#DEBUG_STACK = y

ifndef MAKEOKFLAG

ifeq ($(CROSS), win32) 
 APP_PREFIX = -win32
 CROSS_PATH = /cross/win32
 TARGET = i386-pc-mingw32-
 BUILD_OS = Win32
 STATIC_ALDLIB = y

else ifeq ($(CROSS), armv6h) 
 APP_PREFIX = -armv6h
 CROSS_PATH = /cross/armv6zk
 TARGET = arm-linux-gnueabi-
 BUILD_OS = Linux_arm
 BUILD_ARCH = armv6zk
 BUILD_FPU = vfp
 BUILD_FLOAT_ABI = hard

else ifeq ($(CROSS), armv7a) 
 APP_PREFIX = -armv7a
 CROSS_PATH = /cross/armv7-a
 TARGET = arm-linux-gnueabi-
 BUILD_OS = Linux_arm
 BUILD_ARCH = armv7-a
 BUILD_FPU = vfpv3-d16
 BUILD_FLOAT_ABI = hard

else ifeq ($(CROSS), x86_64) 
 APP_PREFIX = -x86_64
 CROSS_PATH = /cross/x86_64
 TARGET = x86_64-linux-gnu-
 BUILD_OS = Linux_x86_64

else ifeq ($(CROSS), x86_x32) 
 APP_PREFIX = -x86_x32
 CROSS_PATH = /cross/x86_x32
 TARGET = x86_64-linux-gnux32-
 BUILD_OS = Linux_x86_x32

else ifdef CROSS
 $(error $(CROSS) is not supported)

else
 ifdef HOST 
  TARGET = $(HOST)-
 endif

 BUILD_OS := ${shell case `$(TARGET)gcc -dumpmachine` in \
 arm-*) echo Linux_arm ;; \
 mingw32*) echo Win32 ;; \
 i[34567]86-*) echo Linux ;; \
 x86_64-*-gnux32) echo Linux_x86_x32 ;; \
 x86_64-*) echo Linux_x86_64 ;; \
 esac }

 ifndef BUILD_OS
  TMPE := ${shell echo `$(TARGET)gcc -dumpmachine`}
  $(error $(TMPE) is not supported)
 endif

 ifndef BUILD_ARCH
  ifeq ($(BUILD_OS), Linux_arm) 
    BUILD_ARCH := ${shell case `uname -m` in \
    armv7*) echo armv7-a ;; \
    esac }
  endif
 endif

 ifndef UPDV
  UPDV = y
 endif
endif

ifndef STATIC_ALDLIB
 STATIC_ALDLIB = y
endif

ifndef SHARED_ALDLIB
 SHARED_ALDLIB = n
endif

ifeq ($(BUILD_OS), Linux)
 CPU_TYPE = i386

else ifeq ($(BUILD_OS), Win32)
 USE_uClibc =
 CPU_TYPE = i386

else ifeq ($(BUILD_OS), Linux_arm) 
 CPU_TYPE = arm

else ifeq ($(BUILD_OS), Linux_x86_64) 
 CPU_TYPE = x86_64

else ifeq ($(BUILD_OS), Linux_x86_x32) 
 CPU_TYPE = x86_x32

endif

ifdef USE_uClibc
 CROSS_PATH := $(CROSS_PATH)/usr/uC/static
 TARGET = $(CPU_TYPE)-uclibc-
endif

ifdef CROSS_PATH
 PATH := $(CROSS_PATH)/bin:$(PATH)
 export PATH
 CROSS_BINPATH = "$(CROSS_PATH)/bin/"
# cross-gcc warning -I/usr/include
 GCC_NoHostPath = d
 export GCC_NoHostPath
endif

CC = $(TARGET)gcc
LD = $(TARGET)ld
AR = $(TARGET)ar
STRIP = $(TARGET)strip
WINDRES = $(TARGET)windres
INSTALL = install
RM = rm -f
ifeq ($(BUILD_OS), Win32) 
 LN = cp
 CP = cp -a
else
 LN = ln -s
 CP = cp -a
endif

GCC_VER_STR = $(shell $(CROSS_BINPATH)$(CC) -dumpversion)
GCC_VER = $(subst ., ,$(GCC_VER_STR))
GCC_MAJOR_VER = $(word 1,$(GCC_VER))
#GCC_MINOR_VER = $(word 2,$(GCC_VER))

LDFLAGS += ${shell case $(GCC_VER_STR) in \
 4.[56789].*) echo -static-libgcc ;; \
 esac }

ifdef DEBUG
 ifndef CFLAGS
  CFLAGS = -O

  ifdef BUILD_ARCH
   CFLAGS += -march=$(BUILD_ARCH)
  endif
  ifdef BUILD_FPU
   CFLAGS += -mfpu=$(BUILD_FPU)
  endif
  ifdef BUILD_FLOAT_ABI
   CFLAGS += -mfloat=$(BUILD_FLOAT_ABI)
  endif
  CFLAGS += -Wall -Wwrite-strings -Wmissing-declarations -Wundef \
	-Wshadow -Wpointer-arith -Wredundant-decls -Wunused-macros \
	-Winit-self -Wcast-qual -std=gnu99
 endif
 CFLAGS += -g
else
 ifndef CFLAGS
  CFLAGS  = -O2
# -Wcast-qual -Wcast-align
# -Wunused-parameter -Wdeclaration-after-statement -Wconversion
# -Wsign-compare

  ifeq ($(BUILD_OS), Linux) 
   CFLAGS += -march=i386 -mpreferred-stack-boundary=2 -falign-functions=1 \
	-falign-jumps=1 -falign-labels=1 -falign-loops=1 -fno-tree-loop-optimize
#	-fno-guess-branch-probability

  else ifeq ($(BUILD_OS), Linux_x86_64) 
   CFLAGS += -mtune=nocona
# -fno-guess-branch-probability

  else ifeq ($(BUILD_OS), Linux_x86_x32) 
   CFLAGS += -mtune=nocona

  else ifeq ($(BUILD_OS), Win32) 
   CFLAGS += -march=i386 -mpreferred-stack-boundary=2 -falign-functions=1 \
	-falign-jumps=1 -falign-labels=1 -falign-loops=1 
#	-fno-guess-branch-probability

  else ifeq ($(BUILD_OS), Linux_arm) 
#   CFLAGS += -march=armv6 -mcpu=arm1176jzf-s -mfpu=vfp -mfloat-abi=softfp
   CFLAGS += -munaligned-access
  endif

  ifdef BUILD_ARCH
   CFLAGS += -march=$(BUILD_ARCH)
  endif
  ifdef BUILD_FPU
   CFLAGS += -mfpu=$(BUILD_FPU)
  endif
  ifdef BUILD_FLOAT_ABI
   CFLAGS += -mfloat-abi=$(BUILD_FLOAT_ABI)
  endif

  CFLAGS += -Wall -Wwrite-strings -Wmissing-declarations -Wundef \
	-Wshadow -Wpointer-arith -Wredundant-decls -Wunused-macros \
	-Winit-self -Wcast-qual -std=gnu99
 endif

 ifndef DEBUG_STACK
  LDFLAGS += -s
#  CFLAGS += -fomit-frame-pointer
  CFLAGS += -fomit-frame-pointer -fno-asynchronous-unwind-tables
  STRIP_BIN = y
 endif

 ifeq ($(GCC_MAJOR_VER),4)
  CFLAGS += -fgnu89-inline
  ifneq ($(BUILD_OS), Win32)
   LDFLAGS += -Wl,--gc-sections
   CFLAGS += -ffunction-sections -fdata-sections
  endif
 endif

endif

ifneq ($(BUILD_OS), Win32)
LDFLAGS += $(shell if $(CROSS_BINPATH)$(LD) --copy-dt-needed-entries -V >/dev/null 2>&1; \
    then echo "-Wl,--copy-dt-needed-entries"; fi)
endif

loc_obj_PHAT=obj$(CROSS_PREFIX)
loc_obj_shPHAT=obj$(CROSS_PREFIX)

ifeq ($(SHARED_ALDLIB), y) 
 ifeq ($(BUILD_OS), Linux_x86_64) 
  loc_obj_shPHAT=objSH$(CROSS_PREFIX)
 endif
 ifeq ($(BUILD_OS), Linux_x86_x32) 
  loc_obj_shPHAT=objSH$(CROSS_PREFIX)
 endif
 ifdef BUILD_ARCH
  ifeq ($(BUILD_ARCH), armv7-a)
   loc_obj_shPHAT=objSH$(CROSS_PREFIX)
  endif
 endif
endif
#ifeq ($(BUILD_OS), Linux)
# CPU_TYPE = i386
#
#else 
ifeq ($(BUILD_OS), Win32)
 CFLAGS += -D_OS_WIN32
# CPU_TYPE = i386
 USE_PTHREAD =
 USE_ACL =

else ifeq ($(BUILD_OS), Linux_arm) 
# CPU_TYPE = arm
 USE_JPEG_TURBO =

endif

ifeq ($(BUILD_OS), Win32) 
 EXE = .exe
else
 EXE = $(APP_PREFIX)
 PXLIBS = -L$(CROSS_PATH)/usr/X11/lib
export PXLIBS XLIBRARY
endif

ifndef USE_ASM
 USE_ASM = $(CPU_TYPE)
endif

ifdef USE_uClibc
 CFLAGS += -DUSE_uC
 CROSS_PREFIX = $(APP_PREFIX)-uC
else
 CROSS_PREFIX = $(APP_PREFIX)
endif

ifeq ($(STATIC_ALDLIB), y)
 LIBRARY = $(TOP)/lib/$(loc_obj_PHAT)/libaldccp.a
 XLIBRARY = -lX11
else
 LIBRARY = -laldccp -ldl -lpthread
 XLIBRARY = -laldccpw -laldccp -lX11
 USE_JPEG = y
 USE_Z = y
endif

ifdef USE_PTHREAD
 CFLAGS += -DUSE_PTHREAD
 LIBRARY += -lpthread
endif

ifdef USE_ACL
 LIBRARY += -lacl
endif

ifneq (,$(wildcard $(CROSS_PATH)/usr/include/ft2build.h))
 CFLAGS += -DHAVE_FT
 HAVE_FT = y
else ifneq (,$(wildcard $(CROSS_PATH)/usr/include/freetype2/ft2build.h))
 CFLAGS += -DHAVE_FT -I$(CROSS_PATH)/usr/include/freetype2
 HAVE_FT = y
else ifneq (,$(wildcard $(CROSS_PATH)/usr/X11/include/freetype2/ft2build.h))
 CFLAGS += -DHAVE_FT -I$(CROSS_PATH)/usr/X11/include/freetype2
 HAVE_FT = y
else
 HAVE_FT = n
endif

#LD_LIB_DEFAULT  0=dl  1=shared  2=static
ifndef LD_LIB_DEFAULT
 LD_LIB_DEFAULT = 0
endif

ifeq ($(HAVE_FT), y) 
 ifdef USE_FREETYPE
   LIBRARY += -lfreetype
   USE_Z = y
  endif
endif


LD_LIB_JPEG = 1

ifneq ($(LD_LIB_JPEG), 0) 
 ifdef USE_JPEG
  ifdef USE_JPEG_TURBO
   ifeq ($(BUILD_OS), Linux)
    LIBRARY += $(TOP)/lib/Xapp/i386_lib/libjpeg-turbo.a
   else ifeq ($(BUILD_OS), Linux_x86_64)
    LIBRARY += -ljpeg
   else ifeq ($(BUILD_OS), Linux_x86_x32)
    LIBRARY += -ljpeg
   else
    LIBRARY += -ljpeg-turbo
   endif
  else
   LIBRARY += -ljpeg
  endif
 endif
endif

ifeq ($(BUILD_OS), Win32) 
  LD_LIB_PNG = 1
  USE_Z = y
endif

#LD_LIB_PNG  0=dl  1=shared  2=static
ifndef LD_LIB_PNG
 LD_LIB_PNG = $(LD_LIB_DEFAULT)
endif

ifneq ($(LD_LIB_PNG), 0) 
 ifeq ($(LD_LIB_PNG), 2) 
  ifeq (,$(wildcard $(CROSS_PATH)/usr/lib/libpng.a))
   LD_LIB_PNG = 1
  endif 
 endif 

 ifdef USE_PNG
  ifeq ($(LD_LIB_PNG), 1) 
   LIBRARY += -lpng
  else
   LIBRARY += $(CROSS_PATH)/usr/lib/libpng.a
   USE_M = y
   USE_Z = y
  endif 
 endif
endif

ifdef USE_Z
 LD_LIB_Z = $(LD_LIB_DEFAULT)
 ifeq ($(LD_LIB_Z), 0) 
  LD_LIB_Z = 1
 endif
endif

#LD_LIB_Z  0=dl  1=shared  2=static
ifndef LD_LIB_Z
 LD_LIB_Z = 0
endif

ifneq ($(LD_LIB_Z), 0) 
 ifeq ($(LD_LIB_Z), 2) 
  ifeq (,$(wildcard $(CROSS_PATH)/usr/lib/libz.a))
   LD_LIB_Z = 1
  endif 
 endif 

 ifeq ($(LD_LIB_Z), 1) 
  LIBRARY += -lz
 else
  LIBRARY += $(CROSS_PATH)/usr/lib/libz.a
 endif 
endif

ifdef USE_M
 LIBRARY += -lm
endif

OBJ_PHAT = $(loc_obj_PHAT)
OBJ_shPHAT = $(loc_obj_shPHAT)

MAKEOKFLAG = y


export MAKEOKFLAG CC LD AR STRIP WINDRES INSTALL CFLAGS LDFLAGS LIBRARY GCC_VER_STR

export CROSS CROSS_PREFIX CROSS_PATH USE_ASM RM LN CP BUILD_OS EXE
export OBJ_PHAT loc_obj_PHAT OBJ_shPHAT loc_obj_shPHAT STATIC_ALDLIB SHARED_ALDLIB
export USE_uClibc USE_PTHREAD USE_ACL USE_JPEG DEBUG_STACK UPDV

export LD_LIB_PNG LD_LIB_JPEG HAVE_FT

endif  #------ MAKEOKFLAG  -----------------------------------------------

INCL += -I$(TOP)/include
DEPH += $(addprefix $(TOP)/include/,$(DEPSYSH)) $(TOP)/vernr Makefile Mobj
ifeq ($(STATIC_ALDLIB), y)
 PCLIBS = -L$(TOP)/lib/$(OBJ_PHAT)
 DEPLIB += $(TOP)/lib/$(OBJ_PHAT)/libaldccp.a
else
 PCLIBS = -L$(TOP)/lib/$(OBJ_shPHAT)
 DEPLIB += $(TOP)/lib/$(OBJ_shPHAT)/libaldccp.so
endif
PATHLIBS = $(PCLIBS) $(PXLIBS) 


ifneq ($(BUILD_OS), Win32) 
 ifndef USE_uClibc
  ifeq (,$(wildcard $(CROSS_PATH)/usr/include/X11/Xlib.h))
    ifeq (,$(wildcard $(CROSS_PATH)/usr/X11/include/X11/Xlib.h))
      $(error no X11 include found !!!)
    else
      INCL += -I$(CROSS_PATH)/usr/X11/include
    endif
  endif
 endif
endif

#ifeq ($(STATIC_ALDLIB), y)
# RPATH = 
#else
# ifdef RPATH_ALDLIB
#  RPATH = -Wl,-rpath,$(CURDIR)/$(TOP)/lib
# else
#  RPATH = 
# endif
#endif

lib_obj_PHAT = $(TOP)/lib/$(OBJ_PHAT)
lib_obj_shPHAT = $(TOP)/lib/$(OBJ_shPHAT)

prefix = /opt/aldccp_inst$(APP_PREFIX)
bindir = $(prefix)/bin
libdir = $(prefix)/lib
includedir = $(prefix)/include
datadir = $(prefix)/share
