/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public License
 as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version. 

 This library is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU Lesser General Public License for more details. 

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA
*/

#ifndef	__APPXBASE_H__
#define	__APPXBASE_H__    1

#include "appbase.h"

#ifdef _OS_WIN32

#include "w32x11.h"

#else /* _OS_WIN32 */

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>

#endif /* ! _OS_WIN32 */

#ifndef __BIXFORMAT_H__
#include "bixformat.h"
#endif

#define XC_X_cursor		0
#define XC_arrow		2                 
#define XC_bottom_left_corner	12
#define XC_bottom_right_corner	14
#define XC_bottom_side		16
#define XC_dot			38
#define XC_fleur		52
#define XC_left_side		70
#define XC_pirate		88
#define XC_right_side		96
#define XC_sb_h_double_arrow	108
#define XC_sb_v_double_arrow	116
#define XC_top_left_arrow	132
#define XC_top_left_corner	134
#define XC_top_right_corner	136
#define XC_top_side		138
#define XC_watch		150
#define XC_xterm		152
#define XC_none			199
#define XC_MAX			200


#define msgXEv			1
#define msgKey			2
#define msgPaint		3
#define msgFocus		4

#define msgResize		6
#define msgVisibleFormShow	7
#define msgGetFocus_ap		8
#define msgGetWrcStat		9
#define msgRePaint		10
#define msgRePaintAll		11
#define msgPaintEvEnd		12
#define msgDestroy		13
#define msg_dnd			14
#define msgf_dndEnter        0
#define msgf_dndPosition     1
#define msgf_dndDrop         2
#define msgf_dndStatus       3

#define msgAHsever           100

#define msgErrLogOn          200
#define msgFTP_Reply         201
#define msgFTP_Err           202
#define msgFTP_TreansStat    203

extern char *display_name;
extern bool (*On_wmError)(Display *dpy, XErrorEvent *e);

#ifdef Test_Root_WinId
extern Window TestRootWinId;
extern Window OrgRootWinId;
#endif

typedef struct TPic {
  Pixmap pic;
  Pixmap mask;
  int width;
  int height;
} TPic;

typedef struct {
  TPic en;
  TPic dis;
} TPicEnDis;

typedef struct TIconWin {
  XID WinID;
  TPic *Pic;
  int fu; //b0=Map
} TIconWin;

typedef struct {
    Window root;
    int x;
    int y;
    int width;
    int height;
    int border_width;
    int depth;
} TGeometry;

extern bool get_geometry(Drawable d, TGeometry *g);

typedef struct {
  int num;
  int left;
  int top;
  int right;
  int bottom;
#ifdef _OS_WIN32
  HMONITOR mon_id;
#endif
} TMonitor;

#define MAX_MONITORS 16
#define dpyM_Active 1
#define dpyM_AutoReScane (1<<1)
#define dpyM_Change      (1<<2)

typedef struct {
  int count;
  int PrimaryIdx;
  int stat;
  void (*OnChange)();
  TMonitor mon[MAX_MONITORS];
} TdpyMonitors;

extern TdpyMonitors dpyMonitors;

extern void MonitorsInit();

//******************* graf.c
extern dword REGP1 GetColor_RGB(dword RGB);
extern dword AddColor(dword RGB, dword RGB_add);
extern dword SubColor(dword RGB, dword RGB_sub);


extern GC CreateGC(long gcm, XGCValues *gcv);
extern void FreeGC(GC gc);
extern GC CreateGC_RGB(long RGB);
extern void ChangeGC(GC TGC, long gcm, XGCValues *gcv);
extern GC GetPaintGC(int RGB);

extern void DrawLine(Window WinID, GC TGC, int X1, int Y1, int X2, int Y2);
extern void DrawLineRGB(Window WinID, int RGB, int X1, int Y1, int X2, int Y2);
extern void DrawRect(Window WinID, GC TGC, int x, int y, int w, int h);
extern void DrawRectRGB(Window WinID, int RGB, int x, int y, int w, int h);
extern void DrawRectFocus(Window WinID, int x, int y, int w, int h);
extern void StdBorderPaint(Window WinID, int BorderFu, int x, int y, int w, int h);

extern void Fill_Rect(Window WinID, GC gc, int x, int y, int w, int h);
extern void FillRectRGB(Window WinID, int RGB, int x, int y, int w, int h);

extern void Set_Icon_gcv(TPic *pic, long gcm, int x, int y);
extern void DrawPic(Window WinID, TPic *pic, int x, int y);
extern void DrawPic_wh(Window WinID, TPic *pic, int picx, int pixy, int width, int height, int x, int y);
extern void ClrPic(TPic *pic);

//******************* ahico.c
extern TPic REGP2 *CreateAHicoPic(const char *icoHead, TPic *pic);
extern TPic REGP2 *CreateAHicoDisPic(const char *icoHead, TPic *pic);

extern TPic REGP3 *CreateAHicoPic_EnDis(const char *icoHead, TPicEnDis *pic_ed,
							 bool disable);

extern struct TPixMap *CreateAHicoImg(const char *icoHead);
extern struct TPixMap *CreateAHicoImg_depth(const char *icoHead, int rdepth);

//******************* grid/fileicons.c
#define fico_Folder	1
#define fico_Link	2
#define fico_Chardev	3
#define fico_Blockdev	4
#define fico_FIFO	5
#define fico_Socket	6
#define fico_None	16
#define fico_Txt	17
#define fico_Exec	18
#define fico_SO		19
#define fico_Tgz	20
#define fico_Shell	21
#define fico_Pascal	22
#define fico_C		23
#define fico_HTML	24
#define fico_Binary	25
#define fico_Make	26
#define fico_Font_A	27
#define fico_Font_Fix	28


extern TPic *get_fileIcon(int idx);
extern TPic *get_fileDisIcon(int idx);

extern TPic *get_fileIcon32(int idx);

//----------
typedef struct {
  void (*InitIconPaths)(TpcIni *ini);
  TPic *(*GetDeskIcon)(int ddkt, int size, const char *name, int fu, const char *default_zico);
} ThGetDeskIcon;

extern const ThGetDeskIcon *hGetDeskIcon; //NULL, hGetDeskIcon_file or hGetDeskIcon_loc

extern const ThGetDeskIcon hGetDeskIcon_file;    //file icons else buildin icons
extern const ThGetDeskIcon hGetDeskIcon_buildin; //use only buildin icons

#define gdifu_getico0 0
#define gdifu_getico1 1
#define gdifu_getico2 2
#define gdifu_getico3 3
#define gdifu_geticomask 0x03

#define gdifu_noUnknown (1<<8)
#define gdifu_noZoom (1<<9)
#define gdifu_noError (1<<10)

#define gdifu_PathScalable (1<<16)
//#define gdifu_svg (1<16)
#define gdifu_mask 0x00ffffff

#define gdifu_pvUnknown (1<<24)
#define gdifu_pvZoom (1<<25)

extern TPic *GetDeskIcon(int ddkt, int size, const char *name, int fu, const char *default_zico);
//harddisk_mount_pic_32x32 = 
//  GetDeskIcon(ddkt_FSDevice, 32, "32x32", "harddisk", gdifu_getico1);
//----------
extern TPic *GetZIcon(const char *zico);
extern TPic *GetDisZIcon(const char *zico);

#ifdef USE_ONLY_ZICON
#define GetDeskIcon_z(a,b,c,d,e) GetZIcon(e)
#else
#define GetDeskIcon_z GetDeskIcon
#endif

//******************* font.c
#define keyst_Press		1
#define keyst_Release		(1<<1)
#define keyst_WinsIsDis		(1<<2)
typedef struct TKeyEvent {
  int	keyst;
  dword state;		/* key or button mask */
  dword keycode;	/* detail */
  KeySym keysym;
} TKeyEvent;

        /* TFont.fu */
#include "fontX.h"

typedef struct TVDrawString {
  AlgTlgs
  int Xoff;
  int Yoff;
  TFontX *Font;
  dword ForeColor, BackColor;

  int SelA, SelE;
  struct TStrColorDatArr *scda;
  Window WinID;

  int TabSpaceWidth;
  struct TWins *Wins;
  bool OnlyLg;
  bool TabSpaceOn;
  bool TabSpaceDot;
  bool UTF8on;
  bool Image;				//DrawImageString
  bool ClsToEnd;
  bool SelToEnd;
  bool nc;

} TVDrawString;

#define scdfu_override_utf8	0x01
#define scdfu_UTF8on		(1<<1)

typedef struct TStrColorDat {
  AlgTlgs
  TFontX *Font;
  int ForeColor, BackColor;
  int fu;
} TStrColorDat;

typedef struct TStrColorDatArr {
  int items;
  TStrColorDat scd[];
} TStrColorDatArr;

TStrColorDatArr *set_scda_arr(void **scda, int items);

//-misc-fixed-bold-r-normal-*-13-120-75-75-c-70-iso10646-1
// 0    1     2    3 4      5 6  7   8  9  1 1  1        1
//                                         0 1  2        3
typedef struct {
  Tlgs lgs[14];
} TFontLgs;
extern bool FontLgsArrFill(TFontLgs *fl, const char *pc);

extern bool FontExists(const char *fontname);
extern char *GetFullFontName(const char *fontname);
extern TFontX *MakeFont(const char *fontname, const char *defaultfont);

extern int DrawColorString(int x, int y, TVDrawString *vds);
extern int TextWidthColorString(TVDrawString *vds);
extern void DrawLineString(const char *s, int x, int y, int *max_w, int *max_h, TVDrawString *vds);
extern int GetChar_XPos(int idx, TVDrawString *vds);
extern int GetMouseX_CharPos(int x, TVDrawString *vds);

extern bool keysymtostr(KeySym *keysym, char *ret,  bool UTF8on);
extern int SendKeyCode(dword keycode, bool is_press);
extern int SendKeySym(KeySym keysym, bool is_press);

//state = ShiftMask,LockMask,ControlMask,...
extern int SendKeyToWins(struct TWins *wins, dword keysym, dword state, bool is_press);

extern char *vds_path_cut(TVDrawString *vds, const char *s, int max_w);

//******************* getVds_WChar.c
extern dword REGP3 GetVds_WChar(int idx, TVDrawString *vds);
extern dword REGP3 GetVdsNext_WChar(int *idx, TVDrawString *vds);
extern dword REGP3 GetVdsPrev_WChar(int *idx, TVDrawString *vds);

//******************* fontart.c
extern bool GetFontSXTypeIdx(int idx, const char *sx, const char **ret_pc, 
				int *ret_lg);
extern bool SetFontSXTypeIdx(int idx, const char *sx, const char *sins, 
				char **ret_sx);
extern bool SetFontSXVTypeIdx(int idx, char **sx, const char *sins);

#define FONT_Any	0
#define FONT_Medium	1
#define FONT_MediumK	2
#define FONT_Bold	4
#define FONT_BoldK	8
#define FONT_SMask	0x0f
#define FONT_NoResize	(1<<8)

extern bool GetFontArtName(const char *sx, int font_art, char **ret_sx);
extern int GetFont_Art(const char *sx);	//ret = font_art

extern bool MakeFontArt(const char *FontName, TFontX **ppFont, int font_art);

extern bool ChkScaleFont(const char *sx);
extern bool ChkFixedFont(const char *sx);
extern int GetFont_Size(const char *sx);
extern char *GetFont_NextSize(const char *sx, int size);

//******************* appbase.c

typedef struct Tobjmsg {
  int msg;
  struct TWins *wins;
  int f;
  int m;
  void *p;
} Tobjmsg;

typedef struct TXev_msg {
  int msg;
  struct TWins *wins;
  int f;
  int m;
  XEvent *ev;
} TXev_msg;

typedef int (TEvProc)(Tobjmsg *msg);


extern const char *Desktop_Entry;

typedef struct {
  TgpList *gpList;
  union {
    char *Exec;
    const char *Exec_const;
  };
  int fu;
  char *ret_cmd;
//  bool err;	//WIN32 exec proc not found
} TDesktopCMD;

extern bool get_desktop_dskcmd(TDesktopCMD *dskcmd);
extern char *get_desktop_CMD(TpcIni *ini, TgpList *gpList);

extern int ddkt_desktop_stat(const char *fname, Tstat64 *st);
extern int ddkt_desktop(const char *fname);
extern int exec_desktop(char *fname, int ddkt, int fu, int *retfu);
extern void dotdesktop_new(const char *fn, const char *Name, const char *Icon, 
			const char *s1, const char *s2, const char *s3, int ddkt);


typedef struct {
  struct TWins *apph;
  struct TWins *oldapph;
  int x;
  int y;
  int x_root;
  int y_root;
  int button;
  int state;
  int obj_left;
  int obj_top;
  int obj_width;
  int obj_height;
  int UpDownCount;
  Time time_ev;
  bool DblClickAkt;
  bool WinIsEnabled;
  bool nc2;
  bool nc3;
} TMouseDown;

extern TMouseDown MouseDown;

#include "stdwin.h"


#ifndef _OS_WIN32
typedef Window REGP1 (*THandleXEvent)(TXev_msg *msg);
#endif

#define MaxSelRetDat	10
#define MaxSELECT_Atoms 7


typedef struct {
  int wtyp;
  TIconWin IconWin[16];  //0=dnd_No 1=dnd_Txt 2=dnd_Folder
  int dnd_Schr;
  int dndSel_LastX, dndSel_LastY;
  Atom dndSel_rAtoms[16];
  int dndSel_rAtomsCount;

  XID dndSel_SrcWin;
  unsigned int dndSel_SrcVer;

  XID dndSel_DestWin;

  Atom dndM_SELECT_Atoms[MaxSELECT_Atoms];
  int dndM_SELECT_AtomsCount;
  XID dndM_SrcWin;
  TWins *SrcWins;
//  XID dndM_SrcClientWin;
  XID dndM_DestWin;
  int dndM_WinNr;
  int dndM_WinOkNr;
  int dndM_WinX, dndM_WinY;
  Atom dndM_Action;
  Tlgs SelTxt;
  Tlgs SelTxt_URL;
  bool (*dnd_Message)(XClientMessageEvent *ev);
  bool (*dndM_Message)(XClientMessageEvent *ev);
  void (*SetSelection)(TWins *ws, Atom *nAtom, int nAtomCount);
  void (*MoveSet)(int sch, XID destWinID, int x, int y);
  XID (*GetMoveWin)();
  bool (*SelectionReq)(XSelectionRequestEvent *ev, char **ppc, int *lg, int *iss);
/* *** ACHTUNG auf Reienfolge achten *** */
  Atom XA_MOTIF_DRAG_AND_DROP_MESSAGE, _MOTIF_DRAG_INITIATOR_INFO, XA_MOTIF_DRAG_RECEIVER_INFO;
  Atom XA_MOTIF_DRAG_WINDOW, _MOTIF_DRAG_TARGETS;
  Atom XmTRANSFER_SUCCESS, XmTRANSFER_FAILURE;
  Atom XA_XdndActionAsk, XA_XdndActionCopy, XA_XdndActionLink;
  Atom XA_XdndActionMove, XA_XdndActionList, XA_XdndActionPrivate;
  Atom XA_XdndAware, XA_XdndEnter, XA_XdndPosition, XA_XdndLeave;
  Atom XA_XdndStatus, XA_XdndSelection, XA_XdndTypeList;
  Atom XdndProxy, XA_XdndDrop, XA_XdndFinished;
} Tdnd;

#define evFilter_sel_rd 0
#define evFilter_sel_wr 1
#define evFilter_max 2

typedef struct {
  void (*Filter_PropEv)(XPropertyEvent *ev);
  Atom FilterAtom;
} TPropEventFilter;

extern Tptr_arr *PaintOnEvEnd_List;
extern int PaintOnEvEnd_Count;
#ifndef _OS_WIN32
extern XMotionEvent MotionEvent;
#endif

typedef struct TappXDat {
  Display *dpy;
  XID RootW;
  int default_screen;
  int Root_width, Root_height, RootW_depth;
  bool UTF8on;
  bool FileSystemUTF8on;
  bool xnc2, xnc3;

  TForm *StartForm;
  void (*OnAppStart)();
  void (*OnAppClose)();
  TWins *Wins;

  TWins *KeyInWin;
  TWins *KeyInForm;
  TWins *GrabWin;
  dword last_KeyState;

  TWins *InfoWin;
  dword InfoWin_us;

  dword Default_Color;
  TFontX *Default_Font;
  TFontX *Default_bold_Font;

  GC IconGC;
  XGCValues Icon_gcv;
  GC Default_GC;
  GC bordL_GC;
  GC bordH_GC;
  GC GC_FFFFFF;
  GC GC_C0C0C0;
  GC GC_404040;
  GC GC_000000;
  GC GC_000000_Dot;
  GC Select_GC;
  GC Select_GC_Back;
  GC FocusLine_GC;
  Pixmap Focus_bitmap;
  GC Paint_GC;
  int Paint_Color;
  GC SB_GC;

  Cursor XCursors[XC_MAX];
  int Default_MouseZ_Wheel;

#ifdef _OS_WIN32
  int LogPixel;
#else
/* ****** ACHTUNG !!! Atom reihenfolge in appXbase.c !!!! ****** */
  Atom XA_WM_DELETE_WINDOW;
  Atom XA_WM_STATE;
  Atom XA_WM_PROTOCOLS;
  Atom XA_MOTIF_WM_HINTS;
  Atom XA_ALD_selection;
  Atom XA_NONE;
  Atom XA_CLIPBOARD;
  Atom XA_TARGETS;
  Atom XA_INCR;
  Atom XA_UTF8_STRING;
  Atom XA_TEXT_PLAIN;
  Atom XA_URL_STRING;
  Atom XA_IMAGE_PNG;
  Atom XA_KDE_CUT;
  Atom XA_KDE_URL_STRING;
  Atom XA_NET_WM_STATE;
  Atom XA_NET_WM_USER_TIME;
  Atom XA_NET_WM_STATE_ABOVE;
  Atom XA_NET_WM_PID;
  Atom XA_NET_WM_ICON;
/* ****** ACHTUNG  !!! Atom reihenfolge ende ****** */

  Atom CLIPBOARD_ATOM;
#endif /* _OS_WIN32 */

  Atom SELECT_Atoms[MaxSELECT_Atoms];
  int SELECT_AtomsCount;
  int SelCut;		//"0"=false "1"=true
  int SelArt;		//0=String 1=UTF-8 String
  bool SelWait;
  bool appCursorWait;
  bool nc2_c1;
  bool nc3_c1;

  Tlgs SelTxt;
  Tlgs SelTxt_URL;

  Tlgs SelRet_lgs;
  Tlgs tmp_SelRet_lgs;

#ifndef _OS_WIN32
  struct TPixMap *Selpxm;
  Atom SelRetAtom;
  int SelRetFormat;
  Window SelHandleWin;
#endif /* _OS_WIN32 */
  TPropEventFilter PropEvFilter[evFilter_max];

  Tptr_arr *pathPicRoot;
  TDecor *DefaultWinsDecor;
  TDecor *DefaultButtonDecor;

  Tdnd *dnd;

  XKeyEvent *Tmp_XKeyEv;
} TappXDat;

extern TappXDat appXDat;
extern dword appXDat_Default_Color;

extern void initXapp(int fu, int argc, char **argv);

#ifdef _OS_WIN32
extern GC WinsDefault_GC;
#else
extern void appXeventDefaultInit();
extern void REGP2 set_XEventHandle(int evtype, THandleXEvent EvProc);
extern THandleXEvent REGP1 get_XEventHandle(int evtype);
extern Window REGP1 exec_XEventHandle(TXev_msg *msg);

extern void REGP1 NumLook_korr(TKeyEvent *KeyEv);

extern void Send_MSG_MotionEvent();
#endif

extern void X11DispatchEvent();
extern void Wins_PaintOnEvEnd();

extern void TWinsRegistration(TWins *wins);
extern void TWinsDeregister(TWins *wins);

extern TWins *GetTWins(Window WinID);
extern bool TWins_ChkExists(TWins *self);
#ifdef MEM_CHK
extern int GetTWinsCount();
#endif

extern void appSetCursorWait(TWins *wins, bool Value);

extern bool GetXWinAbs_rc(XID win, TWRect *rc);
extern TWins *GetNextFocEl(TWins *wins, bool ret, bool over);


extern int Exec_Win_Ev(Tobjmsg *msg, Window win);
extern int Send_MSG(Tobjmsg *msg, TWins *wins);

extern void MapWinID(Window WinID, int fu);  //fu 0=UnMap 1=Map 2=MapRaised
extern void AppHandleWinMessages();

extern void Std_GrabPointer(TWins *gw, Cursor GrabCursor);
extern void Std_UngrabPointer();

extern int TWins_EvProc(Tobjmsg *msg);

extern void AzAtom(const char *F, int L, const char *s, Atom a);

//************************** Caret ********************************************
extern void Caret_Init();
#define CARET_OFF 0
#define CARET_ON  1
extern void Caret_Draw(int fu);
extern void Caret_Setup(Window WinID, int left, int top, int width, int height);
extern void Caret_SetPos(Window WinID, int left, int top);
extern void TWins_Caret_SetNewPos(TWins* self, int left, int top);
extern void TWins_CaretRect_SetNewPos(TWins* self, int x, int y, int w);
extern void TWins_Caret_Clr(TWins* self);
extern bool TWins_Caret_Chk(TWins* self); //TWins->WinID == Caret->WinID

//************************** Selection ****************************************
extern void SetSelection(int count, ...);
extern void SelClr();
extern void SelRetClr();
extern bool GetPasteStr_lgs(int enc_utf8);
extern bool Chk_GetSelection(int count, ...);

extern int Get_Preferred_DropEffect();

#ifdef _OS_WIN32
extern bool Clipboard_Open();	//False to chk appError(EMSG_FromLastBuf, NULL);
extern void Clipboard_Close();
extern void Clipboard_Add();
#else
extern void SelectionInit();
extern bool GetPasteDaten(Atom SrcAtom, Atom XA_SelAtom);
extern bool Get_XA_TARGETS();
extern Atom get_nSelAtom(Atom a);
#endif

//************************** dnd ****************************************
#ifdef DRAG_AND_DROP

#define dnd_version		4

extern void TWins_dnd(TWins *self, TOndndMouseMove OndndMouseMove, TOndndDrop OndndDrop);

#define  dndM_Move_Auto        	 0
#define  dndM_Move_Enter       	 1
#define  dndM_Move_Position    	 2
#define  dndM_Move_Leave       	 3
#define  dndM_Move_Drop        	 4
#define  dndM_Move_Close       	 5

typedef struct TMotifDragReceiverInfo {
  unsigned char byte_order;
  unsigned char protocol_version;
  unsigned char protocol_style;
  unsigned char pad;
  int proxy_window;
  short num_drop_sites;
  short padding;
  int total_size;
} TMotifDragReceiverInfo;

extern Tdnd *Tdnd_Atoms_Create();
//void Tdnd_Create(Window Form_WinID, int fu);
extern void Tdnd_Create();
extern void Tdnd_master_Create();
//bool dnd_ClientMessage(XEvent *ev);
extern void dndM_MoveSet(int sch, XID destWinID, int x, int y);
extern XID dndM_GetMoveWin();
extern Atom dnd_AisText();

extern bool dnd_ClientMessage(XClientMessageEvent *ev);

#endif  /* DRAG_AND_DROP */

//************************** SysTray ****************************************
extern bool GetSysTrayWin(Window *ret_SysTrayWin, TWRect *ret_rc);
extern void SendSysTrayWin(Window SysTrayWin, Window WinID);

//************************** Xapp/get_gravity.c *******************************
extern void get_gravity_offset(int Gravity, int width, int height, int *left, int *top);


//************************** TPic ******************************************
/****** pic_cache.c ******/

extern void InitIconPaths();
extern TPic *pic_load(const char *FileName, TPic *pic_mem);  //+path_rel2abs

//************************** Decor ******************************************
#define imBst_Mask	1
#define imBst_MaskAkt	(1<<1)

extern TDecor *TDecor_Create(void *dat);
extern TDecor *TDecor_dup(TDecor **Decor);

typedef struct TBordMaskHead {
  byte HeadType;
  byte DatX;
  byte DatY;
  byte CornerLen;
  byte Borderwidth;	//bit0..3 = Borderwidth
  byte nc5;
  byte nc6;
  byte nc7;
  int8_t dat[];
} TBordMaskHead;


extern TDecor *TDecor_w3b3_Create(const TBordMaskHead *upDat, const TBordMaskHead *downDat);


extern TDecor *TDecor_button_w3b3_Create();
extern void TDecor_button_SetBorder(TDecor *Decor, char *up, char *down);

extern const TBordMaskHead TDecoBuW3B3_up_Tab;
extern const TBordMaskHead TDecoBuW3B3_down_Tab;
extern const TBordMaskHead TDecoBuR3B3_up_Tab;
extern const TBordMaskHead TDecoBuR3B3_down_Tab;
extern const TBordMaskHead TDecoBuR5B4_up_Tab;
extern const TBordMaskHead TDecoBuR5B4_down_Tab;

#endif /* __APPXBASE_H__ */
