/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public License
 as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version. 

 This library is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU Lesser General Public License for more details. 

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA
*/

#ifndef	__APPBASE_H__
#define	__APPBASE_H__    1

#include "sysbase.h"
#include "sysbaseio.h"
#include <sys/time.h>

//#ifndef _OS_WIN32
//#include <sys/wait.h>
//#endif

//******************** Error *******************************
#define Timer_Enabled		1
#define Timer_Single		(1<<1)
#define Timer_OnTimerAndFree	(1<<2)
#define Timer_NoLostOnTimer	(1<<3)

#define TAlgItem TgTimer

typedef struct TgTimer {
  AlgTItem
  void *obj;
  void (*OnTimer)(struct TgTimer*);
  dword OrgInterval;
  dword func;
/* private */
  dword intern_func;
  dword RunInterval;
  dword Tmp_ms;
} TgTimer;

#undef TAlgItem

extern TgTimer *gTimerX_Add(dword func, dword Interval_ms, void (*OnTimer)(struct TgTimer* gTimer));
extern void TgTimer_NewInterval(TgTimer *self, dword Interval_ms);
extern void gTimer_Init();
// time_ms 16667..500000ms --> 60..2Hz
extern void TgTimer_SetMaxTime(dword time_ms);
extern void TgTimer_SetMinTime(dword time_ms);

typedef struct TPid_Array {
    struct TPid_Array *next;
    int pid;
    int tag;
    void *obj;
    void (*On_pid_close)(int pid, int stat_loc, struct TPid_Array*);
} TPid_Array;

typedef struct Tfd_list {
#ifdef _OS_WIN32
  struct Tfd_list *next;
  void (*Proc)(int fd, struct Tfd_list*);
  int fd;  //file descriptors in Winsock
  int fu;
#else  /* _OS_WIN32 */
  void (*Proc)(FileHandle fd, struct Tfd_list*);
  FILE *io;
#endif /* _OS_WIN32 */
  void *obj;	//in TExecProc Texec_pipe*
  int m;	//in TExecProc Texec_pipe->pp[m]
} Tfd_list;

typedef void (*TEv_OnPipe)(FileHandle fd, Tfd_list *fdl);

#ifdef _OS_WIN32
extern void (*FOnSocketFD)(bool sel);
extern bool AddAppMsgEvent(HANDLE h);
extern bool DelAppMsgEvent(HANDLE h);
#else /* _OS_WIN32 */
extern void REGP1 fd_set_count(FileHandle fd);
#endif /* _OS_WIN32 */

typedef struct TEv_Array {
    struct TEv_Array *next;
    int tag;
    void *obj;
    void (*On_appEv)(struct TEv_Array *appEv);
    bool run_mk, nc1, nc2, nc3;
} TEv_Array;

#define appFu_select_t_NULL	1
#define appFu_select_t0_single	(1<<1)
#define appFu_select_t1_single	(1<<2)
#define appFu_select_t_single	(1<<3)


typedef void (*TOnAHServ)(void *sh);

typedef struct {
  int self_pid;
  char *self_exe;
  const char *sHOME;
  char *ALDini_dir;
  char *inifile_global;
  char *inifile_app;
  dword appFu;
  bool gQUIT_Ev, gSIGPIPE_Ev, Sleep, ba_nc3;
  bool AppRun, AppExit, ba_nc6, ba_nc7;
  void *obj;
  TPid_Array *Pid_Array;
  void (*OnPipeError)(int fd, int err);

  TEv_Array *appEv_Array;
  int gTimerNewEv;

  void (*TimerDispatch)();
  int x_fd;
  void (*X11DispatchEvent)();
  void (*Wins_PaintOnEvEnd)();
  void (*thd_private_EV)();

  struct timeval app_tv;
  uint64_t app_us;
  dword app_ms;
  dword Hz30;
  dword tapp_us;
  struct timeval start_tv;
#ifdef _OS_WIN32
  HANDLE app_Event;	//used in tthread.c  SetEvent(appDat.app_Event);
#else
  int fd_set_count;
  Tfd_list *fd_list;
  byte AktPipes[FD_SETSIZE];  //(b7=OutPipe+b0=SendAutoDat)
#endif
  TpcIni *globalIni;
  TpcIni *appIni;

  uint64_t app_start_us;
  dword app_start_ms;
  dword app_start_sec;

  struct Tpthd_arr *pthd_arr;
  void (*MSG_Buf_Ev)();
} TappDat;

extern TappDat appDat;

#define app_NoIni	1
extern void initapp(int fu, int argc, char **argv);

extern void AppRun();
extern void Close();

extern bool AppProcessMessages();
extern bool AppHandleMessage();
extern bool AppHandleMessageRun();
extern void AppSetWait0_single();
extern void AppSetWait1_single();
extern void AppSetWait_single(int us);

#if AppEvCountAz10s
extern bool DisableAppEvCountAz10s;
#endif

extern void setDefaultIniDat(const char *DefaultIniDat);

extern bool fd_in_wait_sigle(int fd, int us);

typedef void (*TOn_pid_close)(int pid, int stat_loc, TPid_Array *Pid_Array);
extern TPid_Array *Add_pid_Handle(int pid, TOn_pid_close On_pid_close);
extern TPid_Array *Get_pid_Handle(int pid);
extern void Free_pid_Handle(int pid, int stat_loc);

extern TEv_Array *Add_appEv(void (*On_appEv)(TEv_Array *appEv));
extern void Free_appEv(TEv_Array *appEv);

typedef void (*TOnPipe)(int fd, Tfd_list *fd_list);
#define InPipe		1
#define OutPipe		0x80
#ifndef _OS_WIN32
cHIDDEN Tfd_list *AddPipeProc_TmpRes(FileHandle fd, int fu, TOnPipe OnPipe, void *obj);
extern void AddPipeProc(FileHandle fd, int fu, TOnPipe OnPipe, void *obj);
#define SocketAddPipeProc AddPipeProc
/* SocketAddPipeProc in net_io.h + socket.c */

#define clpp_fd		0x01
#define clpp_io		0x02
#define clpp_fdx	0x04	/* force close(fd) */
extern void REGP2 ClosePipeProc(FileHandle fd, int fu);
extern void REGP1 pClosePipe(FileHandle *fd);
extern void REGP1 ClosePipe(FileHandle fd);
#endif /* _OS_WIN32 */

#define carfu_cmd	1
extern bool ChkAppRun(const char *fn, int fu, char **ppc);

extern TpcIni *get_appIni();
extern void free_appIni(TpcIni *pcIni);

extern TpcIni *get_globalIni();
extern void free_globalIni(TpcIni *pcIni);

//*********************** app/app_ChkIsThread.c **********************
extern bool (*app_ChkIsThread)();
extern bool ChkIsThread();

//*********************** app/cmdtoargs.c ****************************
extern void cmdtoargs(const char *cmd, int *argc, char ***argv);
extern void args_free(char **argv);

//*********************** app/exec_pipe.c ****************************

typedef struct _Texec_pipe Texec_pipe;

#define fd0_outfd		0x00
#define fd1_infd		0x01
#define fd2_errfd		0x02
#define fd3_Doutfd		0x03
#define fd4_Dinfd		0x04

#define evtyp_outfd		0x00
#define evtyp_infd		0x01
#define evtyp_errfd		0x02
#define evtyp_Doutfd		0x03
#define evtyp_Dinfd		0x04
#define evtyp_fd_Close_offset	0x10
#define evtyp_outfd_Close	0x10
#define evtyp_infd_Close	0x11
#define evtyp_errfd_Close	0x12
#define evtyp_Doutfd_Close	0x13
#define evtyp_Dinfd_Close	0x14
#define evtyp_ProcStart		0x20
#define evtyp_ProcClose		0x21

/* expipe->CloseFlag data in lb (in lib/app/exec_proc.c) */
#define epCF_infd		(1<<1)
#define epCF_errfd		(1<<2)
#define epCF_Dinfd		(1<<4)
#define epCF_UserError		(1<<8)
#define epCF_Error		(epCF_errfd | epCF_UserError)

typedef void (*TOnExecProcEv)(Texec_pipe* expip, int evtyp);

typedef struct {
  int fd;
  TLineBuf *lb;
  void (*Proc)(int fd, Tfd_list *fd_list);
} Tprocpipe;

#define AlgTexec_pipe  AlgTTC0 \
  char *args; \
  void *owner; \
  void (*OnExecProcEv)(TCSelf* expip, int evtyp); \
  char *cd_path; \
  char *env; \
  TPid_Array *pida; \
  void (*On_pid_close)(int pid, int stat_loc, TPid_Array *Pid_Array); \
  Tprocpipe pp[5]; \
  int tag; \
  int CloseFlag;

#define TCSelf Texec_pipe
struct _Texec_pipe{
  AlgTexec_pipe
  void *p1;
  char *fname;
};
#undef TCSelf

extern Texec_pipe *Texec_pipe_Create();
extern bool Texec_pipe__Destroy(Texec_pipe *self);

#define expipfu_args_free		1
#define expipfu_addPID_H		(1<<1)

extern int exec_pipe_argv(char **argv, int fu, Texec_pipe *expip);

//*********************** app/exec_pc.c ****************************
extern int exec_pc_co_path(const char *args, TOn_pid_close On_pid_close, char *path);
extern int exec_pc_co(const char *args, TOn_pid_close On_pid_close);
extern int exec_pc(const char *args);

//*********************** app/exec_proc.c ****************************

#define outfd_Open		1
#define infd_Open		(1<<1)
#define errfd_Open		(1<<2)
#define Doutfd_Open		(1<<3)
#define Dinfd_Open		(1<<4)

extern int ExecuteProcApp_argv_ex(const char *cmd, int argc, char **argv, int fu, 
				TOnExecProcEv OnExecProcEv, void *owner, 
				Texec_pipe **ret_expip, 
				const char *cd_path, const char *env, int tag);

extern int ExecuteProcApp_argv(const char *args, int argc, char **argv, int fu, 
				TOnExecProcEv OnExecProcEv, void *owner, 
				Texec_pipe **ret_expip, int tag);
extern int ExecuteProcAppTag(const char *cmd, int fu, 
				TOnExecProcEv OnExecProcEv, void *owner, 
				Texec_pipe **ret_expip, int tag);
extern int ExecuteProcApp(const char *args, int fu, 
				TOnExecProcEv OnExecProcEv, void *owner, 
				Texec_pipe **ret_expip);
extern int ExecuteProc(const char *args);

//*********************** misc ****************************

#ifdef MEM_CHK
#ifdef MEM_CHKappAZ
extern bool MEM_AZ_Disable;
#endif
#endif

#ifdef TEST_NOTIFY_TIME
extern dword test_notify_us;
extern dword test__us;
#endif


#endif /* __APPBASE_H__ */
