/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public License
 as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version. 

 This library is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU Lesser General Public License for more details. 

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA
*/

#ifndef __MEDIA_IO_H__
#define __MEDIA_IO_H__    1

#include "tthread.h"

// stat
#define iost_init		1
#define iost_run		(1<<1)
#define iost_end		(1<<2)
#define iost_eof		(1<<2)
#define iost_empty		(1<<3)
#define iost_eof_empty		(iost_eof | iost_empty)

#define iost_err		(1<<4)
#define iost_OverFlow		(1<<5)

#define iost_io_in_sleep	(1<<16)
#define iost_io_out_sleep	(1<<17)
#define iost_io_sleep		(iost_io_in_sleep | iost_io_out_sleep)

/*____________________________ AV_IO Stream _____________________________*/
#define TV_VS_STYP	0x20764841
#define TV_VS_KTYP_V1_0 0xA55A5510

#define DatType_XRaw		0x40
#define DatType_VStream		0x01
#define DatType_AStream		0x02
#define DatType_UStream		0x03

#define DatType_Raw		0x41
#define DatType_VideoRaw	0x41
#define DatType_AudioRaw	0x42
#define DatType_URaw		0x43

#define DatType_TImage		0x80

#define iocf_compress_zlib	1

#define iocf_compress_k_diff	1

typedef struct {
  ic32  styp;		//0x00 "AHv " 0x20764841
  dword ktyp;		//0x04 0xA55A5510 = Version 1.0
  word  head_size;	//0x08 
  word  ex_off;		//0x0a default=0
  dword data_size;	//0x0c
  byte  typ;		//0x10 DatType_VStream,DatType_AStream,...
  byte  stat;		//0x11 
  byte  compress;	//0x12 0=none
  byte  compress_k;	//0x13 0=full-frame
} TAV_Stream;

#define AVStreamHead_size 0x14

#define AlgTAV_Stream ic32 styp; \
  dword ktyp; \
  word  head_size; \
  word  ex_off; \
  dword data_size; \
  byte  typ; \
  byte  stat; \
  byte  compress; \
  byte  compress_k;

extern void *TAV_Stream_alloc(word  head_size, dword data_size, int typ);

/*____________________________ Audio_IO _________________________________*/
typedef struct {
  AlgTAV_Stream
  int   rate;		//0x14 SampleRate 44100, 48000,...
  byte  channels;	//0x18 1=Mono, 2=Stereo
  byte  sample_bits;	//0x19 8,16,20,24
  byte  frame_byts;	//0x1a Frame Bytes 1,2,4
  byte  frame_typ;	//0x1b 0=PCM signed 1=PCM unsigned
  dword frames;		//0x1c
  dword lfn;		//0x20
  word  nc2c;		//0x24
  word  sum;		//0x26
  byte  data[];		//0x28
} TAudio_Stream;

#define AStreamHead_size 0x28

#define AlgTIOCAudioF dword data_size; \
  byte typ; \
  byte stat; \
  byte compress; \
  byte compress_k; \
  int   rate; \
  byte  channels; \
  byte  sample_bits; \
  byte  frame_byts; \
  byte  frame_typ; \
  dword frames;

typedef struct {
  AlgTIOCAudioF
} TIOCAudioF;

#define TIOCAudioF_size 20

typedef struct {
  dword stat;
  TIOCAudioF *iof;
  union {
    TAudio_Stream *as;
    byte *raw;
  } t;
} TAFrame;

/*____________________________ VIDEO_IO ________________________________*/
typedef struct {
  ic32  styp;		//0x00 "AHv " 0x20764841
  dword ktyp;		//0x04 0xA55A5510 = Version 1.0
  word  head_size;	//0x08 
  word  ex_off;		//0x0a default=0
  dword vdat_size;	//0x0c
  byte  typ;		//0x10 default=DatType_VStream
  byte  stat;		//0x11 
  byte  compress;	//0x12 0=none 1=zlib 2=jpeg
  byte  compress_k;	//0x13 0=full-frame 1=diff-frame
  byte  format;		//0x14 2=pxm_4b_24bpp 10=pxm_YUV420 11=pxm_YUV422
  byte  fps;		//0x15 0x03=fps_25 0x43=fps_50
  word  width;		//0x16
  word  height;		//0x18
  word  bytes_per_line; //0x1a
  word  xo;		//0x1c
  word  xl;		//0x1e
  word  yo;		//0x20
  word  yl;		//0x22
  byte  asp;		//0x24 0=1:1
  byte  nc25;		//0x25
  word  nc26;		//0x26
  dword lfn;		//0x28
  word  nc2c;		//0x2c
  word  sum;		//0x2e
  byte vdata[];		//0x30
} TVideo_Stream;

//#define Video_Stream_size 0x30
#define VStreamHead_size 0x30

#define AlgTIOCVideoF dword vdat_size; \
  byte typ; \
  byte stat; \
  byte compress; \
  byte compress_k; \
  byte format; \
  byte fps; \
  word width; \
  word height; \
  word bytes_per_line; \
  word xo, xl; \
  word yo, yl; \
  byte asp; \
  byte nc25; \
  word nc26; 


typedef struct {
  AlgTIOCVideoF
} TIOCVideoF;

#define TIOCF_size 28

typedef struct {
  dword stat;
  TIOCVideoF *iof;
  union {
    TVideo_Stream *vs;
    struct TImage *img;
    byte *raw;
  } t;
} TVFrame;


/*____________________________ MEDIA_IO ________________________________*/
#define friorstat_ret_s_Free	0x01
#define friorstat_ret_raw_Free	0x02

typedef struct {
  int thdMSG;	//thdMSG (0==Stream)
  int fu;
  int tag;

  union {
    TAFrame a;
    TVFrame v;
  } org;


  union {
    TAFrame a;
    TVFrame v;
  } ret;

  union {
    TAFrame a;
    TVFrame v;
  } dest;
  
  int buf_frames;
} TMediaFrame;

#define AlgTMedia_TTC AlgTTC \
  struct TMedia_TTC *next_get_link; \
  struct TMedia_TTC *next_set_link; \
  int stat; \
  int (*get_stat)(struct TMedia_TTC *tmttc); \
  int (*get_frame)(struct TMedia_TTC *tmttc, TMediaFrame *mf); \
  int (*On_frame)(struct TMedia_TTC *tmttc, TMediaFrame *mf, int *pfst); \
  int (*get_set_stat)(struct TMedia_TTC *tmttc); \
  int (*set_frame)(struct TMedia_TTC *tmttc, TMediaFrame *mf); \
  bool (*setup)(struct TMedia_TTC *tmttc, TThread_MSG *msg);

typedef struct TMedia_TTC {
  AlgTMedia_TTC
} TMedia_TTC;

typedef int (*TMTTC_GET_STAT)(TMedia_TTC *tmttc);
typedef int (*TMTTC_GET_FRAME)(TMedia_TTC *tmttc, TMediaFrame *mf);
typedef int (*TMTTC_ON_FRAME)(TMedia_TTC *tmttc, TMediaFrame *mf, int *pfst);
typedef bool (*TMTTC_SETUP)(TMedia_TTC *tmttc, TThread_MSG *msg);

extern int Media_TTC_get_stat(TMedia_TTC *tmttc);
extern int Media_TTC_get_frame(TMedia_TTC *tmttc, TMediaFrame *mf);


typedef struct TMedia_IO {
  AlgTTC
  TMedia_TTC *a_ttc;	//audio
  TMedia_TTC *v_ttc;	//video
  TMedia_TTC *u_ttc;

  void *owner; 
  int fu; 
  int stat;
  char *dev_name;

  bool (*start)(struct TMedia_IO *self); 
  void (*stop)(struct TMedia_IO *self); 
  void (*close_process)(struct TMedia_IO *self);
  int (*OnMSG)(struct TMedia_IO *self, TThread_MSG *msg);

} TMedia_IO;


extern bool Media_IO_ChkType(TMedia_IO *self);
extern void Media_IO_Stop(TMedia_IO *self);
extern bool Media_IO_Start(TMedia_IO *self);
extern void Media_IO_CloseProcess(TMedia_IO *self);

extern TMedia_IO *TMedia_FileRD_AHvs_Create(int fu);


#endif /* __MEDIA_IO_H__ */
