/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public License
 as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version. 

 This library is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU Lesser General Public License for more details. 

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA
*/

#ifndef __NET_IO_H__
#define __NET_IO_H__    1

#ifndef __FILEMSG_H__
 #include "filemsg.h"
#endif

#ifdef _OS_WIN32
 #include <winsock.h>
// #include <ws2tcpip.h>
#else
 #include <arpa/inet.h>
#endif

#define MAX_AHSERV_BUF		BUFSIZ

// *********************** socket ****************************
typedef struct THWaddr {
  byte mac[8];
//  int  len;	//6=mac48 8=mac64
} THWaddr;

#ifndef _OS_WIN32
extern int make_named_socket(char *filename);

//HWname eth0,..    or use /sys/class/net/eth0/address
extern bool SocketGetHWaddr(THWaddr *hwaddr, const char *HWname);

typedef struct TIFaddrIPv4 {
  union {
    struct sockaddr	addr;
    struct sockaddr_in	addr_in;
  };
  union {
    struct sockaddr	broadaddr;
    struct sockaddr_in	broadaddr_in;
  };
  union {
    struct sockaddr	netmask;
    struct sockaddr_in	netmask_in;
  };
//  struct sockaddr dstaddr;
} TIFaddrIPv4;

bool SocketGetIFaddrIPv4(TIFaddrIPv4 *ifaddr, const char *HWname);

#endif

#if (defined _OS_WIN32 && !defined _WINSOCK2_H)
typedef int socklen_t;
#define INET6_ADDRSTRLEN 16
#endif

typedef struct {
  union {
    struct sockaddr		addr;
    struct sockaddr_in		addr_in;
#if (!defined _OS_WIN32 || defined _WINSOCK2_H)
    struct sockaddr_in6		addr_in6;
    struct sockaddr_storage	addr_storage;
#endif
  };
} TSockAddrStorage;

typedef struct {
  union {
    struct sockaddr	addr;
    struct sockaddr_in	addr_in;
  };
} TSockAddr;

extern int  SocketOpen(int domain, int itype, int protocol);
#ifdef _OS_WIN32
extern void SocketClose(int sock);
#else  /* _OS_WIN32 */
#define SocketClose(a) file_close(a, NULL)
#endif /* _OS_WIN32 */
extern int SocketGetLastError();
extern const char *SocketGetLastError_cpc();
extern const char *SocketGetIP_cpc(struct sockaddr *sa);

#ifdef _OS_WIN32
extern Tfd_list *SocketAddPipeProc(int fd, int fu, TOnPipe OnPipe, void *obj);
#endif

// *********************** AHserv ****************************

 /* TdServHead.bef  */
#define hSHb_Ok              3
#define hSHb_ERR             4
#define hSHb_SET_Login       8  /* ServToClient = Error */
#define hSHb_StrgMSG		10
#define hSHb_VerStr		11

#define hSHb_NOP             255

/* -- OK --     sh.HeadArt = hServHead sh.bef = hSHb_Ok 
FTP:
  sh.m --> FTP server return codes
  sh.fu == 0 --> OK
  sh.fu == 1 --> Error
  sh.strg == scSHsg_MSG  --> msg string in sh.dat
*/
#define hSHfuOk_ERROR		1

#define  scSHsg_MSG           	 18

 /* TdServHead.strg   (TdServHead.bef == hSHb_StrgMSG)  */
#define hSHStM_Ok		0x00
#define hSHStM_Connect		0x01
#define hSHStM_user		0x02
#define hSHStM_passwd		0x03
#define hSHStM_ls		0x04
#define hSHStM_WrOpen		0x10
#define hSHStM_RdOpen		0x11
// ERROR = hSHStM_Connect,hSHStM_user,... + hSHStM_Error
#define hSHStM_xOk		0x40
#define hSHStM_Error		0x80

typedef struct TdServFileStat {
  AlgTdServHead
  Tstat64 st;
} TdServFileStat;
#define TdServFile TdServFileStat

typedef struct TdServFile_00 {
  AlgTdServHead
  Tstat64 st[0];
} TdServFile_00;

/* ________________________ TdData _________________________________ */
#define hSHbd_Dat	0
#define hSHbd_FileSize	1
 /* TdData.fu   (TdServHead.bef == hSHbd_Dat) */
#define hSHfu_EOF	1
#define hSHfu_send	(1<<1)
#define hSHfu_onlyInfo	(1<<2)
#define hSHfu_EOF_ERROR	(1<<3)

#define sh_DataSize	D32B.i64[0]
#define sh_DataNowSize	D32B.i64[1]
#define sh_LoadTime_us	D32B.i64[2]
#define sh_ErrorNr	D32B.i[6]
typedef struct TdData {
  AlgTdServHead
  union {
	char c[32];
	byte b[32];
	short i16[16];
	int i[8];
	dword dw[8];
	float f[8];
	int64_t i64[4];
	double f64[4];
  } D32B;

  char dat[1];
} TdData;

//*********************** pipe ****************************
cHIDDEN bool GetSyncPipeServHead(TLineBuf *tpb, TdServHead **sh, int *tsf);


#define ioErr_Unknown_host	1000
#define ioErr_socket_failed	1001
#define ioErr_connection_failed	1002
#define ioErr_getsockname	1003
#define ioErr_ServClosCon	1004
#define ioErr_Internal		1005
#define ioErr_setsockopt	1006
//#define ioErr_bind		1007
#define ioErr_not_connect	1008
#define ioErr_DecodeCmd		1009
//#define ioErr_Syntax		1010
#define ioErr_NoConnMax		1049

#define ioErr_FileNotFound	1100
#define ioErr_PathNotFound	1101
//in tty mode no output err >= 2000
#define ioErr_NoLogin		2000

#define AlgTioMsg    int lg; \
  char HeadArt; \
  char bef; \
  char fu; \
  char Strg; \
  int pid; \
  int ID; \
  int UserID; \
  int UserTag; \
  unsigned short int m;

typedef struct TioMsg {
  AlgTioMsg
  char Dat[];
} TioMsg;


/* ftpappclient.c */
/* TFtpClientNextCMD.nxfu */
#define fcnxfu_EndOk		1
#define fcnxfu_SkipOk		(1<<1)
#define fcnxfu_ErrorIsOk	(1<<2)

typedef struct TFtpClientNextCMD {
  struct TFtpClientNextCMD *next;
  void (*OnFileMsg)(struct TFileMsg *fmsg); //NULL use TFtpClientCMD.OnFileMsg
  char *bef;
  char *cmd;
  dword nxfu;
  int stat;  //ftpcmdmk_Get,ftpcmdmk_Put
  dword akt_fkt;  //facfkt_STAT
} TFtpClientNextCMD;

/* TFtpClientCMD.stat */
#define ftpcmdmk_send		1
#define ftpcmdmk_OK		(1<<1)
#define ftpcmdmk_Get		(1<<4)
#define ftpcmdmk_Put		(1<<5)
#define ftpcmdmk_End		(1<<6)
#define ftpcmdmk_Stop		(1<<7)
#define ftpcmdmk_StopIsSend	(1<<8)

/* OnRxD_End EndCode */
#define facEndCode_OK		1
//#define facEndCode_ERROR	2
#define facCMD_Free		2
#define facSendPipe		3
#define facEndCode_MASK		0xff

typedef struct TFtpClientCMD {
  AlgTNet
  struct TFtpClientCMD *next;
  struct TFtpAppClient *ftpac;
  TFtpClientNextCMD *fcnx;
  char *bef;
  char *cmd;
  int stat;
  dword nxfu;
  dword akt_fkt;  //facfkt_STAT
} TFtpClientCMD;

#define ftpacmk_SendOpen	1
#define ftpacmk_Open		(1<<1)
#define ftpacmk_TimeEnd		(1<<2)
#define ftpacmk_ErrorOpen	(1<<3)
#define ftpacmk_GoFree		(1<<4)
#define ftpacmk_ClientClose	(1<<5)
#define ftpacmk_PipeRead	(1<<6)

/* TFtpAppClient.fkt */
#define facfkt_CHMOD	1
#define facfkt_UTIME	(1<<1)
#define facfkt_CHOWN	(1<<2)
#define facfkt_STAT	(1<<8)  /* ftp: */


typedef struct TFtpAppClient {
  AlgTNet
  struct TFtpAppClient *next;
  struct TFtpAppClient **ppfac; //root FtpAppClient
  int pid;
  Texec_pipe *expip;
  int stat;
  dword us;
  int dofree_sec;
  TFtpClientCMD *ftpcmd;
  dword fkt;
  char *VerStr;
  int VerNr;
} TFtpAppClient;

cHIDDEN int TFtpAppClient_TimeOut;

cHIDDEN TFtpAppClient *TFtpAppClient_get(TglobPath *gp); //, int facfu);
cHIDDEN TFtpClientCMD *TFtpClientCMD_Create(const char *bef, const char *cmd, TglobPath *gp);
cHIDDEN bool TFtpAppClient_CMD_add(TFtpAppClient *self, TFtpClientCMD *ftpcmd);

cHIDDEN bool TFtpClientCMD_Stop(TFtpClientCMD *self);

cHIDDEN dword TFtpAppClient_get_fkt(TglobPath *gp);

//TFtpAppClient *TFtpAppClient_Create(char *url);
//extern int TFtpAppClient_Akt();



extern Tstat64 *ftp_stat_gp(TglobPath *gp, TNetErrRet *ErrRet, int fu); //ACHTUNG use AppHandleMessage() !!!

extern bool ftp_unlink_gp(TglobPath *gp, TNetErrRet *ErrRet); //ACHTUNG use AppHandleMessage() !!!
extern bool ftp_rmdir_gp(TglobPath *gp, TNetErrRet *ErrRet);  //ACHTUNG use AppHandleMessage() !!!
extern bool ftp_chmod_gp(TglobPath *gp, TNetErrRet *ErrRet, int mode); //ACHTUNG use AppHandleMessage() !!!
extern bool ftp_utime_gp(TglobPath *gp, TNetErrRet *ErrRet, time_t mtime); //ACHTUNG use AppHandleMessage() !!!
extern bool ftp_mkdir_gp(TglobPath *gp, TNetErrRet *ErrRet, int mode); //ACHTUNG use AppHandleMessage() !!!
extern bool ftp_mkdir_gp_NoReScan(TglobPath *gp, TNetErrRet *ErrRet, int mode); //ACHTUNG use AppHandleMessage() !!!



#endif /* __NET_IO_H__ */
