/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public License
 as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version. 

 This library is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU Lesser General Public License for more details. 

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA
*/

#ifndef	__TTHREAD_H__
#define	__TTHREAD_H__    1

#ifdef _OS_WIN32
#include <winbase.h>
#else /* ! _OS_WIN32 */
#include <pthread.h>
#endif /* ! _OS_WIN32 */

#define thdMSG_OnlyEvent	0
#define thdMSG_CloseProcess	1
#define thdMSG_Init		4
#define thdMSG_Open		5
#define thdMSG_Close		6
#define thdMSG_Start		7
#define thdMSG_Stop		8
#define thdMSG_Err		9
#define thdMSG_Setup_Data	10
#define thdMSG_Status		11
#define thdMSG_EOF		12
#define thdMSG_Result		13

#define thdMSG_XCopy		32
#define thdMSG_XCopy_RunStat	33

#define thdMSG_Mask		0x00ffffff
#define thdMSG_rv		0x01000000
#define thdMSG_rv_mmMSG_free	0x03000000
#define thdMSG_NoSyncEv		0x04000000

#define uthd_t_none	0
#define uthd_t_int	1
#define uthd_t_short	2
#define uthd_t_char	3
#define uthd_t_dword	4
#define uthd_t_word	5
#define uthd_t_byte	6
#define uthd_t_pc	7
#define uthd_t_p	8
#define uthd_t_ttc	9
#define uthd_t_Mask	0x0f
#define uthd_t_MemFree	0x80

typedef struct {
  byte type;
  byte m1;
  byte m2;
  byte m3;
  union {
    int i;
    short s[2];
    char c[4];
    dword dw;
    word w[2];
    byte b[4];
    char *pc;
    void *p;
    TTC0 *ttc;
  };
} TuthdMSG;

typedef struct {
  int thdMSG;
  int tag;
  TuthdMSG u;
} TThread_MSG;

void res_thdMSG(TThread_MSG *msg);

typedef struct TThread_pv *Tthd_pv;

#define AlgTThread AlgTTC; \
  TThread_MSG msg; \
  int (*OnMsgEvent)(struct TThread *thd, TThread_MSG *msg, TThread_MSG *ret_msg); \
  int (*DoExecute)(struct TThread *thd); \
  FileHandle fd; \
  struct TThread *app_private_thd; \
  Tthd_pv thd_pv;

typedef struct TThread {
  AlgTThread
} TThread;

TThread *TThread_Create();
bool TThread_Start(TThread *self);
void TThread_Stop(TThread *self);

void REGP1 TThread_lock(TThread *self);
void REGP1 TThread_unlock(TThread *self);

/* ------------- Msg App to Thread -------------------------------- */
void app2thd_pMSG(TThread *self, int send_sync, TThread_MSG *msg);
void app2thd_MSG(TThread *self, int send_sync, int thdMSG, int tag);

/* ------------- Msg Thread to App -------------------------------- */
void thd2app_pMSG(TThread *self, int send_sync, TThread_MSG *msg);
void thd2app_MSG(TThread *self, int send_sync, int thdMSG, int tag);
int thd2app_MSG_Wait(TThread *self, int thdMSG, int tag, TThread_MSG *ret_msg);

/* Result */
#define thdWait_TimeOut		0
#define thdWait_msg		1
#define thdWait_fd		2
int TThread_Wait(TThread *self, dword TimeOut_us);

/*_____________________________ Sync ____________________________________*/
void TThread_Sync_app();
void TThread_Sync_pth(TThread *self);

#endif /* __TTHREAD_H__ */
