/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public License
 as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version. 

 This library is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 See the GNU Lesser General Public License for more details. 

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA
*/

#ifndef	__XCOPY_H__
#define	__XCOPY_H__    1

#ifndef __SYSBASE_H__
 #include "sysbase.h"
#endif

#ifndef __SYSBASEIO_H__
 #include "sysbaseio.h"
#endif


//*********************** XCopy ***********************************
#define xpyMsg_ovwDir		1
#define xpyMsg_ovwFile		2

#define xpyErr_Alg		0x1000
#define xpyErr_Msg		0x2000

#define xpyErr_SrcNotExist	1
#define xpyErr_DestNotExist	2
#define xpyErr_OverWriteType	3
#define xpyErr_OvwTypeDestDir	4
#define xpyErr_SameFile		5
#define xpyErr_DelFile		6
#define xpyErr_OpenRD		7
#define xpyErr_OpenWR		8
#define xpyErr_MkDir		9
#define xpyErr_MkLink		10
#define xpyErr_MkNod		11
#define xpyErr_Type		12

#define xpyMsg_src		0x10000
#define xpyMsg_dest		0x20000

#define xpyMsg_NrMask		0x00ff

#define xcySkip_OldFile		1
#define xcySkip_ovwDir		(1<<1)
#define xcySkip_ovwFile		(1<<2)
#define xcySkip_Chmod		(1<<3)
#define xcySkip_Chown		(1<<4)
#define xcySkip_Utime		(1<<5)
#define xcySkip_Preserv		(xcySkip_Chmod | xcySkip_Chown | xcySkip_Utime)
#define xcySkip_Dir		(1<<6)	 /* used in cpwdMode_cut / Del */
#define xcySkip_DestNotExist	(1<<7)

#define xcyForce_ovwDir		(1<<9)
//#define xcyForce_delDir		(1<<10)   /* used in cpwdMode_cut */
#define xcyForce_ovwFile	(1<<11)
#define xcyForce_ovwType	(1<<12)
#define xcyForce_followlinks	(1<<13)

#define xcySkip_SrcNotExist	(1<<16)
#define xcySkip_errSameFile	(1<<17)
#define xcySkip_errOpenRD	(1<<18)
#define xcySkip_errOpenWR	(1<<19)
#define xcySkip_errDelFile	(1<<20)
#define xcySkip_OverWriteType	(1<<21)
#define xcySkip_OvwTypeDesrDir	(1<<22)
#define xcySkip_errMkDir	(1<<23)
#define xcySkip_errMkLink	(1<<24)
#define xcySkip_errMkNod	(1<<25)
#define xcySkip_errChmod	(1<<26)
#define xcySkip_errChown	(1<<27)
#define xcySkip_errUtime	(1<<28)
#define xcySkip_errType		(1<<29)


typedef struct TXCopy_wd *Txpy_wd;

#define xcyRetEv_CANCEL  0
#define xcyRetEv_SKIP	 0x01
#define xcyRetEv_REPEAT	 0x02
#define xcyRetEv_BefMask 0x07
#define xcyRetEv_ALL	 0x08

typedef int (*ThCopyEv)(Txpy_wd wd, int xcyMsg);

#define xcyFU_INPUT		0
#define xcyFU_OUTPUT		1

typedef struct ThXCopy {
  bool (*ChkType)(TglobPath *gp, bool dest);
  struct TXCopy_ld *(*create)(int fu, Txpy_wd wd, TglobPath *gp);
} ThXCopy;

extern Txpy_wd XCopy_Create(void *owner, ThCopyEv hcyEv,
			TglobPath *gp_src, TglobPath *gp_dest, int skip);

extern bool TXCopy_SetSrc(Txpy_wd wd, TglobPath *gp_src);

extern ThXCopy *XCopy_GetHande(TglobPath *gp, bool dest);

#define xcySFU_Preserv		0x01
#define xcySFU_Del		0x02
#define xcySFU_befmask		0xff
extern bool XCopy_Start(Txpy_wd wd, Tstat64 *st_src, int xcySFU);
extern void XCopy_Stop(Txpy_wd wd);

extern bool XCopy_ChkWait(Txpy_wd wd);
extern bool XCopy_ChkErr(Txpy_wd wd);

#define cpwdMode_cut	1
extern void XCopy_SetMode(Txpy_wd wd, dword mode);
extern dword XCopy_GetMode(Txpy_wd wd);

extern void XCopy_SetSkip(Txpy_wd wd, dword skip);
extern dword XCopy_GetSkip(Txpy_wd wd);
extern void XCopy_MsgSetSkip(Txpy_wd wd, int xcyMsg, dword skip, bool all);

extern const char *XCopy_GetErr_pc(Txpy_wd wd, int xcyMsg);
extern void XCopy_NameToPath(Txpy_wd wd);
extern void XCopy_PathToName(Txpy_wd wd);
extern void XCopy_ChangeName(Txpy_wd wd, const char *fname);
extern void XCopy_ChangeDestName(Txpy_wd wd, const char *fname);
extern bool XCopy_ChkDestDirDouble(Txpy_wd wd);

extern Tstat64 *XCopy_GetSrcStat(Txpy_wd wd);    //stat is dup
extern Tstat64 *XCopy_GetDestStat(Txpy_wd wd);   //stat is dup

extern TglobPath *XCopy_GetSrc_gp(Txpy_wd wd);  //gp is dup
extern TglobPath *XCopy_GetDest_gp(Txpy_wd wd);  //gp is dup

extern char *XCopy_GetSrc_gp_fn(Txpy_wd wd, int gptfu);
extern char *XCopy_GetDest_gp_fn(Txpy_wd wd, int gptfu);

extern void *XCopy_GetOwner(Txpy_wd wd);

extern const char *XCopy_GetMsg_pc(Txpy_wd wd, int xcyMsg);

typedef struct {
  int FileCount;
  uint64_t GesDataSize;
  uint64_t GesDataTime;
  uint64_t DataSize;
  uint64_t DataNowSize;
  uint64_t DataNowTime;
} TXCopyStats;

extern bool XCopy_GetStats(Txpy_wd wd, TXCopyStats *xcyStats);
extern bool XCopy_DestDirIsCreate(Txpy_wd wd);


extern void ThXCopy_Reg(int n, ...);
extern ThXCopy hXCopy_ld_File;
extern ThXCopy hXCopy_ld_Ftp;

#endif /* __XCOPY_H__ */
