/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"

Pixmap XCreateBitmapFromData(Display *display, Drawable d, const char *data,
		unsigned int width, unsigned int height) {
// printf("%s(%d): XCreateBitmapFromData: d=0x%x %d %d\n", FL, (int)d, width, height);

  Pixmap res;
  TBITMAP_Dat *bmDat;
  char *dest0;

	res = XCreatePixmap(display, d, width, height, 1);
	bmDat = Get_BITMAP_Dat(res);
	if (!bmDat || (bmDat->ui_type != sizeof(DIBSECTION)) || !data) return 0;

	dest0 = bmDat->ui.DIBS.dsBm.bmBits;

  int ix, iy, xw;
  const char *src;
  char *dest;
	xw = (width + 7) >> 3;
	src = data;
	dest0 += bmDat->bytes_per_line * (height - 1);
	iy = height;
	while (iy) {
		dest = dest0;
		ix = xw;
		while (ix) {
		    *dest = ~(((*src & 0x01) << 7) | ((*src & 0x02) << 5) |
			    ((*src & 0x04) << 3) | ((*src & 0x08) << 1) |
			    ((*src & 0x10) >> 1) | ((*src & 0x20) >> 3) |
			    ((*src & 0x40) >> 5) | ((*src & 0x80) >> 7));
		    dest++; src++;
		    ix--;
		}
		dest0 -= bmDat->bytes_per_line;
		iy--;
	}
	return res;
}

Pixmap XCreateBitmapFromData_W32noDIB(Display *display, Drawable d, const char *data,
		unsigned int width, unsigned int height) {

  HBITMAP hbitmap;
  TBITMAP_Dat *bmDat;
  int m, bytes_per_line;
  char *imgdat;
  int ix, iy, xw;
  char *dest0;

	bytes_per_line = Get_bytes_per_line(1, width);
	imgdat = xcalloc(bytes_per_line * height);

  const char *src;
  char *dest;
	dest0 = imgdat;
	xw = (width + 7) >> 3;
	src = data;
	iy = height;
	while (iy) {
		dest = dest0;
		ix = xw;
		while (ix) {
		    *dest = ~(((*src & 0x01) << 7) | ((*src & 0x02) << 5) |
			    ((*src & 0x04) << 3) | ((*src & 0x08) << 1) |
			    ((*src & 0x10) >> 1) | ((*src & 0x20) >> 3) |
			    ((*src & 0x40) >> 5) | ((*src & 0x80) >> 7));
		    dest++; src++;
		    ix--;
		}
		dest0 += bytes_per_line;
		iy--;
	}

	hbitmap = CreateBitmap(width, height, 1, 1, imgdat);
	if (!hbitmap) goto ende;

	bmDat = xcalloc(sizeof(TBITMAP_Dat));
	bmDat->hbitmap = hbitmap;
	bmDat->depth = 1;

	m = GetObject(hbitmap, sizeof(DIBSECTION), &bmDat->ui.DIBS);
	if (!m) {
	    xfree(bmDat);
	    goto ende;
	}
	bmDat->ui_type = m;
	bmDat->bytes_per_line = bytes_per_line;
	Add_BITMAP_Dat(bmDat);
 
ende:
	xfree(imgdat);
	return hbitmap;
}
