/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"

#define ft_normal_Roman  0x01
#define ft_normal_Italic 0x02
#define ft_bold_Roman    0x04
#define ft_bold_Italic   0x08

//#define ft__normal 0x03
#define ft__bold   0x0c
//#define ft__Roman  0x05
#define ft__Italic 0x0a

#define ft_ALL 0x0f

typedef struct TFontEnumFontCB_nx {
  struct TFontEnumFontCB_nx *next;
  LOGFONTW lf;
  int FontType;
  int ft;
} TFontEnumFontCB_nx;

typedef struct {
  TFontEnumFontCB_nx *nx;
  HDC hdc;
  int maskFontType;
  int ft;
  int find_H;
  int find_W;
  int unicode;  //0=all 1=ASCII 2=UNI

  int maxNames;

  char **Result;
  int ResArrLg;
  int ResCount;
} TFontEnumFontCB;

//#define e4st_tmOK 1

#define AlgFontEnum4  int ft; \
  HDC hdc; \
  int Height; \
  int Width; \
  int FontType; \
  LOGFONTW lf; \
  TEXTMETRICW tm_nR; \
  TEXTMETRICW tm_nI; \
  TEXTMETRICW tm_bR; \
  TEXTMETRICW tm_bI;

typedef struct {
  AlgFontEnum4
} TFontEnum4;

typedef struct {
  TFontEnumFontCB *fefCB;
  TFontEnumFontCB_nx *nx;
  int minH;
  int maxH;
  int AktH;
  bool allH;
  bool Font_Ok;
  bool Bold;
  bool Kursiv;
  AlgFontEnum4
} TFontEnumAddDat;


//-adobe-times-medium-r-normal--18-180-75-75-p-94-iso8859-1
//-misc-fixed-bold-r-normal-*-13-120-75-75-c-70-iso10646-1
// 0    1     2    3 4      5 6  7   8  9  1 1  1        1
//                                         0 1  2        3

static const char *CharSet_W32toX(int CharSet) {
	switch (CharSet) {
	case SYMBOL_CHARSET:  return "SYMBOL";	//2
	case MAC_CHARSET:     return "MAC";	//77
//	case SHIFTJIS_CHARSET:return "*";		//128
//	case HANGUL_CHARSET:return "*";		//129
//	case JOHAB_CHARSET:return "*";		//130
//	case GB2312_CHARSET:return "*";		//134
//	case CHINESEBIG5_CHARSET:return "*";		//136
	case GREEK_CHARSET:   return "7";		//161
	case TURKISH_CHARSET: return "9";		//162
//	case VIETNAMESE_CHARSET:return "*";		//163
	case HEBREW_CHARSET:  return "8";		//177
//	case ARABIC_CHARSET:  return "8";		//178
	case BALTIC_CHARSET:  return "4";		//186
	case RUSSIAN_CHARSET: return "5";		//204
	case THAI_CHARSET:    return "11";	//222
	case EASTEUROPE_CHARSET: return "2";	//238
	case OEM_CHARSET:     return "OEM";	//255
	}
	return "1";
}

static int CharSet_XtoW32(const char *cs) {
  int m;
	if (!cs || !*cs || (*cs == '*'))   return DEFAULT_CHARSET;
	if (!pccasecmpASCII(cs, "SYMBOL")) return SYMBOL_CHARSET;
	if (!pccasecmpASCII(cs, "OEM"))    return OEM_CHARSET;
	if (!pccasecmpASCII(cs, "MAC"))    return MAC_CHARSET;

	m = pctoint(cs);
	switch (m) {
	case 2: return EASTEUROPE_CHARSET;
	case 4: return BALTIC_CHARSET;
	case 5: return RUSSIAN_CHARSET;
	case 7: return GREEK_CHARSET;
	case 8: return HEBREW_CHARSET;
	case 9: return TURKISH_CHARSET;
	case 11: return THAI_CHARSET;
	}
	return ANSI_CHARSET;
}

static bool SetFont_tm(TFontEnum4 *fe, TEXTMETRICW *tm) {
  HDC hdc;
  HFONT hf, hfo;
  LOGFONTW *lf = &fe->lf;
  bool res = True;

	hdc = fe->hdc;
	if (!hdc) hdc = GetDC(0);
	Fill0(tm, sizeof(TEXTMETRICW));
	hf = CreateFontIndirectW(lf);
	if (!hf) return False;
	hfo = SelectObject(hdc, hf);
	if (!GetTextMetricsW(hdc, tm)) res = False;
	SelectObject(hdc, hfo);
	DeleteObject(hf);
	if (!fe->hdc) ReleaseDC(0, hdc);
#if 0
 char tmp[256];

	WCharsToUtf8(tmp, 256, lf->lfFaceName, pwclen(lf->lfFaceName), UC_force_ASCII);
 printf("SetFont_tm: h=%-2d w=%d %-3d --> h=%-2d il=%d w=%d %-3d  \"%s\"\n",
	(int)lf->lfHeight,
	(int)lf->lfWidth,
	(int)lf->lfWeight, 
	(int)tm->tmHeight,
	(int)tm->tmInternalLeading,
	(int)tm->tmAveCharWidth,
	(int)tm->tmWeight, 
	tmp);
#endif
	return res;
}

static bool SetFont_tm_ft(TFontEnum4 *fe, TEXTMETRICW *tm, int ft) {
  bool res = SetFont_tm(fe, tm);
	if (res && (ft & ft__bold))    res = tm->tmWeight > FW_MEDIUM;
	if (res && !(ft & ft__bold))   res = tm->tmWeight <= FW_MEDIUM;
	if (res && (ft & ft__Italic))  res = tm->tmItalic != 0;
	if (res && !(ft & ft__Italic)) res = tm->tmItalic == 0;
	return res;
}

#if 0
static void e4_print(TEXTMETRICW *tm, char *pc, bool ok) {
	if (ok) {
 printf("%s h=%-2ld w=%-2ld %3ld | ", pc, tm->tmHeight - tm->tmInternalLeading, tm->tmAveCharWidth, tm->tmWeight);
	} else {
 printf("%s h=-- w=-- --- | ", pc);
	}
}
#endif
static void SetFontEnum4(TFontEnum4 *fe, int ft) {
  HDC hdc = fe->hdc;

	if (!fe->hdc) fe->hdc = GetDC(0);
	fe->ft = 0;
	fe->lf.lfWeight = FW_NORMAL;
	fe->lf.lfItalic = 0;
	if (ft & ft_normal_Roman) {
	    if (SetFont_tm_ft(fe, &fe->tm_nR, ft_normal_Roman)) fe->ft |= ft_normal_Roman;
	}
	if (ft & ft_normal_Italic) {
	    fe->lf.lfItalic = 1;
	    if (SetFont_tm_ft(fe, &fe->tm_nI, ft_normal_Italic)) fe->ft |= ft_normal_Italic;
	}
	fe->lf.lfWeight = FW_BOLD;
	fe->lf.lfItalic = 0;
	if (ft & ft_bold_Roman) {
	    if (SetFont_tm_ft(fe, &fe->tm_bR, ft_bold_Roman)) fe->ft |= ft_bold_Roman;
	}
	if (ft & ft_bold_Italic) {
	    fe->lf.lfItalic = 1;
	    if (SetFont_tm_ft(fe, &fe->tm_bI, ft_bold_Italic)) fe->ft |= ft_bold_Italic;
	}
#if 0
 printf("0x%x h=%-2ld w=%-2ld | ", fe->ft, fe->lf.lfHeight, fe->lf.lfWidth);
 e4_print(&fe->tm_nR, "nR", (fe->ft & ft_normal_Roman) != 0);
 e4_print(&fe->tm_nI, "nI", (fe->ft & ft_normal_Italic) != 0);
 e4_print(&fe->tm_bR, "bR", (fe->ft & ft_bold_Roman) != 0);
 e4_print(&fe->tm_bI, "bI", (fe->ft & ft_bold_Italic) != 0);
 printf("\n");
#endif
	if (!hdc) ReleaseDC(0, fe->hdc);
	fe->hdc = hdc;
}

static int GetFontEnum4_x(TFontEnum4 *fe) {
  TEXTMETRICW *tm = NULL;
	if (fe->ft & ft_normal_Roman) tm = &fe->tm_nR; else
	if (fe->ft & ft_bold_Roman)   tm = &fe->tm_bR; else
	if (fe->ft & ft_normal_Italic) tm = &fe->tm_nI; else
	if (fe->ft & ft_bold_Italic)  tm = &fe->tm_bI;
	if (!tm) return -7;
	return tm->tmHeight - tm->tmInternalLeading;
}

static int CALLBACK EnumFontCallBack(ENUMLOGFONTEXW *lpelfe, NEWTEXTMETRICEXW *lpntme, int FontType, LPARAM lParam) { 
  TFontEnumFontCB *fefCB = (TFontEnumFontCB*)lParam;
  TFontEnumFontCB_nx *nx;

	if (!(FontType & fefCB->maskFontType)) return 1;
	if ((fefCB->unicode == 1) && (lpntme->ntmTm.tmLastChar > 255)) return 1;
	if ((fefCB->unicode == 2) && (lpntme->ntmTm.tmLastChar <= 255)) return 1;

#if 0
 char tmp[256];

//	WCharsToUtf8(tmp, 256, lpelfe->elfFullName, pwclen(lpelfe->elfFullName), UC_force_ASCII);
	WCharsToUtf8(tmp, 256, lpelfe->elfLogFont.lfFaceName, pwclen(lpelfe->elfLogFont.lfFaceName), UC_force_ASCII);
 printf("FontCallBack: %d %-16s ", FontType, tmp);
	WCharsToUtf8(tmp, 256, lpelfe->elfStyle, pwclen(lpelfe->elfStyle), UC_force_ASCII);
 printf("%-13s ", tmp);
	WCharsToUtf8(tmp, 256, lpelfe->elfScript, pwclen(lpelfe->elfScript), UC_force_ASCII);
 printf("%-16s ", tmp);

 printf(" h=%-2d w=%-2d wx=%-2d  lh=%-2d w=%d %-3d pf=0x%x cs=%d\n",
	(int)lpntme->ntmTm.tmHeight,
//	(int)lpntme->ntmTm.tmAscent,
//	(int)lpntme->ntmTm.tmDescent,
	(int)lpntme->ntmTm.tmAveCharWidth,
	(int)lpntme->ntmTm.tmMaxCharWidth,
	(int)lpelfe->elfLogFont.lfHeight,
	(int)lpelfe->elfLogFont.lfWeight,
	(int)lpelfe->elfLogFont.lfItalic,
	lpelfe->elfLogFont.lfPitchAndFamily,
	lpelfe->elfLogFont.lfCharSet);
#endif

	nx = fefCB->nx;
	while (nx && ((nx->FontType != FontType) ||
		     (nx->lf.lfCharSet != lpelfe->elfLogFont.lfCharSet) ||
		     !pwccmp(nx->lf.lfFaceName, lpelfe->elfLogFont.lfFaceName))) {
		nx = nx->next;
	}

	if (!nx) {
	    nx = xmalloc(sizeof(TFontEnumFontCB_nx));
	    nx->next = fefCB->nx;
	    fefCB->nx = nx;
	    pnmove(&nx->lf, &lpelfe->elfLogFont, sizeof(LOGFONTW));
	    nx->FontType = FontType;
	}

	if (nx) {
	    if (lpelfe->elfLogFont.lfWeight > FW_NORMAL) {
		if (lpelfe->elfLogFont.lfItalic) {
		    nx->ft |= ft_bold_Italic;
		} else {
		    nx->ft |= ft_bold_Roman;
		}
	    } else {
		if (lpelfe->elfLogFont.lfItalic) {
		    nx->ft |= ft_normal_Italic;
		} else {
		    nx->ft |= ft_normal_Roman;
		}
	    }
	}

	return 1;
} 

#define MM_FONT_MIN 11
#define MM_FONT_MAX 40

static char *GetX11_Fontstr(TFontEnum4 *fe, int ft) {
  TEXTMETRICW *tm = NULL;
  int i6, i11;
  const char *s0, *s2, *s3, *s4, *s8, *s10, *s12, *s13;
  char s1[80];
  char buf[256];

	if (ft & ft_normal_Roman) {
	    if (fe->ft & ft_normal_Roman) tm = &fe->tm_nR;
	} else if (ft & ft_normal_Italic) {
	    if (fe->ft & ft_normal_Italic) tm = &fe->tm_nI;
	} else if (ft & ft_bold_Roman) {
	    if (fe->ft & ft_bold_Roman) tm = &fe->tm_bR;
	} else if (ft & ft_bold_Italic) {
	    if (fe->ft & ft_bold_Italic) tm = &fe->tm_bI;
	}
	if (!tm) return NULL;

	if (!s0) s0 = "*";
	if (!s8) s8 = "*";
	WCharsToUtf8(s1, sizeof(s1), fe->lf.lfFaceName, pwclen(fe->lf.lfFaceName), UC_force_ASCII);

	s2 = "medium";
	if (tm->tmWeight > FW_MEDIUM) s2 = "bold";
	s3 = "r";
	if (tm->tmItalic) s3 = "o";

	s4 = "normal";

	i6 = 0;
	i11 = 0;
	if (fe->Height > 0) i6 = fe->Height;

	s8 = "0";
	if (fe->FontType & RASTER_FONTTYPE) {
	    s0 = "raster";
	    s8 = "96";
//	    if (fe->FontType & RASTER_FONTTYPE) {
		i11 = tm->tmAveCharWidth * 10;
//	    }
	} else if (fe->FontType & DEVICE_FONTTYPE) {
	    s0 = "Device";
	} else if (fe->FontType & TRUETYPE_FONTTYPE) {
	    s0 = "TrueType";
	} else {
	    s0 = "*";
	}

	s10 = "p";
	if ((fe->lf.lfPitchAndFamily & 0x03) == FIXED_PITCH) {
	    s10 = "m";
	    int m = 0;
	    if (fe->ft & ft_normal_Roman) m = fe->tm_nR.tmAveCharWidth;
	    if (fe->ft & ft_normal_Italic) {
		if (!m) m = fe->tm_nI.tmAveCharWidth;
		else if (m != fe->tm_nI.tmAveCharWidth) m = -1;
	    }
	    if (fe->ft & ft_bold_Roman) {
		if (!m) m = fe->tm_bR.tmAveCharWidth;
		else if (m != fe->tm_bR.tmAveCharWidth) m = -1;
	    }
	    if (fe->ft & ft_bold_Italic) {
		if (!m) m = fe->tm_bI.tmAveCharWidth;
		else if (m != fe->tm_bI.tmAveCharWidth) m = -1;
	    }
	    if (m > 0) s4 = "fixed";
	}

	if (tm->tmLastChar > 255) {
	    s12 = "iso10646";
	} else {
	    s12 = "iso8859";
	}
	s13 = CharSet_W32toX(fe->lf.lfCharSet);

	snprintf(buf, sizeof(buf), 
		"-%s-%s-%s-%s-%s--%d-%d-%s-%s-%s-%i-%s-%s",
		s0,s1,s2,s3,s4,i6,i6*10,s8,s8,s10,i11,s12,s13);
// printf("%s\n", buf);
	return pcdup(buf);
}

static void AddFont(TFontEnumAddDat *feAdd, int ft) {
  TFontEnumFontCB *fefCB;
  char *pc = GetX11_Fontstr((TFontEnum4*)&feAdd->ft, ft);

	if (!pc) return;
	fefCB = feAdd->fefCB;
	if (fefCB->ResCount+1 >= fefCB->ResArrLg) {
	    if (fefCB->ResArrLg) {
		fefCB->ResArrLg += 256;
		fefCB->Result = xrealloc(fefCB->Result, sizeof(char*) * fefCB->ResArrLg);
	    } else {
		fefCB->ResArrLg = 256;
		fefCB->Result = xmalloc(sizeof(char*) * 256);
	    }
	}
	fefCB->Result[fefCB->ResCount] = pc;
	fefCB->ResCount++;
	fefCB->Result[fefCB->ResCount] = NULL;
}

static void AddFont4(TFontEnumAddDat *feAdd) {
  int ft = feAdd->fefCB->ft;
	if (ft & ft_normal_Roman) AddFont(feAdd, ft_normal_Roman);
	if (ft & ft_normal_Italic) AddFont(feAdd, ft_normal_Italic);
	if (ft & ft_bold_Roman) AddFont(feAdd, ft_bold_Roman);
	if (ft & ft_bold_Italic) AddFont(feAdd, ft_bold_Italic);
}

static bool SetFontFindMinMax(TFontEnumAddDat *feAdd) {
  LOGFONTW *lf = &feAdd->lf;
  TFontEnumFontCB_nx *nx = feAdd->nx;
  int find_H = feAdd->fefCB->find_H;

	pnmove(lf, &nx->lf, sizeof(LOGFONTW));
	lf->lfUnderline = 0;
	lf->lfStrikeOut = 0;
	if (find_H >= 0) {
	    lf->lfHeight = -find_H;
	    lf->lfWidth = 0;
	    SetFontEnum4((TFontEnum4*)&feAdd->ft, ft_ALL);
	    if (GetFontEnum4_x((TFontEnum4*)&feAdd->ft) == find_H) {
		feAdd->Height = find_H;
		AddFont4(feAdd);
		return True;
	    }
	    return False;
	}

	feAdd->minH = 0;
	feAdd->maxH = 0;
	if (nx->FontType & RASTER_FONTTYPE) {
 char tmp[256];
	    WCharsToUtf8(tmp, 256, lf->lfFaceName, pwclen(lf->lfFaceName), UC_force_ASCII);
	    if (!pccasecmpASCII(tmp, "Fixedsys") ||
		!pccasecmpASCII(tmp, "System")) {
// printf("%s(%d):  th=%ld tl=%ld\n", FL, tm.tmHeight, tm.tmInternalLeading);
		lf->lfHeight = 0;
		lf->lfWidth = 0;
		SetFontEnum4((TFontEnum4*)&feAdd->ft, ft_ALL);
		feAdd->Height = GetFontEnum4_x((TFontEnum4*)&feAdd->ft);
		if (feAdd->Height < 0) return False;
		AddFont4(feAdd);
		return True;

	    } else if (!pccasecmpASCII(tmp, "Small Fonts")) {
		feAdd->minH = 3;
		feAdd->maxH = 9;
//		lf->lfHeight = -9;

	    } else if (!pccasecmpASCII(tmp, "MS Sans Serif")) {
		feAdd->minH = 11;
		feAdd->maxH = 32;
//		lf->lfHeight = -11;

	    } else if (!pccasecmpASCII(tmp, "MS Serif")) {
		feAdd->minH = 8;
		feAdd->maxH = 32;
//		lf->lfHeight = -8;

	    } else if (!pccasecmpASCII(tmp, "Courier")) {
		feAdd->minH = 13;
		feAdd->maxH = 20;
//		lf->lfHeight = -13;

	    } else if (!pccasecmpASCII(tmp, "Terminal")) {
		feAdd->minH = 6;
		feAdd->maxH = 19;
//		lf->lfHeight = -6;

	    } else {
		lf->lfHeight = -MM_FONT_MIN;
		SetFontEnum4((TFontEnum4*)&feAdd->ft, ft_ALL);
		feAdd->minH = GetFontEnum4_x((TFontEnum4*)&feAdd->ft);
		if (feAdd->minH < 0) return False;
		if (feAdd->minH < MM_FONT_MIN) feAdd->minH = MM_FONT_MIN;

		lf->lfHeight = -79;
		SetFontEnum4((TFontEnum4*)&feAdd->ft, ft_ALL);
		feAdd->maxH = GetFontEnum4_x((TFontEnum4*)&feAdd->ft);
		if (feAdd->maxH > MM_FONT_MAX) feAdd->maxH = MM_FONT_MAX;

//		lf->lfHeight = -feAdd->minH;
// printf("%s(%d): RASTER th=%ld tl=%ld %s\n", FL, feAdd->minH, feAdd->maxH, tmp);
	    }

	} else {
	    lf->lfHeight = -13;
	    lf->lfWidth = 0;
	    SetFontEnum4((TFontEnum4*)&feAdd->ft, ft_ALL);
	    feAdd->Height = GetFontEnum4_x((TFontEnum4*)&feAdd->ft);
	    if (feAdd->Height < 0) return False;
	    feAdd->Height = 0;
	    AddFont4(feAdd);
	    return True;
	}

	if ((feAdd->minH <= 0) || (feAdd->maxH <= 0)) return False;

	lf->lfWidth = 0;
	for (int i=feAdd->minH;i<=feAdd->maxH;i++) {
	    feAdd->lf.lfHeight = -i;
	    SetFontEnum4((TFontEnum4*)&feAdd->ft, ft_ALL);
//	    if (find_W >= 0) feAdd->lf.lfWidth = find_W;
	    if (GetFontEnum4_x((TFontEnum4*)&feAdd->ft) == i) {
		feAdd->Height = i;
		AddFont4(feAdd);
	    }

	}

	return False;
}

/*
FontNameRegistry-Foundry-FamilyName-WeightName-Slant-
SetwidthName-AddStyleName-PixelSize-PointSize-
ResolutionX-ResolutionY-Spacing-AverageWidth-
CharSetRegistry-CharSetCoding

3  r=Roman (no slant) i=Italic (slant left) o=Oblique (slant left)
   ri=Reverse italic (slant right)
   ro=Reverse oblique (slant right) 

10 m = monospaced (fixed pitch), 
   p = proportional (variable pitch), 
   c = cell

*/
//-adobe-times-medium-r-normal--18-180-75-75-p-94-iso8859-1
//-misc-fixed-bold-r-normal-*-13-120-75-75-c-70-iso10646-1
// 0    1     2    3 4      5 6  7   8  9  1 1  1        1
//                                         0 1  2        3

static int getX_FontType(const char *fontname) {
  int L;
  const char *rpc;
	if (GetFontSXTypeIdx(0, fontname, &rpc, &L) && (L > 0)) {
	    if (!pcncasecmpASCII(rpc, "fixed", L)) return RASTER_FONTTYPE;
	    if (!pcncasecmpASCII(rpc, "raster", L)) return RASTER_FONTTYPE;
	    if (!pcncasecmpASCII(rpc, "TrueType", L)) return TRUETYPE_FONTTYPE;
	    if (!pcncasecmpASCII(rpc, "vector", L)) return DEVICE_FONTTYPE;
	}
	return RASTER_FONTTYPE | DEVICE_FONTTYPE | TRUETYPE_FONTTYPE;
}

static int getX_Pitch(const char *fontname) {
  int L;
  const char *rpc;
	if (GetFontSXTypeIdx(10, fontname, &rpc, &L) && (L == 1)) {
	    if (*rpc == 'p') return VARIABLE_PITCH;
	    if (*rpc == 'm') return FIXED_PITCH;
	    if (*rpc == 'c') return FIXED_PITCH;
	}
	return DEFAULT_PITCH;
}

char **XListFonts(Display *dpy, const char *pattern, int maxNames, int *actualCount) {
  TFontEnumFontCB fefCB;
  LOGFONTW lf;
  int L;
  const char *rpc;
  int m2m3 = 0;

	Fill0(&fefCB, sizeof(TFontEnumFontCB));
	Fill0(&lf, sizeof(LOGFONTW));
	fefCB.maxNames = maxNames;
	fefCB.find_H = -1;
	fefCB.find_W = -1;
	fefCB.maskFontType = getX_FontType(pattern);

//	lf.lfCharSet = ANSI_CHARSET;
	lf.lfCharSet = DEFAULT_CHARSET;
	if (pattern) {
	  if (GetFontSXTypeIdx(1, pattern, &rpc, &L)) {
	    if (*rpc != '*') {
		Utf8ToWChars(lf.lfFaceName, LF_FACESIZE, rpc, L, UC_force_ASCII);
	    }
	  }
	  if (GetFontSXTypeIdx(2, pattern, &rpc, &L)) {
	    if (*rpc != '*') {
		if (!pcncasecmpASCII(rpc, "bold", L)) m2m3 |= 0x02;
		else m2m3 |= 0x01;
	    }
	  }
	  if (GetFontSXTypeIdx(3, pattern, &rpc, &L)) {
	    if (*rpc != '*') {
		if (!pcncasecmpASCII(rpc, "o", L) || 
		    !pcncasecmpASCII(rpc, "i", L)) m2m3 |= 0x08;
		else m2m3 |= 0x04;
	    }
	  }

	  if (GetFontSXTypeIdx(6, pattern, &rpc, &L)) {
	    if (*rpc != '*') {
		fefCB.find_H = pctoint(rpc);
	    }
	  }

	  if (GetFontSXTypeIdx(11, pattern, &rpc, &L)) {
	    if (*rpc != '*') {
		fefCB.find_W = pctoint(rpc) / 10;
	    }
	  }
	  if (GetFontSXTypeIdx(12, pattern, &rpc, &L)) {
	    if ((L == 8) && pcncmp(rpc,"iso10646", 8)) {
		fefCB.unicode = 2;
	    } else if ((L == 7) && pcncmp(rpc,"iso8859", 7)) {
		fefCB.unicode = 1;
	    }
	  }
	  if (GetFontSXTypeIdx(13, pattern, &rpc, &L) && L) {
	    lf.lfCharSet = CharSet_XtoW32(rpc);
	  }
	}

// printf("%s(%d): \"%s\"  typ=0x%x h=%d w=%d 0x%x ", FL, pattern, fefCB.maskFontType, fefCB.find_H, fefCB.find_W, m2m3);

	if (!(m2m3 & 0x03)) m2m3 |= 0x03;
	if (!(m2m3 & 0x0c)) m2m3 |= 0x0c;

	if ((m2m3 & 0x05) == 0x05) fefCB.ft |= ft_normal_Roman;
	if ((m2m3 & 0x09) == 0x09) fefCB.ft |= ft_normal_Italic;
	if ((m2m3 & 0x06) == 0x06) fefCB.ft |= ft_bold_Roman;
	if ((m2m3 & 0x0a) == 0x0a) fefCB.ft |= ft_bold_Italic;
// printf("--> 0x%x ft=0x%x\n", m2m3, fefCB.ft);

	fefCB.hdc = GetDC(0);

	EnumFontFamiliesExW(fefCB.hdc, &lf, (FONTENUMPROCW)EnumFontCallBack, (LPARAM)&fefCB, 0);

  TFontEnumFontCB_nx *nx;
  TFontEnumAddDat feAdd;

	feAdd.fefCB = &fefCB;
	feAdd.hdc = fefCB.hdc;

	nx = fefCB.nx;
	while (nx && (fefCB.ResCount < maxNames)) {
#if 0
 char tmp[256];

	WCharsToUtf8(tmp, 256, nx->lf.lfFaceName, pwclen(nx->lf.lfFaceName), UC_force_ASCII);
 printf("nx: typ=0x%x h=%-2ld 0x%x \"%s\"\n", nx->FontType,
	nx->lf.lfHeight, 
	nx->ft, tmp);
#endif
	    feAdd.nx = nx;
	    if (nx->FontType & fefCB.maskFontType) {
		feAdd.FontType = nx->FontType;
		SetFontFindMinMax(&feAdd);
	    }
	    nx = nx->next;
	}

	ReleaseDC(0, fefCB.hdc);

	while (fefCB.nx) {
	    nx = fefCB.nx;
	    fefCB.nx = nx->next;
	    xfree(nx);
	}

	if (actualCount) *actualCount = fefCB.ResCount;
	return fefCB.Result;
}

/*____________________________________________________________________________*/
int XFreeFontNames(char **list) {       
  char **pps = list;
	if (!list) return 0;
	while (*pps) {xfree(*pps); pps++;}
	xfree(list);
	return 0;
}

/*____________________________________________________________________________*/
int
XFreeFont(Display *dpy, XFontStruct *fs) { 
	if (!fs) return 0;
	DeleteObject(fs->fid);
	xfree(fs);
	return 0;
}

/*____________________________________________________________________________*/
bool X11fontname_to_LOGFONTW(LOGFONTW *lf, const char *name) {
  int L;
  const char *rpc;

	Fill0(lf, sizeof(LOGFONTW));
	if (!GetFontSXTypeIdx(1, name, &rpc, &L)) return False;

	if (L >= LF_FACESIZE) L = LF_FACESIZE-1;
	Utf8ToWChars(lf->lfFaceName, LF_FACESIZE, rpc, L, UC_force_ASCII);

//	lf->lfCharSet = ANSI_CHARSET;
	lf->lfCharSet = DEFAULT_CHARSET;
	lf->lfHeight = 13;
	lf->lfWeight = FW_REGULAR;
//	lf->lfEscapement = 0;
//	lf->lfOutPrecision = OUT_DEVICE_PRECIS;
//	lf->lfClipPrecision = CLIP_CHARACTER_PRECIS;
//	lf->lfQuality = DEFAULT_QUALITY;
//	lf->lfPitchAndFamily = FIXED_PITCH;

	if (GetFontSXTypeIdx(2, name, &rpc, &L)) {
	    if (!pcncasecmpASCII(rpc, "bold", L)) lf->lfWeight = FW_BOLD;
	}

	if (GetFontSXTypeIdx(3, name, &rpc, &L)) {
	    if (*rpc != '*') {
		if (!pcncasecmpASCII(rpc, "o", L) || 
		    !pcncasecmpASCII(rpc, "i", L)) lf->lfItalic = 1;
	    }
	}

	if (GetFontSXTypeIdx(6, name, &rpc, &L)) {
	    if (*rpc != '*') lf->lfHeight = pctoint(rpc);
	}

	lf->lfPitchAndFamily = getX_Pitch(name);

	if (GetFontSXTypeIdx(11, name, &rpc, &L)) {
	    if (*rpc != '*') lf->lfWidth = pctoint(rpc) / 10;
	}

	if (GetFontSXTypeIdx(13, name, &rpc, &L) && L) {
	    lf->lfCharSet = CharSet_XtoW32(rpc);
	}
	lf->lfHeight = -lf->lfHeight;
	return True;
}

XFontStruct *XLoadQueryFont(Display *dpy, const char *name) {
  HFONT hf, fid;
  XFontStruct *xfs;
  HDC hdc;
  LOGFONTW lf;
  TEXTMETRICW tm;

	if (!X11fontname_to_LOGFONTW(&lf, name)) return NULL;


	xfs = NULL;

	fid = CreateFontIndirectW(&lf);
	if (fid) {
	    hdc = GetDC(0);
	    hf = SelectObject(hdc, fid);
	    if (GetTextMetricsW(hdc, &tm)) {
		xfs = xcalloc(sizeof(XFontStruct));
		xfs->fid = fid;
		xfs->ascent = tm.tmAscent;
		xfs->descent = tm.tmDescent;
#if 0
     printf("--> %s(%d): h=%ld, a=%ld d=%ld w=%ld l=%ld oh=%ld\"%s\"\n", FL,
	    tm.tmHeight,
	    tm.tmAscent,
	    tm.tmDescent,
	    tm.tmAveCharWidth,
	    tm.tmInternalLeading,
	    tm.tmOverhang, 
	    name
    ); 
#endif
	    }
	    SelectObject(hdc, hf);
	    ReleaseDC(0, hdc);
	}
	return xfs;
}
