/*
 Copyright (C) 2010 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"
#include "image.h"

bool image_to_DIB(TImage *img, BITMAPINFOHEADER *p, byte *dest_data) {
  byte *dest0, *src0;
  int bytes_per_line;

	if (!img || !img->data || DIB_chk_error(p) || !dest_data) return False;
	dest0 = dest_data;
	src0 = img->data;

	if (((img->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) &&
	    (p->biBitCount == 16)) {
// printf("%s(%d): image_to_DIB 16,16\n", FL);
	    bytes_per_line = ((p->biWidth * 2) + 3) & 0xfffffffc;
	    dest0 += bytes_per_line * (img->height - 1);
	    for (int y=img->height;y>0;y--) {
		word *dest = (word*)dest0;
		word *src = (word*)src0;
		for (int x=img->width;x>0;x--) {
		    *dest = ((*src & 0xf800)>>1)|((*src & 0x7c0)>>1)|(*src & 0x1f);
		    dest++; src++;
		}
		src0 += img->bytes_per_line;
		dest0 -= bytes_per_line;
	    }

	} else if (((img->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) &&
		   (p->biBitCount == 24)) {

	    appError(EMSG_ERR1|EMSG_ToLastBuf|EMSG_H_LF, 
			" bmiHeader.biBitCount 16,32 is %d",  p->biBitCount);
	    return False;

	} else if (((img->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) &&
		   (p->biBitCount == 32)) {
// printf("%s(%d): image_to_DIB 16,32\n", FL);
	    bytes_per_line = p->biWidth * 4;
	    dest0 += bytes_per_line * (img->height - 1);
	    for (int y=img->height;y>0;y--) {
		dword *dest = (dword*)dest0;
		word  *src = (word*)src0;
		for (int x=img->width;x>0;x--) {
		    *dest = ((*src & 0xf800)<<8)|((*src & 0x7e0)<<5)|((*src & 0x1f)<<3);
		    dest++; src++;
		}
		src0 += img->bytes_per_line;
		dest0 -= bytes_per_line;
	    }


	} else if (p->biBitCount == 16) {

// printf("%s(%d): image_to_DIB 32,16\n", FL);
	    bytes_per_line = ((p->biWidth * 2) + 3) & 0xfffffffc;
	    dest0 += bytes_per_line * (img->height - 1);
	    for (int y=img->height;y>0;y--) {
		word  *dest = (word*)dest0;
		dword *src  = (dword*)src0;
		for (int x=img->width;x>0;x--) {
		    *dest = ((*src & 0xf80000)>>9)|((*src & 0xf800)>>6)|((*src & 0xf8)>>3);
		    dest++; src++;
		}
		src0 += img->bytes_per_line;
		dest0 -= bytes_per_line;
	    }

	} else if (p->biBitCount == 24) {
// printf("%s(%d): image_to_DIB 32,24\n", FL);
	    bytes_per_line = ((p->biWidth * 3) + 3) & 0xfffffffc;
	    dest0 += bytes_per_line * (img->height - 1);
	    for (int y=img->height;y>0;y--) {
		byte *dest = dest0;
		byte *src  = src0;
		for (int x=img->width;x>0;x--) {
		    *dest = *src; dest++; src++;
		    *dest = *src; dest++; src++;
		    *dest = *src; dest++;
		    src += 2;
		}
		src0 += img->bytes_per_line;
		dest0 -= bytes_per_line;
	    }

	} else if (p->biBitCount == 32) {
// printf("%s(%d): image_to_DIB 32,32\n", FL);
	    bytes_per_line = p->biWidth * 4;
	    dest0 += bytes_per_line * (img->height - 1);
	    for (int y=img->height;y>0;y--) {
		dword *dest = (dword*)dest0;
		dword *src = (dword*)src0;
		for (int x=img->width;x>0;x--) {
		    *dest = *src;
		    dest++; src++;
		}
		src0 += img->bytes_per_line;
		dest0 -= bytes_per_line;
	    }
	} else return False;
	return True;

}
