/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"

static int CF_PREFERREDDROPEFFECT;

static int Clipboard_RefCount;
static bool Clipboard_Empty = True;

void SelClr() {
	Tlgs_Clr(&appXDat.SelTxt);
	Tlgs_Clr(&appXDat.SelTxt_URL);
}

void SelRetClr() {
	Tlgs_Clr(&appXDat.SelRet_lgs);
}

bool Clipboard_Open() {
// printf("%s(%d): %d %d\n", FL, Clipboard_RefCount, Clipboard_Empty);
	appError_LastClr();
	if (!Clipboard_RefCount) {
	    if (!OpenClipboard(0)) {
		appError(EMSG_ERR1|EMSG_SYS|EMSG_ToLastBuf|EMSG_H_LF,
			 " Clipboard not Open");
		return False;
	    }
	    Clipboard_Empty = False;
	}
	Clipboard_RefCount++;
	return True;
}

void Clipboard_Close() {
	if (!Clipboard_RefCount) return;
	Clipboard_RefCount--;
	if (!Clipboard_RefCount) CloseClipboard();
}

static void Clipboard_Clear() {
	if (!Clipboard_Open()) return;
	EmptyClipboard();
	Clipboard_Close();
}

void Clipboard_Add() {
	if (Clipboard_RefCount && !Clipboard_Empty) {
	    Clipboard_Clear();
	    Clipboard_Empty = True;
	}
}

/*____________________________________________________________________________*/
void SetSelection(int count, ...) {
  va_list ap;
  int i, lg, memsize;
  HGLOBAL hGlobMem;
  char *p;

	appXDat.SELECT_AtomsCount = count;
	va_start (ap, count);
	for (i = 0; (i < count) && (i <= MaxSELECT_Atoms); i++) {
		appXDat.SELECT_Atoms[i] = va_arg (ap, int);
	}
	va_end (ap);

	lg = appXDat.SelTxt.lg;
	memsize = lg+2;
	if (appXDat.SELECT_Atoms[0] == CF_UNICODETEXT) {
	    lg = Chk_fio_UTF8(appXDat.SelTxt.s, lg, NULL);
	    memsize = (lg+1) << 1;
	}

	if (Clipboard_Open()) {
	    hGlobMem = GlobalAlloc(GMEM_MOVEABLE+GMEM_DDESHARE, memsize);
	    if (hGlobMem) {
		p = GlobalLock(hGlobMem);
		if (appXDat.SELECT_Atoms[0] == CF_UNICODETEXT) {
		    Utf8ToWChars((word*)p, memsize>>1, appXDat.SelTxt.s, appXDat.SelTxt.lg, 0);
		} else {
		    ppcnmove((void*)&p, appXDat.SelTxt.s, appXDat.SelTxt.lg);
		    p++; *p = 0;
		}
		Clipboard_Add();
		SetClipboardData(appXDat.SELECT_Atoms[0], hGlobMem);
		GlobalUnlock(hGlobMem);
	    }
	    Clipboard_Close();
	}
	Tlgs_Clr(&appXDat.SelTxt);
	appError(EMSG_FromLastBuf, NULL);
}

bool Chk_GetSelection(int count, ...) {
  va_list ap;
  HGLOBAL hGlobMem = NULL;

	if ((count <= 0) || !Clipboard_Open()) return False;

	va_start(ap, count);
	while (!hGlobMem && (count > 0)) {
        int an = va_arg(ap, int);
	    hGlobMem = GetClipboardData(an);
	    count--;
	}
	va_end(ap);
	Clipboard_Close();
	return (hGlobMem != NULL);
}

bool GetPasteStr_lgs(int enc_utf8) {
  HGLOBAL hGlobMem;
  void *p;
  int lg, cf;

	SelRetClr(); //Tlgs_Clr(&appXDat.SelRet_lgs);
	Clipboard_Open();

	cf = CF_TEXT;
	if (enc_utf8) cf = CF_UNICODETEXT;

	hGlobMem = GetClipboardData(cf);
	if (hGlobMem) {
	    p = GlobalLock(hGlobMem);
	    if (enc_utf8) {
		lg = WCharsToUtf8(NULL, 0, (word*)p, pwclen((word*)p), 0);
		appXDat.SelRet_lgs.lg = lg;
		appXDat.SelRet_lgs.s = xmalloc(lg+1);
		WCharsToUtf8(appXDat.SelRet_lgs.s, lg+1, (word*)p, pwclen((word*)p), 0);

	    } else {
		lg = pclen(p);
		if (lg) {
		    appXDat.SelRet_lgs.lg = lg;
		    appXDat.SelRet_lgs.s = xmalloc(lg+1);
		    pcnmove(appXDat.SelRet_lgs.s, p, lg);
		}
	    }
	    GlobalUnlock(hGlobMem);
	}

	Clipboard_Close();
	return (appXDat.SelRet_lgs.lg > 0);
}

//---------------------------------------------------------------------
int Get_Preferred_DropEffect() {
	if (!CF_PREFERREDDROPEFFECT) {
	    CF_PREFERREDDROPEFFECT = RegisterClipboardFormat("Preferred DropEffect");
	}
	return CF_PREFERREDDROPEFFECT;
}
