/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"
#include "tpixmap.h"

#define TranspArray	1
#define Color0isMask	(1<<1)
#define CountArray	(1<<2)

typedef struct TAHicoHead {
  byte fu;
  byte cols;
  short width;
  short height;
  int DataSize;
    //..... Data
} TAHicoHead;

static byte *GetMask(const byte *dat, int w, int h) {
  int i2, m;
  TBitsIO mbits;
  byte *MaskDat, *pm;

	mbits.io.rd_byte = NULL;
	SetBits8Data(&mbits, dat);
	MaskDat = xcalloc(((w+7)>>3) * h);
	pm = MaskDat;
	for (;h>0;h--) {
	    i2 = w;
	    while (i2 > 0) {
		m = i2;
		if (m > 8) m = 8;
		*pm = GetBits8(&mbits, m);
		pm++; i2 -= 8;
	    }
	}
	return MaskDat;
}

/* ********************************************************
 +-----+-----+-----+-----+--       --+-----+-----
 |  2  |   1 |  1  |  1  |           |  1  |   xxx
 | lg  | next| lg^ | next|  ....     | loop|  ...data..
 +-----+--+--+-----+-----+--       --+--+--+-----
          +- 1 = next lg bit            +- 0= n data
                                        +- 1= n loop data
******************************************************** */
typedef struct {
  TBitsIO bits;
  int col;
  int z;
  int k;
  bool mlp;	//True=n loop data
} TGCCA;

static bool GetColor_CA(TGCCA *ca) {
  int z; 
  unsigned int q;

	if (!ca->z) {
	    ca->mlp = False;
	    q = 2;
	    z = GetBits8(&ca->bits, 2);
	    while (GetBits8(&ca->bits, 1)) {
		z |= GetBits8(&ca->bits, 1) << q;
		q++;
		if (q > 15) return False;
	    }
	    z++;
	    if (GetBits8(&ca->bits, 1)) {	// n loop data
		ca->mlp = True;
		z += 2;
		ca->col = GetBits8(&ca->bits, ca->k);
	    }
	    ca->z = z;
	}
	if (!ca->mlp) {
	    ca->col = GetBits8(&ca->bits, ca->k);
	}
	ca->z--;
	return True;
}

static int GetCol_k(int cols) {
  int k = 0;
	while (cols > 0) {k++; cols >>= 1;}
	return k;
}

TPixMap *CreateAHicoImg(const char *icoHead) {
	return CreateAHicoImg_depth(icoHead, appXDat.RootW_depth);
}

TPixMap *CreateAHicoImg_depth(const char *icoHead, int rdepth) {
  int w, h, fu, cols, m, k, kx, col, i;
  XImage *img;
  TPixMap *pxm;
  byte *pdb;
  int *pdi;
  const word *pCol16;
  word *pdw;
  byte *MaskDat;
  const byte *pColNr;
  const TAHicoHead *icoh;
  int ColList256[256];
  TBitsIO cbits;
  TBitsIO mbits;
  TGCCA ca;

	if (!icoHead) return NULL;
	if (rdepth < 16) return NULL;

	icoh = (const TAHicoHead*)icoHead;
	w = icoh->width;
	h = icoh->height;
	fu = icoh->fu;
	cols = icoh->cols;
	kx = pxm_2b_16bpp;
	if (rdepth > 16) kx = pxm_4b_24bpp;
	pxm = TPixMap_Create_whLD(w, h, kx);
	if (!pxm) return NULL;
	TPixMap_CreateXimg(pxm);
	img = pxm->Ximg;

	pCol16 = (const word*)(icoHead + 10); //sizeof(TAHicoHead));
	for (i=0; i < cols; i++) {
	    if (rdepth == 16) ColList256[i] = *pCol16; 
	    else ColList256[i] = RGB16To24(*pCol16);
	    pCol16++;
	}

	cbits.io.rd_byte = NULL;
	mbits.io.rd_byte = NULL;

	MaskDat = NULL;
	if (fu & CountArray) {
	  Fill0(&ca, sizeof(ca));
	  if (fu & TranspArray) {
	  } else {
	    m = cols;
	    if (!(fu & Color0isMask)) m--;
	    ca.k = GetCol_k(m);
	    SetBits8Data(&ca.bits, (const byte*)pCol16);

	    if (fu & Color0isMask) {
	      MaskDat = xcalloc(((w+7)>>3) * h);
	      SetBits8Data(&mbits, MaskDat);
	      pdb = (byte*)img->data;
	      for (;h;h--) {
		pdw = (word*)pdb;
		pdi = (int*)pdb;
		for (i=w;i;i--) {
		  if (!GetColor_CA(&ca)) goto err;
		  col = m = 0;
		  if (ca.col) {col = ColList256[ca.col-1]; m = 1;}
		  if (rdepth == 16) {*pdw = col; pdw++;} 
			       else {*pdi = col; pdi++;}
		  SetBits8(&mbits, m, 1);
		}
		SetBits8End(&mbits);
		pdb += img->bytes_per_line;
	      }

	    } else {
	      pdb = (byte*)img->data;
	      for (;h;h--) {
		pdw = (word*)pdb;
		pdi = (int*)pdb;
		for (i=w;i;i--) {
		  if (!GetColor_CA(&ca)) goto err;
		  col = ColList256[ca.col];
		  if (rdepth == 16) {*pdw = col; pdw++;} 
			       else {*pdi = col; pdi++;}
		}
		pdb += img->bytes_per_line;
	      }
	    }
	  }

	} else {

	    pColNr = (const byte*)pCol16;
	    if (fu & TranspArray) {
		MaskDat = GetMask(pColNr, w, h);
		SetBits8Data(&mbits, pColNr);
		pColNr += (w*h+7) >> 3;
	    }
	    SetBits8Data(&cbits, pColNr);

	    k = GetCol_k(cols-1);
	    pdb = (byte*)img->data;
	    for (;h;h--) {
		pdw = (word*)pdb;
		pdi = (int*)pdb;
		for (i=w;i;i--) {
		    if (MaskDat && !GetBits8(&mbits, 1)) col = 0; else 
			col = ColList256[GetBits8(&cbits, k)];
		    if (rdepth == 16) {*pdw = col; pdw++;} 
				 else {*pdi = col; pdi++;}
		}
		pdb += img->bytes_per_line;
	    }
	}

	if (MaskDat) {
	    pxm->mask_data = MaskDat;
	    pxm->stat &= ~pmxStat_XMaskPixAkt;
	    pxm->stat |= (pmxStat_HandleMask|pmxStat_MaskImgAkt);
	}

	return pxm;
err:
	Tfree(pxm);
	return NULL;
}

static void pxm2pic(TPixMap *pxm, TPic *pic) {
	pic->width = pxm->width;
	pic->height = pxm->height;
	TPixMap_XPutImage(pxm);
	pic->pic = pxm->pmxID;
	pic->mask = pxm->maskID;
	pxm->pmxID = pxm->maskID = 0;
	Tfree(pxm);
}

TPic REGP2 *CreateAHicoPic(const char *icoHead, TPic *pic) {
  TPixMap *pxm;

	if (pic && pic->pic) return pic;
	if (!pic) pic = xmalloc(sizeof(*pic));
	Fill0(pic, sizeof(*pic));

	pxm = CreateAHicoImg(icoHead);
	if (!pxm) return NULL;
//	bitmask2alpha((TImage*)pxm);

	pxm2pic(pxm, pic);
	return pic;
}

TPic REGP2 *CreateAHicoDisPic(const char *icoHead, TPic *pic) {
  int i, col, h;
  TPixMap *pxm;
  int col1, col2;
  TBitsIO mbits;
  word *pdw;
  int *pdi;
  byte *pdb;
  bool Mask1, Mask2;

	if (pic && pic->pic) return pic;
	if (!pic) pic = xmalloc(sizeof(*pic));
	Fill0(pic, sizeof(*pic));

	pxm = CreateAHicoImg(icoHead);
	if (!pxm) return NULL;
//	bitmask2alpha((TImage*)pxm);

	if (pxm->mask_data) {
	    mbits.io.rd_byte = NULL;
	    SetBits8Data(&mbits, pxm->mask_data);

	    col1 = 0x808080;
	    col2 = 0xffffff;
	    if (appXDat.RootW_depth == 16) {
		col1 = RGB24To16(col1);
		col2 = RGB24To16(col2);
	    }

	    pdb = (byte*)pxm->Ximg->data;
	    for (h=pxm->height;h;h--) {
		pdw = (word*)pdb;
		pdi = (int*)pdb;
		i = pxm->width;
		Mask2 = GetBits8(&mbits, 1);
		while (i) {
		    col = col1;
		    Mask1 = Mask2;
		    if (i > 1) Mask2 = GetBits8(&mbits, 1); else Mask2 = False;
		    if (Mask1 && !Mask2) col = col2;
		    if (appXDat.RootW_depth == 16) {
			*pdw = col; pdw++;
		    } else {
			*pdi = col; pdi++;
		    }
		    i--;
		}
		while (mbits.k) GetBits8(&mbits, 1);
		pdb += pxm->Ximg->bytes_per_line;
	    }
	}

	pxm2pic(pxm, pic);
	return pic;
}

extern TPic REGP3 *CreateAHicoPic_EnDis(const char *icoHead, TPicEnDis *pic_ed,
							 bool disable) {
	if (disable) return CreateAHicoDisPic(icoHead, &pic_ed->dis);
	return CreateAHicoPic(icoHead, &pic_ed->en);
}

