/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"


typedef struct {
  Window WinID;
  TWins *wins;
} THWinID;

typedef struct {
  int count;
  THWinID arr[];
} TArrWinID;

static TArrWinID *ArrID[32];

static int get_idx(int WinID) {
  int result = WinID + (WinID >> 16);
	return (result + (result >> 5) + (result >> 10)) & 0x1f;
}

void TWinsRegistration(TWins *wins) {
  int count, ic;
  TArrWinID *ap, **ppap;
  THWinID *arr;

	ppap = &ArrID[get_idx((int)wins->WinID)];
	ap = *ppap;
	if (!ap) goto new_mem;

	count = ap->count;
	arr = ap->arr;
	while (count > 0) {
	    if (!arr->WinID) {
		arr->WinID = wins->WinID;
		arr->wins = wins;
		return;
	    }
	    arr++; count--;
	}
  //get more mem
	count = ap->count;
	ic = 8;
	if (count >= 31) ic = 16;
	if (count >= 63) ic = 32;
	goto init;
new_mem:
	count = 0;
	ic = 7;
init:
	set_arr((void*)ppap, sizeof(TArrWinID), count, count+ic, sizeof(THWinID));
	ap = *ppap;
	ap->count += ic;
	ap->arr[count].WinID = wins->WinID;
	ap->arr[count].wins = wins;
}

void TWinsDeregister(TWins *wins) {
  int i, count;
  TArrWinID *ap;
  THWinID *arr;

	if (appXDat.GrabWin == wins) Std_UngrabPointer();

	for (i=0; i < PaintOnEvEnd_Count; i++) {
	    if (PaintOnEvEnd_List->val[i] == wins) PaintOnEvEnd_List->val[i] = NULL;
	}

	ap = ArrID[get_idx((int)wins->WinID)];
	if (ap) {
	    count = ap->count;
	    arr = ap->arr;
	    while (count > 0) {
		if (arr->WinID == wins->WinID) {
		    arr->WinID = 0;
		    arr->wins = NULL;
		    return;
		}
		arr++; count--;
	    }
	}
// printf("%s(%d): Error: Deregister %d 0x%x 0x%x\n", FL, idx, (int)wins, (int)wins->WinID);
}

TWins *GetTWins(Window WinID) {
  int count;
  TArrWinID *ap;
  THWinID *arr;

	ap = ArrID[get_idx((int)WinID)];
	if (ap) {
	    count = ap->count;
	    arr = ap->arr;
	    while (count > 0) {
		if ((arr->WinID == WinID) &&
		    (ttc_chk_struct(arr->wins))) return arr->wins;
		arr++; count--;
	    }
	}
// printf("%s(%d): Error: GetTWins 0x%x\n", FL, (int)WinID);
	return NULL;
}

bool TWins_ChkExists(TWins *self) {
  int y, count;
  TArrWinID *ap, **ppap;
  THWinID *arr;

	if (!self) return False;
	ppap = ArrID;
	for (y=0;y<32;y++) {
	    ap = *ppap;
	    if (ap) {
		count = ap->count;
		arr = ap->arr;
		while (count > 0) {
		    if (arr->wins == self) {
			return  (self->magic == TTC_MAGIC) && 
				(self->wtcsize >= sizeof(TWins)) &&
				((self->RunStatus & runst_GoDestroy) == 0) &&
				(arr->WinID == self->WinID);
		    }
		    arr++; count--;
		}
	    }
	    ppap++;
	}
	return False;
}

#ifdef MEM_CHK
int GetTWinsCount() {
  int result = 0;
	for (int i=0;i<32;i++) {
	    if (ArrID[i]) for (int x=0;x<ArrID[i]->count;x++) {
		if (ArrID[i]->arr[x].WinID) result++;
	    }
	}
	return result;
}
#endif
