/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include <unistd.h>
#include <winsock.h>

extern void (*FAppRun)();

OSVERSIONINFO WinVERSIONINFO;
TappXDat appXDat;
TMouseDown MouseDown;

dword appXDat_Default_Color = 0xc0c0c0;
GC WinsDefault_GC;

Tptr_arr *PaintOnEvEnd_List;
int PaintOnEvEnd_Count;

static HMODULE appInstance;

static Display dpy;
static dword appTimer_50ms;

static bool Terminate = False;

static void CALLBACK appTimerProc_50ms(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime) {
	W32_app_Timer_50ms(dwTime);
}


#ifdef APP_INI_DAT
static const char DefaultIniDat[] = {
  "[Global]\n"
  "Font=-*-Arial-medium-r-*-*-13-*-*-*-*-*-*-1\n"
  "FontBold=-*-Arial-bold-r-*-*-13-*-*-*-*-*-*-1\n"
  "; DateFormat 0=d-m-y 1=m-d-y 2=y-m-d\n"
  "DateFormat=0\n"
  "FileSystemUTF8=0\n"
  "MouseZ_Wheel=3\n"
  "\n"
  "[Icon]\n"
  ";IconPaths=\n"
  "\n"
  "[App]\n"
  ";dotdesktop=\n"
  ";dotdesktop_mimelnk=\n"
  "\n"
  "[Dlg_File]\n"
  "Az0_ItemE_Dbl=True\n"
  "Az0_PathE_Dbl=False\n"
  "Az1_ItemE_Dbl=True\n"
  "Az1_PathE_Dbl=False\n"
};
#endif

#define g_width 2
#define g_height 2
static char g_bits[] = {0x02, 0x01};

/*____________________________________________________________________________*/
static void InitStartForm() {
  Tobjmsg msg;

	msg.msg = msgVisibleFormShow;
	msg.f = msg.m = 0;
	msg.p = NULL;

  TWins *wins = appXDat.Wins;
	while (wins) {
	    if (wins->wtyp & wtyp_Form) Send_MSG(&msg, wins);
	    wins = wins->next;
	}
}

static void XAppRun() {
	InitStartForm();
	AppProcessMessages();
	if (appXDat.OnAppStart) appXDat.OnAppStart();
NotClose:
	while (AppHandleMessage());
	if (appXDat.OnAppClose) appXDat.OnAppClose();
	if (!appDat.gQUIT_Ev) goto NotClose;
	appDat.AppExit = True;

	KillTimer(0, appTimer_50ms);
}

/*____________________________________________________________________________*/
void initXapp(int fu, int argc, char **argv) {
  word *pwc;

	initapp(fu, argc, argv);
	FAppRun = XAppRun;

	WinVERSIONINFO.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (!GetVersionEx(&WinVERSIONINFO)) {
	    printf("%s(%d): ERROR: GetVersionEx()\n", FL);
	}
// printf("%s(%d): %lu.%lu %lu %lu %s\n", FL, WinVERSIONINFO.dwMajorVersion, WinVERSIONINFO.dwMinorVersion, WinVERSIONINFO.dwBuildNumber, WinVERSIONINFO.dwPlatformId, WinVERSIONINFO.szCSDVersion);

	InitEvCALLBACKJumpTable();

	set_ptr_arr(&PaintOnEvEnd_List, 254);

	appDat.self_pid = GetCurrentThreadId();

	MonitorsInit();

// printf("%d %d\n", appXDat.Root_width, appXDat.Root_height);

	appInstance = GetModuleHandle(0);
	pwc = xcalloc(4096*2);
	GetModuleFileNameW(appInstance, pwc, 4096);
	appDat.self_exe = pwctoUTF8(pwc);
	xfree(pwc);
	appDat.sHOME = GetFilePath(appDat.self_exe);
	appDat.ALDini_dir = pcadd(appDat.sHOME, ".ALDini");

	if ((fu & app_NoIni) == 0) {
	    if (!file_chk_exist(appDat.ALDini_dir, findDIR)) {
  char buf[MAX_PATH];
  buf[0] = 0;
  HKEY hk = 0;
  DWORD BufSize = MAX_PATH;
  DWORD ret_type = 0;
		if (!RegOpenKeyExA(HKEY_USERS, 
			".DEFAULT\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders", 
			0, KEY_QUERY_VALUE, &hk)) {
//	  printf("%s(%d): hk=0x%x 0x%x\n", FL, (int)hk, (int)HKEY_USERS);

		    if (!RegQueryValueExA(hk, "AppData", NULL, &ret_type, buf, &BufSize)) {
//	  printf("%s(%d): 0x%x 0x%x\n", FL, (int)ret_type, (int)REG_EXPAND_SZ);
			if (ret_type == REG_EXPAND_SZ) {
			    char *tmp = pcdup(buf);
			    buf[0] = 0;
			    ExpandEnvironmentStringsA(tmp, buf, MAX_PATH);
			    xfree(tmp);
			}
//	  printf("%s(%d): Reg    =%s\n", FL, buf);
		    }
		    RegCloseKey(hk);
		    appDat.sHOME = pcdup(buf);
		    ppcchange_add_n(&appDat.ALDini_dir, 2, buf, "\\ALDccp");
		}
	    }
	    file_mkdir_parents_ae(appDat.ALDini_dir, 0750, EMSG_ERR_EXIT);
	}

	appDat.inifile_global = pcadd(appDat.ALDini_dir, "\\global.ini");
#ifdef APP_INI_DAT
	if (!(fu & app_NoIni)) {
	    if (!file_chk_exist(appDat.inifile_global, F_OK)) {
		FileHandle fd;
		file_open_wr_ae(appDat.inifile_global, &fd, 0640, EMSG_ERR_EXIT);
		file_write(fd, NULL, DefaultIniDat, sizeof(DefaultIniDat)-1, NULL);
		file_close(fd, NULL);
	    }
	}
#endif
	appXDat.UTF8on = true;
	appXDat.FileSystemUTF8on = true;

	appXDat.RootW = 0;
  HDC hdc = GetDC(0);
	appXDat.RootW_depth = GetDeviceCaps(hdc, BITSPIXEL);
	appXDat.LogPixel = GetDeviceCaps(hdc, LOGPIXELSY);
// printf("%s(%d): RootW_depth=%d\n", FL, appXDat.RootW_depth);
	if (appXDat.RootW_depth > 24) appXDat.RootW_depth = 24;

// printf("%d\n", GetDeviceCaps(hdc, BITSPIXEL));
	ReleaseDC(0, hdc);


	appTimer_50ms = SetTimer(0, 0, 50, appTimerProc_50ms);

/*
      SendMessage(appHandle, WM_SETICON, 1, LoadIcon((HINSTANCE) NULL, IDI_APPLICATION));
      SetClassLong(appHandle, GCL_HICON, LoadIcon((HINSTANCE) NULL, IDI_APPLICATION));
*/

	appDat.X11DispatchEvent = X11DispatchEvent;
	appDat.Wins_PaintOnEvEnd = Wins_PaintOnEvEnd;


	dpy.appDat = &appDat;
	dpy.appXDat = &appXDat;
	dpy.appInstance = appInstance;
//	dpy.lpfnWndProc = WindowProc;

	appXDat.dpy = &dpy;

  long gcm;
  XGCValues gcv;
	Fill0(&gcv, sizeof(XGCValues));
	appXDat.Default_Color = appXDat_Default_Color;
	appXDat.Paint_Color = appXDat_Default_Color;
#ifdef USE_X11GC_plane_mask
	gcm = GCFunction | GCPlaneMask | GCGraphicsExposures | GCLineWidth;
	gcv.plane_mask = AllPlanes;
#else
	gcm = GCFunction | GCGraphicsExposures | GCLineWidth;
#endif
//	gcv.line_width = 0;
	gcv.function = GXcopy;
	gcv.fill_style = FillSolid;
//	gcv.graphics_exposures = 0;
	appXDat.IconGC = CreateGC(gcm, &gcv);

	WinsDefault_GC = CreateGC_RGB(appXDat.Default_Color);
	appXDat.Default_GC = CreateGC_RGB(appXDat.Default_Color);
	appXDat.bordH_GC = CreateGC_RGB(0xe0e0e0);
	appXDat.bordL_GC = CreateGC_RGB(0x808080);
	appXDat.GC_FFFFFF = CreateGC_RGB(0xffffff);
	appXDat.GC_C0C0C0 = CreateGC_RGB(0xc0c0c0);
	appXDat.GC_404040 = CreateGC_RGB(0x404040);
	appXDat.GC_000000 = CreateGC_RGB(0x000000);
	appXDat.Paint_GC = CreateGC_RGB(appXDat.Paint_Color);
	appXDat.Select_GC_Back = CreateGC_RGB(0x00007f);

#ifdef USE_X11GC_plane_mask
	gcm = GCFunction | GCPlaneMask | GCGraphicsExposures | GCLineWidth |
	      GCForeground | GCFillStyle;
#else
	gcm = GCFunction | GCGraphicsExposures | GCLineWidth | GCForeground | GCFillStyle;
#endif
	gcv.function  = GXnand;
//	gcv.foreground = GetColor_RGB(0xffff80);
//	gcv.function  = GXorReverse;
	gcv.foreground = GetColor_RGB(0xffff80);
	appXDat.Select_GC = CreateGC(gcm, &gcv);

	gcv.function = GXxor;
	appXDat.Focus_bitmap = XCreateBitmapFromData_W32noDIB(appXDat.dpy, appXDat.RootW, g_bits, g_width, g_height);
	gcm |= GCStipple | GCSubwindowMode | GCBackground;
	gcv.line_width = 1;
	gcv.foreground = GetColor_RGB(0x000000);
	gcv.background = GetColor_RGB(0xc0c0c0);
	gcv.fill_style = FillOpaqueStippled;
	gcv.stipple = appXDat.Focus_bitmap;
	gcv.subwindow_mode = IncludeInferiors;
	appXDat.FocusLine_GC = CreateGC(gcm, &gcv);

	gcv.function = GXandInverted;
	appXDat.GC_000000_Dot = CreateGC(gcm, &gcv);

	gcv.function = GXcopy;
	gcv.foreground = GetColor_RGB(0xffffff);
	appXDat.SB_GC = CreateGC(gcm, &gcv);


	appXDat.XCursors[XC_top_left_arrow] = LoadCursor(0, IDC_ARROW);
	appXDat.XCursors[XC_xterm] = LoadCursor(0, IDC_IBEAM);

	appXDat.XCursors[XC_top_side] = LoadCursor(0, IDC_SIZENS);
	appXDat.XCursors[XC_bottom_side] = appXDat.XCursors[XC_top_side];
	appXDat.XCursors[XC_sb_v_double_arrow] = appXDat.XCursors[XC_top_side];

	appXDat.XCursors[XC_left_side] = LoadCursor(0, IDC_SIZEWE);
	appXDat.XCursors[XC_right_side] = appXDat.XCursors[XC_left_side];
	appXDat.XCursors[XC_sb_h_double_arrow] = appXDat.XCursors[XC_left_side];

	appXDat.XCursors[XC_top_left_corner] = LoadCursor(0, IDC_SIZENWSE);
	appXDat.XCursors[XC_bottom_right_corner] = appXDat.XCursors[XC_top_left_corner];

	appXDat.XCursors[XC_top_right_corner] = LoadCursor(0, IDC_SIZENESW);
	appXDat.XCursors[XC_bottom_left_corner] = appXDat.XCursors[XC_top_right_corner];

#ifdef APP_INI_DAT
	TpcIni *ini;
	const char *smFont, *sbFont;
	const char *cmFont, *cbFont;
	cmFont = "-*-Arial-medium-r-*-*-13-*-*-*-*-*-*-1";
	cbFont = "-*-Arial-bold-r-*-*-13-*-*-*-*-*-*-1";
	smFont = "-*-MS Sans Serif-medium-r-*-*-11-*-*-*-*-*-*-1";
	sbFont = "-*-MS Sans Serif-bold-r-*-*-11-*-*-*-*-*-*-1";
	ini = NULL;
	if (!(fu & app_NoIni)) {
	    ini = TpcIni_Create(appDat.inifile_global);

//	    appXDat.UTF8on = TpcIni_RD_boolean(ini, "Global", "UTF8", 0);
//	    appXDat.FileSystemUTF8on = TpcIni_RD_boolean(ini, "Global", "FileSystemUTF8", 0);
	    appXDat.Default_MouseZ_Wheel = TpcIni_RD_int(ini, "Global", "MouseZ_Wheel", 3);

	    GlobDateFormat = TpcIni_RD_int(ini, "Global", "DateFormat", 0);

	    if (FontExists(cmFont)) {
		smFont = cmFont;
		sbFont = cbFont;
	    }
	    cmFont = TpcIni_RD(ini, "Global", "Font");
	    cbFont = TpcIni_RD(ini, "Global", "FontBold");
	}
	if (!appXDat.Default_Font) 
		appXDat.Default_Font      = MakeFont(cmFont, smFont);
	if (!appXDat.Default_bold_Font) 
		appXDat.Default_bold_Font = MakeFont(cbFont, sbFont);
	Tfree(ini);

#else

	appXDat.Default_Font = MakeFont("-*-Arial-medium-r-*-*-13-*-*-*-*-*-*-1", 
					 "-*-MS Sans Serif-medium-r-*-*-11-*-*-*-*-*-*-1");
	appXDat.Default_bold_Font = MakeFont("-*-Arial-bold-r-*-*-13-*-*-*-*-*-*-1", 
					      "-*-MS Sans Serif-bold-r-*-*-11-*-*-*-*-*-*-1");
#endif
	Font_SetDefault((TFont*)appXDat.Default_Font, true);
	Font_SetDefault((TFont*)appXDat.Default_bold_Font, true);
}

int Send_MSG(Tobjmsg *msg, TWins *wins) {
	if (!msg || !ttc_chk_struct(wins) || !wins->EvProc) return 0;
	msg->wins = wins;
	return wins->EvProc(msg);
}

int Exec_Win_Ev(Tobjmsg *msg, Window win) {
  TWins *wins = GetTWins(win);
	return Send_MSG(msg, wins);
}

static bool DispatchEvent() {
  MSG Msg;
	if (PeekMessageW(&Msg, 0, 0, 0, PM_REMOVE)) {
	    if (Msg.message == WM_QUIT) {
		Terminate = True;
		appDat.gQUIT_Ev = True;
		return False;
	    }
//	    TranslateMessage (&Msg);
	    DispatchMessageW(&Msg);
	    return True;
	}
	return False;
}

void X11DispatchEvent() {
	while (DispatchEvent());
}

void Wins_PaintOnEvEnd() {
  Tobjmsg msg;
  int count;
  void **val;

	if (PaintOnEvEnd_Count > 0) {
	    msg.msg = msgPaint;
	    count = PaintOnEvEnd_Count;
	    PaintOnEvEnd_Count = 0;
	    val = pndup(PaintOnEvEnd_List->val, sizeof(void*) * count);
	    
	    for (int i=0; i < count; i++) {
		msg.wins = val[i];
		if (ttc_chk_struct(msg.wins) && msg.wins->EvProc)
					msg.wins->EvProc(&msg);
	    }
	    xfree(val);
	}
}

/*____________________________________________________________________________*/

void MapWinID(Window WinID, int fu) {  //fu 0=UnMap 1=Map 2=MapRaised
	switch(fu) {
	case 0:
		XUnmapWindow(appXDat.dpy, WinID);
		break;
	case 1:
		XMapWindow(appXDat.dpy, WinID);
		break;
	case 2:
		XMapRaised(appXDat.dpy, WinID);
		break;
	}
}

void AppHandleWinMessages() {
	if (appDat.X11DispatchEvent) appDat.X11DispatchEvent();
	if (appDat.Wins_PaintOnEvEnd) appDat.Wins_PaintOnEvEnd();
}
