/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"

typedef struct TCaretDat {
  TgTimer *timer;
#ifndef _OS_WIN32
  GC gc;
#endif
  Window WinID;
  int x, y, h, w;
  bool isOn, isPaint;
} TCaretDat;

static TCaretDat CaretDat;

//************************** Caret ********************************************
#ifdef _OS_WIN32
static void CaretPaint() {
  HDC hdc = GetDC(CaretDat.WinID);
	BitBlt(hdc, CaretDat.x, CaretDat.y, 1, CaretDat.h, 0, 0, 0, DSTINVERT);
	ReleaseDC(CaretDat.WinID, hdc);
}
#else
static void CaretPaint() {
	if (appXDat.dpy) {
	    if (CaretDat.w < 2) {
		DrawLine(CaretDat.WinID, CaretDat.gc, CaretDat.x, CaretDat.y, CaretDat.x, CaretDat.y + CaretDat.h - 1);
	    } else {
		Fill_Rect(CaretDat.WinID, CaretDat.gc, CaretDat.x, CaretDat.y, CaretDat.w, CaretDat.h);
	    }
	}
}
#endif

static void Caret_OnTimer(TgTimer *ti) {
	if (CaretDat.WinID && CaretDat.isOn) {
	    CaretPaint();
	    CaretDat.isPaint = !CaretDat.isPaint;
	}
}

void Caret_Draw(int fu) {
	if (CaretDat.x < 0 || !CaretDat.WinID) fu = 0;
	if ((fu & 1) != (CaretDat.isOn && CaretDat.isPaint)) {
	    CaretPaint();
	}
	CaretDat.isPaint = CaretDat.isOn = (fu & 1);
}

void Caret_Setup(Window WinID, int left, int top, int width, int height) {
	if (!CaretDat.timer) Caret_Init();
	Caret_Draw(CARET_OFF);
	CaretDat.WinID = WinID;
	CaretDat.x = left;
	CaretDat.y = top;
	CaretDat.w = width;
#ifdef _OS_WIN32
	CaretDat.h = height;
#else
	CaretDat.h = height;
#endif
}

void Caret_SetPos(Window WinID, int left, int top) {
	Caret_Setup(WinID, left, top, CaretDat.w, CaretDat.h);
}

void TWins_Caret_SetNewPos(TWins* self, int left, int top) {
  int height;

	if (!self) return;
	if (!Font_GetMetrics((TFont*)self->vds.Font, &height, NULL)) return;
	Caret_Setup(self->WinID, left, top, 0, height);
}

void TWins_CaretRect_SetNewPos(TWins* self, int x, int y, int w) {
	TWins_Caret_SetNewPos(self, x, y);
	CaretDat.w = w;
}

void TWins_Caret_Clr(TWins* self) {
	if (CaretDat.WinID == self->WinID) {
	    Caret_Draw(0);
	    CaretDat.WinID = 0;
	}
}

bool TWins_Caret_Chk(TWins* self) {
	return (CaretDat.WinID == self->WinID);
}

void Caret_Init() {
	if (CaretDat.timer) return;
#ifndef _OS_WIN32
  XGCValues *gcv = xcalloc(sizeof(XGCValues));

	gcv->function = GXxor;
	gcv->fill_style = FillSolid;
	gcv->foreground = GetColor_RGB(0xffffff);
#ifdef USE_X11GC_plane_mask
	gcv->plane_mask = AllPlanes;
	CaretDat.gc = CreateGC(GCFunction | GCPlaneMask | GCGraphicsExposures | 
		GCLineWidth | GCForeground | GCFillStyle, gcv);
#else
	CaretDat.gc = CreateGC(GCFunction | GCGraphicsExposures | GCLineWidth |
	      GCForeground | GCFillStyle, gcv);
#endif
	xfree(gcv);
#endif /* _OS_WIN32 */
	CaretDat.timer = gTimerX_Add(Timer_Enabled, 500, Caret_OnTimer);
}

