/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"
#include <sys/stat.h>


const char *Desktop_Entry = "Desktop Entry";

typedef struct {
  const char *a;
  const int ddkt;
} TScan_ddkt;

static const TScan_ddkt TScan_ddkt_tab[5] = {
  {"Application",   ddkt_Application},
  {"FSDevice",      ddkt_FSDevice},
  {"Directory",     ddkt_Directory},
  {"ALDeskBuildin", ddkt_ALDeskBuildin},
  {"MimeType", ddkt_Mime}
};

int dotdesktop_get_ddkt(const char *dType) {
  int i;
	for (i=0;i<5;i++) {
	    if (!pccasecmpASCII(dType, TScan_ddkt_tab[i].a)) {
		return TScan_ddkt_tab[i].ddkt;
	    }
	}
	return 0;
}


int ddkt_desktop_stat(const char *fname, Tstat64 *st) {
  const char *dType;
  TpcIni *ini;
  int ret = 0;
	if (st) {
	ini = NULL;
	if (st->st & st_Reg) {
	    if (st->FileType == stft_desktop) {
		ini = TpcIni_Create(fname);
		dType = TpcIni_RD_pc(ini, Desktop_Entry, "Type", NULL);
		ret = dotdesktop_get_ddkt(dType);
	    }
	  }
	  if (ini) Tfree(ini);
	}
	return ret;
}

int ddkt_desktop(const char *fname) {
  Tstat64 *st;
  int ret;

	st = file_stat_fn(fname, fsFollowLink | fsGetFileType | fsNoFileName | fsNoLinkName);
	ret = ddkt_desktop_stat(fname, st);
	xfree(st);
	return ret;
}
