/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"

bool FontExists(const char *fontname) {
  int L;
  char **FontNameList;
	FontNameList = XListFonts(appXDat.dpy, fontname, 1, &L);
	XFreeFontNames(FontNameList);
	return (L > 0);
}

char *GetFullFontName(const char *fontname) {
  int L;
  char **FontNameList;
  char *res = NULL;
	if (!fontname || !*fontname) return NULL;
	FontNameList = XListFonts(appXDat.dpy, fontname, 1, &L);
	if (L > 0) res = pcdup(*FontNameList);
	XFreeFontNames(FontNameList);
	return res;
}

TFontX *MakeFont(const char *fontname, const char *defaultfont) {
  char *fontpc = NULL;
  TFontX *F;

	if (!fontname) goto DefFont; //fontname = "fixed";
	fontpc = GetFullFontName(fontname);
	if (!fontpc) {
	    printf("WARNING -- can t get font \"%s\", trying \"%s\"\n", fontname, defaultfont);
DefFont:
	    fontpc = GetFullFontName(defaultfont);
	}

	if (!fontpc) {
#ifdef _OS_WIN32
	    fontpc = pcdup("-*-system-*-*-*-*-*-*-*-*-*-*-*-*");
#else
	    fontpc = pcdup("fixed");
#endif
	    printf("WARNING -- can t get font \"%s\", trying \"%s\"\n", defaultfont, fontpc);
	}
	F = TFontX_Create(fontpc);
	xfree(fontpc);
	return F;
}

//**************** X11 ****************
static int DrawStringSeg(int x, int y, int lgs, const char *pcs, 
			TVDrawString *vds, TFontX *font, 
			dword ForeColor, dword BackColor) {
  int i, lg, w, x0, xn, xt, yt, height, ascent;
  const char *pcsa;
  dword flags;

	if (!pcs) return x;
	if (!Font_GetMetrics((TFont*)font, &height, &ascent)) return x;
	flags = 0;
	if (vds->UTF8on) flags |= FONTFG_UTF8;
	if (vds->Image)  flags |= FONTFG_ImageString;
	Font_SetFlags((TFont*)font, flags);
//	Font_ClrOrigin((TFont*)Font);
	Font_SetColor((TFont*)font, ForeColor);
	Font_SetBackColor((TFont*)font, BackColor);

	pcsa = pcs;
	if (vds->TabSpaceOn && (vds->TabSpaceWidth > 0)) {
	  for (i=0; i<lgs; i++) {
	    if (*pcs == 0x09) {
	      w = pcs - pcsa;
	      x0 = x;
	      if (w > 0) x0 = x + Font_GetWidth((TFont*)font, pcsa, w); //vds_nWidth(vds, pcsa, w);
	      xn = (x0 / vds->TabSpaceWidth + 1) * vds->TabSpaceWidth;
	      if (!vds->OnlyLg && (xn >= -vds->Xoff)) {
		if ((w > 0) && (x0 >= -vds->Xoff)) {
		  FontX_PaintTo(font, &vds->WinID, vds->Xoff + x, 
				vds->Yoff + ascent + y, pcsa, w);
		}
		xt = x0 + vds->Xoff;
		yt = y + vds->Yoff;
		FillRectRGB(vds->WinID, BackColor, xt, yt, xn - xt, height);
		if (vds->TabSpaceDot) {
		  yt += ascent;
		  DrawLine(vds->WinID, appXDat.GC_000000, xt, yt, xt + 1, yt);
		  DrawLine(vds->WinID, appXDat.GC_000000, xt, yt - 1, xt, yt);
		}
	      }
	      x = xn;
	      pcsa = pcs + 1;
	    }
	    pcs++;
	  }
	  lg = pcs - pcsa;
	} else lg = lgs;

	if (lg > 0) {
	    x0 = x + Font_GetWidth((TFont*)font, pcsa, lg); //vds_nWidth(vds, pcsa, lg);
	    if (!vds->OnlyLg && (x0 >= -vds->Xoff)) {
		FontX_PaintTo(font, &vds->WinID, vds->Xoff + x, 
			      vds->Yoff + ascent + y, pcsa, lg);
	    }
	    x = x0;
	}
	return x;
}

int DrawColorString(int x, int y, TVDrawString *vds) {
  int x0, yo, L, it, lg, yh, xa, xe, a, e;
  TStrColorDat *scd;
  char *pc;
  int height;
  bool UTF8on;

	if (!vds) return x;
	L = vds->lg;
	pc = vds->s;
	x0 = x;

	if (vds->scda && vds->scda->items) {
	    UTF8on = vds->UTF8on;
	    it = vds->scda->items;
	    scd = vds->scda->scd;
	    while ((it > 0) && (L > 0) && scd->s) {
		lg = scd->s - pc;

		if (lg > L) lg = L;
		if (lg > 0) {
		    vds->UTF8on = UTF8on;
		    x = DrawStringSeg(x, y, lg, pc, vds, vds->Font, vds->ForeColor, vds->BackColor);
		    L -= lg;
		    pc += lg;
		}
		lg = scd->lg;
		if (lg > L) lg = L;
		if (lg > 0) {
		    if (scd->fu & scdfu_override_utf8) {
			vds->UTF8on = (scd->fu & scdfu_UTF8on) != 0;
		    }
		    x = DrawStringSeg(x, y, lg, pc, vds, scd->Font, scd->ForeColor, scd->BackColor);
		    L -= lg;
		    pc += lg;
		}
		it--; scd++;
	    }
	    vds->UTF8on = UTF8on;
	}

	if (L > 0) {
	    x = DrawStringSeg(x, y, L, pc, vds, vds->Font, vds->ForeColor, vds->BackColor);
// printf("%dx%d %d %s %x %x %x\n", x, y, L, pc, vds->WinID, vds->TmpForeColor, vds->TmpBackColor);
	}
	if (vds->OnlyLg) return x;

	Font_GetMetrics((TFont*)vds->Font, &height, NULL);
	yo = y + vds->Yoff;
	if (vds->Wins && vds->ClsToEnd) {
	  yh = height;
	  xa = x + vds->Xoff;
	  if (xa < 0) xa = 0;
	  e = vds->Wins->width - xa;
	  if (e > 0) {
	    if (vds->SelToEnd) {
		Fill_Rect(vds->WinID, appXDat.Select_GC_Back, xa, yo, e, yh);
	    } else {
		FillRectRGB(vds->WinID, vds->BackColor, xa, yo, e, yh);
	    }
	  }
	}

	if ((vds->SelA != vds->SelE) || vds->SelToEnd) {
	    xe = x + vds->Xoff;
	    xa = 0;
	    if (vds->SelToEnd != 2) {
		a = vds->SelA;
		e = vds->SelE;
		if (a > e) SWAP_INT(a, e); //{e = a; a = vds->SelE;}

		xa = GetChar_XPos(a, vds) + x0;
		if (!vds->SelToEnd) xe = GetChar_XPos(e, vds) + x0;
	    }
	    if (xe > xa)
		Fill_Rect(vds->WinID, appXDat.Select_GC, xa, yo, xe-xa, height);
	}

	return x;
}

int TextWidthColorString(TVDrawString *vds) {
  int res;
  bool OnlyLg = vds->OnlyLg;

	vds->OnlyLg = True;
	res = DrawColorString(0, 0, vds);
	vds->OnlyLg = OnlyLg;
	return res;
}

void DrawLineString(const char *s, int x, int y, int *max_w, int *max_h, TVDrawString *vds) {
  int mx, lg, m;
  const char *pc;
  int height;

	Font_GetMetrics((TFont*)vds->Font, &height, NULL);
	*max_w = 0;
	*max_h = height;

	if (!vds || !s) return;
	mx = x;
	pc = s;
	while (*pc) {
	    if ((*pc == 10) || ((*pc == 13) && (pc[1] == 10))) {
		lg = pc - s;
		if (lg > 0) {
		    m = DrawStringSeg(x, y, lg, s, vds, vds->Font, vds->ForeColor, vds->BackColor);
		    if (m > mx) mx = m;
		}
		if (*pc == 13) pc++;
		s = pc+1;
		y += height;
		*max_h += height;
	    }
	    pc++;
	}

	lg = pclen(s);
	if (lg > 0) {
	    m = DrawStringSeg(x, y, lg, s, vds, vds->Font, vds->ForeColor, vds->BackColor);
	    if (m > mx) mx = m;
	}

	*max_w = mx;
}
