/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"

#define TFontX_handle struct TFX_fh *fh
#define TFontX_pv struct TFX_pv *fpv;
#include "appXbase.h"

typedef struct TFX_pv {
  AlgTFontPvBase
  int OriginX, OriginY;
  int XOrigin, YOrigin;
  GC gc;
  dword gcColor, gcBackColor;
  TWRect ClipRc;
} TFX_pv;

typedef struct TFX_fh {
  AlgTTC0
  XFontStruct *xfont;
  GC gc;
  dword Color, BackColor;
  TFontX *create_TFontX;
  int  lnk_count;
  bool IsDefault, ncb1, ncb2, ncb3;
} TFX_fh;

typedef struct TFX_idx {
  struct TFX_idx *next;
  TFX_fh *fh;
  char FontFileName[];
} TFX_idx;

static TFX_idx *gFX_idx;

static TFX_fh *gFX_GetHandle(const char *FontFileName) {
  TFX_idx *fti;

	fti = gFX_idx;
	while (fti && !pccmp(FontFileName, fti->FontFileName)) fti = fti->next;
	if (!fti) return NULL;
	fti->fh->lnk_count++;
	return fti->fh;
}

static TFX_idx *GetTFX_idx(TFX_fh *fh) {
  TFX_idx *fti;

	fti = gFX_idx;
	while (fti && (fti->fh != fh)) fti = fti->next;
	return fti;
}

static void gFX_AddHandle(TFX_fh *fh, const char *FontFileName) {
  TFX_idx *fti, **ppfti;
  int lg;

// printf("%s(%d): gFT_AddHandle 0x%08x %s\n", FL, (int)fh, FontFileName);
	lg = pclen(FontFileName);
	fti = xcalloc(sizeof(TFX_idx) + lg + 1);
	fti->fh = fh;
	pcnmove(fti->FontFileName, FontFileName, lg);
	ppfti = &gFX_idx;
	while (*ppfti) ppfti = &(*ppfti)->next;
	*ppfti = fti;
}

static void gFX_DelHandle(TFX_fh *fh) {
  TFX_idx *fti, **ppfti;

	ppfti = &gFX_idx;
	while (*ppfti && ((*ppfti)->fh != fh)) ppfti = &(*ppfti)->next;
	fti = *ppfti;
	if (fti) {
	    *ppfti = fti->next;
	    xfree(fti);
	}
}
/* _____________________________________________________________________ */

static int TFontX_GetWidth(TFontX *self, const char *s, int len) {
  word *pwc;
  int  result, L;
  word wcs[256];

	if (!(self->fpv->flags & FONTFG_UTF8)) {
	    return XTextWidth(self->fh->xfont, s, len);
	}

	if (len < 256) {
	    L = 256;
	    pwc =  wcs;
	} else {
	    L = len + 1;
	    pwc = xmalloc(L * 2);
	}
#ifdef _OS_WIN32
	L = Utf8ToWChars(pwc, L, s, len, 0);
#else
	L = Utf8ToWChars(pwc, L, s, len, UC_WCharBE);
#endif
	if (L <= 0) return 0;

	result = XTextWidth16(self->fh->xfont, (XChar2b*)pwc, L);
	if (pwc !=  wcs) xfree(pwc);
	return result;
}

static bool TFontX_GetMetrics(TFontX *self, int *height, int *ascent) {
  XFontStruct *xfont;

	xfont = self->fh->xfont;
	if (!xfont) return false;
	if (height) *height = xfont->ascent + xfont->descent;
	if (ascent) *ascent = xfont->ascent;
	return true;
}

static bool TFontX_PaintTo(TFontX *self, Window *WinID, int x, int y, 
			   const char *s, int len) 
{
  TFX_fh *fh;
  TFX_pv *pv;
  Window id;
  long gcm;
  GC gc;
  int L;
  word *pwc;
  XGCValues gcv;
  word  wcs[256];

	if (!WinID || !*WinID) return false;
	if ((int)*WinID == TTC_MAGIC) {
	    printf("%s(%d): ERROR: TFontX_PaintTo is TTC\n", FL);
	    return false;
	}
	id = *WinID;
	fh = self->fh;
#ifdef _OS_WIN32
	y -= fh->xfont->ascent;
#endif
	pv = self->fpv;

	gcm = 0;
	if (pv->gc) {
	    gc = pv->gc;
	    if (pv->gcColor != pv->Color)	gcm = GCForeground;
	    if (pv->gcBackColor != pv->BackColor) gcm |= GCBackground;
	    pv->gcColor     = pv->Color;
	    pv->gcBackColor = pv->BackColor;
	} else {
	    gc = fh->gc;
	    if (fh->Color != pv->Color)		gcm = GCForeground;
	    if (fh->BackColor != pv->BackColor) gcm |= GCBackground;
	    fh->Color     = pv->Color;
	    fh->BackColor = pv->BackColor;
	}
	if (!gc) return false;

	if (gcm) {
	    Fill0(&gcv, sizeof(XGCValues));
	    gcv.foreground = GetColor_RGB(pv->Color);
	    gcv.background = GetColor_RGB(pv->BackColor);
	    ChangeGC(gc, gcm, &gcv);
	}

	if (pv->flags & FONTFG_OriginAdd) {
	    x += pv->OriginX;
	    y += pv->OriginY;
	}


	if (!(pv->flags & FONTFG_UTF8)) {
// printf("%s(%d): 0x%06x 0x%06x\n", FL, (int)WinID, (int)self->fh->gc);
	    if (pv->flags & FONTFG_ImageString) {
		XDrawImageString(appXDat.dpy, id, gc, x, y, s, len);
	    } else {
		XDrawString(appXDat.dpy, id, gc, x, y, s, len);
	    }
	    if (pv->flags & FONTFG_OriginAdd) {
		pv->OriginX += XTextWidth(fh->xfont, s, len);
	    }
	} else {

	    if (len < 256) {
		L = 256;
		pwc =  wcs;
	    } else {
		L = len + 1;
		pwc = xmalloc(L * 2);
	    }
#ifdef _OS_WIN32
	    L = Utf8ToWChars(pwc, L, s, len, 0);
#else
	    L = Utf8ToWChars(pwc, L, s, len, UC_WCharBE);
#endif
	    if (L > 0) {
		if (pv->flags & FONTFG_ImageString) {
		    XDrawImageString16(appXDat.dpy, id, gc, x, y, (XChar2b*)pwc, L);
		} else {
		    XDrawString16(appXDat.dpy, id, gc, x, y, (XChar2b*)pwc, L);
		}
		if (pv->flags & FONTFG_OriginAdd) {
		    pv->OriginX += XTextWidth16(fh->xfont, (XChar2b*)pwc, L);
		}
	    }
	    if (pwc !=  wcs) xfree(pwc);
	}
	return true;
}

/* static bool TFontX_SetFlags(TFontX *self, dword flags) {
//  TFX_pv *pv;

//	pv = self->fpv;
	self->fpv->flags = flags;
	return true;
} */

/* static bool TFontX_SetColor(TFontX *self, dword Color) {
	self->fpv->Color = Color;
	return true;
}

static bool TFontX_SetBackColor(TFontX *self, dword Color) {
	self->fpv->BackColor = Color;
	return true;
} */

static bool FontX_SetDefault(TFontX *self, bool Value) {
	if (self->fh->create_TFontX != self) {
	    printf("%s(%d): ERROR FontX=0x%08lx is not base FontX=0x%08lx\n", FL, 
				(long)self, (long)self->fh->create_TFontX);
	    return false;
	}
	self->fh->IsDefault = Value;
	return true;
}

static GC CreateFontX_GC(XFontStruct *xfont, dword Color, dword BackColor) {
  long gcm;
  XGCValues gcv;

	Fill0(&gcv, sizeof(XGCValues));
	gcv.font       = xfont->fid;
	gcv.plane_mask = AllPlanes;
	gcv.function   = GXcopy;
	gcv.fill_style = FillSolid;
	gcv.foreground = GetColor_RGB(Color);
	gcv.background = GetColor_RGB(BackColor);
	gcm = GCFunction | GCPlaneMask | GCGraphicsExposures | GCLineWidth | 
	      GCForeground | GCBackground | GCFillStyle | GCFont;
	return CreateGC(gcm, &gcv);
}

static bool TFontX_SetClipRc(TFontX *self, TWRect *ClipRc) {
  TFX_pv *pv;
  XRectangle rc;
  XGCValues gcv;

	pv = self->fpv;
	if (!ClipRc || (ClipRc->width <= 0) || (ClipRc->height <= 0)) {
	    if (pv->ClipRc.width && pv->ClipRc.height && pv->gc) {
	        Fill0(&gcv, sizeof(XGCValues));
		ChangeGC(pv->gc, GCClipXOrigin|GCClipYOrigin|GCClipMask, &gcv);
	    }
	    pv->ClipRc.width = 0;
	    pv->ClipRc.height = 0;
	    return true;
	}

	if (self->fh->IsDefault && (self->fh->create_TFontX == self)) {
	    printf("%s(%d): ERROR: can not SetClipRc on base Font\n", FL);
	    return false;
	}

	pv = self->fpv;
	pv->ClipRc = *ClipRc;

	if (!pv->gc) {
	    pv->gc = CreateFontX_GC(self->fh->xfont, pv->Color, pv->BackColor);
	}
	if (!pv->gc) return false;

	rc.x = pv->ClipRc.left;
	rc.y = pv->ClipRc.top;
	rc.width = pv->ClipRc.width;
	rc.height = pv->ClipRc.height;
	XSetClipRectangles(appXDat.dpy, pv->gc, 0, 0, &rc, 1, 0);

#ifdef _OS_WIN32
// printf("%s(%d): TODO FontX_SetClipRc\n", FL);
#endif

	return true;
}

/*____________________________ Public __________________________________*/
bool FontX_PaintTo(TFontX *self, Window *WinID, int x, int y, const char *s, int len) {
	if (!self || !self->PaintTo) return false;
	return self->PaintTo(self, WinID, x, y, s, len);
}

const char *FontX_GetFontName(TFontX *self) {
  TFX_idx *fti;

	if (!self || !ttc_chk_is_wtyp(self->fh, wtyp_FontHandle_X)) return NULL;
	fti = GetTFX_idx(self->fh);
	if (!fti) return NULL;
	return fti->FontFileName;
}

XFontStruct *FontX_GetXFontStruct(TFontX *self) {
	if (!self || !ttc_chk_is_wtyp(self->fh, wtyp_FontHandle_X)) return NULL;
	return self->fh->xfont;
}

/*_____________________________ Destroy _________________________________*/
static bool TFontHandleX__Destroy(TFX_fh *self) {

	self->lnk_count--;
	if (self->lnk_count > 0) return true;
	if (self->IsDefault) return true;

	gFX_DelHandle(self);
	FreeGC(self->gc);
	if (appXDat.dpy) XFreeFont(appXDat.dpy, self->xfont);
	return false;
}

static bool TFontX__Destroy(TFontX *self) {
	if (self->fh->IsDefault && (self->fh->create_TFontX == self)) return true;
	FreeGC(self->fpv->gc);
	xfree(self->fpv);
	Tfree(self->fh);
	return false;
}

/* ____________________________ Create __________________________________ */
static TFontX *TFontX_CreateDuplicate(TFontX *orig) {
  TFX_idx *fti;
  TFontX *self;

	if (!orig) return NULL;
#if FONT_CHK_STRUCT
	if (orig->wtyp != wtyp_FontX) {
	    printf("%s(%d): ERROR: TFontX_CreateDuplicate Orig. is not TFontX\n", FL);
	    return NULL;
	}
#endif
	fti = GetTFX_idx(orig->fh);
	if (!fti) return NULL;

	self = TFontX_Create(fti->FontFileName);
	return self;
}

TFontX *TFontX_Create(const char *FontFileName) {
  TFontX *self;
  TFX_fh *fh;
  XFontStruct *xfont;

	fh = gFX_GetHandle(FontFileName);
	if (!fh) {
	    xfont = XLoadQueryFont(appXDat.dpy, FontFileName);
// printf("%s(%d): 0x%08x %s\n", FL, (int)xfont, FontFileName);
	    if (!xfont) return NULL;
	    fh = ttc_create(sizeof(TFX_fh), wtyp_FontHandle_X);
	    fh->destroy = (TDESTROY)TFontHandleX__Destroy;
	    fh->xfont = xfont;
	    fh->lnk_count = 1;
	    fh->Color = 0x000000;
	    fh->BackColor = 0xc0c0c0;
	    fh->gc = CreateFontX_GC(fh->xfont, fh->Color, fh->BackColor);
	    gFX_AddHandle(fh, FontFileName);
	}

	self = ttc_create(sizeof(TFontX), wtyp_FontX);
	self->destroy = (TDESTROY)TFontX__Destroy;
	self->fh = fh;
	if (!fh->create_TFontX) fh->create_TFontX = self;

	self->fpv = xcalloc(sizeof(TFX_pv));
	self->fpv->Color = 0x000000;
	self->fpv->BackColor = 0xc0c0c0;

	self->CreateDuplicate	= TFontX_CreateDuplicate;

//	self->SetColor		= TFontX_SetColor;
//	self->SetBackColor	= TFontX_SetBackColor;
//	self->SetFlags		= TFontX_SetFlags;
	self->SetDefault	= FontX_SetDefault;
	self->GetWidth		= TFontX_GetWidth;
	self->GetMetrics	= TFontX_GetMetrics;
	self->SetClipRc		= TFontX_SetClipRc;
//	self->ClrClipRc		= TFontX_ClrClipRc;
	self->PaintTo		= TFontX_PaintTo;
	return self;
}
