/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

//#include "appbase.h"
//#include "fontX_FT_pv.h"

#ifdef HAVE_FT

#include "appbase.h"
#include "fonts_FT_handle.h"

#define TFontX_FT_handle struct TFontHandle_FT *fh
#define TFontX_FT_pv struct TFXFT_pv *fpv;

#include "font_FT.h"

#include "appXbase.h"
#include "tpixmap.h"

#include "fonts_FT_pv.h"


typedef struct TFXFT_pv {
  AlgTFontFTPvBase
  TPixMap *back_pxm;
} TFXFT_pv;

static bool TFontX_FT_PaintTo(TFontX_FT *self, Window *WinID, int x, int y, 
			   const char *s, int len) 
{
  TFXFT_pv *pv;
  int height, ascent, width, pxm_width, lineX, lineY, lineH, lineW, ox, oy;
//  int cleft, ctop;
  TPixMap *line_pxm;
  GC gc;
  TWRect *cRc;

	if (!WinID || !*WinID) return false;
	if ((int)*WinID == TTC_MAGIC) {
	    printf("%s(%d): ERROR: TFontX_FT_PaintTo is TTC\n", FL);
	    return false;
	}

	if (!Font_GetMetrics((TFont*)self, &height, &ascent)) return false;

	lineX = 0;
	lineW = 0;
	lineY = 0;
	lineH = 0;
	y -= ascent;
	pv = self->fpv;
	cRc = &pv->ClipRc;
	if ((cRc->width > 0) && (cRc->height > 0)) {
//   printf("%s(%d): xo=%3d yo=%3d x=%3d y=%3d w=%3d h=%3d\n", FL, cRc->XOrigin, cRc->YOrigin,
//		cRc->left, cRc->top, cRc->width, cRc->height);

//	    y += cRc->YOrigin;
//	    ctop = cRc->top + cRc->YOrigin;
	    lineH = cRc->top + cRc->height - y;
	    if (lineH <= 0) return true;

	    lineY = cRc->top - y;
	    if (lineY >= height) return true;
	    if (lineY < 0) lineY = 0;

//	    x += cRc->XOrigin;
//	    cleft = cRc->left + cRc->XOrigin;
	    lineW = cRc->left + cRc->width - x;

	    lineX = cRc->left - x;
// printf("%s(%d): %d %d %d\n", FL, lineW, lineX, cRc->left);
	    if (lineX > 0) {
		lineW -= lineX;
	    } else {
		lineX = 0;
	    }
	    if (lineW <= 0) return true;
// printf("%s(%d): \n", FL);
	}

	width = Font_GetWidth((TFont*)self, s, len);
// printf("%s(%d): w=%d h=%d a=%d  x=%d y=%d\n", FL, width, height, ascent, x, y);
	if (!width) return true;
	line_pxm = self->fh->line_pxm;
	if (line_pxm && (line_pxm->width < width) && 
		        (line_pxm->width < appXDat.Root_width))
	{
// printf("%s(%d): Del %d %d\n", FL, line_pxm->width, width);
	    self->fh->line_pxm = NULL;
	    ppTfree(&line_pxm);
	}

	if (!line_pxm) {
	    pxm_width = (width + 128) & ~0x0f;
// printf("%s(%d): Add %d\n", FL, pxm_width);
	    if (pxm_width > appXDat.Root_width) pxm_width = appXDat.Root_width;
	    line_pxm = TPixMap_Create_whLD(pxm_width, height, 0);
	    TPixMap_img_malloc(line_pxm);
	    self->fh->line_pxm = line_pxm;
	}

	if (!line_pxm) return false;
// dword t = GetTime_us();

	if (pv->BackImage) {
// printf("%s(%d): x=%d y=%d a=%d %d\n", FL, x, y, ascent, pv->BackImage->height);
	    ox = x - pv->BiOriginX;
	    oy = 0;
	    if (!(pv->BiFlags & BiFlags_IsLineImage)) {
		oy = y - ascent - pv->BiOriginY;
	    }
	    image_copy((TImage*)line_pxm, 0, 0, pv->BackImage, ox, oy, width, height, 0);
	    
	} else {
	    TImage_FillRectRGB((TImage*)line_pxm, pv->BackColor, 0, 0, width, height);
	}
	if (self->ToImage) self->ToImage(self, (TImage*)line_pxm, 0, ascent, s, len);

	TPixMap_XPutImage(line_pxm);
	gc = line_pxm->gc;
	if (!gc) gc = appXDat.Default_GC;

	if (lineH) {
	    if (lineH >= line_pxm->height) lineH = line_pxm->height;
	} else {
	    lineH = line_pxm->height;
	}
	lineH -= lineY;

	if (lineX) width -= lineX;
	if (width <= 0) return true;
	if (lineW && (width > lineW)) width = lineW;

	XCopyArea(appXDat.dpy, (Drawable)line_pxm->pmxID, *WinID, gc,
		  lineX, lineY, width, lineH, x + lineX, y + lineY);
// printf("%s(%d): %dus\n", FL, GetTime_us() - t);

	return true;
}

/* static bool FontX_FT_SetClipRc(TFont_FT *self, TClipRc *ClipRc) {
	self->fpv->ClipRc = *ClipRc;
	return false;
} */

/*____________________________ Public __________________________________*/
bool FontX_FT_ToImage(TFontX_FT *self, struct TImage *img, int x, int y, const char *s, int len) {
	if (!self || !self->ToImage) return false;
	return self->ToImage(self, img, x, y, s, len);
}

bool FontX_FT_StrToImage(TFontX_FT *self, struct TImage *img, int x, int y, const char *s) {
	if (!self || !self->ToImage) return false;
	return self->ToImage(self, img, x, y, s, pclen(s));
}

bool FontX_FT_SetBackImage(TFontX_FT *self, struct TImage *BackImage,
			dword BiFlags, int BiOriginX, int BiOriginY)
{
	if (!self || (self->wtyp != wtyp_FontX_FT)) return false;
	self->fpv->BackImage = BackImage;
	self->fpv->BiFlags = BiFlags;
	self->fpv->BiOriginX = BiOriginX;
	self->fpv->BiOriginY = BiOriginY;
	return true;
}
/*_____________________________ Destroy _________________________________*/
/* static bool TFontX_FT__Destroy(TFontX_FT *self) {
	return TFontS_FT__Destroy(self);
} */
/* ____________________________ Create __________________________________ */
static TFontX_FT *TFontX_FT_CreateDuplicate(TFontX_FT *orig) {
  TFT_idx *fti;
  TFontX_FT *self;

	if (!orig) return NULL;
#if FONT_CHK_STRUCT
	if (orig->wtyp != wtyp_FontX_FT) {
	    printf("%s(%d): ERROR: TFont_FT_CreateDuplicate Orig. is not TFontX_FT\n", FL);
	    return NULL;
	}
#endif
	fti = TFontHandle_FT_Get_FT_idx(orig->fh);
	if (!fti) return NULL;

	self = TFontX_FT_Create(fti->FontFileName, fti->font_height, fti->font_dpi);
	self->fpv->flags = (orig->fpv->flags & FONTFG_UTF8);
	return self;
}

TFontX_FT *TFontX_FT_Create(const char *FontFileName, 
				int font_height, int font_dpi)
{
  TFontX_FT *self;

	self = (TFontX_FT*)TFont_FT_AlgCreate(FontFileName, 
					font_height, font_dpi,
					sizeof(TFontX_FT), wtyp_FontX_FT, 
					sizeof(TFXFT_pv));
	if (!self) return NULL;
//	self->destroy = (TDESTROY)TFontX_FT__Destroy;
//	self->fpv->Color = 0x000000;
//	self->fpv->BackColor = 0xc0c0c0;

	self->CreateDuplicate	= TFontX_FT_CreateDuplicate;

	self->PaintTo		= TFontX_FT_PaintTo;
	return self;
}

#else /* HAVE_FT */

#include "appbase.h"

TFontS *TFontImg_FT_Create(const TFontHandle_FT *fh) {
 printf("%s(%d): ERROR: no freetype support (no ft2build.h !!!)\n", FL);
	return NULL;
}

#endif /* HAVE_FT */
