/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "font.h"

bool Font_SetFlag(TFont *self, dword flag) {
	if (!self) return false;
	if (flag) {
	    if (self->SetFlag) return self->SetFlag(self, flag);
	    self->fpv->flags |= flag;
	}
	return true;
}

bool Font_ClrFlag(TFont *self, dword flag) {
	if (!self) return false;
	if (flag) {
	    if (self->ClrFlag) return self->ClrFlag(self, flag);
	    self->fpv->flags &= ~flag;
	}
	return true;
}

bool Font_SetFlags(TFont *self, dword flags) {
	if (!self) return false;
	if (self->SetFlags) return self->SetFlags(self, flags);
	self->fpv->flags = flags;
	return true;
}

bool Font_SetUTF8(TFont *self, bool UTF8on) {
	if (!self) return false;
	self->fpv->flags &= ~FONTFG_UTF8;
	if (UTF8on) self->fpv->flags |= FONTFG_UTF8;
	return true;
}

bool Font_SetColor(TFont *self, dword Color) {
	if (!self) return false;
	if (self->SetColor) return self->SetColor(self, Color);
	self->fpv->Color = Color;
	return true;
}

bool Font_SetBackColor(TFont *self, dword Color) {
	if (!self) return false;
	if (self->SetBackColor) return self->SetBackColor(self, Color);
	self->fpv->BackColor = Color;
	return true;
}

int Font_GetWidth(TFont *self, const char *s, int len) {
	if (!self || !self->GetWidth || (len <= 0)) return 0;
	return self->GetWidth(self, s, len);
}

bool Font_GetMetrics(TFont *self, int *height, int *ascent) {
	if (height) *height = 0;
	if (ascent) *ascent = 0;
	if (!self || !self->GetMetrics) return false;
	return self->GetMetrics(self, height, ascent);
}

int Font_GetGlyphs(TFont *self) {
	if (!self || !self->GetGlyphs) return 0;
	return self->GetGlyphs(self);
}

bool Font_ClrOrigin(TFont *self) {
	if (!self || !self->ClrOrigin) return false;
	return self->ClrOrigin(self);
}

bool Font_SetOrigin(TFont *self, int x, int y) {
	if (self) {
	    if (self->SetOrigin) return self->SetOrigin(self, x , y);
	    if (self->SetOriginFloat) return self->SetOriginFloat(self, x , y);
	}
	return false;
}

bool Font_SetDefault(TFont *self, bool Value) {
	if (!self || !self->SetDefault) return false;
	return self->SetDefault(self, Value);
}

bool Font_GetOrigin(TFont *self, int *x, int *y) {
  float fx, fy;

	if (x) *x = 0;
	if (y) *y = 0;
	if (!self) return false;
	if (self->GetOrigin) return self->GetOrigin(self, x , y);
	if (!self->GetOriginFloat || !self->GetOriginFloat(self, &fx , &fy)) return false;
	if (x) *x = fx;
	if (y) *y = fy;
	return true;
}

bool Font_SetOriginFloat(TFont *self, float x, float y) {
	if (self) {
	    if (self->SetOriginFloat) return self->SetOriginFloat(self, x , y);
	    if (self->SetOrigin) return self->SetOrigin(self, x , y);
	}
	return false;
}

bool Font_GetOriginFloat(TFont *self, float *x, float *y) {
  int ix, iy;

	if (x) *x = 0;
	if (y) *y = 0;
	if (!self) return false;
	if (self->GetOriginFloat) return self->GetOriginFloat(self, x , y);
	if (!self->GetOrigin || !self->GetOrigin(self, &ix , &iy)) return false;
	if (x) *x = (float)ix;
	if (y) *y = (float)iy;
	return true;
}

bool Font_GetVector(TFont *self, int *x, int *y, const char *s, int len) {
  float fx, fy;

	if (x) *x = 0;
	if (y) *y = 0;
	if (!self) return false;
	if (self->GetVector) return self->GetVector(self, x , y, s, len);
	if (!self->GetVectorFloat || !self->GetVectorFloat(self, &fx , &fy, s, len)) return false;
	if (x) *x = (int)fx;
	if (y) *y = (int)fy;
	return true;
}

bool Font_StrGetVector(TFont *self, int *x, int *y, const char *s) {
	return Font_GetVector(self, x, y, s, pclen(s));
}

bool Font_GetVectorFloat(TFont *self, float *x, float *y, const char *s, int len) {
  int ix, iy;

	if (x) *x = 0;
	if (y) *y = 0;
	if (!self) return false;
	if (self->GetVectorFloat) return self->GetVectorFloat(self, x , y, s, len);
	if (!self->GetVector || !self->GetVector(self, &ix , &iy, s, len)) return false;
	if (x) *x = (float)ix;
	if (y) *y = (float)iy;
	return true;
}

bool Font_StrGetVectorFloat(TFont *self, float *x, float *y, const char *s) {
	return Font_GetVectorFloat(self, x, y, s, pclen(s));
}

bool Font_SetClipRc(TFont *self, TWRect *ClipRc) {
	if (!self || !self->SetClipRc) return false;
	return self->SetClipRc(self, ClipRc);
}

bool Font_ClrClipRc(TFont *self) {
	return Font_SetClipRc(self, NULL);
}

bool Font_ChkPaintToIsImage(long *toobj) {
  TTC0 *tc;
	tc = (TTC0*)toobj;
	if ((tc->magic == TTC_MAGIC) && 
	    (tc->wtcsize > sizeof(TTC0)) &&
	    (tc->wtcsize <= TTC_MAXDEFAULDSIZE))
	{
	    if ((tc->wtyp & wtyp_TImage_MASK) == wtyp_TImage) return true;
 printf("%s(%d): ERROR: Font_PaintTo no TImage\n", FL);
	}
	return false;
}

bool Font_PaintTo(TFont *self, long *toobj, int x, int y, const char *s, int len) {

	if (!self || !toobj) return false;
	if (Font_ChkPaintToIsImage(toobj) && self->NC_ToImage) {
	    return self->NC_ToImage(self, toobj, x, y, s, len);
	}

	if (!self->PaintTo) return false;
	return self->PaintTo(self, toobj, x, y, s, len);
}

bool Font_StrPaintTo(TFont *self, long *toobj, int x, int y, const char *s) {
	return Font_PaintTo(self, toobj, x, y, s, pclen(s));
}

TFont *Font_CreateDuplicate(TFont *self) {
	if (!self || !self->CreateDuplicate) return NULL;
	return self->CreateDuplicate(self);
}

bool Font_PointX_GetCharData(TFont *self, TFontCharData *fcd, int x, 
					const char *s, int len) 
{
  int ascent, i;

	if (!self || !fcd) return false;
	Fill0(fcd, sizeof(TFontCharData));
	fcd->char_s = s;
	if (!Font_GetMetrics(self, &fcd->char_height, &ascent)) return false;
	fcd->isUTF8 = (self->fpv->flags & FONTFG_UTF8) != 0;
	if (!s || (len <= 0)) return true;

	i = 0;
	while (i < len) {
	    fcd->char_count = 1;
	    if (fcd->isUTF8) fcd->char_count = get_Utf8Chars(fcd->char_s);
	    fcd->char_width = Font_GetWidth(self, fcd->char_s, fcd->char_count);
// printf("%s(%d): %d %d %d\n", FL, x, wlcd->char_x, wlcd->char_width);
	    if (x < (fcd->char_x + fcd->char_width - (fcd->char_width / 4))) break;
	    fcd->char_x   += fcd->char_width;
	    fcd->char_s   += fcd->char_count;
	    fcd->char_idx += fcd->char_count;
	    i += fcd->char_count;
	    fcd->char_width = 0;
	    fcd->char_count = 0;
	}
	return true;
}

bool Font_StrIdx_GetCharData(TFont *self, TFontCharData *fcd, 
				const char *s, int idx) 
{
	if (!self || !fcd) return false;
	Fill0(fcd, sizeof(TFontCharData));
	fcd->char_idx = idx;
	fcd->char_s = s + idx;
	if (!Font_GetMetrics(self, &fcd->char_height, NULL)) return false;
	fcd->isUTF8 = (self->fpv->flags & FONTFG_UTF8) != 0;
	if (!s) return true;
	fcd->char_x = Font_GetWidth(self, s, idx);
	fcd->char_count = 1;
	if (fcd->isUTF8) fcd->char_count = get_Utf8Chars(fcd->char_s);
	fcd->char_width = Font_GetWidth(self, fcd->char_s, fcd->char_count);
	return true;
}
