/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#ifdef HAVE_FT

#include "appbase.h"

#include "fonts_FT_handle.h"

#define TFont_FT_handle struct TFontHandle_FT *fh
#define TFont_FT_pv struct TFFT_pv *fpv;
#include "font_FT.h"

#include "fonts_FT_pv.h"
#include <math.h>

typedef struct TFFT_pv {
  AlgTFontFTPvBase
} TFFT_pv;

static bool chk_and_set(TFont_FT *self, TImage *img) {
  TFFT_pv *pv;
  TFontHandle_FT *fh;
  TFCache *fcache_glyph;
  FT_GlyphSlot  slot;
  int w, m, sbpl;
  int dLeft, dTop, dmaxW, dmaxH;

	fh = self->fh;
	pv = self->fpv;
	fcache_glyph = fh->fcache_glyph;
	if (fcache_glyph) {
	    if (!fcache_glyph->data) return false;
	    w = fcache_glyph->bitmap_width;
	    pv->grows = fcache_glyph->bitmap_rows;
	    pv->sb0   = fcache_glyph->data;
	} else {
	    slot = fh->face->glyph;
	    w    = slot->bitmap.width;
	    pv->grows = slot->bitmap.rows;
	    pv->sb0   = slot->bitmap.buffer;
	}

	sbpl = w;
	if (fh->fixes_size) sbpl = ((sbpl + 31) & ~31) >> 3;

	dLeft = 0;
	dTop = 0;
	dmaxH = img->height;
	dmaxW = img->width;
	if (pv->chk_ClipRc) {
	    dLeft = pv->ClipRc.left;
	    if (pv->ClipRc.left > (pv->ixo + w)) return false;

	    dTop = pv->ClipRc.top;
	    if (dTop >= (pv->iyo + pv->grows)) return false;
	    if (dTop > pv->iyo) {
		m = dTop - pv->iyo;
		pv->grows -= m;
		pv->sb0 += sbpl * m;
		pv->iyo = dTop;
	    }

	    if ((pv->ClipRc.left + pv->ClipRc.width) < dmaxW)
		dmaxW = pv->ClipRc.left + pv->ClipRc.width;

	    if ((pv->ClipRc.top + pv->ClipRc.height) < dmaxH) 
		dmaxH = pv->ClipRc.top + pv->ClipRc.height;
	}

	if ((pv->iyo + pv->grows) > dmaxH) pv->grows = dmaxH - pv->iyo;
	if ((pv->grows) <= 0) return false;

	if ((pv->ixo + w) > dmaxW) w = dmaxW - pv->ixo;
	if (fh->fixes_size) {
	    pv->fixes_o = 8;
	    if (pv->ixo < dLeft) {
		m = dLeft - pv->ixo;
		pv->sb0 += (m & ~0x07) >> 3;
		pv->fixes_o = -m & 0x07;
		if (!pv->fixes_o) pv->fixes_o = 8;
		pv->ixo = dLeft;
	    }
	    pv->fixes_o_shift = 8 - pv->fixes_o;

	} else {
	    if (pv->ixo < dLeft) {
		w -= dLeft - pv->ixo;
		pv->sb0 += dLeft - pv->ixo;
		pv->ixo = dLeft;
	    }
	}
	pv->sbpl = sbpl;
	pv->gwidth = w;
	return w >= 0;
}

static void REGP3 glyph_argb16(TFont_FT *self, TImage *img) {
  TFFT_pv *pv;
  int x, y, yl, sbpl;
  byte *sb0, *sb, *db0;
  word *dw;
  dword A, A2, R, G, B;

	if (!chk_and_set(self, img)) return;
	pv = self->fpv;
	sbpl = pv->sbpl;
	sb0 = pv->sb0;

	yl = pv->iyo;
	db0 = img->data + (yl * img->bytes_per_line);
	for (y=pv->grows;y>0;y--) {
	    if ((dword)yl < (dword)img->height) {
		sb = sb0;
		dw = ((word*)db0) + pv->ixo;
		for (x=pv->gwidth;x>0;x--) {
		    A = *sb;
		    if (A >= 0xf8) {
			*dw = pv->col;
		    } else if (A >= 0x08) {
			A2 = 0x100 - A;
			R = ((pv->R * A) + ((*dw & 0xf800) * A2)) & 0xf80000;
			G = ((pv->G * A) + ((*dw & 0x07e0) * A2)) & 0x07e000;
			B = ((pv->B * A) + ((*dw & 0x001f) * A2)) & 0x001f00;
			*dw = (R | G | B) >> 8;
		    }
		    sb++; dw++;
		}
	    }
	    sb0 += sbpl;
	    db0 += img->bytes_per_line;
	    yl++;
	}
}

static void REGP3 glyph_fixed16(TFont_FT *self, TImage *img) {
  TFFT_pv *pv;
  int x, y, sbpl, dbpl, bc;
  byte *sb0, *sb, *db0;
  word *dw;
  dword sc, yl, dmaxH;

	if (!chk_and_set(self, img)) return;
	pv = self->fpv;
	sbpl = pv->sbpl;
	sb0 = pv->sb0;

	dmaxH = (dword)img->height;
	dbpl = img->bytes_per_line;
	yl = (dword)pv->iyo;

	db0 = img->data + (yl * dbpl);
	for (y=pv->grows;y>0;y--) {
	    if (yl < dmaxH) {
		sb = sb0;
		dw = ((word*)db0) + pv->ixo;
		bc = pv->fixes_o;
		sc = *sb; sb++;
		if (pv->fixes_o_shift) sc <<= pv->fixes_o_shift;
		for (x=pv->gwidth;x>0;x--) {
		    if (!bc) {sc = *sb; sb++; bc = 8;}
		    if (sc & 0x80) *dw = pv->col;
		    sc <<= 1; bc--; dw++;
		}
	    }
	    sb0 += sbpl; db0 += dbpl; yl++;
	}
}

static void REGP3 glyph_argb32(TFont_FT *self, TImage *img) {
  TFFT_pv *pv;
  int x, y, yl, sbpl;
  byte *sb0, *sb, *db0;
  dword *dw;
  dword A, A2, R, G, B;

	if (!chk_and_set(self, img)) return;
	pv = self->fpv;
	sbpl = pv->sbpl;
	sb0 = pv->sb0;

	yl = pv->iyo;
	db0 = img->data + (yl * img->bytes_per_line);
	for (y=pv->grows;y>0;y--) {
	    if ((dword)yl < (dword)img->height) {
		sb = sb0;
		dw = ((dword*)db0) + pv->ixo;
		for (x=pv->gwidth;x>0;x--) {
		    A = *sb;
		    if (A >= 0xf8) {
			*dw = pv->col;
		    } else if (A >= 0x08) {
			A2 = 0x100 - A;
			R = ((pv->R * A) + ((*dw & 0xff0000) * A2)) & 0xff000000;
			G = ((pv->G * A) + ((*dw & 0x00ff00) * A2)) & 0x00ff0000;
			B = ((pv->B * A) + ((*dw & 0x0000ff) * A2)) & 0x0000ff00;
			*dw = (R | G | B) >> 8;
		    }
		    sb++; dw++;
		}
	    }
	    sb0 += sbpl;
	    db0 += img->bytes_per_line;
	    yl++;
	}
}

static void REGP3 glyph_fixed32(TFont_FT *self, TImage *img) {
  TFFT_pv *pv;
  int x, y, sbpl, dbpl, bc;
  byte *sb0, *sb, *db0;
  dword *dw;
  dword sc, yl, dmaxH, col;

	if (!chk_and_set(self, img)) return;
	pv = self->fpv;
	sbpl = pv->sbpl;
	sb0 = pv->sb0;

	col = pv->col;
	dmaxH = (dword)img->height;
	dbpl = img->bytes_per_line;
	yl = (dword)pv->iyo;

	db0 = img->data + (yl * dbpl);
	for (y=pv->grows;y>0;y--) {
	    if (yl < dmaxH) {
		sb = sb0;
		dw = ((dword*)db0) + pv->ixo;
		bc = pv->fixes_o;
		sc = *sb; sb++;
		if (pv->fixes_o_shift) sc <<= pv->fixes_o_shift;
		for (x=pv->gwidth;x>0;x--) {
		    if (!bc) {sc = *sb; sb++; bc = 8;}
		    if (sc & 0x80) *dw = col;
		    sc <<= 1; bc--; dw++;
		}
	    }
	    sb0 += sbpl; db0 += dbpl; yl++;
	}
}

static bool TFont_FT_ToImage(TFont_FT *self, TImage *img, int x, int y, 
			      const char *s, int len) 
{
  TFontHandle_FT *fh;
  TFFT_pv *pv;
  int cdec;
  dword c;
  const char *cpc;
  int bitmap_left, bitmap_top, advance;
//  TFCache *fcache_glyph;
  FT_GlyphSlot  slot;
  FT_Vector pen;

	if (!img) return false;
	if (img->magic != TTC_MAGIC) {
	    printf("%s(%d): ERROR: TFont_FT_ToImage is not TTC\n", FL);
	    return false;
	}

	if ((img->width <= 0) || (img->height <= 0)) return false;
	if (len <= 0) len = pclen(s);
	if (len <= 0) return true;

	pv = self->fpv;
	fh = self->fh;

	pv->chk_ClipRc = ((TImage*)fh->line_pxm != img) &&
			 (pv->ClipRc.width > 0) &&
			 (pv->ClipRc.height > 0);
//  printf("%s(%d): chk_ClipRc=%d\n", FL, pv->chk_ClipRc);

	if ((img->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) {
	    pv->to_img = glyph_argb16;
	    if (fh->fixes_size) pv->to_img = glyph_fixed16;
	    pv->R = (pv->Color >> 8) & 0xf800;
	    pv->G = (pv->Color >> 5) & 0x07e0;
	    pv->B = (pv->Color >> 3) & 0x001f;
	    pv->col = pv->R | pv->G | pv->B;
	} else {
	    pv->to_img = glyph_argb32;
	    if (fh->fixes_size) pv->to_img = glyph_fixed32;
	    pv->col = pv->Color;
	    pv->R = pv->col & 0xff0000;
	    pv->G = pv->col & 0x00ff00;
	    pv->B = pv->col & 0x0000ff;
	}

	pen.x = 0;
	pen.y = 0;
	FT_Set_Transform(fh->face, &pv->matrix, &pen);

	if (pv->flags & FONTFG_OriginAdd) {
	    pen.x = pv->pen.x;
	    pen.y = pv->pen.y;
//	} else {
//	    pen.x = 0;
//	    pen.y = 0;
	}

	slot = fh->face->glyph;
	cpc = s;
	cdec = 1;
	if (pv->use_angle) {
	    while (len > 0) {
		fh->fcache_glyph = NULL;
		if (pv->flags & FONTFG_UTF8) {
		    c = Utf8CharToWChar_rlg(cpc, &cdec);
		} else {
		    c = (byte)*cpc;
		}
		FT_Set_Transform(fh->face, &pv->matrix, &pen);
		if (!FT_Load_Char(fh->face, c, FT_LOAD_RENDER)) {
		    pv->ixo = x  + slot->bitmap_left;
		    pv->iyo = y - slot->bitmap_top;
		    if (pv->iyo < img->height) {
			pv->to_img(self, img);
		    }
		    pen.x += slot->advance.x;
		    pen.y += slot->advance.y;
		}
		cpc += cdec;
		len -= cdec;
	    }

	} else {
	    while (len > 0) {
		if (pv->flags & FONTFG_UTF8) {
		    c = Utf8CharToWChar_rlg(cpc, &cdec);
		} else {
		    c = (byte)*cpc;
		}

		if (TFontHandle_FT_Load_Glyph(fh, c)) {
		    if (fh->fcache_glyph) {
			bitmap_left = fh->fcache_glyph->bitmap_left;
			bitmap_top = fh->fcache_glyph->bitmap_top;
			advance = fh->fcache_glyph->advance;
		    } else {
			bitmap_left = slot->bitmap_left;
			bitmap_top = slot->bitmap_top;
			advance = slot->advance.x;
		    }
		    pv->ixo = x + (pen.x / 64) + bitmap_left;
		    pv->iyo = y + (pen.y / 64) - bitmap_top;
		    if (pv->iyo < img->height) {
			pv->to_img(self, img);
		    }
		    pen.x += advance;
		}
		cpc += cdec;
		len -= cdec;
	    }
	}

	if (pv->flags & FONTFG_OriginAdd) {
	    pv->pen.x = pen.x;
	    pv->pen.y = pen.y;
	}
	return true;
}

/* _____________________________________________________________________ */
static int GetAdvance(TFontHandle_FT *self, dword c) {
	if (!TFontHandle_FT_Load_Glyph(self, c)) return 0;
	if (self->fcache_glyph) return self->fcache_glyph->advance;
	return self->face->glyph->advance.x;
}

static bool TFont_FT_GetVectorFloat(TFont_FT *self, float *x, float *y, 
				const char *s, int len) 
{
  TFontHandle_FT *fh;
  TFFT_pv *pv;
  int cdec;
  dword c;
  const char *cpc;
  FT_GlyphSlot  slot;
  FT_Vector pen;
  bool UTF8, use_angle;

	if (len <= 0) len = pclen(s);
	if (len <= 0) return true;

	pv = self->fpv;
	fh = self->fh;
	pen.x = 0;
	pen.y = 0;
	FT_Set_Transform(fh->face, &pv->matrix, &pen);

	slot = fh->face->glyph;
	UTF8 = (pv->flags & FONTFG_UTF8) != 0;
	use_angle = pv->use_angle;
	cpc = s;
	cdec = 1;
	while (len > 0) {
	    if (UTF8) {
		c = Utf8CharToWChar_rlg(cpc, &cdec);
	    } else {
		c = (byte)*cpc;
	    }
	    if (use_angle) {
		FT_Set_Transform(fh->face, &pv->matrix, &pen);
		if (!FT_Load_Char(fh->face, c, FT_LOAD_RENDER)) {
		    pen.x += slot->advance.x;
		    pen.y += slot->advance.y;
		}
	    } else {
		pen.x += GetAdvance(fh, c);
	    }
	    cpc += cdec;
	    len -= cdec;
	}
	if (x) *x = (float)pen.x / 64.0;
	if (y) *y = (float)pen.y / 64.0;
	return true;
}

static int TFont_FT_GetWidth(TFont_FT *self, const char *s, int len) {
  float x;
	x = 0;
	if (!TFont_FT_GetVectorFloat(self, &x, NULL, s, len)) return 0;
	return (int) x;
}

/*static bool TFont_FT_GetVectorFloat(TFont_FT *self, float *x, float *y, 
				const char *s, int len) 
{
	return TFont_FT_GetVector(self, x, y, s, len);
} */

/* _____________________________________________________________________ */
static void SetMatrix(TFFT_pv *self, float angle) {
  float gw;
	self->angle = angle;
	self->use_angle = ((angle > 0.01) || (angle < -0.01));
	gw = angle * M_PI_2 / 90;
	self->matrix.xx = (FT_Fixed)( cosf(gw) * 0x10000);
	self->matrix.xy = (FT_Fixed)(-sinf(gw) * 0x10000);
	self->matrix.yx = (FT_Fixed)( sinf(gw) * 0x10000);
	self->matrix.yy = (FT_Fixed)( cosf(gw) * 0x10000);
}

static bool TFont_FT_SetColor(TFont_FT *self, dword Color) {
	self->fpv->Color = Color;
	return true;
}

static bool TFont_FT_SetBackColor(TFont_FT *self, dword Color) {
	self->fpv->BackColor = Color;
	return true;
}

static bool TFont_FT_SetFlags(TFont_FT *self, dword flags) {
	self->fpv->flags = flags;
	return true;
}

static bool TFont_FT_SetDefault(TFont_FT *self, bool Value) {
	if (self->fh->create_TFont_FT != self) {
	    printf("%s(%d): ERROR Font_FT=0x%08lx is not base Font_FT=0x%08lx\n", FL, 
				(long)self, (long)self->fh->create_TFont_FT);
	    return false;
	}
	self->fh->IsDefault = Value;
	return true;
}

/*
static bool TFont_FT_SetUTF8(TFont_FT *self, bool UTF8on) {
	self->UTF8on = UTF8on;
	return true;
}

static bool TFont_FT_SetAngle(TFont_FT *self, float angle) {
//	if (!self || self->fixes_size) return False;
	if (self->fixes_size) return False;
	if (self->angle != angle) {
	    SetMatrix(self, angle);
	}
	return true;
}
*/
static bool TFont_FT_ClrOrigin(TFont_FT *self) {
	self->fpv->pen.x = 0;
	self->fpv->pen.y = 0;
//	FT_Set_Transform(self->fh->face, &self->fpv->matrix, &self->fpv->pen);
	return true;
}

static bool TFont_FT_SetOriginFloat(TFont_FT *self, float x, float y) {
	self->fpv->pen.x = (FT_Fixed)(x * 64);
	self->fpv->pen.y = (FT_Fixed)(y * 64);
//	FT_Set_Transform(self->fh->face, &self->fpv->matrix, &self->fpv->pen);
	return true;
}

static bool TFont_FT_GetOrigin(TFont_FT *self, float *x, float *y) {
  TFFT_pv *pv = self->fpv;
	if (x) *x = (float)pv->pen.x / 64.0;
	if (y) *y = (float)pv->pen.y / 64.0;
	return true;
}

static bool TFont_FT_GetMetrics(TFont_FT *self, int *height, int *ascent) {
  TFontHandle_FT *fh = self->fh;
	if (height) *height = fh->metrics_height;
	if (ascent) *ascent = fh->metrics_ascent;
	return true;
}

static int TFont_FT_GetGlyphs(TFont_FT *self) {
	return self->fh->glyphs;
}

static bool TFont_FT_SetClipRc(TFont_FT *self, TWRect *ClipRc) {
  TFFT_pv *pv;

	pv = self->fpv;
	if (!ClipRc || (ClipRc->width <= 0) || (ClipRc->height <= 0)) {
	    pv->ClipRc.width = 0;
	    pv->ClipRc.height = 0;
	    return true;
	}

	if (self->fh->IsDefault && (self->fh->create_TFont_FT == self)) {
//	if (self->fh->create_TFont_FT == self) {
	    printf("%s(%d): ERROR: can not SetClipRc on base default Font\n", FL);
	    return false;
	}

//   printf("%s(%d): xo=%3d yo=%3d x=%3d y=%3d w=%3d h=%3d\n", FL, pv->ClipRc.XOrigin, pv->ClipRc.YOrigin,
//		pv->ClipRc.left, pv->ClipRc.top, pv->ClipRc.width, pv->ClipRc.height);

	pv->ClipRc = *ClipRc;
// printf("%s(%d): TODO Font_FT_SetClipRc\n", FL);
	return false;
}

/*____________________________ Public __________________________________*/
bool Font_FT_ToImage(TFont_FT *self, struct TImage *img, int x, int y, const char *s, int len) {
	if (!self || !self->ToImage) return false;
	return self->ToImage(self, img, x, y, s, len);
}

bool Font_FT_StrToImage(TFont_FT *self, struct TImage *img, int x, int y, const char *s) {
	if (!self || !self->ToImage) return false;
	return self->ToImage(self, img, x, y, s, pclen(s));
}

/*_____________________________ Destroy _________________________________*/
static bool TFont_FT__Destroy(TFont_FT *self) {
  TFFT_pv *pv;
  TFontHandle_FT *fh;

	fh = self->fh;
	if (fh->IsDefault && (fh->create_TFont_FT == self)) return true;

	pv = self->fpv;
	xfree(pv);
	Tfree(fh);
	return false;
}

/* ____________________________ Create __________________________________ */
static TFont_FT *TFont_FT_CreateDuplicate(TFont_FT *orig) {
  TFT_idx *fti;
  TFont_FT *self;

	if (!orig) return NULL;
#if FONT_CHK_STRUCT
	if ((orig->wtyp != wtyp_Font_FT) && (orig->wtyp != wtyp_FontX_FT)) {
	    printf("%s(%d): ERROR: TFont_FT_CreateDuplicate Orig. is not TFont_FT\n", FL);
	    return NULL;
	}
#endif
	fti = TFontHandle_FT_Get_FT_idx(orig->fh);
	if (!fti) return NULL;

	self = TFont_FT_Create(fti->FontFileName, fti->font_height, fti->font_dpi);
	return self;
}

TFont_FT *TFont_FT_AlgCreate(const char *FontFileName, int font_height, int font_dpi,
			 int tcsize, int wtyp, int pvsize) {
  TFontHandle_FT *fh;
  TFont_FT *self;
  TFFT_pv *pv;

	fh = TFontHandle_FT_Create(FontFileName, font_height, font_dpi);
	if (!fh) return NULL;
	if (!tcsize) tcsize = sizeof(TFont_FT);
	if (!wtyp) wtyp = wtyp_Font_FT;
	if (!pvsize) pvsize = sizeof(TFFT_pv);

	self = ttc_create(tcsize, wtyp);
	self->destroy = (TDESTROY)TFont_FT__Destroy;
	self->fh = fh;
	if (!fh->create_TFont_FT) fh->create_TFont_FT = self;
	pv = xcalloc(pvsize);
	self->fpv = pv;
	pv->Color = 0x000000;
	pv->BackColor = 0xc0c0c0;

	SetMatrix(pv, 0.0);

	self->CreateDuplicate	= TFont_FT_CreateDuplicate;

	self->SetColor		= TFont_FT_SetColor;
	self->SetBackColor	= TFont_FT_SetBackColor;

	self->SetFlags		= TFont_FT_SetFlags;
	self->SetDefault	= TFont_FT_SetDefault;
	self->GetWidth		= TFont_FT_GetWidth;
	self->GetMetrics	= TFont_FT_GetMetrics;
	self->GetVectorFloat	= TFont_FT_GetVectorFloat;

	self->ClrOrigin 	= TFont_FT_ClrOrigin;
	self->SetOriginFloat	= TFont_FT_SetOriginFloat;
	self->GetOriginFloat	= TFont_FT_GetOrigin;
	self->SetClipRc		= TFont_FT_SetClipRc;

	self->PaintTo		= TFont_FT_ToImage;
	self->ToImage		= TFont_FT_ToImage;
	self->GetGlyphs		= TFont_FT_GetGlyphs;

	return self;
}

TFont_FT *TFont_FT_Create(const char *FontFileName, int font_height, int font_dpi) {
	return TFont_FT_AlgCreate(FontFileName, font_height, font_dpi,
		sizeof(TFont_FT), wtyp_Font_FT, sizeof(TFFT_pv));
}

#else /* HAVE_FT */

TFont_FT *TFont_FT_Create(const char *FontFileName, int font_height, int font_dpi) {
 printf("%s(%d): ERROR: no freetype support (no ft2build.h !!!)\n", FL);
	return NULL;
}

#endif /* ! HAVE_FT */
