/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "fonts_FT_handle.h"
//#include "fonts.h"
//#include "fonts_FT_pv.h"

#ifdef HAVE_FT

static TFT_idx *gFT_idx;

static TFontHandle_FT *gFT_GetHandle(const char *FontFileName, int font_height, int font_dpi) {
  TFT_idx *fti;

	fti = gFT_idx;
	while (fti) {
	    if (pccmp(FontFileName, fti->FontFileName) &&
		(font_height == fti->font_height) &&
		(font_dpi == fti->font_dpi))
	    {
		fti->fh->lnk_count++;
		return fti->fh;
	    }
	    fti = fti->next;
	}
	return NULL;
}

static void gFT_AddHandle(TFontHandle_FT *fh, const char *FontFileName, int font_height, int font_dpi) {
  TFT_idx *fti, **ppfti;
  int lg;

// printf("%s(%d): gFT_AddHandle 0x%08x %s\n", FL, (int)fh, FontFileName);
	lg = pclen(FontFileName);
	fti = xcalloc(sizeof(TFT_idx) + lg + 1);
	fti->fh = fh;
	pcnmove(fti->FontFileName, FontFileName, lg);
	fti->font_height = font_height;
	fti->font_dpi = font_dpi;
	fti->fh = fh;
	ppfti = &gFT_idx;
	while (*ppfti) ppfti = &(*ppfti)->next;
	*ppfti = fti;
}

static void gFT_DelHandle(TFontHandle_FT *fh) {
  TFT_idx *fti, **ppfti;

	ppfti = &gFT_idx;
	while (*ppfti && ((*ppfti)->fh != fh)) ppfti = &(*ppfti)->next;
	fti = *ppfti;
	if (fti) {
	    *ppfti = fti->next;
	    xfree(fti);
	}
}

/* _____________________________________________________________________ */
static void TFontHandle_FT_Add_GlyphToCache(TFontHandle_FT *self, int glyph_index) {
  TFCache *fcache;
  FT_GlyphSlot slot;
  int i;

	slot = self->face->glyph;
	fcache = &self->fcache[glyph_index];
	fcache->advance = slot->advance.x;
	if ((fcache->advance & 0x3f) == 0) {
	    fcache->bitmap_width = slot->bitmap.width;
	    fcache->bitmap_rows = slot->bitmap.rows;
	    fcache->bitmap_left = slot->bitmap_left;
	    fcache->bitmap_top  = slot->bitmap_top;
	    if (self->fixes_size) {
		i = (((fcache->bitmap_width + 31) & ~31) >> 3);
	    } else {
		i = fcache->bitmap_width;
	    }
	    fcache->data = pndup(slot->bitmap.buffer, i * fcache->bitmap_rows);
	}
}

bool REGP3 TFontHandle_FT_Load_Glyph(TFontHandle_FT *self, dword c) {
  int gidx;

	self->fcache_glyph = NULL;
#if TFGIDX_MAX > 0
	if (c < TFGIDX_MAX) {
	    if (self->glyph_idx[c] >= 0) {
		gidx = self->glyph_idx[c];
	    } else {
		gidx = FT_Get_Char_Index(self->face, c);
		self->glyph_idx[c] = gidx;
	    }
	} else {
	    gidx = FT_Get_Char_Index(self->face, c);
	}
#else
	gidx = FT_Get_Char_Index(self->face, c);
#endif
	if ((self->fcache_max > gidx) && (self->fcache[gidx].advance >= 0)) {
	    self->fcache_glyph = &self->fcache[gidx];
	    return true;
	}
// printf("%s(%d): 0x%04x\n", FL, c);
	if (FT_Load_Glyph(self->face, gidx, FT_LOAD_RENDER)) return false;
	if ((self->fcache_max > gidx) && (self->fcache[gidx].advance < 0)) {
	    TFontHandle_FT_Add_GlyphToCache(self, gidx);
	}
	return true;
}

TFT_idx *TFontHandle_FT_Get_FT_idx(TFontHandle_FT *self) {
  TFT_idx *fti;

	if (!self) return NULL;
#if FONT_CHK_STRUCT
	if (self->wtyp != wtyp_FontHandle_FT) {
	    printf("%s(%d): ERROR: TFontHandle_FT_Get_FT_idx() is not wtyp_FontHandle_FT\n", FL);
	    return NULL;
	}
#endif
	fti = gFT_idx;
	while (fti && (fti->fh != self)) fti = fti->next;
	return fti;
}
/*_____________________________ Destroy _________________________________*/
static bool TFontHandle_FT__Destroy(TFontHandle_FT *self) {
  int i;
  TFCache *fcache;

	self->lnk_count--;
	if (self->lnk_count > 0) return true;

	if (self->IsDefault) {
	    return true;
	}

	gFT_DelHandle(self);
	fcache = self->fcache;
	if (fcache) {
	    for (i=self->fcache_max;i>0;i--) {
		xfree(fcache->data);
		fcache++;
	    }
	    xfree(self->fcache);
	}
	Tfree(self->line_pxm);
	if (self->face) FT_Done_Face(self->face);
	if (self->library) FT_Done_FreeType(self->library);
	return False;
}

/* ____________________________ Create __________________________________ */
/* TFontHandle_FT *TFontHandle_FT_CreateDuplicate(TFontHandle_FT *orig) {

	if (!orig) return NULL;
	if (orig->wtyp != wtyp_FontHandle_FT) {
	    printf("%s(%d): ERROR: TFontHandle_FT_CreateDuplicate Orig. is not wtyp_FontHandle_FT\n", FL);
	    return NULL;
	}
	return orig;
} */

TFontHandle_FT *TFontHandle_FT_Create(const char *FontFileName, int font_height, int font_dpi) {
  TFontHandle_FT *self;
  FT_Error err;
  int i;
  TFCache *fcache;

	self = gFT_GetHandle(FontFileName, font_height, font_dpi);
	if (self) return self;

	self = ttc_create(sizeof(TFontHandle_FT), wtyp_FontHandle_FT);
	self->destroy = (TDESTROY)TFontHandle_FT__Destroy;
	self->lnk_count = 1;

	err = FT_Init_FreeType(&self->library);  /* initialize library */
	if (err) {
	    printf("%s(%d): ERROR: FT library initialization\n", FL);
end_err:
	    TFontHandle_FT__Destroy(self);
	    xfree(self);
	    return NULL;
	}

	err = FT_New_Face(self->library, FontFileName, 0, &self->face);  
	if (err == FT_Err_Unknown_File_Format) {
	    printf("%s(%d): ERROR: FT font format is unsupported %s\n", FL, FontFileName);
	    goto end_err;
	} else if (err) {
	    printf("%s(%d): ERROR: FT file could not opened %s\n", FL, FontFileName);
	    goto end_err;
	}

// printf("%s(%d): %s\n", FL, self->face->family_name);
// printf("%s(%d): %s\n", FL, self->face->style_name);

	self->fixes_size = FT_IS_SCALABLE(self->face) == 0;
	if (font_height <= 0) {
	    font_height = 12;
	    if (self->fixes_size) font_height = self->face->available_sizes->y_ppem / 64;
	}
	self->font_height = font_height;
	if (font_dpi > 0) {
	    self->font_dpi = font_dpi;
	    err = FT_Set_Char_Size(self->face, font_height * 64, 0, font_dpi, 0);
	} else {
	    err = FT_Set_Pixel_Sizes(self->face, 0, font_height);
	}
	if (err) printf("%s(%d): ERROR: FT Set Pixel Sizes %d\n", FL, font_height);

	self->glyphs = self->face->num_glyphs;
	self->metrics_height = self->face->size->metrics.height / 64;
	self->metrics_ascent = self->face->size->metrics.ascender / 64;

	i = FT_Get_Char_Index(self->face, '_');
	if (!FT_Load_Glyph(self->face, i, FT_LOAD_RENDER)) {
	    FT_GlyphSlot slot = self->face->glyph;
	    i = self->metrics_ascent - slot->bitmap_top + slot->bitmap.rows;
	    if (i > self->metrics_height) self->metrics_height++;
	}
// printf("%s(%d): %ld %d %d\n", FL, self->glyphs, self->metrics_height, self->metrics_ascent);
#if TFGIDX_MAX > 0
	for (i=0;i<TFGIDX_MAX;i++) self->glyph_idx[i] = -1;
#endif

#if TFCACHE_MAX == 0
	self->fcache_max = 0;
#else
#if TFCACHE_MAX < 128
# error "TFCACHE_MAX < 128"
#endif
	self->fcache_max = self->glyphs;
	if (self->fcache_max < 0) self->fcache_max = 0;
	if (self->fcache_max > TFCACHE_MAX) self->fcache_max = TFCACHE_MAX;
	if (self->fcache_max > 0) {
	    self->fcache = xcalloc(sizeof(TFCache) * self->fcache_max);
	    fcache = self->fcache;
	    for (i=self->fcache_max;i>0;i--) {fcache->advance = -1; fcache++;}
	    for (i=0x20;i<0x80;i++) TFontHandle_FT_Load_Glyph(self, i);
#if TFCACHE_MAX > 191
	    for (i=0xa0;i<0x100;i++) TFontHandle_FT_Load_Glyph(self, i);
#endif
	}
#endif

	gFT_AddHandle(self, FontFileName, font_height, font_dpi);
	return self;
}

#else /* HAVE_FT */

TFontHandle_FT *TFontHandle_FT_Create(const char *FontFileName, int font_height, int font_dpi) {
 printf("%s(%d): ERROR: no freetype support (no ft2build.h !!!)\n", FL);
	return NULL;
}

#endif /* ! HAVE_FT */

