/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "font.h"
#include "image.h"
#include "twrapline.h"


static void TWrapLine_ClrLineData(TWrapLine *self) {
  TWrapLineData *wld;

	Fill0(&self->wlcd, sizeof(self->wlcd));
	while (self->wld) {
	    wld = self->wld;
	    self->wld = wld->next;
	    xfree(wld);
	}
}

static void PaintLineData(TWrapLine *self, long *toobj) {
  TWrapLineData *wld;
  int x, y, w, h, lg;
  dword Color, BackColor, FontFlags;
  const char *s;
  TWRect rc;

	wld = self->wld;
	while (wld) {
	    FontFlags = wld->font->fpv->flags;
	    Font_ClrOrigin(wld->font);
	    Font_ClrFlag(wld->font, FONTFG_OriginAdd);
	    Font_ClrOrigin(wld->font);
	    Font_SetColor(wld->font, wld->Color);
	    Font_SetBackColor(wld->font, wld->BackColor);
	    x = self->ClipRc.left + wld->x;
	    y = self->ClipRc.top + self->font_yo + wld->y;

	    if (wld->SelA == wld->SelE) {
		Font_PaintTo(wld->font, toobj, x, y, wld->s, wld->len);

	    } else {
		if ((wld->font->wtyp == wtyp_FontX) && 
		    !(wld->font->fpv->flags & FONTFG_ImageString))
		{
		    Font_PaintTo(wld->font, toobj, x, y, wld->s, wld->len);
		    x = self->ClipRc.left + wld->SelRc.left;
		    y = self->font_yo + wld->SelRc.top;
		    w = wld->SelRc.width;
		    h = wld->SelRc.height;
		    if (y < 0) {
			h += y;
			y = 0;
		    }

		    if ((h > 0) && (self->DoXFillSelectRect)) {
			rc.left = x;
			rc.top = y;
			rc.width = w;
			rc.height = h;
			self->DoXFillSelectRect(self, toobj, &rc);
//			Fill_Rect(*toobj, appXDat.Select_GC, x, y, w, h);
		    }

		} else {
		    s = wld->s;
		    if (wld->SelA > 0) {
			Font_PaintTo(wld->font, toobj, x, y, s, wld->SelA);
			x += Font_GetWidth(wld->font, s, wld->SelA);
			s += wld->SelA;
		    }

		    Color = ~(wld->Color & 0xffff80);
		    Font_SetColor(wld->font, Color);
		    BackColor = ~(wld->BackColor & 0xffff80);

		    if (!(wld->font->fpv->flags & FONTFG_ImageString) && 
			Font_ChkPaintToIsImage(toobj)) 
		    {
			rc.left = self->ClipRc.left + wld->SelRc.left;
			rc.top = self->font_yo + wld->SelRc.top;
			rc.width = wld->SelRc.width;
			rc.height = wld->SelRc.height;
			if (rc.top < 0) {
			    rc.height += rc.top;
			    rc.top = 0;
			}

			if (rc.height > 0) {
			    TImage_FillRectRGB_rc((TImage*)toobj, BackColor, &rc);
			}

		    } else {
			Font_SetBackColor(wld->font, BackColor);
		    }

		    lg = wld->SelE - wld->SelA;
		    Font_PaintTo(wld->font, toobj, x, y, s, lg);

		    Font_SetColor(wld->font, wld->Color);
		    Font_SetBackColor(wld->font, wld->BackColor);

		    if (wld->len > wld->SelE) {
			x += Font_GetWidth(wld->font, s, lg);
			s += lg;
			Font_PaintTo(wld->font, toobj, x, y, s, wld->len - wld->SelE);
		    }
		}
	    }
	    wld->font->fpv->flags = FontFlags;
	    wld = wld->next;
	}
}

static void TWrapLine_ClrSelect(TWrapLine *self) {
  TWrapLineData *wld;

	self->SelA = 0;
	self->SelE = 0;
	wld = self->wld;
	while (wld) {
	    wld->SelA = 0;
	    wld->SelE = 0;
	    wld = wld->next;
	}
}

static void TWrapLine_SetSelect(TWrapLine *self, int SelA, int SelE) {
  int i, ca, ce, ascent;
  TWrapLineData *wld;
  const char *s;

	if (SelA == SelE) {
	    if (self->SelA != self->SelE) TWrapLine_ClrSelect(self);
	    self->SelA = SelA;
	    self->SelE = SelA;
	} else {
	    self->SelA = SelA;
	    self->SelE = SelE;
	    if (SelA > SelE) {i = SelA; SelA = SelE; SelE = i;}
	}
	ca = 0;
	wld = self->wld;
	while (wld) {
	    if (wld->next) {
		ce = wld->next->char_idx;
	    } else {
		ce = ca + wld->len;
	    }
// printf("%s(%d): ca=%d ce=%d SelA=%d SelE=%d\n", FL, ca, ce, SelA, SelE);
	    wld->SelA = 0;
	    wld->SelE = 0;
	    
	    if ((ce > SelA) && (ca < SelE)) {
		wld->SelA = SelA - ca;
		if (wld->SelA < 0) wld->SelA = 0;
		wld->SelE = SelE - ca;
		if (wld->SelE > wld->len) wld->SelE = wld->len;
	        if (Font_GetMetrics(wld->font, &wld->SelRc.height, &ascent)) {
		    wld->SelRc.top = wld->y - ascent;
		    wld->SelRc.left = wld->x;
		    s = wld->s;
		    wld->SelRc.left += Font_GetWidth(wld->font, s, wld->SelA);
		    s += wld->SelA;
		    wld->SelRc.width = Font_GetWidth(wld->font, s, wld->SelE - wld->SelA);
		}
// printf("%s(%d): SelA=%d SelE=%d x=%d y=%d w=%d h=%d\n", FL, wld->SelA, wld->SelE, wld->SelRc.left,
//			wld->SelRc.top, wld->SelRc.width, wld->SelRc.height);
	    }

	    ca = ce;
	    wld = wld->next;
	}
}

static void AddLineData(TWrapLine *self, const char *s, int len, bool NewLine) {
  TWrapLineData *wld, **ppwld;

	
	if (len < 0) return;

	wld = self->wld;
	while (wld && wld->next) wld = wld->next;
// if (wld) printf("%s(%d): %d\n", FL, wld->len);

	if (wld && (wld->y == self->yo) && (wld->orig_s == self->line_tmp) &&
		  (wld->font == self->font))
	{
	    wld->len = (s + len) - wld->s;
// printf("%s(%d): %d\n", FL, wld->len);
	} else {

	    wld = xcalloc(sizeof(TWrapLineData));
	    wld->len = len;
	    wld->s = s;
	    wld->x = self->xo;
	    wld->y = self->yo;
	    wld->font = self->font;
	    wld->orig_s = self->line_tmp;
	    wld->char_idx = s - wld->orig_s;
	    wld->Color = self->font->fpv->Color;
	    wld->BackColor = self->font->fpv->BackColor;

	    ppwld = &self->wld;
	    while (*ppwld) ppwld = &(*ppwld)->next;
	    *ppwld = wld;
	}

	if (wld) wld->line_nr = self->LineCount;

	if (NewLine) {
	    self->LineCount++;
	    self->xo = 0;
	    self->yo += self->line_height;
	} else if (len > 0) {
	    self->xo += Font_GetWidth(self->font, s, len);
	}
}

static const byte *SkipBlank(const byte *s) {
	while (*s && (*s <= ' ')&&(*s != 0x0d)&&(*s != 0x0a)) s++;
	return s;
}


static bool SetLineData(TWrapLine *self, const char *s, int len) {
  int w, height, ascent, lg;
  const byte *pc, *pca, *pcn;

	self->LineCount = 0;
	if (!Font_GetMetrics(self->font, &height, &ascent)) return false;
	if (!s || (len <= 0)) return true;
	if ((self->ClipRc.width <= 0) || (self->ClipRc.height <= 0)) return true;
	if (self->line_height <= 0) self->line_height = height;
	if (self->font_yo == 0) self->font_yo = ascent;
	self->LineCount = 1;

	pc = (const byte*)s;
	pca  = pc;
	pcn = NULL;

	if (!self->xo) while (*pc == ' ') pc++;

	while (*pc) {
	    while (*pc && (*pc > ' ') && (*pc != ',') && (*pc != '.') && (*pc != '=')) pc++;
	    if (!*pc) break;
	    if ((*pc == ',') || (*pc == '.') || (*pc == '=')) pc++;


	    if ((*pc == 0x0a) || (*pc == 0x0d)) {
		if (pcn) {
		    w = self->xo + Font_GetWidth(self->font, (const char*)pca, pc - pca);
		    if (w >= self->ClipRc.width) {
			AddLineData(self, (const char*)pca, pcn - pca, true);
			pca = SkipBlank(pcn);
			pcn = NULL;
		    }
		}
		lg = pc - pca;
		if (lg) AddLineData(self, (const char*)pca, lg, true);
		if (*pc == 0x0d) pc++;
		if (*pc == 0x0a) pc++;
		AddLineData(self, (const char*)pc, 0, (*pc == 0x0d) || (*pc == 0x0a));
		pca = pc;
		pcn = NULL;

	    } else if (pcn) {
		w = self->xo + Font_GetWidth(self->font, (const char*)pca, pc - pca);
		if (w < self->ClipRc.width) {
		    pcn = pc;
		    pc = SkipBlank(pc);
		} else {
		    AddLineData(self, (const char*)pca, pcn - pca, true);
		    pca = SkipBlank(pcn);
		    pcn = NULL;
		}

	    } else {
		w = self->xo + Font_GetWidth(self->font, (const char*)pca, pc - pca);
		if (w >= self->ClipRc.width) {
		    AddLineData(self, (const char*)pca, pc - pca, true);
		    pca = SkipBlank(pc);
		    pcn = NULL;
		} else {
		    pcn = pc;
		    pc = SkipBlank(pc);
		}
	    }
	}

	if (*pca) {
	    if (pcn) {
		w = self->xo + Font_GetWidth(self->font, (const char*)pca, pc - pca);
		if (w >= self->ClipRc.width) {
		    AddLineData(self, (const char*)pca, pcn - pca, true);
		    pca = SkipBlank(pcn);
		}
	    }
// printf("%s(%d): %d\n", FL, pc - pca);
	    lg = pc - pca;
	    if (lg) AddLineData(self, (const char*)pca, pc - pca, false);
	}

	return true;
}

/* _____________________________________________________________________ */
/* static bool TWrapLine_SetOrigin(TWrapLine *self, int x, int y) {
	self->x = x;
	self->y = y;
	return true;
} */

static bool TWrapLine_SetClipRc(TWrapLine *self, TWRect *ClipRc) {
	self->ClipRc = *ClipRc;
	TWrapLine_ClrLineData(self);
	self->NewFlags |= TWLNF_USE_UPDATE_ALL;
	return true;
}

static bool TWrapLine_SetLine(TWrapLine *self, const char *s, int len) {
	self->line_len = len;
	ppcnchange(&self->line_tmp, s, len);
	TWrapLine_ClrLineData(self);
	self->NewFlags |= TWLNF_USE_UPDATE_ALL;
	return true;
}

static bool TWrapLine_UpdateLine(TWrapLine *self) {
	self->NewFlags = 0;
	self->xo = 0;
	self->yo = 0;
	Fill0(&self->wlcd, sizeof(self->wlcd));
	if (!self->font) return true;
	SetLineData(self, self->line_tmp, self->line_len);
	return true;
}

static bool TWrapLine_PaintTo(TWrapLine *self, long *toobj) {
	if (self->NewFlags) {
	    if (!self->UpdateLine || !self->UpdateLine(self)) return false;
	}
	if (!toobj || !self->font || !self->wld) return true;
	Font_SetClipRc(self->font, &self->ClipRc);
	PaintLineData(self, toobj);
	return true;
}

static bool TWrapLine_GetCharWlcd_XY(TWrapLine *self, int x, int y) {
  TWrapLineData *wld;
  int bottom, height, ascent, char_idx;

	Fill0(&self->wlcd, sizeof(self->wlcd));
	if (!self->wld) return false;

	x -= self->ClipRc.left;
	y -= self->ClipRc.top;
	if ((x < 0) || (y < 0) ||
	    (x >= self->ClipRc.width) || 
	    (y >= self->ClipRc.height)) return false;

	wld = self->wld;
	char_idx = 0;
	while (wld) {
	    Font_GetMetrics(wld->font, &height, &ascent);
	    bottom = wld->y + self->font_yo - ascent + height;
	    if (y <= bottom) break;
	    if (!wld->next) break;
	    if (wld->orig_s == wld->next->orig_s) {
		char_idx = wld->next->char_idx;
	    } else {
		char_idx += wld->len;
	    }
	    wld = wld->next;
	}
	self->wlcd.wld = wld;

	if (wld) {
// printf("%s(%d): %d %s\n", FL, x, wld->s);
	    if (!Font_PointX_GetCharData(wld->font, (TFontCharData*)(void*)&self->wlcd, 
				x, wld->s, wld->len)) return false;
	}
	self->wlcd.char_idx += char_idx;
	self->wlcd.char_x += self->ClipRc.left;
	self->wlcd.char_top = wld->y + self->font_yo - ascent + self->ClipRc.top;
	return true;
}

static bool TWrapLine_GetCharWlcd(TWrapLine *self, int char_idx) {
  TWrapLineData *wld;
  int height, ascent;

	Fill0(&self->wlcd, sizeof(self->wlcd));
	if (!self->wld) {
	    if (!Font_GetMetrics(self->font, &height, &ascent)) return false;
	    self->wlcd.char_height = height;
	    self->wlcd.char_x = self->ClipRc.left;
	    self->wlcd.char_top = self->font_yo - ascent + self->ClipRc.top;
	    return true;
	}

	wld = self->wld;
	while (wld && wld->next) {
	    if (wld->next->char_idx > char_idx) break;
	    wld = wld->next;
	}
	self->wlcd.wld = wld;
	char_idx -= wld->char_idx;
	if (char_idx > wld->len) char_idx = wld->len;
// printf("%s(%d): char_idx=%d\n", FL, char_idx);
	if (!Font_StrIdx_GetCharData(wld->font, (TFontCharData*)(void*)&self->wlcd, 
				wld->s, char_idx)) return false;
// printf("%s(%d): %s\n", FL, wld->s);
	Font_GetMetrics(wld->font, &height, &ascent);
	
	self->wlcd.char_x += self->ClipRc.left;
	self->wlcd.char_top = wld->y + self->font_yo - ascent + self->ClipRc.top;
	return true;
}
/*____________________________ Public __________________________________*/
/* bool WrapLine_SetOrigin(TWrapLine *self, int x, int y) {
	if (!self || !self->SetOrigin) return false;
	return self->SetOrigin(self, x, y);
} */

bool WrapLine_SetClipRc(TWrapLine *self, TWRect *ClipRc) {
	if (!self || !self->SetClipRc) return false;
	return self->SetClipRc(self, ClipRc);
}

bool WrapLine_SetFont(TWrapLine *self, TFont *font) {
	if (!self) return false;
	if (self->SetFont) return self->SetFont(self, font);
	if (self->font != font) {
	    self->NewFlags |= TWLNF_USE_UPDATE_ALL;
	    self->font = font;
	}
	return true;
}

bool WrapLine_Set(TWrapLine *self, const char *s, int len) {
	if (!self || !self->SetLine) return false;
	return self->SetLine(self, s, len);
}

bool WrapLine_SetStr(TWrapLine *self, const char *s) {
	return WrapLine_Set(self, s, pclen(s));
}

bool WrapLine_UpdateLine(TWrapLine *self) {
	if (!self || !self->UpdateLine) return false;
	return self->UpdateLine(self);
}

bool WrapLine_PaintTo(TWrapLine *self, long *toobj) {
	if (!self || !self->PaintTo) return false;
	return self->PaintTo(self, toobj);
}

bool WrapLine_GetCharWlcd_XY(TWrapLine *self, int x, int y) {
	if (!self || !self->GetCharWlcd_XY) return false;
	return self->GetCharWlcd_XY(self, x, y);
}
bool WrapLine_GetCharWlcd(TWrapLine *self, int char_idx) {
	if (!self || !self->GetCharWlcd) return false;
	return self->GetCharWlcd(self, char_idx);
}

void WrapLine_SetSelect(TWrapLine *self, int SelA, int SelE) {
	if (self && self->SetSelect) self->SetSelect(self, SelA, SelE);
}

void WrapLine_ClrSelect(TWrapLine *self) {
	if (self && self->ClrSelect) self->ClrSelect(self);
}
/*_____________________________ Destroy _________________________________*/
bool TWrapLine__Destroy(TWrapLine *self) {
	TWrapLine_ClrLineData(self);
	xfree(self->line_tmp);
	return false;
}

/* ____________________________ Create __________________________________ */
TWrapLine *TWrapLine_AlgCreate(int tcsize, int wtyp) {
  TWrapLine *self;

	self = ttc_create(tcsize, wtyp);
	self->destroy = (TDESTROY)TWrapLine__Destroy;

//	self->SetOrigin		= TWrapLine_SetOrigin;
	self->SetClipRc		= TWrapLine_SetClipRc;
	self->SetLine		= TWrapLine_SetLine;
	self->UpdateLine	= TWrapLine_UpdateLine;
	self->PaintTo		= TWrapLine_PaintTo;
	self->GetCharWlcd_XY	= TWrapLine_GetCharWlcd_XY;
	self->GetCharWlcd	= TWrapLine_GetCharWlcd;
	self->SetSelect		= TWrapLine_SetSelect;
	self->ClrSelect		= TWrapLine_ClrSelect;

	return self;
}

TWrapLine *TWrapLine_Create() {
  TWrapLine *self;

	self = TWrapLine_AlgCreate(sizeof(TWrapLine), wtyp_WrapLine);
	return self;
}
