/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"
#include <setjmp.h>

#ifdef _OS_WIN32

#include <wingdi.h>

#else /* _OS_WIN32 */

#ifndef WORD
typedef unsigned short WORD;
#endif
#ifndef DWORD
typedef unsigned int DWORD;
#endif
#ifndef LONG
typedef long LONG;
#endif

typedef struct tagBITMAPFILEHEADER {
	WORD	bfType;
	DWORD	bfSize;
	WORD	bfReserved1;
	WORD	bfReserved2;
	DWORD	bfOffBits;
} __attribute__((packed)) BITMAPFILEHEADER,*LPBITMAPFILEHEADER,*PBITMAPFILEHEADER;

/*
#pragma pack(push,2)
typedef struct tagBITMAPFILEHEADER {
	WORD	bfType;
	DWORD	bfSize;
	WORD	bfReserved1;
	WORD	bfReserved2;
	DWORD	bfOffBits;
} BITMAPFILEHEADER,*LPBITMAPFILEHEADER,*PBITMAPFILEHEADER;
#pragma pack(pop)
*/

typedef struct tagBITMAPINFOHEADER{
	DWORD	biSize;
	LONG	biWidth;
	LONG	biHeight;
	WORD	biPlanes;
	WORD	biBitCount;
	DWORD	biCompression;
	DWORD	biSizeImage;
	LONG	biXPelsPerMeter;
	LONG	biYPelsPerMeter;
	DWORD	biClrUsed;
	DWORD	biClrImportant;
} __attribute__((packed)) BITMAPINFOHEADER,*LPBITMAPINFOHEADER,*PBITMAPINFOHEADER;

#define BI_RGB 0
#define BI_RLE8 1
#define BI_RLE4 2
#define BI_BITFIELDS 3

#endif

typedef struct {
  DWORD Rmask;
  DWORD Gmask;
  DWORD Bmask;
  DWORD Amask;
  DWORD nc0;
  DWORD nc1;
} BITMAP_V3_EXTRA;

typedef struct TtmpIO {
  BITMAPFILEHEADER bmHF;
  BITMAPINFOHEADER bmIH;
  byte *bmpLine;
  int SrcLineLg;
  byte *tmp_RLE;
  jmp_buf setjmp_buffer;
} TtmpIO;


static void read_DATA(TImage *self, void *data, int lg) {
	if (fread(data, 1, lg, self->in_fp) != lg) 
			longjmp(self->tIO->setjmp_buffer, 1);
}

static void read_WORD(TImage *self, WORD *w) {
	read_DATA(self, w, 2);
}

static void read_DWORD(TImage *self, DWORD *dw) {
	read_DATA(self, dw, 4);
}

static void read_Line(TImage *self) {
  TtmpIO *Dat = self->tIO;
  int lg, i;
  int BitCount;
  byte *sb, *db;

	BitCount = Dat->bmIH.biBitCount;
	sb = Dat->bmpLine;
	lg = Dat->SrcLineLg;
	if ((Dat->bmIH.biCompression == BI_RLE4) || (Dat->bmIH.biCompression == BI_RLE8)) {
	    if (Dat->bmIH.biCompression == BI_RLE4) lg = (lg+1) >> 1;
	    pnmove(sb, Dat->tmp_RLE, lg);
	    Dat->tmp_RLE += lg;

	} else {
	    if (BitCount == 1) lg = ((lg+31) >> 5) << 2;
	    if (BitCount == 4) lg = ((lg+7) >> 3) << 2;
	    read_DATA(self, sb, lg);
	}

	if (BitCount == 1) {
	    sb = &Dat->bmpLine[((Dat->bmIH.biWidth+7)>>3)-1];
	    db = &Dat->bmpLine[Dat->bmIH.biWidth-1];
	    dword c = *sb;
	    for (i=Dat->bmIH.biWidth;i;i--) {
		int k = ((8-i) & 7);
		*db = (c >> k) & 0x01;
		if (k == 7) {
		    sb--;
		    c = *sb;
		}
		db--;
	    }

	} else if (BitCount == 4) {
	    sb = &Dat->bmpLine[((Dat->bmIH.biWidth+1)>>1)-1];
	    db = &Dat->bmpLine[Dat->bmIH.biWidth-1];
	    for (i=Dat->bmIH.biWidth;i;i--) {
		if (i & 1) {
		    *db = (*sb >> 4) & 0x0f;
		    sb--;
		} else {
		    *db = *sb & 0x0f;
		}
		db--;
	    }
	}

}

int read_bmp(TImage *self) {
  TtmpIO tIO;
  BITMAP_V3_EXTRA bmV3;
  int i, x, y, res;
  int incRGB, incA;
  dword m;
  int kR, kB, kG, kA;
  int bR, bB, bG, bA;
  int bitR, bitB, bitG, bitA;
  dword *palette_dat = NULL;
  byte *bmpLine = NULL;
  byte *tmp_RLE = NULL;
  byte *sb;
  word *sw;
  char buf[32];
  char *buf_pc;
  bool ALPHA;

	self->tIO = &tIO;
	Fill0(&tIO, sizeof(tIO));
	buf_pc = buf;
	if (setjmp(tIO.setjmp_buffer)) {
	    res = IMGRD_ERR_STREAM;
	    goto eClose;
	}

	res = IMGRD_ERR_HEAD;
	read_DATA(self, &tIO.bmHF.bfType, 14);
	if (tIO.bmHF.bfType != 0x4D42) goto eClose;

	bmV3.Rmask = 0x00FF0000;
	bmV3.Gmask = 0x0000FF00;
	bmV3.Bmask = 0x000000FF;
	bmV3.Amask = 0x00000000;

	read_DWORD(self, &tIO.bmIH.biSize);
	if (tIO.bmIH.biSize == 12) {
	    read_WORD(self, (void*)&tIO.bmIH.biWidth);
	    read_WORD(self, (void*)&tIO.bmIH.biHeight);
	    read_WORD(self, &tIO.bmIH.biPlanes);
	    read_WORD(self, &tIO.bmIH.biBitCount);
	    tIO.bmIH.biCompression = BI_RGB;
	    ppcmove(&buf_pc, "V0 ");

	} else if (tIO.bmIH.biSize == 40) {
	    read_DATA(self, &tIO.bmIH.biWidth, 36);
	    if (tIO.bmIH.biCompression == BI_BITFIELDS) {
		read_DATA(self, &bmV3.Rmask, 12);
		ppcmove(&buf_pc, "V2 ");
	    } else {
		ppcmove(&buf_pc, "V1 ");
	    }

	} else if ((tIO.bmIH.biSize >= 56) && (tIO.bmIH.biSize <= 64)) {
	    read_DATA(self, &tIO.bmIH.biWidth, 36);
	    read_DATA(self, &bmV3.Rmask, tIO.bmIH.biSize-40);
	    ppcmove(&buf_pc, "V3 ");

	} else {
// printf("%s(%d): %d %s\n", FL, tIO.bmIH.biSize, self->in_gp->name);
	    goto eClose;
	}

	res = IMGRD_ERR_FORMAT;

	self->width = tIO.bmIH.biWidth;
	self->height = tIO.bmIH.biHeight;
	if (self->height < 0) self->height = -self->height;
	if ((self->width < 0) || 
	    (self->width > 32000) || 
	    (self->height > 32000)) goto eClose;

	if (self->pxm_LD & pxmLD_OnlyInfo) return True;
// printf("%s(%d): %dx%d\n", FL, self->width, self->height);


	switch (tIO.bmIH.biCompression) {
	case BI_RGB:
	    if (tIO.bmHF.bfOffBits == (14+tIO.bmIH.biSize)) {
		switch (tIO.bmIH.biBitCount) {
		case 16:
		    bmV3.Amask = 0x8000;
		case 15:
		    bmV3.Rmask = 0x7C00;
		    bmV3.Gmask = 0x03E0;
		    bmV3.Bmask = 0x001F;
		    break;
//		case 24:
		case 32:
		    bmV3.Amask = 0xFF000000;
		    break;
		}
	    }
	    break;

	case BI_BITFIELDS:
//	    break;
	case BI_RLE4:
	case BI_RLE8:
	    break;

	default:
	    goto eClose;
	} 


	kR = kG = kB = kA = 0;
	bitR = bitG = bitB = bitA = 0;
	if (bmV3.Rmask) {
	    m = bmV3.Rmask;
	    while ((m & 1) == 0) {kR++; m>>=1;}
	    while (m & 1) {bitR++; m>>=1;}
	}
	if (bmV3.Bmask) {
	    m = bmV3.Bmask;
	    while ((m & 1) == 0) {kB++; m>>=1;}
	    while (m & 1) {bitB++; m>>=1;}
	}
	if (bmV3.Gmask) {
	    m = bmV3.Gmask;
	    while ((m & 1) == 0) {kG++; m>>=1;}
	    while (m & 1) {bitG++; m>>=1;}
	}
	ALPHA = False;
	if (bmV3.Amask) {
	    ALPHA = True;
	    m = bmV3.Amask;
	    while ((m & 1) == 0) {kA++; m>>=1;}
	    while (m & 1) {bitA++; m>>=1;}
	}
	
	if (!TImage_SetMem(self)) {
	    res = IMGRD_ERR_MEM;
	    goto eClose;
	}

	if (tIO.bmIH.biBitCount <= 8) {
	    palette_dat = xcalloc(sizeof(dword) * 256);
	    if (!tIO.bmIH.biClrUsed) tIO.bmIH.biClrUsed = 1 << tIO.bmIH.biBitCount;
	    if (tIO.bmIH.biSize == 12) {
		for (i=0;i<(int)tIO.bmIH.biClrUsed;i++) {
		    read_DATA(self, &palette_dat[i], 3);
		}
	    } else {
		read_DATA(self, palette_dat, tIO.bmIH.biClrUsed * 4);
	    }
	}

	switch (tIO.bmIH.biBitCount) {
	case 1:
	case 4:
	case 8:
	    i = ((tIO.bmIH.biWidth + 3) >> 2) << 2;
	    break;
	case 15:
	case 16:
	    i = (((tIO.bmIH.biWidth * 2)+3) >> 2) << 2;
	    break;
	case 24:
	    i = (((tIO.bmIH.biWidth * 3)+3) >> 2) << 2;
	    break;
	case 32:
	    i = tIO.bmIH.biWidth * 4;
	    break;
	default:
	    goto eClose;
	}
	bmpLine = xmalloc(i);

	tIO.bmpLine = bmpLine;
	tIO.SrcLineLg = i;

	if (fseek(self->in_fp, tIO.bmHF.bfOffBits, SEEK_SET)) {
	    res = IMGRD_ERR_STREAM;
	    goto eClose;
	}

	if ((tIO.bmIH.biCompression == BI_RLE4) || (tIO.bmIH.biCompression == BI_RLE8)) {
  ic32 ic;
  int rle_count = self->width * self->height;
	    tIO.SrcLineLg = self->width;
	    tmp_RLE = xcalloc(rle_count+255);
	    tIO.tmp_RLE = tmp_RLE;
	    sb = tmp_RLE;

	    while (rle_count > 0) {
		read_DATA(self, ic.b, 2);
		if (ic.b[0]) {
		    int lg = ic.b[0];
		    rle_count -= lg;
		    if (tIO.bmIH.biCompression == BI_RLE4) lg = lg >> 1;
		    for (i=lg;i;i--) *sb++ = ic.b[1];

		} else if (ic.b[1] > 2) {
		    int lg = ic.b[1];
		    rle_count -= lg;
		    if (tIO.bmIH.biCompression == BI_RLE4) lg = lg >> 1;
		    read_DATA(self, sb, lg);
		    sb += lg;
		    if (lg & 1) read_DATA(self, ic.b, 1);

		} else if (ic.b[1] == 2) {  //Deltarecord ???
		    read_DATA(self, ic.b, 2);

		} else if (ic.b[1] == 0) {

		} else break;
	    }
	}

  bool rgb565 = (self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp;

  byte *db, *db0, *pixAlpha, *pixAlpha0;
  dword col;
  dword R, G, B, A, mR, mG, mB, mA;

	if (rgb565) {
	    mR = 0xf800;   mG = 0x07e0;   mB = 0x001f;   mA = 0x00ff;
	    bR = 16-bitR;  bG = 11-bitG;  bB = 5-bitB;   bA = 8-bitA;
	} else {
	    mR = 0xff0000; mG = 0x00ff00; mB = 0x0000ff; mA = 0xff000000;
	    bR = 24-bitR;  bG = 16-bitG;  bB = 8-bitB;   bA = 32-bitA;
	}

	if (bR<0) {kR -= bR; bR = 0;}
	if (bG<0) {kG -= bG; bG = 0;}
	if (bB<0) {kB -= bB; bB = 0;}
	if (bA<0) {kA -= bA; bA = 0;}

	pixAlpha0 = NULL;
	if (ALPHA) self->stat |= pmxStat_has_alpha;
	if (rgb565 && ALPHA) {
	    pixAlpha0 = xmalloc(self->width * self->height);
	    self->rgb565_alpha_data = pixAlpha0;
	}

	if (tIO.bmIH.biHeight < 0) {
	    incRGB = self->bytes_per_line;
	    incA = self->width;
	    db0 = self->data;
	} else {
	    incRGB = -self->bytes_per_line;
	    incA = -self->width;
	    db0 = self->data + (self->bytes_per_line * (self->height-1));
	    pixAlpha0 += self->width * (self->height-1);
	}

	for (y = self->height;y;y--) {
	    read_Line(self);
	    db = db0;
	    pixAlpha = pixAlpha0;

	    if (tIO.bmIH.biBitCount == 16) {
		sw = (word*)bmpLine;
		for (x = tIO.bmIH.biWidth;x;x--) {
		    R = (((*sw & bmV3.Rmask) >> kR) << bR) & mR;
		    G = (((*sw & bmV3.Gmask) >> kG) << bG) & mG;
		    B = (((*sw & bmV3.Bmask) >> kB) << bB) & mB;
		    col = R | G | B;
		    A = (((*sw & bmV3.Amask) >> kA) << bA) & mA;
		    if (rgb565) {
			if (ALPHA) *pixAlpha++ = A;
			*((word*)db) = col;
			db += 2;
		    } else {
			*((dword*)db) = A | col;
			db += 4;
		    }
		    sw++;
		}

	    } else if (tIO.bmIH.biBitCount == 24) {
		sb = bmpLine;
		for (x = tIO.bmIH.biWidth;x;x--) {
		    col = sb[0] | (sb[1] << 8) | (sb[2] << 16);
		    R = (((col & bmV3.Rmask) >> kR) << bR) & mR;
		    G = (((col & bmV3.Gmask) >> kG) << bG) & mG;
		    B = (((col & bmV3.Bmask) >> kB) << bB) & mB;
		    col = R | G | B;
		    if (rgb565) {
			*((word*)db) = col;
			db += 2;
		    } else {
			*((dword*)db) = col;
			db += 4;
		    }
		    sb += 3;
		}

	    } else if (tIO.bmIH.biBitCount == 32) {
		dword *sd = (dword*)bmpLine;
		for (x = tIO.bmIH.biWidth;x;x--) {
		    R = (((*sd & bmV3.Rmask) >> kR) << bR) & mR;
		    G = (((*sd & bmV3.Gmask) >> kG) << bG) & mG;
		    B = (((*sd & bmV3.Bmask) >> kB) << bB) & mB;
		    col = R | G | B;
		    A = (((*sd & bmV3.Amask) >> kA) << bA) & mA;
		    if (rgb565) {
			if (ALPHA) *pixAlpha++ = A;
			*((word*)db) = col;
			db += 2;
		    } else {
			*((dword*)db) = A | col;
			db += 4;
		    }
		    sd++;
		}

	    } else {  //biBitCount 1,4,8
		sb = bmpLine;
		for (x = tIO.bmIH.biWidth;x;x--) {
		    col = palette_dat[*sb];
		    if (rgb565) {
			*((word*)db) = ((col>>8)&mR)|((col>>5)&mG)|((col>>3)&mB);
			db += 2;
		    } else {
			*((dword*)db) = col;
			db += 4;
		    }
		    sb++;
		}
	    }
	    pixAlpha0 += incA;
	    db0 += incRGB;
	}

	alpha_bm_stat(self);

	if (!(self->pxm_LD & pxmLD_NoImgInfoStr)) {

	    if (self->stat & pmxStat_has_alpha_tgl) {*buf_pc = 'A'; buf_pc++;}
	    ppcmove(&buf_pc, "RGB");
	    if (self->stat & pmxStat_has_alpha_tgl) inttoppc(&buf_pc, bitA);
	    inttoppc(&buf_pc, bitR);
	    inttoppc(&buf_pc, bitG);
	    inttoppc(&buf_pc, bitB);

	    if (tIO.bmIH.biBitCount <= 8) {
		*buf_pc = ' '; buf_pc++;
		inttoppc(&buf_pc, tIO.bmIH.biClrUsed);
		*buf_pc = 'c'; buf_pc++; 
	    }
	    if (tIO.bmIH.biCompression == BI_RLE4) ppcmove(&buf_pc, " RLE4");
	    if (tIO.bmIH.biCompression == BI_RLE8) ppcmove(&buf_pc, " RLE8");
	    *buf_pc = 0;

	    self->img_info = pcdup(buf);
	}

	res = IMGRD_OK;

eClose:
	xfree(tmp_RLE);
	xfree(bmpLine);
	xfree(palette_dat);

	return res;
}

static bool ChkType(TglobPath *gp, const char *fn, FILE *F) {
	return TImage_ChkPattern(gp, fn, hImageRD_bmp.pattern);
}

const ThImageLoader hImageRD_bmp = {
  "bmp",
  filetyp_BMP,
  ChkType,
  read_bmp
};
