/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"
#include "lzw.h"
#include <setjmp.h>

typedef struct TtmpIO {
  const char *err_txt;
  jmp_buf setjmp_buffer;

  TImageExGif *AktExGif;
  int LzwLg;
  byte *LzwData;
} TtmpIO;

static void read_DATA(TImage *self, void *data, int lg) {
	if (fread(data, 1, lg, self->in_fp) != lg) 
			longjmp(self->tIO->setjmp_buffer, 1);
}

static bool read_DATA_Lzw(TLzw *lzw, void *data, int lg) {
  TtmpIO *tIO = ((TImage*)lzw->owner)->tIO;
	if (!tIO->LzwData || (lg > tIO->LzwLg)) return False;
	pnmove(data, tIO->LzwData, lg);
	tIO->LzwLg -= lg;
	tIO->LzwData += lg;
	return True;
}

static int read_WORD_int(TImage *self) {
  int w = 0;
	read_DATA(self, &w, 2);
	return w;
}

static dword read_BYTE_dword(TImage *self) {
  int w = 0;
	read_DATA(self, &w, 1);
	return w;
}

static dword read_RGB_dword(TImage *self) {
  byte buf[4];
	read_DATA(self, buf, 3);
	return (buf[0]<<16) | (buf[1]<<8) | buf[2];
}

static bool ExGif__Destroy(TImageExGif *self) {
	Tfree(self->next);
	xfree(self->LzwData);
	xfree(self->GlobColorTab);
	xfree(self->ColorTab);
	xfree(self->Comment);
	return False;
}

static TImageExGif *TImageExGif_Create() {
  TImageExGif *self;

	self = ttc_create(sizeof(TImageExGif), wtyp_TImageExGif);
	self->destroy = (TDESTROY)ExGif__Destroy;
	self->TransparentIdx = 0x100;
	return self;
}

static int SetColorTab(TImage *self, dword bef, dword **ppColorTab) {
  int res = 0;
	if (bef & 0x80) {
	    res = 1 << ((bef & 0x07) + 1);
	    int ColorCount = res;
	    dword *dw = xcalloc(1024);
	    *ppColorTab = dw;
	    while (ColorCount) {
		*dw = read_RGB_dword(self);
		dw++; ColorCount--;
	    }
	}
	return res;
}

int read_gif(TImage *self) {
  TtmpIO *tIO;
  int res;
  byte buf[16];
  TImageExGif *ExGif;

	res = IMGRD_ERR_HEAD;
	tIO = xcalloc(sizeof(TtmpIO));
	self->tIO = tIO;
	tIO->err_txt = "Format: ";
	if (setjmp(tIO->setjmp_buffer)) {
err:
	    appError(EMSG_ERR1, "%s(%d): %s \"%s\"", FL, tIO->err_txt, self->in_gp->name);
	    goto Close02;
	}

 buf[6] = 0;
	read_DATA(self, buf, 6);
	if (!pcncmp((char*)buf, "GIF87a", 6) && !pcncmp((char*)buf, "GIF89a", 6)) goto err;

	self->width = read_WORD_int(self);
	self->height = read_WORD_int(self);
	if (!self->width || !self->height ||
	    (self->width > 32000) || 
	    (self->height > 32000)) {
	    res = IMGRD_ERR_FORMAT;
	    goto err;
	}

	if (self->pxm_LD & pxmLD_OnlyInfo) goto EndeOk;

	read_DATA(self, buf, 3);

	ExGif = TImageExGif_Create();
	self->ExImg = (TImageExImg*)ExGif;

	ExGif->GlobColorCount = SetColorTab(self, buf[0], &ExGif->GlobColorTab);
	ExGif->BackGroundColor = buf[1];

	if (!TImage_SetMem(self)) goto Close02;

RecLoop:
	read_DATA(self, buf, 1);
// printf("%s(%d): Type=0x%02x\n", FL, buf[0]);


	if (buf[0] == ',') {
	    ExGif->left = read_WORD_int(self);
	    ExGif->top = read_WORD_int(self);
	    ExGif->width = read_WORD_int(self);
	    ExGif->height = read_WORD_int(self);

	    read_DATA(self, buf, 1);
// printf("%s(%d): %dx%d %d %d 0x%02x\n", FL, ExGif->width, ExGif->height, ExGif->left, ExGif->top, buf[0]);
	    ExGif->ColorCount = SetColorTab(self, buf[0], &ExGif->ColorTab);
	    ExGif->Interlace = (buf[0] & 0x40) != 0;

	    int LzwGesLg = 1;
	    byte *LzwData = xmalloc(2);
	    read_DATA(self, LzwData, 1);
	    while (1) {
		int i = read_BYTE_dword(self);
		LzwData[LzwGesLg] = i;
		if (!i) break;
		LzwGesLg++;
		LzwData = xrealloc(LzwData, LzwGesLg + i + 1);
		read_DATA(self, LzwData + LzwGesLg, i);
		LzwGesLg += i;
	    }
	    ExGif->LzwLg = LzwGesLg;
	    ExGif->LzwData = LzwData;
	    ExGif->next = (TImageExImg*)TImageExGif_Create();
	    ExGif = (TImageExGif*)ExGif->next;

	} else if (buf[0] == '!') {	    
	    int ExtGesLg = 0;
	    byte *ExtData = NULL;
	    int ExtCode = read_BYTE_dword(self);

	    while (1) {
		int i = read_BYTE_dword(self);
		if (!i) break;
		ExtData = xrealloc(ExtData, ExtGesLg + i);
		read_DATA(self, &ExtData[ExtGesLg], i);
		ExtGesLg += i;
	    }
// printf("%s(%d): ext=0x%02x lg=%d ", FL, ExtCode, ExtGesLg);

	    if (ExtCode == 0xf9) { //Graphic Control Extension
		if (ExtGesLg != 4) goto err;
		if (ExtData[0] & 0x01) {
		    ExGif->TransparentIdx = ExtData[3];
		}
		ExGif->DisposalMethod = (ExtData[0] >> 2) & 0x07;
		ExGif->DelayTime = ExtData[0] | (ExtData[1] << 8);

// printf(" %02x %02x %02x %02x", ExtData[0], ExtData[1], ExtData[2], ExtData[3]);
	    } else if (ExtCode == 0xfe) { //Comment Extension
		ExGif->Comment = (char*)ExtData;
// printf(" %s", ExtData);
		ExtData = NULL;
	    }
// printf("\n");
	    xfree(ExtData);
	    goto RecLoop;

	} else if (buf[0] != ';') {
	    goto err;
	}

	res = DecodeExGif(self, (TImageExGif*)self->ExImg);
	goto Close02;

EndeOk:
	res = IMGRD_OK;
Close02:
	xfree(tIO);
	self->tIO = NULL;
	return res;
}

static int IntlOffTab[5] = {0, 0, 4, 2, 1};
static int IntlAddTab[5] = {1, 8, 8, 4, 2};

int DecodeExGif(TImage *self, TImageExGif *ExGif) {
  TtmpIO *tIO;
  TLzw *Lzw;
  int maxX, maxY;
  dword *colTab, Transparent;
  byte *db0, *lbuf, *pixAlpha, *maskdb0;
  int res, IntlIdx, IntlCount, bytes_per_line, mask_bytes_per_line, LineNr;
  TImageExGif *GlobExGif;
  bool rgb565;

	if (!self) return False;
	if (!self->data) return IMGRD_ERR_MEM;
	GlobExGif = (TImageExGif*)self->ExImg;
	if (!GlobExGif) return False;
	if (!ExGif) ExGif = GlobExGif;

	IntlCount = 1;
	IntlIdx = 0;
	if (ExGif->Interlace) {
	    IntlCount = 4;
	    IntlIdx = 1;
	}

	res = IMGRD_ERR_FORMAT;
	pixAlpha = lbuf = NULL;

	tIO = xcalloc(sizeof(TtmpIO));
	self->tIO = tIO;

	tIO->AktExGif = ExGif;
	tIO->LzwLg = ExGif->LzwLg;
	tIO->LzwData = ExGif->LzwData;

	Lzw = TLzw_Create();
	Lzw->owner = self;
	Lzw->Lzw_rd = read_DATA_Lzw;
	if (!TLzw_Setup_rd(Lzw)) goto err;

	Transparent = ExGif->TransparentIdx;


	colTab = ExGif->ColorTab;
	if (!colTab) colTab = GlobExGif->GlobColorTab;
	if (!colTab) goto err;
	maxX = self->width - ExGif->left;
	if (maxX > ExGif->width) maxX = ExGif->width;
	maxY = self->height - ExGif->top;
	if (maxY > ExGif->height) maxY = ExGif->height;
	if ((maxX <= 0) || (maxY <= 0) || 
	    (ExGif->width <= 0) || (ExGif->height <= 0)) goto EndeOk;

	rgb565 = (self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp;

	lbuf = xmalloc(self->width);
	if (rgb565) {
	    pixAlpha = self->rgb565_alpha_data;
	    if (!pixAlpha) {
		pixAlpha = xcalloc(self->width * self->height);
		self->rgb565_alpha_data = pixAlpha;
	    }
	    pixAlpha += (ExGif->top * self->width);
	}

// printf("%s(%d): %dx%d %d %d\n", FL, ExGif->width, ExGif->height, ExGif->Interlace, rgb565);

	while (IntlCount) {
	    bytes_per_line = self->bytes_per_line * IntlAddTab[IntlIdx];
	    mask_bytes_per_line = self->width * IntlAddTab[IntlIdx];
	    LineNr = IntlOffTab[IntlIdx];
	    db0 = self->data + ((ExGif->top + LineNr) * self->bytes_per_line);
	    maskdb0 = self->rgb565_alpha_data + ((ExGif->top + LineNr) * self->width);

	    while (LineNr < maxY) {
		if (!TLzw_Decompress(Lzw, lbuf, ExGif->width) ||
		    Lzw->Error) goto err;

		if (rgb565) {
		    pixAlpha = maskdb0 + ExGif->left;
		    byte *tb = lbuf;
		    word *db = (word*)db0 + ExGif->left;
		    for (int x=maxX; x; x--) {
			if (Transparent == *tb) {
			    *pixAlpha++ = 0x00;
			} else {
			    *pixAlpha++ = 0xff;
			    dword col = colTab[*tb];
			    *db = ((col & 0xf80000) >> 8) | 
				  ((col & 0xfc00) >> 5) |
				  ((col & 0xf8) >> 3);
			}
			db++; tb++;
		    }
		    maskdb0 += mask_bytes_per_line;

		} else {
		    byte *tb = lbuf;
		    dword *db = (dword*)db0 + ExGif->left;
		    for (int x=maxX; x; x--) {
			if (Transparent == *tb) {
			    *db = 0;
			} else {
			    *db = colTab[*tb] | 0xff000000;
			}
			db++; tb++;
		    }
		}
		LineNr += IntlAddTab[IntlIdx];
		db0 += bytes_per_line;
	    }
	    IntlIdx++;
	    IntlCount--;
	}
	alpha_bm_stat(self);
EndeOk:
	res = IMGRD_OK;
err:
	xfree(lbuf);
	Tfree(Lzw);
	xfree(tIO);
	self->tIO = NULL;
	return res;
}

static bool ChkType(TglobPath *gp, const char *fn, FILE *F) {
  char m[6];

	if (F) {
	    rewind(F);
	    if ((fread(&m, 1, 6, F) == 6) &&
		(pcncmp(m, "GIF87a", 6) || pcncmp(m, "GIF89a", 6))) return True;
	}
	return TImage_ChkPattern(gp, fn, hImageRD_gif.pattern);
}

const ThImageLoader hImageRD_gif = {
  "gif",
  filetyp_GIF,
  ChkType,
  read_gif
};

