/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"
#include "jpeg_pv.h"

#if LD_LIB_JPEG

#define dl_jpeg_create_compress jpeg_create_compress
#define dl_jpeg_destroy_compress jpeg_destroy_compress
#define dl_jpeg_stdio_dest jpeg_stdio_dest
#define dl_jpeg_set_defaults jpeg_set_defaults
#define dl_jpeg_set_quality jpeg_set_quality
#define dl_jpeg_start_compress jpeg_start_compress
#define dl_jpeg_write_scanlines jpeg_write_scanlines
#define dl_jpeg_finish_compress jpeg_finish_compress

#else /* LD_LIB_JPEG */

extern void *jpegLibHandle;
extern bool jpeglib_WR_load;
extern bool jpeglib_Err;

extern bool LoadLib_JPEG();

#define dl_jpeg_create_compress(cinfo) \
    dl_jpeg_CreateCompress((cinfo), JPEG_LIB_VERSION, \
			(size_t) sizeof(struct jpeg_compress_struct))

void (*dl_jpeg_CreateCompress)(j_compress_ptr cinfo,
				      int version, size_t structsize);
void (*dl_jpeg_destroy_compress)(j_compress_ptr cinfo);
void (*dl_jpeg_stdio_dest)(j_compress_ptr cinfo, FILE * outfile);
void (*dl_jpeg_set_defaults)(j_compress_ptr cinfo);
void (*dl_jpeg_set_quality)(j_compress_ptr cinfo, int quality, boolean force_baseline);
void (*dl_jpeg_start_compress)(j_compress_ptr cinfo, boolean write_all_tables);
JDIMENSION (*dl_jpeg_write_scanlines)(j_compress_ptr cinfo,
				      JSAMPARRAY scanlines, JDIMENSION num_lines);
void (*dl_jpeg_finish_compress)(j_compress_ptr cinfo);

static bool LoadLib_JPEG_write() {
	if (jpeglib_WR_load) return True;
	if (!jpegLibHandle && !LoadLib_JPEG()) return False;
	dl_jpeg_CreateCompress = load_lib(jpegLibHandle, "jpeg_CreateCompress");
	dl_jpeg_destroy_compress = load_lib(jpegLibHandle, "jpeg_destroy_compress");
	dl_jpeg_stdio_dest = load_lib(jpegLibHandle, "jpeg_stdio_dest");
	dl_jpeg_set_defaults = load_lib(jpegLibHandle, "jpeg_set_defaults");
	dl_jpeg_set_quality = load_lib(jpegLibHandle, "jpeg_set_quality");
	dl_jpeg_start_compress = load_lib(jpegLibHandle, "jpeg_start_compress");
	dl_jpeg_write_scanlines = load_lib(jpegLibHandle, "jpeg_write_scanlines");
	dl_jpeg_finish_compress = load_lib(jpegLibHandle, "jpeg_finish_compress");

	jpeglib_WR_load = True;
	return True;
} 

#endif /* LD_LIB_JPEG == 0 */

/*____________________________________________________________________________*/

struct my_error_mgr {
  struct jpeg_error_mgr pub;	/* "public" fields */
  jmp_buf setjmp_buffer;	/* for return to caller */
};
/*____________________________________________________________________________*/
int write_jpeg(TImage *self) {
  struct jpeg_compress_struct cinfo;
  struct my_error_mgr jerr;
  JSAMPROW row_pointer[1];	/* pointer to JSAMPLE row[s] */
  bool gray;

#if LD_LIB_JPEG == 0
	if (!LoadLib_JPEG_write()) return False;
#endif

  /* Step 1: allocate and initialize JPEG compression object */
	jpeg_error_init((j_common_ptr)&cinfo, &jerr);
	if (setjmp(jerr.setjmp_buffer)) {
	    dl_jpeg_destroy_compress(&cinfo);
	    return IMGWR_ERR_HEAD;
	}

	dl_jpeg_create_compress(&cinfo);
  /* Step 2: specify data destination (eg, a file) */
	gray = (self->out_fu & fuWR_gray) != 0;
	dl_jpeg_stdio_dest(&cinfo, self->out_fp);
  /* Step 3: set parameters for compression */
	cinfo.image_width = self->width; 	/* image width and height, in pixels */
	cinfo.image_height = self->height;
	cinfo.input_components = 3;	/* # of color components per pixel */
	cinfo.in_color_space = JCS_RGB; /* colorspace of input image */
	if (gray) {
	    cinfo.input_components = 1;	/* # of color components per pixel */
	    cinfo.in_color_space = JCS_GRAYSCALE; /* colorspace of input image */
	}

	dl_jpeg_set_defaults(&cinfo);
  int quality = (self->out_fu >> 8) & 0xff;
	if (quality <= 0) quality = 95;
	dl_jpeg_set_quality(&cinfo, quality, TRUE);
//	jpeg_simple_progression(&cinfo);
  /* Step 4: Start compressor */
	dl_jpeg_start_compress(&cinfo, TRUE);
  /* Step 5: while (scan lines remain to be written) */
  /*           jpeg_write_scanlines(...); */
	row_pointer[0] = xmalloc(cinfo.image_width * 3);

  bool rgb565 = (self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp;
  byte *sb0 = self->data;
	for (int y=self->height;y;y--) {
  byte *sb, *db;
  int r, g, b;

	    sb = sb0;
	    db = row_pointer[0];
	    for (int x=self->width;x;x--) {
		if (rgb565) {
		    dword c = *((word*)sb);
		    b = (c<<3) & 0xf8; g = (c>>3) & 0xfc; r = (c>>8) & 0xf8;
		    sb += 2;
		} else {
		    b = sb[0]; g = sb[1]; r = sb[2];
		    sb += 4;
		}

		if (gray) {
		    *db++ = ((r + g + b) * 0x55a) >> 12;  //div 3
		} else {
		    *db++ = r; *db++ = g; *db++ = b;
		}
	    }
	    dl_jpeg_write_scanlines(&cinfo, row_pointer, 1);
	    sb0 += self->bytes_per_line;
	}

  /* Step 6: Finish compression */
	dl_jpeg_finish_compress(&cinfo);
	xfree(row_pointer[0]);
	dl_jpeg_destroy_compress(&cinfo);

	return IMGWR_OK;
}

static bool ChkType(TglobPath *gp, const char *fn, FILE *F) {
	return TImage_ChkPattern(gp, fn, hImageWR_jpeg.pattern);
}

const ThImageLoader hImageWR_jpeg = {
  "jpg jpeg",
  filetyp_JPEG,
  ChkType,
  write_jpeg
};
