/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"

//#ifdef _OS_WIN32
//#define DYN_LIBPNG 1
//#endif

#if LD_LIB_PNG

#include "png.h"

//int pngVer = 3;
const char *pngVerStr = PNG_LIBPNG_VER_STRING;

#define dl_png_create_read_struct png_create_read_struct
#define dl_png_destroy_read_struct png_destroy_read_struct
#define dl_png_create_info_struct png_create_info_struct
#define dl_png_init_io png_init_io
#define dl_png_read_info png_read_info
#define dl_png_get_IHDR png_get_IHDR
#define dl_png_set_expand png_set_expand
#define dl_png_set_gray_to_rgb png_set_gray_to_rgb
#define dl_png_read_rows png_read_rows
#define dl_png_read_end png_read_end
//#define dl_png_get_valid png_get_valid
#define dl_png_set_strip_16 png_set_strip_16
#define dl_png_set_packing png_set_packing
#define dl_png_set_interlace_handling png_set_interlace_handling
#define dl_png_read_update_info png_read_update_info
#define dl_png_get_channels png_get_channels

#else /* LD_LIB_PNG */

#include <setjmp.h>

typedef struct {
  jmp_buf jmpbuf;	/* used in png_error */
//   ...
} png_struct;

extern void *(*dl_png_create_read_struct) (char *user_png_ver,
		void *error_ptr, void *error_fn, void *warn_fn);
extern void *(*dl_png_destroy_read_struct) (png_struct **png_ptr_ptr,
		void **info_ptr_ptr, void **end_info_ptr_ptr);
extern void *(*dl_png_create_info_struct) (void *png_ptr);

extern void (*dl_png_init_io) (void *png_ptr, FILE *fp);


void (*dl_png_read_info) (void *png_ptr, void *info_ptr);
dword (*dl_png_get_IHDR) (void *png_ptr, void *info_ptr,
		dword *width, dword *height,
		int *bit_depth, int *color_type, int *interlace_method,
		int *compression_method, int *filter_method);
void (*dl_png_set_expand) (void *png_ptr);
void (*dl_png_set_gray_to_rgb) (void *png_ptr);
void (*dl_png_read_rows) (void *png_ptr, void *row, void *display_row, unsigned int num_rows);
//void (*png_read_row) (void *png_ptr, void *row, void *display_row);
void (*dl_png_read_end) (void *png_ptr, void *info_ptr);

//int (*dl_png_get_valid) (void *png_ptr, void *info_ptr, int flag);
void (*dl_png_set_strip_16) (void *png_ptr);
void (*dl_png_set_packing) (void *png_ptr);
int (*dl_png_set_interlace_handling) (void *png_ptr);
//void (*png_do_read_interlace) (void *png_ptr);
void (*dl_png_read_update_info) (void *png_ptr, void *info_ptr);
byte (*dl_png_get_channels) (void *png_ptr, void *info_ptr);

extern void *pngLibHandle;
//extern int pngVer;
extern char *pngVerStr;
extern bool pnglib_RD_load;
extern bool pnglib_WR_load;
extern bool pnglib_Err;

static void LoadLib_PNG_Proc() {
	dl_png_read_info = load_lib(pngLibHandle, "png_read_info");
//	dl_png_get_valid = load_lib(pngLibHandle, "png_get_valid");
	dl_png_get_IHDR = load_lib(pngLibHandle, "png_get_IHDR");
	dl_png_set_expand = load_lib(pngLibHandle, "png_set_expand");
	dl_png_set_gray_to_rgb = load_lib(pngLibHandle, "png_set_gray_to_rgb");
	dl_png_set_packing = load_lib(pngLibHandle, "png_set_packing");
	dl_png_set_strip_16 = load_lib(pngLibHandle, "png_set_strip_16");
	dl_png_set_interlace_handling = load_lib(pngLibHandle, "png_set_interlace_handling");
//	png_set_interlace_handling = load_lib(pngLibHandle, "png_set_interlace_handling");
//	png_do_read_interlace = load_lib(pngLibHandle, "png_do_read_interlace");
	dl_png_read_rows = load_lib(pngLibHandle, "png_read_rows");
//	png_read_row = load_lib(pngLibHandle, "png_read_row");
	dl_png_read_update_info = load_lib(pngLibHandle, "png_read_update_info");
	dl_png_read_end = load_lib(pngLibHandle, "png_read_end");
	dl_png_get_channels = load_lib(pngLibHandle, "png_get_channels");
}

static bool LoadLib_PNG_read() {
	if (pnglib_RD_load) return True;
	if (!pngLibHandle && !LoadLib_PNG()) return False;
	LoadLib_PNG_Proc();
	pnglib_RD_load = True;
	return True;
} 

#endif /* LD_LIB_PNG == 0 */

#if LD_LIB_PNG
static void __attribute__ ((noreturn)) png_rd_error_exit(png_structp png_ptr, png_const_charp msg) {
#else
static void __attribute__ ((noreturn)) png_rd_error_exit(png_struct *png_ptr, char *msg) {
#endif
 printf("%s(%d): ERROR: %s\n", FL, msg);
  longjmp(png_ptr->jmpbuf, 1);
}

int read_png(TImage *self) {
  png_struct *png_ptr;
#if LD_LIB_PNG
  png_info *info_ptr = NULL;
  png_info *end_info = NULL;
#else /* LD_LIB_PNG */
  void *info_ptr = NULL;
  void *end_info = NULL;
#endif /* LD_LIB_PNG */
  int bit_depth, color_type, interlace_type, channels;
  int passes;
  byte *pngLine, *pixAlpha;
  byte *sb, *db0, *db;
  bool GRAY, ALPHA;

#if LD_LIB_PNG == 0
	if (!LoadLib_PNG_read()) return False;
#endif
	self->width = 0;
	self->height = 0;
	self->data = NULL;
	self->rgb565_alpha_data = NULL;

	png_ptr = dl_png_create_read_struct(pngVerStr, NULL, png_rd_error_exit, NULL);
	if (!png_ptr) return False;

	info_ptr = dl_png_create_info_struct(png_ptr);
	if (!info_ptr) {
Close02:
		dl_png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
		return False;
	}
	if (setjmp(png_ptr->jmpbuf)) goto Close02;

	end_info = dl_png_create_info_struct(png_ptr);
	if (!end_info) goto Close02;
	
	dl_png_init_io(png_ptr, self->in_fp);
	dl_png_read_info(png_ptr, info_ptr);

	dl_png_get_IHDR(png_ptr, info_ptr, (void*)&self->width, (void*)&self->height, &bit_depth, &color_type, &interlace_type, 0, 0);
	if ((self->width <= 0) || (self->height <= 0)) goto Close02;

	if (self->pxm_LD & pxmLD_OnlyInfo) {
	    dl_png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
	    return True;
	}

	GRAY = ((color_type & 3) == 0);

	if (bit_depth == 16) {
		dl_png_set_strip_16(png_ptr);
	} else if (bit_depth < 8) dl_png_set_packing(png_ptr);

	passes = 0;
	if (interlace_type != 0) {
	    passes = dl_png_set_interlace_handling(png_ptr);
	}
	if (GRAY) dl_png_set_gray_to_rgb(png_ptr);
	dl_png_set_expand(png_ptr);

	dl_png_read_update_info(png_ptr, info_ptr);
//	png_get_IHDR(png_ptr, info_ptr, &w, &h, &bit_depth, &color_type, &interlace_type, 0, 0);
	channels = dl_png_get_channels(png_ptr, info_ptr);

	if (channels != 4 && channels != 3) {
	    appError(EMSG_ERR1, "%s(%d) PNG channels=%d ??? \"%s\"", FL, channels, self->in_gp->name);
	    goto Close02;
	}

	if (!TImage_SetMem(self)) goto Close02;

	ALPHA = (channels == 4);
	pixAlpha = NULL;
	pngLine = xmalloc(self->width * 4);

	if (passes > 1 ) {
	    for (int i2 = 1; i2<passes;i2++) {
		sb = (byte*)pngLine;
		for (int y=self->height; y; y--) {
		    dl_png_read_rows(png_ptr, &sb, NULL, 1);
		}
	    }
	}

  bool rgb565 = (self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp;
	if (rgb565 && ALPHA) pixAlpha = xmalloc(self->width * self->height);
	self->rgb565_alpha_data = pixAlpha;
	if (ALPHA) self->stat |= pmxStat_has_alpha;

	db0 = self->data;
	for (int y=self->height; y; y--) {
	    db = db0;
	    sb = (byte*)pngLine;
	    dl_png_read_rows(png_ptr, &sb, NULL, 1);

	    if (ALPHA) {		//channels == 4
	      for (int x=self->width; x; x--) {
		  if (rgb565) {
		    *((word*)db) = ((sb[0]&0xf8)<<8)|((sb[1]&0xfc)<<3)|((sb[2]&0xf8)>>3);
		    db += 2;
		    *pixAlpha++ = sb[3];
		  } else {
		    *db++ = sb[2]; *db++ = sb[1]; *db++ = sb[0]; *db++ = sb[3];
		  }
		  sb += 4;
	      }
	    } else {	//channels == 3
	      for (int x=self->width; x; x--) {
		  if (rgb565) {
		    *((word*)db) = ((sb[0]&0xf8)<<8)|((sb[1]&0xfc)<<3)|((sb[2]&0xf8)>>3);
		    db += 2;
		  } else {
		    *((dword*)db) = (sb[0]<<16) | (sb[1]<<8) | sb[2] | 0xff000000;
		    db += 4;
		  }
		  sb += 3;
	      }
	    }
	    db0 += self->bytes_per_line;
	}

	dl_png_read_end(png_ptr, end_info);
	dl_png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);

	xfree(pngLine);
	alpha_bm_stat(self);
	return True;
}


static bool ChkType(TglobPath *gp, const char *fn, FILE *F) {
  int m[2];

	if (F) {
	    rewind(F);
	    if ((fread(&m, 1, 8, F) == 8) &&
		((m[0] == 0x474e5089) && (m[1] == 0x0a1a0a0d))) return True;
	}
	return TImage_ChkPattern(gp, fn, hImageRD_png.pattern);
}

static bool GetLibErr() {
#if LD_LIB_PNG
	return False;
#else
	return pnglib_Err;
#endif
}

const ThImageLoader hImageRD_png = {
  "png",
  filetyp_PNG,
  ChkType,
  read_png,
  GetLibErr
};

