/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"
#include <setjmp.h>

#define USE_INBUFER 1

typedef struct TtmpIO {
#if USE_INBUFER
  int idx;
  int incount;
#endif
  int fu;	//1=buffer on
  const char *err_txt;
  jmp_buf setjmp_buffer;
  byte txt[32];
#if USE_INBUFER
  byte line[BUFSIZ];
#endif
} TtmpIO;

static void read_DATA(TImage *self, void *data, int lg) {
	if (fread(data, 1, lg, self->in_fp) != lg) 
			longjmp(self->tIO->setjmp_buffer, 1);
}

#if USE_INBUFER
static dword buf_getc(TImage *self) {
	if (self->tIO->incount == EOF) return EOF;
	if (self->tIO->idx >= self->tIO->incount) {
	    self->tIO->incount = fread(self->tIO->line, 1, BUFSIZ, self->in_fp);
	    if (self->tIO->incount <= 0) {
		self->tIO->incount = EOF;
		return EOF;
	    }
	    self->tIO->idx = 0;
	}
	return self->tIO->line[self->tIO->idx++];
}
#endif

static dword read_VAL(TImage *self) {
  dword value, b;
  bool bskip = False;
  bool valOk = False;
	value = 0;
loop:
#if USE_INBUFER
	if (self->tIO->fu & 1) b = buf_getc(self); else
#endif
	b = getc(self->in_fp);

	if (b == EOF) longjmp(self->tIO->setjmp_buffer, 1);

	if (b <= 0x20) {
	    if ((b == '\n') || (b == '\r')) bskip = False;
	    if (!bskip && valOk) return value;
	    goto loop;
	}
	if (bskip) goto loop;
	if (b == '#') {
	    bskip = True;
	    goto loop;
	}
	if ((b < '0') && (b > '9')) {
	    self->tIO->err_txt = "Format: not unsigned integer";
	    longjmp(self->tIO->setjmp_buffer, 1);
	}
	value = (value * 10) + (b & 0x0f);
	valOk = True;
	goto loop;
}

static char *read_TXT(TImage *self) {
  dword b;
  byte *pc;
  bool bskip = False;

	pc = self->tIO->txt;
	*pc = 0;
loop:
	b = getc(self->in_fp);
	if (b == EOF) longjmp(self->tIO->setjmp_buffer, 1);

	if (b <= 0x20) {
	    if ((b == '\n') || (b == '\r')) bskip = False;
	    if (!bskip && (pc > self->tIO->txt)) {
		*pc = 0;
		return (char*)self->tIO->txt;
	    }
	    goto loop;
	}
	if (bskip) goto loop;
	if (b == '#') {
	    bskip = True;
	    goto loop;
	}
	if ((b < 'A') && (b > 'W')) {
	    self->tIO->err_txt = "Format: not txt";
	    longjmp(self->tIO->setjmp_buffer, 1);
	}
	*pc = b; pc++;
	goto loop;
}

#define PNM_P5 1
#define PNM_P6 2
#define PNM_P7 3


int read_pnm(TImage *self) {
  ic32 ic;
  int pnm_maxval;
  int BixByte;
  byte *src_buf = NULL;
  TtmpIO *tIO;
  int pnmType;
  bool res = False;

	tIO = xcalloc(sizeof(TtmpIO));
	self->tIO = tIO;
	tIO->err_txt = "Format: ";
	if (setjmp(tIO->setjmp_buffer)) {
err:
	    appError(EMSG_ERR1, "%s(%d): %s \"%s\"", FL, tIO->err_txt, self->in_gp->name);
	    goto Close02;
	}

	pnmType = 0;
	BixByte = 0;
	read_DATA(self, &ic, 2);
	if (ic.w[0] == 0x3350) {
	    tIO->fu = 1;
	} else if (ic.w[0] == 0x3550) {
	    BixByte = 1;
	    pnmType = PNM_P5;
	} else if (ic.w[0] == 0x3650) {
	    BixByte = 3;
	    pnmType = PNM_P6;
	} else if (ic.w[0] == 0x3750) {
	    pnmType = PNM_P7;
	    BixByte = 3;
	} else goto err;

	pnm_maxval = 0;
	if (pnmType == PNM_P7) {
	    while (1) {
		char *pc = read_TXT(self);
		if (pccmp(pc, "WIDTH")) {
		    self->width = read_VAL(self);
		} else if (pccmp(pc, "HEIGHT")) {
		    self->height = read_VAL(self);
		} else if (pccmp(pc, "DEPTH")) {
		    BixByte = read_VAL(self);
		} else if (pccmp(pc, "MAXVAL")) {
		    pnm_maxval = read_VAL(self);
		} else if (pccmp(pc, "TUPLTYPE")) {
		    read_TXT(self);
		} else if (pccmp(pc, "ENDHDR")) {
		    goto P7_End;
		} else goto err;
	    }
P7_End:
	    if ((BixByte != 3)&&(BixByte != 4)) goto err;
	    if (BixByte == 3) pnmType = PNM_P6;
	} else {
	    self->width = read_VAL(self);
	    self->height = read_VAL(self);
	    pnm_maxval = read_VAL(self);
	}
// printf("%s(%d): %d %d %d %d\n", FL, self->width, self->height, pnm_maxval, depth);
	if (!pnm_maxval) goto err;
	if (pnm_maxval == 255) pnm_maxval = 0;
	if (self->pxm_LD & pxmLD_OnlyInfo) goto EndeOk;
	if (!TImage_SetMem(self)) goto err;

//  int lg = self->width * BixByte;
  byte *db, *db0, *sb, *pixAlpha;
  dword A = 255;
  bool rgb565 = (self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp;
  bool ALPHA = (pnmType == PNM_P7) && !(self->pxm_LD & pxmLD_NoMask);

	pixAlpha = NULL;
	if (rgb565 && ALPHA) pixAlpha = xmalloc(self->width * self->height);
	self->rgb565_alpha_data = pixAlpha;
	if (ALPHA) self->stat |= pmxStat_has_alpha;

	BixByte *= self->width;
	if (BixByte) src_buf = xmalloc(BixByte);
	db0 = self->data;
	for (int y=self->height;y;y--) {
	    if (src_buf) read_DATA(self, src_buf, BixByte);
	    db = db0;
	    sb = src_buf;
	    for (int x=self->width;x;x--) {
  dword R, G, B;
		if (sb) {
		    R = *sb++;
		    if (pnmType == PNM_P6) {
			G = *sb++;
			B = *sb++;
			if (pnmType == PNM_P7) {
			    if (ALPHA) A = *sb;
			    sb++;
			}
		    } else {
			G = B = R;
		    }
		} else {
		    R = read_VAL(self);
		    G = read_VAL(self);
		    B = read_VAL(self);
		}

		if (pnm_maxval) {
		    R = (R * 255) / pnm_maxval;
		    G = (G * 255) / pnm_maxval;
		    B = (B * 255) / pnm_maxval;
		    A = (A * 255) / pnm_maxval;
		}

		if (rgb565) {
		    *((word*)db) = ((R & 0xf8)<<8)|((G & 0xfc)<<3)|((B & 0xf8)>>3);
		    db += 2;
		    if (pixAlpha) *pixAlpha++ = A;
		} else {
		    *db++ = B;
		    *db++ = G;
		    *db++ = R;
		    *db++ = A;
		}
	    }
	    db0 += self->bytes_per_line;
	}
	alpha_bm_stat(self);
EndeOk:
	res = True;
Close02:
	xfree(src_buf);
	xfree(tIO);
	return res;
}

static bool ChkType(TglobPath *gp, const char *fn, FILE *F) {
	return TImage_ChkPattern(gp, fn, hImageRD_pnm.pattern);
}

const ThImageLoader hImageRD_pnm = {
  "pnm ppm pam",
  filetyp_PNM,
  ChkType,
  read_pnm
};
