/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"

static char *wr_colval(char *pc, int value) {
	ppcmove_v32(&pc, value, kSPACE_ADD);
	return pc;
}

static void write_fb(TImage *self, char *pc) {
	fwrite(self->tIO, 1, pc - (char*)self->tIO, self->out_fp);
}

int write_pnm(TImage *self) {
  int m;
  char *buf, *pc;
  const char *cpc;
  dword A;
  bool ALPHA;
  bool TxtPlain;
  bool rgb565;

	TxtPlain = (self->out_fu & fuWR_txtplain) != 0;
	ALPHA = !(self->out_fu & fuWR_NoMask) && alpha_akt(self) && !TxtPlain;
	if (ALPHA) {
	    cpc = GetFileExt_pc(self->out_gp->name);
	    if (pccasecmpASCII(cpc, "pnm") && pccasecmpASCII(cpc, "pam")) {
		ALPHA = False;
	    }
	}
	buf = xmalloc((self->width*4)+64);
	self->tIO = (void*)buf;
	pc = buf;
	if (ALPHA) {
	    ppcmove(&pc, "P7\nWIDTH ");
	    inttoppc(&pc, self->width);
	    ppcmove(&pc, "\nHEIGHT ");
	    inttoppc(&pc, self->height);
	    ppcmove(&pc, "\nDEPTH 4\nMAXVAL 255\nENDHDR\n");
	} else {
	    if (TxtPlain) ppcmove(&pc, "P3\n"); else ppcmove(&pc, "P6\n");
//	    WrInteger_ppc(self->width, &pc); *pc = ' '; pc++;
	    pc = wr_colval(pc, self->width);
	    inttoppc(&pc, self->height);
	    ppcmove(&pc, "\n255\n");
	}
	write_fb(self, pc);

	rgb565 = (self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp;

  byte *sb0 = self->data;
  byte *sbA = self->rgb565_alpha_data;
	A = 255;
	for (int y=self->height;y;y--) {
	    pc = buf;
	    m = 0;
	    byte *sb = sb0;
	    for (int x=self->width;x;x--) {
  dword R, G, B;
		if (rgb565) {
		    dword c = *((word*)sb);
		    sb += 2;
		    B = (c & 0x1f)<<3;
		    G = (c & 0x7e0)>>3;
		    R = (c & 0xf800)>>8;
		    if (sbA) A = *sbA++;
		} else {
		    B = *sb++;
		    G = *sb++;
		    R = *sb++;
		    A = *sb++;
		}

		if (TxtPlain) {
		    pc = wr_colval(pc, R);
		    pc = wr_colval(pc, G);
		    pc = wr_colval(pc, B);
		    m++;
		    if (m > 5) {
			m = 0;
			pc[-1] = '\n';
			write_fb(self, pc);
			pc = buf;
		    }
		} else {
		    *pc++ = R;
		    *pc++ = G;
		    *pc++ = B;
		    if (ALPHA) *pc++ = A;
		}
	    }

	    if (pc > buf) {
		if (TxtPlain) pc[-1] = '\n';
		write_fb(self, pc);
	    }
	    sb0 += self->bytes_per_line;
	}

	xfree(buf);
	return IMGWR_OK;
}

static bool ChkType(TglobPath *gp, const char *fn, FILE *F) {
	return TImage_ChkPattern(gp, fn, hImageWR_pnm.pattern);
}

const ThImageLoader hImageWR_pnm = {
  "pnm ppm pam",
  filetyp_PNM,
  ChkType,
  write_pnm
};
