/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"

typedef struct {
  ic32 t;
  int rgb;
} TxpmColDat;

typedef struct TtmpIO {
  char *line;
  int line_lg;
  int colors;
  int chr_per_pixel;
  TxpmColDat *colTab;
  TxpmColDat *last_colTab;
  ic32 none;
  int retst;
  int hashTab[4096];
} TtmpIO;

static dword getcharhash(dword b) {
	return (b + (b >> 12) + (b >> 20)) & 0xfff;
}

static char *ReadLine(TImage *self) {
	return fgets(self->tIO->line, self->tIO->line_lg, self->in_fp);
}

static char *skipb(char *s) {
	while (*s && ((*s == ' ') || (*s == 0x09))) s++;
	return s;
}

static bool read_xpm_colTab(TImage *self) {
  char *s;
  int i, m;
  TtmpIO *tIO = self->tIO;
  int count = self->tIO->colors;
  TxpmColDat *colTab = self->tIO->colTab;

loop:
	s = ReadLine(self);
	if (!s) return False;
	if (*s != '"') goto loop;

	s++;
	for (i=0; i<tIO->chr_per_pixel; i++) {
	    if (!*s) return False;
	    colTab->t.c[i] = *s;
	    s++;
	}
	s = skipb(s);

	while ((*s == 'm') || (*s == 's')) {
	    s = skipb(s+1);
	    while (*s && ((*s != ' ') && (*s != 0x09))) s++;
	    s = skipb(s);
	}
	if (*s != 'c') return False;
	s = skipb(s+1);
	if (!*s) return False;
	    /* Chk Color name */
	if (!pcncasecmpASCII(s, "none", 4)) {
	    tIO->none.dw = colTab->t.dw;
	    tIO->colors--;
	} else {
	    m = GetColorByName(s);
	    colTab->rgb = m;
	    if ((self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp)
		colTab->rgb = ((m >> 8)&0xf800)|((m >> 5)&0x7e0)|((m >> 3)&0x1f);
	    colTab->rgb |= 0xff000000;
	    colTab++;
	}
	count--;
	if (count) goto loop;
	return (tIO->colors > 0);
}

static int __attribute__ ((noinline)) getcolor_xc(TtmpIO *tIO, char **s) {
  ic32 t;
  int i;
  char *sc, *pc;
  TxpmColDat *colTab;

	t.dw = 0;
	tIO->retst = -1;	//Error
	i = tIO->chr_per_pixel;
	pc = *s;
	sc = t.c;
	while (i && *pc) {*sc = *pc; sc++; pc++; i--;}
	*s = pc;	//Set next s
	if (i) return 0;

	tIO->retst = 1;	//Mask
	if (tIO->none.dw == t.dw) return 0;

	tIO->retst = 0;	//Ok
	if (t.dw == tIO->last_colTab->t.dw) return tIO->last_colTab->rgb;

	i = getcharhash(t.dw);
	i = tIO->hashTab[i];
	colTab = tIO->colTab + i;
	for (;i<tIO->colors;i++) {
	    if (t.dw == colTab->t.dw) {
		tIO->last_colTab = colTab;
		return colTab->rgb;
	    }
	    colTab++;
	}

	tIO->retst = -2;	//Error NO Color
	return 0;
}

 
static bool __attribute__ ((noinline)) read_xpm_colDat(TImage *self) {
  char *s;
  int count = self->height;
  byte *data = self->data;
  byte *pixAlpha = NULL;
  bool rgb565;

	rgb565 = (self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp;
	if (self->tIO->none.dw != 0) {
	    self->stat |= pmxStat_has_alpha;
	    if (rgb565) pixAlpha = xmalloc(self->width * self->height);
	}
	self->rgb565_alpha_data = pixAlpha;

loop:
	s = ReadLine(self);
	if (!s) return False;
	if (*s != '"') goto loop;
	s++;

  byte *db = data;
	for (int x=self->width;x;x--) {

	    dword v = getcolor_xc(self->tIO, &s);
	    if (rgb565) {
		*((word*)db) = v; db += 2;
	    } else {
		*((dword*)db) = v; db += 4;
	    }
	    if (pixAlpha) {
		dword A = 0;
		if (v & 0x80000000) A = 255;
		*pixAlpha++ = A;
	    }
	}
	data += self->bytes_per_line;
	count--;
	if (count) goto loop;

	alpha_bm_stat(self);

	return True;
}

static int rd_xpm_sort_color(TxpmColDat *a, TxpmColDat *b, int fu) {
	return getcharhash(a->t.dw) - getcharhash(b->t.dw);
}

static bool __attribute__ ((noinline)) read_xpm_head(TImage *self) {
  const char *s;
  TtmpIO *tIO = self->tIO;

	Fill0(tIO, sizeof(TtmpIO));
	tIO->line_lg = 4096;
	tIO->line = xmalloc(tIO->line_lg);

	s = ReadLine(self);
	if (!pcncmp(s, "/* XPM */", 9)) return False;
loop:
	s = ReadLine(self);
	if (!s) return False;
	if (*s != '"') goto loop;
	s++;
	self->width = ppctoint(&s);
	self->height = ppctoint(&s);
	tIO->colors = ppctoint(&s);
	tIO->chr_per_pixel = ppctoint(&s);
	return (self->width > 0) && (self->height > 0) && 
		(tIO->colors > 0) && 
		(tIO->chr_per_pixel > 0) && (tIO->chr_per_pixel <= 4);
}

int read_xpm(TImage *self) {
  int m, k;
  TxpmColDat *colTab;
  TxpmColDat **pcolTab;
  TtmpIO tIO;
  bool res = False;

	self->tIO = &tIO;
	if (!read_xpm_head(self)) goto ende;

	if (self->pxm_LD & pxmLD_OnlyInfo) {
	    res = True;
	    goto ende;
	}

	m = tIO.chr_per_pixel * self->width + 8;
	if (m > tIO.line_lg) {
	    tIO.line_lg = m;
	    tIO.line = xrealloc(tIO.line, m);
	}

	tIO.colTab = xcalloc(sizeof(TxpmColDat) * tIO.colors);
	if (!read_xpm_colTab(self)) goto ende;

	pcolTab = xcalloc(sizeof(TxpmColDat*) * tIO.colors);
	for (int i=0;i<tIO.colors;i++) pcolTab[i] = &tIO.colTab[i];

	QuickSort_PL((void*)pcolTab, 0, tIO.colors-1, 0, 
		     (TCompareFunction)rd_xpm_sort_color);

	colTab = xcalloc(sizeof(TxpmColDat) * tIO.colors);
	for (int i=0;i<tIO.colors;i++) colTab[i] = *pcolTab[i];

	xfree(pcolTab);
	xfree(tIO.colTab);
	tIO.colTab = colTab;

	m = -1;
	for (int i=0;i<tIO.colors;i++) {
	    k = getcharhash(colTab->t.dw);
	    if (m != k) {
		m = k;
		tIO.hashTab[m] = i;
	    }
	    colTab++;
	}

	tIO.last_colTab = tIO.colTab;
	if (!TImage_SetMem(self)) goto ende;

	res = True;
	read_xpm_colDat(self);

ende:
	xfree(tIO.colTab);
	xfree(tIO.line);
	self->tIO = NULL;
	return res;
}

static bool ChkType(TglobPath *gp, const char *fn, FILE *F) {
	return TImage_ChkPattern(gp, fn, hImageRD_xpm.pattern);
}

const ThImageLoader hImageRD_xpm = {
  "xpm",
  filetyp_XPM,
  ChkType,
  read_xpm
};
