/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"

static int getcolhash(int c) {
	return ((c >> 12) & 0xf00) + ((c >> 8) & 0xf0) + ((c >> 4) & 0x0f);
}

static int write_xpm_sort_color(int a, int b, int fu) {
  int res = getcolhash(a) - getcolhash(b);
	if (!res) res = (a & 0x00ffffff)-(b & 0x00ffffff);
	return res;
}

static const char chrtab[64] = {".#abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"};

int write_xpm(TImage *self) {

  dword mask;
  char *pc, *pcn, *line;
  const char *cpc;
  byte *sb0, *sbm;
  int i, x, y, mk, col, Nchr, colmask;
  int *ColArr, *TxtArr, *cal, ColCount, *pint;
  int *hash64;
  bool ALPHA, rgb565;

	colmask = 0xffffff;
	ColCount = TImage_GetColorCount(self, pxgcc_RGB_ALL | pxgcc_ColorCount_AddMask);
	if (ColCount >= 0x40000) { //64*64*64=262144
	    ColCount = TImage_GetColorCount(self, pxgcc_RGB_666 | pxgcc_ColorCount_AddMask);
	    colmask = 0xfcfcfc;
	}
	TImage_GetColorMap(self);

	if (!self->cmap || !self->cmap->col_map) return IMGWR_ERR_FORMAT;

	ColArr = (int*)self->cmap->col_map;
	ALPHA = !(self->out_fu & fuWR_NoMask) && alpha_akt(self);

	y = 0;
	if (ALPHA) y++;
	QuickSort_PL((void*)ColArr, y, ColCount-1, 0, 
		     (TCompareFunction)write_xpm_sort_color);

	Nchr = 1;
	if (ColCount > 64) Nchr++;
	if (ColCount > 4096) Nchr++;
	TxtArr = xcalloc(sizeof(int)*(ColCount+2));

	pc = (char*)TxtArr;
	for (y=0;y<ColCount;y++) {
	    i = y;
	    for (x=Nchr;x;x--) {
		pc[x-1] = chrtab[i & 0x3f];
		i >>= 6;
	    }
	    pc += 4;
	}

	i = (self->width * Nchr) + 8;
	if (i < 256) i = 256;
	line = xmalloc(i);

	fputs("/* XPM */\n", self->out_fp);
	pc = ChangeFileExt(GetFileName_pc(self->out_gp->name), NULL);
	pcn = line;

	sprintf(line, "static char *%s[]={\n", pc);
	fputs(line, self->out_fp);
	xfree(pc);

	sprintf(line, "\"%d %d %d %d\",\n", self->width, self->height, ColCount, Nchr);
	fputs(line, self->out_fp);

	y = 0;
	pint = ColArr;
	if (ALPHA) {
	    pcn = line + 1;
	    pc = (char*)TxtArr;
	    for (i=Nchr;i;i--) {*pcn = *pc; pcn++; pc++;}
	    ppcmove(&pcn, " c None\",\n");
	    fputs(line, self->out_fp);
	    y++;
	    pint++;
	}

	hash64 = xcalloc(sizeof(int) * 4096);
	mk = -1;
	for (;y<ColCount;y++) {
	    col = getcolhash(*pint);
	    if (col != mk) {
		hash64[col] = y;
		mk = col;
	    }
	    pcn = line + 1;
	    pc = (char*)(TxtArr + y);
	    for (i=Nchr;i;i--) {*pcn = *pc; pcn++; pc++;}
	    ppcmove(&pcn, " c #");
	    hextoppc(&pcn, *pint, 6);
	    ppcmove(&pcn, "\",\n");
	    fputs(line, self->out_fp);
	    pint++;
	}

	rgb565 = (self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp;
	sbm = self->rgb565_alpha_data;
	sb0 = self->data;
	for (y=self->height;y;y--) {

	    pcn = line + 1;
	    x = self->width;

	    if (rgb565) {
		word *sw = (word*)sb0;
		while (x--) {
		  i = 0;
		  if (!sbm || !(*sbm++ & 0x80)) {
//wrcol16:
		    col = ((*sw & 0xf800)<<8)|((*sw & 0x7e0)<<5)|((*sw & 0x1f)<<3);
		    i = hash64[getcolhash(col)];
		    cal = ColArr + i;
		    while ((i<ColCount) && (*cal != col)) {cal++; i++;}
		  }
		  pc = (char*)(TxtArr+i);
		  for (i=Nchr;i;i--) {*pcn = *pc; pcn++; pc++;}
		  sw++;
		}
	    } else {
		dword *sdw = (dword*)sb0;
		while (x--) {
		  col = *sdw++;
		  if (!ALPHA) goto wrcol24;
		  mask = col & 0x80000000;
		  if (sbm) mask = *sbm++ & 0x80;

		  i = 0;
		  if (mask) {
wrcol24:
		    col &= colmask;
		    i = hash64[getcolhash(col)];
		    cal = ColArr + i;
		    while ((i<ColCount) && (*cal != col)) {cal++; i++;}
		  }
		  pc = (char*)(TxtArr+i);
		  for (i=Nchr;i;i--) {*pcn = *pc; pcn++; pc++;}
		}
	    }

	    cpc = "\",\n";
	    if (y == 1) cpc = "\"},\n";
	    ppcmove(&pcn, cpc);

	    fputs(line, self->out_fp);
	    sb0 += self->bytes_per_line;
	}

	xfree(hash64);
	xfree(line);
	xfree(TxtArr);
	return IMGWR_OK;
}

static bool ChkType(TglobPath *gp, const char *fn, FILE *F) {
	return TImage_ChkPattern(gp, fn, hImageWR_xpm.pattern);
}

const ThImageLoader hImageWR_xpm = {
  "xpm",
  filetyp_XPM,
  ChkType,
  write_xpm
};
