/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"
#include "tpixmap.h"
#include "tdecor.h"

static void decor_SetRegion(Window WinID, char *MaskDat, int bw, int x, int y) {
  Pixmap res;
	res = XCreateBitmapFromData(appXDat.dpy, appXDat.RootW, MaskDat, bw, bw);
	XShapeCombineMask(appXDat.dpy, WinID, 0, x, y, res, 3);
	XFreePixmap(appXDat.dpy, res);
}

typedef struct {
  union {
    char b[8];
    dword dw[2];
  };
  dword Mw, bw;
  const int8_t *cm;
} Tmd;

static bool SetMaskDat(Tmd *md, int xo, int yo) {
  dword m, k, bw;
  char *db;
  const int8_t *sb;

	md->dw[0] = md->dw[1] = 0;
	bw = md->bw;
	db = md->b;
	sb =  md->cm + xo + (yo * md->Mw);
	for (int y=bw;y;y--) {
	    m = 0;
	    k = 1;
	    for (int x=0;x<bw;x++) {
		if (sb[x] == 1) m |= k;
		k <<= 1;
	    }
	    *db = m; db++;
	    sb += md->Mw;
	}
	return ((md->dw[0] | md->dw[1]) != 0);
}

static int Get_bw(TRX_Dat *dat) {
  int bw = dat->BordMaskHead->Borderwidth & 0x0f;
	if (bw > 8) bw = 8;
	return bw;
}

void decor_RegionBR2_5(TRX_Dat *dat, int *stat) {
  dword Mh;
  Tmd md;

	if (*stat & imBst_MaskAkt) {
		*stat &= ~(Deco_MaskAkt|Deco_SizeOk);
		XShapeCombineMask(appXDat.dpy, dat->WinID, 0, 0, 0, 0, 0);
	}

	if (!dat->BordMaskHead) return;
	md.cm = dat->BordMaskHead->dat;
	md.Mw = dat->BordMaskHead->DatX;
	Mh = dat->BordMaskHead->DatY;
	if ((md.Mw > 32) || (Mh > 32)) return;
	*stat |= Deco_SizeOk;
	md.bw = Get_bw(dat);

	if (SetMaskDat(&md, 0, 0)) {
	  *stat |= Deco_MaskAkt;
	  decor_SetRegion(dat->WinID, md.b, md.bw, 0, 0);
	}

	if (SetMaskDat(&md, md.Mw - md.bw, 0)) {
	  *stat |= Deco_MaskAkt;
	  decor_SetRegion(dat->WinID, md.b, md.bw, dat->width - md.bw, 0);
	}

	if (SetMaskDat(&md, 0, Mh - md.bw)) {
	  *stat |= Deco_MaskAkt;
	  decor_SetRegion(dat->WinID, md.b, md.bw, 0, dat->height - md.bw);
	}

	if (SetMaskDat(&md, md.Mw - md.bw, Mh - md.bw)) {
	  *stat |= Deco_MaskAkt;
	  decor_SetRegion(dat->WinID, md.b, md.bw, dat->width - md.bw, dat->height - md.bw);
	}
}

bool decor_paintBR2_5(TRX_Dat *dat) {
  TPixMap *imgH, *imgV;
  long gcm;
  XGCValues gcv;
  int cornerlg, bw, xrep, yrep, cornerlg2x;

	if (!dat->BordMaskHead) return False;
	dat->BordMaskDat = dat->BordMaskHead->dat; // + dat->BordMaskHead->DatOff;
	dat->Mw = dat->BordMaskHead->DatX;
	dat->Mh = dat->BordMaskHead->DatY;
	cornerlg = dat->BordMaskHead->CornerLen;

	if ((dat->Mw > 32) || (dat->Mh > 32) || (cornerlg > 16)) return False;
	cornerlg2x = cornerlg * 2;
	xrep = dat->Mw - cornerlg2x - 1;
	if (xrep < 0) return False;
	yrep = dat->Mh - cornerlg2x - 1;
	if (yrep < 0) return False;
	dat->cornerlg = cornerlg;
	bw = Get_bw(dat);


	if (dat->org_pxm) {
	    imgH = TPixMap_Create_depth(dat->org_pxm->width, bw, 0);
	    imgV = TPixMap_Create_depth(bw, dat->org_pxm->height, 0);
	} else {
	    imgH = TPixMap_Create_depth(cornerlg, bw, 0);
	    imgV = TPixMap_Create_depth(bw, cornerlg, 0);
	}

	TPixMap_CreateXimg(imgH);
	TPixMap_CreateXimg(imgV);

	Fill0(&gcv, sizeof(XGCValues));
	gcv.line_width = bw;
	gcv.function = GXcopy;
	gcv.fill_style = FillTiled;
	gcv.plane_mask = AllPlanes;
	gcm = GCFunction | GCPlaneMask | GCGraphicsExposures | 
		GCLineWidth | GCFillStyle;
	dat->tgc = CreateGC(gcm, &gcv);

	imgH->gc = dat->tgc;
	imgV->gc = dat->tgc;

	dat->paint_pxm = imgH;

	dat->dx = dat->left;
	dat->dy = dat->top;
	dat->xMoff = 0;
	dat->yMoff = 0;
	dat->lx = cornerlg;
	dat->ly = bw;
	dat->xrep = cornerlg;
	dat->yrep = bw;
	dat->plg = cornerlg;
	TR3AddTab_PaintH(dat);

	dat->dx += dat->width - cornerlg;
	dat->xMoff = dat->Mw - cornerlg;
	TR3AddTab_PaintH(dat);

	dat->dx = dat->left;
	dat->dy += dat->height - bw;
	dat->xMoff = 0;
	dat->yMoff = dat->Mh - bw;
	TR3AddTab_PaintH(dat);

	dat->dx += dat->width - cornerlg;
	dat->xMoff = dat->Mw - cornerlg;
	TR3AddTab_PaintH(dat);


	if (dat->width > cornerlg2x) {
	    dat->xrep = xrep;
	    dat->dx = dat->left + cornerlg;
	    dat->dy = dat->top;
	    dat->xMoff = cornerlg;
	    dat->yMoff = 0;
	    if (dat->org_pxm) {
		dat->lx = dat->width - cornerlg2x;
		if (dat->lx > imgH->width) dat->lx = imgH->width;
	    }
	    dat->plg = dat->width - cornerlg2x;
	    TR3AddTab_PaintH(dat);

	    dat->dy += dat->height - bw;
	    dat->yMoff = dat->Mh - bw;
	    TR3AddTab_PaintH(dat);
	}

	dat->paint_pxm = imgV;
	dat->lx = bw;
	dat->xrep = bw;
	if (cornerlg > bw) {
	    dat->yrep = dat->paint_pxm->height;
	    dat->dx = dat->left;
	    dat->dy = dat->top + bw;
	    dat->xMoff = 0;
	    dat->yMoff = bw;
	    dat->ly = cornerlg - bw;
	    dat->plg = cornerlg - bw;
	    TR3AddTab_PaintVL(dat);

	    dat->dy = dat->top + dat->height - cornerlg;
	    dat->yMoff = dat->Mh - cornerlg;
	    TR3AddTab_PaintVL(dat);

	    dat->dx = dat->left + dat->width - bw;
	    dat->xMoff = dat->Mw - bw;
	    TR3AddTab_PaintVL(dat);

	    dat->dy = dat->top + bw;
	    dat->yMoff = bw;
	    TR3AddTab_PaintVL(dat);
	}

	if (dat->height > cornerlg2x) {
	    dat->yrep = yrep;
	    dat->dx = dat->left;
	    dat->dy = cornerlg;
	    dat->xMoff = 0;
	    dat->yMoff = cornerlg;
	    dat->ly = dat->height - cornerlg2x;
	    if (dat->ly > dat->paint_pxm->height) dat->ly = dat->paint_pxm->height;
	    dat->plg = dat->height - cornerlg2x;
	    TR3AddTab_PaintVL(dat);

	    dat->dx += dat->width - bw;
	    dat->xMoff = dat->Mw - bw;
	    TR3AddTab_PaintVL(dat);
	}

	FreeGC(dat->tgc);

	imgH->gc = NULL;
	imgV->gc = NULL;

	Tfree(imgH);
	Tfree(imgV);

	return True;
}

