/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/


#define picAkt 0x01
#define picAkt1 0x02
#define picAkt2 0x04
#define picAkt3 0x08

typedef struct TPicCache {
  struct TPicCache *next;
  int stat;
  TPic pic[4];
  int namelg;
  char name[];
} TPicCache;

typedef struct TPicCache_nxn {
  struct TPicCache_nxn *next;
  TPicCache *pch;
  int nxn_lg;
  char nxn[];
} TPicCache_nxn;

typedef struct {
  const char *name;
  const char *ext;
  const char *default_zico;
  int namelg;
  int ddkt;
  int size;
  const char *fn;
  TPicCache_nxn *pchnxn;
} Tpchh;


typedef struct TNotFound {
  struct TNotFound *next;
  char *name;
} TNotFound;


static const char mount5x5_AHico[] = {
 0x01,0x03,0x05,0x00,0x05,0x00,0x00,0x00,0x18,0x00,0xE3,0x19,0xAC,0x64,0xAD,0x76,
 0x10,0x73,0xFF,0x01,0x89,0xA2,0x12,0x00};

static const char mount9x9_AHico[] = {
 0x06,0x07,0x09,0x00,0x09,0x00,0x00,0x00,0x34,0x00,0x00,0x00,0xC0,0x00,0x44,0x2D,
 0xC6,0x33,0x07,0x57,0x32,0x8F,0xDE,0xF7,0x2C,0x82,0x5C,0x08,0x1F,0xC5,0xF0,0x4E,
 0x82,0xE1,0x3D,0x21,0xEC,0xE1,0xDD,0x09,0xE0,0xDD,0x4E,0xE0,0xDD,0x7A,0xE2,0x9D,
 0x1B,0x8A,0x61,0x05};

static const char stop5x5_AHico[] = {
 0x01,0x03,0x05,0x00,0x05,0x00,0x00,0x00,0x1A,0x00,0x00,0x10,0x62,0xA0,0x45,0xF1,
 0xEE,0xFF,0xEF,0x00,0x04,0x2A,0xA9,0xA1,0x42,0x00};

static const char stop9x9_AHico[] = {
 0x06,0x03,0x09,0x00,0x09,0x00,0x00,0x00,0x28,0x00,0x00,0x00,0x00,0xF8,0xFF,0xFF,
 0x08,0x96,0x44,0xA1,0x4C,0x41,0xEA,0x4D,0xA4,0xBB,0x16,0x0A,0xA3,0x37,0xA5,0xBB,
 0x46,0x2A,0x13,0x14,0x1A,0x95,0x60,0x08};

static TPicCache_nxn *PicCache_nxn = NULL;
static TNotFound *cNotFound = NULL;

static void PicCache_pxm2pic(TPicCache *pch, TPixMap *pxm, int idx) {
	TPixMap_XPutImage(pxm);
	pch->pic[idx].width = pxm->width;
	pch->pic[idx].height = pxm->height;
	pch->pic[idx].pic = pxm->pmxID;
	pch->pic[idx].mask = pxm->maskID;
	pch->stat |= picAkt << idx;
	pxm->pmxID = pxm->maskID = 0;
}

static void PicCache_pxm_add_pic(TPixMap *pxm, const char *pcpic) {
  TPixMap *pxmadd;
	pxmadd = CreateAHicoImg(pcpic);
	bitmask2alpha((TImage*)pxmadd);
#if 1
	image_copy((TImage*)pxm, pxm->width - pxmadd->width, 
				 pxm->height - pxmadd->height, 
		   (TImage*)pxmadd, 0, 0, 0, 0, IMGCOPY_MASK);
#else
	image_copy((TImage*)pxmadd, (TImage*)pxm, 0, 0, 
		    pxmadd->width, pxmadd->height, 
		    pxm->width - pxmadd->width, 
		    pxm->height - pxmadd->height, IMGCOPY_MASK);
#endif
	if (pxm->stat & pmxStat_has_alpha_tgl) alpha2bitmask((TImage*)pxm);
	Tfree(pxmadd);
}

static TPicCache *getfiledeskicon(Tpchh *pchh, int fu) {
  TPixMap *pxm, *pxmdup;
  TPicCache *pch;
  const char *pc;

#ifndef USE_NO_FILEIMAGE
	pxm = NULL;
	if (!(fu & gdifu_pvUnknown)) pxm = TPixMap_Loader(pchh->fn, NULL, 0);
	if (!pxm) {
#endif
	    if (fu & gdifu_noUnknown) return NULL;
	    pc = pchh->default_zico;
	    if (!pc) {
		pc = zico16unknown;
		if (pchh->size > 30) pc = zico32unknown;
	    }
	    pxm = CreateAHicoImg(pc);
	    bitmask2alpha((TImage*)pxm);
#ifndef USE_NO_FILEIMAGE
	}
#endif
	if ((pxm->height != pchh->size) && (fu & gdifu_pvZoom)) {
	    pxmdup = TPixMap_dup_new_size(pxm, pchh->size, pchh->size);
	    Tfree(pxm);
	    pxm = pxmdup;
	}

	pch = xcalloc(sizeof(TPicCache) + pclen(pchh->name) + 1);
	pch->next = pchh->pchnxn->pch;
	pchh->pchnxn->pch = pch;
	pcmove(pch->name, pchh->name);

	pch->namelg = pchh->namelg;

	PicCache_pxm2pic(pch, pxm, 0);

	if (pchh->ddkt == ddkt_FSDevice) {
	    pxmdup = (TPixMap*)TImage_Create_dup_struct((TImage*)pxm, dupfu_copy_all);

	    pc = mount5x5_AHico;
	    if (pchh->size > 22) pc = mount9x9_AHico;
	    PicCache_pxm_add_pic(pxm, pc);
	    PicCache_pxm2pic(pch, pxm, 1);
	    Tfree(pxm);

	    pxm = pxmdup;
	    pc = stop5x5_AHico;
	    if (pchh->size > 22) pc = stop9x9_AHico;
	    PicCache_pxm_add_pic(pxm, pc);
	    PicCache_pxm2pic(pch, pxm, 2);
	}
	Tfree(pxm);
	return pch;
}

static TPicCache *getPicCache(Tpchh *pchh) {
  TPicCache *pch = pchh->pchnxn->pch;

	while (pch && ((pch->namelg != pchh->namelg) ||
		       !pccmp(pch->name, pchh->name))) {
	    pch = pch->next;
	}
	return pch;
}

#ifndef USE_NO_FILEIMAGE
static const char *findin[5] = {
  "actions/", "mimetypes/", "apps/", "devices/", "filesystems/"
};

static void InitIconPaths_src(TpcIni *ini) {
  int i;
  char *pc;
  const char *token;
  TpcIni *tini = ini;

	if (!tini) tini = TpcIni_Create(appDat.inifile_global);
	set_ptr_arr(&appXDat.pathPicRoot, 16);
	i = 0;
#ifdef _OS_WIN32
	char *w32ico = pcadd_n(3, "\"", appDat.ALDini_dir, "\\share\\icons\"");
	token = TpcIni_RD_pc(tini, "Icon", "IconPaths", w32ico);
#else
	token = TpcIni_RD_pc(tini, "Icon", "IconPaths", ALDccpPREFIX"/share/icons/");
#endif
	while (token && (i<15)) {
	    pc = NULL;
	    token = GetToken(token, &pc);
	    appXDat.pathPicRoot->val[i] = pc;
	    i++;
	}
#ifdef _OS_WIN32
	xfree(w32ico);
#endif
	if (tini != ini) Tfree(tini);
}
#endif /* ! USE_NO_FILEIMAGE */

static TPic *GetDeskIcon_src(int ddkt, int size, const char *name, int fu, const char *default_zico) {
  int idx;
  int nxn_lg = 0;
#ifndef USE_NO_FILEIMAGE
  int fiidx = 0;
  int ZoomAuto = 0;
  char *pcfi;
#endif /* ! USE_NO_FILEIMAGE */
  char *pc;
  const char *pcn, *cpc;
  TPicCache *pch;
  Tpchh pchh;
  char bufnxn[150];
  char buf[256];

	if (!name || !*name) return GetZIcon(default_zico);
	fu &= gdifu_mask;

	pchh.name = name;
	pchh.ext = GetFileExt_dot_pc(name);
	pchh.namelg = pclen(name);
	if (pchh.namelg > (sizeof(buf)-64)) return NULL;
	if (pchh.ext) pchh.namelg = pchh.ext - name;

	pchh.ddkt = ddkt;
	pchh.size = size;
	pchh.default_zico = default_zico;
	pchh.fn = buf;

	if (!appXDat.pathPicRoot) InitIconPaths(NULL);
	if (!PicCache_nxn) PicCache_nxn = xcalloc(sizeof(TPicCache_nxn)+4);

	pchh.pchnxn = PicCache_nxn;
	if (*name == '/') {
	    pchh.fn = name;
	    pch = getPicCache(&pchh);
#ifndef USE_NO_FILEIMAGE
	    if (!pch) {
		if (!(fu & gdifu_noZoom)) fu |= gdifu_pvZoom;
		pch = getfiledeskicon(&pchh, fu);
	    }
#endif /* ! USE_NO_FILEIMAGE */
	} else {
	    pc = bufnxn;
	    cpc = GetFileName_pc(name);
	    if (cpc != name) {
		if ((cpc - name) > (sizeof(bufnxn)-1)) return NULL;
		ppcnmove(&pc, name, cpc - name);
		pchh.name = cpc;
	    }
	    *pc = 0;

#ifdef USE_NO_FILEIMAGE
	    if (!bufnxn[0]) {
#else
	    pcfi = pc;
	    if (bufnxn[0]) {
		fiidx = 99;	//no scan
	    } else {
#endif /* ! USE_NO_FILEIMAGE */
		if (fu & gdifu_PathScalable) {
		    pcn = "scalable"; 
		} else {
		    pcn = "16x16";
		    if (size > 70) pcn = "128x128"; else
		    if (size > 34) pcn = "64x64"; else
		    if (size > 24) pcn = "32x32"; else
		    if (size > 20) pcn = "22x22";
		}
#ifndef USE_NO_FILEIMAGE
loop_zoom:
#endif /* ! USE_NO_FILEIMAGE */
		pc = bufnxn;
		ppcmove(&pc, pcn);
		*pc = '/'; pc++; *pc = 0;

#ifndef USE_NO_FILEIMAGE
		pcfi = pc;
		pcn = NULL;
		if (ddkt == ddkt_Application) {
		    pcn = "apps/";
		} else if (ddkt == ddkt_FSDevice) {
		    pcn = "devices/";
		} else if ((ddkt == ddkt_Directory) || 
			   (ddkt == ddkt_Actions) || 
			   (ddkt == ddkt_ALDeskBuildin)) {
		    pcn = "actions/";
		} else if (ddkt == ddkt_Mime) {
		    pcn = "mimetypes/";
		}
		if (pcn) ppcmove(&pc, pcn);
#endif /* ! USE_NO_FILEIMAGE */
	    }

	    nxn_lg = pclen(bufnxn);
	    if (nxn_lg) {
  TPicCache_nxn *anxn, *onxn;
		onxn = pchh.pchnxn;
loop_nxn:
		anxn = onxn->next;
		if (anxn && ((anxn->nxn_lg != nxn_lg) ||
			    !pccmp(anxn->nxn, bufnxn))) {
		    onxn = anxn;
		    goto loop_nxn;
		}
		if (!anxn) {
		    anxn = xcalloc(sizeof(TPicCache_nxn) + nxn_lg + 1);
		    onxn->next = anxn;
		    pcmove(anxn->nxn, bufnxn);
		    anxn->nxn_lg = nxn_lg;
		}
		pchh.pchnxn = anxn;
	    }
	    pch = getPicCache(&pchh);
#ifndef USE_NO_FILEIMAGE
	    if (!*pcfi && (fiidx < 5)) {
		pcmove(pcfi, findin[fiidx]);
		fiidx++;
	    }
fi_loop:
	    idx = 0;
	    while (!pch) {
		pcn = appXDat.pathPicRoot->val[idx];
		if (!pcn) break;
		idx++;
		if (TestIsPath(pcn)) {
		    pc = buf;
		    ppcmove(&pc, pcn);
		    if (pc[-1] != '/') {*pc = '/'; pc++;}
		    ppcmove(&pc, bufnxn);
		    ppcmove(&pc, pchh.name);
		    bool ok = False;
		    if (pchh.ext) {
			ok = TestIsFile(buf);
			if (!ok) pc -= pclen(pchh.ext);
		    }

		    if (!ok && (TImageRD_AllFiletyp & filetyp_PNG)) {
			pcmove(pc, ".png");
			ok = TestIsFile(buf);
		    }

		    if (!ok && (TImageRD_AllFiletyp & filetyp_XPM)) {
			pcmove(pc, ".xpm");
			ok = TestIsFile(buf);
		    }

		    if (!ok && (TImageRD_AllFiletyp & filetyp_BMP)) {
			pcmove(pc, ".bmp");
			ok = TestIsFile(buf);
		    }

		    if (ok) pch = getfiledeskicon(&pchh, fu);
		}
	    }
	    if (!pch && (fiidx < 5)) {
		pcmove(pcfi, findin[fiidx]);
		fiidx++;
		goto fi_loop;
	    }
#endif /* ! USE_NO_FILEIMAGE */

	}

#ifndef USE_NO_FILEIMAGE
	if (!pch && nxn_lg && (fiidx == 5) && !(fu & gdifu_noZoom) && !(fu & gdifu_pvZoom)) {
	    fu |= gdifu_pvZoom;
	    if ((ZoomAuto == 0) && (pchh.size < 32)) {
		fiidx = 0;
		pcn = "32x32";
		ZoomAuto++;
		goto loop_zoom;
	    } else if ((ZoomAuto == 1) && (pchh.size < 48)) {
		pcn = "48x48";
		ZoomAuto++;
		goto loop_zoom;
	    }
	}
#endif /* ! USE_NO_FILEIMAGE */


	if (!pch) {
	    if (!pchh.default_zico) {
		TNotFound *cnf = cNotFound;
		while (cnf && !pccmp(cnf->name, pchh.name)) {
		    cnf = cnf->next;
		}
		if (!cnf) {
		    if (!(fu & gdifu_noError)) {
			printf("%s(%d) Icon not found \"%s\"\n", FL, pchh.name);
		    }
		    cnf = xcalloc(sizeof(TNotFound));
		    cnf->name = pcdup(pchh.name);
		    cnf->next = cNotFound;
		    cNotFound = cnf;
		}
	    }
	    pch = getfiledeskicon(&pchh, fu | gdifu_pvUnknown);
	}
	if (!pch || !(pch->stat & picAkt)) return NULL;

	idx = 0;
	fu &= gdifu_geticomask;
	if (fu == gdifu_getico1) {
	    if (pch->stat & picAkt1) idx = 1;
	} else if (fu == gdifu_getico2) {
	    if (pch->stat & picAkt2) idx = 2;
	} else if (fu == gdifu_getico3) {
	    if (pch->stat & picAkt3) idx = 3;
	}
	return &pch->pic[idx];
}
