/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"

typedef struct {
  const int sr, sg, sb;
} TCR_RGB_SHIFT;

static const TCR_RGB_SHIFT rgbShift15[6] = {
  {11, 6, 0},
  {11, 0, 6},
  { 6,11, 0},
  { 6, 0,16},
  { 0,11, 6},
  { 0, 6,11}
};

static const TCR_RGB_SHIFT rgbShift24[6] = {
  {16, 8, 0},
  {16, 0, 8},
  { 8,16, 0},
  { 8, 0,16},
  { 0,16, 8},
  { 0, 8,16}
};

bool TImage_ColorOrder(TImage *self, int order) {
  int sr, sg, sb, h, w;
  byte *sd0;
  dword A, R, G, B;

	if (!self || !self->bytes_per_line || 
	    (order < 0) || (order >= 6)) return false;
	if (order == IMGCOLORORDER_RGB_TO_RGB) return true;

	if ((self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) {
	    sr = rgbShift15[order].sr;
	    sg = rgbShift15[order].sg;
	    sb = rgbShift15[order].sb;

	    sd0 = self->data;
	    for (h=self->height;h>0;h--) {
		word *sd = (word*)sd0;
		for (w=self->width;w>0;w--) {
		    R = (*sd & 0xf800) >> 11;
		    G = (*sd & 0x07c0) >> 6;
		    B =  *sd & 0x001f;
		    *sd = (R << sr) | (G << sg) | (B << sb); 
		    sd++;
		}
		sd0 += self->bytes_per_line;
	    }

	} else {
	    sr = rgbShift24[order].sr;
	    sg = rgbShift24[order].sg;
	    sb = rgbShift24[order].sb;

	    sd0 = self->data;
	    for (h=self->height;h>0;h--) {
		dword *sd = (dword*)sd0;
		for (w=self->width;w>0;w--) {
		    A = *sd & 0xff000000;
		    R = (*sd & 0x00ff0000) >> 16;
		    G = (*sd & 0x0000ff00) >> 8;
		    B =  *sd & 0x000000ff;
		    *sd = A | (R << sr) | (G << sg) | (B << sb); 
		    sd++;
		}
		sd0 += self->bytes_per_line;
	    }
	}
	self->stat &= ~pmxStat_XPixAkt;
	return true;
}
