/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"

#define USE_pxm_2b_16bpp 1

static void image_copy32_mask(TImage *dest, int dest_x, int dest_y,
			       TImage *src, int x, int y, int w, int h) {
  byte *sd0, *dd0;

	sd0 = src->data  + (src->bytes_per_line * y) + (x * 4);
	dd0 = dest->data + (dest->bytes_per_line * dest_y) + (dest_x * 4);
	for (;h>0;h--) {
	    dword *sd = (dword*)sd0;
	    dword *dd = (dword*)dd0;
	    for (int i=w;i>0;i--) {
		if (*sd & 0x80000000) *dd = *sd; 
		dd++; sd++;
	    }
	    sd0 += src->bytes_per_line;
	    dd0 += dest->bytes_per_line;
	}
}

#if USE_pxm_2b_16bpp
static void __attribute__ ((noinline)) image_copy16_blend(TImage *dest, int dest_x, int dest_y,
			       TImage *src, int x, int y, int w, int h) {
  byte *sd0, *sm0, *dd0, *sm;
  dword R, G, B, A1, A2;

	sd0 = src->data  + (src->bytes_per_line * y) + (x * 2);
	sm0 = src->rgb565_alpha_data + (src->width * y) + x;
	dd0 = dest->data + (dest->bytes_per_line * dest_y) + (dest_x * 2);
	for (;h>0;h--) {
	    word *sd = (word*)sd0;
	    word *dd = (word*)dd0;
	    sm = sm0;
	    for (int i=w;i>0;i--) {
		A1 = *sm;
		if (A1 >= 0xf8) {
		    *dd = *sd;
		} else if (A1 > 0x08) {
		    A2 = 0x100 - A1;
		    R = ((((*sd & 0xf800) * A1) + ((*dd & 0xf800) * A2)) >> 8) & 0xf800;
		    G = ((((*sd & 0x07e0) * A1) + ((*dd & 0x07e0) * A2)) >> 8) & 0x07e0;
		    B = ((((*sd & 0x001f) * A1) + ((*dd & 0x001f) * A2)) >> 8) & 0x001f;
		    *dd = R | G | B;
		}
		dd++; sd++; sm++;
	    }
	    sd0 += src->bytes_per_line;
	    sm0 += src->width;
	    dd0 += dest->bytes_per_line;
	}
}
#endif /* USE_pxm_2b_16bpp */

static void image_copy32_blend(TImage *dest, int dest_x, int dest_y,
			       TImage *src, int x, int y, int w, int h) {
  byte *sd0, *dd0;
  dword R, G, B, A1, A2;

	sd0 = src->data  + (src->bytes_per_line * y) + (x * 4);
	dd0 = dest->data + (dest->bytes_per_line * dest_y) + (dest_x * 4);
	for (;h>0;h--) {
	    dword *sd = (dword*)sd0;
	    dword *dd = (dword*)dd0;
	    for (int i=w;i>0;i--) {
		A1 = (*sd >> 24) & 0xff;
		if (A1 == 0xff) {
		    *dd = *sd;
		} else if (A1) {
		    A2 = 0x100 - A1;
		    R = ((((*sd & 0xff0000) * A1) + ((*dd & 0xff0000) * A2)) >> 8) & 0xff0000;
		    G = ((((*sd & 0x00ff00) * A1) + ((*dd & 0x00ff00) * A2)) >> 8) & 0x00ff00;
		    B = ((((*sd & 0x0000ff) * A1) + ((*dd & 0x0000ff) * A2)) >> 8) & 0x0000ff;
		    *dd = R | G | B | (Clip255[A1 + ((*dd >> 24) & 0xff)] << 24);
		} 
		dd++; sd++;
	    }
	    sd0 += src->bytes_per_line;
	    dd0 += dest->bytes_per_line;
	}
}

static void image_copy16(TImage *dest, int dest_x, int dest_y,
			 TImage *src, int x, int y, int w, int h) {
  byte *sd0, *dd0;

	sd0 = src->data  + (src->bytes_per_line * y) + (x * 2);
	dd0 = dest->data + (dest->bytes_per_line * dest_y) + (dest_x * 2);
	for (;h>0;h--) {
	    word *sd = (word*)sd0;
	    word *dd = (word*)dd0;
	    for (int i=w;i>0;i--) {
		*dd = *sd; 
		dd++; sd++;
	    }
	    sd0 += src->bytes_per_line;
	    dd0 += dest->bytes_per_line;
	}
}

static void image_copy32(TImage *dest, int dest_x, int dest_y,
			 TImage *src, int x, int y, int w, int h) {
  byte *sd0, *dd0;

	sd0 = src->data  + (src->bytes_per_line * y) + (x * 4);
	dd0 = dest->data + (dest->bytes_per_line * dest_y) + (dest_x * 4);
	for (;h>0;h--) {
	    dword *sd = (dword*)sd0;
	    dword *dd = (dword*)dd0;
	    for (int i=w;i>0;i--) {
		*dd = *sd; 
		dd++; sd++;
	    }
	    sd0 += src->bytes_per_line;
	    dd0 += dest->bytes_per_line;
	}
}

void image_copy(TImage *dest, int dest_x, int dest_y,
		TImage *src, int x, int y, int w, int h, int fu) {
  TImage *nsrc, *ndest;
  bool alpha;

	if (!src || !dest) return;
	if (w <= 0) w = src->width;
	if (h <= 0) h = src->height;

	if ((x + w) > src->width) w = src->width - x;
	if ((y + h) > src->height) h = src->height - y;
	if ((dest_x + w) > dest->width) w = dest->width - dest_x;
	if ((dest_y + h) > dest->height) h = dest->height - dest_y;
	if (w <= 0) return;
	if (h <= 0) return;

	nsrc = src;
	ndest = dest;

// printf("%s(%d): 0x%07x 0x%07x\n", FL, (int)src->rgb565_alpha_data, (int)dest->rgb565_alpha_data);
// dword t = GetTime_us();

	if (((src->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) &&
	    ((dest->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp)) {

	    if (!fu || !src->rgb565_alpha_data) {
		image_copy16(ndest, dest_x, dest_y, nsrc, x, y, w, h);
		goto set_stat;
	    } 
#if USE_pxm_2b_16bpp
	    if ((fu & IMGCOPY_MASK_BLEND) && !dest->rgb565_alpha_data) {
		image_copy16_blend(ndest, dest_x, dest_y, nsrc, x, y, w, h);
		goto set_stat;
	    }
#endif /* USE_pxm_2b_16bpp */
	}

	alpha = (src->stat & pmxStat_ckh_alpha) || (dest->stat & pmxStat_ckh_alpha);
	if ((src->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) {
	    nsrc = TImage_rgb565to888(src);
	}
	if ((dest->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) {
	    ndest = TImage_rgb565to888(dest);
	}

	if (alpha && (fu & IMGCOPY_MASK_BLEND)) {
	    image_copy32_blend(ndest, dest_x, dest_y, nsrc, x, y, w, h);
	} else if (alpha && (fu & IMGCOPY_MASK)) {
	    image_copy32_mask(ndest, dest_x, dest_y, nsrc, x, y, w, h);
	} else {
	    image_copy32(ndest, dest_x, dest_y, nsrc, x, y, w, h);
	}
	if (src != nsrc) Tfree(nsrc);

set_stat:
	dest->stat &= ~(pmxStat_XPixAkt|pmxStat_XMaskPixAkt);
	if (dest != ndest) {
	    dest->stat &= ~pmxStat_MaskImgAkt;
	    xfree(dest->mask_data);
	    dest->mask_data = NULL;
	    alpha_bm_stat(ndest);
	    if (!dest->rgb565_alpha_data && alpha_chk_mask(ndest, 0xff)) {
		dest->rgb565_alpha_data = xmalloc(dest->width * dest->height);
	    }

	    dest->stat &= ~pmxStat_has_alpha_tgl;
	    dest->stat |= (ndest->stat & pmxStat_has_alpha_tgl);
	    TImage_rgb888move565(ndest, dest);
	    Tfree(ndest);
	}
//  if (!(dest_x|dest_y|x|y)) printf("%s(%d): 0x%02x %dus w=%d h=%d\n", FL, fu, GetTime_us() - t, w, h);
}
