/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"
#include "pen_pv.h"
#include "cairo/cairo.h"

#define TCSelf TPenCairo
typedef struct TPenCairo {
  AlgTPen
} TPenCairo;
#undef TCSelf

static void line_to(TPenCairo *self, float x, float y) {
	if ((self->x == x) && (self->y == y)) return;
	self->x = x;
	self->y = y;
	cairo_line_to(self->img->struct_draw, x, y);
	cairo_stroke (self->img->struct_draw);
}

static void move_to(TPenCairo *self, float x, float y) {
	self->x = x;
	self->y = y;
	cairo_move_to(self->img->struct_draw, x, y);
}

static void set_line_width(TPenCairo *self, float width) {
	if (self->line_width != width) {
	    self->line_width = width;
	    cairo_set_line_width(self->img->struct_draw, width);
	}
}

static void set_color(TPenCairo *self, dword Color) {
	if (self->Color != Color) {
	    self->Color = Color;
	    cairo_set_source_rgba(self->img->struct_draw, 
			((float)((Color >> 16) & 0xff)) / 255, 
			((float)((Color >> 8 ) & 0xff)) / 255,
			((float)(Color & 0xff)) / 255,
			((float)((Color >> 24) & 0xff)) / 255);
	}
}

static void cairo_flush(TPenCairo *self) {
	cairo_stroke(self->img->struct_draw);
}
/*_____________________________ Destroy _________________________________*/
static bool TPen__Destroy(TPen *self) {
	if (self->img) {
	    cairo_destroy(self->img->struct_draw);
	    self->img->struct_draw = NULL;
	}
	return false;
}

/*_____________________________ Create __________________________________*/
void TImagePenCairo_Create(TImage *img) {
  TPenCairo *self;
  cairo_surface_t *surface;

	if (!img) return;
	TPen_Create(img, sizeof(TPenCairo), PEN_ART_CAIRO);
	self = (TPenCairo*)img->Pen;
	if (!self) return;
	self->destroy = (TDESTROY)TPen__Destroy;
	self->set_color = set_color;
	self->set_line_width = set_line_width;
	self->move_to = move_to;
	self->line_to = line_to;
	self->flush = cairo_flush;

	if ((img->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) {
	    surface = cairo_image_surface_create_for_data(img->data, 
				CAIRO_FORMAT_RGB16_565, img->width, img->height, 
				img->bytes_per_line);
	} else {
	    surface = cairo_image_surface_create_for_data(img->data, 
				CAIRO_FORMAT_ARGB32, img->width, img->height, 
				img->bytes_per_line);
	}
	img->struct_draw = cairo_create(surface);
	cairo_surface_destroy(surface);
	cairo_set_line_width(img->struct_draw, self->line_width);
}
