/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"
#include "image_pv.h"

static ThImageLoader **hImageRD;
static ThImageLoader **hImageWR;

dword TImageRD_AllFiletyp;
dword TImageWR_AllFiletyp;

bool TImage_ChkPattern(TglobPath *gp, const char *fn, const char *Pattern) {
  const char *FileExt, *indata, *token;
  int lg;

	FileExt = fn;
	if (gp) FileExt = gp->name;

	FileExt = GetFileExt_pc(FileExt);
	if (!FileExt) return False;
	lg = pclen(FileExt);

	indata = Pattern;
	while (indata) {
	    int tlg;
	    indata = pcGetToken(indata, &token, &tlg);
	    if ((lg == tlg) && 
		!pcncasecmpASCII(FileExt, token, tlg)) return True;
	}
	return False;
}

ThImageLoader *TImage_get_hLoader(const char *fn, TglobPath *gp, 
				dword pxm_LD, FILE *F, bool wr) {
  ThImageLoader **pphLD, *hLD;

	if (!fn && !gp && !F) return NULL;

	pphLD = hImageRD;
	if (wr) pphLD = hImageWR;
	if (pphLD) {
loop:
	    hLD = *pphLD;
	    if (hLD) {
		if (hLD->ChkType && hLD->ChkType(gp, fn, F)) return hLD;
		pphLD++;
		goto loop;
	    }
	}

	if (!(pxm_LD & pxmLD_OnlyInfo) &&
	    !(pxm_LD & pxmLD_NoERR)) {
	    const char *msg = "read";
	    if (wr) msg = "write";
	    char *tmp = get_DisplayFileStr(fn, gp);
	    appError(EMSG_ERR1, "image %s, unknown file format \n \"%s\"", msg, tmp);
	    xfree(tmp);
	}
	return NULL;
}

FILE REGP3 *TImageIO_OpenStream(TglobPath *gp, dword pxm_LD, bool wr) {
  FILE *F;
  const char *iofkt;

	iofkt = "rb";
	if (wr) iofkt = "wb";

	if (pxm_LD & pxmLD_NoERR) {
	    F = net_stream_open_gp_noERR(gp, iofkt);
	} else {
	    F = net_stream_open_gp(gp, iofkt);
	}
	if (F == PTR_FAILED) F = NULL;
	return F;
}

FILE *TImageIO_GetStream(const char *fn, TglobPath *gp, dword pxm_LD, 
			ThImageLoader **pphLD, bool wr) {
  ThImageLoader *hLD;
  FILE *F;
  TglobPath *gpt;

	hLD = TImage_get_hLoader(fn, gp, pxm_LD, NULL, wr);
	if (pphLD) *pphLD = hLD;
	if (!hLD) return NULL;

	gpt = gp;
	if (!gpt) gpt = pctogp(fn);
	F = TImageIO_OpenStream(gpt, pxm_LD, wr);
	if (gpt != gp) gpFree(gpt);
	return F;
}


char *TImageIO_GetAllFilePattern(bool wr) {
  char *result;
  ThImageLoader **pphLD;

	pphLD = hImageRD;
	if (wr) pphLD = hImageWR;
	if (!pphLD) return NULL;

	result = NULL;
	while (*pphLD) {
	    if ((*pphLD)->ChkType && (*pphLD)->ioproc && (*pphLD)->pattern) {
		ppcadd_n(&result, 3, "\"", (*pphLD)->pattern, "\" ");
	    }
	    pphLD++;
	}
	return result;
}

static dword SetAllFiletyp(ThImageLoader **pphLD) {
  dword result;

	if (!pphLD) return 0;
	result = 0;
	while (*pphLD) {
	    if ((*pphLD)->ChkType && (*pphLD)->ioproc) result |= (*pphLD)->filetyp;
	    pphLD++;
	}
	return result;
}

void ThImageRD_Reg(int n, ...) {
  va_list args;
	va_start(args, n);
	pp_void_va_add((void*)&hImageRD, n, args);
	va_end(args);
	TImageRD_AllFiletyp = SetAllFiletyp(hImageRD);
}

void ThImageWR_Reg(int n, ...) {
  va_list args;
	va_start(args, n);
	pp_void_va_add((void*)&hImageWR, n, args);
	va_end(args);
	TImageWR_AllFiletyp = SetAllFiletyp(hImageWR);
}
