/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "image.h"

#define USE_MMX_ReMap 0

#if USE_MMX_ReMap
void getnearest_init_mmx() {
  static const dword mmxconst[4]={0xffffffff,0x0000ffff,0x00010001,0x00000001};
 __asm__ volatile(
		"movq		(%0), %%mm6 \n"
		"movq		8(%0), %%mm7 \n"
		:: "r"(mmxconst));
}

void end_mmx() {
 __asm__ volatile(
		"emms \n");
}

dword getnearest_mmx(TColMap *cmap, ic32 col) {
  dword idx;

 __asm__ volatile(
		"mov		%1, %%edx \n"
		"mov		%%edx, %%ebx \n"
		"mov		$0xffffff, %%ecx \n"
		"mov		%2, %%esi \n"
		"movd		%3, %%mm0 \n"
		"punpcklbw	%%mm2, %%mm0 \n"
	      "0: \n"
		"movd		(%%esi), %%mm1 \n"
		"add		$4, %%esi \n"
		"punpcklbw	%%mm2, %%mm1 \n"
		"psubw		%%mm0, %%mm1 \n"
		"pand		%%mm6, %%mm1 \n"
		"pmaddwd	%%mm1, %%mm1 \n"
		"packssdw	%%mm1, %%mm1 \n"
		"pmaddwd	%%mm7, %%mm1 \n"
		"movd		%%mm1, %%edi \n"
		"cmp		%%edi, %%ecx \n"
	      "jle 1f \n"
		"mov		%%edi, %%ecx \n"
		"mov		%%ebx, %%edx \n"
	      "1: \n"

		"dec		%%ebx \n"
	      "jne 0b \n"
		"mov		%2, %%esi \n"
		"mov		%1, %%eax \n"
		"sub		%%edx, %%eax \n"
		"mov		(%%esi,%%eax,4), %0\n"
		: "=%eax"(idx)
		: "o"(cmap->quant_count), "o"(cmap->quant_col_map), "m"(col)
		: "%ebx", "%ecx", "%edx", "%esi", "%edi");
	return idx;
}
#else
static dword getnearest(TImage *self, ic32 *col) {
  int d, k, min, idx;
  ic32 *quant;

	min = 0xffffff;
	idx = 0;
	quant = self->cmap->quant_col_map;
	for (int i=self->cmap->quant_count;i;i--) {
	    d = quant->b[0] - col->b[0]; k  = d * d;
	    d = quant->b[1] - col->b[1]; k += d * d;
	    d = quant->b[2] - col->b[2]; k += d * d;
	    quant++;
	    if (min > k) {min = k; idx=i;}
	}
	return self->cmap->quant_col_map[self->cmap->quant_count - idx].dw;
}
/* static dword getnearest_f8(TImage *self, ic32 *col) {
  int r, g, b, k;
  int min = 0xffffff;
  int idx = 0;
  ic32 *quant = self->cmap->quant_col_map;

	for (int i=self->cmap->quant_count;i;i--) {
	    b = (quant->b[0] & 0xfc) - col->b[0];
	    g = (quant->b[1] & 0xfc) - col->b[1];
	    r = (quant->b[2] & 0xfc) - col->b[2];
	    quant++;
	    k = r*r + g*g + b*b;
	    if (min > k) {min = k; idx=i;}
	}
	return self->cmap->quant_col_map[self->cmap->quant_count - idx].dw;
} */
#endif

static void ReMapColor16(TImage *self, dword *coltab) {
  int eoff, h;
  dword k;
  word *pws;

	pws = (word*)self->data;
	eoff = (self->bytes_per_line >> 1) - self->width;
	h = self->height;
	while (h) {   
	  if (coltab) {
	    for (int i=self->width;i;i--) {
		k = ((*pws & 0xffc0)>>1)|(*pws & 0x1f);
		*pws = coltab[k];
		pws++;
	    }
	  } else {
	    for (int i=self->width;i;i--) {
		k = ((*pws & 0xf800)<<8)|((*pws & 0x7e0)<<5)|((*pws & 0x1f)<<3);
#if USE_MMX_ReMap
		k = getnearest_mmx(self->cmap, (ic32)k);
#else
		k = getnearest(self, (ic32*)&k);
#endif
		*pws = ((k>>8)&0xf800)|((k>>5)&0x7e0)|((k>>3)&0x1f);
		pws++;
	    }
	  }
	  pws += eoff;
	  h--;
	}
}

static void ReMapColor24(TImage *self, dword *coltab) {
  int eoff, h, k;
  dword *pws;
//  bool ALPHA = (self->stat & pmxStat_has_alpha) != 0;

//	if (ALPHA) 
	alpha_data8_get(self);
	pws = (dword*)self->data;
	eoff = (self->bytes_per_line >> 2) - self->width;
	h = self->height;
	while (h) {
	  if (coltab) {
	    for (int i=self->width;i;i--) {
		k = ((*pws & 0xfc)>>2)|((*pws & 0xfc00)>>4)|((*pws & 0xfc0000)>>6);
		*pws = coltab[k];
		pws++;
	    }
	  } else {
	    for (int i=self->width;i;i--) {
#if USE_MMX_ReMap
		*pws = getnearest_mmx(self->cmap, (ic32)*pws);
#else
		*pws = getnearest(self, (ic32*)pws);
#endif
		pws++;
	    }
	  }
	  pws += eoff;
	  h--;
	}
//	if (ALPHA) {
	    alpha_data8_set(self);
	    ppfree(&self->rgb565_alpha_data);
//	}
}

static dword *setcoltab(TImage *self) {
  ic32 ic;
  dword *res, *coltab, k;

	if ((self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) {
	    if ((self->width * self->height) < 0x8000) return NULL;
	    res = coltab = xmalloc(sizeof(ic32) * 0x8000);
	    for (int i=0;i<0x8000;i++) {
		ic.dw = ((i & 0x1f)<<3)|((i & 0x3e0)<<6)|((i & 0x7c00)<<9);
#if USE_MMX_ReMap
		k = getnearest_mmx(self->cmap, ic);
#else
		k = getnearest(self, &ic);
#endif
		*coltab = ((k>>8)&0xf800)|((k>>5)&0x7e0)|((k>>3)&0x1f);
		coltab++;
	    }
	} else {
	    if ((self->width * self->height) < 0x40000) return NULL;
	    res = coltab = xmalloc(sizeof(ic32) * 0x40000);
	    for (int i=0;i<0x40000;i++) {
		ic.dw = ((i & 0x3f)<<2)|((i & 0xfc0)<<4)|((i & 0x3f000)<<6);
#if USE_MMX_ReMap
		*coltab = getnearest_mmx(self->cmap, ic);
#else
		*coltab = getnearest(self, &ic);
#endif
		coltab++;
	    }
	}
	return res;
}

bool TImage_ReMapColor(TImage *self) {
  dword *coltab;

	if (!self->cmap || (self->cmap->quant_count < 1)) return False;

#if USE_MMX_ReMap
	getnearest_init_mmx();
#endif
	coltab = setcoltab(self);

	if ((self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) {
	    ReMapColor16(self, coltab);
	    self->stat &= ~pmxStat_XPixAkt;
	} else if ((self->pxm_LD & pxmLD_bppMask) == pxm_4b_24bpp) {
	    ReMapColor24(self, coltab);
	    self->stat &= ~pmxStat_XPixAkt;
	}
#if USE_MMX_ReMap
	end_mmx();
#endif
	xfree(coltab);
	return True;
}
