/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tpixmap.h"
#include "tpixmap_pv.h"


/*____________________________________________________________________________*/
  /* Reset GC duplicate */
#if 0   /* subPixMap */
static void TPixMap_subResGC(TPixMap *pxm, TPixMap *subpxm) {
	if (pxm->gc == subpxm->gc) subpxm->gc = NULL;
	if (pxm->line_gc == subpxm->line_gc) subpxm->line_gc = NULL;
	if (pxm->mask_gc == subpxm->mask_gc) subpxm->mask_gc = NULL;
}

static void TPixMap_subDestroyGC(TPixMap *pxm, TPixMap *subpxm) {
  int i;
  TPixMap *tmppxm;
 /* TODO Del tmppxm->subPixMap GC's not testet */
	for (i=0;i<subpxm->subPixMapCount;i++) {
	    tmppxm = subpxm->subPixMap[i];
	    if (tmppxm) {
		if (tmppxm->subPixMapCount) TPixMap_subDestroyGC(pxm, tmppxm);
		TPixMap_subResGC(pxm, tmppxm);
	    }
	}
}

static void TPixMap_subDestroy(TPixMap *pxm) {
  int i;
  TPixMap *tmppxm;

	for (i=0;i<pxm->subPixMapCount;i++) {
	    tmppxm = pxm->subPixMap[i];
	    if (tmppxm) {
		pxm->subPixMap[i] = NULL;
 /* TODO Del tmppxm->subPixMap GC's not testet */
		if (tmppxm->subPixMapCount) TPixMap_subDestroyGC(pxm, tmppxm);
		TPixMap_subResGC(pxm, tmppxm);
//		TPixMap_Destroy(tmppxm);
		Tfree(tmppxm);
	    }
	}
	xfree(pxm->subPixMap);
	pxm->subPixMap = NULL;
}
#endif /* subPixMap */

static void TPixMap_FreeImage(TPixMap *pxm) {
	if (pxm->Ximg) {
#ifdef USE_XSHM
	    if (pxm->stat & pmxStat_SharedMemImg) {
		XShmDetach(pxm->dpy, &pxm->shminfo);
		XDestroyImage(pxm->Ximg);
		shmdt(pxm->shminfo.shmaddr);
		shmctl(pxm->shminfo.shmid, IPC_RMID, 0);
	    } else {
#endif
		xfree(pxm->Ximg->data);
		pxm->Ximg->data = NULL;
		XDestroyImage(pxm->Ximg);
#ifdef USE_XSHM
	    }
#endif
	}
	pxm->data = NULL;
	pxm->Ximg = NULL;
	pxm->bytes_per_line = 0;
}


/*____________________________________________________________________________*/
void TPixMap_SetXShm(TPixMap *self) {
#ifdef USE_XSHM
  int XShm_major, XShm_minor, m;
	if (XShmQueryVersion(self->dpy, &XShm_major, &XShm_minor, &m)) {
	    self->stat |= pmxStat_SharedMemImg;
	}
#endif
}

#ifdef USE_XSHM
static XImage *TPixMap_CreateXShmImg(TPixMap *self) {
  XImage *Ximg = NULL; //self->Ximg;
  int depth = 24;
	if ((self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) depth = 16;
	if ((self->stat & pmxStat_SharedMemImg) && (appXDat.RootW_depth == depth)) {
	    Ximg = XShmCreateImage(self->dpy, DefaultVisual(self->dpy, appXDat.default_screen),
			appXDat.RootW_depth, ZPixmap,
			NULL, &self->shminfo, self->width, self->height);
	}
	if (!Ximg) {
	    self->stat &= ~pmxStat_SharedMemImg;
	    return NULL;
	}
	self->shminfo.shmid = shmget(IPC_PRIVATE,
			Ximg->bytes_per_line * Ximg->height, IPC_CREAT|0777);
	self->shminfo.shmaddr = Ximg->data = shmat(self->shminfo.shmid, 0, 0);
	XShmAttach(self->dpy, &self->shminfo);
	XSync(self->dpy, 0);
	self->data = (byte*)Ximg->data;
	self->bytes_per_line = Ximg->bytes_per_line;
	self->memfu &= ~memfu_data_free;
	return Ximg;
}
#endif

/*____________________________________________________________________________*/
void TPixMap_CreateXimg(TPixMap *self) {
  XImage *Ximg;
  int bitmap_pad, depth, k;

	if (!self) return;
	TPixMap_FreeImage(self);
	if (self->height <= 0) return;
#ifdef USE_XSHM
	Ximg = TPixMap_CreateXShmImg(self);
	if (Ximg) goto imgOK;
#endif
	k = 2;
	bitmap_pad = 32;
	depth = 24;
	if ((self->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) {
	    k = 1;
	    bitmap_pad = 16;
	    depth = 16;
	} else if (appXDat.RootW_depth <= 16) {
	    depth = 32;
	} else if (depth < appXDat.RootW_depth) {
	    depth = appXDat.RootW_depth;
	}

	Ximg = XCreateImage(self->dpy, DefaultVisual(self->dpy, appXDat.default_screen),
			depth, ZPixmap, 0, NULL,
			self->width, self->height, bitmap_pad, 0);

	if (Ximg && Ximg->bytes_per_line) {
	    if (Ximg->bytes_per_line < (self->width << k)) {
 printf("%s(%d): err Ximg->bytes_per_line=%d soll=%d\n", FL, Ximg->bytes_per_line, self->width << k);
		Ximg->bytes_per_line = self->width << k;
	    }
	    Ximg->data = xcalloc(Ximg->bytes_per_line * self->height);
	    self->memfu |= memfu_data_free;
	}

#ifdef USE_XSHM
imgOK:
#endif
	if (Ximg) {
	    self->data = (byte*)Ximg->data;
	    self->bytes_per_line = Ximg->bytes_per_line;
	}
	self->Ximg = Ximg;
	self->stat &= ~pmxStat_XPixAkt;
	self->stat |= pmxStat_ImgAkt;
}

//bool TPixMap__Destroy(TPixMap *self);

int TPixMap_img_malloc(TPixMap *self) {
	if (!self->dpy) self->dpy = appXDat.dpy;
#ifdef USE_XSHM
	if (self->pxm_LD & pxmLD_USE_XSHM) TPixMap_SetXShm(self);
#endif
	TPixMap_CreateXimg(self);
	if (self->destroy == (TDESTROY)TImage__Destroy)
		self->destroy = (TDESTROY)TPixMap__Destroy;

	return (self->data != NULL);
}

/*____________________________________________________________________________*/
/* Load PixMap to XImage */
void TPixMap_XGetImage(TPixMap *self) {
  XImage *Ximg = self->Ximg;

	if (!self) return;
	if (!(self->stat & pmxStat_ImgAkt)) {
#ifdef USE_XSHM
	    if (!Ximg) Ximg = TPixMap_CreateXShmImg(self);

	    if (self->stat & pmxStat_SharedMemImg) {
		XShmGetImage(self->dpy, self->pmxID, Ximg, 0, 0, 0xffffffff);
	    } else {
#endif
		if (Ximg) {
		    MEM_CHK_CLR(Ximg->data);
		    XDestroyImage(Ximg);
		    self->data = NULL;
		}
		Ximg = XGetImage(self->dpy, (Drawable)self->pmxID, 0, 0, 
				    self->width, self->height, 0xffffffff, ZPixmap);
		if (Ximg) {
		    MEM_CHK_ADD(Ximg->data, Ximg->bytes_per_line*self->height);
		    self->data = (byte*)Ximg->data;
		    self->bytes_per_line = Ximg->bytes_per_line;
		    self->memfu |= memfu_data_free;
		}
		self->Ximg = Ximg;
#ifdef USE_XSHM
	    }
#endif
	    self->stat |= pmxStat_ImgAkt;
	}

	if (self->maskID && (self->stat & pmxStat_HandleMask) && 
			   !(self->stat & pmxStat_MaskImgAkt)) {
  int lg, y;
  byte *sb0, *db;
	    xfree(self->mask_data);
	    Ximg = XGetImage(self->dpy, (Drawable)self->maskID, 0, 0, 
			  self->width, self->height, 0xffffffff, ZPixmap);
	    if (Ximg) {
		lg = (self->width+7) >> 3;
		if (Ximg->bytes_per_line > lg) {
		    db = xmalloc(lg * self->height);
		    self->mask_data = db;
		    sb0 = (byte*)Ximg->data;
		    y = self->height;
		    while (y) {
			nmove((void*)&db, sb0, lg);
			sb0 += Ximg->bytes_per_line; y--;
		    }
		} else {
		    MEM_CHK_ADD(Ximg->data,Ximg->bytes_per_line*self->height);
		    self->mask_data = (byte*)Ximg->data;
		    Ximg->data = NULL;
		}
		XDestroyImage(Ximg);
		self->stat |= pmxStat_MaskImgAkt;
	    } else {
  printf("%s(%d): ERROR no mask image ???? \n", FL);
	    }
	}
//	XFlush(appXDat.dpy);
}

/*____________________________________________________________________________*/
void REGP1 pxm_create_maskID(TPixMap *pxm) {

	if (!pxm->mask_data || (pxm->stat & pmxStat_XMaskPixAkt)) return;
	pxm->stat |= pmxStat_XMaskPixAkt;
	if (pxm->maskID) XFreePixmap(pxm->dpy, pxm->maskID);
	pxm->maskID = XCreateBitmapFromData(pxm->dpy, 
		    appXDat.RootW, (char*)pxm->mask_data,
		    pxm->width, pxm->height);
}

static void pxm_chk_pmxID(TPixMap *self) {
	if (!self->pmxID) self->pmxID = XCreatePixmap(self->dpy, appXDat.RootW, 
				self->width, self->height, appXDat.RootW_depth);
#ifdef _OS_WIN32
	if (!self->gc) self->gc = CreateGC_RGB(appXDat.Default_Color);
#else
	if (!self->gc) self->gc = appXDat.Default_GC;
#endif
}

/* Save XImage to PixMap */
void TPixMap_XPutImage(TPixMap *pxm) {

	pxm_chk_pmxID(pxm);
	if (!(pxm->stat & pmxStat_XPixAkt) && pxm->Ximg) {
#ifdef USE_XSHM
	  if (pxm->stat & pmxStat_SharedMemImg) {
	    XShmPutImage(pxm->dpy, pxm->pmxID, pxm->gc, pxm->Ximg,
			0, 0, 0, 0, pxm->width, pxm->height, False);
	  } else {
	    XPutImage(pxm->dpy, pxm->pmxID, pxm->gc, pxm->Ximg,
			0, 0, 0, 0, pxm->width, pxm->height);
	  }
#else
	  XPutImage(pxm->dpy, (Drawable)pxm->pmxID, pxm->gc, pxm->Ximg,
		  0, 0, 0, 0, pxm->width, pxm->height);
#endif
	  pxm->stat |= pmxStat_XPixAkt;
	}

	if (pxm->stat & pmxStat_HandleMask) {
	    pxm_create_maskID(pxm);
	}

//	XSync(dpy, 0);
//	XFlush(dpy);
}

/*____________________________ Free __________________________________*/

bool TPixMap__Destroy(TPixMap *self) {
	TPixMap_FreeImage(self);
#if 0   /* subPixMap */
	if (self->subPixMapCount) TPixMap_subDestroy(self);
#endif
	FreeGC(self->gc);
	FreeGC(self->line_gc);
	FreeGC(self->mask_gc);
	if (self->pmxID) XFreePixmap(self->dpy, self->pmxID);
	if (self->maskID) XFreePixmap(self->dpy, self->maskID);
	xfree(self->vds);
	return TImage__Destroy((TImage*)self);
}

/*____________________________ Create ________________________________*/
/* depth = 16 or 24 */
TPixMap *TPixMap_Create_depth(int width, int height, dword depth) {
  TPixMap *pxm;

	pxm = ttc_create(sizeof(TPixMap), wtyp_TPixMap);
	pxm->destroy = (TDESTROY)TPixMap__Destroy;
	pxm->width = width;
	pxm->height = height;
	pxm->dpy = appXDat.dpy;

	if (!depth) depth = appXDat.RootW_depth;
	pxm->pxm_LD = pxm_4b_24bpp;
	if (depth == 16) pxm->pxm_LD = pxm_2b_16bpp;
	pxm->img_malloc = (Timg_malloc)TPixMap_img_malloc;
	return pxm;
}

TPixMap *TPixMap_Create_whLD(int width, int height, dword pxm_LD) {
  TPixMap *pxm;
	pxm = ttc_create(sizeof(TPixMap), wtyp_TPixMap);
	pxm->destroy = (TDESTROY)TPixMap__Destroy;
	pxm->width = width;
	pxm->height = height;
	pxm->dpy = appXDat.dpy;

	if (!(pxm_LD & pxmLD_bppMask)) {
	    if (appXDat.RootW_depth == 16) {
		pxm_LD |= pxm_2b_16bpp;
	    } else {
		pxm_LD |= pxm_4b_24bpp;
	    }
	}
	pxm->pxm_LD = pxm_LD;
	pxm->img_malloc = (Timg_malloc)TPixMap_img_malloc;
	return pxm;
}
