/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"
#include <zlib.h>
#include "zsrc.h"
#include "tpixmap.h"

TPixMap *TPixMap_zsrcLoadPNG(const ThZsrc *zsrc) {
  TPixMap *pxm;
  const int *cpint;
  FILE *f;
  TZsrcDecomp zsrc_dc;

	if (!ZsrcDecompress(zsrc, &zsrc_dc)) return NULL;

	pxm = NULL;
	cpint = (const int*)zsrc_dc.data;
	if ((zsrc_dc.size < 16) || 
	    (cpint[0] != 0x474e5089) || (cpint[1] != 0x0a1a0a0d))
	{
	    appError(EMSG_ERR, " no PNG header");

	} else {
#ifdef _OS_WIN32
	    f = tmpfile();
	    fwrite(zsrc_dc.data, 1, zsrc_dc.size, f); 
	    rewind(f);
#else
	    xchar fmem;
	    fmem.us_const = zsrc_dc.data;
	    f = fmemopen(fmem.s, zsrc_dc.size, "r");
#endif
	    if (f) {
		pxm = TPixMap_Create_whLD(0, 0, 0);
		pxm->in_fp = f;
		if (!read_png((TImage*)pxm)) {
		    ppTfree(&pxm);
		}
		fclose(f);
	    }
	}

	xfree(zsrc_dc.buf);
	return pxm;
}

TPic *TPic_zsrcLoadPNG(const ThZsrc *zsrc, bool MaskSet) {
  TPixMap *pxm;
  TPic *pic;

	pxm = TPixMap_zsrcLoadPNG(zsrc);
	if (!pxm) return NULL;

	TPixMap_XPutImage(pxm);
	pic = xcalloc(sizeof(TPic));
	pic->width = pxm->width;
	pic->height = pxm->height;
	pic->pic = pxm->pmxID;
	pxm->pmxID = 0;
	if (MaskSet) {
	    pic->mask = pxm->maskID;
	    pxm->maskID = 0;
	}
	Tfree(pxm);

	return pic;
}
