/*
 Copyright (C) 2010-2011 by Hoefer Alfred

 Licensed under the LGPL v2.1, see the file COPYING.LIB
*/

#include "appXbase.h"
#include "tpixmap.h"
#include "media_io.h"
#include "video_ioc.h"

typedef struct TVideo_IOC_private {
  AlgTTC1
  int	s_format;	//pxm_2b_16bpp pxm_4b_24bpp pxm_YUV422
  int	d_format;	//pxm_2b_16bpp pxm_4b_24bpp pxm_YUV422
  bool	color_ok, bnc2, bnc3, bnc4;

  void *l_src;
  void *l_dest;
  int w;
  void (*conv_line)(struct TVideo_IOC_private *pv);
  int ColorSat;		//0..256..512
  int minmax_mk;
  byte R_Tab[256];
  byte G_Tab[256];
  byte B_Tab[256];
  byte Rk[256];		//colorKorr 2b,3b,4b,5b,6b Tab
  byte Gk[256];		//colorKorr 2b,3b,4b,5b,6b Tab
  byte Bk[256];		//colorKorr 2b,3b,4b,5b,6b Tab
} TVideo_IOC_private;

static void set_reduce_color(int k, byte *pb, int mask, bool negative) {
  int i, m0, m1, m2, step, val, n;


	if (negative) pb += 255;

	if ((k < 1) || (k > 129)) {
	    if (negative) {
		for (i=0;i<256;i++) {*pb = i & mask; pb--;}
	    } else {
		for (i=0;i<256;i++) {*pb = i & mask; pb++;}
	    }
	    return;
	}
	if (k < 2) k = 2;

	if (k & 1) {
	    step = 256 / (k & 0xfe);
	    m1 = 256 / (k & 0xfe);
	    m0 = m2 = m1 / 2;
	} else {
	    step = 256 / (k - 1);
	    m1 = 256 / k;
	    m0 = m2 = 0;
	}
// printf("%s(%d): m0=%d m1=%d m2=%d step=0x%x\n", FL, m0, m1, m2, step);

	n = k;
	val = 0;
	if (negative) {
	    if (k & 1) {
		for (;m0;m0--) {*pb = val & mask; pb--;}
		n -= 2;
		val += step;
	    }
	    for (;n;n--) {
		if (val > 255) val = 255;
		for (i=m1;i;i--) {*pb = val & mask; pb--;}
		val += step;
	    }
	    if (k & 1) {
		if (val > 255) val = 255;
		for (;m2;m2--) {*pb = val & mask; pb--;}
	    }
	} else {
	    if (k & 1) {
		for (;m0;m0--) {*pb = val & mask; pb++;}
		n -= 2;
		val += step;
	    }
	    for (;n;n--) {
		if (val > 255) val = 255;
		for (i=m1;i;i--) {*pb = val & mask; pb++;}
		val += step;
	    }
	    if (k & 1) {
		if (val > 255) val = 255;
		for (;m2;m2--) {*pb = val & mask; pb++;}
	    }
	}
}

static void set_RGB_Korr(TVideo_IOC *ioc, TVideo_Color_Korr *tvckr) {
  TVideo_IOC_private *pv = ioc->private;
  int i, k, c;
  int Brightness;
  int Contrast;
  int Red;
  int Green;
  int Blue;
  byte *tR = pv->R_Tab;
  byte *tG = pv->G_Tab;
  byte *tB = pv->B_Tab;

	if (tvckr) {
	    Brightness = tvckr->Brightness;
	    Contrast = tvckr->Contrast + 256;
	    Red = tvckr->Red;
	    Green = tvckr->Green;
	    Blue = tvckr->Blue;
	} else {
	    Brightness = 0;
	    Contrast = 256;
	    Red = 0;
	    Green = 0;
	    Blue = 0;
	}

	for (i=0;i<256;i++) {
	    k = i + Brightness - ((i-128) - (((i-128)*Contrast) >> 8));

	    c = k + ((i * Red) >> 8);
	    if (c < 0) c = 0; if (c>255) c = 255;
	    *tR++ = c;

	    c = k + ((i * Green) >> 8);
	    if (c < 0) c = 0; if (c>255) c = 255;
	    *tG++ = c;

	    c = k + ((i * Blue) >> 8);
	    if (c < 0) c = 0; if (c>255) c = 255;
	    *tB++ = c;
	}
}

/*_________________ 2b_16bpp -> 2b_16bpp _____________________________ */
static void conv_line_2b16_2b16(TVideo_IOC_private *pv) {
  word *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, c, r, g, b;
	    for (i=pv->w;i>0;i--) {
		c = *sw++;
		r = pv->Rk[pv->R_Tab[(c >> 8) & 0xf8]];
		g = pv->Gk[pv->G_Tab[(c >> 3) & 0xfc]];
		b = pv->Bk[pv->B_Tab[(c << 3) & 0xf8]];
		*dw++ = (r<<8) | (g<<3) | (b>>3);
	    }
}

static void conv_line_2b16_2b16_gray(TVideo_IOC_private *pv) {
  word *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, c, y;
	    for (i=pv->w;i>0;i--) {
		c = *sw++;
		y = pv->Rk[pv->R_Tab[(c >> 8) & 0xf8]] * 77;
		y += pv->Gk[pv->G_Tab[(c >> 3) & 0xfc]] * 150;
		y += pv->Bk[pv->B_Tab[(c << 3) & 0xf8]] * 29;
		y >>= 8;
//		if (y > 255) y = 255;
		*dw++ = ((y&0xf8)<<8) | ((y&0xfc)<<3) | ((y&0xf8)>>3);
	    }
}

static void conv_line_2b16_2b16_gray_lin(TVideo_IOC_private *pv) {
  word *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, c, y;
	    for (i=pv->w;i>0;i--) {
		c = *sw++;
		y = pv->Rk[pv->R_Tab[(c >> 8) & 0xf8]];
		y += pv->Gk[pv->G_Tab[(c >> 3) & 0xfc]];
		y += pv->Bk[pv->B_Tab[(c << 3) & 0xf8]];
		y = (y * 0x55a) >> 12;  //div 3
		*dw++ = ((y&0xf8)<<8) | ((y&0xfc)<<3) | ((y&0xf8)>>3);
	    }
}

static void conv_line_2b16_2b16_sat(TVideo_IOC_private *pv) {
  word *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, c, r, g, b, y, u, v;
  int c1 = 145 * pv->ColorSat;
  int c2 = 183 * pv->ColorSat;
	for (i=pv->w;i>0;i--) {
	    c = *sw++;
	    r = pv->Rk[pv->R_Tab[(c >> 8) & 0xf8]];
	    g = pv->Gk[pv->G_Tab[(c >> 3) & 0xfc]];
	    b = pv->Bk[pv->B_Tab[(c << 3) & 0xf8]];
// RGB -> YUV
//    Y = 0.299*R + 0.587*G + 0.114*B
//    U = (B-Y)*0.565
//    V = (R-Y)*0.713
	    y = (r * 77) + (g * 150) + (b * 29);
	    c = y >> 8;
	    u = ((b - c) * c1) >> 16;
	    v = ((r - c) * c2) >> 16;
// YUV -> RGB
//    R = Y + 1.403*V
//    G = Y - 0.344*U - 0.714*V
//    B = Y + 1.770*U
	    c =  (Clip255[(y + (v * 359)) >> 8] & 0xf8) << 8;
	    c |= (Clip255[((y - (u * 88) - (v * 183)) >> 8)] & 0xfc) << 3;
	    *dw++ = c | ((Clip255[(y + (453 * u)) >> 8] & 0xf8) >> 3);
	}
}

extern char *TRxvAddTab0;
static void conv_line_2b16_2b16_sat_lin(TVideo_IOC_private *pv) {
  word *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, c, r, g, b, y;
  int c1 = pv->ColorSat;

	for (i=pv->w;i>0;i--) {
	    c = *sw++;
	    r = pv->Rk[pv->R_Tab[(c >> 8) & 0xf8]];
	    g = pv->Gk[pv->G_Tab[(c >> 3) & 0xfc]];
	    b = pv->Bk[pv->B_Tab[(c << 3) & 0xf8]];
	    y = ((r + g + b) * 0x55a) >> 12;  //div 3
	    r = Clip255[y + (((r - y) * c1) >> 8)] & 0xf8;
	    g = Clip255[y + (((g - y) * c1) >> 8)] & 0xfc;
	    b = Clip255[y + (((b - y) * c1) >> 8)] & 0xf8;
	    *dw++ = (r << 8) | (g << 3) | (b >> 3);
	}
}
/*_________________ 4b_24bpp -> 2b_16bpp _____________________________ */
static void conv_line_4b24_2b16(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, r, g, b;
	    for (i=pv->w;i>0;i--) {
		b = pv->Bk[pv->B_Tab[*sw++]];
		g = pv->Gk[pv->G_Tab[*sw++]];
		r = pv->Rk[pv->R_Tab[*sw]];
		sw += 2;
		*dw++ = (r<<8) | (g<<3) | (b>>3);
	    }
}

static void conv_line_4b24_2b16_gray(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, y;
	    for (i=pv->w;i>0;i--) {
		y = pv->Rk[pv->R_Tab[*sw++]] * 29;
		y += pv->Gk[pv->G_Tab[*sw++]] * 150;
		y += pv->Bk[pv->B_Tab[*sw]] * 77;
		sw += 2;
		y >>= 8;
		if (y > 255) y = 255;
		*dw++ = ((y&0xf8)<<8) | ((y&0xfc)<<3) | ((y&0xf8)>>3);
	    }
}

static void conv_line_4b24_2b16_gray_lin(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, y;
	    for (i=pv->w;i>0;i--) {
		y = pv->Rk[pv->R_Tab[*sw++]];
		y += pv->Gk[pv->G_Tab[*sw++]];
		y += pv->Bk[pv->B_Tab[*sw]];
		sw += 2;
		y = (y * 0x55a) >> 12;  //div 3
		*dw++ = ((y&0xf8)<<8) | ((y&0xfc)<<3) | ((y&0xf8)>>3);
	    }
}

static void conv_line_4b24_2b16_sat(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, c, r, g, b, y, u, v;
  int c1 = 145 * pv->ColorSat;
  int c2 = 183 * pv->ColorSat;
	for (i=pv->w;i>0;i--) {
	    b = pv->Bk[pv->B_Tab[*sw++]];
	    g = pv->Gk[pv->G_Tab[*sw++]];
	    r = pv->Rk[pv->R_Tab[*sw]];
	    sw += 2;
// RGB -> YUV
	    y = (r * 77) + (g * 150) + (b * 29);
	    u = ((b-(y>>8)) * c1) >> 16;
	    v = ((r-(y>>8)) * c2) >> 16;
// YUV -> RGB
	    c =  (Clip255[(y + (v * 359)) >> 8] & 0xf8) << 8;
	    c |= (Clip255[((y - (u * 88) - (v * 183)) >> 8)] & 0xfc) << 3;
	    *dw++ = c | ((Clip255[(y + (453 * u)) >> 8] & 0xf8) >> 3);
	}
}

static void conv_line_4b24_2b16_sat_lin(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  word *dw = pv->l_dest;
  int i, r, g, b, y;
  int c1 = pv->ColorSat;
	for (i=pv->w;i>0;i--) {
	    b = pv->Bk[pv->B_Tab[*sw++]];
	    g = pv->Gk[pv->G_Tab[*sw++]];
	    r = pv->Rk[pv->R_Tab[*sw]];
	    sw += 2;
	    y = ((r + g + b) * 0x55a) >> 12;  //div 3
	    r = Clip255[y + (((r - y) * c1) >> 8)] & 0xf8;
	    g = Clip255[y + (((g - y) * c1) >> 8)] & 0xfc;
	    b = Clip255[y + (((b - y) * c1) >> 8)] & 0xf8;
	    *dw++ = (r << 8) | (g << 3) | (b >> 3);
	}
}
/*_________________ 4b_24bpp -> 4b_24bpp _____________________________ */
static void conv_line_4b24_4b24(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  dword *dw = pv->l_dest;
  dword rgb;
	for (int i=pv->w;i>0;i--) {
	    rgb = pv->Bk[pv->B_Tab[*sw++]];
	    rgb |= pv->Gk[pv->G_Tab[*sw++]] << 8;
	    rgb |= pv->Rk[pv->R_Tab[*sw++]] << 16;
	    rgb |= *sw++ << 24;
	    *dw++ = rgb;
	}
}

static void conv_line_4b24_4b24_gray(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  dword *dw = pv->l_dest;
  dword a, y;
	    for (int i=pv->w;i>0;i--) {
		y = pv->Rk[pv->R_Tab[*sw++]] * 29;
		y += pv->Gk[pv->G_Tab[*sw++]] * 150;
		y += pv->Bk[pv->B_Tab[*sw++]] * 77;
		a = *sw++ << 24;
		y >>= 8;
		if (y > 255) y = 255;
		*dw++ = a | (y<<16) | (y<<8) | y;
	    }
}

static void conv_line_4b24_4b24_gray_lin(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  dword *dw = pv->l_dest;
  dword a, y;
	    for (int i=pv->w;i>0;i--) {
		y = pv->Rk[pv->R_Tab[*sw++]];
		y += pv->Gk[pv->G_Tab[*sw++]];
		y += pv->Bk[pv->B_Tab[*sw++]];
		a = *sw++ << 24;
		y = (y * 0x55a) >> 12;  //div 3
		*dw++ = a | (y<<16) | (y<<8) | y;
	    }
}

static void conv_line_4b24_4b24_sat(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  dword *dw = pv->l_dest;
  int i, a, r, g, b, y, u, v;
  int c1 = 145 * pv->ColorSat;
  int c2 = 183 * pv->ColorSat;
	    for (i=pv->w;i>0;i--) {
		b = pv->Bk[pv->B_Tab[*sw++]];
		g = pv->Gk[pv->G_Tab[*sw++]];
		r = pv->Rk[pv->R_Tab[*sw++]];
		a = *sw++ << 24;
// RGB -> YUV
		y = (r * 77) + (g * 150) + (b * 29);
		u = ((b-(y>>8)) * c1) >> 16;
		v = ((r-(y>>8)) * c2) >> 16;
// YUV -> RGB
		r = (y + (v * 359)) >> 8;
		a |= Clip255[r] << 16;
		g = ((y - (u * 88) - (v * 183)) >> 8);
		a |= Clip255[g] << 8;
		b = (y + (453 * u)) >> 8;
		*dw++ = a | Clip255[b];
	    }
}

static void conv_line_4b24_4b24_sat_lin(TVideo_IOC_private *pv) {
  byte *sw = pv->l_src;
  dword *dw = pv->l_dest;
  int i, a, r, g, b, y;
  int c1 = pv->ColorSat;
	    for (i=pv->w;i>0;i--) {
		b = pv->Bk[pv->B_Tab[*sw++]];
		g = pv->Gk[pv->G_Tab[*sw++]];
		r = pv->Rk[pv->R_Tab[*sw++]];
		a = *sw++ << 24;
		y = ((r + g + b) * 0x55a) >> 12;  //div 3
		r = y + (((r - y) * c1) >> 8);
		g = y + (((g - y) * c1) >> 8);
		b = y + (((b - y) * c1) >> 8);
		a |= Clip255[r] << 16;
		a |= Clip255[g] << 8;
		*dw++ = a | Clip255[b];
	    }
}


/*_____________________________ SetColor ________________________________*/
static bool TVideo_IOC_ColorRGB_SetColor(TVideo_IOC *ioc, TVideo_Color_Korr *tvckr) {
  TVideo_IOC_private *pv = ioc->private;
	pv->color_ok = False;
	return True;
}

static bool _IOC_SetColor(TVideo_IOC *ioc) {
  TVideo_IOC_private *pv = ioc->private;
  TVideo_Color_Korr *tvckr;
  int m1, m2, k;
  bool sbpp16, sbpp24, dbpp16, dbpp24;
  bool mk;

	tvckr = ioc->tvckr;

	k = pv->s_format; //ioc->svf->frame_typ;
	sbpp16 = (k == pxm_2b_16bpp);
	sbpp24 = (k == pxm_4b_24bpp)||(k == pxm_4b_24bpp8a);
	k = pv->d_format; //ioc->dvf->frame_typ;
	dbpp16 = (k == pxm_2b_16bpp);
	dbpp24 = (k == pxm_4b_24bpp)||(k == pxm_4b_24bpp8a);


	m1 = m2 = 0xff;
	if (dbpp16) {
	    m1 = 0xf8;
	    m2 = 0xfc;
	}
	mk = ((ioc->iocfu & VIOCFU_NEGATIV) != 0);
	if (tvckr) {
	    set_reduce_color(tvckr->R_reduce, pv->Rk, m1, mk);
	    set_reduce_color(tvckr->G_reduce, pv->Gk, m2, mk);
	    set_reduce_color(tvckr->B_reduce, pv->Bk, m1, mk);
	}
	set_RGB_Korr(ioc, tvckr);
#if 0
//	set_reduce_color(255, pv->Rk, m1, True);
 int l16 = 0;
 int m16 = 0;
 byte *pb = pv->G_Tab;

 for (int i=0;i<256;i++) {
    if (!(i & 0x0f)) {printf("%2d ", l16); l16++;}
    printf("0x%02x,", *pb);
    m16++; pb++;
    if ((i & 0x03) == 0x03) printf(" ");
    if ((i & 0x0f) == 0x0f) {printf("\n"); m16 = 0;}
 } 
#endif
// printf("%s(%d): %d\n", FL, ioc->ColorSat);

	if (tvckr) {
	    pv->ColorSat = tvckr->ColorSat + 256;
	    if (pv->ColorSat < 0) pv->ColorSat = 0;
	    if (pv->ColorSat > 512) pv->ColorSat = 512;
	} else {
	    pv->ColorSat = 256;
	}

	if (sbpp16 && dbpp16) {
	    if (pv->ColorSat == 256) {
		pv->conv_line = conv_line_2b16_2b16;
	    } else if (!pv->ColorSat) {
		if (ioc->iocfu & VIOCFU_GRAY_NATURE) {
		    pv->conv_line = conv_line_2b16_2b16_gray;
		} else {
		    pv->conv_line = conv_line_2b16_2b16_gray_lin;
		}
	    } else {
		if (ioc->iocfu & VIOCFU_GRAY_NATURE) {
		    pv->conv_line = conv_line_2b16_2b16_sat;
		} else {
		    pv->conv_line = conv_line_2b16_2b16_sat_lin;
		}
	    }

	} else if (sbpp24 && dbpp16) {
	    if (pv->ColorSat == 256) {
		pv->conv_line = conv_line_4b24_2b16;
	    } else if (!pv->ColorSat) {
		if (ioc->iocfu & VIOCFU_GRAY_NATURE) {
		    pv->conv_line = conv_line_4b24_2b16_gray;
		} else {
		    pv->conv_line = conv_line_4b24_2b16_gray_lin;
		}
	    } else {
		if (ioc->iocfu & VIOCFU_GRAY_NATURE) {
		    pv->conv_line = conv_line_4b24_2b16_sat;
		} else {
		    pv->conv_line = conv_line_4b24_2b16_sat_lin;
		}
	    }

	} else if (sbpp24 && dbpp24) {
	    if (pv->ColorSat == 256) {
		pv->conv_line = conv_line_4b24_4b24;
	    } else if (!pv->ColorSat) {
		if (ioc->iocfu & VIOCFU_GRAY_NATURE) {
		    pv->conv_line = conv_line_4b24_4b24_gray;
		} else {
		    pv->conv_line = conv_line_4b24_4b24_gray_lin;
		}
	    } else {
		if (ioc->iocfu & VIOCFU_GRAY_NATURE) {
		    pv->conv_line = conv_line_4b24_4b24_sat;
		} else {
		    pv->conv_line = conv_line_4b24_4b24_sat_lin;
		}
	    }

	} else {
	    pv->conv_line = NULL;
	}


	return (pv->conv_line != NULL);
}

/*_____________________________ Conv ____________________________________*/
static bool get_frame(TVideo_IOC *self, TFrame_IO *fio) {
  TVideo_IOC_private *pv = self->private;
  TImage *simg, *dimg;

	if (!fio) return False;
	if (fio->thdMSG) {
	    if (!self->next_link || !self->next_link->get_frame) return False;
	    return self->next_link->get_frame(self->next_link, fio);
	}

	if (!fio->org.v.iof || !fio->ret.v.iof) return False;
	if ((fio->org.v.iof->typ == DatType_TImage) &&
	    (fio->ret.v.iof->typ == DatType_TImage)) {
	    simg = fio->org.v.t.img;
	    dimg = fio->ret.v.t.img;
	    if (!simg || !dimg) return False;

	    if (!pv->color_ok || 
		(pv->s_format != (simg->pxm_LD & pxmLD_bppMask)) ||
		(pv->d_format != (dimg->pxm_LD & pxmLD_bppMask))) {

		pv->s_format = simg->pxm_LD & pxmLD_bppMask;
		pv->d_format = dimg->pxm_LD & pxmLD_bppMask;
		pv->color_ok = _IOC_SetColor(self);
	    }
	    if (!pv->color_ok) return False;

  byte *src, *dest;
	    pv->w = simg->width;
	    src = simg->data;
	    dest = dimg->data;
	    for (int y=simg->height;y>0;y--) {
		pv->l_src = src;
		pv->l_dest = dest;
		pv->conv_line(pv);
		src += simg->bytes_per_line;
		dest += dimg->bytes_per_line;
	    }
	    return True;

	}
	return False;
}
/*_____________________________ Init ____________________________________*/

static bool TVideo_IOC_ColorRGB_Init(TVideo_IOC *ioc) {
	TVideo_IOC_PV_Init(ioc, sizeof(TVideo_IOC_private), VIOCART_RGB_to_RGB);
	ioc->setcolor = TVideo_IOC_ColorRGB_SetColor;
//	ioc->conv = TVideo_IOC_ColorRGB_Conv;
	return True;
}

/*_____________________________ Free ____________________________________*/

/*_____________________________ Create __________________________________*/
TVideo_IOC *TVideo_IOC_ColorRGB_Create() {
  TVideo_IOC *self = TVideo_IOC_Create();

//	ioc->art = VIOCART_RGB_to_RGB;
	self->init = TVideo_IOC_ColorRGB_Init;
	self->get_frame = (TVTTC_GET_FRAME)get_frame;
	self->setcolor = TVideo_IOC_ColorRGB_SetColor;
//	ioc->conv = TVideo_IOC_ColorRGB_Conv;
	self->tvckr = xcalloc(sizeof(TVideo_Color_Korr));
//	ioc->private = xcalloc(sizeof(TVideo_IOC_private));
	TVideo_IOC_PV_Init(self, sizeof(TVideo_IOC_private), VIOCART_RGB_to_RGB);
	return self;
}
