/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"

#ifndef _OS_WIN32
#include <pthread.h>
#endif

extern void (*FappMutex_lock)();
extern void (*FappMutex_unlock)();

#ifdef _OS_WIN32
static CRITICAL_SECTION loc_mutex;
static bool isInit;

static void app_lock() {
	EnterCriticalSection(&loc_mutex);
}

static void app_unlock() {
	LeaveCriticalSection(&loc_mutex);
}

void appMutex_init() {
	if (isInit) return;
	isInit = True;
	InitializeCriticalSection(&loc_mutex);
	FappMutex_lock = app_lock;
	FappMutex_unlock = app_unlock;
}

#else
static pthread_mutex_t loc_mutex;
static bool isInit;

static void app_lock() {
	pthread_mutex_lock(&loc_mutex);
}

static void app_unlock() {
	pthread_mutex_unlock(&loc_mutex);
}

void appMutex_init() {
	if (isInit) return;
	isInit = True;
	pthread_mutex_init(&loc_mutex, NULL);
	FappMutex_lock = app_lock;
	FappMutex_unlock = app_unlock;
}
#endif

