/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include <errno.h>

static void TExecProc_DoEv(Texec_pipe *expipe, int evtyp) {
	if (expipe->OnExecProcEv) expipe->OnExecProcEv(expipe, evtyp);
}

static void SetCloseFlag(Texec_pipe *expipe, int nr) {
  Tprocpipe *pp = &expipe->pp[nr];
	ClosePipeProc(pp->fd, clpp_fd);
	pp->fd = -1;
	if (pp->lb && (pp->lb->lg > 0)) expipe->CloseFlag |= (1 << nr);
	TExecProc_DoEv(expipe, nr + evtyp_fd_Close_offset);
}

static void TExecProc_inProc(int fd, Tfd_list* fdl) {
  Texec_pipe *expipe = fdl->obj;
  int nr = fdl->m;

	if (TLineBuf_fd_Add(expipe->pp[nr].lb, fd)) {
	    TExecProc_DoEv(expipe, nr);
	} else {
	    SetCloseFlag(expipe, nr);
	}
}

static void inProc_close(Texec_pipe *expipe, int nr) {
	if (expipe->pp[nr].fd > 0) {
	    if (TLineBuf_fd_Add(expipe->pp[nr].lb, expipe->pp[nr].fd)) {
		TExecProc_DoEv(expipe, nr);
	    }
	    SetCloseFlag(expipe, nr);
	}
}

static void TExecProc_close(int pid, int stat_loc, TPid_Array *pida) {
//  int i;
  Texec_pipe *expipe;

	expipe = pida->obj;
	if (expipe->pp[0].fd > 0) ClosePipeProc(expipe->pp[0].fd, clpp_fdx);
	inProc_close(expipe, 1);
	inProc_close(expipe, 2);
	if (expipe->pp[3].fd > 0) ClosePipeProc(expipe->pp[3].fd, clpp_fdx);
	inProc_close(expipe, 4);

	TExecProc_DoEv(expipe, evtyp_ProcClose);
/*	for (i=0;i<5;i++) ppTfree(&expipe->pp[i].lb);
	xfree(expipe->args);
	xfree(expipe->cd_path);
	xfree(expipe->env);
	xfree(expipe->fname);
	xfree(expipe); */
	Tfree(expipe);
	pida->obj = NULL;
}

// fu = outfd_Open | infd_Open | errfd_Open
int ExecuteProcApp_argv_ex(const char *cmd, int argc, char **argv, int fu, 
			TOnExecProcEv OnExecProcEv, void *owner, 
			Texec_pipe **ret_expip, 
			const char *cd_path, const char *env, int tag)
{
  int pid, expipfu;
  Texec_pipe *expipe = NULL;

	if (!argc || !argv) return 0;
	expipfu = expipfu_args_free | expipfu_addPID_H;
	if (OnExecProcEv || cd_path || env) {
	    expipe = Texec_pipe_Create();
	    expipe->owner = owner;
	    expipe->cd_path = pcdup(cd_path);
	    expipe->env = pcdup(env);
	    expipe->tag = tag;
	    expipe->args = pcdup(cmd);
	    expipe->OnExecProcEv = OnExecProcEv;
	    expipe->On_pid_close = TExecProc_close;

	    if (fu & outfd_Open) {	//out_fd
		expipe->pp[0].Proc = (void*)1;
	    }

	    if (fu & infd_Open) {	//out_fd
		expipe->pp[1].lb = TLineBuf_Create();
		expipe->pp[1].Proc = TExecProc_inProc;
	    }

	    if (fu & errfd_Open) {	//err_fd
		expipe->pp[2].lb = TLineBuf_Create();
		expipe->pp[2].Proc = TExecProc_inProc;
	    }

	    if (fu & Doutfd_Open) {	//Dout_fd
		expipe->pp[3].Proc = (void*)1;
	    }

	    if (fu & Dinfd_Open) {	//Din_fd
		expipe->pp[4].lb = TLineBuf_Create();
		expipe->pp[4].Proc = TExecProc_inProc;
	    }
	}
	pid = exec_pipe_argv(argv, expipfu, expipe);
	if (pid > 0) {
	    if (expipe) TExecProc_DoEv(expipe, evtyp_ProcStart);
	}
	if (ret_expip) *ret_expip = expipe;

	if (!pid && expipe) {
	    TExecProc_DoEv(expipe, evtyp_ProcClose);
	}
	return pid;
}

int ExecuteProcApp_argv(const char *cmd, int argc, char **argv, int fu, 
			TOnExecProcEv OnExecProcEv, void *owner, 
			Texec_pipe **ret_expip, int tag) {
	return ExecuteProcApp_argv_ex(cmd, argc, argv, fu, OnExecProcEv, owner,
					ret_expip, NULL, NULL, tag);
}

// fu = outfd_Open | infd_Open | errfd_Open
int ExecuteProcAppTag(const char *cmd, int fu, TOnExecProcEv OnExecProcEv, 
		      void *owner, Texec_pipe **ret_expip, int tag) {
  int argc;
  char **argv = NULL;

	if (!cmd || !*cmd) return 0;
	cmdtoargs(cmd, &argc, &argv);
	return ExecuteProcApp_argv(cmd, argc, argv, fu, OnExecProcEv, owner, ret_expip, tag);
}

int ExecuteProcApp(const char *cmd, int fu, TOnExecProcEv OnExecProcEv, 
		   void *owner, Texec_pipe **ret_expip) {
	return ExecuteProcAppTag(cmd, fu, OnExecProcEv, owner, ret_expip, 0);
}

int ExecuteProc(const char *args) {
	return ExecuteProcApp(args, 0, NULL, NULL, NULL);
}
