/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"
#include <unistd.h>
#include <sys/stat.h>

bool TestIsFile(const char *fn) {
#ifdef _OS_WIN32
	if (!fn) return False;
	return file_chk_exist(fn, findREG);
#else
  struct stat64 buf;
	if (!fn || stat64(fn, &buf)) return False;
	if ((buf.st_mode & S_IFMT) != S_IFREG) return False;
	return True;
#endif
}

bool TestIsPath(const char *fn) {
#ifdef _OS_WIN32
	if (!fn) return False;
	return file_chk_exist(fn, findDIR);
#else
  struct stat64 buf;
	if (!fn || stat64(fn, &buf)) return False;
	if ((buf.st_mode & S_IFMT) != S_IFDIR) return False;
	return True;
#endif
}

char *GetFilePath(const char *fn) {
  const char *pc, *pce;

	if (!fn) return NULL;
	pc = fn;
	pce = NULL;
	while (*pc) {
	    if (TestSlash(*pc)) pce = pc;
	    pc++;
	}
	if (!pce) return NULL;
	return pcndup(fn, pce-fn+1);
}

const char *GetFileExt_dot_pc(const char *fn) {
  const char *pc, *pce;

	pc = GetFileName_pc(fn);
	if (!pc) return NULL;
	pce = NULL;
	while (*pc) {
	    if (*pc == '.') pce = pc;
	    pc++;
	}
	return pce;
}

const char *GetFileExt_pc(const char *fn) {
  const char *pc;

	pc = GetFileExt_dot_pc(fn);
	if (pc) {
	    pc++;
	    if (!*pc) pc = NULL;
	}
	return pc;
}

