/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "../net_xxx.h"

static void TNet_SYS_ERROR(TNet *self) {
	self->ende = True;
	self->error = True;
	self->run = False;

	self->fmsg.art = fmArt_AlgErr;
	self->fmsg.typD = fmTyp_char;
	self->fmsg.D.s = get_DisplayFileStr(NULL, &self->gp);
	ppcadd2(&self->fmsg.D.s, "\n      ", SysError_pc());
	TNet_msg((TNet*)self);
	xfree(self->fmsg.D.s);
}

static void nlb_close(TNetLineBuf *self, FileHandle fd) {
#ifdef _OS_WIN32
	file_close(fd, NULL);
#else
	if (self->ioflag & ioflag_NoPipe) {
	    file_close(fd, NULL);
	} else {
	    ClosePipeProc(fd, clpp_fd);
	}
#endif
	self->iofd = FileHandleNULL;
	self->run = False;
}

static void TNetLineBuf_Write(TNetLineBuf *self, FileHandle fd) {
  int blg;
  char *pc;

	TNetLineBuf_sync_wr(self);
	blg = TLineBuf_Get(self->lb, BUFSIZ, &pc);
	if (blg > 0) {
	    self->error = !file_write(fd, NULL, pc, blg, NULL);
	    self->ioinfo.DataNowSize += blg;
	    self->ioinfo.Time_us = GetCurTimeDiv_us64(self->ref_us);
	    blg = TLineBuf_Sub(self->lb, blg);
	}

	if ((self->lb->eof && !blg) || self->error) {
	    self->ioinfo.stat |= fmIOInfo_EOF;
	    self->ende = True;
	    nlb_close(self, fd);
	}
	TfreeLock(self);
	TNet_send_ioinfo((TNet*)self);
	TfreeUnLock(self);
}

static void TNetLineBuf_Read(TNetLineBuf *self, FileHandle fd) {
	if (self->OnDataAdd) self->OnDataAdd((TNetLineBuf*)self, False);
	self->ioinfo.DataNowSize += TLineBuf_fd_Add(self->lb, fd);
	self->ioinfo.Time_us = GetCurTimeDiv_us64(self->ref_us);
	if (self->OnDataAdd) self->OnDataAdd((TNetLineBuf*)self, True);
	TNetLineBuf_sync_rd(self);
	if (self->lb->eof) nlb_close(self, fd);
	TNet_send_ioinfo((TNet*)self);
}

#ifndef _OS_WIN32
static void TNetLineBuf_OnPipeWrite(FileHandle fd, Tfd_list *fdl) {
	TNetLineBuf_Write(fdl->obj, fd);
}

static void TNetLineBuf_OnPipeRead(FileHandle fd, Tfd_list *fdl) {
	TfreeLock(fdl->obj);
	TNetLineBuf_Read(fdl->obj, fd);
	TfreeUnLock(fdl->obj);
}
#endif

static bool TNetLineBuf_wait(TNetLineBuf *self) {
  bool mk;

	if (!self->omod) return False;

	if (self->ioflag & ioflag_NoPipe) {
	    if (self->omod == NetOpenMod_Write) {
		while (self->run) {
		    TNetLineBuf_Write(self, self->iofd);
		    AppProcessMessages();
		}
	    } else if (self->omod == NetOpenMod_Read) {
		while (self->run) {
		    TfreeLock(self);
		    TNetLineBuf_Read(self, self->iofd);
		    mk = self->ende && !self->error;
		    if (TfreeUnLock(self)) return mk;
		    AppProcessMessages();
		}
	    } else {
		self->ende = True;
		self->error = True;
	    }
	} else {
	    while (AppHandleMessage() && !self->ende);
	}
	return self->ende && !self->error;
}


static bool TNetLineBuf_open(TNetLineBuf *self, int omod) {

	if (!self->lb || self->run) return False;
#ifdef _OS_WIN32
	self->ioflag |= ioflag_NoPipe;
#else
// on "/proc/" or "/sys/" not always end-of-file event ????? 
	if ((pcncmp(self->gp.path, "/proc/", 6)) ||
	    (pcncmp(self->gp.path, "/sys/",5))) {
	    self->ioflag |= ioflag_NoPipe;
	} else {
	    self->ioflag &= ~ioflag_NoPipe;
	}
#endif

	TNet_Reset((TNet*)self);
	if (omod == NetOpenMod_Read) {
	    self->omod = omod;
	    TLineBuf_Clr(self->lb);
	    if (file_open_rd_gp(&self->gp, NULL, &self->iofd)) {
	        self->run = True;
#ifndef _OS_WIN32
		if (!(self->ioflag & ioflag_NoPipe)) 
		    AddPipeProc(self->iofd, InPipe, TNetLineBuf_OnPipeRead, self);
#endif
		return True;
	    }
	    TNet_SYS_ERROR((TNet*)self);

	} else if (omod == NetOpenMod_Write) {
	    self->omod = omod;
	    if (file_open_wr_gp(&self->gp, NULL, &self->iofd, 0)) {
		if (self->lb->wr_memlook) {
		    self->lb->lg = self->lb->datalg;
		    self->lb->pc_rdidx = NULL;
		}
	        self->run = True;
#ifndef _OS_WIN32
		if (!(self->ioflag & ioflag_NoPipe))
		    AddPipeProc(self->iofd, OutPipe, TNetLineBuf_OnPipeWrite, self);
#endif
		return True;
	    }
	    TNet_SYS_ERROR((TNet*)self);
	}
	self->omod = 0;
	return False;
}

static bool TNetFileLineBuf_set_gp(TNetLineBuf *self, TglobPath *gp) {
	if (self->run || !gp_test_local_drive(gp)) return False;
	gp_pc_free(&self->gp);
	gp_dup_gpdata(&self->gp, gp);
	return True;
}
/*_____________________________ Destroy _________________________________*/
static bool TNetFileLineBuf__Destroy(TNetLineBuf *self) {
	if (self->run) nlb_close(self, self->iofd);
	return TNet__Destroy((TNet*)self);
}


/*_____________________________ Create __________________________________*/
TNetLineBuf *TNetFileLineBuf_Create(void *owner, TglobPath *gp) {
  TNetLineBuf *self;

	self = TNet_Create(sizeof(TNetLineBuf), wtyp_TNetLineBuf, gp);
	self->destroy = (TDESTROY)TNetFileLineBuf__Destroy;
	self->owner = owner;
	self->set_gp = (void*)TNetFileLineBuf_set_gp;
	self->open = TNetLineBuf_open;
	self->wait = TNetLineBuf_wait;

	return self;
}
