/*
 Copyright (C) 2010-2011 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"

#include <pwd.h>
#include <grp.h>

static TUserList  *UserList = NULL;
static TGroupList *GroupList = NULL;

/* __________________________ UserList ___________________________________*/
static void ul_free() {
	while (UserList) {
	    TUserList *ul = UserList;
	    UserList = ul->next;
	    xfree(ul);
	}
}

static TUserList **ul_add(TUserList **ppul, int uid, char *name) {

	while (*ppul) ppul = &(*ppul)->next;
	int i = pclen(name);
	if (i < fs_UID_GID_MaxMem) {
	    TUserList *ul = xcalloc(sizeof(TUserList));
	    pcnmove(ul->name, name, i);
	    ul->uid = uid;
	    *ppul = ul;
	}
	return ppul;
}

static void ul_init() {
  struct passwd *pwd;
  TUserList **ppul;
	ppul = &UserList;
	setpwent();
loop_pwd:
	pwd = getpwent();
	if (pwd) {
	    ppul = ul_add(ppul, pwd->pw_uid, pwd->pw_name);
	    goto loop_pwd;
	}
	endpwent();
}

static TUserList *ul_new_add(int uid) {
  char *pc;
  struct passwd *pwd;
  struct passwd tpwd;
  char tmpf[1024];	//NSS_BUFLEN_PASSWD

	pc = tmpf;
	if (!getpwuid_r(uid, &tpwd, tmpf, sizeof(tmpf), &pwd) && pwd) {
	    pc = pwd->pw_name;
	} else inttopc(pc, uid);
	return *ul_add(&UserList, uid, pc);
}

TUserList *ul_getby_uid(int uid) {
	if (!UserList) ul_init();

  TUserList *ul = UserList;
	while (ul && (ul->uid != uid)) ul = ul->next;
	if (!ul) ul = ul_new_add(uid);
	return ul;
}

TUserList *ul_getby_name(const char *name) {
	if (!UserList) ul_init();

  TUserList *ul = UserList;
	while (ul && !pccmp(ul->name, name)) ul = ul->next;
	return ul;
}

TUserList *ul_get_next(TUserList *ul) {
	if (!UserList) ul_init();
	if (!ul) return UserList;
	return ul->next;
}


/* _________________________ GroupList ___________________________________*/
static void gl_free() {
	while (GroupList) {
	    TGroupList *gl = GroupList;
	    GroupList = gl->next;
	    xfree(gl);
	}
}

static TGroupList **gl_add(TGroupList **ppgl, int gid, char *name) {

	while (*ppgl) ppgl = &(*ppgl)->next;
	int i = pclen(name);
	if (i < fs_UID_GID_MaxMem) {
	    TGroupList *gl = xcalloc(sizeof(TGroupList));
	    pcnmove(gl->name, name, i);
	    gl->gid = gid;
	    *ppgl = gl;
	}
	return ppgl;
}

static void gl_init() {
  struct group *grp;
  TGroupList **ppgl;
	ppgl = &GroupList;
	setgrent();
loop_grp:
	grp = getgrent();
	if (grp) {
	    ppgl = gl_add(ppgl, grp->gr_gid, grp->gr_name);
	    goto loop_grp;
	}
	endgrent();
}

static TGroupList *gl_new_add(int gid) {
  char *pc;
  struct group *grp;
  struct group tgrp;
  char tmpf[1024];

	pc = tmpf;
	if (!getgrgid_r(gid, &tgrp, tmpf, sizeof(tmpf), &grp) && grp) {
	    pc = grp->gr_name;
	} else inttopc(pc, gid);
	return *gl_add(&GroupList, gid, pc);
}

TGroupList *gl_getby_gid(int gid) {
	if (!GroupList) gl_init();

  TGroupList *gl = GroupList;
	while (gl && (gl->gid != gid)) gl = gl->next;
	if (!gl) gl = gl_new_add(gid);
	return gl;
}

TGroupList *gl_getby_name(const char *name) {
	if (!GroupList) gl_init();

  TGroupList *gl = GroupList;
	while (gl && !pccmp(gl->name, name)) gl = gl->next;
	return gl;
}

TGroupList *gl_get_next(TGroupList *gl) {
	if (!GroupList) gl_init();
	if (!gl) return GroupList;
	return gl->next;
}


/* __________________________ ul gl _____________________________________*/
void ul_gl_free() {
	ul_free();
	gl_free();
}

void ul_gl_init() {
	ul_gl_free();
	ul_init();
	gl_init();
}
