/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"
#include <unistd.h>
#include <sys/stat.h>

#ifndef S_IRGRP
//#define	S_IRGRP	(S_IRUSR >> 3)	/* Read by group.  */
#define	S_IWGRP	(S_IWUSR >> 3)	/* Write by group.  */
//#define	S_IXGRP	(S_IXUSR >> 3)	/* Execute by group.  */
#endif

#ifndef S_IROTH
//#define	S_IROTH	(S_IRGRP >> 3)	/* Read by others.  */
#define	S_IWOTH	(S_IWGRP >> 3)	/* Write by others.  */
//#define	S_IXOTH	(S_IXGRP >> 3)	/* Execute by others.  */
#endif

TMime *app_mime = NULL;
ThGetFileHeadInfo *hGetFileHeadInfo = NULL;

#ifndef _OS_WIN32
#ifdef USE_ACL

#include <sys/acl.h>
#include <acl/libacl.h>

static void file_stat_acl(char *f, Tstat64 *st) {
  acl_t acl;

	acl = acl_get_file(f, ACL_TYPE_ACCESS);
	if (!acl) return;
	st->st |= st_has_ACL;
	if (acl_entries(acl) > 3) {
	    st->st |= st_has_ACL_attr;
	} else if (st->st & st_Dir) {
	    acl_free(acl);
	    acl = acl_get_file(f, ACL_TYPE_DEFAULT);
	    if (acl_entries(acl) > 0) st->st |= st_has_ACL_attr;
	}
	acl_free(acl);
}
#endif
#endif /* ! _OS_WIN32 */


static TExFileDat *SetUp_ExFileDat(TExFileDat *ExDat, const char *s, int lg, int exfdt) {
	ExDat->type = exfdt;
	ExDat->lg = lg;
	if (s) pcxmove(ExDat->dat, s, lg-1);
	return (TExFileDat*)(ExDat->dat + lg);
}


Tstat64 *file_stat_A(const char *pathfn, const char *fname, int fu) {
  int sst, mlg, flg, llg, Link_size, m;
  Tstat64 *result;
  TExFileDat *ExDat;
#ifdef _OS_WIN32
  charOS *fnOS;
  struct _stati64 buf;
  dword dwFileAttributes;
  char tmpf[4096];
#else
#define fnOS pathfn
  struct stat64 buf;
  char *lnk;
#endif

	if (!pathfn || !fname) return NULL;
	sst = 0;
	if ((fname[0] == '.') && fname[1] &&
	    (fname[1] != '.')) sst = st_Hidden;

#ifdef _OS_WIN32
	fnOS = path_to_local_OS(pathfn);
	int wlg = pwclen(fnOS);
	if ((wlg > 1) && (fnOS[wlg-1] == '\\') && 
		         (fnOS[wlg-2] != ':')) fnOS[wlg-1] = 0;

	dwFileAttributes = GetFileAttributesW(fnOS);
	if (dwFileAttributes == 0xFFFFFFFF) {
	    xfree(fnOS);
	    return NULL;
	}

	if (_wstati64(fnOS, &buf)) {
	    xfree(fnOS);
	    return NULL;
	}

	if (dwFileAttributes & FILE_ATTRIBUTE_READONLY) {
	    buf.st_mode &= ~(S_IWUSR|S_IWGRP|S_IWOTH);
	}
	if (dwFileAttributes & FILE_ATTRIBUTE_HIDDEN) {
	    if (!((fname[0] == '.') && 
		(!fname[1] ||
		((fname[1] == '.') && !fname[2])))) {
		sst = st_Hidden;
	    }
	}

#else /* _OS_WIN32 */
	if (fu & fsLinkDirekt) {
	    if (stat64(fnOS, &buf)) return NULL;
	} else {
	    if (lstat64(fnOS, &buf)) return NULL;
	}
#endif /* ! _OS_WIN32 */

	mlg = sizeof(Tstat64);
	if (fu & fsAdd_UID_GID_mem) {
	    mlg += (sizeof(TExFileDat) + fs_UID_GID_MaxMem) * 2;
	}
	if (fu & fsAdd_Type_mem) {
	    mlg += sizeof(TExFileDat) + fs_Type_MaxMem;
	}

	flg = pclen(fname) + 1;
	if (!(fu & fsNoFileName)) mlg += sizeof(TExFileDat) + flg;

	Link_size = llg = 0;
#ifndef _OS_WIN32
	lnk = NULL;
	if ((buf.st_mode & S_IFMT) == S_IFLNK) {
	    sst |= st_Link;
	    Link_size = buf.st_size;
	    lnk = xmalloc(4080);
	    llg = readlink(fnOS, lnk, 4080);
	    lnk[llg] = 0;
	    llg++;
	    if (fu & fsFollowLink) stat64(fnOS, &buf);
	    if (!(fu & fsNoLinkName)) mlg += sizeof(TExFileDat) + llg;
	}
#endif

	result = xcalloc(mlg);
	result->StructSize = mlg;
	result->dev = buf.st_dev;
	result->mode = buf.st_mode;
	result->nlink = buf.st_nlink;
	result->uid = buf.st_uid;
	result->gid = buf.st_gid;
	result->rdev = buf.st_rdev;
	result->size = buf.st_size;
	result->atime = buf.st_atime;
	result->mtime = buf.st_mtime;
	result->ctime = buf.st_ctime;
#ifdef _OS_WIN32
	result->dwFileAttributes = dwFileAttributes;
#endif
	result->LinkSize = Link_size;

//	if (!(buf.st_mode & S_IWUSR)) sst |= st_ReadOnly;
//	if (!file_chk_access_st(result, W_OK)) sst |= st_ReadOnly;

	m = buf.st_mode & S_IFMT;
	if (m == S_IFDIR) {
	    result->FileType = stft_Folder; sst |= st_Dir;
	} else if (m == S_IFREG) {
	    sst |= st_Reg;
	} else if (m == S_IFCHR) {
	    result->FileType = stft_Chardev;
	} else if (m == S_IFBLK) {
	    result->FileType = stft_Blockdev;
	} else if (m == S_IFIFO) {
	    result->FileType = stft_FIFO;
#ifndef _OS_WIN32
	} else if (m == S_IFSOCK) {
	    result->FileType = stft_Socket;
#endif
	}

	result->st = sst;

	if (!result->FileType && (fu & fsGetFileType)) {
	    dword MinSize;
	    result->FileType = get_file_stft_ext(fname, &result->sub_mime, &MinSize);
	    if (MinSize > result->size) result->FileType = stft_None;
	}

	if ((fu & fsGetFileTypeHead) && (sst & st_Reg) &&
#ifdef _OS_WIN32
	    !(dwFileAttributes & FILE_ATTRIBUTE_HIDDEN) &&
#endif
	    (!result->FileType || (result->FileType == stft_NoFound)) && 
	    hGetFileHeadInfo) {
	    hGetFileHeadInfo->get_head_mime_gp_st_fn(result, pathfn, fname);
	}

#ifndef _OS_WIN32
#ifdef USE_ACL
	file_stat_acl(f, statx);
#endif
#endif /* ! _OS_WIN32 */
	ExDat = &result->ExDat;
	if (!(fu & fsNoFileName)) {
	    ExDat = SetUp_ExFileDat(ExDat, fname, flg, exfdt_FileName);
	}
#ifndef _OS_WIN32
	if ((sst & st_Link) && !(fu & fsNoLinkName)) {
	    ExDat = SetUp_ExFileDat(ExDat, lnk, llg, exfdt_LinkName);
	}
	xfree(lnk);
#endif /* ! _OS_WIN32 */

	if (fu & fsAdd_UID_GID_mem) {
#ifdef _OS_WIN32
  TUserGroupList *ul = NULL;
  TUserGroupList *gl = NULL;
  PSECURITY_DESCRIPTOR pSD = (PSECURITY_DESCRIPTOR)&tmpf;
  DWORD lpnLengthNeeded = 0;

	    if (GetFileSecurityW(fnOS, OWNER_SECURITY_INFORMATION | GROUP_SECURITY_INFORMATION,
				pSD, sizeof(tmpf), &lpnLengthNeeded)) {
		PSID pid;
		BOOL od;
		if (GetSecurityDescriptorOwner(pSD, &pid, &od)) {
		    ul = ul_getby_uid(pid);
		}
		if (GetSecurityDescriptorGroup(pSD, &pid, &od)) {
		    gl = gl_getby_gid(pid);
		}
	    }
	    ExDat->type = exfdt_owner;
	    ExDat->lg = fs_UID_GID_MaxMem;
	    if (ul) pcxmove(ExDat->dat, ul->Name, fs_UID_GID_MaxMem);
	    ExDat = (TExFileDat*)&ExDat->dat[fs_UID_GID_MaxMem];
#else /* _OS_WIN32 */
	    TUserList *ul = ul_getby_uid(result->uid);
	    ExDat = SetUp_ExFileDat(ExDat, ul->name, fs_UID_GID_MaxMem, exfdt_owner);
#endif /* ! _OS_WIN32 */


#ifdef _OS_WIN32
	    ExDat->type = exfdt_group;
	    ExDat->lg = fs_UID_GID_MaxMem;
	    if (gl) pcxmove(ExDat->dat, gl->Name, fs_UID_GID_MaxMem);
	    ExDat = (TExFileDat*)&ExDat->dat[fs_UID_GID_MaxMem];
#else
	    TGroupList *gl = gl_getby_gid(result->gid);
	    ExDat = SetUp_ExFileDat(ExDat, gl->name, fs_UID_GID_MaxMem, exfdt_group);
#endif /* ! _OS_WIN32 */
	}

	if (fu & fsAdd_Type_mem) {
	    ExDat->type = exfdt_type;
	    ExDat->lg = fs_Type_MaxMem;
	    if ((result->FileType == stft_Chardev) || (result->FileType == stft_Blockdev)) {
		const char *pcn = "Char";
		if (result->FileType == stft_Blockdev) pcn = "Blk";
		sprintf(ExDat->dat, "%s %3d:%3d", pcn, (int)((result->rdev >> 8) & 0xff), (int)(result->rdev & 0xff));
	    }
	    ExDat = (TExFileDat*)(ExDat->dat + fs_Type_MaxMem);
	}
	ExDat->fu = exfdf_lastdat;

	if ((fu & fsGetFileMime) && 
#ifdef _OS_WIN32
	    !(dwFileAttributes & FILE_ATTRIBUTE_HIDDEN) &&
#endif
	    app_mime && app_mime->stat_set_mime_id) {
// printf("%s(%d): 0x%x 0x%x %s\n", FL, fu, statx, fname);
	    app_mime->stat_set_mime_id(app_mime, result, fname);
	}

#ifdef _OS_WIN32
	xfree(fnOS);
#endif
	return result;
}

Tstat64 *file_stat_gp(TglobPath *gp, int fu) {
  char *pathfn;
  const char *fname;
  Tstat64 *result;

	pathfn = gptopc(gp, gptfu_del_slash|gptfu_noRootArt);
	fname = gp->name;
	if (!fname) fname = ".";
	result = file_stat_A(pathfn, fname, fu);
	xfree(pathfn);
	return result;
}

Tstat64 *file_stat_gp_net(TglobPath *gp, TNetErrRet *ErrRet, int fsfu) {
	return file_stat_gp(gp, fsfu);
}

Tstat64 *file_stat_fn(const char *FileName, int fu) {
  const char *fname;

	FileName = path_korr_file(FileName);
	fname = GetFileName_pc(FileName);
	if (!fname) return NULL;
	if (!*fname) fname = ".";   // /tmp/ --> /tmp/.
	return file_stat_A(FileName, fname, fu);
}
