/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "net_io.h"
#include "filemsg.h"
#include "tdirlist.h"

static const char *hSHb_Txt[11] = {
  "hSHb_QUIT",			// 0
  "hSHb_appSTART",		// 1
  "hSHb_appQUIT",		// 2
  "hSHb_Ok",			// 3
  "hSHb_ERR",			// 4
  "hSHb_LS",			// 5
  "hSHb_LS_Refresh",		// 6
  "hSHb_LS_Del_uID_uTAG",	// 7
  "hSHb_SET_Login",		// 8  /* ServToClient = Error */
  "hSHb_FileStat",		// 9
  "hSHb_StrgMSG"		// 10
};

static const char *get_hSHb(int bef) {
	if ((bef < 0) || (bef > 10)) return "";
	return hSHb_Txt[bef];
}





void file_msg_debug(TFileMsg *fmsg, const char *name) {
  const char *art = "  ?   ";

	if (!name) name = "any_fmsg  ";
	switch (fmsg->art) {
	case fmArt_AlgErr:	art = "AlgErr "; break;
	case fmArt_errLogin:	art = "errLog "; break;
	case fmArt_errConnect:	art = "errCon "; break;
	case fmArt_NewLogin:	art = "NewLog "; break;
	case fmArt_strg:	art = "strg   "; break;
	case fmArt_TdServ:	art = "TdServ "; break;
	case fmArt_ftpappclient:art = "ftpappc"; break;
	case fmArt_DirList:	art = "DirList"; break;
	case fmArt_DataIO:	art = "DataIO "; break;
	}
  printf("%s (%d)%s wtyp=0x%06x Ok=%d\n", name, fmsg->art, art, fmsg->wtyp, fmsg->Ok);
//	fmsg->Ok = True;

	if (fmsg->typD == fmTyp_int) {
  printf(" D  int         %d (0x%04x)\n", fmsg->D.m, fmsg->D.m);
//	    if ((fmsg->art == fmArt_DataIO) && 
//	        (fmsg->typF == fmTyp_lb)) fmsg->Ok = False;

	} else if (fmsg->typD == fmTyp_char) {
  printf(" D  char        %s\n", fmsg->D.s);
	} else if (fmsg->typD == fmTyp_stat) {
  printf(" D  st          size=%u\n", (dword)fmsg->D.st->size);
	} else if (fmsg->typD == fmTyp_ioinfo) {
  printf(" D  info        soll=%u size=%u us=%u stat=0x%04x\n", 
		(dword)fmsg->D.ioinfo->DataSize, (dword)fmsg->D.ioinfo->DataNowSize, (dword)fmsg->D.ioinfo->Time_us, fmsg->D.ioinfo->stat);
	} else if (fmsg->typD != fmTyp_none) {
  printf(" D  %d\n", fmsg->typD);
	}


	if (fmsg->typF == fmTyp_stat) {
#ifdef _OS_WIN32
  printf(" F  st          size=%u\n", (dword)fmsg->F.st->size);
#else
#if defined(__x86_64__) && !defined(__ILP32__)
  printf(" F  st          size=%ld\n", fmsg->F.st->size);
#else
  printf(" F  st          size=%lld\n", fmsg->F.st->size);
#endif
#endif
	} else if (fmsg->typF == fmTyp_lb) {
  printf(" F  lb          mem=%ld lg=%ld\n", fmsg->F.lb->memlg, fmsg->F.lb->lg);
	} else if (fmsg->typF == fmTyp_sh) {
  printf(" F  sh          bef=%d -> %s\n", fmsg->F.sh->bef, get_hSHb(fmsg->F.sh->bef));
	} else if (fmsg->typF == fmTyp_shd) {
	    if (fmsg->F.shd->fu & hSHfu_EOF) {
#ifdef _OS_WIN32
  printf(" F  shd us=%6u   size=%u size=%u  EOF\n", (dword)fmsg->F.shd->sh_LoadTime_us, (dword)fmsg->F.shd->sh_DataSize, (dword)fmsg->F.shd->sh_DataNowSize);
#else
#if defined(__x86_64__) && !defined(__ILP32__)
  printf(" F  shd us=%6lu   size=%lu size=%lu  EOF\n", fmsg->F.shd->sh_LoadTime_us, fmsg->F.shd->sh_DataSize, fmsg->F.shd->sh_DataNowSize);
#else
  printf(" F  shd us=%6llu   size=%llu size=%llu  EOF\n", fmsg->F.shd->sh_LoadTime_us, fmsg->F.shd->sh_DataSize, fmsg->F.shd->sh_DataNowSize);
#endif
#endif
	    } else { 
#ifdef _OS_WIN32
  printf(" F  shd us=%6u   size=%u size=%u\n", (dword)fmsg->F.shd->sh_LoadTime_us, (dword)fmsg->F.shd->sh_DataSize, (dword)fmsg->F.shd->sh_DataNowSize);
#else
#if defined(__x86_64__) && !defined(__ILP32__)
  printf(" F  shd us=%6lu   size=%lu size=%lu\n", fmsg->F.shd->sh_LoadTime_us, fmsg->F.shd->sh_DataSize, fmsg->F.shd->sh_DataNowSize);
#else
  printf(" F  shd us=%6llu   size=%llu size=%llu\n", fmsg->F.shd->sh_LoadTime_us, fmsg->F.shd->sh_DataSize, fmsg->F.shd->sh_DataNowSize);
#endif
#endif
	    }
	} else if (fmsg->typF == fmTyp_dfl) {
  printf(" F  dfl         %s\n", fmsg->F.dfl->fn);

	} else if (fmsg->typF != fmTyp_none) {
  printf(" F  %d\n", fmsg->typF);
	}

	if (fmsg->typR == fmTyp_gp) {
	    printf(" R  gp          ");
	    if (fmsg->R.gp->RootArt) printf("%s//%s", fmsg->R.gp->RootArt, fmsg->R.gp->URL);
	    printf("%s", fmsg->R.gp->path);
	    if (fmsg->R.gp->name) printf("%s", fmsg->R.gp->name);
	    printf("\n");

	} else if (fmsg->typR != fmTyp_none) {
  printf(" R  %d\n", fmsg->typR);
	}

//	fmsg->Ok = (fmsg->art != fmArt_AlgErr) &&
//		   (fmsg->art != fmArt_errLogin);
 fflush(stdout);
}
