/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "ftp_net.h"
#include "ftp_cache.h"

typedef struct TFtpCacheEvData {
  struct TFtpCacheEvData *next;
  void *obj;
  TNetCacheEv CacheEv;
} TFtpCacheEvData;


typedef struct TFtpCache {
  struct TFtpCache *next;
  char *URL;
  char *user;
  char *path;
  TNetDataCache *dc;
  TFtpCacheEvData *evd;
  dword ms;
} TFtpCache;

static TFtpCache *FtpCache;

static TFtpCache *get_fc(TglobPath *gp) {
  TFtpCache *fc;
	fc = FtpCache;
	while (fc) {
	    if (pccmp(fc->URL,  gp->URL) &&
		pccmp(fc->user, gp->user) &&
		pccmp(fc->path, gp->path)) break;
	    fc = fc->next;
	}
	return fc;
}

static TFtpCache **get_ppfc(TglobPath *gp) {
  TFtpCache **ppfc;
	ppfc = &FtpCache;
	while (*ppfc) {
	    if (pccmp((*ppfc)->URL,  gp->URL) &&
		pccmp((*ppfc)->user, gp->user) &&
		pccmp((*ppfc)->path, gp->path)) break;
	    ppfc = &(*ppfc)->next;
	}
	return ppfc;
}

static TFtpCache *get_fc_Add(TglobPath *gp) {
  TFtpCache *fc, **ppfc;

	fc = get_fc(gp);
	if (!fc) {
	    ppfc = &FtpCache;
	    while (*ppfc) ppfc = &(*ppfc)->next;
	    fc = xcalloc(sizeof(TFtpCache));
	    *ppfc = fc;
	    fc->URL = pcdup(gp->URL);
	    fc->user = pcdup(gp->user);
	    fc->path = pcdup(gp->path);
	    fc->ms = GetTime_ms();
	}
	return fc;
}

static TNetDataCache **get_ppdc_name(TNetDataCache **ppdc, const char *name) {
	while (*ppdc) {
	    if (pccmp((*ppdc)->name,  name)) break;
	    ppdc = &(*ppdc)->next;
	}
	return ppdc;
}

static TNetDataCache **get_ppdc_st(TNetDataCache **ppdc, Tstat64 *st) {
	return get_ppdc_name(ppdc, Get_ExFileDat_pc(st, exfdt_FileName));
}

static TNetDataCache *get_dc_name(TNetDataCache *dc, const char *name) {
	while (dc) {
	    if (pccmp(dc->name,  name)) break;
	    dc = dc->next;
	}
	return dc;
}

static bool get_fc_dc(TglobPath *gp, TFtpCache **ppfc, TNetDataCache **ppdc) {
  TFtpCache *fc;
  TNetDataCache *dc;

	if (!gp || !gp->name) return False;
	fc = get_fc(gp);
	if (!fc) return False;
	if (ppfc) {
	    *ppfc = fc;
	    if (!ppdc) return True;
	}
	dc = get_dc_name(fc->dc, gp->name);
	if (!dc) return False;
	if (ppdc) *ppdc = dc;
	return True;
}

static void send_CacheEv(TFtpCacheEvData *evd, TNetDataCache *dc, int fu) {
	while (evd) {
	    if (evd->CacheEv) evd->CacheEv(evd->obj, dc, fu);
	    evd = evd->next;
	}
}

static void Cache_SetDirCount(TglobPath *gp, TFtpCache *fc, bool sendEv) {
  TglobPath gptmp;
  TNetDataCache *dc, *dctmp;
  TFtpCache *fctmp;
  int count = 0;

	if ((gp->path[0] == '/') && !gp->path[1]) return;
	gp_dup_gpdata_path(&gptmp, gp);
	gp_PathToName(&gptmp);
	if (!get_fc_dc(&gptmp, &fctmp, &dctmp)) goto end;

	dc = fc->dc;
	while (dc) {
	    if (dc->st->st & st_Dir) count++;
	    dc = dc->next;
	}

	if (dctmp->st->nlink != count) {
	    dctmp->st->nlink = count;
	    if (sendEv) send_CacheEv(fctmp->evd, dctmp, ndcfu_CHANGE);
	}

end:
	gp_pc_free(&gptmp);
}

static TNetDataCache *Cache_ReMove_name(TglobPath *gp, bool sendEv) {
  TFtpCache *fc;
  TNetDataCache *dc, **ppdc;

	if (!gp || !gp->name) return NULL;
	fc = get_fc(gp);
	if (!fc) return NULL;

	ppdc = get_ppdc_name(&fc->dc, gp->name);
	dc = *ppdc;
	if (!dc) return NULL;
	*ppdc = dc->next;
	dc->next = NULL;
	if (sendEv) send_CacheEv(fc->evd, dc, ndcfu_DEL);
	if (dc->st->st & st_Dir) Cache_SetDirCount(gp, fc, sendEv);
	return dc;
}

static TNetDataCache *get_new_dc(Tstat64 *st) {
  TNetDataCache *result;
	result = xcalloc(sizeof(TNetDataCache) + st->StructSize + 2);
	pnmove(result->st, st, st->StructSize);
	result->name = Get_ExFileDat_pc(result->st, exfdt_FileName);
	return result;
}

TNetDataCache *FtpCache_add_dc_st(TNetDataCache **ppdc, Tstat64 *st, int *pp_ndcfu) {
  TNetDataCache *dc;
  const char *name;
  int ndcfu;
  TNetDataCache *Result = NULL;

	ndcfu = 0;
	name = Get_ExFileDat_pc(st, exfdt_FileName);
	while (*ppdc && !pccmp((*ppdc)->name,  name)) ppdc = &(*ppdc)->next;

	if (*ppdc) {
	    dc = *ppdc;
	    if (dc->st->st & st_Link) goto ende;

	    if ((dc->st->mtime == st->mtime) &&
		(dc->st->size  == st->size)  &&
		(dc->st->mode  == st->mode)  &&
		(dc->st->nlink == st->nlink) &&
		(dc->st->uid   == st->uid)   &&
		(dc->st->gid   == st->gid)) goto ende;

	    ndcfu = ndcfu_CHANGE;
	    Result = get_new_dc(st);
	    Result->next = dc->next;
	    *ppdc = Result;
	    xfree(dc);

	} else {
	    ndcfu = ndcfu_ADD;
	    Result = get_new_dc(st);
	    *ppdc = Result;
	}
ende:
	if (pp_ndcfu) *pp_ndcfu = ndcfu;
	return Result;
}


void FtpCache_add_st(TglobPath *gp, Tstat64 *st, bool sendEv) {
  TFtpCache *fc;
  TNetDataCache *dc;
  int ndcfu;

	if (!gp || !st) return;
	fc = get_fc_Add(gp);

	dc = FtpCache_add_dc_st(&fc->dc, st, &ndcfu);
	if (dc && sendEv) send_CacheEv(fc->evd, dc, ndcfu);
}

void FtpCache_chmod(TglobPath *gp, int mode, int mask) {
  TFtpCache *fc;
  TNetDataCache *dc;
  int omod;

	if (!get_fc_dc(gp, &fc, &dc)) return;
	omod = (dc->st->mode & ~mask) | (mode & mask);
	if (dc->st->mode != omod) {
	    dc->st->mode = omod;
	    send_CacheEv(fc->evd, dc, ndcfu_CHANGE);
	}
}

void FtpCache_mtime(TglobPath *gp, time_t mtime) {
  TFtpCache *fc;
  TNetDataCache *dc;

	if (!get_fc_dc(gp, &fc, &dc)) return;
	if (dc->st->mtime != mtime) {
	    dc->st->atime = mtime;
	    dc->st->mtime = mtime;
	    dc->st->ctime = mtime;
	    send_CacheEv(fc->evd, dc, ndcfu_CHANGE);
	}
}

static TNetDataCache *get_lnk_dc(TNetDataCache *dc0, const char *name, int max) {
  TNetDataCache *dc;
  const char *pc = name;

	if (!*pc || !max) return NULL;
	while (*pc && TestNoSlash(*pc)) pc++;
	if (*pc) return NULL;
	dc = get_dc_name(dc0, name);
	if (dc && (dc->st->st & st_Link)) {
	    dc = get_lnk_dc(dc0, Get_ExFileDat_pc(dc->st, exfdt_LinkName), max-1);
	}
	return dc;
}

static bool setup_lnk(TNetDataCache *dc0, TNetDataCache *dc) {
  TNetDataCache *ndc;

	if (dc->st->st & st_Link) {
	    ndc = get_lnk_dc(dc0, Get_ExFileDat_pc(dc->st, exfdt_LinkName), 8);
	    if (ndc) {
		if ((dc->st->size  == ndc->st->size) &&
		    (dc->st->mode  == ndc->st->mode) &&
		    (dc->st->mtime == ndc->st->mtime) &&
		    (dc->st->st    == (ndc->st->st | st_Link)) &&
		    (dc->st->FileType == ndc->st->FileType)) return False;

		dc->st->size = ndc->st->size;
		dc->st->mode = ndc->st->mode;
		dc->st->mtime = ndc->st->mtime;
		dc->st->st = ndc->st->st | st_Link;
		dc->st->FileType = ndc->st->FileType;
		return True;
	    }
	}
	return False;
}

void FtpCache_chklnk(TglobPath *gp, bool sendEv) {
  TFtpCache *fc;
  TNetDataCache *dc;

	if (!gp) return;
	fc = get_fc(gp);
	if (!fc) return;

	dc = fc->dc;
	while (dc) {
	    if (setup_lnk(fc->dc, dc)) {
		if (sendEv) send_CacheEv(fc->evd, dc, ndcfu_CHANGE);
	    }
	    dc = dc->next;
	}
}

void FtpCache_rename(TglobPath *gp, const char *new_name, bool sendEv) {
  int m;
  TFtpCache *fc;
  TNetDataCache *dc, *new_dc;
  TExFileDat *ExDat;

	if (!gp || !gp->name) return;

	fc = get_fc(gp);
	if (!fc) return;
	dc = get_dc_name(fc->dc, gp->name);
	if (!dc) return;
	ExDat = Get_TExFileDat(dc->st, exfdt_FileName);
	if (!ExDat) return;
	if (sendEv) send_CacheEv(fc->evd, dc, ndcfu_DEL);

	m = pclen(new_name) + 1 - ExDat->lg;
	if (m <= 0) {
	    pcmove(ExDat->dat, new_name);

	} else {
	    int lg, l1;

	    lg = sizeof(TNetDataCache) + dc->st->StructSize + 2;
#ifdef POINTER64BIT
	    l1 = (int64_t)ExDat->dat - (int64_t)dc;
#else
	    l1 = (int)ExDat->dat - (int)dc;
#endif
	    new_dc = xcalloc(lg + m);
	    new_dc->next = dc->next;
	    byte *a = (byte*)dc;
	    byte *b = (byte*)new_dc;
	    pnmove(b, a, l1);
	    a += l1;
	    b += l1+m;
	    pnmove(b, a, lg-l1);

	    ExDat = Get_TExFileDat(new_dc->st, exfdt_FileName);
	    ExDat->lg = pclen(new_name) + 1;
	    pcmove(ExDat->dat, new_name);
	    new_dc->name = ExDat->dat;

	    TNetDataCache **ppdc = get_ppdc_name(&fc->dc, gp->name);
	    *ppdc = new_dc;
	    xfree(dc);
	    dc = new_dc;
	}
	if (sendEv) send_CacheEv(fc->evd, dc, ndcfu_ADD);
}

bool FtpCache_mv(TglobPath *gp, TglobPath *new_gp, bool sendEv) {
  TFtpCache *fc;
  TNetDataCache **ppdc;
  TNetDataCache *src_dc = Cache_ReMove_name(gp, sendEv);

	if (!src_dc) return False;
	if (!new_gp) goto err;
	fc = get_fc(new_gp);
	if (!fc) goto err;
	ppdc = get_ppdc_st(&fc->dc, src_dc->st);
	if (!*ppdc) {
	    *ppdc = src_dc;
	    if (sendEv) send_CacheEv(fc->evd, src_dc, ndcfu_ADD);
	    if (src_dc->st->st & st_Dir) Cache_SetDirCount(new_gp, fc, sendEv);
	    return True;
	}
err:
	xfree(src_dc);
	return False;
}

TNetDataCache *FtpCache_get_dc(TglobPath *gp) {
  TFtpCache *fc;

	if (!gp) return NULL;

	fc = get_fc(gp);
	if (!fc) return NULL;
	return fc->dc;
}

Tstat64 *FtpCache_get_st(TglobPath *gp) {
  TFtpCache *fc;
  TNetDataCache *dc;
  const char *name;

	if (!gp) return NULL;

	fc = get_fc(gp);
	if (!fc) return NULL;

	name = gp->name;
	if (!name) name = ".";
	dc = get_dc_name(fc->dc, name);
	if (dc) return dc->st;
	return NULL;
}

TNetDataCache *FtpCache_get_dc0(TglobPath *gp) {
  TFtpCache *fc;
	if (!gp) return NULL;

	fc = get_fc(gp);
	if (!fc) return NULL;
	return fc->dc;
}

bool FtpCache_ChkDirOk(TglobPath *gp) {
  TNetDataCache *dc = FtpCache_get_dc0(gp);
	return get_dc_name(dc, ".") != NULL;
}

/* ___________________________ Clr ______________________________________ */
static void FtpCache_Clr_dc_ev(TNetDataCache **ppdc, TFtpCacheEvData *evd) {
  TNetDataCache *dc = *ppdc;
	if (dc) {
	    *ppdc = dc->next;
	    if (evd) send_CacheEv(evd, dc, ndcfu_DEL);
	    xfree(dc);
	}
}

void FtpCache_Clr_dc(TNetDataCache **ppdc) {
	FtpCache_Clr_dc_ev(ppdc, NULL);
}

void FtpCache_Clr_dc_all(TNetDataCache **ppdc) {
	while (*ppdc) FtpCache_Clr_dc_ev(ppdc, NULL);
}

void FtpCache_ClrAll() {
  TFtpCache *fc;

	while (FtpCache) {
	    fc = FtpCache;
	    FtpCache = FtpCache->next;
	    FtpCache_Clr_dc_all(&fc->dc);
	    xfree(fc->path);
	    xfree(fc->user);
	    xfree(fc->URL);
	    xfree(fc);
	}
}


void FtpCache_Clr_name(TglobPath *gp, bool sendEv) {
  TNetDataCache *dc = Cache_ReMove_name(gp, sendEv);
	xfree(dc);
}

void FtpCache_Clr(TglobPath *gp, bool sendEv) {
  TFtpCache *fc, **ppfc;
  TNetDataCache *dc;

	if (!gp || !gp->path) return;
	ppfc = get_ppfc(gp);
	fc = *ppfc;
	if (!fc) return;

	if (!fc->evd) {
	    *ppfc = fc->next;
	}

	while (fc->dc) {
	    dc = fc->dc;
	    fc->dc = fc->dc->next;
	    if (sendEv) send_CacheEv(fc->evd, dc, ndcfu_DEL);
	    xfree(dc);
	}

	if (!fc->evd) {
	    xfree(fc->path);
	    xfree(fc->user);
	    xfree(fc->URL);
	    xfree(fc);
	}
}

void FtpCache_Sync_dc(TglobPath *gp, TNetDataCache **sync_ppdc) {
  TFtpCache *fc;
  TNetDataCache *dc, *odc, **ppdc, **s_ppdc;
  TFtpCacheEvData *evd;

	if (!gp || !*sync_ppdc) return;

	dc = *sync_ppdc;
	while (dc) {
	    setup_lnk(*sync_ppdc, dc);
	    dc = dc->next;
	}

	fc = get_fc_Add(gp);
	dc = fc->dc;
	while (dc) {
	    dc->stat |= ndcstat_DelMk;
	    dc = dc->next;
	}

	evd = fc->evd;

	s_ppdc = sync_ppdc;
	while (*s_ppdc) {
	    ppdc = get_ppdc_name(&fc->dc, (*s_ppdc)->name);
	    if (*ppdc) {
		(*ppdc)->stat &= ~ndcstat_DelMk;
		dc = *s_ppdc;
		*s_ppdc = dc->next;

//		if (!(dc->st->st & st_Link)) {
		    odc = *ppdc;
		    if ((odc->st->size  != dc->st->size) ||
			(odc->st->mode  != dc->st->mode) ||
			(odc->st->mtime != dc->st->mtime)) {
			odc->st->size  = dc->st->size;
			odc->st->mode  = dc->st->mode;
			odc->st->mtime = dc->st->mtime;
			send_CacheEv(evd, odc, ndcfu_CHANGE);
		    }
//		}
		xfree(dc);
	    } else {
		s_ppdc = &(*s_ppdc)->next;
	    }
	}

	ppdc = &fc->dc;
	while (*ppdc) ppdc = &(*ppdc)->next;
	*ppdc = *sync_ppdc;
	*sync_ppdc = NULL;
	while (*ppdc) {
	    send_CacheEv(evd, *ppdc, ndcfu_ADD);
	    ppdc = &(*ppdc)->next;
	}


	ppdc = &fc->dc;
	while (*ppdc) {
	    if ((*ppdc)->stat & ndcstat_DelMk) {
		FtpCache_Clr_dc_ev(ppdc, evd);
	    } else {
		ppdc = &(*ppdc)->next;
	    }
	}
}

/* ___________________________ EV _______________________________________ */
void FtpCache_DelEv(TglobPath *gp, void *obj) {
  TFtpCache *fc;
  TFtpCacheEvData *evd, **ppevd;

	if (!gp) return;
	fc = get_fc_Add(gp);

	ppevd = &fc->evd;
	while (*ppevd && ((*ppevd)->obj != obj)) ppevd = &(*ppevd)->next;
	evd = *ppevd;
	if (!evd) return;
	*ppevd = evd->next;
	xfree(evd);
}

TNetCache_DelEv FtpCache_AddEv(TglobPath *gp, void *obj, TNetCacheEv CacheEv) {
  TFtpCache *fc;
  TFtpCacheEvData *evd, **ppevd;

	if (!gp || !CacheEv) return NULL;
	fc = get_fc_Add(gp);

	evd = fc->evd;
	while (evd) {
	    if (evd->obj == obj) break;
	    evd = evd->next;
	}

	if (!evd) {
	    ppevd = &fc->evd;
	    while (*ppevd) ppevd = &(*ppevd)->next;
	    evd = xcalloc(sizeof(TFtpCacheEvData));
	    *ppevd = evd;
	    evd->obj = obj;
	}
	evd->CacheEv = CacheEv;
	return FtpCache_DelEv;
}
