/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "ftp_net.h"
#include "ftp_cache.h"
#include "tdirlist.h"

typedef struct {
  AlgTNetFtp
  TDirListLoader *dll;
  dword open_us;
} TNetDirList;

typedef struct Tprivate {
  TNetDirList *nftp;
  int mk;
} Tprivate;

static void DoBefEv(TDirListLoader *self, TFileMsg *fmsg, int bef) {
	fmsg->typD = fmTyp_int;
	fmsg->D.m = bef | self->obef;
	fmsg->typF = 0;
	fmsg->F.dfl = NULL;
	file_msg_ev(fmsg, (TTC0*)self->owner, self->OnFileMsg);
}

static void DoftplBefEv(TNetDirList *ndl, int bef) {
  TDirListLoader *dll = ndl->dll;
  TFileMsg fmsg;

	if (!dll) return;
	file_msg_init(&fmsg, fmArt_DirList);

	fmsg.typR = fmTyp_gp;
	fmsg.R.gp = dll->gp;
	DoBefEv(dll, &fmsg, bef);
}

static void TFtpCacheEv(TDirListLoader *self, TNetDataCache *dc, int fu) {
  Tprivate *pv;
  TFileMsg fmsg;
  TDirFileList *dfl;

	file_msg_init(&fmsg, fmArt_DirList);
	fmsg.typR = fmTyp_gp;
	fmsg.R.gp = self->gp;

	pv = self->private;
	dfl = *self->ppdfl;
	while (dfl && !pccmp(dc->name, dfl->fn)) dfl = dfl->next;
	if (dfl) {
	    if (!pv->mk) {
		pv->mk = 0x01;
		DoBefEv(self, &fmsg, dlbef_Open);
	    }

	    fmsg.typD = fmTyp_int;
	    fmsg.typF = fmTyp_dfl;
	    fmsg.F.dfl = dfl;
	    if ((dfl->st->st & st_Dir) != (dc->st->st & st_Dir)) {
		fmsg.D.m = dlbef_NeedSort | self->obef;
		file_msg_ev(&fmsg, (TTC0*)self->owner, self->OnFileMsg);
	    }

	    dfl->st->size =  dc->st->size;
	    dfl->st->mode =  dc->st->mode;
	    dfl->st->mtime =  dc->st->mtime;
	    dfl->st->st = dc->st->st;
	    dfl->st->FileType = dc->st->FileType;

	    fmsg.D.m = dlbef_Change | self->obef;
	    file_msg_ev(&fmsg, (TTC0*)self->owner, self->OnFileMsg);
	}
}

/*__________________________ dll_open appEv _____________________________*/
static void OnFileMsg_LS(TFileMsg *fmsg) {
  TFtpClientCMD *ftpcmd;
  TNetDirList *self;
  TDirListLoader *dll;

	if (!TNetFtp_fmsg_GetSelf(fmsg, &ftpcmd, (void*)&self)) return;

	if (TNetFtp_Chk_ftpappclient((TNetFtp*)self, fmsg)) {
	    if (self->ende) {
		DoftplBefEv(self, dlbef_ListEnd);
		DoftplBefEv(self, dlbef_Close);

		dll = self->dll;
		dll->obef = dlbef_RescanRun;
		FtpCache_AddEv(dll->gp, dll, (TNetCacheEv)TFtpCacheEv);
		FtpCache_chklnk(dll->gp, True);
		FtpCache_DelEv(dll->gp, dll);
		if (((Tprivate*)dll->private)->mk) {
		    DoftplBefEv(self, dlbef_ListEnd);
		}
		DoftplBefEv(self, dlbef_Close);
	    }

	} else if (fmsg->art == fmArt_DataIO) {

	    if (fmsg->typD == fmTyp_stat) {

		dll = self->dll;
		FtpCache_add_st(fmsg->R.gp, fmsg->D.st, False);
		if (chk_file_ok(fmsg->D.st, dll->gp->path)) {
		    TDirListLoader_New_st(dll, fmsg->D.st, fmsg);
		}
		if (cpTime(self->open_us + 100000, appDat.app_us)) {
		    DoftplBefEv(self, dlbef_Sync);
		    self->open_us = appDat.app_us;
		}
		fmsg->Ok = True;
		fmsg->ev_Stop = True;
	    }
	}
}

static void OnFileMsg_CD(TFileMsg *fmsg) {
  TFtpClientCMD *ftpcmd;
  TNetDirList *self;
  TDirListLoader *dll;

	if (!TNetFtp_fmsg_GetSelf(fmsg, &ftpcmd, (void*)&self)) return;

	dll = self->dll;
	if (TNetFtp_Chk_ftpappclient((TNetFtp*)self, fmsg)) {
	    if (self->ende) {
		DoftplBefEv(self, dlbef_Open);
		if (self->error) {
		    DoftplBefEv(self, dlbef_ListEnd);
		    DoftplBefEv(self, dlbef_Close);
		/* Start Rescan */
		    dll->obef = dlbef_RescanRun;
		    DoftplBefEv(self, dlbef_Close);

		} else {
		    self->open_us = appDat.app_us;
		}
	    }
	} else if (dll) {
	    file_msg_ev_next(fmsg, (TTC0*)dll->owner, dll->OnFileMsg);
	}
}

/*_____________________________ Open ____________________________________*/

static bool TDirList_ld_ftp_Open(TDirListLoader *self) {

  TNetDataCache *dc;
  char *path;
  Tprivate *pv;
  TFileMsg fmsg;
  bool Ok;

	file_msg_init(&fmsg, fmArt_DirList);
	fmsg.typR = fmTyp_gp;
	fmsg.R.gp = self->gp;
	fmsg.typD = fmTyp_int;

	self->obef = dlbef_ScanRun;

	dc = FtpCache_get_dc(self->gp);
	if (dc) {
	    DoBefEv(self, &fmsg, dlbef_Open);
	    while (dc) {
		if (chk_file_ok(dc->st, self->gp->path)) {
		    TDirListLoader_New_st(self, dc->st, &fmsg);
		}
		dc = dc->next;
	    }
	    DoBefEv(self, &fmsg, dlbef_ListEnd);
	    DoBefEv(self, &fmsg, dlbef_Close);
	    self->obef = dlbef_RescanRun;
	    DoBefEv(self, &fmsg, dlbef_Open);
	    DoBefEv(self, &fmsg, dlbef_Close);
	    return True;
	}

	pv = self->private;
	pv->nftp = TNetFtp_Create(sizeof(TNetDirList), wtyp_TNetDirList, NULL, self->gp);
	pv->nftp->dll = self;
	pv->nftp->omod = NetOpenMod_Read;
	pv->nftp->open_us = appDat.app_us;

	TFtpClientNextCMD *fcnx = xcalloc(sizeof(TFtpClientNextCMD));
	fcnx->bef = pcdup("ls -al");
	fcnx->OnFileMsg = OnFileMsg_LS;

	path = pcadd_n(3, "\"", self->gp->path, "\"");
	Ok = TNetFtp_ftpac_add_fcnx((TNetFtp*)pv->nftp, "cd", path, 
				OnFileMsg_CD, 0, fcnx);
	xfree(path);
	if (!Ok) {
	    ppTfree(&pv->nftp);
	    return False;
	}

	return True;
}

/*_____________________________ Close ___________________________________*/
static void TDirList_ld_ftp_Close(TDirListLoader *self) {
//  T_ftp_list *ftpl = ftp_list;
//	del_dll_open_next(self);
//	while (ftpl && (ftpl->dll != self)) ftpl = ftpl->next;
//	if (ftpl) ftpl->mk |= ftplmk_dll_del;
}

/*_____________________________ Destroy _________________________________*/
static bool TDirListLoader__Destroy(TDirListLoader *self) {
  Tprivate *pv;

	if (self->NetCache_DelEv) self->NetCache_DelEv(self->gp, self);
	if (self->close) self->close(self);
	if (!(self->memfu & dlmemfu_gpNotFree)) gpFree(self->gp);
	pv = self->private;
	Tfree(pv->nftp);

	xfree(pv);
	return False;
}

/*_____________________________ Create __________________________________*/
static TDirListLoader *TDirList_ld_ftp_Create(void *owner, TDirFileList **ppdfl, TOnFileMsg OnFileMsg) {
  TDirListLoader *self;

	self = ttc_create(sizeof(TDirListLoader), wtyp_TDirListLoader);
	self->destroy = (TDESTROY)TDirListLoader__Destroy;
	self->ppdfl = ppdfl;
	self->add_ppdfl = ppdfl;
	self->OnFileMsg = OnFileMsg;
	self->owner = owner;
	self->private = xcalloc(sizeof(Tprivate));
	self->open = TDirList_ld_ftp_Open;
	self->close = TDirList_ld_ftp_Close;
	self->scanfu = fsGetFileAll_UGT;
	return self;
}

const ThDirListLoader hdirlist_ld_ftp = {
  ftp_chk_RootArt_URL,
  TDirList_ld_ftp_Create,
//  TDirList_ld_ftp_ChkType,
  0
};
