/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "ftp_net.h"
#include "ftp_cache.h"
#include <errno.h>

typedef struct {
  AlgTNetFtp
  Tstat64 *st;
  char *fn;
  char *lnk;
  TNetDataCache *dc;
  int step;
  bool ForceReScan;
} TNetFtpStat;


static void OnFileMsg_STAT(TFileMsg *fmsg) {
  TFtpClientCMD *ftpcmd;
  TNetFtpStat *self;

	if (!TNetFtp_fmsg_GetSelf(fmsg, &ftpcmd, (void*)&self)) return;

	if (TNetFtp_Chk_ftpappclient((TNetFtp*)self, fmsg)) {

	    if (self->ende) {
		if (fmsg->typR == fmTyp_gp) {
		    if (self->ForceReScan) {
			FtpCache_Sync_dc(fmsg->R.gp, &self->dc);
		    }
		    FtpCache_chklnk(fmsg->R.gp, True);
		}
	    }

	} else if (fmsg->art == fmArt_DataIO) {

	    if (fmsg->typD == fmTyp_stat) {
		if (self->ForceReScan) {
		    FtpCache_add_dc_st(&self->dc, fmsg->D.st, NULL);
		} else {
		    FtpCache_add_st(fmsg->R.gp, fmsg->D.st, True);
		}
		fmsg->Ok = True;
		fmsg->ev_Stop = True;
	    }

	} else TNetFtp_OnFileMsg_ChkAlgErr((TNetFtp*)self, fmsg);
}

Tstat64 *ftp_stat_gp(TglobPath *gp, TNetErrRet *ErrRet, int fu) {
  dword fkt;
  TNetFtpStat *self;
  char *pc, *fn, *path;
  Tstat64 *res = NULL;
  TNetErrRet tmpErrRet;

	path = NULL;
	if (!ErrRet) {
	    ErrRet = &tmpErrRet;
	    net_err_init(ErrRet);
	}

	fkt = TFtpAppClient_get_fkt(gp);
	if (!(fkt & facfkt_STAT)) {
	    fu &= ~fsSingleReScan;
	    fu |= fsForceReScan;
	}

	if (fu & fsSingleReScan) {
	    if (!FtpCache_ChkDirOk(gp)) {
		fu &= ~fsSingleReScan;
		fu |= fsForceReScan;
	    }
	}

	if (!(fu & (fsSingleReScan | fsForceReScan))) {
	    res = FtpCache_get_st(gp);
	    if (res) goto end_Ok;
	}

	if (!gp->name || !*gp->name) {
	    fn = pcdup(gp->path);
	    int lg = pclen(fn);
	    if (lg > 1) fn[lg-1] = 0;
	    ppcchange(&fn, GetFileName_pc(fn));
	} else {
	    fn = pcdup(gp->name);
	}

	if (fn) {
	    pc = fn;
	    while (*pc) {
		if (*pc == ' ') *pc = '?';
		pc++;
	    }
	}

	path = pcadd_n(3, "\"", gp->path, "\"");

	self = TNetFtp_Create(sizeof(TNetFtpStat), wtyp_TNetStat, NULL, gp);
	self->ErrRet = ErrRet;
	self->omod = NetOpenMod_Read;
	self->fn = fn;

	TFtpClientNextCMD *fcnx = xcalloc(sizeof(TFtpClientNextCMD));
	if ((fu & (fsSingleReScan|fsForceReScan)) == fsSingleReScan) {
	    fcnx->bef = pcdup("status");
	    fcnx->cmd = pcdup(self->fn);
	    fcnx->akt_fkt = facfkt_STAT;
	    fcnx->next = xcalloc(sizeof(TFtpClientNextCMD));
	    fcnx->next->bef = pcdup("ls -al");
	    fcnx->next->cmd = pcdup(self->fn);
	    fcnx->next->nxfu = fcnxfu_SkipOk;
	} else {
	    self->ForceReScan = (fu & fsForceReScan) != 0;
	    fcnx->bef = pcdup("ls -al");
	}
	fcnx->OnFileMsg = OnFileMsg_STAT;


	if (!TNetFtp_ftpac_add_fcnx((TNetFtp*)self, "cd", path, 
				TNetFtp_OnFileMsg, 0, fcnx)) goto end_err;
	if (!TNetFtp_wait((TNetFtp*)self)) goto end_err;

	if (ErrRet && (ErrRet->errFlag & NetErrFlag_NewLogin)) {
	    ppcchange(&gp->user, ErrRet->new_user);
	    ppcchange(&gp->passwd, ErrRet->new_passwd);
	}

	res = FtpCache_get_st(gp);

end_err:
	FtpCache_Clr_dc_all(&self->dc);
	xfree(self->fn);
	xfree(self->lnk);
	Tfree(self);
end_Ok:
	xfree(path);
	if (res) res = tstat_dup(res);
	return res;
}

static bool ftp_chk_exist_gp(TglobPath *gp, int findAtr) {
  Tstat64 *st = ftp_stat_gp(gp, NULL, fsNoFileName|fsNoLinkName|fsLinkDirekt);
	if (!st) return False;
	mode_t st_mode = st->mode;
	xfree(st);
	return file_stat_mode_match(findAtr, st_mode);
}

static bool ftp_chk_open_gp(TglobPath *gp, TNetErrRet *ErrRet, int fu) {
  TNetFtp *self;
  bool result = False;

	self = TNetFtp_Create(sizeof(TNetFtpStat), wtyp_TNetFtp, NULL, gp);
	self->ErrRet = ErrRet;
	self->omod = NetOpenMod_Read;
	if (TNetFtp_ftpac_add((TNetFtp*)self, NULL, NULL, TNetFtp_OnFileMsg)) {
	    result = TNetFtp_wait((TNetFtp*)self);
	}
	Tfree(self);
	return result;
}

const ThNetStatLoader hNetStatLd_ftp = {
  ftp_chk_RootArt,
  .Create.lb = NULL,
  ftp_stat_gp,
  ftp_chk_exist_gp,
  ftp_chk_open_gp
};
