/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "net_io.h"
#include "ftp_io.h"
#include <errno.h>
#include <unistd.h>

static void TFtp_RecvData(int fd, Tfd_list *fdl) {
  int lg, fu = 0;
  TFtp *ftp = fdl->obj;
  bool eof;

	if (ftp->OnRecvData) ftp->OnRecvData(ftp, fd, fdl, &fu);
	if ((fu & 1) == 0) {
	    lg = TLineBuf_fd_Add(&ftp->RdDataBuf, fd);
	    ftp->TransfDatLg += lg;
	    eof = ftp->RdDataBuf.eof;
	    if (ftp->OnFtpDataRecv) ftp->OnFtpDataRecv(ftp);
	    if (eof) {
		TFtp_closeconn(ftp);
		ftp->ReplyBufAkt = False;
		while (ReplyBuf_Line(ftp));
	    }
	}
}


static void TFtp_SendData(int fd, Tfd_list *fdl) {
  TFtp *ftp = fdl->obj;
  TLineBuf *lb = ftp->WrDataBuf;
  bool fd_akt = (lb != NULL);

	if (fd_akt) {
  int fuEOL = 0;
  Tlgs lgs;
	    if (TLineBuf_GetData_lgs(lb, &lgs, BUFSIZ, &fuEOL)) {
		write(fd, lgs.s, lgs.lg);
		ftp->TransfDatLg += lgs.lg;
	    } else {
		appDat.AktPipes[fd] = OutPipe + 2;
	    }
	    if (ftp->OnFtpDataSend) ftp->OnFtpDataSend(ftp);
	    if (fuEOL & fio_EOF) {
		ppTfree(&ftp->WrDataBuf);
	    }

	} else {
	    TFtp_closeconn(ftp);
	    ftp->ReplyBufAkt = False;
	    while (ReplyBuf_Line(ftp));
// fprintf(stderr, "%s(%d) TFtp_SendData B closeconn %d\n", FL, fd);
// fflush(stderr);
	    if (ftp->OnFtpDataSend) ftp->OnFtpDataSend(ftp);
	}

}

static void DoStrgMSG(TFtp *ftp, int msg) {
	if (ftp->DoStrgMSG) ftp->DoStrgMSG(ftp, msg);
}

bool TFtp_BaseBefSend(TFtp *ftp) {
  const char *s;
  bool res;

loop:
	res = False;
	if (ftp->ftpSh & ftpSh_Connect) {
	    ppcfree(&ftp->AktPath);
	    DoStrgMSG(ftp, hSHStM_Connect);
	    ftp->ftpSchr = ftpSchr_Connect;
	    res = TFtp_Open(ftp);

	} else if (ftp->ftpSh & ftpSh_USER) {
	    ppcfree(&ftp->AktPath);
	    DoStrgMSG(ftp, hSHStM_user);
	    ftp->ftpSchr = ftpSchr_USER;
	    s = "anonymous"; 
	    if (ftp->gp.user && *ftp->gp.user) s = ftp->gp.user;
	    res = TFtp_Command(ftp, "USER %s", s);

	} else if (ftp->ftpSh & ftpSh_PASS) {
	    DoStrgMSG(ftp, hSHStM_passwd);
	    ftp->ftpSchr = ftpSchr_PASS;
	    if ((ftp->gp.passwd) && (*ftp->gp.passwd)) {
		res = TFtp_Command(ftp, "PASS %s", ftp->gp.passwd);
	    } else {
		res = TFtp_Command(ftp, "PASS ");
	    }

	} else if (ftp->ftpSh & ftpSh_SYST) {
	    ftp->ftpSchr = ftpSchr_SYST;
	    res = TFtp_Command(ftp, "SYST");

	} else if (ftp->ftpSh & ftpSh_TYPE_I) {
	    ftp->ftpSchr = ftpSchr_TYPE_I;
	    int ires = TFtp_changetype(ftp, TYPE_I);
	    res = ires > 0;
	    if (ires == 2) {
		ftp->ftpSh &= ~ftpSh_TYPE_I;
		goto loop;
	    }


	} else if (ftp->ftpSh & ftpSh_CWD) {
	    ftp->ftpSchr = ftpSchr_CWD;
	    char *path = path_add(ftp->gp.path, NULL);
	    res = TFtp_Command(ftp, "CWD %s", path);
	    xfree(path);

	} else if (ftp->ftpSh & ftpSh_InitConn) {
	    ftp->ftpSchr = ftpSchr_InitConn;
	    ftp->sendport = True;
	    res = TFtp_initconn(ftp);

/*	} else if (ftp->ftpSh & ftpSh_SIZE) {
	    ftp->ftpSchr = ftpSchr_SIZE;
	    char *pc = ftp->cmd_LS;
	    if (ftp->ftpSh & ftpSh_RETR) pc = ftp->srcFile;
	    if (ftp->ftpSh & ftpSh_STOR) pc = ftp->destFile;
	    res = TFtp_Command(ftp, "SIZE %s", pc);
*/
	} else if (ftp->ftpSh & ftpSh_MLST) {
	    ftp->ftpSchr = ftpSchr_MLST;
	    if (ftp->cmd_LS && *ftp->cmd_LS) {
		res = TFtp_Command(ftp, "MLST %s", ftp->cmd_LS);
	    } else {
		res = TFtp_Command(ftp, "MLST");
	    }

	} else if (ftp->ftpSh & ftpSh_MLSD) {
	    ftp->ftpSchr = ftpSchr_MLSD;
	    if (ftp->cmd_LS && *ftp->cmd_LS) {
		res = TFtp_Command(ftp, "MLSD %s", ftp->cmd_LS);
	    } else {
		res = TFtp_Command(ftp, "MLSD");
	    }

	} else if (ftp->ftpSh & ftpSh_LIST) {
	    DoStrgMSG(ftp, hSHStM_ls);
	    ftp->ftpSchr = ftpSchr_LIST;
	    if (ftp->ftpSh & ftpSh_RETR) {
		res = TFtp_Command(ftp, "LIST -al %s", ftp->srcFile);
	    } else if (ftp->ftpSh & ftpSh_STOR) {
		res = TFtp_Command(ftp, "LIST -al %s", ftp->destFile);
	    } else if (ftp->cmd_LS && *ftp->cmd_LS) {
		res = TFtp_Command(ftp, "LIST %s", ftp->cmd_LS);
	    } else {
		res = TFtp_Command(ftp, "LIST -al");
	    }

	} else if (ftp->ftpSh & ftpSh_RETR) {
	    ftp->ftpSchr = ftpSchr_RETR;
	    res = TFtp_Command(ftp, "RETR %s", ftp->srcFile);

	} else if (ftp->ftpSh & ftpSh_STOR) {
	    ftp->ftpSchr = ftpSchr_STOR;
	    ftp->TransfDatLg = 0;
	    ftp->TransfDatLg_Last_MSG = 0;
	    res = TFtp_Command(ftp, "STOR %s", ftp->destFile);

	} else if (ftp->ftpSh & ftpSh_STAT) {
	    ftp->ftpSchr = ftpSchr_STAT;
	    if (ftp->cmd_LS && *ftp->cmd_LS) {
		res = TFtp_Command(ftp, "STAT %s", ftp->cmd_LS);
	    } else {
		res = TFtp_Command(ftp, "STAT");
	    }

	} else if (ftp->ftpSh & ftpSh_QUIT) {
	    ftp->ftpSchr = ftpSchr_QUIT;
	    res = TFtp_Command(ftp, "QUIT");

	}
	return res;
}

bool TFtp_BaseBefRecv(TFtp *ftp, int code, char *msg) {
  int lg;
  const char *cpc, *toc;

	if (ftp->ftpSh & ftpSh_Connect) {
	    ftp->ftpSh &= ~ftpSh_Connect;
	    if (code >= 300) {
		ftp->ftpSh = ftpSh_QUIT;
	    }
	    TFtp_BaseBefSend(ftp);

	} else if (ftp->ftpSh & ftpSh_USER) {
	    ftp->ftpSh &= ~ftpSh_USER;
	    if ((code == 331) || (code == 332)) {
		ftp->ftpSh |= ftpSh_PASS;
	    } else if (code == 230) {
		ftp->ftpSh &= ~ftpSh_PASS;
	    } else {
		TFtp_Error(ftp, ioErr_NoLogin, "Login incorrect\nftp:\"%s\"\n", msg);
		ftp->ftpSh = ftpSh_QUIT;
	    }
	    TFtp_BaseBefSend(ftp);

	} else if (ftp->ftpSh & ftpSh_PASS) {
	    ftp->ftpSh &= ~ftpSh_PASS;
	    if (code != 230) {
		TFtp_Error(ftp, ioErr_NoLogin, "Login incorrect\nftp:\"%s\"\n", msg);
		ftp->ftpSh = ftpSh_QUIT;
	    }
	    TFtp_BaseBefSend(ftp);

	} else if (ftp->ftpSh & ftpSh_SYST) {
	    ftp->ftpSh &= ~ftpSh_SYST;
	    if (code == 215) {
		cpc = pcGetToken(msg, &toc, &lg);
		pcGetToken(cpc, &toc, &lg);
		if ((lg == 4) && pcncmp("UNIX", toc, lg)) {
		    ftp->ftpSh |= ftpSh_TYPE_I;
		}
	    }
	    TFtp_BaseBefSend(ftp);

	} else if (ftp->ftpSh & ftpSh_TYPE_I) {
	    ftp->ftpSh &= ~ftpSh_TYPE_I;
	    if (code == 200) {
		ftp->curtype = ftp->newtype;
	    }
	    TFtp_BaseBefSend(ftp);

	} else if (ftp->ftpSh & ftpSh_CWD) {
	    ftp->ftpSh &= ~ftpSh_CWD;
	    ppcfree(&ftp->AktPath);
	    if (code >= 500) {
		TFtp_Error(ftp, ioErr_PathNotFound, "No such directory\n\"%s\"", ftp->gp.path);
		ftp->ftpSh = 0;
	    } else {
		ftp->AktPath = path_add(ftp->gp.path, NULL);
		TFtp_BaseBefSend(ftp);
	    }

	} else if (ftp->ftpSh & ftpSh_InitConn) {
	    ftp->ftpSh &= ~ftpSh_InitConn;
	    if (ftp->passivemode) {
		if (code == 500) {
		    ftp->ftpSh |= ftpSh_InitConn;
		    ftp->passivemode = False;
	        } else if ((code != 227) || !TFtp_initconn_pasv(ftp)) {
		    ftp->ftpSh = 0;
		}
	    } else {
		if (code != 200) {
		    ftp->ftpSh = 0;
		}
	    }
	    TFtp_BaseBefSend(ftp);

	} else if (ftp->ftpSh & ftpSh_MLST) {
 fprintf(stderr, "ftpSh_MLST code=%d\n", code);
	    if (code == 250) {
		ftp->ftpSh &= ~ftpSh_MLST;
		TFtp_BaseBefSend(ftp);
	    } else {
		ftp->ftpSh = 0;
		TFtp_Error(ftp, code, msg);
		DoStrgMSG(ftp, hSHStM_ls | hSHStM_Error);
	    }

	} else if (ftp->ftpSh & ftpSh_MLSD) {
	    if (code == 150) {
		TLineBuf_Clr(&ftp->RdDataBuf);
		ClrReplyBuf(ftp);
		if (TFtp_dataconn(ftp)) {
		    ftp->ReplyBufAkt = True;
		    AddPipeProc(ftp->data, InPipe, TFtp_RecvData, ftp);
		} else {
		    ftp->ftpSh = 0;
		}
	    } else if (code == 226) {
	        ftp->ftpSh &= ~ftpSh_MLSD;
		TFtp_BaseBefSend(ftp);

	    } else {
		TFtp_closeconn(ftp);
		ftp->ftpSh = 0;
		TFtp_Error(ftp, code, msg);
		DoStrgMSG(ftp, hSHStM_ls | hSHStM_Error);
	    }

	} else if (ftp->ftpSh & ftpSh_LIST) {
	    if (code == 150) {
		TLineBuf_Clr(&ftp->RdDataBuf);
		ClrReplyBuf(ftp);
		if (TFtp_dataconn(ftp)) {
		    ftp->ReplyBufAkt = True;
		    AddPipeProc(ftp->data, InPipe, TFtp_RecvData, ftp);
		} else {
		    ftp->ftpSh = 0;
		}

	    } else if (code == 226) {
		ftp->ftpSh &= ~ftpSh_LIST;
		if (ftp->ftpSh & ftpSh_RETR) {
		    if (ftp->srcFileStat) {
			ftp->ftpSh |= ftpSh_InitConn;
		    } else {
			ftp->ftpSh &= ~ftpSh_RETR;
//			ftp->ftpSh_Err = ioErr_FileNotFound;
			TFtp_Error(ftp, ioErr_FileNotFound, "No such file\n\"%s\"", ftp->srcFile);
		    }
		} else if (ftp->ftpSh & ftpSh_STOR) {
		    ftp->ftpSh |= ftpSh_InitConn;
		}
		TFtp_BaseBefSend(ftp);
		if (!(ftp->ftpSh & ftpSh_RETR))
				DoStrgMSG(ftp, hSHStM_ls | hSHStM_xOk);
	    } else {
		TFtp_closeconn(ftp);
		ftp->ftpSh = 0;
		TFtp_Error(ftp, code, msg);
		DoStrgMSG(ftp, hSHStM_ls | hSHStM_Error);
	    }
/*
	} else if (ftp->ftpSh & ftpSh_SIZE) {
	    ftp->ftpSh &= ~ftpSh_SIZE;
  TdData shD;
	    Fill0(&shD, sizeof(shD));
	    shD.lg = sizeof(TdData);
	    shD.HeadArt = hServData;
	    shD.bef = hSHbd_FileSize;

	    if ((code >= 500) && (code < 530)) {
		shD.sh_ErrorNr = ioErr_Syntax;
	    } else if ((code >= 530) && (code < 540)) {
		shD.sh_ErrorNr = ioErr_NoLogin;
	    } else if (code >= 550) {
		shD.sh_ErrorNr = ioErr_PathNotFound;
	    } else {
		s = pcGetToken(msg, &toc, &lg);
		pcGetToken(s, &toc, &lg);
// fprintf(stderr, "%s(%d): ftpSh_SIZE %s\n", FL, toc);
		shD.sh_DataSize = pctoint64(toc);
		if (ftp->srcFileStat) ftp->srcFileStat->st.size = shD.sh_DataSize;
	    }
	    write(4, &shD, shD.lg);

	    TFtp_BaseBefSend(ftp);
*/
	} else if (ftp->ftpSh & ftpSh_RETR) {
	    if (code == 150) {
		TLineBuf_Clr(&ftp->RdDataBuf);
		ClrReplyBuf(ftp);
		if (TFtp_dataconn(ftp)) {
		    DoStrgMSG(ftp, hSHStM_RdOpen);
		    ftp->ReplyBufAkt = True;
		    AddPipeProc(ftp->data, InPipe, TFtp_RecvData, ftp);
		} else {
		    ftp->ftpSh = 0;
		}

	    } else if (code == 226) {
		ftp->ftpSh &= ~ftpSh_RETR;
		TFtp_BaseBefSend(ftp);
		DoStrgMSG(ftp, hSHStM_xOk);

	    } else {
		TFtp_closeconn(ftp);
		cpc = pcGetToken(msg, &toc, &lg);
		TFtp_Error(ftp, code, "%s", cpc);
		ftp->ftpSh = 0;
		DoStrgMSG(ftp, hSHStM_Error);
	    }

	} else if (ftp->ftpSh & ftpSh_STOR) {
// fprintf(stderr, "%s(%d): ftpSh_STOR %d %d\n", FL, code, (int)ftp->WrDataBuf);
	    if (code == 150) {
		Tfree(ftp->WrDataBuf);
		ftp->WrDataBuf = TLineBuf_Create();
		ClrReplyBuf(ftp);
		if (TFtp_dataconn(ftp)) {
		    ftp->DoStrgMSG(ftp, hSHStM_WrOpen);
		    ftp->ReplyBufAkt = True;
		    AddPipeProc(ftp->data, OutPipe+1, TFtp_SendData, ftp);
		} else {
		    ftp->ftpSh = 0;
		}

	    } else if (code == 226) {
		ftp->ftpSh &= ~ftpSh_STOR;
		TFtp_BaseBefSend(ftp);
		DoStrgMSG(ftp, hSHStM_xOk);

	    } else {
		TFtp_closeconn(ftp);
		if (code == 553) {
		    TFtp_Error(ftp, 553, "STOR write data failed \"no access ???\"\n\"%s\"\n", ftp->destFile);
		} else if (code >= 400) {
		    TFtp_Error(ftp, code, "STOR write data failed \n\"%s\"\n", ftp->destFile);
		} else {
		    TFtp_Error(ftp, code, "STOR write data unknown msg=%d \n\"%s\"\n", code, ftp->destFile);
		}
		ftp->ftpSh = 0;
		DoStrgMSG(ftp, hSHStM_Error);
	    }

	} else if (ftp->ftpSh & ftpSh_STAT) {
	    if (code < 300) {
		ftp->ftpSh &= ~ftpSh_STAT;
		TFtp_BaseBefSend(ftp);
		DoStrgMSG(ftp, hSHStM_xOk);

	    } else {
		cpc = pcGetToken(msg, &toc, &lg);
		TFtp_Error(ftp, code, "%s", cpc);

		ftp->ftpSh = 0;
		DoStrgMSG(ftp, hSHStM_Error);
	    }

	} else if (ftp->ftpSh & ftpSh_QUIT) {
		ftp->ftpSh = 0;

	} else return False;
	return True;
}
