/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "net_io.h"
#include "ftp_io.h"
#include <time.h>
#include <sys/stat.h>

static const char *S_months[12] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                       "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

static void Set__ExFileDat(TExFileDat **ExDat, const char *s, int lg, int exfdt) {
  TExFileDat *ex = *ExDat;
  char *pc;

	if (lg > 0) {
	    ex->type = exfdt;
	    ex->fu = 0;
	    ex->lg = lg;
	    pc = ex->dat;
	    *pc = 0;
	    if (s) pcxmove(pc, s, lg-1);
	    *ExDat = (TExFileDat*)(ex->dat + lg);
	    (*ExDat)->fu = exfdf_lastdat;
	} 
}

TdServFile *ftptoServFile(const char *ls, int *tsf) {
  int i, lg, stm;
  int l_owner, l_group, l_name, l_link;
  const char *p_owner, *p_group, *p_name, *p_link;

  TdServFile *res;
  Tstat64 *tstat;
  TExFileDat *ExDat;
  time_t currTime;
  struct tm tmptm;

	*tsf &= 0xff;
	if (!ls || !*ls) return NULL;
	res = xcalloc(sizeof(TdServFileStat));
	res->lg = sizeof(TdServFileStat);
	tstat = &res->st;

	tstat->uid = 0xffffffff;
	tstat->gid = 0xffffffff;

	while (*ls && ((byte)*ls < 0x21))  ls++;
	if (!*ls) goto Error;
	tstat->st = 0;

	stm = 0;
	switch(*ls) {
	case '-': 
		tstat->mode = S_IFREG; 
		tstat->st = st_Reg;
		break;
	case 'd':
		tstat->mode = S_IFDIR;
		stm = stft_Folder;
		tstat->st = st_Dir;
		break;
	case 's': tstat->mode = S_IFSOCK; stm = stft_Socket; break;
	case 'b': tstat->mode = S_IFBLK; stm = stft_Blockdev; break;
	case 'c': tstat->mode = S_IFCHR; stm = stft_Chardev; break;
	case 'l':
		tstat->mode = S_IFLNK;
		tstat->st = st_Link;
		break;
	case 'p': tstat->mode = S_IFIFO; stm = stft_FIFO; break;
	default: goto Error;
	}
	tstat->FileType = stm;
	ls++;

	while (*ls && (*ls == ' ')) ls++;
	if (!*ls) goto Error;
	if  (*ls == 'r') tstat->mode |= S_IRUSR; ls++;
	if  (*ls == 'w') tstat->mode |= S_IWUSR; ls++;
	if  (*ls == 'x') tstat->mode |= S_IXUSR; ls++;
	if  (*ls == 'r') tstat->mode |= S_IRGRP; ls++;
	if  (*ls == 'w') tstat->mode |= S_IWGRP; ls++;
	if  (*ls == 'x') tstat->mode |= S_IXGRP; ls++;
	if  (*ls == 'r') tstat->mode |= S_IROTH; ls++;
	if  (*ls == 'w') tstat->mode |= S_IWOTH; ls++;
	if  (*ls == 'x') tstat->mode |= S_IXOTH; ls++;

	tstat->nlink = ppctoint(&ls);

	while (*ls && (*ls == ' ')) ls++;
	p_owner = ls; l_owner = 0;
	while (*ls && ((byte)*ls > ' ')) {ls++; l_owner++;}

	while (*ls && (*ls == ' ')) ls++;
	p_group = ls; l_group = 0;
	while (*ls && ((byte)*ls > ' ')) {ls++; l_group++;}

	while (*ls && (*ls == ' ')) ls++;
	if ((byte)*ls >= 'A') {	//ftpd has no group ???
// -r--r--r--  1 500     18362 May 15  2006 LICENSE.GPL
	    tstat->size = pctoint64(p_group);
	    p_group = p_owner;
	    l_group = l_owner;
	} else {
// drwxr-xr-x    3 root       root             4096 Jan 26  2008 .AHwm
	    tstat->size = ppctoint64(&ls);
	}

	currTime = time(NULL);
	gmtime_r(&currTime, &tmptm);
	tmptm.tm_sec = 0;
	tmptm.tm_min = 0;
	tmptm.tm_hour = 0;

  int tm_mon = tmptm.tm_mon;

	while (((byte)*ls < 'A') && *ls) ls++;
	for (i=0;i<12;i++) {
	    if (!pcncasecmpASCII(ls, S_months[i], 3)) {
		tmptm.tm_mon = i;
		ls += 3;
		break;
	    }
	}

	tmptm.tm_mday = ppctoint(&ls);
	i = ppctoint(&ls);
	if (*ls == ':') {
	    tmptm.tm_hour = i;
	    ls++;
	    tmptm.tm_min = ppctoint(&ls);
	    if (tm_mon < tmptm.tm_mon) tmptm.tm_year--;
	} else {
	    tmptm.tm_year = i - 1900;
	}
	tstat->atime = timegm(&tmptm); //timelocal(tmptm); //mktime(tmptm);
	tstat->mtime = tstat->atime;
	tstat->ctime = tstat->atime;

	while (*ls && (*ls == ' ')) ls++;
	l_name = 0;
	p_name = ls;

	while ((byte)*ls > 0x1f) {
		if ((ls[0] == ' ') && ls[1] && (ls[1] == '-') &&
		    ls[2] && (ls[2] == '>') && ls[3] && (ls[3] == ' ')) {
			ls += 4;
			break;
		};
		ls++; l_name++;
	}

	while (*ls && (*ls == ' ')) ls++;
	l_link = 0;
	p_link = ls;
	while ((byte)*ls > 0x1f) {ls++; l_link++;}

	if ((*tsf & ftptsf_RootPath) && (l_name == 2) && (*p_name == '.') && (p_name[1] == '.')) {
		*tsf |= ftptsf_isDotFile;
		goto Error;
	} else {
	    if ((l_name > 1) && (*p_name == '.') && (p_name[1] != '.')) {
		tstat->st |= st_Hidden;
	    }
	    lg = 0;
	    if (l_name)  lg += l_name  + sizeof(TExFileDat) + 1;
	    if (l_link)  lg += l_link  + sizeof(TExFileDat) + 1;
	    if (l_owner) lg += l_owner + sizeof(TExFileDat) + 1;
	    if (l_group) lg += l_group + sizeof(TExFileDat) + 1;
	    lg += sizeof(TExFileDat) + fs_Type_MaxMem;
	    res->lg = lg + sizeof(TdServFileStat);
	    res = xrealloc(res, res->lg);
	    tstat = &res->st;
	    tstat->StructSize = lg + sizeof(Tstat64);
	    res->HeadArt = hServFileStat;
	    ExDat = &res->st.ExDat;
	    if (l_name) Set__ExFileDat(&ExDat, p_name, l_name+1, exfdt_FileName);
	    if (l_link) Set__ExFileDat(&ExDat, p_link, l_link+1, exfdt_LinkName);
	    if (l_owner) Set__ExFileDat(&ExDat, p_owner, l_owner+1, exfdt_owner);
	    if (l_group) Set__ExFileDat(&ExDat, p_group, l_group+1, exfdt_group);
	    Set__ExFileDat(&ExDat, NULL, fs_Type_MaxMem, exfdt_type);

	    if (!tstat->FileType) {
		dword MinSize;
		tstat->FileType = get_file_stft_ext(tstat->ExDat.dat, &tstat->sub_mime, &MinSize);
		if (MinSize > tstat->size) tstat->FileType = stft_None;
	    }
	}
	return res;
Error:
	xfree(res);
	return NULL;
}
