/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include <fcntl.h>

typedef struct {
  const byte ArtNr;
  const byte SubId;
  const word MinSize;
  const char s[10];
} TDefaultFileExtArt;

//extension
static const TDefaultFileExtArt DefaultFileExtArt_tab[] = {
  {stft_Pascal,  0,    0, "pas\0"},
  {stft_Txt,     0,    0, "txt\0"},
  {stft_Txt,     0,    0, "log\0"},
  {stft_Txt,     0,    0, "m4\0"},
  {stft_Txt,     0,    0, "conf\0"},
  {stft_Txt,     0,    0, "la\0"},
  {stft_Txt,     0,    0, "aldfile\0"},
#ifdef _OS_WIN32
  {stft_Txt,     0,    3, "ini\0"},
#endif
  {stft_gzip,    mTgz_gzip,	64, "tgz\0"},
  {stft_gzip,    mTgz_zip,	16, "zip\0"},
  {stft_gzip,    0,		16, "bz\0"},
  {stft_gzip,    0,		16, "bz2\0"},
  {stft_gzip,    0,		16, "gz\0"},
  {stft_gzip,    0,		16, "xz\0"},
  {stft_ar,      0,		16, "a\0"},
  {stft_ar,      mAr_tar,	99, "tar\0"},
  {stft_gzip,    0,		16, "z\0"},
  {stft_C,       0,    0, "c\0"},
  {stft_C,       0,    0, "h\0"},
  {stft_C,       0,    0, "cc\0"},
  {stft_C,       0,    0, "cpp\0"},
  {stft_CXX,     0,    0, "c++\0"},
  {stft_CXX,     0,    0, "cxx\0"},
  {stft_HTML,    0,    0, "htm\0"},
  {stft_HTML,    0,    0, "mht\0"},
  {stft_HTML,    0,    0, "html\0"},
  {stft_Binary,  0,   16, "o\0"},
  {stft_Binary,  0,   16, "dcu\0"},
  {stft_SO,      0,  256, "so\0"},
  {stft_SO,      0,  256, "dll\0"},
  {stft_Font_A,  0,   16, "pfa\0"},
  {stft_Font_A,  0,   16, "pfb\0"},
  {stft_Font_A,  0,   16, "ttf\0"},
  {stft_Font_A,  0,   16, "ttc\0"},
  {stft_Font_Fix,0,   16, "bdf\0"},
  {stft_Font_Fix,0,   16, "pcf\0"},
  {stft_Font_Fix,0,   16, "snf\0"},
  {stft_desktop, 0,    0, "desktop\0"},
  {stft_desktop, 0,    0, "directory\0"},
  {stft_IMAGE,   0,   16, "xpm\0"},
  {stft_IMAGE,   0,   32, "png\0"},
  {stft_IMAGE,   0,   32, "jpg\0"},
  {stft_IMAGE,   0,   32, "jpeg\0"},
  {stft_IMAGE,   0,   32, "bmp\0"},
  {stft_IMAGE,   0,    8, "ppm\0"},
  {stft_IMAGE,   0,    8, "pam\0"},
  {stft_IMAGE,   0,    8, "pnm\0"},
  {stft_IMAGE,   0,   16, "gif\0"},
  {stft_ASM,     0,    0, "S\0"},
  {stft_ASM,     0,    0, "inc\0"},
  {stft_Win32Exe,0,  256, "exe\0"},
  {stft_Shell,   1,    0, "spec\0"},
  {stft_PDF,     0,   32, "pdf\0"},
  {stft_ALDapj,  0,    0, "apj\0"},
  {stft_ALDapj,  0,    0, "aldapj\0"},
  {stft_office,  0,  256, "doc\0"},
  {stft_office,  0,   16, "xls\0"},
  {stft_rpm,     0,  256, "rpm\0"},
  {0, 0,  0, ""}
};

static const TDefaultFileExtArt DefaultFile2ExtArt_tab[] = {
  {stft_gzip,    mTgz_gzip,	64, "tar.gz\0"},
  {stft_gzip,    mTgz_bzip2,	64, "tar.bz2\0"},
  {stft_gzip,    mTgz_xz,	64, "tar.xz\0"},
  {0, 0, 0, ""}
};

static const char *GetFileExt_2dot(const char *pc) {
  const char *pce, *pce2;

	pce = NULL;
	pce2 = NULL;
	if (pc) {
	    while (*pc) {
		if (*pc == '.') {pce2 = pce; pce = pc;}
		pc++;
	    }
	}
	if (pce2) pce2++;
	return pce2;
}

static const TDefaultFileExtArt *scan_stft(const char *pc, const TDefaultFileExtArt *dfea) {
	if (pc) {
	    while (dfea->ArtNr) {
		if (!pccasecmpASCII(dfea->s, pc)) return dfea;
		dfea++;
	    } 
	}
	return NULL;
}

word get_file_stft_ext(const char *fn, word *SubId, dword *MinSize) {
  dword rSubId, rMinSize;
  int stm;
  const char *pc, *s;
  const TDefaultFileExtArt *dfea;

	rSubId = rMinSize = 0;
	stm = stft_NoFound;
	if (!fn || !*fn) goto ende;
	s = GetFileName_pc(fn);

	dfea = scan_stft(GetFileExt_2dot(s), DefaultFile2ExtArt_tab);
	if (!dfea) dfea = scan_stft(GetFileExt_pc(s), DefaultFileExtArt_tab);
	if (dfea) {
	    rSubId = dfea->SubId;
	    rMinSize = dfea->MinSize;
	    stm = dfea->ArtNr;
	    goto ende;
	}

	if (!pccasecmpASCII(s, "makefile") || !pcncasecmpASCII(s, "makefile.", 9)) {
	    stm = stft_Make;
	    goto ende;
	}
	if (!pccasecmpASCII(s, "README")) {
	    stm = stft_Txt;
	    goto ende;
	}
	if (!pccasecmpASCII(s, "configure")) {
	    stm = stft_Shell;
	    goto ende;
	}

	pc = GetFileExt_pc(s);
	if (pc) {
	    pc--;
LoopSO:
	    pc--;
	    while ((pc > s) && (*pc != '.') && (*pc != '/')) pc--;
	    if ((pc > s) && (*pc == '.')) {
		if ((pc[1] == 's') && (pc[2] == 'o') && (pc[3] == '.')) {
		    rMinSize = 256;
		    stm = stft_SO;
		    goto ende;
		} goto LoopSO;
	    }
	}

ende:
	if (MinSize) *MinSize = rMinSize;
	if (SubId)   *SubId = rSubId;
	return stm;
}
