/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"
#include <unistd.h>
#include "tmime.h"

typedef struct {
  byte *data;
  size_t lg;
//  size_t BufSize;
  TglobPath *gp;
  const char *fname;    //only name (no path)
  int stft;
  int sub_mime;
} THeadInfo;

static bool ChkHeadIs(byte **ppc, int *plg) {
  byte *pc = *ppc;
  int lg = *plg;

	if ((lg < 8) || (*pc != '#')) return False;
	pc++; lg--;
	while ((lg > 0) && (*pc == ' ')) {pc++; lg--;}
	if ((lg < 6) || (*pc != '!')) return False;
	pc++; lg--;
	while ((lg > 0) && (*pc == ' ')) {pc++; lg--;}
	*plg = lg;
	*ppc = pc;
	return *pc == '/';
}

static bool CpHeadIs(const byte *s, int lg, const byte **ppc) {
  const byte *pc = *ppc;

	while ((lg > 0) && (*pc > ' ') && (*pc == *s)) {pc++; s++; lg--;}
	if ((*pc <= ' ') && (*s <= ' ')) return True;
	while (*pc > ' ') pc++;
	while (*pc == ' ') pc++;
	*ppc = pc;
	return False;
}

static const char *s_sh_tab = "/bin/bash /bin/sh /bin/ash";

static bool __attribute__ ((noinline)) ChkHeadIsBash(byte *pc, int lg) {
  const byte *pct;

	if (lg > 256) lg = 256;
	if (ChkHeadIs(&pc, &lg)) {
	    pct = (const byte*)s_sh_tab;
loop:
	    if (CpHeadIs(pc, lg, &pct)) return True;
	    if (*pct) goto loop;
	}
	return False;
}

static const char *s_perl = "/bin/perl";

static bool __attribute__ ((noinline)) ChkHeadIsPerl(byte *pc, int lg) {
  const byte *pct;

	if (lg > 256) lg = 256;
	if (ChkHeadIs(&pc, &lg)) {
	    while (lg && (*pc == '/')) {
		pct = (const byte*)s_perl;
		if (CpHeadIs(pc, lg, &pct)) return True;
		pc++; lg--;
		while ((lg > 0) && (*pc > ' ') && (*pc != '/')) {pc++; lg--;}
	    }
	}
	return False;
}

#define MAX_office_Mime 33
static const char *office_Mime_tab[MAX_office_Mime] = {
  "1vnd.oasis.opendocument.database",	"1vnd.sun.xml.base",
  "1vnd.oasis.opendocument.spreadsheet","1vnd.oasis.opendocument.spreadsheet-template",
  "1vnd.sun.xml.calc",			"1vnd.sun.xml.calc.template",
  "1vnd.stardivision.calc",		"1vnd.stardivision.chart",
  "1msexcel",				"1vnd.ms-excel",
  "1vnd.oasis.opendocument.graphics",	"1vnd.oasis.opendocument.graphics-template",
  "1vnd.sun.xml.draw",			"1vnd.sun.xml.draw.template",
  "1vnd.stardivision.draw",		"1vnd.openofficeorg.extension",
  "1vnd.oasis.opendocument.formula",	"1vnd.sun.xml.math",
  "1vnd.stardivision.math",		"1vnd.oasis.opendocument.text",
  "1vnd.oasis.opendocument.text-template","1vnd.oasis.opendocument.text-web",
  "1vnd.oasis.opendocument.text-master","1vnd.sun.xml.writer",
  "1vnd.sun.xml.writer.template",	"1vnd.sun.xml.writer.global",
  "1vnd.stardivision.writer",		"1msword",
  "1vnd.ms-word",			"1x-doc",
  "1rtf",				"1vnd.wordperfect",
  "1wordperfect"
};

static bool __attribute__ ((noinline)) chk_office_Mime(char *pc, int *sub_mime) {
  int i;
  const char *pcn;

	for (i=0;i<MAX_office_Mime;i++) {
	    pcn = office_Mime_tab[i];
	    pcn++;
	    if (pcncmp(pc, pcn, pclen(pcn))) {
		*sub_mime = i;
		return True;
	    }
	}
	return False;
}

const char *get_office_MimeType(int sub_mime) {
	if ((sub_mime < 0) || (sub_mime >= MAX_office_Mime)) return NULL;
	return office_Mime_tab[sub_mime];
}

static void chk_Txt_sub_mime(THeadInfo *sif) {
  const char *pc = NULL;
	if (sif->gp) pc = sif->gp->name;
	if (!pc) pc = sif->fname;
	if (!pc) return;
	pc = GetFileExt_pc(pc);
	if (!pc) return;
	if ((*pc >= '0') && (*pc <= '8') && !pc[1]) {
	    if (sif->data[0] == '.') {
		sif->sub_mime = mTxt_troff;
	    }
	}
}

static bool ChkAsciiCharU(dword c, dword *stat) {
	if ((c == 0x0A) || (c == 0x0D)) {
	    *stat |= 0x03;
	    return True;
	}
	return ((c >= 0x20) && (c < 0x80))
		|| ((c >= 0xA0) && (c < 0xFB00))
		||  (c == 0x09) || (c == 0x0c) || (c == 0x1B);
}

#define NoLF_Ok 1

static bool ChkTxt(THeadInfo *sif, int fkt) {
  byte *pb;

	pb = sif->data;
	if (!(sif->lg & 1) && (sif->lg > 3) && (*((word*)pb) == 0xfeff)) {
// BOM Unicode
	    dword stat = 0;
	    word *pw = (word*)pb;
	    int lg = 510;
	    if (sif->lg < 512) lg = sif->lg - 2;
	    lg = (lg >> 1) - 1;
	    pw++;
	    while ((lg > 0) && ChkAsciiCharU(*pw, &stat)) {
		pw++;
		lg--;
	    }
	    if (!lg && ((stat & 0x07) || (fkt & NoLF_Ok))) {
		sif->stft = stft_Txt;
		sif->sub_mime = mTxt_Unicode;
		return True;
	    }

	} else if (!(sif->lg & 1) && (sif->lg > 3)&&(*((word*)pb) == 0xfffe)) {
// BOM UnicodeBE
	    dword stat = 0;
	    int lg = 510;
	    if (sif->lg < 512) lg = sif->lg - 2;
	    lg = (lg >> 1) - 1;
	    pb += 2;
	    while ((lg > 0) && ChkAsciiCharU((pb[0] << 8) | pb[1], &stat)) {
		pb += 2;
		lg--;
	    }
	    if (!lg && ((stat & 0x07) || (fkt & NoLF_Ok))) {
		sif->stft = stft_Txt;
		sif->sub_mime = mTxt_UnicodeBE;
		return True;
	    }

	} else if ((sif->lg > 3)&&(pb[0]==0xef)&&(pb[1]==0xbb)&&(pb[2]==0xbf)) {
// BOM UTF8
	    dword stat = 0;
//	    int lg = sif->lg - 3;
	    int lg = 252;
	    if (sif->lg < 256) lg = sif->lg - 4;
	    pb += 3;
	    while ((lg > 0) && ChkAsciiCharU(*pb, &stat)) {pb++;lg--;}
	    if (!lg && ((stat & 0x07) || (fkt & NoLF_Ok))) {
		sif->stft = stft_Txt;
		sif->sub_mime = mTxt_UTF8;
		return True;
	    }

	} else {
// ASCII
	    dword stat = 0;
	    int lg = 255;
	    if (sif->lg < 256) lg = sif->lg - 1;
	    while ((lg > 0) && ChkAsciiCharU(*pb, &stat)) {pb++; lg--;}
	    if (!lg && ((stat & 0x07) || (fkt & NoLF_Ok))) {
		if (ChkHeadIsBash(sif->data, sif->lg)) {
		    sif->stft = stft_Shell;
		    return True;
		}
		if (ChkHeadIsPerl(sif->data, sif->lg)) {
		    sif->stft = stft_sh_perl;
		    return True;
		}
		sif->stft = stft_Txt;
		chk_Txt_sub_mime(sif);
		return True;
//	    } else {
// printf("%s(%d): %d 0x%x\n", FL, lg, *pb);
	    }
	}
	return False;
}

static void get_file_head_mime(THeadInfo *sif) {
  int i;
  byte *pb, *pbn;
  int *pint, *pint2;

	sif->stft = stft_None;
	sif->sub_mime = 0;
	if (!sif->data || (sif->lg <= 0)) return;
	pb = sif->data;

	if (sif->lg > 32) {
	    pint = (int*)pb;
	    switch (*pint) {

	    case 0x474e5089:	//".PNG"
		sif->sub_mime = mImg_png;
		sif->stft = stft_IMAGE;
		return;

	    case 0x6d783f3c:	//"<?xm"
		sif->stft = stft_XML;
		return;

	    case 0x464c457f:	//"ELF"
		if (pb[16] == 0x02) sif->stft = stft_Exec; else sif->stft = stft_SO;
		return;

	    case 0x04034b50:	//"PK.."
		switch (pb[4]) {
		case 0x00:
		case 0x09:
		case 0x0a:
		case 0x0b:
		    sif->sub_mime = mTgz_zip;  //application/zip
		    sif->stft = stft_gzip;
		    return;
		case 0x14:
		    if (pcncmp((char*)pb+30, "mimetype", 8) &&
			chk_office_Mime((char*)pb+50, &sif->sub_mime)) {
			sif->stft = stft_office;
			return;
		    } else {
			sif->sub_mime = mTgz_zip;  //application/zip
			sif->stft = stft_gzip;
			return;
		    }
		}
		break;

	    case 0x46464952:	//"RIFF"
		if (pcncmp((char*)pb+8, "WAVEfmt ", 8)) {
		    sif->sub_mime = mAudio_x_wav;
		    sif->stft = stft_audio;
		    return;
		} else if (pcncmp((char*)pb+8, "AVI LIST", 8)) {
		    sif->sub_mime = mVideo_x_msvideo;
		    sif->stft = stft_video;
		    return;
		}
		break;

	    case 0xb3010000:	//MPEG sequences
	    case 0xba010000:	//MPEG sequences
	    case 0xbb010000:	//MPEG sequences
		sif->sub_mime = mVideo_mpeg;
		sif->stft = stft_video;
		return;
	    case 0xA3DF451A:
		sif->sub_mime = mVideo_x_matroska;
		sif->stft = stft_video;
		return;
	    case 0x5367674F:	//OggS
		sif->sub_mime = mVideo_ogg;
		sif->stft = stft_video;
		return;
	    }

	    switch (*pint & 0x00ffffff) {
	    case 0x00535746:	//"FWS"
	    case 0x00535743:	//"CWS"
		sif->sub_mime = mVideo_x_shockwave_flash;
		sif->stft = stft_video;
		return;
	    case 0x00564c46:	//"FLV"
		sif->sub_mime = mVideo_x_flv;
		sif->stft = stft_video;
		return;
	    }

	    pint = (int*)(pb + 4);
	    pint2 = (int*)(pb + 8);
	    switch (*pint) {
	    case 0x70797466:	//"ftyp"
		switch (*pint2) {
		case 0x6d6f7369:		//"isom"
		case 0x326f7369:		//"iso2"
		case 0x3134706d:		//"mp41"
		case 0x3234706d:		//"mp42"
		case 0x34706d6d:		//"mmp4"
		    sif->sub_mime = mVideo_mp4;
		    sif->stft = stft_video;
		    return;
		case 0x31637661:		//"avc1"
		    sif->sub_mime = mVideo_3gpp;
		    sif->stft = stft_video;
		    return;
		}

		switch (*pint2 & 0x00ffffff) {
		case 0x0041344d:		//"M4A"
		case 0x0056344d:		//"M4V"
		    sif->sub_mime = mVideo_mp4;
		    sif->stft = stft_video;
		    return;
		case 0x0050344d:		//"M4P"
		case 0x0042344d:		//"M4B"
		    sif->sub_mime = mVideo_quicktime;
		    sif->stft = stft_video;
		    return;
		case 0x00706733:		//"3gp"
		    sif->sub_mime = mVideo_3gpp;
		    sif->stft = stft_video;
		    return;
		}

		switch (*pint2 & 0x0000ffff) {
		case 0x00007471:		//"qt"
		    sif->sub_mime = mVideo_quicktime;
		    sif->stft = stft_video;
		    return;
		}
		break;

	    case 0x766f6f6d:	//"moov"
	    case 0x7461646d:	//"mdat"
		sif->sub_mime = mVideo_quicktime;
		sif->stft = stft_video;
		return;
	    }
	}

	if (ChkTxt(sif, 0)) return;

	if (sif->lg > 500) {		//test audio/mpeg
	    pbn = sif->data; //pb;
	    i = sif->lg - 32;
	    while (i && !*pbn) {i--; pbn++;}

	    if ((pbn[0] == 0xff) && 
		((pbn[1] & 0xe0) == 0xe0) &&
		((pbn[1] & 0x18) != 0x08) &&
		((pbn[1] & 0x06) != 0x00) &&
		((pbn[2] & 0xf0) != 0x00) &&
		((pbn[2] & 0xf0) != 0xf0) &&
		((pbn[2] & 0x0c) != 0x0c)) {
		sif->sub_mime = mAudio_mpeg;
		sif->stft = stft_audio;
		return;
	    }
	}

	if (ChkTxt(sif, NoLF_Ok)) return;

}

static void get_mem_head_mime(byte *data, size_t size, TglobPath *gp, word *stft, word *sub_mime) {
  THeadInfo sif;
	sif.data = data;
	sif.lg = size;
//	sif.BufSize = size;
//	if (sif.lg > 512) sif.lg = 512;
	sif.gp = gp;
	sif.fname = gp->name;
	get_file_head_mime(&sif);
	if (stft) *stft = sif.stft;
	if (sub_mime) *sub_mime = sif.sub_mime;
}

static bool get_File_head_mime_st_fn(Tstat64 *st, const char *pathfn, const char *fname) {
  FileHandle fd;
  THeadInfo sif;
  bool ok;
  char buf[512];

	if (!file_chk_access_st(st, R_OK)) return False;

	if (!fname) fname = GetFileName_pc(pathfn);
	if (pcncmp(pathfn, "/proc/", 6)) {
		if (pccmp(fname, "kmsg") ||
		    pccmp(fname, "kcore") ||
		    pccmp(fname, "locks") ||
		    pccmp(fname, "mem")) return False;
	}
	if (!file_open_rd(pathfn, NULL, &fd)) return False;
	size_t lg;
//	sif.BufSize = sizeof(buf);
	ok = file_read(fd, NULL, buf, sizeof(buf), &lg);
	sif.lg = lg;
	file_close(fd, NULL);
	if (!ok) return False;

	sif.data = (byte*)buf;
	sif.gp = NULL;
	sif.fname = fname;
	get_file_head_mime(&sif);
	st->FileType = sif.stft;
	st->sub_mime = sif.sub_mime;
	return True;
}


ThGetFileHeadInfo hGetFileHeadInfo_buildin = {
//  get_file_head_mime,
  get_mem_head_mime,
  get_File_head_mime_st_fn
};

