/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"

#include "tmime.h"

//1=application
//2=text
//3=image
//4=audio
//5=video

#if stft_ListEnde != 43
#error stft_ListEnde <> 43
#endif

static const char *fileMimeTyp_tab[8] = {
	NULL,			"application/",
	"text/",		"image/",
	"audio/",		"video/",
	NULL,			NULL
};

static const char *fileMime_tab[stft_ListEnde] = {
	NULL,			"1x-directory",
	NULL,			"1x-character-device",
	"1x-block-device",	"1x-fifo",
	"1x-socket",		NULL,
	NULL,	NULL,	NULL,	NULL,	NULL,	NULL,	NULL,	NULL,
	NULL,			"2plain",		//16..17
	"1x-executable",	"1x-sharedlib",
	"1x-archive",		"1x-gzip",
	"2x-shellscript",	"2x-pascal",
	"2x-c",			"2x-c++",
	"2html",		"1x-object",
	"2x-makefile",		NULL,
	NULL,			"2plain",		//30..31
	"2x-c",			NULL,
	"1x-ms-dos-executable",	"2x-perl",
	"1pdf",			"2aldapj",
	NULL,			NULL,
	NULL,			"1xml",
	"1x-rpm"
};

static const char *txt_Mime_tab[8] = {
	NULL,			"2plain",
	"2plain",		"2plain",
	"2troff",		NULL,
	NULL,			NULL
};

static const char *ar_Mime_tab[8] = {
	NULL,			"1x-tar",
	NULL,			NULL,
	NULL,			NULL,
	NULL,			NULL
};

static const char *gzip_Mime_tab[8] = {
	NULL,			"1x-gzip",
	"1zip",			"1x-bzip2",
	"1x-xz",		NULL,
	NULL,			NULL
};

static const char *img_Mime_tab[8] = {
	NULL,			"3png",
	NULL,			NULL,
	NULL,			NULL,
	NULL,			NULL
};

static const char *audio_Mime_tab[8] = {
	NULL,			"4x-wav",
	"4mpeg",		NULL,
	NULL,			NULL,
	NULL,			NULL
};

static const char *video_Mime_tab[16] = {
	NULL,			"5x-msvideo",
	"5mpeg",		"5mp4",
	"53gpp",		"1x-shockwave_flash",
	"5x-flv",		"5quicktime",
	"5x-matroska",		"1ogg",
	NULL,			NULL,
	NULL,			NULL,
	NULL,			NULL
};


typedef struct {
  const char *ext;
  const char *mime;
} TImgMime;

static TImgMime TImgMime_tab[] = {
  {"xpm",  "2x-c"},
  {"png",  "3png"},
  {"jpg",  "3jpeg"},
  {"jpeg", "3jpeg"},
  {"bmp",  "3x-bmp"},
  {"ppm",  "3x-portable-pixmap"},
  {"pam",  "3x-portable-pixmap"},
  {"pnm",  "3x-portable-pixmap"},
  {NULL, NULL}
};

char *stat_get_bi_mimetype(Tstat64 *st) {
  const char *pc;
  int m;
  dword k;
  word idx = st->FileType;

	if (idx >= stft_ListEnde) idx = stft_None;

	k = st->sub_mime;

	if (idx == stft_IMAGE) {
	    if ((k > 0) && (k < 8)) {
		pc = img_Mime_tab[k];
		if (pc) goto set_mime_str;
	    }

	    pc = Get_ExFileDat_pc(st, exfdt_FileName);
	    pc = GetFileExt_pc(pc);
	    if (pc) {
  const TImgMime *im = TImgMime_tab;
		while (im->ext) {
		    if (!pccasecmpASCII(im->ext, pc)) {
			pc = im->mime;
			goto set_mime_str;
		    }
		    im++;
		}
	    }
	    return NULL;
	}

	if (k && (idx == stft_Txt)) {
	    if (k > 7) return NULL;
	    pc = txt_Mime_tab[k];
	    goto set_mime_str;
	}

	if ((idx == stft_ar) && (k < 8)) {
	    pc = ar_Mime_tab[k];
	    if (pc) goto set_mime_str;
	}

	if ((idx == stft_gzip) && (k < 8)) {
	    pc = gzip_Mime_tab[k];
	    if (pc) goto set_mime_str;
	}

	if (idx == stft_audio) {
	    if (k > 7) return NULL;
	    pc = audio_Mime_tab[k];
	    goto set_mime_str;
	}

	if (idx == stft_video) {
	    if (k > 15) return NULL;
	    pc = video_Mime_tab[k];
	    goto set_mime_str;
	}

	if (idx == stft_office) {
	    pc = get_office_MimeType(st->sub_mime);
	    goto set_mime_str;    
	}

	pc = fileMime_tab[idx];
set_mime_str:
	if (pc) {
	    m = *pc & 0x07; pc++;
	    return pcadd(fileMimeTyp_tab[m], pc);
	}
	return NULL;
}

