/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#ifndef _OS_WIN32
#include <sys/wait.h>
#endif

#include <dlfcn.h>
#include <setjmp.h>

void *magiclib_Handle = NULL;
void *mime_lib = NULL;
void *all_lib = NULL;
bool magiclib_Err = False;
bool magic_all_Err = False;
bool magic_mime_Err = False;

//#define	MAGIC_NONE		0x000000 /* No flags */
//#define	MAGIC_DEBUG		0x000001 /* Turn on debugging */
#define	MAGIC_SYMLINK		0x000002 /* Follow symlinks */
#define	MAGIC_COMPRESS		0x000004 /* Check inside compressed files */
//#define	MAGIC_DEVICES		0x000008 /* Look at the contents of devices */
#define	MAGIC_MIME_TYPE		0x000010 /* Return the MIME type */
//#define	MAGIC_CONTINUE		0x000020 /* Return all matches */
//#define	MAGIC_CHECK		0x000040 /* Print warnings to stderr */
//#define	MAGIC_PRESERVE_ATIME	0x000080 /* Restore access time on exit */
//#define	MAGIC_RAW		0x000100 /* Don't translate unprintable chars */
//#define	MAGIC_ERROR		0x000200 /* Handle ENOENT etc as real errors */
//#define	MAGIC_MIME_ENCODING	0x000400 /* Return the MIME encoding */
//#define MAGIC_MIME		(MAGIC_MIME_TYPE|MAGIC_MIME_ENCODING)
//#define	MAGIC_APPLE		0x000800 /* Return the Apple creator and type */
//#define	MAGIC_NO_CHECK_COMPRESS	0x001000 /* Don't check for compressed files */
//#define	MAGIC_NO_CHECK_TAR	0x002000 /* Don't check for tar files */
//#define	MAGIC_NO_CHECK_SOFT	0x004000 /* Don't check magic entries */
//#define	MAGIC_NO_CHECK_APPTYPE	0x008000 /* Don't check application type */
//#define	MAGIC_NO_CHECK_ELF	0x010000 /* Don't check for elf details */
//#define	MAGIC_NO_CHECK_TEXT	0x020000 /* Don't check for text files */
//#define	MAGIC_NO_CHECK_CDF	0x040000 /* Don't check for cdf files */
//#define	MAGIC_NO_CHECK_TOKENS	0x100000 /* Don't check tokens */
//#define MAGIC_NO_CHECK_ENCODING 0x200000 /* Don't check text encodings */

/* Defined for backwards compatibility (renamed) */
//#define	MAGIC_NO_CHECK_ASCII	MAGIC_NO_CHECK_TEXT

/* Defined for backwards compatibility; do nothing */
//#define	MAGIC_NO_CHECK_FORTRAN	0x000000 /* Don't check ascii/fortran */
//#define	MAGIC_NO_CHECK_TROFF	0x000000 /* Don't check ascii/troff */

typedef struct magic_set *magic_t;

magic_t (*magic_open)(int);
void (*magic_close)(magic_t);

const char *(*magic_file)(magic_t, const char *);
const char *(*magic_descriptor)(magic_t, int);
const char *(*magic_buffer)(magic_t, const void *, size_t);

const char *(*magic_error)(magic_t);
int (*magic_setflags)(magic_t, int);

int (*magic_load)(magic_t, const char *);
int (*magic_compile)(magic_t, const char *);
int (*magic_check)(magic_t, const char *);
int (*magic_errno)(magic_t);

static bool LoadLib_magic() {
	if (magiclib_Err) return False;
	if (magiclib_Handle) return True;

	magiclib_Handle = dlopen("libmagic.so.1", RTLD_LAZY);
	if (!magiclib_Handle) {
	    magiclib_Err = True;
	    printf("%s(%d) ERROR: can't load libmagic.so.1\n", FL);
	    return False;
	}
	magic_open =  load_lib(magiclib_Handle, "magic_open");
	magic_close = load_lib(magiclib_Handle, "magic_close");
	magic_load =  load_lib(magiclib_Handle, "magic_load");
	magic_file =  load_lib(magiclib_Handle, "magic_file");
	magic_error = load_lib(magiclib_Handle, "magic_error");

	if (!magic_open || !magic_close || !magic_load || !magic_file || !magic_error) {
	    dlclose(magiclib_Handle);
	    magiclib_Err = True;
	    printf("%s(%d) ERROR: can't init libmagic.so.1\n", FL);
	    return False;
	}

	return True;
}

/* bool libmagic_all_open() {
	if (magiclib_Err) return False;
	if (all_lib && magiclib_Handle) return True;
	if(magic_all_Err || !LoadLib_magic()) return False;
	all_lib = magic_open(MAGIC_SYMLINK|MAGIC_COMPRESS);
	if (all_lib && magic_load(all_lib, NULL) == -1) {
	    printf("%s(%d) ERROR loading with NULL file: %s\n", FL, magic_error(all_lib));
	    magic_close(all_lib);
	    magic_all_Err = True;
	    all_lib = NULL;
	    return False;
	}
	return (mime_lib != NULL);
} */

static bool libmagic_mime_open() {
	if (magiclib_Err) return False;
	if (mime_lib && magiclib_Handle) return True;
	if(magic_mime_Err || !LoadLib_magic()) return False;
	mime_lib = magic_open(MAGIC_SYMLINK|MAGIC_COMPRESS|MAGIC_MIME_TYPE);
	if (mime_lib && magic_load(mime_lib, NULL) == -1) {
	    printf("%s(%d) ERROR loading with NULL file: %s\n", FL, magic_error(mime_lib));
	    magic_close(mime_lib);
	    magic_mime_Err = True;
	    mime_lib = NULL;
	    return False;
	}
	return (mime_lib != NULL);
}


sigjmp_buf openAbort;

static void __attribute__ ((noreturn)) abortOpen(int n) {
        siglongjmp(openAbort, 1);
}

char *libmagic_get_mime(const char *fn) {
  char *pc, *res;
  sighandler_t old_sig;

	if (!libmagic_mime_open()) return NULL;
	old_sig = signal(SIGSEGV, abortOpen);
	if (sigsetjmp(openAbort, 0)) {
	    signal(SIGSEGV, old_sig);
	    magiclib_Err = True;
	    appError(EMSG_ERR2 | EMSG_tLibSIGSEGV, "Segmentation fault -> libmagic.so.1");
	    return NULL;
	}
	res = pcdup(magic_file(mime_lib, fn));
	signal(SIGSEGV, old_sig);

	pc = res;
	if (pc) {
	    while (*pc) {
		if ((*pc == ' ')||(*pc == '`')||(*pc == '\'')) {
		    xfree(res);
		    return NULL;
		}
		pc++;
	    }
	}
	
	return res;
}
