/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "tdirlist.h"

#include "tmime_pv.h"

typedef struct {
  TMimeObj *mo;
  TMimeDat_app *mda;
  const char *mimetype;
} TFindMime;

/* ----------------------------------------
mime="image/png"
mt="image/bmp;image/g3fax;image/gif;image/jpeg;image/png"
---------------------------------------- */
bool mimetype2app_chk_mime(const char *mime, const char *mt) {
  const char *pca, *pce;
  int lg = pclen(mime);

	if (!mt || !lg) return False;
	pca = pce = mt;
	while (*pce) {
	    if (*pce == ';') {
		int m = pce - pca;
		if ((m == lg) && 
		    !pcncasecmpASCII(mime, pca, m)) return True;
		pce++;
		pca = pce;
	    } else pce++;
	}
	return ((pce > pca) && !pcncasecmpASCII(mime, pca, pce-pca));
}

static bool find_type2app(TFindMime *fm) {
  TMimeDat_app *mda;

	if (!fm->mo) return False;
	mda = fm->mo->mda;
	while (mda) {
	    if (mimetype2app_chk_mime(fm->mimetype, mda->mimetype)) {
		fm->mda = mda;
	        return True;
	    }
	    mda = mda->next;
	}
	return False;
}

static bool find_path_type2app(TFindMime *fm) {

	if (!fm->mo) return False;
	while (fm->mo) {
	    if (find_type2app(fm)) return True;
	    fm->mo = fm->mo->next;
	}
	return False;
}

/* _____________________ deskname2app __________________________ */
static bool find_deskname2app(TFindMime *fm) {
  TMimeDat_app *mda;

	if (!fm->mo) return False;
	mda = fm->mo->mda;
	while (mda) {
	    if (pccmp(fm->mimetype, mda->fn)) {
		fm->mda = mda;
	        return True;
	    }
	    mda = mda->next;
	}
	return False;
}

static bool find_path_deskname2app(TFindMime *fm) {
	if (!fm->mo) return False;
	while (fm->mo) {
	    if (find_deskname2app(fm)) return True;
	    fm->mo = fm->mo->next;
	}
	return False;
}

bool TMime_type2app_getdat(TMime *self, const char* mimetype, TMimeDat *md) {
  TMime_private *pv;
  TMimeObj **ppmo;
  TFindMime fm;

	if (!self || !mimetype || !md) return False;
	pv = self->private;
	Fill0(md, sizeof(TMimeDat));
	md->type = ddkt_Application;

//	pcn = mimetype;   //zB.: image/png
//	pcd = mtyppath;   //image/png --> image

	if (pv->overwrite_ini) {
	    fm.mimetype = TpcIni_RD_pc(pv->overwrite_ini, "exec", mimetype, NULL);
	    if (fm.mimetype) {
	        ppmo = pv->mos;
		while (*ppmo) {
		    fm.mo = *ppmo;
		    if (find_path_deskname2app(&fm)) goto fm_OK;
		    ppmo++;
		}
	    }
	}

	ppmo = pv->mos;
	fm.mimetype = mimetype;
	while (*ppmo) {
	    fm.mo = *ppmo;
	    if (find_path_type2app(&fm)) goto fm_OK;
	    ppmo++;
	}
	return False;

fm_OK:
	md->id = fm.mda;
	md->path = fm.mo->altpath;
	if (fm.mo->ObjNy) md->path = fm.mo->ObjNy->dl->gp.path;
	md->fn = fm.mda->fn;
	md->exec = fm.mda->exec;
	md->icon = fm.mda->icon;
	md->mimetype = fm.mda->mimetype;
	return True;
}

// xxx.dektop = pcadd_n(3, md.path, md.fn, ".desktop");
bool mimetype2app_getdat(const char *mimetype, TMimeDat *md) {
	return TMime_type2app_getdat(app_mime, mimetype, md);
}

