/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "tdirlist.h"

#include "tmime_pv.h"

typedef struct {
  TMimeObj *mo;
  TMimeDat_type *mdt;
  const char *mimetype;
} TFindMime;

static bool find_type2lnk(TFindMime *fm) {
  TMimeDat_type *mdt;

	if (!fm->mo) return False;
	mdt = fm->mo->mdt;
	while (mdt) {
// printf("%s(%d): %s %s\n", FL, fm->mimetype, mdt->mimetype);
	    if (pccmp(fm->mimetype, mdt->mimetype)) {
		fm->mdt = mdt;
	        return True;
	    }
	    mdt = mdt->next;
	}
	return False;
}

static bool find_path_type2lnk(TFindMime *fm) {

	if (!fm->mo) return False;
	while (fm->mo) {
	    if (find_type2lnk(fm)) return True;
	    fm->mo = fm->mo->next;
	}
	return False;
}

static const char *app_skip[] = {
  "x-directory",
  "x-block-device",
  "x-character-device",
  "x-fifo",
  "x-socket",
  "x-executable",
  "x-sharedlib",
  NULL
};

bool TMime_type2lnk_getdat(TMime *self, const char* mimetype, TMimeDat *md) {
  TMime_private *pv;
  const char *pc;
  const char **ppc;
  TMimeObj **ppmo;
  TFindMime fm;

	if (!self || !mimetype || !md) return False;
	pv = self->private;

	Fill0(md, sizeof(TMimeDat));
	md->type = ddkt_Mime;

	if (pcncmp(mimetype, "application/", 12)) {
	    pc = mimetype + 12;
	    ppc = app_skip;
	    while (*ppc) {
		if (pccmp(*ppc, pc)) return False;
		ppc++;
	    }
	}
	if (pccmp(mimetype, "text/x-shellscript")) return False;


	ppmo = pv->mos;
	fm.mimetype = mimetype;

	while (*ppmo) {
	    fm.mo = *ppmo;
	    if (find_path_type2lnk(&fm)) goto fm_OK;
	    ppmo++;
	}
	return False;

fm_OK:
	md->id = fm.mdt;
	md->path = fm.mo->altpath;
	if (fm.mo->ObjNy) md->path = fm.mo->ObjNy->dl->gp.path;
	md->fn = fm.mdt->fn;
	md->icon = fm.mdt->icon;
	md->mimetype = fm.mdt->mimetype;
	return True;
}

bool mimetype2lnk_getdat(const char* mimetype, TMimeDat *md) {
	return TMime_type2lnk_getdat(app_mime, mimetype, md);
}
