/*
 Copyright (C) 2010-2011 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "net_io.h"
#include <errno.h>

#include "net_xxx.h"

static ThNetStatLoader **hNetStatLd = NULL;

/* _________________________ stat _______________________________________ */
Tstat64 *net_stat_gp(TglobPath *gp, TNetErrRet *ErrRet, int fu) {
  ThNetStatLoader *hLD;

	if (gp_test_local_drive(gp)) {
	    return file_stat_gp(gp, fu);

	} else {
	    hLD = GetNet4_hLD((void*)hNetStatLd, gp, 2, hNet_appERR);
	    if (!hLD) {
		errno = EPERM;
		if (ErrRet) {
		    ErrRet->sys_errno = EPERM;
		    ErrRet->errFlag = NetErrFlag_NoDev|NetErrFlag_NoLoader;
		}
		return PTR_FAILED;
	    }
	    return hLD->stat(gp, ErrRet, fu);
	}
}
Tstat64 *net_stat(const char *fn, TNetErrRet *ErrRet, int fu) {
  Tstat64 *res;
  TglobPath *gp = pctogp(fn);
	res = net_stat_gp(gp, ErrRet, fu);
	gpFree(gp);
	return res;
}
/* _________________________ chk_exist __________________________________ */
bool net_chk_exist_gp(TglobPath *gp, int findAtr) {
  ThNetStatLoader *hLD;
	if (gp_test_local_drive(gp)) {
	    return file_chk_exist_gp(gp, findAtr);
	} else {
	    hLD = GetNet4_hLD((void*)hNetStatLd, gp, 3, hNet_appERR);
	    if (!hLD) return False;
	    return hLD->chk_exist(gp, findAtr);
	}
}
bool net_chk_exist(const char *fn, int findAtr) {
  bool res;
  TglobPath *gp = pctogp(fn);
	res = net_chk_exist_gp(gp, findAtr);
	gpFree(gp);
	return res;
}
/* _________________________ chk_open ___________________________________ */
bool net_chk_open_gp(TglobPath *gp, TNetErrRet *ErrRet, int fu) {
  ThNetStatLoader *hLD;

	if (gp_test_local_drive(gp)) {
	    return True;

	} else {
	    hLD = GetNet4_hLD((void*)hNetStatLd, gp, 4, hNet_appERR);
	    if (!hLD) {
		errno = EPERM;
		if (ErrRet) {
		    ErrRet->sys_errno = EPERM;
		    ErrRet->errFlag = NetErrFlag_NoDev|NetErrFlag_NoLoader;
		}
		return False;
	    }
	    return hLD->chk_open(gp, ErrRet, fu);
	}
}

bool net_chk_open(const char *fn, TNetErrRet *ErrRet, int fu) {
  bool res;
  TglobPath *gp = pctogp(fn);
	res = net_chk_open_gp(gp, ErrRet, fu);
	gpFree(gp);
	return res;
}

void ThNetStatLoader_Reg(int n, ...) {
  va_list args;
	va_start(args, n);
	pp_void_va_add((void*)&hNetStatLd, n, args);
	va_end(args);
}
