/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "net_io.h"

#ifndef _OS_WIN32

#include <unistd.h>
//#include <sys/socket.h>
#include <linux/if.h>
#include <sys/ioctl.h>
#include <errno.h>

//or use /sys/class/net/eth0/address
bool SocketGetHWaddr(THWaddr *hwaddr, const char *HWname) {
  int lg, sock;
  struct ifreq ifr;
  bool result;

	if (!hwaddr) return false;
	
	Fill0(hwaddr, sizeof(*hwaddr));
	lg = pclen(HWname);
	if (!lg || (lg >= sizeof(ifr.ifr_name))) return false;

	sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (sock < 0) return false;

	result = false;
	Fill0(ifr.ifr_name, sizeof(ifr.ifr_name));
	Fill0(&ifr.ifr_hwaddr, sizeof(ifr.ifr_hwaddr));
	pcnmove(ifr.ifr_name, HWname, lg);
	if (ioctl(sock, SIOCGIFHWADDR, &ifr) >= 0) {
//	    hwaddr->len = 6;
	    pnmove(hwaddr->mac, &ifr.ifr_hwaddr.sa_data, 8);
	    result = true;
	}
	close(sock);
	return result;
}

bool SocketGetIFaddrIPv4(TIFaddrIPv4 *ifaddr, const char *HWname) {
  int lg, sock;
  struct ifreq ifr;
  bool result;

	if (!ifaddr) return false;
	Fill0(ifaddr, sizeof(*ifaddr));
	lg = pclen(HWname);
	if (!lg || (lg >= sizeof(ifr.ifr_name))) return false;

	sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (sock < 0) return false;

	result = false;
	Fill0(ifr.ifr_name, sizeof(ifr.ifr_name));
	Fill0(&ifr.ifr_hwaddr, sizeof(ifr.ifr_hwaddr));
	ifr.ifr_addr.sa_family = AF_INET;
	pcnmove(ifr.ifr_name, HWname, lg);
	if (ioctl(sock, SIOCGIFADDR, &ifr) >= 0) {
	    ifaddr->addr = ifr.ifr_addr;
//	    pcnmove(ifr.ifr_name, HWname, lg);
//	    if (ioctl(sock, SIOCGIFDSTADDR, &ifr) >= 0) ifaddr->dstaddr = ifr.ifr_dstaddr;
	    pcnmove(ifr.ifr_name, HWname, lg);
	    if (ioctl(sock, SIOCGIFBRDADDR, &ifr) >= 0) ifaddr->broadaddr = ifr.ifr_broadaddr;
	    pcnmove(ifr.ifr_name, HWname, lg);
	    if (ioctl(sock, SIOCGIFNETMASK, &ifr) >= 0) ifaddr->netmask = ifr.ifr_netmask;
	    result = true;
	}
	close(sock);
	return result;
}
#endif /* !_OS_WIN32 */

#ifdef _OS_WIN32
typedef struct TSockFD {
  struct TSockFD *next;
  int sock;
} TSockFD;

static TSockFD *SockFD;
static Tfd_list *WSAfdl;
static fd_set WSA_fd_set; 
static WSADATA wsaData;
static char buf_WSA[16];

/* static void AddSockFD(int sock) {
  TSockFD **ppsfd;
	ppsfd = &SockFD;
	while (*ppsfd) ppsfd = &(*ppsfd)->next;
	*ppsfd = xcalloc(sizeof(TSockFD));
	(*ppsfd)->sock = sock;
} */

static void DelSockFD(int sock) {
  TSockFD *sfd, **ppsfd;
  Tfd_list *fdl, **ppfdl;

	ppsfd = &SockFD;
	while (*ppsfd && ((*ppsfd)->sock != sock)) ppsfd = &(*ppsfd)->next;
	sfd = *ppsfd;
	if (sfd) {
	    closesocket(sock);
	    *ppsfd = sfd->next;
	    xfree(sfd);
	}

	ppfdl = &WSAfdl;
	while (*ppfdl && ((*ppfdl)->fd != sock)) ppfdl = &(*ppfdl)->next;
	fdl = *ppfdl;
	if (fdl) {
	    *ppfdl = fdl->next;
	    xfree(fdl);
	}
}

static void OnSocketFD(bool sel) {
  Tfd_list *fdl;
//  struct timeval tv;

	if (sel) {
//	    tv.tv_sec = 0;
//	    tv.tv_usec = 100000;
	    if (select(0, &WSA_fd_set, NULL, NULL, NULL) != 0) {
		fdl = WSAfdl;
		while (fdl) {
		    if (FD_ISSET(fdl->fd, &WSA_fd_set)) {
			if (fdl->Proc) fdl->Proc(fdl->fd, fdl);
		    }
		    fdl = fdl->next;
		}
	    }

	} else {
	    FD_ZERO(&WSA_fd_set);
	    fdl = WSAfdl;
	    while (fdl) {
		FD_SET(fdl->fd, &WSA_fd_set);
		fdl = fdl->next;
	    }
	}
}

Tfd_list *SocketAddPipeProc(int fd, int fu, TOnPipe OnPipe, void *obj) {
  Tfd_list *fdl, **ppfdl;

	if (!SockFD) {
	    appError(EMSG_ERR, "no WSAStartup()");
	    return NULL;
	}
// printf("%s(%d): %d max=%d\n", FL, fd, FD_SETSIZE);

	if (fd < 0) {
	    appError(EMSG_ERR1, "WSA fd=%d", fd);
	    return NULL;
	}
	if (WSA_fd_set.fd_count >= FD_SETSIZE) {
	    appError(EMSG_ERR1, "WSA fd=%d idx=%d >= max.idx=%d", fd, WSA_fd_set.fd_count, FD_SETSIZE);
	    return NULL;
	}

	if (!FOnSocketFD) FOnSocketFD = OnSocketFD;
	fdl = NULL;
	ppfdl = &WSAfdl;
	while (*ppfdl) {
	    fdl = *ppfdl;
	    if (fdl->fd == fd) break;
	    ppfdl = &fdl->next;
	}
	if (!fdl) {
	    fdl = xcalloc(sizeof(TSockFD));
	    *ppfdl = fdl;
	}

	fdl->fd= fd;
	fdl->fu= fu;
	fdl->Proc = OnPipe;
	fdl->obj = obj;
	return fdl;
}
#endif /* _OS_WIN32 */

int SocketOpen(int domain, int itype, int protocol) {
#ifdef _OS_WIN32
  int result;
  TSockFD **ppsfd;

	if (!SockFD) {
	    if (WSAStartup(MAKEWORD(2, 0), &wsaData) != 0) { /* Load Winsock 2.0 DLL */
		appError(EMSG_ERR1, "WSAStartup() failed");
		return -1;
	    }
	}
	result = socket(domain, itype, protocol);
	if (result >= 0) {
	    ppsfd = &SockFD;
	    while (*ppsfd) {
 if ((*ppsfd)->sock == result) {
  printf("%s(%d): SocketOpen fd=%d double ???\n", FL, result);
  return result;
 }
		ppsfd = &(*ppsfd)->next;
	    }
	    *ppsfd = xcalloc(sizeof(TSockFD));
	    (*ppsfd)->sock = result;
 if (!result)  printf("%s(%d): SocketOpen fd=0\n", FL);
	}
	return result;
#else  /* _OS_WIN32 */
	return socket(domain, itype, protocol);
#endif /* !_OS_WIN32 */
}

#if 1
#ifdef _OS_WIN32
void SocketClose(int sock) {
	if (sock <= 0) return;
	DelSockFD(sock);
	if (!SockFD) WSACleanup();   /* Cleanup Winsock */
}
#endif /* _OS_WIN32 */
#else
void SocketClose(int sock) {
	if (sock <= 0) return;
#ifdef _OS_WIN32
//	if (sock < 0) return;
	DelSockFD(sock);
	if (!SockFD) WSACleanup();   /* Cleanup Winsock */
#else  /* _OS_WIN32 */
	if (appDat.fd_list && (sock < appDat.fd_set_count)) {
	    appDat.fd_list[sock].Proc = NULL;
	}
	appDat.AktPipes[sock] = 0;
	close(sock);
#endif /* !_OS_WIN32 */
}
#endif

const char *SocketGetLastError_cpc() {
#ifdef _OS_WIN32
	if (!SockFD) return NULL;
	inttopc(buf_WSA, WSAGetLastError());
	return buf_WSA;
#else  /* _OS_WIN32 */
	return SysError_pc();
#endif /* !_OS_WIN32 */
}

int SocketGetLastError() {
#ifdef _OS_WIN32
	if (!SockFD) return 0;
	return WSAGetLastError();
#else  /* _OS_WIN32 */
	return errno;
#endif /* !_OS_WIN32 */
}

#if (!defined _OS_WIN32 || defined _WINSOCK2_H)
static char buf_ADDR[INET6_ADDRSTRLEN];  //INET6_ADDRSTRLEN 46
#endif

const char *SocketGetIP_cpc(struct sockaddr *sa) {
#if (defined _OS_WIN32 && !defined _WINSOCK2_H)
	return inet_ntoa(((struct sockaddr_in*)sa)->sin_addr);
#else  /* _OS_WIN32 */
	if (sa->sa_family == AF_INET) {
	    return inet_ntoa(((struct sockaddr_in*)sa)->sin_addr);
	} else {
	    buf_ADDR[sizeof(buf_ADDR)-1] = 0;
	    inet_ntop(sa->sa_family, &(((struct sockaddr_in6*)sa)->sin6_addr), 
		buf_ADDR, sizeof(buf_ADDR));
	    return buf_ADDR;
	}
#endif /* !_OS_WIN32 */
}

