/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "tdirlist.h"

ThDirListLoader **hdirlist_ld = NULL;
ThDirListNotify **hdirlist_ny = NULL;

/*_____________________________ appEv ___________________________________*/
static void SendDflEv(TDirList *self, TDirFileList *dfl, int bef) {
  TFileMsg fmsg;

	file_msg_init_Dm(&fmsg, fmArt_DirList, bef);
	if (dfl) {
	    fmsg.typF = fmTyp_dfl;
	    fmsg.F.dfl = dfl;
	}
	fmsg.typR = fmTyp_gp;
	fmsg.R.gp = &self->gp;
	file_msg_ev_next(&fmsg, (TTC0*)self, self->OnFileMsg);
}

static void TDirList_Clr(TDirList *self) {
  TDirFileList *dfl;

	if (!self->dfl) return;
	SendDflEv(self, NULL, dlbef_Clr);
	while (self->dfl) {
	    dfl = self->dfl;
	    self->dfl = dfl->next;
	    xfree(dfl);
	}
	SendDflEv(self, NULL, dlbef_ClrEnd);
}

static void dl_AddToDfl(TDirList *self, TFileMsg *fmsg) {
  int bef;
  TDirFileList **ppdfl;

	bef = fmsg->D.m & (dlbef_ScanRun|dlbef_RescanRun|dlbef_NotifyRun);
	bef |= dlbef_Add;

	ppdfl = &self->dfl;
	while (*ppdfl) ppdfl = &(*ppdfl)->next;

	*ppdfl = dfl_create(fmsg->F.st);

	SendDflEv(self, *ppdfl, bef);
}

static void TDirList_OnFileMsg(TFileMsg *fmsg) {
  TDirList *self;
  int bef;

	if (!file_msg_chk_wtyp(fmsg, wtyp_TDirList, FL)) return;

	self = fmsg->obj;
	if (fmsg->art == fmArt_NewLogin) {
	    if (fmsg->typR == fmTyp_gp) {
		ppcchange_cmp(&self->gp.user, fmsg->R.gp->user);
		ppcchange_cmp(&self->gp.passwd, fmsg->R.gp->passwd);
	    }
	    goto send_dflEv;
	}

	if ((fmsg->art != fmArt_DirList) ||
	    (fmsg->typD != fmTyp_int)) goto send_dflEv;

	fmsg->Ok = True;
	bef = fmsg->D.m;

	switch (bef & dlbef_BefMask) {
	case dlbef_New_st:  //Add file to dfl
	    dl_AddToDfl(self, fmsg);
	    return;

	case dlbef_DelSelf:
	    self->dlny = NULL;
	    if (self->ldr) {
		Tfree(self->ldr);
		self->ldr = NULL;
	    }
	    TDirList_Clr(self);
	    self->runstat = dlrunstat_NotOpen;
	    goto send_dflEv;

	case dlbef_Destroy:
	    if (self->ldr) {
		Tfree(self->ldr);
		self->ldr = NULL;
		self->runstat = dlrunstat_NotOpen;
	    }
	    return;
	}

	if (bef & dlbef_ScanRun) {
	    switch (bef & dlbef_BefMask) {

	    case dlbef_Open:
		self->runstat = dlrunstat_ScanRun;
		break;

	    case dlbef_ListEnd:
		if (self->bef & dlobef_Rescan) {
		    self->ldr->rescan_dfl = self->dfl;
		    self->ldr->scanfu = fsGetFileAll_UGT;
		}
		self->runstat = dlrunstat_ScanEnd;
		break;
	    }

	} else if (bef & dlbef_RescanRun) {
	    switch (bef & dlbef_BefMask) {

	    case dlbef_Open:
		self->runstat = dlrunstat_ReScanRun;
		break;

	    case dlbef_Close:
		if (self->dlny && self->dlny->start) {
		    self->dlny->start(self->dlny);
		    self->runstat = dlrunstat_NotifyRun;
		} else {
		    self->runstat = dlrunstat_ReScanEnd;
		}
		Tfree(self->ldr);
		self->ldr = NULL;
		break;
	    }
	}

send_dflEv:
	file_msg_ev_next(fmsg, (TTC0*)self, self->OnFileMsg);
}

/*_____________________________ ReScan __________________________________*/
void TDirList_ReScan(TDirList *self) {
	if (!self || !self->dlny || !self->dlny->ReScan) return;
	if (self->runstat >= dlrunstat_NotifyRun) self->dlny->ReScan(self->dlny);
}

/*_____________________________ Mime ____________________________________*/
void TDirList_ReScanMime(TDirList *self) {
  TDirFileList *dfl;
  void *id;
  bool mk;

	if (!app_mime || !app_mime->stat_set_mime_id) return;

	mk = False;
	dfl = self->dfl;
	while (dfl) {
	    if (dfl->st->st & st_Reg) {
		id = dfl->st->mime_id;
		app_mime->stat_set_mime_id(app_mime, dfl->st, Get_ExFileDat_pc(dfl->st, exfdt_FileName));
		if (id != dfl->st->mime_id) {
		    mk = True;
		    SendDflEv(self, dfl, dlbef_NotifyRun | dlbef_Change);
		}
	    }
	    dfl = dfl->next;
	}
	if (mk) SendDflEv(self, NULL, dlbef_NotifyRun | dlbef_ListEnd);
}

/*_____________________________ Open ____________________________________*/

bool TDirList_Open_pp(TDirList *self, int bef, ThDirListLoader **hddl, ThDirListNotify **hdlny) {
  bool UseRescan;

	if (!self || !hddl) return False;
// printf("%s(%d):        TDirList_Open         0x%x %s\n", FL, (int)self, self->gp.path);
	Tfree(self->ldr);
	self->ldr = NULL;
	self->bef = bef;
	self->runstat = dlrunstat_NotOpen;

	while (*hddl && !(*hddl)->ChkType(&self->gp)) hddl++;
	if (!*hddl) return False;
	self->last_dfl = NULL;

	self->ldr = (*hddl)->create(self, &self->dfl, TDirList_OnFileMsg);
	self->appFu_OR = (*hddl)->appFu_OR;

	if (self->bef & dlobef_Rescan) self->ldr->scanfu = fsFollowLink|fsGetFileType|fsGetFileMime|fsAdd_UID_GID_mem|fsAdd_Type_mem;

	UseRescan = True;
	switch (self->bef & dlobef_level_mask) {
	case dlobef_level_0:
	    self->ldr->scanfu = 0;
	    UseRescan = False;
	    break;
	case dlobef_level_1:
	    self->ldr->scanfu = fsFollowLink;
	    UseRescan = False;
	    break;
	case dlobef_level_2:
	    self->ldr->scanfu = fsFollowLink|fsGetFileType;
	    break;
	} 

	self->ldr->memfu = dlmemfu_gpNotFree;
	self->ldr->gp = &self->gp;
	self->ldr->patt = self->patt;
	self->ldr->rescan_dfl = NULL;
	if ((self->bef & dlobef_Notify) && hdlny) {
	    while (*hdlny && !self->dlny) {

		self->dlny = (*hdlny)->create(self, TDirList_OnFileMsg);
		if (self->dlny) {
		    self->dlny->patt = self->patt;
		}
		hdlny++;
	    }
	    if (self->dlny) {
		self->dlny->UseRescan = UseRescan;
	    }
	}

	self->runstat = dlrunstat_Create;
	if (self->ldr->open && self->ldr->open(self->ldr)) {
// printf("%s(%d): dlbef_Start %s\n", FL, self->gp.path);
	    SendDflEv(self, NULL, dlbef_Start);
	    return True;
	}
	self->runstat = dlrunstat_NotOpen;
	return False;
}

bool TDirList_Open(TDirList *self, int bef) {
	return TDirList_Open_pp(self, bef, hdirlist_ld, hdirlist_ny);
}

void TDirList_Open_Wait(TDirList *self, int runstat) {
  int appFu;
	if (self->runstat < 0) return;
	if (self->runstat < runstat) {
	    appFu = appDat.appFu;
	    appDat.appFu |= self->appFu_OR; //appFu_select_t_NULL;
	    while (AppHandleMessage() && (self->runstat >= 0) &&
					 (self->runstat < runstat));
	    appDat.appFu = appFu;
	}
}

/*_____________________________ Destroy _________________________________*/
static bool TDirList__Destroy(TDirList *self) {
// printf("%s(%d):        TDirList__Destroy     0x%x %s\n", FL, (int)self, self->gp.path);

	if (self->NetCache_DelEv) self->NetCache_DelEv(&self->gp, self);
	ppTfree(&self->dlny);
	ppTfree(&self->ldr);
	TDirList_Clr(self);
	if (!(self->memfu & dlmemfu_gpNotFree)) gp_pc_free(&self->gp);
//	xfree(self->private);
	Tfree(self->patt);
	return False;
}

/*_____________________________ Create __________________________________*/
static TDirList *dl_Create() {
  TDirList *self;
	self = ttc_create(sizeof(TDirList), wtyp_TDirList);
	self->destroy = (TDESTROY)TDirList__Destroy;
	return self;
}

TDirList *TDirList_Create_gp(TglobPath *gp) {
  TDirList *self = dl_Create();
	gp_dup_gpdata(&self->gp, gp);
	return self;
}

TDirList *TDirList_Create_gp_name(TglobPath *gp) {
  TDirList *self = TDirList_Create_gp(gp);
	gp_NameToPath(&self->gp);
	return self;
}

TDirList *TDirList_Create(const char *fn) {
  TDirList *self = dl_Create();
	pcsetgp(&self->gp, fn);
	return self;
}

TDirList *TDirList_Create_name(const char *fn) {
  TDirList *self = TDirList_Create(fn);
	gp_NameToPath(&self->gp);
	return self;
}

void ThDirList_Reg(int n, ...) {
  va_list args;
	va_start(args, n);
	pp_void_va_add((void*)&hdirlist_ld, n, args);
	va_end(args);
}

void ThDirList_ny_Reg(int n, ...) {
  va_list args;
	va_start(args, n);
	pp_void_va_add((void*)&hdirlist_ny, n, args);
	va_end(args);
}
