/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "tdirlist.h"
#include "tdirlist_pv.h"

static void st_copy_uid_gid(Tstat64 *org, Tstat64 *new, int funk) {
  const char *sExDat;
  xchar dExDat;
	sExDat = Get_ExFileDat_pc(new, funk);
	dExDat.s_const = Get_ExFileDat_pc(org, funk);
	if (sExDat && dExDat.s) pcxmove(dExDat.s, sExDat, fs_UID_GID_MaxMem);
}

void REGP2 stat_test_copy_uid_gid(Tstat64 *org, Tstat64 *new) {
	if (org->uid != new->uid) {
	    org->uid = new->uid;
	    st_copy_uid_gid(org, new, exfdt_owner);
	}
	if (org->gid != new->gid) {
	    org->gid = new->gid;
	    st_copy_uid_gid(org, new, exfdt_group);
	}
}

dword REGP2 chk_new_stat(Tstat64 *org, Tstat64 *new) {
  dword res = 0;

	if (org->mtime != new->mtime) res |= chstat_mtime;
	if (org->size  != new->size)  res |= chstat_size;
	if (org->mode  != new->mode)  res |= chstat_mode;
	if (org->nlink != new->nlink) res |= chstat_nlink;
	if (org->uid   != new->uid)   res |= chstat_uid;
	if (org->gid   != new->gid)   res |= chstat_gid;

#ifdef _OS_WIN32
	if (org->dwFileAttributes   != new->dwFileAttributes) res |= chstat_dwFileAttributes;
#endif
	return res;
}

dword REGP2 chk_new_stat_type(Tstat64 *org, Tstat64 *new) {
  dword res = chk_new_stat(org, new);
	if (org->FileType != new->FileType) res |= chstat_FileType;
	if (org->mime_id  != new->mime_id)  res |= chstat_mime_id;
	if (org->sub_mime != new->sub_mime) res |= chstat_sub_mime;
	return res;
}

void REGP2 dl_copy_new_stat(Tstat64 *org, Tstat64 *new) {
	    org->mtime = new->mtime;
	    org->size = new->size;
	    org->mode = new->mode;
#ifdef _OS_WIN32
	    org->dwFileAttributes = new->dwFileAttributes;
#endif
	    org->nlink = new->nlink;
	    stat_test_copy_uid_gid(org, new);
}

void REGP2 dl_copy_new_stat_type(Tstat64 *org, Tstat64 *new) {
	dl_copy_new_stat(org, new);
	org->mime_id = new->mime_id;
	org->FileType = new->FileType;
	org->sub_mime = new->sub_mime;
}

dword REGP3 TDirListNotify_chk_dlf_Change(TDirListNotify *self, TDirFileList *dfl, TglobPath *gp) {
  Tstat64 *stat, *org;
  int m;
  dword chstat;

	m = fsFollowLink | fsNoFileName |
		fsNoLinkName | fsGetFileType | fsAdd_UID_GID_mem;
	if (!self->UseRescan) m = fsFollowLink | fsNoFileName | fsNoLinkName;

	org = dfl->st;
	stat = file_stat_gp(gp, m);
	if (!stat) return False; //???
	chstat = chk_new_stat(org, stat);
	if (chstat) {
/*	printf("0x%x=0x%x 0x%x=0x%x 0x%x=0x%x 0x%x=0x%x 0x%x=0x%x 0x%x=0x%x\n",
		org->mtime, stat->mtime, org->size,  stat->size,
	        org->mode,  stat->mode,  org->nlink, stat->nlink,
	        org->uid,   stat->uid,   org->gid,   stat->gid);
*/
// printf("%s(%d): Notify %s %d\n", FL, dfl->fn, stat->FileType);
	    if (self->UseRescan &&
		((org->FileType != stat->FileType) ||
		 (org->FileType == stft_NoFound))) dfl->nymk |= dflnymk_Rescan;
	    dl_copy_new_stat(org, stat);
	} else if (dfl->nymk & dflnymk_Rescan) {
// printf("%s(%d): Notify %s\n", FL, dfl->fn);
	    dfl->nymk &= ~dflnymk_Rescan;
	    xfree(stat);
	    stat = file_stat_gp(gp, fsFollowLink | fsNoFileName |
				     fsNoLinkName | fsGetFileType | fsGetFileTypeHead);
	    if (stat) {
		chstat = chk_new_stat(org, stat);
		if (org->FileType != stat->FileType) chstat |= chstat_FileType;
//		if (org->mime_id  != stat->mime_id)  res |= chstat_mime_id;
		if (org->sub_mime != stat->sub_mime) chstat |= chstat_sub_mime;

		org->FileType = stat->FileType;
		org->sub_mime = stat->sub_mime;
	    }
	}
	xfree(stat);
	dfl->chstat = chstat;
	return chstat;
}

void REGP3 TDirListNotify_DodllEv(TDirListNotify *self, TDirFileList **dfl, int bef) {
  TFileMsg fmsg;
	if (!self->OnFileMsg) return;

	file_msg_init_Dm(&fmsg, fmArt_DirList, dlbef_NotifyRun | bef);

	if (!(self->nymk & dlnymk_MsgOpen)) {
	    self->nymk |= dlnymk_MsgOpen;
	} 

	if (dfl && *dfl) {
	    fmsg.typF = fmTyp_dfl;
	    fmsg.F.dfl = *dfl;
	}

//	if (self->gp) {
	    fmsg.typR = fmTyp_gp;
	    fmsg.R.gp = &self->dl->gp;
//	}

	file_msg_ev(&fmsg, (TTC0*)self->dl, self->OnFileMsg);

	if (dfl) *dfl = fmsg.F.dfl;
}

TDirFileList REGP2 **get_ppdfl(TDirFileList **ppdfl, TDirFileList *dfl) {
	while (*ppdfl && (*ppdfl != dfl)) ppdfl = &(*ppdfl)->next;
	return ppdfl;
}

TDirFileList REGP2 *TDirListNotify_Del(TDirListNotify *self, TDirFileList *dfl) {
  TDirFileList **ppdfl, *dfl_next;

	dfl_next = dfl->next;
	ppdfl = get_ppdfl(&self->dl->dfl, dfl);
	if (*ppdfl) {
	    *ppdfl = dfl->next;
	    dfl->chstat = chstat_Del;
// printf("%s(%d): TDirListNotify_Del\n", FL);
	    TDirListNotify_DodllEv(self, &dfl, dlbef_Del);
	    xfree(dfl);
	}
	return dfl_next;
}

TDirFileList REGP2 *TDirListNotify_Add_st(TDirListNotify *self, Tstat64 *stat) {
  TDirFileList *dfl, **ppdfl;

	dfl = dfl_create(stat);
	dfl->chstat = chstat_Add;
	ppdfl = &self->dl->dfl;
	while (*ppdfl) ppdfl = &(*ppdfl)->next;
	*ppdfl = dfl;
	return dfl;
}


TDirFileList REGP2 *TDirListNotify_Add(TDirListNotify *self, TglobPath *gp) {
  TDirFileList *dfl;
  Tstat64 *stat;

	stat = file_stat_gp(gp, fsFollowLink|fsGetFileType|fsGetFileMime|fsAdd_UID_GID_mem|fsAdd_Type_mem);
//  printf("%s(%d): iNotify_add_dfl %s 0x%x\n", FL, gp->name, (int)stat);
	if (!stat) return NULL;
	dfl = TDirListNotify_Add_st(self, stat);

	if (stat->FileType == stft_NoFound) dfl->nymk |= dflnymk_Rescan;
	xfree(stat);
	return dfl;
}

void REGP1 TDirListNotify_SyncEv(TDirListNotify *self) {
	if (self->nymk & dlnymk_MsgOpen) TDirListNotify_DodllEv(self, NULL, dlbef_Sync);
}

void REGP1 TDirListNotify_ScanEnd(TDirListNotify *self) {
	if (self->nymk & dlnymk_MsgOpen) {
	    TDirListNotify_DodllEv(self, NULL, dlbef_ListEnd);
	    TDirListNotify_DodllEv(self, NULL, dlbef_Close);
	    self->nymk &= ~dlnymk_MsgOpen;
	}
}
