/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "tdirlist.h"
#include "tdirlist_pv.h"

#define APP_SYNC_TIME 20000

typedef struct Tprivate {
  Alg_dirlist_nypv
} Tprivate;

void nypv_ChkFile(TDirListNotify *self) {
  Tprivate *pv = self->private;
  TDirFileList *dfl = pv->tmpdfl;
  char *fn = pv->gp.name;
  dword chstat;

	if (!fn || !*fn) return;
	if (!chk_pattern_match(self->patt, fn, NULL)) return;
	while (dfl) {
	    if (pccmp(fn, dfl->fn)) break;
	    dfl = dfl->next;
	}

	if (!dfl) {
	    dfl = self->dl->dfl;
	    while (dfl) {
		if (pccmp(fn, dfl->fn)) break;
		if (pv->tmpdfl == dfl) {
		    dfl = NULL;
		} else {
		    dfl = dfl->next;
		}
	    }
	}

	if (dfl) {
	    pv->tmpdfl = dfl->next;
	    dfl->nymk &= ~dflnymk_DelMk;

	    if (dfl->nymk & dflnymk_Add) {
		TDirListNotify_DodllEv(self, &dfl, dlbef_Add);
		dfl->nymk &= ~dflnymk_Add;
	    }
	    chstat = TDirListNotify_chk_dlf_Change(self, dfl, &pv->gp);
	    if (dfl->nymk & dflnymk_Rescan) pv->change_akt = True;
	    if (chstat) {
		TDirListNotify_DodllEv(self, &dfl, dlbef_Change);
		if (!(dfl->st->st & st_Dir)) {
		    dfl->nymk |= dflnymk_WrClose;
		    pv->change_akt = True;
		}
	    } else if (dfl->nymk & dflnymk_WrClose) {
		TDirListNotify_DodllEv(self, &dfl, dlbef_WrClose);
		dfl->nymk &= ~dflnymk_WrClose;
	    }

	} else {
	    dfl = TDirListNotify_Add(self, &pv->gp);
	    if (dfl) {
		TDirListNotify_DodllEv(self, &dfl, dlbef_Add);
		if (!(dfl->st->st & st_Dir)) {
		    dfl->nymk |= dflnymk_WrClose;
		    pv->change_akt = True;
		}
	    }
	}
}

bool nypv_OpenDir(TDirListNotify *self) {
  Tprivate *pv = self->private;
  TDirFileList *dfl;

	pv->change_akt = False;
	pv->schr++;
	pv->tmpdfl = NULL;
#ifdef _OS_WIN32
 char *fname = gptopc(&pv->gp, gptfu_noRootArt|gptfu_noName);
	word *tmpwc = get_pcw_filename(fname, add_end_asterisk);
	pv->OS.dh = FindFirstFileW(tmpwc, &pv->OS.fid);
	xfree(tmpwc);
	xfree(fname);
#else
	pv->OS.dh = opendir(pv->gp.path);
#endif
	if (!pv->OS.dh) {
	    TDirListNotify_DodllEv(self, NULL, dlbef_DelSelf);
	    Tfree(self);
#ifdef TEST_NOTIFY_TIME
	    test_notify_us += GetCurTimeDiv_us(t);
#endif
	    return False;
	}

	dfl = self->dl->dfl;
	while (dfl) {
	    dfl->nymk |= dflnymk_DelMk;
	    dfl = dfl->next;
	}
	return True;
}

bool nypv_ScanDir(TDirListNotify *self) {
  Tprivate *pv = self->private;
  TDirFileList *dfl;
  dword us;
  char *pc;

	us = GetTime_us();
#ifdef _OS_WIN32
	while (pv->OS.dh) {
	    pc = pwctoUTF8(pv->OS.fid.cFileName);
	    if ((pc[0] != '.') || (pc[1] != 0)) {
		pv->gp.name = pc;
		nypv_ChkFile(self);
		pv->gp.name = NULL;
	    }
	    xfree(pc);
	    if (!FindNextFileW(pv->OS.dh, &pv->OS.fid)) {
		FindClose(pv->OS.dh);
		pv->OS.dh = NULL;
	    } else {
		if (GetCurTimeDiv_us(us) > APP_SYNC_TIME) {
		    TDirListNotify_SyncEv(self);
		    return False;
		}
	    }
	}
#else
	while (pv->OS.dh) {
  struct dirent64 entry;
  struct dirent64 *ret_entry;
	    readdir64_r(pv->OS.dh, &entry, &ret_entry);
	    if (ret_entry) {
		pc = ret_entry->d_name;
		if ((pc[0] != '.') || (pc[1] != 0)) {
		    pv->gp.name = pc;
		    nypv_ChkFile(self);
		    pv->gp.name = NULL;
		    if (GetCurTimeDiv_us(us) > APP_SYNC_TIME) {
			TDirListNotify_SyncEv(self);
#ifdef TEST_NOTIFY_TIME
			test_notify_us += GetCurTimeDiv_us(t);
#endif
			return False;
		    }
		}
	    } else {
		closedir(pv->OS.dh);
		pv->OS.dh = NULL;
	    }
	}
#endif
	dfl = self->dl->dfl;
	while (dfl) {
	    if (dfl->nymk & dflnymk_DelMk) {
		dfl = TDirListNotify_Del(self, dfl);
	    } else {
		dfl = dfl->next;
	    }
	}
	return True;
}

