/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "tdirlist.h"
#include <sys/stat.h>

static int TDirList_CpF_Directory(TDirFileList *dfl1, TDirFileList *dfl2) {
  int m1, m2;

	if (pccmp(dfl1->fn, dfl2->fn)) {
	    return 0;
//	} else if (pccmp(dfl1->fn, "..")) {
	} else if (dfl1->fn && (*(const word*)dfl1->fn == 0x2e2e) && !dfl1->fn[2]) {
	    return -1;
//	} else if (pccmp(dfl2->fn, "..")) {
	} else if (dfl2->fn && (*(const word*)dfl2->fn == 0x2e2e) && !dfl2->fn[2]) {
	    return 1;
	}
	m1 = m2 = 0;
	if (dfl1->st->st & st_Dir) m1 = 1;
	if (dfl2->st->st & st_Dir) m2 = 1;
	return m2 - m1;
}

static int TDirList_CpS(const char *s1, const char *s2, int fu) {
	if (fu & dlsortFu_Case) return pccasecmp_fi(s1, s2, 0); 
	return pccmp_i(s1, s2);
}

static int TDirList_CpF_Name_UP(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1;
	m1 = TDirList_CpF_Directory(dfl1, dfl2);
	if (!m1) m1 = TDirList_CpS(dfl1->fn, dfl2->fn, fu);
	return m1;
}
static int TDirList_CpF_Name_DOWN(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1;
	m1 = TDirList_CpF_Directory(dfl1, dfl2);
	if (!m1) m1 = TDirList_CpS(dfl2->fn, dfl1->fn, fu);
	return m1;
}

static int TDirList_CpF_Size_UP(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1 = TDirList_CpF_Directory(dfl1, dfl2);
	if (m1) return m1;
	if (dfl1->st->size > dfl2->st->size) return 1;
	if (dfl1->st->size < dfl2->st->size) return -1;
	return TDirList_CpS(dfl1->fn, dfl2->fn, fu);
}
static int TDirList_CpF_Size_DOWN(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1 = TDirList_CpF_Directory(dfl1, dfl2);
	if (m1) return m1;
	return TDirList_CpF_Size_UP(dfl2, dfl1, fu);
}

static int TDirList_CpF_Type_UP(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1 = TDirList_CpF_Directory(dfl1, dfl2);
	if (m1) return m1;
	m1 = pccasecmp_fi(GetArtExt_pc(dfl1->st), GetArtExt_pc(dfl2->st), 0);
	if (m1) return m1;
	m1 = pccasecmp_fi(GetFileExt_pc(dfl1->fn), GetFileExt_pc(dfl2->fn), 0);
	if (m1) return m1;
	return TDirList_CpS(dfl1->fn, dfl2->fn, fu);
}
static int TDirList_CpF_Type_DOWN(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1 = TDirList_CpF_Directory(dfl1, dfl2);
	if (m1) return m1;
	return TDirList_CpF_Type_UP(dfl2, dfl1, fu);
}

static int TDirList_CpF_Date_UP(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1 = TDirList_CpF_Directory(dfl1, dfl2);
	if (m1) return m1;
	if (dfl1->st->mtime > dfl2->st->mtime) return 1;
	if (dfl1->st->mtime < dfl2->st->mtime) return -1;
	return TDirList_CpS(dfl1->fn, dfl2->fn, fu);
}
static int TDirList_CpF_Date_DOWN(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1 = TDirList_CpF_Directory(dfl1, dfl2);
	if (m1) return m1;
	return TDirList_CpF_Date_UP(dfl2, dfl1, fu);
}

#define MaxSortProc	20
static void *dl_SortProcArr[MaxSortProc] = {
	TDirList_CpF_Name_UP, TDirList_CpF_Name_DOWN,	//Name
	TDirList_CpF_Size_UP, TDirList_CpF_Size_DOWN,	//Size
	TDirList_CpF_Type_UP, TDirList_CpF_Type_DOWN,	//Type
	TDirList_CpF_Date_UP, TDirList_CpF_Date_DOWN,	//Date
	NULL, NULL,		//Permisson
	NULL, NULL,		//User
	NULL, NULL,		//Group
//	TDirList_CpF_Link_UP, TDirList_CpF_Link_DOWN,
	NULL, NULL,		//Link
	NULL, NULL,
	NULL, NULL
};

bool TDirFileList_ChkSortOk(int fu) {
  int i;
	i = fu & 0xff;
	return ((i < MaxSortProc) && (dl_SortProcArr[i] != NULL));
}

void TDirFileList_Sort(TDirFileList **ppdfl, int fu) {
  TDirFileList *dfl;
  int i;
  int count = 0;
  TDirFileList **pptmpdfl, **ppfln, **ppfls;

	if (!ppdfl) return;
//	self->dfl_sort = NULL;
	count = 0;
	dfl = *ppdfl;
	while (dfl) {
	    count++;
	    dfl = dfl->next;
	}
// printf("%s(%d): count=%d\n", FL, count);
	if (count < 2) return;

	pptmpdfl = xmalloc(sizeof(TDirFileList*) * count);
	ppfln = pptmpdfl;
	dfl = *ppdfl;
	while (dfl) {
	    *ppfln = dfl;
	    ppfln++;
	    dfl = dfl->next;
	}
	i = fu & 0xff;
	if ((i < MaxSortProc) && dl_SortProcArr[i]) {
	    QuickSort_PL((char**)pptmpdfl, 0, count-1, fu, dl_SortProcArr[i]);
	}

	ppfls = ppdfl; //&self->dfl_sort;
	ppfln = pptmpdfl;
	for (;count;count--) {
	    *ppfls = *ppfln;
	    ppfls = &(*ppfls)->next;
	    ppfln++;
	}
	*ppfls = NULL;
	xfree(pptmpdfl);
	return;
}
