/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"
#include "xtty.h"

// #define ttyscn_DEBUG	1

#ifdef	ttyscn_DEBUG
void tty_err(byte c, TTermscan *term) {
  int i;
	fprintf(stderr, "\nError ttyscan: l=%d %d \"%02x",term->reslg, term->laststate, term->res[0]);
	for (i=1; i<term->reslg; i++) {
	    fprintf(stderr, "%02x", term->res[i]);
	}
	fprintf(stderr, "\"\n");
}
#endif

static bool tty_scan_esc(byte c, TTermscan *term) {
  bool err = False;

	term->state = 0;
	switch(c) {
	case '[': term->state = tty_square; break;
	case '(': term->state = tty_setG0; break;
	case ')': term->state = tty_setG1; break;

	default:
	    if (term->termart == term_xterm) {
		switch(c) {
		case 'O': term->state = tty_esc_O; break;
		default:
		    err = True;
		    break;
		}

	    } else if (term->termart == term_xterm) {
		err = True;

	    } else {
		err = True;
	    }
	    break;
	}
	return err;
}

static bool tty_scan_esc_O(byte c, TTermscan *term) {
  bool err = False;

	term->state = 0;
	if (term->termart == term_xterm) {
	    switch(c) {
	    case 'P': term->btty_XK = yXK_F1; break;
	    case 'Q': term->btty_XK = yXK_F2; break;
	    case 'R': term->btty_XK = yXK_F3; break;
	    case 'S': term->btty_XK = yXK_F4; break;

	    default: err = True; break;
	    }
	} else {
	    err = True;
	}
	return err;
}

static bool tty_scan_gotpars(byte c, TTermscan *term) {
  bool err = False;

// printf("tty_scan_gotpars: %d %d\n", term->numIdx, term->nums[0]);
	term->state = 0;
	if (term->termart == term_linux) {
	    switch(c) {
	    case '@': term->btty_XK = yXK_Insert; break;
	    case 'A': term->btty_XK = yXK_Up; break;
	    case 'B': term->btty_XK = yXK_Down; break;
	    case 'C': term->btty_XK = yXK_Right; break;
	    case 'D': term->btty_XK = yXK_Left; break;
	    case 0x7e: 
		switch(term->nums[0]) {
		case 3: term->btty_XK = yXK_Delete; break;
		case 5: term->btty_XK = yXK_Page_Up; break;
		case 6: term->btty_XK = yXK_Page_Down; break;
		case 17: term->btty_XK = yXK_F6; break;
		case 18: term->btty_XK = yXK_F7; break;
		case 19: term->btty_XK = yXK_F8; break;
		case 20: term->btty_XK = yXK_F9; break;
		case 21: term->btty_XK = yXK_F10; break;
		case 23: term->btty_XK = yXK_F11; break;
		case 24: term->btty_XK = yXK_F12; break;
		default: err = True; break; }
		break;
	    default: err = True; break; }


	} else if (term->termart == term_xterm) {
	    switch(c) {
	    case '@': term->btty_XK = yXK_Insert; break;
	    case 'A': term->btty_XK = yXK_Up; break;
	    case 'B': term->btty_XK = yXK_Down; break;
	    case 'C': term->btty_XK = yXK_Right; break;
	    case 'D': term->btty_XK = yXK_Left; break;

	    case 0x7e: 
		switch(term->nums[0]) {
		case 3: term->btty_XK = yXK_Delete; break;
		case 5: term->btty_XK = yXK_Page_Up; break;
		case 6: term->btty_XK = yXK_Page_Down; break;
		case 15: term->btty_XK = yXK_F5; break;
		case 17: term->btty_XK = yXK_F6; break;
		case 18: term->btty_XK = yXK_F7; break;
		case 19: term->btty_XK = yXK_F8; break;
		case 20: term->btty_XK = yXK_F9; break;
		case 21: term->btty_XK = yXK_F10; break;
		case 23: term->btty_XK = yXK_F11; break;
		case 24: term->btty_XK = yXK_F12; break;
		default: err = True; break; }
		break;
	    default: err = True; break; }
	} else {
	    err = True;
	}
	return err;
}

static bool tty_scan_funckey(byte c, TTermscan *term) {
  bool err = False;

// printf("tty_scan_funckey: %d %d\n", term->numIdx, term->nums[0]);
	term->state = 0;
	if (term->termart == term_linux) {
	    switch(c) {
	    case 'A': term->btty_XK = yXK_F1; break;
	    case 'B': term->btty_XK = yXK_F2; break;
	    case 'C': term->btty_XK = yXK_F3; break;
	    case 'D': term->btty_XK = yXK_F4; break;
	    case 'E': term->btty_XK = yXK_F5; break;
	    default: err = True; break; }
	} else if (term->termart == term_xterm) {
	    err = True;
	} else {
	    err = True;
	}
	return err;
}


bool term_scan(byte c, TTermscan *term) {
  int i;
  bool err = False;

	i = GetTime_ms();
	if (cpTime(term->time_ms, i)) term->state = 0;
	term->time_ms = i + 100;

	term->laststate = term->state;
loop:
	if (term->state) {
	    if (term->state == tty_esc) {
		err = tty_scan_esc(c, term);

	    } else if (term->state == tty_square) {
		for (i=0; i<8; i++) term->nums[i] = 0;
		term->numIdx = 0;
		term->state = tty_getnums;
		if (c == '[') {
		    term->state = tty_funckey;
		} else {
		    term->ques = (c == '?');
		    if (!term->ques) goto loop;
		}

	    } else if (term->state == tty_getnums) {
//	fprintf(stderr, "tty_getnums\n");
		if ((c >= '0') && (c <= '9')) {
		    term->nums[term->numIdx] *= 10;
		    term->nums[term->numIdx] += c - 0x30;
		} else if (c == ';') {
		    if (term->numIdx < 7) term->numIdx++;
		} else {
		    term->state = tty_gotpars;
		    goto loop;
		}

	    } else if (term->state == tty_gotpars) {
		err = tty_scan_gotpars(c, term);

	    } else if (term->state == tty_funckey) {
		err = tty_scan_funckey(c, term);

	    } else if (term->state == tty_setG0) {
		term->state = 0;
		err = True;

	    } else if (term->state == tty_setG1) {
		term->state = 0;
		err = True;

	    } else if (term->state == tty_esc_O) {
		err = tty_scan_esc_O(c, term);

	    } else {
		err = True;
	    }
	} else {
	    term->reslg = 0;
	    term->btty_XK = 0;
	    switch(c) {
	    case 0x0A: term->btty_XK = yXK_Return; break;
	    case 0x0D: term->btty_XK = yXK_Home; break;
	    case 0x1B: term->state = tty_esc; break;
	    case 0x9B: term->state = tty_square; break;
	    case 0x08: term->btty_XK = yXK_BackSpace; break;
	    case 0x7f: term->btty_XK = yXK_BackSpace; break;
	    default:
		term->btty_XK = c;
		break;
	    }
	}

	term->res[term->reslg] = c;
	term->reslg++;
	term->res[term->reslg] = 0;
	if ((term->reslg > 7) || err) {
#ifdef	ttyscn_DEBUG
	    tty_err(c, term);
#endif
	    term->state = 0;
	}
	term->err = err;
	return (term->state == 0);
}

void TTermscan_Destroy(TTermscan *term) {
	if (!term) return;
	xfree(term);
}

TTermscan *TTermscan_Create() {
  TTermscan *term = xcalloc(sizeof(TTermscan));
  char *s;

	term->time_ms = GetTime_ms() + 100;
	s = getenv("TERM");
	if (s && !pccasecmpASCII(s, "xterm")) term->termart = term_xterm; 
	return term;
}
