/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "xcopy_pv.h"

int xcyChkDest(TXCopy_ld *src, TXCopy_ld *dest, int skip, 
		int *msgOk_skip, int *msg_nr) {

  int src_ifmt, dest_ifmt, result;
  Tstat64 *src_st, *dest_st;
  time_t src_mtime, dest_mtime;
  bool oldf;
  bool slink;

	result = 0;
	src_st = src->st;
	dest_st = dest->st;
	if (!dest_st) {
	    if (skip & xcySkip_DestNotExist) {
		result = xcyChkDest_Skip;
		if ((src_st->mode & S_IFMT) == S_IFDIR) result = xcyChkDest_Skip | xcyChkDest_NoDirCreate;
	    }
// printf("%s(%d): 0x%03x %s%s\n", FL, result, src->gp.path, src->gp.name);
	    return result;
	}

	oldf = False;
	if (skip & xcySkip_OldFile) {
	    src_mtime  = src_st->mtime;
	    dest_mtime = dest_st->mtime;
	/* test ftp time */
	    if ((src->art == ldart_ftp) || (dest->art == ldart_ftp)) {
	/* set ftp time sec to 0 */
		src_mtime -= src_mtime % 60;
		dest_mtime -= dest_mtime % 60;
	/* test ftp time day only --> time hour,min to 0 */
		if (!(src_mtime % 86400) || !(dest_mtime % 86400)) {
		    src_mtime -= src_mtime % 86400;
		    dest_mtime -= dest_mtime % 86400;
		}
	    }
	    oldf = (src_mtime <= dest_mtime);
	}

	slink = (skip & xcyForce_followlinks) == 0;
	src_ifmt = src_st->mode & S_IFMT;
#ifndef _OS_WIN32
	if (slink && (src_st->st & st_Link)) src_ifmt = S_IFLNK;
#endif
	dest_ifmt = dest_st->mode & S_IFMT;
#ifndef _OS_WIN32
	if (slink && (dest_st->st & st_Link)) dest_ifmt = S_IFLNK;
#endif

	if (src_ifmt != dest_ifmt) {
	    if (dest_ifmt == S_IFDIR) {
		*msgOk_skip = xcySkip_OvwTypeDesrDir;
		*msg_nr = xpyErr_Alg | xpyErr_OvwTypeDestDir;
		return xcyChkDest_MsgSkip;
	    }
	    if (skip & xcyForce_ovwType) {
		return xcyChkDest_DelDest;
	    }
	    *msgOk_skip = xcySkip_OverWriteType;
	    *msg_nr = xpyErr_Alg | xpyErr_OverWriteType;
	    return xcyChkDest_MsgSkip;

	} else {

	    switch (src_ifmt) {

	    case S_IFDIR:
		if (oldf) return xcyChkDest_Skip | xcyChkDest_bSkipDir;
		if (skip & xcyForce_ovwDir) return xcyChkDest_Preserv | xcyChkDest_bSkipDir;
		*msgOk_skip = xcySkip_ovwDir;
		*msg_nr = xpyMsg_ovwDir;
		return xcyChkDest_MsgSkip;

	    case S_IFREG:
		if (oldf) return xcyChkDest_Skip;
		if (skip & xcyForce_ovwFile) return 0;
		*msgOk_skip = xcySkip_ovwFile;
		*msg_nr = xpyMsg_ovwFile;
		return xcyChkDest_MsgSkip;

#ifndef _OS_WIN32
	    case S_IFLNK:
		if (pccmp(Get_ExFileDat_pc(src_st, exfdt_LinkName),
			  Get_ExFileDat_pc(dest_st, exfdt_LinkName))) {
		    return xcyChkDest_Skip;
		}
		return xcyChkDest_DelDest;
/*	    case S_IFIFO:
		break; */

	    case S_IFCHR:
	    case S_IFBLK:
	    case S_IFSOCK:
		if (src_st->rdev == dest_st->rdev) return 0;
		if (skip & xcyForce_ovwFile) return xcyChkDest_DelDest;
		*msgOk_skip = xcySkip_ovwFile;
		*msg_nr = xpyMsg_ovwFile;
		return xcyChkDest_MsgSkip;
#endif
	    }
	}
	return 0;
}
