/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "tdirlist.h"
#include "xcopy_pv.h"
#include <utime.h>
#include <fcntl.h>
#if USE_MMAP
 #include <sys/mman.h>
#endif

static bool Get_Dat_RD(TXCopy_wd *wd, int *lg, int *fuEOL, TXCopy_ld *dest) {
  TXCopy_ld *src;
  Tlgs lgs;
  bool res;

	src = wd->xcy_src;
	if (!src) {
	   if (fuEOL) *fuEOL = fio_EOF;
	   return false;
	}
	TThread_lock(src->thd);
	dest = wd->xcy_dest;
	res = TLineBuf_GetData_lgs(src->lb, &lgs, dest->buf_size, fuEOL);
	if (res) {
	    pnmove(dest->buf, lgs.s, lgs.lg);
	    *lg = lgs.lg;
	}
	TThread_unlock(src->thd);
	return res;
}

static int Chk_Dat_RD(TXCopy_wd *wd) {
  TXCopy_ld *src;
  int res = 0;

	src = wd->xcy_src;
	if (!src) return ChkDat_EOF; //0x02;

	TThread_lock(src->thd);
	if (src->lb->lg > 0) res = ChkDat_DataOk; //0x01;
	if (src->lb->eof) res |= ChkDat_EOF; //0x02;
	TThread_unlock(src->thd);
	return res;
}

/*_____________________________ Thread RD _______________________________*/
static bool dev_open_RD(TXCopy_wd *wd) {
  TXCopy_ld *ld = wd->xcy_src;
	if (!ld) return false;
	if (file_open_rd_OS(ld->fnameOS, NULL, &ld->fd)) return true;
	ppcchange(&ld->msg_s, SysError_pc());
	return false;
}

static bool dev_close_RD(TXCopy_wd *wd, TXCopy_ld *src) {
	file_close(src->fd, NULL);
	src->fd = 0;
	return true;
}

static int send_Mt(void *obj, int RunStat, int MsgSt) {
	if (RunStat == thdMSG_XCopy_RunStat) {
	    thd2app_MSG(obj, true, RunStat, MsgSt);
	    return 0;
	}
	thd2app_MSG(obj, true, RunStat, 0);
	return 0;
}

static void RD_pth_Init(TThread *thd, TXCopy_wd *wd) {
	step_Init_RD(thd, wd);
}

static void RD_pth_ChkStart(TThread *thd, TXCopy_wd *wd) {
	step_ChkStart_RD(thd, wd);
}

static void RD_pth_Start(TThread *thd, TXCopy_wd *wd) {
	step_Start_RD(thd, wd);
}

static bool RD_pth_Move(TThread *thd, TXCopy_wd *wd, TXCopy_ld *src) {
  bool rdWait = false;

	TThread_lock(src->thd);
	TLineBuf_fd_Add(src->lb, src->fd);
	if (src->lb->eof) {
	    TThread_unlock(src->thd);
	    file_close(src->fd, NULL);
	    src->fd = 0;
	    src->srt = srt_ok;
	    if (wd->mode & cpwdMode_cut) src->srt = srt_del_wait;
	} else {
	    int lgo = TLineBuf_sync0_mem(src->lb);
	    rdWait = (lgo + src->lb->lg) > wd->MaxBufSize;
	    TThread_unlock(src->thd);
	}
	return rdWait;
}

static int TXCopy_ld_File_Thread_RD_DoExecute(TThread *thd) {
  int w_ret;
  TXCopy_ld *ld = thd->tagp;
  TXCopy_wd *wd = ld->wd;
  bool rdWait = true;

// printf("%s(%d): Thread_RD_DoExecute\n", FL);

loop:
	if (!rdWait && (ld->fd > 0) && ((ld->srt & srt_bef_mask) == srt_move)) {
	    thd->fd = ld->fd;
	    w_ret = TThread_Wait(thd, 4000);
	} else {
	    rdWait = true;
	    thd->fd = 0;
	    w_ret = TThread_Wait(thd, 4000);
	}
// if (w_ret) printf("%s(%d): w_ret=0x%04x thdMSG=0x%04x tag=0x%04x\n", FL, w_ret, thd->msg.thdMSG, thd->msg.tag);

	if (w_ret == thdWait_msg) {
	    switch (thd->msg.thdMSG & thdMSG_Mask) {
	    case thdMSG_CloseProcess:
		goto ende;

	    case thdMSG_Init:
		if (!MSG_Init_RD(wd, thd->msg.tag, ld->fd > 0)) {
 printf("%s(%d): %7d pth RD Init Internal ERROR !!!\n", FL, GetCurTimeDiv_us(wd->us));
		    thd2app_MSG(thd, true, thdMSG_XCopy_RunStat, MsgSt_err);
		}
		break;

	    case thdMSG_Close:
		if ((ld->srt & srt_bef_mask) != srt_0) {
// printf("%s(%d): %7d pth RD Close fd=%d isend=%d\n", FL, GetCurTimeDiv_us(wd->us), ld->fd, ld->init_send);
// printf("%s(%d): %7d pth RD Close mode=0x%x srt=%d msg.tag=0x%x\n", FL, GetCurTimeDiv_us(wd->us), wd->mode, ld->srt, thd->msg.tag);

		    if ((wd->mode & cpwdMode_cut) && 
			(ld->srt == srt_del_wait) &&
			(thd->msg.tag == MsgSt_close)) {
			file_close(ld->fd, NULL);
			ld->fd = 0;
			ld->srt = srt_del;
		    } else {
			goto bef_close_srt_0;
		    }
		}
		break;
	    }

	} else if (w_ret == thdWait_fd) {
// printf("%s(%d): pthdWait_fd\n", FL);
	    rdWait = RD_pth_Move(thd, wd, ld);
	}

bef_loop:
	if ((ld->srt & srt_bef_mask) == srt_init) {
// printf("%s(%d): srt_init\n", FL);
	    RD_pth_Init(thd, wd);
	}

	if ((ld->srt & srt_bef_mask) == srt_ChkStart) {
// printf("%s(%d): srt_ChkStart\n", FL);
	    RD_pth_ChkStart(thd, wd);
	}

	if ((ld->srt & srt_bef_mask) == srt_Start) {
// printf("%s(%d): srt_Start\n", FL);
	    RD_pth_Start(thd, wd);
	}

	if ((ld->srt & srt_bef_mask) == srt_move) {
// printf("%s(%d): srt_move %d %d\n", FL, rdWait, (int)ld->fd);
	    if (rdWait && (ld->fd > 0)) {
		TThread_lock(ld->thd);
		int lgo = TLineBuf_sync0_mem(ld->lb);
		rdWait = (lgo + ld->lb->lg) > wd->MaxBufSize;
		TThread_unlock(ld->thd);
	    }
	}

	if ((ld->srt & srt_bef_mask) == srt_del) {
	    step_Del_RD(wd);
	}

	if ((ld->srt & srt_bef_mask) == srt_ok) {
 if (ld->fd > 0)
 printf("%s(%3d): %7d RD OK\n", FL, GetCurTimeDiv_us(wd->us));
// if (!ld->init_send)
// printf("%s(%3d): %7d RD OK no init_send ???\n", FL, GetCurTimeDiv_us(wd->us));
	    if (ld->init_send) {
		ld->init_send = false;
		thd2app_MSG(thd, true, thdMSG_XCopy_RunStat, MsgSt_close);
	    }
	    ld->srt = srt_0;
	}

	if ((ld->srt & srt_bef_mask) == srt_skip) {
bef_close_srt_0:
	    ld->srt = srt_0;
	    file_close(ld->fd, NULL);
	    ld->fd = 0;
	    thd2app_MSG(thd, true, thdMSG_XCopy_RunStat, MsgSt_skip);
	}

	if (ld->srt & srt_msg_wait) {
	    ld->srt &= ~srt_msg_wait;

// printf("%s(%d): %7d RD srt_msg_wait=0x%05x \n", FL, GetCurTimeDiv_us(wd->us), ld->srt);
	    int ret = thd2app_MSG_Wait(thd, thdMSG_XCopy, 0, NULL);
// printf("%s(%d): srt_msg_wait ret=0x%02x\n", FL, ret);
	    if (ret & xcyRetEv_BefMask) {
		if (ret & xcyRetEv_ALL) wd->skip |= ld->msgOk_skip;
		ld->skip |= wd->skip;
		ld->skip |= ld->msgOk_skip;
		ret &= xcyRetEv_BefMask;
		if (ret == xcyRetEv_SKIP) ld->srt = srt_skip;

// printf("%s(%d): %7d RD skip wd=0x%06x ld=0x%06x\n", FL, GetCurTimeDiv_us(wd->us), wd->skip, ld->skip);
		goto bef_loop;

	    } else {
		ld->srt = srt_0;
		thd2app_MSG(thd, true, thdMSG_XCopy_RunStat, MsgSt_err);
	    }
	}

	goto loop;
ende:
	file_close(ld->fd, NULL);
	ld->fd = 0;

#if 0
 float a;
 float fa = wd->xcs.GesDataSize;
 float b = wd->xcs.GesDataTime;
 float c = GetTime_us64() - wd->ges_us;
	fa /= 0x100000;
	b /= 1000000;
	c /= 1000000;
	if (b > 0) a = fa / b; else a = 0;
	if (c > 0) b = fa / c; else b = 0;
 printf("%s(%d): %7d RD Thread Stop %u %u %fMB/s %fMB/s\n", FL, GetCurTimeDiv_us(wd->us), (dword)wd->xcs.GesDataSize, (dword)wd->xcs.GesDataTime, a, b);
#endif
	return 0;
}

static bool Init_RD(TXCopy_wd *wd, int funkt) {
  TXCopy_ld *ld = wd->xcy_src;

	if (!ld || !TThread_Start(ld->thd)) return false;
	app2thd_MSG(ld->thd, true, thdMSG_Init, funkt);
	return true;
}

static bool Close_RD(TXCopy_wd *wd, int funkt) {
  TXCopy_ld *ld = wd->xcy_src;

	if (!ld || !TThread_Start(ld->thd)) return false;
	app2thd_MSG(ld->thd, true, thdMSG_Close, funkt);
	return true;
}

static void Clear_RD(TXCopy_wd *wd) {
  TXCopy_ld *src = wd->xcy_src;
	xfree(src->st);
	src->st = NULL;
	xcyld_FreeFileName(src);
//	xfree(src->fname);
//	src->fname = NULL;
}

static bool set_gp_RD(TXCopy_ld *xcy, TglobPath *gp) {
	if (!gp_test_local_drive(gp)) return false;
	ppfree(&xcy->st);
//	ppcfree(&xcy->fname);
	xcyld_FreeFileName(xcy);
	ppcchange_cmp(&xcy->gp.path, gp->path);
	ppcchange(&xcy->gp.name, gp->name);
	return true;
}
/*_____________________________ Thread WR _______________________________*/
static void WR_inc_FileCount(TXCopy_wd *wd, dword size, uint64_t *t64) {
	wd->xcs.FileCount++;
	wd->xcs.GesDataSize += size;
	wd->xcs.GesDataTime += GetTime_us64() - *t64;
}

static void WR_pth_Init(TThread *thd, TXCopy_wd *wd) {
  TXCopy_ld *src, *dest;
#ifndef _OS_WIN32
  const char *pc;
#endif
  int xcyChk;
  int src_ifmt;
  uint64_t t64;

	src = wd->xcy_src;
	dest = wd->xcy_dest;

	if (!step_Init_WR(thd, wd, &xcyChk, &src_ifmt)) goto err_skip;

	switch (xcyChk & xcyChkDest_BefMask) {

	case xcyChkDest_DelDest:
	    if (file_unlink_OS(dest->fnameOS, NULL)) {
		xfree(dest->st);
		dest->st = NULL;
	    } else {
		dest->msgOk_skip = xcySkip_errDelFile;
		dest->msg_nr = xpyErr_Alg | xpyErr_DelFile;
		goto err_sys_skip;
	    }
	    break;

	case xcyChkDest_MsgSkip:
	    goto err_skip;

	case xcyChkDest_Skip:
	    dest->srt = srt_skip;
	    return;

	case xcyChkDest_Preserv:
	    dest->srt = srt_preserv;
	    return;
	}

	t64 = GetTime_us64();

	switch (src_ifmt) {

	case S_IFDIR:
	    dest->stmode = wd->mode_dir;
	    if (file_mkdir_OS(dest->fnameOS, NULL, dest->stmode) < 0) {
		dest->msgOk_skip = xcySkip_errMkDir;
		dest->msg_nr = xpyErr_Alg | xpyErr_MkDir;
		wd->xcyChk |= xcyChkDest_NoDirCreate;
		goto err_sys_skip;
	    }
	    add_dest_dir(wd, dest->fname);
	    WR_inc_FileCount(wd, src->st->size, &t64);
	    dest->srt = srt_preserv;
	    return;

	case S_IFREG:
	    dest->atime = 0;
	    dest->mtime = 0;
	    if (!dest->st) dest->stmode = wd->mode_file;
	    dest->srt = srt_open;
	    return;

#ifndef _OS_WIN32
	case S_IFLNK:
	    if (!dest->st) {
		pc = Get_ExFileDat_pc(src->st, exfdt_LinkName);
		if (pc && *pc && (symlink(pc, dest->fname) < 0)) {
		    dest->msgOk_skip = xcySkip_errMkLink;
		    dest->msg_nr = xpyErr_Alg | xpyErr_MkLink;
		    goto err_sys_skip;
		}
		WR_inc_FileCount(wd, src->st->LinkSize, &t64);
	    }
	    dest->srt = srt_ok;
	    return;

	case S_IFIFO:
	    if (!dest->st) {
		dest->stmode = wd->mode_blk;
		if (mkfifo(dest->fname, dest->stmode | S_IFIFO) < 0) {
		    dest->msgOk_skip = xcySkip_errMkNod;
		    dest->msg_nr = xpyErr_Alg | xpyErr_MkNod;
		    goto err_sys_skip;
		}
		WR_inc_FileCount(wd, 0, &t64);
	    }
	    dest->srt = srt_preserv;
	    return;

	case S_IFCHR:
	case S_IFBLK:
	case S_IFSOCK:
	    if (!dest->st) {
		dest->stmode = wd->mode_blk;
		if (mknod(dest->fname, dest->stmode | (src->st->mode & S_IFMT), src->st->rdev) < 0) {
		    dest->msgOk_skip = xcySkip_errMkNod;
		    dest->msg_nr = xpyErr_Alg | xpyErr_MkNod;
		    goto err_sys_skip;
		}
		WR_inc_FileCount(wd, 0, &t64);
	    }
	    dest->srt = srt_preserv;
	    return;
#endif
	}

	dest->srt = srt_ok | srt_msg_wait;
	dest->msgOk_skip = xcySkip_errType;
	dest->msg_nr = xpyErr_Alg | xpyErr_Type;
	goto err_sys_skip;

err_sys_skip:
	ppcchange(&dest->msg_s, SysError_pc());
	dest->msg_nr |= xpyErr_Msg;
err_skip:
	dest->srt |= srt_msg_wait;
	if (dest->skip & dest->msgOk_skip) dest->srt = srt_skip;
}

static bool dev_open_WR(TXCopy_wd *wd) {
  TXCopy_ld *ld = wd->xcy_dest;
	if (file_open_wr_OS(ld->fnameOS, NULL, &ld->fd, ld->stmode)) return true;
	ppcchange(&ld->msg_s, SysError_pc());
	return false;
}

static bool dev_close_WR(TXCopy_wd *wd, TXCopy_ld *dest) {
	file_close(dest->fd, NULL);
	dest->fd = 0;
	return true;
}


static bool WR_pth_Move(TThread *thd, TXCopy_wd *wd) {
	return step_Move_WR(thd, wd);
}

static void WR_pth_Preserv(TThread *thd, TXCopy_wd *wd) {
  TXCopy_ld *src, *dest;  
  uint64_t t;

	dest = wd->xcy_dest;
	dest->srt = srt_ok;
	src = wd->xcy_src;
	if (!(dest->skip & xcySkip_Chmod) && (src->stmode != dest->stmode)) {
	    t = GetTime_us64();
	    if (!file_chmod_OS(dest->fnameOS, NULL, src->stmode) && !(wd->skip & xcySkip_errChmod)) {
 printf("%s(%d): Err \"%s\" chmod=%s\n", FL, dest->fname, SysError_pc());
	    }
	    wd->xcs.GesDataTime += GetTime_us64() - t;
	}

#ifndef _OS_WIN32
	if (!(dest->skip & xcySkip_Chown)
	    && ((src->uid != dest->uid)||(src->gid != dest->gid))) {
	    t = GetTime_us64();
	    if ((chown(dest->fname, src->st->uid, src->st->gid) < 0) && !(wd->skip & xcySkip_errChown)) {
    printf("%s(%d): Err chown=%s\n", FL, SysError_pc());
	    }
	    wd->xcs.GesDataTime += GetTime_us64() - t;
	}
#endif

#ifdef _OS_WIN32
	if (!(dest->skip & xcySkip_Utime) && !(src->st->st & st_Dir)
#else
	if (!(dest->skip & xcySkip_Utime)
#endif
	    && ((src->atime != dest->atime)||(src->mtime != dest->mtime))) {
	    t = GetTime_us64();
	    if ((file_utime_OS(dest->fnameOS, NULL, src->atime, src->mtime) < 0) 
		&& !(wd->skip & xcySkip_errUtime)) {
    printf("%s(%d): Err utime=%s\n", FL, SysError_pc());
	    }
	    wd->xcs.GesDataTime += GetTime_us64() - t;
	}
}

static int TXCopy_ld_File_Thread_WR_DoExecute(TThread *thd) {
  int w_ret;
  TXCopy_ld *ld = thd->tagp;
  TXCopy_wd *wd = ld->wd;
  bool wrWait = true;

// printf("%s(%d): %7d pth WR Thread Start 0x%x\n", FL, GetCurTimeDiv_us(wd->us), (int)thd);

loop:
	thd->fd = 0;
	if (!wrWait && (ld->fd > 0) && ((ld->srt & srt_bef_mask) == srt_move)) {
	    thd->fd = ld->fd;
	}
	w_ret = TThread_Wait(thd, 4000);
	if (w_ret == thdWait_msg) {
	    switch (thd->msg.thdMSG & thdMSG_Mask) {
	    case thdMSG_CloseProcess:
		goto ende;

	    case thdMSG_Init:
		if (!MSG_Init_WR(thd, wd, thd->msg.tag, ld->fd > 0)) {
 printf("%s(%d): pth WR Init Internal ERROR !!!\n", FL);
		}
		break;

/*	    case thdMSG_Open:
		break; */

	    case thdMSG_Close:
//		if ((ld->srt & srt_bef_mask) != srt_0) { Skip old File not work !!!
// printf("%s(%d): %7d pth WR Close\n", FL, GetCurTimeDiv_us(wd->us));
		    ld->srt = srt_skip;
//		}
		break;
	    }

	} else if (w_ret == thdWait_fd) {
	    wrWait = WR_pth_Move(thd, wd);
	}


Bef_loop:
	if ((ld->srt & srt_bef_mask) == srt_init) {
	    WR_pth_Init(thd, wd);
	}

	if ((ld->srt & srt_bef_mask) == srt_open) {
	    if (!step_Open_WR(thd, wd)) ld->srt = srt_0;
	}

	if ((ld->srt & srt_bef_mask) == srt_move) {

	    if (wrWait && (ld->fd > 0)) {
		int m = wd->xcy_src->chk_dat(wd);
		if (m & 0x01) {
		    wrWait = false;
		} else if (m & 0x02) {
		    file_close(ld->fd, NULL);
		    ld->fd = 0;
		    ld->srt = srt_preserv;
		    wrWait = true;
		}
	    }
	}

	if ((ld->srt & srt_bef_mask) == srt_preserv) {
	    WR_pth_Preserv(thd, wd);
	}

	if ((ld->srt & srt_bef_mask) == srt_skip) {
// printf("%s(%3d): %7d WR Skip\n", FL, GetCurTimeDiv_us(wd->us));
	    file_close(ld->fd, NULL);
	    ld->fd = 0;
	    ld->srt = srt_0;
	    thd2app_MSG(thd, true, thdMSG_XCopy_RunStat, MsgSt_skip);
	}

	if ((ld->srt & srt_bef_mask) == srt_ok) {
// printf("%s(%d): %7d WR OK\n", FL, GetCurTimeDiv_us(wd->us));
	    ld->srt = srt_0;
	    thd2app_MSG(thd, true, thdMSG_XCopy_RunStat, MsgSt_close);
	}

	if (ld->srt & srt_msg_wait) {
	    ld->srt &= ~srt_msg_wait;

// printf("%s(%d): %7d WR srt_msg_wait=0x%05x \n", FL, GetCurTimeDiv_us(wd->us), ld->srt);
	    int ret = thd2app_MSG_Wait(thd, thdMSG_XCopy, 0, NULL);
	    if (ret & xcyRetEv_BefMask) {
		if (ret & xcyRetEv_ALL) wd->skip |= ld->msgOk_skip;
		ld->skip |= wd->skip;
		if (wd->stat & xcywdstat_SameFileMk) {
		    ld->skip = 0;
		}
		ld->skip |= ld->msgOk_skip;
		ret &= xcyRetEv_BefMask;
		if (ret == xcyRetEv_SKIP) ld->srt = srt_skip;

// printf("%s(%d): %7d WR skip=0x%05x\n", FL, GetCurTimeDiv_us(wd->us), ld->skip);
		goto Bef_loop;
	    } else {
// printf("%s(%d): %7d WR ERROR srt=0x%05x\n", FL, GetCurTimeDiv_us(wd->us), ld->srt);
		ld->srt = srt_0;
		thd2app_MSG(thd, true, thdMSG_XCopy_RunStat, MsgSt_err);
	    }
	}


	goto loop;
ende:
	file_close(ld->fd, NULL);
	ld->fd = 0;
#if 0
 float a;
 float fa = ld->data_ges_lg;
 float b = ld->data_ges_us;
 float c = GetTime_us64() - wd->ges_us;
	fa /= 0x100000;
	b /= 1000000;
	c /= 1000000;
	if (b > 0) a = fa / b; else a = 0;
	if (c > 0) b = fa / c; else b = 0;
 printf("%s(%d): %7d WR Thread Stop %llu %llu %fMB/s %fMB/s\n", FL, GetCurTimeDiv_us(wd->us), ld->data_ges_lg, ld->data_ges_us, a, b);
#endif
	return 0;
}


static bool Init_WR(TXCopy_wd *wd, int funkt) {
  TXCopy_ld *ld = wd->xcy_dest;

	if (!ld || !TThread_Start(ld->thd)) return false;
	app2thd_MSG(ld->thd, true, thdMSG_Init, funkt);
	return true;
}

static bool dev_WR(TXCopy_wd *wd, byte *data, int lg, int fuEOL) {
  TXCopy_ld *dest = wd->xcy_dest;

	file_write(dest->fd, NULL, data, lg, NULL);
	wd->xcs.DataNowSize += lg;
	wd->xcs.DataNowTime = GetTime_us64() - dest->open_data_us;
	return true;
}

static bool Close_WR(TXCopy_wd *wd, int funkt) {
  TXCopy_ld *ld = wd->xcy_dest;

	if (!ld || !TThread_Start(ld->thd)) return false;
	app2thd_MSG(ld->thd, true, thdMSG_Close, funkt);
	return true;
}

static void Clear_WR(TXCopy_wd *wd) {
  TXCopy_ld *dest = wd->xcy_dest;
	xfree(dest->st);
	dest->st = NULL;
//	xfree(dest->fname);
//	dest->fname = NULL;
	xcyld_FreeFileName(dest);
}

/*_____________________________ AppEv ___________________________________*/
static int TXCopy_ld_WR_OnMSG(TThread *thd, TThread_MSG *msg, TThread_MSG *ret_msg) {
  TXCopy_ld *ld = thd->tagp;
  int bef = msg->thdMSG & thdMSG_Mask;
  TXCopy_wd *wd;

	if (ld && (ld->wtyp == wtyp_TXcopy_ld) && 
	    ld->wd && (ld->wd->wtyp == wtyp_TXcopy)) {
	    wd = ld->wd;

	    if ((bef == thdMSG_XCopy) && wd->hcyEv) {
// printf("%s(%d): %7d WR MSG \n", FL, GetCurTimeDiv_us(wd->us));
		uint64_t t64 = GetTime_us64();
		int res = wd->hcyEv(wd, ld->msg_nr | xpyMsg_dest);
		wd->ges_us += GetTime_us64() - t64;
		return res;

	    } else if (bef == thdMSG_XCopy_RunStat) {

		return handle_Mt_WR(wd, msg->tag);
	    } else {
#ifdef POINTER64BIT
 printf("%s(%d): WR thdMSG=0x%x tag=0x%x u.type=0x%x u.p=0x%lx\n", FL, msg->thdMSG, msg->tag, msg->u.type, (int64_t)msg->u.p);
#else
 printf("%s(%d): WR thdMSG=0x%x tag=0x%x u.type=0x%x u.p=0x%x\n", FL, msg->thdMSG, msg->tag, msg->u.type, (int)msg->u.p);
#endif
	    }

	} else {

#ifdef POINTER64BIT
 printf("%s(%d): WR thdMSG=0x%x tag=0x%x u.type=0x%x u.p=0x%lx\n", FL, msg->thdMSG, msg->tag, msg->u.type, (int64_t)msg->u.p);
#else
 printf("%s(%d): WR thdMSG=0x%x tag=0x%x u.type=0x%x u.p=0x%x\n", FL, msg->thdMSG, msg->tag, msg->u.type, (int)msg->u.p);
#endif
	}
	return 0;
}

static int TXCopy_ld_RD_OnMSG(TThread *thd, TThread_MSG *msg, TThread_MSG *ret_msg) {
  TXCopy_ld *ld = thd->tagp;
  int bef = msg->thdMSG & thdMSG_Mask;
  TXCopy_wd *wd;

	if (ld && (ld->wtyp == wtyp_TXcopy_ld) && 
	    ld->wd && (ld->wd->wtyp == wtyp_TXcopy)) {
	    wd = ld->wd;

	    if ((bef == thdMSG_XCopy) && wd->hcyEv) {
// printf("%s(%d): %7d RD MSG \n", FL, GetCurTimeDiv_us(wd->us));
		uint64_t t64 = GetTime_us64();
		int res = wd->hcyEv(wd, ld->msg_nr | xpyMsg_src);
		wd->ges_us += GetTime_us64() - t64;
		return res;

	    } else if (bef == thdMSG_XCopy_RunStat) {
		return handle_Mt_RD(wd, msg->tag);

	    } else {
#ifdef POINTER64BIT
 printf("%s(%d): RD thdMSG=0x%x tag=0x%x u.type=0x%x u.p=0x%lx\n", FL, msg->thdMSG, msg->tag, msg->u.type, (int64_t)msg->u.p);
#else
 printf("%s(%d): RD thdMSG=0x%x tag=0x%x u.type=0x%x u.p=0x%x\n", FL, msg->thdMSG, msg->tag, msg->u.type, (int)msg->u.p);
#endif
	    }
	} else {

#ifdef POINTER64BIT
 printf("%s(%d): RD thdMSG=0x%x tag=0x%x u.type=0x%x u.p=0x%lx\n", FL, msg->thdMSG, msg->tag, msg->u.type, (int64_t)msg->u.p);
#else
 printf("%s(%d): RD thdMSG=0x%x tag=0x%x u.type=0x%x u.p=0x%x\n", FL, msg->thdMSG, msg->tag, msg->u.type, (int)msg->u.p);
#endif
	}
	return 0;
}


/*_____________________________ Destroy _________________________________*/
static bool TXCopy_ld_File__Destroy(TXCopy_ld *self) {
	Tfree(self->thd);
	file_close(self->fd, NULL);
	gp_pc_free(&self->gp);
	xfree(self->st);
//	xfree(self->fname);
	xcyld_FreeFileName(self);
	xfree(self->msg_s);
	Tfree(self->lb);
	xfree(self->buf);
	return false;
}

/*_____________________________ Create __________________________________*/
static TXCopy_ld *TXCopy_ld_File_Create(int fu, TXCopy_wd *wd, TglobPath *gp) {
  TXCopy_ld *self;

	self = ttc_create(sizeof(TXCopy_ld), wtyp_TXcopy_ld);
	self->destroy = (TDESTROY)TXCopy_ld_File__Destroy;
	self->art = ldart_file;
	self->wd = wd;
	gp_dup_gpdata(&self->gp, gp);

	if (fu & xcyFU_OUTPUT) {
	    if (wd->xcy_src && (wd->xcy_src->art == ldart_file)) {
		self->gp.fu |= gpi_dest;
	    }
	    self->clear = Clear_WR;
	    self->init = Init_WR;
	    self->close = Close_WR;

	    self->send_Mt = send_Mt;
	    self->dev_open = dev_open_WR;
	    self->dev_close = dev_close_WR;
	    self->dev_wr = dev_WR;
	    self->stat_gp = file_stat_gp_net;

	    self->thd = TThread_Create();
	    self->thd->DoExecute = TXCopy_ld_File_Thread_WR_DoExecute;
	    self->thd->OnMsgEvent = TXCopy_ld_WR_OnMSG;
	    self->thd->tagp = self;
	    self->buf_size = 0x2000; //BUFSIZ;
	    self->buf = xmalloc(self->buf_size);
	} else {
	    gp->fu &= ~gpi_dest;
	    self->clear = Clear_RD;
	    self->init = Init_RD;
	    self->close = Close_RD;
	    self->set_gp = set_gp_RD;
	    self->rmdir_gp = file_rmdir_gp;
	    self->unlink_gp = file_unlink_gp;

	    self->send_Mt = send_Mt;
	    self->dev_open = dev_open_RD;
	    self->dev_close = dev_close_RD;

	    self->stat_gp = file_stat_gp_net;

	    self->thd = TThread_Create();
	    self->thd->DoExecute = TXCopy_ld_File_Thread_RD_DoExecute;
	    self->thd->OnMsgEvent = TXCopy_ld_RD_OnMSG;
	    self->thd->tagp = self;
	    self->get_dat = Get_Dat_RD;
	    self->chk_dat = Chk_Dat_RD;
	    self->lb = TLineBuf_Create();
	}

	return self;
}

static bool TXCopy_ld_File_ChkType(TglobPath *gp, bool dest) {
	return gp_test_local_drive(gp);
}

ThXCopy hXCopy_ld_File = {
  TXCopy_ld_File_ChkType,
  TXCopy_ld_File_Create
};
