/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "net_io.h"
#include "xcopy_pv.h"
#include <utime.h>
#include <fcntl.h>
#include <errno.h>

typedef struct {
  TEv_Array *appEv;
  TNetLineBuf *netlb;
  bool IsOpen;
} Tprivate;

static bool Init_RD(TXCopy_wd *wd, int funkt) {
  TXCopy_ld *ld = wd->xcy_src;

//	ld->data_lg = 0;
	ld->srt = srt_0;
	gp_CryptMini16c_enc(&ld->gp);
	return MSG_Init_RD(wd, funkt, false);
}

static bool Close_RD(TXCopy_wd *wd, int funkt) {
  TXCopy_ld *ld = wd->xcy_src;
	if (!ld) return false;
	if ((ld->srt & srt_bef_mask) != srt_0) {
	    ld->srt = srt_skip;
	}
	return true;
}

static void Clear_RD(TXCopy_wd *wd) {
  TXCopy_ld *src = wd->xcy_src;
	xfree(src->st);
	src->st = NULL;
	xfree(src->fname);
	src->fname = NULL;
}

static bool set_gp(TXCopy_ld *xcy, TglobPath *gp) {
	if (!ftp_chk_RootArt(gp)) return false;
	if (!pccmp(xcy->gp.URL, gp->URL)) return false;

	ppfree(&xcy->st);
	ppcfree(&xcy->fname);
	ppcchange_cmp(&xcy->gp.path, gp->path);
	ppcchange(&xcy->gp.name, gp->name);
	return true;
}

/*__________________________ RD Data IO _________________________________*/
static void OnData_RD(TNetLineBuf *self, bool end) {
  TXCopy_wd *wd = self->owner;
  TXCopy_ld *ld = wd->xcy_src;

	if (end) {
	    appMutex_unlock();
	    if (self->lb->eof) ld->srt = srt_ok;
	} else {
	    ((Tprivate*)ld->private)->IsOpen = true;
	    appMutex_lock();
	}
}

static void OnFileMsg_RD(TFileMsg *fmsg) {
  TXCopy_wd *wd;
  TXCopy_ld *ld;

// file_msg_debug(fmsg, NULL);

	if (!file_msg_chk_wtyp(fmsg, wtyp_TNetLineBuf, FL)) return;
	if (!fmsg->owner) return;
	wd = fmsg->owner;
	ld = wd->xcy_src;

	if (fmsg->art == fmArt_strg) {
	    if (fmsg->D.m == hSHStM_RdOpen) {
		((Tprivate*)ld->private)->IsOpen = true;
// printf("%s(%d): Init_RD  ftp Open 0x%06x\n", FL, wd->wtyp);
	    }
	} else if (fmsg->art == fmArt_AlgErr) {
	    if (fmsg->typD == fmTyp_char) {	//capture error
		ppcchange(&ld->msg_s, fmsg->D.s);
		fmsg->Ok = true;
		fmsg->ev_Stop = true;
	    }
	}
}

static bool dev_open_RD(TXCopy_wd *wd) {
  TXCopy_ld *ld = wd->xcy_src;
  Tprivate *pv = ld->private;


	pv->IsOpen = false;
	if (!pv->netlb) pv->netlb = TNetLineBuf_Create(wd, &ld->gp);
	if (!pv->netlb) {
 printf("%s(%d): Init_RD  ftp ERROR\n", FL);
	    return false;
	}

	pv->netlb->ioinfo.DataSize = ld->st->size;
	pv->netlb->OnDataAdd = OnData_RD;
	pv->netlb->OnFileMsg = OnFileMsg_RD;
	pv->netlb->lb = ld->lb;
	if (!pv->netlb->open || !pv->netlb->open(pv->netlb, NetOpenMod_Read)) {
#ifdef POINTER64BIT
 printf("%s(%d): Init_RD  ftp ERROR 0x%lx 0x%lx\n", FL, (int64_t)pv->netlb->lb, (int64_t)pv->netlb->open);
#else
 printf("%s(%d): Init_RD  ftp ERROR 0x%x 0x%x\n", FL, (int)pv->netlb->lb, (int)pv->netlb->open);
#endif
	    return false;
	}
	while (AppHandleMessage() && !pv->netlb->ende && !pv->IsOpen);

	if (!pv->netlb->error) return true;

	Tfree(pv->netlb);
	pv->netlb = NULL;
	return false;
}

static bool dev_close_RD(TXCopy_wd *wd, TXCopy_ld *src) {
  Tprivate *pv = src->private;

	ppTfree(&pv->netlb);
	return true;
}

static int send_Mt_RD(void *obj, int RunStat, int MsgSt) {
  TXCopy_wd *wd = obj;
  TXCopy_ld *ld;

	if (RunStat == thdMSG_XCopy_RunStat) {
	    atomic_or(&appDat.appFu, appFu_select_t0_single);
	    return handle_Mt_RD(wd, MsgSt);
	}
	ld = wd->xcy_src;
	if (ld && wd->hcyEv) return wd->hcyEv(obj, ld->msg_nr | xpyMsg_src);
	return 0;
}

static bool Get_Dat_RD(TXCopy_wd *wd, int *lg, int *fuEOL, TXCopy_ld *dest) {
  TXCopy_ld *src;
  Tlgs lgs;
  bool res;

	src = wd->xcy_src;
	if (!src) {
	   if (fuEOL) *fuEOL = fio_EOF;
	   return false;
	}
	appMutex_lock();
	dest = wd->xcy_dest;
	res = TLineBuf_GetData_lgs(src->lb, &lgs, dest->buf_size, fuEOL);
	if (res) {
	    pnmove(dest->buf, lgs.s, lgs.lg);
	    *lg = lgs.lg;
	}
	appMutex_unlock();
	return res;
}

static int Chk_Dat_RD(TXCopy_wd *wd) {
  TXCopy_ld *src;
  int res = 0;

	src = wd->xcy_src;
	if (!src) return 0x02;

	appMutex_lock();
	if (src->lb->lg > 0) res = 0x01;
	if (src->lb->eof) res |= 0x02;
	appMutex_unlock();
	return res;
}

/*_____________________________ appEv RD ________________________________*/
static void On_appEv_RD(TEv_Array *appEv) {
  TXCopy_ld *ld = appEv->obj;
  TXCopy_wd *wd = ld->wd;

// printf("%s(%d): %7d pth RD srt=0x%x\n", FL, GetCurTimeDiv_us(wd->us), ld->srt);

bef_loop:
	if ((ld->srt & srt_bef_mask) == srt_init) {
// printf("%s(%d): %7d pth RD Init\n", FL, GetCurTimeDiv_us(wd->us));
	    step_Init_RD(wd, wd);
	}
	if ((ld->srt & srt_bef_mask) == srt_ChkStart) {
// printf("%s(%d): %7d pth RD ChkStart\n", FL, GetCurTimeDiv_us(wd->us));
	    step_ChkStart_RD(wd, wd);
	}
	if ((ld->srt & srt_bef_mask) == srt_Start) {
// printf("%s(%d): %7d pth RD Start\n", FL, GetCurTimeDiv_us(wd->us));
	    step_Start_RD(wd, wd);
	}
/*
	if ((ld->srt & srt_bef_mask) == srt_move) {
	    if (rdWait && (ld->fd > 0)) {
		pthread_mutex_lock(&ld->io_mutex);
		int lgo = TLineBuf_sync0_mem(ld->lb);
		rdWait = (lgo + ld->lb->lg) > wd->MaxBufSize;
		pthread_mutex_unlock(&ld->io_mutex);
	    }
	}
*/
	if ((ld->srt & srt_bef_mask) == srt_del) {
// printf("%s(%3d): %7d RD DEL\n", FL, GetCurTimeDiv_us(wd->us));
	    step_Del_RD(wd);
	}

	if ((ld->srt & srt_bef_mask) == srt_ok) {
// printf("%s(%d): %7d pth RD Ok\n", FL, GetCurTimeDiv_us(wd->us));
	    if (ld->dev_close) ld->dev_close(wd, ld);
	    if (ld->init_send) {
		ld->init_send = false;
		ld->send_Mt(wd, thdMSG_XCopy_RunStat, MsgSt_close);
	    }
	    ld->srt = srt_0;
	}

	if ((ld->srt & srt_bef_mask) == srt_skip) {
// printf("%s(%3d): %7d RD Skip lg=%lld max=%d\n", FL, GetCurTimeDiv_us(wd->us), ld->data_lg, ld->lb->memlg);
	    ld->srt = srt_0;
	    if (ld->dev_close) ld->dev_close(wd, ld);
	    ld->send_Mt(wd, thdMSG_XCopy_RunStat, MsgSt_skip);
	}

	if (ld->srt & srt_msg_wait) {
	    ld->srt &= ~srt_msg_wait;

// printf("%s(%d): %7d RD srt_msg_wait=0x%05x \n", FL, GetCurTimeDiv_us(wd->us), ld->srt);
	    int ret = ld->send_Mt(wd, thdMSG_XCopy | thdMSG_rv_mmMSG_free, 0);
	    if (ret & xcyRetEv_BefMask) {
		if (ret & xcyRetEv_ALL) wd->skip |= ld->msgOk_skip;
		ld->skip |= wd->skip;
//		if (wd->stat & xcywdstat_SameFileMk) {
//		    ld->skip = 0;
//		}
		ld->skip |= ld->msgOk_skip;
		ret &= xcyRetEv_BefMask;
		if (ret == xcyRetEv_SKIP) ld->srt = srt_skip;

// printf("%s(%d): %7d RD skip=0x%05x\n", FL, GetCurTimeDiv_us(wd->us), ld->skip);
		goto bef_loop;

	    } else {
		ld->srt = srt_0;
		ld->send_Mt(wd, thdMSG_XCopy_RunStat, MsgSt_err);
	    }
	}

}

/*_____________________________ Thread WR _______________________________*/
static void step_Init_WR_FTP(TXCopy_wd *wd) {
  TXCopy_ld *src, *dest;
  char *pc;
  int xcyChk;
  int src_ifmt;
  uint64_t t64;
  TNetErrRet ErrRet;

	net_err_init(&ErrRet);
	src = wd->xcy_src;
	dest = wd->xcy_dest;

	if (!step_Init_WR(wd, wd, &xcyChk, &src_ifmt)) goto err_skip;

	switch (xcyChk & xcyChkDest_BefMask) {

/*	case xcyChkDest_DelDest:
	    break; */

	case xcyChkDest_MsgSkip:
	    goto err_skip;

	case xcyChkDest_Skip:
	    dest->srt = srt_skip;
	    return;

	case xcyChkDest_Preserv:
	    dest->srt = srt_preserv;
	    return;
	}

	switch (src_ifmt) {

	case S_IFDIR:
	    dest->stmode = wd->mode_dir;
	    t64 = GetTime_us64();
	    if (ftp_mkdir_gp_NoReScan(&dest->gp, &ErrRet, dest->stmode) < 0) {
		dest->msgOk_skip = xcySkip_errMkDir;
		dest->msg_nr = xpyErr_Alg | xpyErr_MkDir;
		wd->xcyChk |= xcyChkDest_NoDirCreate;
		goto err_sys_skip;
	    }
	    wd->xcs.FileCount++;
	    wd->xcs.GesDataSize += src->st->size;
	    wd->xcs.GesDataTime += GetTime_us64() - t64;

	    pc = pcadd(dest->gp.path, dest->gp.name);
	    add_dest_dir(wd, pc);
	    xfree(pc);
	    dest->srt = srt_preserv;
	    return;

	case S_IFREG:
	    dest->atime = 0;
	    dest->mtime = 0;
	    if (!dest->st) dest->stmode = wd->mode_file;
	    dest->srt = srt_open;
	    return;

	case S_IFLNK:
	    ErrRet.err = pcdup("Operation not permitted");
	    dest->msgOk_skip = xcySkip_errMkLink;
	    dest->msg_nr = xpyErr_Alg | xpyErr_MkLink;
	    goto err_sys_skip;

	case S_IFIFO:
	    ErrRet.err = pcdup("Operation not permitted");
	    dest->msgOk_skip = xcySkip_errMkNod;
	    dest->msg_nr = xpyErr_Alg | xpyErr_MkNod;
	    goto err_sys_skip;

	case S_IFCHR:
	case S_IFBLK:
	case S_IFSOCK:
	    ErrRet.err = pcdup("Operation not permitted");
	    dest->msgOk_skip = xcySkip_errMkNod;
	    dest->msg_nr = xpyErr_Alg | xpyErr_MkNod;
	    goto err_sys_skip;
	}

	dest->srt = srt_ok | srt_msg_wait;
	dest->msgOk_skip = xcySkip_errType;
	dest->msg_nr = xpyErr_Alg | xpyErr_Type;
	goto err_sys_skip;

err_sys_skip:
	ppcchange(&dest->msg_s, ErrRet.err);
	net_err_clr(&ErrRet);
	dest->msg_nr |= xpyErr_Msg;
err_skip:
	dest->srt |= srt_msg_wait;
	if (dest->skip & dest->msgOk_skip) dest->srt = srt_skip;
}

static void OnFileMsg_WR(TFileMsg *fmsg) {
  TXCopy_wd *wd;
  TXCopy_ld *dest;

// file_msg_debug(fmsg, NULL);

	if (!file_msg_chk_wtyp(fmsg, wtyp_TNetLineBuf, FL)) return;
	if (!fmsg->owner) return;
	wd = fmsg->owner;
	dest = wd->xcy_dest;
	if (!dest) return;

	if (fmsg->art == fmArt_strg) {
	    if (fmsg->D.m == hSHStM_WrOpen) {
		((Tprivate*)dest->private)->IsOpen = true;
// printf("%s(%d): Init_RD  ftp Open 0x%06x\n", FL, wd->wtyp);
	    }

	} else if (fmsg->art == fmArt_DataIO) {
	    if (fmsg->typD == fmTyp_ioinfo) {
		wd->xcs.DataNowSize = fmsg->D.ioinfo->DataNowSize;
		wd->xcs.DataNowTime = fmsg->D.ioinfo->Time_us;
// printf("%s(%d): Chk_Dat_WR 0x%08llx 0x%08llx\n", FL, dest->ToDevDataSize, wd->xcs.DataNowSize);
	    }

	} else if (fmsg->art == fmArt_AlgErr) {
	    if (fmsg->typD == fmTyp_char) {	//capture error
		ppcchange(&dest->msg_s, fmsg->D.s);
		fmsg->Ok = true;
		fmsg->ev_Stop = true;
	    }
	}
}

static bool dev_open_WR(TXCopy_wd *wd) {
  TXCopy_ld *dest = wd->xcy_dest;
  Tprivate *pv = dest->private;

	pv->IsOpen = false;
	if (!pv->netlb) pv->netlb = TNetLineBuf_Create(wd, &dest->gp);
	if (!pv->netlb) {
 printf("%s(%d): open_WR  ftp ERROR\n", FL);
	    return false;
	}

	pv->netlb->ioinfo.DataSize = wd->xcy_src->st->size;
//	pv->netlb->OnData = OnData_RD;
	pv->netlb->OnFileMsg = OnFileMsg_WR;
	TLineBuf_Clr(dest->lb);
	pv->netlb->lb = dest->lb;
	if (!pv->netlb->open || !pv->netlb->open(pv->netlb, NetOpenMod_Write)) {
#ifdef POINTER64BIT
 printf("%s(%d): open_WR  ftp ERROR 0x%lx 0x%lx\n", FL, (int64_t)pv->netlb->lb, (int64_t)pv->netlb->open);
#else
 printf("%s(%d): open_WR  ftp ERROR 0x%x 0x%x\n", FL, (int)pv->netlb->lb, (int)pv->netlb->open);
#endif
	    return false;
	}
	while (AppHandleMessage() && !pv->netlb->ende && !pv->IsOpen);

// printf("%s(%d): dev_open_WR nlb=0x%06x lb=0x%06x %d%d%d\n", FL, (int)pv->netlb, (int)pv->netlb->lb, pv->netlb->ende, pv->netlb->error, pv->IsOpen);
	if (!pv->netlb->error) return true;

	Tfree(pv->netlb);
	pv->netlb = NULL;
	return false;
}

static int Chk_Dat_WR(TXCopy_wd *wd) {
  TXCopy_ld *dest = wd->xcy_dest;
  int result = 0;

	if (!dest) return ChkDat_EOF; //0x02;
	if (dest->lb->lg) result = ChkDat_DataOk;
//	if (dest->lb->lg > 0x10000) result |= ChkDat_WAIT;
// printf("%s(%d): Chk_Dat_WR 0x%08x\n", FL, dest->lb->lg);

	if ((dest->ToDevDataSize - wd->xcs.DataNowSize) > 0x40000) {
	    result |= ChkDat_WAIT;
//	} else {
// printf("%s(%d): Chk_Dat_WR 0x%08llx 0x%08llx\n", FL, dest->ToDevDataSize, wd->xcs.DataNowSize);
	}

	return result;
}

static bool dev_WR(TXCopy_wd *wd, byte *data, int lg, int fuEOL) {
  TXCopy_ld *dest = wd->xcy_dest;

	TLineBuf_Add(dest->lb, (char*)data, lg);
	dest->lb->eof = (fuEOL & fio_EOF) != 0; 
	return true;
}

static bool dev_close_WR(TXCopy_wd *wd, TXCopy_ld *dest) {
  Tprivate *pv = dest->private;

	if (!pv->netlb) return true;
// printf("%s(%d): dev_close_WR\n", FL);
	dest->lb->eof = true;

	if (pv->netlb->wait) pv->netlb->wait(pv->netlb);

	ppTfree(&pv->netlb);
	return true;
}

static void Clear_WR(TXCopy_wd *wd) {
  TXCopy_ld *dest = wd->xcy_dest;
	xfree(dest->st);
	dest->st = NULL;
	xfree(dest->fname);
	dest->fname = NULL;
}

static bool Init_WR(TXCopy_wd *wd, int funkt) {
  TXCopy_ld *ld = wd->xcy_dest;

	ld->srt = srt_0;
	gp_CryptMini16c_enc(&ld->gp);
// printf("%s(%d): Init_WR %s//%s:%s@%s%s%s\n", FL, ld->gp.RootArt, ld->gp.user, ld->gp.passwd, ld->gp.URL, ld->gp.path, ld->gp.name);
	return MSG_Init_WR(wd, wd, funkt, false);
}

static bool Close_WR(TXCopy_wd *wd, int funkt) {
  TXCopy_ld *dest;
  Tprivate *pv;

	dest = wd->xcy_dest;
	if (!dest) return false;
	pv = dest->private;
	if ((dest->srt & srt_bef_mask) != srt_0) {
	    if (pv->netlb && pv->netlb->stop) pv->netlb->stop(pv->netlb);
	}
	dest->srt = srt_skip;
	return true;
}

static int send_Mt_WR(void *obj, int RunStat, int MsgSt) {
  TXCopy_wd *wd = obj;
  TXCopy_ld *ld;

	if (RunStat == thdMSG_XCopy_RunStat) {
	    atomic_or(&appDat.appFu, appFu_select_t0_single);
	    return handle_Mt_WR(wd, MsgSt);
	}
	ld = wd->xcy_dest;
	if (ld && wd->hcyEv) return wd->hcyEv(obj, ld->msg_nr | xpyMsg_dest);
	return 0;
}

/*_____________________________ appEv WR ________________________________*/
static void On_appEv_WR(TEv_Array *appEv) {
  TXCopy_ld *dest = appEv->obj;
  TXCopy_wd *wd = dest->wd;

bef_loop:
	if ((dest->srt & srt_bef_mask) == srt_init) {
	    step_Init_WR_FTP(wd);
	}

	if ((dest->srt & srt_bef_mask) == srt_open) {
	    if (!step_Open_WR(wd, wd)) dest->srt = srt_0;
	}

	if ((dest->srt & srt_bef_mask) == srt_move) {
// printf("%s(%3d): %7d WR Move\n", FL, GetCurTimeDiv_us(wd->us));
	    step_Move_WR(wd, wd);
	}

	if ((dest->srt & srt_bef_mask) == srt_preserv) {
// printf("%s(%3d): %7d WR Preserv\n", FL, GetCurTimeDiv_us(wd->us));
	    int fu = fsSingleReScan;
	    if (wd->xcy_src->st->st & st_Dir) fu = fsForceReScan;

	    Tstat64 *st = ftp_stat_gp(&dest->gp, NULL, fu);
	    if (st) {
		dest->mtime = st->mtime;
		dest->stmode = st->mode & Mode07777;
		xfree(dest->st);
		dest->st = st;
	    }
	    step_Preserv_WR(wd, wd, TFtpAppClient_get_fkt(&dest->gp));
	}

	if ((dest->srt & srt_bef_mask) == srt_skip) {
// printf("%s(%3d): %7d WR Skip\n", FL, GetCurTimeDiv_us(wd->us));
	    dest->srt = srt_0;
	    if (dest->dev_close) dest->dev_close(wd, dest);
	    dest->send_Mt(wd, thdMSG_XCopy_RunStat, MsgSt_skip);
	}

	if ((dest->srt & srt_bef_mask) == srt_ok) {
// printf("%s(%3d): %7d WR Ok\n", FL, GetCurTimeDiv_us(wd->us));
	    if (dest->dev_close) dest->dev_close(wd, dest);
	    dest->send_Mt(wd, thdMSG_XCopy_RunStat, MsgSt_close);
	    dest->srt = srt_0;
	}

	if (dest->srt & srt_msg_wait) {
// printf("%s(%3d): %7d WR Wait\n", FL, GetCurTimeDiv_us(wd->us));
	    dest->srt &= ~srt_msg_wait;

// printf("%s(%d): %7d WR srt_msg_wait=0x%05x \n", FL, GetCurTimeDiv_us(wd->us), ld->srt);
	    int ret = dest->send_Mt(wd, thdMSG_XCopy | thdMSG_rv_mmMSG_free, 0);
	    if (ret & xcyRetEv_BefMask) {
		if (ret & xcyRetEv_ALL) wd->skip |= dest->msgOk_skip;
		dest->skip |= wd->skip;
		if (wd->stat & xcywdstat_SameFileMk) {
		    dest->skip = 0;
		}
		dest->skip |= dest->msgOk_skip;
		ret &= xcyRetEv_BefMask;
		if (ret == xcyRetEv_SKIP) dest->srt = srt_skip;

// printf("%s(%d): %7d RD skip=0x%05x\n", FL, GetCurTimeDiv_us(wd->us), ld->skip);
		goto bef_loop;

	    } else {
		dest->srt = srt_0;
		dest->send_Mt(wd, thdMSG_XCopy_RunStat, MsgSt_err);
	    }
	}

}

/*_____________________________ Destroy _________________________________*/
static bool TXCopy_ld_Ftp__Destroy(TXCopy_ld *self) {
  Tprivate *pv = self->private;

	Free_appEv(pv->appEv);
	Tfree(pv->netlb);
	gp_pc_free(&self->gp);
	xfree(self->st);
	xfree(self->fname);
	Tfree(self->lb);
	xfree(pv);
	return false;
}

/*_____________________________ Create __________________________________*/
static TXCopy_ld *TXCopy_ld_Ftp_Create(int fu, TXCopy_wd *wd, TglobPath *gp) {
  TXCopy_ld *self;
  Tprivate *pv;

	self = ttc_create(sizeof(TXCopy_ld), wtyp_TXcopy_ld);
	self->destroy = (TDESTROY)TXCopy_ld_Ftp__Destroy;
	self->art = ldart_ftp;
	self->wd = wd;
	gp_dup_gpdata(&self->gp, gp);

	pv = xcalloc(sizeof(Tprivate));
	self->private = pv;

	if (fu & xcyFU_OUTPUT) {
	    if (wd->xcy_src && (wd->xcy_src->art == ldart_ftp)) {
		self->gp.fu |= gpi_dest;
	    }
	    self->clear = Clear_WR;
	    self->init  = Init_WR;
	    self->close = Close_WR;


	    self->chk_dat  = Chk_Dat_WR;

	    self->send_Mt  = send_Mt_WR;
	    self->dev_open = dev_open_WR;
	    self->dev_close = dev_close_WR;
	    self->dev_wr  = dev_WR;
	    self->stat_gp = ftp_stat_gp;
	    self->chmod_gp = ftp_chmod_gp;
	    self->utime_gp = ftp_utime_gp;

	    self->lb = TLineBuf_Create();
	    pv->appEv = Add_appEv(On_appEv_WR);
	    pv->appEv->obj = self;

	    self->buf_size = 0x2000; //BUFSIZ;
	    self->buf = xmalloc(self->buf_size);

	} else {
	    self->gp.fu &= ~gpi_dest;
	    self->clear = Clear_RD;
	    self->init = Init_RD;
	    self->close = Close_RD;
	    self->set_gp = set_gp;

	    self->send_Mt = send_Mt_RD;
	    self->dev_open = dev_open_RD;
	    self->dev_close = dev_close_RD;
	    self->stat_gp = ftp_stat_gp;
	    self->rmdir_gp = ftp_rmdir_gp;
	    self->unlink_gp = ftp_unlink_gp;

	    self->get_dat = Get_Dat_RD;
	    self->chk_dat = Chk_Dat_RD;
	    self->lb = TLineBuf_Create();
	    pv->appEv = Add_appEv(On_appEv_RD);
	    pv->appEv->obj = self;
	    appMutex_init();
	}

	return self;
}

static bool TXCopy_ld_Ftp_ChkType(TglobPath *gp, bool dest) {
	return ftp_chk_RootArt_URL(gp);
}

ThXCopy hXCopy_ld_Ftp = {
  TXCopy_ld_Ftp_ChkType,
  TXCopy_ld_Ftp_Create
};
