/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appbase.h"
#include "tdirlist.h"
#include "xcopy_pv.h"
#include <errno.h>
#include "net_io.h"

bool MSG_Init_WR(void *obj, TXCopy_wd *wd, int MsgSt, bool isOpen) {
  TXCopy_ld *ld = wd->xcy_dest;

	if (!ld) return False;
	ld->skip = wd->skip;
	if (!wd->xcy_src) {
 printf("%s(%d): pth WR Init Internal ERROR !!!\n", FL);
	    ld->send_Mt(obj, thdMSG_XCopy_RunStat, MsgSt_err);
	    return False;

	} else {
// printf("%s(%03d): %7d pth WR Init MsgSt=%d stat=0x%03x\n", FL, GetCurTimeDiv_us(wd->us), MsgSt, wd->stat);
	    if (MsgSt == MsgSt_FileStatGet) {
		int m = 0;
		if (ld->skip && xcyForce_followlinks) m = fsFollowLink;
		ld->st = ld->stat_gp(&ld->gp, NULL, m);
		ld->send_Mt(obj, thdMSG_XCopy_RunStat, MsgSt_FileStatRet);

	    } else {
		ld->srt = srt_init;
	    }
	}
	return True;
}

static bool chk_link_skip(Tstat64 *st, int skip) {
#ifdef _OS_WIN32
	return False;
#else
	return (((st->mode & S_IFMT) == S_IFLNK) ||
		((st->st & st_Link) && !(skip & xcyForce_followlinks)));
#endif
}

bool step_Init_WR(void *obj, TXCopy_wd *wd, int *xcyChk, int *src_ifmt) {
  TXCopy_ld *src, *dest;
  int m;

	src = wd->xcy_src;
	dest = wd->xcy_dest;
	if (!src || !dest) return False;

	dest->uid = wd->uid;
	dest->gid = wd->gid;
	dest->atime = 0;
	dest->mtime = 0;
	dest->stmode = src->stmode;
//	xfree(dest->fname);
//	dest->fname = gptopc(&dest->gp, gptfu_noPasswd|gptfu_del_slash|gptfu_noRootArt);
	xcyld_SetFileName(dest);

	if ((src->art == dest->art) && pccmp(src->fname, dest->fname)) {
	    wd->stat |= xcywdstat_SameFileMk;
	    dest->msgOk_skip = xcySkip_errSameFile;
	    dest->msg_nr = xpyErr_Alg | xpyErr_SameFile;
	    return False;
	}

	*src_ifmt = src->st->mode & S_IFMT;
#ifndef _OS_WIN32
	if (chk_link_skip(src->st, src->skip)) *src_ifmt = S_IFLNK;
#endif

	m = 0;
	if (dest->skip & xcyForce_followlinks) m = fsFollowLink;
	dest->st = dest->stat_gp(&dest->gp, NULL, m);

	m = 0;
	if (dest->st) {
	    dest->uid = dest->st->uid;
	    dest->gid = dest->st->gid;
	    dest->atime = dest->st->atime;
	    dest->mtime = dest->st->mtime;
	    dest->stmode = dest->st->mode & Mode07777;
	}
	    m = xcyChkDest(src, dest, dest->skip,
				&dest->msgOk_skip, &dest->msg_nr);

	if (m & xcyChkDest_bSkipDir) {
	    char *pc = pcadd(src->gp.path, src->gp.name);
	    add_dest_dir(wd, pc);
	    xfree(pc);
	}

	wd->xcyChk |= m;
	*xcyChk = m;
	return True;
}

static void dest_dev_close(TXCopy_wd *wd, TXCopy_ld *dest) {

	if (dest->dev_close) dest->dev_close(wd, dest);
	dest->srt = srt_preserv;
	dest->ToDevDataSize = 0;

	wd->xcs.GesDataSize += wd->xcs.DataSize;
	wd->xcs.DataSize = 0;
	wd->xcs.DataNowSize = 0;
	wd->xcs.DataNowTime = 0;
	wd->xcs.GesDataTime += GetTime_us64() - dest->open_data_us;
// printf("%s(%d): %3d %8lld %8lld   %8lld %8lld %8lld\n", FL, wd->xcs.FileCount, wd->xcs.GesDataSize, wd->xcs.GesDataTime, wd->xcs.DataSize, wd->xcs.DataNowSize, wd->xcs.DataNowTime);
}

bool step_Open_WR(void *obj, TXCopy_wd *wd) {
  int lg, fuEOL;
  TXCopy_ld *src, *dest;

	src = wd->xcy_src;
	dest = wd->xcy_dest;
	if (!src || !dest) return False;

	if (!dest->dev_wr) {
	    dest->srt = srt_ok;
	    return True;
	}
	
	if (!chk_link_skip(src->st, src->skip) && (src->st->st & st_Reg)) {
	    dest->atime = 0;
	    dest->mtime = 0;
	    if (!dest->st) dest->stmode = wd->mode_file;

	    dest->open_data_us = GetTime_us64();
	    if (!dest->dev_open || !dest->dev_open(wd)) {
		if (dest->skip & xcySkip_errOpenWR) {
		    dest->srt = srt_ok;
		    return True;
		}
		dest->msgOk_skip = xcySkip_errOpenWR;
		dest->msg_nr = xpyErr_Alg | xpyErr_Msg | xpyErr_OpenWR;
		dest->srt |= srt_msg_wait;
		return True;
	    }
// printf("%s(%d): %7d WR Open size=%lld\n", FL, GetCurTimeDiv_us(wd->us), src->st->size);
	    wd->xcs.FileCount++;
	    wd->xcs.DataSize = src->st->size;

	    if (!src->st->size) {
		dest_dev_close(wd, dest);
		return True;
	    }

// printf("%s(%d): %7d WR Open\n", FL, GetCurTimeDiv_us(wd->us));
	    fuEOL = 0;

	    if (wd->xcy_src->get_dat(wd, &lg, &fuEOL, dest)) {
		dest->dev_wr(wd, (byte*)dest->buf, lg, fuEOL);
		dest->ToDevDataSize += lg;
//	    } else {
// printf("%s(%d): %7d noData\n", FL, GetCurTimeDiv_us(wd->us));
	    }
// printf("%s(%d): %7d WR Open lg=%lld eof=0x%02x\n", FL, GetCurTimeDiv_us(wd->us), src->st->size, fuEOL);
	    if (fuEOL & fio_EOF) {
		dest_dev_close(wd, dest);
//  printf("%s(%d): %7d WR Move Ende %lld\n", FL, GetCurTimeDiv_us(wd->us), dest->data_lg);
	        return True;
	    }

	    dest->srt = srt_move;
	    return True;
	}
	dest->srt = srt_ok | srt_msg_wait;
	dest->msg_nr = xpyErr_Alg | xpyErr_Type;
	return True;
}

bool step_Move_WR(void *obj, TXCopy_wd *wd) {
  int lg, fuEOL;
  TXCopy_ld *dest;
  bool res, wait;

	dest = wd->xcy_dest;
	if (!dest) return True;

	res = True;
	fuEOL = 0;

	wait = False;
	if (dest->chk_dat) {
	    wait = (dest->chk_dat(wd) & ChkDat_WAIT) != 0;
	}

	if (!wait && wd->xcy_src->get_dat(wd, &lg, &fuEOL, dest)) {
	    dest->dev_wr(wd, (byte*)dest->buf, lg, fuEOL);
	    dest->ToDevDataSize += lg;
	    res = False;
	}

	if (fuEOL & fio_EOF) {
	    dest_dev_close(wd, dest);
	    res = True;
	}
	return res;
}

void step_Preserv_WR(void *obj, TXCopy_wd *wd, dword akt_fkt) {
  TXCopy_ld *src, *dest;  
  TNetErrRet ErrRet;
  uint64_t t;

	dest = wd->xcy_dest;
	src = wd->xcy_src;
	if (!src || !dest) {
	    if (dest) dest->srt = srt_0;
	    return;
	}

	dest->srt = srt_ok;
	if ((akt_fkt & facfkt_CHMOD) && dest->chmod_gp && 
	    !(dest->skip & xcySkip_Chmod) && (src->stmode != dest->stmode)) {

	    t = GetTime_us64();
	    net_err_init(&ErrRet);
	    if (!dest->chmod_gp(&dest->gp, &ErrRet, src->stmode) && !(wd->skip & xcySkip_errChmod)) {
		if (ErrRet.err) {
 printf("%s(%d): Err chmod=%s\n", FL, ErrRet.err);
		} else {
		    errno = ErrRet.sys_errno;
 printf("%s(%d): Err chmod=%s\n", FL, SysError_pc());
		}
	    }
	    net_err_clr(&ErrRet);
	    wd->xcs.GesDataTime += GetTime_us64() - t;
	}

/*
	if ((akt_fkt & facfkt_CHOWN) && dest->chown_gp && 
	    !(dest->skip & xcySkip_Chown)
	    && ((src->uid != dest->uid)||(src->gid != dest->gid))) {
	    if ((chown(dest->fname, src->st->uid, src->st->gid) < 0) && !(wd->skip & xcySkip_errChown)) {
    printf("%s(%d): Err chown=%s\n", FL, SysError_pc());
	    }
	}
*/

	if ((akt_fkt & facfkt_UTIME) && dest->utime_gp && 
	    !(dest->skip & xcySkip_Utime) && 
	    ((src->atime != dest->atime)||(src->mtime != dest->mtime))) {

	    t = GetTime_us64();
	    net_err_init(&ErrRet);
	    if (!dest->utime_gp(&dest->gp, &ErrRet, src->mtime) && !(wd->skip & xcySkip_errUtime)) {
		if (ErrRet.err) {
 printf("%s(%d): Err utime=%s\n", FL, ErrRet.err);
		} else {
		    errno = ErrRet.sys_errno;
 printf("%s(%d): Err utime=%s\n", FL, SysError_pc());
		}
	    }
	    net_err_clr(&ErrRet);
	    wd->xcs.GesDataTime += GetTime_us64() - t;
	}

}

/* __________________________ msg _______________________________________ */
int handle_Mt_WR(TXCopy_wd *wd, int MsgSt) {

	switch (MsgSt & thdMSG_Mask) {
	    case MsgSt_FileStatRet:
// printf("%s(%d): %7d WR MsgSt_FileStatRet\n", FL, GetCurTimeDiv_us(wd->us));
		if (wd->xcy_src) wd->xcy_src->init(wd, MsgSt);
		break;

	    case MsgSt_err:
// printf("%s(%d): %7d WR MsgSt_err ERROR\n", FL, GetCurTimeDiv_us(wd->us));
		wd->stat |= xcywdstat_wr_err;
	    case MsgSt_close:
	    case MsgSt_skip:
// if (MsgSt == MsgSt_skip) printf("%s(%d): %7d WR MsgSt_skip stat=0x%03x\n", FL, GetCurTimeDiv_us(wd->us), wd->stat);
		wd->stat &= ~xcywdstat_wr_run;
		if (wd->stat & xcywdstat_rd_run) {
		    if (wd->xcy_src && wd->xcy_src->close) wd->xcy_src->close(wd, MsgSt);
		}
		break;

	    default:
 printf("%s(%d): %7d WR Msg ??? msg->tag=%d\n", FL, GetCurTimeDiv_us(wd->us), MsgSt);

	}
	return 0;
}
