/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"

//BUFSIZ
#define LBSEGSIZE 0x2000

void TLineBuf_Clr(TLineBuf *self) {
	if (self->memlg > (LBSEGSIZE<<1)) {
	    xfree(self->UTF8_buf);
	    self->UTF8_buf = NULL;
	    self->UTF8_buf_size = 0;
	    xfree(self->pc);
	    self->pc = NULL;
	    self->memlg = 0;
	}
	self->pc_rdidx = NULL;
	self->lg = 0;
	self->datalg = 0;
	self->eof = False;
}

long TLineBuf_sync0_mem(TLineBuf *self) {
  long lgo;

	if (!self->pc_rdidx) return 0;
	if (self->lg <= 0) {
	    self->lg = 0;
	    self->pc_rdidx = NULL;
	    return 0;
	}

	lgo = self->pc_rdidx - self->pc;
	if (!self->wr_memlook && 
	    ((lgo > (self->memlg >> 2)) || 
	     ((lgo > 256) && (self->lg < 256)))) {
	    pcnmove(self->pc, self->pc_rdidx, self->lg);
	    self->pc_rdidx = NULL;
	    return 0;
	}
	return lgo;
}

char *TLineBuf_Set_mem(TLineBuf *self, long lg) {
  long m, lgi, lgo;

	if (!self->pc) {
	    m = lg / (LBSEGSIZE<<1) + 1;	//New=BUFSIZ * 2
	    self->memlg = (LBSEGSIZE<<1) * m;
	    self->pc = xmalloc(self->memlg);
	    return self->pc;
	}

	lgo = TLineBuf_sync0_mem(self);
	lgi = lgo + self->lg;
	if ((self->memlg - lgi) < lg) {
	    m = lg - (self->memlg - lgi);
	    m = m / (LBSEGSIZE<<2) + 1;		//BUFSIZ * 4
	    self->memlg += (LBSEGSIZE<<2) * m;
	    self->pc = xrealloc(self->pc, self->memlg);
	    if (self->pc_rdidx) self->pc_rdidx = &self->pc[lgo];
	}
	return &self->pc[lgi];
}

void TLineBuf_Add(TLineBuf *self, const char *s, long lg) {
  char *pcd;

	if (!s || (lg <= 0)) {
	    self->eof = True;
	    return;
	}
	pcd = TLineBuf_Set_mem(self, lg+1);
	ppcnmove(&pcd, s, lg);
	self->lg += lg;
	self->datalg += lg;
}

long TLineBuf_fd_Add(TLineBuf *self, FileHandle fd) {
  size_t lg;
  char *pcd;

	if (!ChkFileHandleOK(fd)) {
	    self->eof = True;
	    return 0;
	}
	pcd = TLineBuf_Set_mem(self, LBSEGSIZE+1);

	self->rd_ok = file_read(fd, NULL, pcd, LBSEGSIZE, &lg);
	if (self->rd_ok && (lg > 0)) {
	    self->lg += lg;
	    self->datalg += lg;
	    pcd[lg] = 0;
	} else {
	    self->eof = True;
	    lg = 0;
	    *pcd = 0;
	}
	return lg;
}

bool TLineBuf_GetData_lgs(TLineBuf *self, Tlgs *lgs, long max, int *fuEOL) {
  int tmpEOL = 0;

	lgs->s = NULL;
	lgs->lg = 0;
	if (!fuEOL) fuEOL = &tmpEOL;
	if (!self->lg) goto chk_EOF;

	lgs->s = self->pc;
	lgs->lg = self->lg;
	if (self->pc_rdidx) lgs->s = self->pc_rdidx;

	if (max && (lgs->lg > max)) {
	    lgs->lg = max;
	    self->lg -= lgs->lg;
            self->pc_rdidx = &lgs->s[lgs->lg];
	} else {
	    self->lg = 0;
	    self->pc_rdidx = NULL;
	}


chk_EOF:
	if (self->eof && (self->lg < 1)) *fuEOL |= fio_EOF;
	return (lgs->s != NULL);
}

long TLineBuf_Sub(TLineBuf *self, long lg) {
  char *pc = self->pc;

	if (lg <= 0) return 0;
	if (self->pc_rdidx) pc = self->pc_rdidx;

	self->lg -= lg;
	if (self->lg > 0) {
            self->pc_rdidx = pc + lg;
	} else {
	    self->lg = 0;
	    self->pc_rdidx = NULL;
	}
	return self->lg;
}

long TLineBuf_Get(TLineBuf *self, long max, char **ppc) {
  long lg;

	lg = self->lg;
	if (max && (lg > max)) lg = max;

	if (ppc) {
	    if (self->pc_rdidx) {
		*ppc = self->pc_rdidx;
	    } else {
		*ppc = self->pc;
	    }
	}
	return lg;
}

/*_____________________________ Destroy _________________________________*/
static bool TLineBuf__Destroy(TLineBuf *self) {
	xfree(self->UTF8_buf);
	xfree(self->pc);
	return False;
}

/*_____________________________ Create __________________________________*/
TLineBuf *TLineBuf_Create() {
  TLineBuf *self = ttc_create(sizeof(TLineBuf), wtyp_TLineBuf);
	self->destroy = (TDESTROY)TLineBuf__Destroy;
	self->pc = xmalloc(LBSEGSIZE<<1);	//new BUFSIZ * 2
	self->memlg = LBSEGSIZE<<1;
	return self;
}
