/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include <sys/time.h>

#ifdef _OS_WIN32
#include <windows.h>

static int64_t TicksPerSecond;
static bool TPS_OK = False;

void set_app_time(uint64_t *app_us, dword *app_ms, dword *Hz30, long *tv_sec) {
  uint64_t Ticks;
  double tdf;

	if (!TPS_OK) {
	    QueryPerformanceFrequency((void*)&TicksPerSecond);
	    TPS_OK = True;
	}
	QueryPerformanceCounter((void*)&Ticks);
	*app_ms = (dword)(Ticks * 1000 / TicksPerSecond);
	tdf = (double)Ticks / (double)TicksPerSecond;
	*app_us = (uint64_t)(tdf * 1000000);
	*Hz30   = (dword)(tdf * 30);
	*tv_sec = (long)tdf;
}

uint64_t GetTime_us64() {
  uint64_t Ticks;
  double tdf;

	if (!TPS_OK) {
	    QueryPerformanceFrequency((void*)&TicksPerSecond);
	    TPS_OK = True;
	}
	QueryPerformanceCounter((void*)&Ticks);
	tdf = (double)Ticks / (double)TicksPerSecond;
	return (uint64_t)(tdf * 1000000);
}

dword GetTime_ms() {
  int64_t Ticks;
	if (!TPS_OK) {
	    QueryPerformanceFrequency((void*)&TicksPerSecond);
	    TPS_OK = True;
	}
	QueryPerformanceCounter((void*)&Ticks);
	Ticks = Ticks * 1000 / TicksPerSecond;
	return Ticks;
}

#else
uint64_t GetTime_us64() {
  struct timeval tv;
  uint64_t val;
	gettimeofday(&tv, NULL);
	val = (uint64_t)tv.tv_sec * 1000000;
	val += tv.tv_usec;
	return val;
}

dword GetTime_ms() {
  struct timeval tv;
	gettimeofday(&tv, NULL);
	return (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
}
#endif

dword GetTime_us() {
	return GetTime_us64();
}

bool cpTime(dword ref, dword now) {
	return (((now - ref) & 0x80000000) == 0);
}

dword GetCurTimeDiv_us(dword StartTime) {
	return (GetTime_us() - StartTime);
}

uint64_t GetCurTimeDiv_us64(uint64_t StartTime) {
	return (GetTime_us64() - StartTime);
}
