/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"

bool TLines_LoadFromLineBuf(TLines *tls, TLineBuf *lb) {
  Tlgs lgs;
  bool EvStop;

	if (!tls) return false;
	ppfree(&tls->st);
	ppfree(&tls->FileName);
	TLines_Clear(tls);

	if (lb && (lb->lg > 0)) {
	    EvStop = tls->EvStop;
	    tls->EvStop = true;
	    while (TLineBuf_GetLineEOL_lgs(lb, &lgs, &tls->load_fio_fu)) {
		TLines_Add_lgs(tls, &lgs, tls->load_fio_fu);
	    }
	    tls->org_load_fio_fu = tls->load_fio_fu;
	    tls->EvStop = EvStop;
	}
	tls->Changed = false;
	TLines_DoCount(tls);
	return true;
}

bool TLines_LoadFromMem(TLines *tls, const char *s, long size) {
  TLineBuf *lb;
  bool result;

	if (!tls) return false;
	if (s && (size > 0)) {
	    lb = TLineBuf_Create();
	    TLineBuf_Add(lb, s, size);
	    TLineBuf_Add(lb, NULL, 0);	//Set eof
	    result = TLines_LoadFromLineBuf(tls, lb);
	    Tfree(lb);

	} else {
	    result = TLines_LoadFromLineBuf(tls, NULL);
	}

	return result;
}

bool TLines_SaveToLineBuf(TLines *tls, TLineBuf *lb) {
  int idx, fuEOL;
  TColS *pcs;

	if (!tls || !lb) return false;
	TLineBuf_Clr(lb);

	fuEOL = tls->load_fio_fu  & ~fio_Res_Mask;
	if (tls->load_fio_fu & fio_has_BOM) fuEOL |= fio_Chk_BOM;
	idx = 0;

	while (idx < tls->Count) {
	    pcs = tls->lines->cs[idx];
	    if (pcs) {
		fuEOL &= ~fio_EOL_Mask;
		fuEOL |= GetDefaultEOL_fu(tls->DefaultEOL, pcs->fu);
		TLineBuf_SetLineEOL_lgs(lb, (Tlgs*)pcs, &fuEOL);
	    }
	    idx++;
	}
	lb->eof = true;
	return true;
}

bool TLines_SaveToMem(TLines *tls, char **s, long *size) {
  TLineBuf *lb;

	if (!tls || !s) return false;
	*s = NULL;

	lb = TLineBuf_Create();
	TLines_SaveToLineBuf(tls, lb);
	if (lb->lg > 0) {
	    *s = lb->pc;
	    lb->pc = NULL;
	}
	if (size) *size = lb->lg;
	Tfree(lb);
	return true;
}
