/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"

#define USE_MIN_RD_BUF 0

typedef struct {
  TglobPath *gp;
  bool chkHead;
} Tprivate;

static void lb_read(TLines *tls, TLineBuf *lb) {
  Tprivate *pv = tls->pv_io;
	if (pv->chkHead && (lb->lg > 0)) {
	    if (tls->st && hGetFileHeadInfo) {
		hGetFileHeadInfo->get_head_mime_mem((byte*)lb->pc, lb->lg, pv->gp, 
				&tls->st->FileType, &tls->st->sub_mime);
	    }
	    pv->chkHead = False;
	}

	Tlgs lgs;
	while (TLineBuf_GetLineEOL_lgs(lb, &lgs, &tls->load_fio_fu)) {
	    TLines_Add_lgs(tls, &lgs, tls->load_fio_fu);
	}
}

#if USE_MIN_RD_BUF
static void OnFileMsg(TFileMsg *fmsg) {
  TLines *tls;

	if ((fmsg->wtyp != wtyp_TNetLineBuf) ||
	    !ttc_chk_is_wtyp(fmsg->owner, wtyp_TLines)) return;

	tls = fmsg->owner;
	if ((fmsg->art == fmArt_DataIO) && 
	    (fmsg->typD == fmTyp_int) &&
	    (fmsg->D.m == fmReadRecvBuf) &&
	    (fmsg->typF == fmTyp_lb)) {

	    lb_read(tls, fmsg->F.lb);
	    fmsg->Ok = True;
	    fmsg->ev_Stop = True;

	} else {
	    file_msg_ev_next(fmsg, (TTC0*)tls, tls->OnFileMsg);
	}
}
#endif /* USE_MIN_RD_BUF */

bool TLines_LoadFromFile(TLines *tls, const char *FileName) {
  TLineBuf *lb;
  Tprivate pv;
  TNetLineBuf *nlb;
  const char *pcerr;
  bool EvStop;
  bool res = False;

	ppfree(&tls->st);
	TLines_Clear(tls);

	if (FileName) ppcchange_cmp(&tls->FileName, FileName);
	if (!tls->FileName) return False;
	pv.gp = pctogp(tls->FileName);
	gp_CryptMini16c_enc(pv.gp);

	pcerr = NULL;
	tls->st = net_stat_gp(pv.gp, NULL, fsFollowLink|fsGetFileType|fsNoFileName|fsNoLinkName);
	if (!tls->st || (tls->st == PTR_FAILED)) {
	    if (!tls->st) pcerr = "not found";
	    tls->st = NULL;
	} else if (tls->st->st & st_Dir) {
	    pcerr = "is dir !!!";
	}

	if (pcerr) {
	    char *fn = get_DisplayFileStr(NULL, pv.gp);
	    TTC_netError((TTC0*)tls, NULL, EMSG_ERR1, "file %s\n%s", pcerr, fn);
	    xfree(fn);
	    goto end_err0;
	}

	nlb = TNetLineBuf_Create(tls, pv.gp);
	if (!nlb) goto end_err0;

	tls->pv_io = &pv;
#if USE_MIN_RD_BUF
	nlb->OnFileMsg = OnFileMsg;
#endif /* USE_MIN_RD_BUF */
	nlb->ioinfo.DataSize = tls->st->size;

	lb = TLineBuf_Create();
	nlb->lb = lb;

	if (!nlb->open || !nlb->open(nlb, NetOpenMod_Read)) {
	    goto end_err;
	}

	if (!nlb->wait || !nlb->wait(nlb)) {
	    printf("%s(%d): ERROR: nlb->wait Read\n", FL);
	    goto end_err;
	}

	pv.chkHead = (tls->stat_fu & fsGetFileTypeHead) &&
		     ((tls->st->FileType == stft_None) || 
		      (tls->st->FileType == stft_NoFound));

	EvStop = tls->EvStop;
	tls->EvStop = True;
	lb_read(tls, lb);
	tls->org_load_fio_fu = tls->load_fio_fu;
	res = True;
	tls->EvStop = EvStop;

end_err:
	Tfree(nlb);
	Tfree(lb);
end_err0:
	gpFree(pv.gp);
	tls->pv_io = NULL;
	if (!res) TLines_Clear(tls);
	tls->Changed = False;
	TLines_DoCount(tls);
	return res;
}
