/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"


#define USE_MIN_WR_BUF 0

#if USE_MIN_WR_BUF
#define WR_BUFSIZE 0x040000
#endif

typedef struct {
  int idx;
  int fuEOL;
} Tprivate;


static void load_lb(TLines *tls, TLineBuf *lb) {
  Tprivate *pv = tls->pv_io;

#if USE_MIN_WR_BUF
	while ((pv->idx < tls->Count) && (lb->lg < WR_BUFSIZE)) {
#else
	while (pv->idx < tls->Count) {
#endif
	    TColS *pcs = tls->lines->cs[pv->idx];
	    if (pcs) {
		pv->fuEOL &= ~fio_EOL_Mask;
		pv->fuEOL |= GetDefaultEOL_fu(tls->DefaultEOL, pcs->fu);
		TLineBuf_SetLineEOL_lgs(lb, (Tlgs*)pcs, &pv->fuEOL);
	    }
	    pv->idx++;
	}
#if USE_MIN_WR_BUF
	if (pv->idx >= tls->Count) lb->eof = True;
#else
	lb->eof = True;
#endif
}

#if USE_MIN_WR_BUF
static void OnFileMsg(TFileMsg *fmsg) {
  TLines *tls;

	if ((fmsg->wtyp != wtyp_TNetLineBuf) ||
	    !ttc_chk_is_wtyp(fmsg->owner, wtyp_TLines)) return;

	tls = fmsg->owner;
	if ((fmsg->art == fmArt_DataIO) && 
	    (fmsg->typD == fmTyp_int) &&
	    (fmsg->D.m == fmFillSendBuf) &&
	    (fmsg->typF == fmTyp_lb)) {

	    load_lb(tls, fmsg->F.lb);
	    fmsg->Ok = True;
	    fmsg->ev_Stop = True;

	} else {
	    file_msg_ev_next(fmsg, (TTC0*)tls, tls->OnFileMsg);
	}
}
#endif /* USE_MIN_WR_BUF */

bool TLines_SaveToFile(TLines *tls, const char *FileName) {
  TglobPath *gp;
  Tprivate pv;
  TNetLineBuf *nlb;
  TLineBuf *lb = NULL;
  word stft, sub_mime;
  bool TypeHeadChk;
  bool res = False;

	if (FileName) ppcchange_cmp(&tls->FileName, FileName);
	if (!tls->FileName) return False;
	gp = pctogp(tls->FileName);
	gp_CryptMini16c_enc(gp);

	nlb = TNetLineBuf_Create(tls, gp);
	if (!nlb) goto end_err0;

#if USE_MIN_WR_BUF
	nlb->OnFileMsg = OnFileMsg;
#endif /* USE_MIN_WR_BUF */
	nlb->ioinfo.DataSize = TLines_GetSize(tls, 0);

	pv.idx = 0;
	pv.fuEOL = tls->load_fio_fu  & ~fio_Res_Mask;
	if (tls->load_fio_fu & fio_has_BOM) pv.fuEOL |= fio_Chk_BOM;
	tls->pv_io = &pv;

	lb = TLineBuf_Create();
	nlb->lb = lb;
	load_lb(tls, lb);

	TypeHeadChk = (tls->stat_fu & fsGetFileTypeHead) && hGetFileHeadInfo;
	if (TypeHeadChk) {
	    hGetFileHeadInfo->get_head_mime_mem((byte*)lb->pc, lb->lg, gp, &stft, &sub_mime);
	}

	if (!nlb->open || !nlb->open(nlb, NetOpenMod_Write)) {
	    printf("%s(%d): ERROR: nlb->open Write\n", FL);
	    goto end_err0;
	}
	if (!nlb->wait || !nlb->wait(nlb)) {
	    goto end_err0;
	}

	ppTfree(&lb);

	xfree(tls->st);
	tls->st = net_stat_gp(gp, NULL, fsNoFileName | fsGetFileType | fsNoLinkName | fsFollowLink);
	if (tls->st == PTR_FAILED) tls->st = NULL;

	if (tls->st && TypeHeadChk &&
		       ((tls->st->FileType == stft_None) || 
			(tls->st->FileType == stft_NoFound))) {
	    tls->st->FileType = stft;
	    tls->st->sub_mime = sub_mime;
	}
	tls->Changed = False;
	res = True;

end_err0:
	Tfree(nlb);
	Tfree(lb);
	gpFree(gp);
	tls->pv_io = NULL;

	return res;
}
