/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"

void TNodeIns(TNode *prev_child, TNode *prev, TNode *self) {
	if (!self) return;
	self->prev_child = prev_child;
	if (!prev_child && !prev) {
	    self->prev = NULL;
	    self->next = *self->pproot;
	    *self->pproot = self;
	} else if (prev) {
	    self->prev = prev;
	    self->next = prev->next;
	    prev->next = self;
	} else {
	    self->prev = NULL;
	    self->next = prev_child->next_child;
	    prev_child->next_child = self;
	}
	if (self->next) self->next->prev = self;
}

void TNodeReMove(TNode *self) {
	if (!self) return;
	if (self->prev) {
	    self->prev->next = self->next;
	} else if (self->prev_child) {
	    self->prev_child->next_child = self->next;
	}
	if (self->next) {
	    self->next->prev = self->prev;
	    self->next->prev_child = self->prev_child;
	}
	if (self == *self->pproot) *self->pproot = self->next;
	self->prev = self->next = NULL;
}

/*_____________________________ Free ____________________________________*/
bool TNode__Destroy(TNode *self) {
  TNode *n, **ppn;

#if 0
	while (self->next_child) Tfree(self->next_child); //Can Loop (node Lock)
#else
	ppn = &self->next_child;
	while (*ppn) {
	    n = *ppn;
	    Tfree(n);
	    if (n == *ppn) {
#ifdef POINTER64BIT
 printf("%s(%d): TNode Loop 0x%08lx wtyp=0x%x\n", FL, (int64_t)n, n->wtyp);
#else
 printf("%s(%d): TNode Loop 0x%08x wtyp=0x%x\n", FL, (int)n, n->wtyp);
#endif
		ppn = &n->next;
	    }
	}
#endif
	TNodeReMove(self);
	return False;
}

/*_____________________________ Create __________________________________*/
void *TNodeAddTo(TNode **pproot, TNode *prev_child, TNode *prev, int size, int wtyp) {
  TNode *self = TNode_Create(pproot, size, wtyp);

	TNodeIns(prev_child, prev, self);
	return self;
}

void *TNode_Create(TNode **pproot, int size, int wtyp) {
  TNode *self = NULL;

	if (!pproot || (size < sizeof(TNode))) return NULL;
	self = ttc_create(size, wtyp);
	self->destroy = (TDESTROY)TNode__Destroy;
	self->pproot = pproot;
	return self;
}
