/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"
#include "zsrc.h"
#include "zlib.h"

bool ZsrcDecompress(const ThZsrc *zsrc, TZsrcDecomp *ret) {
  int m;
  xchar xc;
  z_stream s;

	if (!ret || !zsrc || !zsrc->data || !zsrc->org_size) return false;

	ret->data   = zsrc->data;
	ret->size   = zsrc->org_size;
	ret->buf    = NULL;
	ret->result = ZsrcOk;

	if (zsrc->compress == zsrc_no_comp) return true;

	Fill0(&s, sizeof(s));

	ret->buf = xcalloc(zsrc->org_size + 256);
	ret->data = ret->buf;
	s.avail_in = zsrc->data_size;
	s.avail_out = zsrc->org_size + 256;
	xc.us_const = zsrc->data;
	s.next_in = xc.us;
	s.next_out = ret->buf;

	if (inflateInit2(&s, -15) != Z_OK) {
	    appError(EMSG_ERR, " zlib_inflateInit");
	    ret->result = ZsrcError;
	}

	m = inflate(&s, Z_FINISH);
	if ((ret->result == ZsrcOk) && (s.total_out != zsrc->org_size)) {
	    appError(EMSG_ERR, " zlib_inflate %d", m);
	    ret->result = ZsrcError;
	}

	inflateEnd(&s);

	if (ret->result == ZsrcOk) return true;
	ret->data = NULL;
	ppfree(&ret->buf);
	return false;
}
