/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include <signal.h>

#define USE_MWMHINTS 1

#if USE_MWMHINTS
/****** MwmUtil.h ******/
typedef struct TMwmHints {
    unsigned long flags;
    unsigned long functions;
    unsigned long decorations;
    long input_mode;
    unsigned long status;
} TMwmHints;

//#define MWM_HINTS_FUNCTIONS     (1L << 0)
#define MWM_HINTS_DECORATIONS   (1L << 1)
//#define MWM_HINTS_INPUT_MODE    (1L << 2)
//#define MWM_HINTS_STATUS        (1L << 3)

//#define MWM_FUNC_ALL            (1L << 0)
//#define MWM_FUNC_RESIZE         (1L << 1)
//#define MWM_FUNC_MOVE           (1L << 2)
//#define MWM_FUNC_MINIMIZE       (1L << 3)
//#define MWM_FUNC_MAXIMIZE       (1L << 4)
//#define MWM_FUNC_CLOSE          (1L << 5)

//#define MWM_DECOR_ALL           (1L << 0)
#define MWM_DECOR_BORDER        (1L << 1)
#define MWM_DECOR_RESIZEH       (1L << 2)
#define MWM_DECOR_TITLE         (1L << 3)
#define MWM_DECOR_MENU          (1L << 4)
#define MWM_DECOR_MINIMIZE      (1L << 5)
#define MWM_DECOR_MAXIMIZE      (1L << 6)

//#define MWM_INPUT_MODELESS 0			/* Input goes to any window */
//#define MWM_INPUT_PRIMARY_APPLICATION_MODAL 1	/* Input does not go to ancestors of this window *
//#define MWM_INPUT_SYSTEM_MODAL 2		/* Input goes only to this window */
//#define MWM_INPUT_FULL_APPLICATION_MODAL 3	/* Input does not go to other windows in this application */
//#define MWM_INPUT_APPLICATION_MODAL MWM_INPUT_PRIMARY_APPLICATION_MODAL

//#define MWM_TEAROFF_WINDOW	(1L<<0)

#endif

static int TForm_XEvent(TForm *self, XEvent *ev) {
  Window JRoot, JParent, *JChildren, JFrame;
  dword NChildren;
  TGeometry gm;

	switch(ev->type) {

	case ConfigureNotify: //22
  /*
  printf("ConfigureNotify: 0x%x 0x%x 0x%x %d %d %dx%d\n", 
  (int)ev->xconfigure.event, (int)ev->xconfigure.window, (int)ev->xconfigure.above,
  ev->xconfigure.x, ev->xconfigure.y,
  ev->xconfigure.width, ev->xconfigure.height);
  */
	    JChildren = NULL;
	    JFrame = ev->xconfigure.window;
	    while (XQueryTree(appXDat.dpy, JFrame, &JRoot, &JParent, &JChildren, &NChildren)) {
		if (JChildren) {XFree(JChildren); JChildren = NULL;}
		if (JParent == appXDat.RootW) break;
		JFrame = JParent;
	    }
	    if (JChildren) XFree(JChildren);
	    if (get_geometry(JFrame, &gm)) {
		self->left = gm.x;
		self->top = gm.y;
	    } else {
		self->left = ev->xconfigure.x;
		self->top = ev->xconfigure.y;
	    }
	    self->width = ev->xconfigure.width;
	    self->height = ev->xconfigure.height;
	    if ((self->xwc.left != self->left) || (self->xwc.top != self->top)) {
		if (self->OnMove) self->OnMove(self);
		self->xwc.left = self->left;
		self->xwc.top = self->top;
	    }
	    if ((self->xwc.width != self->width) || (self->xwc.height != self->height)) {
		self->xwc.width = self->width;
		self->xwc.height = self->height;
		if (self->ResizeStartMk && self->OnResize && self->Visible) 
		    self->OnResize(self);
		TWins_Resize((TWins*)self);
	    }
	    break;

	case ClientMessage:
	    if ((ev->xclient.format == 32) && 
		(ev->xclient.data.l[0] == appXDat.XA_WM_DELETE_WINDOW)) {
		    self->CloseIsWM_DELETE = true;
		    if (self->Close) self->Close(self);
		    return 1;
	    }
	    break;
	}
	return 0;
}

static int TForm_EvProc(Tobjmsg *msg) {
  TForm *self;

	self = (TForm*)msg->wins;
	if (!self) return 0;

	switch(msg->msg) {
	case msgXEv:
	    if (TForm_XEvent(self, (XEvent*)msg->p)) return 1;
	    break;

	case msgVisibleFormShow:
	    if (self->Visible && self->Show) self->Show(self);
	    return 1;

#ifdef DRAG_AND_DROP
/*	case msg_dnd:
		switch(msg->f) {
		case msgf_dndEnter:
			if (appXDat.dnd) return 1;
			break;
		}
		break; */
#endif
	} 

	return TWins_EvProc(msg);
}


void TForm_Close(TForm *self) {
  int CanClose;

	CanClose = FormClose;
	if (self->OnCanClose) self->OnCanClose(self, &CanClose);
	if (CanClose) {
	    if (appXDat.StartForm == self) {
		appDat.gQUIT_Ev = true;
	    } else {
		if (CanClose & FormCloseAndFree) {
		    Tfree(self);
		    return;
		} else {
		    TWins_SetVisible((TWins*)self, false);
		}
	    }
	}
	self->CloseIsVisibleFalse = false;
	self->CloseIsWM_DELETE = false;
}
/*
void TForm_SetNetWmStat(TForm *self, Atom A1, Atom A2, bool set) {
  XEvent ev;

	Fill0(&ev, sizeof(ev));
	ev.xclient.type = ClientMessage;
	ev.xclient.message_type = appXDat.XA_NET_WM_STATE;
	ev.xclient.display = appXDat.dpy;
	ev.xclient.window = self->WinID;
	ev.xclient.format = 32;
	if (set) ev.xclient.data.l[ 0 ] = 1;
	ev.xclient.data.l[ 1 ] = A1;
	ev.xclient.data.l[ 2 ] = A2;
	XSendEvent(appXDat.dpy, appXDat.RootW, false, 
		   SubstructureNotifyMask|SubstructureRedirectMask, &ev);
}
*/
void  TForm_Show(TForm *self) {
  int x, y, xm, ym;
  XID ParentWinID;
  XSizeHints *pSizeHints;
  XTextProperty *pxname, *pxIconName;
  XWMHints *pWMHints;
  XClassHint *FormClass;
  TWRect rc;
  xchar xc;
  bool send_WMProp; //, send_OnTop;
  XSizeHints SizeHints;
  XTextProperty xname, xIconName;
    // NET window states
  long net_wm_stat[6];
  int net_wm_stat_count;
#if USE_MWMHINTS
  TMwmHints *pMwmH;
#endif

	if (self->Visible && self->IsParent) {
	    if (!(self->FormZStyle & fszNoInput))
		XSetInputFocus (appXDat.dpy, self->WinID, RevertToParent, 
				CurrentTime);

	} else {
	    send_WMProp = false;
//	    send_OnTop = false;
	    pSizeHints = NULL;
	    pWMHints = NULL;
	    pxname = NULL;
	    pxIconName = NULL;
	    FormClass = NULL;
	    Fill0(&SizeHints, sizeof(XSizeHints));
	    Fill0(&net_wm_stat, sizeof(net_wm_stat));
	    net_wm_stat_count = 0;

	    if (self->FormPosition != poDefault) {
		ParentWinID = appXDat.RootW;
		if (self->Parent) ParentWinID = self->Parent->WinID;
		if (GetXWinAbs_rc(ParentWinID, &rc)) {
		    x = rc.left + rc.width;
		    y = rc.top + rc.height;
		    xm = rc.left + ((rc.width - self->width) >> 1);
		    ym = rc.top + ((rc.height - self->height) >> 1);
		} else {
		    x = appXDat.Root_width;
		    y = appXDat.Root_height;
		    xm = (x - self->width) >> 1;
		    ym = (y - self->height) >> 1;
		}
		if (xm < 0) xm = 0;
		if (ym < 0) ym = 0;

		if (self->FormPosition == poCenter) {
		    self->left = xm;
		    self->top = ym;
		} else if (self->Gravity) {
		    SizeHints.flags |= PWinGravity;
		    SizeHints.win_gravity = self->Gravity;
		    switch(self->Gravity) {
		    case NorthGravity:
			self->left = xm + self->left;
			break;
		    case NorthEastGravity:
			self->left = x - self->left - self->width;
			break;
		    case WestGravity:
			self->top = ym + self->top;
			break;
		    case CenterGravity:
			self->left = xm + self->left;
			self->top = ym + self->top;
			break;
		    case EastGravity:
			self->left = x - self->left - self->width;
			self->top = ym + self->top;
			break;
		    case SouthWestGravity:
			self->top = y - self->top - self->height;
			break;
		    case SouthGravity:
			self->left = xm + self->left;
			self->top = y - self->top - self->height;
			break;
		    case SouthEastGravity:
			self->left = x - self->left - self->width;
			self->top = y - self->top - self->height;
			break;
		    }
		    self->Gravity = 0;
		}
		SizeHints.x = self->left;
		SizeHints.y = self->top;
		SizeHints.flags |= USPosition;
	    }

	    if (self->FormZStyle & fszNoResize) {
		self->min_width = self->width;
		self->max_width = self->width;
		self->min_height = self->height;
		self->max_height = self->height;
	    }

	    if ((self->min_width > 0) || (self->min_height > 0)) {
		SizeHints.min_width = self->min_width;
		SizeHints.min_height = self->min_height;
		SizeHints.flags |= PMinSize;
	    }
	    if ((self->max_width > 0) || (self->max_height > 0)) {
		SizeHints.max_width = self->max_width;
		SizeHints.max_height = self->max_height;
		SizeHints.flags |= PMaxSize;
	    }

	    if (SizeHints.flags) {
		pSizeHints = &SizeHints;
		send_WMProp = true;
	    }

	    if ((self->Icon && self->Icon->pic) || (self->FormZStyle & fszNoInput)) {
		pWMHints = xcalloc(sizeof(XSizeHints));
		if (self->Icon && self->Icon->pic) {
		    pWMHints->icon_pixmap = self->Icon->pic;
		    if (self->Icon->mask) {
			pWMHints->icon_mask = self->Icon->mask;
			pWMHints->flags |= IconMaskHint;
		    }
		    pWMHints->flags |= IconPixmapHint;
		}
		if (self->FormZStyle & fszNoInput) {
		    pWMHints->input = false;
		    pWMHints->flags |= InputHint;
		}
		send_WMProp = true;
	    }

#if USE_MWMHINTS
	    pMwmH = NULL;
	    if (self->FormZStyle & (fszNoTitle|fszNoBorder|fszFullSize)) {
		pMwmH = xcalloc(sizeof(TMwmHints));
/*
		if (self->FormZStyle & fszFullSize) {
		    pMwmH->flags |= MWM_HINTS_STATUS;
		    pMwmH->status = MWM_TEAROFF_WINDOW;
		    self->FormZStyle |= fszNoTitle | fszNoBorder;
		}

		if (self->FormZStyle & fszNoInput) {
		    pMwmH->flags |= MWM_HINTS_INPUT_MODE;
		    pMwmH->input_mode = MWM_INPUT_MODELESS;
		}
*/
		pMwmH->flags |= MWM_HINTS_DECORATIONS;
		if (!(self->FormZStyle & fszNoTitle)) {
		    pMwmH->decorations = MWM_DECOR_RESIZEH | MWM_DECOR_TITLE |
			 MWM_DECOR_MENU | MWM_DECOR_MINIMIZE | MWM_DECOR_MAXIMIZE;
		}
		if (!(self->FormZStyle & fszNoBorder)) {
		    pMwmH->decorations = MWM_DECOR_BORDER;
		}
	    }
#endif

	    if (!self->IsParent) {
		self->IsParent = true;
#ifndef USE_NANO_X
		if ((self->FormZStyle & fszTransient) && self->Parent) {
		    TForm *fm = GetParentForm(self->Parent);
		    if (fm) XSetTransientForHint(appXDat.dpy, self->WinID, fm->WinID);
		}
#endif /* USE_NANO_X */

		TWins_AdjustSize((TWins*)self);

		if (self->vds.s) {
		    xname.value = (unsigned char*)self->vds.s;
		    xname.nitems = self->vds.lg;
		    xname.encoding = XA_STRING;
		    xname.format = 8;
		    pxname = &xname;
		}
		if (self->IconName) {
		    xc.s_const = self->IconName;
		    xIconName.value = (unsigned char*)xc.s;
		    xIconName.nitems = pclen(self->IconName);
		    xIconName.encoding = XA_STRING;
		    xIconName.format = 8;
		    pxIconName = &xIconName;
		}
		if (self->XFormClass.res_name || self->XFormClass.res_class)
			FormClass = &self->XFormClass;
		send_WMProp = true;
	    }
	    if (send_WMProp)
		XSetWMProperties(appXDat.dpy, self->WinID, pxname, pxIconName,
				 NULL, 0, pSizeHints, pWMHints, FormClass);

	    xfree(pWMHints);
#if USE_MWMHINTS
	    if (pMwmH) {
		XChangeProperty(appXDat.dpy, self->WinID, appXDat.XA_MOTIF_WM_HINTS,
			appXDat.XA_MOTIF_WM_HINTS, 32, PropModeReplace, (const byte*)pMwmH, 5);
		xfree(pMwmH);
	    }
#endif
	    if ((self->FormZStyle & fzsOnTop) && !self->Parent) {
		net_wm_stat[net_wm_stat_count] = appXDat.XA_NET_WM_STATE_ABOVE;
		net_wm_stat_count++;
	    }
	    if (net_wm_stat_count > 0) {
		XChangeProperty(appXDat.dpy, self->WinID, 
				appXDat.XA_NET_WM_STATE, XA_ATOM, 32, 
				PropModeReplace,
				(byte*)&net_wm_stat, net_wm_stat_count);
	    }

	    self->Visible = true;
	    MapWinID(self->WinID, 2);
/*
	    if (send_OnTop)
		TForm_SetNetWmStat(self, appXDat.XA_NET_WM_STATE_ABOVE, 0, true);
*/
	    Resize((TWins*)self);
	}
	if (self->OnShow) self->OnShow(self);

}

static int TForm_DoKeyDown(TForm *self, TKeyEvent *KeyEv) {
	if (self->Root_MenuButton && self->Root_MenuButton->mbuStat) return 1;
	return TWins_DoKeyDown((TWins*)self, KeyEv);
}

static int TForm_DoKeyUp(TForm *self, TKeyEvent *KeyEv) {
	if (!self->Root_MenuButton || !self->Root_MenuButton->mbuStat) {
	    TWins_DoKeyUp((TWins*)self, KeyEv);
	}

	if (!(KeyEv->state & 0x4) && 
            ((KeyEv->keysym == XK_Meta_L) || (KeyEv->keysym == XK_Alt_L))) {
	    if (self->DoMenuButtonExec) self->DoMenuButtonExec(self);
	}
	return 1;
}

static void TForm_SetCaption(TForm *self, const char *s) {
  XTextProperty xtp;

	if (pccmp(s, self->vds.s)) return;
	vds_SetStr(&self->vds, s);
	if (self->IsParent) {
	    xtp.value = (unsigned char*)self->vds.s;
	    xtp.encoding = XA_STRING;
	    xtp.format = 8;
	    xtp.nitems = self->vds.lg;
	    XSetWMProperties(appXDat.dpy, self->WinID, &xtp, NULL, NULL, 0, NULL, NULL, NULL);
	}
}

void TForm_SetVisible(TForm *self, bool Value) {
	if (self->Visible != Value) {
	    if (self->Visible) {
		self->CloseIsVisibleFalse = true;
		if (self->Close) self->Close(self);
	    } else {
		if (self->Show)  self->Show(self);
	    }
	}
}

/*_____________________________ Destroy _________________________________*/
bool TForm__Destroy(TForm *self) {
	return TWins__Destroy((TWins*)self);
}

/*____________________________ Create ___________________________________*/
void TForm_CreateParams(TWins_Create_struct *wcs) {
  TForm *self;
  TForm *parentForm;

	wcs->XAttr.event_mask |= FocusChangeMask;
	self = wcs->wins;
	parentForm = GetParentForm((TWins*)wcs->SubP);
	if (parentForm) {
	    self->Icon = parentForm->Icon;
	    self->IconName = parentForm->IconName;
	}

//	self->destroy = (TDESTROY)TForm__Destroy;
	self->dnd_isEnable = false;
	self->FormStruct = (TWins*)self;

	self->width = 320;
	self->height = 240;

	self->EvProc = TForm_EvProc;
	self->SetCaption = TForm_SetCaption;
	self->SetVisible = TForm_SetVisible;

	self->DoKeyDown = TForm_DoKeyDown;
	self->DoKeyUp = TForm_DoKeyUp;
	self->Show = TForm_Show;
	self->Close = TForm_Close;

	if (!appXDat.StartForm) appXDat.StartForm = self;
}


void TForm_CreateXWin(TWins_Create_struct *wcs) {
  TForm *self = wcs->wins;
	TWins_CreateXWin(wcs);
#ifndef USE_NANO_X
	XSetWMProtocols(appXDat.dpy, self->WinID, &appXDat.XA_WM_DELETE_WINDOW, 1);
#endif /* USE_NANO_X */

	XChangeProperty(appXDat.dpy, self->WinID, 
			appXDat.XA_NET_WM_PID, XA_CARDINAL, 32, PropModeReplace,
			(unsigned char *) &appDat.self_pid, 1);
}

void *TForm_CreatParamAlg_WCSfkt(TWins *Parent, int FormMemSize, dword WCSfkt) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, FormMemSize, wtyp_Form);
	wcs.CreateParams = TForm_CreateParams;
	wcs.CreateXWin = TForm_CreateXWin;
	wcs.SubP = Parent;
	wcs.WCSfkt = WCSfkt;
	TWins_DefaultConf(&wcs);
	((TForm*)(wcs.wins))->Parent = Parent;
	return wcs.wins;
}

void *TForm_CreatParamAlg(TWins *Parent, int FormMemSize) {
	return TForm_CreatParamAlg_WCSfkt(Parent, FormMemSize, 0);
}

TForm *TForm_Create(TWins *Parent) {
	return TForm_CreatParamAlg_WCSfkt(Parent, sizeof(TForm), 0);
}
