/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"

dword TWins_GlobalCreateFlags;


void TWins_Refresh(TWins *self) {
	if (self->PaintOnEvEndMk || !self->IsParent || !self->Visible) return;
	self->PaintOnEvEndMk = true;
	if (PaintOnEvEnd_Count >= PaintOnEvEnd_List->items) 
		set_ptr_arr(&PaintOnEvEnd_List, PaintOnEvEnd_List->items+256);
	PaintOnEvEnd_List->val[PaintOnEvEnd_Count] = self;
	PaintOnEvEnd_Count++;
}

static void TWins_AppMouseDown(TWins *self, XButtonEvent *ev) {
	MouseDown.WinIsEnabled = TestWinIsEnabled(self);
	if (MouseDown.WinIsEnabled && (ev->button == 1)) Set_Focus(self);
	MouseDown.oldapph = MouseDown.apph;
	MouseDown.apph = self;
	MouseDown.x = ev->x;
	MouseDown.y = ev->y;
	MouseDown.x_root = ev->x_root;
	MouseDown.y_root = ev->y_root;
	MouseDown.button = ev->button;
	MouseDown.state = ev->state;
	MouseDown.obj_left = self->left;
	MouseDown.obj_top = self->top;
	MouseDown.obj_width = self->width;
	MouseDown.obj_height = self->height;
	if (self->DoMouseDown) self->DoMouseDown(self, ev);
}

static void TWins_ChkInfoWin(TWins *self, bool Leave) {
  TWins *InfoWin = appXDat.InfoWin;
	if (!Leave && self->InfoWin) {
	    appXDat.InfoWin = self->InfoWin;
	    appXDat.InfoWin_us = appDat.app_us + 400000;
	    if (InfoWin && (InfoWin != self->InfoWin)) {
		SetVisible(InfoWin, false);
	    }
	} else if (InfoWin) {
	    appXDat.InfoWin = NULL;
	    SetVisible(InfoWin, false);
	}
}

int TWins_EvProc(Tobjmsg *msg) {
  TWins *self;
  THdnd *Hdnd;
  XEvent *ev;

	self = msg->wins;
	if (!self) return 0;

	if (msg->msg == msgXEv) {
	    ev = (XEvent *) msg->p;
	    switch(ev->type) {
	    case Expose:
		if ((self->KeyFu & mHasFocus) && (self->CaptionFu & mFocusRecIsPaint)) {
		    XClearWindow(appXDat.dpy, self->WinID);
#ifdef _OS_WIN32
		    self->CaptionFu &= ~mFocusRecIsPaint;
		    return 1;
#endif /* ! _OS_WIN32 */
		}
		self->CaptionFu &= ~mFocusRecIsPaint;
		Refresh(self);
		return 1;

	    case ButtonPress:
		TWins_AppMouseDown(self, &ev->xbutton);
		return 1;

	    case ButtonRelease:
		if (self->DoMouseUp) self->DoMouseUp(self, &ev->xbutton);
		return 1;

	    case MotionNotify:
		TWins_ChkInfoWin(self, false);
		if (self->DoMouseMove) self->DoMouseMove(self, &ev->xmotion);
		return 1;

	    case EnterNotify:
	    case LeaveNotify:
#ifndef _OS_WIN32
		Send_MSG_MotionEvent();
#endif /* ! _OS_WIN32 */
		if (self->DoMouseOver) {
		    bool Leave = ev->type == LeaveNotify;
		    TWins_ChkInfoWin(self, Leave);
		    self->DoMouseOver(self, &ev->xcrossing, Leave);
		}
		return 1;

	    default:
		break;
	    }
	} else {
  TKeyEvent *KeyEv;
	    switch(msg->msg) {

	    case msgKey:
		KeyEv = (TKeyEvent*)msg->p;
		if (!TestWinIsAktiv(self)) {
		    if (!(self->KeyFu & mK_Ev_OnDisWins)) return -1;
		    KeyEv->keyst |= keyst_WinsIsDis;
		}
		if (KeyEv->keyst & keyst_Press) {
		    if (self->DoKeyDown) self->DoKeyDown(self, KeyEv);
		} else if (KeyEv->keyst & keyst_Release) {
		    if (self->DoKeyUp) self->DoKeyUp(self, KeyEv);
		} else return 0;
		return 1;

	    case msgPaint:
		Paint(self);
		return 1;

	    case msgFocus:
		if (self->DoFocus) self->DoFocus(self, msg->m);
		return 1;

	    case msgDestroy:
		Tfree(self);
		return 1;
#ifdef DRAG_AND_DROP
	    case msg_dnd:
// printf("%s(%d): Wins=0x%x f=%d\n", FL, (int)self->WinID, msg->f);
// fflush(stdout);
	      Hdnd = self->Hdnd;
	      if (Hdnd) {
		switch(msg->f) {
		case msgf_dndEnter: return 1;
		case msgf_dndPosition:
		    if (Hdnd->OndndMouseMove)
		      return Hdnd->OndndMouseMove(self, (XMotionEvent*)msg->p);
// printf("%s(%d): Wins=0x%x f=%d 0x%x\n", FL, (int)self, msg->f, self->DodndMouseMove);
		    break;
		case msgf_dndDrop: 
		    if (Hdnd->OndndDrop) 
		      return Hdnd->OndndDrop(self);
		    break;
		case msgf_dndStatus: 
		    if (Hdnd->OndndStatus) 
		      return Hdnd->OndndStatus(self, (XClientMessageEvent*)msg->p);
		    break;
		}
	      }
#endif
	    }
	}
	return 0;
}


bool WinsCtl(TWins *self, TWinsCtl *wctl) {
  bool res = false;
	if (self && wctl) {
	    if (self->WinsCtl) res = self->WinsCtl(self, wctl);
	    if (wctl->flag & wctlf_ptrFree) xfree(wctl->ptr);
	    if (wctl->flag & wctlf_SelfFree) xfree(wctl);
	}
	return res;
}

#ifdef USE_DECOR
bool TWins_Decor(TWins *self, int bef, int mk, void *p) {
  bool res = false;

	if (self->Decor && self->Decor->DoDecor) {
		res = self->Decor->DoDecor(self, bef, mk, p);
	}
	return res;
}
#endif

static void set_Align(TWins *self, TWRect *rc) {
	if (self->Visible) {
	  if (self->Align & alTop) {
	    rc->top += self->height;
	    rc->height -= self->height;
	  } else if (self->Align & alLeft) {
	    rc->left += self->width; 
	    rc->width -= self->width; 
	  } else if (self->Align & alRight) {
	    rc->width -= self->width; 
	  } else if (self->Align & alBottom) {
	    rc->height -= self->height;
	  }
	}
}

void TWins_GetAlignClientRect(TWins *self, TWRect *rc) {
  TWins *wins;

	Get_ClientRect(self->prev_child, rc);

	if (self->Align & (alTop | alLeft)) {
	    wins = self;
	    while (wins->prev) wins = wins->prev;

	    while (wins != self) {
		set_Align(wins, rc);
		wins = wins->next;
	    }
	    if (self->Align & alLeft) {
		wins = self->next;
		while (wins) {
		    if (wins->Visible && (wins->Align & alBottom)) {
			rc->height -= wins->height;
		    }
		    wins = wins->next;
		}
	    }

	} else if (self->Align & (alRight | alBottom)) {
	    wins = self;
	    while (wins->next) wins = wins->next;

	    while (wins != self) {
		set_Align(wins, rc);
		wins = wins->prev;
	    }
	} else if (self->Align & alClient) {
	    wins = self;
	    while (wins->prev) wins = wins->prev;

	    while (wins != self) {
		set_Align(wins, rc);
		wins = wins->next;
	    }
	    wins = self;
	    while (wins->next) wins = wins->next;

	    while (wins != self) {
		set_Align(wins, rc);
		wins = wins->prev;
	    }
	}
}

void TWins_GetClientRect(TWins *self, TWRect *rc) {
	rc->left   = self->BorderOffLeft;
	rc->width  = self->width - (self->BorderOffLeft + self->BorderOffRigth);
	rc->top    = self->BorderOffTop;
	rc->height = self->height - (self->BorderOffTop + self->BorderOffBottom);
}

void TWins_SetSize(TWins *self) {
  int mkPos, mkSize;

	if (!self->IsParent) return;
	TWins_ChkWinSizeMinMax(self, &self->width, &self->height);
	mkPos = 0;
	if (self->left != self->xwc.left) mkPos = CWX;
	if (self->top != self->xwc.top) mkPos |= CWY;
	mkSize = 0;
	if (self->width != self->xwc.width) mkSize = CWWidth;
	if (self->height != self->xwc.height) mkSize |= CWHeight;
	if (!mkPos && !mkSize) return;

	self->xwc.left = self->left;
	self->xwc.top = self->top;
	self->xwc.width = self->width;
	self->xwc.height = self->height;

	if (mkSize && mkPos) {
	    XMoveResizeWindow(appXDat.dpy, self->WinID,
			self->left, self->top, self->width, self->height);
	} else if (mkSize) {
	    XResizeWindow(appXDat.dpy, self->WinID, self->width, self->height);
	} else {
	    XMoveWindow(appXDat.dpy, self->WinID, self->left, self->top);
	}
//	if (mkSize && self->ResizeStartMk && self->OnResize) self->OnResize(self);
	if (mkSize && self->ResizeStartMk && self->DoResize) self->DoResize(self);
#ifdef USE_DECOR
	TWins_Decor(self, decor_NewSize, mkPos | mkSize, NULL);
#endif
} 

bool TWins_AdjustSize(TWins *self) {
  TWRect rc;
  bool result;

	if (self->IsParent && self->prev_child && self->Align &&
	    (self->prev_child->WinID != appXDat.RootW)) {

	    GetAlignClientRect(self, &rc);

	    self->left = rc.left;
	    self->top = rc.top;
	    if (self->Align & alTop) {
		self->width = rc.width;
	    } else if (self->Align & alLeft) {
		self->height = rc.height;
	    } else if (self->Align & alBottom) {
		self->top += rc.height - self->height; self->width = rc.width;
	    } else if (self->Align & alRight) {
		self->left += rc.width - self->width; self->height = rc.height;
	    } else if (self->Align & alClient) {
		self->width = rc.width; self->height = rc.height;
	    }
	}
	result = (self->width != self->xwc.width) || 
		 (self->height != self->xwc.height);
	TWins_SetSize(self);
	return result;
}

bool TWins_DoActivate(TWins *self) {
	if ((self->RunStatus & runst_Activate) ||
	    !self->IsParent || !TestWinIsAktiv(self)) return false;
	self->RunStatus |= runst_Activate;
	if (self->OnActivate) self->OnActivate(self);
	self->RunStatus |= runst_Activate1;
	return true;
}

void TWins_DoResize(TWins *self) {
	if (self->OnResize) self->OnResize(self);
}

void TWins_Resize(TWins *self) {
  TWins *wins;

	if (self->IsParent) {
	    self->AdjustSize(self);
//	    if (!self->ResizeStartMk && self->OnResize && self->Visible) {
//		self->OnResize(self);
	    if (!self->ResizeStartMk && self->Visible) {
		if (self->DoResize) self->DoResize(self);
		self->ResizeStartMk = true;
	    }
	    wins = self->next_child;
	    while (wins) {
		Resize(wins);
		wins = wins->next;
	    }
	}
	if (self->DoActivate) self->DoActivate(self);
}

void TWins_PaintEnCaption(TWins *self, bool En) {
  TVDrawString *vds = &self->vds;
  int ForeColor = vds->ForeColor;
  int fuEOL;

	if (self->CaptionFu & mCaptionAutoUtf8) {
	    fuEOL = 0;
	    if (Chk_fio_UTF8(vds->s, vds->lg, &fuEOL)) {
		vds->UTF8on = (fuEOL & (fio_has_UTF8_char|fio_err_UTF8_char)) == fio_has_UTF8_char;
	    }
	}
	vds->Image = ((self->CaptionFu & mCaptionSolid) != 0);

	if (self->OnPaintCaption && self->OnPaintCaption(self, En)) return;

	vds->BackColor = self->Color;
	if (!En) {
	    vds->ForeColor = AddColor(self->Color, 0x202020);
	    DrawColorString(1, 1, vds);
	    vds->ForeColor = SubColor(self->Color, 0x404040);
	}
	DrawColorString(0, 0, vds);
	vds->ForeColor = ForeColor;
}

void TWins_DoPaint(TWins *self) {
  int x, y, tw, m;
  int height;
  TWRect rc;
  TVDrawString *vds = &self->vds;

	if (!vds->lg && !(self->CaptionFu & mCaptionRePaint)) return;
	if ((self->CaptionFu & mCaptionPaint) && vds->Font) {
	    Font_GetMetrics((TFont*)vds->Font, &height, NULL);
	    if (!(self->CaptionFu & mCaptionNoAutoXY)) {
		Get_ClientRect(self, &rc);
		x = rc.left;
		y = rc.top;
		m = rc.height - height;
		if (self->CaptionFu & mCaptionBottom) {
		    y = rc.top + m;
		} else if ((self->CaptionFu & mCaptionTop) == 0) {
		    y = rc.top + (m >> 1);
		}
		vds->Yoff = self->CaptionOffY + y;

		dword flags = 0;
		if (vds->UTF8on) flags = FONTFG_UTF8;
		if (vds->Image)  flags |= FONTFG_ImageString;
		Font_SetFlags((TFont*)vds->Font, flags);
		tw = Font_GetWidth((TFont*)vds->Font, vds->s, vds->lg);

		if (self->CaptionFu & mCaptionRigth) {
		    x = rc.left + (rc.width - tw);
		} else if ((self->CaptionFu & mCaptionLeft) == 0) {
		    x = rc.left + ((rc.width - tw) >> 1);
		}
		if (self->CaptionFu & mCaptionRigth) {
		    x -= self->CaptionOffX;
		} else {
		    x += self->CaptionOffX;
		}
		vds->Xoff = x;
	    }
	    if (self->CaptionFu & mCaptionRePaint) {
		self->CaptionFu &= ~mCaptionRePaint;
		if (!(self->CaptionFu & mCaptionUseRepaint))
		    FillRectRGB(self->WinID, self->Color, rc.left, 
				vds->Yoff, rc.width, height);
	    }
	    TWins_PaintEnCaption(self, TestWinIsEnabled(self));
	}
}

void TWins_Paint(TWins *self) {
	self->LastPaintTime_us = GetTime_us();
	self->PaintOnEvEndMk = false;
	if (!self->IsParent || !self->Visible) return;
#ifdef _OS_WIN32
	SetTmp_hGC(self->WinID);
#endif /* _OS_WIN32 */
	if (!self->ResizeStartMk) {
	    Resize(self);
	    self->ResizeStartMk = true;
	} else {
	    if (self->DoActivate) self->DoActivate(self);
	}
	if (self->DoPaint) self->DoPaint(self);
	if (self->OnPaint) self->OnPaint(self);
#ifdef USE_DECOR
	if (!TWins_Decor(self, decor_Border, 0, NULL))
#endif
		StdBorderPaint(self->WinID, self->BorderFu, 0, 0, self->width, self->height);
	self->RunStatus |= runst_Paint1;
#ifdef _OS_WIN32
	ReleaseTmp_hGC(self->WinID);
#endif /* _OS_WIN32 */
}

void TWins_RePaint(TWins *self) {
	if (!TestWinIsVisible(self) || !self->IsParent) return;
	self->CaptionFu &= ~mFocusRecIsPaint;
	XClearWindow(appXDat.dpy, self->WinID);
	Refresh(self);
}


static void TWins_RePaintAllNext(TWins *self) {
	while (self) {
	    if (self->next_child) TWins_RePaintAllNext(self->next_child);
	    RePaint(self);
	    self = self->next;
	}
}

static void TWins_ClrAllRepTimerNext(TWins *self) {
	while (self) {
	    if (self->next_child) TWins_ClrAllRepTimerNext(self->next_child);
	    if (self->RepTimer) {
// printf("%s(%d): self->RepTimer = NULL\n", FL);
		ppTfree(&self->RepTimer);
	    }
	    self = self->next;
	}
}

void TWins_SetEnabled(TWins *self, bool Value) {
	if (self->Enabled != Value) {
	    if (!Value) {
		if (self->RepTimer) {
		    Tfree(self->RepTimer);
		    self->RepTimer = NULL;
		}
		TWins_ClrAllRepTimerNext(self->next_child);
	    }

	    self->Enabled = Value;
	    RePaint(self);
	    TWins_RePaintAllNext(self->next_child);
	}
}

static void Clr_Activate(TWins *self) {
	while (self) {
	    if (self->next_child) Clr_Activate(self->next_child);
	    self->RunStatus &= ~runst_Activate;
	    self = self->next;
	}
}

void TWins_SetVisible(TWins *self, bool Value) {
	if (self->Visible != Value) {
	    self->Visible = Value;
	    if (!self->IsParent) return;
	    if (self->Align && self->prev_child) {
		Resize(self->prev_child);
	    } else if (Value) {
		Resize(self);
	    }
	    if (!Value) {
		Clr_Activate(self->next_child);
		self->RunStatus &= ~runst_Activate;
	    }
	    MapWinID(self->WinID, Value ? 1 : 0);
	    if (self->OnVisible) self->OnVisible(self, Value);
	}
}

int TWins_ChkWinSizeMinMax(TWins *self, int *w, int *h) {
  int m, k, result;

	result = 0;
	if (w) {
	    m = *w;
	    k = self->min_width;
	    if (k <= 0) k = 1;
	    if (m < k) {
		m = k;
		result = cWinsMinWidth;
	    }
	    k = self->max_width;
	    if ((k <= 0) || (k > appXDat.Root_width)) k = appXDat.Root_width;
	    if (m > k) {
		m = k;
		result |= cWinsMaxWidth;
	    }
	    *w = m;
	}
	if (h) {
	    m = *h;
	    k = self->min_height;
	    if (k <= 0) k = 1;
	    if (m < k) {
		m = k;
		result |= cWinsMinHeight;
	    }
	    k = self->max_height;
	    if ((k <= 0) || (k > appXDat.Root_height)) k = appXDat.Root_height;
	    if (m > k) {
		m = k;
		result |= cWinsMaxHeight;
	    }
	    *h = m;
	}
	return result;
}

void TWins_SetRC(TWins *self, int left, int top, int width, int height) {
	TWins_ChkWinSizeMinMax(self, &width, &height);

	if ((self->width != width) || (self->height != height)) {
	    self->left = left;
	    self->top = top;
	    self->width = width;
	    self->height = height;
	    Resize(self);
	    if (self->IsParent && self->Align && self->prev_child) Resize(self->prev_child);
	    AppSetWait1_single();

	} else if ((self->left != left) || (self->top != top)) {
	    self->left = left;
	    self->top = top;
	    AdjustSize(self);
	    AppSetWait1_single();
	}
}


void TWins_DoClick(TWins *self, long KeyState, bool DblClick) {
  if (self->OnClick && TestWinIsEnabled(self)) self->OnClick(self, KeyState, DblClick);
}

void TWins_DoMouseDown(TWins *self, XButtonEvent *ev) {
  int m;

	self->MouseClickStop = false;
	if (self->OnMouseDown) self->OnMouseDown(self, ev);

// printf("%s(%d): DoMouseDown 0x%02x 0x%02x 0x%08x\n", FL, ev->state, ev->button, (int)self->WinID);
	if (TestWinIsEnabled(self)) {
	    if (ev->button == 1) {
		m = ev->time - MouseDown.time_ev;
		MouseDown.DblClickAkt = ((m < 350) && 
			(MouseDown.oldapph == self));
	    }
	}
	MouseDown.time_ev = ev->time;
}

void TWins_DoMouseUp(TWins *self, XButtonEvent *ev) {
  bool UpDownOK;
#ifndef _OS_WIN32
  int jx, jy;
  XID UpWin, Jwin;
#endif

// printf("%s(%d): DoMouseUp   0x%02x 0x%02x  0x%08x 0x%08x\n", FL, ev->state, ev->button, (int)MouseDown.apph, (int)self->WinID);

#ifdef DRAG_AND_DROP
  Tdnd *dnd = appXDat.dnd;
	if (dnd && (dnd->dndM_SrcWin != 0) && dnd->MoveSet) {
	    dnd->MoveSet(dndM_Move_Drop, 0, 0, 0);
	    return;
	}
#endif

#ifdef _OS_WIN32
	UpDownOK = (MouseDown.apph == self)
		   && (MouseDown.button == ev->button)
		   && !(MouseDown.state & 0x700);
#else

	UpWin = Jwin = appXDat.RootW;
	while (Jwin && XTranslateCoordinates(appXDat.dpy, appXDat.RootW, Jwin, 
	    ev->x_root, ev->y_root, &jx, &jy, &Jwin)) {
	    if (Jwin) UpWin = Jwin;
// printf("%s(%d): DoMouseUp %dx%d %dx%d %dx%d 0x%08x -> 0x%08x\n", FL, ev->x, ev->y, ev->x_root, ev->y_root, jx, jy, (int)self->WinID, (int)Jwin);
	}

	UpDownOK = (MouseDown.apph == self) 
		   && (MouseDown.button == ev->button)
		   && !(MouseDown.state & 0x700)
		   && (self->WinID == UpWin);
#endif
	MouseDown.button = 0;
	if (TestWinIsEnabled(self)) {
	    if (ev->button == 1) {
		if (!self->MouseClickStop) {
		    if (UpDownOK && self->DoClick) {
			bool DblClick = MouseDown.DblClickAkt && 
				  ((ev->time - MouseDown.time_ev) < 500);
			self->DoClick(self, ev->state, DblClick);
		    }

		    MouseDown.DblClickAkt = false;
		    /* chk self is Destroy */
		    if (!TWins_ChkExists(self)) return;
		}

	    } else if (ev->button == 3) {
// printf("%s(%d): Popup UpDownOK=%d\n", FL, UpDownOK);
		if (UpDownOK && self->PopupMenu) {
		    self->PopupMenu->owner = self;
#ifdef Test_Root_WinId
		    if (TestRootWinId) {
			XTranslateCoordinates(appXDat.dpy, TestRootWinId, OrgRootWinId, 0, 0,
			    &jx, &jy, &Jwin);
			ev->x_root -= jx;
			ev->y_root -= jy;
		    }
#endif
		    TMenu_Show(self->PopupMenu, self, ev->x_root, ev->y_root);
		}
	    }
	}

	if (self->OnMouseUp) self->OnMouseUp(self, ev);
}

void TWins_DoMouseMove(TWins *self, XMotionEvent *ev) {
	if (self->OnMouseMove) self->OnMouseMove(self, ev);
}

void TWins_DoMouseOver(TWins *self, XCrossingEvent *ev, bool Leave) {
	if (self->OnMouseOver) self->OnMouseOver(self, ev, Leave);
}

int TWins_DoKeyDown(TWins *self, TKeyEvent *KeyEv) {
  TWins *ws;
  int ks;

	if (self->OnKeyDown) self->OnKeyDown(self, KeyEv);
// printf("TWins_DoKeyDown: win=%d xkey=%d(0x%x) c=%d(0x%x)\n", (int)self->WinID, ev->keycode, ev->keycode, (int)*keysym, (int)*keysym);
	/* chk self is Destroy */
	if (!TWins_ChkExists(self)) return 1;

	if ((KeyEv->state & (ShiftMask | ControlMask))) return 0;

	ks = KeyEv->keysym;
	if (((ks == XK_Right) && (self->KeyFu & mK_Right)) ||
	    ((ks == XK_Down) && (self->KeyFu & mK_Down)) ||
	    ((ks == XK_Return) && (self->KeyFu & mK_ReturnTab)) ||
	    ((ks == XK_Tab) && (self->KeyFu & mK_Tab))) {
		ws = GetNextFocEl(self, false, ks == XK_Tab);
		if (ws) Set_Focus(ws);
		return 1;

	} else if (((ks == XK_Left) && (self->KeyFu & mK_Left)) ||
		   ((ks == XK_Up) && (self->KeyFu & mK_Up))) {
		ws = GetNextFocEl(self, true, false);
		if (ws) Set_Focus(ws);
		return 1;
	}
	return 0;
}

int TWins_DoKeyUp(TWins *self, TKeyEvent *KeyEv) {

	if (self->OnKeyUp) self->OnKeyUp(self, KeyEv);
	return 1;
}

void TWins_DoFocus(TWins *self, int Change) {
  int WinsBase;

	if (Change & WinsFocusIN) {
	    TForm *tmpWin = GetParentForm(self);
	    appXDat.KeyInWin = self;
	    if (tmpWin) {
		tmpWin->FocusWin = self;
		WinsBase = self->wtyp & wtyp_WinsBaseMask;
		if ((WinsBase != wtyp_MenuButton) && (WinsBase != wtyp_Menu)) {
		    tmpWin->LastNoMenuFocusWin = self;
		}
	    }
	    self->KeyFu |= mHasFocus;
	} else {

	    self->KeyFu &= ~mHasFocus; 
	}
	if (self->OnFocus) self->OnFocus(self, Change);
}

void Set_Focus(TWins *self) {
  TForm *tmpWin;
  Tobjmsg msg;

	if (!TestWinIsEnabled(self)) return;
	if ((self->KeyFu & (mCanFocus | mHasFocus)) != mCanFocus) return;

	tmpWin = GetParentForm(self);
	msg.msg = msgFocus;
	msg.m = WinsFocusChange;
	if (tmpWin && tmpWin->FocusWin) {
	    Exec_Win_Ev(&msg, tmpWin->FocusWin->WinID);  //Focus Out
	}
	msg.m = WinsFocusIN|WinsFocusChange;
	Exec_Win_Ev(&msg, self->WinID);
}

void vds_SetStr(TVDrawString *vds, const char *s) {
	ppcchange(&vds->s, s);
	vds->lg = pclen(s);
	vds->scda = NULL;
}

void TWins_SetCaption(TWins *self, const char *s) {
	if (pccmp(s, self->vds.s)) return;
	self->CaptionFu |= mCaptionRePaint;
	vds_SetStr(&self->vds, s);
	if (self->OnSetCaption) self->OnSetCaption(self, s);
	if (self->vds.lg && !(self->CaptionFu & mCaptionUseRepaint)) {
	    Refresh(self); 
	} else {
	    RePaint(self);
	}
}

void TWins_SetUTF8(TWins *self, bool UTF8) {
  TWinsCtl wctl;

	wctl.bef = wctlb_SetUTF8;
	wctl.flag = 0;
	wctl.ic.c[0] = UTF8;
	WinsCtl(self, &wctl);
}

int TWins_Scale(TWins *self, float fx, float fy) {
  TWinsCtl wctl;

	wctl.bef = wctlb_Scale;
	wctl.flag = 0;
	wctl.ic.f[0] = fx;
	wctl.ic.f[1] = fy;
	WinsCtl(self, &wctl);
	return wctl.flag;
}

static void TWins_ScaleNextChild(TWins *self, float fx, float fy) {
 dword flag;

	while (self) {
	    flag = TWins_Scale(self, fx, fy);
	    if (!(flag & SCALERESULT_SKIP_NEXT)) {
		TWins_ScaleNextChild(self->next_child, fx, fy);
	    }
	    self = self->next;
	}
}

void TWins_ScaleAll(TWins *self, float fx, float fy) {
 dword flag;

	if (self) {
	    flag = TWins_Scale(self, fx, fy);
	    if (!(flag & SCALERESULT_SKIP_NEXT)) {
		TWins_ScaleNextChild(self->next_child, fx, fy);
	    }
	}
}

void TWins_ScaleAllNextChild(TWins *self, float fx, float fy) {
	if (self) TWins_ScaleNextChild(self->next_child, fx, fy);
}

void TWins_SetUTF8Auto(TWins *self, bool UTF8Auto) {
  TWinsCtl wctl;

	wctl.bef = wctlb_CaptionFu;
	wctl.flag = 0;
	wctl.ic.dw[0] = mCaptionAutoUtf8;
	wctl.ic.dw[1] = CtlCaptionFu_Clr;
	if (UTF8Auto) wctl.ic.dw[1] = CtlCaptionFu_Set;
	WinsCtl(self, &wctl);
}

void TWins_SetColor(TWins *self, dword Color) {
  XSetWindowAttributes XAttr;
	if (self->Color != Color) {
	    self->Color = Color;
	    Fill0(&XAttr, sizeof(XSetWindowAttributes));
	    XAttr.background_pixel = GetColor_RGB(Color);
	    XChangeWindowAttributes(appXDat.dpy, self->WinID, CWBackPixel, &XAttr);
	    RePaint(self);
	}
	self->vds.BackColor = Color;
}

static void SetBorderLRTB(TWins *self) {
  int m = 0;

	if (self->BorderFu & mBorder) {
	    m = 1;
	    if (self->BorderFu & mBorderW2) m++;
	}
	TWins_SetBorderWidth(self, m);
}

void TWins_SetBorder(TWins *self, int BorderFu) {
	if (self->BorderFu != BorderFu) {
	    self->BorderFu = BorderFu;
	    SetBorderLRTB(self);
	    AdjustSize(self);
//	    Refresh(self);
	}
}

bool TWins_Ctl(TWins *self, TWinsCtl *wctl) {
  TFontX *font;
  int i;

	if (wctl->bef == wctlb_SetFont) {
	    font = self->vds.Font;
	    self->vds.Font = MakeFont(wctl->ptr, FontX_GetFontName(appXDat.Default_Font));
	    Tfree(font);
	    return true;

	} else if (wctl->bef == wctlb_SetUTF8) {
	    if (self->vds.UTF8on != wctl->ic.c[0]) {
		self->vds.UTF8on = wctl->ic.c[0];
		RePaint(self);
	    }
	    return true;

	} else if (wctl->bef == wctlb_CaptionFu) {
	    dword value = self->CaptionFu;
	    if (wctl->ic.dw[1] == CtlCaptionFu_Set) {
		value |= wctl->ic.dw[0];
	    } else if (wctl->ic.dw[1] == CtlCaptionFu_Clr) {
		value &= ~wctl->ic.dw[0];
	    } else {
		value = wctl->ic.dw[0];
	    }
	    if (self->CaptionFu != value) {
		self->CaptionFu = value;
		Refresh(self);
	    }
	    return true;

	} else if (wctl->bef == wctlb_Scale) {
	    if (!self->Scale) {
		self->Scale = xcalloc(sizeof(TScale));
		self->Scale->ScaleType = SCALETYP_BASE;
		self->Scale->left   = self->left;
		self->Scale->top    = self->top;
		self->Scale->width  = self->width;
		self->Scale->height = self->height;
		self->Scale->fx = 1;
		self->Scale->fy = 1;
	    }
	    i = 0;
	    if (self->OnScale) {
		i = self->OnScale(self, wctl->ic.f[0], wctl->ic.f[1]);
	    }
	    if (!(i & SCALERESULT_NO_FX_FY)) {
		self->Scale->fx = wctl->ic.f[0];
		self->Scale->fy = wctl->ic.f[1];
	    }
	    if (!(i & SCALERESULT_NO_SetRC)) {
// printf("%s(%d): %f %f\n", FL, self->Scale->fx, self->Scale->fy);
//		if (!(self->Align & alClient)) {
		    SetRC(self, (int)((float)self->Scale->left   * self->Scale->fx),
				(int)((float)self->Scale->top    * self->Scale->fy),
				(int)((float)self->Scale->width  * self->Scale->fx),
				(int)((float)self->Scale->height * self->Scale->fy));
//		}
	    }
	    wctl->flag = i;
	    return true;
	}
	return false;
}

void Set_Parent(TWins *self, TWins *Parent) {
	if (self && self->SetParent) self->SetParent(self, Parent);
}

bool Get_IsParent(TWins *self) {
	if (!self) return false;
	if (self->GetIsParent) return self->GetIsParent(self);
	return self->IsParent;
}

void SetCaption(TWins *self, const char *s) {
	if (self && self->SetCaption) self->SetCaption(self, s);
}

const char *GetCaption_pc(TWins *self) {
	if (!self) return NULL;
	if (self->GetCaption) return self->GetCaption(self);
	return self->vds.s;
}

void SetColor(TWins *self, dword col) {
	if (self && self->SetColor) self->SetColor(self, col);
}

void SetBorder(TWins *self, int BorderFu) {
	if (self && self->SetBorder) self->SetBorder(self, BorderFu);
}

void SetFont(TWins *self, const char *name) {
  TWinsCtl wctl;
	if (!self) return;
	wctl.bef = wctlb_SetFont;
	wctl.flag = 0;
	wctl.ptr_s_const = name;
	WinsCtl(self, &wctl);
}

void SetEnabled(TWins *self, bool Value) {
	if (self && self->SetEnabled) self->SetEnabled(self, Value);
}

void SetVisible(TWins *self, bool Value) {
	if (self && self->SetVisible) self->SetVisible(self, Value);
}

void Set_Cursor(TWins *self, int Value) {
  XSetWindowAttributes XAttr;

	if (!self) return;
	if (self->cursor_nr != Value) {
	    self->cursor_nr = Value;
	    Fill0(&XAttr, sizeof(XSetWindowAttributes));
	    XAttr.cursor = appXDat.XCursors[Value];
	    XChangeWindowAttributes(appXDat.dpy, self->WinID, CWCursor, &XAttr);
	}
}

void SetPosition(TWins *self, int left, int top) {
	if (self && self->SetRC) self->SetRC(self, left, top, self->width, self->height);
}
void SetLeft(TWins *self, int left) {
	if (!self) return;
	SetPosition(self, left, self->top);
}
void SetTop(TWins *self, int top) {
	if (!self) return;
	SetPosition(self, self->left, top);
}
void SetWinSize(TWins *self, int Width, int Height) {
	if (!self) return;
	if (self->SetRC) self->SetRC(self, self->left, self->top, Width, Height);
}
void SetWidth(TWins *self, int Width) {
	if (!self) return;
	SetWinSize(self, Width, self->height);
}
void SetHeight(TWins *self, int Height) {
	if (!self) return;
	SetWinSize(self, self->width, Height);
}
void SetRC(TWins *self, int left, int top, int width, int height) {
	if (self && self->SetRC) self->SetRC(self, left, top, width, height);
}

void Paint(TWins *self) {
	if (self && self->Paint) self->Paint(self);
}

void RePaint(TWins *self) {
	if (self && self->RePaint) self->RePaint(self);
}

void Refresh(TWins *self) {
	if (self && self->Refresh) self->Refresh(self);
}

void Resize(TWins *self) {
	if (self && self->Resize) self->Resize(self);
}

void Get_ClientRect(TWins *self, TWRect *rc) {
	if (self && self->GetClientRect) self->GetClientRect(self, rc);
}

void GetAlignClientRect(TWins *self, TWRect *rc) {
	if (self && self->GetAlignClientRect) self->GetAlignClientRect(self, rc);
}

void AdjustSize(TWins *self) {
	if (self && self->AdjustSize) self->AdjustSize(self);
}

void *GetParentWins(TWins *self) {
	if (!self) return NULL;
	return self->prev_child;
}

void *GetParentWins_wtyp(TWins *self, int wtyp) {
	while (self) {
	    self = self->prev_child;
	    if (!self || 
		(self->magic != TTC_MAGIC) || 
		(self->wtcsize < sizeof(TWins))) return NULL;
	    if (self->wtyp == wtyp) return self;
	}
	return NULL;
}

void *GetParentForm(TWins *self) {
  TWins *wins = self;
	if (wins && !(wins->wtyp & wtyp_Form)) wins = wins->FormStruct;
	if (wins && !(wins->wtyp & wtyp_Form)) {
	    printf("%s(%d): ERROR: FormStruct wtyp=0x%06x not wtyp_Form\n", FL, wins->wtyp);
	    wins = NULL;
	}
	return wins;
}

bool TestWinIsEnabled(TWins *self) {
  TWins *wins = self;
	if (!wins) return false;
	while (wins->Enabled && wins->prev_child && 
		!(wins->wtyp & wtyp_Form)) wins = wins->prev_child;
	return wins->Enabled;
}

bool TestWinIsVisible(TWins *self) {
  TWins *wins = self;
	if (!wins) return false;
	while (wins->Visible && wins->prev_child && 
	       !(wins->wtyp & wtyp_Form)) wins = wins->prev_child;
	return wins->Visible;
}

bool TestWinIsAktiv(TWins *self) {
  TWins *wins = self;
	if (!wins) return false;
	while (wins->Enabled && wins->Visible && wins->prev_child && 
	       !(wins->wtyp & wtyp_Form)) wins = wins->prev_child;
	return (wins->Enabled && wins->Visible);
}

bool TestWinCanFocus(TWins *self) {
	return (TestWinIsAktiv(self) && (self->KeyFu & mCanFocus));
}

void TWins_SetBorderWidth(TWins *self, int BorderWidth) {
  int fu;
	if (!self) return;
	fu = self->BorderFu;
	if (!(fu & mBorderNoLeft))   self->BorderOffLeft = BorderWidth;
	if (!(fu & mBorderNoRigth))  self->BorderOffRigth = BorderWidth;
	if (!(fu & mBorderNoTop))    self->BorderOffTop = BorderWidth;
	if (!(fu & mBorderNoBottom)) self->BorderOffBottom = BorderWidth;
}

void TWins_SetDefaultBorderWidth(TWins *self) {
	if (!self) return;
	if (self->BorderOffLeft | self->BorderOffRigth | 
	    self->BorderOffTop | self->BorderOffBottom) return;
	SetBorderLRTB(self);
}
/*____________________________ Destroy __________________________________*/
bool TWins__Destroy(TWins *self) {
  TForm *form;

  if (self->RunStatus & runst_GoDestroy) {
     printf("TWins__Destroy: win=0x%x wtyp=0x%x\n", (unsigned int)self->WinID, self->wtyp);  
  }

	self->RunStatus |= runst_GoDestroy;
	TNode__Destroy((TNode*)self);
	TWinsDeregister(self);

	if (self->OnDestroy) self->OnDestroy(self);
	if (appXDat.InfoWin == self) appXDat.InfoWin = NULL;
	form = GetParentForm(self);
	if (form && (form->FocusWin == self)) form->FocusWin = NULL;
	self->vds.lg = 0;
	xfree(self->vds.s);
	Tfree(self->vds.Font);
	if (appXDat.dpy) XDestroyWindow(appXDat.dpy, self->WinID);
	Tfree(self->InfoWin);
#ifdef USE_DECOR
	TWins_Decor(self, decor_Destroy, 0, NULL);
#endif
	return false;
}

/*____________________________ Create ___________________________________*/
void TWins_ReParent(TWins *self, TWins *Parent, int left, int top) {
	if (!self || !Parent) return;
	TNodeReMove((TNode*)self);
	TWins_Add(self, Parent);
	XReparentWindow(appXDat.dpy, self->WinID, Parent->WinID, left, top);
}

void TWins_SetParent(TWins *self, TWins *Parent) {

	if (!self) return;
	if ((self->prev_child != Parent) && Parent) {
	    TWins_ReParent(self, Parent, self->left, self->top);
	}

	if (!self->IsParent) {
	    self->IsParent = true;
	    TWins_SetDefaultBorderWidth(self);
	    AdjustSize(self);
	    if (self->Visible) MapWinID(self->WinID, 2);
	}
}

void TWins_CreateXWin(TWins_Create_struct *wcs) {
  TWins *self = wcs->wins;
	wcs->XAttr.background_pixel = GetColor_RGB(self->Color);
	wcs->XAttr.cursor = appXDat.XCursors[self->cursor_nr];

	self->xwc.left = self->left;
	self->xwc.top = self->top;
	self->xwc.width = self->width;
	self->xwc.height = self->height;

	self->WinID = XCreateWindow (appXDat.dpy, wcs->ParentWinID, 
			self->left, self->top, self->width, self->height,
			0, CopyFromParent, InputOutput,
			CopyFromParent, wcs->XMask, &wcs->XAttr);
	self->vds.WinID = self->WinID;
// printf("%s(%d): 0x%x\n", FL, self->vds.WinID);
#ifdef _OS_WIN32
	self->OS_var.background_pixmap = wcs->XAttr.background_pixmap;
	self->OS_var.OverrideRedirect = ((wcs->XMask & CWOverrideRedirect) && 
					  wcs->XAttr.override_redirect);
#endif
	TWinsRegistration(self);
}

void TWins_Add(TWins *self, TWins *Parent) {
  TWins *prev;

	if (Parent) {
	    self->pproot = &Parent->next_child;
	} else {
	    self->pproot = &appXDat.Wins;
	}
	prev = *self->pproot;
	if (prev) while (prev->next) prev = prev->next;
	TNodeIns((TNode*)Parent, (TNode*)prev, (TNode*)self);
}


void TWins_DefaultConf(TWins_Create_struct *wcs) {
  TWins *self;
  TWins *Parent;

	self = wcs->wins;
	self->destroy = (TDESTROY)TWins__Destroy;
	self->EvProc = TWins_EvProc;

	Parent = wcs->Parent;
	TWins_Add(self, Parent);

	if (Parent) {
	    wcs->ParentWinID = Parent->WinID;
	    self->Color = Parent->Color;
	    self->subClass = Parent->subClass;
	    self->FormStruct = GetParentForm(Parent);
	    self->vds.Font = Parent->vds.Font;
	    self->vds.ForeColor = Parent->vds.ForeColor;
	    self->vds.BackColor = Parent->vds.BackColor;
	    self->vds.UTF8on = Parent->vds.UTF8on;
	    self->cursor_nr =  Parent->cursor_nr;

	    if (TWins_GlobalCreateFlags & TWGCF_PARENT_POPUPMENU) {
		self->PopupMenu = Parent->PopupMenu;
	    }

	} else {
	    self->cursor_nr = XC_top_left_arrow;
	    wcs->ParentWinID = appXDat.RootW;
	    self->Color = appXDat.Default_Color;
	    self->vds.Font = appXDat.Default_Font;
#ifdef _OS_WIN32
	    self->vds.UTF8on = true;
#endif
	}
	self->vds.Wins = self;

	if (self->width <= 0) self->width = 32;
	if (self->height <= 0) self->height = 16;
	self->Align = alNone;
	self->Enabled = true;
	self->Visible = true;

	self->SetParent = TWins_SetParent;
	self->SetCaption = TWins_SetCaption;
	self->SetColor = TWins_SetColor;
	self->SetBorder = TWins_SetBorder;
	self->SetEnabled = TWins_SetEnabled;
	self->SetVisible = TWins_SetVisible;


	self->SetRC = TWins_SetRC;
	self->Refresh = TWins_Refresh;
	self->WinsCtl = TWins_Ctl;

	self->GetClientRect = TWins_GetClientRect;
	self->GetAlignClientRect = TWins_GetAlignClientRect;
	self->AdjustSize = TWins_AdjustSize;
	self->Resize = TWins_Resize;
	self->DoResize = TWins_DoResize;
	self->Paint = TWins_Paint;
	self->RePaint = TWins_RePaint;

	self->DoPaint = TWins_DoPaint;
	self->DoClick = TWins_DoClick;
	self->DoMouseDown = TWins_DoMouseDown;
	self->DoMouseUp = TWins_DoMouseUp;
	self->DoMouseMove = TWins_DoMouseMove;
	self->DoMouseOver = TWins_DoMouseOver;
	self->DoKeyDown = TWins_DoKeyDown;
	self->DoKeyUp = TWins_DoKeyUp;
	self->DoFocus = TWins_DoFocus;
	self->DoActivate = TWins_DoActivate;

#ifdef DRAG_AND_DROP
//	self->DodndMouseMove = TWins_DodndMouseMove;
//	self->DodndDrop = TWins_DodndDrop;
//	self->DodndStatus = TWins_DodndStatus;
#endif

	wcs->XMask = CWBackPixel | CWEventMask | CWCursor; // or CWSaveUnder;

	wcs->XAttr.event_mask = ButtonPressMask | ButtonMotionMask |
                      ButtonReleaseMask | PointerMotionMask | EnterWindowMask |
                      LeaveWindowMask | ExposureMask | KeyPressMask | KeyReleaseMask |
                      StructureNotifyMask;

	if (wcs->WCSfkt & WCSfkt_OverrideRedirect) {
	    wcs->XMask |= CWOverrideRedirect;
	    wcs->XAttr.override_redirect = 1;
	}

	if (wcs->CreateParams) wcs->CreateParams(wcs);

	TWins_SetDefaultBorderWidth(self);
	
	if (wcs->CreateXWin) wcs->CreateXWin(wcs);
}

void TWins_Create_struct_init(TWins_Create_struct *wcs, int tcsize, int wtyp) {
	Fill0(wcs, sizeof(TWins_Create_struct));
	wcs->CreateXWin = TWins_CreateXWin;
	wcs->wins = ttc_create(tcsize, wtyp);
}

TWins *TWins_Create(TWins *Parent) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TWins), wtyp_Wins);
	wcs.Parent = Parent;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}

TWins *TWins_Create_wh(TWins *Parent, int Left, int Top, int Width, int Height) {
  TWins *self;
	self = TWins_Create(Parent);
	self->top = Top;
	self->left = Left;
	self->width = Width;
	self->height = Height;
	return self;
}

TWins *TWins_CreateB(TWins *Parent, int Left, int Top, int Width, int Height, int BorderFu) {
  TWins *self;
	self = TWins_Create(Parent);
	self->BorderFu = BorderFu;
	SelfCreateS(self, Left, Top, Width, Height, NULL);
	return self;
}

TWins *TWins_CreateBA(TWins *Parent, int Left, int Top, int Width, int Height, int BorderFu, int Align) {
  TWins *self;
	self = TWins_Create(Parent);
	self->BorderFu = BorderFu;
	self->Align = Align;
	SelfCreateS(self, Left, Top, Width, Height, NULL);
	return self;
}

void REGP3 SelfCreateS(TWins *self, int Left, int Top, int Width, int Height, const char *s) {
	self->top = Top;
	self->left = Left;
	self->width = Width;
	self->height = Height;
	if (s) {
	    self->CaptionFu |= mCaptionPaint;
	    SetCaption(self, s);
	}
	Set_Parent(self, self->prev_child);
}
