/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"

//*********************** Button ************************

void TButton_DoPaint(TButton *self) {
  int w = 2;
	TWins_DoPaint((TWins*)self);
	if ((self->KeyFu & mHasFocus) && !(self->CaptionFu & mFocusRecIsPaint)) {
#ifdef USE_DECOR
	  if (!TWins_Decor((TWins*)self, decor_FocusIn, 0, &w))
#endif
	    DrawRectFocus(self->WinID, w, w, self->width - (w + w), self->height - (w + w));
	    self->CaptionFu |= mFocusRecIsPaint;
	}
}

int TButton_DoKeyDown(TButton *self, TKeyEvent *KeyEv) {
	if ((KeyEv->keysym == XK_space) || (KeyEv->keysym == XK_Return)) {
	    self->BorderFu |= mBorderL;
	    Refresh((TWins*)self);
	}
	return TWins_DoKeyDown((TWins*)self, KeyEv);
}

int TButton_DoKeyUp(TButton *self, TKeyEvent *KeyEv) {
	if ((KeyEv->keysym == XK_space) || (KeyEv->keysym == XK_Return)) {
	    if ((self->BorderFu & mBorderL) && self->DoClick)
		    self->DoClick(self, KeyEv->state, False);
	    if (!TWins_ChkExists((TWins*)self)) return 1;
	    self->BorderFu &= ~mBorderL;
	    Refresh((TWins*)self);
	}
	return TWins_DoKeyUp((TWins*)self, KeyEv);
}

void TButton_DoMouseDown(TButton *self, XButtonEvent *ev) {
	if ((ev->button == 1) && TestWinIsEnabled((TWins*)self)) {
	    self->BorderFu |= mBorderL;
	    Refresh((TWins*)self);
	}
	TWins_DoMouseDown((TWins*)self, ev);
}

void TButton_DoMouseUp(TButton *self, XButtonEvent *ev) {
	if ((ev->button == 1) && TestWinIsEnabled((TWins*)self)) {
	    self->BorderFu &= ~mBorderL;
	    Refresh((TWins*)self);
	}
	TWins_DoMouseUp((TWins*)self, ev);
}

void TButton_DoFocus(TButton *self, int Change) {
	TWins_DoFocus((TWins*)self, Change);
	if (!(Change & WinsFocusIN)) self->BorderFu &= ~mBorderL;
	TWins_RePaint((TWins*)self);
}

/*____________________________ Create ___________________________________*/
void TButton_CreateParams(TWins_Create_struct *wcs) {
  TButton *self = wcs->wins;
#ifdef USE_DECOR
  TDecor *Decor;
#endif

	self->DoPaint = TButton_DoPaint;
	self->DoMouseDown = TButton_DoMouseDown;
	self->DoMouseUp = TButton_DoMouseUp;
	self->DoFocus = TButton_DoFocus;
	self->DoKeyDown = TButton_DoKeyDown;
	self->DoKeyUp = TButton_DoKeyUp;

	self->CaptionFu = mCaptionPaint;
	self->KeyFu = mCanFocus | mK_Right | mK_Down | mK_Left | mK_Up | mK_Tab;
	self->BorderFu = mBorder | mBorderW2 | mBorderBu;

#ifdef USE_DECOR
	Decor = appXDat.DefaultButtonDecor;
	if (Decor && Decor->CreateDup) {
	      self->Decor = Decor->CreateDup(&appXDat.DefaultButtonDecor);
	}
#endif
}

TButton *TButton_Create(TWins *Parent) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TButton), wtyp_Button);
	wcs.Parent = Parent;
	wcs.CreateParams = TButton_CreateParams;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}

TButton *TButton_CreateS(TWins *Parent, int Left, int Top, int Width, int Height, const char *s) {
  TButton *self;

	self = TButton_Create(Parent);
	SelfCreateS((TWins*)self, Left, Top, Width, Height, s);
	return self;
}
