/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "cicons.h"

//*********************** TComboBox ************************

void TComboBox_Resize(TComboBox *self) {
  TWRect rc;
  TMenu *me = self->popup;

	TWins_Resize((TWins*)self);
	if (self->IsParent) {
	    me->min_width = self->width;
	    me->max_width = self->width;
	    me->min_height = me->ItemMinHeight;
	    Get_ClientRect((TWins*)self, &rc);
	    SetRC(self->button, rc.left + rc.width - 14, rc.top-1, 15, rc.height+2);
	}
}


static void TComboBoxBu_Paint(TWins *self) {
  int x, y;

	TWins_Paint(self);
	x = (self->width-9) >> 1;
	y = (self->height-9) >> 1;
	DrawPic(self->WinID, get_pic9_vsb_plus(False), x, y);
}

static void TComboBoxPopup_DoMenuOpen(TMenu *me) {
  TComboBox *cb = (TComboBox*)me->owner;
  TMenuItem *mi;

	if (cb->OnPopUp) cb->OnPopUp(cb, me);
	if (cb->SelItem) {
	    mi = me->MenuItem;
	    while (mi) {
		if (mi == cb->SelItem) {
		    me->SelItem = mi;
		    break;
		}
		mi = mi->next;
	    }
	}
	TMenu_DoMenuOpen(me);
}

static void TComboBoxBu_DoMouseDown(TWins *self, XButtonEvent *ev) {
  TComboBox *cb = (TComboBox*)self->prev_child;
  TMenu *me = cb->popup;
  TWRect rc;

	if ((ev->button == 1) && TestWinIsEnabled(self)) {
	    self->BorderFu |= mBorderL;
	    Refresh(self);
	    GetXWinAbs_rc(cb->WinID, &rc);
	    TMenu_Show(me, (TWins*)cb, rc.left, rc.top + cb->height);
	}
	TWins_DoMouseDown(self, ev);
}

static void Popup_OnClick(TMenu *self, long KeyState, bool DblClick) {
  TComboBox *cb;
  TMenuItem *SelItem = self->SelItem;

	if (SelItem) {
	    cb = (TComboBox*)self->owner;
	    cb->SelItem = SelItem;
	    SetCaption((TWins*)cb, SelItem->s);
	    if (cb->OnItemSel) cb->OnItemSel(cb, SelItem);
	}
}

static void TComboBoxPopup_OnMenuClose(TMenu *self) {
  TComboBox *cb = (TComboBox*)self->owner;
	cb->button->BorderFu &= ~mBorderL;
	Refresh(cb->button);
}

TMenuItem *TComboBox_AddItem(TComboBox *self, const char *s, long fu, void (*OnClick)(struct TMenu*, long, struct TMenuItem*)) {
  TMenuItem *mi = TMenu_AddItem(self->popup, s, fu, OnClick);
	if (fu & miAdd_setSel) {
	    self->SelItem = mi;
	    SetCaption((TWins*)self, s);
	}
	return mi;
}

void TComboBox_DelItem(TComboBox *self, TMenuItem *mi) {
	if (self->SelItem == mi) self->SelItem = NULL;
	TMenu_DelItem(self->popup, mi);
}

void TComboBox_DelItemS(TComboBox *self, const char *s) {
  TMenuItem *mi = self->popup->MenuItem;
  TMenuItem *tmpmi;

	while (mi) {
	    tmpmi = mi->next;
	    if (pccmp(s, mi->s)) TComboBox_DelItem(self, mi);
	    mi = tmpmi;
	}
}

TMenuItem *TComboBox_GetItemI(TComboBox *self, int idx) {
	return TMenu_GetItemI(self->popup, idx);
}

void TComboBox_ClsMenu(TComboBox *self) {
	TMenu_Cls(self->popup);
	self->SelItem = NULL;
}

static bool TComboBox_Ctl(TComboBox *self, TWinsCtl *wctl) {
	WinsCtl((TWins*)self->popup, wctl);
	return TWins_Ctl((TWins*)self, wctl);
}
// ___________________________ Create ________________________________

static void TComboBox_DefaultConf(TWins_Create_struct *wcs) {
  TComboBox *self = wcs->wins;
	TWins_DefaultConf(wcs);
	self->Resize = TComboBox_Resize;
	self->WinsCtl = TComboBox_Ctl;

	self->button = TWins_Create((TWins*)self);
	Set_Cursor(self->button, wcs->Parent->cursor_nr);
	self->button->BorderFu = mBorder | mBorderW2 | mBorderBu;
	self->button->width = 15;
	self->button->Paint = TComboBoxBu_Paint;
	self->button->DoMouseDown = TComboBoxBu_DoMouseDown;
	SetColor(self->button, appXDat.Default_Color);
	Set_Parent(self->button, (TWins*)self);

	self->popup = TMenu_Create((TWins*)self);
	self->popup->owner = (TWins*)self;
	self->popup->OnClick = Popup_OnClick;
	self->popup->OnMenuClose = TComboBoxPopup_OnMenuClose;
	self->popup->DoMenuOpen = TComboBoxPopup_DoMenuOpen;
}

TComboBox *TComboBox_Create(TWins *Parent) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TComboBox), wtyp_Edit);
	wcs.Parent = Parent;
	wcs.CreateParams = TEdit_CreateParams;
	TComboBox_DefaultConf(&wcs);
	return wcs.wins;
}

TComboBox *TComboBox_CreateS(TWins *Parent, int Left, int Top, int Width, int Height, const char *s) {
  TComboBox *self;

	self = TComboBox_Create(Parent);
	SelfCreateS((TWins*)self, Left, Top, Width, Height, s);
	return self;
}
