/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"

typedef struct TEditPassWD {
  dword Mod;

  char *Orig;
  int  OrigLg;

  char *PasswdChar;

  char *AzPasswd;
  int  AzPasswdCharCount;
  int  AzPasswdLg;

  char *CaretPasswd;
  int  CaretPasswdCharCount;
  int  CaretPasswdLg;

} TEditPassWD;

//*********************** TEdit ************************
static char *GetAzPasswd(TEdit *self, int count) {
  int i, u8c;
  char *pc, *result;
  const char *ps, *pt;

	u8c = 1;
	ps = NULL;
	if (self->PassWD->PasswdChar) {
	    u8c = get_Utf8Chars(self->PassWD->PasswdChar);
	    if ((u8c == 1)||((u8c > 1) && self->vds.UTF8on)) ps = self->PassWD->PasswdChar;
	}
	if (!ps) {
	    ps = "*";
	    u8c = 1;
	}

	result = xcalloc((u8c * count) + 1);
	pc = result;
	while (count > 0) {
	    pt = ps;
	    for (i=u8c;i>0;i--) {
		*pc = *pt; pc++; pt++;
	    }
	    count--;
	}
	return result;
}

static void SetAzPasswd(TEdit *self, int count) {
  TEditPassWD *pw;

	pw = self->PassWD;
	if (pw->AzPasswdCharCount == count) return;
	ppcfree(&pw->AzPasswd);
	pw->AzPasswd   = GetAzPasswd(self, count);
	pw->AzPasswdLg = pclen(pw->AzPasswd);
	pw->AzPasswdCharCount = count;
}

static int ConvCharIdx(TEdit *self, char *orig, char *s, int idx) {
  int i, oc, result;

	if (!idx || !self->vds.UTF8on) return idx;
	oc = 0;
	while (idx > 0) {
	    oc++;
	    i = get_Utf8Chars(orig);
	    if (i <= 0) break;
	    orig += i;
	    idx -= i;
	}

	result = 0;
	while (oc > 0) {
	    oc--;
	    i = get_Utf8Chars(s);
	    if (i <= 0) break;
	    s += i;
	    result += i;
	}
	return result;
}

static void SetCaretPasswd(TEdit *self) {
  int count;
  TEditPassWD *pw;

	pw = self->PassWD;
	if (!pw) return;
	count = pw->OrigLg;
	if (self->vds.UTF8on) count = Chk_fio_UTF8(pw->Orig, pw->OrigLg, 0);
	if (pw->CaretPasswdCharCount == count) return;
	ppcfree(&pw->CaretPasswd);
	pw->CaretPasswd   = GetAzPasswd(self, count);
	pw->CaretPasswdLg = pclen(pw->CaretPasswd);
	pw->CaretPasswdCharCount = count;
}

void TEdit_DoPaint(TEdit *self) {
  int SelA ,SelE;
  TEditPassWD *pw;

	if (self->KeyFu & mHasFocus) Caret_Draw(0);
	pw = self->PassWD;
	if (pw) {
	    if (pw->Mod & ED_PASSWD_CHARSYNC) {
		self->vds.s  = pw->CaretPasswd;
		self->vds.lg = pw->CaretPasswdLg;
	    } else {
		if ((pw->OrigLg > 0) || (pw->Mod & ED_PASSWD_SIM_EMPTY)) {
		    self->vds.s  = pw->AzPasswd;
		    self->vds.lg = pw->AzPasswdLg;
		} else {
		    self->vds.s  = NULL;
		    self->vds.lg = 0;
		}
	    }
	    SelA = self->vds.SelA;
	    SelE = self->vds.SelE;
	    if (self->vds.SelA != self->vds.SelE) {
		self->vds.SelA = ConvCharIdx(self, pw->Orig, pw->CaretPasswd, SelA);
		self->vds.SelE = ConvCharIdx(self, pw->Orig, pw->CaretPasswd, SelE);
	    }

	    TWins_DoPaint((TWins*)self);
	    self->vds.SelA = SelA;
	    self->vds.SelE = SelE;

	} else {
	    TWins_DoPaint((TWins*)self);
	}
	if ((self->KeyFu & mHasFocus) && !self->NoCaret) Caret_Draw(1);
}

static void ed_SetCaret(TEdit *self, int idx) {
  int x;

	if (self->NoCaret) return;
	if (self->PassWD) {
	    self->vds.s  = self->PassWD->CaretPasswd;
	    self->vds.lg = self->PassWD->CaretPasswdLg;
	    idx = ConvCharIdx(self, self->PassWD->Orig, self->vds.s, idx);
	}

	x = GetChar_XPos(idx, &self->vds);
	TWins_Caret_SetNewPos((TWins*)self, x, self->vds.Yoff);
	Caret_Draw(1);
}

static void pw_pop_orig(TEdit *self) {
	if (self->PassWD) {
	    self->vds.s  = self->PassWD->Orig;
	    self->vds.lg = self->PassWD->OrigLg;
	}
}

static void pw_push_orig(TEdit *self) {
	if (self->PassWD) {
	    self->PassWD->Orig = self->vds.s;
	    self->PassWD->OrigLg = self->vds.lg;
	}
}

static void pw_pop_az(TEdit *self) {
	if (self->PassWD) {
	    self->vds.s  = self->PassWD->CaretPasswd;
	    self->vds.lg = self->PassWD->CaretPasswdLg;
	}
}

static void TEdit_SetCaretIdx(TEdit *self, int idx, bool sel) {

	pw_pop_orig(self);
	if (self->KeyFu & mHasFocus) Caret_Draw(0);
	if (idx < 0) idx = 0;
	if (idx > self->vds.lg) idx = self->vds.lg;
	self->CaretCharIdx = idx;
	self->vds.SelE = idx;
	if (!sel) self->vds.SelA = idx;
	if (TestWinIsVisible((TWins*)self)) {
	    Paint((TWins*)self);
	    if (self->KeyFu & mHasFocus) ed_SetCaret(self, idx);
	}
}

void TEdit_SetSelAll(TEdit *self) {
	if (self->NoCaret) return;
	pw_pop_orig(self);
	self->vds.SelA = 0;
	TEdit_SetCaretIdx(self, self->vds.lg, true);
}

static void TEdit_ClsSel(TEdit *self, int *idx) {
  int i;

	pw_pop_orig(self);
	if (self->vds.SelA != self->vds.SelE) {
	    ppclg_delsel(&self->vds.s, &self->vds.lg, self->vds.SelA, self->vds.SelE);
	    if (self->vds.SelA < self->vds.SelE) {
		i = self->vds.SelA;
	    } else {
		i = self->vds.SelE;
	    }
	    if (i < 0) i = 0;
	    *idx = i;
	}
	pw_push_orig(self);
}

static bool ed_Strg_C(TEdit* self) {
  char *pc;
	if (self->PassWD || (self->vds.SelA == self->vds.SelE)) return false;
	if (!self->PassWD) {
	    pc = strgetsel(self->vds.s, self->vds.SelA, self->vds.SelE);
	    ppcchange(&appXDat.SelTxt.s, pc);
	    appXDat.SelTxt.lg = pclen(appXDat.SelTxt.s);
	    SetSelection(1, XA_STRING);
	}
	return true;
}


int TEdit_DoKeyDown(TEdit *self, TKeyEvent *KeyEv) {
  int idx;
  char tmp[8];
  bool sel = false;
  bool chg = false;

// printf("ks=0x%x \"%s\" keycode=%d(0x%x)  state=0x%x\n", *keysym, (char*)keysym, ev->keycode, ev->keycode, ev->state);
	pw_pop_orig(self);
	if (self->NoCaret) {
	    idx = self->vds.lg;
	    self->vds.SelA = idx;
	    self->vds.SelE = idx;
	    self->CaretCharIdx = idx;
	} else {
	    idx = self->CaretCharIdx;
	}
	TWins_DoKeyDown((TWins*)self, KeyEv);

	if (KeyEv->keysym == XK_Left) {
	    if (self->NoCaret) return 1;
	    GetVdsPrev_WChar(&idx, &self->vds);
	    sel = ((KeyEv->state & 1) != 0);

	} else if (KeyEv->keysym == XK_Right) {
	    if (self->NoCaret) return 1;
	    GetVdsNext_WChar(&idx, &self->vds);
	    sel = ((KeyEv->state & 1) != 0);

// ******* Strg_C *******
	} else if ((KeyEv->keycode == 54) && (KeyEv->state & 0x4)) {
	    if (self->NoCaret) return 1;
	    ed_Strg_C(self);
	    return 1;

	} else if (self->ReadOnly) {
	    return 0;

	} else if (KeyEv->keysym == XK_BackSpace) {
	    if (self->vds.SelA == self->vds.SelE) GetVdsPrev_WChar(&self->vds.SelA, &self->vds);
	    TEdit_ClsSel(self, &idx);
	    chg = true;

	} else if (KeyEv->keysym == XK_Delete) {
	    if (self->vds.SelA == self->vds.SelE) GetVdsNext_WChar(&self->vds.SelE, &self->vds);
	    TEdit_ClsSel(self, &idx);
	    chg = true;

// ******* Strg_X *******
	} else if ((KeyEv->keycode == 53) && (KeyEv->state & 0x4)) {
	    if (ed_Strg_C(self)) {
		TEdit_ClsSel(self, &idx);
		chg = true;
	    }
// ******* Strg_V *******
	} else if ((KeyEv->keycode == 55) && (KeyEv->state & 0x4)) { //Strg_V
	    TEdit_ClsSel(self, &idx);
	    if (GetPasteStr_lgs(self->vds.UTF8on)) {
		ppclg_ins_slg(&self->vds.s, &self->vds.lg, 
			      appXDat.SelRet_lgs.s, appXDat.SelRet_lgs.lg, idx);
		idx += pclen(appXDat.SelRet_lgs.s);
		SelRetClr();
		chg = true;
	    }

	} else if (self->PassWD && (KeyEv->keysym == XK_space)) {
	    return 1;

	} else if (keysymtostr(&KeyEv->keysym, tmp, self->vds.UTF8on)) {
	    if (!self->DoCharAllow || self->DoCharAllow(self, KeyEv, idx)) {
		TEdit_ClsSel(self, &idx);
		ppclg_ins_slg(&self->vds.s, &self->vds.lg, 
			      tmp, pclen(tmp), idx);
		GetVdsNext_WChar(&idx, &self->vds);
		chg = true;
	    }
	} else return 0;

	self->vds.lg = pclen(self->vds.s);
	if (chg) {
	    if (self->PassWD) {
		self->PassWD->Mod &= ~ED_PASSWD_SIM_EMPTY;
	    }
	    self->TxtModify = true;
	    if (self->OnChange) self->OnChange(self);
	}
	self->CaptionFu |= mCaptionRePaint;
	pw_push_orig(self);
	SetCaretPasswd(self);
	TEdit_SetCaretIdx(self, idx, sel);
	return 1;
}

void TEdit_DoMouseDown(TEdit *self, XButtonEvent *ev) {
  int idx;
	if ((ev->button == Button1) && 
	    TestWinIsEnabled((TWins*)self) &&
	    !self->NoCaret)
	{
	    pw_pop_az(self);
	    idx = GetMouseX_CharPos(ev->x, &self->vds);
	    if (self->PassWD) idx = ConvCharIdx(self, self->vds.s, self->PassWD->Orig, idx);
	    TEdit_SetCaretIdx(self, idx, false);
	}
	TWins_DoMouseDown((TWins*)self, ev);
}

void TEdit_DoMouseMove(TEdit *self, XMotionEvent *ev) {
  int idx;
	TWins_DoMouseMove((TWins*)self, ev);
	if ((ev->state & Button1Mask) && 
	    (MouseDown.apph == (TWins*)self) &&
	    !self->NoCaret)
	{
	    pw_pop_az(self);
	    idx = GetMouseX_CharPos(ev->x, &self->vds);
	    if (self->PassWD) idx = ConvCharIdx(self, self->vds.s, self->PassWD->Orig, idx);
	    if (self->CaretCharIdx != idx) 
			TEdit_SetCaretIdx(self, idx, true);
	}
}

void TEdit_DoClick(TEdit *self, long KeyState, bool DblClick) {
  bool sel = DblClick && (self->KeyFu & mCanFocus);

	if (sel) TEdit_SetSelAll(self);
	TWins_DoClick((TWins*)self, KeyState, DblClick);
	if (sel) Paint((TWins*)self);
}

void TEdit_DoFocus(TEdit *self, int Change) {

	if (Change & WinsFocusIN) {
	    if ((self->RunStatus & runst_Paint1) == 0) Paint((TWins*)self);
	    TWins_DoFocus((TWins*)self, Change);
	    if (self->PassWD) {
		self->vds.s  = self->PassWD->Orig;
		self->vds.lg = self->PassWD->OrigLg;
	    }
	    if (self->AutoSel) {
		TEdit_SetSelAll(self);
	    } else {
		ed_SetCaret(self, self->CaretCharIdx);
	    }

	} else {
	    if (Change & WinsFocusChange) {
		self->vds.SelA = self->vds.SelE = 0;
		Refresh((TWins*)self);
	    }
	    Caret_Draw(0);
	    TWins_DoFocus((TWins*)self, Change);
	}
}

void TEdit_SetCaption(TEdit *self, const char *s) {
	self->TxtModify = false;
	pw_pop_orig(self);
	if (!pccmp(self->vds.s, s)) {
	    if (self->KeyFu & mHasFocus) Caret_Draw(0);

	    self->CaptionFu |= mCaptionRePaint;
	    vds_SetStr(&self->vds, s);
	    if (self->OnSetCaption) self->OnSetCaption(self, s);
	    pw_push_orig(self);

	    SetCaretPasswd(self);
	    TEdit_SetCaretIdx(self, self->CaretCharIdx, false);
	    if (self->vds.lg && !(self->CaptionFu & mCaptionUseRepaint)) {
		Refresh((TWins*)self); 
	    } else {
		RePaint((TWins*)self);
	    }
	    if (self->OnChange) self->OnChange(self);
	}
}

void TEdit_SetReadOnly(TEdit *self, bool ReadOnly) {
  dword Color;

	if (!self) return;
	if (self->ReadOnly != ReadOnly) {
	    self->ReadOnly = ReadOnly;
	    if (ReadOnly) {
		Color = SubColor(self->Color, self->ColorReadOnlyOffset);
	    } else {
		Color = AddColor(self->Color, self->ColorReadOnlyOffset);
	    }
	    SetColor((TWins*)self, Color);
	}
}

static void TEditPassWD_Clr(TEdit *self) {
  TEditPassWD *pw;

	pw = self->PassWD;
	if (!pw) return;
	if (pw->AzPasswd != pw->CaretPasswd) ppcfree(&pw->AzPasswd);
	ppcfree(&pw->CaretPasswd);
	pw->AzPasswdCharCount = 0;
	pw->AzPasswdLg = 0;
	pw->CaretPasswdCharCount = 0;
	pw->CaretPasswdLg = 0;
}

static void TEditPassWD_Free(TEdit *self) {
	if (!self->PassWD) return;
	pw_pop_orig(self);
	TEditPassWD_Clr(self);
	ppfree(&self->PassWD);
}

void TEdit_SetPasswdMod(TEdit *self, dword Mod, const char *PasswdChar) {
  TEditPassWD *pw;
  bool mk;

	if (!self) return;

	mk = false;
	pw = self->PassWD;
	if (Mod & ED_PASSWD_ON) {
	    if (!pw) {
		pw = xcalloc(sizeof(TEditPassWD));
		self->PassWD = pw;
		pw_push_orig(self);
	    } else {
		if ((pw->Mod == Mod) && pccmp(pw->PasswdChar, PasswdChar)) return;
	    }
	    TEditPassWD_Clr(self);
	    ppcchange(&pw->PasswdChar, PasswdChar);
	    pw->Mod = Mod;
	    SetCaretPasswd(self);
	    if (!(Mod & ED_PASSWD_CHARSYNC)) {
		SetAzPasswd(self, 8);
	    }
	    mk = true;

	} else if (pw) {
	    pw_pop_orig(self);
	    TEditPassWD_Free(self);
	    mk = true;
	}

	if (mk) {
	    self->vds.SelA = 0;
	    self->vds.SelE = 0;
	    self->CaretCharIdx = 0;
	    Caret_Draw(0);
	    TWins_Caret_SetNewPos((TWins*)self, self->vds.Xoff, self->vds.Yoff);
	    RePaint((TWins*)self);
	}
}

const char *TEdit_GetCaption_pc(TEdit *self) {
	if (self->PassWD) return self->PassWD->Orig;
	return self->vds.s;
}

/*____________________________ Destroy __________________________________*/
bool TEdit__Destroy(TEdit *self) {
	TWins_Caret_Clr((TWins*)self);
	TEditPassWD_Free(self);
	return TWins__Destroy((TWins*)self);
}

/*____________________________ Create ___________________________________*/
void TEdit_CreateParams(TWins_Create_struct *wcs) {
  TEdit *self = wcs->wins;

	Caret_Init();

	self->destroy = (TDESTROY)TEdit__Destroy;
	self->SetCaption = TEdit_SetCaption;
	self->GetCaption = TEdit_GetCaption_pc;

	self->DoPaint = TEdit_DoPaint;
	self->DoMouseDown = TEdit_DoMouseDown;
	self->DoMouseMove = TEdit_DoMouseMove;
	self->DoClick = TEdit_DoClick;

	self->DoFocus = TEdit_DoFocus;
	self->DoKeyDown = TEdit_DoKeyDown;

	self->CaptionFu = mCaptionPaint | mCaptionSolid | mCaptionLeft;
	self->KeyFu = mCanFocus | mK_Down | mK_Up | mK_Tab;
	self->BorderFu = mBorder | mBorderL | mBorderW2;
	self->Color = 0xffffff;
	self->ColorReadOnlyOffset = 0x181818;
	self->vds.BackColor = self->Color;
	self->CaptionOffX = 1;
//	self->AutoSel = true;
	self->cursor_nr = XC_xterm;

	self->CaptionFu |= mCaptionRePaint;
}

TEdit *TEdit_Create(TWins *Parent) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TEdit), wtyp_Edit);
	wcs.Parent = Parent;
	wcs.CreateParams = TEdit_CreateParams;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}

TEdit *TEdit_CreateS(TWins *Parent, int Left, int Top, int Width, int Height, const char *s) {
  TEdit *self;

	self = TEdit_Create(Parent);
	SelfCreateS((TWins*)self, Left, Top, Width, Height, s);
	return self;
}

