/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "twrapline.h"

//*********************** TEdit ************************
static void TEditML_SetWrapLine(TEditML *self) {
//  dword flags;

	if (!WrapLine_SetFont(self->wl, (TFont*)self->vds.Font)) return;

//	flags = 0;
//	if (self->vds.UTF8on) flags = FONTFG_UTF8;
//	Font_SetFlag((TFont*)self->vds.Font, flags);

	Font_SetUTF8((TFont*)self->vds.Font, self->vds.UTF8on);
	Font_SetColor((TFont*)self->vds.Font, self->vds.ForeColor);
	Font_SetBackColor((TFont*)self->vds.Font, self->vds.BackColor);

	WrapLine_Set(self->wl, self->vds.s, self->vds.lg);
	WrapLine_UpdateLine(self->wl);
}

static void TEditML_DoPaint(TEditML *self) {
	if (self->KeyFu & mHasFocus) Caret_Draw(CARET_OFF);

//	Font_ClrOrigin((TFont*)self->vds.Font);
	if (!self->wl->wld && (self->vds.lg > 0)) {
	    TEditML_SetWrapLine(self);
	}
	WrapLine_PaintTo(self->wl, (void*)&self->WinID);


//	    TWins_DoPaint((TWins*)self);
	if (self->KeyFu & mHasFocus) Caret_Draw(CARET_ON);
}

static void ed_SetCaret(TEditML *self, int idx) {
//  TWrapLineCharData wlcd;

	if (!WrapLine_GetCharWlcd(self->wl, idx)) return;
// printf("%s(%d): x=%d y=%d idx=%d\n", FL, wlcd.char_x, wlcd.char_top, idx);
	Caret_Setup(self->WinID, self->wl->wlcd.char_x, self->wl->wlcd.char_top, 0, 
			self->wl->wlcd.char_height);
	Caret_Draw(CARET_ON);
}

static void TEditML_SetCaretIdx(TEditML *self, int idx, bool sel) {
  int SelA, SelE;
  bool NewPaint;

	if (self->KeyFu & mHasFocus) Caret_Draw(CARET_OFF);
	if (idx < 0) idx = 0;
	if (idx > self->vds.lg) idx = self->vds.lg;
	self->CaretCharIdx = idx;

	SelA = self->wl->SelA;
	SelE = self->wl->SelE;
	NewPaint = false;
	if (sel) {
	    if (SelE != self->CaretCharIdx) {
		WrapLine_SetSelect(self->wl, SelA, self->CaretCharIdx);
		NewPaint = True;
	    }
	} else {
	    if (SelA != SelE) NewPaint = True;    
	    WrapLine_SetSelect(self->wl, self->CaretCharIdx, self->CaretCharIdx);   
	}
	if (!TestWinIsVisible((TWins*)self)) return;

	if (NewPaint || (self->CaptionFu & mCaptionRePaint)) {
	    if (self->CaptionFu & mCaptionRePaint) {
		self->CaptionFu &= ~mCaptionRePaint;
		RePaint((TWins*)self);
	    } else {
		Refresh((TWins*)self);
	    }
	}
	if (self->KeyFu & mHasFocus) ed_SetCaret(self, idx);
}

static void TEditML_SetSelAll(TEditML *ed) {
	ed->wl->SelA = 0;
	TEditML_SetCaretIdx(ed, ed->vds.lg, True);
}

static void TEditML_ClsSel(TEditML *self, int *idx) {
  int SelA, SelE;

	SelA = self->wl->SelA;
	SelE = self->wl->SelE;
	if (SelA != SelE) {
	    ppclg_delsel(&self->vds.s, &self->vds.lg, SelA, SelE);
	    if (SelA > SelE) SelA = SelE;
	    if (SelA < 0) SelA = 0;
	    *idx = SelA;
	    TEditML_SetWrapLine(self);
	}
}

static bool ed_Strg_C(TEditML* self) {
  char *pc;
	if (self->wl->SelA == self->wl->SelE) return False;
	pc = strgetsel(self->vds.s, self->wl->SelA, self->wl->SelE);
	ppcchange(&appXDat.SelTxt.s, pc);
	appXDat.SelTxt.lg = pclen(appXDat.SelTxt.s);
	SetSelection(1, XA_STRING);
	return True;
}


static int TEditML_DoKeyDown(TEditML *self, TKeyEvent *KeyEv) {
  int idx, y;
  TWrapLineCharData *wlcd;
  TWrapLineData *wld;
  char tmp[8];
  bool sel = False;
  bool chg = False;

// printf("ks=0x%04x \"%s\" keycode=%d(0x%x)  state=0x%x\n", (int)KeyEv->keysym, (char*)&KeyEv->keysym, KeyEv->keycode, KeyEv->keycode, KeyEv->state);
	idx = self->CaretCharIdx;
	TWins_DoKeyDown((TWins*)self, KeyEv);
	wlcd = &self->wl->wlcd;

	if (KeyEv->keysym == XK_Left) {
	    GetVdsPrev_WChar(&idx, &self->vds);
	    sel = ((KeyEv->state & 1) != 0);

	} else if (KeyEv->keysym == XK_Right) {
	    GetVdsNext_WChar(&idx, &self->vds);
	    sel = ((KeyEv->state & 1) != 0);

	} else if (KeyEv->keysym == XK_Up) {
	    wld = wlcd->wld;
	    if (!self->vds.lg || !wld) return 1;
	    if (wld->line_nr <= 0) return 1;
	    y = wlcd->char_top - (wlcd->char_height / 2);
	    if (y < 0) y = 0;

	    if (!WrapLine_GetCharWlcd_XY(self->wl, wlcd->char_x, y)) return 1;
	    idx = wlcd->char_idx;
	    sel = ((KeyEv->state & 1) != 0);

	} else if (KeyEv->keysym == XK_Down) {
	    wld = wlcd->wld;
	    if (!self->vds.lg || !wld) return 1;
	    if (wld->line_nr >= self->wl->LineCount) return 1;
	    y = wlcd->char_top + wlcd->char_height + (wlcd->char_height / 2);

	    if (!WrapLine_GetCharWlcd_XY(self->wl, wlcd->char_x, y)) return 1;
	    idx = wlcd->char_idx;
	    sel = ((KeyEv->state & 1) != 0);

// ******* Strg_C *******
	} else if ((KeyEv->keycode == 54) && (KeyEv->state & 0x4)) {
	    ed_Strg_C(self);
	    return 1;

	} else if (self->ReadOnly) {
	    return 0;

	} else if (KeyEv->keysym == XK_BackSpace) {
	    if (self->wl->SelA == self->wl->SelE) GetVdsPrev_WChar(&self->wl->SelA, &self->vds);
	    TEditML_ClsSel(self, &idx);
	    chg = True;

	} else if (KeyEv->keysym == XK_Delete) {
	    if (self->wl->SelA == self->wl->SelE) GetVdsNext_WChar(&self->wl->SelE, &self->vds);
	    TEditML_ClsSel(self, &idx);
	    chg = True;

// ******* Strg_X *******
	} else if ((KeyEv->keycode == 53) && (KeyEv->state & 0x4)) {
	    if (ed_Strg_C(self)) {
		TEditML_ClsSel(self, &idx);
		chg = True;
	    }
// ******* Strg_V *******
	} else if ((KeyEv->keycode == 55) && (KeyEv->state & 0x4)) { //Strg_V
	    TEditML_ClsSel(self, &idx);
	    if (GetPasteStr_lgs(self->vds.UTF8on)) {
		ppclg_ins_slg(&self->vds.s, &self->vds.lg, 
			      appXDat.SelRet_lgs.s, appXDat.SelRet_lgs.lg, idx);
		idx += pclen(appXDat.SelRet_lgs.s);
		SelRetClr();
		chg = True;
	    }

	} else if (KeyEv->keysym == XK_Return) {
	    TEditML_ClsSel(self, &idx);
	    if (!self->MaxLineCount || 
		((self->MaxLineCount > 1) &&
		 (self->MaxLineCount > self->wl->LineCount)))
	    {
		ppclg_ins_slg(&self->vds.s, &self->vds.lg, "\n", 1, idx);
		idx++;
	    }
	    chg = True;

	} else if (KeyEv->keysym == XK_Tab) {
	    return 0;

	} else if (keysymtostr(&KeyEv->keysym, tmp, self->vds.UTF8on)) {
	    if (!self->DoCharAllow || self->DoCharAllow(self, KeyEv, idx)) {
		TEditML_ClsSel(self, &idx);
		ppclg_ins_slg(&self->vds.s, &self->vds.lg, 
			      tmp, pclen(tmp), idx);
		GetVdsNext_WChar(&idx, &self->vds);
		chg = True;
	    }
	} else return 0;

	self->vds.lg = pclen(self->vds.s);
	if (chg) {
	    if (self->OnChange) self->OnChange(self);
	    TEditML_SetWrapLine(self);
	    self->CaptionFu |= mCaptionRePaint;
	}
	TEditML_SetCaretIdx(self, idx, sel);
	return 1;
}

static void TEditML_DoMouseDown(TEditML *self, XButtonEvent *ev) {

	if ((ev->button == Button1) && TestWinIsEnabled((TWins*)self)) {
	    if (WrapLine_GetCharWlcd_XY(self->wl, ev->x, ev->y)) {
		TEditML_SetCaretIdx(self, self->wl->wlcd.char_idx, false);
	    }
	}
	TWins_DoMouseDown((TWins*)self, ev);
}

static void TEditML_DoMouseMove(TEditML *self, XMotionEvent *ev) {

	TWins_DoMouseMove((TWins*)self, ev);
	if ((ev->state & Button1Mask) && (MouseDown.apph == (TWins*)self)) {
	    if (WrapLine_GetCharWlcd_XY(self->wl, ev->x, ev->y)) {
		if (self->CaretCharIdx != self->wl->wlcd.char_idx) 
			TEditML_SetCaretIdx(self, self->wl->wlcd.char_idx, true);
	    }
	}
}

static void TEditML_DoClick(TEditML *self, long KeyState, bool DblClick) {
  bool sel = DblClick && (self->KeyFu & mCanFocus);

	if (sel) TEditML_SetSelAll(self);
	TWins_DoClick((TWins*)self, KeyState, DblClick);
	if (sel) Paint((TWins*)self);
}

static void TEditML_DoFocus(TEditML *self, int Change) {

	if (Change & WinsFocusIN) {
	    if ((self->RunStatus & runst_Paint1) == 0) Paint((TWins*)self);
	    TWins_DoFocus((TWins*)self, Change);
	    if (self->AutoSel) {
		TEditML_SetSelAll(self);
	    } else {
		ed_SetCaret(self, self->CaretCharIdx);
	    }

	} else {
	    if (Change & WinsFocusChange) {
		self->wl->SelA = self->wl->SelE = 0;
		Refresh((TWins*)self);
	    }
	    Caret_Draw(CARET_OFF);
	    TWins_DoFocus((TWins*)self, Change);
	}
}

static void TEditML_SetCaption(TEditML *self, const char *s) {
	if (!pccmp(self->vds.s, s)) {
	    if (self->KeyFu & mHasFocus) Caret_Draw(CARET_OFF);
	    TWins_SetCaption((TWins*)self, s);
	    if (self->OnChange) self->OnChange(self);
	    TEditML_SetWrapLine(self);
	}
	TEditML_SetCaretIdx(self, 0, False);
}

/* void TEditML_SetReadOnly(TEditML *self, bool ReadOnly) {
  dword Color;
	if (self->ReadOnly != ReadOnly) {
	    self->ReadOnly = ReadOnly;
	    if (ReadOnly) {
		Color = SubColor(self->Color, self->ColorReadOnlyOffset);
	    } else {
		Color = AddColor(self->Color, self->ColorReadOnlyOffset);
	    }
	    SetColor((TWins*)self, Color);
	}
} */

static void TEditML_Resize(TEditML *self) {
  TWRect rc;

	TWins_Resize((TWins*)self);
	Get_ClientRect((TWins*)self, &rc);
	WrapLine_SetClipRc(self->wl, &rc);
}

/*____________________________ Destroy __________________________________*/
bool TEditML__Destroy(TEditML *self) {
	TWins_Caret_Clr((TWins*)self);
	Tfree(self->wl);
	return TWins__Destroy((TWins*)self);
}

/*____________________________ Create ___________________________________*/
void TEditML_CreateParams(TWins_Create_struct *wcs) {
  TEditML *self = wcs->wins;

	Caret_Init();

	self->destroy = (TDESTROY)TEditML__Destroy;
	self->SetCaption = TEditML_SetCaption;

	self->DoPaint      = TEditML_DoPaint;
	self->DoMouseDown  = TEditML_DoMouseDown;
	self->DoMouseMove  = TEditML_DoMouseMove;
	self->DoClick      = TEditML_DoClick;

	self->DoFocus	= TEditML_DoFocus;
	self->DoKeyDown	= TEditML_DoKeyDown;

	self->Resize	= TEditML_Resize;

//	self->CaptionFu = mCaptionPaint | mCaptionSolid | mCaptionLeft;
	self->KeyFu = mCanFocus; // | mK_Down | mK_Up | mK_Tab;
	self->BorderFu = mBorder | mBorderL | mBorderW2;
	self->Color = 0xffffff;
	self->ColorReadOnlyOffset = 0x181818;
	self->vds.BackColor = self->Color;
	self->CaptionOffX = 1;
//	self->AutoSel = True;
	self->cursor_nr = XC_xterm;

	self->wl = TWrapLine_Create();
	WrapLine_SetFont(self->wl, (TFont*)self->vds.Font);

//	self->CaptionFu |= mCaptionRePaint;
}

TEditML *TEditML_Create(TWins *Parent) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TEditML), wtyp_Edit);
	wcs.Parent = Parent;
	wcs.CreateParams = TEditML_CreateParams;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}

TEditML *TEditML_CreateS(TWins *Parent, int Left, int Top, int Width, int Height, const char *s) {
  TEditML *self;

	self = TEditML_Create(Parent);
	SelfCreateS((TWins*)self, Left, Top, Width, Height, s);
	return self;
}

