/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"

//*********************** TGrid ************************

typedef struct {
  int idx;
  int cell;
  int cfix;
  int dynpos;
  int fixpos;
  int cellx;
  int cellxo;
  int cellw;
  int w;
  int max_w;
  TcrDat *cd;
} Tnext_Cell;


static bool get_next_Cell(Tnext_Cell *ncl) {

	if (ncl->idx >= ncl->cell) return False;
	ncl->idx++;
	
	ncl->cellxo = 0;
	if (!ncl->cd[ncl->idx].Visible) {
	    ncl->cellw = 0;
	} else {
	    ncl->w = ncl->cellw = ncl->cd[ncl->idx].w;
	    if (ncl->idx < ncl->cfix) {
		ncl->cellx = ncl->fixpos;
		ncl->fixpos += ncl->w;
	    } else {
		if (ncl->dynpos < 0) {
		    ncl->cellw += ncl->dynpos;
		    ncl->cellxo = ncl->w - ncl->cellw;
		}
		ncl->cellx = ncl->fixpos + ncl->dynpos;
		ncl->dynpos += ncl->w;
	    }
	}
	if (ncl->cellx >= ncl->max_w) return False;
	return True;
}

static void set_next_Cell0(TGrid *gr, Tnext_Cell *ncl, bool col) {
	ncl->idx = -1;
	ncl->w = 0;
	ncl->cellx = 0;
	ncl->cellw = 0;
	ncl->fixpos = 0;
	if (col) {
	    ncl->cell = gr->ColCount-1;
	    ncl->cfix = gr->FixedCols;
	    ncl->cd = gr->ColDat;
	    ncl->max_w = gr->width;
	    ncl->dynpos = -TScrollBar_GetCount(gr->HSB);
	} else {
	    ncl->cell = gr->RowCount-1;
	    ncl->cfix = gr->FixedRows;
	    ncl->cd = gr->RowDat;
	    ncl->max_w = gr->height;
	    ncl->dynpos = -TScrollBar_GetCount(gr->VSB);
	}
}

static int get_max_x(TGrid *self, TcrDat *cd, int anf, int end) {
  int x;
  int res = 0;
	for (x=anf; x<end; x++) {
	    if (cd[x].Visible) res += cd[x].w;
	}
	return res;
}

void TGrid_DoSetUp_VHSB(TGrid *self) {
  int m, v, z;

	v = get_max_x(self, self->RowDat, 0, self->RowCount);
	TScrollBar_SetMaxCount_ButtonSize(self->VSB, v, self->height);
	m = ((self->height - self->FixedRows * self->DefaultRowHeight) / self->DefaultRowHeight) - 1;
	if (m < 1) m = 1;
	self->VSB->LargeChange = m * self->DefaultRowHeight;

	v = get_max_x(self, self->ColDat, 0, self->ColCount);
	TScrollBar_SetMaxCount_ButtonSize(self->HSB, v, self->width);
	z = self->FixedCols;
	if (z > self->ColCount) z = self->ColCount;
	m = self->width - get_max_x(self, self->ColDat, 0, z);
	if (m < self->DefaultColWidth) m = self->DefaultColWidth;
	self->HSB->LargeChange = m;
}

void TGrid_Chk_VHSB(TGrid *self) {
	if (self->DoSetUp_VHSB) {
	    self->DoSetUp_VHSB(self);
	} else {
	    TGrid_DoSetUp_VHSB(self);
	}
}

void TGrid_EditEnd(TGrid *gr) {
	if (gr->EditWins && gr->DoEditEnd) gr->DoEditEnd(gr);
}

void TGrid_Resize(TGrid *self) {
	TGrid_EditEnd(self);
	TWins_Resize((TWins*)self);
	TGrid_Chk_VHSB(self);
}

void TGrid_DoPaint(TGrid *self) {
  int xmax, ymax;
  int fu = 0;
  TWRect rc;
  Tnext_Cell ncly, nclx;
  TPaintCellDat pcd;

	self->CellGC->vds.UTF8on = self->vds.UTF8on;
	self->PaintGC->XOrigin = 0;
	self->PaintGC->YOrigin = 0;
	TClassGC_SetForeGround(self->PaintGC, self->Color);
	if (!self->ColDat || !self->RowDat) {
	    TClassGC_FillRect(self->PaintGC, 0, 0, self->width, self->height);
	    return;
	}

	self->rcPaint.left = get_max_x(self, self->ColDat, 0, self->FixedCols);
	self->rcPaint.top = get_max_x(self, self->RowDat, 0, self->FixedRows);

	xmax = get_max_x(self, self->ColDat, 0, self->ColCount);
	ymax = get_max_x(self, self->RowDat, 0, self->RowCount);

	xmax -= TScrollBar_GetCount(self->HSB);
	if (xmax < self->rcPaint.left) xmax = self->rcPaint.left;
	ymax -= TScrollBar_GetCount(self->VSB);
	if (ymax < self->rcPaint.top) ymax = self->rcPaint.top;

	if (self->width > xmax) {    //test rest cls
	    TClassGC_FillRect(self->PaintGC, xmax, 0, self->width - xmax, ymax);
	}
	if (self->height > ymax) {    //test rest cls
	    TClassGC_FillRect(self->PaintGC, 0, ymax, self->width, self->height - ymax);
	}


	pcd.xa = pcd.xe = pcd.ya = pcd.ye = -1;
	set_next_Cell0(self, &ncly, False);
	while (get_next_Cell(&ncly)) {
	  if (ncly.cellw > 0) {
	    if ((ncly.idx >= self->FixedRows) && (pcd.ya == -1)) pcd.ya = ncly.idx;
	    pcd.ye = ncly.idx;

	    set_next_Cell0(self, &nclx, True);
	    while (get_next_Cell(&nclx)) {
		if (nclx.cellw > 0) {
		    if (pcd.ya == pcd.ye) {
			if ((nclx.idx >= self->FixedCols) && (pcd.xa == -1)) pcd.xa = nclx.idx;
			pcd.xe = nclx.idx;
		    }

		    rc.width = nclx.cellw;
		    rc.left = nclx.cellxo;
		    self->CellGC->XOrigin = nclx.cellx;
		    self->PaintGC->XOrigin = nclx.cellx;

		    rc.height = ncly.cellw;
		    rc.top = ncly.cellxo;
		    self->CellGC->YOrigin = ncly.cellx;
		    self->PaintGC->YOrigin = ncly.cellx;

		    if (self->DoPaintCell &&
			self->DoPaintCell(self, nclx.idx, ncly.idx, &rc, &fu)) {
			if (fu & GrDPC_DrawSelect) {
			    TClassGC_FillSelectRect_rc(self->CellGC, &rc);
			}
			if (fu & GrDPC_DrawFocusRc) {
			    TClassGC_DrawFocusRect_rc(self->CellGC, &rc);
			}
		    }
//		    if (self->DoPaintSelectCell) self->DoPaintSelectCell(self, nclx.idx, ncly.idx, &rc, &fu);
		}
	    }
	  }
	}
//  printf("%s(%d): %d %d  %d %d\n", FL, pcd.xa, pcd.xe, pcd.ya, pcd.ye);
	if (self->OnPaintCellEnd) self->OnPaintCellEnd(self, &pcd);
}

static bool GetCell_ncl(TGrid *self, int ACol, int ARow, Tnext_Cell *nclx, Tnext_Cell *ncly) {
	if ((ACol >= self->ColCount) || (ARow >= self->RowCount)) return False;

	set_next_Cell0(self, nclx, True);
	nclx->cell = ACol;
	while (get_next_Cell(nclx));
	if (nclx->cellw <= 0) return False;

	set_next_Cell0(self, ncly, False);
	ncly->cell = ARow;
	while (get_next_Cell(ncly));
	if (ncly->cellw <= 0) return False;

	return True;
}

bool TGrid_GetCell_AbsRC(TGrid *gr, int ACol, int ARow, TWRect *rc) {
  Tnext_Cell nclx, ncly;

	if (!GetCell_ncl(gr, ACol, ARow, &nclx, &ncly)) return False;
	rc->width = nclx.cellw;
	rc->left = nclx.cellx + nclx.cellxo;
	rc->height = ncly.cellw;
	rc->top = ncly.cellx + ncly.cellxo;
	return True;
}

bool TGrid_GetCell_RC(TGrid *gr, int ACol, int ARow, TWRect *rc) {
  Tnext_Cell nclx, ncly;

	if (!GetCell_ncl(gr, ACol, ARow, &nclx, &ncly)) return False;
	rc->width = nclx.cellw;
	rc->left = nclx.cellxo;
	gr->CellGC->XOrigin = gr->PaintGC->XOrigin = nclx.cellx;

	rc->height = ncly.cellw;
	rc->top = ncly.cellxo;
	gr->CellGC->YOrigin = gr->PaintGC->YOrigin = ncly.cellx;
	return True;
}

void TGrid_PaintCell(TGrid *self, int ACol, int ARow, int *fu) {
  TWRect rc;

	if (TGrid_GetCell_RC(self, ACol, ARow, &rc)) {
	    if (self->DoPaintCell &&
		self->DoPaintCell(self, ACol, ARow, &rc, fu)) {

		if (*fu & GrDPC_DrawSelect) {
		    TClassGC_FillSelectRect_rc(self->CellGC, &rc);
		}
		if (*fu & GrDPC_DrawFocusRc) {
		    TClassGC_DrawFocusRect_rc(self->CellGC, &rc);
		}
	    }
	}
}

bool TGrid_DoPaintCell(TGrid *gr, int ACol, int ARow, TWRect *rc, int *fu) {
  int ForeColor, m;
  TWRect rcc;

	rcc.left = rc->left;
	rcc.top = rc->top;
	rcc.width = rc->width;
	rcc.height = rc->height;

	if ((gr->FixedCols > ACol) || (gr->FixedRows > ARow)) {
	    ForeColor = appXDat.Default_Color;

	    if (ARow < gr->FixedRows) {
		m = gr->FixedRowsFu;
		if (gr->FixedRowsColor != 0x80000000) ForeColor = gr->FixedRowsColor;
	    } else {
		m = gr->FixedColsFu;
		if (gr->FixedColsColor != 0x80000000) ForeColor = gr->FixedColsColor;
	    }

	    if (!(m & grFixNoBorder)) {
		TClassGC_BorderPaint_rc(gr->PaintGC, mBorder, &rcc);
		rcc.left++;
		rcc.top++;
		rcc.width -= 2;
		rcc.height -= 2;
	    }
	    if ((rcc.width < 1) || (rcc.height < 1)) return False;

	} else {
	    ForeColor = gr->Color;
	    if (gr->ColDat[ACol].Color != 0x80000000) ForeColor = gr->ColDat[ACol].Color;
	    if (gr->RowDat[ARow].Color != 0x80000000) ForeColor = gr->RowDat[ARow].Color;
	}
	TClassGC_SetForeGround(gr->PaintGC, ForeColor);
	TClassGC_FillRect_rc(gr->PaintGC, &rcc);
	TClassGC_SetClipRc(gr->CellGC, &rcc);
	gr->CellGC->vds.BackColor = ForeColor;

	return True;
}

static bool TGrid_MouseXYToCellXY(TGrid *self, int mx, int my) {
  Tnext_Cell ncl;
  TMouseCell *mc = &self->MouseCell;

	mc->InCell = False;
	set_next_Cell0(self, &ncl, False);
	while (get_next_Cell(&ncl)) {
	    if ((ncl.cellw > 0) && ((ncl.cellx + ncl.cellxo) < my) && 
		((ncl.cellx + ncl.cellxo + ncl.cellw) >= my)) goto yok;
	}
	return False;
yok:
	mc->Row = ncl.idx;
	mc->rc.top = ncl.cellxo;
	mc->rc.height = ncl.cellw;
	mc->YOrigin = ncl.cellx;
	mc->y = my - ncl.cellx;

	set_next_Cell0(self, &ncl, True);
	while (get_next_Cell(&ncl)) {
	    if ((ncl.cellw > 0) && ((ncl.cellx + ncl.cellxo) < mx) && 
		((ncl.cellx + ncl.cellxo + ncl.cellw) >= mx)) goto xok;
	}
	return False;
xok:
	mc->Col = ncl.idx;
	mc->rc.left = ncl.cellxo;
	mc->rc.width = ncl.cellw;
	mc->XOrigin = ncl.cellx;
	mc->x = mx - ncl.cellx;

	mc->fixCol = (self->FixedCols > mc->Col);
	mc->fixRow = (self->FixedRows > mc->Row);
	mc->InCell = True;
	return True;

}

static bool TGrid_ChkMouseSnap(TGrid *self) {
  int tmpCursor;
  TMouseCell *mc = &self->MouseCell;
  bool Result = False;

	tmpCursor = XC_top_left_arrow;
	if (mc->InCell) {
	    if (mc->fixRow && (self->FixedColsFu & grFixResize) &&
		((mc->x + 4) > (mc->rc.left + mc->rc.width))) {
		tmpCursor = XC_sb_h_double_arrow;
		Result = True;
	    } else if (mc->fixCol && (self->FixedRowsFu & grFixResize) &&
		((mc->y + 4) > (mc->rc.top + mc->rc.height))) {
		tmpCursor = XC_sb_v_double_arrow;
		Result = True+1;
	    }
	}
	Set_Cursor((TWins*)self, tmpCursor);
	return Result;
}

void TGrid_ChkSelect_RC(TGrid *gr, int ACol, int ARow) {
  int m, xl, top, height, left, width;

	if ((ARow < 0) || (ARow >= gr->RowCount)) return;
	if ((ACol < 0) || (ACol >= gr->ColCount)) return;

	top = get_max_x(gr, gr->RowDat, 0, ARow);
	height = gr->RowDat[ARow].w;
	if (!gr->RowDat[ARow].Visible) return;

	left = get_max_x(gr, gr->ColDat, 0, ACol);
	width = gr->ColDat[ACol].w;
	if (!gr->ColDat[ACol].Visible) return;

	if (gr->VSB->Visible) {
	    m = top - TScrollBar_GetCount(gr->VSB);
	    if (m < gr->rcPaint.top) {
		TScrollBar_SetCount_ev(gr->VSB, top - gr->rcPaint.top);
	    }
	    m = gr->height - (top + height) + TScrollBar_GetCount(gr->VSB);
	    if (m < 0) {
		TScrollBar_SetCount_ev(gr->VSB, (top + height) - gr->height);
	    }
	}

	if (!(gr->SelectFu & grSelFu_NoAutoHSB) && gr->HSB->Visible) {
	    xl = gr->rcPaint.left;
	    m = left - TScrollBar_GetCount(gr->HSB) - xl;
	    if (m < 0) {
		TScrollBar_SetCount_ev(gr->HSB, left - xl);
	    }
	    m = gr->width - (left + width) + TScrollBar_GetCount(gr->HSB);
	    if (m < 0) {
		TScrollBar_SetCount_ev(gr->HSB, (left + width) - gr->width);
	    }
	}
}

void TGrid_DoMouseDown(TGrid *self, XButtonEvent *ev) {
  int bef;

	TGrid_EditEnd(self);
	TGrid_MouseXYToCellXY(self, ev->x, ev->y);
	TWins_DoMouseDown((TWins*)self, ev);
	if ((ev->button != 1) || !self->Enabled) return;

	self->RowColResize = TGrid_ChkMouseSnap(self);
	if (self->RowColResize) {
	    Std_GrabPointer((TWins*)self, appXDat.XCursors[self->cursor_nr]);
	    self->MoveX = self->ColDat[self->MouseCell.Col].w;
	    self->MoveY = self->RowDat[self->MouseCell.Row].w;
	}
	if (!self->MouseCell.InCell) return;

	bef = grBefFu_Select;
	switch(ev->state & ~0x12) {
	case 1:  //ctrl
	    bef = grBefFu_Select | grBefFu_ctrl;
	    break;
	case 4:  //strg
	    bef = grBefFu_Select | grBefFu_strg;
	    break;
	}

	if (self->DoSelect) self->DoSelect(self, bef);
}

void TGrid_DoMouseUp(TGrid *self, XButtonEvent *ev) {
#if 0
  int VSBc, k;

	if ((ev->button == Button4) || (ev->button == Button5)) {
	    TWins_DoMouseUp((TWins*)self, ev);
	    VSBc = TScrollBar_GetCount(self->VSB);
	    k = self->DefaultRowHeight * self->MouseZ_Wheel;
	    if (ev->button == Button4) VSBc -= k; else VSBc += k;
	    TScrollBar_SetCount_ev(self->VSB, VSBc);
#else
	if (TScrollBar_MouseZ_Wheel(self->VSB, ev)) {
	    TWins_DoMouseUp((TWins*)self, ev);
#endif
	} else if (ev->button == Button1) {
	    TGrid_MouseXYToCellXY(self, ev->x, ev->y);
	    Std_UngrabPointer();
	    TWins_DoMouseUp((TWins*)self, ev);
	    if (self->RowColResize) TGrid_Chk_VHSB(self);
	    self->RowColResize = False;
	    TGrid_ChkMouseSnap(self);
	} else {
	    TWins_DoMouseUp((TWins*)self, ev);
	}
}

void TGrid_DocrResize(TGrid *self, int idx, bool isCol) {

	if (self->OncrResize) self->OncrResize(self, idx, isCol);
	Refresh((TWins*)self);
}

void TGrid_DoMouseMove(TGrid *self, XMotionEvent *ev) {
  int  nw, idx;
  bool isCol;

	if (!self->RowColResize) TGrid_MouseXYToCellXY(self, ev->x, ev->y);
	TWins_DoMouseMove((TWins*)self, ev);
	if ((ev->state & 0x100) == 0) {
	    TGrid_ChkMouseSnap(self);
	    self->RowColResize = False;
	} else if (self->RowColResize) {

	    isCol = self->RowColResize == True;
	    if (isCol) {
		nw = ev->x - MouseDown.x;
//		if (self->MouseCell.rc.width + nw < 6) nw = 6 - self->MouseCell.rc.width;
		nw += self->MoveX;
		idx = self->MouseCell.Col;
	    } else {
		nw = ev->y - MouseDown.y;
//		if (self->MouseCell.rc.height + nw < 6) nw = 6 - self->MouseCell.rc.height;
		nw += self->MoveY;
		idx = self->MouseCell.Row;
	    }
	    TGrid_Set_crSize(self, idx, isCol, nw);
	}
}

void TGrid_SetColCount(TGrid *self, int Value) {
  int i, m;
  TcrDat *cc;

	if (Value < 0) Value = 0;
	if (self->ColCount != Value) {
	    m = self->ColCount;
	    set_arr((void*)&self->ColDat, 0, self->ColCount, Value, sizeof(TcrDat));
	    for (i=m; i<Value; i++) {
		cc = &self->ColDat[i];
		cc->w = self->DefaultColWidth;
		cc->Color = 0x80000000;
		cc->Visible = True;
	    }
	    self->ColCount = Value;
	    if (self->ResizeStartMk) TGrid_Chk_VHSB(self);
	    Refresh((TWins*)self);
	}
}

void TGrid_SetRowCount(TGrid *self, int Value) {
  int i, m;
  TcrDat_Row *cr;

	if (Value < 0) Value = 0;
	if (self->RowCount != Value) {
	    m = self->RowCount;
	    if (m > Value) {
	      for (i=Value; i<m; i++) {
		cr = &self->RowDat[i];
		if (self->OnRowFree) self->OnRowFree(self, cr);
	      }
	    }
	    set_arr((void*)&self->RowDat, 0, self->RowCount, Value, sizeof(TcrDat_Row));
	    for (i=m; i<Value; i++) {
		cr = &self->RowDat[i];
		cr->w = self->DefaultRowHeight;
		cr->Color = 0x80000000;
		cr->Visible = True;
	    }
	    self->RowCount = Value;
	    if (self->ResizeStartMk) TGrid_Chk_VHSB(self);
	    Refresh((TWins*)self);
	}
}

static int crSizeChk(TcrDat *cr, int wh) {
	if (cr->min && (wh < cr->min)) wh = cr->min;
	if (cr->max && (wh > cr->max)) wh = cr->max;
	if (wh < 3) wh = 3;
	if (wh > 4000) wh = 4000;
	return wh;
}

void TGrid_Set_crMinMax(TGrid *self, int idx, bool isCol, int min, int max) {
  TcrDat *cr = self->ColDat;
  int mx = self->ColCount;

	if (!isCol) {
	    cr = self->RowDat;
	    mx = self->RowCount;
	}
	if ((idx < 0) || (idx >= mx)) return;
	cr += idx;

	if (min < 0) min = 0;
	if (min > 4000) min = 4000;
	cr->min = min;

	if (max < 0) max = 0;
	if (max > 4000) max = 4000;
	cr->max = max;

	cr->w = crSizeChk(cr, cr->w);
}
/* ------ TGrid Set Size ------ */
void TGrid_Set_crSize(TGrid *self, int idx, bool isCol, int wh) {
  TcrDat *cr = self->ColDat;
  int mx = self->ColCount;

	if (!isCol) {
	    cr = self->RowDat;
	    mx = self->RowCount;
	}
	if ((idx < 0) || (idx >= mx)) return;
	cr += idx;
	wh = crSizeChk(cr, wh);
	if (cr->w != wh) {
	    cr->w = wh;
	    TGrid_Chk_VHSB(self);
	    if (self->DocrResize) self->DocrResize(self, idx, isCol);
	}
}

void TGrid_SetColWidth(TGrid *gr, int Col, int Width) {
	TGrid_Set_crSize(gr, Col, True, Width);
}
void TGrid_SetRowHeight(TGrid *gr, int Row, int Height) {
	TGrid_Set_crSize(gr, Row, False, Height);
}

/* ------ TGrid Set Visible ------ */
static void TGrid_Set_crVisible(TGrid *self, int idx, bool isCol, bool Visible) {
  TcrDat *cr = self->ColDat;
  int mx = self->ColCount;

	if (!isCol) {
	    cr = self->RowDat;
	    mx = self->RowCount;
	}
	if ((idx < 0) || (idx >= mx)) return;
	cr += idx;
	if (cr->Visible != Visible) {
	    cr->Visible = Visible;
	    TGrid_Chk_VHSB(self);
	    if (self->DocrResize) self->DocrResize(self, idx, isCol);
	}
}

void TGrid_SetColVisible(TGrid *gr, int Col, bool Visible) {
	TGrid_Set_crVisible(gr, Col, True, Visible);
}
void TGrid_SetRowVisible(TGrid *gr, int Row, bool Visible) {
	TGrid_Set_crVisible(gr, Row, False, Visible);
}

static void TGrid_VSB_OnCount(TScrollBar *sb, int Count) {
  TGrid *gr = (TGrid*)sb->owner;
	TGrid_EditEnd(gr);
	Refresh((TWins*)gr);
}

static void TGrid_HSB_OnCount(TScrollBar *sb, int Count) {
  TGrid *gr = (TGrid*)sb->owner;
	TGrid_EditEnd(gr);
	Refresh((TWins*)gr);
}

void TGrid_SetMouseZ_Wheel(TGrid *self, int Value) {
	TScrollWin_SetMouseZ_Wheel((TScrollWin*)self, Value);
}

// ___________________________ Free __________________________________
bool TGrid__Destroy(TGrid *self) {
	Tfree(self->CellGC);
	Tfree(self->PaintGC);
	xfree(self->ColDat);
	xfree(self->RowDat);
	return TWins__Destroy((TWins*)self);
}

// ___________________________ Create ________________________________
void TGrid_CreateParams(TWins_Create_struct *wcs) {
  TGrid *self = wcs->wins;

	TScrollWin_CreateParams(wcs);

	self->destroy = (TDESTROY)TGrid__Destroy;
	self->Resize =  TGrid_Resize;
	self->DoPaint = TGrid_DoPaint;
//	self->DoSelect = TGrid_DoSelect;
	self->DoMouseDown = TGrid_DoMouseDown;
	self->DoMouseUp =   TGrid_DoMouseUp;
	self->DoMouseMove = TGrid_DoMouseMove;
	self->DocrResize = TGrid_DocrResize;

	self->DoPaintCell = TGrid_DoPaintCell;
//	self->DoPaintSelectCell = TGrid_DoPaintSelectCell;

	self->FixedCols = 1;
	self->FixedRows = 1;
	self->SelectIdxRow = -1;
	self->SelectIdxCol = -1;

	self->DefaultColWidth = 60;
	self->DefaultRowHeight = 18;
	self->FixedColsColor = 0x80000000;     // = no Color
	self->FixedRowsColor = 0x80000000;     // = no Color

//	self->KeyFu = mCanFocus | mK_Right | mK_Down | mK_Left | mK_Up | mK_Tab;
	self->KeyFu = mCanFocus | mK_Tab;

	self->VSB->OnCount = TGrid_VSB_OnCount;
	self->VSB->SmallChange = self->DefaultRowHeight;
	self->HSB->OnCount = TGrid_HSB_OnCount;
	self->HSB->SmallChange = 8;

	TGrid_SetColCount(self, 5);
	TGrid_SetRowCount(self, 3);

}

void TGrid_DefaultConf(TWins_Create_struct *wcs) {
  TGrid *self = wcs->wins;
	TScrollWin_DefaultConf(wcs);
	self->CellGC = TClassGC_Create((TWins*)self);
	self->PaintGC = TClassGC_Create((TWins*)self);
}

TGrid *TGrid_Create(TWins *Parent) {
  TWins_Create_struct wcs;
	TWins_Create_struct_init(&wcs, sizeof(TGrid), wtyp_Grid);
	wcs.Parent = Parent;
	wcs.CreateParams = TGrid_CreateParams;
	TGrid_DefaultConf(&wcs);
	return wcs.wins;
}

/*
TGrid *TGrid_CreateU(TWins *Parent, int Left, int Top, int Width, int Height, int BorderFu, int Align, bool mParent) {
  TGrid *self;
  TWins *pw;
	self = TGrid_Create(Parent);
	pw = self->pw;
	pw->top = Top;
	pw->left = Left;
	pw->width = Width;
	pw->height = Height;
//	((TGrid*)self)->pws->BorderFu = BorderFu;
//	if (Align) self->Align = Align;

	if (BorderFu >= 0) pw->BorderFu = BorderFu;
	self->Align = alClient;
	if (Align) pw->Align = Align;

	if (mParent) Set_Parent((TWins*)self, Parent);
	return self;
}
*/

TGrid *TGrid_Create_alClient(TWins *Parent, int BorderFu) {
  TGrid *self;
	self = TGrid_Create(Parent);
	if (BorderFu >= 0) self->pw->BorderFu = BorderFu;
	self->Align = alClient;
	self->pw->Align = alClient;
	Set_Parent((TWins*)self, Parent);
	return self;
}

