/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "cicons.h"

//*********************** Menu ************************

static TMenu *RootMenueWin;
static TMenu *AktMenueWin;


static void TMenu_PaintItem(TMenu *me, TMenuItem *mi) {
  int x, y1, hi, y, fonth;
  bool mk;

	if (!mi || (mi->fu & miNotVisible)) return;
	x = me->SideXOff;
	y1 = mi->y1 + me->ItemYOff;
	if (mi->fu & miSeparator) {
	    x += 4;
	    DrawLine(me->WinID, appXDat.bordL_GC, x, y1, me->width-5, y1);
	    y1++;
	    DrawLine(me->WinID, appXDat.bordH_GC, x, y1, me->width-5, y1);
	} else {

	    hi = mi->y2 - mi->y1;
	    x += 2;
	    XClearArea(appXDat.dpy, me->WinID, x, y1, me->width-4, hi, 0);
	    x++;

	    if ((mi->fu & miCanCheck) && (mi->fu & miChecked)) {
		DrawPic(me->WinID, get_pic16_ok(False), x, y1+1);
	    }
	    x += me->pic_OffX;
	    if (mi->pic) DrawPic(me->WinID, mi->pic, x, y1+1);

	    me->vds.Xoff = x + me->CaptionOffX + 1;
	    Font_GetMetrics((TFont*)me->vds.Font, &fonth, NULL);
	    me->vds.Yoff = y1 + 1 + ((hi - fonth) >> 1);
	    me->vds.s = mi->s;
	    me->vds.lg = mi->lg;
	    mk = me->Enabled && !(mi->fu & miNotEnabled);
	    TWins_PaintEnCaption((TWins*)me, mk);
	    if (me->s2_XOff && mi->s2 && !(mi->fu & miDisable_s2)) {
		me->vds.Xoff = me->s2_XOff;
		me->vds.s = mi->s2;
		me->vds.lg = pclen(mi->s2);
		TWins_PaintEnCaption((TWins*)me, mk);
	    }
	    me->vds.s = NULL;
	    me->vds.lg = 0;
	    
	    if (mi->next_menu) {
  XPoint points[3];
		y = y1 + (hi >> 1);
		points[0].x = me->width - 8;
		points[0].y = y - 4;
		points[1].x = me->width - 8;
		points[1].y = y + 4;
		points[2].x = me->width - 4;
		points[2].y = y;
		XFillPolygon(appXDat.dpy, me->WinID, appXDat.GC_000000, &points[0], 3, Convex, CoordModeOrigin);
	    }

	    if (mi == me->SelItem) {
		x = me->SideXOff + 2;
		if (mi->pic) {
		    if (me->pic_OffX > 2) {
			Fill_Rect(me->WinID, appXDat.Select_GC, x, y1, me->pic_OffX, hi);
			x += me->pic_OffX;
		    }
		    StdBorderPaint(me->WinID, mBorder, x, y1, mi->pic->width+2, mi->pic->height+2);
		    x += me->CaptionOffX;
		}
		Fill_Rect(me->WinID, appXDat.Select_GC, x, y1, me->width-x-3, hi);
	    }
	}
}
void TMenu_DoPaint(TMenu *self) {
  TMenuItem *mi;
	mi = self->MenuItem;
	while (mi) {
	    TMenu_PaintItem(self, mi);
	    mi = mi->next;
	}
}

static void TMenu_NewSelItem(TMenu *self, TMenuItem *mi) {
  TMenuItem *mi2;
  TWRect rc;
  int yb;
  TMenu *me;

	if (!RootMenueWin || (self->SelItem == mi)) return;
#ifdef _OS_WIN32
	SetTmp_hGC(self->WinID);
#endif /* _OS_WIN32 */
	mi2 = self->SelItem;
	self->SelItem = mi;
	if (mi) {
	    Get_ClientRect((TWins*)self, &rc);
	    yb = rc.top + rc.height;
	    if (mi->y2 > (yb - self->ItemYOff)) {
		self->ItemYOff = yb - mi->y2;
		TMenu_DoPaint(self);
	    } else if ((mi->y1 + self->ItemYOff) < rc.top) {
		self->ItemYOff = -mi->y1;
		TMenu_DoPaint(self);
	    }
	}

	if (mi2) {
	    if (mi2->next_menu) {
		TMenu_Close(mi2->next_menu);
	    }
	    TMenu_PaintItem(self, mi2);
	}
	TfreeLock(self);
	if (mi) {
	    TMenu_PaintItem(self, mi);
	    if (mi->next_menu && !(mi->fu & miNotEnabled)) {
		self->next_tmp_menu = mi->next_menu;
		self->next_tmp_menu->prev_tmp_menu = self;
		me = mi->next_menu;
		TfreeLock(me);
		TMenu_Show(me, self->owner, self->left + self->width - 3, self->top + mi->y1);
		me->DoOwner = self->DoOwner;
		TfreeUnLock(me);
	    }
	}
	if (RootMenueWin) {
	    if (AktMenueWin != self) {
		AktMenueWin = self;
		Set_Focus((TWins*)self);
		Std_GrabPointer((TWins*)self, appXDat.XCursors[XC_arrow]);
	    }
	    StdBorderPaint(self->WinID, self->BorderFu, 0, 0, self->width, self->height);
	}
#ifdef _OS_WIN32
	ReleaseTmp_hGC(self->WinID);
#endif /* _OS_WIN32 */
	TfreeUnLock(self);
}

static void TMenu_DoMouseDown(TMenu *self, XButtonEvent *ev) {
	if ((ev->x < 0) || (ev->x > self->width) || 
	    (ev->y < 0) || (ev->y > self->height)) {
	      TMenu_Close(RootMenueWin);
	}
}

static void TMenu_DoMouseUp(TMenu *self, XButtonEvent *ev) {
	if ((ev->x < 0) || (ev->x > self->width) || 
	    (ev->y < 0) || (ev->y > self->height)) return;

	TWins_DoMouseUp((TWins*)self, ev);
}

static void TMenu_DoMouseMove(TMenu *self, XMotionEvent *ev) {
  TMenuItem *mi;
  int y;

	TWins_DoMouseMove((TWins*)self, ev);
	if ((ev->x < self->SideXOff) || (ev->x > self->width) || 
	    (ev->y < 0) || (ev->y > self->height) ||
	    !RootMenueWin) return;

	mi = self->MenuItem;
	y = ev->y - self->ItemYOff;
	while (mi) {
	    if ((y >= mi->y1) && (y <= mi->y2) && 
		!(mi->fu & (miSeparator|miCanNotSelect|miNotVisible)) ) break;
	    mi = mi->next;
	}
	TMenu_NewSelItem(self, mi);
}

void TMenu_DoClick(TMenu *self, long KeyState, bool DblClick) {
  TMenu *um, *push_me;

	if (self->SelItem &&
	    !(self->SelItem->fu & (miSeparator|miNotVisible|miNotEnabled)) &&
	    !self->SelItem->next_menu) {

	    Std_UngrabPointer();

	    um = RootMenueWin;
	    while (um) {
		MapWinID(um->WinID, 0);
		um = um->next_tmp_menu;
	    }

	    if ((self->SelItem->fu & miAutoToggle) == miAutoToggle) {
		self->SelItem->fu ^= miChecked;
	    }
	    AktMenueWin = NULL;
	    push_me = RootMenueWin;
	    RootMenueWin = NULL;

	    TfreeLock(self);
	    if (self->OnClick) self->OnClick(self, KeyState, False);
	    if (self->SelItem && self->SelItem->OnClick) self->SelItem->OnClick(self, KeyState, self->SelItem);

	    RootMenueWin = push_me;
	    if (RootMenueWin == self) {
		TMenu_Close(RootMenueWin);
		TfreeUnLock(self);
	    } else {
		TfreeUnLock(self);
		TMenu_Close(RootMenueWin);
	    }
	}
}

static TMenuItem *FindMenuItem(TMenuItem *pmi, bool up) {
	while (pmi) {
	    if (!(pmi->fu & (miSeparator|miNotVisible|miNotEnabled|miCanNotSelect))) return pmi;
	    if (up) pmi = pmi->prev; else pmi = pmi->next;
	}
	return NULL;
}

static int TMenu_DoKeyDown(TMenu *self, TKeyEvent *KeyEv) {
  TMenuItem *pmi;

	if (KeyEv->keysym == XK_Down) {
	    if (self->SelItem) pmi = self->SelItem->next; else pmi = self->MenuItem;
	    pmi = FindMenuItem(pmi, False);
	    if (!pmi) pmi = FindMenuItem(self->MenuItem, False);
	    TMenu_NewSelItem(self, pmi);

	} else if (KeyEv->keysym == XK_Up) {
	    pmi = self->SelItem;
	    if (pmi) pmi = pmi->prev;
	    pmi = FindMenuItem(pmi, True);
	    if (!pmi) {
		pmi = self->MenuItem;
		while (pmi->next) pmi = pmi->next;
		pmi = FindMenuItem(pmi, True);
	    }
	    TMenu_NewSelItem(self, pmi);

	} else if (KeyEv->keysym == XK_Right) {
	    if (self->SelItem && self->SelItem->next_menu) {
		pmi = FindMenuItem(self->SelItem->next_menu->MenuItem, False);
		if (pmi) TMenu_NewSelItem(self->SelItem->next_menu, pmi);
	    } else if (self->DoOwner) self->DoOwner(self->owner, mioXK_Right);

	} else if (KeyEv->keysym == XK_Left) {
	    if (self->prev_tmp_menu) {
		pmi = self->SelItem;
		self->SelItem = NULL;
		TMenu_PaintItem(self, pmi);
		pmi = self->prev_tmp_menu->SelItem;
		self->prev_tmp_menu->SelItem = NULL;
		TMenu_NewSelItem(self->prev_tmp_menu, pmi);
	    } else if (self->DoOwner) self->DoOwner(self->owner, mioXK_XK_Left);
	}
	return 1;
}

static int TMenu_DoKeyUp(TMenu *self, TKeyEvent *KeyEv) {
	if ((KeyEv->keysym == XK_Return) || (KeyEv->keysym == XK_space)) {
	    TMenu_DoClick(self, KeyEv->state, False);

	} else if (KeyEv->keysym == XK_Escape) {
	    TMenu_Close(RootMenueWin);
	}
	return 1;
}

void TMenu_DoMenuOpen(TMenu *me) {
	if (me->OnMenuOpen) me->OnMenuOpen(me);
}

void TMenu_DoShow(TMenu *self, int Left, int Top) {
  int w, h, fh, fhi, m, xg, yg, ws2, UpDownCount;
  TMenuItem *mi;
  bool CanCheck_mk = False;
  bool pic_mk = False;

	if (!(self->RunStat & meRS_OPEN)) return;
	if (self->Visible) return;

	ws2 = 0;
	self->DoOwner = NULL;
	self->Visible = True;
	self->left = Left;
	self->ItemYOff = 0;
	self->SelItem = NULL;
	self->pic_OffX = 0;
	if (self->DoMenuOpen) {
	    UpDownCount = MouseDown.UpDownCount;
	    self->DoMenuOpen(self);
	    if (UpDownCount != MouseDown.UpDownCount) {
 printf("%s(%d): TMenu_DoShow Mouse UpDownCount %d <> %d\n", FL, UpDownCount, MouseDown.UpDownCount);
		self->RunStat &= ~(meRS_OPEN|meRS_SubLeft);
		if (self->DoMenuClose) self->DoMenuClose(self);
		return;
	    }
	}
	w = self->SideXOff + 32;
	h = 2;
	Font_GetMetrics((TFont*)self->vds.Font, &fh, NULL);
	fh++;
	if (fh < self->ItemMinHeight) fh = self->ItemMinHeight;
	mi = self->MenuItem;
	while (mi) {
	    if (!(mi->fu & miNotVisible)) {
		mi->y1 = h;
		mi->fu &= ~miSeparator;
		if (mi->lg) {
		    if ((mi->lg == 1) && (mi->s[0] == '-')) {
			mi->fu |= miSeparator;
			h += 2;
		    } else {
			self->vds.s = mi->s;
			self->vds.lg = mi->lg;
			m = TextWidthColorString(&self->vds) + self->SideXOff + 14;
			if (m > w) w = m;
			if (mi->s2 && !(mi->fu & miDisable_s2)) {
			    self->vds.s = mi->s2;
			    self->vds.lg = pclen(mi->s2);
			    m = TextWidthColorString(&self->vds);
			    if (m > ws2) ws2 = m;
			}
			self->vds.s = NULL;
			self->vds.lg = 0;

			if (mi->fu & miCanCheck) {
			    CanCheck_mk = True;
			    if (self->CaptionOffX < 18) self->CaptionOffX = 18;
			}

			fhi = fh;
			if (mi->pic) {
			    pic_mk = True;
			    m = mi->pic->width + 2;
			    if (m > 34) m = 34;
			    if (m < 16) m = 16;
			    self->CaptionOffX = m;
			    m = mi->pic->height + 2;
			    if (m > 34) m = 34;
			    if (fhi < m) fhi = m;
			}
			h += fhi;
		    }
		}
	    }
	    mi->y2 = h;
	    mi = mi->next;
	}
	if (CanCheck_mk && pic_mk) self->pic_OffX = 18;
	if (ws2) ws2 += 16;
	m = w + ws2 + self->CaptionOffX + self->pic_OffX;
	if ((self->min_width > 0) && (m < self->min_width)) m = self->min_width;
	if ((self->max_width > 0) && (m > self->max_width)) m = self->max_width;
	self->width = m;
	if (ws2) self->s2_XOff = m + 4 - ws2;
// printf("%s(%d): %d %d %d\n", FL, m, ws2, self->s2_XOff);
	h += 2;
	if ((self->min_height > 0) && (h < self->min_height)) h = self->min_height;
	if ((self->max_height > 0) && (h > self->max_height)) h = self->max_height;

	get_gravity_offset(self->Gravity, self->width, h, &xg, &yg);
	self->left -= xg;
	if ((self->left + self->width) > appXDat.Root_width) {
	    self->left = appXDat.Root_width - self->width;
	    self->RunStat |= meRS_SubLeft;
	    if (self->prev_tmp_menu) {
		self->left = self->prev_tmp_menu->left - self->width;
	    }

	} else if (self->prev_tmp_menu && (self->prev_tmp_menu->RunStat & meRS_SubLeft)) {
	    self->left = self->prev_tmp_menu->left - self->width;
	}


	if (self->left < 0) {
	    self->left = 0;
	    self->RunStat &= ~meRS_SubLeft;
	}

	Top -= yg;
	m = appXDat.Root_height - 16;
	if ((Top + h) > m) {
	    Top = m - h;
	    if (Top < 0) Top = 0;
	}
	self->top = Top;

	if ((Top + h) > m) {
	    h = m - Top;
	    if (h < 2) h = 2;
	}
	self->height = h;

	self->IsParent = True;
	Resize((TWins*)self);
	MapWinID(self->WinID, 2);
	if (!RootMenueWin) {
	    Std_GrabPointer((TWins*)self, appXDat.XCursors[XC_arrow]);
	    RootMenueWin = self;
	    AktMenueWin = self;
	    Set_Focus((TWins*)self);
	}
}

void TMenu_DoClose(TMenu *self) {
	if (self->OnMenuClose) self->OnMenuClose(self);
}

void TMenu_Close(TMenu *self) {
	if (!self) self = RootMenueWin;
	if (!self) return;
	if (self->prev_tmp_menu) {
	    self->prev_tmp_menu->next_tmp_menu = NULL;
	    self->prev_tmp_menu = NULL;
	}
	self->RunStat &= ~(meRS_OPEN|meRS_SubLeft);
	if (!self->Visible) return;
	self->Visible = False;
	if (self->next_tmp_menu) TMenu_Close(self->next_tmp_menu);
	self->next_tmp_menu = NULL;
	MapWinID(self->WinID, 0);
	if (AktMenueWin == self) {
	    AktMenueWin = NULL;
	    if (appXDat.GrabWin == (TWins*)self) Std_UngrabPointer();
	}
	if (RootMenueWin == self) {
//	    XFlush(appXDat.dpy);
  TForm *Form = GetParentForm((TWins*)self);
	    if (Form) {
		if (TWins_ChkExists(Form->LastNoMenuFocusWin)) 
			Set_Focus(Form->LastNoMenuFocusWin);
		if (self->DoOwner) self->DoOwner(self->owner, mioClose);
	    }
	    RootMenueWin = NULL;
	}
	if (self->DoMenuClose) self->DoMenuClose(self);
}

bool TMenuItem__Destroy(TMenuItem *self) {
	if (self->next_menu && 
	    (self->next_menu->RunStat & meRS_ParentCanDestroy)) Tfree(self->next_menu);
	xfree(self->s2);
	xfree(self->s);
	return False;
}

static TMenuItem *TMenu_DoMenuItemCreate(TMenu *self) {
  TMenuItem *mi;
	mi = ttc_create(sizeof(TMenuItem), wtyp_MenuItem);
	mi->destroy = (TDESTROY)TMenuItem__Destroy;
	return mi;
}

void TMenuItem_SetCaption(TMenuItem *mi, const char *s) {
	if (mi && ppcchange_cmp(&mi->s, s)) mi->lg = pclen(s);
}

TMenuItem *TMenu_InsItem(TMenu *self, const char *s, int idx) {
  TMenuItem *mi, *prv;
  TMenuItem **ppmi;

	if (!self->DoMenuItemCreate) return NULL;
	prv = NULL;
	ppmi = &self->MenuItem;
	while (*ppmi && idx) {
	    prv = *ppmi;
	    ppmi = &prv->next;
	    idx--;
	}
	mi = self->DoMenuItemCreate(self);
	mi->prev = prv;
	mi->next = *ppmi;
	if (mi->next) mi->next->prev = mi;
	*ppmi = mi;
	ppcchange(&mi->s, s);
	mi->lg = pclen(s);
	return mi;
}

void TMenu_DelItem(TMenu *self, TMenuItem *mi) {
	if (mi) {
	    if (mi == self->SelItem) self->SelItem = NULL;
	    if (mi == self->MenuItem) {
		self->MenuItem = mi->next;
		if (self->MenuItem) self->MenuItem->prev = NULL;
	    } else {
		if (mi->prev) mi->prev->next = mi->next;
		if (mi->next) mi->next->prev = mi->prev;
	    }
	    Tfree(mi);
	}
}

void TMenu_DelItemS(TMenu *self, const char *s) {
  TMenuItem *mi = self->MenuItem;
  TMenuItem *tmpmi;

	while (mi) {
	    tmpmi = mi->next;
	    if (pccmp(s, mi->s)) TMenu_DelItem(self, mi);
	    mi = tmpmi;
	}
}

void TMenu_Cls(TMenu *self) {
  TMenuItem *mi, *nmi;
	mi = self->MenuItem;
	while (mi) {
	    nmi = mi->next;
	    TMenu_DelItem(self, mi);
	    mi = nmi;
	}
	self->SelItem = NULL;
}

TMenuItem *TMenu_AddItem(TMenu *self, const char *s, long fu, void (*OnClick)(struct TMenu*, long, struct TMenuItem*)) {
  TMenuItem *mi;
	if ((fu & miAddInv) == 0) mi = TMenu_InsItem(self, s, 0x7fffffff);
		else mi = TMenu_InsItem(self, s, 0);
	mi->fu = fu & 0xffffff;
	mi->OnClick = OnClick;
	if (fu & miAdd_setSel) self->SelItem = mi;
	return mi;
}

TMenuItem *TMenu_GetItemI(TMenu *self, int idx) {
  TMenuItem *mi = self->MenuItem;
	while (mi && (idx > 0)) {
	    mi = mi->next;
	    idx--;
	}
	return mi;
}

TMenuItem *TMenu_GetItemS(TMenu *self, const char *s) {
  TMenuItem *mi = self->MenuItem;
  TMenuItem *tmpmi;

	while (mi) {
	    tmpmi = mi->next;
	    if (pccmp(s, mi->s)) return mi;
	    mi = tmpmi;
	}
	return NULL;
}

TMenu *GetAktMenueWin() {
  TMenu *result;
	result = AktMenueWin;
	if (!result) result = RootMenueWin;
	return result;
}
// ___________________________ Destroy _______________________________
bool TMenu__Destroy(TMenu *self) {
	TMenu_Cls(self);
	return TWins__Destroy((TWins*)self);
}

// ___________________________ Create ________________________________
void TMenu_CreateParams(TWins_Create_struct *wcs) {
  TMenu *self = wcs->wins;
	self->Visible = False;

	self->destroy = (TDESTROY)TMenu__Destroy;
	self->DoMenuItemCreate = TMenu_DoMenuItemCreate;
	self->PopupMenu = NULL;

	self->SetVisible = NULL;
	self->DoPaint = TMenu_DoPaint;
	self->DoMouseDown = TMenu_DoMouseDown;
	self->DoMouseUp = TMenu_DoMouseUp;
	self->DoMouseMove = TMenu_DoMouseMove;
	self->DoClick = TMenu_DoClick;
	self->DoKeyDown = TMenu_DoKeyDown;
	self->DoKeyUp = TMenu_DoKeyUp;
	self->DoMenuOpen = TMenu_DoMenuOpen;
	self->DoMenuClose = TMenu_DoClose;
	self->DoShow = TMenu_DoShow;

	self->BorderFu = mBorder | mBorderW2 | mBorderBu;
//	self->Color = 0xe0e0e0;
	self->KeyFu = mCanFocus;
	self->ItemMinHeight = 18;

	wcs->XMask |= CWOverrideRedirect;
	wcs->XAttr.override_redirect = 1;
	wcs->ParentWinID = appXDat.RootW;
}

TMenu *TMenu_Create(TWins *Form) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TMenu), wtyp_Menu);
	wcs.Parent = Form;
	wcs.CreateParams = TMenu_CreateParams;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}



