/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"

//*********************** MenuButton ************************

void TMenuButton_mio(TWins *self, int fu);

static void TMenuButton_DoPaint(TMenuButton *self) {
  TPic *pic;
	pic = self->pic;
	if (pic) {
	    if (!TestWinIsEnabled((TWins*)self) && self->dis_pic) pic = self->dis_pic;
	    DrawPic(self->WinID, pic, 1, 1);
	}
	TWins_DoPaint((TWins*)self);
}

static TMenuButton *getnext(TMenuButton *mb) {
  TWins *wins = mb->next;
	while (wins && (wins->wtyp != wtyp_MenuButton)) wins = wins->next;
	return (TMenuButton*)wins;
}

static TMenuButton *getprev(TMenuButton *mb) {
  TWins *wins = mb->prev;
	while (wins && (wins->wtyp != wtyp_MenuButton)) wins = wins->prev;
	return (TMenuButton*)wins;
}

void TMenuButton_SetPosition(TMenuButton *mb) {
  TMenuButton *mbu = mb->mbuRoot;
  int x = 0;
  int sw;
	if (mbu) x = mbu->left;
	while (mbu) {
	    if (mbu->Visible) {
		if (mbu->AutoSize) {
		    sw = TextWidthColorString(&mbu->vds) + 4;
		    if (mbu->pic) {
			sw += mbu->pic->width + 2;
			mbu->CaptionOffX = (mbu->pic->width >> 1);
		    } else sw += 4;
		    if (sw < 18) sw = 18;
		    SetRC((TWins*)mbu, x, mbu->top, sw, mbu->height);
		}
		x += mbu->width;
	    }
	    mbu = getnext(mbu);
	}
}

static void TMenuButton_SetVisible(TMenuButton *self, bool Value) {
	TWins_SetVisible((TWins*)self, Value);
	TMenuButton_SetPosition(self);
}

static void TMenuButton_Border(TMenuButton *self, int fu) {
  TMenuButton *mbu = self->mbuRoot;

	while (mbu) {
	    if (mbu->BorderFu) DrawRect(mbu->WinID, appXDat.Default_GC, 0, 0, mbu->width, mbu->height);
	    mbu->BorderFu = 0;
	    mbu = getnext(mbu);
	}

	if (fu == -1) fu = self->mbuRoot->mbuStat;
	if (fu) {
	    if (fu == 1) self->BorderFu = mBorder;
	    if (fu == 2) self->BorderFu = mBorder | mBorderL;
	    Refresh((TWins*)self);
	}
}


static void TMenuButton_MenuOpen(TMenuButton *self) {
  TWRect rc;
	self->mbuRoot->mbuStat = 2;
	if (self->menu) {
	    GetXWinAbs_rc(self->WinID, &rc);
	    rc.top += self->height;
	    TMenu_Show(self->menu, (TWins*)self, rc.left, rc.top);
	    self->menu->DoOwner = TMenuButton_mio;
//	    self->menu->owner = (TWins*)self;
	}
	TMenuButton_Border(self, -1);
}

static void TMenuButton_ReOpen(TMenuButton *old, TMenuButton *new) {
  TMenuButton *root;
	if (old) root = old->mbuRoot; else
	if (new) root = new->mbuRoot; else return;

	TMenu_Close(NULL);
	if (new) {
	    Set_Focus((TWins*)new);
	    if (new->menu) {
		if (root->mbuStat == 2) {
		    TMenuButton_MenuOpen(new);
		} else {
		    TMenuButton_Border(new, -1);
		}
	    } else {
		TMenuButton_Border(new, 1);
	    }
	}
}

static void TMenuButton_next(TMenuButton *mbu) {
  TMenuButton *tmb;
	tmb = getnext(mbu);
	while (tmb && !TestWinCanFocus((TWins*)tmb)) tmb = getnext(tmb);
	if (!tmb) {
	    tmb = mbu->mbuRoot;
	    while (tmb && !TestWinCanFocus((TWins*)tmb)) tmb = getnext(tmb);
	}
	TMenuButton_ReOpen(mbu, tmb);
}

static void TMenuButton_prev(TMenuButton *mbu) {
  TMenuButton *tmb, *ttmb;
	tmb = getprev(mbu);
	while (tmb && !TestWinCanFocus((TWins*)tmb)) tmb = getprev(tmb);
	if (!tmb) {
	    ttmb = tmb = mbu;
	    while (ttmb) {
		tmb = ttmb;
		ttmb = getnext(ttmb);
	    }
	    while (tmb && !TestWinCanFocus((TWins*)tmb)) tmb = getprev(tmb);
	}
	TMenuButton_ReOpen(mbu, tmb);
}

void TMenuButton_mio(TWins *self, int fu) {
  TMenuButton *sw = (TMenuButton*)self;
	if (fu == mioClose) {
	    TMenuButton_Border(sw, 0);
	    sw->mbuRoot->mbuStat = 0;
	} else if (fu == mioXK_XK_Left) {
	    TMenuButton_prev(sw);
	} else if (fu == mioXK_Right) {
	    TMenuButton_next(sw);
	}
}


static void TMenuButton_Close(TMenuButton *self) {
  TForm *Form = GetParentForm((TWins*)self);

	TMenu_Close(NULL);
	self->mbuRoot->mbuStat = 0;
	TMenuButton_Border(self, 0);
	if (Form && TWins_ChkExists(Form->LastNoMenuFocusWin)) {
	    Set_Focus(Form->LastNoMenuFocusWin);
	}
} 

void TMenuButton_DoMouseDown(TMenuButton *self, XButtonEvent *ev) {
	if ((ev->x < 0) || (ev->x > self->width) || (ev->y < 0) || (ev->y > self->height)) {
	    TMenu_Close(NULL);
	    return;
	}
	TWins_DoMouseDown((TWins*)self, ev);

	if ((ev->button == 1) && TestWinIsEnabled((TWins*)self)) {
	    if (self->menu) {
		if (self->mbuRoot->mbuStat == 0) {
		    TMenuButton_MenuOpen(self);
		} else {
		    TMenuButton_Close(self);
		}
	    } else {
		TMenuButton_Border(self, 2);
	    }
	}
}

void TMenuButton_DoMouseUp(TMenuButton *self, XButtonEvent *ev) {
	TWins_DoMouseUp((TWins*)self, ev);
	if ((ev->button == 1) && !self->menu) {
	    TMenuButton_Close(self);
	}
}

void TMenuButton_DoClick(TMenuButton *self, long KeyState, bool DblClick) {
	if (!self->menu) TWins_DoClick((TWins*)self, KeyState, DblClick);
}

static TMenuButton *EnterMenuButton;

static void TMenuButton_DoMouseOver(TMenuButton *self, XCrossingEvent *ev, bool Leave) {
  int fu;

	if (!self->mbuRoot) return;
	fu = self->mbuRoot->mbuStat;
	if (Leave) {
	    if (fu == 0) TMenuButton_Border(self, 0);
	} else {
	    switch (fu) {
	    case 0:
		TMenuButton_Border(self, 1);
		break;
	    case 1:
		TMenuButton_ReOpen(NULL, self);
		break;
	    case 2:
		if (EnterMenuButton != self)
			TMenuButton_ReOpen(NULL, self);
		break;
	    }
	    EnterMenuButton = self;
	}
	TWins_DoMouseOver((TWins*)self, ev, Leave);
}

int TMenuButton_DoKeyDown(TMenuButton *self, TKeyEvent *KeyEv) {

	if (KeyEv->keysym == XK_Left) {
		TMenuButton_prev(self);
	} else if (KeyEv->keysym == XK_Right) {
		TMenuButton_next(self);
	} else if (KeyEv->keysym == XK_Down) {
		TMenuButton_MenuOpen(self);
	} else if (((KeyEv->keysym == XK_space) || 
		    (KeyEv->keysym == XK_Return)) && !self->menu) {
		TMenuButton_Border(self, 2);
	} else return 0;
	return 1;
}

int TMenuButton_DoKeyUp(TMenuButton *self, TKeyEvent *KeyEv) {
	if (((KeyEv->keysym == XK_space) || 
	     (KeyEv->keysym == XK_Return)) && !self->menu) {
	    TMenuButton_Close(self);
	    TMenuButton_DoClick(self, KeyEv->state, False);
	}
	return 1;
}

static void TMenuButton_SetCaption(TMenuButton *self, const char *s) {
	TWins_SetCaption((TWins*)self, s);
	TMenuButton_SetPosition(self);
//	Refresh(self);
}

static void TMenuButton_Exec(TForm *Form) {
  TMenuButton *mb = Form->Root_MenuButton;
	if (!mb) return;
	if (mb->mbuStat) {
	    if (TWins_ChkExists(Form->LastNoMenuFocusWin)) 
			Set_Focus(Form->LastNoMenuFocusWin);
	    mb->mbuStat = 0;
	    TMenu_Close(NULL);
	} else {
	    mb->mbuStat = 1;
	    Set_Focus((TWins*)mb);
	}
	TMenuButton_Border(mb, -1);
}

void TMenuButton_SetAsRoot(TForm *Form, TMenuButton *mbu) {
	Form->Root_MenuButton = mbu;
	Form->DoMenuButtonExec = TMenuButton_Exec;
}


/*_____________________________ Create __________________________________*/
static void TMenuButton_CreateParams(TWins_Create_struct *wcs) {
  TMenuButton *self, *mbuRoot, *mbu;
  int x;

	self = wcs->wins;
	mbuRoot = wcs->SubP;
	self->SetCaption = TMenuButton_SetCaption;
	self->SetVisible = TMenuButton_SetVisible;

	self->DoPaint = TMenuButton_DoPaint;
	self->DoMouseDown = TMenuButton_DoMouseDown;
	self->DoMouseUp = TMenuButton_DoMouseUp;
	self->DoClick = TMenuButton_DoClick;
	self->DoMouseOver = TMenuButton_DoMouseOver;
	self->DoKeyDown = TMenuButton_DoKeyDown;
	self->DoKeyUp = TMenuButton_DoKeyUp;

	self->CaptionFu = mCaptionPaint;
	self->KeyFu = mCanFocus;
	self->AutoSize = True;

	if (mbuRoot) {
	    if (mbuRoot->wtyp & wtyp_Form) {
		self->height = 18;
		self->top = 2;
		self->left = 1;
		TMenuButton_SetAsRoot((TForm*)mbuRoot, self);
		self->mbuRoot = self;
	    } else if (mbuRoot->wtyp == wtyp_MenuButton) {

		self->mbuRoot = (TMenuButton*)mbuRoot;
		mbu = self->mbuRoot;
		x = mbu->left; // + mbu->width;
		while (mbu) {
		    x += mbu->width;
		    mbu = getnext(mbu);
		}
//		mbu->next = self;
		self->left = x;
		self->top = mbuRoot->top;
		self->height = mbuRoot->height;
	    } else {
#ifdef POINTER64BIT
 printf("%s(%d): TODO mbuRoot=0x%08lx wtyp=0x%x\n", FL, (int64_t)mbuRoot, mbuRoot->wtyp);
#else
 printf("%s(%d): TODO mbuRoot=0x%08x wtyp=0x%x\n", FL, (unsigned int)mbuRoot, mbuRoot->wtyp);
#endif
	    }
	} else {
	    self->mbuRoot = self;
	    self->top = 2;
	    self->left = 1;
	}
}

TMenuButton *TMenuButton_Create(TWins *Parent, TMenuButton *mbuRoot) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TMenuButton), wtyp_MenuButton);
	wcs.Parent = Parent;
	wcs.CreateParams = TMenuButton_CreateParams;
	wcs.SubP = mbuRoot;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}

TMenuButton *TMenuButton_CreateS(TWins *Parent, TMenuButton *mbuRoot, const char *s) {
  TMenuButton *self = TMenuButton_Create(Parent, mbuRoot);
	if (s) SetCaption((TWins*)self, s);
	Set_Parent((TWins*)self, Parent);
	return self;
}
