/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"

TPage *TPage_GetNext(TPage *pg, bool ChkVisible) {
  TWins *wins = pg->next;
	if (ChkVisible) {
	    while (wins && ((wins->wtyp != wtyp_TPage) || 
			    !((TPage*)wins)->PageVisible)) wins = wins->next;
	} else {
	    while (wins && (wins->wtyp != wtyp_TPage)) wins = wins->next;
	}
	return (TPage*)wins;
}

TPage *TPage_GetPrev(TPage *pg, bool ChkVisible) {
  TWins *wins = pg->prev;
	if (ChkVisible) {
	    while (wins && ((wins->wtyp != wtyp_TPage) || 
			    !((TPage*)wins)->PageVisible)) wins = wins->prev;
	} else {
	    while (wins && (wins->wtyp != wtyp_TPage)) wins = wins->prev;
	}
	return (TPage*)wins;
}

TPageControl *TPage_GetTPageControl(TPage *pg) {
	if (!pg || !ttc_chk_is_wtyp(pg->PageControl, wtyp_TPageControl)) return NULL;
	return pg->PageControl;
}

TPage *GetParentPage(TWins* wins) {
	while (wins) {
	    if (wins->wtyp == wtyp_TPage) break;
	    wins = wins->prev_child;
	}
	return (TPage*)wins;
}

static void TPage_SetCaption(TPage *self, const char *s) {
  TPageControl *pgc;

	if (pccmp(self->vds.s, s)) return;
	vds_SetStr(&self->vds, s);
	if (self->OnSetCaption) self->OnSetCaption(self, s);
	pgc = self->PageControl;
	if (pgc && pgc->ResizeStartMk) {
	    pgc->ResizeStartMk = False;
	    Refresh((TWins*)pgc);
	}
}

void TPage_SetPageAktiv(TPage *self) {
  int fu = 0;
  TPage *pg;
  TPageControl *pgc;

	if (!self || !self->IsParent) return;
	pgc = TPage_GetTPageControl(self);
	if (pgc && (pgc->AktPage != self)) {
	    if (pgc->OnChangeTab) pgc->OnChangeTab(pgc, pgc->AktPage, self, &fu);
	    if (!(fu & pgNotChange)) {
		pgc->AktPage = self;
		pg = TPageControl_GetFirstPage(pgc, False);
		while (pg) {
		    if (pg != self) TWins_SetVisible((TWins*)pg, False);
		    pg = TPage_GetNext(pg, False);
		}
		TWins_SetVisible((TWins*)self, True);
		pgc->ResizeStartMk = False;
		Refresh((TWins*)pgc);
		if (self->FocusWin) Set_Focus(self->FocusWin);
		if (pgc->OnPageActivate) pgc->OnPageActivate(pgc, pgc->AktPage);
	    }
	}
}

static void TPage_SetVisible(TPage *self, bool Value) {
  TPageControl *pgc;
	if (self->PageVisible != Value) {
	    self->PageVisible = Value;
	    pgc = TPage_GetTPageControl(self);
	    if (pgc && (pgc->RunStatus & runst_Paint1)) Refresh((TWins*)pgc);
	}
}

/* ____________________ TPageControl ____________________________________ */
static void TPageControl_GetClientRect(TPageControl *self, TWRect *rc) {
	rc->left = 2;
	rc->top = self->TabsGesHeight + 2;
	rc->width = self->width - 4;
	rc->height = self->height - self->TabsGesHeight - 4;
	if (rc->height < 4) rc->height = 4;
}

static void TPageControl_SetPagesSize(TPageControl *self) {
  int i, x, w, mt0, h, xa, xe;
  TPage *pg;
  TVDrawString *vds;

	for (i=0; i<16; i++) self->TabMaxX[i] = -1;

	mt0 = self->xv1 = self->xv2 = self->Ynr = 0;
	x = 2;
	pg = TPageControl_GetFirstPage(self, False);
	while (pg) {
	    if (pg->PageVisible) {
		w = 0;
		vds = &pg->vds;
		if (vds->s) {
		    vds->WinID = self->WinID;
		    if (*vds->s == ' ') {
			*vds->s = '*';
			w = TextWidthColorString(vds);
			*vds->s = ' ';
		    } else {
			w = TextWidthColorString(vds);
		    }
		}
		pg->Wtxt = w;
		w += self->TabTxtOff;
		if (w < self->MinTabWidth) w = self->MinTabWidth;
		if (w < 16) w = 16;

		if ((mt0 > 0) && ((x + w + 4) > self->width)) {
		    self->Ynr++;
		    x = 2;
		    mt0 = 0;
		} else mt0++;

		xa = x;
		xe = x + w;
		if (pg->Visible) {
		    xa = x - 2;
		    xe = xe + 2;
		    self->AktYnr = self->Ynr;
		    self->xv1 = xa;
		    self->xv2 = xe+1;
		}
		pg->TabXa = xa;
		pg->TabXe = xe;
		pg->TabYnr = self->Ynr;
		x = x + w + 1;
		if (self->Ynr < 16) self->TabMaxX[self->Ynr] = x;
	    } //if PageVisible
	    pg = TPage_GetNext(pg, False);
	}
	h = self->TabsHeight * (self->Ynr + 1) - self->Ynr * 2;
	if (self->Ynr > 0) h = h + 2;
	self->TabsGesHeight = h;
}

static void TPageControl_Resize(TPageControl *self) {
  TPage *pg;
  TWRect rc;

	if (self->IsParent) {
	    TWins_AdjustSize((TWins*)self);
	    if (!self->ResizeStartMk && self->OnResize && self->Visible) {
		self->OnResize(self);
		self->ResizeStartMk = True;
	    }
	    TPageControl_SetPagesSize(self);
	    Get_ClientRect((TWins*)self, &rc);
	    pg = TPageControl_GetFirstPage(self, False);
	    while (pg) {
		pg->left = rc.left;
		pg->top = rc.top;
		pg->width = rc.width;
		pg->height = rc.height;
		Resize((TWins*)pg);
		pg = TPage_GetNext(pg, False);
	    }
	    if (self->AktPage && 
		!(self->RunStatus & runst_Activate) &&
		self->OnPageActivate) self->OnPageActivate(self, self->AktPage);
	}
	if (self->DoActivate) self->DoActivate(self);

}

static void TPageControl_PaintTabs(TPageControl *self, int y0, int Ynr, GC gc) {
  int x, x2, y, y2, xt;
  TPage *pg, *pg_prev;
  Window WinID;
  TVDrawString *vds;

	pg_prev = NULL;
	y2 = self->TabsGesHeight;
	WinID = self->WinID;
	if (Ynr < 16) Fill_Rect(WinID, gc, 0, y0+3, self->TabMaxX[Ynr], y2-y0-3);
	pg = TPageControl_GetFirstPage(self, True);
	while (pg) {
	    if (pg->TabYnr == Ynr) {
		x = pg->TabXa;
		x2 = pg->TabXe;
		y = y0;
		if (pg->Visible) {
		    Fill_Rect(WinID, gc, x, y, x2-x, y2-y+1);
		} else {
		    y += 2;
		}
		pg->TabYo = y;
		pg->TabYu = y0 + self->TabsHeight;

		xt = ((x2 - x) - pg->Wtxt) >> 1;
		vds = &pg->vds;
		vds->WinID = WinID;
		vds->Xoff = x+xt;
		vds->Yoff = y+2;
		TWins_PaintEnCaption((TWins*)pg, TestWinIsEnabled((TWins*)pg));

		if (!pg_prev || !pg_prev->Visible) {
		    DrawLine(WinID, appXDat.GC_FFFFFF, x, y+2, x, y2);
		    DrawLine(WinID, appXDat.GC_FFFFFF, x, y+2, x+2, y);
		}

		DrawLine(WinID, appXDat.GC_FFFFFF, x+2, y, x2-2, y);
		DrawLine(WinID, appXDat.bordL_GC, x2-2, y, x2, y+2);
		DrawLine(WinID, appXDat.GC_000000, x2, y+2, x2, y2);
		pg_prev = pg;
	    } //if FPageVisible 
	    pg = TPage_GetNext(pg, True);
	} 
}

static void TPageControl_DrawFocus(TPageControl *self) {
  TPage *Page = self->AktPage;
  if (Page) DrawRectFocus(self->WinID, Page->TabXa+2, Page->TabYo+2,
                                       Page->TabXe-Page->TabXa-4,
                                       Page->TabYu-Page->TabYo-4);
}

static void TPageControl_DoPaint(TPageControl *self) {
  int i, y, w, h;
  GC gc;

	gc = GetPaintGC(self->Color);
	Fill_Rect(self->WinID, gc, 0, 0, self->width, self->TabsGesHeight+2);

	y = 0;
	for (i=self->Ynr; i>=0 ; i--) {
	    if (self->AktYnr != i) {
		TPageControl_PaintTabs(self, y, i, gc);
		y = y + self->TabsHeight - 2;
	    }
	}
	if (self->Ynr > 0) y += 2;
	TPageControl_PaintTabs(self, y, self->AktYnr, gc);

	if (self->AktPage && (self->KeyFu & mHasFocus)) {
	    TPageControl_DrawFocus(self);
	}
	w = self->width-1;
	h = self->height-1;
	y = self->TabsGesHeight;

	if (self->xv1 != self->xv2) DrawLine(self->WinID, appXDat.GC_FFFFFF, 0, y, self->xv1, y);
	DrawLine(self->WinID, appXDat.GC_FFFFFF, self->xv2, y, w, y);

	DrawLine(self->WinID, appXDat.GC_FFFFFF, 0, y, 0, h);
	DrawLine(self->WinID, appXDat.GC_404040, 0, h, w, h);
	DrawLine(self->WinID, appXDat.GC_404040, w, y, w, h);
}

static void TPageControl_DoMouseDown(TPageControl *self, XButtonEvent *ev) {
  TPage *pg;

	TWins_DoMouseDown((TWins*)self, ev);
	if (ev->button != 1) return;

	pg = TPageControl_GetFirstPage(self, False);
	while (pg) {
	    if (pg->PageVisible && TestWinIsEnabled((TWins*)pg)) {
		if ((ev->y >= pg->TabYo) && (ev->y <= pg->TabYu)) {
		    if ((ev->x >= pg->TabXa) && (ev->x <= pg->TabXe)) {
			TPage_SetPageAktiv(pg);
			break;
		    }
		}
	    }
	    pg = TPage_GetNext(pg, False);
	}
}

static int TPageControl_DoKeyDown(TPageControl *self, TKeyEvent *KeyEv) {

	if (self->OnKeyDown) self->OnKeyDown(self, KeyEv);

	if (self->AktPage) {
	    if ((KeyEv->keysym == XK_Right) && (self->KeyFu & mK_Right)) {
		TPage *pg = TPage_GetNext(self->AktPage, True);
		if (!pg) pg = TPageControl_GetFirstPage(self, True);
		TPage_SetPageAktiv(pg);
		return 1;
	    } else if ((KeyEv->keysym == XK_Left) && (self->KeyFu & mK_Left)) {
		TPage *pg = TPage_GetPrev(self->AktPage, True);
		if (!pg) pg = TPageControl_GetLastPage(self, True);
		TPage_SetPageAktiv(pg);
		return 1;
	    } 
	}
	return 0;
}

static void TPageControl_DoFocus(TPageControl *self, int Change) {
	TWins_DoFocus((TWins*)self, Change);
	TPageControl_DrawFocus(self);
}

/*________________________ Ctl __________________________________________*/
TPage *TPageControl_GetFirstPage(TPageControl *self, bool ChkVisible) {
  TPage *pg;
	if (!self) return NULL;
	pg = (TPage*)self->next_child;
	if (!pg) return NULL;
	if (pg->wtyp != wtyp_TPage) pg = TPage_GetNext(pg, False);
	if (!pg) return NULL;

	if (pg->PageVisible || !ChkVisible) return pg;
	return TPage_GetNext(pg, ChkVisible);
}

TPage *TPageControl_GetLastPage(TPageControl *self, bool ChkVisible) {
  TPage *pg, *tpg;

	tpg = TPageControl_GetFirstPage(self, False);
	if (!tpg) return NULL;
	pg = tpg;
	while (tpg) {
	    pg = tpg;
	    tpg = TPage_GetNext(tpg, False);
	}
	if (pg->PageVisible || !ChkVisible) return pg;
	return TPage_GetPrev(pg, ChkVisible);
}

int TPageControl_GetPageCount(TPageControl *self, bool ChkVisible) {
  int result = 0;
  TPage *pg = TPageControl_GetFirstPage(self, ChkVisible);
	while (pg) {
	    result++;
	    pg = TPage_GetNext(pg, ChkVisible);
	}
	return result;
}

// ___________________________ Destroy _______________________________
static bool TPage__Destroy(TPage *self) {
  TPageControl *pgc = TPage_GetTPageControl(self);
  TPage *pg;

	if (pgc && !(pgc->RunStatus & runst_GoDestroy)) {
	    pg = NULL;
	    if (pgc->AktPage == self) {
		pg = TPage_GetPrev(self, True);
		if (!pg) pg = TPage_GetNext(self, True);
	    }
	    pgc->ResizeStartMk = False;
	    Refresh((TWins*)pgc);
	    TPage_SetPageAktiv(pg);
	}

	return TWins__Destroy((TWins*)self);
}

/*________________________ Create TPage __________________________________*/
static void TPage_CreateParams(TWins_Create_struct *wcs) {
  TPage *self = wcs->wins;

	self->destroy = (TDESTROY)TPage__Destroy;
	self->Visible = False;
	self->PageVisible = True;
	self->SetCaption = TPage_SetCaption;
	self->PageControl = (TPageControl*)wcs->Parent;
}

void *TPage_CreateParamAlg(TPageControl *Parent, int PageMemSize) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, PageMemSize, wtyp_TPage);
	wcs.Parent = (TWins*)Parent;
	wcs.CreateParams = TPage_CreateParams;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}

TPage *TPageControl_AddNewPage_Size(TPageControl *self, const char *Caption, 
				int PageMemSize) {
  TPage *pg;

	pg = TPage_CreateParamAlg(self, PageMemSize);
	pg->SetVisible = TPage_SetVisible;
	SetCaption((TWins*)pg, Caption);

	if (!self->AktPage) {
	    self->AktPage = pg;
	    pg->Visible = True;
	}

	Set_Parent((TWins*)pg, (TWins*)self);
	if (self->RunStatus & runst_Paint1) Refresh((TWins*)self);
	return pg;
}

TPage *TPageControl_AddNewUserPage(TPageControl *self, const char *Caption, 
		struct TPage* (*DoTPageCreate)(struct TPageControl *Parent)) {
  TPage *pag;

	pag = NULL;
	if (DoTPageCreate) pag = DoTPageCreate(self);
	if (!pag) return NULL;
	pag->SetVisible = TPage_SetVisible;
	SetCaption((TWins*)pag, Caption);

	if (!self->AktPage) {
	    self->AktPage = pag;
	    pag->Visible = True;
	}

	Set_Parent((TWins*)pag, (TWins*)self);
	if (self->RunStatus & runst_Paint1) Refresh((TWins*)self);
	return pag;
}


/* static TPage *TPage_Create(TPageControl *Parent) {
	return TPage_CreateParamAlg(Parent, sizeof(TPage));
} */

TPage *TPageControl_AddNewPage(TPageControl *self, const char *Caption) {
//	return TPageControl_AddNewUserPage(pagc, Caption, TPage_Create);
	return TPageControl_AddNewPage_Size(self, Caption, sizeof(TPage));
}

/*___________________ Create TPageControl ______________________________*/
static void TPageControl_CreateParams(TWins_Create_struct *wcs) {
  TPageControl *self = wcs->wins;
	self->width = 240;
	self->height = 160;
	self->TabsHeight = 18;
	self->MinTabWidth = 20;
	self->TabTxtOff = 6;

	self->DoPaint = TPageControl_DoPaint;
	self->GetClientRect = TPageControl_GetClientRect;
//	self->AdjustSize = TPageControl_AdjustSize;
	self->Resize = TPageControl_Resize;
	self->DoMouseDown = TPageControl_DoMouseDown;
	self->DoKeyDown = TPageControl_DoKeyDown;
	self->DoFocus = TPageControl_DoFocus;
//	self->KeyFu = mCanFocus|mK_Right|mK_Left;
}

TPageControl *TPageControl_Create(TWins *Parent) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TPageControl), wtyp_TPageControl);
	wcs.Parent = Parent;
	wcs.CreateParams = TPageControl_CreateParams;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}

TPageControl *TPageControl_CreateA(TWins *Parent) {
  TPageControl *self = TPageControl_Create(Parent);
	self = TPageControl_Create(Parent);
	self->Align = alClient;
	Set_Parent((TWins*)self, Parent);
	return self;
}
