/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"

static void TSplitterH_DoMouseDown(TSplitterH *self, XButtonEvent *ev) {
  TWins *wins;

	TWins_DoMouseDown((TWins*)self, ev);
	if (ev->button != 1) return;
	self->topwin = self->bottomwin = NULL;
	wins = self->prev;
	if (!wins) return;
	self->topwin = wins;
	self->md_top = self->top;
	wins = self->next;
	if (!wins) return;
	self->bottomwin = wins;
	self->bottomwin_bottom = self->bottomwin->top + self->bottomwin->height;
}

static void TSplitterH_DoMouseMove(TSplitterH *self, XMotionEvent *ev) {
  int yo, w, wk, wks, wr;
  bool vis;
	TWins_DoMouseMove((TWins*)self, ev);
	if (!self->topwin || !(ev->state & Button1MotionMask) ||
	    !TestWinIsEnabled((TWins*)self)) return;

	yo = ev->y_root - MouseDown.y_root;
	w = (self->md_top - self->topwin->top) + yo;
	wk = 3;
	if (self->topwin->min_height > wk) wk = self->topwin->min_height;
	wks = wk >> 2;
	if (wks > 20) wks = 20;
	wks = wk - wks;
	vis = w >= wks;
	if (w < wk) w = wk;
	if (self->bottomwin) {
	    wr = self->bottomwin_bottom - self->md_top - self->height - yo;
	    if (self->bottomwin->min_height < 3) wr -= 3; else 
					       wr -= self->bottomwin->min_height;
	} else {
	    wr = self->prev_child->height - self->md_top - self->height - yo;
	}
	if (wr < 0) w += wr;

	self->rmk |= 0x01;
	SetVisible(self->topwin, vis);
	SetHeight(self->topwin, w);
	self->rmk &= ~0x01;
}
/*_____________________________ Destroy _________________________________*/

/*_____________________________ Create __________________________________*/
static void TSplitterH_CreateParams(TWins_Create_struct *wcs) {
  TSplitterH *self = wcs->wins;

	self->cursor_nr = XC_sb_v_double_arrow;
	self->height = 4;
	self->Align = alTop;
	self->BorderFu = mBorder;
	self->DoMouseDown = TSplitterH_DoMouseDown;
	self->DoMouseMove = TSplitterH_DoMouseMove;
}

TSplitterH *TSplitterH_CreateP(TWins *Parent) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TSplitterH), wtyp_SplitterH);
	wcs.Parent = Parent;
	wcs.CreateParams = TSplitterH_CreateParams;
	TWins_DefaultConf(&wcs);
	Set_Parent((TWins*)wcs.wins, Parent);
	return wcs.wins;
}
