/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "dlg.h"
#include "cicons.h"


static void TMsgBoxBu_OnClick(TButton *self, long KeyState, bool DblClick) {
//	if (DblClick) return;
  TMsgBox *mb = (TMsgBox*)self->FormStruct;
	mb->RetCode = self->tag;
	mb->intern_approc_count++;
	if (mb->OnButton) mb->OnButton(mb);
	mb->intern_approc_count--;
	if (mb->intern_approc_count > 0) return;
	SetVisible((TWins*)mb, (mb->RetCode == 0));
}

static void TMsgBox_OnResize(TMsgBox *self) {
  int i, w, o;

	w = (msgbuWidth + 6) * self->buCount - 6;
	o = (self->width - w) >> 1;
	for (i=0; i<self->buCount; i++) {
		self->buN[i]->left = o;
		o = o + msgbuWidth + 6;
		self->buN[i]->top = self->height - 2 - msgbuHeight;
	}
}

static void TMsgBox_OnPaint(TMsgBox *self) {
  int x, w, h, y;

	self->vds.OnlyLg = false;
	x = 10;
	if (self->pic) {
	    y = (self->height - msgbuHeight - self->pic->height) / 2;
	    if (y < 5) y = 5;
	    DrawPic(self->WinID, self->pic, 5, y);
	    x += self->pic->width;
	}
	self->vds.BackColor = self->Color;
	DrawLineString(self->Msg, x, self->y_off, &w, &h, &self->vds);
}

static const char *msgBuTxt[16] = {
    "OK",		"OK ALL",	"ABORT",	"RETRY",
    "IGNORE",		"YES",		"YES ALL",	"NO",
    "SKIP",		"SKIP ALL",	"REPEAT",	"FORCE !",
    "12",		"13",		"14",		"CANCEL"};

static void TMsgBox_OnCanClose(TMsgBox *self, int *CanClose) {
	if (self->intern_approc_count > 0) {
	    self->RetCode = msgCANCEL;
	    self->intern_approc_count++;
	    if (self->OnButton) self->OnButton(self);
	    self->intern_approc_count--;
	    *CanClose = false;
	}
}

TMsgBox *TMsgBox_Create(TWins *parent, const char *Msg, dword fu, TMsgBoxParam *mbp) {
  TMsgBox *self;
  int i, m, k, w, h, buCount, fb;
  dword akt, foc;

	akt = 0;
	if (mbp) akt = mbp->akt;

	if (parent && (!TWins_ChkExists((TWins*)parent) ||
		      (parent->RunStatus & runst_GoDestroy))) parent = NULL;

	if (fu & msgBell) {
#ifndef _OS_WIN32
	    XBell(appXDat.dpy, 0);
#endif
	}

	self = TForm_CreatParamAlg(parent, sizeof(TMsgBox));

	self->Visible = false;
	self->FormZStyle = fszTransient | fszNoResize;
	if (akt & mbpakt_x_y) {
	    self->top = mbp->top;
	    self->left = mbp->left;
	    self->FormPosition = poDesigned;
	} else {
	    self->FormPosition = poCenter;
	}

	if (akt & mbpakt_UTF8) {
	    self->vds.UTF8on = mbp->UTF8on;
	} else {
	    m = 0;
	    if (Chk_fio_UTF8(Msg, pclen(Msg), &m)) {
		self->vds.UTF8on = (m & (fio_has_UTF8_char|fio_err_UTF8_char)) == fio_has_UTF8_char;
	    }
	}
	if (akt & mbpakt_OnButton) self->OnButton = mbp->OnButton;

	self->OnResize = TMsgBox_OnResize;
	self->OnPaint =  TMsgBox_OnPaint;
	self->OnCanClose = TMsgBox_OnCanClose;
	self->Msg = Msg;
	self->y_off = 5;
	if (akt & mbpakt_IconName) {
	    self->IconName = mbp->IconName;

	} else if ((fu & msgIcoERROR) == msgIcoERROR) {
	    self->Icon = GetDeskIcon(ddkt_Actions, 16, "error", 
					gdifu_noZoom, zico16error);
	    self->pic = GetDeskIcon(ddkt_Actions, 32, "error", 
					gdifu_noZoom, zico32error);

	} else if ((fu & msgIcoERROR) == msgIcoINFO) {
	    self->Icon = GetDeskIcon(ddkt_Actions, 16, "info", 
					gdifu_noZoom, zico16info);
	    self->pic = GetDeskIcon(ddkt_Actions, 32, "info", 
					gdifu_noZoom, zico32info);

	} else if ((fu & msgIcoERROR) == msgIcoWARNING) {
	    self->Icon = GetDeskIcon(ddkt_Actions, 16, "warning", 
					gdifu_noZoom, zico16warning);
	    self->pic = GetDeskIcon(ddkt_Actions, 32, "warning", 
					gdifu_noZoom, zico32warning);
	}

	foc = 0;
	if (akt & mbpakt_FocusButton) foc = mbp->FocusButton;
	
	buCount = 0;
	k = 1;
	fb = 0;
	if ((fu & 0xffff) == 0) fu |= msgOK;
	for (i=0; i<16;i++) {
		if (foc == k) fb = buCount;
		if (fu & k) {
			self->buN[buCount] = TButton_CreateS((TWins*)self, 0, 0, msgbuWidth, msgbuHeight, msgBuTxt[i]);
			self->buN[buCount]->OnClick = TMsgBoxBu_OnClick;
			self->buN[buCount]->tag = k;
#ifdef USE_DECOR
//			self->buN[buCount]->PDecor = TDecor_Create_dup(appXDat.DefaultButtonDecor);
#endif
			buCount++;
			if (buCount > 4) break;
		}
		k = k << 1;
	}
	self->FocusWin = (TWins*)self->buN[fb];

	if (fu & msgFontFixed) {
#ifdef _OS_WIN32
	    SetFont((TWins*)self, "-*-Courier New-medium-r-normal--13-*-*-*-*-*-*-1");
#else
	    SetFont((TWins*)self, "-adobe-courier-medium-r-normal--12-120-75-75-*-*-iso8859-1");
#endif /* ! _OS_WIN32 */
	}

	self->vds.OnlyLg = true;
	DrawLineString(Msg, 0, 0, &w, &h, &self->vds);
	w += 20;
	if (self->pic) {
	    w += self->pic->width;
	    if (h < self->pic->height) {
		self->y_off += (self->pic->height - h) / 2;
		h = self->pic->height;
	    }
	}

	self->buCount = buCount;
	m = (msgbuWidth + 6) * buCount;
	if (m > w) w = m;
	if (w < 120) w = 120;
	self->width = w;

	self->height = h + msgbuHeight + 15;
	if ((akt & mbpakt_Caption) && mbp->Caption) SetCaption((TWins*)self, mbp->Caption);

	if (akt & mbpakt_scr) {
	    self->left += (mbp->scrwidth - self->width) / 2;
	    self->top += (mbp->scrheight - self->height) / 2;
	    self->FormPosition = poDesigned;
	}

	return self;
}

int MsgBox_Show_mbp(TWins *parent, const char *Msg, dword fu, TMsgBoxParam *mbp) {
  TMsgBox *self;
  int m;

	self = TMsgBox_Create(parent, Msg, fu, mbp);
	SetVisible((TWins*)self, true);
	while (TestWinIsVisible((TWins*)self) && AppHandleMessage());
	if (mbp) {
	    mbp->top = self->top;
	    mbp->left = self->left;
	}
	m = self->RetCode;
	Tfree(self);
	return m;
}

int MsgBox_Show(TWins *parent, const char *Caption, const char *Msg, dword fu, dword foc) {
  TMsgBoxParam mbp;
	mbp.akt = mbpakt_FocusButton|mbpakt_Caption;
	mbp.Caption = Caption;
	mbp.FocusButton = foc;
	return MsgBox_Show_mbp(parent, Msg, fu, &mbp);
}
