/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "dlg.h"

static bool NewFile_Dlg_ChkStrOk(const char *s) {
	if (!pclen(s) || (*s == ' ')) return False;
	while (*s && (*s != '*')) s++;
	return (*s == 0);
}

static int NewFile_Dlg_MkNew(TNewFile_Dlg *fw, char *s) {
  int result = 0;
  TNetErrRet ErrRet;

	if (NewFile_Dlg_ChkStrOk(s)) {
	    net_err_init(&ErrRet);
	    if ((fw->fu & 0x0f) == ffnNewDirectory) {
		result = net_mkdir_gp(fw->gp, &ErrRet, 0);

	    } else if ((fw->fu & 0x0f) == ffnNewTxtFile) {
		result = net_file_new_va_gp(fw->gp, &ErrRet, "\n");
	    }
	    if (!result) {
		NetError_Dlg(fw->Parent, fw->gp, 0, "Create new", &ErrRet);
	    }
	    net_err_clr(&ErrRet);
	}
	return result;
}

static void NewFile_Dlg_buOk_Click(TButton *self, long KeyState, bool DblClick) {
  TNewFile_Dlg *fw = (TNewFile_Dlg*)self->FormStruct;

//	if (DblClick) return;
	SetVisible((TWins*)self->FormStruct, False);
	fw->RetCode = NewFile_Dlg_MkNew(fw, fw->edFile->vds.s);
}

static void NewFind_Dlg_edFile_OnKeyUp(TEdit *self, TKeyEvent *KeyEv) {
	if (KeyEv->keysym == XK_Return) NewFile_Dlg_buOk_Click((TButton*)self, 0, False);
}

static void NewFile_Dlg_edFile_OnChange(TEdit *ed) {
  TNewFile_Dlg *fw = (TNewFile_Dlg*)ed->FormStruct;
  bool mk = NewFile_Dlg_ChkStrOk(ed->vds.s);
	if (mk) {
	    ppcchange(&fw->gp->name, ed->vds.s);
	    mk = !file_chk_exist_gp(fw->gp, findALL);
	}
	SetEnabled((TWins*)fw->buOk, mk);
}

static void NewFind_Dlg_lblNewPath_OnSetCaption(TWins *self, const char *s) {
  TNewFile_Dlg *fw = (TNewFile_Dlg*)self->FormStruct;

	NewFile_Dlg_edFile_OnChange(fw->edFile);
}

static void NewFile_Dlg_buCancel_Click(TButton *self, long KeyState, bool DblClick) {
	SetVisible(self->FormStruct, False);
}

static void NewFile_Dlg_Show(TNewFile_Dlg *self) {
	TForm_Show((TForm*)self);
	self->RetCode = 0;
}

static void NewFile_Dlg_OnResize(TNewFile_Dlg *self) {
  int m;

	m = self->width >> 1;
	self->buCancel->left = m + 5;
	self->buCancel->top = self->height - 2 - self->buOk->height;
	self->buOk->left = m - 5 - self->buCancel->width;
	self->buOk->top = self->height - 2 - self->buCancel->height;
	self->lblNew->width = self->width - 4 - self->lblNew->left;
	self->lblNewPath->width = self->width - 4 - self->lblNewPath->left;
	self->edFile->width = self->width - 4 - self->edFile->left;
}

bool NewFile_Dlg_Execute(TNewFile_Dlg *fw) {
  char *s;
  bool mk;

	if ((fw->fu & 0x0f) == ffnNewDirectory) {
	    mk = net_mkdir_gp_test(fw->gp);
	} else if ((fw->fu & 0x0f) == ffnNewTxtFile) {
	    mk = net_file_new_va_gp_test(fw->gp);
	} else {
	    mk = gp_test_local_drive(fw->gp);
	}

	if (!mk) {
	    MsgBox_Error((TWins*)fw->Parent, "cant Create file or dir on\n\"%s\\\"", fw->gp->RootArt);
	    return False;
	}

	s = get_DisplayFileStr(NULL, fw->gp);
	SetCaption(fw->lblNewPath, s);
	xfree(s);

	fw->RetCode = 0;
	fw->FocusWin = (TWins*)fw->edFile;

	SetVisible((TWins*)fw, True);
	while (TestWinIsVisible((TWins*)fw) && AppHandleMessage());
	return (fw->RetCode == 1);
}

static bool NewFile_Dlg__Destroy(TNewFile_Dlg *self) {
	gpFree(self->gp);
	return TWins__Destroy((TWins*)self);
}

TNewFile_Dlg *Create_NewFile_Dlg(TWins *parent, int fu) {
  TNewFile_Dlg *self;

	self = TForm_CreatParamAlg(parent, sizeof(TNewFile_Dlg));
	self->Visible = False;
	self->FormZStyle = fszTransient;
	self->FormPosition = poCenter;
	self->Show = NewFile_Dlg_Show;
	self->OnResize = NewFile_Dlg_OnResize;
	self->destroy = (TDESTROY)NewFile_Dlg__Destroy;
	self->fu = fu;

	self->width = 260;
	self->height = 90;
	self->min_width = 240;
	self->min_height = 90;

	self->lblNew = TWins_CreateLabel((TWins*)self, 2, 4, 120, 16, NULL, 0, 0);
	self->lblNewPath = TWins_CreateLabel((TWins*)self, 2, 20, 120, 16, NULL, 0, 0);

	self->lblNewPath->OnSetCaption = NewFind_Dlg_lblNewPath_OnSetCaption;

	self->edFile = TEdit_CreateS((TWins*)self, 2, 40, 120, 20, NULL);
	self->edFile->OnKeyUp = NewFind_Dlg_edFile_OnKeyUp;
	self->edFile->OnChange = NewFile_Dlg_edFile_OnChange;

	self->buOk = TButton_CreateS((TWins*)self, 100, 2, 64, 24, "OK");
	self->buOk->OnClick = NewFile_Dlg_buOk_Click;
	self->buOk->Enabled = False;
	self->buCancel = TButton_CreateS((TWins*)self, 100, 24, 64, 24, "Cancel");
	self->buCancel->OnClick = NewFile_Dlg_buCancel_Click;

	if ((fu & 0x0f) == ffnNewDirectory) {
	    SetCaption((TWins*)self, "New Directory Dlg");
	    SetCaption(self->lblNew, "Create new Directory on:");
	} else {
	    SetCaption((TWins*)self, "New File Dlg");
	    SetCaption(self->lblNew, "Create new File on:");
	}
	return self;
}

