/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tlineedit.h"

void TLineEdit_DoChange(TLineEdit *le, int bef, TKeyS *KeyS, Tlgs *Ins_lgs) {
  int xa = le->SelAx, ya = le->SelAy;
  int xe = le->CaretCharIdx, ye = le->CaretCharIdy;
  int yya, m;
  Tlgs Tmp_lgs;
  Tlgs Del_lgs;
  TColS *cs;
  bool urnmk = ((bef & (ur_Undo | ur_Redo)) == 0);
  bool chEOL;

	if (bef & ur_InsKey) {
		Tmp_lgs.s = KeyS->c;
		Tmp_lgs.lg = pclen(KeyS->c);
	} else if (bef & ur_InsStr) {
		Tmp_lgs.s = Ins_lgs->s;		//in undo reset lgs to NULL 
		Tmp_lgs.lg = Ins_lgs->lg;
	} else {
		Tmp_lgs.s = NULL;
		Tmp_lgs.lg = 0;
	}
	if ((bef & ur_DelSel) && !TLineEdit_ChkIsSel(le)) bef &= ~ur_DelSel;

	if (bef & ur_DelSel) {
		Del_lgs.s = NULL;
		Del_lgs.lg = 0;
		if (le->OnChange && urnmk) {
		    TLines_GetSel_Tlgs(le->tls, &Del_lgs, xa, ya, xe, ye, 0);
		}
		TLines_DelSel(le->tls, &le->SelAx, &le->SelAy, le->CaretCharIdx, le->CaretCharIdy);
		le->CaretCharIdx = le->SelAx;
		le->CaretCharIdy = le->SelAy;
		if (le->OnChange) {
		    m = bef & ~(ur_InsKey | ur_InsStr);
		    le->OnChange(le, m, KeyS, &Del_lgs, xa, ya, xe, ye);
		    Tlgs_Clr(&Del_lgs);
		}
		xa = le->SelAx;
		ya = le->SelAy;
		xe = le->CaretCharIdx; 
		ye = le->CaretCharIdy;
	}
	yya = le->CaretCharIdy;
	if ((bef & (ur_InsKey | ur_InsStr)) != 0) {
	    m = bef & ~ur_DelSel;
	    chEOL = (le->CaretCharIdy >= le->tls->Count);
	    TLines_Paste_lgs(le->tls, &le->CaretCharIdy, &le->CaretCharIdx, &Tmp_lgs, 0);
	    if (le->OnChange) {
		le->OnChange(le, m, KeyS, Ins_lgs, xa, ya, le->CaretCharIdx, le->CaretCharIdy);
	    }
	    if (chEOL) {
		cs = TLines_Get_pcs(le->tls, le->tls->Count-1);
		if (cs && ((cs->fu & fio_EOL_Mask) == fio_EOLnone)) cs->fu = fio_EOLdefault;
	    }
	}

	if (bef && le->highlight && le->highlight->ChangeLine) {
	    le->highlight->ChangeLine(le->highlight, le->tls, yya, le->CaretCharIdy, clfu_use_yaye);
	}
}


void TLineEdit_Strg_X(TLineEdit *le) {
	TLineEditBase_Strg_C((TLineEditBase*)le);
	TLineEdit_DoChange(le, ur_DelSel, NULL, NULL);
}

void TLineEdit_Strg_V(TLineEdit *le) {

	if (!GetPasteStr_lgs(le->vds.UTF8on)) return;
	TLineEdit_DoChange(le, ur_DelSel | ur_InsStr, NULL, &appXDat.SelRet_lgs);
	SelRetClr();

}

static bool TLineEdit_keyplugin_edit(TLineEdit *le, TKeyEvent *KeyEv, int *res) {
  int urmk;
  TColS *cs;
  TKeyS KeyS;
  TVDrawString *vds;

	if (le->ReadOnly) return False;

	vds = &le->vds;

	if (KeyEv->state & 0x4) {
	    switch(KeyEv->keycode) {
	    case 53:		//Strg_X
		TLineEdit_Strg_X(le);
		break;

	    case 55:		//Strg_V
		TLineEdit_Strg_V(le);
		break;

	    default:
		return False;
	    }

	} else {
	    switch(KeyEv->keysym) {
	    case XK_BackSpace:
		urmk = ur_DelSel;
		if (!TLineEdit_ChkIsSel(le)) {
		    if (le->CaretCharIdx > 0) {
			GetVdsPrev_WChar(&le->CaretCharIdx, vds);
		    } else if (le->CaretCharIdy > 0) {
			le->CaretCharIdy--;
			cs = TLines_Get_pcs(le->tls, le->CaretCharIdy);
			if (cs)
			  le->CaretCharIdx = cs->lg;
			else le->CaretCharIdx = 0;
		    }
		    urmk |= ur_DelChar;
		}
		TLineEdit_DoChange(le, urmk, NULL, NULL);
		break;

	    case XK_Delete:
		urmk = ur_DelSel;
		if (!TLineEdit_ChkIsSel(le)) {
		    if (le->CaretCharIdx < le->vds.lg) {
//			le->SelAx++;
			GetVdsNext_WChar(&le->CaretCharIdx, vds);
		    } else if (le->CaretCharIdy < le->tls->Count) {
			le->SelAy++;
			le->SelAx = 0;
		    }
		    urmk |= ur_DelCharA;
		}
		TLineEdit_DoChange(le, urmk, NULL, NULL);
		break;

	    case XK_Return:
		KeyS.c[0] = '\n';
		KeyS.c[1] = 0;
		TLineEdit_DoChange(le, ur_InsKey | ur_DelSel, &KeyS, NULL);
		break;

	    case XK_Tab:
		KeyS.c[0] = '\t';
		KeyS.c[1] = 0;
		TLineEdit_DoChange(le, ur_InsKey | ur_DelSel, &KeyS, NULL);
		break;

	    default:
		if (keysymtostr(&KeyEv->keysym, KeyS.c, le->vds.UTF8on)) {
		    TLineEdit_DoChange(le, ur_InsKey | ur_DelSel, &KeyS, NULL);
		} else return False;
	    }
	}

	le->SelAx = le->CaretCharIdx;
	le->SelAy = le->CaretCharIdy;
	TLineEdit_ChkCaret(le, 0);
	TLineEdit_SetCaret(le);
	Refresh((TWins*)le);
	*res = 1;
	return True;
}

TPlugIn_KeyDown *TLineEdit_keyplugin_edit_Create() {
  TPlugIn_KeyDown *plug;
	plug = ttc_create(sizeof(TPlugIn_KeyDown), wtyp_LineEditKeyPlugin);
	plug->KeyDownPlugIn = (TKeyDownPlugIn)TLineEdit_keyplugin_edit;
	return plug;
}
