/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tlineedit.h"

static char *new_line_add(char *pca, int ct, int cp) {
  int m;
  char *pc;
  char line[256];

	m = 255;
	pc = line;
	while ((ct > 0) && (m > 0)) {
	    *pc = '\t';
	    pc++; ct--; m--;
	}
	while ((cp > 0) && (m > 0)) {
	    *pc = ' ';
	    pc++; cp--; m--;
	}
	*pc = 0;
	return pcadd(line, pca);
}

static void line_insert(TLines *tls, int idx) {
  TColS * pcs;
  int ct,cp, m;
  byte *pca;
  char *pc;

	pcs = TLines_Get_pcs(tls, idx);
	if (!pcs) return;
	if ((pcs->lg <= 0) || !pcs->s) return;

	ct = 0;
	cp = 0;
	pca = (byte*)pcs->s;
	while (*pca && (*pca <= 0x20)) {
	    if (*pca == 0x20) cp++;
	    if (*pca == '\t') {ct++; cp = 0;}
	    pca++;
	}
	if (!*pca) return;

	m = cp / 8;
	if (m > 0) {
	    ct += m;
	    cp -= (m * 8);
	}

	cp = (cp / 2) * 2;
	if (cp >= 6) {
	    ct++;
	    cp = 0;
	} else cp += 2;

	pc = new_line_add((char*)pca, ct, cp);
	xfree(pcs->s);
	pcs->s = pc;

	pcs->lg = pclen(pcs->s);
	TLines_DoLineChange(tls, idx);
}

static void line_del(TLines *tls, int idx) {
  TColS * pcs;
  int ct,cp, m;
  byte *pca;
  char *pc;

	pcs = TLines_Get_pcs(tls, idx);
	if (!pcs) return;
	if ((pcs->lg <= 0) || !pcs->s) return;

	ct = 0;
	cp = 0;
	pca = (byte*)pcs->s;
	while (*pca && (*pca <= 0x20)) {
	    if (*pca == 0x20) cp++;
	    if (*pca == '\t') {ct++; cp = 0;}
	    pca++;
	}
	if (!*pca) return;

	m = cp / 8;
	if (m > 0) {
	    ct += m;
	    cp -= (m * 8);
	}

	cp = (cp / 2) * 2;
	if (cp < 2) {
	    cp = 6;
	    if (ct > 0) ct--; else cp = 0;
	} else cp -= 2;

	pc = new_line_add((char*)pca, ct, cp);
	xfree(pcs->s);
	pcs->s = pc;

	pcs->lg = pclen(pcs->s);
	TLines_DoLineChange(tls, idx);
}


static bool TLineEdit_keyplugin_strg_u_i(TLineEdit *le, TKeyEvent *KeyEv, int *res) {
  int ya, ye, m;
  TKeyS KeyS;
  Tlgs lgs;

	if (le->ReadOnly ||
	    le->SelAx || le->CaretCharIdx) return False;

	lgs.s = NULL;
	lgs.lg = 0;
	ya = le->SelAy;
	ye = le->CaretCharIdy;
	if (ya == ye) return False;
	if (ya > ye) {m=ya; ya=ye; ye=m;}
	m = ya;

	if (KeyEv->state & 0x4) {
	    if (KeyEv->keycode == 31) goto ctl_I;
	    if (KeyEv->keycode == 30) goto ctl_U;
	} else  {
	    if (KeyEv->keysym == XK_Tab) goto ctl_I;
	    if (KeyEv->keysym == XK_ISO_Left_Tab) goto ctl_U;
	}
	return False;

ctl_I:
	while (m < ye) {
	    line_insert(le->tls, m);
	    m++;
	}
	goto ctl_End;
ctl_U:
	TLines_GetSel_Tlgs(le->tls, &lgs, 0, ya, 0, ye, 0);
	while (m < ye) {
	    line_del(le->tls, m);
	    m++;
	}

ctl_End:
	KeyS.d[0] = KeyEv->keysym;
	KeyS.w[2] = KeyEv->keycode;
	KeyS.w[3] = KeyEv->state;
	if (le->OnChange) le->OnChange(le, ur_plugin | ur_keycode, &KeyS, &lgs, 0, ya, 0, ye);
	TLineEdit_AllAtrNew(le);
	*res = 1;
	return True;
}

static bool TLineEdit_keyplugin_strg_u_i_ReUndo(TLineEdit *le, TUnRedoLine *url) {
  int m;

	if (!(url->fu & ur_keycode)) return False;

	m = url->ya;
	if (url->KeyS.w[3] & 0x4) {
	    if (url->KeyS.w[2] == 31) goto ctl_I;
	    if (url->KeyS.w[2] == 30) goto ctl_U;
	} else  {
	    if (url->KeyS.d[0] == XK_Tab) goto ctl_I;
	    if (url->KeyS.d[0] == XK_ISO_Left_Tab) goto ctl_U;
	}
	return False;

ctl_I:
	while (m < url->ye) {
	    if (url->fu & ur_Redo) {
		line_insert(le->tls, m);
	    } else {
		line_del(le->tls, m);
	    }
	    m++;
	}
	goto ctl_End;
ctl_U:
	if (url->fu & ur_Redo) {
	    while (m < url->ye) {
		line_del(le->tls, m);
		m++;
	    }
	} else {
	    le->SelAx = le->CaretCharIdx = 0;
	    le->SelAy = url->ya;
	    le->CaretCharIdy = url->ye;
	    TLineEdit_DoChange(le, ur_DelSel | ur_InsStr | ur_Undo, 
			      NULL, &url->lgs);
	}

ctl_End:
	le->SelAx = le->CaretCharIdx = 0;
	le->SelAy = url->ya;
	le->CaretCharIdy = url->ye;
	TLineEdit_AllAtrNew(le);
	return True;
}

TPlugIn_KeyDown *TLineEdit_keyplugin_strg_u_i_Create() {
  TPlugIn_KeyDown *plug;
	plug = ttc_create(sizeof(TPlugIn_KeyDown), wtyp_LineEditKeyPlugin);
	plug->KeyDownPlugIn = (TKeyDownPlugIn)TLineEdit_keyplugin_strg_u_i;
	plug->ReUndo = (TKeyUnRedoPlugIn)TLineEdit_keyplugin_strg_u_i_ReUndo;
	return plug;
}
