/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tlineedit.h"

static void TLabelMultiLine_SetupLines(TLabelMultiLine *self) {
  int w;
  char *pc, *pcn;
  bool mk;

	if (!self->tls) return;
	mk = self->tls->Count > 0;
	if (mk) TLines_Clear(self->tls);

	if (!Get_IsParent((TWins*)self) || !TestWinIsVisible((TWins*)self)) return;
	if (mk) Refresh((TWins*)self);
	if (self->lgs.lg <= 0) return;

	self->vds.s = self->lgs.s;
	pc = self->lgs.s;
	pcn = NULL;
loop_line:
	while (*pc && (*pc != ' ') && (*pc != 0x0a) && (*pc != 0x0d)) pc++;

	if (!*pc || (*pc == 0x0a) || (*pc == 0x0d)) {
	    if (pcn) {
		self->vds.lg = pc - self->vds.s;
		w = TextWidthColorString(&self->vds);
		if (w >= self->width) {
		    TLines_nAdd(self->tls, self->vds.s, pcn - self->vds.s);
		    if (*pcn == ' ') pcn++;
		    if (*pcn == 0x0d) pcn++;
		    if (*pcn == 0x0a) pcn++;
		    self->vds.s = pcn;
		    pcn = NULL;
		}
	    }

	    TLines_nAdd(self->tls, self->vds.s, pc - self->vds.s);
	    if (*pc) {
		if (*pc == 0x0d) pc++;
		if (*pc == 0x0a) pc++;
	    }
	    self->vds.s = pc;
	    pcn = NULL;
	} else if (pcn) {
	    self->vds.lg = pc - self->vds.s;
	    w = TextWidthColorString(&self->vds);
	    if (w < self->width) {
		pcn = pc;
		if (*pc == ' ') pc++;
	    } else {
		TLines_nAdd(self->tls, self->vds.s, pcn - self->vds.s);
		if (*pcn == ' ') pcn++;
		self->vds.s = pcn;
		pcn = NULL;
	    }
	} else {
	    self->vds.lg = pc - self->vds.s;
	    w = TextWidthColorString(&self->vds);
	    if (w >= self->width) {
		TLines_nAdd(self->tls, self->vds.s, pc - self->vds.s);
		if (*pc == ' ') pc++;
		self->vds.s = pc;
	    } else {
		pcn = pc;
		if (*pc == ' ') pc++;
	    }
	}
	if (*pc) goto loop_line;
	self->vds.s = NULL;
	self->vds.lg = 0;
}

static const char *TLabelMultiLine_GetCaption(TLabelMultiLine *self) {
	return self->lgs.s;
}

static void TLabelMultiLine_SetCaption(TLabelMultiLine *self, const char *s) {
	if (!pccmp(self->lgs.s, s)) {
	    self->lgs.lg = pclen(s);
	    ppcnchange(&self->lgs.s, s, self->lgs.lg);
	    TLabelMultiLine_SetupLines(self);
	}
}

static void TLabelMultiLine_SetBorder(TLabelMultiLine *self, int BorderFu) {
	if (self && self->pw && self->pw->SetBorder) 
		self->pw->SetBorder(self->pw, BorderFu);
}

/*____________________________ ev _______________________________________*/
static void TLabelMultiLine_SetVisible(TLabelMultiLine *self, bool Value) {
	if (self->Visible != Value) {
	    TWins_SetVisible((TWins*)self, Value);
	    if (Value) TLabelMultiLine_SetupLines(self);
	}
}

static bool TLabelMultiLine_AdjustSize(TLabelMultiLine *self) {
  bool result;
	result = TWins_AdjustSize((TWins*)self);
	if (result) TLabelMultiLine_SetupLines(self);
	return result;
}

static void TLabelMultiLine_DoPaint(TLabelMultiLine *self) {
  int i, y, m;
	m = TLines_GetLineCount(self->tls);
	y = 0;
	for (i=0; i < m; i++) {
	    y = TLineEditBase_PaintLine((TLineEditBase*)self, i, y);
	    if (y > self->height) break;
	}
}

/*____________________________ Destroy __________________________________*/
static bool TLabelMultiLine__Destroy(TLabelMultiLine *self) {
	Tfree(self->tls);
	Tlgs_Clr(&self->lgs);
	return TWins__Destroy((TWins*)self);
}

/*_____________________________ Create __________________________________*/
static void TLabelMultiLine_CreateParams(TWins_Create_struct *wcs) {
  TLabelMultiLine *self = wcs->wins;

	TScrollWin_CreateParams(wcs);
	self->destroy = (TDESTROY)TLabelMultiLine__Destroy;

	self->Align = alClient;
	self->pw->BorderFu = 0;

	self->tls = TLines_Create();
	self->tls->Owner = self;

	self->SetCaption = TLabelMultiLine_SetCaption;
	self->GetCaption = TLabelMultiLine_GetCaption;
	self->SetVisible = TLabelMultiLine_SetVisible;
	self->AdjustSize = TLabelMultiLine_AdjustSize;
	self->DoPaint = TLabelMultiLine_DoPaint;
	self->SetBorder = TLabelMultiLine_SetBorder;

	SetVisible((TWins*)self->VSB, false);
	SetVisible((TWins*)self->HSB, false);
	self->Color = wcs->Parent->Color;
}

TLabelMultiLine *TLabelMultiLine_Create(TWins *Parent) {
  TWins_Create_struct wcs;
	TWins_Create_struct_init(&wcs, sizeof(TLabelMultiLine), wtyp_LabelMultiLine);
	wcs.Parent = Parent;
	wcs.CreateParams = TLabelMultiLine_CreateParams;
	TScrollWin_DefaultConf(&wcs);
	return wcs.wins;
}
