/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tlineedit.h"

//*********************** TLineEdit ************************
TPlugIn_KeyDown *TLineEdit_keyplugin_Global = NULL;

static void set_SB(TScrollBar *SB, int max, int lew) {
	if (SB->Visible && ((SB->MaxCount != max) || 
			    (SB->ButtonCountSize != lew))) {
	    SB->MaxCount = max;
	    SB->ButtonCountSize = lew;
	    SB->LargeChange = lew - 1;
	    Refresh((TWins*)SB);
	}
}

#define GetLineFontHeight(a) TLineEditBase_GetLineFontHeight((TLineEditBase*)a)

static void TLineEdit_Adjust_VHSB(TLineEdit *self) {
  int m, wu, lew;
  TVDrawString *vds = &self->vds;

	vds->s_const = "        ";
	vds->lg = 8;
	vds->scda = NULL;
	vds->TabSpaceWidth = TextWidthColorString(vds);
	vds->s_const = "u";
	vds->lg = 1;
	wu = TextWidthColorString(vds);
	vds->s = NULL;
	vds->lg = 0;

	if (wu < 1) wu = 1;
	self->HSB->SmallChange = wu;
	self->VisibleLines = self->height / GetLineFontHeight(self);
	m = self->tls->Count;
	if (!(self->LEfu & LEfu_VSBnoVisibleLines)) m += self->VisibleLines - 1;
	set_SB(self->VSB, m, self->VisibleLines);

	lew = self->width + 4;
	if (lew < 2) lew = 2;
	m = self->width;
	if (self->MaxStrX > lew) m = self->MaxStrX + wu;

	set_SB(self->HSB, m, lew);
}

static bool TLineEdit_AdjustSize(TLineEdit *self) {
  bool result;
	result = TWins_AdjustSize((TWins*)self);
	TLineEdit_Adjust_VHSB(self);
	return result;
}

#define TLineEdit_Set_vds(a, b) TLineEditBase_Set_vds((TLineEditBase*)a, b)

static void TLineEdit_Caret_Draw(TLineEdit *le, int fu) {
	Caret_Draw(fu);
	if (le->highlight && le->highlight->SetCaret) {
	    le->highlight->SetCaret(le->highlight, le->tls, le->CaretCharIdx, le->CaretCharIdy, fu);
	}
}

static void TLineEdit_DoPaint(TLineEdit *self) {
  int i, y, m;
	if (self->KeyFu & mHasFocus) TLineEdit_Caret_Draw(self, 0);
	self->vds.ClsToEnd = True;

	self->vds.Image = True;
	self->vds.BackColor = self->Color;
	m = TLineEdit_GetLineCount(self);
	y = 0;
	for (i=self->VSB->Count; i < m; i++) {
	    y = TLineEditBase_PaintLine((TLineEditBase*)self, i, y);
	    if (y > self->height) break;
	}
	if (y < self->height)
	    FillRectRGB(self->WinID, self->Color, 0, y, self->width, self->height - y);

	if (self->KeyFu & mHasFocus) TLineEdit_Caret_Draw(self, 1);
}

void TLineEdit_GetChar_xy(TLineEdit *le, int idx, int idy, int *ret_x, int *ret_y, int *ret_h) {
  int i, y, x;
  TColS *cs;

	*ret_x = -1; //*ret_y = -1;
	if (idy < le->VSB->Count) return;
	y = 0;
	for (i=le->VSB->Count; i<idy; i++) {
	    if (!TLineEdit_Set_vds(le, i)) return;
	    y += GetLineFontHeight(le);
	    if (y > le->height) return;
	}
	cs = TLineEdit_Set_vds(le, idy);
	if (!cs) return;
	le->vds.Xoff = -le->HSB->Count;
	x = GetChar_XPos(idx, &le->vds);
	if (x < le->width) {
	    *ret_x = x;
	    *ret_y = y;
	    *ret_h = GetLineFontHeight(le);
	}
}

void TLineEdit_GetCaretIdxy(TLineEdit *le, int x, int y, int *idx, int *idy) {
  TColS *cs;
	*idx = 0; *idy = 0;
	if (le->tls->Count <= 0) return;
  int iy = (y / GetLineFontHeight(le)) + le->VSB->Count;
	if (iy >= le->tls->Count) iy = le->tls->Count;
	le->vds.Xoff = -le->HSB->Count;
	cs = TLineEdit_Set_vds(le, iy);
	if (cs) *idx = GetMouseX_CharPos(x, &le->vds);
	*idy = iy;
}

static void ed_Caret_SetNewPos(TLineEdit *le, int x) {
  int y;
	y = (le->CaretCharIdy - le->VSB->Count) * GetLineFontHeight(le);
	if ((x > le->width) || (y < 0) || (y > le->height)) x = -1;
	TWins_Caret_SetNewPos((TWins*)le, x, y);
	if (x >= 0) Caret_Draw(1);
}

void TLineEdit_SetCaret(TLineEdit *le) {
  TColS *cs;
  int x;

	x = 0;
	Caret_Draw(0);
	cs = TLineEdit_Set_vds(le, le->CaretCharIdy);
	if (cs) {
	    le->vds.Xoff = -le->HSB->Count;
	    x = GetChar_XPos(le->CaretCharIdx, &le->vds); // - le->HSB->Count;
	    ed_Caret_SetNewPos(le, x);
	} else if (le->CaretCharIdy >= le->tls->Count) {
	    le->CaretCharIdy = le->tls->Count;
	    x = -le->HSB->Count;
	    ed_Caret_SetNewPos(le, x);
	}
	if (!le->OldX) le->OldCaretX = x;
	if (le->OnSetCaret) le->OnSetCaret(le, cs);
}

static void TLineEdit_SetCaret_if_Akt(TLineEdit *le) {
	if (TWins_Caret_Chk((TWins*)le)) TLineEdit_SetCaret(le);
}

void TLineEdit_ChkCaret(TLineEdit *le, int fu) {
  TColS *cs;
  int x, xa, m, ma, lh, ym, lx, la;

	lh = le->height / GetLineFontHeight(le);
	ym = 0;
	if (fu & LE_ChkCaret_Ymiddle) ym = lh >> 1;
	if (le->CaretCharIdy < le->VSB->Count) {
	    TScrollBar_SetCount_ev(le->VSB, le->CaretCharIdy - ym);
	} else if (le->CaretCharIdy >= (le->VSB->Count + lh)) {
	    lh -= ym;
	    TScrollBar_SetCount_ev(le->VSB, le->CaretCharIdy - lh + 1);
	}

	cs = TLineEdit_Set_vds(le, le->CaretCharIdy);
	if (cs) {
	    if (le->CaretCharIdx > cs->lg) le->CaretCharIdx = cs->lg;
	    le->vds.Xoff = 0;
	    m = GetChar_XPos(le->CaretCharIdx, &le->vds);
	    x = m - le->HSB->Count;

	    if (TLineEdit_ChkIsSel(le)) {
		ma = GetChar_XPos(le->SelAx, &le->vds);
		xa = ma - le->HSB->Count;
		lx = x - xa;
		la = lx;
		if (la < 0) la = -la;

		if ((appXDat.last_KeyState & Button1Mask) &&
		    (MouseDown.apph == (TWins*)le)) {
  // MouseMove and MouseDown
		} else {

		  if (la < le->width) {
		    if (lx > 0) {
		      if (xa < 0) {x = xa; m = ma;}
		    } else {
		      if (xa > le->width) {x = xa; m = ma;}
		    }
		  } else {
		    if (lx > 0) {
		      if (x <= le->width) {x -= le->width; m -= le->width;}
		    }
		  }
		}
	    }

	    if (x < 0) {
		TScrollBar_SetCount_ev(le->HSB, m);
	    } else if (x > (le->width - 4)) {
		TScrollBar_SetCount_ev(le->HSB, m - le->width + 4);
	    }
	}
}

static void TLineEdit_DoMouseDown(TLineEdit *self, XButtonEvent *ev) {
	TWins_DoMouseDown((TWins*)self, ev);
	if ((ev->button == Button1) && TestWinIsEnabled((TWins*)self)) {
	    self->OldX = False;
	    TLineEdit_GetCaretIdxy(self, ev->x, ev->y, &self->CaretCharIdx, &self->CaretCharIdy);
	    if (TLineEdit_ChkIsSel(self)) Refresh((TWins*)self);
	    self->SelAx = self->CaretCharIdx;
	    self->SelAy = self->CaretCharIdy;
	    TLineEdit_SetCaret(self);
	}
}


static void TLineEdit_DoMouseUp(TLineEdit *self, XButtonEvent *ev) {
	TWins_DoMouseUp((TWins*)self, ev);
	TScrollBar_MouseZ_Wheel(self->VSB, ev);
}


static void TLineEdit_DoMouseMove(TLineEdit *self, XMotionEvent *ev) {
  int x, y, yo;

	TWins_DoMouseMove((TWins*)self, ev);
	if ((ev->state & Button1Mask) && (MouseDown.apph == (TWins*)self)) {
	    yo = MouseDown.y - ev->y;
	    if (yo > 0) {
		if (yo > 4) yo = 4;
	    } else {
		if (yo < -4) yo = -4;
	    }
	    x = self->CaretCharIdx;
	    y = self->CaretCharIdy;
	    TLineEdit_GetCaretIdxy(self, ev->x, ev->y+yo, &self->CaretCharIdx, &self->CaretCharIdy);
	    if ((x != self->CaretCharIdx) || (y != self->CaretCharIdy)) {
		TLineEdit_ChkCaret(self, 0);
		TLineEdit_SetCaret(self);
		Refresh((TWins*)self);
	    }
	}
}

static void TLineEdit_DoClick(TLineEdit *self, long KeyState, bool DblClick) {
  TColS *cs;
  int rx, ry, xa, xe;

	TWins_DoClick((TWins*)self, KeyState, DblClick);
	if (!DblClick) return;

	TLineEdit_GetCaretIdxy(self, MouseDown.x, MouseDown.y, &rx, &ry);
	cs = TLineEdit_Set_vds(self, ry);
	if (!cs || (self->vds.lg == 0)) return;
	xa = xe = rx;
	while ((xa > 0) && (GetTzC(self->TzC, GetVdsPrev_WChar(&rx, &self->vds)) == 1)) {
	    xa = rx;
	}
	if (xa < xe) xe--;
	while (GetTzC(self->TzC, GetVdsNext_WChar(&xe, &self->vds)) == 1);
	if (xa != xe) TLineEdit_SetSel(self, xa, ry, xe, ry);
}

void TLineEdit_SetSel(TLineEdit *le, int xa, int ya, int xe, int ye) {
	le->SelAx = xa;
	le->SelAy = ya;
	le->CaretCharIdx = xe;
	le->CaretCharIdy = ye;
	TLineEdit_SetCaret(le);
	Refresh((TWins*)le);
}

TPlugIn_KeyDown *TLineEdit_GetKeyDownPlugIn(TLineEdit *le) {
  TPlugIn_KeyDown *plug = NULL;
	if (le) plug = le->PlugIn_KeyDown;
	if (!plug && !le->Disable_keyplugin_Global) plug = TLineEdit_keyplugin_Global;
	return plug;
}

static int TLineEdit_DoKeyDown(TLineEdit *self, TKeyEvent *KeyEv) {
  int m;

	TWins_DoKeyDown((TWins*)self, KeyEv);

	if (appXDat.SelWait) return 0;
	TLineEdit_Set_vds(self, self->CaretCharIdy);

	if ((KeyEv->keycode == 54) && (KeyEv->state & 0x4)) { //Strg_C
	    TLineEditBase_Strg_C((TLineEditBase*)self);
	    return 1;
	}

  TPlugIn_KeyDown *plug = TLineEdit_GetKeyDownPlugIn(self);
	m = 0;
	while (plug && !plug->KeyDownPlugIn((TWins*)self, KeyEv, &m)) {    
	    plug = plug->next;
	}
	return m;
}

static void TLineEdit_DoFocus(TLineEdit *self, int Change) {
	TWins_DoFocus((TWins*)self, Change);
	if (Change & WinsFocusIN) {
	    TLineEdit_SetCaret(self);
	} else {
	    Caret_Draw(0);
	}
}

static void TLineEdit_VSB_OnCount(TScrollBar* sb, int Count) {
  TLineEdit *le = (TLineEdit*)sb->owner;

	TLineEdit_SetCaret_if_Akt(le);
	if (le->VSB_DoCount) le->VSB_DoCount((TWins*)sb, Count);
	Refresh((TWins*)le);
}

static void TLineEdit_HSB_OnCount(TScrollBar* sb, int Count) {
  TLineEdit *le = (TLineEdit*)sb->owner;

	TLineEdit_SetCaret_if_Akt(le);
	if (le->HSB_DoCount) le->HSB_DoCount((TWins*)sb, Count);
	Refresh((TWins*)le);
}

static void TLineEdit_MaxStrLen(TLineEdit *le, int ya, int ye) {
  int i, x, mx;

	mx = 0;
	if (ye > le->tls->Count) ye = le->tls->Count;
	for (i = ya; i < ye; i++) {
	    TLineEdit_Set_vds(le, i);
	    x = TextWidthColorString(&le->vds);
	    if (x > mx) mx = x;
	}
	mx += 10;
	if (mx > le->MaxStrX) le->MaxStrX = mx;
}

static void TLineEdit_tls_OnLineChange(TLines *tls, int idx) {
  TLineEdit *le = (TLineEdit*)tls->Owner;
	TLineEdit_MaxStrLen(le, idx, idx+1);
	TLineEdit_Adjust_VHSB(le);
}

static void TLineEdit_tls_OnCount(TLines *tls) {
  TLineEdit *le = (TLineEdit*)tls->Owner;

	TLineEdit_Adjust_VHSB(le);
}

void TLineEdit_Clr(TLineEdit *self) {
	if (!self) return;
	self->CaretCharIdx = 0;
	self->CaretCharIdy = 0;
	self->SelAx = 0;
	self->SelAy = 0;
	self->OldCaretX = 0;
	self->MaxStrX = 0;
	self->vds.scda = NULL;
	self->vds.s = NULL;
	TLines_Clear(self->tls);
	TLineEdit_SetCaret_if_Akt(self);
	Refresh((TWins*)self);
}

void TLineEdit_Clr_c(TLineEdit *self) {
	if (!self) return;
	TLines_Clear_c(self->tls);
}

void TLineEdit_AllAtrNew(TLineEdit *self) {
	self->MaxStrX = 0;
	self->tls->EvStop = False;
	TLineEdit_MaxStrLen(self, 0, self->tls->Count);
	TLineEdit_Adjust_VHSB(self);

	Refresh((TWins*)self);

	if (self->highlight && self->highlight->ChangeLine) {
	    AppHandleMessage();
	    self->highlight->ChangeLine(self->highlight, self->tls, 0, 0, clfu_all);
	    Refresh((TWins*)self);
	}
}

void TLineEdit_SetCaretXY(TLineEdit *self, int x, int y) {
	self->CaretCharIdx = x;
	self->SelAx = x;
	self->CaretCharIdy = y;
	self->SelAy = y;
	TLineEdit_ChkCaret(self, 0);
	TLineEdit_SetCaret(self);
}

void TLineEdit_AddLine_lgs(TLineEdit *self, Tlgs *lgs) {
	if (self->wtyp != wtyp_LineEdit) return;
	TLines_Add_lgs(self->tls, lgs, 0);
	Refresh((TWins*)self);
}

void TLineEdit_AddLine(TLineEdit *self, const char *s) {
  Tlgs lgs;
	lgs.s_const = s;
	lgs.lg = pclen(s);
	TLineEdit_AddLine_lgs(self, &lgs);
}


static bool TLineEdit_Ctl(TLineEdit *self, TWinsCtl *wctl) {
	if (wctl->bef == wctlb_SetFont) TLineEdit_Clr_c(self);

	TWins_Ctl((TWins*)self, wctl);
	if (self->highlight && self->highlight->hlCtl) {
	    self->highlight->Font = self->vds.Font;
	    self->highlight->hlCtl(self->highlight, wctl);
	}

	if (wctl->bef == wctlb_SetFont) {
	    TLineEdit_AllAtrNew(self);
	    return True;
	}
	return False;
}

int TLineEdit_GetLineCount(TLineEdit *self) {
	if (!self || !self->tls) return 0;
	return self->tls->Count;
}

void TLineEdit_SetMouseZ_Wheel(TLineEdit *self, int Value) {
	TScrollWin_SetMouseZ_Wheel((TScrollWin*)self, Value);
}

/*____________________________ Destroy __________________________________*/
bool TLineEdit__Destroy(TLineEdit *self) {

	TWins_Caret_Clr((TWins*)self);
	Tfree(self->highlight);
	self->highlight = NULL;

	Tfree(self->UnRedo);
	Tfree(self->tls);

	while (self->PlugIn_KeyDown) {
	    TPlugIn_KeyDown *plug = self->PlugIn_KeyDown;
	    self->PlugIn_KeyDown = plug->next;
	    Tfree(plug);
	}

	self->tls = NULL;
	self->vds.s = NULL;
	self->vds.lg = 0;
	self->vds.scda = NULL;
	return TWins__Destroy((TWins*)self);
}

/*_____________________________ Create __________________________________*/
static void TLineEdit_CreateParams(TWins_Create_struct *wcs) {
  TLineEdit *self = wcs->wins;

	Caret_Init();

	TScrollWin_CreateParams(wcs);

	self->TzC = (const Tss*)&Default_TzC;

	self->tls = TLines_Create();
	self->tls->Owner = (int*) self;
	self->tls->OnLineChange = TLineEdit_tls_OnLineChange;
	self->tls->OnCount = TLineEdit_tls_OnCount;
	self->vds.TabSpaceOn = True;
	self->vds.TabSpaceDot = True;

	self->destroy = (TDESTROY)TLineEdit__Destroy;
	self->AdjustSize = TLineEdit_AdjustSize;
	self->DoPaint = TLineEdit_DoPaint;
	self->WinsCtl = TLineEdit_Ctl;

	self->DoMouseDown = TLineEdit_DoMouseDown;
	self->DoMouseUp = TLineEdit_DoMouseUp;
	self->DoMouseMove = TLineEdit_DoMouseMove;
	self->DoClick = TLineEdit_DoClick;

	self->DoKeyDown = TLineEdit_DoKeyDown;

	self->DoFocus = TLineEdit_DoFocus;

	self->VSB->OnCount = TLineEdit_VSB_OnCount;
	self->HSB->OnCount = TLineEdit_HSB_OnCount;

	self->KeyFu = mCanFocus;
	self->fio_Strg_C_on = True;

	self->cursor_nr = XC_xterm;
}

TLineEdit *TLineEdit_Create(TWins *Parent) {
  TWins_Create_struct wcs;
	TWins_Create_struct_init(&wcs, sizeof(TLineEdit), wtyp_LineEdit);
	wcs.Parent = Parent;
	wcs.CreateParams = TLineEdit_CreateParams;
	TScrollWin_DefaultConf(&wcs);
	return wcs.wins;
}

TLineEdit *TLineEdit_CreateA(TWins *Parent, int Left, int Top, int Width, int Height, int Align) {
  TLineEdit *self;
	self = TLineEdit_Create(Parent);
	self->pw->top = Top;
	self->pw->left = Left;
	self->pw->width = Width;
	self->pw->height = Height;
	self->Align = alClient;
	self->pw->Align = Align;
	Set_Parent((TWins*)self, Parent);
	return self;
}
