/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tlineedit.h"

int REGP1 TLineEditBase_GetLineFontHeight(TLineEditBase *self) {
  int fonth;
	Font_GetMetrics((TFont*)self->vds.Font, &fonth, NULL);
	return fonth + self->LineOffset;
}

void REGP1 TLineEditBase_Res_vds(TLineEditBase *self) {
	self->vds.s = NULL;
	self->vds.lg = 0;
	self->vds.scda = NULL;
}

TColS REGP2 *TLineEditBase_Set_vds(TLineEditBase *self, int idy) {
  TColS *cs = TLines_Get_pcs(self->tls, idy);
	if (cs) {
	    self->vds.s = cs->s;
	    self->vds.lg = cs->lg;
	    self->vds.scda = (TStrColorDatArr*)cs->c;
	    return cs;
	}
	TLineEditBase_Res_vds(self);
	return NULL;
}


bool REGP1 TLineEditBase_ChkIsSel(TLineEditBase *self) {
	return ((self->SelAx != self->CaretCharIdx) || 
		(self->SelAy != self->CaretCharIdy));
}

int TLineEditBase_PaintLine(TLineEditBase *self, int idy, int y) {
  int xa, xe, ya, ye;
  TVDrawString *vds;
  TColS *cs = TLineEditBase_Set_vds(self, idy);

	if (!cs) return y;
	vds = &self->vds;
	vds->SelE = vds->SelA;
	vds->SelToEnd = False;

	xa = self->SelAx;
	xe = self->CaretCharIdx;
	ya = self->SelAy;
	ye = self->CaretCharIdy;

	if ((xa != xe) || (ya != ye)) {
	    if (ya == ye) {
		if (idy == ya) {
		    vds->SelA = xa;
		    vds->SelE = xe;
		}
	    } else {
		if (ya > ye) {
		    SWAP_INT(xa, xe);
		    SWAP_INT(ya, ye);
		}
		if ((idy >= ya) && (idy <= ye)) {
		    if (self->LEBfu & LEBfu_SelRect) {
			vds->SelA = xa;
			vds->SelE = xe;

		    } else if (self->LEBfu & LEBfu_SelRectXoffAkt) {
			if (idy == ya) {
			    vds->SelA = xa;
			    vds->SelE = self->SelXoffE;
			} else if (idy == ye) {
			    vds->SelA = self->SelXoffA;
			    vds->SelE = xe;
			} else {
			    vds->SelA = self->SelXoffA;
			    vds->SelE = self->SelXoffE;
			}

		    } else {
			if (idy == ya) {
			    vds->SelA = xa;
			    vds->SelE = cs->lg;
			    vds->SelToEnd = True;
			} else if (idy == ye) {
			    vds->SelA = 0;
			    vds->SelE = xe;
			} else {
			    vds->SelA = 0;
			    vds->SelE = cs->lg;
			    vds->SelToEnd = 2;
			}
		    }
		}
	    }
	}

	vds->Xoff = -self->HSB->Count;
	vds->BackColor = self->Color;
	if ((cs->fu & fio_EOL_Mask) == fio_EOLnone) {
	    vds->BackColor = 0xffe0e0;
	} else {
	    TColS *csn = TLines_Get_pcs(self->tls, idy-1);
	    if (csn && ((csn->fu & fio_EOL_Mask) == fio_EOLnone)) {
		vds->BackColor = 0xffe8f0;
	    }
	}
	if (self->LineOffset > 0) {
	    int fonth;
	    Font_GetMetrics((TFont*)self->vds.Font, &fonth, NULL);
	    int yo = y + fonth;
	    if (self->LineOffset == 1) {
		DrawLineRGB(self->WinID, self->Color, 0, yo, self->width, yo);
	    } else {
		FillRectRGB(self->WinID, self->Color, 0, yo, self->width, yo + self->LineOffset);
	    }
	}
	DrawColorString(0, y, vds);

	return y + TLineEditBase_GetLineFontHeight(self);
}

bool TLineEditBase_GetSel_lgs(TLineEditBase *self, Tlgs *lgs, int fu) {
	lgs->s = NULL;
	lgs->lg = 0;
	if (!TLineEditBase_ChkIsSel(self)) return False;
	TLines_GetSel_Tlgs(self->tls, lgs, self->SelAx, self->SelAy, 
			  self->CaretCharIdx, self->CaretCharIdy, fu);
	return True;
}


void TLineEditBase_Strg_C(TLineEditBase *self) {
  Tlgs lgs;
  int fio_fu = 0;
  bool utf8;
	if (self->fio_Strg_C_on) fio_fu = fio_Strg_C;
	if (TLineEditBase_GetSel_lgs(self, &lgs, fio_fu)) {
	    SelClr();

	    appXDat.SelTxt.s = lgs.s;
	    appXDat.SelTxt.lg = lgs.lg;

	    utf8 = self->vds.UTF8on;
	    if (utf8) {
		int fuEOL = 0;
		Chk_fio_UTF8_lgs(&lgs, &fuEOL);
	        if ((fuEOL & fio_has_0x80_char) && 
		    !(fuEOL & fio_has_UTF8_char)) utf8 = False;
	    }

	    if (utf8) {
#ifdef _OS_WIN32
		SetSelection(2, CF_UNICODETEXT, XA_STRING);
#else
		SetSelection(2, appXDat.XA_UTF8_STRING, XA_STRING);
#endif
	    } else {
		SetSelection(1, XA_STRING);
	    }
	}
}
