/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tlineedit.h"

static bool TLineEdit_Undo_ChkCaret(TLineEdit *le, TUnRedoLine *url) {
  int lh, fonth;
  bool res;

	Font_GetMetrics((TFont*)le->vds.Font, &fonth, NULL);
	lh = le->height / fonth;
	res = ((url->ya < le->VSB->Count) || (url->ya >= (le->VSB->Count + lh)));
	if (res) {
	    le->SelAx = url->xa;
	    le->SelAy = url->ya;
	    le->CaretCharIdx = url->xa;
	    le->CaretCharIdy = url->ya;
	    TLineEdit_ChkCaret(le, LE_ChkCaret_Ymiddle);
	    TLineEdit_SetCaret(le);
	    Refresh((TWins*)le);
	}
	return res;
}

bool TLineEdit_Undo(TLineEdit *le, TUnRedoLine *url) {
  int xa, ya;
  TPlugIn_KeyDown *plug;

	xa = url->xa;
	ya = url->ya;

	if (TLineEdit_Undo_ChkCaret(le, url)) return False;

	if (url->fu & ur_plugin) {
	    plug = TLineEdit_GetKeyDownPlugIn(le);
	    while (plug) {
		if (plug->ReUndo && plug->ReUndo((TWins*)le, url)) return True; 
		plug = plug->next;
	    }
	}

	if (url->fu & (ur_InsKey | ur_InsStr)) {
	    le->SelAx = url->xa;
	    le->SelAy = url->ya;
	    le->CaretCharIdx = url->xe;
	    le->CaretCharIdy = url->ye;
	    TLineEdit_DoChange(le, ur_DelSel | ur_Undo, &url->KeyS, &url->lgs);

	} else if (url->fu & ur_DelSel) {
	    if (ya == url->ye) {
		if (xa > url->xe) xa = url->xe;
	    } else {
		if (ya > url->ye) {xa = url->xe; ya = url->ye;}
	    }
		le->SelAx = xa;
		le->SelAy = ya;
		le->CaretCharIdx = xa;
		le->CaretCharIdy = ya;
		TLineEdit_DoChange(le, ur_InsStr | ur_Undo, &url->KeyS, &url->lgs);
		if (url->fu & ur_DelChar) {
		    le->SelAx = le->CaretCharIdx;
		    le->SelAy = le->CaretCharIdy;
		} if (url->fu & ur_DelCharA) {
		    le->CaretCharIdx = le->SelAx;
		    le->CaretCharIdy = le->SelAy;
		}
	}

	TLineEdit_ChkCaret(le, 0);
	TLineEdit_SetCaret(le);

	Refresh((TWins*)le);
	return True;
}

bool TLineEdit_Redo(TLineEdit *le, TUnRedoLine *url) {
  TPlugIn_KeyDown *plug;

	if (TLineEdit_Undo_ChkCaret(le, url)) return False;

	if (url->fu & ur_plugin) {
	    plug = TLineEdit_GetKeyDownPlugIn(le);
	    while (plug) {
		if (plug->ReUndo && plug->ReUndo((TWins*)le, url)) return True;
		plug = plug->next;
	    }
	}

	if (url->fu & (ur_InsKey | ur_InsStr)) {
	    le->SelAx = url->xa;
	    le->SelAy = url->ya;
	    le->CaretCharIdx = url->xa;
	    le->CaretCharIdy = url->ya;
	    TLineEdit_DoChange(le, url->fu, &url->KeyS, &url->lgs);

	} else if (url->fu & ur_DelSel) {
	    le->SelAx = url->xa;
	    le->SelAy = url->ya;
	    le->CaretCharIdx = url->xe;
	    le->CaretCharIdy = url->ye;
	    TLineEdit_DoChange(le, url->fu, &url->KeyS, &url->lgs);

	} else {
 printf("TLineEdit_Redo: Error fu=0x%x\n", url->fu);
		return True;
	}

	le->SelAx = le->CaretCharIdx;
	le->SelAy = le->CaretCharIdy;
	TLineEdit_ChkCaret(le, 0);
	TLineEdit_SetCaret(le);
	Refresh((TWins*)le);

	return True;
}
