/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"

//*********************** undo - redo ************************

static void TUnRedo_ClrLine(TUnRedo *ur, int idx) {
	Tlgs_Clr(&ur->lines[idx].lgs);
	ur->lines[idx].fu = 0;
	ur->lines[idx].KeyS.d[0] = 0;
}

static void TUnRedo_DelLine0(TUnRedo *ur) {
  int z;
	if (!ur->count) return;
	ur->count--;
	ur->idx = ur->count;
	TUnRedo_ClrLine(ur, 0);
	z = sizeof(TUnRedoLine) * (ur->max_items-1);
	pnmove(&ur->lines[0], &ur->lines[1], z);
}

void TUnRedo_Clr(TUnRedo *ur) {
  int i;

	if (!ur->count) return;
	for (i=0;i<ur->count;i++) {
	    TUnRedo_ClrLine(ur, i);
	}
	ur->idx = 0;
	ur->count = 0;
}

void TUnRedo_AddLine(TUnRedo *ur, int fu, TKeyS *KeyS, Tlgs *lgs, int xa, int ya, int xe, int ye) {
  int i;
  TUnRedoLine *url;

	if (fu & (ur_Undo | ur_Redo | ur_Nop)) return;
	if (ur->idx < ur->count) {
	    for (i = ur->idx; i < ur->count; i++) TUnRedo_ClrLine(ur, i);
	    ur->count = ur->idx;
	}
	if (ur->count >= ur->max_items) TUnRedo_DelLine0(ur);

	i = ur->count;
	ur->idx = i;
	url = &ur->lines[i];
	url->fu = fu;
	url->xa = xa;
	url->ya = ya;
	url->xe = xe;
	url->ye = ye;
	url->KeyS.d[0] = 0;
	if (KeyS) {
	    url->KeyS.d[0] = KeyS->d[0];
	    url->KeyS.d[1] = KeyS->d[1];
	    url->KeyS.d[2] = KeyS->d[2];
	    url->KeyS.d[3] = KeyS->d[3];
	}

	url->lgs.s = NULL;
	url->lgs.lg = 0;
	if (lgs) {
	    if (fu & ur_DupTlgs) {
		Tlgs_Add(&url->lgs, lgs);
	    } else {
		url->lgs.lg = lgs->lg;
		url->lgs.s = lgs->s;
		lgs->s = NULL;
		lgs->lg = 0;
	    }
	} 

	ur->count++;
	ur->idx = ur->count;
} 

TUnRedoLine *TUnRedo_GetUndo(TUnRedo *ur) {
  TUnRedoLine *url;
	if ((ur->count > 0) && (ur->idx > 0)) {
	    ur->idx--;
	    url = &ur->lines[ur->idx];
	    url->fu |= ur_Undo;
	    url->fu &= ~ur_Redo;
	    return url;
	} else return NULL;
}

TUnRedoLine *TUnRedo_GetRedo(TUnRedo *ur) {
  TUnRedoLine *url;

	if ((ur->count > 0) && (ur->idx < ur->count)) {
	    url = &ur->lines[ur->idx];
	    url->fu &= ~ur_Undo;
	    url->fu |= ur_Redo;
	    ur->idx++;
	    return url;
	} else return NULL;
}

void TUnRedo_Undo_Old(TUnRedo *ur) {
	if ((ur->count > 0) && (ur->idx < ur->count)) ur->idx++;
}
void TUnRedo_Redo_Old(TUnRedo *ur) {
	if ((ur->count > 0) && (ur->idx > 0)) ur->idx--;
}

void TUnRedo_GetStat(TUnRedo *ur, bool *undo, bool *redo) {
	*undo = False;
	*redo = False;
	if (ur && (ur->count > 0)) {
	    *undo = (ur->idx > 0);
	    *redo = (ur->count > ur->idx);
	}
}

/* void TUnRedo_Get_xy(TUnRedoLine *url, int *x, int *y, int max) {
  int m, xa, ya, xe, ye;

	xa = url->xa;
	ya = url->ya;
	xe = url->xe;
	ye = url->ye;
	if (ya > ye) {
	    m = xa; xa = xe; xe = m;
	    m = ya; ya = ye; ye = m;
	} if ((ya == ye) && (xa > xe)) {
	    m = xa; xa = xe; xe = m;
	}
	if (max) {
	    *x = xe;
	    *y = ye;
	} else {
	    *x = xa;
	    *y = ya;
	}
} */

static bool TUnRedo__Destroy(TUnRedo *ur) {
	TUnRedo_Clr(ur);
	return False;
}

TUnRedo *TUnRedo_Create(int max_items) {
  TUnRedo *ur;
  int size;
	if (max_items < 3) max_items = 3;
	size = sizeof(TUnRedo) + sizeof(TUnRedoLine) * max_items;
	ur = ttc_create(size, wtyp_TLinesUnRedo);
	ur->destroy = (TDESTROY)TUnRedo__Destroy;
	ur->max_items = max_items;
	return ur;
}
