/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"


static void chkmimax(TDVal *dv, double *val) {
	if (*val > dv->max) *val = dv->max;
	if (*val < dv->min) *val = dv->min;
}

static void TDVal_SetValStr(TDVal *dv) {
  double dval = dv->dval;
  int nk = dv->nk;
  char buf[32];

	if (dv->dv_fu & tdv_fu_KorrOn) {
	    dval /= dv->korr;
	    nk = dv->korr_nk;
	}
	doubletopc(buf, dval, nk);
	SetCaption((TWins*)dv, buf);
}


static void TDVal_DoPaint(TDVal *self) {
	if (!self->vds.s) TDVal_SetValStr(self);
	TEdit_DoPaint((TEdit*)self);
}

void TDVal_DoValChange(TDVal *self, double val, int fu) {
	if ((self->dval != val) || self->ForceChange_mk) {
	    self->Change_mk = false;
	    self->ForceChange_mk = false;
	    chkmimax(self, &val);
	    self->dval = val;
	    if (self->OnValChange && (fu & tdvfu_ChangeEv)) 
			self->OnValChange(self, self->dval);
	    TDVal_SetValStr(self);
	}
}

static double TDVal_RdDouble(TDVal *dv) {
  double dval = pctodouble(dv->vds.s);
	if (dv->dv_fu & tdv_fu_KorrOn) {
	    dval *= dv->korr;
	}
	return dval;
}

static int TDVal_DoKeyDown(TDVal *self, TKeyEvent *KeyEv) {
  int res;
  bool ent = (KeyEv->keysym == XK_Return);

	if ((KeyEv->keycode == 55) && (KeyEv->state & 0x4)) {
	    self->Change_mk = true;
	} else if (KeyEv->keysym == XK_BackSpace) {
	    if (self->CaretCharIdx > 0) self->Change_mk = true;
	} else if (KeyEv->keysym == XK_Delete) {
	    self->Change_mk = true;
	}

	res = TEdit_DoKeyDown((TEdit*)self, KeyEv);
	if (ent && !self->ReadOnly) {
	    TDVal_DoValChange(self, TDVal_RdDouble(self), tdvfu_ChangeEv);
	}
	return res;
}

static bool TDVal_DoCharAllow(TDVal *self, TKeyEvent *KeyEv, int idx) {
  char *pc = self->vds.s;
  char *tpc = NULL;
  int i = 0;
  int vk = 0;
  int nk = 0;
  int nk0 = self->nk;
  int lg;
  bool dot = false;
  bool neg = false;
  bool idx_nk = false;
  bool res;

	if (self->dv_fu & tdv_fu_KorrOn) {
	    nk0 = self->korr_nk;
	}

	if (self->vds.SelA != self->vds.SelE) {
	    tpc = pcdup(pc);
	    lg = self->vds.lg;
	    ppclg_delsel(&tpc, &lg, self->vds.SelA, self->vds.SelE);
	    if (self->vds.SelA < self->vds.SelE) {
		idx = self->vds.SelA;
	    } else {
		idx = self->vds.SelE;
	    }
	    if (idx < 0) idx = 0;
	    pc = tpc;
	}

	if (pc) {
	    while (*pc) {
		switch (*pc) {
		case '-': neg = true; break;
		case '.':
		case ',': dot = true; idx_nk = (idx > i); break;
		default: if (dot) nk++; else vk++;
		}
		pc++;
		i++;
	    }
	}
	xfree(tpc);

	res = false;
	if ((KeyEv->keysym >= '0') && (KeyEv->keysym <= '9')) 
		res = ((!neg || (idx != 0)) && 
			(!idx_nk || (nk0 > nk)));
	if ((KeyEv->keysym == '.') || (KeyEv->keysym == ',')) 
		res = (!dot && nk0);
	if (KeyEv->keysym == '-') 
		res = (!neg && (idx == 0));
	if (res) self->Change_mk = true;
	return res;
}

void TDVal_Set_Value(TDVal *self, double dval) {
	if (self->dval == dval) return;
	self->dval = dval;
	chkmimax(self, &self->dval);
	if (self->dval != dval) {
	    self->ForceChange_mk = true;
	    self->Change_mk = true;
	}
	TDVal_SetValStr(self);
}

void TDVal_Set_Value_ev(TDVal *self, double dval) {
	if (self->dval == dval) return;
	self->dval = dval;
	chkmimax(self, &self->dval);
	self->ForceChange_mk = false;
	if (self->OnValChange) self->OnValChange(self, self->dval);
	TDVal_SetValStr(self);
}

void TDVal_Set_Min(TDVal *dv, double dval) {
	dv->min = dval;
	if (dv->dval < dval) {
	    dv->dval = dval;
	    TDVal_SetValStr(dv);
	}
}

void TDVal_Set_Max(TDVal *dv, double dval) {
	dv->max = dval;
	if (dv->dval > dval) {
	    dv->dval = dval;
	    TDVal_SetValStr(dv);
	}
}

static void TDVal_DoMouseDown(TDVal *self, XButtonEvent *ev) {
	TEdit_DoMouseDown((TEdit*)self, ev);
	self->MouseDown_dval = TDVal_RdDouble(self);
	self->MouseMove_Sel_mk = 0;
}

static void TDVal_DoMouseMove(TDVal *self, XMotionEvent *ev) {
  int x;

	if (!(ev->state & Button1Mask)) {
	    TEdit_DoMouseMove((TEdit*)self, ev);
	    return;
	}

	x = 0;
	if ((self->dv_fu & tdv_fu_MouseMove_add) &&
	    (self->MouseMove_Sel_mk != 1) &&
	    (MouseDown.apph == (TWins*)self)) {

	    if (ev->y < 0) x = -ev->y;
	    if (ev->y > self->height) x = -(ev->y - self->height);

	    if (x) {
		self->MouseMove_Sel_mk = 2;
		x += ev->x - MouseDown.x;
	    }
	}

	if (self->MouseMove_Sel_mk == 2) {
	    TDVal_Set_Value_ev(self, self->MouseDown_dval + 
				     (x * self->MouseMove_add_dval));
	    TWins_DoMouseMove((TWins*)self, ev);
	} else {
	    TEdit_DoMouseMove((TEdit*)self, ev);
	    if (self->vds.SelA != self->vds.SelE) {
		self->MouseMove_Sel_mk = 1;
	    }
	}
}

static void TDVal_DoFocus(TDVal *self, int Change) {
	if (Change & WinsFocusIN) {
	    self->CaptionFu |= mCaptionRePaint;
	    Paint((TWins*)self);
	    TEdit_DoFocus((TEdit*)self, Change);
	} else {
	    TEdit_DoFocus((TEdit*)self, Change);
	    self->CaptionFu |= mCaptionRePaint;
	    if ((self->dv_fu & tdv_fu_LostFocusEnter) && 
		!self->ReadOnly && self->Change_mk) {
		TDVal_DoValChange(self, TDVal_RdDouble(self), tdvfu_ChangeEv);
	    } else {
		TDVal_SetValStr(self);
	    }
	    Refresh((TWins*)self);
	}
}

void TDVal_KorrOnOff(TDVal *dv, int OnOff) {
	if (OnOff && ((dv->dv_fu & tdv_fu_KorrOn) == 0)) {
	    dv->dv_fu |= tdv_fu_KorrOn;
	} else if (!OnOff && (dv->dv_fu & tdv_fu_KorrOn)) {
	    dv->dv_fu &= ~tdv_fu_KorrOn;
	} else return;
	TDVal_SetValStr(dv);
	Refresh((TWins*)dv);
}

void TDVal_SetUpKorr(TDVal *dv, double korr, int nk, int OnOff) {
	if ((dv->korr != korr) || (dv->korr_nk != nk)) {
	    dv->korr = korr;
	    dv->korr_nk = nk;
	    TDVal_SetValStr(dv);
	    Refresh((TWins*)dv);
	}
	TDVal_KorrOnOff(dv, OnOff);
}

int TDVal_GetIntegerMult(TDVal *self, double multfakt) {
	return (self->dval * multfakt + 0.5);
}

int TDVal_GetInteger(TDVal *self) {
	return (self->dval + 0.5);
}
/* ______________________ Create _____________________________________ */
static void TDVal_CreateParams(TWins_Create_struct *wcs) {
  TDVal *self = wcs->wins;

	TEdit_CreateParams(wcs);
	self->AutoSel = true;
	self->dv_fu = tdv_fu_LostFocusEnter;
	self->min = -9999999;
	self->max = 9999999;
	self->korr = 25.4;	// mm/Inch
	self->nk = 3;
	self->korr_nk = 3;
	self->DoPaint = TDVal_DoPaint;
	self->DoKeyDown = TDVal_DoKeyDown;
	self->DoCharAllow = TDVal_DoCharAllow;
	self->DoMouseDown = TDVal_DoMouseDown;
	self->DoMouseMove = TDVal_DoMouseMove;
	self->DoFocus = TDVal_DoFocus;
	self->CaptionFu &= ~mCaptionLeft;
	self->CaptionFu |= mCaptionRigth;
}

TDVal *TDVal_Create(TWins *Parent) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TDVal), wtyp_TDVal);
	wcs.Parent = Parent;
	wcs.CreateParams = TDVal_CreateParams;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}

TDVal *TDVal_CreateS(TWins *Parent, int Left, int Top, int Width, int Height) {
  TDVal *self;

	self = TDVal_Create(Parent);
	SelfCreateS((TWins*)self, Left, Top, Width, Height, NULL);
	return self;
}

TDVal *TDVal_CreateSV4(TWins *Parent, int Left, int Top, int Width, int Height, 
			double val, double min, double max, int nk) {
  TDVal *self;

	self = TDVal_Create(Parent);
	self->min = min;
	self->max = max;
	self->nk = nk;
	chkmimax(self, &val);
	self->dval = val;
	SelfCreateS((TWins*)self, Left, Top, Width, Height, NULL);
	return self;
}
