/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tdirlist.h"
#include <sys/stat.h>

void TMenuFile_DoPaint(TMenuFile *self) {
  int y, h;

	TMenu_DoPaint((TMenu*)self);
	if (self->side_pic) {
	    h = self->side_pic->height - self->height + 4;
	    y = 2;
	    if (h < 0) {
		y -=h;
		h = 0;
	    }
	    DrawPic_wh(self->WinID, self->side_pic, 0, h,
		    self->SideXOff, self->height - 4, 2, y);
	}
}

static bool TMenuFileItem__Destroy(TMenuFileItem *self) {
	if (self->wtyp == wtyp_MenuFileItem) {
	    xfree(self->filename);
	    xfree(self->itemname);
	}
	return TMenuItem__Destroy((TMenuItem*)self);
}

static TMenuFileItem *TMenuFile_DoMenuItemCreate(TMenuFile *self) {
  TMenuFileItem *mi;
	mi = ttc_create(sizeof(TMenuFileItem), wtyp_MenuFileItem);
	mi->destroy = (TDESTROY)TMenuFileItem__Destroy;
	return mi;
}

static int TMenuFile_ChkSubMenu(char *s) {
  int res = 0;
  TDirListObjNy *ObjNy;
  TDirFileList *dfl0, *dfl;

	ObjNy = TDirListObjNy_Create(NULL);
	ObjNy->findAtr = findREG | findDIR;
	dfl0 = TDirListObjNy_GetDflSingle(ObjNy, s);

	dfl = dfl0;
	while (dfl) {
	    dfl0 = dfl->next;
	    if (dfl->st->st & st_Dir) {
		res |= 0x01;
	    } else if (dfl->st->st & st_Reg) {
		if (dfl->st->FileType == stft_desktop) res |= 0x02;
	    }
	    xfree(dfl);
	    dfl = dfl0;
	}

	return res;
}

static void TMenuFile_Sort(TMenuFile *self, const char *sort) {
  TMenuFileItem *mi;
  int count, lg, i;
  const char *indata, *token;
  TMenuFileItem **miarr, **ppmi, *miprev;
  bool inv = (self->mfk & mfk_sort_inv);
  bool mkHead = False;

	count = 0;
	mi = self->MenuItem;
	while (mi) {
	    mi->mfik &= ~mfik_sort_ok;
	    if ((mi->fu & miHead) && !mkHead) {
		mkHead = True;
		count++;
	    }
	    count++;
	    mi = mi->next;
	}
	if (count < 2) return;

	indata = sort;
	while (indata && *indata) {
	    indata = pcGetToken(indata, &token, &lg);
	    if (token && (*token == '-')) {
		token++;
		if (*token == 't') count++;
	    }
	}
	miarr = xcalloc((count+1) * sizeof(void*));
	
	indata = sort;
	ppmi = miarr;

	if (mkHead) {
	    mi = self->MenuItem;
	    while (mi) {
		if (mi->fu & miHead) {
		    mi->mfik |= mfik_sort_ok;
		    *ppmi = mi;
		    ppmi++;
		}
		mi = mi->next;
	    }
	    mi = (TMenuFileItem*)TMenu_AddItem((TMenu*)self, "-", 0, NULL);
	    mi->mfik |= mfik_sort_ok;
	    *ppmi = mi;
	    ppmi++;
	}

	if (inv) ppmi = &miarr[count-1];

	while (indata && *indata) {
	    indata = pcGetToken(indata, &token, &lg);
	    if (token && (*token == '-')) {
		token++;
		if (*token == 't') {
		    mi = (TMenuFileItem*)TMenu_AddItem((TMenu*)self, "-", 0, NULL);
		    mi->mfik |= mfik_sort_ok;
		    *ppmi = mi;
		    if (inv) ppmi--; else ppmi++;
		}
	    } else {
		mi = self->MenuItem;
		while (mi) {
		    if ((pclen(mi->itemname) == lg) && pcncmp(mi->itemname, token, lg)) {
			mi->mfik |= mfik_sort_ok;
			*ppmi = mi;
			if (inv) ppmi--; else ppmi++;
			mi = NULL;
		    } else mi = mi->next;
		}
	    }
	}
	mi = self->MenuItem;
	while (mi) {
	    if ((mi->mfik & mfik_sort_ok) == 0) {
		*ppmi = mi;
		if (inv) ppmi--; else ppmi++;
	    }
	    mi = mi->next;
	}
	ppmi = miarr;
	miprev = NULL;
	for (i=0; i<count; i++) {
	    mi = *ppmi;
	    mi->prev = miprev;
	    miprev = mi;
	    ppmi++;
	    mi->next = *ppmi;
	}
	mi->next = NULL;
	self->MenuItem = *miarr;
	xfree(miarr);
}

static void TMenuFile_InitDat_dfl(TMenuFile *self, TDirFileList *dfl) {
  int chk, mifu;
  char *fn;
  TMenuFileItem *mi;
  TMenuFile *tmp_mf;
  TpcIni *ini;
  TDesktopHead ddkh;
  bool mkDirHead;
  bool ok;

	fn = pcdup(self->path);

	if (dfl->st->st & st_Dir) {
	    path_add_Slash_ppc(&fn, dfl->fn);
	    tmp_mf = NULL;
	    mifu = 0;
	    chk = TMenuFile_ChkSubMenu(fn);
	    if (chk) {
		if (self->OnNextMenuCreate) {
		    tmp_mf = self->OnNextMenuCreate(self);
		    if (!tmp_mf->OnNextMenuCreate) 
			tmp_mf->OnNextMenuCreate = self->OnNextMenuCreate;
		} else {
		    tmp_mf = TMenuFile_Create((TWins*)self);
		}
		tmp_mf->OnClick = self->OnClick;
		ppcchange(&tmp_mf->path, fn);
	    } else mifu = miNotEnabled;

	    ppcadd(&fn, ".directory");
	    ini = TpcIni_Create(fn);
	    ddkh.name = dfl->fn;
	    TPic *pic = NULL;
	    if (dotdesktop_get_head(ini, &ddkh)) {
		pic = GetDeskIcon(ddkh.ddkt, self->icon_size, ddkh.icon, 0, NULL);
	    }
	    mi = (TMenuFileItem*)TMenu_AddItem((TMenu*)self, ddkh.name, mifu, NULL);
	    Tfree(ini);
	    mi->pic = pic;
	    ppcchange(&mi->itemname, dfl->fn);
	    mi->next_menu = tmp_mf;

	} else if (dfl->st->st & st_Reg) {

	    mkDirHead = ((self->mfk & mfk_AddDirHead) && pccmp(dfl->fn, ".directory"));
	    if ((dfl->st->FileType == stft_desktop) || mkDirHead) {
		path_add_ppc(&fn, dfl->fn);
		ini = TpcIni_Create(fn);
		ddkh.name = dfl->fn;
		TPic *pic = NULL;
		ok = True;
		if (dotdesktop_get_head(ini, &ddkh)) {
		    if (self->OnAddFilter) ok = self->OnAddFilter(self, dfl, &ddkh);
		    if (ok) pic = GetDeskIcon(ddkh.ddkt, self->icon_size, ddkh.icon, 0, NULL);
		} else {
		    if (self->OnAddFilter) ok = self->OnAddFilter(self, dfl, NULL);
		}
		if (ok) {
		    mi = (TMenuFileItem*)TMenu_AddItem((TMenu*)self, ddkh.name, 0, NULL);
		    mi->pic = pic;
		    mi->itemname = ChangeFileExt(dfl->fn, NULL);
		    ppcchange(&mi->filename, dfl->fn);
		    if (mkDirHead) mi->fu |= miHead | miCanNotSelect;
		}
		Tfree(ini);
	    }
	}
	xfree(fn);
}

static void TMenuFile_InitDat(TMenuFile *self) {
  char *dfile;
  const char *sort, *icon;
  TDirListObjNy *ObjNy;
  TDirFileList *dfl0, *dfl;
  TpcIni *ini;

	if (self->OnInitDat) self->OnInitDat(self, False);
	if (self->mfk & mfk_init_ok) return;
	self->mfk |= mfk_init_ok;
	if (pclen(self->path) < 3) return;
	if (!path_end_isSlash(self->path)) path_add_Slash_ppc(&self->path, NULL);

	ObjNy = TDirListObjNy_Create(self);
	ObjNy->findAtr = findDIR | findREG | findIsRootPath;
	if ((self->mfk & mfk_AddDirHead) == 0) ObjNy->findAtr |= findNoHidden;
	dfl0 = TDirListObjNy_GetDflSingle(ObjNy, self->path);
	TDirFileList_Sort(&dfl0, dlsortFu_Name);

	dfl = dfl0;
	while (dfl) {
	    dfl0 = dfl->next;
	    TMenuFile_InitDat_dfl(self, dfl);
	    xfree(dfl);
	    dfl = dfl0;
	}

	dfile = path_add(self->path, ".directory");
	if (TestIsFile(dfile)) {
	    ini = TpcIni_Create(dfile);
	    sort = TpcIni_RD(ini, Desktop_Entry, "ALDsort");
	    if (sort) TMenuFile_Sort(self, sort);
	    icon = TpcIni_RD(ini, Desktop_Entry, "ALDSideIcon");
	    if (icon) {
		self->side_pic = GetDeskIcon(0, 0, icon, 0, NULL);
	    }
	    Tfree(ini);
	}
	xfree(dfile);
	if (self->OnInitDat) self->OnInitDat(self, True);
}


static void TMenuFile_DoShow(TMenuFile *self, int Left, int Top) {
	TMenuFile_InitDat(self);
	TMenu_DoShow((TMenu*)self, Left, Top);
}

static void TMenuFile_DoMenuClose(TMenuFile *self) {
	TMenu_DoClose((TMenu*)self);
	if (self->mfk & mfk_AutoFree) {
	    TMenu_Cls((TMenu*)self);
	    self->mfk &= ~mfk_init_ok;
	}
}
/*____________________________ Destroy __________________________________*/
bool TMenuFile__Destroy(TMenuFile *self) {
// printf("%s(%d): TMenuFile__Destroy 0x%08x\n", FL, (int)self);
	xfree(self->path);
	return TMenu__Destroy((TMenu*)self);
}

/*_____________________________ Create __________________________________*/
void TMenuFile_CreateParams(TWins_Create_struct *wcs) {
  TMenuFile *self = wcs->wins;

	TMenu_CreateParams(wcs);

	self->icon_size = 16;
	self->destroy = (TDESTROY)TMenuFile__Destroy;
	self->DoMenuItemCreate = TMenuFile_DoMenuItemCreate;

	self->DoPaint = TMenuFile_DoPaint;
	self->DoShow = TMenuFile_DoShow;
	self->DoMenuClose = TMenuFile_DoMenuClose;
	self->RunStat = meRS_ParentCanDestroy;
}

TMenuFile *TMenuFile_Create(TWins *Form) {
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TMenuFile), wtyp_MenuFile);
	wcs.Parent = Form;
	wcs.CreateParams = TMenuFile_CreateParams;
	TWins_DefaultConf(&wcs);
	return wcs.wins;
}
