/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tfilewin.h"
#include "dlg.h"

typedef struct {
  TDirListObjNy *ObjNy;
  TDirFileList *dfl;
  TTreeVNode *dummy_tvn;
  TTreeVNode *dummy_tvn_next;
  TglobPath gp;
} Tprivate;


void TTreeVFile_SortNode(TDirListObjNy *ObjNy, int sortfu, TTreeVNode *child) {
  TDirFileList *dfl;
  TTreeVNode *tvn, **pptvn, *prev, *prev_child;

	if (!ObjNy || !child) return;
	TDirListObjNy_Sort(ObjNy, sortfu);
	dfl = ObjNy->dfl;
	pptvn = &child->next_child;
	prev_child = child;
	prev = NULL;
	while (dfl) {
	    if (dfl->tagp) {
		tvn = dfl->tagp;
		tvn->prev = prev;
		tvn->prev_child = prev_child;
		tvn->next = NULL;
		prev = tvn;
		prev_child = NULL;
		*pptvn = tvn;
		pptvn = &tvn->next;
	    }
	    dfl = dfl->next;
	}
	if (child && child->private) {
	    TTreeVNode *dummy_tvn = ((Tprivate*)child->private)->dummy_tvn;
	    while (dummy_tvn) {
		dummy_tvn->prev = prev;
		dummy_tvn->prev_child = prev_child;
		dummy_tvn->next = NULL;
		prev = dummy_tvn;
		prev_child = NULL;
		*pptvn = dummy_tvn;
		pptvn = &dummy_tvn->next;

		dummy_tvn = ((Tprivate*)dummy_tvn->private)->dummy_tvn_next;
	    }
	}
}

static TTreeVNode *get_dummy_node(TTreeVNode *tvn, const char *name) {
  TTreeVNode *ptvn;

	if (!tvn || (tvn->wtyp != wtyp_TTreeVFileNode)) return NULL;
	ptvn = ((Tprivate*)tvn->private)->dummy_tvn;
	while (ptvn) {
	    if (pccmp(TTreeVNode_GetCaption(ptvn), name)) break;
	    ptvn = ((Tprivate*)ptvn->private)->dummy_tvn_next;
	}
	return ptvn;
}

static void TTreeVFile_get_s_lg(TTreeVNode *tvn, const char **s, int *lg) {
  const char *pc = NULL;
	if (tvn && ((Tprivate*)tvn->private)->dfl) {
	    pc = ((Tprivate*)tvn->private)->dfl->fn;
	}
	*s = pc;
	*lg = pclen(pc);
}

static void TTreeVFile_OnFileMsg(TFileMsg *fmsg) {
  TDirListObjNy *self;
  TTreeVNode *ntvfn;
  TTreeVNode *tvfn;
  Tprivate *pv, *pvn;
  TDirFileList *dfl;
  TTreeVFile *tvf;
  int bef;

	if (!file_msg_chk_wtyp(fmsg, wtyp_TDirListObjNy, FL)) return;

	self = fmsg->obj;
	tvfn = self->owner;
	if (!tvfn || !tvfn->tv) return;
	tvf = (TTreeVFile*)tvfn->tv;
	if ((fmsg->art != fmArt_DirList) ||
	    (fmsg->typD != fmTyp_int)) {
	    if (tvf->OnFileMsg) tvf->OnFileMsg(tvf, tvfn, fmsg);
	    return;
	}

	fmsg->Ok = True;
	bef = fmsg->D.m;
	dfl = fmsg->F.dfl;

	pv = tvfn->private;

	switch (bef & dlbef_BefMask) {
	case dlbef_Add:
	    tvf->AnyNodeChange = True;
	    ntvfn = TTreeVFileNode_GetDummyNode_name(tvfn, dfl->fn);
	    if (!ntvfn) ntvfn = TTreeVFile_AddNode(tvf, tvfn, NULL, NULL);
	    ntvfn->get_s_lg = TTreeVFile_get_s_lg;
	    dfl->tagp = ntvfn;
	    pvn = ntvfn->private;
	    pvn->dfl = dfl;
	    ntvfn->pic = get_fileIcon(stft_Folder);
	    if (dfl->st->nlink > 2) ntvfn->st |= tvnSt_ForceCanExp;
	    if (dfl->st->st & st_Hidden) ntvfn->st |= tvnSt_NotVisible;
	    break;

	case dlbef_Del:
	    tvf->AnyNodeChange = True;
	    ppTfree(&dfl->tagp);
	    break;

	case dlbef_Change:
	    ntvfn = dfl->tagp;
	    if (ntvfn) {
		int st = ntvfn->st;
		ntvfn->st &= ~tvnSt_ForceCanExp;
		if (dfl->st->nlink > 2) ntvfn->st |= tvnSt_ForceCanExp;
		if (st != ntvfn->st) tvf->AnyNodeChange = True;
	    }
	    break;

	case dlbef_DelSelf:
	    tvf->AnyNodeChange = True;

	    fmsg->Ok = False;	//in TDirListObjNy   Tfree(ObjNy);
	    if (tvf->OnFileMsg) tvf->OnFileMsg(tvf, tvfn, fmsg);
	    pv->ObjNy = NULL;
	    if ((tvfn->art & tfvfart_MASK) != tfvfart_ROOT) Tfree(tvfn);
	    return;

	case dlbef_NeedSort:
	    tvf->AnyNodeChange = True;
	    break;

	case dlbef_Sync:
	case dlbef_ListEnd:
	    if (tvf->AnyNodeChange) {
		TTreeVFile_SortNode(self, 0, tvfn);
		TTreeV_Update_ScrollBar((TTreeV*)tvf, NULL);
		tvf->AnyNodeChange = False;
	    }
	    break;

	}
	if (tvf->OnFileMsg) tvf->OnFileMsg(tvf, tvfn, fmsg);
}

static void ch_user_pw_gp(TglobPath *dgp, TglobPath *sgp) {
	if (!dgp->user && sgp->user) ppcchange(&dgp->user, sgp->user);
	if (!dgp->passwd && sgp->passwd) ppcchange(&dgp->passwd, sgp->passwd);
}
bool TTreeVFile_SetSel_gp(TTreeVFile *tvf, TTreeVNode *tvfn) {
  char *dpc;
  const char *cpc;
  TTreeVNode *tvfnx;
  Tprivate *pv;

	if (!tvfn) return False;
	dpc = NULL;

	ppcfree(&tvf->sel_gp.user);
	ppcfree(&tvf->sel_gp.passwd);

	tvfnx = tvfn;
	while (tvfnx && (tvfnx->wtyp == wtyp_TTreeVFileNode) && !(tvfnx->art & tfvfart_MASK)) {
	    pv = tvfnx->private;
	    ch_user_pw_gp(&tvf->sel_gp, &pv->gp);
	    cpc = TTreeVNode_GetCaption(tvfnx);
	    if (cpc && *cpc) {
		ppcins(&dpc, "/", 0);
		ppcins(&dpc, cpc, 0);
	    }
	    tvfnx = TTreeV_Get_prev_child((TTreeV*)tvf, tvfnx);
	}

	if (TTreeVFileNode_ChkIsRootNode(tvfnx)) {
	    pv = tvfnx->private;
  char *pc = pv->gp.path;
	    if (pc && *pc) ppcins(&dpc, pc, 0);

	    tvf->sel_gp.fu = pv->gp.fu;
	    ppcchange(&tvf->sel_gp.RootArt, pv->gp.RootArt);
	    ppcchange(&tvf->sel_gp.URL,     pv->gp.URL);
	    ch_user_pw_gp(&tvf->sel_gp, &pv->gp);
	    xfree(tvf->sel_gp.path);
	    tvf->sel_gp.path = dpc;

  TDirList *dl = TDirListObjNy_Get_dl_gp(&tvf->sel_gp);
	    if (dl) {  //set akt user,passwd
		if (dl->gp.user) {
		    ppcchange_cmp(&pv->gp.user, dl->gp.user);
		    ppcchange_cmp(&tvf->sel_gp.user, dl->gp.user);
		}
		if (dl->gp.passwd) {
		    ppcchange_cmp(&pv->gp.passwd, dl->gp.passwd);
		    ppcchange_cmp(&tvf->sel_gp.passwd, dl->gp.passwd);
		}
	    }
	    return True;
	}
	xfree(dpc);
	return False;
}

static void tvn_ObjNy_Free(TTreeVNode *tvn) {
	if (tvn->private) ppTfree(&((Tprivate*)tvn->private)->ObjNy);
}

static void fnode_collaps_child(TTreeVNode *tvfn) {
  TTreeVNode *fnx;

	fnx = tvfn->next_child;
	while (fnx) {
	    if (fnx->next_child) fnode_collaps_child(fnx);
	    tvn_ObjNy_Free(fnx);
	    Tfree(fnx);
	    fnx = tvfn->next_child;
	}
}

static void TTreeVFile_DoCollaps(TTreeVFile *self, TTreeVNode *tvn) {
  TglobPath *gp;
  TTreeVNode *tvnx = tvn->next_child;

	if (tvn->wtyp != wtyp_TTreeVFileNode) return;
	while (tvnx) {
	    if (tvnx->next_child && !(tvnx->st & tvnSt_Expanded)) {
		fnode_collaps_child(tvnx);
	        if (tvnx->wtyp == wtyp_TTreeVFileNode) {
		    tvn_ObjNy_Free(tvnx);
		}
	    }
	    tvnx = tvnx->next;
	}
	if (TTreeVFile_SetSel_gp(self, tvn)) {
	    gp = &self->sel_gp;
	    if (gp_test_local_drive(gp) && 
	        (pcncmp(gp->path, "/dev/", 5) ||
		 pcncmp(gp->path, "/proc/", 6) ||
		 pcncmp(gp->path, "/sys/", 5))) {
		fnode_collaps_child(tvn);
		tvn_ObjNy_Free(tvn);
	    }
	}
	TTreeV_DoCollaps((TTreeV*)self, tvn);
}

/* _________________________ Expand _________________________________ */
static void tvn_ObjNy_Create(TTreeVFile *self, TTreeVNode *tvn, TglobPath *gp) {
  TDirListObjNy *ObjNy;
  Tprivate *pv;

	if (!tvn) return;
	pv = tvn->private;
	ObjNy = TDirListObjNy_Create(tvn);
	pv->ObjNy = ObjNy;
	ObjNy->findAtr = findDIR | findIsRootPath;
	ObjNy->OnFileMsg = TTreeVFile_OnFileMsg;
	ObjNy->chstat_Filter = chstat_Add|chstat_Del|chstat_nlink;
	if (!TDirListObjNy_Open_gp(ObjNy, gp)) {
	    tvn_ObjNy_Free(tvn);
	} else {
	    dword t = GetTime_ms() + 700;
	    while ((pv->ObjNy->dl->runstat >= 0) && (pv->ObjNy->dl->runstat < dlrunstat_ScanEnd) &&
		   AppHandleMessage() && !cpTime(t, GetTime_ms()));
	}
}

void TTreeVFile_ChkOpenNode(TTreeVFile *self, TTreeVNode *tvn) {
  Tprivate *pv;

	if (!tvn || (tvn->wtyp != wtyp_TTreeVFileNode)) return;
	pv = tvn->private;
	if (pv->ObjNy) return;

	if (!TTreeVFile_SetSel_gp(self, tvn)) return;
	if (self->OnChkOpenNode) self->OnChkOpenNode(self, &tvn);

	tvn_ObjNy_Create(self, tvn, &self->sel_gp);
}

static void TTreeVFile_DoExpand(TTreeVFile *tvf, TTreeVNode *tvn) {

	if (TTreeVFile_SetSel_gp(tvf, tvn)) {
	    TTreeVFile_ChkOpenNode(tvf, tvn);
	}
	TTreeV_DoExpand((TTreeV*)tvf, tvn);
}

char *TTreeVFile_GetNodeFileName(TTreeVFile *self, TTreeVNode *tvn) {
	if (!TTreeVFile_SetSel_gp(self, tvn)) return NULL;
	return gptopc(&self->sel_gp, gptfu_noName);
}

char *TTreeVFile_GetNodeFileName_passwd(TTreeVFile *self, TTreeVNode *tvn) {
	if (!TTreeVFile_SetSel_gp(self, tvn)) return NULL;
	return gptopc(&self->sel_gp, gptfu_PasswdTxt |gptfu_noName);
}

TDirFileList *TTreeVFileNode_Get_dfl(TTreeVNode *tvn) {
	if (!ttc_chk_is_wtyp(tvn, wtyp_TTreeVFileNode) || 
	    !tvn->private) return NULL;
	return ((Tprivate*)tvn->private)->dfl;
}

TglobPath *TTreeVFileNode_Get_gp(TTreeVNode *tvn) {
	if (!TTreeVFileNode_ChkIsRootNode(tvn)) return NULL; 
	return &((Tprivate*)tvn->private)->gp;
}

bool TTreeVFileNode_TestIsDummyNode(TTreeVNode *tvn) {
  const char *s;
	s = TTreeVNode_GetCaption(tvn);
	tvn = TTreeVNode_get_prev_child(tvn); //clr dummy_node
	return get_dummy_node(tvn, s) != NULL;
}

bool TTreeVFileNode_ChkIsRootNode(TTreeVNode *tvn) {
	return ttc_chk_is_wtyp(tvn, wtyp_TTreeVFileNode) &&
	       ((tvn->art & tfvfart_MASK) == tfvfart_ROOT) &&
	       tvn->private;
}

TTreeVNode *TTreeVFileNode_FindNode(TTreeVNode *root_tvn, const char *path) {
	if (!TTreeVFileNode_ChkIsRootNode(root_tvn)) return NULL;
	if (!path || !*path || (TestSlash(*path) && !path[1])) return root_tvn;

	while (*path && root_tvn) {
	    while (*path && TestSlash(*path)) path++;
	    const char *pce = path;
	    while (*pce && (*pce != '/')) pce++;
	    if (path != pce) {
		root_tvn = TTreeVNode_get_by_name_lg(root_tvn->next_child, path, pce-path, True);
	    }
	    path = pce;
	}
	return root_tvn;
}


TTreeVNode *TTreeVFileNode_GetDummyNode_name(TTreeVNode *tvn, const char *name) {
  TTreeVNode **pptvn;

	if (!tvn || (tvn->wtyp != wtyp_TTreeVFileNode)) return NULL;
	pptvn = &((Tprivate*)tvn->private)->dummy_tvn;
	while (*pptvn) {
	    if (pccmp(TTreeVNode_GetCaption(*pptvn), name)) break;
	    pptvn = &((Tprivate*)(*pptvn)->private)->dummy_tvn_next;
	}
	tvn = *pptvn;
	if (!tvn) return NULL;
	*pptvn = ((Tprivate*)tvn->private)->dummy_tvn_next;
	tvn->art &= ~tfvfart_Dummy;
	return tvn;
}

TTreeVNode *TTreeVFile_GetSelectNode(TTreeVFile *self) {
	if (!self) return NULL;
	return self->SelectNode;
}

TglobPath *TTreeVFile_GetSelect_gp(TTreeVFile *self, bool prev) {
  TTreeVNode *tvn = self->SelectNode;
	if (!tvn) return NULL;
	if (prev) tvn = TTreeVNode_get_prev_child(tvn);
	if (!TTreeVFile_SetSel_gp(self, tvn)) return NULL;
	return &self->sel_gp;
}

TDirFileList *TTreeVFile_GetSelect_dfl(TTreeVFile *self) {
	return TTreeVFileNode_Get_dfl(self->SelectNode);
}

void TTreeVFile_SetMouseZ_Wheel(TTreeVFile *self, int Value) {
	TTreeV_SetMouseZ_Wheel((TTreeV*)self, Value);
}

/*_____________________________ Paint Node ______________________________*/
void TTreeVFile_DoPaintNode(TTreeV *tv, TTreeVNode *tvn, int fu) {
  Tprivate *pv = tvn->private;

#if TVN_CHK_EXPAND
	if ((fu & tvnfu_TestExpand) && 
	    !TTreeVNode_TestNodeIsExpand(tvn, tvnCF_ALLChild)) return;
#endif
	TTreeV_DoPaintNode(tv, tvn, fu);
	if ((fu & tvnfu_CalcSizeOnly) || !pv) return;
	if (tvn->pic && pv->dfl && (pv->dfl->st->st & st_Link)) {
	    DrawPic(tv->WinID, get_fileIcon(stft_Link), tvn->left+1, tvn->top+11);
	}
}

/*_____________________________ Destroy _________________________________*/
static void TTreeVFileNode_ObjNy_Del(TTreeVNode *self) {
  Tprivate *pv = self->private;
  TDirFileList *dfl;
  TDirListObjNy *ObjNy;

	if (pv->dfl) {
	    pv->dfl->tagp = NULL;
	    pv->dfl = NULL;
	}
	if (pv->ObjNy) {
	    ObjNy = pv->ObjNy;
	    pv->ObjNy = NULL;
	    dfl = ObjNy->dfl;
	    while (dfl) {dfl->tagp = NULL; dfl = dfl->next;}
	    ObjNy->owner = NULL;
	    Tfree(ObjNy);
	}
}

bool TTreeVFileNode__Destroy(TTreeVNode *self) {
  Tprivate *pv = self->private;
  TTreeVNode *tvn;
  const char *s;

	s = TTreeVNode_GetCaption(self);
	tvn = TTreeVNode_get_prev_child(self); //clr dummy_node
	TTreeVFileNode_GetDummyNode_name(tvn, s);
	TTreeVFileNode_ObjNy_Del(self);
	gp_pc_free(&pv->gp);
	xfree(pv);

	return TTreeVNode__Destroy(self);
}

/*_____________________________ Create Node _____________________________*/

TTreeVNode *TTreeVFile_AddNode(TTreeVFile *self, TTreeVNode *prev_child, 
				TTreeVNode *prev, const char* s) {
  TTreeVNode *tvn;

	tvn = TNodeAddTo((void*)&self->Nodes, (TNode*)prev_child, (TNode*)prev, sizeof(TTreeVNode), wtyp_TTreeVFileNode);
	TTreeVNode_Param(tvn, (TTreeV*)self, s);
	tvn->destroy = (TDESTROY)TTreeVFileNode__Destroy;
	tvn->private = xcalloc(sizeof(Tprivate));
	return tvn;
}

TTreeVNode *TTreeVFile_AddDummyNode(TTreeVFile *self, TTreeVNode *prev_child, 
				TTreeVNode *prev, const char* s) {
  TTreeVNode *tvn, *tvxn;
  Tprivate *pv;

	tvn = TTreeVFile_AddNode(self, prev_child, prev, s);
	if (prev_child && prev_child->private) {
	    pv = prev_child->private;
	    tvxn = pv->dummy_tvn;
	    if (tvxn) {
		while (((Tprivate*)tvxn->private)->dummy_tvn_next) tvxn = ((Tprivate*)tvxn->private)->dummy_tvn_next;
		((Tprivate*)tvxn->private)->dummy_tvn_next = tvn;
	    } else {
		pv->dummy_tvn = tvn;
	    }
	    tvn->art |= tfvfart_Dummy;
	}
	return tvn;
}

void TTreeVFile_OpenRoodNode(TTreeVFile *self, TTreeVNode *tvn) {
  Tprivate *pv;

	if (!TTreeVFileNode_ChkIsRootNode(tvn)) return;
	pv = tvn->private;
	if (pv->ObjNy) return;

	tvn_ObjNy_Create(self, tvn, &pv->gp);
} 

TTreeVNode *TTreeVFile_AddRoodNode(TTreeVFile *self, TTreeVNode *prev_child, 
			TTreeVNode *prev, const char* s, const char *fn_root) {
  TTreeVNode *tvn = TTreeVFile_AddNode(self, prev_child, prev, s);
  Tprivate *pv = tvn->private;

	tvn->art = tfvfart_ROOT;
	pcsetgp(&pv->gp, fn_root);
	return tvn;
}

TTreeVNode *TTreeVFile_AddDummyRoodNode(TTreeVFile *self, TTreeVNode *prev_child, 
		TTreeVNode *prev, const char* s, const char *fn_root, TglobPath *gp) {
  TTreeVNode *tvn = TTreeVFile_AddDummyNode(self, prev_child, prev, s);
  Tprivate *pv = tvn->private;

	tvn->art = tfvfart_ROOT | tfvfart_Dummy;
	if (gp) {
	    gp_pc_free(&pv->gp);
	    gp_dup_gpdata_url(&pv->gp, gp);
	    pv->gp.path = pcdup("/");
	} else {
	    pcsetgp(&pv->gp, fn_root);
	}
	return tvn;
}

/*_____________________________ Create __________________________________*/
static void TTreeVFile_CreateParams(TWins_Create_struct *wcs) {
  TTreeVFile *self = wcs->wins;

	TTreeV_CreateParams(wcs);
	self->DoExpand = (TtvNodeEv)TTreeVFile_DoExpand;
	self->DoCollaps = (TtvNodeEv)TTreeVFile_DoCollaps;
	self->DoPaintNode = TTreeVFile_DoPaintNode;
}

TTreeVFile *TTreeVFile_Create(TWins *Parent) {
  TWins_Create_struct wcs;
	TWins_Create_struct_init(&wcs, sizeof(TTreeVFile), wtyp_TTreeVFile);
	wcs.Parent = Parent;
	wcs.CreateParams = TTreeVFile_CreateParams;
	TScrollWin_DefaultConf(&wcs);
	return wcs.wins;
}

TTreeVFile *TTreeVFile_CreateA(TWins *Parent, int Left, int Top, int Width, int Height, int Align) {
  TTreeVFile *self;
	self = TTreeVFile_Create(Parent);
	self->pw->top = Top;
	self->pw->left = Left;
	self->pw->width = Width;
	self->pw->height = Height;
	self->Align = alClient;
	if (Align) self->pw->Align = Align;
	Set_Parent((TWins*)self, Parent);
	return self;
}
