/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "appXbase.h"
#include "tfontwin.h"

typedef struct {
  char *org_name;
} Tprivate;

//-misc-fixed-bold-r-normal-*-13-120-75-75-c-70-iso10646-1
// 0    1     2    3 4      5 6  7   8  9  1 1  1        1
//                                         0 1  2        3

static bool getb_TFontLgs(TTreeVFont *self, char *pc, TFontLgs *fl) {
  char *pca;

	if (!FontLgsArrFill(fl, pc)) return False;

	pca = self->dpi89;
#ifdef _OS_WIN32
	if (pccmp(pca, "96-96")) {
	    int m = pctoint(fl->lgs[8].s);
	    return m > 0;
#else
	if (pccmp(pca, "75-75") || pccmp(pca, "100-100")) {
	    if ((fl->lgs[11].lg == 1) && (*fl->lgs[11].s == '0')) return False;
#endif
	} else if (pccmp(pca, "0-0")) {
	    if ((fl->lgs[11].lg != 1) || (*fl->lgs[11].s != '0')) return False;
	    if ((fl->lgs[10].lg = 1) && (*fl->lgs[10].s == 'c')) return False;
	}
	return True;
}


static TTreeVNode *get_node(TTreeVNode *node, int tag, char *s, int lg) {
	while (node && ((node->tag != tag) || 
			(node->lg != lg) || 
			!pcncmp(node->s, s, lg))) {
	    node = node->next;
	}
	return node;
}

static bool TTreeVFontNode__Destroy(TTreeVNode *self) {
  Tprivate *pv = self->private;
	if (pv) {
	    xfree(pv->org_name);
	    xfree(pv);
	}
	return TTreeVNode__Destroy(self);
}

static TTreeVNode *AddNode(TTreeVFont *tv, TTreeVNode *prev_child, char *s, int lg) {
  TTreeVNode *tvn;

	tvn = TTreeV_AddNextNode((TTreeV*)tv, prev_child, NULL);
	tvn->destroy = (TDESTROY)TTreeVFontNode__Destroy;
	tvn->s = pcndup(s, lg);
	tvn->lg = lg;
	return tvn;
}

static int QS_size_node(TTreeVNode *tvn1, TTreeVNode *tvn2, int fu) {
  int m1, m2, res;
  char *pc1, *pc2;
	m1 = pctoint(tvn1->s);
	m2 = pctoint(tvn2->s);
	res = m1 - m2;
	if (!res) {
	    pc1 = tvn1->s;
	    while (*pc1 && (*pc1 != '-')) pc1++;
	    pc2 = tvn2->s;
	    while (*pc2 && (*pc2 != '-')) pc2++;
	    if (*pc1 && *pc2) {
		pc1++; pc2++;
		m1 = pctoint(pc1);
		m2 = pctoint(pc2);
		res = m1 - m2;
	    }
	}
	return res;
}

static int QS_name_node(TTreeVNode *tvn1, TTreeVNode *tvn2, int fu) {
  int res;
	res = pccmp_i(tvn1->s, tvn2->s);
	if (!res) res = tvn1->tag - tvn2->tag;
	return res;
}

static void sort_node(TTreeVNode **pptvn, TCompareFunction cpf) {
  int count;
  TTreeVNode *tvn, *prev_child, **pptvn0;

	if (!pptvn || !*pptvn) return;
	count = 0;
	tvn = *pptvn;
	prev_child = tvn->prev_child;
	while (tvn) {count++; tvn = tvn->next;}
	if (count < 2) return;

	tvn = *pptvn;
	pptvn0 = xcalloc(sizeof(void*) * (count+1));
	for (int i=0;i<count;i++) {
	    pptvn0[i] = tvn;
	    tvn->prev_child = NULL;
	    tvn = tvn->next;
	}
	QuickSort_PL((char**)pptvn0, 0, count-1, 0, cpf);

	*pptvn = pptvn0[0];
	pptvn0[0]->prev_child = prev_child;
	tvn = NULL;
	for (int i=0;i<count;i++) {
	    pptvn0[i]->prev = tvn;
	    tvn = pptvn0[i];
	    tvn->next = pptvn0[i+1];
	}
	xfree(pptvn0);
}

void TreeVFont_List(TTreeVFont *self) {
  int L, lg;
  char **FontNameList, *name, *fn, *pc;
  TTreeVNode *node, *node1, *node2, *node3, *node4;
  TFontLgs fl;
  int tag0;
  char buf[64];

	TTreeV_Cls((TTreeV*)self);
	
//			-*-*-*-*-*-*-*-*-75-75-*-*-iso10646-1
	fn = pcadd_n(4, "-*-*-*-*-*-*-*-*-", self->dpi89, "-*-*-", self->Charset12);
	FontNameList = XListFonts(appXDat.dpy, fn, 99999, &L);
	xfree(fn);
	if (!FontNameList) return;

	for (int i=0;i<L;i++) {
	    name = FontNameList[i];

	    if (getb_TFontLgs(self, name, &fl)) {
		tag0 = 0x00;
		if ((fl.lgs[10].lg = 1) && (*fl.lgs[10].s == 'p')) tag0 = 0x10;

		node = get_node(self->Nodes, tag0, fl.lgs[0].s, fl.lgs[0].lg);
		if (!node) {
		    node = AddNode(self, NULL, fl.lgs[0].s, fl.lgs[0].lg);
		    node->tag = tag0;
		    if (tag0 & 0x10) {
			node->pic = get_fileIcon(stft_Font_A);
		    } else {
			node->pic = get_fileIcon(stft_Font_Fix);
		    }
		    node->st |= tvnSt_Expanded;
		}

		node1 = get_node(node->next_child, 1, fl.lgs[1].s, fl.lgs[1].lg);
		if (!node1) {
		    node1 = AddNode(self, node, fl.lgs[1].s, fl.lgs[1].lg);
		    node1->tag = 1;
		}

		node2 = get_node(node1->next_child, 2, fl.lgs[4].s, fl.lgs[4].lg);
		if (!node2) {
		    node2 = AddNode(self, node1, fl.lgs[4].s, fl.lgs[4].lg);
		    node2->tag = 2;
		}

		pc = buf;
		ppcnmove(&pc, fl.lgs[6].s, fl.lgs[6].lg);
		if ((tag0 & 0x10) == 0) {
		    *pc = ' '; pc++;
		    *pc = '('; pc++;
		    lg = fl.lgs[10].lg + fl.lgs[11].lg + 1;
		    ppcnmove(&pc, fl.lgs[10].s, lg);
		    *pc = ')'; pc++; *pc = 0;
		}
		lg = pclen(buf);
		node3 = get_node(node2->next_child, 3, buf, lg);
		if (!node3) {
		    node3 = AddNode(self, node2, buf, lg);
		    node3->tag = 3;
		}

		lg = fl.lgs[2].lg + fl.lgs[3].lg + 1;
		node4 = AddNode(self, node3, fl.lgs[2].s, lg);
		node4->tag = 4;

		Tprivate *pv = xcalloc(sizeof(Tprivate));
		pv->org_name = pcdup(name);
		node4->private = pv;

		switch (GetFont_Art(name)) {
		case FONT_MediumK: node4->Font = self->mediumK_font; break;
		case FONT_Bold:    node4->Font = self->bold_font; break;
		case FONT_BoldK:   node4->Font = self->boldK_font; break;
		}
	    }

	}
	XFreeFontNames(FontNameList);

	sort_node(&self->Nodes, (TCompareFunction)QS_name_node);

	node = self->Nodes;
	while (node) {
	    sort_node(&node->next_child, (TCompareFunction)QS_name_node);
	    node = node->next;
	}

	node = self->Nodes;
	while (node) {
	    node1 = node->next_child;
	    while (node1) {
		node2 = node1->next_child;
		while (node2) {
		    sort_node(&node2->next_child, (TCompareFunction)QS_size_node);
		    node2 = node2->next;
		}
		node1 = node1->next;
	    }
	    node = node->next;
	}
	Refresh((TWins*)self);
}


static void TTreeVFont_DoNodeClick(TTreeVFont *self, TTreeVNode *tvn, bool DblClick) {
  int size;
  char *pc, *tpc;
  const char *c0, *c1, *c2, *c32, *c4;
  TTreeVNode *tvn1, *tvn2, *tvn3, *tvn4;
  Tprivate *pv;
  char bufs[16];
  char buf[200];

	tvn1 = tvn2 = tvn3 = tvn4 = NULL;
	size = self->size6;

	if ((tvn->tag & 0x0f) == 4) {
	    pv = tvn->private;
	    if (pv && pv->org_name) {
		ppcchange(&self->selFontName, pv->org_name);
		goto end_ev;
	    }
	    tvn4 = tvn;
	    tvn = TTreeV_Get_prev_child((TTreeV*)self, tvn);
	}
	if ((tvn->tag & 0x0f) == 3) {
	    tvn3 = tvn;
	    tvn = TTreeV_Get_prev_child((TTreeV*)self, tvn);
	}
	if ((tvn->tag & 0x0f) == 2) {
	    tvn2 = tvn;
	    tvn = TTreeV_Get_prev_child((TTreeV*)self, tvn);
	}
	if ((tvn->tag & 0x0f) == 1) {
	    tvn1 = tvn;
	    tvn = TTreeV_Get_prev_child((TTreeV*)self, tvn);
	}

	c4 = "*-*";
	if (tvn4) c4 = tvn4->s;

	c32 = "*-*";
	if (tvn->tag & 0x10) c32 = "p-*";

	if (tvn3) {
	    size = pctoint(tvn3->s);
	    pc = tvn3->s;
	    while (*pc && (*pc != '(')) pc++;
	    if (*pc) {
		pc++;
		tpc = bufs;
		while (*pc && (*pc != ')')) {
		    *tpc = *pc; tpc++; pc++;
		}
		*tpc = 0;
		c32 = bufs;
	    }
	}

	c2 = "*";
	if (tvn2) c2 = tvn2->s;

	if (!tvn1) tvn1 = tvn->next_child;
	c1 = tvn1->s;

	c0 = "*";
	if ((tvn->tag & 0x0f) == 0) {
	    c0 = tvn->s;
	}
	snprintf(buf, sizeof(buf), "-%s-%s-%s-%s-*-%d-*-%s-%s-%s", 
		c0, c1, c4, c2, size, self->dpi89, c32, self->Charset12);

	ppcchange(&self->selFontName, buf);

end_ev:
	if (self->OnNodeClick) self->OnNodeClick((TTreeV*)self, tvn, DblClick);
}

void TTreeVFont_ExpandDefaultNode(TTreeVFont *self) {
  TTreeVNode *tvn0, *tvn1;
  TFontLgs fl;

	if (!FontLgsArrFill(&fl, self->selFontName)) return;
	tvn0 = self->Nodes;
loop_tvn:
	tvn0 = TTreeVNode_get_by_name_lg(tvn0, fl.lgs[0].s, fl.lgs[0].lg, True);
	if (tvn0) {
	    tvn1 = TTreeVNode_get_by_name_lg(tvn0->next_child, fl.lgs[1].s, fl.lgs[1].lg, True);
	    if (!tvn1) {
		tvn0 = tvn0->next;
		goto loop_tvn;
	    }
	    TTreeV_NodeExpand((TTreeV*)self, tvn0);
	    self->SelectNode = tvn1;
	}
}

void TTreeVFont_SetDefaultParam(TTreeVFont *self, const char *fontname) {
  TFontLgs fl;

	ppcchange_cmp(&self->selFontName, fontname);
#ifdef _OS_WIN32
	ppcchange_cmp(&self->dpi89, "96-96");
#else
	ppcchange_cmp(&self->dpi89, "75-75");
#endif
	ppcchange_cmp(&self->Charset12, "iso8859-1");
	self->size6 = 12;

	if (FontLgsArrFill(&fl, fontname)) {
	    if (fl.lgs[8].lg == 1) {
		if (*fl.lgs[8].s == '0') {
		    ppcchange(&self->dpi89, "0-0");
		} else if (*fl.lgs[11].s == '*') {
		    ppcchange(&self->dpi89, "*-*");
		}
#ifndef _OS_WIN32
	    } else if (pcncmp(fl.lgs[8].s, "100", 3)) {
		ppcchange(&self->dpi89, "100-100");
#endif
	    }
	    ppcchange(&self->Charset12, fl.lgs[12].s);
	    self->size6 = pctoint(fl.lgs[6].s);
	}
}

/*_____________________________ Destroy _________________________________*/
static bool TTreeVFont__Destroy(TTreeVFont *self) {
	xfree(self->selFontName);
	xfree(self->dpi89);
	xfree(self->Charset12);
	Tfree(self->mediumK_font);
	Tfree(self->bold_font);
	Tfree(self->boldK_font);
	return TTreeV__Destroy((TTreeV*)self);
}

/*_____________________________ Create __________________________________*/
static void TTreeVFont_CreateParams(TWins_Create_struct *wcs) {
  TTreeVFont *self = wcs->wins;
  const char *cpc;

	TTreeV_CreateParams(wcs);
	self->destroy = (TDESTROY)TTreeVFont__Destroy;
	self->DoNodeClick = (void*)TTreeVFont_DoNodeClick;

//	pc = self->vds.Font->name;
	cpc = FontX_GetFontName(self->vds.Font);
 printf("%s(%d): TODO MakeFontArt\n", FL);
	self->mediumK_font = appXDat.Default_Font;
	self->bold_font = appXDat.Default_Font;
	self->boldK_font = appXDat.Default_Font;
//	MakeFontArt(pc, &self->mediumK_font, FONT_MediumK);
//	MakeFontArt(pc, &self->bold_font,    FONT_Bold);
//	MakeFontArt(pc, &self->boldK_font,   FONT_BoldK);

	TTreeVFont_SetDefaultParam(self, cpc);
}

TTreeVFont *TTreeVFont_Create(TWins *Parent) {
  TWins_Create_struct wcs;
	TWins_Create_struct_init(&wcs, sizeof(TTreeVFont), wtyp_TTreeVFont);
	wcs.Parent = Parent;
	wcs.CreateParams = TTreeVFont_CreateParams;
	TScrollWin_DefaultConf(&wcs);
	return wcs.wins;
}

TTreeVFont *TTreeVFont_CreateA(TWins *Parent, int Left, int Top, int Width, int Height, int Align) {
  TTreeVFont *self;
	self = TTreeVFont_Create(Parent);
	self->pw->top = Top;
	self->pw->left = Left;
	self->pw->width = Width;
	self->pw->height = Height;
	self->Align = alClient;
	if (Align) self->pw->Align = Align;
	Set_Parent((TWins*)self, Parent);
	return self;
}
