/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#ifndef __ALD_CALC_H__
#define __ALD_CALC_H__    1

#include "appXbase.h"
#include "zsrc.h"
#include "tpixmap.h"
#include <math.h>

#if 1
 #ifdef __NO_LONG_DOUBLE_MATH
  #define TDouble double
 #else
  #define TDouble long double
 #endif
#else
 #define TDouble double
#endif

#define wtyp_ButtonCalc		wtyp_USER_Wins
#define wtyp_DisplayCalc	wtyp_USER_Wins + 1

#define KFK_MASK	0xf00
#define KFK_INPUT	0x100
#define KFK_BEF		0x200
#define KFK_MATH	0x300
#define KFK_MATH2OP	0x400
#define KFK_FUNKT	0x500

#define KFF_EXP		0x1000

#define KFB_INV		(1<<16)
#define KFB_RAD		(1<<17)
#define KFB_HEX		(1<<18)
#define KFB_HEX_ALT	(1<<19)

#define KF_NONE		-1

#define KF_MASK		0x0ff
#define KF_0		KFK_INPUT
#define KF_1		(KFK_INPUT + 1)
#define KF_2		(KFK_INPUT + 2)
#define KF_3		(KFK_INPUT + 3)
#define KF_4		(KFK_INPUT + 4)
#define KF_5		(KFK_INPUT + 5)
#define KF_6		(KFK_INPUT + 6)
#define KF_7		(KFK_INPUT + 7)
#define KF_8		(KFK_INPUT + 8)
#define KF_9		(KFK_INPUT + 9)
#define KF_A		(KFB_HEX + KFK_INPUT + 10)
#define KF_B		(KFB_HEX + KFK_INPUT + 11)
#define KF_C		(KFB_HEX + KFK_INPUT + 12)
#define KF_D		(KFB_HEX + KFK_INPUT + 13)
#define KF_E		(KFB_HEX + KFF_EXP + KFK_INPUT + 14)
#define KF_F		(KFB_HEX + KFK_INPUT + 15)
#define KF_COMMA	(KFK_INPUT + 16)

#define KF_CLR		KFK_BEF
#define KF_BACK		(KFK_BEF + 1)
#define KF_GETM		(KFK_BEF + 2)
#define KF_SETM		(KFK_BEF + 3)
#define KF_PLUSMIN	(KFK_BEF + 4)

#define KF_ADD		KFK_MATH2OP
#define KF_SUB		(KFK_MATH2OP + 1)
#define KF_MULT		(KFK_MATH2OP + 2)
#define KF_DIV		(KFK_MATH2OP + 3)
#define KF_XY		(KFK_MATH2OP + KFB_HEX_ALT + KFB_INV + 4)

#define KF_RESULT	KFK_MATH
#define KF_PARENL	(KFK_MATH + 5)
#define KF_PARENR	(KFK_MATH + 6)

#define KF_LN		(KFK_MATH + KFB_HEX_ALT + KFB_INV + 7)
#define KF_LOG		(KFK_MATH + KFB_HEX_ALT + KFB_INV + 8)
#define KF_SIN		(KFK_MATH + KFB_HEX_ALT + KFB_INV + 9)
#define KF_COS		(KFK_MATH + KFB_HEX_ALT + KFB_INV + 10)
#define KF_TAN		(KFK_MATH + KFB_HEX_ALT + KFB_INV + 11)

#define KF_KERW		(KFK_MATH + KFB_HEX_ALT + 12)
#define KF_X2		(KFK_MATH + KFB_INV + 13)
#define KF_SQRT		(KFK_MATH + KFB_INV + 14)

#define KF_INV		KFK_FUNKT
#define KF_RAD		(KFK_FUNKT + 1)
#define KF_HEX		(KFK_FUNKT + 2)



typedef struct {
  dword KF;
  int x, y, w, h;
  int yo, yo2;
  const byte *img_data;
  const byte *img_data_alt;
  const byte *img_data_hex;
} TKeyfk;

typedef struct {
  byte art;
  byte w, h;
  byte data[];
} Timg_data;

extern void fill_img(TImage *img, dword color, int offx, int offy, 
			const byte *data, int data_w, int data_h);

#define TCSelf TButtonCalc
typedef struct TButtonCalc {
  AlgTButtonPic
  bool OnOffSwitch;
  bool AltFunkt;
  bool bbnc2;
  bool bbnc3;
  int KF;
  dword KFB;
  int XK_sym, XK_sym2;
  const TKeyfk *Keyfk;
  TPixMap *pxm;
  TPic static_pic;
  int img_w;
  int img_h;
  const byte *img_byte;
  const Timg_data *img_data;
  int img_data_idx;
} TButtonCalc;
#undef TCSelf

extern TButtonCalc *TButtonCalc_CreateB(TWins *Parent, int left, int top, 
						int width, int height);
extern TButtonCalc *TButtonCalc_Create_Keyfk(TWins *Parent, int offx, int offy, 
						const TKeyfk *Keyfk);

//extern TButtonCalc *TButtonCalc_CreateS(TWins *Parent, int offx, int offy, const TKeyfk *Keyfk);
extern void TButtonCalc_Set_XK_sym(TButtonCalc *self, int KF);
extern void ButtonCalc_ImgFill(TButtonCalc *self);
extern TButtonCalc *ButtonCalc_KeyEvent(TWins *wins, TKeyEvent *KeyEv, bool Down);
extern void ButtonCalc_SetImgData(TButtonCalc *self, const Timg_data *img_data);
extern TButtonCalc *ButtonCalc_find_KF(TWins *wins, int KF);
extern void ButtonCalc_Set_KFB(TWins *wins, int KFB);

typedef struct {
  const Timg_data *img_data;
  int width;
  int height;
  const byte *data;
} TCharData;

#define OP_FLOAT 1
#define OP_INT64 (1<<1)
//#define OP_HEX (1<<2)

#define DPC_NEG		1
#define DPC_DOT		(1<<1)
#define DPC_EXP		(1<<2)
#define DPC_EXP_VZ	(1<<3)
typedef struct {
  dword   dpc_st;
  int64_t iOP;
  TDouble  fOP;
} TMOP;

#define TDisplayCalc_MAXGLYPHS 256	

#define TCSelf TDisplayCalc
typedef struct TDisplayCalc {
  AlgTWins
  TMOP op;
  int num, nk, exp_num;
  dword dpc_st;
  char *pc;
  int img_data_idx;
  int GlyphH, GlyphXo;
//  bool neg, com;
  bool mod_hex, mod_exp, skip_sc, bnc_3;
  TPixMap *pxm;
  TCharData cd[TDisplayCalc_MAXGLYPHS];
} TDisplayCalc;
#undef TCSelf

extern TDisplayCalc *TDisplayCalc_Create(TWins *Parent, 
				int left, int top, int width, int height);

extern void DisplayCalc_SetCharData(TDisplayCalc *self, int CharIdx, 
				const Timg_data *img_data);
extern void DisplayCalc_SetImgDataIdx(TDisplayCalc *self, int idx);

extern void DisplayCalc_OPx_Display(TDisplayCalc *self, dword flag);
extern void DisplayCalc_AddChar(TDisplayCalc *self, int c);

extern bool DisplayCalc_Get_dpc_st(TDisplayCalc *self, const char *s, 
			dword *pdpc_st, int *pnum, int *pnk, int *pexp_num);

typedef struct TOPStack {
  struct TOPStack *next;
  TMOP op;
  int KF;
  int paren_count;
} TOPStack;

#endif /* __ALD_CALC_H__ */
