/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "aldcalc.h"


static void TDisplayCalc_DoPaint(TDisplayCalc *self) {

	if (!self->pxm) return;

	TPixMap_PaintToWin(self->pxm, self->WinID, 1, 1);
	TWins_DoPaint((TWins*)self);
}

static bool pxm_set_size(TDisplayCalc *self) {
  int w, h;

	w = self->width - 2;
	h = self->height - 2;
	if (self->pxm && (self->pxm->width  == w) &&
			 (self->pxm->height == h)) return false;
	Tfree(self->pxm);
	self->pxm = TPixMap_Create_whLD(w, h, 0);
	TPixMap_img_malloc(self->pxm);
	TPixMap_ClsRGB(self->pxm, self->Color);
	return true;
}

static int get_char_width(TDisplayCalc *self) {
  int lg, result;
  const byte *cpc;

	lg = self->vds.lg;
	cpc = (const byte*)self->vds.s;
	if (!cpc || (lg <= 0)) return 0;
	result = 0;
	while (lg) {
	    result += self->cd[*cpc].width;
	    cpc++;
	    lg--;
	}
	return result;
}

static void SetPxmChar(TDisplayCalc *self) {
  int lg, cw, xo, yo, yt, ym, num, nk, m;
  const byte *cpc;
  TCharData *cd;
  bool cisnum, isnk, mk;

// printf("%s(%d): %d %d\n", FL, self->num, self->nk);
	TPixMap_ClsRGB(self->pxm, self->Color);

	cw = get_char_width(self);
	if (cw <= 0) return;
	num = self->num;
	nk  = self->nk;
	xo = self->pxm->width - cw - self->GlyphXo;
	yo = ((self->height - self->GlyphH) / 2) - 2;
	if (yo < 0) yo = 0;
	ym = self->GlyphH + yo + 4;

	lg = self->vds.lg;
	cpc = (const byte*)self->vds.s;
	isnk = false;
	while (lg) {
	    cisnum = ((*cpc >= '0')&&(*cpc <= '9')) || 
		    (self->mod_hex &&(*cpc >= 'A')&&(*cpc <= 'F'));
	    if (*cpc == '.') isnk = true;

	    if (cisnum && ((num > 0) || (isnk && (nk > 0)))) {
		if (self->mod_hex) {
		    mk = !(num & 0x01);
		    yt = ym - 3;
		    if (num & 0x02) yt = ym - 1;
		} else {
		    yt = ym - 3;
		    m = num / 3;
		    mk = (m * 3) == num;
		}
		if (mk) {
		    TImage_DrawLineRGB((TImage*)self->pxm, 0xc0c0c0, xo, yt, xo, ym);
		}
		if (isnk) nk--;
		num--;
	    }

	    cd = &self->cd[*cpc];
	    fill_img((TImage*)self->pxm, 0x000000, xo, yo, 
			cd->data, cd->width, cd->height);

	    xo += cd->width;
	    cpc++;
	    lg--;
	}
}

static void TDisplayCalc_SetCaption(TDisplayCalc *self, const char *s) {
  char *pc;
  bool skip_sc;

	skip_sc = self->skip_sc;
	self->skip_sc = false;
	if (pccmp(self->vds.s, s)) return;
	TWins_SetCaption((TWins*)self, s);

	self->dpc_st = 0;
	self->num = 0;
	self->nk = 0;
	self->exp_num = 0;
	pc = self->vds.s;
	if (!pc) {
	    self->op.dpc_st = 0;
	    self->op.iOP = 0;
	    self->op.fOP = 0;
	    SetPxmChar(self);
	    return;
	}

	if (self->mod_hex) {
	    while (*pc) {self->num++; pc++;}
	    if (!skip_sc) {
		self->op.iOP = pctohex64(self->vds.s);
		self->op.fOP = self->op.iOP;
	    }

	} else {
	    DisplayCalc_Get_dpc_st(self, pc, &self->dpc_st,
			&self->num, &self->nk, &self->exp_num);
// printf("%s(%d): %d %d\n", FL, self->num, self->nk);
	    if (!skip_sc) {
		self->op.fOP = pctodouble(self->vds.s);
		if (self->dpc_st & DPC_EXP) {
		    if (self->op.fOP > INT64_MAX) {
			self->op.iOP = 0x7fffffffffffffff;
		    } else if (self->op.fOP < INT64_MIN) {
			self->op.iOP = 0x8000000000000000;
		    } else {
			self->op.iOP = self->op.fOP;
		    }
		} else {
		    self->op.iOP = pctoint64(self->vds.s);
		}
	    }
	}
	self->op.dpc_st = self->dpc_st;
	SetPxmChar(self);
}

static void TDisplayCalc_OnResize(TDisplayCalc *self) {
	if (pxm_set_size(self)) {
	    SetPxmChar(self);
	}
}
/*____________________________ Public ___________________________________*/
void DisplayCalc_SetCharData(TDisplayCalc *self, int CharIdx, const Timg_data *img_data) {

	if (!self || !img_data) return;
	if (img_data && (img_data->art == 1)) {
	    if ((CharIdx > 0) && (CharIdx < TDisplayCalc_MAXGLYPHS)) {
		if (self->GlyphH < img_data->h) self->GlyphH = img_data->h;
		self->cd[CharIdx].img_data = img_data;
		self->cd[CharIdx].width  = img_data->w;
		self->cd[CharIdx].height = img_data->h;
		self->cd[CharIdx].data   = img_data->data;
	    }
	}
}

void DisplayCalc_SetImgDataIdx(TDisplayCalc *self, int idx) {
  int i, n;
  const Timg_data *imgd;
  const byte *data;

	if (!self) return;
	if (idx < 0) idx = 0;
	if (self->img_data_idx == idx) return;
	self->GlyphH = 0;
	self->img_data_idx = idx;
	for (i=0;i<TDisplayCalc_MAXGLYPHS;i++) {
	    imgd = self->cd[i].img_data;
	    if (imgd) {
		n = self->img_data_idx;
		while (n > 0) {
		    data = imgd->data + (imgd->w * imgd->h);
		    if (*data != 1) break;
		    imgd = (const Timg_data*)(data);
		    n--;
		}
		if (self->GlyphH < imgd->h) self->GlyphH = imgd->h;
		self->cd[i].width  = imgd->w;
		self->cd[i].height = imgd->h;
		self->cd[i].data   = imgd->data;
	    }
	}
	SetPxmChar(self);
	Refresh((TWins*)self);
}

void DisplayCalc_OPx_Display(TDisplayCalc *self, dword flag) {
  int decipoint;
  char *pc;
  char buf[32];

	buf[0] = '0';
	buf[1] = 0;
	if (flag & OP_FLOAT) {
	    if (self->op.fOP > INT64_MAX) {
		self->op.iOP = 0x7fffffffffffffff;
	    } else if (self->op.fOP < INT64_MIN) {
		self->op.iOP = 0x8000000000000000;
	    } else {
		self->op.iOP = self->op.fOP;
	    }
	    if (self->mod_hex) {
		hex64topc(buf, self->op.iOP, 0);
	    } else {
		if (isinf(self->op.fOP)) {
		    pc = buf;
		    if (self->op.fOP < 0) {*pc = '-'; pc++;}
		    pcmove(pc, "inf");
#ifdef NAN
		} else if (isnan(self->op.fOP)) {
		    pcmove(buf, "nan");
#endif
		} else {
		    decipoint = DECIPOINT_AUTO;
		    if (self->mod_exp) decipoint = DECIPOINT_AUTO|DECIPOINT_EXP;
		    doubletopc(buf, self->op.fOP, decipoint);
		}
	    }
	} else if (flag & OP_INT64) {
	    self->op.fOP = self->op.iOP;
	    if (self->mod_hex) {
		hex64topc(buf, self->op.iOP, 0);
	    } else {
		int64topc(buf, self->op.iOP);
	    }
	}
	self->skip_sc = true;
	SetCaption((TWins*)self, buf);
}

void DisplayCalc_AddChar(TDisplayCalc *self, int c) {
  int lg;
  const char *cpc;
  char buf[32];

	cpc = GetCaption_pc((TWins*)self);
	lg = pclen(cpc);
	if (lg && (lg < 30)) {
	    pcnmove(buf, cpc, lg);
	    if ((c == '.') || (c == ',')) {
		buf[lg] = c;
		buf[lg + 1] = 0;

	    } else if ((buf[0] == '0') && !buf[1]) {
		buf[0] = c;

	    } else if ((buf[0] == '-') && (buf[1] == '0') && !buf[2]) {
		buf[1] = c;

	    } else {
		buf[lg] = c;
		buf[lg + 1] = 0;
	    }

	} else {
	    if ((c == '.') || (c == ',')) {
		buf[0] = '0';
		buf[1] = c;
		buf[2] = 0;
	    } else {
		buf[0] = c;
		buf[1] = 0;
	    }
	}
	SetCaption((TWins*)self, buf);
}


bool DisplayCalc_Get_dpc_st(TDisplayCalc *self, const char *s, dword *pdpc_st,
			int *pnum, int *pnk, int *pexp_num) 
{
  int num, nk, exp_num;
  dword dpc_st;
  const char *cpc;

	dpc_st = 0;
	num = 0;
	nk = 0;
	exp_num = 0;
	cpc = s;
	while (*cpc) {
	    if (*cpc == '-') {
		if (dpc_st & DPC_EXP) {
		    dpc_st |= DPC_EXP_VZ;
		} else {
		    dpc_st |= DPC_NEG;
		}

	    } else if (*cpc == '+') {
		if (dpc_st & DPC_EXP) {
		    dpc_st |= DPC_EXP_VZ;
		}

	    } else if ((*cpc == '.')||(*cpc == ',')) {
		dpc_st |= DPC_DOT;

	    } else if ((*cpc == 'E')||(*cpc == 'e')) {
		if ((num + nk) > 0) {
		    dpc_st |= DPC_EXP;
		} else break;

	    } else {
		if (dpc_st & DPC_EXP) {
		    exp_num++;
		} else {
		    if (dpc_st & DPC_DOT) {
			nk++;
		    } else {
			num++;
		    }
		}
	    }
	    cpc++;
	}
	if (pdpc_st)  *pdpc_st = dpc_st;
	if (pnum)     *pnum = num;
	if (pnk)      *pnk = nk;
	if (pexp_num) *pexp_num = exp_num;
	return (num + nk) > 0;
}
/*____________________________ Destroy __________________________________*/
static bool TDisplayCalc__Destroy(TDisplayCalc *self) {
	ppTfree(&self->pxm);
	return TWins__Destroy((TWins*)self);
}

/*____________________________ Create ___________________________________*/
static void TDisplayCalc_CreateParams(TWins_Create_struct *wcs) {
  TDisplayCalc *self;
  TWRect *wrc;

	self = wcs->wins;
	wrc = wcs->SubP;
	self->left   = wrc->left;
	self->top    = wrc->top;
	self->width  = wrc->width;
	self->height = wrc->height;
	self->destroy = (TDESTROY)TDisplayCalc__Destroy;
	self->DoPaint = TDisplayCalc_DoPaint;
	self->SetCaption = TDisplayCalc_SetCaption;
	self->BorderFu = mBorder|mBorderL;
	self->Color = 0xffffff;
}

TDisplayCalc *TDisplayCalc_Create(TWins *Parent, 
			int left, int top, int width, int height)
{
  TDisplayCalc *self;
  TWins_Create_struct wcs;
  TWRect wrc;

	wrc.left   = left;
	wrc.top    = top;
	wrc.width  = width;
	wrc.height = height;

	TWins_Create_struct_init(&wcs, sizeof(TDisplayCalc), wtyp_DisplayCalc);
	wcs.Parent = Parent;
	wcs.CreateParams = TDisplayCalc_CreateParams;
	wcs.SubP = &wrc;
	TWins_DefaultConf(&wcs);
	self = wcs.wins;
	pxm_set_size(self);
	
	Set_Parent((TWins*)self, Parent);
	self->OnResize = TDisplayCalc_OnResize;
	self->GlyphXo = 2;
	return self;
}
