/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "aldcalc.h"
#include "tpixmap.h"

static void fill_img_16(TImage *img, dword color, int offx, int offy, 
			const byte *data, int data_w, int data_h) 
{
  int x, y;
  dword vCol, vR, vG, vB;
  dword A, A2, R, G, B;
  const byte *sb;
  byte *db0;
  word *dw;

	vR = (color >> 8) & 0xf800;
	vG = (color >> 5) & 0x07e0;
	vB = (color >> 3) & 0x001f;
	vCol = vR | vG | vB;

	db0 = img->data + (offx * 2) + (offy * img->bytes_per_line);
	sb = data;
	for (y=data_h;y>0;y--) {
	    dw = (word*)db0;
	    for (x=data_w;x>0;x--) {
		A = *sb;
		if (A >= 0xf8) {
		    *dw = vCol;
		} else if (A >= 0x08) {
		    A2 = 0x100 - A;
		    R = ((vR * A) + ((*dw & 0xf800) * A2)) & 0xf80000;
		    G = ((vG * A) + ((*dw & 0x07e0) * A2)) & 0x07e000;
		    B = ((vB * A) + ((*dw & 0x001f) * A2)) & 0x001f00;
		    *dw = (R | G | B) >> 8;
		}
		sb++; dw++;
	    }
	    db0 += img->bytes_per_line;
	}
}

static void fill_img_32(TImage *img, dword color, int offx, int offy, 
			const byte *data, int data_w, int data_h) 
{
  int x, y;
  dword vCol, vR, vG, vB;
  dword A, A2, R, G, B;
  const byte *sb;
  byte *db0;
  dword *dw;

	vR = color & 0xff0000;
	vG = color & 0x00ff00;
	vB = color & 0x0000ff;
	vCol = vR | vG | vB;

	db0 = img->data + (offx * 4) + (offy * img->bytes_per_line);
	sb = data;
	for (y=data_h;y>0;y--) {
	    dw = (dword*)db0;
	    for (x=data_w;x>0;x--) {
		A = *sb;
		if (A >= 0xf8) {
		    *dw = vCol;
		} else if (A >= 0x08) {
		    A2 = 0x100 - A;
		    R = ((vR * A) + ((*dw & 0xff0000) * A2)) & 0xff000000;
		    G = ((vG * A) + ((*dw & 0x00ff00) * A2)) & 0x00ff0000;
		    B = ((vB * A) + ((*dw & 0x0000ff) * A2)) & 0x0000ff00;
		    *dw = (R | G | B) >> 8;
		}
		sb++; dw++;
	    }
	    db0 += img->bytes_per_line;
	}
}

void fill_img(TImage *img, dword color, int offx, int offy, 
			const byte *data, int data_w, int data_h)
{
	if (offx < 0) offx = 0;
	if (offy < 0) offy = 0;
	if ((offx + data_w) > img->width) return;
	if ((offy + data_h) > img->height) return;


	if (appXDat.RootW_depth == 16) {
	    fill_img_16(img, color, offx, offy, data, data_w, data_h);
	} else {
	    fill_img_32(img, color, offx, offy, data, data_w, data_h);
	}
}
