/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"
#include "image.h"
#include "font_FT.h"

#define APPNAME "genkimg"

#define REDUCED_RC	1

/*
---------------------------------------------------------------------------
-k format = 1

0x01, width, height
---------------------------------------------------------------------------
-k format = 4

  7   6   5   4   3   2   1   0   7   6   5   4   3   2   1   0
+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
| 1 |   height          |    off y          |   width           |
+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+
5bit height    + 1  (1..32)
5bit offset y       (0..31)
5bit width     + 1  (1..32)
*/


/* ------------------------------------------------------------------- */
static const char *Help_Msg = {
  "\n"
  "Usage: "APPNAME" -f Fontname -d Char\n"
  "\n"
  "  -c Caption           default Data\n"
  "  -d Data\n"
  "  -f Fontname\n"
  "  -n CName             default \"img_data\"\n"
  "  -s Fontsize          default 16\n"
  "  -C t,b,l,r           clip top,bottom,left,right\n"
  "  -O t,l               min. offset top,left\n"
  "  --bit                bit mask\n"
  "  --asm                \n"
  "  --fix                no reduced rc\n"
  "  -k format            default 1\n"
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: "APPNAME" version "ALD_VERSION"  (build %d  %s)\n"
};

const int build_v = ALD_BUILD_V;
const char *build_date = __DATE__;

int main (int argc, char **argv) {
  int i, x, y;
  int font_height[4], font_idx;
  int left, right, top, bottom, cformat, mask, k, km;
  int width, height, ascent, fuEOL;
  int clip_top, clip_bottom, clip_left, clip_right, off_top, off_left;
  TImage *img;
  char *val, *FontName, *CName, *Caption;
//  const char *cpc;
  byte *sb0, *sb;
  TFont_FT *font;
  char *Text;
  bool help, err, clip, bit_mode, asm_mode, reduced_rc;

	help = false;
	err  = false;
	clip = false;
	bit_mode = false;
	asm_mode = false;
	reduced_rc = true;
	CName = NULL;
	Text = NULL;
	Caption = NULL;
	FontName = NULL;
	clip_top = 0;
	clip_bottom = 0;
	clip_left = 0;
	clip_right = 0;
	off_top = 0;
	off_left = 0;
	cformat = 1;
	font_height[0] = 16;
	font_height[1] = 0;
	font_height[2] = 0;
	font_height[3] = 0;
	font_idx = 0;
	i = 1;
	while (!err && (i < argc)) {
	    val = argv[i];
	    if (*val == '-') {
		val++;
		switch (*val) {
		case 'C':
		    clip = true;
		    err = cmd_get_int_pp(argc, argv, &val, &i, &clip_top);
		    if (!err && *val) err = cmd_get_int_pp(argc, argv, &val, &i, &clip_bottom);
		    if (!err && *val) err = cmd_get_int_pp(argc, argv, &val, &i, &clip_left);
		    if (!err && *val) err = cmd_get_int_pp(argc, argv, &val, &i, &clip_right);
		    break;
		case 'O':
		    err = cmd_get_int_pp(argc, argv, &val, &i, &off_top);
		    if (!err && *val) err = cmd_get_int_pp(argc, argv, &val, &i, &off_left);
		    break;
		case 'h':
		    help = true;
		    break;
		case 'c':
		    err = cmd_get_pc(argc, argv, val, &i, &Caption);
		    break;
		case 'd':
		    err = cmd_get_pc(argc, argv, val, &i, &Text);
		    break;
		case 'f':
		    err = cmd_get_pc(argc, argv, val, &i, &FontName);
		    break;
		case 'k':
		    err = cmd_get_int_pp(argc, argv, &val, &i, &cformat);
		    break;
		case 'n':
		    err = cmd_get_pc(argc, argv, val, &i, &CName);
		    break;
		case 's':
//		    err = cmd_get_int(argc, argv, val, &i, &font_height);
		    err = cmd_get_int_pp(argc, argv, &val, &i, &font_height[0]);
		    if (!err && *val) err = cmd_get_int_pp(argc, argv, &val, &i, &font_height[1]);
		    if (!err && *val) err = cmd_get_int_pp(argc, argv, &val, &i, &font_height[2]);
		    if (!err && *val) err = cmd_get_int_pp(argc, argv, &val, &i, &font_height[3]);
		    break;
		case '-':
		    val++;
		    if (pcncmp(val, "help", 4)) {
			help = true;
		    } else if (pcncmp(val, "asm", 3)) {
			asm_mode = true;
		    } else if (pcncmp(val, "bit", 3)) {
			bit_mode = true;
		    } else if (pcncmp(val, "fix", 3)) {
			reduced_rc = false;
		    } else err = true;
		    break;
		default: 
		    err = True;
		}
	    } else {
		Text = val;
	    }
	    i++;
	}

//	err = err || !InFile;
	if (help || err || !Text || !FontName) {
	    printf(Help_Msg, build_v, build_date);
	    exit(EXIT_FAILURE);
	}
	if (!Caption) Caption = Text;

loop:
	font = TFont_FT_Create(FontName, font_height[font_idx], 0);
	if (font_idx && !font) goto Ende;
	if (!font) exit(EXIT_FAILURE);

	fuEOL = 0;
	Chk_fio_UTF8(Text, pclen(Text), &fuEOL);
	if ((fuEOL & (fio_has_UTF8_char|fio_err_UTF8_char)) == fio_has_UTF8_char) {
	    Font_SetUTF8((TFont*)font, true);
	}

	Font_GetMetrics((TFont*)font, &height, &ascent);
	width = Font_GetWidth((TFont*)font, Text, pclen(Text));
	if (font_idx && (width <= 0)) goto Ende;
	if (width <= 0) exit(EXIT_FAILURE);

// printf("%s(%d): %d %d %d\n", FL, width, height, ascent);

	img = TImage_Create_wht(0, 0, width, height, pxm_4b_24bpp);
//	TImage_SetMem(img);
//	TImage_ClsRGB(img, 0x010203);
	Font_SetColor((TFont*)font, 0xffffff);
	Font_FT_ToImage(font, img, 0, ascent, Text, pclen(Text));

#if REDUCED_RC
	if (reduced_rc) {
	    left = width;
	    right = 0;
	    top = height;
	    bottom = 0;
	    sb0 = img->data;
	    for (y=0;y<height;y++) {
		sb = sb0;
		for (x=0;x<width;x++) {
		    if (*sb) {
			if (left > x)   left = x;
			if (right < x)  right = x;
			if (top > y)    top = y;
			if (bottom < y) bottom = y;
		    }
		    sb += 4;
		}
		sb0 += img->bytes_per_line;
	    }
	} else {
	    left = 0;
	    right = 0;
	    top = 0;
	    bottom = 0;
	}
	if (top < off_top) top = off_top;
	if (left < off_left) left = off_left;
#else
	left = 0;
	top = 0;
#endif /* REDUCED_RC */

//	cpc = CName;
//	if (!cpc) cpc = "img_data";

	if ((left > right) || (top > bottom)) {
	    if (font_idx) goto Ende;
	    printf("# error Unsupported char! s%d_%s\n", font_height[font_idx], Caption);
	    exit(EXIT_FAILURE);
	}

	if (clip) {
	    left = clip_left;
	    width -= (clip_left + clip_right);
	    top = clip_top;
	    height -= (clip_top + clip_bottom);
	} else {
#if REDUCED_RC
	    if (reduced_rc) {
		width  = 1 + right - left;
		height = 1 + bottom - top;
	    }
#endif /* REDUCED_RC */
	}

	if ((width <= 0) || (height <= 0)) {
	    if (font_idx) goto Ende;
	    printf("# error char s%d_%s w=%d h=%d\n", font_height[font_idx], Caption, width, height);
	    exit(EXIT_FAILURE);
	}


	if (!font_idx) {
	    if (CName) {
		if (asm_mode) {
		    printf("%s_%s:", CName, Caption);
		} else {
		    printf("static const byte %s_%s[] = {", CName, Caption);
		}
	    } else {
		if (asm_mode) {
		    printf("img_data_s%d_%s:", font_height[font_idx], Caption);
		} else {
		    printf("static const byte img_data_s%d_%s[] = {", font_height[font_idx], Caption);
		}
	    }
	}

	if (cformat == 4) {
	    printf("  /* W=%d H=%d Yo=%d */\n", width, height, top - off_top);
	} else {
	    printf("\n");
	}

	if (bit_mode) {
/*
	    if (asm_mode) {
		printf(" .byte ");
	    } else {
		printf("  ");
	    }
*/
	    if (cformat == 1) {
		if (asm_mode) {
		    printf(" .byte 0x01,0x%02x,0x%02x,", width, height);
		} else {
		    printf("  0x01,0x%02x,0x%02x,\n", width, height);
		}
	    } else if (cformat == 4) {
		k = 0x8000 + (((height - 1) & 0x1f) << 10) + 
		    (((top - off_top) & 0x1f) << 5) + ((width - 1) & 0x1f);
// printf("%d %d %d\n", height, top - off_top, width);
		if (asm_mode) {
		    printf(" .byte 0x%02x,0x%02x,", k & 0xff, k >> 8);
		} else {
		    printf("  0x%02x,0x%02x,", k >> 8, k & 0xff);
		}

	    }

	    sb0 = img->data + (left * 4) + (img->bytes_per_line * top);
	    for (y=height;y>0;y--) {
		sb = sb0;
		mask = 0;
		k = 0;
		km = 1;
		for (x=width;x>0;x--) {
		    if (*sb & 0x80) {
			mask |= km;
		    }
		    km <<= 1;

		    k++;
		    if ((k & 0x07) == 0) {
		        if ((x == 1) && (y == 1)) {
			    printf("0x%02x", mask);
			} else {
			    printf("0x%02x,", mask);
			}
			mask = 0;
			km = 1;
		    }
		    sb += 4;
		}

		if ((k & 0x07) != 0) {
		    if (y == 1) {
			printf("0x%02x", mask);
		    } else {
			printf("0x%02x,", mask);
		    }
		}

//		printf("\n");
		sb0 += img->bytes_per_line;
	    }
	    if (asm_mode) {
		printf("\n");
	    } else {
		printf("};\n");
	    }

	} else {

	    if (cformat == 1) {
		if (asm_mode) {
		    printf(" .byte 0x%02x,0x%02x,0x%02x\n", 1, width, height);
		} else {
		    printf("  0x%02x,0x%02x,0x%02x,\n", 1, width, height);
		}
	    }

	    sb0 = img->data + (left * 4) + (img->bytes_per_line * top);
	    for (y=height;y>0;y--) {
		if (asm_mode) {
		    printf(" .byte ");
		} else {
		    printf("  ");
		}
		sb = sb0;
		for (x=width;x>0;x--) {
		    if (asm_mode && (x == 1)) {
			printf("0x%02x", *sb);
		    } else {
			printf("0x%02x,", *sb);
		    }
		    sb += 4;
		}
		printf("\n");
		sb0 += img->bytes_per_line;
	    }
	    ppTfree(&img);
	    ppTfree(&font);
	    if (font_idx < 3) {
		font_idx++;
		if (font_height[font_idx] > 0) goto loop;
	    }
Ende:
	    if (asm_mode) {
		printf(" .byte 0x00\n");
	    } else {
		printf("  0x00};\n");
	    }
	}

	return 0;
}
