/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under LGPL v2.1, see file COPYING.LIB in this tarball
*/

#include "aldcalc.h"

static TButtonCalc *down_button;

static bool ButtonCalc_Set_img_data(TButtonCalc *self, int w, int h, const byte *data) {
  TPixMap *pxm;

	if (self->img_byte == data) return false;
	ppTfree(&self->pxm);
	self->pic = NULL;
	self->dispic = NULL;
	self->img_w = 0;
	self->img_h = 0;
	self->img_byte = NULL;
	if (!data || (w <= 0) || (h <= 0)) return false;
	self->img_w = w;
	self->img_h = h;
	self->img_byte = data;

	pxm = TPixMap_Create_whLD(w + 4, h + 4, 0);
	TPixMap_img_malloc(pxm);
	self->pxm = pxm;
	return true;
}

static TButtonCalc *find_ButtonCalc_Key(TWins *wins, int XK) {
  TButtonCalc *result;

	result = NULL;
	while (wins) {
	    if (wins->next_child) result = find_ButtonCalc_Key(wins->next_child, XK);
	    if (result) return result;
	    if ((wins->wtyp == wtyp_ButtonCalc) &&
		((((TButtonCalc*)wins)->XK_sym  == XK) || 
		 (((TButtonCalc*)wins)->XK_sym2 == XK))) return (TButtonCalc*)wins;
	    wins = wins->next;
	}
	return NULL;
}

static TButtonCalc *find_ButtonCalc_KF(TWins *wins, int KF) {
  TButtonCalc *result;

	result = NULL;
	while (wins) {
	    if (wins->next_child) result = find_ButtonCalc_KF(wins->next_child, KF);
	    if (result) return result;
	    if ((wins->wtyp == wtyp_ButtonCalc) &&
		(((TButtonCalc*)wins)->KF == KF)) return (TButtonCalc*)wins;
	    wins = wins->next;
	}
	return NULL;
}

static void SetUp_ButtonCalc_KFB(TWins *wins, int KFB) {
  TButtonCalc *bc;
  const Timg_data *img_data;
  dword kmask, k;
  bool mk;

	while (wins) {
	    if (wins->next_child) SetUp_ButtonCalc_KFB(wins->next_child, KFB);
	    if (wins->wtyp == wtyp_ButtonCalc) {
		bc = (TButtonCalc*)wins;
		kmask = bc->KF & KFB_INV;
		if (bc->KF & KFB_HEX_ALT) kmask |= KFB_HEX;
		k = KFB & kmask;
		if (kmask && (bc->KFB != k)) {
		    bc->KFB = k;
		    if (k & KFB_HEX) {
			img_data = (const Timg_data*)bc->Keyfk->img_data_hex;
		    } else if (k & KFB_INV) {
			img_data = (const Timg_data*)bc->Keyfk->img_data_alt;
		    } else {
			img_data = (const Timg_data*)bc->Keyfk->img_data;
		    }
		    ButtonCalc_SetImgData(bc, img_data);
		    ButtonCalc_ImgFill(bc);
		    RePaint(wins);
		}
		if (bc->KF & KFB_HEX) {
		    mk = ((KFB & KFB_HEX) != 0) || (bc->KF & KFF_EXP);
		    if (bc->Enabled != mk) {
			bc->Enabled = mk;
			ButtonCalc_ImgFill(bc);
			Refresh(wins);
		    }
		}
	    }
	    wins = wins->next;
	}
}

/*____________________________ ev _______________________________________*/
static void TButtonCalc_DoPaint(TButtonCalc *self) {
	self->pic_offX = 0;
	self->pic_offY = 0;
	if (self->Keyfk) {

	    if (self->KFB & KFB_HEX) {
		self->pic_offY = self->Keyfk->yo;
	    } else if (self->KFB & KFB_INV) {
		self->pic_offY = self->Keyfk->yo2;
	    } else {
		self->pic_offY = self->Keyfk->yo;
	    }
//	    self->pic_offY = self->Keyfk->yo;
//	    if (self->AltFunkt) self->pic_offY = self->Keyfk->yo2;
	}
	if (self->BorderFu & mBorderL) {
	    self->pic_offX++;
	    self->pic_offY++;
	}
	TButtonPic_DoPaint((TButtonPic*)self);
}

static void TButtonCalc_DoMouseDown(TButtonCalc *self, XButtonEvent *ev) {
	if (self->OnOffSwitch) {
	    if ((ev->button == 1) && TestWinIsEnabled((TWins*)self)) {
		if (self->BorderFu & mBorderL) {
		    self->BorderFu &= ~mBorderL;
		} else {
		    self->BorderFu |= mBorderL;
		}
		TWins_DoClick((TWins*)self, 0, false);
		Refresh((TWins*)self);
	    }
	    TWins_DoMouseDown((TWins*)self, ev);

	} else {
	    TButton_DoMouseDown((TButton*)self, ev);
	}
}

static void TButtonCalc_DoMouseUp(TButtonCalc *self, XButtonEvent *ev) {
	if (self->OnOffSwitch) {
	    self->MouseClickStop = true;
	    TWins_DoMouseUp((TWins*)self, ev);
	} else {
	    if (!self->pic) self->MouseClickStop = true;
	    TButton_DoMouseUp((TButton*)self, ev);
	}
}

static bool ButtonCalc_UpdateImgDataIdx(TButtonCalc *self) {
  const Timg_data *imgd1;
  int idx;
  const byte *data;

	if (!self->img_data) return false;
	if (self->img_data->art == 1) {
	    imgd1 = self->img_data;
	    idx = self->img_data_idx;
	    while (idx > 0) {
		data = imgd1->data + (imgd1->w * imgd1->h);
		if (*data != 1) break;
		imgd1 = (const Timg_data*)(data);
		idx--;
	    }
	    return ButtonCalc_Set_img_data(self, imgd1->w, imgd1->h, imgd1->data);
	}
	ButtonCalc_Set_img_data(self, 0, 0, NULL);
	return false;
}

static int TButtonCalc_OnScale(TButtonCalc *self, float fx, float fy) {
	self->img_data_idx = 0;
	if ((fx > 1.2) && (fy > 1.18)) {
	    self->img_data_idx = 1;
	}
	if ((fx > 1.4) && (fy > 1.35)) {
	    self->img_data_idx = 2;
	}
	if ((fx > 1.8) && (fy > 1.7)) {
	    self->img_data_idx = 3;
	}
	if (self->img_data) {
	    if (ButtonCalc_UpdateImgDataIdx(self)) {
		ButtonCalc_ImgFill(self);
		Refresh((TWins*)self);
	    }
	}
	return 0;
}

/*____________________________ Public ___________________________________*/
void ButtonCalc_ImgFill(TButtonCalc *self) {
	if (!self->pxm || !self->img_byte) return;

	TPixMap_ClsRGB(self->pxm, self->Color);

	if (((self->img_w + 4) > self->pxm->width) ||
	    ((self->img_h + 4) > self->pxm->height))
	{
	    printf("%s(%d): ERROR: %dx%d > %dx%d\n", FL, 
			self->img_w + 2, self->pxm->width,
			self->img_h + 2, self->pxm->height);
	    return;
	}

	fill_img((TImage*)self->pxm, SubColor(self->Color, 0x181818), 1, 1,
			self->img_byte, self->img_w, self->img_h);
	fill_img((TImage*)self->pxm, AddColor(self->Color, 0x101010), 3, 3,
			self->img_byte, self->img_w, self->img_h);

	if (self->Enabled) {
	    fill_img((TImage*)self->pxm, 0x202020, 2, 2,
			self->img_byte, self->img_w, self->img_h);
	} else {
	    fill_img((TImage*)self->pxm, SubColor(self->Color, 0x202020), 2, 2,
			self->img_byte, self->img_w, self->img_h);
	}

//	self->pxm->stat &= ~pmxStat_XPixAkt;
	TPixMap_XPutImage(self->pxm);
	self->static_pic.width = self->pxm->width;
	self->static_pic.height = self->pxm->height;
	self->static_pic.pic = self->pxm->pmxID;
	self->pic = &self->static_pic;
}

void ButtonCalc_SetImgData(TButtonCalc *self, const Timg_data *img_data) {

	if (!self) return;
	if (img_data && (img_data->art == 1)) {
	    self->img_data = img_data;
	    ButtonCalc_UpdateImgDataIdx(self);
	} else {
	    self->img_data = NULL;
	    ButtonCalc_Set_img_data(self, 0, 0, NULL);
	}
}

TButtonCalc *ButtonCalc_KeyEvent(TWins *wins, TKeyEvent *KeyEv, bool Down) {
  TButtonCalc *result;

	result = find_ButtonCalc_Key(wins, KeyEv->keysym);

	if (down_button) {
	    if (ttc_chk_is_wtyp(down_button, wtyp_ButtonCalc)) {
		down_button->BorderFu &= ~mBorderL;
		Refresh((TWins*)down_button);
	    }
	    down_button = NULL;
	}

	if (Down && result && !(KeyEv->state & ControlMask)) {
	    down_button = result;
	    result->BorderFu |= mBorderL;
	    Refresh((TWins*)result);
	    if (result->OnClick) result->OnClick(result, 0, false);
	}
	return result;
}

TButtonCalc *ButtonCalc_find_KF(TWins *wins, int KF) {
	return find_ButtonCalc_KF(wins, KF);
}

void ButtonCalc_Set_KFB(TWins *wins, int KFB) {
	SetUp_ButtonCalc_KFB(wins, KFB);
}

/*____________________________ Destroy __________________________________*/
static bool TButtonCalc__Destroy(TButtonCalc *self) {
	ppTfree(&self->pxm);
	return TWins__Destroy((TWins*)self);
}

/*____________________________ Create ___________________________________*/
static TButtonCalc *TButtonCalc_Create(TWins *Parent) {
  TButtonCalc *self;
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TButtonCalc), wtyp_ButtonCalc);
	wcs.Parent = Parent;
	wcs.CreateParams = TButtonPic_CreateParams;
	TWins_DefaultConf(&wcs);
	self = wcs.wins;
	self->destroy = (TDESTROY)TButtonCalc__Destroy;
	self->KeyFu &= ~mCanFocus;
	self->DoMouseDown = TButtonCalc_DoMouseDown;
	self->DoMouseUp = TButtonCalc_DoMouseUp;
	self->DoPaint = TButtonCalc_DoPaint;
	self->OnScale = TButtonCalc_OnScale;
	return self;
}

typedef struct {
  int KF, XK, XK2;
} TKF_XK;
   
static const TKF_XK TKF_XK_Tab[] = {
  {KF_0,	XK_0,		0},
  {KF_1,	XK_1,		0},
  {KF_2,	XK_2,		0},
  {KF_3,	XK_3,		0},
  {KF_4,	XK_4,		0},
  {KF_5,	XK_5,		0},
  {KF_6,	XK_6,		0},
  {KF_7,	XK_7,		0},
  {KF_8,	XK_8,		0},
  {KF_9,	XK_9,		0},
  {KF_A,	XK_A,		XK_a},
  {KF_B,	XK_B,		XK_b},
  {KF_C,	XK_C,		XK_c},
  {KF_D,	XK_D,		XK_d},
  {KF_E,	XK_E,		XK_e},
  {KF_F,	XK_F,		XK_f},
  {KF_COMMA,	XK_comma,	XK_period},
  {KF_RESULT,	XK_Return,	0},
  {KF_CLR,	XK_Delete,	0},
  {KF_ADD,	XK_plus,	0},
  {KF_SUB,	XK_minus,	0},
  {KF_MULT,	XK_asterisk,	0},
  {KF_DIV,	XK_slash,	0},
  {0,	0}
};

TButtonCalc *TButtonCalc_CreateB(TWins *Parent, int left, int top, 
						int width, int height) 
{
  TButtonCalc *self;

	self = TButtonCalc_Create(Parent);
	SelfCreateS((TWins*)self, left, top, width, height, NULL);
	return self;
}

void TButtonCalc_Set_XK_sym(TButtonCalc *self, int KF) {
  const TKF_XK *kfxk;

	kfxk = TKF_XK_Tab;
	while (kfxk->KF && (kfxk->KF != KF)) kfxk++;
	if (kfxk->KF) {
	    self->XK_sym  = kfxk->XK;
	    self->XK_sym2 = kfxk->XK2;
	}
}

TButtonCalc *TButtonCalc_Create_Keyfk(TWins *Parent, int offx, int offy, 
						const TKeyfk *Keyfk) 
{
  TButtonCalc *self;

	self = TButtonCalc_CreateB(Parent, Keyfk->x + offx, Keyfk->y + offy, 
					   Keyfk->w, Keyfk->h);
	self->Keyfk = Keyfk;
	self->KF = Keyfk->KF;

	ButtonCalc_SetImgData(self, (const Timg_data*)Keyfk->img_data);
	TButtonCalc_Set_XK_sym(self, Keyfk->KF);

	return self;
}
