/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldconsole.h"
#include <unistd.h>

static void menuEdit_OnMenuOpen(TMenu *self) {
  TFormAHconsole *fmain = (TFormAHconsole*)self->FormStruct;
  TMenuItem *mi;
  bool sh_slave, sel;

	sel = TLineEdit_ChkIsSel(fmain->le);
	sh_slave = chk_sh_slave(fmain, NULL);
	mi = self->MenuItem;
	while (mi) {
	    if (mi->tag == 1) {
		mi->fu |= miNotEnabled;
		if (sel) mi->fu &= ~miNotEnabled;

		if (sh_slave) {
		    ppcfree(&mi->s2);
		} else {
		    if (!mi->s2) mi->s2 = pcdup("Strg c");
		}

	    } else if (mi->tag == 2) {
		if (sh_slave) {
		    ppcfree(&mi->s2);
		} else {
		    if (!mi->s2) mi->s2 = pcdup("Strg v");
		}
	    }
	    mi = mi->next;
	}
}

static void mEdit_Copy(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormAHconsole *fmain = (TFormAHconsole*)self->FormStruct;
	TLineEditBase_Strg_C((TLineEditBase*)fmain->le);
}

static void mEdit_Paste(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormAHconsole *fmain = (TFormAHconsole*)self->FormStruct;
	if (!GetPasteStr_lgs(fmain->le->vds.UTF8on)) return;
	write(fmain->pty->ptyfd, appXDat.SelRet_lgs.s, appXDat.SelRet_lgs.lg);
	SelRetClr();
}

static void mEdit_SelectAll(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormAHconsole *fmain = (TFormAHconsole*)self->FormStruct;
	TLineEdit_SetSel(fmain->le, 0, 0, 0, fmain->le->tls->Count);
}

static const char *No_Find_Msg = {
  "\"%s\"\n"
  "End of document\n"
  "Continue from the beginning ?"
};

void EditFindExec(TFormAHconsole *fmain) {
  TLines *tls = fmain->le->tls;
  TFind_Dlg *Find_Dlg = fmain->Find_Dlg;
  int m;
  char line[1024];

	if (fmain->EditFind_run) return;
	fmain->EditFind_run = True;
	if (!Find_Dlg->edFind->vds.s) {
	    Find_Dlg_SetUTF8(fmain->Find_Dlg, fmain->le->vds.UTF8on);
	    Find_Dlg_Execute(Find_Dlg, NULL);
	}
	if (Find_Dlg->edFind->vds.s) {
	    tls->FindXa = fmain->le->CaretCharIdx;
	    tls->FindYa = fmain->le->CaretCharIdy;
loopAnf:
	    tls->FindNoCase = ((TChkBox*)Find_Dlg->cbCase)->Checked;
	    if (TLines_FindS(tls, Find_Dlg->edFind->vds.s)) {
		TLineEdit_SetSel(fmain->le, 
			tls->FindXa, tls->FindYa, tls->FindXe, tls->FindYe);
		TLineEdit_ChkCaret(fmain->le, LE_ChkCaret_Ymiddle);
	    } else {
		snprintf(line, 1024, No_Find_Msg, Find_Dlg->edFind->vds.s);
		m = MsgBox_Show((TWins*)fmain, "Find - ALDedit", line, msgYES | msgNO, msgYES);
		if (m == msgYES) {
		    tls->FindXa = 0;
		    tls->FindYa = 0;
		    goto loopAnf;
		}
	    }
	}
	fmain->EditFind_run = False;
}

void Edit_Find(TFormAHconsole *fmain) {
  Tlgs lgs;

	if (fmain->mEdit_Find_run) return;
	fmain->mEdit_Find_run = True;
	TLineEditBase_GetSel_lgs((TLineEditBase*)fmain->le, &lgs, 0);
	Find_Dlg_SetUTF8(fmain->Find_Dlg, fmain->le->vds.UTF8on);
	if (Find_Dlg_Execute(fmain->Find_Dlg, lgs.s)) EditFindExec(fmain);
	Tlgs_Clr(&lgs);
	fmain->mEdit_Find_run = False;
}

static void mEdit_Find(TMenu *self, long KeyState, TMenuItem *smi) {
	Edit_Find((TFormAHconsole*)self->FormStruct);
}

/* ______________________ Create _____________________________________ */
void MenuEdit_Create(TFormAHconsole *fmain) {
  TMenuItem *mi;
	fmain->mbEdit = TMenuButton_Create(fmain->PanelTop, fmain->mbFile);
	SetCaption((TWins*)fmain->mbEdit, "Edit");
	Set_Parent((TWins*)fmain->mbEdit, fmain->PanelTop);

	fmain->menuEdit = TMenu_Create((TWins*)fmain);
	fmain->menuEdit->OnMenuOpen = menuEdit_OnMenuOpen;
	fmain->mbEdit->menu = fmain->menuEdit;

	mi = TMenu_AddItem(fmain->menuEdit, "Copy", miNotEnabled, mEdit_Copy);
	mi->tag = 1;
	mi = TMenu_AddItem(fmain->menuEdit, "Paste", 0, mEdit_Paste);
	mi->tag = 2;
	TMenu_AddItem(fmain->menuEdit, "Select All", 0, mEdit_SelectAll);
	TMenu_AddItem(fmain->menuEdit, "-", 0, NULL);
	mi = TMenu_AddItem(fmain->menuEdit, "Find", 0, mEdit_Find);
	mi->s2 = pcdup("Strg f");

}
