/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldconsole.h"
#include "cicons.h"

static void mSetting_Clipbord(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormAHconsole *fmain = (TFormAHconsole*)self->FormStruct;
  TMenuItem *mi;
	mi = fmain->menuClipbord->MenuItem;
	while (mi) {mi->fu &= ~miChecked; mi = mi->next;}
	smi->fu |= miChecked;
	if (smi->tag == 1) {
		appXDat.CLIPBOARD_ATOM = XA_PRIMARY;
	} else if (smi->tag == 2) {
		appXDat.CLIPBOARD_ATOM = appXDat.XA_CLIPBOARD;
	}
}

void ChkTermUTF8(TFormAHconsole *fmain, bool force_open) {
  char *LANG;
  Tleterm *term;
  XFontStruct *xfont;
  bool UTF8on;

	LANG = NULL;
	term = fmain->leterm;
	UTF8on = (fmain->miUTF8->fu & miChecked) != 0;
	term->UTF8on = UTF8on;

	if (UTF8on && (fmain->miUTF8_en_US->fu & miChecked)) {
	    LANG = pcdup("en_US.UTF-8");

	} else if (fmain->LANG_utf8 != UTF8on) {
	    char *pc = fmain->LANG;
	    while (*pc && (*pc != '.') && (*pc != '@')) pc++;
	    if (fmain->LANG_utf8) {
		LANG = pcndup(fmain->LANG, pc - fmain->LANG);
	    } else {
		LANG = pcnnadd(fmain->LANG, pc - fmain->LANG, ".UTF-8", 6);
	    }
	}

	if (pccmp(fmain->pty->LANG, LANG)) {
	    xfree(LANG);

	} else {
	    term->idx = 0;
	    Tpty_close(fmain->pty);
	    xfree(fmain->pty->LANG);
	    fmain->pty->LANG = LANG;
	    AppHandleMessage();
	    force_open = True;
	}

	if (UTF8on) {
	    term->AktFont = term->FontUTF8;
	    term->AktFontBold = term->FontBoldUTF8;
	} else {
	    term->AktFont = term->Font;
	    term->AktFontBold = term->FontBold;
	}
	xfont = FontX_GetXFontStruct(term->AktFont);
	term->FontHasUTF8 = (xfont && ((xfont->min_byte1 | 
					xfont->max_byte1) != 0));

	if (force_open) {
	    Tpty_open(fmain->pty);
	}
	Refresh((TWins*)fmain->le);
}


static void miUTF8_OnClick(TMenu *self, long KeyState, TMenuItem *smi) {
	ChkTermUTF8(appDat.obj, False);
}

static void ChkSet_MenuItem(TMenuItem *mi, char *pc, int tag) {
	while (mi) {
	    if (mi->tag == tag) {
		mi->fu &= ~miChecked;
		if (pccmp(pc, mi->s)) mi->fu |= miChecked;
	    }
	    mi = mi->next;
	}
}

void SetTermSize(TFormAHconsole *fmain, int new_col, int new_row) {
  char *pc;
  Tpty *pty = fmain->pty;
  char tmps[40];

	Tpty_SetSize(pty, new_col, new_row);
	Tleterm_SetSize(fmain->leterm, new_col, new_row);
	snprintf(tmps, 40, "w=%d h=%d", pty->wsize.ws_col, pty->wsize.ws_row);
	SetCaption(fmain->SizeAzBox, tmps);
	SetLeTermRect(fmain, 0);

	inttopc(tmps, pty->wsize.ws_col);
	ChkSet_MenuItem(fmain->menuTermSizeCol->MenuItem, tmps, 1);

	inttopc(tmps, pty->wsize.ws_row);
	ChkSet_MenuItem(fmain->menuTermSizeRow->MenuItem, tmps, 2);

	pc = tmps;
	inttoppc(&pc, pty->wsize.ws_col);
	*pc = 'x'; pc++;
	inttoppc(&pc, pty->wsize.ws_row);
	ChkSet_MenuItem(fmain->menuTermSize->MenuItem, tmps, 3);

}

static void miSizeCol_OnClick(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormAHconsole *fmain = appDat.obj;
  TMenuItem *mi;
  int m;
	mi = self->MenuItem;
	while (mi) {
	    mi->fu &= ~miChecked;
	    if (mi == smi) {
		mi->fu |= miChecked;
		m = pctoint(mi->s);
		SetTermSize(fmain, m, 0);
	    }
	    mi = mi->next;
	}
}

static void miSizeRow_OnClick(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormAHconsole *fmain = appDat.obj;
  TMenuItem *mi;
  int m;
	mi = self->MenuItem;
	while (mi) {
	    mi->fu &= ~miChecked;
	    if (mi == smi) {
		mi->fu |= miChecked;
		m = pctoint(mi->s);
		SetTermSize(fmain, 0, m);
	    }
	    mi = mi->next;
	}
}

static void miSizeColRow_OnClick(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormAHconsole *fmain = appDat.obj;
  const char *pc = smi->s;
  int w, h;
	w = ppctoint(&pc);
	h = ppctoint_scan(&pc, True);
	SetTermSize(fmain, w, h);
}

static void menuSetting_OnMenuOpen(TMenu *self) {
  TFormAHconsole *fmain = appDat.obj;
  bool ok = ((*fmain->leterm->line == '>') && (fmain->leterm->lg >= 2));

	if (ok) {
	    for (int i=1;i<fmain->leterm->lg;i++) {
		if (fmain->leterm->line[i] != ' ') { ok = False; break;}
	    }
	}

	if (ok) {
	    fmain->miUTF8->fu &= ~miNotEnabled;
	    if (fmain->LANG) fmain->miUTF8_en_US->fu &= ~miNotEnabled;
	} else {
	    fmain->miUTF8->fu |= miNotEnabled;
	    fmain->miUTF8_en_US->fu |= miNotEnabled;
	}
}

/* ______________________ Create _____________________________________ */
void MenuSetting_Create(TFormAHconsole *fmain) {
  TMenuItem *mi, *mi2;
  int m;

	fmain->mbSetting = TMenuButton_Create(fmain->PanelTop, fmain->mbFile);
	SetCaption((TWins*)fmain->mbSetting, "Settings");
	Set_Parent((TWins*)fmain->mbSetting, fmain->PanelTop);

	fmain->menuSetting = TMenu_Create((TWins*)fmain);
	fmain->mbSetting->menu = fmain->menuSetting;
	fmain->menuSetting->OnMenuOpen = menuSetting_OnMenuOpen;

	mi = TMenu_AddItem(fmain->menuSetting, "Term size", 0, NULL);
	  fmain->menuTermSize = TMenu_Create((TWins*)fmain);
	  mi->next_menu = fmain->menuTermSize;
	  mi2 = TMenu_AddItem(fmain->menuTermSize, "Col X", 0, NULL);
	    fmain->menuTermSizeCol = TMenu_Create((TWins*)fmain);
	    mi2->next_menu = fmain->menuTermSizeCol;
	    TMenu_AddItem(fmain->menuTermSizeCol, "64", miCanCheck, miSizeCol_OnClick)->tag = 1;
	    TMenu_AddItem(fmain->menuTermSizeCol, "80", miCanCheck, miSizeCol_OnClick)->tag = 1;
	    TMenu_AddItem(fmain->menuTermSizeCol, "96", miCanCheck, miSizeCol_OnClick)->tag = 1;
	    TMenu_AddItem(fmain->menuTermSizeCol, "132", miCanCheck, miSizeCol_OnClick)->tag = 1;
	    TMenu_AddItem(fmain->menuTermSizeCol, "160", miCanCheck, miSizeCol_OnClick)->tag = 1;
	  mi2 = TMenu_AddItem(fmain->menuTermSize, "Row Y", 0, NULL);
	    fmain->menuTermSizeRow = TMenu_Create((TWins*)fmain);
	    mi2->next_menu = fmain->menuTermSizeRow;
	    TMenu_AddItem(fmain->menuTermSizeRow, "18", miCanCheck, miSizeRow_OnClick)->tag = 2;
	    TMenu_AddItem(fmain->menuTermSizeRow, "24", miCanCheck, miSizeRow_OnClick)->tag = 2;
	    TMenu_AddItem(fmain->menuTermSizeRow, "30", miCanCheck, miSizeRow_OnClick)->tag = 2;
	    TMenu_AddItem(fmain->menuTermSizeRow, "36", miCanCheck, miSizeRow_OnClick)->tag = 2;
	    TMenu_AddItem(fmain->menuTermSizeRow, "48", miCanCheck, miSizeRow_OnClick)->tag = 2;

	TMenu_AddItem(fmain->menuTermSize, "80x24", miCanCheck, miSizeColRow_OnClick)->tag = 3;
	TMenu_AddItem(fmain->menuTermSize, "80x30", miCanCheck, miSizeColRow_OnClick)->tag = 3;
	TMenu_AddItem(fmain->menuTermSize, "96x24", miCanCheck, miSizeColRow_OnClick)->tag = 3;
	TMenu_AddItem(fmain->menuTermSize, "96x30", miCanCheck, miSizeColRow_OnClick)->tag = 3;
	TMenu_AddItem(fmain->menuTermSize, "96x36", miCanCheck, miSizeColRow_OnClick)->tag = 3;
	TMenu_AddItem(fmain->menuTermSize, "132x36", miCanCheck, miSizeColRow_OnClick)->tag = 3;


	mi = TMenu_AddItem(fmain->menuSetting, "Clipbord Mode", 0, NULL);
	  fmain->menuClipbord = TMenu_Create((TWins*)fmain);
	  mi->next_menu = fmain->menuClipbord;
	  m = miCanCheck;
	  if (appXDat.CLIPBOARD_ATOM != appXDat.XA_CLIPBOARD) m |= miChecked;
	  TMenu_AddItem(fmain->menuClipbord, "X11 (XA_PRIMARY)", m, mSetting_Clipbord)->tag = 1;
	  m = miCanCheck;
	  if (appXDat.CLIPBOARD_ATOM == appXDat.XA_CLIPBOARD) m |= miChecked;
	  TMenu_AddItem(fmain->menuClipbord, "QT3 (XA_CLIPBOARD)", m, mSetting_Clipbord)->tag = 2;

	TMenu_AddItem(fmain->menuSetting, "-", 0, NULL);
	mi = TMenu_AddItem(fmain->menuSetting, "Font", 0, NULL);
	mi->pic = get_fileIcon(stft_Font_A);
	  fmain->menuFont = (TMenu*)TMenu_Create((TWins*)fmain);
	  mi->next_menu = fmain->menuFont;
	  TMenu_AddItem(fmain->menuFont, "Set Font", miNotEnabled, NULL)->pic = get_fileIcon(stft_Font_A);
//	  TMenu_AddItem(fmain->menuFont, "Save Font as Default", miNotEnabled, NULL)->pic = get_sysIcon(sico_filesave);
	  TMenu_AddItem(fmain->menuFont, "Save Font as Default", miNotEnabled, NULL)->pic = get_pic16_filesave(False);
//	TMenu_AddItem(fmain->menuSetting, "Font CharSet", 0, NULL);
	fmain->miUTF8 = TMenu_AddItem(fmain->menuSetting, "Encoding UTF-8", 
			miAutoToggle, miUTF8_OnClick);

	fmain->miUTF8_en_US = TMenu_AddItem(fmain->menuSetting, "utf8 use \"en_US.UTF-8\"", 
			miAutoToggle, miUTF8_OnClick);

	TMenu_AddItem(fmain->menuSetting, "-", 0, NULL);
	fmain->miAutoReconnect = TMenu_AddItem(fmain->menuSetting, "Auto reconnect", 
			miChecked|miAutoToggle, NULL);

}
