/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldconsole.h"
#include <unistd.h>
#include <sys/wait.h>
#include <sys/resource.h>
#include <errno.h>
#include "xtty.h"

/*static const char *pargv[2] = {
  "/bin/bash", NULL
}; */

bool Tpty_open(Tpty *self) {
  int pid;
  int tty;
  char *pargv[2];
  struct rlimit rlp;

	self->TermPid = 0;
	self->ptyfd = posix_openpt(O_RDWR);
	if (self->ptyfd <= 0) self->ptyfd = getpt();
	if (self->ptyfd <= 0) return False;
	if (grantpt(self->ptyfd) || unlockpt(self->ptyfd) || 
	    ptsname_r(self->ptyfd, self->tty_name, sizeof(self->tty_name))) {
	    close(self->ptyfd);
	    return False;
	}
	pargv[0] = pcdup("/bin/bash");
	pargv[1] = NULL;

	pid = fork();
	if (pid == 0) {
	    getrlimit(RLIMIT_NOFILE, &rlp);
	    for (int i=0;i<rlp.rlim_cur;i++) close(i);
	    tty = open(self->tty_name, O_RDWR);
	    dup2(tty, STDIN_FILENO);
	    dup2(tty, STDOUT_FILENO);
	    dup2(tty, STDERR_FILENO);
	    if (tty > 2) close(tty);

	    if (self->cwd && chdir(self->cwd)) {
		printf("ERROR: chdir failed=\"%s\" [%s]\n", self->cwd, strerror(errno));
	    }

	    if (setsid() < 0) {
		printf("ERROR: %s(%d): failed to set process group\n", FL);
	    }
	    if (self->term_type == term_xterm) {
//		setenv("TERM", "xterm-xfree86", 1);
		setenv("TERM", "xterm", 1);
	    } else {
		setenv("TERM", "linux", 1);
	    }

	    if (self->LANG && *self->LANG) {
		if (self->LC_xxxx & 0x01) setenv("LC_ALL",   self->LANG, 1);
		if (self->LC_xxxx & 0x02) setenv("LC_CTYPE", self->LANG, 1);
		setenv("LANG", self->LANG, 1);
	    }

	    setenv("PS1", "> ", 1);
	    execvp(pargv[0], pargv);
	    fprintf(stderr, "ERROR: exec failed \"%s\" \"%s\"\n", strerror(errno), pargv[0]);
	    _exit(EXIT_FAILURE);
	}
	if (pid <= 0) {
	    close(self->ptyfd);
	    self->ptyfd = 0;
	    self->TermPid = 0;
	    xfree(pargv[0]);
	    return False;
	}

	ioctl(self->ptyfd, TIOCSWINSZ, &self->wsize);
	self->TermPid = pid;
	AddPipeProc(self->ptyfd, InPipe, self->OnInPipe, NULL);
	self->path_deleted = False;
	xfree(pargv[0]);
	return True;
}

void Tpty_close(Tpty *self) {
	if (self->TermPid) kill(self->TermPid, SIGQUIT);
	self->TermPid = 0;
	if (self->ptyfd > 0) ClosePipeProc(self->ptyfd, clpp_fd);
	self->ptyfd = 0;
}

void Tpty_SetSize(Tpty *self, int col, int row) {
	if (col <= 0) col = self->wsize.ws_col;
	if (row <= 0) row = self->wsize.ws_row;

	if ((self->wsize.ws_col == col) && (self->wsize.ws_row == row)) return;
	self->wsize.ws_col = col;
	self->wsize.ws_row = row;
	if (self->ptyfd) {
	    ioctl(self->ptyfd, TIOCSWINSZ, &self->wsize);
	}
}

/* ______________________ Destroy ____________________________________ */
static bool Tpty__Destroy(Tpty *self) {
	Tpty_close(self);
	xfree(self->line);
	return False;
}

/* ______________________ Create _____________________________________ */
Tpty *Tpty_Create() {
  Tpty *self;
	self = ttc_create(sizeof(Tpty), wtyp_Tpty);
	self->destroy = (TDESTROY)Tpty__Destroy;
	self->wsize.ws_col = 80;
	self->wsize.ws_row = 24;
	self->max = 16384;
	self->line = xmalloc(self->max);
	return self;
}
