/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldedit.h"
#include "net_io.h"
#include "tdirlist.h"
#include "cicons.h"


//#define LdF_Add_Page		0x01
//#define LdF_ChkFileIsSaved	0x02
#define LdF_fu_apj		0x100
#define LdF_fu_ReLoad		0x200

// *************************
static int app_argc;
static char **app_argv;

static TLineEdit *Add_LineEdit(TWins *Parent);


static void OnChangeTab(TPageControl *pagc, TPage *fromPage, TPage *toPage, int *fu) {
  TFormEdit *fed = (TFormEdit*)pagc->FormStruct;
  char *fn, *fnNoPasswd;

	if (toPage && toPage->owner) {
	    fed->AktLE = (TLineEdit*)toPage->owner;
	    Chk_UnRedo(fed);
	    fn = fed->AktLE->tls->FileName;
	    if (fn) {
		fnNoPasswd = get_DisplayFileStr(fn, NULL);
		SetCaption((TWins*)fed, fnNoPasswd);
		xfree(fnNoPasswd);
	    } else {
		SetCaption((TWins*)fed, toPage->vds.s);
	    }
	    UpdateSettingMenu(fed);
	}
}

static void SetPageCaptionToFileName(TPage *pag) {
  TLines *tls;
  TglobPath *gp;
	if (pag && pag->owner) {
	    tls = ((TLineEdit*)pag->owner)->tls;
	    if (pclen(tls->FileName) > 0) {
		gp = pctogp(tls->FileName);
		ppcins(&gp->name, " ", 0);
		SetCaption((TWins*)pag, gp->name);
		gpFree(gp);
	    }
	}
}


void Add_Page(TFormEdit *fed, char *fn, const char *Caption) {
  TPage *pg;
  TLines *tls;
  char *s, *stmp;
  bool ok;

	pg = NULL;
	if (fn) {
	    s = get_DisplayFileStr(fn, NULL);
	    pg = TPageControl_GetFirstPage(fed->PageControl, False);
	    while (pg) {
		if (pg->owner) {
		    tls = ((TLineEdit*)pg->owner)->tls;
		    stmp = get_DisplayFileStr(tls->FileName, NULL);
		    ok = pccmp(s, stmp);
		    xfree(stmp);
		    if (ok) break;
		}
		pg = TPage_GetNext(pg, False);
	    }
	    xfree(s);
	    Caption = GetFileName_pc(fn);
	}

	if (pg) {
	    fed->AktLE = (TLineEdit*)pg->owner;
	    TPage_SetPageAktiv(pg);
	} else {
	    pg = (TPage*)TPageControl_AddNewPage(fed->PageControl, Caption);
	    fed->AktLE = Add_LineEdit((TWins*)pg);
#ifdef _OS_WIN32
	    fed->AktLE->vds.UTF8on = False;
#endif
	    pg->FocusWin = (TWins*)fed->AktLE;
	    pg->owner = (TWins*)fed->AktLE;
	    TPage_SetPageAktiv(pg);
	}
}

static bool Get_le_FileSaved(TLineEdit *le) {
	return (le && !le->tls->Changed && 
	        (le->user_b0 == (le->tls->DefaultEOL & fio_EOL_Mask)));
}

bool ChkAllFileSaved(TFormEdit *fed) {
  TPage *pg;
  TLineEdit *le;
  bool res = True;
  bool refr = False;

	if (fed->PageControl) {
	    pg = TPageControl_GetFirstPage(fed->PageControl, False);
	    while (pg) {
		le = (TLineEdit*)pg->owner;
		if (le) {
		    if (!Get_le_FileSaved(le)) {
			res = False;
			if (pg->vds.s && (*pg->vds.s == ' ')) {
			    *pg->vds.s = '*';
			    refr = True;
			}
		    } else {
			if (pg->vds.s && (*pg->vds.s == '*')) {
			    *pg->vds.s = ' ';
			    refr = True;
			}
		    }
		}
		pg = TPage_GetNext(pg, False);
	    }
	    if (refr && fed->PageControl->ResizeStartMk) {
		fed->PageControl->ResizeStartMk = False;
		Refresh((TWins*)fed->PageControl);
	    }
	} else {
	    res = Get_le_FileSaved(fed->AktLE);
	}

	SetEnabled((TWins*)fed->mbSave, !res);
	if (res) fed->miSave->fu |= miNotEnabled; else fed->miSave->fu &= ~miNotEnabled;
	if (res) fed->miLESave->fu |= miNotEnabled; else fed->miLESave->fu &= ~miNotEnabled;
	return res;
}

static void PanelBottom_OnResize(TWins *self) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
  int x;
	x = self->width - fed->buNextPage->width;
	SetLeft((TWins*)fed->buNextPage, x);
	x -= fed->buPrevPage->width + 1;
	SetLeft((TWins*)fed->buPrevPage, x);
}

static void buPrevPage_OnPaint(TButton *self) {
  TPic *pic;
	pic = get_pic9_hsb_minus(False);
	DrawPic(self->WinID, pic, 4, 4);
}

static void buNextPage_OnPaint(TButton *self) {
  TPic *pic;
	pic = get_pic9_hsb_plus(False);
	DrawPic(self->WinID, pic, 4, 4);
}

static void buPrevPage_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	TPage_Move(fed->PageControl->AktPage, NULL, pgMove_ChkVisible|pgMove_Prev);
}

static void buNextPage_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	TPage_Move(fed->PageControl->AktPage, NULL, pgMove_ChkVisible);
}


static void OnLdWrTimer(TgTimer *gTimer) {
  TFormEdit *fed = gTimer->obj;
	SetVisible((TWins*)fed->ProgressBar, False);
	SetVisible(fed->kpsBox, False);
	fed->LdWrTimer = NULL;
}

static void StartLdWrTimer(TFormEdit *fed) {
	if (fed->LdWrTimer) Tfree(fed->LdWrTimer);
	fed->LdWrTimer = gTimerX_Add(Timer_Enabled | Timer_OnTimerAndFree, 6000, OnLdWrTimer);
	fed->LdWrTimer->obj = fed;
	SetVisible((TWins*)fed->ProgressBar, False);
}

static void Az_LdWr_On(TFormEdit *fed) {
	fed->us_mk = 0;
	fed->mkDataNowSize = 0;
	TProgressBar_SetProz(fed->ProgressBar, 0);
	SetLeft((TWins*)fed->ProgressBar, fed->CharAzBox->left);
	SetLeft(fed->kpsBox, fed->CharAzBox->left + fed->ProgressBar->width + 1);
	SetVisible((TWins*)fed->ProgressBar, True);
	SetVisible(fed->kpsBox, True);
}

void LoadFile(TFormEdit *fed, TLineEdit *le, char *pfn, int fu, TRestoreCaret *rsc) {
  TpcIni *ini;
  int m;
  char *fn, *fnNoPasswd;
  const char *cpc;

// printf("%s(%d) LoadFile: %s\n", FL, pfn);
	if (!pfn || !*pfn) return;

	fn = path_rel2abs(pfn, NULL);
#ifdef _OS_WIN32
	char *tfn = NT4_GetLongPathName(fn);
	if (tfn) {xfree(fn); fn = tfn;}
#endif

	if (fed->PageControl) {
  TglobPath *gp;
	    gp = pctogp(fn);
	    if (fu & 0x01) {
		Add_Page(fed, fn, gp->name);
	    }
	    gpFree(gp);
	}

	if (!le) le = fed->AktLE;
	if ((fu & 0x02) && ((ChkFileIsSaved(le) == msgCANCEL))) { 
	    xfree(fn);
	    return;
	}
	if (!le) return;  //???

	fed->IO_Run = True;
	TUnRedo_Clr(le->UnRedo);
	SetEnabled((TWins*)fed->mbUndo, False);
	SetEnabled((TWins*)fed->mbRedo, False);
	if (!fed->PageControl) {
	    SetEnabled((TWins*)fed->mbSave, False);
	    fed->miSave->fu |= miNotEnabled;
	    fed->miLESave->fu |= miNotEnabled;
	}

	Tfree(le->highlight);
	le->highlight = NULL;


	le->tls->load_fio_fu = fio_Chk_BOM;
	Az_LdWr_On(fed);

	fnNoPasswd = get_DisplayFileStr(fn, NULL);
	if (le == fed->AktLE) SetCaption((TWins*)fed, fnNoPasswd);


	le->tls->stat_fu = fsGetFileType | fsGetFileTypeHead;
	le->tls->EvStop = True;
	le->ReadOnly = !TLineEdit_LoadFromFile(le, fn);

	le->user_b0 = le->tls->DefaultEOL & fio_EOL_Mask;
	le->user_b1 = True;

	if (((le->tls->org_load_fio_fu & fio_txtTyp_Mask) || 
	     (le->tls->org_load_fio_fu & fio_has_UTF8_char)) && 
	     !(le->tls->org_load_fio_fu & fio_err_UTF8_char)) {
	    le->vds.UTF8on = True;
	    fed->miUTF8->fu |= miChecked; 
	}

	if (!le->ReadOnly && rsc) {
	    TLineEdit_AllAtrNew(le);
	    RestoreCaret(le, rsc);
	}

	Refresh((TWins*)le);
	AppProcessMessages();
	le->tls->EvStop = False;

	if ((fu & LdF_fu_apj) == 0) {
	    int dupi = -1;
	    char *pc0;
	    char line[32];
	    char *itm[REOPEN_ITEMS];

	    Fill0(itm, sizeof(itm));

	    ini = get_appIni();
	    pc0 = line;
	    ppcmove(&pc0, "LastFile");
	    int idx = 1;
	    for (int i=0; i<REOPEN_ITEMS; i++) {
		inttopc(pc0, i);
		cpc = TpcIni_RD(ini, APPNAME, line);
		if (!cpc) break;
		
		TglobPath *gp = pctogp(cpc);
		CryptMini16c_enc_reg(gp->passwd);
		char *fnt = get_DisplayFileStr(NULL, gp);
		gpFree(gp);
		if (pccmp(fnNoPasswd, fnt)) {
		    xfree(fnt);
		    dupi = i;
		} else {
		    xfree(fnt);
		    itm[idx] = pcdup(cpc);
		    idx++;
		    if (idx >= REOPEN_ITEMS) break;
		}
	    }

	    if (dupi) {
		TglobPath *gp = pctogp(fn);
		char *pw = CryptMini16c_dec(gp->passwd);
		if (pw) {
		    xfree(gp->passwd);
		    gp->passwd = pw;
		}
		itm[0] = gptopc(gp, gptfu_PasswdTxt);
		gpFree(gp);

		for (int i=0; i<REOPEN_ITEMS; i++) {
		    if (!itm[i]) break;
		    inttopc(pc0, i);
		    TpcIni_WR(ini, APPNAME, line, itm[i]);
		}
		if (!appDat.appIni) TpcIni_Save(ini);
	    }

	    for (int i=0; i<REOPEN_ITEMS; i++) xfree(itm[i]);
	    free_appIni(ini);
	}
	xfree(fnNoPasswd);


	m = stft_None;
	int sub_mime = 0;
	if (le->tls->st) {
	    m = le->tls->st->FileType;
	    sub_mime = le->tls->st->sub_mime;
	}
	Set_highlight(fed, le, m, sub_mime);

	fed->IO_Run = False;

	StartLdWrTimer(fed);

	if (fed->PageControl) {
	  ChkAllFileSaved(fed);
	  if (!(fu & LdF_fu_ReLoad)) {
	    SetPageCaptionToFileName(GetParentPage((TWins*)le));
	    if (fed->ProjFileIni) {
		char *s = path_abs2rel(fn, fed->ProjPath);
		cpc = TpcIni_RD(fed->ProjFileIni, s, "topxy");
		xfree(s);
		m = ppctoint(&cpc);
		TScrollBar_SetCount_ev(le->VSB, m);
		le->CaretCharIdx = ppctoint(&cpc);
		le->CaretCharIdy = ppctoint(&cpc);

		TLineEdit_ChkCaret(le, 0);
		le->SelAx = le->CaretCharIdx;
		le->SelAy = le->CaretCharIdy;
		TLineEdit_SetCaret(le);
	    }
	  }
	}

	xfree(fn);
	UpdateSettingMenu(fed);
}

// ******* Menu File *******
static void SaveFile(TLineEdit *le, char *fn) {
  TFormEdit *fed = (TFormEdit*)le->FormStruct;
  TLines *tls = le->tls;
  int FileType = 0;

	if (tls->st) FileType = tls->st->FileType;
	Az_LdWr_On(fed);
	TLineEdit_SaveToFile(le, fn);

	le->user_b0 = tls->DefaultEOL & fio_EOL_Mask;


	if (tls->st && (tls->st->FileType != FileType)) {
	    Set_highlight(fed, le, tls->st->FileType, tls->st->sub_mime);
	    UpdateSettingMenu(fed);
	}

	ChkAllFileSaved(fed);

	StartLdWrTimer(fed);
}

void FileSaveAs(TLineEdit *le) {
  TFormEdit *fed;
  char *s = NULL;
  Tstat64 *st = NULL;
  int m;
  char line[512];
  bool err;

	if (!le) return;
	fed = (TFormEdit*)le->FormStruct;

	fed->IO_Run = True;
	if (!fed->FileSave_Dlg) {
	    mFile_FileSave_Dlg_Init(fed);
	    if (le->tls->FileName) 
		File_Dlg_SetNamePath(fed->FileSave_Dlg, le->tls->FileName);
	}
	if (File_Dlg_Execute(fed->FileSave_Dlg)) {
	    File_Dlg_GetEdFileName(fed->FileSave_Dlg, &s);

	    err = False;
	    st = net_stat(s, NULL, fsFollowLink | fsNoFileName | fsNoLinkName);
	    if (st == PTR_FAILED) {
end_err:
		xfree(s);
		fed->IO_Run = False;
		return;
	    }
	      if (st) {
		char *azfn = get_DisplayFileStr(s, NULL);
		if (st->st & st_Reg) {
		  snprintf(line, sizeof(line), "Overwrite exists file \n%s", azfn);
		  m = MsgBox_Show((TWins*)fed, "Warning !!!", line, 
			  msgIcoWARNING | msgYES | msgNO, msgNO);
		  if (m == msgNO) err = True;
		} else {
		  snprintf(line, sizeof(line), "Can't write file \n%s", azfn);
		  MsgBox_Show((TWins*)fed, "ERROR !!!", line, 
			  msgIcoERROR | msgYES, msgYES);
		  err = True;
		}
		xfree(azfn);
	      }

	    xfree(st);
	    if (err) goto end_err;
	    SaveFile(le, s);
	    SetCaption((TWins*)fed, s);
	    if (fed->PageControl) {
		SetPageCaptionToFileName(GetParentPage((TWins*)le));
	    }
	    xfree(s);
	}
	fed->IO_Run = False;
}


static void SaveFileChk(TFormEdit *fed, TLineEdit *le) {
	if (le->tls->FileName) {
	    fed->IO_Run = True;
	    SaveFile(le, NULL);
	    fed->IO_Run = False;
	} else {
	    FileSaveAs(le);
	}
}

void File_Save(TFormEdit *fed) {
  TPage *pg;
  TLineEdit *le;
	if (fed->PageControl) {
	    pg = TPageControl_GetFirstPage(fed->PageControl, False);
	    while (pg) {
		le = (TLineEdit*)pg->owner;
		if (le && !Get_le_FileSaved(le)) SaveFileChk(fed, le);
		pg = TPage_GetNext(pg, False);
	    }
	} else {
	    SaveFileChk(fed, fed->AktLE);
	}
}

static void LineEdit_OnChange(TLineEdit *le, int fu, TKeyS *KeyS, Tlgs *lgs, int xa, int ya, int xe, int ye) {
  TFormEdit *fed = (TFormEdit*)le->FormStruct;
	TUnRedo_AddLine(le->UnRedo, fu, KeyS, lgs, xa, ya, xe, ye);
	if (fed->PageControl) {
	    ChkAllFileSaved(fed);
	} else {
	    SetEnabled((TWins*)fed->mbSave, True);
	    fed->miSave->fu &= ~miNotEnabled;
	    fed->miLESave->fu &= ~miNotEnabled;
	}
	Chk_UnRedo(fed);
}

static void LineEdit_VSB_AzCount(TFormEdit *fed) {
  char tmps[32];
  int m;

	m = fed->AktLE->tls->Count;
	snprintf(tmps, 32, "%d:%5d", m, fed->LCount);
	SetCaption(fed->SBVPosBox, tmps);
}

static void LineEdit_OnSetCaret(TLineEdit *le, TColS *cs) {
  TFormEdit *fed = (TFormEdit*)le->FormStruct;
  const char *s;
  char tmps[32];
  int x = le->CaretCharIdx;
  int y = le->CaretCharIdy;
  int c, m;
	if (cs) {
	    if (cs->s && (cs->lg > x)) { // && cs->s[x]) {
		if (le->vds.UTF8on) {
		    c = Utf8CharToWChar(cs->s + x);
		    if (c & Char_ErrorUTF8) c = (unsigned char)cs->s[x];
		} else {
		    c = (byte)cs->s[x];
		}
		snprintf(tmps, 32, "%d / 0x%02x", c, c);
		s = tmps;

	    } else {
		m = GetDefaultEOL_fu(le->tls->DefaultEOL, cs->fu);
		if (m == fio_EOLunix) {
			s = "EOL LF";
		} else if (m == fio_EOLwindos) {
			s = "EOL CRLF";
		} else if (m == fio_EOL_0) {
			s = "EOL 0";
		} else s = "EOL";
	    }
	} else s = "EOF";
	SetCaption(fed->CharAzBox, s);

	if (le->vds.UTF8on && cs && cs->s) {
	    x = Chk_fio_UTF8(cs->s, x, NULL);
	}

	snprintf(tmps, 32, "x=%d  y=%d", x, y+1);
	SetCaption(fed->CursPosBox, tmps);
	LineEdit_VSB_AzCount(fed);
}

static void LineEdit_VSB_DoCount(TWins *self, int Count) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	fed->LCount = Count;
	LineEdit_VSB_AzCount(fed);
}

static void LineEdit_OnKeyDown(TLineEdit *self, TKeyEvent *KeyEv) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	if (KeyEv->keysym == XK_F3) {
	    EditFind(fed);
	    KeyEv->keysym = 0;
	} else if ((KeyEv->keycode == 41) && ((KeyEv->state & 0x4) != 0)) { //Strg_F
	    TFormEdit_Find(fed);
	    KeyEv->keysym = 0;
	}
}

static void OnIO(TFileMsg *fmsg) {
  TFormEdit *fed = appDat.obj;
  int m;
  double kbs;
  char line[80];

	if ((fmsg->art == fmArt_DataIO) && (fmsg->typD == fmTyp_ioinfo)) {

	    if (fmsg->D.ioinfo->DataSize && (fed->mkDataNowSize != fmsg->D.ioinfo->DataNowSize)) {
		fed->mkDataNowSize = fmsg->D.ioinfo->DataNowSize;
		if (fed->mkDataNowSize <fmsg->D.ioinfo->DataSize) {
		    kbs = ((double)fed->mkDataNowSize * 1000) / 
			   (double)fmsg->D.ioinfo->DataSize;
		    m = kbs;
		} else m = 1000;
		TProgressBar_SetProz(fed->ProgressBar, m);
	    }
	    if ((fmsg->D.ioinfo->Time_us > fed->us_mk) || 
		(fmsg->D.ioinfo->stat & fmIOInfo_EOF)) {
		fed->us_mk += 100000;
		kbs = (double)fmsg->D.ioinfo->DataNowSize * 1024 / 
		      (double)fmsg->D.ioinfo->Time_us;
		snprintf(line, sizeof(line), "%8.1f KB/s", kbs);
		SetCaption(fed->kpsBox, line);
	    }
	}
}


void StoreCaret(TLineEdit *le, TRestoreCaret *rsc) {
	rsc->x = le->CaretCharIdx;
	rsc->y = le->CaretCharIdy;
	rsc->vsb_count = TScrollBar_GetCount(le->VSB);
}

void RestoreCaret(TLineEdit *le, TRestoreCaret *rsc) {
	if (rsc) {
	    TScrollBar_SetCount_ev(le->VSB, rsc->vsb_count);
	    le->SelAx = le->CaretCharIdx = rsc->x;
	    le->SelAy = le->CaretCharIdy = rsc->y;
	    TLineEdit_SetCaret(le);
	}
}

void Chk_LE_Reload(TFormEdit *fed, TLineEdit *le) {
  TLines *tls;
  Tstat64 *tmpst, *st;
  char *s;
  char tmps[1024];
  int m;

	if (!le) le = fed->AktLE;

	if (!le || fed->MsgBox_Reload_Run) return;
	fed->MsgBox_Reload_Run = True;
	tls = le->tls;


	if (!fed->IO_Run && tls && le->user_b1) {
	  st = tls->st;
	  if (st && test_local_drive(tls->FileName)) {
	    tmpst = file_stat_fn(tls->FileName, fsNoFileName | fsNoLinkName | fsFollowLink);
	    if (tmpst && (tmpst->mtime != st->mtime)) {
	      s = get_DisplayFileStr(tls->FileName, NULL); //pctogptopc(tls->FileName, gptfu_noPasswd);
	      snprintf(tmps, 1024, "%s\n\ntime/date changed. Reload?", s);
	      xfree(s);
	      m = MsgBox_Show((TWins*)fed, "Information !!!", 
			  tmps, msgYES | msgNO, msgYES);
	      if (m == msgYES) {
		  TRestoreCaret rsc;

		  StoreCaret(le, &rsc);
		  LoadFile(fed, le, tls->FileName, LdF_fu_ReLoad, &rsc);

		  fed->IO_Run = True;
		  Set_Focus((TWins*)fed->AktLE);
		  fed->IO_Run = False;
	      } else {
		  le->user_b1 = False;
		  fed->miChkReload->fu &= ~miChecked;
	      }
	    }
	    xfree(tmpst);
	  }
	}
	fed->MsgBox_Reload_Run = False;
}

static void LineEdit_OnMouseOver(TLineEdit *self, XCrossingEvent *ev, bool Leave) {
  TFormEdit *fed;
	if (!Leave) {
	    fed = (TFormEdit*)self->FormStruct;
	    if (!fed->MsgBox_Reload_Run &&
		!fed->IO_Run &&
		!fed->File_Dlg_Run &&
		!fed->Help_Info_Run &&
		!fed->Help_cmd_Run &&
		!fed->EditFind_run &&
		!fed->mEdit_Find_run &&
		!fed->EditReplace_run &&
		!fed->mEdit_Replace_run &&
		!GetAktMenueWin()) {
		Chk_LE_Reload(fed, NULL);
	    }
	}
}

static void OnAppStart() {
  TFormEdit *fed = (TFormEdit*)appDat.obj;
  int i;
  char *pc, *token;
  const char *cpc, *lpf;
  TWins *lle = NULL;

	if (fed->PageControl) {
	    if (fed->ProjFile) {
		fed->ProjFileIni = TpcIni_Create(fed->ProjFile);
		pc = path_rel2abs(fed->ProjFile, NULL);
		fed->ProjPath = GetFilePath(pc);
		xfree(pc);
// dword t=GetTime_ms();
		lpf = TpcIni_RD_pc(fed->ProjFileIni, "File", "LastPage", NULL);
		cpc = TpcIni_RD_pc(fed->ProjFileIni, "File", "files", NULL);
		while (cpc) {
		    cpc = GetToken(cpc, &token);
		    char *fn = path_rel2abs(token, fed->ProjPath);
		    LoadFile(fed, NULL, fn, 0x01 | LdF_fu_apj, NULL);
		    if (lpf && pccmp(lpf, token)) lle = (TWins*)fed->AktLE;
		    xfree(fn);
		    xfree(token);
		}
// printf("%s(%d): %dms\n", FL, GetTime_ms() - t);
		if (lle) TPage_SetPageAktiv(GetParentPage(lle));

	    } else if (fed->argv_fileIdx) {
		for (i=fed->argv_fileIdx; i<app_argc; i++) {
		    if (*app_argv[i] != '-') {
			LoadFile(fed, NULL, app_argv[i], 0x01, NULL);
		    }
		}
	    }

	} else {
	    if (fed->argv_fileIdx && (app_argc > fed->argv_fileIdx)) 
		LoadFile(fed, NULL, app_argv[fed->argv_fileIdx], 0x01, NULL);
	    fed->IO_Run = True;
	    Set_Focus((TWins*)fed->AktLE);
	    fed->IO_Run = False;
	}
	TpcIni_Save(appDat.appIni);
	ppTfree(&appDat.appIni);
}


int ChkFileIsSaved(TLineEdit *le) {
  TFormEdit *fed = (TFormEdit*)le->FormStruct;
  int res = -1;

	if (le && !Get_le_FileSaved(le)) {
	  Chk_LE_Reload(fed, le);
	  if (!Get_le_FileSaved(le)) {
	    char *efn = get_DisplayFileStr(le->tls->FileName, NULL);
	    res = MsgBox_Show_Error((TWins*)fed, "Warning !!!", 
		msgYES|msgNO|msgCANCEL, msgYES, 0,
		"The current Document has been modified.\n\"%s\"\n\nWould you like to save it ?", 
		efn);
	    xfree(efn);
	    if (res == msgYES) {
		appError_Mk = 0;
		File_Save(fed);
		if (appError_Mk) res = msgCANCEL;
	    }
	  } else res = msgCANCEL;
	}
	return res;
}

static void OnCanClose(TFormEdit *self, int *CanClose) {
  int m;
  TPage *pg;
  TLineEdit *le;

	if (self->mbSave->Enabled) {
	    if (self->MsgBox_Reload_Run) {
		*CanClose = False;
	    } else {
		if (self->PageControl) {
		  pg = TPageControl_GetFirstPage(self->PageControl, False);
		  while (pg) {
		    le = (TLineEdit*)pg->owner;
		    if (le && !Get_le_FileSaved(le)) {
			TPage_SetPageAktiv(pg);
			m = ChkFileIsSaved(self->AktLE);
			if (m == msgCANCEL) {
			    Set_Focus((TWins*)self->AktLE);
			    *CanClose = False;
			    break;
			} else if (m == msgNO) break;
		    }
		    pg = TPage_GetNext(pg, False);
		  }
		} else {

		  m = ChkFileIsSaved(self->AktLE);
		  if (m == msgCANCEL) {
		      Set_Focus((TWins*)self->AktLE);
		      *CanClose = False; 
		  }
		}
	    }
	}
	if (self->PageControl) File_SaveProj(self, 0x01);
}

static TLineEdit *Add_LineEdit(TWins *Parent) {
  TFormEdit *fed = (TFormEdit*)Parent->FormStruct;
  TLineEdit *le;
  TpcIni *pcIni;
  int sbc;
  const char *s;

	pcIni = TpcIni_Create(appDat.inifile_app);

	le = TLineEdit_CreateA(Parent, 0, 0, 200, 200, alClient);
	le->user_b0 = le->tls->DefaultEOL & fio_EOL_Mask;
#ifdef _OS_WIN32
	le->LineOffset = TpcIni_RD_int(pcIni, APPNAME, "LineOffset", 1);
	s = TpcIni_RD_pc(pcIni, APPNAME, "Font", "-*-Courier New-medium-r-normal--13-*-*-*-*-*-*-1");
	fed->miUTF8->fu &= ~miNotEnabled;
#else
	le->LineOffset = TpcIni_RD_int(pcIni, APPNAME, "LineOffset", 0);
	s = TpcIni_RD_pc(pcIni, APPNAME, "Font", "-adobe-courier-medium-r-normal--12-120-75-75-*-*-*-1");
#endif /* ! _OS_WIN32 */

	SetFont((TWins*)le, s);

	sbc = TpcIni_RD_int(pcIni, APPNAME, "MouseZ_Wheel", 3);
	TLineEdit_SetMouseZ_Wheel(le, sbc);
	le->OnKeyDown = LineEdit_OnKeyDown;
	le->OnChange = LineEdit_OnChange;
	le->OnSetCaret = LineEdit_OnSetCaret;
	le->OnMouseOver = LineEdit_OnMouseOver;
	le->VSB_DoCount = LineEdit_VSB_DoCount;
	le->UnRedo = TUnRedo_Create(200);
	le->PopupMenu = fed->menuLE;

	Tfree(pcIni);
	return le;
}

/*_____________________________ ErrorMsg ________________________________*/

static void Global_OnFileMsg(TFileMsg *fmsg) {

// file_msg_debug(fmsg, NULL);

	if ((fmsg->art == fmArt_errLogin) || (fmsg->art == fmArt_AlgErr)) {
	    default_Global_OnFileMsg(fmsg);

	} else if (fmsg->art == fmArt_strg) {
	    OnIO(fmsg);

	} else if (fmsg->art == fmArt_DataIO) {

	    OnIO(fmsg);
	}
}

/*_____________________________ Start ___________________________________*/

const char *Help_Msg = {
  "\n"
  "Usage: aldedit or aldeditpage [options] file\n"
  "\n"
  "  -P                   page mode\n"
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: aldedit version "ALD_VERSION"  (build %d  %s)\n"
};

#ifdef _OS_WIN32
static const char *DefaultIniDat = {
  "[aldedit]\n"
  "MouseZ_Wheel=4\n"
  "Width=581\n"
  "Height=588\n"
  "Font=-*-Courier New-medium-r-normal--13-*-*-*-*-*-*-1\n"
  };
#else
static const char *DefaultIniDat = {
  "[aldedit]\n"
  "MouseZ_Wheel=4\n"
  "Width=581\n"
  "Height=588\n"
  "Font=-adobe-courier-medium-r-normal--12-120-75-75-*-*-iso8859-1\n"
  };

static const char zico16aldedit[] = {
 0x06,0x0F,0x10,0x00,0x10,0x00,0x00,0x00,0x9B,0x00,0x80,0x00,0xA6,0x21,0x07,0x22,
 0x0A,0x33,0xA1,0x30,0xE6,0x39,0x28,0x42,0xE9,0x4A,0xC1,0x79,0x8E,0x73,0xD3,0x9C,
 0xE8,0xC3,0x73,0xC5,0xDB,0xDE,0xDF,0xFF,0x2C,0x4C,0x2B,0x20,0x4A,0x10,0x2D,0x4C,
 0xBF,0x2B,0x0A,0x92,0xC2,0xF4,0x3F,0xA2,0x20,0x81,0x24,0x38,0xFD,0xEF,0x26,0x82,
 0x14,0x22,0x08,0x35,0xFD,0xEF,0x56,0x36,0x44,0x28,0x02,0xD0,0xF8,0x8D,0x55,0x32,
 0x0E,0x8A,0x04,0x34,0xFE,0x8D,0x57,0x72,0xB3,0x89,0x04,0x74,0xFE,0xCD,0x2D,0x37,
 0x67,0xD9,0xB9,0x2A,0xFE,0xA3,0xEA,0xCE,0x59,0x87,0x7F,0x13,0xA0,0xEF,0xE8,0xBE,
 0x5A,0x87,0x0F,0x81,0xF0,0x28,0xB6,0x2F,0x9E,0xE3,0x43,0x24,0x34,0xBA,0xAE,0xA7,
 0xF9,0x10,0x0A,0x8C,0xBA,0x9E,0xE7,0x43,0x28,0x30,0xEA,0x0A,0x9A,0x0F,0xA1,0x20,
 0x77,0x8A,0x30,0xFA,0x1F,0x2C,0xC0,0x29,0x84,0x06,0x0B};

#endif /* ! _OS_WIN32 */

const int build_v = ALD_BUILD_V;
const char *build_date = __DATE__;

int main (int argc, char **argv) {

  int argv_fileIdx = 0;
  TFormEdit *fed;
  TpcIni *ini;
  char *ProjFile;
  const char *cpc;
  int i;
  char *val;
#ifdef _OS_WIN32
  char *pc;
#endif
  bool help = False;
  bool err = False;
  bool use_pages;
  bool use_apj = False;

#ifdef _OS_WIN32
	cmdUTF8korr(&argc, &argv);
	cpc = GetFileName_pc(argv[0]);

	use_pages = False;
	if (pccasecmp(cpc, "aldeditpage.exe")) {
	    use_pages = True;
	}
#else
	cpc = GetFileName_pc(argv[0]);
	use_pages = False;
	if (pccmp(cpc, "aldeditpage")) {
	    use_pages = True;
	}
#endif
	app_argc = argc;
	app_argv = argv;

	i = 1;
	while (!err && !help && (i<argc)) {
	    val = argv[i];
	    if (*val == '-') {
argv_loop:
		val++;
		switch (*val) {
		case 0: break;
		case 'p':
		case 'P':
		    use_pages = True;
		    goto argv_loop;
		case 'h':
		    help = True;
		    goto argv_loop;
		case '-':
		    val++;
		    if (pcncmp(val, "help", 4)) {
			help = True;
#ifdef Test_Root_WinId
		    } else if (pcncmp(val, "TestRootWinID", 13)) {
			i++;
			val = argv[i];
			if (val && (*val != '-')) {
			    TestRootWinId = pctoint(val);
			} else err = True;
#endif
		    } else err = True;
		    break;
		default: 
		    err = True;
		}
	    } else {
		if (!argv_fileIdx) {
		    argv_fileIdx = i;
		} else {
		    use_pages = True;
		}
	    }
	    i++;
	}

	if (use_pages && !use_apj && argv_fileIdx) {
	    cpc = argv[argv_fileIdx];
	    if (cpc && *cpc) {
		cpc = GetFileExt_pc(cpc);
		use_apj = (pccasecmpASCII(cpc, "apj") == 0) || 
			  (pccasecmpASCII(cpc, "aldapj") == 0);
	    }
	}

	ProjFile = NULL;
	if (use_apj) {
	    if (argv_fileIdx) {
		ProjFile = argv[argv_fileIdx];
		if (!ProjFile || !*ProjFile) err = True;
		argv_fileIdx = 0;
	    } else err = True;
	}

	if (help || err) {
	    printf(Help_Msg, build_v, build_date);
	    exit(err);
	}

	initXapp(0, argc, argv);
#ifdef _OS_WIN32

	pc = pcadd(appDat.ALDini_dir, "/share/icons/16x16/apps");
	ZsrcToPath(&hZsrc_aldedit16_png, pc, NULL, ZsrcFu_OnlyNotExists);
	xfree(pc);

	pc = pcadd(appDat.ALDini_dir, "/share/icons/32x32/apps");
	ZsrcToPath(&hZsrc_aldedit32_png, pc, NULL, ZsrcFu_OnlyNotExists);
	xfree(pc);

	i = 0;
	pc = pcadd(appDat.ALDini_dir, "/applications");
	if (ZsrcToPath(&hZsrc_aldedit_desktop, pc, NULL, ZsrcFu_OnlyNotExists) == ZsrcOk) i = 1;
	if (ZsrcToPath(&hZsrc_aldeditpage_desktop, pc, NULL, ZsrcFu_OnlyNotExists) == ZsrcOk) i = 1;
	xfree(pc);

	if (i) {
	    pc = pcadd(appDat.ALDini_dir, "/share/applications");
	    ZsrcToPath(&hZsrc_aldedit_desktop, pc, NULL, ZsrcFu_OnlyNotExists);
	    xfree(pc);

	    pc = pcadd(appDat.ALDini_dir, "/Menu/OpenWith");
	    ZsrcToPath(&hZsrc_aldedit_desktop, pc, NULL, ZsrcFu_OnlyNotExists);
	    ZsrcToPath(&hZsrc_aldeditpage_desktop, pc, NULL, ZsrcFu_OnlyNotExists);
	    xfree(pc);
	}

	appDat.inifile_app = pcadd(appDat.ALDini_dir, "\\edit.ini");
	ThDirList_Reg(1, &hdirlist_ld_File);
	ThDirList_ny_Reg(1, &hdirlist_ny_File);
#else  /* _OS_WIN32 */
	appDat.inifile_app = pcadd(appDat.ALDini_dir, "/edit.ini");
//	ThDirList_Reg(1, &hdirlist_ld_File);
	ThDirList_Reg(2, &hdirlist_ld_File, &hdirlist_ld_ftp);
	ThDirList_ny_Reg(2, &hdirlist_nyi_File, &hdirlist_ny_File);
	ThNetLineBufLoader_Reg(1, &hNetLineBufLd_ftp);
	ThNetStatLoader_Reg(1, &hNetStatLd_ftp);
#endif /* ! _OS_WIN32 */
	ThImageRD_Reg(1, &hImageRD_xpm);
#ifdef USE_PRINT_DLG
	ThImageWR_Reg(1, &hImageWR_png);
#endif


	hGetFileHeadInfo = &hGetFileHeadInfo_buildin;

	fed = TForm_CreatParamAlg(NULL, sizeof(TFormEdit));
	appDat.obj = fed;

#ifdef USE_DECOR
	appXDat.DefaultButtonDecor = TDecor_button_w3b3_Create();
	appXDat.DefaultWinsDecor = TDecor_w3b3_Create(&TDecoBuR3B3_up_Tab, &TDecoBuR3B3_down_Tab);
#endif
#ifndef _OS_WIN32
	fed->Icon = GetDeskIcon_z(ddkt_Application, 16, APPNAME, 
				    gdifu_noZoom, zico16aldedit);
#endif

	fed->argv_fileIdx = argv_fileIdx;
	fed->ProjFile = pcdup(ProjFile);

	setDefaultIniDat(DefaultIniDat);

	ini = TpcIni_Create(appDat.inifile_app);
	appDat.appIni = ini;

	fed->OnCanClose = OnCanClose;
	SetCaption((TWins*)fed, "FormMain");
	fed->IconName = APPNAME;
	fed->height = TpcIni_RD_int(ini, APPNAME, "Height", 500);
	fed->width = TpcIni_RD_int(ini, APPNAME, "Width", 440);
	fed->min_width = 280;
	fed->min_height = 88;

	fed->PanelTop = TWins_CreateBA((TWins*)fed, 0, 0, 200, 21, mBorder | mBorderL, alTop);


// ******* Menu *******
	MenuFile_Create(fed, use_pages);
	MenuEdit_Create(fed);
	MenuTools_Create(fed);
	MenuSetting_Create(fed);
	MenuHelp_Create(fed);
	MenuSave_Create(fed);
	MenuUnRedo_Create(fed);

	TMenuButton_SetPosition(fed->mbFile);

// ******* Menu LineEdit *******
	MenuEditLE_Create(fed, use_pages);

// ******* LineEdit *******
	TLineEdit_AddKeyDownPlugIn(NULL, TLineEdit_keyplugin_edit_Create());
	TLineEdit_AddKeyDownPlugIn(NULL, TLineEdit_keyplugin_strg_u_i_Create());
	TLineEdit_AddKeyDownPlugIn(NULL, TLineEdit_keyplugin_cursor_Create());

	if (use_pages) {
	    fed->PageControl = TPageControl_CreateA((TWins*)fed);
	    fed->PageControl->OnChangeTab = OnChangeTab;
	} else {
	    fed->AktLE = Add_LineEdit((TWins*)fed);
#ifdef _OS_WIN32
	    fed->AktLE->vds.UTF8on = False;
#endif
	}

// ******* PanelBottom *******
	fed->PanelBottom = TWins_CreateBA((TWins*)fed, 0, 0, 200, 18, 0, alBottom);

	fed->CursPosBox = TWins_CreateLabel(fed->PanelBottom, 0, 1, 99, 17, "0:  0", 0, mBorder | mBorderL);
	fed->SBVPosBox = TWins_CreateLabel(fed->PanelBottom, 100, 1, 90, 17, "0", 0, mBorder | mBorderL);
	fed->CharAzBox = TWins_CreateLabel(fed->PanelBottom, 191, 1, 96, 17, "255/FF", 0, mBorder | mBorderL);

	fed->ProgressBar = TProgressBar_CreateU(fed->PanelBottom, 254, 1, 100, 17);
	fed->kpsBox = TWins_CreateLabel(fed->PanelBottom, 356, 1, 90, 17, NULL, 0, mBorder | mBorderL);

	SetVisible((TWins*)fed->ProgressBar, False);
	SetVisible(fed->kpsBox, False);
	fed->LdWrTimer = NULL;

	fed->Find_Dlg = Create_Find_Dlg((TWins*)fed);
	fed->Replace_Dlg = Create_Replace_Dlg((TWins*)fed);

	Glob_OnFileMsg = Global_OnFileMsg;
	appError_ev = default_Error_ev;
	appXDat.OnAppStart = OnAppStart;

	if (use_pages) {
	  fed->PanelBottom->OnResize = PanelBottom_OnResize;
	  fed->buPrevPage = TButton_CreateS(fed->PanelBottom, 220, 1, 17, 17, NULL);
	  fed->buPrevPage->KeyFu = 0;
	  fed->buPrevPage->OnPaint = buPrevPage_OnPaint;
	  fed->buPrevPage->OnClick = buPrevPage_OnClick;
	  fed->buNextPage = TButton_CreateS(fed->PanelBottom, 200, 1, 17, 17, NULL);
	  fed->buNextPage->KeyFu = 0;
	  fed->buNextPage->OnPaint = buNextPage_OnPaint;
	  fed->buNextPage->OnClick = buNextPage_OnClick;
	}
	AppRun();

#ifdef MEM_CHK
  Tfree(fed->File_Dlg);
  Tfree(fed->Find_Dlg);
  Tfree(fed->Replace_Dlg);
  Tfree(fed);
  int lMCt;
  size_t lMsize;
  MEM_CHK_Count(&lMCt, &lMsize);
  printf("Debug %s(%d): MEM_CHK_Count=%3d  size=%ld wins=%d\n", FL, lMCt, (long)lMsize, GetTWinsCount());
#endif

	return 0;
}



