/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldedit.h"
#include "cicons.h"

static void menuEdit_ChkTag1(TFormEdit *fed, TMenuItem *mi) {
  TLineEdit *le = fed->AktLE;
  bool mk = False;
  bool xaxe0 = False;

	if (fed->PageControl && 
	    !TPageControl_GetPageCount(fed->PageControl, False)) le = NULL;

	if (le) mk = TLineEdit_ChkIsSel(le);
	if (mk) xaxe0 = (!le->SelAx && !le->CaretCharIdx);
	while (mi) {
	    if (mi->tag == 0) {
		mi->fu &= ~miNotEnabled;
		if (!le) mi->fu |= miNotEnabled;
	    } else if (mi->tag == 1) {
		if (mk) mi->fu &= ~miNotEnabled; else mi->fu |= miNotEnabled;
	    } else if (mi->tag == 2) {
		if (xaxe0) mi->fu &= ~miNotEnabled; else mi->fu |= miNotEnabled;
	    }
	    mi = mi->next;
	}
}

static void menuEdit_OnMenuOpen(TMenu *self) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	menuEdit_ChkTag1(fed, fed->menuEdit->MenuItem);
}

static void menuLE_OnMenuOpen(TMenu *self) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	menuEdit_ChkTag1(fed, fed->menuLE->MenuItem);
}

/* ------------------------------------------------- */
static void mEdit_Cut(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	TLineEdit_Strg_X(fed->AktLE);
}

static void mEdit_Copy(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	TLineEditBase_Strg_C((TLineEditBase*)fed->AktLE);
}

static void mEdit_Paste(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	TLineEdit_Strg_V(fed->AktLE);
}

static void mEdit_SelectAll(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
	TLineEdit_SetSel(fed->AktLE, 0, 0, 
		0, fed->AktLE->tls->Count);
}

static const char *No_Find_Msg = {
  "\"%s\"\n"
  "End of document\n"
  "Continue from the beginning ?"
};


void EditFind(TFormEdit *fed) {
  TLines *tls = fed->AktLE->tls;
  TFind_Dlg *Find_Dlg = fed->Find_Dlg;
  int m;
  char line[1024];

	if (fed->EditFind_run) return;
	fed->EditFind_run = True;
	if (!Find_Dlg->edFind->vds.s) {
	    Find_Dlg_SetUTF8(fed->Find_Dlg, fed->AktLE->vds.UTF8on);
	    Find_Dlg_Execute(Find_Dlg, NULL);
	}
	if (Find_Dlg->edFind->vds.s) {
	    tls->FindXa = fed->AktLE->CaretCharIdx;
	    tls->FindYa = fed->AktLE->CaretCharIdy;
loopAnf:
	    tls->FindNoCase = ((TChkBox*)Find_Dlg->cbCase)->Checked;
	    if (TLines_FindS(tls, Find_Dlg->edFind->vds.s)) {
		TLineEdit_SetSel(fed->AktLE, 
			tls->FindXa, tls->FindYa, tls->FindXe, tls->FindYe);
		TLineEdit_ChkCaret(fed->AktLE, LE_ChkCaret_Ymiddle);
	    } else {
		snprintf(line, 1024, No_Find_Msg, Find_Dlg->edFind->vds.s);
		m = MsgBox_Show((TWins*)fed, "Find - ALDedit", line, msgYES | msgNO, msgYES);
		if (m == msgYES) {
		    tls->FindXa = 0;
		    tls->FindYa = 0;
		    goto loopAnf;
		}
	    }
	}
	fed->EditFind_run = False;
}

void TFormEdit_Find(TFormEdit *fed) {
  Tlgs lgs;

	if (fed->mEdit_Find_run) return;
	fed->mEdit_Find_run = True;
	TLineEditBase_GetSel_lgs((TLineEditBase*)fed->AktLE, &lgs, 0);
	Find_Dlg_SetUTF8(fed->Find_Dlg, fed->AktLE->vds.UTF8on);
	if (Find_Dlg_Execute(fed->Find_Dlg, lgs.s)) EditFind(fed);
	Tlgs_Clr(&lgs);
	fed->mEdit_Find_run = False;
}

static void mEdit_Find(TMenu *self, long KeyState, TMenuItem *smi) {
	TFormEdit_Find((TFormEdit*)self->FormStruct);
}

static bool sFind_Caret(TFormEdit *fed, TLineEdit *le) {
  TLines *tls = le->tls;
	tls->FindXa = le->CaretCharIdx;
	tls->FindYa = le->CaretCharIdy;
	if (!TLines_FindS(tls, fed->Find_S)) return False;
	le->SelAx = tls->FindXa;
	le->SelAy = 	tls->FindYa;
	le->CaretCharIdx = tls->FindXe;
	le->CaretCharIdy = tls->FindYe;
	return True;
}

static void Replace_MsgBox_OnButton(TMsgBox *mb) {
  TFormEdit *fed = (TFormEdit*)mb->Parent;
  TLineEdit *le = fed->AktLE;
  int ms;
  bool mk;

	if (mb->intern_approc_count > 1) {
	    mb->OnButton_UserMk = 0;
	    return;
	}
	if ((mb->RetCode != msgYES) && 
	    (mb->RetCode != msgYESALL) && 
	    (mb->RetCode != msgNO)) return;

	mb->OnButton_UserMk = 1;
	ms = GetTime_ms() + 40;
loop:
	if ((mb->RetCode == msgYES) || (mb->RetCode == msgYESALL)) {
	    TLineEdit_DoChange(le, ur_DelSel, NULL, NULL);
	    if (fed->Replace_lgs.lg) 
		TLineEdit_DoChange(le, ur_InsStr | ur_DupTlgs, NULL, &fed->Replace_lgs);
	}

	mk = sFind_Caret(fed, le);
	if (mk && mb->OnButton_UserMk) {
	    if (mb->RetCode == msgYESALL) {
		if (cpTime(ms, GetTime_ms())) {
		    TLineEdit_SetCaret(le);
		    TLineEdit_ChkCaret(le, LE_ChkCaret_Ymiddle);
		    Refresh((TWins*)le);
		    AppProcessMessages();
		    ms = GetTime_ms() + 40;
		}
		goto loop;
	    }
	}
	TLineEdit_SetCaret(le);
	TLineEdit_ChkCaret(le, LE_ChkCaret_Ymiddle);
	Refresh((TWins*)le);
	if (mk && ((mb->RetCode == msgYES) || (mb->RetCode == msgNO))) mb->RetCode = 0;
}

static void EditReplace(TFormEdit *fed) {
  TLineEdit *le = fed->AktLE;
  TLines *tls = le->tls;
  char line[1024];
  char *s, *srp;
  int x, y;

	if (fed->EditReplace_run) return;
	fed->EditReplace_run = True;
	if (TLineEdit_ChkIsSel(le)) {
	    x = le->SelAx;
	    y = le->SelAy;
	    if (y == le->CaretCharIdy) {
		if (le->SelAx > le->CaretCharIdx) x = le->CaretCharIdx;
	    } else if (y > le->CaretCharIdy) {
		x = le->CaretCharIdx;
		y = le->CaretCharIdy;
	    }
	    TLineEdit_SetSel(le, x, y, x, y);
	}

	if (!fed->Replace_Dlg->edFind->vds.s) {
	    Replace_Dlg_SetUTF8(fed->Replace_Dlg, fed->AktLE->vds.UTF8on);
	    Replace_Dlg_Execute(fed->Replace_Dlg, NULL);
	}
	s = fed->Replace_Dlg->edFind->vds.s;
	if (s) {
	    srp = fed->Replace_Dlg->edRepl->vds.s;
	    tls->FindNoCase = fed->Replace_Dlg->cbCase->Checked;

	    fed->Find_S = s;
	    fed->Replace_lgs.s = srp;
	    fed->Replace_lgs.lg = pclen(srp);

	    if (sFind_Caret(fed, le)) {
		TLineEdit_SetCaret(le);
		TLineEdit_ChkCaret(le, LE_ChkCaret_Ymiddle);
		Refresh((TWins*)le);

		TMsgBoxParam mbp;
		mbp.akt = mbpakt_x_y|mbpakt_FocusButton|mbpakt_OnButton|mbpakt_UTF8;
		mbp.left = fed->Replace_Dlg->left;
		mbp.top = fed->Replace_Dlg->top;
		mbp.Caption = "Replace - ALDedit";
		mbp.FocusButton = msgYES;
		mbp.OnButton = Replace_MsgBox_OnButton;
		mbp.UTF8on = fed->AktLE->vds.UTF8on;
		
		snprintf(line, sizeof(line), "Replace\n\"%s\"\n\nWith\n\"%s\"", s, srp);
		MsgBox_Show_mbp((TWins*)fed, line, 
			msgYES | msgYESALL | msgNO | msgCANCEL, &mbp);
		fed->Replace_Dlg->left = mbp.left;
		fed->Replace_Dlg->top = mbp.top;
	    }
	}
	fed->EditReplace_run = False;
}

static void mEdit_Replace(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
  Tlgs lgs;

	if (fed->mEdit_Replace_run) return;
	fed->mEdit_Replace_run = True;
	TLineEditBase_GetSel_lgs((TLineEditBase*)fed->AktLE, &lgs, 0);
	Replace_Dlg_SetUTF8(fed->Replace_Dlg, fed->AktLE->vds.UTF8on);
	if (Replace_Dlg_Execute(fed->Replace_Dlg, lgs.s)) EditReplace(fed);
	fed->Replace_Dlg->FormPosition = poDesigned;
	Tlgs_Clr(&lgs);
	fed->mEdit_Replace_run = False;
}

static void mEdit_LinePlus2(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
  TKeyEvent KeyEv;
	Fill0(&KeyEv, sizeof(TKeyEvent));
	KeyEv.keycode = 31;
	KeyEv.keyst = keyst_Press;
	KeyEv.state = 0x04;
	if (fed->AktLE && fed->AktLE->DoKeyDown) 
	    fed->AktLE->DoKeyDown(fed->AktLE, &KeyEv);
}

static void mEdit_LineMinus2(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormEdit *fed = (TFormEdit*)self->FormStruct;
  TKeyEvent KeyEv;
	Fill0(&KeyEv, sizeof(TKeyEvent));
	KeyEv.keycode = 30;
	KeyEv.keyst = keyst_Press;
	KeyEv.state = 0x04;
	if (fed->AktLE && fed->AktLE->DoKeyDown) 
	    fed->AktLE->DoKeyDown(fed->AktLE, &KeyEv);
}

/* ______________________ Create _____________________________________ */
void MenuEdit_Create(TFormEdit *fed) {
  TMenuItem *mi;

	fed->mbEdit = TMenuButton_Create(fed->PanelTop, fed->mbFile);
	SetCaption((TWins*)fed->mbEdit, "Edit");
	Set_Parent((TWins*)fed->mbEdit, fed->PanelTop);

	fed->menuEdit = TMenu_Create((TWins*)fed);
	fed->menuEdit->OnMenuOpen = menuEdit_OnMenuOpen;
	fed->mbEdit->menu = fed->menuEdit;
	mi = TMenu_AddItem(fed->menuEdit, "Cut", miNotEnabled, mEdit_Cut);
	mi->tag = 1;
	mi->s2 = pcdup("Strg x");
	mi = TMenu_AddItem(fed->menuEdit, "Copy", miNotEnabled, mEdit_Copy);
	mi->tag = 1;
	mi->s2 = pcdup("Strg c");
	mi = TMenu_AddItem(fed->menuEdit, "Paste", 0, mEdit_Paste);
	mi->s2 = pcdup("Strg v");
	TMenu_AddItem(fed->menuEdit, "Select All", 0, mEdit_SelectAll);
	TMenu_AddItem(fed->menuEdit, "-", 0, NULL);
	mi = TMenu_AddItem(fed->menuEdit, "Find", 0, mEdit_Find);
	mi->s2 = pcdup("Strg f");
	TMenu_AddItem(fed->menuEdit, "Replace", 0, mEdit_Replace);
	TMenu_AddItem(fed->menuEdit, "-", 0, NULL);
	mi = TMenu_AddItem(fed->menuEdit, "Select line +2", 0, mEdit_LinePlus2);
	mi->tag = 2;
	mi->s2 = pcdup("Strg i");
	mi = TMenu_AddItem(fed->menuEdit, "Select line -2", 0, mEdit_LineMinus2);
	mi->s2 = pcdup("Strg u");
	mi->tag = 2;

}

void MenuEditLE_Create(TFormEdit *fed, bool use_pages) {
  TMenuItem *mi;

	fed->menuLE = (TMenu*)TMenu_Create((TWins*)fed);
	fed->menuLE->OnMenuOpen = menuLE_OnMenuOpen;

	if (use_pages) {
	  TMenu_AddItem(fed->menuLE, "Close Page", 0, mFile_ClosePage);
	  mi = TMenu_AddItem(fed->menuLE, "Change Page", 0, mFile_ChangePage);
	  mi->pic = get_pic16_fileopen(False);
	  TMenu_AddItem(fed->menuLE, "-", 0, NULL);
	}

	mi = TMenu_AddItem(fed->menuLE, "Open...", 0, mFile_Open);
	mi->pic = get_pic16_fileopen(False);
	mi = TMenu_AddItem(fed->menuLE, "Reopen", 0, NULL);
		mi->next_menu = fed->menuReopen;
	TMenu_AddItem(fed->menuLE, "-", 0, NULL);
	fed->miLESave = TMenu_AddItem(fed->menuLE, "Save", miNotEnabled, mFile_Save);
	fed->miLESave->pic = get_pic16_filesave(False);
	mi = TMenu_AddItem(fed->menuLE, "Save As...", 0, mFile_SaveAs);
	mi->pic = get_pic16_filesave(False);
	TMenu_AddItem(fed->menuLE, "-", 0, NULL);
	mi = TMenu_AddItem(fed->menuLE, "Cut", miNotEnabled, mEdit_Cut);
	mi->tag = 1;
	mi = TMenu_AddItem(fed->menuLE, "Copy", miNotEnabled, mEdit_Copy);
	mi->tag = 1;
	TMenu_AddItem(fed->menuLE, "Paste", 0, mEdit_Paste);
	TMenu_AddItem(fed->menuLE, "Select All", 0, mEdit_SelectAll);
	TMenu_AddItem(fed->menuLE, "-", 0, NULL);
	TMenu_AddItem(fed->menuLE, "Find", 0, mEdit_Find);
	TMenu_AddItem(fed->menuLE, "Replace", 0, mEdit_Replace);
}
