/*
 Copyright (C) 2010-2011 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldedit.h"

/* ------------------------------------------------- */

typedef struct {
  TFormEdit *fmain;
  TLineEdit *le;
  TLines *tls;
  TLineBuf *lb;
  int sb_y, ca_x, ca_y;
  int fuEOL;
  bool UTF8on;
} Tle_conv;

static bool cv_init(TMenu *self, Tle_conv *cv) {
	cv->fmain = (TFormEdit*)self->FormStruct;
	cv->le = cv->fmain->AktLE;

	if (!cv->le) return False;
	cv->tls = cv->le->tls;
	if (!cv->tls || !cv->tls->Count) return False;

	cv->fuEOL = 0;
  int idx = 0;
	while (idx < cv->tls->Count) {
	    TColS *pcs = cv->tls->lines->cs[idx];
	    if (pcs) Chk_fio_UTF8(pcs->s, pcs->lg, &cv->fuEOL);
	    idx++;
	}
	return True;
}

static void lb_load_le_clr(Tle_conv *cv, int fuEOL) {
  TLineEdit *le = cv->le;
  int idx;

	cv->sb_y = TScrollBar_GetCount(le->VSB);
	cv->ca_x = le->CaretCharIdx;
	cv->ca_y = le->CaretCharIdy;
	cv->lb = TLineBuf_Create();
	idx = 0;
	while (idx < cv->tls->Count) {
	    TColS *pcs = cv->tls->lines->cs[idx];
	    if (pcs) {
		fuEOL &= ~fio_EOL_Mask;
		fuEOL |= GetDefaultEOL_fu(cv->tls->DefaultEOL, pcs->fu);
		TLineBuf_SetLineEOL_lgs(cv->lb, (Tlgs*)pcs, &fuEOL);
	    }
	    idx++;
	}

	TUnRedo_Clr(le->UnRedo);
	TLineEdit_Clr(le);
	SetEnabled((TWins*)cv->fmain->mbUndo, False);
	SetEnabled((TWins*)cv->fmain->mbRedo, False);

  Tlgs lgs;
	while (TLineBuf_GetLineEOL_lgs(cv->lb, &lgs, &cv->fuEOL)) {
	    TLines_Add_lgs(cv->tls, &lgs, cv->fuEOL);
	}
	Tfree(cv->lb);
}

static void le_refresh(Tle_conv *cv) {
  TLineEdit *le = cv->le;

	TScrollBar_SetCount_ev(le->VSB, cv->sb_y);
	le->CaretCharIdx = cv->ca_x;
	le->CaretCharIdy = cv->ca_y;
	TLineEdit_ChkCaret(le, 0);
	le->SelAx = le->CaretCharIdx;
	le->SelAy = le->CaretCharIdy;
	TLineEdit_SetCaret(le);

	le->vds.UTF8on = cv->UTF8on;
	cv->fmain->miUTF8->fu &= ~miChecked;
	if (cv->UTF8on) cv->fmain->miUTF8->fu |= miChecked;

	TLineEdit_AllAtrNew(le);
}

static void mAsciiToUTF8(TMenu *self, long KeyState, TMenuItem *smi) {
  Tle_conv cv;

	if (!cv_init(self, &cv)) return;

	if ((cv.fuEOL & (fio_has_UTF8_char | fio_err_UTF8_char)) == fio_has_UTF8_char) {
	    if (MsgBox_Show((TWins*)cv.fmain, "WARNING", 
			"WARNING: Text is UTF8", 
			msgIcoWARNING|msgOK|msgCANCEL, msgCANCEL) == msgCANCEL) return;
	}

	cv.fuEOL = fio_txtTyp_Unicode;
	lb_load_le_clr(&cv, fio_txtTyp_Unicode | fio_force_ASCII);

	cv.UTF8on = True;
	le_refresh(&cv);
}

static void mU16ToUTF8(TMenu *self, long KeyState, TMenuItem *smi) {
  Tle_conv cv;

	if (!cv_init(self, &cv)) return;
// printf("%s(%d): 0x%04x\n", FL, cv.fuEOL);

	if ((cv.fuEOL & (fio_has_UTF8_char | fio_err_UTF8_char)) == fio_has_UTF8_char) {
	    if (MsgBox_Show((TWins*)cv.fmain, "ERROR", 
			"ERROR: Text is UTF8", 
			msgIcoERROR|msgFORCE|msgCANCEL, msgCANCEL) != msgFORCE) return;
	} else if (!(cv.fuEOL & fio_has_0x00_char)) {
	    if (MsgBox_Show((TWins*)cv.fmain, "ERROR", 
			"ERROR: Text is not U16 (Unicode)", 
			msgIcoERROR|msgFORCE|msgCANCEL, msgCANCEL) != msgFORCE) return;
	} else {
	    if (MsgBox_Show((TWins*)cv.fmain, "WARNING", 
			"WARNING: Text loss of data !!!", 
			msgIcoWARNING|msgOK|msgCANCEL, msgCANCEL) == msgCANCEL) return;
	}

	cv.fuEOL = fio_txtTyp_Unicode;
	lb_load_le_clr(&cv, fio_txtTyp_ANSI);

	cv.UTF8on = True;
	le_refresh(&cv);
}

static void mUTF8ToAscii(TMenu *self, long KeyState, TMenuItem *smi) {
  Tle_conv cv;

	if (!cv_init(self, &cv)) return;

	if (cv.fuEOL & fio_has_gt_0xff_char) {
	    if (MsgBox_Show((TWins*)cv.fmain, "WARNING", 
			"WARNING: Text loss of data !!!", 
			msgIcoWARNING|msgOK|msgCANCEL, msgCANCEL) == msgCANCEL) return;
	}

	cv.fuEOL = fio_txtTyp_Unicode | fio_force_ASCII;
	lb_load_le_clr(&cv, fio_txtTyp_Unicode);


	cv.UTF8on = (cv.fuEOL & (fio_has_UTF8_char|fio_err_UTF8_char)) == fio_has_UTF8_char;
	le_refresh(&cv);
}

/* ______________________ Create _____________________________________ */
void MenuTools_Create(TFormEdit *fmain) {
  TMenuItem *mi;

	fmain->mbTools = TMenuButton_Create(fmain->PanelTop, fmain->mbFile);
	SetCaption((TWins*)fmain->mbTools, "Tools");
	Set_Parent((TWins*)fmain->mbTools, fmain->PanelTop);

	fmain->menuTools = TMenu_Create((TWins*)fmain);
	fmain->mbTools->menu = fmain->menuTools;


	mi = TMenu_AddItem(fmain->menuTools, "Convert", 0, NULL);
	  fmain->menuConvert = TMenu_Create((TWins*)fmain);
	  mi->next_menu = fmain->menuConvert;
	  TMenu_AddItem(fmain->menuConvert, "ASCII -> UTF8", 0, mAsciiToUTF8);
	  TMenu_AddItem(fmain->menuConvert, "UTF8 -> ASCII", 0, mUTF8ToAscii);
	  TMenu_AddItem(fmain->menuConvert, "U16 -> UTF8", 0, mU16ToUTF8);
}
