/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldesk.h"

static bool di_ChkDeskPos(TDesktopIcon *self, TwmRoot *wmr, int cx, int cy) {
  bool ok = True;
  TWins *wins = wmr->next_child;
	while (ok && wins) {
	    if ((wins != (TWins*)self) && (wins->wtyp == wtyp_TwmDeskIcon)) {
		if (((wins->left > (cx-29)) && 
		    (wins->left < (cx+29))) &&
		    ((wins->top  > (cy-44)) && 
		    (wins->top  < (cy+44)))) ok = False;
	    }
	    wins = wins->next;
	}
	return ok;
}

static int di_GetMaxRight() {
	return dpyMonitors.mon[0].right - 32;
}
static int di_GetMaxBottom() {
	return dpyMonitors.mon[0].bottom - 72;
}

static void di_GetNewDeskPos(TDesktopIcon *self, TwmRoot *wmr, int *pcx, int *pcy) {
  int cx, cy;
  int max_right = di_GetMaxRight();
  int max_bottom = di_GetMaxBottom();

	cx = 15; cy = 5;
	while (cy < max_bottom) {
	    if (cx > max_right) {cx = 15; cy += 55;}
	    if (di_ChkDeskPos(self, wmr, cx, cy)) {
		*pcx = cx;
		*pcy = cy;
		return;
	    }
	    cx += 55;
	}
}

static void TDesktopIcon_DoPaint(TDesktopIcon *self) {

	if (!self->pic) return;
	if (self->pic->mask != self->LastIcoMask) {
	    self->LastIcoMask = self->pic->mask;
	    CombineMask(self->WinID, self->pic->mask);
	}
	DrawPic(self->WinID, self->pic, 0, 0);
}

static void TDesktopIconTw_DoPaint(TWins *self) {
  TDesktopIcon *di = (TDesktopIcon*)self->owner;

	if (di->glob->mod & (wmDImod_HighLight | wmDImod_Transparent)) {
	    di->WinCaption->vds.ForeColor = 0x404040;
	    DrawColorString(2, 0, &self->vds);
	    DrawColorString(2, 2, &self->vds);
	    DrawColorString(1, 1, &self->vds);
	    DrawColorString(3, 1, &self->vds);
	    di->WinCaption->vds.ForeColor = 0xffffff;
	    DrawColorString(2, 1, &self->vds);
	} else {
	    DrawColorString(2, 1, &self->vds);
	}
}

TDesktopIcon REGP1 *get_owner_DesktopIcon(TWins *self) {
  TWins *owner;

	if (self->wtyp == wtyp_TwmDeskIcon) return (TDesktopIcon*)self;
	owner = self->owner;
	if (owner && (owner->wtyp == wtyp_Wins)) owner = owner->owner;
	if (!owner || (owner->wtyp != wtyp_TwmDeskIcon)) return NULL;
	return (TDesktopIcon*)owner;
}

static void TDesktopIcon_OnTimer(TgTimer *gti) {
  TWins *ws = gti->obj;
  TDesktopIcon *di = get_owner_DesktopIcon(ws);

	if (!di) return;
	di->IcoButtonUpTimer = NULL;
	Set_Cursor(ws, XC_fleur);
}

static void TDesktopIcon_DoMouseDown(TWins *self, XButtonEvent *ev) {
  TDesktopIcon *di = get_owner_DesktopIcon(self);

	if (!di) return;
	if (self->cursor_nr == XC_watch) return;
	ppTfree(&di->IcoButtonUpTimer);
	MouseDown.obj_left = di->left;
	MouseDown.obj_top = di->top;
	if (ev->button == Button1) {
	    di->IcoButtonUpTimer = gTimerX_Add(Timer_Enabled | Timer_OnTimerAndFree, 400, TDesktopIcon_OnTimer);
	    di->IcoButtonUpTimer->obj = self;
	}
	TWins_DoMouseDown(self, ev);
}

static void TDesktopIcon_DoMouseUp(TWins *self, XButtonEvent *ev) {
  TDesktopIcon *di;
  TpcIni *ini;
  const char *fn;
  char Dgr[32], Dxy[32];

	di = get_owner_DesktopIcon(self);
	if (!di) return;
	if (self->cursor_nr == XC_watch) return;
	ppTfree(&di->IcoButtonUpTimer);
	TWins_DoMouseUp(self, ev);

	ini = NULL;
	if (self->cursor_nr == XC_fleur) {

	    if (!di_ChkDeskPos(di, (TwmRoot*)di->prev_child, di->left, di->top)) {
		int x = di->left;
		int y = di->top;
		di_GetNewDeskPos(di, (TwmRoot*)di->prev_child, &x, &y);
		SetPosition((TWins*)di, x, y);
		SetPosition(di->WinCaption, x + di->xo, y + di->yo);
	    }

	    fn = GetFileName_pc(di->fname);
	    ini = TpcIni_Create(((TwmRoot*)di->prev_child)->IconPosFile);
	    snprintf(Dgr, 32, "%dx%d", dpyMonitors.mon[0].right, dpyMonitors.mon[0].bottom);
	    snprintf(Dxy, 32, "%d %d", di->left, di->top);
	    TpcIni_WR(ini, fn, Dgr, Dxy);
	    TpcIni_Save(ini);
		
	} else if (appXDat.GrabWin) {
//	    if (ev->button != Button1) Std_UngrabPointer();

	} else if (ev->button == Button1) {
	    switch(di->ddkt) {
	    case ddkt_Application:
		ExecDesktopFile(di->fname, NULL);
		break;

	    case ddkt_FSDevice:
		ini = TpcIni_Create(di->fname);
		if (di->pic == di->pic3) {
		    meDeskIcon_mount_Click(di, ini);
		} else if (di->pic == di->pic2) {
		    meDeskIcon_umount_Click(di, ini);
		}
		break;
	    } //switch
	}
	Tfree(ini);
	if (self->cursor_nr != XC_watch) Set_Cursor(self, XC_top_left_arrow);
}

static void TDesktopIcon_DoMouseMove(TWins *self, XMotionEvent *ev) {
  int x, y;
  TDesktopIcon *di = get_owner_DesktopIcon(self);

	if (!di) return;
	if ((self->cursor_nr == XC_fleur) &&
	    (ev->state & Button1Mask) && 
	    (MouseDown.apph == (TWins*)self))
	{

	    x = MouseDown.obj_left + (ev->x_root - MouseDown.x_root);
	    y = MouseDown.obj_top + (ev->y_root - MouseDown.y_root);

	    if (x > di_GetMaxRight()) x = di_GetMaxRight();
	    if (y > di_GetMaxBottom()) y = di_GetMaxBottom();
	    if (x < 0) x = 0;
	    if (y < 0) y = 0;

	    x = (x / 5) * 5;
	    y = (y / 5) * 5;
	    SetPosition((TWins*)di, x, y);
	    SetPosition(di->WinCaption, x + di->xo, y + di->yo);
	}
	TWins_DoMouseMove(self, ev);
}

static bool TDesktopIcon_ChkMouseOverIcoCaption(TDesktopIcon *di, int x, int y) {
  TWins *tw = di->WinCaption;

	x -= tw->left;
	y -= tw->top;
	return ((x >= 0) && (x < tw->width) && (y >= 0) && (y < tw->height));
}

static void TDesktopIcon_DoMouseOver(TWins *self, XCrossingEvent *ev, bool Leave) {
  TDesktopIcon *di = get_owner_DesktopIcon(self);

	if (!di) return;
	if (Leave) {
	    if (!(di->WinCaption->KeyFu & mHasFocus) && 
		!TDesktopIcon_ChkMouseOverIcoCaption(di, ev->x_root, ev->y_root))
		    TDesktopIcon_SetFunkt(di, DIFu_Default);
	} else {
	    if (di->WinCaption->KeyFu & mHasFocus) {
		TDesktopIcon_SetFunkt(di, DIFu_Focus);
	    } else {
		TDesktopIcon_SetFunkt(di, DIFu_HighLight);
	    }
	}
	TWins_DoMouseOver(self, ev, Leave);
}

static void TDesktopIconTw_DoFocus(TWins *self, int Change) {
	TWins_DoFocus(self, Change);
	int m = DIFu_Default;
	if (Change & WinsFocusIN) m = DIFu_Focus;
	TDesktopIcon_SetFunkt((TDesktopIcon*)self->owner, m);
}

static void TDesktopIcon_Restack(TDesktopIcon *di) {
  Window win[4];

	if (!di) return;
	if (di->glob->top_di && (di->glob->top_di != di)) {

	    win[0] = di->glob->top_di->WinID;
	    win[1] = di->glob->top_di->WinCaption->WinID;
	    win[2] = di->WinID;
	    win[3] = di->WinCaption->WinID;
	    XRestackWindows(appXDat.dpy, win, 4);

	    win[0] = di->WinID;
	    win[1] = di->WinCaption->WinID;
	    win[2] = di->glob->top_di->WinID;
	    win[3] = di->glob->top_di->WinCaption->WinID;
	    XRestackWindows(appXDat.dpy, win, 4);

	}
	di->glob->top_di = di;
}

static void TDesktopIcon_ClrMask(TDesktopIcon *di) {
	if (di->stat & wmDIst_MaskAkt) {
	    di->stat &= ~wmDIst_MaskAkt;
	    CombineMask(di->WinCaption->WinID, 0);
	}
}

void TDesktopIcon_SetFunkt(TDesktopIcon *di, int fu) {
  TPixMap *pxm;

	switch(fu & 0xffff) {

	case DIFu_Focus:
	    TDesktopIcon_ClrMask(di);
	    di->stat &= ~wmDIst_HlAkt;
	    di->WinCaption->vds.ForeColor = di->glob->TwFocFontColor;
	    SetColor(di->WinCaption, di->glob->TwFocColor);
	    break;

	case DIFu_HighLight:
	    TDesktopIcon_ClrMask(di);
	    TDesktopIcon_Restack(di);
	    if (di->glob->mod & wmDImod_HighLight) {
		di->stat |= wmDIst_HlAkt;
		di->WinCaption->vds.ForeColor = di->glob->TwHlFontColor;
		SetColor(di->WinCaption, di->glob->TwHlColor);
	    } else {
		di->WinCaption->vds.ForeColor = di->glob->TwFontColor;
		SetColor(di->WinCaption, di->glob->TwColor);
	    }
	    break;

	default:
	    if (di->glob->mod & wmDImod_Transparent) {
	      if (!(di->stat & wmDIst_MaskAkt) || (fu & DIFu_Force)) {
		di->stat |= wmDIst_MaskAkt;
		pxm = TPixMap_Create_depth(di->WinCaption->width, di->WinCaption->height, appXDat.RootW_depth);
		TPixMap_CreateXimg(pxm);
		TPixMap_SetFont(pxm, di->glob->DesktopIcoFont);
		TPixMap_ClsRGB(pxm, 0xffffff);
		di->WinCaption->vds.ForeColor = 0x000000;
		TPixMap_DrawString(pxm, di->WinCaption->vds.s, 2, 0);
		TPixMap_DrawString(pxm, di->WinCaption->vds.s, 2, 2);
		TPixMap_DrawString(pxm, di->WinCaption->vds.s, 1, 1);
		TPixMap_DrawString(pxm, di->WinCaption->vds.s, 3, 1);
		TPixMap_DrawString(pxm, di->WinCaption->vds.s, 2, 1);
		TPixMap_XGetImage(pxm);

		TPixMap_SetMaskRGB(pxm, 0xffffff);
		TPixMap_WinCombineMask(pxm, di->WinCaption->WinID);
		Tfree(pxm);
		di->WinCaption->vds.ForeColor = 0xffffff;
		SetColor(di->WinCaption, 0x000000);
	      }
	    } else {
		TDesktopIcon_ClrMask(di);
		di->WinCaption->vds.ForeColor = di->glob->TwFontColor;
		SetColor(di->WinCaption, di->glob->TwColor);
	    }
	    di->stat &= ~wmDIst_HlAkt;
	    break;
	}
}

void TDesktopIcon_ScanMousePos(TWins *self, XMotionEvent *ev, TWins *wins) {
  TDesktopIcon *di;
  TWins *tw;

	while (wins) {
	    if (wins->wtyp == wtyp_TwmDeskIcon) {
		di = (TDesktopIcon*)wins;
		tw = di->WinCaption;
		if (TDesktopIcon_ChkMouseOverIcoCaption(di, ev->x, ev->y)) {
		    if (tw->KeyFu & mHasFocus) {
			TDesktopIcon_SetFunkt(di, DIFu_Focus);
		    } else {
			TDesktopIcon_SetFunkt(di, DIFu_HighLight);
		    }
		    return;
		} else {
		    if (!(tw->KeyFu & mHasFocus))
			    TDesktopIcon_SetFunkt(di, DIFu_Default);
		}
	    }
	    wins = wins->next;
	}
}

void TDesktopIcon_SetSize(TDesktopIcon *di) {
  TWins *tw;
  int w, fheight;

	tw = di->WinCaption;
	w = TextWidthColorString(&tw->vds) + 4;
	if (w < 32) w = 32;
	di->xo = (di->width - w) / 2;
	di->yo = di->height + 2;
	Font_GetMetrics((TFont*)tw->vds.Font, &fheight, NULL);
	SetRC(tw, di->left + di->xo, di->top + di->yo, w, fheight + 2);
}

static int get_opt_len(const char *cpc) {
  int i = 0;
  const char *pc = cpc;
	while (*pc && (*pc != ' ') && (*pc != ',')) {pc++; i++;}
	return i;
}

static bool __attribute__ ((noinline)) ChkFSDeviceOk(TpcIni *ini, dword *difs, bool nyrun) {
  const char *mp, *url, *exec, *opt, *cpc, *pass;
  int i;
  char *pc, *pct;
  char buf[40];
  bool has_user, has_guest, has_ro;

	*difs = 0;
	mp = TpcIni_RD(ini, Desktop_Entry, "MountPoint");
	if (!mp || pccmp(mp, "/")) return false;
	url = TpcIni_RD(ini, Desktop_Entry, "Dev");
	if (!url) return false;
	opt = TpcIni_RD(ini, Desktop_Entry, "Options");

	pass = NULL;
	has_user = false;
	has_guest = false;
	has_ro = false;
	if (opt) {
	    cpc = opt;
	    while (*cpc && (*cpc != ' ')) {
		if (pcncmp(cpc, "password=", 9)) {
		    pass = cpc + 9;
		} else if (pcncmp(cpc, "pass=", 5)) {
		    pass = cpc + 5;
		} else if (pcncmp(cpc, "user=", 5)) {
		    has_user = true;
		} else if (pcncmp(cpc, "guest", 5) && (get_opt_len(cpc) == 5)) {
		    has_guest = true;
		} else if (pcncmp(cpc, "ro", 2) && (get_opt_len(cpc) == 2)) {
		    has_ro = true;
		}
		while (*cpc && (*cpc != ' ') && (*cpc != ',')) cpc++;
		if (*cpc && (*cpc != ' ')) cpc++;
	    }

	    if (pass) {
		i = get_opt_len(pass);
		if (i > 32) goto err_pass;
		pcnmove(buf, pass, i);
		if (i && (i <= 16)) {
		    pc = CryptMini16c_dec(buf);
		    pct = pcndup(opt, pass - opt);
		    ppcadd2(&pct, pc, pass + i);
		    xfree(pc);
		    TpcIni_WR(ini, Desktop_Entry, "Options", pct);
		    xfree(pct);
		    TpcIni_Save(ini);
		    return !nyrun;

		} else if (i == 32) {
		    pc = CryptMini16c_enc(buf);
		    if (!pc) goto err_pass;
		    xfree(pc);
		    *difs |= DIFS_CRYPT_PASS;

		} else if (i) {
err_pass:
		    printf("ERROR : password fault \"%s\"\n", ini->FileName);
		    return false;
		}
	    }
	}

	if (!has_ro && TpcIni_RD_boolean(ini, Desktop_Entry, "ReadOnly", False)) {
	    *difs |= DIFS_ADD_READONLY;
	}

	exec = TpcIni_RD(ini, Desktop_Entry, "Exec");
	if (!exec) return true;

	cpc = exec;
	while (*cpc && (*cpc != '.')) cpc++;
	if (!*cpc) return true;
	cpc++;

	if (pccmp(cpc, "nfs") || pccmp(cpc, "nfs4")) {
	    *difs |= DIFS_NFS;
	    return chkdesk_nfs_url(url);
	}

	if (pccmp(cpc, "cifs")) {
	    *difs |= DIFS_CIFS;
	    if (!chkdesk_cifs_url(url)) return false;
	    if (!pass && !has_user) {	//ok guest
		if (!has_guest) *difs |= DIFS_ADD_GUEST;
		return true;
	    }
	    if (!pass) {		//ok add empty password
		*difs |= DIFS_ADD_EMPTY_PASS;
		return true;
	    }
	}
	return true;
}

static void FsOptAdd(char **ppc, const char *s, int *mk) {
	if (*mk) {
	    ppcmove(ppc, ","); 
	} else {
	    ppcmove(ppc, " -o ");
	    *mk = 1;
	}
	ppcmove(ppc, s);
}

void SetFSDeviceOpt(TDesktopIcon *self, TpcIni *ini, char **ppc) {
  const char *opt, *cpc, *pass;
  int mk, i;
  char *pc, *pct;

	mk = 0;
	if (self->difs & DIFS_ADD_GUEST)	FsOptAdd(ppc, "guest", &mk);
	if (self->difs & DIFS_ADD_EMPTY_PASS)	FsOptAdd(ppc, "password=", &mk);
	if (self->difs & DIFS_ADD_READONLY)	FsOptAdd(ppc, "ro", &mk);
	opt = TpcIni_RD(ini, Desktop_Entry, "Options");
	if (!opt) return;
	if (self->difs & DIFS_CRYPT_PASS) {

	    pass = NULL;
	    if (mk) ppcmove(ppc, ","); else ppcmove(ppc, " -o ");
	    cpc = opt;
	    while (*cpc && (*cpc != ' ')) {
	        if (pcncmp(cpc, "password=", 9)) {
		    pass = cpc + 9;
		    break;
		} else if (pcncmp(cpc, "pass=", 5)) {
		    pass = cpc + 5;
		    break;
		}
		while (*cpc && (*cpc != ' ') && (*cpc != ',')) cpc++;
		if (*cpc && (*cpc != ' ')) cpc++;
	    }

	    if (pass) {
		ppcnmove(ppc, opt, pass - opt);
		i = get_opt_len(pass);
		pct = pcndup(pass, i);
		pc = CryptMini16c_enc(pct);
		ppcmove(ppc, pc);
		xfree(pc);
		xfree(pct);
		ppcmove(ppc, pass + get_opt_len(pass));
	    }

	} else {
	    FsOptAdd(ppc, opt, &mk);
	}
}

/*_____________________________ Destroy _________________________________*/

static bool TDesktopIcon__Destroy(TDesktopIcon *self) {
	if (self->WinCaption) {
	    self->WinCaption->vds.Font = NULL;
	    ppTfree(&self->WinCaption);
	}
	xfree(self->fname);
	xfree(self->s2);
	if (self->glob->top_di == self) self->glob->top_di = NULL;
	return TWins__Destroy((TWins*)self);
}

void TDesktopIcon_Destroy_fname(char *fn) {
  TWins *wins = wmRoot->next_child;
  TDesktopIcon *wmdi;

	while (wins) {
	    if (wins->wtyp == wtyp_TwmDeskIcon) {
		wmdi = (TDesktopIcon*)wins;
		if (pccmp(wmdi->fname, fn)) {
		    Tfree(wmdi);
		    return;
		}
	    }
	    wins = wins->next;
	}
}

/*_____________________________ Create __________________________________*/
static TDesktopIcon *TDesktopIcon_Create(TwmRoot *wmr, char *fname, bool nyrun) {
  TDesktopIcon *self;
  TWins_Create_struct wcs;
  TWins *tw;
  int ddkt;
  dword difs;
  const char *pc, *fn, *name, *icon;
  TpcIni *inid = NULL;
  TPic *pic;

	ddkt = ddkt_desktop(fname);
	if (ddkt < 1) return NULL;
	self = NULL;

	fn = GetFileName_pc(fname);
	inid = TpcIni_Create(fname);
	name = TpcIni_RD_pc(inid, Desktop_Entry, "Name", fn);
	icon = TpcIni_RD(inid, Desktop_Entry, "Icon");
	if (!name) goto Ende;
	difs = 0;
	if (ddkt == ddkt_FSDevice) {
	    if (!ChkFSDeviceOk(inid, &difs, nyrun)) goto Ende;
	}

	pic = GetDeskIcon(ddkt, 32, icon, gdifu_getico0, NULL);
	if (!pic) goto Ende;

	TWins_Create_struct_init(&wcs, sizeof(TDesktopIcon), wtyp_TwmDeskIcon);
	wcs.Parent = (TWins*)wmr;
	wcs.CreateXWin = NULL;
	TWins_DefaultConf(&wcs);
	self = wcs.wins;
	self->cursor_nr = XC_top_left_arrow;
	self->fname = pcdup(fname);

	self->glob = &wmr->di_glob;
	if (!self->glob->DesktopIcoFont) self->glob->DesktopIcoFont = appXDat.Default_Font;
	self->destroy = (TDESTROY)TDesktopIcon__Destroy;
	self->DoPaint = TDesktopIcon_DoPaint;
	self->DoMouseDown = (void*)TDesktopIcon_DoMouseDown;
	self->DoMouseUp   = (void*)TDesktopIcon_DoMouseUp;
	self->DoMouseMove = (void*)TDesktopIcon_DoMouseMove;
	self->DoMouseOver = (void*)TDesktopIcon_DoMouseOver;

	tw = TWins_Create((TWins*)wmr);
	self->WinCaption = tw;
	tw->owner = (TWins*)self;
	tw->vds.Font = self->glob->DesktopIcoFont;
	tw->DoPaint = TDesktopIconTw_DoPaint;
	tw->DoMouseDown = TDesktopIcon_DoMouseDown;
	tw->DoMouseUp = TDesktopIcon_DoMouseUp;
	tw->DoMouseOver = TDesktopIcon_DoMouseOver;
	tw->DoFocus = TDesktopIconTw_DoFocus;
	tw->DoMouseMove = TDesktopIcon_DoMouseMove;
	tw->PopupMenu = wmr->menu_Default_DeskIcon;
	Set_Cursor(tw, XC_top_left_arrow);


	self->ddkt = ddkt;
	self->PopupMenu = wmr->menu_Default_DeskIcon;
	if (self->ddkt == ddkt_FSDevice) {
	    self->difs = difs;
	    self->pic3 = pic; //hdd_unmount
	    self->pic2 = GetDeskIcon(ddkt, 32, icon, gdifu_getico1, NULL); //hdd_mount
	    self->pic4 = GetDeskIcon(ddkt, 32, icon, gdifu_getico2, NULL); //hdd_iseject
	    self->pic = self->pic4; //hdd_iseject

	    self->s2 = TpcIni_RDstrdup(inid, Desktop_Entry, "Dev", NULL);
	    if (self->difs & (DIFS_NFS|DIFS_CIFS)) {
// printf("%s(%d): %s\n", FL, self->s2);
		add_nfs_info(self->s2);
	    } else {
		add_fs_info(GetFileName_pc(self->s2), NULL, NULL);
	    }

	} else {
	    self->pic = pic;
	}

  bool ok;
  char line[200];
  TpcIni *inisz = TpcIni_Create(wmr->IconPosFile);
	snprintf(line, sizeof(line), "%dx%d", dpyMonitors.mon[0].right, dpyMonitors.mon[0].bottom);
	ok = False;
	pc = TpcIni_RD(inisz, fn, line);
	if (pc) {
	    self->left = ppctoint(&pc);
	    self->top = ppctoint(&pc);
	    ok = di_ChkDeskPos(self, wmr, self->left, self->top);
	}
	if (!ok) di_GetNewDeskPos(self, wmr, &self->left, &self->top);
	Tfree(inisz);

	if (self->left > di_GetMaxRight()) self->left = di_GetMaxRight();
	if (self->top > di_GetMaxBottom()) self->top = di_GetMaxBottom();
	if (self->left < 0) self->left = 0;
	if (self->top < 0) self->top = 0;

	self->width = 32;
	self->height = 32;
	TWins_CreateXWin(&wcs);

	SetCaption(tw, name);

	Set_Parent((TWins*)self, (TWins*)wmr);
	XLowerWindow(appXDat.dpy, self->WinID);
	Set_Parent(tw, (TWins*)wmr);
	XLowerWindow(appXDat.dpy, self->WinCaption->WinID);

	TDesktopIcon_SetSize(self);
	TDesktopIcon_SetFunkt(self, DIFu_Default);

Ende:
	Tfree(inid);
	return self;
}

static void DeskIcons_OnFileMsg(TFileMsg *fmsg) {
  TwmRoot *wmr;
  char *rpe;
  int bef, r_bef;
  TDirFileList *dfl;
  char line[1024];

	bef = get_bef_dflEv(fmsg, &r_bef);
	if (!bef) {
	    if ((r_bef & dlbef_RescanRun) && 
		((r_bef & dlbef_BefMask) == dlbef_Close)) {
//  printf("%s(%d): 0x%04x\n", FL, r_bef);
		Find_CDrom();
		TwmRoot_AzMount(NULL);
	    }
	    return;
	}
	dfl = fmsg->F.dfl;
	wmr = ((TDirListObjNy*)fmsg->obj)->owner;

	rpe = line;
	ppcmove(&rpe, wmr->DesktopPath);
	ppcmove(&rpe, dfl->fn);

	switch (bef) {
	case dlbef_Add:
	    TDesktopIcon_Create(wmr, line, (r_bef & dlbef_NotifyRun) != 0);
	    break;
	case dlbef_WrClose:
	    TDesktopIcon_Destroy_fname(line);
	    TDesktopIcon_Create(wmr, line, (r_bef & dlbef_NotifyRun) != 0);
	    break;
	case dlbef_Del:
	    TDesktopIcon_Destroy_fname(line);
	    break;
	}
}


void TwmRoot_InitDeskIcons(TwmRoot *wmr) {
  TDirListObjNy *ObjNy;

	ObjNy = TDirListObjNy_Create(wmr);
	ObjNy->OnFileMsg = DeskIcons_OnFileMsg;
	ObjNy->findAtr = findREG;
	if (!TDirListObjNy_Open(ObjNy, wmr->DesktopPath)) {
	    Tfree(ObjNy);
	} else {
	    wmr->ObjNy_DeskIcons = ObjNy;
	}
}
