/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldesk.h"
#include "../../lib/Ximg/decor/tdecor.h"

typedef struct TwmFrameBorder_rxbx {
  int wtyp;
  int size;
  const TBordMaskHead *Dat;
  const TBordMaskHead *NC;
} TwmFrameBorder_rxbx;

static bool TwmFrameBorder_DoDecor(TWins *wins, int bef, int mk, void *p) {
  TRX_Dat rxdat;
  TwmFrameBorder_rxbx *dat;
  TDecor *Decor = wins->Decor;

	if (!Decor) return False;
	dat = Decor->DecorDat;
	if (!dat || (dat->wtyp != wtyp_TDecorDat)) return False;

	rxdat.WinID = wins->WinID;
	rxdat.Color = wins->Color;
	rxdat.org_pxm = NULL;
//	rxdat.bw = 0;
	rxdat.width = wins->width;
	rxdat.height = wins->height;
	rxdat.top = 0;
	rxdat.left = 0;
	rxdat.BordMaskHead = dat->Dat;

	switch(bef) {
	case decor_NewSize:
	    if (mk & (CWWidth | CWHeight)) {
		decor_RegionBR2_5(&rxdat, &Decor->stat);
	    }
	    return True;

	case decor_Border:
	    if ((Decor->stat & Deco_SizeOk) == 0) {
		decor_RegionBR2_5(&rxdat, &Decor->stat);
	    }
	    decor_paintBR2_5(&rxdat);
	    return True;

	case decor_ClrAndFree:
	    rxdat.BordMaskHead = NULL;
	    decor_RegionBR2_5(&rxdat, &Decor->stat);
	    if (Decor->wtyp == wtyp_TDecorDup) {
		xfree(Decor);
		wins->Decor = NULL;
	    }
	    return True;

	case decor_Destroy:
	    if (Decor->wtyp == wtyp_TDecorDup) {
		xfree(Decor);
	    }
	    return True;

	default:
	    return False;
	}

	return False;
}


void TwmFrameBorder_Create(TwmRoot *wmr, TpcIni *Ini) {
  int m;
  TpcIni *IniMk = Ini;
  TwmFrameBorder_rxbx *dat;
  const TBordMaskHead *bmh = NULL;
//  Tapph *ap;
  TwmFrame *wmf;

	if (!Ini) Ini = TpcIni_Create(appDat.inifile_app);

	m = TpcIni_RD_int(Ini, "Style", "FormStyle", 0);
	switch (m) {
	case 1: bmh = &TDecoBuR3B3_up_Tab; break;
	case 2: bmh = &TDecoBuW3B3_up_Tab; break;
	case 3: bmh = &TDecoBuR5B4_up_Tab; break;
	}
	if (IniMk != Ini) Tfree(Ini);

	if (!wmRoot->Default_Decor) {
	    if (bmh) {
		dat = xcalloc(sizeof(TwmFrameBorder_rxbx));
		dat->wtyp = wtyp_TDecorDat;
		dat->size = sizeof(TwmFrameBorder_rxbx);
		dat->Dat = bmh;
		wmr->Default_Decor = TDecor_Create(dat);
		wmr->Default_Decor->DoDecor = TwmFrameBorder_DoDecor;
		wmf = (TwmFrame*)appXDat.Wins;
		while (wmf) {
		    if (wmf->wtyp == wtyp_TwmFrame) {
//			wmf = (TwmFrame*)ap->wins;
			wmf->Decor = TDecor_dup(&wmRoot->Default_Decor);
			if ((wmf->wmWin->Desk == wmRoot->CurrentDesk) && wmf->Visible) {
			    TWins_Decor((TWins*)wmf, decor_Border, 0, NULL);
			}
		    }
		    wmf = (TwmFrame*)wmf->next;
		}
	    }
	} else {
	    dat = wmr->Default_Decor->DecorDat;
	    if (bmh && (bmh != dat->Dat)) {
		dat->Dat = bmh;
		wmf = (TwmFrame*)appXDat.Wins;
		while (wmf) {
		    if (wmf->wtyp == wtyp_TwmFrame) {
			wmf->Decor->stat &= ~Deco_SizeOk;
			if ((wmf->wmWin->Desk == wmRoot->CurrentDesk) && wmf->Visible) {
			    TWins_Decor((TWins*)wmf, decor_Border, 0, NULL);
			}
		    }
		    wmf = (TwmFrame*)wmf->next;
		}

	    } else if (!bmh) {
		wmf = (TwmFrame*)appXDat.Wins;
		while (wmf) {
		    if (wmf->wtyp == wtyp_TwmFrame) {
			TWins_Decor((TWins*)wmf, decor_ClrAndFree, 0, NULL);
		    }
		    wmf = (TwmFrame*)wmf->next;
		}
		xfree(dat);
		xfree(wmr->Default_Decor);
		wmr->Default_Decor = NULL;
	    } 
	}
}
