/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldesk.h"
#include "unistd.h"
#include <sys/wait.h>

TwmRoot *wmRoot;
int ExtQuitBef;
#if USE_NoFocusWin
Window NoFocusWin;
#endif

#ifdef	USE_DEBUG_WIN
Window last_event_window;
int last_event_type;
#endif

bool Chk_another_WM;
bool XwmError_Stop;
bool NoAutoStart;

static bool XwmError(Display *dpy, XErrorEvent *e) {
  bool res;

	if (Chk_another_WM) {
	    printf("ERROR: another WM is running\n");
	    exit(1);
	}
	if (XwmError_Stop) return true;

#ifdef ALL_XwmError
	res = false;
#else
	res = (e->error_code == BadWindow);
#endif

#ifdef	USE_DEBUG_WIN
 char msg[80], req[80], number[80];
    if (!res) {
	XGetErrorText(dpy, e->error_code, msg, sizeof(msg));
	sprintf(number, "%d", e->request_code);
	XGetErrorDatabaseText(dpy, "XRequest", number, "<unknown>", req, sizeof(req));
	Debug_msg(0, "ALDesk XwmError: %s(0x%lx): %s\n", req, e->resourceid, msg);
	Debug_msg(0, "  --------->>: last_ev_w=0x%x last_ev_typ=%d\n", (int)last_event_window, last_event_type);
    }
#endif
	return res;
}

static void AutoStart_OnFileMsg(TFileMsg *fmsg) {
  char *pc;

	if (get_bef_dflEv(fmsg, NULL)) {
	    if (fmsg->F.dfl->st->FileType == stft_desktop) {
		pc = pcadd(fmsg->R.gp->path, fmsg->F.dfl->fn);
		ExecDesktopFile(pc, NULL);
		xfree(pc);
	    }
	}
}

static void AutoStart() {
  TDirListObjNy *ObjNy;

	ObjNy = TDirListObjNy_Create(NULL);
	ObjNy->OnFileMsg = AutoStart_OnFileMsg;
	ObjNy->findAtr = findREG;
	if (TDirListObjNy_Open(ObjNy, wmRoot->AutostartPath)) {
	    TDirListObjNy_Open_Wait(ObjNy);
	}
	Tfree(ObjNy);
}

/*_____________________________ Monitors ________________________________*/
static void MonitorsReScane(TgTimer *gti) {
	MonitorsInit();
}

/*_____________________________ Start ___________________________________*/
static const char *path_tab[] = {
  "/Menu/OpenWith/",
  NULL
};

static void OnAppStart() {
  char *pc;
  const char **ppc;

	ppc = path_tab;
	while (*ppc) {
	    pc = pcadd(appDat.ALDini_dir, *ppc);
	    file_mkdir_parents_ae(pc, 0750, EMSG_ERR1);
	    xfree(pc);
	    ppc++;
	}

	AppProcessMessages();
	SyncX();
	AppHandleMessage();

	TwmRoot_global(wmRoot);
	SetBootDev();
	TwmRoot_InitDeskIcons(wmRoot);

//	gTimerX_Add(Timer_Enabled, 2000, TwmRoot_AzMount);

	if (wmRoot->Taskbar && wmRoot->Taskbar->Visible) {
	    XRaiseWindow(appXDat.dpy, wmRoot->Taskbar->WinID);
	}

	SyncX();
	AppProcessMessages();

#if USE_SYS_BLOCK
	if (wmRoot->ObjNy_DeskIcons) {
	    while (wmRoot->ObjNy_DeskIcons->dl->runstat < dlrunstat_NotifyRun) {
		AppHandleMessage();
	    }
	}
	InitIcons_AutoMount(wmRoot);
#endif
	AppHandleMessage();

	MonitorsInit();
	if ((dpyMonitors.stat & dpyM_Active) && !(dpyMonitors.stat & dpyM_AutoReScane)) {
	    gTimerX_Add(Timer_Enabled, 2000, MonitorsReScane);
	}
	AppHandleMessage();
	if (!NoAutoStart) AutoStart();
}

//static bool exit_msg_run = false;
static bool force_exit = false;

static void MsgBox_ExitEv(Texec_pipe* expip, int evtyp) {
	if (evtyp == evtyp_ProcClose) {
	    if (pcncmp(expip->pp[evtyp_infd].lb->pc, "YES", 3)) {
		force_exit = true;
		Close();
	    }
//	    exit_msg_run = false;
	}
}

static bool ChkAnyWinOpen() {
  TWins *wins;
	wins = appXDat.Wins;
	while (wins) {
	    if (wins->wtyp == wtyp_TwmFrame) return true;
	    wins = wins->next;
	}
	return false;
}

static void OnAppClose() {
  dword t;
  char *pc;
  char line[1024];
  bool OpenMk, SysShutDown;

	SysShutDown = ((ExtQuitBef & 3) == 3);
	exec_Buildin(NULL, "close_all_win", NULL, 0);
	if (force_exit) goto CloseWmClose;

	appDat.gQUIT_Ev = false;

	t = GetTime_ms() + 2000;
	if (SysShutDown) t += 10000;
loop_t:
	OpenMk = ChkAnyWinOpen();
	if (OpenMk) {
//		XSync(appXDat.dpy, 0);
	    AppHandleMessage();
	    if (!cpTime(t, GetTime_ms())) goto loop_t;
	    OpenMk = ChkAnyWinOpen();
	}

	if (OpenMk) {
	    if (SysShutDown) {
		exec_Buildin(NULL, "close_all_win", NULL, 0);
		AppHandleMessage();
		goto CloseWmClose;
	    }
//	    if (!exit_msg_run) {
		pc = line;
#ifdef USE_BULDIN_APP
		pc += sprintf(pc, "%s aldmessagebox ", appDat.self_exe);
#else
		ppcmove(&pc, "aldmessagebox ");
#endif
#ifdef Test_Root_WinId
		if (TestRootWinId) {
			pc += sprintf(pc, "--TestRootWinID 0x%x ", (int)TestRootWinId);
		}
#endif
		ppcmove(&pc, " -fYES,NO -dNO \"Not all Windows closed\\nForce Exit ???\"");
		ExecuteProcApp(line, infd_Open, MsgBox_ExitEv, NULL, NULL);
//	    }
	    return;
	}

CloseWmClose:
	appDat.gQUIT_Ev = true;
// ******** Close Taskbar AppletWin ********
	if (wmRoot->Taskbar) {
	    TWins *wins = wmRoot->Taskbar->tbSysTray->next_child;
	    while (wins) {
		TWins *winsn = wins->next;
		if (wins->wtyp == wtyp_TTaskbarApplet) {
			TwmWin_Close((TwmWin*)wins);
		}
		wins = winsn;
	    }
	}

	if (SysShutDown) {
	    FileHandle fd;
	    if (!file_open_wr("/var/run/ALDesk/aldesk", NULL, &fd, 0640)) {
		appError(EMSG_ERR, "Cannot create /var/run/ALDesk/aldesk");
	    } else {
		file_write(fd, NULL, "ShutDown", 8, NULL);
		file_close(fd, NULL);
	    }
	}
}

void menu_desktop_info_OnClick(TMenu *menu, long KeyState, TMenuItem* mi) {
  char *pc;
  char line[1024]; 

	pc = line;
#ifdef USE_BULDIN_APP
	pc += sprintf(pc, "%s Copyright_MsgBox \"%s   (build %d  %s)\"", 
			appDat.self_exe, ALD_VERSION, ALD_BUILD_V, __DATE__);
#else
	pc += sprintf(pc, "Copyright_MsgBox \"%s %s   (build %d  %s)\"", 
			appDat.self_exe, ALD_VERSION, ALD_BUILD_V, __DATE__);
#endif
#ifdef Test_Root_WinId
	if (TestRootWinId) {
	    sprintf(pc, "--TestRootWinID 0x%x ", (int)TestRootWinId);
	}
#endif
	ExecuteProc(line);
}

static const char *Help_Msg = {
  "Usage: "APPNAME" [options]\n"
  "\n"
  "  -f File              Ini-File (default ~/.ALDccp/global.ini)\n"
  "  -d display_name      :1\n"
  "  --NoAutoStart        not exec ~/.ALDccp/Autostart/*.deskdop\n"
  "  --NoTaskbar\n"
#ifdef Test_Root_WinId
  "  --TestRootWinID      wm run in owner window\n"
#endif
#ifdef	USE_DEBUG_WIN
  "  --debug              show debug window\n"
#endif
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: "APPNAME" version "ALD_VERSION"  (build %d  %s)\n"
};

int main (int argc, char **argv) {
  int i;
  char *val;
  bool err, help, NoTaskbar;

	err = false;
	help = false;
	NoTaskbar = false;
	cmdline_slash_korr(argc, argv);
#ifdef USE_BULDIN_APP
  TBuildin_app *bi_app;
	bi_app = Get_Buildin_pc(argv[0]);
	if (bi_app) return bi_app(argc, argv);
	if ((argc > 1) && (*argv[1] != '-')) {
	    cmdline_del_argv0(&argc, argv);
	    bi_app = Get_Buildin_pc(argv[0]);
	    if (bi_app) return bi_app(argc, argv);
	    BuildinHelp();
	    return EXIT_FAILURE;
	}
#endif

	NoAutoStart = false;
	XwmError_Stop = false;
	i = 1;
	while (!err && (i<argc)) {
	    val = argv[i];
	    if (*val == '-') {
		val++;
		switch (*val) {
		case 'd': 
		    i++;
		    err = (i >= argc);
		    if (!err) {
			    display_name = argv[i];
		    }
		    break;
		case 'h': 
		    help = true; break;
		case '-':
		    val++;
		    if (pcncmp(val, "NoAutoStart", 11)) {
			NoAutoStart = true;
		    } else if (pcncmp(val, "NoTaskbar", 9)) {
			NoTaskbar = true;
#ifdef Test_Root_WinId
		    } else if (pcncmp(val, "TestRootWinID", 13)) {
			i++;
			val = argv[i];
			if (val && *val != '-') {
				TestRootWinId = pctoint(val);
			} else err = true;
#endif
		    } else if (pcncmp(val, "debug", 5)) {
#ifdef	USE_DEBUG_WIN
			DebugOn = true;
#endif
		    } else err = true;
		    break;
		default:
		    err = true;
		}
	    } else err = true;
	    i++;
	}
	if (err || help) {
	    printf(Help_Msg, ALD_BUILD_V, __DATE__);
#ifdef USE_BULDIN_APP
	    printf("\nor:\n");
	    BuildinHelp();
#endif
	    if (err) exit(1); else exit(0);
	}

	while (waitpid(-1, NULL, WNOHANG) > 0);

	initXapp(0, argc, argv);
	ThDirList_Reg(1, &hdirlist_ld_File);
//	ThDirList_ny_Reg(1, &hdirlist_ny_File);
	ThDirList_ny_Reg(2, &hdirlist_nyi_File, &hdirlist_ny_File);

	hGetDeskIcon = &hGetDeskIcon_file;
	ThImageRD_Reg(4, &hImageRD_xpm, &hImageRD_png, &hImageRD_jpeg, 
			 &hImageRD_bmp);

	hGetFileHeadInfo = &hGetFileHeadInfo_buildin;

	appDat.inifile_app = pcadd(appDat.ALDini_dir, "/"APPNAME".ini");
	ChkSetDefault();

	Chk_another_WM = true;

	On_wmError = XwmError;

//	XSetErrorHandler(CatchRedirectError);
	InitEventHandlerJumpTable();

	Create_globGC();

	wmRoot = TwmRoot_Create(NULL);
// return 0;
	Chk_another_WM = false;

#if USE_NoFocusWin
//create NoFocusWin
  XSetWindowAttributes *attr = xcalloc(sizeof(XSetWindowAttributes));
	attr->event_mask = KeyPressMask | FocusChangeMask;
	attr->override_redirect = 1;
	NoFocusWin = XCreateWindow(appXDat.dpy, appXDat.RootW, -10, -10, 10, 10, 0, 0,
                               InputOnly, CopyFromParent,
                               CWEventMask | CWOverrideRedirect, attr);
	xfree(attr);
	XMapWindow(appXDat.dpy, NoFocusWin);
// printf("%s(%d) NoFocusWin=0x%x\n", FL, (int)NoFocusWin);
#endif

//	setNumlock(true);

	if (!NoTaskbar) wmRoot->Taskbar = TTaskbar_Create();

	appXDat.OnAppStart = OnAppStart;
	appXDat.OnAppClose = OnAppClose;

//	appError_ev = Error_ev;

#ifdef USE_DEBUG_WIN
	if (DebugOn) Chk_DebugWinRun();
#endif

	AppRun();
// printf("%s(%d): \n", FL);
//	Tfree(wmRoot->Taskbar);
// printf("%s(%d): \n", FL);
#ifdef MEM_CHK
  int lMCt;
  dword lMsize;
  MEM_CHK_Count(&lMCt, &lMsize);
  printf("Debug %s(%d): MEM_CHK_Count=%3d  size=%d wins=%d\n", FL, lMCt, lMsize, GetTWinsCount());
#endif

	usleep(50000);
	return 0;
}
