/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/
 
#include "appXbase.h"
#include "tpixmap.h"
#include "tdirlist.h"

#ifdef USE_BULDIN_APP
#include "buildin_app/buildin_app.h"
#endif

#define APPNAME "aldesk"

#define SAVE_PARENT_WIN 1
#define USE_NoFocusWin 1

#define USE_PROC_PART 1
#define USE_SYS_BLOCK 0

#if USE_PROC_PART
#undef USE_SYS_BLOCK
#define USE_SYS_BLOCK 0
#endif

#define NEW_FLAGS_08400 1

struct TDesktopIcon;

typedef struct TwmDeskIcon_Global {
  int mod;
  int TwColor;
  int TwHlColor;
  int TwFocColor;
  int TwFontColor;
  int TwHlFontColor;
  int TwFocFontColor;
  TFontX *DesktopIcoFont;
  char *DesktopIcoFontName;
  struct TDesktopIcon* top_di;
} TwmDeskIcon_Global;

#define PART_TABLE_MAX 16

#define FSSTAT_EXIST_MK	(1<<16)

typedef struct TFSDevice_part_table {
  struct TFSDevice_part_table *next;
  int stat;
  char dev[16];
  int type[PART_TABLE_MAX];
  TDirList *dl;
} TFSDevice_part_table;

#define FSINFO_OK	1
#define FSINFO_CDROM	(1<<1)
#define FSINFO_NFS	(1<<2)
#define FSINFO_CIFS	(1<<3)
#define FSINFO_AZ_OK	0x0f
#define FSINFO_DEVLOCK	(1<<7)
#define FSINFO_HIDE	(1<<8)
#define FSINFO_MOUNT	(1<<9)


typedef struct TFSDevice_info {
  struct TFSDevice_info *next;
  int nr;
  int stat;
  byte major;
  byte minor;
  byte ncb2, ncb3;
  int part_type;
  char size[16];
  char type[16];
  char dev0[16];
  char dev[4][16];
} TFSDevice_info;

TFSDevice_info *add_fs_info(const char *dev, const char *dev0, int *r_idx);
TFSDevice_info *get_fs_info(const char *dev);

typedef struct {
  TFSDevice_info *next;
  int nr;
  int stat;
  byte ncb0, ncb1, ncb2, ncb3;
  int part_type;
  char size[16];
  char type[16];
  char dev0[16];
  char dev[192];
} TNFSDevice_info;

bool chkdesk_nfs_url(const char *dev);
bool chkdesk_cifs_url(const char *dev);
TNFSDevice_info *add_nfs_info(const char *dev);

#define DIFS_NFS		(1<<2)
#define DIFS_CIFS		(1<<3)
#define DIFS_ADD_GUEST		(1<<8)
#define DIFS_ADD_EMPTY_PASS	(1<<9)
#define DIFS_ADD_READONLY	(1<<10)
#define DIFS_CRYPT_PASS		(1<<11)

//#define DIFS_ADD_MASK	(DIFS_ADD_GUEST|DIFS_ADD_EMPTY_PASS|DIFS_ADD_READONLY)

#define TCSelf TDesktopIcon
typedef struct TDesktopIcon {
  AlgTWins
  int ddkt;		// dotdesktop.c
  int stat;
  int xo, yo;
  char *fname;
  TWins *WinCaption;
  TPic *pic;	//Paint pic
  TPic *pic2;
  TPic *pic3;
  TPic *pic4;
  TPic *temp_pic;
  char *s2;
  TgTimer *IcoButtonUpTimer;
  int LastIcoMask;
  TwmDeskIcon_Global *glob;
  int mtime;
  char *cmd2; //umount eject
  dword difs;
} TDesktopIcon;
#undef TCSelf

TDesktopIcon REGP1 *get_owner_DesktopIcon(TWins *self);
void SetFSDeviceOpt(TDesktopIcon *self, TpcIni *ini, char **ppc);


#define wmRst_ImgIsOk	1

#define TCSelf TwmRoot
typedef struct TwmRoot {
  AlgTForm
  int CurrentDesk;
  int DeskStartPid;
  int DesktopColor;
  char *DesktopImage;
  char *DesktopIcoColorMod;
  int stat;

  char *TitleFont;
  TFontX *wmTitleFont;
  TFontX *wmTitleFontUTF8;
  TPic *TitleH;
  TPic *TitleL;

  struct TwmWin *Focus_wmWin;
  struct TwmFrame *Top_wmf;

  TMenuFile *WinOptMenu;
  TMenu *WinOptMenu_moveDesk;
  TMenuFile *menuStart;
  TMenu *menu_Default_DeskIcon;
  struct TTaskbar *Taskbar;
#ifdef USE_DECOR
  TDecor *Default_Decor;
#endif
  char *AutostartPath;
  char *DesktopPath, *IconPosFile;
  char *MenuPath;
  char *AutoMountPath;
  char *sysblockPath;
  struct TBuildinKeys *BKy_key_up;
  struct TBuildinKeys *BuildinKeys;
  struct TwmWin *wmw_KeyDown;
  TPixMap *Selection_pxm;
#ifdef DRAG_AND_DROP
  TWins *xdnd_widget;
#endif
  TwmDeskIcon_Global di_glob;
  int BorderSnap;
  int DateFormat;
  Atom XA_ALDesk_DESTROY;

  TDirListObjNy *ObjNy_DeskIcons;
  TFSDevice_info *fs_info;
  TFSDevice_part_table *part_table;
} TwmRoot;
#undef TCSelf

TwmRoot *TwmRoot_Create(TWins *ws);
void TwmRoot_ClearStartNotify(int pid);
void menuStart_OnClick(TMenuFile *self, long KeyState, bool DblClick);

void REGP1 pcmove_directory(char *pc);
void REGP2 pcmove_desktop(char *pc, const char *name);
void Menu_OnMenuOpen(TMenu *self);
void Menu_OnMenuClose(TMenu *self);

void menu_start(TwmRoot *wmr);
void menu_desktop(TwmRoot *wmr);
void menu_FormOpt(TwmRoot *wmr);
void menu_DeskIcon(TwmRoot *wmr);
void meDeskIcon_mount_Click(TDesktopIcon *di, TpcIni *ini);
void meDeskIcon_umount_Click(TDesktopIcon *di, TpcIni *ini);

extern TwmRoot *wmRoot;

typedef struct {
  Atom XA_NET_WM_STATE_ABOVE;
  Atom XA_NET_WM_STATE_BELOW;
  Atom XA_NET_WM_STATE_MAXIMIZED_VERT;
  Atom XA_NET_WM_STATE_MAXIMIZED_HORZ;
  Atom XA_NET_WM_STATE_FULLSCREEN;
} TNetWMAtom;
typedef enum {
    NET_WM_NONE			= 0,
    NET_WM_STATE_ABOVE		= 1,
    NET_WM_STATE_BELOW		= 2
} TNetWMnum;
extern TNetWMAtom NetWMAtom;

TNetWMnum GetNetWMnum(Atom a);

extern int ExtQuitBef;
#if USE_NoFocusWin
extern Window NoFocusWin;
#endif
extern bool XwmError_Stop;
#ifdef	USE_DEBUG_WIN
extern Window last_event_window;
extern int last_event_type;
#endif

// #define wmDIst_pic1_akt		1
#define wmDIst_pic1_aktmk	(1<<1)
#define wmDIst_pic2_akt		(1<<2)
#define wmDIst_pic2_aktmk	(1<<3)
#define wmDIst_pic2chk		(1<<4)
//#define wmDIst_NotVisible	(1<<5)

#define wmDIst_MaskAkt		(1<<8)
#define wmDIst_HlAkt		(1<<9)

#define wmDImod_HighLight	1
#define wmDImod_Transparent	(1<<1)


//TDesktopIcon *TDesktopIcon_Create(TwmRoot *wmr, char *fname);
void TDesktopIcon_Destroy_fname(char *fn);
#define DIFu_Default		0
#define DIFu_Focus		1
#define DIFu_HighLight		2
#define DIFu_Force		(1<<16)
void TDesktopIcon_SetFunkt(TDesktopIcon *di, int fu);
void TDesktopIcon_ScanMousePos(TWins *self, XMotionEvent *ev, TWins *app);
void TDesktopIcon_SetSize(TDesktopIcon *di);

void TwmRoot_InitDeskIcons(TwmRoot *wmr);
TWins *TwmDeskIcon_Create(TwmRoot *wmr, char *fname);
void TwmRoot_AzMount(TgTimer *gti);

void TwmRoot_global(TwmRoot *wmr);

  /* desktop.c */
void TwmRoot_SetUpFont(TwmRoot *wmr, TpcIni *Ini);
void TwmRoot_SetBackground(TwmRoot *wmr, TpcIni *Ini);
void TwmRoot_SetBuildinStr(TwmRoot *self, TpcIni *Ini);
void ChkSetDefault();

//#define fl_MAPPED	1
#define fl_GrabButton		(1<<8)
#define fl_FormIsMaximize	(1<<9)

#define tfl_BORDER	1
#define tfl_TITLE	(1<<1)
#define tfl_MINIMIZE	(1<<2)
#define tfl_MINMAXIMIZE	(1<<3)
#if NEW_FLAGS_08400
#define tfl_NOINPUT	(1<<7)
#endif
#define tfl_NOAUTOPOS	(1<<8)
//#define tfl_ABSPOS	(1<<9)
#define tfl_ABOVE	(1<<12)

typedef struct {
  char *icon_name;
  char *WMName;
  char *WMIconName;
  XWMHints *WMHints;   
  XClassHint classhint;
} TwmWin_tcreate;

#define TCSelf TwmWin
typedef struct TwmWin {
  AlgTWins
  int Desk;
  int flags;
  int tflags;
  int WinGravity;
  int pid_NET_WM;
  int norm_left;
  int norm_top;
  int norm_width;
  int norm_height;
  XID window_group;
  struct TwmFrame *Frame;
#ifdef SAVE_PARENT_WIN
  Window SaveParentWinID;
#endif
  int border_width;
  TwmWin_tcreate *tcreate;  //only tmp used in create wmWin
} TwmWin;
#undef TCSelf

TwmWin *TwmWin_Create(Window win);
TwmWin *TwmWin_Create_base(Window win, XWindowAttributes *attr);
bool TwmWin__Destroy(TwmWin *self);

void TwmWin_Close(TwmWin *wmWin);
TwmWin *TwmWin_Find_WinsX_Ev(Window win);
void TwmWin_NetWMState(TwmWin *wmw, XClientMessageEvent *ev);
void TwmWin_PropertyNotify(TwmWin *wmw, XPropertyEvent *ev);

void TwmWin_GrabFocusWin(TwmWin *wmWin, bool grab);
void REGP3 TwmWin_CalcGravity(TwmWin *self, int *x, int *y);
int TwmWin_ChkSize(TwmWin *wmw, int *width, int *height);

#define wmwPos_SendPos		1
#define wmwPos_SendPosAlways	3
void TwmWin_SetupSize(TwmWin *wmw, int left, int top, int width, int height, int sendEvent);

void REGP2 TwmWin_SetMapState(TwmWin *wmWin, int State);

void menu_desktop_info_OnClick(TMenu *menu, long KeyState, TMenuItem* mi);

int GetNetWMPid(Window WinID);

/****** TwmFrame.c ******/
#define TCSelf TwmFrame
typedef struct TwmFrame {
  AlgTWins
  int Border_width;
  int Caption_height;
  Window transientfor;
  TwmWin *wmWin;
  Window wmWinID;
  struct TwmTitle *wmTitle;
  TWins *buTaskbarWin;
  int final_x, final_y, final_w, final_h;
} TwmFrame;
#undef TCSelf

extern GC MoveBorderGC;

TwmFrame *TwmFrame_Create(TwmWin *wmw, XWindowAttributes *attr);
void TwmFrame_GetClientRect(TwmFrame *self, TWRect *rc);
void TwmFrame_ClrAll_Hilite(TwmFrame *wmf);
TwmFrame *GetTop_DeskWin(TwmFrame *wmf);
void TwmFrame_SetOnTop(TwmFrame *wmf, int fu);

/****** TwmFrameBorder.c ******/
void TwmFrameBorder_Create(TwmRoot *wmr, TpcIni *Ini);

/****** TwmTitle.c ******/
#define wmtfl_hilite		1
//#define wmtfl_pic_memfree	(1<<16)
//#define wmtfl_pic_winfree	(1<<17)

#define TCSelf TwmTitle
typedef struct TwmTitle {
  AlgTWins
  int flags;
  TWins *buWinOptMenu;
  TButton *buMinimize;
  TButton *buMaximize;
  TButton *buClose;
  TPic *pic;
  TPic mpic;
} TwmTitle;
#undef TCSelf

TwmTitle *TwmTitle_Create(TwmFrame *wmf);
void TwmTitle_SetHilite(TwmTitle *wmt, bool Hilite);


/****** events.c ******/
void InitEventHandlerJumpTable();
void DeskDispatchEvent();

/****** desktop_exec.c ******/
void ExecDesktopFile(char *fname, TWins *wins);
void menu_ExecDesktopFile(TMenuFile *mf, long KeyState, TWins *wins);

#define exShowErrMsgBox	0x01
#define exNoStartPid	0x02
#define exAddTimer	0x04
#define exGrabPointer	0x08

void TwmDeskIcon_App_ExecEv(Texec_pipe* expipe, int evtyp);

/****** glob.c ******/

extern int glob_Border_width;
extern int glob_Caption_height;

void Create_globGC();
void CombineMask(Window WinID, Pixmap mask);
//void MoveOutline(Window root, int x, int y, int width, int height);
TPic *GetPic_directory(char *fpath, char **Paths1, char **Paths2, char **Caption);
TPic *GetPic_directory16(char *fpath, char **Caption);

void ShowExtMsgBoxEv(const char *msg, TOnExecProcEv OnExecProcEv);
void ShowExtMsgBox(const char *msg, ...);
void wmExcCMD_head(char **buf, const char *proc);
void wmExcCMD_headX(char **buf, const char *proc);

/****** misc.c ******/
bool chk_dev_removable(char *fn);

void SetXInputFocus(Window WinID);

void GrabXServer();
void UngrabXServer();
void SyncX();

int get_bef_dflEv(TFileMsg *fmsg, int *r_Bef);

/****** taskbar.c ******/
#define TASKBAR_HIDDE_NO	0
#define TASKBAR_HIDDE_YES	1
#define TASKBAR_HIDDE_AUTO	2

#define HIDDE_AUTO_COUNT	7

#define TCSelf TTaskbar
typedef struct TTaskbar {
  AlgTWins
  TWins *buStart;
  TPic *buStart_pic;
  TWins *tbCPU;
  TWins *tbMEM;
  TWins *tbTime;
  TWins *tbSysTray;
  TgTimer *tbTimer;
  TWins *tbDesktop;
  TWins *tbDeskButton[4];
  TWins *buTaskbarWin[4];
  TDecor *tbbDecor;
  Atom XA_NET_SYSTEM_TRAY_S;
  Atom XA_NET_SYSTEM_TRAY_OPCODE;
  int TimerZ;
  int Oldtm_sec;
  int hidde, hidde_tCount;
  int cpuh, scpuh, memh, cmemh, swph;

  dword oCPU_USER, oCPU_NICE, oCPU_SYS, oCPU_IDLE;
  int cpuFD;
  int memFD;
  char buf[3000];
} TTaskbar;
#undef TCSelf

TTaskbar *TTaskbar_Create();
void TTaskbar_AddWin(TTaskbar *tb, TwmFrame *wmf);
void TTaskbar_DelWin(TTaskbar *tb, TwmFrame *wmf);
void TTaskbar_DeskMoveWin(TTaskbar *tb, TwmFrame *wmf, int ToNewDesk);
void TTaskbar_Add_AppletWin(TTaskbar *tb, TwmWin *wmw);
bool TTaskbar_Chk_AppletWin_EvProc(Tobjmsg *msg, Window win);
void TTaskbar_SetUp(TwmRoot *wmRoot, TpcIni *appini);
void TTaskbar_Handle_SysTray_ev(TTaskbar *self, XClientMessageEvent *ev);
int  Taskbar_GetBottomOffset();
bool Taskbar_HandleKeyFunk(bool kf_open);

void REGP2 TTaskbar_buWin_SetUpDown(TWins *self, bool down);

/****** auto_mount.c ******/
void SetBootDev();
void Find_CDrom();

#if USE_SYS_BLOCK
void InitIcons_AutoMount(TwmRoot *wmr);
#endif

#if USE_PROC_PART
void Timer_AutoFsDev_proc_part();
#endif



/****** builtin.c ******/
#define cond_0		0
#define cond_ChkOnly	1

typedef struct TBuildinKeys {
  struct TBuildinKeys *next;
  int code, mod, modmask;
  KeySym sym;
  bool nc0; //isSym;
  bool nc1;
  bool nc2;
  bool nc3;
  char Action[20];
  char *cmd;
} TBuildinKeys;

void BuildinStrScane(TwmRoot *wmr, const char *s);
int exec_Buildin(TWins *ap, const char *Action, const char *cmd, int cond);
bool chk_AnyMenuOpen();

/****** keystat.c ******/
void setNumlock(bool on);
void SetUpKey(TpcIni *Ini);

/****** testwin.c ******/

#ifdef	USE_DEBUG_WIN
extern bool DebugOn;
void Chk_DebugWinRun();
void Debug_msg(int lev, const char *s, ...);
#else
#define Debug_msg(lev, fmt, ...)
#endif


/****** MwmUtil.h ******/
typedef struct TMwmHints {
    unsigned long flags;
    unsigned long functions;
    unsigned long decorations;
    long input_mode;
    unsigned long status;
} TMwmHints;

#define MWM_HINTS_FUNCTIONS     (1L << 0)
#define MWM_HINTS_DECORATIONS   (1L << 1)
#define MWM_HINTS_INPUT_MODE    (1L << 2)
#define MWM_HINTS_STATUS        (1L << 3)

#define MWM_FUNC_ALL            (1L << 0)
#define MWM_FUNC_RESIZE         (1L << 1)
#define MWM_FUNC_MOVE           (1L << 2)
#define MWM_FUNC_MINIMIZE       (1L << 3)
#define MWM_FUNC_MAXIMIZE       (1L << 4)
#define MWM_FUNC_CLOSE          (1L << 5)

#define MWM_DECOR_ALL           (1L << 0)
#define MWM_DECOR_BORDER        (1L << 1)
#define MWM_DECOR_RESIZEH       (1L << 2)
#define MWM_DECOR_TITLE         (1L << 3)
#define MWM_DECOR_MENU          (1L << 4)
#define MWM_DECOR_MINIMIZE      (1L << 5)
#define MWM_DECOR_MAXIMIZE      (1L << 6)

#define MWM_INPUT_MODELESS 0
#define MWM_INPUT_PRIMARY_APPLICATION_MODAL 1
#define MWM_INPUT_SYSTEM_MODAL 2
#define MWM_INPUT_FULL_APPLICATION_MODAL 3
#define MWM_INPUT_APPLICATION_MODAL MWM_INPUT_PRIMARY_APPLICATION_MODAL

#define MWM_TEAROFF_WINDOW	(1L<<0)

