/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/
#ifndef STANDALONE_BUILDIN
#include "buildin_app.h"
#endif
#include "appXbase.h"
#include "tfilewin.h"
#include "dlg.h"
#include "image.h"

#ifdef _OS_WIN32

#ifdef DRAG_AND_DROP
#undef DRAG_AND_DROP
#endif

#ifdef Test_Root_WinId
#undef Test_Root_WinId
#endif

#endif /* _OS_WIN32 */

static const char *bi_FileBox_Help_Msg = {
  "\n"
  "Usage: aldfilebox [options] \"Path\"\n"
  "\n"
  "  aldfilebox -x10 -y10 -t\"File Open\" \"/home/*\"\n"
  "\n"
  "  -x -y                positon x y\n"
  "  -W -H                size width height\n"
  "  -t                   Title\n"
  "  -p                   pattern  (-p=*.xpm,*.png)\n"
  "  --ft                 File typ (--ft=17,23,24) (in sysbase.h)\n"
  "                         txt=17 Shell=21 C=23,24 HTML=25 Make=27 \n"
  "  --ra                 RootArt:/File\n"
#ifdef DRAG_AND_DROP
  "  --dnd                drag and drop enabled\n"
#endif
  "  -n                   no return msg\n"
  "  -h, --help           Prints out this screen\n"
  "  return Filename\n"
};

#ifdef STANDALONE_BUILDIN
int main (int argc, char **argv) {
	cmdline_slash_korr(argc, argv);
#else
int bi_FileBox_main (int argc, char **argv) {
#endif
  int i, m;
  char *pc, *title, *path;
  char *pattern;
  unsigned short ftyp[16];
  int ftypZ = 0;
  int x = 0;
  int y = 0;
  int width = 0;
  int height = 0;
  TFile_Dlg *File_Dlg;
  int RetName_gp_fu = gptfu_noRootArt;
  bool useXY = False;
  bool help = False;
  bool err = False;
  bool dnd = False;
  bool RetMsg = True;

	title = NULL;
	path = NULL;
	pattern = NULL;
	ftyp[0] = 0;
	for (i=1; i<argc; i++) {
	    pc = argv[i];
	    if (*pc == '-') {
		pc++;
		if (*pc == 'h') {
		    help = True;
		} else if (*pc == 'n') {
		    RetMsg = False;
		} else if (*pc == 'p') {
		    while (*pc && *pc != '=') pc++;
		    if (*pc) pc++;
		    while (*pc && *pc == ' ') pc++;
		    if (*pc) pattern = pc; else err = True;
		} else if (*pc == 't') {
		    pc++;
		    title = pc;
		    if (*title == '\"') {
			title++;
			m = pclen(title)-1;
			if (m > 0 && title[m] == '\"') title[m] = 0;
		    }
		} else if (*pc == 'W') {
		    pc++;
		    width = pctoint(pc);
		} else if (*pc == 'H') {
		    pc++;
		    height = pctoint(pc);
		} else if (*pc == 'x') {
		    pc++;
		    x = pctoint(pc);
		    useXY = True;
		} else if (*pc == 'y') {
		    pc++;
		    y = pctoint(pc);
		    useXY = True;
		} else if (*pc == '-') {
		    pc++;
		    if (pcncmp(pc, "ft", 2)) {
			while (*pc && *pc != '=') pc++;
			if (*pc) pc++;
			while (*pc && *pc == ' ') pc++;
			if (*pc) {
			    while (*pc) {
				ftyp[ftypZ] = ppctoint_scan((void*)&pc, True);
				ftypZ++;
				if (ftypZ > 15) ftypZ = 15;
				ftyp[ftypZ] = 0;
			    }
			} else err = True;
		    } else if (pcncmp(pc, "help", 4)) {
			help = True;
		    } else if (pcncmp(pc, "ra", 3)) {
			RetName_gp_fu &= ~gptfu_noRootArt;
		    } else if (pcncmp(pc, "dnd", 3)) {
			dnd = True;
#ifdef Test_Root_WinId
		    } else if (pcncmp(pc, "TestRootWinID", 13)) {
			i++;
			pc = argv[i];
			if (pc && *pc != '-') {
				TestRootWinId = pctoint(pc);
			} else err = True;
#endif
		    } else err = True;
		} else err = True;
	    } else {
		path = pc;
		if (*path == '\"') {
		    path++;
		    m = pclen(path)-1;
		    if (m > 0 && path[m] == '\"') path[m] = 0;
		}
	    }
	}

	if (help || err) {
		printf(bi_FileBox_Help_Msg);
		exit(1);
	}

	initXapp(0, argc, argv);

#ifdef USE_DECOR
	appXDat.DefaultButtonDecor = TDecor_button_w3b3_Create();
#endif

	ThImageRD_Reg(3, &hImageRD_xpm, &hImageRD_png, &hImageRD_jpeg); 
//			 &hImageRD_gif, &hImageRD_bmp, &hImageRD_pnm);

#ifdef _OS_WIN32
	ThDirList_Reg(1, &hdirlist_ld_File);
	ThDirList_ny_Reg(1, &hdirlist_ny_File);
#else
	ThDirList_Reg(1, &hdirlist_ld_File);
//	ThDirList_Reg(2, &hdirlist_ld_File, &hdirlist_ld_ftp);
	ThDirList_ny_Reg(2, &hdirlist_nyi_File, &hdirlist_ny_File);
#endif

	hGetFileHeadInfo = &hGetFileHeadInfo_buildin;

	File_Dlg = Create_FileAlg_Dlg(NULL);
	file_dlg_InitAzPic(File_Dlg);
	if (useXY) {
	    File_Dlg->FormPosition = poDesigned;
	    File_Dlg->top = y;
	    File_Dlg->left = x;
	}

	if (width) File_Dlg->width = width;
	if (height) File_Dlg->height = height;

	if (title) {
	    SetCaption((TWins*)File_Dlg, title);
	} else {
	    SetCaption((TWins*)File_Dlg, "File Open");
	}

#ifdef DRAG_AND_DROP
	if (dnd) TFileWin_dnd_master_init(File_Dlg->FileWin);
// printf("%s(%d):  FormW=0x%x FileW=0x%x\n", FL, (int)File_Dlg->WinID, (int)File_Dlg->FileWin->WinID);
#endif

	path = path_rel2abs(path, NULL);
	File_Dlg_SetNamePath(File_Dlg, path);
	File_Dlg->RetName_gp_fu = RetName_gp_fu;

	if (pattern || ftyp[0]) {
	    File_Dlg_AddPattern(File_Dlg, pattern, ftyp);
	}

	if (File_Dlg_Execute(File_Dlg)) {
	    pc = NULL;
	    File_Dlg_GetEdFileName(File_Dlg, &pc);
	    if (RetMsg) printf("%s\n", pc);
	}
	return 0;
}
