/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/
#ifndef STANDALONE_BUILDIN
#include "buildin_app.h"
#endif
#include "appXbase.h"
#include "dlg.h"

static bool RetMsg;

static const char *bi_MsgBox_Help_Msg = {
  "\n"
  "Usage: aldmessagebox [options] \"Caption\"\n"
  "\n"
  "  aldmessagebox -fOK,NO,SKIP -dNO -x10 -y10 -t\"Message Box\" \"Test msg\"\n"
  "\n"
  "  -f Button function   OK      OKALL   ABORT   RETRY\n"
  "                       IGNORE  YES     YESALL  NO\n"
  "                       SKIP    SKIPALL REPEAT  CANCEL\n"
  "  -d default Button    OK..CANCEL\n"
  "  -x -y                positon x y\n"
  "  -s                   screen width x height -s640x480\n"
  "  -t                   Title / Message\n"
  "  -n                   no return Button msg\n"
  "  -w                   w0=warning w1=error w1=info\n"
  "  -h, --help           Prints out this screen\n"
  "  return Button        OK..CANCEL\n"
};

static const char *ButtenFu[] = {
 "OK",		"OKALL",	"ABORT",	"RETRY",
 "IGNORE",	"YES",		"YESALL",	"NO",
 "SKIP",	"SKIPALL",	"REPEAT",	NULL,
 NULL,		NULL,		NULL,		"CANCEL",
 NULL,		NULL,		NULL,		NULL};

static void Scanfu(const char *pc, int lg, unsigned int *res) {
  int i;
  unsigned int k = 1;
	for (i=0;i<16;i++) {
	    if (ButtenFu[i] && (pclen(ButtenFu[i]) == lg) && !pcncasecmpASCII(ButtenFu[i], pc, lg)) {
		*res |= k;
		return;
	    }
	    k += k;
	}
}

static unsigned int Scanfu_pc(const char *pc) {
  int lg;
  unsigned int res = 0;
  const char *npc = pc;
	lg = 0;
	while (*npc) {
	    if (*npc == ',') {
		Scanfu(pc, lg, &res);
		lg = 0;
		npc++;
		pc = npc;
	    } else {
		lg++; npc++;
	    }
	}
	if (lg) Scanfu(pc, lg, &res);
	return res;
}

TMsgBox *MsgBox;

static void bi_MsgBox_OnAppClose() {
  int i;
  int m = MsgBox->RetCode;
  const char *pc = "NULL";
	if (!RetMsg) return;
	for (i=0;i<16;i++) {
	    if (m & 1) {pc = ButtenFu[i]; break;}
	    m = m >> 1;
	}
	printf("%s\n", pc);
}

#ifdef STANDALONE_BUILDIN
int main (int argc, char **argv) {
#else
int bi_MsgBox_main (int argc, char **argv) {
#endif
  int i, m;
  char *pc, *pcs, *msg;
  TMsgBoxParam mbp;
  dword fu = msgOK;
  dword fu_ico = 0;
  bool help = False;

	Fill0(&mbp, sizeof(mbp));

	RetMsg = True;
	msg = NULL;
	for (i=1; i<argc; i++) {
	    pc = argv[i];
	    if (*pc == '-') {
		pc++;
		if (*pc == 'f') {
		    pc++;
		    fu = Scanfu_pc(pc);
		} else if (*pc == 'd') {
		    pc++;
		    mbp.FocusButton = Scanfu_pc(pc);
		    mbp.akt |= mbpakt_FocusButton;
		} else if (*pc == 'n') {
		    RetMsg = False;
		} else if (*pc == 's') {
		    pc++;
		    mbp.scrwidth = ppctoint_scan((void*)&pc, True);
		    mbp.scrheight = ppctoint_scan((void*)&pc, True);
		    mbp.akt |= mbpakt_scr;
		} else if (*pc == 't') {
		    pc++;
		    if (*pc == '\"') {
			pc++;
			m = pclen(pc)-1;
			if (m > 0 && pc[m] == '\"') pc[m] = 0;
		    }
		    mbp.Caption = pc;
		    mbp.akt |= mbpakt_Caption;

		} else if (*pc == 'w') {
		    pc++;
		    if (*pc == '0') fu_ico = msgIcoWARNING; else
		    if (*pc == '1') fu_ico = msgIcoERROR; else
		    if (*pc == '2') fu_ico = msgIcoINFO;
		} else if (*pc == 'x') {
		    pc++;
		    mbp.left = pctoint(pc);
		    mbp.akt |= mbpakt_x_y;
		} else if (*pc == 'y') {
		    pc++;
		    mbp.top = pctoint(pc);
		    mbp.akt |= mbpakt_x_y;
#ifdef Test_Root_WinId
		} else if (*pc == '-') {
		    pc++;
		    if (pcncmp(pc, "TestRootWinID", 13)) {
			i++;
			pc = argv[i];
			if (pc && *pc != '-') {
				TestRootWinId = pctoint(pc);
			} else help = True;
		    } else help = True;
#endif
		} else help = True;
	    } else {
		if (*pc == '\"') {
		    pc++;
		    msg = pcdup(pc);
		    m = pclen(msg)-1;
		    if (m > 0 && msg[m] == '\"') msg[m] = 0;
		} else msg = pcdup(pc);
	    }
	}

	if (help) {
	    printf(bi_MsgBox_Help_Msg);
	    exit(1);
	}

	initXapp(0, argc, argv);
	appXDat.OnAppClose = bi_MsgBox_OnAppClose;
#ifdef USE_DECOR
	appXDat.DefaultButtonDecor = TDecor_button_w3b3_Create();
#endif
	if (msg) {
	    pc = msg;
	    pcs = msg;
	    while (*pcs) {
		*pc = *pcs;
		if ((*pcs == '\\') && (pcs[1] == 'n')) {
			*pc = '\n'; pcs++;
		}
		pc++; pcs++;
	    }
	}

	MsgBox = TMsgBox_Create(NULL, msg, fu | fu_ico, &mbp);
	MsgBox->Visible = True;
#ifdef MEM_CHK
#ifdef MEM_CHKappAZ
#ifndef STANDALONE_BUILDIN
	MEM_AZ_Disable = True;
#endif
#endif
#endif
	AppRun();
	return 0;
}
