/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldesk.h"
#include <sys/wait.h>

typedef struct {
  const char *key;
  int  mod;
} TxModKey;

static const TxModKey xModKey[]= {
  {"Shift",ShiftMask},
  {"Lock", LockMask},
  {"Control",ControlMask},
  {"Strg",ControlMask},
  {"Mod1",Mod1Mask},
  {"Mod2",Mod2Mask},
  {"Mod3",Mod3Mask},
  {"Mod4",Mod4Mask},
  {"Mod5",Mod5Mask},
  {0,0}
};

static int KeyFunk_GetKeycode(const char *s, int lg, KeySym *keysym) {
  char line[80];
  int res = 0;
  char *ks;
	ks = line;
	ppcnmove(&ks, s, lg);
	res = pctoint(line);
	if (!res) {
	    *keysym = XStringToKeysym(line);
	    if (*keysym) res = XKeysymToKeycode(appXDat.dpy, *keysym);
// printf("%s(%d): s=%s ks=0x%04x code=%d\n", FL, line, (int)*keysym, res);
	}
	return res;
}

static bool AddKeyFunk(TwmRoot *wmr, const char *s) {
  char *pc;
  const char *pce, *sx, *toc;
  int i, lg, toclg;
  int mod = 0;
  int modmask = 0xff;
  int code = 0;
  KeySym sym;
  bool mk, err = False;

	sx = pcGetToken(s, &toc, &toclg);
	sx = pcGetToken(sx, &toc, &toclg);
	if (!toc || toclg > 19) return False;
kloop:
	pce = s;
	while (((byte)*pce > 0x20) && (*pce != '-') && (*pce != '+')) pce++;
	mk = False;
	lg = pce - s;
	if (lg > 0 && ((*pce == '+')||(*pce == '-'))) {
	    i = 0;
	    while (!mk && xModKey[i].key) {
		mk = (pclen(xModKey[i].key) == lg) 
		     && !pcncasecmpASCII(xModKey[i].key, s, lg);
		if (mk && (*pce == '-')) mod |= xModKey[i].mod;
		if (mk && (*pce == '+')) modmask &= ~xModKey[i].mod;
		i++;
	    }
	    if (!mk) err = True;
	    pce++;
	    s = pce;
	    if (!err) goto kloop;
	}
	sym = 0;
	if (!err && lg > 0) {
	    code = KeyFunk_GetKeycode(s, lg, &sym);
// printf("code=%d sym=0x%04x mod=0x%02x mask=0x%02x %d \"%s\"\n", code, (int)sym, mod, modmask, lg, s);
	    err = (code == 0);
	}
	if (err) return False;

  TBuildinKeys *BKy, **ppBKy;
	ppBKy = &wmr->BuildinKeys;
	while (*ppBKy) ppBKy = &(*ppBKy)->next;

	BKy = xcalloc(sizeof(TBuildinKeys));
	*ppBKy = BKy;
	BKy->mod = mod;
	BKy->modmask = modmask;
	BKy->code = code;
	BKy->sym = sym;

	pc = BKy->Action;
	ppcnmove(&pc, toc, toclg);
	if (sx) BKy->cmd = pcdup(sx);
// printf("%s %s\n", BKy->Action, BKy->cmd); 

	return True;
}

void BuildinStrScane(TwmRoot *wmr, const char *s) {
  int m;
  const char *indata, *toc;

	indata = s;
	if (!indata) return;
	indata = pcGetToken(indata, &toc, &m);
	if (toc) {
	    if ((m == 3) && !pcncasecmpASCII(toc, "Key", 3)) {
		indata = pcGetToken(indata, &toc, &m);
		if ((m == 3) && !pcncasecmpASCII(toc, "add", 3)) {
		    if (!AddKeyFunk(wmr, indata)) {
			printf("%s(%d) ERROR: Key add >%s<\n", FL, indata);
			return;
		    }
		}
	    }
	}
}

typedef struct Tfk_Daten {
  TWins *wins;
  const char *cmd;
  int cond;
  int ret;
} Tfk_Daten;

typedef struct Tfk_Buildin {
  const char *Action;
  void REGP1 (*fk_)(Tfk_Daten *fk_Daten);
} Tfk_Buildin;

static TwmWin *fk_get_wmw(Tfk_Daten *fk_Daten) {
	if (!fk_Daten || !fk_Daten->wins) return NULL;
	if (fk_Daten->wins->wtyp != wtyp_TwmWin) return NULL;
	return (TwmWin*)fk_Daten->wins;
}

static void REGP1 fk_close_win(Tfk_Daten *fk_Daten) {
	TwmWin_Close(fk_get_wmw(fk_Daten));
}

static void REGP1 fk_kill_win(Tfk_Daten *fk_Daten) {
  TwmWin *wmw = fk_get_wmw(fk_Daten);
	if (!wmw) return;
	if (wmw->pid_NET_WM) {
	    kill(wmw->pid_NET_WM, SIGKILL);
	} else {
	    TwmWin_Close(wmw);
	}
}

static void REGP1 fk_close_all_win(Tfk_Daten *fk_Daten) {
  TwmFrame *wmf, *wmfn;

	wmf = (TwmFrame*)appXDat.Wins;
	while (wmf) {
	    wmfn = (TwmFrame*)wmf->next;
	    if (wmf->wtyp == wtyp_TwmFrame) {
		TwmWin_Close(wmf->wmWin);
		AppHandleMessage();
	    }
	    wmf = wmfn;
	}
}

static void REGP1 fk_minmax_win(Tfk_Daten *fk_Daten) {
  int h;
  TwmWin *wmw = fk_get_wmw(fk_Daten);
	if (!wmw) return;
	if (fk_Daten->cond == cond_ChkOnly) return;

	if (wmw->flags & fl_FormIsMaximize) {
	    wmw->flags &= ~fl_FormIsMaximize;
	    TwmWin_SetupSize(wmw, wmw->norm_left, wmw->norm_top, wmw->norm_width, wmw->norm_height, False);
	} else {
	    wmw->flags |= fl_FormIsMaximize;
	    wmw->norm_left = wmw->Frame->left;
	    wmw->norm_top = wmw->Frame->top;
	    wmw->norm_width = wmw->Frame->width;
	    wmw->norm_height = wmw->Frame->height;
	    h = appXDat.Root_height - Taskbar_GetBottomOffset();
	    TwmWin_SetupSize(wmw, 0, 0, appXDat.Root_width, h, wmwPos_SendPos);
	}
}

static void REGP1 fk_exit(Tfk_Daten *fk_Daten) {
	Close();
}

static void REGP1 fk_exec(Tfk_Daten *fk_Daten) {
	if (fk_Daten->cond == cond_ChkOnly) return;
// printf("%s(%d): 0x%x %s %d\n", FL, (int)fk_Daten->ap, fk_Daten->cmd, fk_Daten->cond);
	ExecuteProc(fk_Daten->cmd);
}

/* ****** Menue ****** */
bool chk_AnyMenuOpen() {
	return (wmRoot->menuStart->Visible || 
		wmRoot->WinOptMenu->Visible || 
		wmRoot->PopupMenu->Visible);
}



static void REGP1 fk_close_menu(Tfk_Daten *fk_Daten) {
	if (fk_Daten->cond == cond_ChkOnly) {
	    fk_Daten->ret = chk_AnyMenuOpen();
	    return;
	}
	if (wmRoot->menuStart->Visible) {
	    TMenu_Close((TMenu*)wmRoot->menuStart);
	} else if (wmRoot->WinOptMenu->Visible) {
	    TMenu_Close((TMenu*)wmRoot->WinOptMenu);
	} else if (wmRoot->PopupMenu->Visible) {
	    TMenu_Close(wmRoot->PopupMenu);
	} else if (fk_Daten) fk_Daten->ret = False;
}

static bool _fk_close_menu(Tfk_Daten *fk_Daten) {
  bool res;
	fk_close_menu(fk_Daten);
	res = fk_Daten->ret;
	fk_Daten->ret = True;
	return res;
}

static void REGP1 fk_menu_start(Tfk_Daten *fk_Daten) {
  TWins *buStart;
	if (fk_Daten->cond == cond_ChkOnly) return;
	if (_fk_close_menu(fk_Daten)) {
	    Taskbar_HandleKeyFunk(false);
	    return;
	}
	if (!wmRoot->menuStart || !Taskbar_HandleKeyFunk(true)) return;
	buStart = wmRoot->Taskbar->buStart;
	if (!buStart) return;

  TWRect rc;
	if (!GetXWinAbs_rc(buStart->WinID, &rc)) return;
	TMenu_Show((TMenu*)wmRoot->menuStart, (TWins*)wmRoot, rc.left-2, rc.top-2);
	buStart->BorderFu |= mBorderL;
	Refresh(buStart);
}

static void REGP1 fk_menu_form_opt(Tfk_Daten *fk_Daten) {
  TwmWin *wmw;
	if (fk_Daten->cond == cond_ChkOnly) return;
	if (chk_AnyMenuOpen()) return;
	wmw = fk_get_wmw(fk_Daten);
	if (!wmw || !wmRoot->WinOptMenu) return;
	if (!wmw->Frame || !wmw->Frame->wmTitle) return;
  TWins *buWinOptMenu = wmw->Frame->wmTitle->buWinOptMenu;
	if (!buWinOptMenu) return;

  TWRect rc;
	if (!GetXWinAbs_rc(buWinOptMenu->WinID, &rc)) return;
	TMenu_Show((TMenu*)wmRoot->WinOptMenu, (TWins*)wmw, rc.left, rc.top+buWinOptMenu->height);
}

static void snapshot_WinID(Window WinID) {
	wmRoot->Selection_pxm = TPixMap_Create_XID(WinID, 0);
 Debug_msg(0, "fk_snapshot 0x%08lx\n", (long)wmRoot->Selection_pxm);
	if (wmRoot->Selection_pxm) {
	    TPixMap_XGetImage(wmRoot->Selection_pxm);
	    wmRoot->Selection_pxm->pmxID = 0;
	    wmRoot->Selection_pxm->maskID = 0;
	    wmRoot->Selection_pxm->stat &= ~(pmxStat_XPixAkt|pmxStat_XMaskPixAkt);
	}
	SetSelection_TPixMap(wmRoot->Selection_pxm);
}

static void REGP1 fk_snapshot(Tfk_Daten *fk_Daten) {
	if (fk_Daten->cond == cond_ChkOnly) return;

	Tfree(wmRoot->Selection_pxm);
	snapshot_WinID(wmRoot->WinID);
}
static void REGP1 fk_snapshot_form(Tfk_Daten *fk_Daten) {
  TWins *win;

	if (fk_Daten->cond == cond_ChkOnly) return;
	ppTfree(&wmRoot->Selection_pxm);
 Debug_msg(0, "snapshot_form 0x%08lx 0x%08lx\n", (long)wmRoot->Focus_wmWin, (long)appXDat.GrabWin);

	win = appXDat.GrabWin;
	if (!win && wmRoot->Focus_wmWin) {
	    win = (TWins*)wmRoot->Focus_wmWin;
	    if (wmRoot->Focus_wmWin->Frame) win = (TWins*)wmRoot->Focus_wmWin->Frame;
	}

	Window WinID = None;
	if (win) WinID = win->WinID;
	snapshot_WinID(WinID);
}

struct Tfk_Buildin fk_Buildin[] = {
//  {"exec", fk_exec},
  {"close_win", fk_close_win},
  {"kill_win", fk_kill_win},
  {"close_all_win", fk_close_all_win},
  {"minmax_win", fk_minmax_win},
  {"Exit", fk_exit},
  {"exec", fk_exec},
  {"menu_form_opt", fk_menu_form_opt},
  {"menu_start", fk_menu_start},
  {"close_menu", fk_close_menu},
  {"snapshot", fk_snapshot},
  {"snapshot_form", fk_snapshot_form},
  {NULL, NULL}
};

int exec_Buildin(TWins *wins, const char *Action, const char *cmd, int cond) {
  Tfk_Daten fk_Daten;
  int i;

	fk_Daten.wins = wins;
	fk_Daten.cmd = cmd;
	fk_Daten.cond = cond;
	fk_Daten.ret = True;
	i = 0;
	while (fk_Buildin[i].Action) {
	    if (!pccasecmpASCII(fk_Buildin[i].Action, Action)) {
		fk_Buildin[i].fk_(&fk_Daten);
		return fk_Daten.ret;
	    }
	    i++;
	}
	printf("ERROR: aldesk Buildin \"%s\" not found\n", Action);
	return False;
}
