/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldesk.h"
#include <time.h>
#include <unistd.h>
#include <fcntl.h>

#define PROCSTAT "/proc/stat"
#define PROCMEMINFO "/proc/meminfo"

#define tb_DefaultWidth 160

void REGP2 TTaskbar_buWin_SetUpDown(TWins *self, bool down) {
	if (!self) return;
	self->BorderFu &= ~mBorderL;
	if (down) self->BorderFu |= mBorderL;
	Refresh(self);
}

static void TTaskbar_buWin_OnMouseDown(TWins *self, XButtonEvent *ev) {
  TwmFrame *wmf = (TwmFrame*)self->tag;
	if (ev->button == Button1) {
	    SetVisible((TWins*)wmf, True);
	    if (wmRoot->Focus_wmWin != wmf->wmWin) {
		SetXInputFocus(wmf->wmWinID);
	    }
	} else if (ev->button == Button3) {
	    if (wmRoot->WinOptMenu) {
		wmRoot->WinOptMenu->owner = (TWins*)wmf->wmWin;
		TMenu_Show((TMenu*)wmRoot->WinOptMenu, self, ev->x_root, ev->y_root);
	    }
	}
}

static void TTaskbar_buWin_OnPaint(TWins *self) {
  TwmFrame *wmf = (TwmFrame*)self->tag;
	if (wmf && wmf->wmTitle) {
	    DrawPic(self->WinID, wmf->wmTitle->pic, 3, 3);
	}
}

static int TTaskbar_buWin_GetMax_w(TWins *self) {
	int max_w = self->width - self->CaptionOffX - 4;
	if (max_w < 60) max_w = 60;
	return max_w;
}

static void TTaskbar_buWin_OnResize(TWins *self) {
  TwmFrame *wmf = (TwmFrame*)self->tag;

	if (!wmf || !wmf->wmTitle) return;
	int max_w = TTaskbar_buWin_GetMax_w(self);
	char *pc = vds_path_cut(&wmf->vds, wmf->vds.s, max_w);
	if (!pccmp(pc, self->vds.s)) vds_SetStr(&self->vds, pc);
	xfree(pc);
}

static void TTaskbar_buWin_SetCaption(TWins *self, const char *s) {
	int max_w = TTaskbar_buWin_GetMax_w(self);
	char *pc = vds_path_cut(&self->vds, s, max_w);
	TWins_SetCaption(self, pc);
	xfree(pc);
}

static void TTaskbar_SetPosSizeWin(TTaskbar *self, int Desk) {
  TWins *tbw;
  int count = 0;
  int w, x;

	if (Desk != wmRoot->CurrentDesk) return;
	tbw = self->buTaskbarWin[Desk];
	while (tbw) {count++; tbw = tbw->owner;}
	if (!count) return;

	x = self->buStart->left + self->buStart->width + 2;
	w = self->width - x - self->tbMEM->width - self->tbTime->width -
		self->tbSysTray->width - self->tbDesktop->width - 6;
	w = w / count;
	if (w > tb_DefaultWidth) w = tb_DefaultWidth;
	if (w < 4) w = 4;
	tbw = self->buTaskbarWin[Desk];
	while (tbw) {
	    SetRC(tbw, x, tbw->top, w, tbw->height);
	    x += w;
	    tbw = tbw->owner;
	}
}

void TTaskbar_AddWin(TTaskbar *self, TwmFrame *wmf) {
  TWins **pptbw, *tbw;

	if (!self || !wmf || !wmf->wmTitle) return;
	tbw = TWins_CreateLabel((TWins*)self, 60, 2, tb_DefaultWidth, 22, "New", 
				mCaptionLeft, mBorder | mBorderBu | mBorderW2);
#ifdef POINTER64BIT
	tbw->tag = (int64_t)wmf;
#else
	tbw->tag = (int)wmf;
#endif
	tbw->Visible = (wmf->wmWin->Desk == wmRoot->CurrentDesk);
	tbw->CaptionOffX = 3;
	if (wmf->wmTitle->pic) tbw->CaptionOffX += wmf->wmTitle->pic->width + 1;

	tbw->OnPaint = TTaskbar_buWin_OnPaint;
	tbw->OnMouseDown = TTaskbar_buWin_OnMouseDown;
	tbw->OnResize = TTaskbar_buWin_OnResize;
	tbw->SetCaption = TTaskbar_buWin_SetCaption;

#ifdef USE_DECOR
	tbw->Decor = TDecor_dup(&self->tbbDecor);
#endif

	Set_Parent((TWins*)tbw, (TWins*)self);

	pptbw = &self->buTaskbarWin[wmf->wmWin->Desk];
	while (*pptbw) pptbw = &(*pptbw)->owner;
	*pptbw = tbw;

	wmf->buTaskbarWin = tbw;
	TTaskbar_SetPosSizeWin(self, wmf->wmWin->Desk);
}

static TWins **get_pptbw(TTaskbar *self, TwmFrame *wmf) {
  TWins **pptbw;
	pptbw = &self->buTaskbarWin[wmf->wmWin->Desk];
	while (*pptbw && ((void*)(*pptbw)->tag != (void*)wmf)) pptbw = &(*pptbw)->owner;
	return pptbw;
}

void TTaskbar_DelWin(TTaskbar *self, TwmFrame *wmf) {
  TWins **pptbw, *tbw;

	if (!self || !wmf || !wmf->buTaskbarWin) return;
	wmf->buTaskbarWin = NULL;
	pptbw = get_pptbw(self, wmf);
	if (*pptbw) {
	    tbw = *pptbw;
	    *pptbw = tbw->owner;
	    Tfree(tbw);
	}
	TTaskbar_SetPosSizeWin(self, wmf->wmWin->Desk);
}

void TTaskbar_DeskMoveWin(TTaskbar *self, TwmFrame *wmf, int ToNewDesk) {
  TWins **pptbw, *tbw;

	if (!self || !wmf || !wmf->buTaskbarWin) return;
	pptbw = get_pptbw(self, wmf);
	if (!*pptbw) return;

	tbw = *pptbw;
	*pptbw = tbw->owner;
	tbw->owner = NULL;

	pptbw = &self->buTaskbarWin[ToNewDesk];
	while (*pptbw) pptbw = &(*pptbw)->owner;
	*pptbw = tbw;

	SetVisible(tbw, False);
	TTaskbar_SetPosSizeWin(self, wmf->wmWin->Desk);

	MapWinID(wmf->WinID, 0);
	wmf->wmWin->Desk = ToNewDesk;
}

static void buStart_OnMouseDown(TWins *self, XButtonEvent *ev) {
	exec_Buildin(self, "menu_start", NULL, cond_0);
}

static void buStart_OnPaint(TWins *self) {
	DrawPic(self->WinID, ((TTaskbar*)self->prev_child)->buStart_pic, 1, 1);
}

static void TTaskbar_ChangeDeskWin(TTaskbar *self, int Desk) {
  TWins *tbw;

	tbw = self->buTaskbarWin[wmRoot->CurrentDesk];
	while (tbw) {SetVisible(tbw, False); tbw = tbw->owner;}

	tbw = self->buTaskbarWin[Desk];
	while (tbw) {SetVisible(tbw, True); tbw = tbw->owner;}
}

static void SetCurrentDesk(int Desk) {
  TwmFrame *wmf;
  TwmFrame *topF = NULL;

	wmf = (TwmFrame*)appXDat.Wins;
	while (wmf) {
	    if (wmf->wtyp == wtyp_TwmFrame) {
		if (wmf->wmWin->Desk == Desk) {
		    if (wmf->Visible) {
			if (!topF) topF = wmf;
			MapWinID(wmf->WinID, 1);
		    }
		} else {
		    MapWinID(wmf->WinID, 0);
		}
	    }
	    wmf = (TwmFrame*)wmf->next;
	}
	if (topF) SetXInputFocus(topF->wmWinID);
}

static void tbDeskButton_DoMouseDown(TWins *self, XButtonEvent *ev) {
  TTaskbar *tb = (TTaskbar*)self->owner;

	if (wmRoot->CurrentDesk == self->tag) return;
	for (int i=0;i<4;i++) TTaskbar_buWin_SetUpDown(tb->tbDeskButton[i], False);
	tb->tbDeskButton[self->tag]->BorderFu |= mBorderL;
	SetCurrentDesk(self->tag);
	TTaskbar_ChangeDeskWin(tb, self->tag);
	wmRoot->CurrentDesk = self->tag;
	TTaskbar_SetPosSizeWin(tb, self->tag);
}

typedef struct TtbMemInfo {
  dword Total, Free, Cached, SwapTotal, SwapFree;
} TtbMemInfo;

#define USE_REOPEN 0

#if USE_REOPEN == 0
static int tb_read(TTaskbar *tb, int *pfd) {
  int fd = *pfd;
	if (fd <= 0) return 0;
	lseek(fd, 0, 0);
	int L = read(fd, tb->buf, sizeof(tb->buf)-1);
	if (L < 0) {
	    printf("%s(%d): Read error on fd=%d\n", FL, fd);
	    close(fd);
	    *pfd = 0;
	}
	return L;
}
#endif

static int __attribute__ ((noinline)) Get_tbMemInfo(TTaskbar *tb, TtbMemInfo *MemInfo) {
  int resmk = 0;
  char *tline;

#if USE_REOPEN
	int fd = open(PROCMEMINFO, O_RDONLY);
	if (fd > 0) {
	    int L = read(fd, tb->buf, sizeof(tb->buf)-1);
	    close(fd);
	    if (L > 10) {
#else
	int L = tb_read(tb, &tb->memFD);
	if (L > 10) {
#endif
// dword t = GetTime_us();
	    tb->buf[L] = 0;
	    tline = tb->buf;
	    while (*tline && (resmk != 0x37)) {
		if (*tline == 'M') {
		    if (pcncmp("MemTotal:", tline, 9)) {
			tline += 9;
			MemInfo->Total = pctoint(tline);
			resmk |= 0x01;
		    } else if (pcncmp("MemFree:", tline, 8)) {
			tline += 8;
			MemInfo->Free = pctoint(tline);
			resmk |= 0x02;
		    }
		} else if (*tline == 'C') {
		    if (pcncmp("Cached:", tline, 7)) {
			tline += 7;
			MemInfo->Cached = pctoint(tline);
			resmk |= 0x04;
		    }
		} else if (*tline == 'S') {
		    if (pcncmp("SwapTotal:", tline, 10)) {
			tline += 10;
			MemInfo->SwapTotal = pctoint(tline);
			resmk |= 0x10;
		    } else if (pcncmp("SwapFree:", tline, 9)) {
			tline += 9;
			MemInfo->SwapFree = pctoint(tline);
			resmk |= 0x20;
		    }
		}
		while (*tline && (*tline != '\n')) tline++;
		if (*tline == '\n') tline++;
	    }
// printf("%s(%d): %dus\n", FL, GetTime_us() - t);
#if USE_REOPEN
	    }
#endif
	}
	return resmk;
}

static void TTaskbar_tbMEM_OnPaint(TWins *self) {
	StdBorderPaint(self->WinID, mBorder, 0, 0, 5, self->height);
	StdBorderPaint(self->WinID, mBorder, 5, 0, 5, self->height);
}

static void TTaskbar_SetDefaultTop(TTaskbar *self) {
	SetTop((TWins*)self, appXDat.Root_height - self->height);
}

static void DateInfo_Time(TWins *self, struct tm *locti) {
  char line[80];
  const char *pc = "%a %d-%m-%Y %H:%M:%S";

	if (!locti) return;
	if (wmRoot->DateFormat == 1) {
	    pc = "%a %m-%d-%Y %H:%M:%S";
	} else if (wmRoot->DateFormat == 2) {
	    pc = "%a %Y-%m-%d %H:%M:%S";
	}

	strftime(line, sizeof(line)-1, pc, locti);
	SetCaption(self, line);
}

static void TTaskbar_OnTimer(TgTimer *ti) {
  TTaskbar *tb;
  Window win;
  int cpuh, scpuh, azheight;
  dword Total, Free, Cached;
  char *pc;
  XTimeCoord *tc, *tcn;
  int nevents;
  char line[256];
  TtbMemInfo MemInfo;

	tb = ti->obj;
	tb->TimerZ++;

	azheight = tb->tbCPU->width - 2;
	cpuh = 0;
	scpuh = 0;
// dword t = GetTime_us();
#if USE_REOPEN
	int fd = open(PROCSTAT, O_RDONLY);
	if (fd > 0) {
	    int L = read(fd, tb->buf, sizeof(tb->buf)-1);
	    close(fd);
	    if (L > 10) {
#else
	int L = tb_read(tb, &tb->cpuFD);
	if (L > 10) {
#endif
	    tb->buf[L] = 0;
	    pc = tb->buf;
	    while (*pc && ((pc[0] != 'c') || 
			   (pc[1] != 'p') || 
			   (pc[2] != 'u') ||
			   (pc[3] != ' '))) pc++;
	    if (*pc == 'c') {
		pc += 4;
  dword dw;
  int cput, syst, gest;
		dw = tb->oCPU_USER;
		tb->oCPU_USER = ppctoint((void*)&pc) + ppctoint((void*)&pc);
		cput = tb->oCPU_USER - dw;

//		dw = tb->oCPU_NICE;
//		tb->oCPU_NICE = ppctoint((void*)&pc);
//		cput += tb->oCPU_NICE - dw;

		dw = tb->oCPU_SYS;
		tb->oCPU_SYS = ppctoint((void*)&pc);
		syst = tb->oCPU_SYS - dw;
		cput += syst;

		dw = tb->oCPU_IDLE;
		tb->oCPU_IDLE = ppctoint((void*)&pc);
		gest = cput + (tb->oCPU_IDLE - dw);
		if (gest > 0) {
		    cpuh = (cput * azheight) / gest;
		    scpuh = (syst * azheight) / gest;
		}
	    }
#if USE_REOPEN
	    }
#endif
	}
	if ((tb->cpuh != cpuh) || (tb->scpuh != scpuh)) {
	    tb->cpuh = cpuh;
	    tb->scpuh = scpuh;
	    if (cpuh < azheight) FillRectRGB(tb->tbCPU->WinID, tb->tbCPU->Color, 
					    cpuh + 1, 1, azheight - cpuh, 4);
	    if (cpuh > 0) FillRectRGB(tb->tbCPU->WinID, 0xc000ff, 1, 1, cpuh, 4);
	    if (scpuh > 0) FillRectRGB(tb->tbCPU->WinID, 0xff0000, 1, 1, scpuh, 4);
	}

	if (tb->TimerZ & 0x01) {
	    azheight = tb->tbMEM->height - 2;
	    win = tb->tbMEM->WinID;
	    int MemInfoMk = Get_tbMemInfo(tb, &MemInfo);
	    if ((MemInfoMk & 0x07) == 0x07) {
		Total = MemInfo.Total >> 4; //Div16
		if (Total) {
		  Free = MemInfo.Free >> 4;
		  int memh = ((Total - Free) * azheight) / Total;
		  Cached = MemInfo.Cached >> 4;
		  int cmemh = ((Total - (Free + Cached)) * azheight) / Total;
		  if ((tb->memh != memh) || (tb->cmemh != cmemh)) {
		    tb->memh = memh;
		    tb->cmemh = cmemh;
		    if (memh < azheight) 
		      FillRectRGB(win, tb->tbCPU->Color, 1, 1, 3, azheight - memh);
		    if (memh > 0)
		      FillRectRGB(win, 0x808000, 1, azheight - memh + 1, 3, memh);
		    if (cmemh < azheight) 
		      FillRectRGB(win, 0x4040e0, 1, azheight - cmemh + 1, 3, cmemh);
		  }
		}
	    }
	    if ((MemInfoMk & 0x30) == 0x30) {
		Total = MemInfo.SwapTotal >> 4; //Div16
		if (Total) {
		  Free = MemInfo.SwapFree >> 4;
		  int swph = ((Total - Free) * azheight) / Total;
		  if (tb->swph != swph) {
		    tb->swph = swph;
		    if (swph < azheight) 
		      FillRectRGB(win, tb->tbCPU->Color, 6, 1, 3, azheight - swph);
		    if (swph > 0)
		      FillRectRGB(win, 0x808000, 6, azheight-swph + 1, 3, swph);
		  }
		}
	    }
	} else {

	    time_t vti = time(NULL);
	    struct tm *locti = localtime(&vti);
	    if (locti && (tb->Oldtm_sec != locti->tm_sec)) {
		tb->Oldtm_sec = locti->tm_sec;
#ifdef MEM_CHK
		int lMCt;
		MEM_CHK_Count(&lMCt, NULL);

		snprintf(line, sizeof(line)-1, "%d", lMCt);
#else
		strftime(line, sizeof(line)-1, "%H:%M:%S", locti);
#endif
		SetCaption(tb->tbTime, line);
		if (tb->tbTime->InfoWin->Visible) {
		    DateInfo_Time(tb->tbTime->InfoWin, locti);
		}
	    }
	}
// printf("%s(%d): %dus\n", FL, GetTime_us() - t);


	if (tb->hidde == TASKBAR_HIDDE_AUTO) {
	    if ((tb->hidde_tCount > 0) && !(tb->buStart->BorderFu & mBorderL)) {

		nevents = 0;
		tc = XGetMotionEvents(appXDat.dpy, appXDat.RootW, 
				      MotionEvent.time, CurrentTime, &nevents);
		if (tc) {
		    tcn = tc;
		    if (nevents > 1) tcn += nevents - 1;
		    MotionEvent.time = tcn->time;
		    MotionEvent.x_root = tcn->x;
		    MotionEvent.y_root = tcn->y;
		    XFree(tc);
		}

		if (MotionEvent.y_root >= (appXDat.Root_height  - tb->height)) {
		    TTaskbar_SetDefaultTop(tb);
		    tb->hidde_tCount = HIDDE_AUTO_COUNT;
		} else {
		    tb->hidde_tCount--;
		    if (!tb->hidde_tCount) {
			SetTop((TWins*)tb, appXDat.Root_height - 2);
		    } else if (tb->hidde_tCount < (HIDDE_AUTO_COUNT - 2)) {
			SetTop((TWins*)tb, tb->top + 1);
		    }
		}
	    }
	}
}

static void DateInfo_OnVisible(TWins *self, bool Visible) {
  time_t vti;
  struct tm *locti;

	if (Visible) {
	    XRaiseWindow(appXDat.dpy, self->WinID);
	    vti = time(NULL);
	    locti = localtime(&vti);
	    DateInfo_Time(self, locti);
	} else {
	    Tlgs_Clr((void*)&self->vds);
	}
}

static void TTaskbar_OnResize(TTaskbar *self) {
  int x, h;

	x = self->width - self->tbMEM->width - 2;
	h = self->height - 2; 
	SetRC(self->tbMEM, x, 1, self->tbMEM->width, h);

	x -= self->tbTime->width + 1;
	SetLeft(self->tbCPU, x);
	SetLeft(self->tbTime, x);
	x -= self->tbSysTray->width + 1;
	SetLeft(self->tbSysTray, x);
	x -= self->tbDesktop->width + 1;
	SetLeft(self->tbDesktop, x);

	x = self->width - self->tbTime->InfoWin->width - 1;
	SetPosition(self->tbTime->InfoWin, x, self->top - self->tbTime->InfoWin->height - 1);
}

static void TTaskbar_OnMouseOver(TTaskbar *self, XCrossingEvent *ev, bool Leave) {
	if (self->hidde == TASKBAR_HIDDE_AUTO) {
	    if (!Leave) {
//		self->hidde_tCount = 6;
//	    } else {
		self->hidde_tCount = HIDDE_AUTO_COUNT;
		TTaskbar_SetDefaultTop(self);
	    }
	}
}

static int add_x_InfoCalcX(int width, int x) {
	if (width > 32) return x + 32;
	if (width < 16) return x + 16;
	return x + width + 1;
}

static int TTaskbar_InfoCalcX(TTaskbar *tb) {
  int x;
  TWins *wins;

	x = 0;
	wins = tb->tbSysTray->next_child;
	while (wins) {
	    x = add_x_InfoCalcX(wins->width, x);
	    wins = wins->next;
	}
	return x;
}

static void TTaskbar_InfoSetSize(TTaskbar *self) {
  int w;
  int x;
  TWins *wins;

	w = TTaskbar_InfoCalcX(self) + 2;
	SetWidth(self->tbSysTray, w);
	TTaskbar_OnResize(self);

	x = 1;
	wins = self->tbSysTray->next_child;
	while (wins) {
	    if (wins->wtyp == wtyp_TTaskbarApplet) {
		if (wins->left != x) {
		    wins->left = x;
		    Window pWinID = wins->WinID;
		    if (((TwmWin*)wins)->SaveParentWinID) pWinID = ((TwmWin*)wins)->SaveParentWinID;
		    XMoveWindow(appXDat.dpy, pWinID, wins->left, wins->top);
		}
		x = add_x_InfoCalcX(wins->width, x);
	    }
	    wins = wins->next;
	}
}

static int TTaskbar_AppletWin_EvProc(Tobjmsg *msg) {
  long supplied = 0;
  TWins *self;
  XEvent *ev;
  XSizeHints SizeHints;

	self = msg->wins;
	if (msg->msg == msgXEv) {
	    ev = (XEvent*)msg->p;

	    switch(ev->type) {
	    case UnmapNotify:
		Tfree(self);
		return 1;

	    case ConfigureRequest:
		return 1;

	    case PropertyNotify:
		if (ev->xproperty.atom == XA_WM_NORMAL_HINTS) {
		    if (XGetWMNormalHints(appXDat.dpy, self->WinID, &SizeHints,
					  &supplied) == 0)
		    {
			    SizeHints.flags = 0;
		    }
		    if (SizeHints.flags & (USSize | PSize)) {
		      self->width = SizeHints.width;
		      self->height = SizeHints.height;
		      TTaskbar_InfoSetSize((TTaskbar*)self->owner);
		    }
		}
		return 1;
	    }
	}
	return TWins_EvProc(msg);
}

bool TTaskbar_Chk_AppletWin_EvProc(Tobjmsg *msg, Window win) {
  TWins *wins;

	wins = wmRoot->Taskbar->tbSysTray->next_child;
	while (wins) {
	    if ((wins->WinID == win) && (wins->wtyp == wtyp_TTaskbarApplet)) {
		Send_MSG(msg, wins);
		return True;
	    }
	    wins = wins->next;
	}
	return False;
}

void TTaskbar_SetUp(TwmRoot *wmr, TpcIni *appini) {
  const char *cpc;
  int hidde;
  TTaskbar *tb;

	if (!wmr || !appini) return;
	tb = wmr->Taskbar;
	if (!tb) return;

	cpc = TpcIni_RD_pc(appini, "Taskbar", "Visible", "yes");
	hidde = TASKBAR_HIDDE_NO;
	if (pccasecmp(cpc, "0") || pccasecmp(cpc, "no")) {
	    hidde = TASKBAR_HIDDE_YES;
	} else if (pccasecmp(cpc, "auto")) {
	    hidde = TASKBAR_HIDDE_AUTO;
	}
	if (tb->hidde != hidde) {
	    tb->hidde = hidde;
	    SetVisible((TWins*)tb, hidde != TASKBAR_HIDDE_YES);
	    if (hidde == TASKBAR_HIDDE_NO) {
		TTaskbar_SetDefaultTop(tb);
	    }
	}
}

int Taskbar_GetBottomOffset() {
	if (!wmRoot || !wmRoot->Taskbar) return 0;
//	if (!wmRoot->Taskbar->Visible) return 0;
	if (wmRoot->Taskbar->hidde == TASKBAR_HIDDE_YES)  return 0;
	if (wmRoot->Taskbar->hidde == TASKBAR_HIDDE_AUTO) return 2;
	return wmRoot->Taskbar->height;
}

bool Taskbar_HandleKeyFunk(bool kf_open) {
  TTaskbar *self;

	self = wmRoot->Taskbar;
	if (!self) return false;
	if (kf_open) {
	    if (self->hidde == TASKBAR_HIDDE_YES) {
		if (!self->Visible) {
		    XRaiseWindow(appXDat.dpy, self->WinID);
		    SetVisible((TWins*)self, true);
		    return false;
		}
	    } else if (self->hidde == TASKBAR_HIDDE_AUTO) {
		self->hidde_tCount = HIDDE_AUTO_COUNT;
		TTaskbar_SetDefaultTop(self);
	    }
	} else {
	    if (self->hidde == TASKBAR_HIDDE_YES) {
		SetVisible((TWins*)self, false);
	    }
	}
	return true;
}
/* _____________________ AppletWin ___________________________ */
//bool TwmWin__Destroy(TwmWin *self);

static bool TwmWin_AppletWin__Destroy(TwmWin *self) {
	TwmWin__Destroy(self);
	TTaskbar_InfoSetSize((TTaskbar*)self->owner);
	return False;
}

void TTaskbar_Add_AppletWin(TTaskbar *self, TwmWin *wmw) {
  int x;
  Window pWinID;

	TWins_Add((TWins*)wmw, self->tbSysTray);
	wmw->EvProc = TTaskbar_AppletWin_EvProc;

	wmw->wtyp = wtyp_TTaskbarApplet;
	wmw->owner = (TWins*)self;
	wmw->destroy = (TDESTROY)TwmWin_AppletWin__Destroy;

	wmw->top = 1;
	XSelectInput(appXDat.dpy, wmw->WinID, 
				StructureNotifyMask | ResizeRedirectMask |
				PointerMotionMask | PointerMotionHintMask);

	pWinID = self->tbSysTray->WinID;
	x = 1;
	if (wmw->SaveParentWinID) {
	    pWinID = wmw->SaveParentWinID;
	    x = 0;
	}

	XReparentWindow(appXDat.dpy, wmw->WinID, pWinID, x, x);
	MapWinID(wmw->WinID, 2);
	TWinsRegistration((TWins*)wmw);

	TTaskbar_InfoSetSize(self);
}

#define SYSTEM_TRAY_REQUEST_DOCK  0
//#define SYSTEM_TRAY_BEGIN_MESSAGE   1
//#define SYSTEM_TRAY_CANCEL_MESSAGE  2

void TTaskbar_Handle_SysTray_ev(TTaskbar *self, XClientMessageEvent *ev) {
  Window WinID;
  TwmWin *wmw;
  XWindowAttributes attr;
  XSetWindowAttributes XAttr;

	if (ev->data.l[1] == SYSTEM_TRAY_REQUEST_DOCK) {
	    WinID = ev->data.l[2];
	    if (WinID == None) return;
	    TwmRoot_ClearStartNotify(GetNetWMPid(WinID));

/* Chk Win exist */
	    if (GetTWins(WinID)) return;

	    wmw = TwmWin_Create_base(WinID, &attr);
	    if (!wmw) return;
	    if (wmw->width > 32) wmw->width = 32;
	    if (wmw->width < 4)  wmw->width = 4;
	    wmw->height = 20;
	    wmw->left = 0;
	    wmw->top =  0;

	    Fill0(&XAttr, sizeof(XSetWindowAttributes));
	    XAttr.cursor = appXDat.XCursors[wmw->cursor_nr];
	    XAttr.background_pixel = GetColor_RGB(self->tbSysTray->Color);

	    XAttr.event_mask = (SubstructureNotifyMask | ResizeRedirectMask |
			   PointerMotionMask);
	    wmw->SaveParentWinID = XCreateWindow(appXDat.dpy, self->tbSysTray->WinID, 
			1, 1, wmw->width, wmw->height,
			0, CopyFromParent, InputOutput, CopyFromParent,
			CWBackPixel | CWEventMask | CWCursor, &XAttr);
	    XMapWindow(appXDat.dpy, wmw->SaveParentWinID);
	    UngrabXServer();

	    TTaskbar_Add_AppletWin(self, wmw);
	}; //???
}

#if USE_REOPEN == 0
static void TTaskbar_Close(TTaskbar *self) {
	if (self->memFD > 0) close(self->memFD);
	self->memFD = 0;
	if (self->cpuFD > 0) close(self->cpuFD);
	self->cpuFD = 0;
}

static bool TTaskbar_Ctl(TTaskbar *self, TWinsCtl *wctl) {
	if (wctl->bef == wctlb_AtExit) {
// printf("%s(%d): TTaskbar_Ctl %d %d\n", FL, self->memFD, self->cpuFD);
	    TTaskbar_Close(self);
	    return True;
	}
	return TWins_Ctl((TWins*)self, wctl);
}
#endif
/* _____________________ Create DateInfo ______________________ */
static void TTaskbar_DateInfo_CreateParams(TWins_Create_struct *wcs) {
  TWins *self;

	self = wcs->wins;
	self->Visible = False;
	self->Color = 0xe0e0a0;
	self->BorderFu = mBorder|mBorderL;
	self->CaptionFu = mCaptionPaint;

	vds_SetStr(&self->vds, "Sun 19-09-2010 00:00:00");
	self->width = TextWidthColorString(&self->vds) + 8;
	Font_GetMetrics((TFont*)self->vds.Font, &self->height, NULL);
	if (self->height < 6) self->height = 6;
	self->CaptionOffY = 1;
	Tlgs_Clr((void*)&self->vds);

	wcs->XMask |= CWOverrideRedirect;
	wcs->XAttr.override_redirect = 1;
	wcs->ParentWinID = appXDat.RootW;
}

static TWins *TTaskbar_DateInfo_Create(TWins *Parent) {
  TWins *self;
  TWins_Create_struct wcs;

	TWins_Create_struct_init(&wcs, sizeof(TWins), wtyp_Wins);
	wcs.Parent = Parent;
	wcs.CreateParams = TTaskbar_DateInfo_CreateParams;
	TWins_DefaultConf(&wcs);
	self = wcs.wins;

	TWins_SetParent(self, Parent);
	self->OnVisible = DateInfo_OnVisible;
	return self;
}

/*____________________________ Destroy __________________________________*/
static bool TTaskbar__Destroy(TTaskbar *self) {
	if (appXDat.dpy) XSetSelectionOwner(appXDat.dpy, 
				self->XA_NET_SYSTEM_TRAY_S, None, CurrentTime);
#if USE_REOPEN == 0
	TTaskbar_Close(self);
#endif
	return TWins__Destroy((TWins*)self);
}
/* _____________________ Create _______________________________ */
static void TTaskbar_CreateParams(TWins_Create_struct *wcs) {
  TTaskbar *self = wcs->wins;

	self->destroy = (TDESTROY)TTaskbar__Destroy;
	self->top = appXDat.Root_height - 26;
	self->width = appXDat.Root_width;
	self->height = 26;

	self->OnResize = TTaskbar_OnResize;
#if USE_REOPEN == 0
	self->WinsCtl = TTaskbar_Ctl;
#endif
	self->BorderFu = mBorder;
}

#if USE_REOPEN == 0
static int tb_fd_open(const char *fn) {
  int fd = open(fn, O_RDONLY);
	if (fd > 0) {
	    fcntl(fd, F_SETFD, FD_CLOEXEC);
	} else {
	    printf("%s(%d): Can not open \"%s\"\n", FL, fn);
	}
	return fd;
}
#endif

TTaskbar *TTaskbar_Create(TWins *Parent) {
  TTaskbar *self;
  TWins_Create_struct wcs;
  char *pc;
  TpcIni *ini;
  TDesktopHead ddkh;
  char buf[512];

	TWins_Create_struct_init(&wcs, sizeof(TTaskbar), wtyp_TTaskbar);
	wcs.CreateParams = TTaskbar_CreateParams;
	wcs.WCSfkt = WCSfkt_OverrideRedirect;
	TWins_DefaultConf(&wcs);
	self = wcs.wins;
	Set_Parent((TWins*)self, NULL);
	self->OnMouseOver = TTaskbar_OnMouseOver;

	pc = buf;
	ppcmove(&pc, wmRoot->menuStart->path);
	ppcmove(&pc, ".directory");
	ini = TpcIni_Create(buf);
	ddkh.name = "Start";
	if (dotdesktop_get_head(ini, &ddkh)) {
	    self->buStart_pic = GetDeskIcon(ddkh.ddkt, 16, ddkh.icon, 0, NULL);
	}
	self->buStart = TWins_CreateLabel((TWins*)self, 2, 2, 60, 22, ddkh.name,
					  0, mBorder);
	Tfree(ini);

	self->buStart->OnMouseDown = buStart_OnMouseDown;
	self->buStart->OnPaint = buStart_OnPaint;
	self->buStart->vds.Font = appXDat.Default_bold_Font;
	if (self->buStart_pic) {
	    self->buStart->CaptionFu |= mCaptionLeft;
	    self->buStart->CaptionOffX = self->buStart_pic->width;
	}

	self->tbMEM = TWins_CreateB((TWins*)self, 66, 1, 10, 22, 0);
	self->tbMEM->OnPaint = TTaskbar_tbMEM_OnPaint;

	self->tbCPU = TWins_CreateB((TWins*)self, 66, 18, 60, 6, mBorder | mBorderL);

	self->tbTime = TWins_CreateLabel((TWins*)self, 66, 2, 60, 16, "00:00:00", 
							0, mBorder | mBorderL);
	self->tbTime->vds.Font = appXDat.Default_bold_Font;
	self->tbTime->InfoWin = TTaskbar_DateInfo_Create(self->tbTime);

	self->tbSysTray = TWins_CreateB((TWins*)self, 66, 2, 2, 22, mBorder | mBorderL);

	sprintf(buf, "_NET_SYSTEM_TRAY_S%u", DefaultScreen(appXDat.dpy));
	self->XA_NET_SYSTEM_TRAY_S = XInternAtom(appXDat.dpy, buf, 0);
	self->XA_NET_SYSTEM_TRAY_OPCODE = XInternAtom(appXDat.dpy, 
					"_NET_SYSTEM_TRAY_OPCODE", 0);
	XSetSelectionOwner(appXDat.dpy, self->XA_NET_SYSTEM_TRAY_S,
			    self->tbSysTray->WinID, CurrentTime);

	if (XGetSelectionOwner(appXDat.dpy, self->XA_NET_SYSTEM_TRAY_S) == self->tbSysTray->WinID) {
	    XClientMessageEvent ev;
	    Fill0(&ev, sizeof(ev));
	    ev.type = ClientMessage;
	    ev.window = appXDat.RootW;
	    ev.message_type = XInternAtom(appXDat.dpy, "MANAGER", False);
	    ev.format = 32;
	    ev.data.l[0] = CurrentTime;
	    ev.data.l[1] = self->XA_NET_SYSTEM_TRAY_S;
	    ev.data.l[2] = self->tbSysTray->WinID;
	    XSendEvent(appXDat.dpy, appXDat.RootW, 0, StructureNotifyMask, (XEvent*)&ev);
	}

	self->tbDesktop = TWins_CreateB((TWins*)self, 66, 2, 31, 22, 0);

  int i;
  int x = 0;
  int y = 0;
  int m = mBorder | mBorderL;
	for (i=0;i<4;i++) {
	    self->tbDeskButton[i] = TWins_CreateB(self->tbDesktop, x, y, 15, 11, m);
	    self->tbDeskButton[i]->DoMouseDown = tbDeskButton_DoMouseDown;
	    self->tbDeskButton[i]->tag = i;
	    self->tbDeskButton[i]->owner = (TWins*)self;
	    x += 16;
	    m = mBorder;
	    if (i == 1) {x = 0; y = 11;}
	}

#ifdef USE_DECOR
	self->tbbDecor = TDecor_w3b3_Create(&TDecoBuR3B3_up_Tab, &TDecoBuR3B3_down_Tab);
#endif

#if USE_REOPEN == 0
	self->cpuFD = tb_fd_open(PROCSTAT);
	self->memFD = tb_fd_open(PROCMEMINFO);
#endif
	self->tbTimer = gTimerX_Add(Timer_Enabled, 250, TTaskbar_OnTimer);
	self->tbTimer->obj = self;

	return self;
}

