/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldfile.h"
#include "dlg.h"
#include "tdirlist.h"
#include "FormExecWith.h"

typedef struct {
  char *exec;
} T_node_private;

#define TCSelf TFormExecWith
typedef struct TFormExecWith {
  AlgTForm
  TWins *pnlTop;
    TEdit *edExec;
  TTreeV *tv;
  TWins *pnlBottom;
    TButton *buDefault;
    TButton *buCancle;
    TButton *buOk;
    TButton *buFind;

  TPic pic_ok;
  TPic pic_no;
  char *path_default;
  const char *path_default_Caption;
  char *pc_mime;
  dword FileType;
  char *fname;
  int RetCode;
  bool local_drive;
  bool mime0_Ok;
} TFormExecWith;
#undef TCSelf

static int LastPosX, LastPosY;

static const char zico_ok[] = {
 0x01,0x03,0x06,0x00,0x06,0x00,0x00,0x00,0x1A,0x00,0xC3,0x19,0xCC,0x64,0xCD,0x76,
 0x38,0xFE,0x79,0x0C,0x01,0x86,0x61,0x86,0x45,0x00};

static const char zico_no[] = {
 0x01,0x02,0x06,0x00,0x06,0x00,0x00,0x00,0x16,0x00,0x00,0x40,0x00,0xA0,0xC0,0xEC,
 0x31,0xDE,0x0C,0x55,0x55,0x01};


static bool tvn__Destroy(TTreeVNode *self) {
  T_node_private *pv = self->private;
	xfree(pv->exec);
	xfree(pv);
	return TTreeVNode__Destroy(self);
}

static TTreeVNode *tv_AddNode(TTreeV *self, TTreeVNode *prev_child, 
			      TTreeVNode *prev, const char* s) {
  TTreeVNode *tvn;
	tvn = TTreeV_AddNode(self, prev_child, prev, s);
	tvn->destroy = (TDESTROY)tvn__Destroy;
	tvn->private = xcalloc(sizeof(T_node_private));
	return tvn;
}

static bool chk_fexec(const char *pc) {
	if (!pc) return false;
	while (*pc) {
	    if (*pc == '%') {
		pc++;
		if ((*pc == 'f') || (*pc == 'F') ||
		    (*pc == 'u') || (*pc == 'U')) return true;
	    }
	    pc++;
	}
	return false;
}

#define MimeAkt (1<<4)
#define MimeOk  (1<<5)
#define MimeOkNotDefault (3<<5)

static void add_dir(TTreeV *tv, TTreeVNode *tvn0, TglobPath *gp0, const char *path) { 
  char *pc;
  const char *cpc;
  TglobPath *gp;
  TDirList *dl;
  TDirFileList *dfl;
  TTreeVNode *tvn = NULL;
  TpcIni *ini;
  TDesktopHead ddkh;
  T_node_private *pv;
  TFormExecWith *xf = (TFormExecWith*)tv->FormStruct;

	gp = gp_dup(gp0);
	gp_NameToPath(gp);
	gp->name = pcdup(path);
	gp_NameToPath(gp);
	dl = TDirList_Create_gp(gp);
	if (TDirList_Open(dl, dlobef_level_2)) {
	    TDirList_Open_Wait(dl, dlrunstat_ReScanEnd);
	    TDirFileList_Sort(&dl->dfl, 0);
	    dfl = dl->dfl;
	    while (dfl) {
		if (dfl->st->FileType == stft_Folder) {
		    if (dfl->fn[0] != '.') {
			tvn = tv_AddNode(tv, tvn0, tvn, dfl->fn);
			tvn->tag = 1;
			tvn->pic = get_fileIcon(stft_Folder);
			add_dir(tv, tvn, gp, dfl->fn);
		    }
		} else if (!(dfl->st->st & st_Hidden) &&
			   (dfl->st->FileType == stft_desktop)) {

		    gp->name_const = dfl->fn;
		    pc = gptopc(gp, gptfu_noPasswd|gptfu_noRootArt);
		    gp->name = NULL;
		    ini = TpcIni_Create(pc);
		    xfree(pc);
		    dotdesktop_get_head(ini, &ddkh);

		    if ((ddkh.ddkt == ddkt_Application) &&
			chk_fexec(ddkh.exec)) {
			tvn = tv_AddNode(tv, tvn0, tvn, dfl->fn);
			pv = tvn->private;
			pv->exec = pcdup(ddkh.exec);
			tvn->pic = GetDeskIcon(ddkt_Application, 16, ddkh.icon, 
				    gdifu_noUnknown, NULL);
			if (!tvn->pic) tvn->pic = get_fileIcon(dfl->st->FileType);

			if (xf->pc_mime) {
			    tvn->tag |= MimeAkt;
			    cpc = TpcIni_RD(ini, "Desktop Entry", "MimeType");
			    if (mimetype2app_chk_mime(xf->pc_mime, cpc)) {
				tvn->tag |= MimeOk;
			    } else if (pcncmp(xf->pc_mime, "text/", 5)) {
				if (mimetype2app_chk_mime("text/plain", cpc)) 
						tvn->tag |= MimeOkNotDefault;
			    }
			}
		    }
		    Tfree(ini);
		}
		dfl = dfl->next;
	    }
	}

	Tfree(dl);
	gpFree(gp);
}
/* --------------------------- Ev ------------------------------------ */
static void edExec_OnChange(TEdit *self) {
  TFormExecWith *xf = (TFormExecWith*)self->FormStruct;
	SetEnabled((TWins*)xf->buOk, chk_fexec(GetCaption_pc((TWins*)xf->edExec)));
}

static void tv_OnNodeClick(TTreeV *tv, TTreeVNode *tvn, bool DblClick) {
}

static const char *tv_GetCaption(TFormExecWith *form, TTreeVNode *tvn) {
  const char *result = TTreeVNode_GetCaption(tvn);
	if (pccmp(form->path_default_Caption, result)) result = form->path_default;
	return result;
}

static void tv_OnNodeSelect(TTreeV *tv, TTreeVNode *tvn) {
  TFormExecWith *form = (TFormExecWith*)tv->FormStruct;
  char *fname, *pcn;
  int tag;
  TTreeVNode *tvnx;
  T_node_private *pv;

	if (!tvn || (tvn->tag & 0x01)) goto SelErr;
	pv = tvn->private;
	if (!pv || !pv->exec) goto SelErr;
	tag = tvn->tag;

	fname = pcdup(tv_GetCaption(form, tvn));

	tvnx = tvn;
	while (tvnx) {
	    tvnx = TTreeVNode_get_prev_child(tvnx);
	    if (tvnx) {
		tag = tvnx->tag;
		pcn = pcadd_n(3, tv_GetCaption(form, tvnx), "/", fname);
		xfree(fname);
		fname = pcn;
	    }
	}
	if (!(tag & 0x02)) {
	    pcn = pcadd_n(3, appDat.ALDini_dir, "/Menu/OpenWith/", fname);
	    xfree(fname);
	    fname = pcn;
	}

	xfree(form->fname);
	form->fname = fname;
	SetCaption((TWins*)form->edExec, pv->exec);
	form->FileType = stft_desktop;
	if (pcncmp(fname, form->path_default, pclen(form->path_default))) {
	    SetCaption((TWins*)form->buDefault, "Del Default");
	    SetEnabled((TWins*)form->buDefault, true);
	    form->buDefault->tag = 1;
	} else if (form->mime0_Ok && ((tvn->tag & MimeOkNotDefault) == MimeOk)) {
	    SetCaption((TWins*)form->buDefault, "Add to Default");
	    SetEnabled((TWins*)form->buDefault, true);
	    form->buDefault->tag = 2;
	} else {
SelErr:
	    SetCaption((TWins*)form->buDefault, "Default");
	    SetEnabled((TWins*)form->buDefault, false);
	    form->buDefault->tag = 0;
	}
}

static void buFind_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormExecWith *xf = (TFormExecWith*)self->FormStruct;
  TFile_Dlg *File_Dlg;
  char *fname;
  const char *cpc;
  word ftyp[16];
  Tstat64 *st;

	SetCaption((TWins*)xf->edExec, NULL);
	xf->FileType = 0;

	File_Dlg = Create_FileAlg_Dlg((TWins*)xf);
	ftyp[0] = stft_Exec;
	ftyp[1] = stft_Shell;
	ftyp[2] = stft_desktop;
	ftyp[3] = stft_sh_perl;
	ftyp[4] = 0;
	File_Dlg_AddPattern(File_Dlg, NULL, ftyp);
	File_Dlg->RetName_gp_fu |= gptfu_noRootArt;

	fname = NULL;
	if (File_Dlg_Execute(File_Dlg)) {
	    File_Dlg_GetEdFileName(File_Dlg, &fname);
	}
	Tfree(File_Dlg);

	st = file_stat_fn(fname, fsFollowLink|fsGetFileType|fsGetFileTypeHead);
	if (st) {
	    xf->FileType = st->FileType;
	    xfree(st);
	    cpc = " %u";
	    if (xf->local_drive) cpc = " %f";
	    ppcadd(&fname, cpc);
	    SetCaption((TWins*)xf->edExec, fname);
	}
	xfree(xf->fname);
	xf->fname = fname;
}

static int msg_CopyEv(Txpy_wd wd, int xcyMsg) {
  int bef = xcyMsg & xpyMsg_NrMask;
  int res = 0;
  TFormExecWith *mf;

	mf = XCopy_GetOwner(wd);
	SetEnabled((TWins*)mf->buCancle, false);
	if (xcyMsg & xpyErr_Alg) {
	    
	    switch(bef) {
	    case xpyErr_OverWriteType:
	    case xpyErr_OvwTypeDestDir:
	    case xpyErr_SameFile:
		res = TFormOverWrite_Show((TWins*)mf, wd, xcyMsg, 0x07);
		break;
	    default:
		res = xcy_ErrorMsgShow((TWins*)mf, wd, xcyMsg);
	    }
	} else {

	    switch(bef) {
	    case xpyMsg_ovwDir:
	    case xpyMsg_ovwFile:
		res = TFormOverWrite_Show((TWins*)mf, wd, xcyMsg, 0x1f);
		break;
	    default:
		res = xcy_ErrorMsgShow((TWins*)mf, wd, xcyMsg);
	    }
	}
	SetEnabled((TWins*)mf->buCancle, true);
	return res;
}

static void buDefault_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormExecWith *xf = (TFormExecWith*)self->FormStruct;
  char *pc;
  TTreeVNode *tvn, *tvn0, *xtvn;
  TglobPath *gp_src;
  TglobPath *gp_dest;
  Txpy_wd xpy_wd;

	if (TestWinIsEnabled((TWins*)xf->buOk)) Set_Focus((TWins*)xf->buOk); 
					   else Set_Focus((TWins*)xf->buCancle);
	SetEnabled((TWins*)self, false);
	if (!xf->tv->SelectNode || !xf->fname) return;

	if (self->tag == 1) {
	    if (file_unlink(xf->fname, NULL)) {
		TTreeVNode_Del(xf->tv->SelectNode);
		ppcfree(&xf->fname);
		SetCaption((TWins*)xf->edExec, NULL);
	    }
	} else if (self->tag == 2) {

	    tvn = xf->tv->Nodes;
	    while (tvn && !pccmp(xf->path_default_Caption, TTreeVNode_GetCaption(tvn))) {
		tvn = tvn->next;
	    }
	    if (!tvn) return;
	    tvn0 = tvn;
	    tvn = TTreeVNode_get_next_child(tvn);
	    while (tvn) {
		xtvn = TTreeVNode_get_next(tvn);
		if (tvn->tag & MimeOk) {
		    pc = path_add(xf->path_default, TTreeVNode_GetCaption(tvn));
		    if (file_unlink(pc, NULL)) TTreeVNode_Del(tvn);
		    xfree(pc);
		}
		tvn = xtvn;
	    }

	    gp_src = pctogp(xf->fname);
	    gp_dest = pctogp(xf->path_default);
	    gp_NameToPath_AddName(gp_dest, gp_src->name);

	    xpy_wd = XCopy_Create(xf, msg_CopyEv, gp_src, gp_dest, xcyForce_ovwFile);
	    if (xpy_wd) {
		XCopy_Start(xpy_wd, NULL, 0);
		while (AppHandleMessage() && XCopy_ChkWait(xpy_wd));
		Tfree(xpy_wd);
		tvn = tv_AddNode(xf->tv, tvn0, NULL, TTreeVNode_GetCaption(xf->tv->SelectNode));
		tvn->pic = xf->tv->SelectNode->pic;
		tvn->tag = xf->tv->SelectNode->tag;
		T_node_private *pv = xf->tv->SelectNode->private;
		if (pv && pv->exec) {
		    ((T_node_private*)tvn->private)->exec = pcdup(pv->exec);
		}
		Refresh((TWins*)xf->tv);
	    }

	    gpFree(gp_dest);
	    gpFree(gp_src);
	}
}

static void buCancle_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormExecWith *xf = (TFormExecWith*)self->FormStruct;
	TForm_Close((TForm*)xf);
}

static void buOk_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormExecWith *xf = (TFormExecWith*)self->FormStruct;
	xf->RetCode = 1;
	TForm_Close((TForm*)xf);
}

static void tv_OnKeyUp(TTreeV *self, TKeyEvent *KeyEv) {
  TFormExecWith *xf = (TFormExecWith*)self->FormStruct;

	if ((KeyEv->keysym == XK_Return) && 
	    chk_fexec(GetCaption_pc((TWins*)xf->edExec))) {
	    xf->RetCode = 1;
	    TForm_Close((TForm*)xf);
	}
}


static void pnlTop_OnResize(TWins *self) {
  TFormExecWith *xf = (TFormExecWith*)self->FormStruct;
	
	SetWidth((TWins*)xf->edExec, self->width-2);
}

static void pnlBottom_OnResize(TWins *self) {
  TFormExecWith *xf = (TFormExecWith*)self->FormStruct;
  int x;
	x = self->width - xf->buCancle->width;
	SetLeft((TWins*)xf->buCancle, x);

	x -= xf->buOk->width + 10;
	SetLeft((TWins*)xf->buOk, x);
}

static void tv_OnPaintNode(TTreeV *self, TTreeVNode *tvn, int fu) {
  TFormExecWith *xf = (TFormExecWith*)self->FormStruct;

	if ((fu & tvnfu_CalcSizeOnly) || !(tvn->tag & MimeAkt)) return;

  TPic *pic = &xf->pic_no;
	if (tvn->tag & MimeOk) pic = &xf->pic_ok;
	DrawPic(self->WinID, pic, tvn->left-4, tvn->top+11);
}

/* ___________________________________________________________________ */
static const char *mime_ext[5] = {
  "application/x-",
  "image/x-",
  "application/",
  "image/",
  NULL
};

char *inFormExecWith_chk_app_mime(const char *fname, TMimeDat *md) {
  int i;
  const char *cpc;
  char *res = NULL;
  TMimeDat tmd;
  char buf[32];

	cpc = GetFileExt_pc(fname);
	if (!cpc) return NULL;
	if (!md) md = &tmd;

	pcxmove(buf, cpc, 31);
	pctoLcaseASCII(buf);

	i = 0;
	while (mime_ext[i]) {
	    res = pcadd(mime_ext[i], buf);
	    if (mimetype2app_getdat(res, md)) return res;
	    ppcfree(&res);
	    i++;
	}
	return NULL;
}

/* ____________________________ Form Ev ______________________________ */
static void xf_OnResize(TFormExecWith *self) {
	if (!self->ResizeStartMk) return;
  TpcIni *ini = TpcIni_Create(appDat.inifile_app);
	TpcIni_WR_int(ini, "FormExecWith", "Width", self->width);
	TpcIni_WR_int(ini, "FormExecWith", "Height", self->height);
	TpcIni_SaveAndFree(ini);
}

/* ____________________________ Destroy ______________________________ */
static bool TFormExecWith__Destroy(TFormExecWith *self) {
	xfree(self->path_default);
	xfree(self->pc_mime);
	xfree(self->fname);
	ClrPic(&self->pic_ok);
	ClrPic(&self->pic_no);
	return TForm__Destroy((TForm*)self);
}

/* ____________________________ Create _______________________________ */
bool TFormExecWith_Show(TWins *parent, const char *fname, Tstat64 *st, TMimeDat *md) {
  char *pc;
  const char *cpc;
  int lsHOME;
  TFormExecWith *self;
  TpcIni *ini;
  char *token, *ddPath;

	if (!md) return false;
	Fill0(md, sizeof(TMimeDat));

	self =  TForm_CreatParamAlg(parent, sizeof(TFormExecWith));
	self->destroy = (TDESTROY)TFormExecWith__Destroy;
	self->FormZStyle = fszTransient;
	if (LastPosX || LastPosY) {
	    self->left = LastPosX;
	    self->top = LastPosY;
	    self->FormPosition = poDesigned;
	} else {
	    self->FormPosition = poCenter;
	}
	ini = TpcIni_Create(appDat.inifile_app);
	self->width =  TpcIni_RD_int(ini, "FormExecWith", "Width", 300);
	self->height = TpcIni_RD_int(ini, "FormExecWith", "Height", 400);
	Tfree(ini);
	self->min_width = 260;
	self->min_height = 200;
	self->OnResize = xf_OnResize;

	pc = pcadd_n(3, "exec file: \"", GetFileName_pc(fname), "\" with");
	SetCaption((TWins*)self, pc);
	xfree(pc);


	self->path_default_Caption = "Default exec";
	self->path_default = pcadd(appDat.ALDini_dir, "/share/applications");
#ifdef _OS_WIN32
	slash_korr(self->path_default, true);
#endif
	self->pnlTop = TWins_CreateBA((TWins*)self, 0, 0, 200, 38, mBorder | mBorderL, alTop);
	self->pnlTop->OnResize = pnlTop_OnResize;
	self->pnlTop->CaptionFu = mCaptionPaint | mCaptionLeft | mCaptionTop;
	self->pnlTop->CaptionOffX = 2;
	self->pnlTop->CaptionOffY = 1;

	self->tv = TTreeV_CreateA((TWins*)self, 0, 0, 50, 50, alClient);
	self->tv->OnNodeClick = tv_OnNodeClick;
	self->tv->OnNodeSelect = tv_OnNodeSelect;
	self->tv->OnPaintNode = tv_OnPaintNode;
	self->tv->OnKeyUp = tv_OnKeyUp;
	self->pnlBottom = TWins_CreateBA((TWins*)self, 0, 0, 200, 48, 0, alBottom);
	self->pnlBottom->OnResize = pnlBottom_OnResize;

//pnlTop
	self->edExec = TEdit_CreateS(self->pnlTop, 1, 18, 100, 19, NULL);
	self->edExec->OnChange = edExec_OnChange;
//pnlBottom
	self->buDefault = TButton_CreateS(self->pnlBottom, 1, 1, 128, 22, "Default");
	SetEnabled((TWins*)self->buDefault, false);
	self->buDefault->OnClick = buDefault_OnClick;

	self->buCancle = TButton_CreateS(self->pnlBottom, 1, 25, 64, 22, "CANCEL");
	self->buCancle->OnClick = buCancle_OnClick;
	self->buOk = TButton_CreateS(self->pnlBottom, 1, 25, 64, 22, "Ok");
	self->buOk->OnClick = buOk_OnClick;
	SetEnabled((TWins*)self->buOk, false);
	self->buFind = TButton_CreateS(self->pnlBottom, 1, 25, 64, 22, "Find");
	self->buFind->OnClick = buFind_OnClick;


// find Mime
	self->pc_mime = stat_get_mimetype(st, fname);
	self->mime0_Ok = self->pc_mime != NULL;
	if (!self->pc_mime) self->pc_mime = inFormExecWith_chk_app_mime(fname, NULL);
#if 1
#ifndef _OS_WIN32
	if (!self->pc_mime) {
	    TPathInfo pathi;
	    if (get_PathInfo(fname, &pathi) && (pathi.fu & pathi_Local)) {
		self->local_drive = true; //test_local_drive(fname);
		self->pc_mime = libmagic_get_mime(pathi.path_anf);
	    }
	}
#endif
#endif
//  printf("%s(%d): 0x%08x %s -> %s\n", FL, (int)st, fname, self->pc_mime);

#if 0
  struct TMimeObj *mo, *mon;
  struct TMimeDat_app *mda;
	mo = TMimeEnum(NULL);
	while (mo) {
// printf("%s(%d): mo  0x%04x %s\n", FL, (int)mo, TMimeObjPath(mo));
	    mon = mo;
	    while (mon) {
		char *path = TMimeObjPath(mon);
 printf("%s(%d): mon 0x%04x %s\n", FL, (int)mon, path);
		xfree(path);
		mda = TMimeObjAppEnum(mon, NULL);
		while (mda) {
 printf("%s(%d): mda 0x%04x %s\n", FL, (int)mda, TMimeDatAppName(mda));


		    mda = TMimeObjAppEnum(mon, mda);
		}
		mon = TMimeObjEnum(mon);
	    }
	    mo = TMimeEnum(mo);
	}
#endif

  TglobPath *gp;
  TTreeVNode *tvn0;
  TTreeVNode *tvn = NULL;

	pc = pcadd(appDat.ALDini_dir, "/Menu/OpenWith/");
	gp = pctogp(pc);
	xfree(pc);
	add_dir(self->tv, NULL, gp, NULL);
	gpFree(gp);

	ini = TpcIni_Create(appDat.inifile_global);
#ifdef _OS_WIN32
	ddPath = pcadd_n(6, "\"", self->path_default, "\" \"", 
			appDat.ALDini_dir, "/applications\" ", 
			TpcIni_RD_pc(ini, "App", "dotdesktop", NULL));
#else
	ddPath = pcadd_n(6, self->path_default, " ",
			TpcIni_RD_pc(ini, "App", "dotdesktop", "/usr/share/applications"),
			" ", ALDccpPREFIX, "/share/applications");
#endif
	Tfree(ini);
	lsHOME = pclen(appDat.sHOME);

	tvn = self->tv->Nodes;
	while (tvn && tvn->next) tvn = tvn->next;
	tvn0 = tvn;

  const char *indata = ddPath;
	while (indata) {
	    indata = GetToken(indata, &token);
	    if (*token == '~') {
		pc = pcadd(appDat.sHOME, token+1);
		xfree(token);
		token = pc;
	    }

#ifdef _OS_WIN32
	    slash_korr(token, true);
#else
	    slash_korr(token, false);
#endif
	    char *add_token = token;
	    if (pcncmp(appDat.sHOME, token, lsHOME)) {
		add_token = pcadd("~", token + lsHOME);
	    }

	    TTreeVNode *xtvn = tvn0;
	    bool Ok = true;
	    while (xtvn && Ok) {
		Ok = !pccmp(add_token, TTreeVNode_GetCaption(xtvn));
		xtvn = xtvn->next;
	    }

	    if (Ok && TestIsPath(token)) {
		const char *cap = add_token;
		if (pccmp(self->path_default, token)) cap = self->path_default_Caption;

		tvn = TTreeV_AddNode((TTreeV*)self->tv, NULL, tvn, cap);
		tvn->tag = 3;
		tvn->pic = get_fileIcon(stft_Folder);
		gp = pctogp(token);
		add_dir(self->tv, tvn, gp, NULL);
		gpFree(gp);
	    }
	    if (add_token != token) xfree(add_token);
	    xfree(token);
	}

	xfree(ddPath);

	CreateAHicoPic(zico_ok, &self->pic_ok);
	CreateAHicoPic(zico_no, &self->pic_no);

	if (mimetype2app_getdat(self->pc_mime, md)) {
	    if (chk_fexec(md->exec)) {
		SetCaption((TWins*)self->edExec, md->exec);
		self->FileType = stft_desktop;
	    }
	}
	md->exec = NULL;

	pc = pcadd("Mime: ", self->pc_mime);
	SetCaption(self->pnlTop, pc);
	xfree(pc);

	Set_Focus((TWins*)self->buCancle);
	TForm_Show((TForm*)self);

	while (TestWinIsVisible((TWins*)self) && AppHandleMessage());

	LastPosX = self->left;
	LastPosY = self->top;

	if (self->RetCode == 1) {
	    cpc = GetCaption_pc((TWins*)self->edExec);
	    if (chk_fexec(cpc)) {
		md->exec = pcdup(cpc);
	    }
	}

	Tfree(self);
	return md->exec != NULL;
}
