/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldfile.h"
#include "cicons.h"

static int LastPosX, LastPosY;

#define TCSelf TFormOverWrite
typedef struct TFormOverWrite {
  AlgTForm

  TWins *pnlTop;
  TWins *pnlMiddle;
  TWins *pnlBottom;

  TEdit *edFrom;
  TEdit *edTo;

  TEdit *edFile;

  TWins *lblSizeS;
  TWins *lblTypeS;
  TWins *lblDateS;
  TWins *lblSizeD;
  TWins *lblTypeD;
  TWins *lblDateD;

  TButton *buRename;
  TButton *buSkip;
  TButton *buSkipAll;
  TButton *buOverWrite;
  TButton *buOverWriteAll;
  TButton *buCancle;

  TglobPath *gp_src;
  TglobPath *gp_dest;

  Txpy_wd wd;
  int xcyMsg;

  int RetCode;
} TFormOverWrite;
#undef TCSelf

/* _____________________________ Ev __________________________________ */
static void buRename_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormOverWrite *mf = GetParentForm((TWins*)self);
	XCopy_ChangeDestName(mf->wd, GetCaption_pc((TWins*)mf->edFile));
	mf->RetCode = xcyRetEv_REPEAT;
	TForm_Close((TForm*)mf);
}

static void buSkip_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormOverWrite *mf = GetParentForm((TWins*)self);
	mf->RetCode = xcyRetEv_SKIP;
	TForm_Close((TForm*)mf);
}

static void buSkipAll_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormOverWrite *mf = GetParentForm((TWins*)self);
	XCopy_MsgSetSkip(mf->wd, mf->xcyMsg, 0, True);
	mf->RetCode = xcyRetEv_SKIP | xcyRetEv_ALL;
	TForm_Close((TForm*)mf);
}

static void set_ov_skip(TFormOverWrite *mf, bool all) {
  int bef = mf->xcyMsg & xpyMsg_NrMask;
  int skip = 0;
	if (!(mf->xcyMsg & xpyErr_Alg)) {
	    if (bef == xpyMsg_ovwDir) {
		skip = xcyForce_ovwDir;
	    } else if (bef == xpyMsg_ovwFile) {
		skip = xcyForce_ovwFile;
	    }
	}
	if (skip) {
	    XCopy_MsgSetSkip(mf->wd, mf->xcyMsg, skip, all);
	    mf->RetCode = xcyRetEv_REPEAT;
	}
}

static void buOverWrite_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormOverWrite *mf = GetParentForm((TWins*)self);
	set_ov_skip(mf, False);
	TForm_Close((TForm*)mf);
}

static void buOverWriteAll_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormOverWrite *mf = GetParentForm((TWins*)self);
	set_ov_skip(mf, True);
	TForm_Close((TForm*)mf);
}

static void buCancle_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormOverWrite *mf = GetParentForm((TWins*)self);
	TForm_Close((TForm*)mf);
}

static void edFile_OnChange(TEdit *self) {
  TFormOverWrite *mf = (TFormOverWrite*)GetParentForm((TWins*)self);
  const char *pc;

	pc = GetCaption_pc((TWins*)mf->edFile);
	SetEnabled((TWins*)mf->buRename, !pccmp(pc, mf->gp_dest->name));
}

/* ____________________________ Resize _______________________________ */
static void pnlTop_OnResize(TWins *self) {
  TFormOverWrite *mf = (TFormOverWrite*)GetParentForm(self);
  int w;
	w = self->width - 43;
	if (w < 40) w = 40;
	SetWidth((TWins*)mf->edFrom, w);
	SetWidth((TWins*)mf->edTo, w);
}

static void pnlMiddle_OnResize(TWins *self) {
  TFormOverWrite *mf = (TFormOverWrite*)GetParentForm(self);
  int w;
	w = self->width - 43;
	if (w < 40) w = 40;
	SetWidth((TWins*)mf->edFile, w);
}


static void pnlBottom_OnResize(TWins *self) {
  TFormOverWrite *cf = (TFormOverWrite*)GetParentForm(self);
  int wg;
  float wo, xm;

	wg = cf->buRename->width + cf->buSkip->width + cf->buSkipAll->width + 
	     cf->buOverWrite->width + cf->buOverWriteAll->width + cf->buCancle->width; 

	wo = (self->width + 1 - wg);
	wo /= 7;

	xm = wo;
	SetLeft((TWins*)cf->buRename, xm);
	xm += wo + cf->buRename->width;
	SetLeft((TWins*)cf->buSkip, xm);
	xm += wo + cf->buSkip->width;
	SetLeft((TWins*)cf->buSkipAll, xm);
	xm += wo + cf->buSkipAll->width;
	SetLeft((TWins*)cf->buOverWrite, xm);
	xm += wo + cf->buOverWrite->width;
	SetLeft((TWins*)cf->buOverWriteAll, xm);
	xm += wo + cf->buOverWriteAll->width;
	SetLeft((TWins*)cf->buCancle, xm);
}


/* ____________________________ Destroy ______________________________ */
static bool TFormOverWrite__Destroy(TFormOverWrite *self) {
	gpFree(self->gp_src);
	gpFree(self->gp_dest);
	return TWins__Destroy((TWins*)self);
}

/* static void mf_OnCanClose(TForm *self, bool *CanClose) {
	*CanClose = FormCloseAndFree;
} */

/* ____________________________ Create _______________________________ */
int TFormOverWrite_Show(TWins *Parent, Txpy_wd wd, int xcyMsg, int fu) {
  TFormOverWrite *self;
  char *pc;
  const char *cpc;
  Tstat64 *st;
  char buf[80];

	if (!wd) return 0;

	self = TForm_CreatParamAlg(Parent, sizeof(TFormOverWrite));
	self->destroy = (TDESTROY)TFormOverWrite__Destroy;
	self->FormZStyle = fszTransient;

	if (LastPosX || LastPosY) {
	    self->left = LastPosX;
	    self->top = LastPosY;
	    self->FormPosition = poDesigned;
	} else {
	    self->FormPosition = poCenter;
	}
	self->width = 440;
	self->height = 130;
	self->min_width = 410;
	self->min_height = 130;

	if (xcyMsg & xpyErr_Alg) {
	    cpc = "ERROR";
	    self->Icon = GetDeskIcon_z(ddkt_Actions, 16, "error", 
					gdifu_noZoom, zico16error);
	} else {
	    cpc = "WARNING";
	    self->Icon = GetDeskIcon_z(ddkt_Actions, 16, "warning", 
					gdifu_noZoom, zico16warning);
	}
	pc = pcadd_n(3, cpc, ": Copy  ", XCopy_GetMsg_pc(wd, xcyMsg));
	SetCaption((TWins*)self, pc);
	xfree(pc);


	self->wd = wd;
	self->xcyMsg = xcyMsg;

	self->gp_src = XCopy_GetSrc_gp(wd);
	self->gp_dest = XCopy_GetDest_gp(wd);

	self->pnlTop = TWins_CreateBA((TWins*)self, 0, 0, 200, 41, mBorder, alTop);
	self->pnlTop->OnResize = pnlTop_OnResize;
	self->pnlMiddle = TWins_CreateBA((TWins*)self, 0, 0, 200, 21, mBorder, alClient);
	self->pnlMiddle->OnResize = pnlMiddle_OnResize;
	self->pnlBottom = TWins_CreateBA((TWins*)self, 0, 0, 200, 28, mBorder, alBottom);
	self->pnlBottom->OnResize = pnlBottom_OnResize;

	TWins_CreateLabel(self->pnlTop, 2,  3, 28, 16, "from:", mCaptionLeft, 0);
	TWins_CreateLabel(self->pnlTop, 2, 22, 28, 16, "to :", mCaptionLeft, 0);

	pc = get_DisplayPathStr(NULL, self->gp_src);
	self->edFrom = TEdit_CreateS(self->pnlTop, 40, 2, 45, 18, pc);
	xfree(pc);
	self->edFrom->KeyFu |= mNoKeyFocus;
	self->edFrom->ReadOnly = True;
	SetColor((TWins*)self->edFrom, 0xe0e0e0);
	TWins_SetUTF8((TWins*)self->edFrom, appXDat.FileSystemUTF8on);

	pc = get_DisplayPathStr(NULL, self->gp_dest);
	self->edTo = TEdit_CreateS(self->pnlTop, 40, 21, 45, 18, pc);
	xfree(pc);
	self->edTo->KeyFu |= mNoKeyFocus;
	self->edTo->ReadOnly = True;
	SetColor((TWins*)self->edTo, 0xe0e0e0);
	TWins_SetUTF8((TWins*)self->edTo, appXDat.FileSystemUTF8on);


	TWins_CreateLabel(self->pnlMiddle, 2,  3, 28, 16, "file:", mCaptionLeft, 0);

	self->edFile = TEdit_CreateS(self->pnlMiddle, 40, 2, 45, 18, self->gp_dest->name);
	if (!(fu & 0x01)) {
	    self->edFile->KeyFu |= mNoKeyFocus;
	    self->edFile->ReadOnly = True;
	    SetColor((TWins*)self->edFile, 0xe0e0e0);
	}
	TWins_SetUTF8((TWins*)self->edFile, appXDat.FileSystemUTF8on);
	self->edFile->OnChange = edFile_OnChange;


	TWins_CreateLabel(self->pnlMiddle, 2, 23, 28, 16, "from:", mCaptionLeft, 0);
	TWins_CreateLabel(self->pnlMiddle, 2, 42, 28, 16, "to :", mCaptionLeft, 0);

	st = XCopy_GetSrcStat(wd);
	if (st) {
	    pc = buf;
	    file_str_size_ppc(&pc, st->size, 0);
	    self->lblSizeS = TWins_CreateLabel(self->pnlMiddle, 40, 22, 60, 18, buf, mCaptionRigth, mBorder|mBorderL);
	    pc = buf;
	    int64toppc(&pc, st->size);
	    ppcmove(&pc, " B");
	    self->lblTypeS = TWins_CreateLabel(self->pnlMiddle,102, 22, 80, 18, buf, mCaptionRigth,  mBorder|mBorderL);
	    file_str_time_HMS_pc(buf, st->mtime);
	    self->lblDateS = TWins_CreateLabel(self->pnlMiddle,184, 22, 146,18, buf, 0,  mBorder|mBorderL);
	    xfree(st);
	}
	st = XCopy_GetDestStat(wd);
	if (st) {
	    pc = buf;
	    file_str_size_ppc(&pc, st->size, 0);
	    self->lblSizeD = TWins_CreateLabel(self->pnlMiddle, 40, 41, 60, 18, buf, mCaptionRigth, mBorder|mBorderL);
	    pc = buf;
	    int64toppc(&pc, st->size);
	    ppcmove(&pc, " B");
	    self->lblTypeD = TWins_CreateLabel(self->pnlMiddle,102, 41, 80, 18, buf, mCaptionRigth,  mBorder|mBorderL);
	    file_str_time_HMS_pc(buf, st->mtime);
	    self->lblDateD = TWins_CreateLabel(self->pnlMiddle,184, 41, 146,18, buf, 0,  mBorder|mBorderL);
	    xfree(st);
	}

	self->buRename = TButton_CreateS(self->pnlBottom, 2, 2, 64, 24, "Rename");
	self->buSkip = TButton_CreateS(self->pnlBottom, 2, 2, 64, 24, "Skip");
	self->buSkipAll = TButton_CreateS(self->pnlBottom, 2, 2, 64, 24, "Skip All");
	self->buOverWrite = TButton_CreateS(self->pnlBottom, 2, 2, 64, 24, "Overwrite");
	self->buOverWriteAll = TButton_CreateS(self->pnlBottom, 2, 2, 80, 24, "Overwrite All");
	self->buCancle = TButton_CreateS(self->pnlBottom, 2, 2, 64, 24, "Cancle");
	SetEnabled((TWins*)self->buRename, False);
	SetEnabled((TWins*)self->buSkip, (fu & 0x02) != 0);
	SetEnabled((TWins*)self->buSkipAll, (fu & 0x04) != 0);
	SetEnabled((TWins*)self->buOverWrite, (fu & 0x08) != 0);
	SetEnabled((TWins*)self->buOverWriteAll, (fu & 0x10) != 0);

	self->buRename->OnClick    = buRename_OnClick;
	self->buSkip->OnClick      = buSkip_OnClick;
	self->buSkipAll->OnClick   = buSkipAll_OnClick;
	self->buOverWrite->OnClick = buOverWrite_OnClick;
	self->buOverWriteAll->OnClick = buOverWriteAll_OnClick;
	self->buCancle->OnClick    = buCancle_OnClick;

	Set_Focus((TWins*)self->buCancle);

	TForm_Show((TForm*)self);

	while (TestWinIsVisible((TWins*)self) && AppHandleMessage());

	LastPosX = self->left;
	LastPosY = self->top;

	int RetCode = self->RetCode;
	Tfree(self);
	return RetCode;
}
