/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldfile.h"

#define TCSelf TFormPropNet
typedef struct TFormPropNet {
  AlgTForm
  TEdit *edTvNode;
    TPic *default_pic;

  TWins *lblFileName;
  TEdit *edFileName;

  TWins *lblHost;
  TEdit *edHostName;

  TWins *lblUser;
  TEdit *edUser;

  TWins *lblPasswd;
  TEdit *edPasswd;

  TWins *lblAll;


  TWins *pnlBottom;
  TButton *buSave;
  TButton *buCancel;
  
  TglobPath *gp, *new_gp;
  char *BasePath, *fn, *NodeName, *NodeIconName;
  char *NodeAddPath, *SyncName;
  bool NewIni;
  bool NewPasswd;
  bool SkipChange;
} TFormPropNet;
#undef TCSelf

static void chk_buSave_en(TFormPropNet *mf) {
  bool mk;
	mk = mf->NewIni || mf->NewPasswd ||
	     !pccmp(mf->NodeName, GetCaption_pc((TWins*)mf->edTvNode)) ||
	     !pccmp(mf->gp->URL,  mf->new_gp->URL) || 
	     !pccmp(mf->gp->user, mf->new_gp->user);

	if (mk && !pccmp(mf->fn, GetCaption_pc((TWins*)mf->edFileName))) {
	    char *fn = pcadd_n(3, mf->BasePath, 
				GetCaption_pc((TWins*)mf->edFileName), ".aldfile");
	    mk = !file_chk_exist(fn, findALL);

	    xfree(fn);
	}

	SetEnabled((TWins*)mf->buSave, mk);
}

static void refresh_lblAll(TFormPropNet *mf) {
  char *pc;
  bool Sync;

	pc = gptopc(mf->new_gp, 0);
	SetCaption(mf->lblAll, pc);
	xfree(pc);
	Sync = pccmp(mf->SyncName, GetCaption_pc((TWins*)mf->edTvNode));

	if (mf->new_gp->user) {
	    ppcchange_add_n(&mf->SyncName, 3, mf->new_gp->user, "@", mf->new_gp->URL);
	    pc = pcadd(mf->NodeAddPath, mf->SyncName);
	    SetCaption((TWins*)mf->edFileName, pc);
	    xfree(pc);
	} else {
	    ppcchange(&mf->SyncName, mf->new_gp->URL);
	    pc = pcadd(mf->NodeAddPath, mf->new_gp->URL);
	    SetCaption((TWins*)mf->edFileName, pc);
	    xfree(pc);
	}
	if (Sync) SetCaption((TWins*)mf->edTvNode, mf->SyncName);
	chk_buSave_en(mf);
}

static void edTvNode_OnChange(TEdit *self) {
	chk_buSave_en(GetParentForm((TWins*)self));
}

static void edTvNode_OnFocus(TEdit *self, int Change) {
  TFormPropNet *mf = GetParentForm((TWins*)self);

	if (((Change & (WinsFocusIN|WinsFocusChange)) == WinsFocusChange) &&
	    !pclen(GetCaption_pc((TWins*)self))) {

	    if (mf->new_gp->user) {
		char *pc = pcadd_n(3, mf->new_gp->user, "@", mf->new_gp->URL);
		SetCaption((TWins*)self, pc);
		xfree(pc);
	    } else {
		SetCaption((TWins*)self, mf->new_gp->URL);
	    }
	}
}

static void edTvNode_OnPaint(TEdit *self) {
  TFormPropNet *mf = GetParentForm((TWins*)self);
	if (mf->default_pic) DrawPic(self->WinID, mf->default_pic, 3, 3);
}

static void edHostName_OnChange(TEdit *self) {
  TFormPropNet *mf = GetParentForm((TWins*)self);
	if (mf->SkipChange) return;
	mf->SkipChange = true;
//	ppcchange_trim(&mf->new_gp->URL, GetCaption_pc((TWins*)self));
	xfree(mf->new_gp->URL);
	mf->new_gp->URL = pctrim(GetCaption_pc((TWins*)self));
	if (!mf->new_gp->URL) mf->new_gp->URL = pcdup(mf->gp->URL);
	refresh_lblAll(mf);
	mf->SkipChange = false;
}

static void edUser_OnChange(TEdit *self) {
  TFormPropNet *mf = GetParentForm((TWins*)self);
	if (mf->SkipChange) return;
	mf->SkipChange = true;
//	ppcchange_trim(&mf->new_gp->user, GetCaption_pc((TWins*)self));
	xfree(mf->new_gp->user);
	mf->new_gp->user = pctrim(GetCaption_pc((TWins*)self));
	refresh_lblAll(mf);
	mf->SkipChange = false;
}

static void edPasswd_OnChange(TEdit *self) {
  TFormPropNet *mf = GetParentForm((TWins*)self);

//	ppcchange_trim(&mf->new_gp->passwd, GetCaption_pc((TWins*)self));
	xfree(mf->new_gp->passwd);
	mf->new_gp->passwd = pctrim(GetCaption_pc((TWins*)self));
	mf->NewPasswd = true;
	refresh_lblAll(mf);
}


static void buSave_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormPropNet *mf = GetParentForm((TWins*)self);
  char *fn, *cpasswd;
  TpcIni *ini;

	NetPasswdSave(mf->new_gp);
	cpasswd = CryptMini16c_dec(mf->new_gp->passwd);

	fn = pcadd_n(3, mf->BasePath, 
			GetCaption_pc((TWins*)mf->edFileName), ".aldfile");

	ini = TpcIni_Create(fn);
	TpcIni_WR(ini, "def", "Caption", GetCaption_pc((TWins*)mf->edTvNode));
	TpcIni_WR(ini, "def", "url", mf->new_gp->URL);
	TpcIni_WR(ini, "def", "user", mf->new_gp->user);
	TpcIni_WR(ini, "def", "passwd", cpasswd);
	TpcIni_WR(ini, "def", "icon", mf->NodeIconName);

	TpcIni_SaveAndFree(ini);
	file_chmod(fn, NULL, 0600);

	xfree(fn);
	xfree(cpasswd);
	TForm_Close((TForm*)mf);
}

static void buCancel_OnClick(TButton *self, long KeyState, bool DblClick) {
	TForm_Close(GetParentForm((TWins*)self));
}

static void pnlBottom_OnResize(TWins *self) {
  TFormPropNet *mf = GetParentForm(self);
  int x = self->width;
	x -= mf->buCancel->width;
	SetLeft((TWins*)mf->buCancel, x);
	x -= mf->buSave->width + 4;
	SetLeft((TWins*)mf->buSave, x);
}

static void form_OnResize(TFormPropNet *self) {
  int w;
	SetWidth((TWins*)self->edTvNode, self->width - 2);
	SetWidth((TWins*)self->lblAll, self->width - 2);

	w = self->width - self->edFileName->left - 1;
	SetWidth((TWins*)self->edFileName, w);

	w = self->width - self->edHostName->left - 1;
	SetWidth((TWins*)self->edHostName, w);
}

/*_____________________________ Destroy _________________________________*/
static bool TFormPropNet__Destroy(TFormPropNet *self) {

	xfree(self->BasePath);
	xfree(self->NodeName);
	xfree(self->NodeIconName);
	xfree(self->NodeAddPath);
	xfree(self->SyncName);

	xfree(self->fn);
	gp_free(self->gp);
	gp_free(self->new_gp);

	return TForm__Destroy((TForm*)self);
}

static void mf_OnCanClose(TFormPropNet *self, int *CanClose) {
	*CanClose = FormCloseAndFree;
}

/*_____________________________ Create __________________________________*/
void TFormPropNet_Show(TMainForm *Parent, Txdfl *xdfl) {
  TFormPropNet *self;
  TTreeVNode *tvn;
  int y;
  dword pwmod;
  char *pc, *NodeAddPath;
  const char *cpc;
  TpcIni *ini;
  bool isFtp, MkNewNode;

	if (!xdfl || !xdfl->istv) return;
	tvn = TTreeVFile_GetSelectNode(xdfl->o.tv);
	if (!tvn) return;
	NodeAddPath = NULL;
	isFtp = MkNewNode = false;
	if (xdfl->gp) {
	    if ((tvn->art & tfvfart_MASK) != tfvfart_ROOT) return;
	    isFtp = ftp_chk_RootArt_URL(xdfl->gp);
	} else {

	    while (tvn) {
		if (tvn == Parent->tvftp) {
		    isFtp = true;
		    break;
		}
		ppcadd_n(&NodeAddPath, 2, tvn->s, "/");
		tvn = TTreeVNode_get_prev_child(tvn);
	    }
	    MkNewNode = true;
	}

	if (!isFtp) {
	    xfree(NodeAddPath);
	    return;
	}

	self = TForm_CreatParamAlg((TWins*)Parent, sizeof(TFormPropNet));
	self->destroy = (TDESTROY)TFormPropNet__Destroy;

	self->FormZStyle = fszTransient;
	self->FormPosition = poCenter;
	self->width = 360;
	self->height = 164;
	self->min_width = 280;
	self->min_height = 164;
	self->OnCanClose = mf_OnCanClose;
	self->OnResize = form_OnResize;

	pc = get_DisplayFileStr(NULL, xdfl->gp);
	ppcchange_add_n(&pc, 2, "Properties ", pc);
	SetCaption((TWins*)self, pc);
	xfree(pc);

	self->BasePath = pcadd(appDat.ALDini_dir, "/Net/ftp/");  //isFtp
	self->NodeIconName = pcdup("ftp");  //isFtp

	if (!MkNewNode) {
	    if (tvn->tagp) {
		self->fn = path_abs2rel(tvn->tagp, self->BasePath);
		ChangeFileExt_ppc(&self->fn, NULL);
	    } else {
		self->fn = pcdup(tvn->s);
	    }
	    self->NodeName = pcdup(tvn->s);

	    self->gp = gp_dup(xdfl->gp);
	    NetPasswdSetup_gp(self->gp);
	    self->new_gp = gp_dup(self->gp);

	    pc = pcadd_n(3, self->BasePath, self->fn, ".aldfile");
	    ini = TpcIni_Create(pc);
	    xfree(pc);

	    self->NewIni = !ini->Exist;

	    if (self->NewIni) {
		self->NewPasswd = self->gp->passwd != NULL;
	    } else {
		pc = TpcIni_RDstrdup(ini, "def", "icon", self->NodeIconName);
		xfree(self->NodeIconName);
		self->NodeIconName = pc;

		cpc = TpcIni_RD(ini, "def", "passwd");
		if (cpc) {
		    char *tmppc = CryptMini16c_enc(cpc);
		    if (tmppc) cpc = tmppc;
		    self->NewPasswd = !pccmp(self->gp->passwd, cpc);
		    xfree(tmppc);
		} else {
		    self->NewPasswd = self->gp->passwd != NULL;
		}
	    }
	    Tfree(ini);
	} else {
	    self->gp = xcalloc(sizeof(TglobPath));
	    self->gp->RootArt = pcdup("ftp:"); //isFtp
	    self->new_gp = gp_dup(self->gp);
	    self->NodeAddPath = NodeAddPath;
	}

	self->SyncName = pcdup(self->NodeName);
	self->edTvNode = TEdit_CreateS((TWins*)self, 1, 1, 200, 22, self->NodeName);
	self->edTvNode->KeyFu |= mK_ReturnTab;
	self->edTvNode->CaptionOffX = 22;
	self->default_pic = tvn->pic;
	self->edTvNode->OnPaint  = edTvNode_OnPaint;
	self->edTvNode->OnChange = edTvNode_OnChange;
	self->edTvNode->OnFocus  = edTvNode_OnFocus;

	y = 24;
	self->lblFileName = TWins_CreateLabel((TWins*)self, 4, y, 70, 20, 
					"File Name:", mCaptionLeft, 0);
	self->edFileName = TEdit_CreateS((TWins*)self, 74, y, 140, 20, self->fn);
	self->edFileName->KeyFu |= mK_ReturnTab;
	TEdit_SetReadOnly(self->edFileName, true);

	y += 24;
	self->lblHost = TWins_CreateLabel((TWins*)self, 4, y, 40, 16, 
					"Host:", mCaptionLeft, 0);
	self->edHostName = TEdit_CreateS((TWins*)self, 74, y, 200, 20, self->gp->URL);
	self->edHostName->KeyFu |= mK_ReturnTab;
	self->edHostName->OnChange = edHostName_OnChange;
	TEdit_SetReadOnly(self->edHostName, !MkNewNode);

	y += 24;
	self->lblUser = TWins_CreateLabel((TWins*)self, 4, y, 70, 16, 
					"Username", mCaptionLeft, 0);
	self->edUser = TEdit_CreateS((TWins*)self, 74, y, 160, 20, self->gp->user);
	self->edUser->KeyFu |= mK_ReturnTab;
	self->edUser->OnChange = edUser_OnChange;
	TEdit_SetReadOnly(self->edUser, !MkNewNode);

	y += 24;
	self->lblPasswd = TWins_CreateLabel((TWins*)self, 4, y, 70, 16, 
					"Passwd", mCaptionLeft, 0);
	self->edPasswd = TEdit_CreateS((TWins*)self, 74, y, 160, 20, NULL);
	self->edPasswd->OnChange = edPasswd_OnChange;
	pwmod = ED_PASSWD_ON;
	if (self->gp->passwd) pwmod = ED_PASSWD_ON|ED_PASSWD_SIM_EMPTY;
	TEdit_SetPasswdMod(self->edPasswd, pwmod, NULL);

	y += 24;
	pc = gptopc(self->new_gp, 0);
	self->lblAll = TWins_CreateLabel((TWins*)self, 1, y, 70, 18, pc, 
					mCaptionLeft, mBorder|mBorderL);
	xfree(pc);

	self->pnlBottom = TWins_CreateBA((TWins*)self, 0, 0, 200, 25, 0, alBottom);

/* pnlBottom */
	self->buSave = TButton_CreateS((TWins*)self->pnlBottom, 20, 1, 64, 24, "SAVE");
	self->buSave->OnClick = buSave_OnClick;
	self->buSave->Enabled = self->NewPasswd;
	self->buCancel = TButton_CreateS((TWins*)self->pnlBottom, 100, 1, 64, 24, "Cancel");
	self->buCancel->OnClick = buCancel_OnClick;
	self->pnlBottom->OnResize = pnlBottom_OnResize;

	self->FocusWin = (TWins*)self->edTvNode;
	if (MkNewNode) self->FocusWin = (TWins*)self->edHostName;

	TForm_Show((TForm*)self);
	return;
}
