/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "appXbase.h"
#include "tdirlist.h"
#include "tfind_grid.h"

TDirFileList *TFindGrid_GetSelect_dfl(TFindGrid *self) {
  TDirFileList *dfl = self->dfl;

	while (dfl && !(dfl->fu & fi_Select)) dfl = dfl->next;
	return dfl;
}

TDirFileList *TFindGrid_Get_dfl(TFindGrid *self, int idx) {
  TDirFileList *dfl = self->dfl;

	while (dfl && (idx > 0)) {dfl = dfl->next; idx--;}
	return dfl;
}

void TFindGrid_AddItem(TFindGrid *self, TDirFileList *dfl) {
	if (!dfl) return;
	if (!self->ppfi) {
	    self->ppfi = &self->dfl;
	    while (*self->ppfi) self->ppfi = &(*self->ppfi)->next;
	}
	*self->ppfi = dfl;
	self->ppfi = &dfl->next;
	self->fiCount++;
}

static void dfl_Clr(TFindGrid *self) {
  TDirFileList *dfl;
	while (self->dfl) {
	    dfl = self->dfl;
	    self->dfl = dfl->next;
	    xfree(dfl);
	}
}

void TFindGrid_Clr(TFindGrid *self) {

	self->SelectIdxRow = -1;
	self->SelectIdxCol = -1;
	TGrid_SetRowCount((TGrid*)self, 1);
	self->ppfi = NULL;
	self->fiCount = 0;
	self->fiCount_LastPaint = 0;
	dfl_Clr(self);
}

/* void TFindGrid_Update(TFindGrid *self) {
	TFindGrid_Sort(self, 0);
	TGrid_SetRowCount((TGrid*)self, self->fiCount + 1);
	self->fiCount_LastPaint = self->fiCount;
} */

/* ------------------------- Sort -------------------------------------- */
#define MaxSortProc	4

static int dl_CpS(const char *s1, const char *s2, int fu) {
  if (fu & dlsortFu_Case) return pccasecmp_fi(s1, s2, 0); else return pccmp_i(s1, s2);
}

static int dl_CpF_Name_UP(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1;
	m1 = dl_CpS(Get_ExFileDat_pc(dfl1->st, exfdt_user), 
		    Get_ExFileDat_pc(dfl2->st, exfdt_user), fu);
	if (!m1) m1 = dl_CpS(dfl1->fn, dfl2->fn, fu);
	return m1;
}
static int dl_CpF_Name_DOWN(TDirFileList *dfl1, TDirFileList *dfl2, int fu) {
  int m1;
	m1 = dl_CpS(Get_ExFileDat_pc(dfl1->st, exfdt_user), 
		    Get_ExFileDat_pc(dfl2->st, exfdt_user), fu);
	if (!m1) m1 = dl_CpS(dfl2->fn, dfl1->fn, fu);
	return m1;
}

static void *dl_SortProcArr[MaxSortProc] = {
	dl_CpF_Name_UP, dl_CpF_Name_DOWN,	//Name
	NULL, NULL
};

void grFind_Sort(TDirFileList **ppdfl, int fu) {
  TDirFileList *dfl;
  int i;
  int count = 0;
  TDirFileList **pptmpdfl, **ppfln, **ppfls;

	if (!ppdfl) return;
	count = 0;
	dfl = *ppdfl;
	while (dfl) {
	    count++;
	    dfl = dfl->next;
	}
// printf("%s(%d): count=%d\n", FL, count);
	if (count < 2) return;

	pptmpdfl = xmalloc(sizeof(TDirFileList*) * count);
	ppfln = pptmpdfl;
	dfl = *ppdfl;
	while (dfl) {
	    *ppfln = dfl;
	    ppfln++;
	    dfl = dfl->next;
	}
	i = fu & 0xff;
	if ((i < MaxSortProc) && dl_SortProcArr[i]) {
	    QuickSort_PL((char**)pptmpdfl, 0, count-1, fu, dl_SortProcArr[i]);
	}

	ppfls = ppdfl; //&self->dfl_sort;
	ppfln = pptmpdfl;
	for (;count;count--) {
	    *ppfls = *ppfln;
	    ppfls = &(*ppfls)->next;
	    ppfln++;
	}
	*ppfls = NULL;
	xfree(pptmpdfl);
	return;
}

void TFindGrid_Sort(TFindGrid *self, int fu) {
	if (self && self->dfl) {
	    self->ppfi = NULL;
	    grFind_Sort(&self->dfl, fu);
	}
}
/* ------------------------- PaintCell --------------------------------- */

static bool grFind_DoPaintCell(TFindGrid *self, int ACol, int ARow, TWRect *rc, int *fu) {
  TClassGC *CellGC;
//  TcrDat_Row *cr;
  TcrDat *cc;
  TVDrawString *vds;
  int cst_x, cst_fu;
  char buf[32];
  TWRect rct;
  bool utf8;

	if (!TGrid_DoPaintCell((TGrid*)self, ACol, ARow, rc, fu)) return False;

	cst_x = 2;
	cst_fu = mStrLeft;
//	cr = &self->RowDat[ARow];
	CellGC = self->CellGC;
	vds = &CellGC->vds;
	cc = &self->ColDat[ACol];
	if (ARow == 0) {
	    vds->s_const = cc->name;
	    vds->lg = pclen(vds->s);
	    TClassGC_DrawColorStringFu(CellGC, 2, 0, mStrLeft);
	} else {
	    vds->s = NULL;
	    TDirFileList *dfl = TFindGrid_Get_dfl(self, ARow-1);
	    if (!dfl) return False;
	    Tstat64 *st = dfl->st;
	    utf8 = self->vds.UTF8on;
	    switch(cc->tag) {
	    case 1:  //Name
		vds->s_const = dfl->fn;
	        vds->lg = pclen(vds->s);
		TClassGC_DrawColorStringFu(CellGC, 20, 0, mStrLeft);
		if (dfl->fu & fi_Select) {
		    TClassGC_FillSelectRect_rc(CellGC, rc);
		}
		TClassGC_DrawPic(CellGC, get_fileIcon(st->FileType), 1, 1);
		if (st->st & st_Link) {
		    TClassGC_DrawPic(CellGC, get_fileIcon(stft_Link), 1,11);
		}
		if ((ARow == self->SelectIdxRow) && (self->KeyFu & mHasFocus))
				TClassGC_DrawFocusRect_rc(CellGC, rc);
		vds->s = NULL;
		break;
	    case 2:  //Path
		vds->s_const = Get_ExFileDat_pc(st, exfdt_user);
		break;
	    case 3:  //Size
		file_str_size_pc(buf, st->size, fstrSize_space);
		vds->s = buf;
//		DrawStrFree = False;
		cst_fu = mStrRigth;
		cst_x = -3;
		break;
	    case fgaColModi:  //4 Modified
		file_str_time_HM_pc(buf, st->mtime);
		vds->s = buf;
		break;
	    case fgaColPerm:  //5 Permission
		file_str_mode_pc(buf, st, StrModT_PlusACL);
		vds->s = buf;
		break;
	    case fgaColCharType:  //6 
		vds->s_const = Get_ExFileDat_pc(st, exfdt_user + 2);
		self->FML_utf8 = vds->s && (*vds->s != 'a');
		break;
	    case fgaColFML:   //7 match
		utf8 = self->FML_utf8;
		if (self->Edit->Visible && 
		    (self->Edit->tag == fgaColFML) &&
		    (self->EditRow == ARow) &&
		    TGrid_GetCell_AbsRC((TGrid*)self, ACol, ARow, &rct)) {
		    SetRC((TWins*)self->Edit, rct.left, rct.top, rct.width, rct.height);
		} else {
		    vds->s_const = Get_ExFileDat_pc(st, exfdt_user + 1);
		}
		break;
	    }

	    if (vds->s) {
		self->CellGC->vds.UTF8on = utf8;
	        vds->lg = pclen(vds->s);
		TClassGC_DrawColorStringFu(CellGC, cst_x, 0, cst_fu);
	    }
	}

	vds->s = NULL;
	vds->lg = 0;
	return True;
}

static void grFind_DoPaint(TFindGrid *self) {
	if (self->fiCount_LastPaint != self->fiCount) {
	    self->fiCount_LastPaint = self->fiCount;
	    TFindGrid_Sort(self, 0);
	    TGrid_SetRowCount((TGrid*)self, self->fiCount + 1);
	}
	TGrid_DoPaint((TGrid*)self);
}

static void grFind_OnPaintCellEnd(TFindGrid *self, TPaintCellDat *pcd) {
	if (self->Edit->Visible && 
	    ((pcd->ya > self->EditRow)||(pcd->ye < self->EditRow))) {
	    SetTop((TWins*)self->Edit, -self->Edit->height);
	}
}

/* ------------------------- Cell Ev --------------------------------- */
static void gr_DoItemEnter(TFindGrid *self) {
  int idx;
  TDirFileList *dfl;

	if (!self->OnItemEnter || (self->SelectIdxRow <= 0)) return;
	idx = 0;
	dfl = self->dfl;
	while (dfl) {
	    idx++;
	    if (idx == self->SelectIdxRow) {
		self->OnItemEnter(self, dfl);
		return;
	    }
	    dfl = dfl->next;
	}
}

static void gr_DoSelect(TFindGrid *self, int fu) {
  TDirFileList *dfl;
  int idx, m;

	if ((fu & grBefFu_Mask) == grBefFu_Select) {
	    if ((fu & grBefFu_Select_Idx) ||
		((self->FixedRows <= self->MouseCell.Row) &&
	         (self->SelectIdxRow != self->MouseCell.Row))) {

		self->SelectIdxCol = 0;
		if (!(fu & grBefFu_Select_Idx)) {
		    self->SelectIdxRow = self->MouseCell.Row;
		}

		idx = 0;
		dfl = self->dfl;
		while (dfl) {
		    idx++;
		    if (dfl->fu & fi_Select) {
			dfl->fu &= ~fi_Select;
			m = 0;
			TGrid_PaintCell((TGrid*)self, 0, idx, &m);
		    }

		    if (idx == self->SelectIdxRow) {
			dfl->fu |= fi_Select;
			m = 0;
			TGrid_PaintCell((TGrid*)self, 0, idx, &m);
			if (self->OnSelect) self->OnSelect(self, 0, idx, grBefFu_Select);
		    }
		    dfl = dfl->next;
		}
		TGrid_ChkSelect_RC((TGrid*)self, self->SelectIdxCol, self->SelectIdxRow);


//		gr_Deselect(self, grBefFu_Deselect);
//		gr_Select(self, self->MouseCell.Col, self->MouseCell.Row);
	    }
	    if (!(fu & grBefFu_Select_Idx) &&
		(self->FixedRows <= self->MouseCell.Row) &&
		!self->MouseCell.Col) {
		gr_DoItemEnter(self);
	    }
	
	}
}

static int TFindGrid_DoKeyDown(TFindGrid *self, TKeyEvent *KeyEv) {
  int m;

	if (TWins_DoKeyDown((TWins*)self, KeyEv)) return 1;

	if ((self->SelectIdxRow <= 0) || (self->fiCount <= 0)) return 0;
	m = self->SelectIdxRow;

	switch(KeyEv->keysym) {
	case XK_Up:
	    m--;
	    break;

	case XK_Down:
	    m++;
	    break;

	case XK_Return:
	    gr_DoItemEnter(self);
	    return 1;

	default:
	    return 1;
	}

	if ((m > self->fiCount) || (m < 1)) return 1;
	self->SelectIdxRow = m;
	gr_DoSelect(self, grBefFu_Select|grBefFu_Select_Idx);
	return 1;
}

static void TFindGrid_DoMouseDown(TFindGrid *self, XButtonEvent *ev) {
	TGrid_DoMouseDown((TGrid*)self, ev);
	if (!self->MouseCell.InCell) return;
	if (self->MouseCell.Row > 0) {
	    if (ev->button == 3) {
		TDirFileList *dfl = TFindGrid_Get_dfl(self, self->MouseCell.Row);
		if (dfl && !(dfl->fu & fls_Select)) {
		    self->SelectIdxRow = self->MouseCell.Row;
		    gr_DoSelect(self, grBefFu_Select|grBefFu_Select_Idx);
		}
	    }
	}
}

static void TFindGrid_DoClick(TFindGrid *self, long KeyState, bool DblClick) {
  int Col, Row;
  TcrDat *cc;
  TDirFileList *dfl;
  Tstat64 *st;
  const char *pc;
  TWRect rc;

	TWins_DoClick((TWins*)self, KeyState, DblClick);
	if (!TestWinIsEnabled((TWins*)self) || !DblClick) return;

	Col = self->MouseCell.Col;
	Row = self->MouseCell.Row;
	cc = &self->ColDat[Col];
	if (Row < self->FixedRows) return;
	if (cc->tag == fgaColFML) {
	    if (!TGrid_GetCell_AbsRC((TGrid*)self, Col, Row, &rc)) return;  
	    dfl = TFindGrid_Get_dfl(self, Row-1);
	    if (!dfl) return;
	    st = dfl->st;
	    pc = Get_ExFileDat_pc(st, exfdt_user + 2);
	    self->Edit->vds.UTF8on = pc && (*pc != 'a');
	    
	    pc = Get_ExFileDat_pc(st, exfdt_user + 1);
	    if (!pc || !*pc) return;
	    self->Edit->tag = fgaColFML;
	    self->EditRow = Row;
	    self->EditCol = Col;

	    SetRC((TWins*)self->Edit, rc.left, rc.top, rc.width, rc.height);
	    SetCaption((TWins*)self->Edit, pc);
	    SetVisible((TWins*)self->Edit, True);
	    Set_Focus((TWins*)self->Edit);
	}
}

static void Edit_OnFocus(TEdit *self, int Change) {
	if (!(Change & WinsFocusIN)) {
	    SetVisible((TWins*)self, False);
	}
}
/*_____________________________ Destroy _________________________________*/
static bool TFindGrid__Destroy(TFindGrid *self) {
	dfl_Clr(self);
	return TGrid__Destroy((TGrid*)self);
}

/*_____________________________ Create __________________________________*/
typedef struct {
  int art;
  int fu;
  int Width;
  int min, max;
  const char *name;
} TgrColD;

static const TgrColD TgrColD_arr[] = {
  {fgaColName, 1, 140,  40, 400, "Name"},
  {fgaColPath, 1, 140,  40, 400, "Path"},
  {fgaColSize, 1,  64,  64,  64, "Size"},
  {fgaColModi, 1, 104,  68, 104, "Modified"},
  {fgaColPerm, 1,  68,  68,  68, "Permission"},
  {fgaColCharType, 1, 40, 40,50, "Char"},
  {fgaColFML,  1, 400,  40,2000, "First Matching Line"}
};

void TFindGrid_SetColVisible(TFindGrid *self, int fga, bool Visible) {
	for (int i=0;i<6;i++) {
	    if (self->ColDat[i].tag == fga) {
		TGrid_SetColVisible((TGrid*)self, i, Visible);
		return;
	    }
	}
}

static void TFindGrid_CreateParams(TWins_Create_struct *wcs) {
  TFindGrid *self;
  char buf[16];
  TpcIni *ini;

	TGrid_CreateParams(wcs);
	self = wcs->wins;
	self->destroy = (TDESTROY)TFindGrid__Destroy;
	self->FixedColsFu |= grFixResize | grFixNoBorder;
	self->FixedColsColor = 0xf0f0f0;
	self->DoSelect = gr_DoSelect;

	ini = get_appIni();
	TGrid_SetColCount((TGrid*)self, 7);
	TGrid_SetRowCount((TGrid*)self, 1);
	pcmove(buf, "GridW_0");
	TcrDat *cr = self->ColDat;
	for (int i=0;i<fgaColLast;i++) {
	    buf[6] = 0x30 + i;
	    TGrid_Set_crMinMax((TGrid*)self, i, True, TgrColD_arr[i].min, TgrColD_arr[i].max);
	    int w = TpcIni_RD_int(ini, app_name, buf, TgrColD_arr[i].Width);
	    TGrid_Set_crSize((TGrid*)self, i, True, w);
	    cr->tag = TgrColD_arr[i].art;
	    cr->Visible = TgrColD_arr[i].fu != 0;
	    cr->name = TgrColD_arr[i].name;
	    cr++;
	}

	self->DoPaint = grFind_DoPaint;
	self->DoPaintCell = grFind_DoPaintCell;
	self->DoKeyDown = TFindGrid_DoKeyDown;
	self->DoMouseDown = TFindGrid_DoMouseDown;
	self->DoClick = TFindGrid_DoClick;
	self->OnPaintCellEnd = grFind_OnPaintCellEnd;
	free_appIni(ini);
}

static void TFindGrid_DefaultConf(TWins_Create_struct *wcs) {
  TFindGrid *self;

	TGrid_DefaultConf(wcs);
	self = wcs->wins;

	self->Edit = TEdit_Create((TWins*)self);
	self->Edit->BorderFu = mBorder | mBorder2D;
	self->Edit->Visible = False;
	self->Edit->ReadOnly = True;
	self->Edit->ReadOnly = True;
	Set_Parent((TWins*)self->Edit, (TWins*)self);
	self->Edit->OnFocus = Edit_OnFocus;
}

TFindGrid *TFindGrid_Create(TWins *Parent) {
  TWins_Create_struct wcs;
	TWins_Create_struct_init(&wcs, sizeof(TFindGrid), wtyp_Grid);
	wcs.Parent = Parent;
	wcs.CreateParams = TFindGrid_CreateParams;
	TFindGrid_DefaultConf(&wcs);
	return wcs.wins;
}
