/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldfile.h"
#include "dlg.h"
#include "FormExecWith.h"
#include "miCopy.h"

#ifdef _OS_WIN32
typedef struct _DROPFILES {
	DWORD pFiles;
	POINT pt;
	BOOL fNC;
	BOOL fWide;
} DROPFILES,*LPDROPFILES;

typedef enum tagDROPEFFECT {
	DROPEFFECT_NONE=0,
	DROPEFFECT_COPY=1,
	DROPEFFECT_MOVE=2,
	DROPEFFECT_LINK=4,
	DROPEFFECT_SCROLL=0x80000000
} DROPEFFECT;
#endif

#define en_all		1
#define en_dfl		(1<<1)
#define en_gp		(1<<2)
#define en_paste	(1<<3)

#define vi_fw		(1<<4)
#define vi_tv		(1<<5)
#define vi_tvnet	(1<<6)
#define vi_tvftp	(1<<7)

#define fkt_cut		(1<<16)

/* ______________________ misc __________________________________________ */
bool get_MenuFileItem_xdfl(TMenuFile *self, Txdfl *xdfl) {
  TMainForm *mf = appDat.obj;
  int wtyp;

	wtyp = self->owner->wtyp;
	if (wtyp == wtyp_MenuButton) {
	    if (mf->LastNoMenuFocusWin) wtyp = mf->LastNoMenuFocusWin->wtyp;
	}

	xdfl->gpList = NULL;
	if (wtyp == wtyp_TTreeVFile) {
	    xdfl->istv = True;
	    xdfl->o.tv = mf->tv;
	    xdfl->dfl = TTreeVFile_GetSelect_dfl(mf->tv);
	    xdfl->gp = TTreeVFile_GetSelect_gp(mf->tv, False);
	} else {
	    xdfl->istv = False;
	    xdfl->o.fw = mf->fw;
	    xdfl->dfl = TFileWin_Get_Select_dfl(mf->fw, NULL);
	    xdfl->gp = TFileWin_Get_gp(mf->fw);
	}
	xdfl->isLocalF = gp_test_local_drive(xdfl->gp);
	return xdfl->dfl != 0;
}

/* ______________________ menuNew _______________________________________ */
static void menuNew_OnMenuOpen(TMenu *self) {
  TMainForm *mf = (TMainForm*)self->FormStruct;
  TglobPath *gp;
  TMenuItem *mi;
  bool mk;

	gp = TFileWin_Get_gp(mf->fw);
	mi = self->MenuItem;
	while (mi) {
	    mi->fu &= ~miNotEnabled;
	    if (mi->tag == ffnNewDirectory) {
		mk = net_mkdir_gp_test(gp);
	    } else if (mi->tag == ffnNewTxtFile) {
		mk = net_file_new_va_gp_test(gp);
	    } else {
		mk = gp_test_local_drive(gp);
	    }

	    if (!mk) mi->fu |= miNotEnabled;
	    mi = mi->next;
	}
}

static void miNewFile_OnClick(TMenu *self, long KeyState, TMenuItem *mi) {
  TNewFile_Dlg *nf_dlg;
  TMainForm *mf = (TMainForm*)self->FormStruct;

	nf_dlg = Create_NewFile_Dlg((TWins*)mf, mi->tag);
	nf_dlg->gp = gp_dup(TFileWin_Get_gp(mf->fw));
	NewFile_Dlg_Execute(nf_dlg);
	Tfree(nf_dlg);
}

static void miNewURL_OnClick(TMenuFile *self, long KeyState, TMenuItem *mi) {
  TMainForm *mf = appDat.obj;
  Txdfl xdfl;

	if (!get_MenuFileItem_xdfl(self, &xdfl)) {
	    if (xdfl.istv) {
		TFormPropNet_Show(mf, &xdfl);
	    }
	}
}

/* ______________________ menu OpenWith ______________________________ */
static bool set_next_xdfl(Txdfl *xdfl) {
	if (xdfl->istv) {
	    xdfl->dfl = NULL;
	} else {
	    xdfl->dfl = TFileWin_Get_Select_dfl(xdfl->o.fw, xdfl->dfl);
	}
	return xdfl->dfl != 0;
}

static TgpList *get_gpList_xdfl(Txdfl *xdfl) {
  TgpList *gpList;

	gpList = xcalloc(sizeof(TgpList));
	gp_dup_gpdata_path(&gpList->gp, xdfl->gp);
	if (xdfl->dfl) gpList->gp.name = pcdup(xdfl->dfl->fn);
	return gpList;
}

static void set_xdfl_gpList(Txdfl *xdfl) {
  TgpList *gpList;

	xdfl->gpList = get_gpList_xdfl(xdfl);
	gpList = xdfl->gpList;

	while (set_next_xdfl(xdfl)) {
	    gpList->next = get_gpList_xdfl(xdfl);
	    gpList = gpList->next;
	}
}

static void free_xdfl(Txdfl *xdfl) {
  TgpList *gpList, *ngpList;

	gpList = xdfl->gpList;
	xdfl->gpList = NULL;
	while (gpList) {
	    ngpList = gpList->next;
	    gp_pc_free(&gpList->gp);
	    xfree(gpList);
	    gpList = ngpList;
	}
}


static bool OpenWith_OnGetDskCmd(TMenuOpenWith *self, TDesktopCMD *dskcmd, int fu) {
  Txdfl xdfl;

	get_MenuFileItem_xdfl((TMenuFile*)self, &xdfl);

	if ((fu & MOWfu_QnlyLocal) && !xdfl.isLocalF) return False;

	if (fu & MOWfu_OnlyDir) {
	    xdfl.gpList = xcalloc(sizeof(TgpList));
	    gp_dup_gpdata_path(&xdfl.gpList->gp, xdfl.gp);
	} else {
	    if (!xdfl.dfl) return False;
	    set_xdfl_gpList(&xdfl);
	}
	dskcmd->gpList = xdfl.gpList;
	xdfl.gpList = NULL;
//	free_xdfl(&xdfl);

	return True;
}

static TMenuOpenWith *OpenWith_OnNextMenuCreate(TMenuOpenWith *self) {
  TMenuOpenWith *next;
	next = TMenuOpenWith_Create((TWins*)self);
	next->OnGetDskCmd = OpenWith_OnGetDskCmd;
	return next;
}

static void menuOpenWithFind_OnClick(TMenuFile *self, long KeyState, TMenuItem *mi) {
  TMainForm *mf = (TMainForm*)self->FormStruct;
  TMimeDat md;
  Txdfl xdfl;
  TDesktopCMD dskcmd;

	if (!get_MenuFileItem_xdfl(self, &xdfl)) return;

	if (TFormExecWith_Show((TWins*)mf->fw, xdfl.dfl->fn, xdfl.dfl->st, &md)) {

	    set_xdfl_gpList(&xdfl);
	    dskcmd.gpList = xdfl.gpList;
	    dskcmd.Exec = md.exec;
	    get_desktop_dskcmd(&dskcmd);
	    if (dskcmd.ret_cmd) {
		exec_pc(dskcmd.ret_cmd);
		xfree(dskcmd.ret_cmd);
	    }
	    free_xdfl(&xdfl);
	    xfree(md.exec);
	}
}

/* ______________________ menu Properties ____________________________ */
static void miProperties_OnClick(TMenuFile *self, long KeyState, TMenuItem* mi) {
  TMainForm *mf = appDat.obj;
  TDirFileList *dfl0, **ppdfl;
  Txdfl xdfl;

	dfl0 = NULL;
	if (!get_MenuFileItem_xdfl(self, &xdfl)) {
	    if (xdfl.istv && xdfl.gp) {
		if (gp_test_local_drive(xdfl.gp)) {
		    goto prop_show;
		} else {
		    TFormPropNet_Show(mf, &xdfl);
		}
	    }
	    return;
	}

	ppdfl = &dfl0;
	*ppdfl = dfl_dup(xdfl.dfl);
	
	while (set_next_xdfl(&xdfl)) {
	    ppdfl = &(*ppdfl)->next;
	    *ppdfl = dfl_dup(xdfl.dfl);
	}
prop_show:
	//dfl0 free in TFormProperty
	TFormProperty_Show((TWins*)mf, xdfl.gp, dfl0, app_mime, xdfl.istv);
}

/* ______________________ menu Copy __________________________________ */
static char *get_sel_str(TMenuFile *self, bool passwd) {
  Txdfl xdfl;
  char *result;
  int gptfu;

	if (!get_MenuFileItem_xdfl(self, &xdfl)) return NULL;
	gptfu = gptfu_noPasswd;
	if (passwd) gptfu = gptfu_PasswdTxt;
	result = NULL;

	if (xdfl.istv) {
	    return gptopc(xdfl.gp, gptfu | gptfu_del_slash);
	}

loop:
	if (!pccmp(xdfl.dfl->fn, "..")) {
	    char *si = xdfl.gp->name;
	    xdfl.gp->name_const = xdfl.dfl->fn;
	    char *tmp = gptopc(xdfl.gp, gptfu);
	    xdfl.gp->name = si;
	    ppcadd2(&result, tmp, "\n");
	    xfree(tmp);
	}
	if (set_next_xdfl(&xdfl)) goto loop;
	return result;
}

static void miCopy_OnClick(TMenuFile *self, long KeyState, TMenuItem* mi) {
  TMainForm *mf = appDat.obj;
  char *pcsel, *pcsel_passwd;
//  bool utf8;

	appXDat.SelCut = (mi->tag & fkt_cut) != 0;
	SelClr();

	pcsel = get_sel_str(self, False);
	if (!pcsel) return;
	pcsel_passwd = get_sel_str(self, True);

//	utf8 = mf->fw->vds.UTF8on;
	miCopy(pcsel, pcsel_passwd, mf->fw->vds.UTF8on);
	xfree(pcsel_passwd);
	xfree(pcsel);
#if 0
#ifdef _OS_WIN32
  char *pc;
  int lg = pclen(pcsel);
  int wlg = lg;
  DROPFILES *DropFile;
  HGLOBAL hGlobMem;

	if (utf8) {
	    int fuEOL = 0;
	    wlg = Chk_fio_UTF8(pcsel, lg, &fuEOL);
	    if ((fuEOL & fio_has_0x80_char) && 
		!(fuEOL & fio_has_UTF8_char)) utf8 = False;
	    if (!utf8) wlg = lg;
	}
	pc = pcsel;
	while (*pc) {
	    if (*pc == '\n') *pc = 0;
	    pc++;
	}

	if (Clipboard_Open()) {
	    wlg++;
	    hGlobMem = GlobalAlloc(GMEM_MOVEABLE | GMEM_SHARE, 
				sizeof(DROPFILES) + (wlg * 2));
	    if (hGlobMem) {
		pc = GlobalLock(hGlobMem);
		DropFile = (DROPFILES*)pc;
		DropFile->pFiles = sizeof(DROPFILES);
		DropFile->pt.x = 0;
		DropFile->pt.y = 0;
		DropFile->fNC = 0;
		DropFile->fWide = 1;
		pc += sizeof(DROPFILES);
		int UCfu = 0;
		if (!utf8) UCfu = UC_force_ASCII;
		Utf8ToWChars((word*)pc, wlg, pcsel, lg, UCfu);
		Clipboard_Add();
		SetClipboardData(CF_HDROP, hGlobMem);
		GlobalUnlock(hGlobMem);
	    }

	    int CF_n = Get_Preferred_DropEffect();
	    if (CF_n) {
		hGlobMem = GlobalAlloc(GMEM_MOVEABLE | GMEM_SHARE, sizeof(int));
		if (hGlobMem) {
		    int *pint = GlobalLock(hGlobMem);

		    if (appXDat.SelCut) {
			*pint = DROPEFFECT_MOVE;
		    } else {
			*pint = DROPEFFECT_COPY;
		    }
		    Clipboard_Add();
		    SetClipboardData(CF_n, hGlobMem);
		    GlobalUnlock(hGlobMem);
		}
	    }
	    Clipboard_Close();
	}
	xfree(pcsel);

#else /* _OS_WIN32 */
	if (utf8) {
	    int fuEOL = 0;
	    Chk_fio_UTF8(pcsel, pclen(pcsel), &fuEOL);
	    if ((fuEOL & fio_has_0x80_char) && 
		!(fuEOL & fio_has_UTF8_char)) utf8 = False;
	}

	pctoUTF8esclgs(pcsel, &appXDat.SelTxt);
	xfree(pcsel);

	pcsel = get_sel_str(self, True);
	utf8 = mf->fw->vds.UTF8on;
	if (utf8) {
	    int fuEOL = 0;
	    Chk_fio_UTF8(pcsel, pclen(pcsel), &fuEOL);
	    if ((fuEOL & fio_has_0x80_char) && 
		!(fuEOL & fio_has_UTF8_char)) utf8 = False;
	}
	pctoUTF8esclgs(pcsel, &appXDat.SelTxt_URL);
	xfree(pcsel);

	if (utf8) {
	    SetSelection(4, appXDat.XA_URL_STRING, appXDat.XA_UTF8_STRING, XA_STRING, appXDat.XA_KDE_CUT);
	} else {
	    SetSelection(3, appXDat.XA_URL_STRING, XA_STRING, appXDat.XA_KDE_CUT);
	}
#endif /* ! _OS_WIN32 */
#endif
}

/* ______________________ menu Del ___________________________________ */
static void miDelete_OnClick(TMenuFile *self, long KeyState, TMenuItem* mi) {
  TMainForm *mf = appDat.obj;
  Txdfl xdfl;
  TTreeVNode *tvn;
  char *pcsel;

	appXDat.SelCut = 0;

	if (!get_MenuFileItem_xdfl(self, &xdfl)) {
	    if (!xdfl.istv) return;
	    tvn = TTreeVFile_GetSelectNode(xdfl.o.tv);
	    if (!tvn || ((tvn->art & tfvfart_MASK) != tfvfart_ROOT)) return;
	    if (!tvn->tagp) {
		if (TTreeVFileNode_TestIsDummyNode(tvn)) {
		    Tfree(tvn);
		}
		return;
	    }

	    pcsel = pcdup(tvn->tagp);
	} else {
	    pcsel = get_sel_str(self, False);
	}

	TFormDel_Show((TWins*)mf->fw, pcsel);
	xfree(pcsel);
}
/* ______________________ menu Paste _________________________________ */
#if 0
static void db_hex(char *p, int x, int lg) {
  char buf[20];
  char *pc;

	printf("%04x  ", x);
	pc = buf;
	if (lg > 16) lg = 16;
	for (int i=0;i<16;i++) {
	  if (i == 8) printf(" ");
	  if (i < lg) {
	      printf("%02x ", *p & 0xff);
	      if (*p >= ' ') *pc = *p; else *pc = ' ';
	      p++;
	  } else {
	      *pc = ' ';
	      printf("   ");
	  }
	  pc++;
	}
	*pc = 0;
	printf("%s\n", buf);
}
#endif

static void miPaste_OnClick(TMenu *self, long KeyState, TMenuItem* mi) {
  TMainForm *mf = appDat.obj;
  char *filelist;
#ifdef _OS_WIN32
  HGLOBAL hGlobMem;
  char *p;
#else
  Atom SelAtom, CutAtom;
#endif
  TglobPath *gp;

	gp = TFileWin_Get_gp(mf->fw);
	if (!gp) return;

	SelRetClr();
	filelist = NULL;
#ifdef _OS_WIN32

	Clipboard_Open();
	hGlobMem = GetClipboardData(CF_HDROP);
	if (hGlobMem) {
	    p = GlobalLock(hGlobMem);

  DROPFILES *DropFile;
	    DropFile = (DROPFILES*)p;
	    if (DropFile->pFiles == sizeof(DROPFILES)) {
		if (DropFile->fWide == 1) {
		    char *pc = p;
		    pc += sizeof(DROPFILES);
		    int lg = 0;
		    word *pwc = (word*)pc;
		    while (pwc[0] || pwc[1]) {
			lg++;
			pwc++;
		    }
		    int lgs = WCharsToUtf8(NULL, 0, (word*)pc, lg + 1, 0);
		    filelist = xmalloc(lgs + 1);
		    WCharsToUtf8(filelist, lgs + 1, (word*)pc, lg + 1, 0);

		    pc = filelist;
		    while (pc[0] || pc[1]) {
			if (!*pc) *pc = '\n';
			pc++;
		    }
		    *pc = '\n';
		}
	    }
#if 0
 int lg = (int)GlobalSize(hGlobMem);
 printf("%s(%d): lg=%d (%04x)\n", FL, lg, lg);
 int x = 0;
 while (lg > 0) {
  db_hex(p, x, lg);
  x += 16;
  p += 16;
  lg -= 16;
 }
#endif
	    GlobalUnlock(hGlobMem);

	    appXDat.SelCut = False;
	    int CF_n = Get_Preferred_DropEffect();
	    if (CF_n) {
		hGlobMem = GetClipboardData(CF_n);
		if (hGlobMem) {
		    int *pint = GlobalLock(hGlobMem);
		    appXDat.SelCut = (*pint & DROPEFFECT_MOVE) != 0;
		    GlobalUnlock(hGlobMem);
		}
	    }
	}
	Clipboard_Close();
#else /* _OS_WIN32 */

	SelAtom = CutAtom = 0;
	if (Get_XA_TARGETS()) {
	    SelAtom = get_nSelAtom(appXDat.XA_URL_STRING);
	    CutAtom = get_nSelAtom(appXDat.XA_KDE_CUT);
	}
	SelRetClr();
	if (!SelAtom) return;

	if (CutAtom) {
	    if (GetPasteDaten(appXDat.CLIPBOARD_ATOM, CutAtom)) {
		appXDat.SelCut = appXDat.SelRet_lgs.s[0] & 0x01;
	    }
	}

	if (GetPasteDaten(appXDat.CLIPBOARD_ATOM, SelAtom)) {
	    if (appXDat.SelRetFormat != 8) {
		SelRetClr();
		return;
	    }
	}

	filelist = UTF8esctopc(appXDat.SelRet_lgs.s);
#endif /* ! _OS_WIN32 */
	TFormCopy_Show((TWins*)mf->fw, filelist, gp);
	xfree(filelist);
}

TglobPath *pcsel_to_tls(char *pcsel, TLines *tls) {
  Tlgs lgs;
  TLines *tmp_tls;
  TColS *cs;
  TglobPath *result;

	tmp_tls = TLines_Create();
	lgs.s = pcsel;
	lgs.lg = pclen(pcsel);
	TLines_Paste_lgs(tmp_tls, NULL, NULL, &lgs, 4096);

	for (int i=0; i<tmp_tls->Count;i++) {
	    cs = TLines_Get_pcs(tmp_tls, i);
	    TLines_Add(tls, GetFileName_pc(cs->s));
	}

	cs = TLines_Get_pcs(tmp_tls, 0);
	result = pctogp(cs->s);
	ppcfree(&result->name);
	Tfree(tmp_tls);

	return result;
}

/* ______________________ OnMenu _____________________________________ */
static void menuEdit_OnMenuOpen(TMenuFile *self) {
  TMenuFileItem *mi;
  Txdfl xdfl;
  int VisibleMk, EnabledMk;
  bool dflok;

	VisibleMk = 0;
	dflok = get_MenuFileItem_xdfl(self, &xdfl);
	if (xdfl.istv) {
	    if (dflok) {
		VisibleMk = vi_tv;
	    } else {
		if (xdfl.gp) {
		    VisibleMk = vi_tvnet;
		    if (gp_test_local_drive(xdfl.gp)) VisibleMk = vi_tv;
		} else {
		    TMainForm *mf = appDat.obj;
		    TTreeVNode *tvn = TTreeVFile_GetSelectNode(xdfl.o.tv);
		    while (tvn) {
			if (tvn == mf->tvftp) {
			    VisibleMk = vi_tvftp;
			    break;
			}
			tvn = TTreeVNode_get_prev_child(tvn);
		    }
		}    
	    }

	} else {
	    if (xdfl.dfl && pccmp(xdfl.dfl->fn, "..")) {
		set_next_xdfl(&xdfl);
	    }
	    VisibleMk = vi_fw;
	}

	EnabledMk = 0;
	if (dflok) EnabledMk = en_dfl;
	if (xdfl.gp) EnabledMk |= en_gp;
#ifdef _OS_WIN32

#if 0
 DWORD enu = 0;
 char buf[256];

	Clipboard_Open();
loop:
	enu = EnumClipboardFormats(enu);
	buf[0] = 0;
	if (enu) GetClipboardFormatNameA(enu, buf, 256);
	printf("%s(%d): 0x%04x %s\n", FL, (int)enu, buf);
	if (enu) goto loop;
	Clipboard_Close();
#endif

	if (XCopy_GetHande(xdfl.gp, True)) {
	    if (Chk_GetSelection(1, CF_HDROP)) EnabledMk |= en_paste;
	}
#else
	if (XCopy_GetHande(xdfl.gp, True)) {
	    if (Chk_GetSelection(1, appXDat.XA_URL_STRING)) EnabledMk |= en_paste;
	}
#endif

	mi = self->MenuItem;
	while (mi) {
	    mi->fu |= miNotVisible|miNotEnabled;
	    if (mi->tag & VisibleMk) mi->fu &= ~miNotVisible;
	    if ((mi->tag & EnabledMk) || (mi->tag & en_all)) mi->fu &= ~miNotEnabled;
	    mi = mi->next;
	}
}

/* ______________________ Create _____________________________________ */
void MenuEdit_Create(TMainForm *self) {
  TMenuItem *mi;

	self->mbEdit = TMenuButton_CreateS(self->pnlTop, self->mbFile, "Edit");

	self->menuEdit = TMenu_Create((TWins*)self);
	self->mbEdit->menu = self->menuEdit;
	self->menuEdit->OnMenuOpen = (TMenuEv)menuEdit_OnMenuOpen;
	mi = TMenu_AddItem(self->menuEdit, "Create New", 0, NULL);
	mi->tag = en_gp|vi_fw;
	  self->menuNew = TMenu_Create((TWins*)self);
	  self->menuNew->OnMenuOpen = menuNew_OnMenuOpen;
	  mi->next_menu = self->menuNew;
	  mi = TMenu_AddItem(self->menuNew, "Directory", 0, miNewFile_OnClick);
	  mi->tag = ffnNewDirectory;
	  mi->pic = get_fileIcon(stft_Folder);
	  mi = TMenu_AddItem(self->menuNew, "Text File", 0, miNewFile_OnClick);
	  mi->tag = ffnNewTxtFile;
	  mi->pic = get_fileIcon(stft_Txt);

	TMenu_AddItem(self->menuEdit, "Create New URL", 0, 
		(TMenuClickEv)miNewURL_OnClick)->tag = en_all|vi_tvftp;


	TMenu_AddItem(self->menuEdit, "-", 0, NULL)->tag = vi_fw;
	self->miOpenWith = TMenu_AddItem(self->menuEdit, "Open With", 0, NULL);
	self->miOpenWith->tag = vi_fw|en_dfl;
	  self->menuOpenWith = TMenuOpenWith_Create((TWins*)self);
	  self->miOpenWith->next_menu = (TMenu*)self->menuOpenWith;
	  self->menuOpenWith->OnGetDskCmd = OpenWith_OnGetDskCmd;
	  self->menuOpenWith->OnNextMenuCreate = OpenWith_OnNextMenuCreate;
	  self->menuOpenWith->path = pcadd(appDat.ALDini_dir, "/Menu/OpenWith/");

//	self->miTools = TMenu_AddItem(self->menuEdit, "Tools", 0, NULL);
//	self->miTools->tag = vi_fw|vi_tv|en_gp;

	TMenu_AddItem(self->menuEdit, "Win Exec With", 0, 
		(TMenuClickEv)menuOpenWithFind_OnClick)->tag = vi_fw|en_dfl;

	TMenu_AddItem(self->menuEdit, "-", 0, NULL)->tag = vi_fw|vi_tv;
	TMenu_AddItem(self->menuEdit, "Copy", 0, 
		(TMenuClickEv)miCopy_OnClick)->tag = vi_fw|vi_tv|en_dfl;
	TMenu_AddItem(self->menuEdit, "Paste", 0, miPaste_OnClick)->tag = vi_fw|en_paste;
	TMenu_AddItem(self->menuEdit, "-", 0, NULL)->tag = vi_fw|vi_tv;
	TMenu_AddItem(self->menuEdit, "Cut", 0, 
		(TMenuClickEv)miCopy_OnClick)->tag = vi_fw|en_dfl|fkt_cut;
	TMenu_AddItem(self->menuEdit, "Delete", 0, 
		(TMenuClickEv)miDelete_OnClick)->tag = vi_fw|vi_tv|vi_tvnet|en_dfl|en_gp;
	TMenu_AddItem(self->menuEdit, "-", 0, NULL)->tag = vi_fw|vi_tv|vi_tvnet|en_dfl|en_gp;
	TMenu_AddItem(self->menuEdit, "Properties", 0, 
		(TMenuClickEv)miProperties_OnClick)->tag = vi_fw|vi_tv|vi_tvnet|en_dfl|en_gp;
}
