/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldfile.h"
#include "tdirlist.h"
#include "cicons.h"

static bool TTreeVFileNode_NetRoot__Destroy(TTreeVNode *self) {
	xfree(self->tagp);
	return TTreeVFileNode__Destroy(self);
}

/*_____________________________ dflEv __________________________________*/
static void v_net_OnFileMsg(TFileMsg *fmsg) {
  TDirListObjNy *self;
  int bef;
  TMainForm *mf;
  TTreeVNode *tvfn;
  TTreeVNode *tvn;
  TDirFileList *dfl;
  char *fn, *url;
  const char *cpc;
  TpcIni *ini;
  TglobPath gp;

	if (!file_msg_chk_wtyp(fmsg, wtyp_TDirListObjNy, FL)) return;

	if ((fmsg->art != fmArt_DirList) ||
	    (fmsg->typD != fmTyp_int)) return;

	mf = appDat.obj;
	self = fmsg->obj;
	tvfn = self->owner;
	bef = fmsg->D.m;
	dfl = NULL;
	if (fmsg->typF == fmTyp_dfl) dfl = fmsg->F.dfl;

	switch (bef & dlbef_BefMask) {
	case dlbef_Add:
	    if (dfl->st->st & st_Dir) {
		tvn = TTreeV_AddNode((TTreeV*)mf->tv, tvfn, NULL, dfl->fn);
		dfl->tagp = tvn;
		tvn->pic = get_fileIcon(stft_Folder);
		fn = pcadd_n(3, self->dl->gp.path, dfl->fn, "/");
  TDirListObjNy *ObjNy;
		ObjNy = TDirListObjNy_Create(tvn);
		ObjNy->findAtr = findIsRootPath|findDIR_NoPattern;
		ObjNy->chstat_Filter = chstat_Add|chstat_Del|chstat_mtime|chstat_nlink;
		ObjNy->OnFileMsg = v_net_OnFileMsg;
		TPattern_set(&ObjNy->patt, "*.aldfile", NULL);
		if (!TDirListObjNy_Open(ObjNy, fn)) {
		    Tfree(ObjNy);
		}
		xfree(fn);

	    } else if (dfl->st->st & st_Reg) {
		fn = pcadd(self->dl->gp.path, dfl->fn);
		ini = TpcIni_Create(fn);
		xfree(fn);
		gp_NULL(&gp);
		gp.RootArt = pcdup("ftp:");
		gp.URL = TpcIni_RDstrdup(ini, "def", "url", NULL);
		gp.user = TpcIni_RDstrdup(ini, "def", "user", NULL);
		gp.passwd = TpcIni_RDstrdup(ini, "def", "passwd", NULL);
		if (gp.passwd) {
		    char *passwd = CryptMini16c_enc(gp.passwd);
		    if (passwd) {
			xfree(gp.passwd);
			gp.passwd = passwd;
		    }
		}
		gp.path = pcdup("/");
		cpc = TpcIni_RD_pc(ini, "def", "Caption", dfl->fn);
		url = gptopc(&gp, gptfu_PasswdTxt | gptfu_noName);

		char *tmppc = ChangeFileExt(dfl->fn, NULL);
		tvn = TTreeVFileNode_GetDummyNode_name(tvfn, tmppc);
		xfree(tmppc);

		if (!tvn) tvn = TTreeVFile_AddRoodNode(mf->tv, tvfn, NULL, cpc, url);
		tvn->destroy = (TDESTROY)TTreeVFileNode_NetRoot__Destroy;
		tvn->tagp = pcdup(ini->FileName);
		dfl->tagp = tvn;
		xfree(url);
		gp_pc_free(&gp);
		Tfree(ini);

		tvn->pic = get_pic16_ftp(False);
	    }
	    break;

	case dlbef_Del:
	    Tfree(dfl->tagp);
	    break;

	case dlbef_Change:
	    tvn = dfl->tagp;
	    if (tvn) {
		fn = pcadd(self->dl->gp.path, dfl->fn);
		ini = TpcIni_Create(fn);
		xfree(fn);
		cpc = TpcIni_RD_pc(ini, "def", "Caption", tvn->s);
		if (ppcchange_cmp(&tvn->s, cpc)) tvn->lg = pclen(tvn->s);
		Tfree(ini);
	    }
	    break;

	case dlbef_ListEnd:
	    if (self->chstat) {
		TTreeVFile_SortNode(self, 0, tvfn);
		TTreeV_Update_ScrollBar((TTreeV*)mf->tv, NULL);
	    }
	    self->chstat = 0;
	}
}

void tv_net_create() {
  TMainForm *mf = appDat.obj;
  TDirListObjNy *ObjNy;
  char *path;

	path = pcadd(appDat.ALDini_dir, "/Net/ftp/");

	ObjNy = TDirListObjNy_Create(mf->tvftp);
	ObjNy->findAtr = findIsRootPath|findDIR_NoPattern;
	ObjNy->chstat_Filter = chstat_Add|chstat_Del|chstat_mtime|chstat_nlink;
	ObjNy->OnFileMsg = v_net_OnFileMsg;
	TPattern_set(&ObjNy->patt, "*.aldfile", NULL);
	if (!TDirListObjNy_Open(ObjNy, path)) {
	    Tfree(ObjNy);
	}
	xfree(path);
}
