/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldfile.h"
#include "cicons.h"
#include "tfilewin.h"

static const char zico12frun000[] = {
 0x01,0x04,0x0C,0x00,0x0C,0x00,0x00,0x00,0x3E,0x00,0x00,0x40,0x1C,0xE7,0x00,0xA5,
 0x10,0xC6,0x60,0xC0,0x3F,0xFE,0xE7,0x7F,0xFE,0xF7,0xFF,0xFF,0xEF,0x7F,0xFE,0xE7,
 0x7F,0xFC,0x03,0x06,0x00,0x28,0x40,0xAD,0x17,0x54,0x7B,0x41,0xB5,0x15,0x54,0x41,
 0x15,0x54,0x41,0x15,0x54,0x5E,0x41,0xED,0x15,0xD4,0x7A,0x01,0x28,0x00};

static const char zico12frun045[] = {
 0x01,0x04,0x0C,0x00,0x0C,0x00,0x00,0x00,0x3C,0x00,0x00,0x40,0x00,0xC6,0x10,0xC6,
 0x1C,0xE7,0x60,0x80,0x1F,0xFC,0xE3,0x7F,0xFE,0xF7,0xFF,0xFF,0xEF,0x7F,0xFE,0xC7,
 0x3F,0xF8,0x01,0x06,0x00,0x0F,0xFC,0x1B,0xFC,0x5B,0xC1,0xBF,0x25,0xFC,0x83,0x3E,
 0xBC,0xC2,0x3F,0x58,0xFE,0x43,0xE5,0x3F,0xE4,0x3F,0xF0,0x00};

static const char zico12frun090[] = {
 0x01,0x04,0x0C,0x00,0x0C,0x00,0x00,0x00,0x3E,0x00,0x00,0x40,0x00,0xA5,0x10,0xC6,
 0x1C,0xE7,0x60,0xC0,0x3F,0xFE,0xE7,0x7F,0xFE,0xF7,0xFF,0xFF,0xEF,0x7F,0xFE,0xE7,
 0x7F,0xFC,0x03,0x06,0x00,0x3C,0xC0,0xFF,0x3F,0xFC,0xFF,0x83,0xFE,0x2F,0x54,0xC1,
 0x16,0x94,0x43,0x15,0xF8,0xBF,0xC2,0xFF,0x3F,0xFC,0xFF,0x03,0x3C,0x00};

static const char zico12frun135[] = {
 0x01,0x04,0x0C,0x00,0x0C,0x00,0x00,0x00,0x3C,0x00,0x00,0x40,0x1C,0xE7,0x00,0xC6,
 0x10,0xC6,0x60,0x80,0x1F,0xFC,0xE3,0x7F,0xFE,0xF7,0xFF,0xFF,0xEF,0x7F,0xFE,0xC7,
 0x3F,0xF8,0x01,0x06,0x00,0x05,0x78,0x15,0xA8,0x57,0xC1,0x7A,0x15,0xD4,0x43,0x15,
 0x54,0xC1,0x17,0x54,0xAD,0x43,0xD5,0x2A,0x54,0x2D,0x50,0x00};

#define MAX_tvnPICS 4

typedef struct TRunImg {
  AlgTTC0
  struct TRunImg *next;
  TTreeVNode *tvn;
  TDirListObjNy *ObjNy;
  int idx;
} TRunImg;


static TPic pics[MAX_tvnPICS];

static TgTimer *TimerRun;

static TRunImg *runimg;


static bool chk_tvn_Ok(TTreeVNode *tvn) {
	return (ttc_chk_is_wtyp(tvn, wtyp_TTreeVFileNode) && tvn->tv);
}

static void TRunImg_OnTimer(TgTimer *gti) {
  TRunImg *rimg, *rimg_next;
  TTreeVNode *tvn;

	if (!runimg) {
	    Tfree(TimerRun);
	    TimerRun = NULL;
	    return;
	}

	rimg = runimg;
	while (rimg) {
	    rimg_next = rimg->next;
	    if (rimg->idx >= MAX_tvnPICS) rimg->idx = 0;
	    tvn = rimg->tvn;
	    if (chk_tvn_Ok(tvn) && (tvn->ttc == (TTC0*)rimg)) {
		TTreeVNode *tvnx = tvn;
loop:
		tvnx = TTreeVNode_get_prev_child(tvnx);
		if (tvnx && (tvnx->st & tvnSt_Expanded)) goto loop;

		if (!tvnx) {
		    DrawPic(tvn->tv->WinID, tvn->pic, tvn->left+1, tvn->top+1);
		    DrawPic(tvn->tv->WinID, &pics[rimg->idx], tvn->left+5, tvn->top+5);
		    rimg->idx++;
		}
	    } else {
		Tfree(rimg);
	    }
	    rimg = rimg_next;
	}
}


static bool TRunImg__Destroy(TRunImg *self) {
  TRunImg **pprimg;

	pprimg = &runimg;
	while (*pprimg && (*pprimg != self)) pprimg = &(*pprimg)->next;
	if (*pprimg) *pprimg = self->next;

	return False;
}

void TRunImg_Add(TTreeVNode *tvn, TDirListObjNy *ObjNy) {
  TRunImg *self, **pprimg;

	if (!chk_tvn_Ok(tvn) || tvn->ttc) return;

	self = runimg;
	while (self && (self->tvn != tvn)) self = self->next;
	if (self) {
	    if (ObjNy) self->ObjNy = ObjNy;
	    return;
	}

	self = ttc_create(sizeof(TRunImg), wtyp_USER);
	self->destroy = (TDESTROY)TRunImg__Destroy;
	tvn->ttc = (void*)self;
	self->tvn = tvn;
	self->ObjNy = ObjNy;
	pprimg = &runimg;
	while (*pprimg) pprimg = &(*pprimg)->next;
	*pprimg = self;

	if (!TimerRun) TimerRun = gTimerX_Add(Timer_Enabled, 66, TRunImg_OnTimer);
}

void TRunImg_Del(TTreeVNode *tvn) {
	if (!chk_tvn_Ok(tvn) || !tvn->ttc) return;
	ppTfree(&tvn->ttc);
	TTreeVFile_DoPaintNode(tvn->tv, tvn, tvnfu_PaintNew);
}

void TRunImg_Del_ObjNy(TDirListObjNy *ObjNy) {
  TRunImg *rimg = runimg;
	while (rimg && (rimg->ObjNy != ObjNy)) rimg = rimg->next;
	if (rimg) TRunImg_Del(rimg->tvn);
}

void init_tvImgRun() {
	CreateAHicoPic(zico12frun000, &pics[0]);
	CreateAHicoPic(zico12frun045, &pics[1]);
	CreateAHicoPic(zico12frun090, &pics[2]);
	CreateAHicoPic(zico12frun135, &pics[3]);
}
