/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldhex.h"
#include "tpixmap.h"
#include "tdirlist.h"

char *InFile;

/* ---------------------- LoadFile ----------------------------------- */
void LoadFile(TFormMain *self, char *pfn) {
  char *fn, *fnNoPasswd;
  TpcIni *ini;

	if (!pfn || !*pfn) return;

	fn = path_rel2abs(pfn, NULL);
#ifdef _OS_WIN32
	char *tfn = NT4_GetLongPathName(fn);
	if (tfn) {xfree(fn); fn = tfn;}
#endif

	if (THexEdit_LoadFile(self->HexEdit, fn)) {

	    SetCaption((TWins*)self->edOffset, "0x00000000");

	    fnNoPasswd = get_DisplayFileStr(fn, NULL);
	    SetCaption((TWins*)self, fnNoPasswd);

	    int dupi = -1;
	    char *pc0;
	    char line[32];
	    char *itm[REOPEN_ITEMS];

	    Fill0(itm, sizeof(itm));

	    ini = get_appIni();
	    pc0 = line;
	    ppcmove(&pc0, "LastFile");
	    int idx = 1;
	    for (int i=0; i<REOPEN_ITEMS; i++) {
		inttopc(pc0, i);
		const char *s = TpcIni_RD(ini, APPNAME, line);
		if (!s) break;
		
		TglobPath *gp = pctogp(s);
		CryptMini16c_enc_reg(gp->passwd);
		char *fnt = get_DisplayFileStr(NULL, gp);
		gpFree(gp);
		if (pccmp(fnNoPasswd, fnt)) {
		    dupi = i;
		} else {
		    if (idx < REOPEN_ITEMS) itm[idx] = pcdup(s);
		    idx++;
		}
		xfree(fnt);
	    }

	    if (dupi) {
		TglobPath *gp = pctogp(fn);
		char *pw = CryptMini16c_dec(gp->passwd);
		if (pw) {
		    xfree(gp->passwd);
		    gp->passwd = pw;
		}
		itm[0] = gptopc(gp, gptfu_PasswdTxt);
		gpFree(gp);

		for (int i=0; i<REOPEN_ITEMS; i++) {
		    if (!itm[i]) break;
		    inttopc(pc0, i);
		    TpcIni_WR(ini, APPNAME, line, itm[i]);
		}
		if (!appDat.appIni) TpcIni_Save(ini);
	    }

	    for (int i=0; i<REOPEN_ITEMS; i++) xfree(itm[i]); 
	    free_appIni(ini);

	    xfree(fnNoPasswd);
	}
	xfree(fn);
}

static void THexEdit_OnReadEnd(THexEdit *self) {
  dword IOfkt;
  TFormMain *form = (TFormMain*)self->FormStruct;
  char line[32];

	IOfkt = THexEdit_GetIOfkt(self);
	SetEnabled((TWins*)form->cbEdit, (IOfkt & IOfkt_CanWrite) != 0);
	SetEnabled((TWins*)form->cbInsert, (IOfkt & IOfkt_CanInsert) != 0);
	hex64to0xpc(line, THexEdit_GetSizeAbs(form->HexEdit), 8);
	TEdit_SetCaption(form->edSize, line);
}

static void SetEnSave(TFormMain *form) {
  bool en = THexEdit_GetChanged(form->HexEdit);

	SetEnabled((TWins*)form->mbSave, en);
	form->miSave->fu |= miNotEnabled;
	if (en) form->miSave->fu &= ~miNotEnabled;
}

static void THexEdit_OnWriteEnd(THexEdit *self) {
	SetEnSave((TFormMain*)self->FormStruct);
}

static void THexEdit_OnDataLoad(THexEdit *self, bool eof) {
#if USE_SB2_COUNT
	miSBx_Sync((TFormMain*)self->FormStruct);
#endif /* USE_SB2_COUNT */
}

static int edOffset_GetNum(TEdit *self) {
  int result;
  char *pc;

	pc = self->vds.s;
	if (!pc) return 0;
	if ((*pc == '0') && (pc[1] == 'x')) {
	    pc += 2;
	} else if (*pc == '$') {
	    pc++;
	}
	result = 0;
	while (*pc) {
	    pc++;
	    result++;
	}
	if (result > 16) result = 16;
	return result;
}

static void THexEdit_OnSetOffset(THexEdit *self, int64_t Offset) {
  TFormMain *form = (TFormMain*)self->FormStruct;
  int num;
  char line[24];

	num = edOffset_GetNum(form->edOffset);
	if (num < 4) num = 4;

	hex64to0xpc(line, Offset, num);
	TEdit_SetCaption(form->edOffset, line);
}

static void THexEdit_OnChange(THexEdit *self, int fu, TKeyS *KeyS, Tlgs *lgs, int xa, int ya, int xe, int ye) {
  TFormMain *form = (TFormMain*)self->FormStruct;

	TUnRedo_AddLine(self->UnRedo, fu, KeyS, lgs, xa, ya, xe, ye);
//	SetEnabled((TWins*)form->mbSave, True);
//	form->miSave->fu &= ~miNotEnabled;
	Chk_UnRedo(form);
	SetEnSave(form);
}

#if USE_SB2_COUNT
static void VSB2_OnCount(TScrollBar *self, int Count) {
  TFormMain *form = (TFormMain*)self->FormStruct;

	if (THexEdit_SetOffset(form->HexEdit, (int64)Count << form->VSB2SegShift)) {
	    Refresh((TWins*)form->HexEdit);
	}
}
#endif

static void cbInsert_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TFormMain *form = (TFormMain*)self->FormStruct;
	THexEdit_SetInsertMode(form->HexEdit, self->Checked);
}


static void cbEdit_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TFormMain *form = (TFormMain*)self->FormStruct;
	THexEdit_SetEditMode(form->HexEdit, self->Checked);
}



static bool edOffset_DoCharAllow(TEdit *self, TKeyEvent *KeyEv, int idx) {
  int hex, lg;
  char *pc, *tpc;
  bool result;

	hex = 0;
	tpc = NULL;
	pc = self->vds.s;

	if (self->vds.SelA != self->vds.SelE) {
	    tpc = pcdup(pc);
	    lg = self->vds.lg;
	    ppclg_delsel(&tpc, &lg, self->vds.SelA, self->vds.SelE);
	    if (self->vds.SelA < self->vds.SelE) {
		idx = self->vds.SelA;
	    } else {
		idx = self->vds.SelE;
	    }
	    if (idx < 0) idx = 0;
	    pc = tpc;
	}

	if (pc) {
	    if ((*pc == '0') && (pc[1] == 'x')) {
		hex = 2;
	    } else if (*pc == '$') {
		hex = 1;
	    }
	}
	xfree(tpc);

	result = (((hex <= idx) && (KeyEv->keysym >= '0') && (KeyEv->keysym <= '9')) ||
		  (hex && (hex <= idx) && 
			  (((KeyEv->keysym >= 'a') && (KeyEv->keysym <= 'f')) ||
			   ((KeyEv->keysym >= 'A') && (KeyEv->keysym <= 'F')))) ||
		  (!hex && (idx == 1) && (KeyEv->keysym == 'x')) || 
		  (!hex && (idx == 0) && (KeyEv->keysym == '$')));
	return result;
}

static void edOffset_OnChange(TEdit *self) {
  TFormMain *form;
  int64_t off;
  const char *pc;

	form = (TFormMain*)self->FormStruct;
	pc = GetCaption_pc((TWins*)self);
	off = pctoint64(pc);
	off &= 0xfffffffffffffff0;
	THexEdit_SetOffset(form->HexEdit, off);
}

static void buOffxx_OnMouseDownRep(TRepButton *self) {
  TFormMain *form;
  const char *pc;
  int64_t off;

	form = (TFormMain*)self->FormStruct;
	pc = GetCaption_pc((TWins*)form->edOffset);
	off = pctoint64(pc);
	off += self->tag * 0x200;
	if (off <= 0) off = 0;
	off &= 0xfffffffffffffff0;
	THexEdit_SetOffset(form->HexEdit, off);
}


/*_____________________________ ErrorMsg ________________________________*/

static void Global_OnFileMsg(TFileMsg *fmsg) {

// file_msg_debug(fmsg, NULL);

	if ((fmsg->art == fmArt_errLogin) || (fmsg->art == fmArt_AlgErr)) {
	    default_Global_OnFileMsg(fmsg);

/*	} else if (fmsg->art == fmArt_strg) {
	    OnIO(fmsg);

	} else if (fmsg->art == fmArt_DataIO) {

	    OnIO(fmsg); */
	}
}

/* ---------------------- AppStart ----------------------------------- */
static void OnAppStart() {
  TFormMain *self = (TFormMain*)appDat.obj;

	LoadFile(self, InFile);

	TpcIni_Save(appDat.appIni);
	ppTfree(&appDat.appIni);

	TChkBox_SetChecked_ev(self->cbEdit, True);
//	Set_Focus((TWins*)self->HexEdit);
}

/* ------------------------------------------------------------------- */
const char *Help_Msg = {
  "\n"
  "Usage: "APPNAME"\n"
  "\n"
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: "APPNAME" version "ALD_VERSION"  (build %d  %s)\n"
};

#ifndef _OS_WIN32
static char zico16aldhex[] = {
 0x06,0x0E,0x10,0x00,0x10,0x00,0x00,0x00,0x97,0x00,0x03,0x19,0x85,0x21,0xC6,0x21,
 0x07,0x2A,0xC9,0x2A,0x44,0x31,0x07,0x3A,0x0C,0x63,0x08,0x7B,0xCE,0x7B,0xD2,0x9C,
 0x14,0xA5,0xDA,0xDE,0x9E,0xF7,0xD4,0x03,0x10,0x00,0x01,0x10,0x00,0x41,0x42,0xA5,
 0x17,0x77,0x71,0x17,0x77,0x31,0x54,0xA4,0xE1,0x1E,0xEE,0xE1,0xAE,0x36,0x45,0xA2,
 0xAE,0xDF,0x58,0xE6,0x48,0xA8,0x87,0x7B,0xA0,0x7F,0x0D,0xE6,0xA6,0x09,0x68,0x78,
 0x78,0xB8,0x23,0x77,0x9A,0x07,0xA8,0x87,0x7B,0xB8,0x2F,0xAB,0xB9,0x07,0xE8,0xBA,
 0x20,0x5F,0xBA,0x04,0xE8,0x3A,0xEC,0xBA,0x04,0xA8,0x3A,0x18,0xEB,0x4F,0x80,0xFA,
 0xF9,0x39,0x5E,0x8F,0x87,0x87,0x07,0xA8,0x9F,0x9F,0x5F,0xBA,0x13,0xEE,0x01,0xEA,
 0x78,0xE2,0x77,0xE8,0x36,0xE1,0x1E,0xA0,0x7E,0x7E,0x8E,0xF7,0x78,0x78,0x78,0x80,
 0xDA,0xBA,0x04,0xA8,0xED,0x45,0x00};
#endif

const int build_v = ALD_BUILD_V;
const char *build_date = __DATE__;

int main (int argc, char **argv) {
  int i;
  char *val;
  const char *cpc;
  TFormMain *self;
  TpcIni *ini;
  bool help = False;
  bool err = False;

#ifdef _OS_WIN32
	cmdUTF8korr(&argc, &argv);
#endif

	InFile = NULL;
	i = 1;
	while (!err && (i < argc)) {
	    val = argv[i];
	    if (*val == '-') {
		val++;
		switch (*val) {
		case 'h':
		    help = True;
		    break;
		case '-':
		    val++;
		    if (pcncmp(val, "help", 4)) {
			help = True;
		    } else err = True;
		    break;
		default: 
		    err = True;
		}
	    } else {
		InFile = val;
	    }
	    i++;
	}

	if (help || err) {
	    printf(Help_Msg, build_v, build_date);
	    exit(err);
	}

	initXapp(0, argc, argv);
	appDat.inifile_app = pcadd(appDat.ALDini_dir, "/"APPNAME".ini");
//	ThDirList_Reg(2, &hdirlist_ld_File, &hdirlist_ld_ftp);
#ifndef _OS_WIN32
	ThNetLineBufLoader_Reg(1, &hNetLineBufLd_ftp);
#endif
#ifdef USE_DECOR
	appXDat.DefaultButtonDecor = TDecor_button_w3b3_Create();
//	appXDat.DefaultWinsDecor = TDecor_w3b3_Create(&TDecoBuR3B3_up_Tab, &TDecoBuR3B3_down_Tab);
	DefaultRepButtonDecor = TDecor_w3b3_Create(&TDecoBuR3B3_up_Tab, &TDecoBuR3B3_down_Tab);
#endif
//	ThImageRD_Reg(1, &hImageRD_xpm);
	hGetFileHeadInfo = &hGetFileHeadInfo_buildin;

	self = TForm_CreatParamAlg(NULL, sizeof(TFormMain));
	appDat.obj = self;
#ifndef _OS_WIN32
	self->Icon = GetDeskIcon_z(ddkt_Application, 16, APPNAME, 
				    gdifu_noZoom, zico16aldhex);
#endif

	ini = TpcIni_Create(appDat.inifile_app);
	appDat.appIni = ini;

	self->IconName = APPNAME;
	self->height = TpcIni_RD_int(ini, APPNAME, "Height", 560);
	self->width = TpcIni_RD_int(ini, APPNAME, "Width", 485);
	self->min_width = 380;
	self->min_height = 100;
//	self->OnResize = TFormMain_OnResize;
//	appDat.obj = self;
	SetCaption((TWins*)self, APPNAME);

	self->PanelTop = TWins_CreateBA((TWins*)self, 0, 0, 200, 21, mBorder | mBorderL, alTop);
	self->PanelTop2 = TWins_CreateBA((TWins*)self, 0, 0, 200, 39, mBorder | mBorderL, alTop);

	self->pnlMiddle = TWins_CreateBA((TWins*)self, 0, 0, 32, 32, 0, alClient);
// ******* Menu *******
	MenuFile_Create(self);
	MenuSetting_Create(self);
	MenuHelp_Create(self);
	MenuSave_Create(self);
	MenuUnRedo_Create(self);
	TMenuButton_SetPosition(self->mbFile);

	self->lblOffset = TWins_CreateLabel(self->PanelTop2, 2, 3, 42, 16, "Offset", mCaptionLeft, 0);
	self->edOffset = TEdit_CreateS(self->PanelTop2, 44, 2, 108, 17, "0x00000000");
	self->edOffset->DoCharAllow = edOffset_DoCharAllow;
	self->edOffset->OnChange = edOffset_OnChange;

	self->lblSize = TWins_CreateLabel(self->PanelTop2, 2, 21, 42, 16, "Size:", mCaptionLeft, 0);
	self->edSize = TEdit_CreateS(self->PanelTop2, 44, 20, 108, 17, "0x00000000");
	self->edSize->ReadOnly = True;

	self->buOffPosPos = TRepButton_CreateS(self->PanelTop2, 154, 2, 20, 17, "<<");
	self->buOffPos    = TRepButton_CreateS(self->PanelTop2, 175, 2, 20, 17, "<");
	self->buOffNeg    = TRepButton_CreateS(self->PanelTop2, 196, 2, 20, 17, ">");
	self->buOffNegNeg = TRepButton_CreateS(self->PanelTop2, 217, 2, 20, 17, ">>");
	self->buOffPosPos->tag = -8;
	self->buOffPosPos->OnMouseDownRep = buOffxx_OnMouseDownRep;
	self->buOffPos->tag = -1;
	self->buOffPos->OnMouseDownRep = buOffxx_OnMouseDownRep;
	self->buOffNeg->tag = 1;
	self->buOffNeg->OnMouseDownRep = buOffxx_OnMouseDownRep;
	self->buOffNegNeg->tag = 8;
	self->buOffNegNeg->OnMouseDownRep = buOffxx_OnMouseDownRep;

	self->cbEdit = TChkBox_CreateS(self->PanelTop2, 260, 3, 44, 16, "Edit");
	self->cbEdit->OnClick = cbEdit_OnClick;

	self->cbInsert = TChkBox_CreateS(self->PanelTop2, 310, 3, 54, 16, "Insert");
	self->cbInsert->OnClick = cbInsert_OnClick;

	self->HexEdit = THexEdit_CreateA(self->pnlMiddle, 0, 0, 200, 200, alClient);
	self->HexEdit->UnRedo = TUnRedo_Create(200);
	self->HexEdit->OnChange = THexEdit_OnChange;
	self->HexEdit->OnReadEnd = THexEdit_OnReadEnd;
	self->HexEdit->OnWriteEnd = THexEdit_OnWriteEnd;
	self->HexEdit->OnDataLoad = THexEdit_OnDataLoad;
	self->HexEdit->OnSetOffset = THexEdit_OnSetOffset;

#if USE_SB2_COUNT
	self->VSB2 = TScrollBarVert_CreateP(self->pnlMiddle);
	self->VSB2->width = 16;
	self->VSB2->LargeChange = 16;
	self->VSB2->Align = alRight;
	SetBorder((TWins*)self->VSB2, mBorder|mBorderL|mBorderNoRigth);
	self->VSB2->OnCount = VSB2_OnCount;
	self->VSB2SegShift = 10;
#endif

	self->PanelBottom = TWins_CreateBA((TWins*)self, 0, 0, 200, 38, 0, alBottom);

#ifdef _OS_WIN32
	cpc = TpcIni_RD_pc(ini, APPNAME, "Font", "-*-Courier New-medium-r-normal--12-*-*-*-*-*-*-1");
#else
	cpc = TpcIni_RD_pc(ini, APPNAME, "Font", "-misc-fixed-medium-r-semicondensed--13-120-75-75-*-*-*-1");
#endif
	SetFont((TWins*)self->edOffset, cpc);
	SetFont((TWins*)self->edSize, cpc);
	SetFont((TWins*)self->HexEdit, cpc);




	THexEdit_SetMaxSegData(self->HexEdit, 0x00100000);


	Glob_OnFileMsg = Global_OnFileMsg;
	appError_ev = default_Error_ev;

	appXDat.OnAppStart = OnAppStart;
//	Set_Focus((TWins*)self->HexEdit);

	AppRun();
	return 0;
}

