/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldhex.h"
#include "tpixmap.h"
#include "cicons.h"

/* ______________________ File Ev ____________________________________ */
static void menuFile_OnMenuOpen(TMenu *self) {
  TFormMain *form = (TFormMain*)self->FormStruct;
  dword IOfkt;

	form->miSaveAs->fu |= miNotEnabled;

	IOfkt = THexEdit_GetIOfkt(form->HexEdit);
// printf("%s(%d): 0x%04x\n", FL, IOfkt);
	if (IOfkt & IOfkt_CanSaveAs) form->miSaveAs->fu &= ~miNotEnabled;
}

static void menuReopen_OnMenuOpen(TMenu *self) {
  TFormMain *form = (TFormMain*)self->FormStruct;
  TpcIni *pcIni;
  int i, fileart;
  char *fnt;
  const char *fn;
  char line[32];
  TMenuItem *mi;
  TglobPath *gp;

	TMenu_Cls(form->menuReopen);
	pcIni = TpcIni_Create(appDat.inifile_app);
	for (i=0; i<REOPEN_ITEMS; i++) {
	    snprintf(line, 32, "LastFile%d", i);
	    fn = TpcIni_RD(pcIni, APPNAME, line);
	    if (!fn) break;
	    gp = pctogp(fn);
	    gp_CryptMini16c_enc(gp);

	    fileart = stft_ErrNoFile;
	    if ((gp->fu & gpi_Local)) {
		Tstat64 *st = file_stat_gp(gp, fsFollowLink|fsNoFileName|fsNoLinkName|fsGetFileType|fsGetFileTypeHead);
		if (st) fileart = st->FileType;
		xfree(st);
	    } else {
		fileart = get_file_stft_ext(fn, NULL, NULL);
	    }

	    if ((fileart != stft_ErrNoFile) && (fileart != stft_Folder)) {
		fnt = get_DisplayFileStr(NULL, gp);
		char *tmp = path_cut(fnt, 74);
		xfree(fnt);
		mi = TMenu_AddItem(form->menuReopen, tmp, miDisable_s2, NULL);
		xfree(tmp);
		mi->s2 = gptopc(gp, gptfu_PasswdTxt);
		mi->pic = get_fileIcon(fileart);
	    }
	    gpFree(gp);
	}
	Tfree(pcIni);
}


static void File_Open(TFormMain *form, int fu) {
  const char *FileName;
  char *s=NULL;

	if (!form->File_Dlg) {
	    form->File_Dlg = Create_File_Dlg((TWins*)form);
	    if (TImageRD_AllFiletyp) file_dlg_InitAzPic(form->File_Dlg);
	}

	FileName = THexEdit_GetFileName(form->HexEdit);
	if (FileName) {
	    s = GetFilePath(FileName);
	    if (s) File_Dlg_SetPath(form->File_Dlg, s);
	    ppcfree(&s);
	}
	
	if (File_Dlg_Execute(form->File_Dlg)) {
	    File_Dlg_GetEdFileName(form->File_Dlg, &s);
	    LoadFile(form, s);
	    xfree(s);
	}
}

static void mFile_Open(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormMain *form = (TFormMain*)self->FormStruct;
	File_Open(form, 0x02);
}

static void menuReopen_OnClick(TMenu *self, long KeyState, bool DblClick) {
  TFormMain *form = (TFormMain*)self->FormStruct;
  char *fn;
	if (form->menuReopen->SelItem) {
	    fn = form->menuReopen->SelItem->s2;
	    if (!fn) fn = form->menuReopen->SelItem->s;
	    LoadFile(form, fn);
	}
}

static void mFile_Save(TMenu *self, long KeyState, TMenuItem *smi) {
	File_Save((TFormMain*)self->FormStruct);
}

static void mFile_SaveAs(TMenu *self, long KeyState, TMenuItem *smi) {
	File_SaveAs((TFormMain*)self->FormStruct);
}

static void mFile_SaveSize(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormMain *form = (TFormMain*)self->FormStruct;
  TpcIni *Ini;

	Ini = TpcIni_Create(appDat.inifile_app);
	TpcIni_WR_int(Ini, APPNAME, "Width", form->width);
	TpcIni_WR_int(Ini, APPNAME, "Height", form->height);
	TpcIni_SaveAndFree(Ini);
}

static void mFile_Exit(TMenu *self, long KeyState, TMenuItem *smi) {
  Close();
}
/* ______________________ Create _____________________________________ */
void MenuFile_Create(TFormMain *self) {
  TMenuItem *mi;

	self->mbFile = TMenuButton_Create(self->PanelTop, (TMenuButton*)self);
	self->mbFile->top = 2;
	self->mbFile->left = 1;
	SetCaption((TWins*)self->mbFile, "File");
	Set_Parent((TWins*)self->mbFile, self->PanelTop);

	self->menuFile = TMenu_Create((TWins*)self);
	self->menuFile->OnMenuOpen = menuFile_OnMenuOpen;
	self->mbFile->menu = self->menuFile;

	mi = TMenu_AddItem(self->menuFile, "Open...", 0, mFile_Open);
	mi->pic = get_pic16_fileopen(False);
	mi = TMenu_AddItem(self->menuFile, "Reopen", 0, NULL);
		self->menuReopen = TMenu_Create((TWins*)self);
		self->menuReopen->OnClick = menuReopen_OnClick;
		self->menuReopen->OnMenuOpen = menuReopen_OnMenuOpen;
		mi->next_menu = self->menuReopen;

	TMenu_AddItem(self->menuFile, "-", 0, NULL);
	self->miSave = TMenu_AddItem(self->menuFile, "Save", miNotEnabled, mFile_Save);
	self->miSave->pic = get_pic16_filesave(False);
	self->miSaveAs = TMenu_AddItem(self->menuFile, "Save As...", 0, mFile_SaveAs);
	self->miSaveAs->pic = get_pic16_filesave(False);


	TMenu_AddItem(self->menuFile, "-", 0, NULL);
	self->miSaveSize = TMenu_AddItem(self->menuFile, "Save Size", 0, mFile_SaveSize);
	self->miSaveSize->pic = get_pic16_filesave(False);

	TMenu_AddItem(self->menuFile, "-", 0, NULL);
	mi = TMenu_AddItem(self->menuFile, "Exit", 0, mFile_Exit);
	mi->pic = GetZIcon(zico16exit);
}

