/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldpic.h"

typedef struct {
  const byte kr, kg, kb;
} Tkrgb;

const Tkrgb Tkrgb_arr[pmxWR_bppMAXnr + 1] = {
  { 0,  0,  0}, //default
  { 5,  5,  5}, //pmxWR_bpp6
  { 5,  9,  5}, //pmxWR_bpp7
  { 9,  9,  5}, //pmxWR_bpp8
  { 9,  9,  9}, //pmxWR_bpp9
  { 9, 17,  9}, //pmxWR_bpp10
  {17, 17, 17}, //pmxWR_bpp12
  {33, 33, 33}, //pmxWR_bpp15
  {33, 65, 33}, //pmxWR_bpp16
  {65, 65, 65}  //pmxWR_bpp18
};

void ALDpic_Chk_Mask(TMainForm *fmain, TPixMap *pxm) {
	if ((TFormAlphaClr_GetModus_IsAkt(fmain->fAlphaClr) & IS_ALPHA_CLR)) {
	    if (pxm->stat & pmxStat_HandleMask) {
		pxm->stat &= ~(pmxStat_XMaskPixAkt | pmxStat_HandleMask);
		if (pxm->maskID) XFreePixmap(appXDat.dpy, pxm->maskID);
		pxm->maskID = 0;
		FreeGC(pxm->mask_gc);
		pxm->mask_gc = NULL;
		pxm->gcv.clip_mask = 0;
		pxm->gcv.clip_x_origin = 0;
		pxm->gcv.clip_x_origin = 0;
	    }
	} else {
	    if ((pxm->stat & pmxStat_has_alpha_tgl) &&
		!(pxm->stat & pmxStat_HandleMask)) {
		if (!pxm->mask_data) alpha2bitmask((TImage*)pxm);
		pxm->stat |= pmxStat_HandleMask;
	    }
	}
}

void ALDpic_Chk_AutoMask(TMainForm *fmain, int fu) {
  TPixMap *pxm = fmain->pxm;

	if (!pxm || (fmain->org_stat & pmxStat_has_alpha)) return;
        if (fmain->bmp_AutoMask) {
	    if (pxm->stat & pmxStat_has_alpha) return;
	    TImage_ColorToAlphaData((TImage*)pxm, *((dword*)(pxm->data + (pxm->bytes_per_line * (pxm->height-1)))));
	    alpha_data8_set((TImage*)pxm);
	    alpha_bm_stat((TImage*)pxm);
	    xfree(pxm->rgb565_alpha_data);
	    pxm->rgb565_alpha_data = NULL;
	    if (fu) ALDpic_resize_pxm(fmain);
	} else {
	    if (!(pxm->stat & pmxStat_has_alpha)) return;
	    alpha_data8_clr((TImage*)pxm);
	    if (fu) ALDpic_resize_pxm(fmain);
	}
}

void ALDpic_Chk_dup_paint_pxm(TMainForm *fmain) {
  TPixMap *pxm;
  dword t;

	ALDpic_Chk_vioc_pxm(fmain);
	t = GetTime_us();
	fmain->paintTime = 0;
	pxm = ALDpic_pxm_get(fmain, ib_paintbase_pxm);
	if (!pxm) return;

	if (appXDat.RootW_depth > 16) return;
	if ((pxm->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp) return;


	if (fmain->paint_pxm && ((pxm->width  != fmain->paint_pxm->width) ||
				(pxm->height != fmain->paint_pxm->height))) {
	    ALDpic_pxm_free(fmain, ib_paint_pxm);
	}

	if (fmain->paint_pxm) {
	    if (fmain->RePaint_pxm) {
		TImage_rgb888move565((TImage*)pxm, (TImage*)fmain->paint_pxm);
		if (!(TFormAlphaClr_GetModus_IsAkt(fmain->fAlphaClr) & IS_ALPHA_CLR) &&
		    (pxm->stat & pmxStat_has_alpha_tgl))
		{
		    alpha2bitmask((TImage*)fmain->paint_pxm);
		} 
	    }
	} else {
	    fmain->paint_pxm = (TPixMap*)TImage_rgb888to565((TImage*)pxm);
	}
	fmain->RePaint_pxm = False;
	fmain->paintTime = GetCurTimeDiv_us(t);
}


static bool Set_vio_pxm(TMainForm *fmain, TPixMap *pxm) {
	if (!fmain->vio_pxm) {
#ifndef _OS_WIN32
	    dword freemem = get_FreeMem(fmain, (pxm->width * pxm->height) >> 8);
	    if (!freemem) {
		clr_vioc_pxm_para(fmain);
		MsgBox_Error((TWins*)fmain, "low mem\nDisable color function");
		return false;
	    }
#endif
	    fmain->vio_pxm = (TPixMap*)TImage_Create_dup_struct((TImage*)pxm, dupfu_init_imgmem);
	}
	return fmain->vio_pxm != NULL;
}

void ALDpic_Chk_vioc_pxm(TMainForm *fmain) {
  TVideo_IOC *vioc = fmain->ioc_colorRGB;
  TVideo_Color_Korr *tvckr = fmain->ioc_colorRGB->tvckr;
  int i, any_akt, AlphaClrModus;
  dword t, AlphaClrColor;
  TPixMap *pxm;
  bool RePaint_pxm, bQuant, bIOdataOk;

	fmain->viocTime = 0;
	pxm = ALDpic_pxm_get(fmain, ib_base_pxm);
	if (!pxm || !tvckr) return;

	t = GetTime_us();
	RePaint_pxm = False;
	bQuant = fmain->fpal->cbColQuantOn->Checked;
	if (fmain->QuantMk != bQuant) {
	    fmain->QuantMk = bQuant;
	    RePaint_pxm = true;
	} else if (bQuant && !fmain->QuantOk) {
	    RePaint_pxm = true;
	}

	AlphaClrModus = TFormAlphaClr_GetModus_IsAkt(fmain->fAlphaClr);
	if (fmain->AlphaClrModus != AlphaClrModus) {
	    fmain->AlphaClrModus = AlphaClrModus;
	    RePaint_pxm = true;
// printf("%s(%d): \n", FL);
	}

	if (AlphaClrModus) {
	    AlphaClrColor = TFormAlphaClr_GetColor(fmain->fAlphaClr);
	    if (fmain->AlphaClrColor != AlphaClrColor) {
		fmain->AlphaClrColor = AlphaClrColor;
		RePaint_pxm = true;
	    }
	}


	if (tvckr->Brightness != fmain->pnlBrightness->val) {
	    tvckr->Brightness = fmain->pnlBrightness->val;
	    RePaint_pxm = True;
	}
	if (tvckr->Contrast != fmain->pnlContrast->val) {
	    tvckr->Contrast = fmain->pnlContrast->val;
	    RePaint_pxm = True;
	}
	if (fmain->gray) {
	    if (tvckr->ColorSat != -256) {
		tvckr->ColorSat = -256;
		RePaint_pxm = True;
	    }
	} else if (tvckr->ColorSat != fmain->pnlColorSat->val) {
	    tvckr->ColorSat = fmain->pnlColorSat->val;
	    RePaint_pxm = True;
	}
	if (tvckr->Red != fmain->pnlRed->val) {
	    tvckr->Red = fmain->pnlRed->val;
	    RePaint_pxm = True;
	}
	if (tvckr->Green != fmain->pnlGreen->val) {
	    tvckr->Green = fmain->pnlGreen->val;
	    RePaint_pxm = True;
	}
	if (tvckr->Blue != fmain->pnlBlue->val) {
	    tvckr->Blue = fmain->pnlBlue->val;
	    RePaint_pxm = True;
	}

	if (vioc->iocfu != fmain->iocfu) {
	    vioc->iocfu = fmain->iocfu;
	    RePaint_pxm = True;
	}

	if (fmain->paint_bpp != fmain->bpp) {
	    fmain->paint_bpp = fmain->bpp;
	    RePaint_pxm = True;
	}

	if (fmain->LastColorOrder != fmain->ColorOrder) {
	    fmain->LastColorOrder = fmain->ColorOrder;
	    RePaint_pxm = True;
	}

	any_akt = tvckr->Brightness | tvckr->Contrast | tvckr->ColorSat |
		  tvckr->Red | tvckr->Green | tvckr->Blue |
		  (vioc->iocfu & VIOCFU_NEGATIV);

	if (!fmain->bpp && !any_akt && !AlphaClrModus && !bQuant && 
	    !fmain->ColorOrder) 
	{
// printf("%s(%d): 0x%08x\n", FL, (int)fmain->vio_pxm);
	    if (fmain->vio_pxm) {
		ALDpic_pxm_free(fmain, ib_vio_pxm);
		fmain->RePaint_pxm = True;
	    }
	    ColorsQuant(fmain);
	    fmain->viocTime = GetCurTimeDiv_us(t);
	    return;
	}

	if (fmain->vio_pxm) {
	    if ((pxm->width != fmain->vio_pxm->width) ||
		(pxm->height != fmain->vio_pxm->height)) {
		ALDpic_pxm_free(fmain, ib_vio_pxm);
		RePaint_pxm = True;
	    }
	}

	bIOdataOk = false;
	if (!fmain->vio_pxm) {
	    if (!Set_vio_pxm(fmain, pxm)) return;
	    RePaint_pxm = true;
	}

	if (AlphaClrModus && RePaint_pxm) {
	    bIOdataOk = true;
	    pnmove(fmain->vio_pxm->data, pxm->data, pxm->bytes_per_line * pxm->height);
	    fmain->AlphaClrOk = false;
	    AlphaClr(fmain);
	}

	if (fmain->ColorOrder && RePaint_pxm) {
	    if (!bIOdataOk) {
		bIOdataOk = true;
		pnmove(fmain->vio_pxm->data, pxm->data, pxm->bytes_per_line * pxm->height);
	    }
	    TImage_ColorOrder((TImage*)fmain->vio_pxm, fmain->ColorOrder);
	}

	if (fmain->bpp || any_akt) {
	  if (RePaint_pxm) {

  TMediaFrame fio;
  TIOCVideoF iof;
  TIOCVideoF ret_iof;

	    Fill0(&fio, sizeof(TFrame_IO));
	    Fill0(&iof, sizeof(TIOCF));
	    Fill0(&ret_iof, sizeof(TIOCF));
	    fio.org.v.iof = &iof;
	    fio.ret.v.iof = &ret_iof;
	    iof.typ = DatType_TImage;
	    ret_iof.typ = DatType_TImage;
	    fio.org.v.t.img = (TImage*)pxm;
	    if (bIOdataOk) fio.org.v.t.img = (TImage*)fmain->vio_pxm;
	    fio.ret.v.t.img = (TImage*)fmain->vio_pxm;

	    i = (fmain->bpp & pmxWR_bppMask) >> 4;
	    if (i > pmxWR_bppMAXnr) i = 0;
	    tvckr->R_reduce = Tkrgb_arr[i].kr;
	    tvckr->G_reduce = Tkrgb_arr[i].kg;
	    tvckr->B_reduce = Tkrgb_arr[i].kb;

	    Video_IOC_SetColor(vioc, NULL);
	    Video_IOC_Get_Frame(vioc, &fio);
//	    Video_IOC_Conv(vioc);

	    bIOdataOk = true;
	  }
	}

	if (bQuant) {
// printf("%s(%d): Quant IOdataOk=%d RePaint_pxm=%d\n", FL, bIOdataOk, RePaint_pxm);
	    if (!bIOdataOk && RePaint_pxm) {
		pnmove(fmain->vio_pxm->data, pxm->data, pxm->bytes_per_line * pxm->height);
	    }
	}

	if (RePaint_pxm) {
// printf("%s(%d): RePaint_pxm\n", FL);
	    fmain->RePaint_pxm = true;
	    fmain->QuantOk = false;
	    fmain->vio_pxm->stat &= ~pmxStat_XPixAkt;
	}
	ColorsQuant(fmain);

	fmain->viocTime = GetCurTimeDiv_us(t);
}
