/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "image.h"

#ifdef MEM_CHK
static int LastMEM_CHK_Count;
static void AzMem() {
  int lMCt;  
  dword lMsize;
	MEM_CHK_Count(&lMCt, &lMsize);
	if (LastMEM_CHK_Count != lMCt) {
		LastMEM_CHK_Count = lMCt;
    printf("Debug %s: MEM_CHK_Count=%3d  size=%d\n", app_name, lMCt, lMsize);
	}
}
#else
 #define AzMem() 
#endif

static int read_extColMap(dword *quant_col_map, const char *val) {
  dword m, r, g, b;

	m = 0;
	while (m < 8) {
	    r = g = b = ppctoint(&val) & 0xff;
	    if ((*val == '.')||(*val == ',')) {
		val++;
		g = ppctoint(&val) & 0xff;
		if ((*val == '.')||(*val == ',')) {
		    val++;
		    b = ppctoint(&val) & 0xff;
		}
	    }
	    quant_col_map[m] = b|(g<<8)|(r<<16);
	    m++;
	    if (*val != ':') break; else val++;
	}
	return m;
}

static const char *Help_Msg = {
  "\n"
  "Usage: imgconv [options] file\n"
  "  input  *.xpm *.png"
#ifdef USE_JPEG
  " *.jpg"
#endif
#ifdef USE_BMP
  " *.bmp"
#endif
#ifdef USE_PNM
  " *.pnm *.ppm *.pam" 
#endif
  "\n"
  "  output *.xpm *.png *.jpg"
#ifdef USE_PNM
  " *.pnm *.ppm *.pam" 
#endif
  "\n"
  "\n"
  "  -o outfile           output file\n"
  "  -c=nn                color count (quantize colors)\n"
  "  -m=r.g.b             add default map color (max.8)\n"
  "                         -m=0.0.0:255.255.255 (black;white)\n"
  "                         or -m=0:255 (black;white)\n"
  "  -z=K                 zoom percent (max. 1..200%)\n"
  "  -Z=WWWxHHH           zoom pixel W*H (max. 1..200%)\n"
#ifdef USE_JPEG
  "  -q=nn                jpg output quality 1..100 (default 95)\n"
#endif
#ifdef USE_PNM
  "  --plain              *.ppm output txt plain\n"
#endif
  "  -h, --help           prints out this screen\n"
};

int main (int argc, char **argv) {
  TImage *img;
  int i;
  char *val, *fn;
  char *InFile, *OutFile;
  int ColorCount = 0;
  dword default_quant_col_map[8];
  int sum, default_quant_count;
  int jpg_quality = 0;
  dword WR_fu = 0;
  int ZoomProz = 0;
  int ZoomW = 0;
  int ZoomH = 0;
  TImage *rsz_img = NULL;
  int pnm_plain = 0;
  bool help = False;
  bool err = False;

	InFile = NULL;
	OutFile = NULL;
	default_quant_count = 0;
	i = 1;
	while (!err && !help && (i<argc)) {
	    val = argv[i];
	    if (*val == '-') {
		val++;
		switch (*val) {
		case 'c':
		    err = cmd_get_int(argc, argv, val, &i, &ColorCount);
		    break;
		case 'm':
		    val++;
		    if (*val == '=') val++;
		    default_quant_count = read_extColMap(default_quant_col_map, val);
		    break;
		case 'o':
		    val++;
		    if (*val == '=') {
			val++;
			OutFile = pcdup(val);
		    } else if (((i+1)<argc) && (*argv[i+1] != '-')) {
			i++;
			OutFile = pcdup(argv[i]);
		    } else err = True;
		    break;
		case 'q':
		    err = cmd_get_int(argc, argv, val, &i, &jpg_quality);
		    break;
		case 'z':
		    err = cmd_get_int(argc, argv, val, &i, &ZoomProz);
		    if (ZoomProz > 200) err = True;
		    break;
		case 'Z':
		    val++;
		    if (*val == '=') val++;
		    ZoomW = ppctoint((void*)&val);
		    if (*val == 'x') {
			val++;
			ZoomH = pctoint(val);
		    } else err = True;
		    break;
		case 'h':
		    help = True;
		    break;
		case '-':
		    val++;
 printf("%s(%d): %s\n", FL, val);
		    if (pcncmp(val, "help", 4)) {
			help = True;
#ifdef USE_PNM
		    } else if (pcncmp(val, "plain", 5)) {
			pnm_plain = fuWR_txtplain;
#endif
		    } else err = True;
		    break;
		default: 
		    err = True;
		}
	    } else {
		InFile = val;
	    }
	    i++;
	}

	err = err || !InFile;
	if (help || err) {
	    printf(Help_Msg);
	    exit(err);
	}

	ThImageRD_Reg(6, &hImageRD_xpm, &hImageRD_png, &hImageRD_jpeg, 
			 &hImageRD_gif, &hImageRD_bmp, &hImageRD_pnm);

	ThImageWR_Reg(4, &hImageWR_xpm, &hImageWR_png, &hImageWR_jpeg,
			 &hImageWR_pnm);

// printf("%s(%d): %s c=%d ecm=%d -> %s\n", FL, InFile, ColorCount, default_quant_count, OutFile);

	fn = path_rel2abs(InFile, NULL);
	img = TImage_Loader(fn, NULL, pxm_4b_24bpp);
	xfree(fn);
	if (!img) exit(1);

// int t = GetTime_us();
	sum = TImage_GetColorCount(img, pxgcc_RGB_ALL);
// printf("%s(%d): GetColorCount t=%dus\n", FL, GetCurTimeDiv_us(t));
	printf("IN: w=%d h=%d colors=%d", img->width, img->height, sum);

	if (!OutFile) {
	    printf("\n");
	    return 0;
	}

	if (ZoomProz) {
	    ZoomW = img->width * ZoomProz / 100;
	    if (ZoomW < 2) ZoomW = 2;
	    ZoomH = img->height * ZoomProz / 100;
	    if (ZoomH < 2) ZoomH = 2;
	}
	if (ZoomW && ZoomH) {
	    if (ZoomW < 2) ZoomW = 2;
	    if (ZoomH < 2) ZoomH = 2;
	    rsz_img = TImage_dup_new_size(img, ZoomW, ZoomH);
	    if (rsz_img) {
		Tfree(img);
		img = rsz_img;
  		sum = TImage_GetColorCount(img, pxgcc_RGB_ALL);
		printf(" --> w=%d h=%d colors=%d", img->width, img->height, sum);
	    }
	}
	printf("\n");



	if (sum > 0) {
	    img->cmap->default_quant_count = default_quant_count;
	    img->cmap->default_quant_col_map = (ic32*)default_quant_col_map;
	}

  ThImageLoader *hLD;
  FILE *F;
	fn = path_rel2abs(OutFile, NULL);
	F = TImageIO_GetStream(fn, NULL, 0, &hLD, True);
	if (!F) exit(1);

	if (ColorCount > 0) {

//    t = GetTime_us();
	    TImage_GetColorMap(img);
//    printf("%s(%d): t=%dus\n", FL, GetCurTimeDiv_us(t));

//    AzMem();
//    t = GetTime_us();
	    TImage_CutColorMap(img, ColorCount, 0);
//	    int n = TImage_CutColorMap(img, ColorCount, 0);
//    printf("%s(%d): t=%dus %d\n", FL, GetCurTimeDiv_us(t), n);
//    AzMem();



//    t = GetTime_us();
	    TImage_ReMapColor(img);
//    printf("%s(%d): ReMapColor t=%dus \n", FL, GetCurTimeDiv_us(t));
	}

	if (hLD->filetyp & filetyp_JPEG) WR_fu |= jpg_quality << 8;
	if (hLD->filetyp & filetyp_PNM)  WR_fu |= pnm_plain;
	
	img->out_gp = pctogp(fn);
	img->out_fu = WR_fu;
	img->out_fp = F;
	hLD->ioproc(img);
	fclose(img->out_fp);
	gpFree(img->out_gp);

 xfree(OutFile);
 xfree(fn);

 Tfree(img);
 AzMem();

	return 0;
}
