/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "aldpic.h"
#include "cicons.h"

/* ______________________ File Ev ____________________________________ */

static void menuFile_OnMenuOpen(TMenu *me) {
  TMainForm *fpic;

	if (!TImageWR_AllFiletyp) return;
	fpic = (TMainForm*)me->FormStruct;
	if (ALDpic_pxm_get(fpic, ib_paintbase_pxm)) {
	    fpic->miSaveAs->pic = get_pic16_filesave(False);
	    fpic->miSaveAs->fu &= ~miNotEnabled;
	} else {
	    fpic->miSaveAs->pic = get_pic16_filesave(True);
	    fpic->miSaveAs->fu |= miNotEnabled;
	}
}

void ALDpic_FileOpen(TMainForm *fpic, char **ppc) {
  char *s, *pc, *pcn;
  Tstat64 *st;
  TDirFileList *dfl;
  TNetErrRet ErrRet;

	s = *ppc;
	net_err_init(&ErrRet);
	st = net_stat(s, &ErrRet, fsFollowLink);

	if (ErrRet.errFlag & NetErrFlag_NoDev) {
	    st = PTR_FAILED;
	} else if (ErrRet.errFlag & NetErrFlag_NewLogin) {
	    TglobPath *gp = pctogp(s);
	    ppcchange(&gp->user, ErrRet.new_user);
	    ppcchange(&gp->passwd, ErrRet.new_passwd);
	    NetPasswdSetup_gp(gp);
	    s = gptopc(gp, gptfu_PasswdTxt);
	    xfree(*ppc);
	    *ppc = s;
	    gp_free(gp);
	}
	net_err_clr(&ErrRet);

	if (!st || (st == PTR_FAILED)) {
	    pc = NULL;
	    if (st != PTR_FAILED) {
		pc = GetFilePath(s);
		while (pc && !net_chk_exist(pc, findDIR)) {
		    path_sub(pc, &pcn);
		    xfree(pc);
		    pc = pcn;
		}
	    }
	    if (!pc) pc = xgetcwd(True);

	    if (pc) {
		TFileWin_LoadDir(fpic->FileWin, pc);
		xfree(pc);
	    } 

	    char *efn = get_DisplayFileStr(s, NULL);
	    MsgBox_Show_Error((TWins*)fpic, "ERROR !!!", 
		msgOK|msgBell, msgOK, EMSG_ERR,
		"file not found\n%s", efn);
	    xfree(efn);

	} else {
	    if (st->st & st_Dir) {
		TFileWin_LoadDir(fpic->FileWin, s);

	    } else if (st->st & st_Reg) {
		pc = GetFilePath(s);
	        TFileWin_LoadDir(fpic->FileWin, pc);
	        xfree(pc);
		dfl = dfl_create(st);
		ALDpic_FW_Load_Pic(fpic->FileWin, dfl, s);
		xfree(dfl);
	    }
	    xfree(st);
	}
}

static void mFile_Open(TMenu *self, long KeyState, TMenuItem *smi) {
  TMainForm *fpic = (TMainForm*)self->FormStruct;
  char *s;

	s = TFileWin_GetAktDir(fpic->FileWin);
	File_Dlg_SetNamePath(fpic->FileOpen_Dlg, s);
	xfree(s);

	if (File_Dlg_Execute(fpic->FileOpen_Dlg)) {
	    s = NULL;
	    File_Dlg_GetEdFileName(fpic->FileOpen_Dlg, &s);
	    ALDpic_FileOpen(fpic, &s);
	    xfree(s);
	}
}

static void mFile_Save(TMenu *self, long KeyState, TMenuItem *smi) {
}

static void ALDpic_Save(TMainForm *fpic, char *fn) {
  TPixMap *pxm;
  ThImageLoader *hLD;
  FILE *F;
  dword out_fu;
  int res;

	pxm = ALDpic_pxm_get(fpic, ib_paintbase_pxm);
	if (!pxm) return;

	F = TImageIO_GetStream(fn, NULL, 0, &hLD, True);
	if (!F) return;

	out_fu = 0;
	if (hLD->filetyp & filetyp_PNG) {
	    out_fu |= fpic->pngCompLev << 8;
	    if (fpic->miPngPallette->fu & miChecked) out_fu |= fuWR_use_pallette;
	}
	if (hLD->filetyp & filetyp_JPEG) out_fu |= fpic->jpgQuality << 8;
	if ((hLD->filetyp & filetyp_PNM) && fpic->pnm_plain) out_fu |= fuWR_txtplain;
	if (TFormAlphaClr_GetModus_IsAkt(fpic->fAlphaClr) & IS_ALPHA_CLR) out_fu |= fuWR_NoMask;
	pxm->out_gp = pctogp(fn);
	pxm->out_fu = out_fu;
	pxm->out_fp = F;
	res = hLD->ioproc((TImage*)pxm);

	if (res == IMGWR_OK) {
	    ppcchange(&fpic->pxmLastSaveAs, fn);

	} else {
	    char *efn = get_DisplayFileStr(NULL, pxm->out_gp);
	    if (res == IMG_ERR_NOSTREAM) {
		MsgBox_Show_Error((TWins*)fpic, "ERROR !!!", 
		    msgOK|msgBell, msgOK, EMSG_ERR,
		    "Can't write image file \n%s\nno %s writer", efn, pxm->out_gp->RootArt);
	    } else {
		MsgBox_Show_Error((TWins*)fpic, "ERROR !!!", 
		    msgOK|msgBell, msgOK, EMSG_ERR,
		    "Can't write image file \n%s\nerrno %d", efn, res);
	    }
	    xfree(efn);
	}

	fclose(pxm->out_fp);
	pxm->out_fp = 0;
	gpFree(pxm->out_gp);
	pxm->out_gp = NULL;
	return;
}

static void ALDpic_SaveAs(TMainForm *fpic, char *fn) {
  int m;
  TglobPath *gp = pctogp(fn);
  Tstat64 *st = NULL;
  char line[1024];

	if (!ALDpic_pxm_get(fpic, ib_paintbase_pxm)) return;
	st = net_stat_gp(gp, NULL, fsNoFileName | fsNoLinkName);
	if (st) {
	    if (st->st & st_Reg) {
		snprintf(line, sizeof(line), "Overwrite exists image file \n%s", fn);
		m = MsgBox_Show((TWins*)fpic, "Warning !!!", line, 
		    msgIcoWARNING|msgBell|msgYES | msgNO, msgNO);
		if (m == msgNO) goto E_err;
	    } else {
		snprintf(line, sizeof(line), "Can't write image file \n%s", fn);
		MsgBox_Show((TWins*)fpic, "ERROR !!!", line, 
		    msgIcoERROR|msgBell|msgYES, msgYES);
		goto E_err;
	    }
	}

	ALDpic_Save(fpic, fn);
E_err:
	gpFree(gp);
	xfree(st);
}

char *GetLinePattern(const char *pattern) {
  const char *indata, *intmp;
  char *result, *token, *tokentmp;

	indata = pattern;
	result = NULL;
	while (indata) {
	    indata = GetToken(indata, &token);
	    intmp = token;
	    while (intmp) {
		intmp = GetToken(intmp, &tokentmp);
		if (result) ppcadd(&result, ",");
		ppcadd_n(&result, 2, "*.", tokentmp);
		xfree(tokentmp);
	    }
	    xfree(token);
	}
	return result;
}

void SetFileDlgSinglePattern(TFile_Dlg *File_Dlg, const char *pattern) {
  const char *indata, *intmp;
  char *ext, *token, *tokentmp;

	indata = pattern;
	while (indata) {
	    ext = NULL;
	    indata = GetToken(indata, &token);
	    intmp = token;
	    while (intmp) {
		intmp = GetToken(intmp, &tokentmp);
		if (ext) ppcadd(&ext, ",");
		ppcadd_n(&ext, 2, "*.", tokentmp);
		xfree(tokentmp);
	    }
	    File_Dlg_AddPattern(File_Dlg, ext, NULL);
	    xfree(ext);
	    xfree(token);
	}
}

static void mFile_SaveAs(TMenu *self, long KeyState, TMenuItem *smi) {
  TMainForm *fpic = (TMainForm*)self->FormStruct;
  TPattern *patt;
  char *s, *pattern;
  const char *ext;

	if (!fpic->FileSave_Dlg) {
	    fpic->FileSave_Dlg = Create_FileSave_Dlg((TWins*)fpic);
	    file_dlg_InitAzPic(fpic->FileSave_Dlg);

	    pattern = TImageIO_GetAllFilePattern(True);
	    s = GetLinePattern(pattern);
	    File_Dlg_AddPattern(fpic->FileSave_Dlg, s, NULL);
	    xfree(s);

	    SetFileDlgSinglePattern(fpic->FileSave_Dlg, pattern);
	    xfree(pattern);
	}

	s = fpic->pxmLastSaveAs;
	if (!s) s = fpic->pxmFile;
	File_Dlg_SetNamePath(fpic->FileSave_Dlg, s);

	if (File_Dlg_Execute(fpic->FileSave_Dlg)) {
	    s = NULL;
	    File_Dlg_GetEdFileName(fpic->FileSave_Dlg, &s);
	    ext = GetFileExt_pc(s);
	    if (!ext) {
		patt = File_Dlg_Get_Patter(fpic->FileSave_Dlg);
		if (!patt || !patt->ppa || !patt->ppa[0] || patt->ppa[1]) {
		   MsgBox_Show((TWins*)fpic, "Information !!!", 
			"Select Image format *.png, *.jpg or *.xpm", 
			msgIcoINFO | msgYES, msgYES);
		   return;
		}
		ext = GetFileExt_pc(patt->ppa[0]);
		ChangeFileExt_ppc(&s, ext);
	    }
	    ALDpic_SaveAs(fpic, s); 
	    xfree(s); 
	}
}

static void ALDpic_mFile_Exit(TMenu *self, long KeyState, TMenuItem *smi) {
  Close();
}

/* ______________________ Create _____________________________________ */
void MenuFile_Create(TMainForm *self) {
  TMenuItem *mi;

	self->mbFile = TMenuButton_Create(self->PanelTop, (TMenuButton*)self);
	self->mbFile->top = 2;
	self->mbFile->left = 1;
	SetCaption((TWins*)self->mbFile, "File");
	Set_Parent((TWins*)self->mbFile, self->PanelTop);

	self->menuFile = (TMenu*)TMenu_Create((TWins*)self);
	self->menuFile->OnMenuOpen = menuFile_OnMenuOpen;
	self->mbFile->menu = self->menuFile;

	mi = TMenu_AddItem(self->menuFile, "Open...", 0, mFile_Open);
	mi->pic = get_pic16_fileopen(False);
	if (TImageWR_AllFiletyp) {
	  TMenu_AddItem(self->menuFile, "-", 0, NULL);
	  mi = TMenu_AddItem(self->menuFile, "Save", miNotEnabled, mFile_Save);
	  mi->pic = get_pic16_filesave(True);
	  self->miSave = mi;
	  mi = TMenu_AddItem(self->menuFile, "Save As...", 0, mFile_SaveAs);
	  mi->pic = get_pic16_filesave(True);
	  self->miSaveAs = mi;
	}
	TMenu_AddItem(self->menuFile, "-", 0, NULL);
	mi = TMenu_AddItem(self->menuFile, "Print Index", 0, ALDpic_mFile_PrintIndex);
	TMenu_AddItem(self->menuFile, "-", 0, NULL);
	TMenu_AddItem(self->menuFile, "Exit", 0, ALDpic_mFile_Exit);
}
