/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "ElMath.h"
#include "dlg.h"
#include "cicons.h"

extern dword t_Win_Ev;

#ifndef _OS_WIN32
static const char ElMath16_AHico[] = {
 0x06,0x08,0x10,0x00,0x10,0x00,0xE7,0xB7,0x6B,0x00,0x1F,0x00,0x86,0x31,0x10,0x84,
 0x14,0xA5,0x18,0xC6,0xDB,0xDE,0xDF,0xFF,0x00,0xF8,0xBD,0xC0,0x7B,0x41,0x77,0x4B,
 0x21,0x77,0x4B,0xD5,0xC8,0xD1,0x1D,0x1D,0x59,0x99,0x95,0x19,0xC9,0x2D,0xD5,0x63,
 0x65,0x56,0x66,0x24,0x47,0x77,0x74,0x14,0x81,0x58,0x66,0x24,0xB7,0x64,0x3F,0x86,
 0x66,0x68,0x41,0x72,0x44,0x23,0x44,0x58,0x66,0x85,0x46,0x72,0x24,0x32,0x42,0x56,
 0x66,0x85,0x46,0x72,0x44,0x33,0x44,0x56,0x66,0x65,0x48,0x72,0x94,0x62,0x43,0x56,
 0x66,0x65,0x48,0x72,0xB7,0x14,0x42,0xBC,0xD2,0x2F,0x00};
#endif

static void menuFile_OnMenuOpen(TMenu *self) {
  TFormElMath *fem;
  TPage_XFilter *pgx;

	fem = (TFormElMath*)self->FormStruct;
	fem->miSaveAs->fu |= miNotEnabled;
	fem->miLoad->fu   |= miNotEnabled;
	if (fem->PageControl->AktPage == (TPage*)fem->pgFilter) {
	    pgx = (TPage_XFilter*)fem->pgFilter->pgc_Filter->AktPage;
	    if (pgx && (pgx->art == PAGE_ART_FILTER)) {
		 if (pgx->SaveData) fem->miSaveAs->fu &= ~miNotEnabled;
		 if (pgx->LoadData) fem->miLoad->fu   &= ~miNotEnabled;
	    }
	}
}

static void miSaveAs_OnClick(TMenu *self, long KeyState, TMenuItem *mi) {
  TFormElMath *fem;
  TPage_XFilter *pgx;
  char *pc, *s;
  int m;
  TpcIni *ini;
  char line[1024];

	fem = (TFormElMath*)self->FormStruct;
	if (fem->PageControl->AktPage != (TPage*)fem->pgFilter) return;
	pgx = (TPage_XFilter*)fem->pgFilter->pgc_Filter->AktPage;
	if (!pgx || (pgx->art != PAGE_ART_FILTER) || !pgx->SaveData) return;

	if (!fem->FileSave_Dlg) {
	    fem->FileSave_Dlg = Create_FileSave_Dlg((TWins*)fem);
	    File_Dlg_AddPattern(fem->FileSave_Dlg, "*.elmath", NULL);
	    SetCaption((TWins*)fem->FileSave_Dlg->edFile, "filter.elmath");
	}

	s = NULL;
	if (File_Dlg_Execute(fem->FileSave_Dlg)) {
	    File_Dlg_GetEdFileName(fem->FileSave_Dlg, &s);
	    if (test_local_drive(s)) {
		pc = path_to_local(s);

		if (TestIsFile(pc)) {
		    snprintf(line, sizeof(line), "Overwrite exists project file \n%s", pc);
		    m = MsgBox_Show((TWins*)fem, "Warning !!!", 
			    line, msgYES | msgNO, msgNO);
		    if (m == msgNO) pc = NULL;
		}
		if (pc) {
// printf("%s(%d): %s\n", FL, pc);
		    ini = TpcIni_Create(pc);
		    pgx->SaveData(pgx, ini);
		    TpcIni_SaveAndFree(ini);
		}
		xfree(pc);

	    } else {
		MsgBox_Show((TWins*)fem, "ERROR", 
			"No local data drive !!!", msgCANCEL | msgIcoERROR, msgCANCEL);
	    }
	}
	xfree(s);
}

static void miLoad_OnClick(TMenu *self, long KeyState, TMenuItem *mi) {
  TFormElMath *fem;
  TPage_XFilter *pgx;
  char *s, *pc;
  TpcIni *ini;

	fem = (TFormElMath*)self->FormStruct;
	if (fem->PageControl->AktPage != (TPage*)fem->pgFilter) return;
	pgx = (TPage_XFilter*)fem->pgFilter->pgc_Filter->AktPage;
	if (!pgx || (pgx->art != PAGE_ART_FILTER) || !pgx->LoadData) return;

	if (!fem->File_Dlg) {
	    fem->File_Dlg = Create_FileSave_Dlg((TWins*)fem);
	    File_Dlg_AddPattern(fem->File_Dlg, "*.elmath", NULL);
//	    SetCaption((TWins*)fem->File_Dlg->edFile, "filter.elmath");
	}

	s = NULL;
	if (File_Dlg_Execute(fem->File_Dlg)) {
	    File_Dlg_GetEdFileName(fem->File_Dlg, &s);
	    if (test_local_drive(s)) {
		pc = path_to_local(s);
		if (TestIsFile(pc)) {
		    ini = TpcIni_Create(pc);
// printf("%s(%d): %s 0x%08x\n", FL, pc, (int)ini);
		    pgx->LoadData(pgx, ini);
		    Tfree(ini);
		}
		xfree(pc);

	    } else {
		MsgBox_Show((TWins*)fem, "ERROR", 
			"No local data drive !!!", msgCANCEL | msgIcoERROR, msgCANCEL);
	    }
	}
	xfree(s);
}

static void Form_ELM_mbMM_MIL_OnClick(TMenuButton *self, long KeyState, bool DblClick) {
  TFormElMath *fem = (TFormElMath*)self->FormStruct;
	fem->mm_mil++;
	if (fem->mm_mil > 1) fem->mm_mil= 0;
	switch (fem->mm_mil) {
	    case 0: 
		SetCaption((TWins*)self, " mm ");
		break;
	    case 1: 
		SetCaption((TWins*)self, " mil ");
		break;
	} 
	pageLuftSpule_SetUp(fem->pgLuftSpule, 1, fem->mm_mil);
	pageMicrostrip_SetUp(fem->pgMicrostrip, 1, fem->mm_mil);
}

static void fem_pgc_OnActivate_pg(TPageControl *pagc, TPage *pag) {
/*
  TFormElMath *fem = (TFormElMath*)pagc->FormStruct;
  TPageControl *pgc_F = fem->pgFilter->pgc_Filter;
  TPageControl *pgc_DF = fem->pgDIG_Filter->pgc;

	if (pag == (TPage*)fem->pgFilter) {
	    if (pag != fem->lastAktSubPage) {
		fem->lastAktSubPage = pag;
		if (pgc_F->OnPageActivate) {
		    pgc_F->OnPageActivate(pgc_F, pgc_F->AktPage);
		}
		if (pgc_DF->OnPageActivate) {
		    pgc_DF->OnPageActivate(pgc_DF, NULL);
		}
	    }

	} else if (pag == (TPage*)fem->pgDIG_Filter) {
	    if (pag != fem->lastAktSubPage) {
		fem->lastAktSubPage = pag;
		if (pgc_F->OnPageActivate) {
		    pgc_F->OnPageActivate(pgc_F, NULL);
		}
		if (pgc_DF->OnPageActivate) {
		    pgc_DF->OnPageActivate(pgc_DF, pgc_DF->AktPage);
		}
	    }

	} else if (fem->lastAktSubPage == (TPage*)fem->pgDIG_Filter) {
		fem->lastAktSubPage = pag;
		if (pgc_DF->OnPageActivate) {
		    pgc_DF->OnPageActivate(pgc_DF, NULL);
		}
	}
*/
}

static void Form_ELM_mFile_Exit(TMenu *self, long KeyState, TMenuItem *smi) {
  Close();
}

static void Form_ELM_OnMove(TFormElMath *self) {
  TpcIni *ti = TpcIni_Create(appDat.inifile_app);
	TpcIni_WR_int(ti, APPNAME, "Top", self->top);
	TpcIni_WR_int(ti, APPNAME, "Left", self->left);
	TpcIni_SaveAndFree(ti);
}

static void Form_ELM_OnResize(TFormElMath *self) {
	if (!self->ResizeStartMk) return;
  TpcIni *ti = TpcIni_Create(appDat.inifile_app);
	TpcIni_WR_int(ti, APPNAME, "Width", self->width);
	TpcIni_WR_int(ti, APPNAME, "Height", self->height);
	TpcIni_SaveAndFree(ti);
}


static void pgc_OnChangeTab(TPageControl *self, TPage *fromPage, TPage *toPage, int *fu) {
  TFormElMath *fem = (TFormElMath*)self->FormStruct;
  TPageControl *pgc;

	pgc = NULL;
	if ((TPage*)fem->pgFilter == fromPage) {
	    pgc = fem->pgFilter->pgc_Filter;
	} else if ((TPage*)fem->pgDIG_Filter == fromPage) {
	    pgc = fem->pgDIG_Filter->pgc;
	}
	if (pgc && pgc->OnChangeTab) pgc->OnChangeTab(pgc, pgc->AktPage, toPage, fu);

	pgc = NULL;
	if ((TPage*)fem->pgFilter == toPage) {
	    pgc = fem->pgFilter->pgc_Filter;
	} else if ((TPage*)fem->pgDIG_Filter == toPage) {
	    pgc = fem->pgDIG_Filter->pgc;
	}
	if (pgc && pgc->OnChangeTab) pgc->OnChangeTab(pgc, fromPage, pgc->AktPage, fu);
}

static void OnAppStart() {
  TFormElMath *fem = appDat.obj;
	fem->pgMicrostrip = ELM_pageMicrostripAdd(fem->PageControl, "Microstrip");
	if (!AppProcessMessages()) return;
	fem->pgFilter = ELM_pageFilterAdd(fem->PageControl, "Filter");
	if (!AppProcessMessages()) return;
	fem->pgDIG_Filter = ELM_pageDIG_Filter(fem->PageControl, "Dig.Filter");

	Tfree(appDat.appIni);
	appDat.appIni = NULL;
	fem->PageControl->OnPageActivate = fem_pgc_OnActivate_pg;
	fem->PageControl->OnChangeTab = pgc_OnChangeTab;
}

static const char *Help_Msg = {
  "\n"
  "Usage: ElMath [options] \n"
  "\n"
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: ElMath version "ALD_VERSION"  (build %d  %s)\n"
};

int build_v = ALD_BUILD_V;
const char *build_date = __DATE__;

int main (int argc, char **argv) {
  TFormElMath *self;
  int i;
  char *val;
  bool help = False;
  bool err = False;

	i = 1;
	while (!err && !help && (i<argc)) {
	    val = argv[i];
	    if (*val == '-') {
		val++;
		switch (*val) {
		case 'h':
		    help = True; break;
		case '-':
		    val++;
		    if (pcncmp(val, "help", 4)) {
			help = True;
#ifdef Test_Root_WinId
		    } else if (pcncmp(val, "TestRootWinID", 13)) {
			i++;
			val = argv[i];
			if (val && *val != '-') {
				TestRootWinId = pctoint(val);
			} else err = True;
#endif
		    } else err = True;
		    break;
		default: 
			err = True;
		}  // switch end
	    } else {
		err = True;
	    }
	    i++;
	}

	if (help || err) {
	    printf(Help_Msg, build_v, build_date);
	    exit(err);
	}

	initXapp(0, argc, argv);
	self = TForm_CreatParamAlg(NULL, sizeof(TFormElMath));
	appDat.obj = self;

#ifdef USE_DECOR
	appXDat.DefaultButtonDecor = TDecor_button_w3b3_Create();
#endif
#ifdef _OS_WIN32
	appDat.inifile_app = pcadd(appDat.ALDini_dir, "\\ElMath.ini");
#else
	appDat.inifile_app = pcadd(appDat.ALDini_dir, "/ElMath.ini");
	self->Icon = GetDeskIcon_z(ddkt_Application, 16, APPNAME, 
				    gdifu_noZoom, ElMath16_AHico);
#endif
	appDat.appIni = TpcIni_Create(appDat.inifile_app);

	SetCaption((TWins*)self, APPNAME);
	self->IconName = APPNAME;

	self->top = TpcIni_RD_int(appDat.appIni, APPNAME, "Top", 0);
	self->left = TpcIni_RD_int(appDat.appIni, APPNAME, "Left", 0);
	self->width = TpcIni_RD_int(appDat.appIni, APPNAME, "Width", 680);
	self->height = TpcIni_RD_int(appDat.appIni, APPNAME, "Height", 380);
	self->min_width = 720;
	self->min_height = 400;
	self->max_width = 1024;
	self->max_height = 540;
	self->FormPosition = poDesigned;
	self->OnMove = Form_ELM_OnMove;
	self->OnResize = Form_ELM_OnResize;

	self->pnlTop = TWins_CreateBA((TWins*)self, 0, 0, 200, 21, 0, alTop);
	self->pnlMiddle = TWins_CreateBA((TWins*)self, 0, 0, 200, 42, mBorder | mBorderL, alClient);

// ******* Menu File *******
	self->mbFile = TMenuButton_Create(self->pnlTop, (TMenuButton*)self);
	self->mbFile->top = 2;
	self->mbFile->left = 1;
	SetCaption((TWins*)self->mbFile, "File");
	Set_Parent((TWins*)self->mbFile, self->pnlTop);
	self->menuFile = TMenu_Create((TWins*)self);
	self->mbFile->menu = self->menuFile;
	self->menuFile->OnMenuOpen = menuFile_OnMenuOpen;

	self->miSaveAs = TMenu_AddItem(self->menuFile, "Save As...", miNotEnabled, miSaveAs_OnClick);
	self->miSaveAs->pic = get_pic16_filesave(False);
	self->miLoad = TMenu_AddItem(self->menuFile, "Load", miNotEnabled, miLoad_OnClick);
	self->miLoad->pic = get_pic16_fileopen(False);
	TMenu_AddItem(self->menuFile, "-", 0, NULL);
	TMenu_AddItem(self->menuFile, "Exit", 0, Form_ELM_mFile_Exit);

// ******* Menu MM/MILL *******
	self->mbMM_MIL = TMenuButton_Create(self->pnlTop, self->mbFile);
	self->mbMM_MIL->OnClick = Form_ELM_mbMM_MIL_OnClick;
	SetCaption((TWins*)self->mbMM_MIL, " mm ");
	Set_Parent((TWins*)self->mbMM_MIL, self->pnlTop);

// ******* Menu Help *******
	MenuHelp_Create(self);


	self->PageControl = TPageControl_Create(self->pnlMiddle);
	self->PageControl->Align = alClient;
	Set_Parent((TWins*)self->PageControl, self->pnlMiddle);

	self->pgLuftSpule = ELM_pageLuftSpuleAdd(self->PageControl, "Luft Spule");
	self->FormSetLC0 = Create_FormSetLC0((TWins*)self);

	TLineEdit_AddKeyDownPlugIn(NULL, TLineEdit_keyplugin_edit_Create());
	TLineEdit_AddKeyDownPlugIn(NULL, TLineEdit_keyplugin_cursor_Create());

	appXDat.OnAppStart = OnAppStart;

	AppRun();


#ifdef MEM_CHK
	Tfree(self);

  int lMCt;
  dword lMsize;
  MEM_CHK_Count(&lMCt, &lMsize);
  printf("Debug %s(%d): MEM_CHK_Count=%3d  size=%d\n", FL, lMCt, lMsize);
#endif

	return 0;
}
