/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "appXbase.h"
#include "tpixmap.h"
#include "dlg.h"
#include "Form_RLC.h"

#define APPNAME "ElMath"

extern int build_v;
extern const char *build_date;

#define PAGE_ART_FILTER		1


#define sxysetfu_nk_mask	0x0f

#define sxysetfu_scale_mask	0xf0
#define sxysetfu_lin		0
#define sxysetfu_log		0x10
#define sxysetfu_log10		0x20
#define sxysetfu_dB10		0x30
#define sxysetfu_dB20		0x40
#define sxysetfu_log_1		0x50
#define sxysetfu_log10_1	0x60
#define sxysetfu_log20_1	0x70
#define sxysetfu_log100_1	0x80
#define sxysetfu_sqrt		0x90

#define sxysetfu_scale_val	0x100

typedef struct {
  float x, y, y2;
  int px, py;
  float v1, v2;
} TScale_Point;

typedef struct TScale_pxy {
  struct TScale_pxy *next;
  int count;
  int Color;
  bool PaintDat;
  bool PaintDatOk;
  bool AddOff;
  bool nc3;
  TScale_Point xy[];
} TScale_pxy;

typedef struct {
  float val;
  int fu;
  int pos;
  char s[16];
} TScale_Dat;

typedef struct {
  char *s;
  int count;
  int fu;
  float min, max;
  TScale_Dat pt[];
} TScale_d;

typedef struct {
  TScale_d *dx;
  TScale_d *dy;
  TScale_pxy *pxy;
} TScaleXY_Dat;

typedef struct TScaleXY_Txt {
  struct TScaleXY_Txt *next;
  int x;
  int y;
  char *s;
} TScaleXY_Txt;

#define TCSelf TScaleXY
typedef struct TScaleXY {
  AlgTWins
  int sx1;
  int sy1;
  int sx2;
  int sy2;
  int Color_az;
  int RasterColor1;
  int RasterColor2;
  float XY_Dat_yOff;
  TScaleXY_Dat *dat;

  int CursFu;		//b0=CursorX On
  int CursorX_Color;
  float CursorX;
  char *CursorX_Str;
  TScaleXY_Txt *txt;
} TScaleXY;
#undef TCSelf

TScaleXY *TScaleXY_Create(TWins *Parent);
void TScaleXY_SetDat_mem(TScaleXY_Dat **dat, int cx, int cy, int cxy);
void TScaleXY_Set_Dat(TScale_d *d, const char *s, float min, float max, int step, int fu);
float TScaleXY_yVal(TScaleXY_Dat *dat, float yVal);
void TScaleXY_Set_Dat_Log(TScaleXY *sxy, int fu, int set_dx_or_dy);
float TScaleXY_GetCursX_ScaleVal(TScaleXY *sxy);
float TScaleXY_GetCursX_YVal(TScaleXY *sxy, TScale_pxy *pxy, TScale_Point **ppsp);

#define TCSelf TFormRLConoff
typedef struct TFormRLConoff {
  AlgTWins
  TPic *pic;
  TForm *Form;
} TFormRLConoff;
#undef TCSelf

TFormRLConoff *TFormRLConoff_CreateU(TWins *Parent, int left, int top, int width, int height, TPic *pic);
void TFormRLConoff_SetForm(TFormRLConoff *fof, TForm *Form);

#define TCSelf TPage_LuftSpule
typedef struct TPage_LuftSpule {
  AlgTPage
  int mm_mil;
  TWins *pnlSpule;
  TDVal *dv_G;
  TDVal *dv_L;
  TDVal *dvD;
  TDVal *dvWnd;
  TDVal *dv_AB;
  TDVal *dv_d;
  TDVal *dv_uH;
  TDVal *dv_LG;

  TDVal *dv_Cap;
  TDVal *dv_MHz;
  TDVal *dv_Imp;

  TPageControl *pgc_LC;
  TPage *pgPar_LC;
  TPage *pgSer_LC;

  TWins *pnlPar_LC;
  TDVal *dv_Rp;
  TDVal *dv_Rs;
  TDVal *dv_Rl;
  TDVal *dv_Rsum;
  TDVal *dv_Z;

  TWins *pnlSer_LC;
  TDVal *dv_s_Rp;
  TDVal *dv_s_Rs;
  TDVal *dv_s_Rl;
  TDVal *dv_s_Rsum;
  TDVal *dv_s_Z;

  TPic picSpule;
  TPic picLCp;
  TPic picLCs;

  TWins *pnlGraf;
  TScaleXY *xy_ind;

} TPage_LuftSpule;
#undef TCSelf

TPage_LuftSpule *ELM_pageLuftSpuleAdd(TPageControl *pagc, const char *Caption);
void pageLuftSpule_SetUp(TPage_LuftSpule *pg, int fu, int iVal);

#define MSG_MATH_FILTER		1
#define MSG_MATH_IMP		2
#define MSG_NEW_SCALE_PARAM	3
#define MSG_SetLC0		4
#define MSG_BEF_MASK		0xff

typedef struct {
  float MHz;
  float yVal;
  float Z;
} TMgsMathImp;


#define FU_RG		1
#define FU_RA		(1<<1)
#define FU_RS		(1<<2)
#define FU_RX		(1<<3)
#define FU_CS		(1<<4)
#define FU_CX		(1<<5)
#define FU_LS		(1<<6)
#define FU_LX		(1<<7)
#define FU_INV		(1<<8)

#define FU_F1		1
#define FU_F2		(1<<1)
#define FU_F3		(1<<2)
#define FU_F4		(1<<3)

#define MF_CS		1
#define MF_RS		(1<<1)
#define MF_CX		(1<<2)
#define MF_RX		(1<<3)
#define MF_F1		(1<<4)
#define MF_F2		(1<<5)
#define MF_F3		(1<<6)
#define MF_F4		(1<<7)
#define MF_F1I		0x100
#define MF_F2I		0x200
#define MF_F3I		0x300
#define MF_F4I		0x400

#define MF_LS		(1<<14)
#define MF_LX		(1<<15)
#define MF_INV		(1<<16)


#define AlgTPage_XFilter struct TPage_Filter *pgfilt; \
  TWins *pnl; \
  TDVal *dv_Rg; \
  TDVal *dv_Ra; \
  TDVal *dv_Rs; \
  TDVal *dv_Rx; \
  TDVal *dv_Cs; \
  TDVal *dv_Cx; \
  TDVal *dv_Ls; \
  TDVal *dv_Lx; \
  TChkBox *cbCsOn; \
  TChkBox *cbLsOn; \
  TChkBox *cbRsOn; \
  TChkBox *cbCxOn; \
  TChkBox *cbLxOn; \
  TChkBox *cbRxOn; \
  TChkBox *cbInvOn; \
  TChkBox *cbF1On; \
  TChkBox *cbF2On; \
  TChkBox *cbF3On; \
  TChkBox *cbF4On; \
  TChkBox *cbF1ImpOn; \
  TChkBox *cbF2ImpOn; \
  TChkBox *cbF3ImpOn; \
  TChkBox *cbF4ImpOn; \
  TFormRLConoff *OnOff_Rg; \
  TFormRLConoff *OnOff_Ra; \
  TFormRLConoff *OnOff_Rs; \
  TFormRLConoff *OnOff_Rx; \
  TFormRLConoff *OnOff_Cs; \
  TFormRLConoff *OnOff_Cx; \
  TFormRLConoff *OnOff_Ls; \
  TFormRLConoff *OnOff_Lx; \
  TFormR *f_Rg; \
  TFormR *f_Ra; \
  TFormR *f_Rs; \
  TFormR *f_Rx; \
  TFormC *f_Cs; \
  TFormC *f_Cx; \
  TFormL *f_Ls; \
  TFormL *f_Lx; \
  const char *gname; \
  float ScaleMHz; \
  int sbScaleCount; \
  bool (*OnMsg)(struct TCSelf *self, int msg, void *p); \
  void (*SaveData)(struct TCSelf *self, TpcIni *ini); \
  void (*LoadData)(struct TCSelf *self, TpcIni *ini);

#define TCSelf TPage_XFilter
typedef struct TPage_XFilter {
  AlgTPage
  AlgTPage_XFilter
} TPage_XFilter;
#undef TCSelf

void XF_SaveData(TPage_XFilter *self, TpcIni *ini);
void XF_LoadData(TPage_XFilter *self, TpcIni *ini);
void XF_SaveOnOff(TPage_XFilter *self, TpcIni *pcIni);
bool XF_MathOut(TPage_XFilter *self, float w, TRLC *btIn, float *ret);
void XF_MathIn(TPage_XFilter *self, float w, TRLC *btIn, float *ret, float *Z);
void XF_SetVisible(TPage_XFilter *self, bool On);
int XF_GetPaint_Fu(TPage_XFilter *self);

typedef struct TAddXFilter {
  TPageControl *pagc;
  struct TPage_Filter *pgfilt;
  const char *gname;
  int psize;
  TpcIni *ini;
  void (*TDValOnValChange)(TDVal *self, double dval);
  void (*fXxx_OnChange)(TFormRLC* fRLC);
  void (*OnRSync)(TFormRLC* fRLC, int val);
  void (*OnLSync)(TFormRLC* fRLC, int val);
  void (*OnCSync)(TFormRLC* fRLC, int val);
  int dvfu;

  void (*cb_OnClick)(TChkBox *self, long KeyState, bool DblClick);
  int cbfu;
  int cbOnOff;
  int cbY;

  void (*cbFxOnClick)(TChkBox *self, long KeyState, bool DblClick);
  void (*cbFxImpOnClick)(TChkBox *self, long KeyState, bool DblClick);
  int cbF1Y;
  int cbF2Y;
  int cbF3Y;
  int cbF4Y;
  int cbFxfu;

  int LC0_Popupfu;
} TAddXFilter;

TPage_XFilter *PageAddXFilter(TAddXFilter *p);


#define TCSelf TPage_HP1
typedef struct TPage_HP1 {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
  TDVal *dv_L1;
  TDVal *dv_L2;
  TDVal *dv_L3;
  TDVal *dv_L4;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
  TFormRLConoff *OnOff_L3;
  TFormRLConoff *OnOff_L4;

  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
  TFormL *f_L1;
  TFormL *f_L2;
  TFormL *f_L3;
  TFormL *f_L4;

} TPage_HP1;
#undef TCSelf

TPage_HP1 *page_Add_HP1(TPageControl *pagc, struct TPage_Filter *pgfilt);
void HP1_SetVisible(TPage_HP1 *pg, bool On);
//bool ImpHP1_Fx(TPage_HP1 *pg, float MHz, float *yVal, float *Z);
//void math_Filter_HP1(TPage_HP1 *pg);

/*
#define TCSelf TPage_HP2
typedef struct TPage_HP2 {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
  TDVal *dv_C4;
  TDVal *dv_L1;
  TDVal *dv_L2;
  TDVal *dv_L3;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
  TFormRLConoff *OnOff_C4;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
  TFormRLConoff *OnOff_L3;

  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
  TFormC *f_C4;
  TFormL *f_L1;
  TFormL *f_L2;
  TFormL *f_L3;

} TPage_HP2;
#undef TCSelf

TPage_HP2 *page_Add_HP2(TPageControl *pagc, struct TPage_Filter *pgfilt);
void HP2_SetVisible(TPage_HP2 *pg, bool On);
*/

#define TCSelf TPage_TP1
typedef struct TPage_TP1 {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
  TDVal *dv_C4;
  TDVal *dv_L1;
  TDVal *dv_L2;
  TDVal *dv_L3;
//  TDVal *dv_L4;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
  TFormRLConoff *OnOff_C4;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
  TFormRLConoff *OnOff_L3;
//  TFormRLConoff *OnOff_L4;

//  TChkBox *cbCsOn;
//  TChkBox *cbRsOn;
//  TChkBox *cbCxOn;
//  TChkBox *cbRxOn;

//  TChkBox *cbF1On;
//  TChkBox *cbF2On;
//  TChkBox *cbF3On;
//  TChkBox *cbF4On;

//  TChkBox *cbF1ImpOn;
//  TChkBox *cbF2ImpOn;
//  TChkBox *cbF3ImpOn;
//  TChkBox *cbF4ImpOn;

  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
  TFormC *f_C4;
  TFormL *f_L1;
  TFormL *f_L2;
  TFormL *f_L3;
//  TFormL *f_L4;

} TPage_TP1;
#undef TCSelf

TPage_TP1 *page_Add_TP1(TPageControl *pagc, struct TPage_Filter *pgfilt);
void TP1_SetVisible(TPage_TP1 *pg, bool On);
//bool ImpTP1_Fx(TPage_TP1 *pg, float MHz, float *yVal, float *Z);
//void math_Filter_TP1(TPage_TP1 *pg);

/*
#define TCSelf TPage_TP2
typedef struct TPage_TP2 {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
//  TDVal *dv_C4;
  TDVal *dv_L1;
  TDVal *dv_L2;
  TDVal *dv_L3;
  TDVal *dv_L4;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
//  TFormRLConoff *OnOff_C4;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
  TFormRLConoff *OnOff_L3;
  TFormRLConoff *OnOff_L4;


  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
//  TFormC *f_C4;
  TFormL *f_L1;
  TFormL *f_L2;
  TFormL *f_L3;
  TFormL *f_L4;

} TPage_TP2;
#undef TCSelf

TPage_TP2 *page_Add_TP2(TPageControl *pagc, struct TPage_Filter *pgfilt);
void TP2_SetVisible(TPage_TP2 *pg, bool On);
*/
/* ****** Bandpass 1 ****** */
#define TCSelf TPage_BP1
typedef struct TPage_BP1 {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
  TDVal *dv_L1;
  TDVal *dv_L2;
  TDVal *dv_L3;
  TDVal *dv_L4;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
  TFormRLConoff *OnOff_L3;
  TFormRLConoff *OnOff_L4;

//  TChkBox *cbCsOn;
//  TChkBox *cbRsOn;
//  TChkBox *cbCxOn;
//  TChkBox *cbRxOn;

//  TChkBox *cbF1On;
//  TChkBox *cbF2On;
//  TChkBox *cbF3On;
//  TChkBox *cbF4On;

//  TChkBox *cbF1ImpOn;
//  TChkBox *cbF2ImpOn;
//  TChkBox *cbF3ImpOn;
//  TChkBox *cbF4ImpOn;

  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
  TFormL *f_L1;
  TFormL *f_L2;
  TFormL *f_L3;
  TFormL *f_L4;

} TPage_BP1;
#undef TCSelf

TPage_BP1 *page_Add_BP1(TPageControl *pagc, struct TPage_Filter *pgfilt);
void BP1_SetVisible(TPage_BP1 *pg, bool On);
//bool ImpBP1_Fx(TPage_BP1 *pg, float MHz, float *yVal, float *Z);
//void math_Filter_BP1(TPage_BP1 *pg);

/* ****** Bandpass 2 ****** */

#define TCSelf TPage_BP2
typedef struct TPage_BP2 {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
  TDVal *dv_C4;
  TDVal *dv_C5;
  TDVal *dv_L1;
  TDVal *dv_L2;
  TDVal *dv_L3;
  TDVal *dv_L4;
  TDVal *dv_L5;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
  TFormRLConoff *OnOff_C4;
  TFormRLConoff *OnOff_C5;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
  TFormRLConoff *OnOff_L3;
  TFormRLConoff *OnOff_L4;
  TFormRLConoff *OnOff_L5;

//  TChkBox *cbCsOn;
//  TChkBox *cbRsOn;
//  TChkBox *cbCxOn;
//  TChkBox *cbRxOn;
//  TChkBox *cbInvOn;

//  TChkBox *cbF1On;
//  TChkBox *cbF2On;
//  TChkBox *cbF3On;
//  TChkBox *cbF4On;

//  TChkBox *cbF1ImpOn;
//  TChkBox *cbF2ImpOn;
//  TChkBox *cbF3ImpOn;
//  TChkBox *cbF4ImpOn;

  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
  TFormC *f_C4;
  TFormC *f_C5;
  TFormL *f_L1;
  TFormL *f_L2;
  TFormL *f_L3;
  TFormL *f_L4;
  TFormL *f_L5;

} TPage_BP2;
#undef TCSelf

TPage_BP2 *page_Add_BP2(TPageControl *pagc, struct TPage_Filter *pgfilt);
void BP2_SetVisible(TPage_BP2 *pg, bool On);
//bool ImpBP2_Fx(TPage_BP2 *pg, float MHz, float *yVal, float *Z);
//void math_Filter_BP2(TPage_BP2 *pg);


#define TCSelf TPage_NF1
typedef struct TPage_NF1 {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
  TDVal *dv_L1;
  TDVal *dv_L2;
  TDVal *dv_L3;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
  TFormRLConoff *OnOff_L3;

//  TChkBox *cbCsOn;
//  TChkBox *cbRsOn;
//  TChkBox *cbCxOn;
//  TChkBox *cbRxOn;

//  TChkBox *cbF1On;
//  TChkBox *cbF2On;
//  TChkBox *cbF3On;

//  TChkBox *cbF1ImpOn;
//  TChkBox *cbF2ImpOn;
//  TChkBox *cbF3ImpOn;

  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
  TFormL *f_L1;
  TFormL *f_L2;
  TFormL *f_L3;

} TPage_NF1;
#undef TCSelf

TPage_NF1 *page_Add_NF1(TPageControl *pagc, struct TPage_Filter *pgfilt);
void NF1_SetVisible(TPage_NF1 *pg, bool On);
//bool ImpNF1_Fx(TPage_NF1 *pg, float MHz, float *yVal, float *Z);
//void math_Filter_NF1(TPage_NF1 *pg);


/* ****** Bandpass 2x ****** */
#define TCSelf TPage_BP2x
typedef struct TPage_BP2x {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
  TDVal *dv_L1;
  TDVal *dv_L2;
//  TDVal *dv_L3;
//  TDVal *dv_L4;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
//  TFormRLConoff *OnOff_L3;
//  TFormRLConoff *OnOff_L4;

//  TChkBox *cbCsOn;
//  TChkBox *cbRsOn;
//  TChkBox *cbCxOn;
//  TChkBox *cbRxOn;

//  TChkBox *cbF1On;
//  TChkBox *cbF2On;
//  TChkBox *cbF3On;
//  TChkBox *cbF4On;

//  TChkBox *cbF1ImpOn;
//  TChkBox *cbF2ImpOn;
//  TChkBox *cbF3ImpOn;
//  TChkBox *cbF4ImpOn;

  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
  TFormL *f_L1;
  TFormL *f_L2;
//  TFormL *f_L3;
//  TFormL *f_L4;

/* F2 */
  TDVal *dv_Ra_F2;
//  TDVal *dv_Rs_F2;
//  TDVal *dv_Rx_F2;
  TDVal *dv_Cs_F2;
  TDVal *dv_Cx_F2;
  TFormRLConoff *OnOff_Ra_F2;
//  TFormRLConoff *OnOff_Rs_F2;
//  TFormRLConoff *OnOff_Rx_F2;
  TFormRLConoff *OnOff_Cs_F2;
  TFormRLConoff *OnOff_Cx_F2;
  TFormR *f_Ra_F2;
//  TFormR *f_Rs_F2;
//  TFormR *f_Rx_F2;
  TFormC *f_Cs_F2;
  TFormC *f_Cx_F2;

  TDVal *dv_C1_F2;
  TDVal *dv_C2_F2;
  TDVal *dv_C3_F2;
  TDVal *dv_L1_F2;
  TDVal *dv_L2_F2;

  TFormRLConoff *OnOff_C1_F2;
  TFormRLConoff *OnOff_C2_F2;
  TFormRLConoff *OnOff_C3_F2;
  TFormRLConoff *OnOff_L1_F2;
  TFormRLConoff *OnOff_L2_F2;
  TFormC *f_C1_F2;
  TFormC *f_C2_F2;
  TFormC *f_C3_F2;
  TFormL *f_L1_F2;
  TFormL *f_L2_F2;

} TPage_BP2x;
#undef TCSelf

TPage_BP2x *page_Add_BP2x(TPageControl *pagc, struct TPage_Filter *pgfilt);
void BP2x_SetVisible(TPage_BP2x *pg, bool On);
//bool ImpBP2x_Fx(TPage_BP2x *pg, float MHz, float *yVal, float *Z);
//void math_Filter_BP2x(TPage_BP2x *pg);

/* Weiche */
#define TCSelf TPage_W1
typedef struct TPage_W1 {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
  TDVal *dv_L1;
  TDVal *dv_L2;

  TDVal *dv_C4;
  TDVal *dv_C5;
  TDVal *dv_L3;
  TDVal *dv_L4;
  TDVal *dv_L5;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
  TFormRLConoff *OnOff_C4;
  TFormRLConoff *OnOff_C5;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
  TFormRLConoff *OnOff_L3;
  TFormRLConoff *OnOff_L4;
  TFormRLConoff *OnOff_L5;

//  TChkBox *cbCsOn;
//  TChkBox *cbRsOn;
//  TChkBox *cbCxOn;
//  TChkBox *cbRxOn;

//  TChkBox *cbF1On;
//  TChkBox *cbF2On;
//  TChkBox *cbF3On;
//  TChkBox *cbF4On;

//  TChkBox *cbF1ImpOn;
//  TChkBox *cbF2ImpOn;
//  TChkBox *cbF3ImpOn;
//  TChkBox *cbF4ImpOn;

  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
  TFormC *f_C4;
  TFormC *f_C5;
  TFormL *f_L1;
  TFormL *f_L2;
  TFormL *f_L3;
  TFormL *f_L4;
  TFormL *f_L5;

} TPage_W1;
#undef TCSelf

TPage_W1 *page_Add_W1(TPageControl *pagc, struct TPage_Filter *pgfilt);
void W1_SetVisible(TPage_W1 *pg, bool On);
//bool ImpW1_Fx(TPage_W1 *pg, float MHz, float *yVal, float *Z);
//void math_Filter_W1(TPage_W1 *pg);

/* ****** SGS 1 ****** */
#define TCSelf TPage_SGS1
typedef struct TPage_SGS1 {
  AlgTPage
  AlgTPage_XFilter

  TDVal *dv_C1;
  TDVal *dv_C2;
  TDVal *dv_C3;
  TDVal *dv_C4;
  TDVal *dv_L1;
  TDVal *dv_L2;
  TDVal *dv_L3;
  TDVal *dv_L4;

  TFormRLConoff *OnOff_C1;
  TFormRLConoff *OnOff_C2;
  TFormRLConoff *OnOff_C3;
  TFormRLConoff *OnOff_C4;
  TFormRLConoff *OnOff_L1;
  TFormRLConoff *OnOff_L2;
  TFormRLConoff *OnOff_L3;
  TFormRLConoff *OnOff_L4;

//  TChkBox *cbCsOn;
//  TChkBox *cbRsOn;
//  TChkBox *cbCxOn;
//  TChkBox *cbRxOn;

//  TChkBox *cbF1On;
//  TChkBox *cbF2On;
//  TChkBox *cbF3On;
//  TChkBox *cbF4On;

//  TChkBox *cbF1ImpOn;
//  TChkBox *cbF2ImpOn;
//  TChkBox *cbF3ImpOn;
//  TChkBox *cbF4ImpOn;

  TFormC *f_C1;
  TFormC *f_C2;
  TFormC *f_C3;
  TFormC *f_C4;
  TFormL *f_L1;
  TFormL *f_L2;
  TFormL *f_L3;
  TFormL *f_L4;

} TPage_SGS1;
#undef TCSelf

TPage_SGS1 *page_Add_SGS1(TPageControl *pagc, struct TPage_Filter *pgfilt);
void SGS1_SetVisible(TPage_SGS1 *pg, bool On);
//bool ImpSGS1_Fx(TPage_SGS1 *pg, float MHz, float *yVal, float *Z);
//void math_Filter_SGS1(TPage_SGS1 *pg);

#define TCSelf TPage_Filter
typedef struct TPage_Filter {
  AlgTPage
  TWins *pnlFilter;

  TPageControl *pgc_Filter;
  TPage_TP1 *TP1;
//  TPage_TP2 *TP2;
  TPage_HP1 *HP1;
//  TPage_HP2 *HP2;
  TPage_BP1 *BP1;
  TPage_BP2 *BP2;
  TPage_NF1 *NF1;
  TPage_BP2x *BP2x;
  TPage_W1 *W1;
  TPage_SGS1 *SGS1;
  TDVal *dv_TP1_Off_dB;

  TWins *pnlGraf;
  TWins *pnlGrafVal;
  TComboBox *cbmaxMHZ;
  TComboBox *cbLog;
  int fuLogX;
//  TComboBox *cbMHZoff;
  float ScaleMHz;
  float ScaleMHzoff_proz;
  float ScaleMHzoff;
  TChkBox *cbRgRa;
  TChkBox *cbdB140;
  TScaleXY *xy_ind;
  TScrollBar *sbDetail;
  TScrollBar *sbScale;

  TChkBox *cbInCsOn;
  TChkBox *cbInRsOn;
  TChkBox *cbOutCxOn;
  TChkBox *cbOutRxOn;

  TChkBox *cbHPInCsOn;
  TChkBox *cbHPOutCxOn;
  TChkBox *cbHPOutRxOn;

  TScale_pxy *yInZ;

} TPage_Filter;
#undef TCSelf

TPage_Filter *ELM_pageFilterAdd(TPageControl *pagc, const char *Caption);
//void TPage_Filter_SetScaleMHz(TPage_Filter *self, float ScaleMHz);

void Draw_xP1_Rg(Window WinID, int *px2, int *pxa, int y, int y2, int fu);
void Draw_xP1_Ra(Window WinID, int x2, int xa, int y, int y2, int fu);
void Draw_xP1_R_Out(Window WinID, int x2, int xa, int y, int y2, int fu, TPic *pic);
void DrawTP1_Lh(Window WinID, int *x, int y, TPic *pic);
void DrawTP1_Ch(Window WinID, int *x, int y, TPic *pic);
void DrawTP1_Cv(Window WinID, int *x, int xo, int y, TPic *pic);
void DrawTP1_Lv(Window WinID, int *x, int xo, int y, TPic *pic);
void SetCursor_Dat_txt(TPage_Filter *pg);

#define PT_TOP		1
#define PTL_TOP		(1<<1)
#define PT_BOTTOM	(1<<2)
#define PTL_BOTTOM	(1<<3)
#define PT_ALL		0x0f

void DrawTP1_Cv_pt(Window WinID, int *x, int xo, int y, TPic *pic, int pt);
void DrawTP1_Lv_pt(Window WinID, int *x, int xo, int y, TPic *pic, int pt);

/* ************* Microstrip ************* */
#define TCSelf TPage_MS1
typedef struct TPage_MS1 {
  AlgTPage
  struct TPage_Microstrip *pgMS;
  TWins *pnl;
  TDVal *dv_W;
  TDVal *dv_G;
  TDVal *dv_H;
  TDVal *dv_L;
  TDVal *dv_T;
  TDVal *dv_Er;

  TChkBox *cbCoplan;
  TChkBox *cbGroundp;

  TDVal *dv_Imp;
  TDVal *dv_Cm;
  TDVal *dv_uHm;
  TDVal *dv_nSm;
  TDVal *dv_C;
  TDVal *dv_uH;
  TDVal *dv_nS;

  TDVal *dv_Freq;
  TDVal *dv_tan;
  TDVal *dv_lo_dbm;
  TDVal *dv_lo_db;
  TDVal *dv_Ohmm;
  TDVal *dv_Ohm;
  TDVal *dv_OhmP;

} TPage_MS1;
#undef TCSelf

TPage_MS1 *page_Add_MS1(TPageControl *pagc, struct TPage_Microstrip *pgMS);

#define TCSelf TPage_Microstrip
typedef struct TPage_Microstrip {
  AlgTPage
  int mm_mil;
  TWins *pnl1;

  TPageControl *pgc;
  TPage_MS1 *MS1;


  TWins *pnlGraf;
  TWins *pnlGrafVal;

  TScaleXY *xy_ind;
  TScrollBar *sbDetail;

  TScale_pxy *yInZ;

} TPage_Microstrip;
#undef TCSelf

TPage_Microstrip *ELM_pageMicrostripAdd(TPageControl *pagc, const char *Caption);
void pageMicrostrip_SetUp(TPage_Microstrip *pg, int fu, int iVal);


/* ************* Digital-Filter ************* */
typedef struct {
  float fA;
  int n;
  int art;
  float fg;
  float a[4];
  float b[4];
} TIRR_ab;

#define TCSelf TPage_IIR1
typedef struct TPage_IIR1 {
  AlgTPage
  struct TPage_DIG_Filter *pgDIGF;
//  TWins *pnl0;
  TWins *pnl;
  TLineEdit *le;

  int c_stat;
  TDVal *dv_cN;
  TDVal *dv_cFreq;
  TChkBox *cb_cToF1;
  TChkBox *cb_cToF2;
  TWins *buToF;
  TIRR_ab ab;
  TChkBox *cb_cHP;

  TChkBox *cb_F1;
  TChkBox *cb_F1T;
  TDVal *dv_N;
  TDVal *dv_a10;
  TDVal *dv_a11;
  TDVal *dv_a12;
  TDVal *dv_a13;
  TDVal *dv_b10;
  TDVal *dv_b11;
  TDVal *dv_b12;
  TDVal *dv_b13;
  TFormRLConoff *OnOff_a10;
  TFormRLConoff *OnOff_a11;
  TFormRLConoff *OnOff_a12;
  TFormRLConoff *OnOff_a13;
  TFormRLConoff *OnOff_b10;
  TFormRLConoff *OnOff_b11;
  TFormRLConoff *OnOff_b12;
  TFormRLConoff *OnOff_b13;
  TFormDigA *f_a10;
  TFormDigA *f_a11;
  TFormDigA *f_a12;
  TFormDigA *f_a13;
  TFormDigA *f_b10;
  TFormDigA *f_b11;
  TFormDigA *f_b12;
  TFormDigA *f_b13;


  TChkBox *cb_F2;
  TChkBox *cb_F2T;
  TDVal *dv_N2;
  TDVal *dv_a20;
  TDVal *dv_a21;
  TDVal *dv_a22;
  TDVal *dv_a23;
  TDVal *dv_b20;
  TDVal *dv_b21;
  TDVal *dv_b22;
  TDVal *dv_b23;
  TFormRLConoff *OnOff_a20;
  TFormRLConoff *OnOff_a21;
  TFormRLConoff *OnOff_a22;
  TFormRLConoff *OnOff_a23;
  TFormRLConoff *OnOff_b20;
  TFormRLConoff *OnOff_b21;
  TFormRLConoff *OnOff_b22;
  TFormRLConoff *OnOff_b23;
  TFormDigA *f_a20;
  TFormDigA *f_a21;
  TFormDigA *f_a22;
  TFormDigA *f_a23;
  TFormDigA *f_b20;
  TFormDigA *f_b21;
  TFormDigA *f_b22;
  TFormDigA *f_b23;

  TChkBox *cbSumOn;
  TChkBox *cbSerOn;

} TPage_IIR1;
#undef TCSelf

TPage_IIR1 *page_Add_Dig_IIR1(TPageControl *pagc, struct TPage_DIG_Filter *pgDIGF);
void math_Filter_IIR1(TPage_IIR1 *pg);

#define TCSelf TPage_DIG_Filter
typedef struct TPage_DIG_Filter {
  AlgTPage
  TWins *pnl1;

  TPageControl *pgc;
  TPage_IIR1 *IIR1;

  TWins *pnlGraf;
  TWins *pnlGrafVal;
//  TComboBox *cbsampKHZ;
  TDVal *dvSampKHZ;
  TComboBox *cbmaxKHZ;
  TDVal *dv_T;
  TComboBox *cbLog;
  int fuLogX;


  float ScaleKHz;
  float ScaleKHzoff_proz;
  float ScaleKHzoff;

  TScaleXY *xy_ind;
  TScale_pxy *pxy1, *pxy2, *pxy3, *pxy4;
  TScrollBar *sbDetail;
  TScrollBar *sbScale;

  TScale_pxy *yInZ;

} TPage_DIG_Filter;
#undef TCSelf

TPage_DIG_Filter *ELM_pageDIG_Filter(TPageControl *pagc, const char *Caption);

void Dig_IIR1_SetVisible(TPage_IIR1 *pg, bool On);
void ELM_IIR1_DataChange(TPage_DIG_Filter *pg);

/* ************* Form Set LC0 ************* */
#define TCSelf TFormSetLC0
typedef struct TFormSetLC0 {
  AlgTForm
  TDVal *dv_Rg;
  TDVal *dv_MHz;
  TDVal *dv_K;
  TChkBox *CsCx;
  TChkBox *LsLx;
  TChkBox *cbFlat;
  TChkBox *cbFlatR;
  TChkBox *cb05dB;
  TChkBox *cb30dB;
  TButton *buOK;
  TPage_XFilter *pgx;
  int nr;
  float R;	//Ohm
  float L;	//uH
  float C;	//pF
  float K;
  float klc;
  float w;	//MHz * M_PI * 2
  float v[10];
  int vi;
  bool UseCsCx;
  bool UseLsLx;

} TFormSetLC0;
#undef TCSelf

TFormSetLC0 *Create_FormSetLC0(TWins *parent);
void ShowFormSetLC0(TFormSetLC0 *self, TPage_XFilter *pgx, int nr);

/* ************* Main Form ************* */
#define TCSelf TFormElMath
typedef struct TFormElMath {
  AlgTForm
  int mm_mil;
  TWins *pnlTop;
  TWins *pnlMiddle;
  TWins *pnlBottom;

  TMenuButton *mbFile;
    TMenu *menuFile;
    TMenuItem *miSaveAs;
    TMenuItem *miLoad;
  TMenuButton *mbMM_MIL;
  TMenuButton *mbHelp;
    TMenu *menuHelp;

  TPageControl *PageControl;
  TPage *lastAktSubPage;
//  TPage *pgLuftSpule;
  TPage_LuftSpule *pgLuftSpule;
  TPage_Microstrip *pgMicrostrip;
  TPage_Filter *pgFilter;
  TPage_DIG_Filter *pgDIG_Filter;

  TFile_Dlg *File_Dlg;
  TFile_Dlg *FileSave_Dlg;

  TFormSetLC0 *FormSetLC0;
} TFormElMath;
#undef TCSelf

void MenuHelp_Create(TFormElMath *fmain);
