/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "ElMath.h"
#include "ElGraf.h"
#include <math.h>

static void str_toGraf(TFormSetLC0 *Wins, const char *s, int x, int y) {
	vds_SetStr(&Wins->vds, s);
	DrawColorString(x, y, &Wins->vds);
}

static void LC0_OnPaint(TFormSetLC0 *self) {
	str_toGraf(self, "Z", self->dv_Rg->left - 30,
			       self->dv_Rg->top + 3);
	DrawPic(self->WinID, get_Pic_cOhm(), 
			self->dv_Rg->left + self->dv_Rg->width + 2,
			self->dv_Rg->top + 5);
	str_toGraf(self, "K", self->dv_K->left - 30,
			       self->dv_K->top + 3);

	str_toGraf(self, "Freq", self->dv_MHz->left - 30,
			       self->dv_MHz->top + 3);
	str_toGraf(self, "MHz", self->dv_MHz->left + self->dv_MHz->width + 1,
			       self->dv_MHz->top + 3);
//	str_toGraf(self, "K", self->dv_K->left + self->dv_K->width + 1,
//			       self->dv_K->top + 3);
}

static void cbF_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TFormSetLC0 *form;

	form = (TFormSetLC0*)self->FormStruct;
	TChkBox_SetChecked(form->cbFlat,  form->cbFlat == self);
	TChkBox_SetChecked(form->cbFlatR, form->cbFlatR == self);
	TChkBox_SetChecked(form->cb05dB,  form->cb05dB == self);
	TChkBox_SetChecked(form->cb30dB,  form->cb30dB == self);
}

const static float TabF_3[12] = {
 1.0000, 2.0000, 1.0000,
 1.4000, 1.6200, 1.4000,
 1.8597, 1.2777, 1.8597,
 3.3487, 0.7117, 3.3487
};
// 1.5963, 1.0967, 1.5963,

const static float TabF_5[20] = {
 0.6180, 1.6180, 2.0000, 1.6180, 0.6180,
 0.8483, 1.7253, 1.9540, 1.7253, 0.8483,
 1.8056, 1.3015, 2.6894, 1.3015, 1.8056,
 3.4817, 0.7618, 4.5381, 0.7618, 3.4817
};
// 1.7058, 1.2296, 2.5408, 1.2296, 1.7058,

const static float TabF_7[28] = {
 0.4450, 1.2470, 1.8019, 2.0000, 1.8019, 1.2470, 0.4450,
 0.6387, 1.6864, 1.8019, 2.0000, 1.8019, 1.6864, 0.6387,
 1.7893, 1.2960, 2.7172, 1.3847, 2.7172, 1.2960, 1.7893,
 3.5182, 0.7723, 4.6386, 0.8039, 4.6386, 0.7723, 3.5182
};
// 1.7372, 1.2583, 2.6381, 1.3444, 2.6381, 1.2583, 1.7372,


const static float TabF_9[36] = {
 0.3473, 1.0000, 1.5321, 1.8794, 2.0000, 1.8794, 1.5321, 1.0000, 0.3473,
 0.3473, 1.1111, 1.6737, 1.8794, 2.0000, 1.8794, 1.6737, 1.1111, 0.3473,
 1.7504, 1.2690, 2.6678, 1.3673, 2.7239, 1.3673, 2.6678, 1.2690, 1.7504,
 3.5340, 0.7760, 4.6692, 0.8118, 4.7272, 0.8118, 4.6692, 0.7760, 3.5340
};

static void buOK_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormSetLC0 *form;
  TFormElMath *FormElMath;
  TPage_Filter *pgFilter;
  float w;
  const float *pf;
  int i;

	FormElMath = appDat.obj;
	pgFilter = FormElMath->pgFilter;
	form = (TFormSetLC0*)self->FormStruct;
	form->vi = 0;
	form->K = 1;
	if (form->dv_K->Visible) form->K = form->dv_K->dval;
	form->UseCsCx = form->CsCx->Visible && form->CsCx->Checked;
	form->UseLsLx = form->LsLx->Visible && form->LsLx->Checked;
	w = form->dv_MHz->dval * M_PI * 2;
	form->w = w;
	form->R = form->dv_Rg->dval;
	form->L = form->R / w;
	form->C = 1000000 / (form->R * w);
 printf("%s(%d): %fuH %fpf\n", FL, form->L, form->C);
	form->klc = 1;

	pf = NULL;

	if ((TPage_XFilter*)pgFilter->W1 == form->pgx) {
	    switch (form->nr) {
	    case 1:
	    case 2:
		form->vi = 3;
		pf = TabF_3;
		break;
	    case 3:
	    case 4:
		form->vi = 5;
		pf = TabF_5;
		break;
	    }

	} else if ((TPage_XFilter*)pgFilter->BP1 == form->pgx) {
	    switch (form->nr) {
	    case 1:
	    case 2:
		form->vi = 3;
		pf = TabF_3;
		form->klc = 2;
		if (form->cbFlatR->Checked) {
		    form->klc = 1.7;
		} else if (form->cb05dB->Checked) {
		    form->klc = 1.4;
		} else if (form->cb30dB->Checked) {
		    form->klc = 1.26;
		}
		break;
	    }

/*	} else if ((TPage_XFilter*)pgFilter->BP2 == form->pgx) {
	    switch (form->nr) {
	    case 1:
		form->vi = 1;
		form->v[0] = 2;
		break;
	    case 2:
		form->vi = 3;
		pf = TabF_3;
		form->klc = 2;
		if (form->cbFlatR->Checked) {
		    form->klc = 1.58;
		} else if (form->cb05dB->Checked) {
		    form->klc = 1.4;
		} else if (form->cb30dB->Checked) {
		    form->klc = 1.33;
		}
		break;
	    }
*/
	} else if ((TPage_XFilter*)pgFilter->NF1 == form->pgx) {
		form->vi = 1;
		form->v[0] = form->dv_K->dval;

	} else {
	    switch (form->nr) {
	    case 1:
		if (form->UseCsCx || form->UseLsLx) {
		    form->vi = 3;
		    pf = TabF_3;
		} else {
		    form->vi = 1;
		    form->v[0] = 2;
		}
		break;
	    case 2:
		if (form->UseCsCx || form->UseLsLx) {
		    form->vi = 5;
		    pf = TabF_5;
		} else {
		    form->vi = 3;
		    pf = TabF_3;
		}
		break;
	    case 3:
		if (form->UseCsCx || form->UseLsLx) {
		    form->vi = 7;
		    pf = TabF_7;
		} else {
		    form->vi = 5;
		    pf = TabF_5;
		}
		break;
	    case 4:
		if (form->UseCsCx || form->UseLsLx) {
		    form->vi = 9;
		    pf = TabF_9;
		} else {
		    form->vi = 7;
		    pf = TabF_7;
		}
		break;
	    }
	}

	if (pf) {
	    if (form->cbFlatR->Checked) {
		pf += form->vi;
	    } else if (form->cb05dB->Checked) {
		pf += form->vi * 2;
	    } else if (form->cb30dB->Checked) {
		pf += form->vi * 3;
	    }
	    for (i=0;i<form->vi;i++) {
		form->v[i] = *pf; pf++;
	    }
	}

	SetVisible((TWins*)form, False);
	if (form->pgx->OnMsg) {
	    form->pgx->OnMsg(form->pgx, MSG_SetLC0, form);
	}
}

void ShowFormSetLC0(TFormSetLC0 *self, TPage_XFilter *pgx, int nr) {
  TPage_Filter *pgFilter;

	if (!pgx || !nr) return;
	self->pgx = pgx;
	self->nr = nr;
	if (pgx->dv_Rg) TDVal_Set_Value(self->dv_Rg, pgx->dv_Rg->dval);

	pgFilter = ((TFormElMath*)appDat.obj)->pgFilter;

	if ((TPage_XFilter*)pgFilter->NF1 == pgx) {
	    SetVisible((TWins*)self->dv_K,    true);
	    SetVisible((TWins*)self->CsCx,    false);
	    SetVisible((TWins*)self->LsLx,    false);
	    SetVisible((TWins*)self->cbFlat,  false);
	    SetVisible((TWins*)self->cbFlatR, false);
	    SetVisible((TWins*)self->cb05dB,  false);
	    SetVisible((TWins*)self->cb30dB,  false);
	} else if ((TPage_XFilter*)pgFilter->TP1 == pgx) {
	    SetVisible((TWins*)self->dv_K,    false);
	    SetVisible((TWins*)self->CsCx,    false);
	    SetVisible((TWins*)self->LsLx,    true);
	    SetVisible((TWins*)self->cbFlat,  true);
	    SetVisible((TWins*)self->cbFlatR, true);
	    SetVisible((TWins*)self->cb05dB,  true);
	    SetVisible((TWins*)self->cb30dB,  true);
	} else if ((TPage_XFilter*)pgFilter->HP1 == pgx) {
	    SetVisible((TWins*)self->dv_K,    false);
	    SetVisible((TWins*)self->CsCx,    true);
	    SetVisible((TWins*)self->LsLx,    false);
	    SetVisible((TWins*)self->cbFlat,  true);
	    SetVisible((TWins*)self->cbFlatR, true);
	    SetVisible((TWins*)self->cb05dB,  true);
	    SetVisible((TWins*)self->cb30dB,  true);
	} else if (((TPage_XFilter*)pgFilter->BP1 == pgx) ||
		   ((TPage_XFilter*)pgFilter->BP2 == pgx)) {
	    SetVisible((TWins*)self->dv_K,    true);
	    SetVisible((TWins*)self->CsCx,    false);
	    SetVisible((TWins*)self->LsLx,    false);
	    SetVisible((TWins*)self->cbFlat,  true);
	    SetVisible((TWins*)self->cbFlatR, true);
	    SetVisible((TWins*)self->cb05dB,  true);
	    SetVisible((TWins*)self->cb30dB,  true);
	} else {
	    SetVisible((TWins*)self->dv_K,    false);
	    SetVisible((TWins*)self->CsCx,    false);
	    SetVisible((TWins*)self->LsLx,    false);
	    SetVisible((TWins*)self->cbFlat,  true);
	    SetVisible((TWins*)self->cbFlatR, true);
	    SetVisible((TWins*)self->cb05dB,  true);
	    SetVisible((TWins*)self->cb30dB,  true);
	}

	SetVisible((TWins*)self, true);
}

static void OnResize(TFormSetLC0 *self) {
  int x, y;

	x = self->width - self->buOK->width - 2;
	y = self->height - self->buOK->height - 2;
	SetPosition((TWins*)self->buOK, x, y);
}

TFormSetLC0 *Create_FormSetLC0(TWins *parent) {
  TFormSetLC0 *self;

	self = TForm_CreatParamAlg(parent, sizeof(TFormSetLC0));
	self->wtyp = wtyp_USER_Form;
	self->FormZStyle = fszTransient;
	self->FormPosition = poDesigned;
	self->width = 226;
	self->height = 137;
	self->min_width = self->width;
	self->min_height = self->height;
	self->max_width = self->width;
	self->max_height = self->height;
	self->Visible = False;
	self->OnPaint = LC0_OnPaint;
	self->OnResize = OnResize;

	self->dv_Rg = TDVal_CreateS((TWins*)self, 32, 2, 58, 18);
	self->dv_Rg->nk = 1;
	self->dv_Rg->dv_fu = tdv_fu_LostFocusEnter;

	self->dv_MHz = TDVal_CreateS((TWins*)self, 32, 22, 58, 18);
	self->dv_MHz->nk = 3;
	self->dv_MHz->dv_fu = tdv_fu_LostFocusEnter;
	TDVal_Set_Value(self->dv_MHz, 100);

	self->dv_K = TDVal_CreateS((TWins*)self, 32, 42, 58, 18);
	self->dv_K->nk = 3;
	self->dv_K->dv_fu = tdv_fu_LostFocusEnter;
	TDVal_Set_Value(self->dv_K, 1);

	self->CsCx = TChkBox_CreateS((TWins*)self, 2, 62, 88, 16, " use Cs,Cx");
	self->LsLx = TChkBox_CreateS((TWins*)self, 2, 82, 88, 16, " use Ls,Lx");

	self->cbFlat  = TChkBox_CreateS((TWins*)self, 150, 2, 56, 16, " Flat");
	self->cbFlatR = TChkBox_CreateS((TWins*)self, 150, 20, 56, 16, " Flat R");
	self->cb05dB  = TChkBox_CreateS((TWins*)self, 150, 38, 56, 16, " 0.5 dB");
	self->cb30dB  = TChkBox_CreateS((TWins*)self, 150, 56, 56, 16, " 3.0 dB");

	self->cbFlat->Checked = true;

	self->cbFlat->AutoToggle  = false;
	self->cbFlatR->AutoToggle = false;
	self->cb05dB->AutoToggle  = false;
	self->cb30dB->AutoToggle  = false;

	self->cbFlat->OnClick  = cbF_OnClick;
	self->cbFlatR->OnClick = cbF_OnClick;
	self->cb05dB->OnClick  = cbF_OnClick;
	self->cb30dB->OnClick  = cbF_OnClick;


	self->buOK = TButton_CreateS((TWins*)self, 12, 42, 48, 22, "OK");
	self->buOK->OnClick = buOK_OnClick;
	return self;
}
