/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "ElMath.h"
//#include "Form_RLC.h"
#include "ElGraf.h"
#include <math.h>

void Form_R_math(TFormR *fR, float w) {
  float Ohm;

//	Ohm = fR->R->dval;
//	if (fR->cbTR->Checked) Ohm = fR->TR->dval;
	Ohm = fR->TR->dval;
	SetUp_R(&fR->rlc, w, Ohm);
	if (fR->cbOptOn->Checked) {
	    if (fR->cbLs->Checked && (fR->Ls->dval > 0))
		RLC_Add_Ser_L(&fR->rlc, w, fR->Ls->dval);
	    if (fR->cbCp->Checked && (fR->Cp->dval > 0))
		RLC_Add_Par_C(&fR->rlc, w, fR->Cp->dval * 0.000001);
	}
}

static void fR_math_loc_Az(TFormR *fR) {
  float w = fR->MHz->dval * M_PI * 2;
	Form_R_math(fR, w);
	fR->Z->nk = 2;
	if (fR->rlc.Z >= 1000) fR->Z->nk = 1;
	if (fR->rlc.Z >= 10000) fR->Z->nk = 0;
	TDVal_Set_Value(fR->Z, fR->rlc.Z);
}

static void fR_Set_Stat_exDVal(TFormR *fR) {
  int Color = 0xffffff;
	if (!fR->exDVal) return;
//	if (fR->cbTR->Checked) {
	    fR->exDVal->ReadOnly = (fR->sbTR->Count != 0);
	    TDVal_Set_Value(fR->exDVal, fR->TR->dval);
//	} else {
//	    fR->exDVal->ReadOnly = False;
//	    TDVal_Set_Value(fR->exDVal, fR->R->dval);
//	}
	if (fR->exDVal->ReadOnly) Color = 0xe0e0e0;
	SetColor((TWins*)fR->exDVal, Color);
}

static void fR_math_loc(TFormR *fR) {
	fR_math_loc_Az(fR);
	fR_Set_Stat_exDVal(fR);
	if (fR->OnChange) fR->OnChange((TFormRLC*)fR);
}

static void fR_SaveDaten(TFormR *fR, TpcIni *ini) {
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
  char *pc;
  int OnOff = 0;
  char buf[512];

	pcIni = ini;
	if (!ini) pcIni = get_appIni();;
	if (fR->cbOptOn->Checked) OnOff |= 0x01;
	if (fR->cbLs->Checked) OnOff |= 0x02;
	if (fR->cbCp->Checked) OnOff |= 0x08;
//	if (fR->cbTR->Checked) OnOff |= 0x10;
	if (fR->cbTRsync->Checked) OnOff |= 0x20;

	pc = buf;
	doubletoppc(&pc, fR->MHz->dval, fR->MHz->nk | kSPACE_ADD);
	doubletoppc(&pc, fR->Ls->dval, fR->Ls->nk | kSPACE_ADD);
	ppcmove(&pc, "0 ");
	doubletoppc(&pc, fR->Cp->dval, fR->Cp->nk | kSPACE_ADD);
	doubletoppc(&pc, fR->TRmin->dval, fR->TRmin->nk | kSPACE_ADD);
	ppcmove(&pc, "0 ");
	inttoppc(&pc, OnOff);

	TpcIni_WR(pcIni, fR->Gruppe, fR->sOpt, buf);
	TpcIni_WR_double(pcIni, fR->Gruppe, fR->sR, fR->R->dval, fR->R->nk);
	if (!ini) {
	    TpcIni_Save(pcIni);
	    free_appIni(pcIni);
	}
}

static void fR_cb_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TFormR *fR = (TFormR*)self->FormStruct;
	fR_math_loc(fR);
	fR_SaveDaten(fR, NULL);
}

static void fR_TR_SetVal(TFormR *fR, int Count) {
  float R = fR->R->dval - (fR->R->dval - fR->TRmin->dval) * 
			    (float)(Count) / (float)(fR->sbTR->MaxCount);
	if (R < 0.001) R = 0.001;
	fR->TR->nk = 1;
	if (R < 10) fR->TR->nk = 2;
	TDVal_Set_Value(fR->TR, R);
}

static void fR_DVal_OnValChange(TDVal *dv, double val) {
  TFormR *fR = (TFormR*)dv->FormStruct;
	fR_TR_SetVal(fR, TScrollBar_GetCount(fR->sbTR));
	fR_math_loc(fR);
	fR_SaveDaten(fR, NULL);
}

static void fR_sbTR_OnCount(TScrollBar* sb, int Count) {
  TFormR *fR = (TFormR*)sb->FormStruct;
  TPage_XFilter *pg = fR->subClass;

	fR_TR_SetVal(fR, Count);
	fR->val_sync = Count;
	if (fR->cbTRsync->Checked) {
	    if (pg->f_Rg && (pg->f_Rg != fR)) 
				Form_R_sync((TFormRLC*)fR, pg->f_Rg, Count);
	    if (pg->f_Rs && (pg->f_Rs != fR)) 
				Form_R_sync((TFormRLC*)fR, pg->f_Rs, Count);
	    if (pg->f_Rx && (pg->f_Rx != fR)) 
				Form_R_sync((TFormRLC*)fR, pg->f_Rx, Count);
	    if (pg->f_Ra && (pg->f_Ra != fR)) 
				Form_R_sync((TFormRLC*)fR, pg->f_Ra, Count);
	    if (fR->OnSync) fR->OnSync((TFormRLC*)fR, Count);
	}
//	if (fR->cbTR->Checked) fR_math_loc(fR);
	 fR_math_loc(fR);
}

void Form_R_sync(TFormRLC *fRLC, TFormR *fR, int val) {
	if (fRLC == (TFormRLC*)fR) return;
	fR->val_sync = val;
	if (fR->cbTRsync->Checked) {
	    TScrollBar_SetCount(fR->sbTR, val);
	    fR_TR_SetVal(fR, val);
	    fR_math_loc_Az(fR);
	    fR_Set_Stat_exDVal(fR);
	}
}

static void Form_R_SetFreq(TFormRLC *fRLC, float val) {
	TDVal_Set_Value(((TFormR*)fRLC)->MHz, val);
	fR_math_loc_Az((TFormR*)fRLC);
}

static void fR_pnlR_OnPaint(TWins *self) {
  TFormR *fR = (TFormR*)self->FormStruct;
  int x = 13;
  int y = 24;
  int y3 = y+18;

	DrawPic(self->WinID, get_Pic_pto(), x, y+2);
	DrawLineRGB(self->WinID, 0, x+7, y+5, x+9, y+5);
	DrawPic(self->WinID, get_Pic_pt(), x+10, y+3);
	DrawLineRGB(self->WinID, 0, x+15, y+5, x+19, y+5);
	DrawPic(self->WinID, get_Pic_Rh(), x+20, y);
	  DrawPic(self->WinID, get_Pic_cR(), x+28, y+10);
	DrawLineRGB(self->WinID, 0, x+40, y+5, x+49, y+5);
	DrawPic(self->WinID, get_Pic_Lh(), x+50, y);
	  DrawPic(self->WinID, get_Pic_cLs(), x+58, y+10);
	DrawLineRGB(self->WinID, 0, x+70, y+5, x+84, y+5);
	DrawPic(self->WinID, get_Pic_pt(), x+75, y+3);
	DrawPic(self->WinID, get_Pic_pto(), x+85, y+2);

	DrawLineRGB(self->WinID, 0, x+12, y+8, x+12, y3+5);
	DrawLineRGB(self->WinID, 0, x+77, y+8, x+77, y3+5);
	DrawLineRGB(self->WinID, 0, x+12, y3+5, x+34, y3+5);
	DrawPic(self->WinID, get_Pic_Ch(), x+35, y3);
	  DrawPic(self->WinID, get_Pic_cCp(), x+50, y3+7);
	DrawLineRGB(self->WinID, 0, x+55, y3+5, x+76, y3+5);

/* TDVal R */
	DrawPic(self->WinID, get_Pic_cR(), fR->R->left-11, fR->R->top+7);
	DrawPic(self->WinID, get_Pic_cOhm(), fR->R->left+fR->R->width+3, fR->R->top+7);
}


static void fR_pnlOpt_OnPaint(TWins *self) {
  TFormR *fR = (TFormR*)self->FormStruct;

	DrawPic(self->WinID, get_Pic_cLs(), fR->Ls->left-15, fR->Ls->top+5);
	DrawPic(self->WinID, get_Pic_cuH(), fR->Ls->left+fR->Ls->width+3, fR->Ls->top+5);
	DrawPic(self->WinID, get_Pic_cCp(), fR->Cp->left-15, fR->Cp->top+5);
	DrawPic(self->WinID, get_Pic_cpF(), fR->Cp->left+fR->Cp->width+3, fR->Cp->top+5);
}

static void fR_pnlZ_OnPaint(TWins *self) {
  TFormR *fR = (TFormR*)self->FormStruct;

	DrawPic(self->WinID, get_Pic_cf(), fR->MHz->left-8, fR->MHz->top+5);
	DrawPic(self->WinID, get_Pic_cMHz(), fR->MHz->left+fR->MHz->width+3, fR->MHz->top+5);
	DrawPic(self->WinID, get_Pic_cZ(), fR->Z->left-8, fR->Z->top+5);
	DrawPic(self->WinID, get_Pic_cOhm(), fR->Z->left+fR->Z->width+3, fR->Z->top+5);

	DrawPic(self->WinID, get_Pic_cOhm(), fR->TR->left+fR->TR->width+3, fR->TR->top+5);
	DrawPic(self->WinID, get_Pic_cOhm(), fR->TRmin->left+fR->TRmin->width+3, fR->TRmin->top+5);
}
static void fR_pnlZ_OnResize(TWins *self) {
  TFormR *fR = (TFormR*)self->FormStruct;
	SetTop((TWins*)fR->TRmin, self->height-20);
	SetTop((TWins*)fR->TR, self->height-38);
	SetTop((TWins*)fR->cbTRsync, self->height-55);
}

static void Form_R_OnMove(TFormR *self) {
	TFormRLC_Save((TFormRLC*)self);
}

static void fR_LoadData(TFormR *self, TpcIni *ini) {
  int OnOff;
  const char *sopt;

	TDVal_Set_Value(self->R, TpcIni_RD_float(ini, self->Gruppe, self->sR, self->R->dval));
	sopt = TpcIni_RD_pc(ini, self->Gruppe, self->sOpt, "0 0.001 0 0.1 1 0 0x10");
	TDVal_Set_Value(self->MHz, ppctodouble(&sopt));
	TDVal_Set_Value(self->Ls, ppctodouble(&sopt));
//	TDVal_Set_Value(self->Rp, ppctodouble(&sopt));
	ppctodouble(&sopt);
	TDVal_Set_Value(self->Cp, ppctodouble(&sopt));
	TDVal_Set_Value(self->TRmin, ppctodouble(&sopt));
	ppctodouble(&sopt);
	OnOff = ppctoint(&sopt);
	TChkBox_SetChecked(self->cbOptOn, OnOff & 0x01);
	TChkBox_SetChecked(self->cbLs, OnOff & 0x02);
//	TChkBox_SetChecked(self->cbRp, OnOff & 0x04);
	TChkBox_SetChecked(self->cbCp, OnOff & 0x08);
//	TChkBox_SetChecked(self->cbTR, OnOff & 0x10);
	TChkBox_SetChecked(self->cbTRsync, OnOff & 0x20);
}


static void LoadData(TFormR *self, TpcIni *ini) {
	if (!ini) return;
	fR_LoadData(self, ini);
	TScrollBar_SetCount(self->sbTR, 0);
	TDVal_Set_Value(self->TR, self->R->dval);
	fR_Set_Stat_exDVal(self);
}

TFormR *Create_Form_R(TWins *parent, const char *Caption, const char *Gruppe, 
			const char *sR, TDVal *exDVal, float val) {
  TFormR *fR;
  TpcIni *pcIni;
  const char *sopt;

	pcIni = get_appIni();

	fR = TForm_CreatParamAlg(parent, sizeof(TFormR));
	fR->wtyp = wtyp_TFormR;
//	fR->Visible = False;
	fR->FormZStyle = fszTransient;
	fR->FormPosition = poDesigned;
	fR->Gruppe = Gruppe;
	fR->sR = sR;
	fR->sOpt = pcadd(sR, "opt");
	fR->sXY = pcadd(sR, "xy");
	fR->FormSave = TFormRLC_Save;
	fR->SaveData = fR_SaveDaten;
	fR->LoadData = LoadData;
	fR->SetFreq = Form_R_SetFreq;

	SetCaption((TWins*)fR, Caption);
	fR->width = 226;
	fR->height = 137;
	fR->min_width = fR->width;
	fR->min_height = fR->height;
	fR->max_width = fR->width;
	fR->max_height = fR->height;
	sopt = TpcIni_RD_pc(pcIni, Gruppe, fR->sXY, "0 0");
	fR->left = ppctoint(&sopt);
	fR->top = ppctoint(&sopt);
	fR->tag = (ppctoint(&sopt) != 0);
	fR->Visible = False; //fR->tag;
	fR->OnMove = Form_R_OnMove;
	fR->exDVal = exDVal;


	fR->pnl1 = TWins_CreateBA((TWins*)fR, 0, 0, 134, 21, 0, alLeft);
	fR->pnlR = TWins_CreateBA(fR->pnl1, 0, 0, 134, 61, mBorder | mBorderL, alTop);
	fR->pnlOpt = TWins_CreateBA(fR->pnl1, 0, 20, 130, 80, mBorder | mBorderL, alClient);
	fR->pnlZ = TWins_CreateBA((TWins*)fR, 136, 0, 130, 80, mBorder | mBorderL, alClient);

/* Panel R */
	fR->pnlR->OnPaint = fR_pnlR_OnPaint;
	fR->R = TDVal_CreateS((TWins*)fR, 22, 2, 70, 21);
	fR->R->nk = 1;
	fR->R->dv_fu = tdv_fu_LostFocusEnter;
	fR->R->OnValChange = fR_DVal_OnValChange;
	SetFont((TWins*)fR->R, "-*-helvetica-bold-r-*-*-14-*-75-75-*-*-*-*");
	if (fR->exDVal) fR->exDVal->owner = (TWins*)fR->R;

/* Panel Optionen */
	fR->pnlOpt->OnPaint = fR_pnlOpt_OnPaint;
	fR->cbOptOn = TChkBox_CreateS(fR->pnlOpt, 2, 2, 56, 16, "Opt On");
	fR->cbOptOn->OnClick = fR_cb_OnClick;
	fR->cbLs = TChkBox_CreateS(fR->pnlOpt, 2, 20, 16, 16, NULL);
	fR->cbLs->OnClick = fR_cb_OnClick;
	fR->cbCp = TChkBox_CreateS(fR->pnlOpt, 2, 58, 16, 16, NULL);
	fR->cbCp->OnClick = fR_cb_OnClick;
	fR->Ls = TDVal_CreateS(fR->pnlOpt, 36, 18, 60, 18);
	fR->Ls->nk = 3;
	fR->Ls->dv_fu = tdv_fu_LostFocusEnter;
	fR->Ls->OnValChange = fR_DVal_OnValChange;
	fR->Cp = TDVal_CreateS(fR->pnlOpt, 36, 56, 60, 18);
	fR->Cp->nk = 2;
	fR->Cp->dv_fu = tdv_fu_LostFocusEnter;
	fR->Cp->OnValChange = fR_DVal_OnValChange;
//	fR->cbTR = TChkBox_CreateS(fR->pnlOpt, 86, 2, 32, 16, "TR");
//	fR->cbTR->OnClick = fR_cb_OnClick;
	fR->sbTR = TScrollBarVert_CreateP(fR->pnlOpt);
	fR->sbTR->Align = alRight;
	fR->sbTR->OnCount = fR_sbTR_OnCount;
	TScrollBar_SetMaxCount(fR->sbTR, 1000);

/* Panel Z */
	fR->pnlZ->OnPaint = fR_pnlZ_OnPaint;
	fR->pnlZ->OnResize = fR_pnlZ_OnResize;
	fR->MHz = TDVal_CreateS(fR->pnlZ, 12, 3, 56, 18);
	fR->MHz->nk = 1;
	fR->MHz->dv_fu = tdv_fu_LostFocusEnter;
	fR->MHz->OnValChange = fR_DVal_OnValChange;

	fR->Z = TDVal_CreateS(fR->pnlZ, 12, 22, 58, 18);
	fR->Z->nk = 2;
	fR->Z->ReadOnly = True;
	fR->Z->KeyFu |= mNoKeyFocus;
	SetColor((TWins*)fR->Z, 0xe8e8e8);

	fR->cbTRsync = TChkBox_CreateS(fR->pnlZ, 2, 101, 60, 16, "TLsync");
	fR->cbTRsync->OnClick = fR_cb_OnClick;
	fR->TR = TDVal_CreateS(fR->pnlZ, 2, 121, 58, 18);
	fR->TR->nk = 1;
	fR->TR->ReadOnly = True;
	fR->TR->KeyFu |= mNoKeyFocus;
	SetColor((TWins*)fR->TR, 0xe8e8e8);
	fR->TRmin = TDVal_CreateS(fR->pnlZ, 2, 121, 58, 18);
	fR->TRmin->nk = 2;
	fR->TRmin->dv_fu = tdv_fu_LostFocusEnter;
	fR->TRmin->OnValChange = fR_DVal_OnValChange;
	TDVal_Set_Min(fR->TRmin, 0.01);


	TDVal_Set_Value(fR->R, val);
	fR_LoadData(fR, pcIni);
	free_appIni(pcIni);

	fR_TR_SetVal(fR, TScrollBar_GetCount(fR->sbTR));

	fR_math_loc(fR);

	return fR;
}

