/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "sysbase.h"
#include "math_line.h"
#include <math.h>

static const byte C_txt[256] = {
  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,
  0x01,0x01,0x01,0x01 ,0x01,0x01,0x01,0x01 ,0x01,0x01,0x00,0x00 ,0x00,0x00,0x00,0x00,

  0x00,0x01,0x01,0x01 ,0x01,0x01,0x01,0x01 ,0x01,0x01,0x01,0x01 ,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01 ,0x01,0x01,0x01,0x01 ,0x01,0x01,0x01,0x00 ,0x00,0x00,0x00,0x01,
  0x00,0x01,0x01,0x01 ,0x01,0x01,0x01,0x01 ,0x01,0x01,0x01,0x01 ,0x01,0x01,0x01,0x01,
  0x01,0x01,0x01,0x01 ,0x01,0x01,0x01,0x01 ,0x01,0x01,0x01,0x00 ,0x00,0x00,0x00,0x00,

  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,

  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00,
  0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00 ,0x00,0x00,0x00,0x00};

typedef struct {
  int lg;
  const char *name;
  double (*funkt)(double v);
} TMathFunk1op;

static TMathFunk1op Tml1op_Tab[] = {
  {3, "sin", sin}, {3, "cos", cos}, {3, "tan", tan},
  {3, "log", log}, {3, "exp", exp},
  {4, "sqrt", sqrt}, {4, "asin", asin}, {4, "acos", acos},
  {4, "atan", atan}, {4, "sinh", sinh}, {4, "cosh", cosh},
  {4, "tanh", tanh}, {4, "logb", logb},
  {5, "log10", log10}, 
#ifndef _OS_WIN32
  {5, "exp10", exp10}, {5, "pow10", pow10},
#endif
  {5, "round", round}, {5, "trunc", trunc},
  {0, NULL, NULL}
};

static bool GetMatF_1op(char *name, int lg, double val, double *ret) {
  TMathFunk1op *ftab = Tml1op_Tab;

	while (ftab->lg) {
	    if ((ftab->lg == lg) && !pcncasecmpASCII(ftab->name, name, lg)) {
		if (ret) *ret = ftab->funkt(val);
		return True;
	    }
	    ftab++;
	}
	return False;
} 

typedef struct {
  int lg;
  const char *name;
  double v;
} TMathKonst;

static TMathKonst TmlKonst_Tab[] = {
  {1, "e", M_E},
  {2, "pi", M_PI},
  {3, "ln2", M_LN2},
  {4, "ln10", M_LN10},
  {0, NULL, 0}
};

static bool ml_get_Konst(TMathLineStruct *ms, char *name, int lg, double *ret) {
  TMathLine *ml = ms->ml;
  TMathKonst *ktab;
	while (ml) {
	    if ((ml->name_lg == lg) && pcncmp(ml->res_name, name, lg)) {
		if (ret) *ret = ml->res;
		return True;
	    }
	    ml = ml->next;
	}

	ktab = TmlKonst_Tab;
	while (ktab->lg) {
	    if ((ktab->lg == lg) && !pcncasecmpASCII(ktab->name, name, lg)) {
		if (ret) *ret = ktab->v;
		return True;
	    }
	    ktab++;
	}
	if (ret) *ret = 0;
	return False;
}

static void skip_space_p(const char **ppc) {
  const char *pc = *ppc;
	while (*pc && ((*pc == ' ') || (*pc == 9))) pc++;
	*ppc = pc;
}

static bool get_Val(char *pc, int lg, char **ppc, double *val) {
  char *pcn;
  double f = 0;
	pcn = pc;
	if (((*pc >= '0') && (*pc <= '9')) || 
	    ((*pc == '.') && ((pc[1] >= '0') && (pc[1] <= '9')))) {

	    if ((*pc == '0') && (pc[1] == 'x')) {
		f = ppctoint((void*)&pcn);
	    } else {
		f = ppctodouble((void*)&pcn);
	    }	    
	} else return False;

	if (val) *val = f;
	if (ppc) *ppc = pcn;
	return True;
}

static bool get_EndVal(TMathLineStruct *ms, char **ppc, int *plg) {
  int n;
  int lg = *plg;
  char *pcn, *pc = *ppc;
  double f = 0;
  bool neg = False;

	if (*pc == '-') {neg = True; pc++; lg--;}
	while (lg && (*pc == ' ')) {pc++; lg--;}
	if (lg<1) goto Error;

	if ((byte)*pc == 0xff) {
	    if (lg<2) goto Error;
	    n = (byte)pc[1] - 0x20;
	    if ((n<0) || (n>=100)) goto Error;
	    f = ms->f[n];
	    pc += 2; lg -= 2;

	} else if (get_Val(pc, lg, &pcn, &f)) {
	    lg -= pcn-pc;
	    pc = pcn;
	    if (lg<0) lg = 0;

	} else {
	    pcn = pc;
	    while (lg && C_txt[(byte)*pcn]) {pcn++; lg--;}
	    n = pcn-pc;
	    if (!ml_get_Konst(ms, pc, n, &f)) goto Error;
	    pc = pcn;
	}

	if (neg) f = -f;
	ms->tmp_res = f;
	*ppc = pc;
	*plg = lg;
	return True;
Error:
	ms->stat |= mlst_Error;
	ms->err_X = ms->err_X_off + (pc - ms->pct);
	return False;
}

static bool AddTmpVal(TMathLineStruct *ms, char *pc, int lg) {
  int n;

	if ((byte)*pc == 0xff) {
	    n = (byte)pc[1] - 0x20;
	    ms->f[n] = ms->tmp_res;
	} else {
	    if (ms->fidx >= 100) {
		ms->err_X = ms->err_X_off + (pc - ms->pct);
		return False;
	    }
	    ms->f[ms->fidx] = ms->tmp_res;
	    *pc = 0xff;
	    pc[1] = ms->fidx + 0x20;
	    ms->fidx++;
	}
	pc += 2; lg -= 2;
	for (;lg>0;lg--) {*pc = ' '; pc++;}
	return True;
}

static bool ml_scan_seg(TMathLineStruct *ms, char *pc, int lg, int stat) {
  TMathLine *ml =  ms->akt;
  char *pcx, *pcxn, *pca;
  int x, xn, m, n;
  double f = 0;
  char c;

	x = lg;
	pcx = pc;
	while (x > 0) {
	    if (*pcx == ' ') {
		pcx++; x--;

	    } else if (*pcx == '(') {
		m = 1;
		xn = x;
		pcxn = pcx;
		while (m && xn) {
		    pcxn++; xn--;
		    if (*pcxn == '(') m++;
		    if (*pcxn == ')') m--;
		}
		m = x-xn-1;
		if (!ml_scan_seg(ms, pcx+1, m, 0)) return False;
		if (!AddTmpVal(ms, pcx, m+2)) return False;
		pcx = pcxn;
		x = xn;
		pcx++; x--;

	    } else if ((*pcx == '+') || (*pcx == '-') || 
		       (*pcx == '*') || (*pcx == '/')) {
		pcx++; x--;

	    } else if (get_Val(pcx, x, &pcxn, NULL)) {
		x -= pcxn-pcx;
		pcx = pcxn;

	    } else {

		pca = pcx;
		while (x && C_txt[(byte)*pcx]) {pcx++; x--;}
		n = pcx-pca;
		if (n<1) goto Error;
		while (x && *pcx == ' ') {pcx++; x--;}
		if (*pcx == '(') {
		    m = 1;
		    xn = x;
		    pcxn = pcx;
		    pcxn++; xn--;
		    while (m && xn) {
			pcxn++; xn--;
			if (*pcxn == '(') m++;
			if (*pcxn == ')') m--;
		    }
		    m = x-xn-1;
		    if (!ml_scan_seg(ms, pcx+1, m, 0)) return False;

		    if (!GetMatF_1op(pca, n, ms->tmp_res, &ms->tmp_res)) {
			pcx = pca;
			goto Error;
		    }
		    if (!AddTmpVal(ms, pca, pcxn-pca+1)) return False;
		    pcx = pcxn;
		    x = xn;
		    pcx++; x--;
		}

	    }
	}

	x = lg;
	pcx = pc;
	pca = pcx;
	while (x > 0) {
	    c = *pcx;
	    if ((c == '+') || (c == '-')) {
		pcx++; x--;
		pca = pcx;

	    } else if ((c == '*') || (c == '/')) {
		n = pcx-pca;
		pcx++; x--;

		pcxn = pca;
		if (!get_EndVal(ms, &pcxn, &n)) return False;
		f = ms->tmp_res;
		if (!get_EndVal(ms, &pcx, &x)) return False;
		if (c == '*') ms->tmp_res *= f; else ms->tmp_res = f / ms->tmp_res;
		if (!AddTmpVal(ms, pca, pcx-pca)) return False;

	    } else {
		pcx++; x--;
	    }
	}

	x = lg;
	pcx = pc;
	if (!get_EndVal(ms, &pcx, &x)) return False;

	f = ms->tmp_res;
	while (x > 0) {
	    if (*pcx == ' ') {
		pcx++; x--;

	    } else if (*pcx == '+') {
		pcx++; x--;
		if (!get_EndVal(ms, &pcx, &x)) return False;
		f += ms->tmp_res;

	    } else if (*pcx == '-') {
		pcx++; x--;
		if (!get_EndVal(ms, &pcx, &x)) return False;
		f -= ms->tmp_res;

	    } else goto Error;
	}

	ml->res = f;
	ms->tmp_res = f;
	return True;
Error:
	ms->stat |= mlst_Error;
	ms->err_X = ms->err_X_off + (pcx - ms->pct);
	return False;
}


int math_line_scan(TMathLineStruct *ms, TMathLine *ml) {
  const char *pc, *pca;
  char *pcx;
  int lg;

	if (!ml) return ml_nop;
	ml->t = ml_nop;
	ms->fidx = 0;
	ms->err_X_off = 0;
	ms->error = 0;
	ms->stat &= mlst_RunMask;
	pc = ml->pc;
	if (!pc || ((byte)*pc < 0x20 && (*pc != 0x09))) return ml_nop;
	skip_space_p(&pc);
	if (!*pc || (*pc == ';') || (*pc == '#')) return ml_nop;
	if ((*pc == '/') && (pc[1] == '/')) return ml_nop;

	if (*pc == '.') {
	    pc++;
	    ms->stat |= mlst_hidden;
	}
	if (!C_txt[(byte)*pc]) goto Error;
	pca = pc;

	while (*pc && C_txt[(byte)*pc]) pc++;
	ml->name_lg = pc - pca;
	ml->res_name = pcndup(pca, ml->name_lg);
	skip_space_p(&pc);
	if (*pc != '=') goto Error;
	pc++;
	skip_space_p(&pc);
	pca = pc;
	while ((*pc > 0x1f || *pc == 0x09) && (*pc != ';') && 
	       ((*pc != '/') || (pc[1] != '/'))) pc++;

	lg = pc-pca;
	if (lg <= 0) goto Error;

	ms->err_X_off = pca - ml->pc;
	pcx = pcdup(pca);
	ms->pct = pcx;
	while (*pcx) {
	    if (*pcx == 0x09) *pcx = ' ';
	    pcx++;
	}
	if (!ml_scan_seg(ms, ms->pct, lg, 0)) return ml_Error;
	xfree(ms->pct);
	ms->pct = NULL;
	ms->res = ml->res;
	ms->res_name = ml->res_name;

	return ml_konst;
Error:
	ms->stat |= mlst_Error;
	ms->err_X = pc - ml->pc;
	return ml_Error;
}

bool math_line(TMathLineStruct *ms, const char *pc) {
  TMathLine *ml, *mltmp, **ppml;
  char *pct, *pcx;
  int i, ml_res;
  double dk;
  char buf[32];

	ms->line_nr++;
	ml = xcalloc(sizeof(TMathLine));
	ml->pc = pc;
	ms->akt = ml;
	ml_res = math_line_scan(ms, ml);
	ms->ml_res = ml_res;
	if (ml_res < 0) {
	    if (ms->OnResult) {
		ms->OnResult(ms, ml_res);
	    } else if (!(ms->stat & mlst_print_Error_off)) {
		pct = pcdup(pc);
		pcx = pct;
		i = 0;
		while (*pcx) {
		    if (i == ms->err_X) *pcx = '^'; else *pcx = ' ';
		    i++; pcx++;
		}
		printf("*************************\n");
		printf("ERROR: line=%d pos=%d\n", ms->line_nr, ms->err_X);
		printf("%s\n", pc);
		printf("%s\n", pct);
		printf("*************************\n");
		xfree(pct);
	    }
	    xfree(ml->res_name);
	    xfree(ml);
	    return False;
	}
	if (ml_res == ml_nop) {
	    if (ms->OnResult) ms->OnResult(ms, ml_res);
	    xfree(ml->res_name);
	    xfree(ml);
	    return True;
	}

	ml->t = ml_res;
	if (!ms->ml) {
	    ms->ml = ml;
	} else {
	    ppml = &ms->ml;
	    mltmp = ms->ml;
	    while (mltmp && !pccmp(ml->res_name, mltmp->res_name)) {
	        ppml = &mltmp->next;
		mltmp = mltmp->next;
	    }
	    if (mltmp) {
		mltmp->res = ml->res;
		ms->res_name = mltmp->res_name;
		xfree(ml->res_name);
		xfree(ml);
	    } else {
		*ppml = ml;
	    }
	}

	if (ms->OnResult) {
	    ms->OnResult(ms, ml_res);
	} else if (ms->stat & mlst_print_all_sum) {
	    if (!(ms->stat & mlst_hidden)) {
		dk = 999999999;
		i = 0;
		while ((i<12) && (dk > ms->res)) {
		    i++;
		    dk *= 0.1;
		}
		pcx = buf;
		ppcmove(&pcx, "%s=%.");
		inttoppc(&pcx, i);
		ppcmove(&pcx, "f\n");
		printf(buf, ms->res_name, ms->res);
	    }
	}

	return True;
}

void math_line_clr(TMathLineStruct *ms) {
  TMathLine *ml, *mltmp;
	ml = ms->ml;
	while (ml) {
	    xfree(ml->res_name);
	    mltmp = ml;
	    ml = ml->next;
	    xfree(mltmp);
	}
	ms->ml = NULL;
	ms->err_X = 0;
}

void math_line_free(TMathLineStruct *ms) {
	math_line_clr(ms);
	xfree(ms);
}
