/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "ElMath.h"
#include "ElGraf.h"
#include <math.h>

bool XF_MathOut(TPage_XFilter *self, float w, TRLC *btIn, float *ret) {
  float res;
  bool cbCxOn, cbLxOn, cbRxOn, cbCsOn, cbLsOn, cbRsOn;

	cbCxOn = self->cbCxOn && self->cbCxOn->Checked;
	cbLxOn = self->cbLxOn && self->cbLxOn->Checked;
	cbRxOn = self->cbRxOn && self->cbRxOn->Checked;
	cbCsOn = self->cbCsOn && self->cbCsOn->Checked;
	cbLsOn = self->cbLsOn && self->cbLsOn->Checked;
	cbRsOn = self->cbRsOn && self->cbRsOn->Checked;

	if (w < 0.000001) {
	    if (cbCxOn || cbCsOn) return 0;
	    res = self->f_Rs->R->dval;
	    if (cbRxOn) res += self->f_Rx->R->dval;
	    *ret = self->f_Ra->R->dval / (self->f_Ra->R->dval + res);
	    return False;
	}
	Form_R_math(self->f_Ra, w);

	res = 1;
	RLC_Copy(btIn, &self->f_Ra->rlc);
	if (cbCxOn || cbLxOn || cbRxOn) {
	    res = btIn->Z;
	    if (self->cbInvOn && self->cbInvOn->Checked) {
		if (cbCsOn) {
		    Form_C_math(self->f_Cs, w);
		    RLC_Add_Ser(btIn, &self->f_Cs->rlc);
		}
		if (cbLsOn) {
		    Form_L_math(self->f_Ls, w);
		    RLC_Add_Ser(btIn, &self->f_Ls->rlc);
		}
		if (cbRsOn) {
		    Form_R_math(self->f_Rs, w);
		    RLC_Add_Ser(btIn, &self->f_Rs->rlc);
		}
	    } else {
		if (cbCxOn) {
		    Form_C_math(self->f_Cx, w);
		    RLC_Add_Ser(btIn, &self->f_Cx->rlc);
		}
		if (cbLxOn) {
		    Form_L_math(self->f_Lx, w);
		    RLC_Add_Ser(btIn, &self->f_Lx->rlc);
		}
		if (cbRxOn) {
		    Form_R_math(self->f_Rx, w);
		    RLC_Add_Ser(btIn, &self->f_Rx->rlc);
		}
	    }
	    res /= btIn->Z;
	}
	*ret = res;
	return True;
}

void XF_MathIn(TPage_XFilter *self, float w, TRLC *btIn, float *ret, float *Z) {
  float res;

	if (self->cbInvOn && self->cbInvOn->Checked) {
	    if (self->cbCxOn && self->cbCxOn->Checked) {
		Form_C_math(self->f_Cx, w);
		RLC_Add_Ser(btIn, &self->f_Cx->rlc);
	    }
	    if (self->cbLxOn && self->cbLxOn->Checked) {
		Form_L_math(self->f_Lx, w);
		RLC_Add_Ser(btIn, &self->f_Lx->rlc);
	    }
	    *Z = 0;
	    if (self->cbRxOn && self->cbRxOn->Checked) {
		Form_R_math(self->f_Rx, w);
		RLC_Add_Ser(btIn, &self->f_Rx->rlc);
	    }

	} else {
	    if (self->cbCsOn && self->cbCsOn->Checked) {
		Form_C_math(self->f_Cs, w);
		RLC_Add_Ser(btIn, &self->f_Cs->rlc);
	    }
	    if (self->cbLsOn && self->cbLsOn->Checked) {
		Form_L_math(self->f_Ls, w);
		RLC_Add_Ser(btIn, &self->f_Ls->rlc);
	    }
	    *Z = 0;
	    if (self->cbRsOn && self->cbRsOn->Checked) {
		Form_R_math(self->f_Rs, w);
		RLC_Add_Ser(btIn, &self->f_Rs->rlc);
	    }
	}
	if (self->f_Rg && self->f_Rg->R->dval > 0) {
	    *Z = btIn->Z;
	    Form_R_math(self->f_Rg, w);
	    RLC_Add_Ser(btIn, &self->f_Rg->rlc);
	}
	res = *ret;
	res /= btIn->Z;
	*ret = res;
}

void DrawTP1_Lh(Window WinID, int *x, int y, TPic *pic) {
  int x2 = *x;
	DrawPic(WinID, get_Pic_Lh(), x2, y-5);
	DrawPic(WinID, pic, x2+5, y+5);
	x2 += 20;
	*x = x2;
}

void DrawTP1_Ch(Window WinID, int *x, int y, TPic *pic) {
  int x2 = *x;
	DrawPic(WinID, get_Pic_Ch(), x2, y-5);
	DrawPic(WinID, pic, x2+5, y+7);
	x2 += 20;
	*x = x2;
}

void DrawTP1_Cv_pt(Window WinID, int *x, int xo, int y, TPic *pic, int pt) {
  int x2 = *x;

	if (pt & PT_TOP) {
	    DrawLineRGB(WinID, 0, x2, y, x2+10+xo, y);
	    x2 += xo;
	    DrawPic(WinID, get_Pic_pt(), x2+3, y-2);
	} else {
	    x2 += xo;
	}

	if (pt & PTL_TOP) {
	    DrawLineRGB(WinID, 0, x2+5, y+3, x2+5, y+4);
	    y += 4;
	}
	DrawPic(WinID, get_Pic_Cv(), x2, y);
	y += 14;
	DrawPic(WinID, pic, x2+7, y);
	y += 6;
	if (pt & PTL_BOTTOM) {
	    DrawLineRGB(WinID, 0, x2+5, y, x2+5, y+4);
	    y += 4;
	} else {
	    DrawLineRGB(WinID, 0, x2+5, y, x2+5, y+2);
	    y += 2;
	}
	if (pt & PT_BOTTOM) {
	    DrawPic(WinID, get_Pic_pt(), x2+3, y);
	}
	x2 += 10;
	*x = x2;
}

void DrawTP1_Cv(Window WinID, int *x, int xo, int y, TPic *pic) {
	DrawTP1_Cv_pt(WinID, x, xo, y, pic, PT_ALL);
}

void DrawTP1_Lv_pt(Window WinID, int *x, int xo, int y, TPic *pic, int pt) {
  int x2 = *x;
//  int y2 = y+30;
	if (pt & PT_TOP) {
	    DrawLineRGB(WinID, 0, x2, y, x2+10+xo, y);
	    x2 += xo;
	    DrawPic(WinID, get_Pic_pt(), x2+3, y-2);
	} else {
	    x2 += xo;
	}

	if (pt & PTL_TOP) {
	    DrawLineRGB(WinID, 0, x2+5, y+3, x2+5, y+5);
	    y += 5;
	}
	DrawPic(WinID, get_Pic_Lv(), x2, y);
	y += 13;
	DrawPic(WinID, pic, x2+10, y);
	y += 7;
	if (pt & PTL_BOTTOM) {
	    DrawLineRGB(WinID, 0, x2+5, y, x2+5, y+3);
	    y += 3;
	} else {
	    DrawLineRGB(WinID, 0, x2+5, y, x2+5, y+2);
	    y += 2;
	}
	if (pt & PT_BOTTOM) {
	    DrawPic(WinID, get_Pic_pt(), x2+3, y);
	}
	x2 += 10;
	*x = x2;
}

void DrawTP1_Lv(Window WinID, int *x, int xo, int y, TPic *pic) {
	DrawTP1_Lv_pt(WinID, x, xo, y, pic, PT_ALL);
}

void Draw_xP1_Rg(Window WinID, int *px2, int *pxa, int y, int y2, int fu) {
  int x2 = *px2;
  int xa;

	DrawLineRGB(WinID, 0, x2+10, y, x2+14, y);
	DrawLineRGB(WinID, 0, x2+10, y, x2+10, y+4);
	DrawPic(WinID, get_Pic_Rg(), x2, y+5);
	DrawLineRGB(WinID, 0, x2+10, y2, x2+14, y2);
	DrawLineRGB(WinID, 0, x2+10, y+25, x2+10, y2);

	x2 += 20;

	DrawPic(WinID, get_Pic_pto(), x2-3, y-3);
	DrawPic(WinID, get_Pic_pto(), x2-3, y2-3);
	xa = x2+3;
	x2 += 4;

	if ((fu & MF_INV) == 0) {
	  if (fu & MF_CS) {
	    DrawLineRGB(WinID, 0, x2, y, x2+4, y);
	    x2 += 5;
	    DrawPic(WinID, get_Pic_Ch(), x2, y-5);
	    DrawPic(WinID, get_Pic_cCs(), x2, y+5);
	    x2 += 20;
	  } else {
	    DrawLineRGB(WinID, 0, x2, y, x2+5, y);
	    x2 += 6;
	  }

	  if (fu & MF_LS) {
	    DrawPic(WinID, get_Pic_Lh(), x2, y-5);
	    DrawPic(WinID, get_Pic_cLs(), x2+5, y+5);
	    x2 += 20;
	  }	
	  if (fu & MF_RS) {
	    DrawPic(WinID, get_Pic_Rh(), x2, y-5);
	    DrawPic(WinID, get_Pic_cRs(), x2+5, y+5);
	    x2 += 20;
	  }
	} else {
	  if (fu & MF_CX) {
	    DrawLineRGB(WinID, 0, x2, y, x2+4, y);
	    x2 += 5;
	    DrawPic(WinID, get_Pic_Ch(), x2, y-5);
	    DrawPic(WinID, get_Pic_cCx(), x2, y+5);
	    x2 += 20;
	  } else {
	    DrawLineRGB(WinID, 0, x2, y, x2+5, y);
	    x2 += 6;
	  }

	  if (fu & MF_LX) {
	    DrawPic(WinID, get_Pic_Lh(), x2, y-5);
	    DrawPic(WinID, get_Pic_cLx(), x2+5, y+5);
	    x2 += 20;
	  }	
	  if (fu & MF_RX) {
	    DrawPic(WinID, get_Pic_Rh(), x2, y-5);
	    DrawPic(WinID, get_Pic_cRx(), x2+5, y+5);
	    x2 += 20;
	  }
	}
	*px2 = x2;
	*pxa = xa;
}

void Draw_xP1_R_Out(Window WinID, int x2, int xa, int y, int y2, int fu, TPic *pic) {
  int yo, y2o;
/* Cx */
	if ((fu & MF_INV) == 0) {
	  if (fu & MF_CX) {
	    DrawPic(WinID, get_Pic_Ch(), x2, y-5);
	    DrawPic(WinID, get_Pic_cCx(), x2+14, y+5);
	    x2 += 15;
	  }

	  if (fu & (MF_RX|MF_LX)) {
	    if (fu & MF_CX) x2 += 4;
	    x2 += 1;
	    if (fu & MF_LX) {
	      DrawPic(WinID, get_Pic_Lh(), x2, y-5);
	      DrawPic(WinID, get_Pic_cLx(), x2+10, y+5);
	      x2 += 20;
	    }
	    if (fu & MF_RX) {
	      DrawPic(WinID, get_Pic_Rh(), x2, y-5);
	      DrawPic(WinID, get_Pic_cRx(), x2+10, y+5);
	      x2 += 18;
	    }
	  }
	} else {
	  if (fu & 0x01) {
	    DrawPic(WinID, get_Pic_Ch(), x2, y-5);
	    DrawPic(WinID, get_Pic_cCs(), x2+14, y+5);
	    x2 += 15;
	  }

	  if (fu & 0x02) {
	    if (fu & 0x04) x2 += 4;
	    x2 += 1;
	    DrawPic(WinID, get_Pic_Rh(), x2, y-5);
	    DrawPic(WinID, get_Pic_cRs(), x2+14, y+5);
	    x2 += 18;
	  }
	}

	x2 += 7;
	DrawLineRGB(WinID, 0, xa, y2, x2, y2);
	xa = x2-7;
	DrawLineRGB(WinID, 0, xa, y, x2, y);
	DrawPic(WinID, get_Pic_pto(), x2, y-3);
	DrawPic(WinID, get_Pic_pto(), x2, y2-3);

	x2 += 10;
	DrawLineRGB(WinID, 0, x2, y, x2+5, y);
	DrawLineRGB(WinID, 0, x2, y2, x2+5, y2);

	yo = (y2 - y - 20) / 2;
	DrawPic(WinID, get_Pic_Rv(), x2, y + yo);
	DrawPic(WinID, pic, x2+10, y + yo + 7);
	x2 += 5;
	if (yo > 0) DrawLineRGB(WinID, 0, x2, y, x2, y + yo);

	y2o = y2 - y - 20 - yo;
	if (y2o > 0) DrawLineRGB(WinID, 0, x2, y2 - y2o, x2, y2);
}

void Draw_xP1_Ra(Window WinID, int x2, int xa, int y, int y2, int fu) {
	Draw_xP1_R_Out(WinID, x2, xa, y, y2, fu, get_Pic_cRa());
}

static void str_toGraf(TWins *Wins, const char *s, int x, int y) {
	vds_SetStr(&Wins->vds, s);
	DrawColorString(x, y, &Wins->vds);
}

static void ELM_TP1_pnlGrafVal_OnPaint(TWins *self) {
  TPage_Filter *pg = self->subClass;
  int x = 2;
  char *s;
	str_toGraf(self, "Freq", x, 3);
	x =  pg->cbmaxMHZ->left + pg->cbmaxMHZ->width + 2;
	s = pcadd(pg->xy_ind->dat->dx->s, " +");
	str_toGraf(self, s, x, 3);
	xfree(s);
	str_toGraf(self, "dB", pg->dv_TP1_Off_dB->left+pg->dv_TP1_Off_dB->width+2, 3);
}

static void SetCursor_sbDetail(TPage_Filter *pg) {
  int nk = 1;
  char *pc;
  float freq;
  char buf[64];

	pg->xy_ind->CursorX = pg->sbDetail->Count * pg->ScaleMHz / 
			      pg->sbDetail->MaxCount;

	freq = TScaleXY_GetCursX_ScaleVal(pg->xy_ind);
	if (pg->ScaleMHz < 1) {
	    pc = buf;
	    floattoppc(&pc, freq * 1000, 3 | kSPACE_ADD);
	    ppcmove(&pc, "kHz");
	} else {
	    if (pg->ScaleMHz < 2000) nk = 2;
	    if (pg->ScaleMHz < 200) nk = 3;
	    if (pg->ScaleMHz < 20) nk = 4;
	    if (pg->ScaleMHz < 2) nk = 5;
	    pc = buf;
	    floattoppc(&pc, freq, nk | kSPACE_ADD);
	    ppcmove(&pc, "MHz");
	}
	ppcchange(&pg->xy_ind->CursorX_Str, buf);

  TWins *wins = appXDat.Wins;
	while (wins) {
	    if (((wins->wtyp == wtyp_TFormR) || 
		 (wins->wtyp == wtyp_TFormL) ||
		 (wins->wtyp == wtyp_TFormC)) &&
		wins->subClass &&
		(wins->subClass == pg->pgc_Filter->AktPage)) {
		if (((TFormRLC*)wins)->SetFreq) {
		    ((TFormRLC*)wins)->SetFreq((TFormRLC*)wins, freq);
		}
	    }
	    wins = wins->next;
	}

	SetCursor_Dat_txt(pg);
}


static void EML_Filter_TP1_Off_dB_OnValChange(TDVal *dv, double val) {
  TPage_Filter *pg = dv->subClass;
	pg->xy_ind->XY_Dat_yOff = val;
	SetCursor_Dat_txt(pg);
	Refresh((TWins*)pg->xy_ind);
}
static TPage *ELM_pageFilter_Create(TPageControl *Parent) {
	return TPage_CreateParamAlg(Parent, sizeof(TPage_Filter));
}

/* ****************** Graf ****************************************************/
static void ELM_TP1_pnlGrafVal_OnResize(TWins *self) {
  TPage_Filter *pg = self->subClass;
  int x;
	vds_SetStr(&self->vds, "Freq");
	x = TextWidthColorString(&self->vds) + 4;
	SetLeft((TWins*)pg->cbmaxMHZ, x);
	vds_SetStr(&self->vds, "MHz");
	x += pg->cbmaxMHZ->width + TextWidthColorString(&self->vds) + 4;

	SetLeft((TWins*)pg->dv_TP1_Off_dB, x);

	vds_SetStr(&self->vds, "dB");
	x += pg->dv_TP1_Off_dB->width + TextWidthColorString(&self->vds) + 10;
	SetLeft((TWins*)pg->cbRgRa, x);

	x += pg->cbRgRa->width + 2;
	SetLeft((TWins*)pg->cbdB140, x);

	x = self->width - pg->cbLog->width - 2;
	SetLeft((TWins*)pg->cbLog, x);
}

static void EML_Filter_xy_ind_OnResize(TScaleXY *self) {
  TPage_Filter *pg = self->subClass;
	SetWidth((TWins*)pg->sbDetail, self->width - pg->xy_ind->sx2 + 22);
}


void SetCursor_Dat_txt(TPage_Filter *pg) {
  int nk;
  char *pc;
  TScaleXY_Txt *xy_txt;
  TPage_XFilter *pgx;
  float yVal;
  TMgsMathImp MgsMathImp;
  char buf[128];

	xy_txt = pg->xy_ind->txt;
	while (xy_txt) {
	    xfree(xy_txt->s);
	    xy_txt->s = NULL;
	    xy_txt = xy_txt->next;
	}

	if (!pg->sbDetail->Count) return;
	MgsMathImp.MHz = TScaleXY_GetCursX_ScaleVal(pg->xy_ind);
	pgx = (TPage_XFilter*)pg->pgc_Filter->AktPage;
	if (pgx && (pgx->art == PAGE_ART_FILTER) && pgx->OnMsg) {
	    if (pgx->OnMsg(pgx, MSG_MATH_IMP, &MgsMathImp)) {
		if (pg->cbRgRa->Checked) {
		    MgsMathImp.yVal *= (pgx->f_Rg->rlc.val + pgx->f_Ra->rlc.val) / pgx->f_Ra->rlc.val;
		}
		pc = buf;
		ppcmove(&pc, "IN Imp = ");
		nk = 0;
		if (MgsMathImp.Z < 100) nk = 1;
		floattoppc(&pc, MgsMathImp.Z, nk);
		xy_txt = pg->xy_ind->txt;
		ppcchange(&xy_txt->s, buf);

		pc = buf;
		xy_txt = xy_txt->next;
		ppcmove(&pc, "I/O = ");
		yVal = log10(MgsMathImp.yVal)*20 + pg->xy_ind->XY_Dat_yOff;
		floattoppc(&pc, yVal, 1);
		ppcmove(&pc, " dB");
		ppcchange(&xy_txt->s, buf);
	    }
	}
}

static void EML_Filter_sbDetail_OnCount(TScrollBar* sb, int Count) {
  TPage_Filter *pg = sb->subClass;
// printf("%s(%d): \n", FL);
	SetCursor_sbDetail(pg);
	TWins_Refresh((TWins*)pg->xy_ind);
}

static void math_Filter_xP(TPage_Filter *pg) {
  TPage_XFilter *pgx;


	pgx = (TPage_XFilter*)pg->pgc_Filter->AktPage;
	if (pgx && pgx->OnMsg && pgx->OnMsg(pgx, MSG_MATH_FILTER, NULL)) return;
}


static void EML_Filter_sbScale_OnCount(TScrollBar* sb, int Count) {
  TPage_Filter *pg = sb->subClass;
  float off;
  int m;
  TPage_XFilter *pgx;

	m = 0;
	if (pg->ScaleMHz < 10) m = 1;
	if (pg->ScaleMHz < 1) m = 2;
	if (pg->ScaleMHz < 0.1) m = 3;
	off = Count * pg->ScaleMHz / 10;
	pg->ScaleMHzoff = off;
	TScaleXY_Set_Dat(pg->xy_ind->dat->dx, "MHz", off, pg->ScaleMHz+off, 2, pg->fuLogX | m);
	SetCursor_sbDetail(pg);
	math_Filter_xP(pg);

	pgx = (TPage_XFilter*)pg->pgc_Filter->AktPage;
	if (pgx) {
	    pgx->sbScaleCount = pg->sbScale->Count;
//	    if (pgx->OnMsg) pgx->OnMsg(pgx, MSG_NEW_SCALE_PARAM);
	}
}

static void TPage_Filter_SetScaleMHz(TPage_Filter *pg) {
  int m;
  float off;

	m = 0;
	if (pg->ScaleMHz < 10) m = 1;
	if (pg->ScaleMHz < 1) m = 2;
	if (pg->ScaleMHz < 0.1) m = 3;

	off = pg->sbScale->Count * pg->ScaleMHz / 10;
	pg->ScaleMHzoff = off;
	TScaleXY_Set_Dat(pg->xy_ind->dat->dx, "MHz", off, pg->ScaleMHz+off, 2, pg->fuLogX | m);
	SetCursor_sbDetail(pg);
	math_Filter_xP(pg);
}

static void ELM_F_cbmaxMHZ_OnClick(TMenu *me, long KeyState, TMenuItem *mi) {
  TPage_Filter *pg = me->subClass;
  int m = 0;
  float off;
  TPage_XFilter *pgx;


	if (pg->cbmaxMHZ->popup == me) {
	    off = pg->ScaleMHz;
	    pg->ScaleMHz = pctodouble(mi->s);
	    m = pg->sbScale->Count;
	    if (m > 0) {
		m = m * off / pg->ScaleMHz;
		TScrollBar_SetCount(pg->sbScale, m);
	    }
	}

	TPage_Filter_SetScaleMHz(pg);

	pgx = (TPage_XFilter*)pg->pgc_Filter->AktPage;
	if (pgx) {
	    pgx->sbScaleCount = pg->sbScale->Count;
	    pgx->ScaleMHz = pg->ScaleMHz;
	    if (pgx->OnMsg) pgx->OnMsg(pgx, MSG_NEW_SCALE_PARAM, NULL);
	}
}

static void ELM_F_cbLog_OnClick(TMenu *me, long KeyState, TMenuItem *mi) {
  TPage_Filter *pg = me->subClass;
  const char *pc;
	pg->fuLogX = sxysetfu_lin;
	pc = GetCaption_pc((TWins*)pg->cbLog);
	if (!pccasecmpASCII(pc, "log -1")) pg->fuLogX = sxysetfu_log_1; else
	if (!pccasecmpASCII(pc, "log10 -1")) pg->fuLogX = sxysetfu_log10_1;
	if (!pccasecmpASCII(pc, "log20 -1")) pg->fuLogX = sxysetfu_log20_1;
	if (!pccasecmpASCII(pc, "log100-1")) pg->fuLogX = sxysetfu_log100_1;
	TScaleXY_Set_Dat_Log(pg->xy_ind, pg->fuLogX, 0);
}

static void SaveFilterOnOff(TPage_Filter *pg, TpcIni *pcIni) {
  int OnOff = 0;
	if (pg->cbRgRa->Checked) OnOff |= 0x01;
//	if (pg->cbdB140->Checked) OnOff |= 0x02;
	TpcIni_WR_int(pcIni, "Filter", "OnOff", OnOff);
}

void XF_SaveData(TPage_XFilter *self, TpcIni *ini) {
	if (!self || !ini) return;
	if (self->f_Rg) self->f_Rg->SaveData(self->f_Rg, ini);
	if (self->f_Rs) self->f_Rs->SaveData(self->f_Rs, ini);
	if (self->f_Rx) self->f_Rx->SaveData(self->f_Rx, ini);
	if (self->f_Ra) self->f_Ra->SaveData(self->f_Ra, ini);

	if (self->f_Cs) self->f_Cs->SaveData(self->f_Cs, ini);
	if (self->f_Cx) self->f_Cx->SaveData(self->f_Cx, ini);

	if (self->f_Ls) self->f_Ls->SaveData(self->f_Ls, ini);
	if (self->f_Lx) self->f_Lx->SaveData(self->f_Lx, ini);
}

void XF_LoadData(TPage_XFilter *self, TpcIni *ini) {
	if (!self || !ini) return;
	if (self->f_Rg) self->f_Rg->LoadData(self->f_Rg, ini);
	if (self->f_Rs) self->f_Rs->LoadData(self->f_Rs, ini);
	if (self->f_Rx) self->f_Rx->LoadData(self->f_Rx, ini);
	if (self->f_Ra) self->f_Ra->LoadData(self->f_Ra, ini);

	if (self->f_Cs) self->f_Cs->LoadData(self->f_Cs, ini);
	if (self->f_Cx) self->f_Cx->LoadData(self->f_Cx, ini);

	if (self->f_Ls) self->f_Ls->LoadData(self->f_Ls, ini);
	if (self->f_Lx) self->f_Lx->LoadData(self->f_Lx, ini);
}

void XF_SaveOnOff(TPage_XFilter *self, TpcIni *pcIni) {
  int OnOff = 0;

	if (self->cbCsOn &&  self->cbCsOn->Checked)  OnOff |= MF_CS;
	if (self->cbLsOn &&  self->cbLsOn->Checked)  OnOff |= MF_LS;
	if (self->cbRsOn &&  self->cbRsOn->Checked)  OnOff |= MF_RS;
	if (self->cbCxOn &&  self->cbCxOn->Checked)  OnOff |= MF_CX;
	if (self->cbLxOn &&  self->cbLxOn->Checked)  OnOff |= MF_LX;
	if (self->cbRxOn &&  self->cbRxOn->Checked)  OnOff |= MF_RX;
	if (self->cbInvOn && self->cbInvOn->Checked) OnOff |= MF_INV;

	if (self->cbF1On && self->cbF1On->Checked) OnOff |= MF_F1;
	if (self->cbF2On && self->cbF2On->Checked) OnOff |= MF_F2;
	if (self->cbF3On && self->cbF3On->Checked) OnOff |= MF_F3;
	if (self->cbF4On && self->cbF4On->Checked) OnOff |= MF_F4;

	if (self->cbF1ImpOn && self->cbF1ImpOn->Checked) OnOff |= MF_F1I;
	if (self->cbF2ImpOn && self->cbF2ImpOn->Checked) OnOff |= MF_F2I;
	if (self->cbF3ImpOn && self->cbF3ImpOn->Checked) OnOff |= MF_F3I;
	if (self->cbF4ImpOn && self->cbF4ImpOn->Checked) OnOff |= MF_F4I;

	TpcIni_WR_int(pcIni, self->gname, "OnOff", OnOff);
}

void XF_SetVisible(TPage_XFilter *self, bool On) {
  TScale_pxy *pxy;

	if (self->f_Rg) SetVisible((TWins*)self->f_Rg, self->f_Rg->tag && On);
	if (self->f_Rs) SetVisible((TWins*)self->f_Rs, self->f_Rs->tag && On);
	if (self->f_Rx) SetVisible((TWins*)self->f_Rx, self->f_Rx->tag && On);
	if (self->f_Ra) SetVisible((TWins*)self->f_Ra, self->f_Ra->tag && On);
	if (self->f_Cs) SetVisible((TWins*)self->f_Cs, self->f_Cs->tag && On);
	if (self->f_Cx) SetVisible((TWins*)self->f_Cx, self->f_Cx->tag && On);
	if (self->f_Ls) SetVisible((TWins*)self->f_Ls, self->f_Ls->tag && On);
	if (self->f_Lx) SetVisible((TWins*)self->f_Lx, self->f_Lx->tag && On);
	if (On) {
	    pxy = self->pgfilt->xy_ind->dat->pxy;
	    while (pxy) {
		pxy->PaintDat = False;
		pxy = pxy->next;
	    }
//	    math_Filter(self);
	    if (self->OnMsg) self->OnMsg(self, MSG_MATH_FILTER, NULL);

	    if (self->cbF1On) 
		((TScale_pxy*)self->cbF1On->tag)->PaintDat = self->cbF1On->Checked;
	    if (self->cbF2On) 
		((TScale_pxy*)self->cbF2On->tag)->PaintDat = self->cbF2On->Checked;
	    if (self->cbF3On) 
		((TScale_pxy*)self->cbF3On->tag)->PaintDat = self->cbF3On->Checked;
	    if (self->cbF4On) 
		((TScale_pxy*)self->cbF4On->tag)->PaintDat = self->cbF4On->Checked;
	}
}

int XF_GetPaint_Fu(TPage_XFilter *self) {
  int result;

	result = 0;
	if (self->cbCsOn &&  self->cbCsOn->Checked)  result = MF_CS;
	if (self->cbLsOn &&  self->cbLsOn->Checked)  result |= MF_LS;
	if (self->cbRsOn &&  self->cbRsOn->Checked)  result |= MF_RS;
	if (self->cbCxOn &&  self->cbCxOn->Checked)  result |= MF_CX;
	if (self->cbLxOn &&  self->cbLxOn->Checked)  result |= MF_LX;
	if (self->cbRxOn &&  self->cbRxOn->Checked)  result |= MF_RX;
	if (self->cbInvOn && self->cbInvOn->Checked) result |= MF_INV;
	return result;
}

static void EML_cbRgRa_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_Filter *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	SaveFilterOnOff(pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	math_Filter_xP(pg);
	SetCursor_Dat_txt(pg);
	Refresh((TWins*)pg->xy_ind);
}

static void Set_70dB_140dB(TPage_Filter *pg) {
	if (pg->cbdB140->Checked) {
	    TScaleXY_Set_Dat(pg->xy_ind->dat->dy, "dB", -140, 40, 2, sxysetfu_dB20);
	} else {
	    TScaleXY_Set_Dat(pg->xy_ind->dat->dy, "dB", -70, 20, 2, sxysetfu_dB20);
	}
}

static void EML_cbdB140_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_Filter *pg = self->subClass;
//  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
//	SaveFilterOnOff(pg, pcIni);
//	TpcIni_SaveAndFree(pcIni);
	Set_70dB_140dB(pg);
	Refresh((TWins*)pg->xy_ind);
}

static void pgc_Filter_OnActivate(TPageControl *pagc, TPage *AktPage) {
  TPage_Filter *pg = pagc->subClass;
  TPage_XFilter *pgx;
  char buf[20];

	if (AktPage && (AktPage->art == PAGE_ART_FILTER)) {
	    pgx = (TPage_XFilter*)AktPage;
	    if (pgx->ScaleMHz >= 0.01) {
		if (pgx->ScaleMHz >= 1) {
		    floattopc(buf, pgx->ScaleMHz, 0);
		} else {
		    floattopc(buf, pgx->ScaleMHz, 1);
		}
		SetCaption((TWins*)pg->cbmaxMHZ, buf);
		pg->ScaleMHz = pgx->ScaleMHz;
		TScrollBar_SetCount(pg->sbScale, pgx->sbScaleCount);
		TPage_Filter_SetScaleMHz(pg);
	    }
	}
}

static void pgc_Filter_OnChangeTab(TPageControl *self, TPage *fromPage, TPage *toPage, int *fu) {
  TPage_Filter *pg = self->subClass;

	TP1_SetVisible(pg->TP1, (TPage*)pg->TP1 == toPage);
//	TP2_SetVisible(pg->TP2, (TPage*)pg->TP2 == toPage);
	HP1_SetVisible(pg->HP1, (TPage*)pg->HP1 == toPage);
//	HP2_SetVisible(pg->HP2, (TPage*)pg->HP2 == toPage);
	BP1_SetVisible(pg->BP1, (TPage*)pg->BP1 == toPage);
	BP2_SetVisible(pg->BP2, (TPage*)pg->BP2 == toPage);
	NF1_SetVisible(pg->NF1, (TPage*)pg->NF1 == toPage);
	BP2x_SetVisible(pg->BP2x, (TPage*)pg->BP2x == toPage);
	W1_SetVisible(pg->W1, (TPage*)pg->W1 == toPage);
	SGS1_SetVisible(pg->SGS1, (TPage*)pg->SGS1 == toPage);

}

static void DVal_Create_Xxx(TWins *Parent, TDVal **dv, TFormRLConoff **onoff, 
			int *x, int y, int w, int nk, TPic *pic, 
			TDValOnValChange OnValChange) {
  int tx = *x;
  TDVal *tdv = TDVal_CreateS(Parent, tx+19, y, w, 19);
	tdv->nk = nk;
	tdv->dv_fu = tdv_fu_LostFocusEnter;
	tdv->OnValChange = OnValChange;
	*dv = tdv;
	*onoff = TFormRLConoff_CreateU(Parent, tx, y+1, 18, 16, pic);
	*x = tx + w + 21;
}

static TFormR *Create_fR(TPage_XFilter *self, TAddXFilter *p,
			const char *s, TDVal *dv, float val) {
  TFormR *fR;
  char *pc;
  char buf[16];

	pc = buf;
	ppcmove(&pc, self->gname);
	*pc = ' '; pc++;
	pcmove(pc, s);
	
	fR = Create_Form_R(appDat.obj, buf, self->gname, s, dv, val);
	fR->subClass = self;
	TDVal_Set_Value(dv, fR->R->dval);
	fR->OnChange = p->fXxx_OnChange;
	fR->OnSync = p->OnRSync;
	return fR;
}
static TFormL *Create_fL(TPage_XFilter *self, TAddXFilter *p,
			const char *s, TDVal *dv, float val) {
  TFormL *fL;
  char *pc;
  char buf[16];

	pc = buf;
	ppcmove(&pc, self->gname);
	*pc = ' '; pc++;
	pcmove(pc, s);

	fL = Create_Form_L(appDat.obj, buf, self->gname, s, dv, val);
	fL->subClass = self;
	TDVal_Set_Value(dv, fL->L->dval);
	fL->OnChange = p->fXxx_OnChange;
	fL->OnSync = p->OnLSync;
	return fL;
}
static TFormC *Create_fC(TPage_XFilter *self, TAddXFilter *p,
			const char *s, TDVal *dv, float val) {
  TFormC *fC;
  char *pc;
  char buf[16];

	pc = buf;
	ppcmove(&pc, self->gname);
	*pc = ' '; pc++;
	pcmove(pc, s);

	fC = Create_Form_C(appDat.obj, buf, self->gname, s, dv, val);
	fC->subClass = self;
	TDVal_Set_Value(dv, fC->C->dval);
	fC->OnChange = p->fXxx_OnChange;
	fC->OnSync = p->OnCSync;
	return fC;
}


static void miLC0_OnClick(TMenu *self, long KeyState, TMenuItem *mi) {
  TFormElMath *fem;
  TPage_XFilter *pgx;

	fem = (TFormElMath*)self->FormStruct;
	if (fem->PageControl->AktPage == (TPage*)fem->pgFilter) {
	    pgx = (TPage_XFilter*)fem->pgFilter->pgc_Filter->AktPage;
	    if (pgx && (pgx->art == PAGE_ART_FILTER)) {
		ShowFormSetLC0(fem->FormSetLC0, pgx, mi->tag);
	    }
	}
}

TPage_XFilter *PageAddXFilter(TAddXFilter *p) {
  TPage_XFilter *self;
  TWins *pnl;
  int x, cbOnOff;
  TScale_pxy *pxy;

	self = (TPage_XFilter*)TPageControl_AddNewPage_Size(p->pagc, p->gname, p->psize);
	self->art = PAGE_ART_FILTER;
	self->subClass = self;
	self->gname = p->gname;
	self->pgfilt = p->pgfilt;

	cbOnOff = p->cbOnOff;
	self->ScaleMHz = 1000;
	if (p->ini) {
	    self->ScaleMHz = TpcIni_RD_float(p->ini, p->gname, "ScaleMHz", 1000);
	    cbOnOff = TpcIni_RD_int(p->ini, p->gname, "OnOff", p->cbOnOff);
	}

	pnl = TWins_CreateBA((TWins*)self, 0, 0, p->pagc->width - 4, 21, 0, alLeft);
	self->pnl = pnl;

/* Rg Rs Rx Ra */
	x = 0;
	if (p->dvfu & FU_RG) {
	    DVal_Create_Xxx(pnl, &self->dv_Rg, &self->OnOff_Rg, 
				&x, 2, 54, 1, get_Pic_cRg(), p->TDValOnValChange);
	    self->f_Rg = Create_fR(self, p, "Rg", self->dv_Rg, 50);
	    TFormRLConoff_SetForm(self->OnOff_Rg, (TForm*)self->f_Rg);
	}
	if (p->dvfu & FU_RS) {
	    DVal_Create_Xxx(pnl, &self->dv_Rs, &self->OnOff_Rs, 
				&x, 2, 54, 1, get_Pic_cRs(), p->TDValOnValChange);
	    self->f_Rs = Create_fR(self, p, "Rs", self->dv_Rs, 50);
	    TFormRLConoff_SetForm(self->OnOff_Rs, (TForm*)self->f_Rs);
	}
	if (p->dvfu & FU_RX) {
	    DVal_Create_Xxx(pnl, &self->dv_Rx, &self->OnOff_Rx, 
				&x, 2, 54, 1, get_Pic_cRx(), p->TDValOnValChange);
	    self->f_Rx = Create_fR(self, p, "Rx", self->dv_Rx, 50);
	    TFormRLConoff_SetForm(self->OnOff_Rx, (TForm*)self->f_Rx);
	}
	if (p->dvfu & FU_RA) {
	    DVal_Create_Xxx(pnl, &self->dv_Ra, &self->OnOff_Ra, 
				&x, 2, 54, 1, get_Pic_cRa(), p->TDValOnValChange);
	    self->f_Ra = Create_fR(self, p, "Ra", self->dv_Ra, 50);
	    TFormRLConoff_SetForm(self->OnOff_Ra, (TForm*)self->f_Ra);
	}



/* Cs Ca */
	x = 0;
	if (p->dvfu & FU_CS) {
	    DVal_Create_Xxx(pnl, &self->dv_Cs, &self->OnOff_Cs, 
				&x, 22, 54, 1, get_Pic_cCs(), p->TDValOnValChange);
	    self->f_Cs = Create_fC(self, p, "Cs", self->dv_Cs, 100);
	    TFormRLConoff_SetForm(self->OnOff_Cs, (TForm*)self->f_Cs);
	}
	if (p->dvfu & FU_CX) {
	    DVal_Create_Xxx(pnl, &self->dv_Cx, &self->OnOff_Cx, 
				&x, 22, 54, 1, get_Pic_cCx(), p->TDValOnValChange);
	    self->f_Cx = Create_fC(self, p, "Cx", self->dv_Cx, 100);
	    TFormRLConoff_SetForm(self->OnOff_Cx, (TForm*)self->f_Cx);
	}

/* Ls Lx */
	if (p->dvfu & (FU_CS|FU_CX)) x += 18;
	if (p->dvfu & FU_LS) {
	    DVal_Create_Xxx(pnl, &self->dv_Ls, &self->OnOff_Ls, 
				&x, 22, 54, 3, get_Pic_cLs(), p->TDValOnValChange);
	    self->f_Ls = Create_fL(self, p, "Ls", self->dv_Ls, 0.1);
	    TFormRLConoff_SetForm(self->OnOff_Ls, (TForm*)self->f_Ls);
	}
	if (p->dvfu & FU_LX) {
	    DVal_Create_Xxx(pnl, &self->dv_Lx, &self->OnOff_Lx, 
				&x, 22, 54, 3, get_Pic_cLx(), p->TDValOnValChange);
	    self->f_Lx = Create_fL(self, p, "Lx", self->dv_Lx, 0.1);
	    TFormRLConoff_SetForm(self->OnOff_Lx, (TForm*)self->f_Lx);
	}



	x = 2;
	if (p->cbfu & FU_CS) {
	    self->cbCsOn = TChkBox_CreateS(pnl, x, p->cbY, 48, 16, "Cs");
	    self->cbCsOn->OnClick = p->cb_OnClick;
	    self->cbCsOn->Checked = ((cbOnOff & MF_CS) != 0);
	    x += 40;
	}
	if (p->cbfu & FU_LS) {
	    self->cbLsOn = TChkBox_CreateS(pnl, x, p->cbY, 48, 16, "Ls");
	    self->cbLsOn->OnClick = p->cb_OnClick;
	    self->cbLsOn->Checked = ((cbOnOff & MF_LS) != 0);
	    x += 40;
	}
	if (p->cbfu & FU_RS) {
	    self->cbRsOn = TChkBox_CreateS(pnl, x, p->cbY, 48, 16, "Rs");
	    self->cbRsOn->OnClick = p->cb_OnClick;
	    self->cbRsOn->Checked = ((cbOnOff & MF_RS) != 0);
	    x += 40;
	}
	x += 10;
	if (p->cbfu & FU_CX) {
	    self->cbCxOn = TChkBox_CreateS(pnl, x, p->cbY, 48, 16, "Cx");
	    self->cbCxOn->OnClick = p->cb_OnClick;
	    self->cbCxOn->Checked = ((cbOnOff & MF_CX) != 0);
	    x += 40;
	}
	if (p->cbfu & FU_LX) {
	    self->cbLxOn = TChkBox_CreateS(pnl, x, p->cbY, 48, 16, "Lx");
	    self->cbLxOn->OnClick = p->cb_OnClick;
	    self->cbLxOn->Checked = ((cbOnOff & MF_LX) != 0);
	    x += 40;
	}
	if (p->cbfu & FU_RX) {
	    self->cbRxOn = TChkBox_CreateS(pnl, x, p->cbY, 48, 16, "Rx");
	    self->cbRxOn->OnClick = p->cb_OnClick;
	    self->cbRxOn->Checked = ((cbOnOff & MF_RX) != 0);
	    x += 40;
	}
	if (p->cbfu & FU_INV) {
	    self->cbInvOn = TChkBox_CreateS(pnl, x, p->cbY, 48, 16, "Inv");
	    self->cbInvOn->OnClick = p->cb_OnClick;
	    self->cbInvOn->Checked = ((cbOnOff & MF_INV) != 0);
//	    x += 40;
	}

	pxy = p->pgfilt->xy_ind->dat->pxy;
	if (p->cbFxfu & FU_F1) {
	    self->cbF1On = TChkBox_CreateS(pnl, 2, p->cbF1Y, 38, 17, "F1");
	    self->cbF1ImpOn = TChkBox_CreateS(pnl, 2, p->cbF1Y+17, 38, 17, "Imp");
	    SetColor((TWins*)self->cbF1On, 0xa0a0ff);
	    SetColor((TWins*)self->cbF1ImpOn, 0xe0e080);
	    self->cbF1On->Checked = ((cbOnOff & 0x010) != 0);
	    self->cbF1ImpOn->Checked = ((cbOnOff & 0x700) == 0x100);
	    self->cbF1On->OnClick = p->cbFxOnClick;
	    self->cbF1On->tag = (long)pxy;
	    self->cbF1ImpOn->OnClick = p->cbFxImpOnClick;
	}
	pxy = pxy->next;
	if (p->cbFxfu & FU_F2) {
	    self->cbF2On = TChkBox_CreateS(pnl, 2, p->cbF2Y, 38, 17, "F2");
	    self->cbF2ImpOn = TChkBox_CreateS(pnl, 2, p->cbF2Y+17, 38, 17, "Imp");
	    SetColor((TWins*)self->cbF2On, 0xa0ffa0);
	    SetColor((TWins*)self->cbF2ImpOn, 0xe0e080);
	    self->cbF2On->Checked = ((cbOnOff & 0x020) != 0);
	    self->cbF2ImpOn->Checked = ((cbOnOff & 0x700) == 0x200);
	    self->cbF2On->OnClick = p->cbFxOnClick;
	    self->cbF2On->tag = (long)pxy;
	    self->cbF2ImpOn->OnClick = p->cbFxImpOnClick;
	}
	pxy = pxy->next;
	if (p->cbFxfu & FU_F3) {
	    self->cbF3On = TChkBox_CreateS(pnl, 2, p->cbF3Y, 38, 17, "F3");
	    self->cbF3ImpOn = TChkBox_CreateS(pnl, 2, p->cbF3Y+17, 38, 17, "Imp");
	    SetColor((TWins*)self->cbF3On, 0xff8080);
	    SetColor((TWins*)self->cbF3ImpOn, 0xe0e080);
	    self->cbF3On->Checked = ((cbOnOff & 0x040) != 0);
	    self->cbF3ImpOn->Checked = ((cbOnOff & 0x700) == 0x300);
	    self->cbF3On->OnClick = p->cbFxOnClick;
	    self->cbF3On->tag = (long)pxy;
	    self->cbF3ImpOn->OnClick = p->cbFxImpOnClick;
	}
	pxy = pxy->next;
	if (p->cbFxfu & FU_F4) {
	    self->cbF4On = TChkBox_CreateS(pnl, 2, p->cbF4Y, 38, 17, "F4");
	    self->cbF4ImpOn = TChkBox_CreateS(pnl, 2, p->cbF4Y+17, 38, 17, "Imp");
	    SetColor((TWins*)self->cbF4On, 0xff80ff);
	    SetColor((TWins*)self->cbF4ImpOn, 0xe0e080);
	    self->cbF4On->Checked = ((cbOnOff & 0x080) != 0);
	    self->cbF4ImpOn->Checked = ((cbOnOff & 0x700) == 0x400);
	    self->cbF4On->OnClick = p->cbFxOnClick;
	    self->cbF4On->tag = (long)pxy;
	    self->cbF4ImpOn->OnClick = p->cbFxImpOnClick;
	}


	if (p->LC0_Popupfu) {
	    pnl->PopupMenu = TMenu_Create((TWins*)pnl);
	    if (p->LC0_Popupfu & FU_F1) {
		TMenu_AddItem(pnl->PopupMenu, "Set F1", 0, miLC0_OnClick)->tag = 1;
	    }
	    if (p->LC0_Popupfu & FU_F2) {
		TMenu_AddItem(pnl->PopupMenu, "Set F2", 0, miLC0_OnClick)->tag = 2;
	    }
	    if (p->LC0_Popupfu & FU_F3) {
		TMenu_AddItem(pnl->PopupMenu, "Set F3", 0, miLC0_OnClick)->tag = 3;
	    }
	    if (p->LC0_Popupfu & FU_F4) {
		TMenu_AddItem(pnl->PopupMenu, "Set F4", 0, miLC0_OnClick)->tag = 4;
	    }
	}

	return self;
}

TPage_Filter *ELM_pageFilterAdd(TPageControl *pagc, const char *Caption) {
  TPage_Filter *self;
  TpcIni *pcIni;
  int m;

	pcIni = get_appIni();

	self = (TPage_Filter*) TPageControl_AddNewUserPage(pagc, Caption, ELM_pageFilter_Create);
	self->subClass = self;

	self->pnlFilter = TWins_CreateBA((TWins*)self, 0, 0, 340, 21,
					mBorder | mBorderL, alLeft);

	self->pgc_Filter = TPageControl_Create(self->pnlFilter);
	self->pgc_Filter->Align = alClient;
	self->pgc_Filter->height = 120;
	self->pgc_Filter->OnPageActivate = pgc_Filter_OnActivate;
	self->pgc_Filter->OnChangeTab = pgc_Filter_OnChangeTab;

	Set_Parent((TWins*)self->pgc_Filter, self->pnlFilter);

/* ****************** Graf ****************************************************/
	self->pnlGraf = TWins_CreateBA((TWins*)self, 0, 0, 310, 21, 
				mBorder | mBorderL, alClient);
/* Graf Dat */
	self->pnlGrafVal = TWins_CreateBA(self->pnlGraf, 0, 0, 310, 21, 
				mBorder, alTop);
	self->pnlGrafVal->OnPaint = ELM_TP1_pnlGrafVal_OnPaint;
	self->pnlGrafVal->OnResize = ELM_TP1_pnlGrafVal_OnResize;
	self->cbmaxMHZ = TComboBox_CreateS(self->pnlGrafVal, 40, 1, 60, 18, "1000");

  static const char *cbMHZ_Tab[15] = {"0.1", "0.2", "0.5", "1", "2", "5", "10", "20", "50", 
		"100", "200", "500", "1000", "2000", "5000"};

	for (int i=0;i<15;i++)
	    TComboBox_AddItem(self->cbmaxMHZ, cbMHZ_Tab[i], 0, ELM_F_cbmaxMHZ_OnClick);

	self->cbLog = TComboBox_CreateS(self->pnlGrafVal, 220, 1, 76, 18, "lin");
	self->cbLog->ReadOnly = true;
  static const char *cbLog_Tab[5] = {"lin", "log -1", "log10 -1", "log20 -1", "log100-1"};
	for (int i=0;i<5;i++)
	    TComboBox_AddItem(self->cbLog, cbLog_Tab[i], 0, ELM_F_cbLog_OnClick);


	self->dv_TP1_Off_dB = TDVal_CreateS(self->pnlGrafVal, 170, 1, 40, 18);
	self->dv_TP1_Off_dB->nk = 1;
	self->dv_TP1_Off_dB->max = 100;
	self->dv_TP1_Off_dB->min = -100;
	self->dv_TP1_Off_dB->dv_fu = tdv_fu_LostFocusEnter;
	self->dv_TP1_Off_dB->OnValChange = EML_Filter_TP1_Off_dB_OnValChange;

	m = TpcIni_RD_int(pcIni, "Filter", "OnOff", 0xffff);

	self->cbRgRa = TChkBox_CreateS(self->pnlGrafVal, 2, 2, 56, 17, "Rg/Ra");
	self->cbRgRa->OnClick = EML_cbRgRa_OnClick;
	self->cbRgRa->Checked = ((m & 0x01) != 0);

	self->cbdB140 = TChkBox_CreateS(self->pnlGrafVal, 2, 2, 56, 17, "140dB");
	self->cbdB140->OnClick = EML_cbdB140_OnClick;
	self->cbdB140->Checked = ((m & 0x02) != 0);

/* Graf Az */
	self->xy_ind = TScaleXY_Create(self->pnlGraf);
	self->xy_ind->Align = alClient;
	self->xy_ind->OnResize = EML_Filter_xy_ind_OnResize;
	self->xy_ind->sx1 = 22;
	self->xy_ind->sy1 = 18;
	self->xy_ind->sy2 = 46;
	self->xy_ind->CursFu = 0x01;
	Set_Parent((TWins*)self->xy_ind, self->pnlGraf);

	self->sbDetail = TScrollBarHorz_CreateP((TWins*)self->xy_ind);
	self->sbDetail->OnCount = EML_Filter_sbDetail_OnCount;
//	SetLeft((TWins*)self->sbDetail, 1);
	TScrollBar_SetMaxCount(self->sbDetail, 20000);
	self->sbDetail->LargeChange = 50;

	self->sbScale = TScrollBarHorz_CreateP((TWins*)self->xy_ind);
	self->sbScale->BorderFu = mBorder | mBorderL | 
			mBorderNoLeft | mBorderNoRigth | mBorderNoBottom;
	self->sbScale->Align = alBottom;
	SetHeight((TWins*)self->sbScale, 16);
	TScrollBar_SetMaxCount(self->sbScale, 300);
	self->sbScale->LargeChange = 5;
	self->sbScale->OnCount = EML_Filter_sbScale_OnCount;


	TScaleXY_SetDat_mem(&self->xy_ind->dat, 21, 19, 1001);

  TScale_pxy *pxy = self->xy_ind->dat->pxy;
	pxy->Color = 0x0000c0;
	pxy->AddOff = true;

	pxy->next = xcalloc(sizeof(TScale_pxy) + sizeof(TScale_Point)*1001);
	pxy = pxy->next;
	pxy->count = 1001;
	pxy->Color = 0x00c000;
	pxy->AddOff = true;

	pxy->next = xcalloc(sizeof(TScale_pxy) + sizeof(TScale_Point)*1001);
	pxy = pxy->next;
	pxy->count = 1001;
	pxy->Color = 0xc00000;
	pxy->AddOff = true;

	pxy->next = xcalloc(sizeof(TScale_pxy) + sizeof(TScale_Point)*1001);
	pxy = pxy->next;
	pxy->count = 1001;
	pxy->Color = 0xc000c0;
	pxy->AddOff = true;

	pxy->next = xcalloc(sizeof(TScale_pxy) + sizeof(TScale_Point)*1001);
	pxy = pxy->next;
	self->yInZ = pxy;
	pxy->count = 1001;
	pxy->Color = 0xc0c000;


	self->TP1 = page_Add_TP1(self->pgc_Filter, self);
//	self->TP2 = page_Add_TP2(self->pgc_Filter, self);
	self->HP1 = page_Add_HP1(self->pgc_Filter, self);
//	self->HP2 = page_Add_HP2(self->pgc_Filter, self);
	self->BP1 = page_Add_BP1(self->pgc_Filter, self);
	self->BP2 = page_Add_BP2(self->pgc_Filter, self);
	self->NF1 = page_Add_NF1(self->pgc_Filter, self);
	self->BP2x = page_Add_BP2x(self->pgc_Filter, self);
	self->W1 = page_Add_W1(self->pgc_Filter, self);
	self->SGS1 = page_Add_SGS1(self->pgc_Filter, self);

	pxy = self->xy_ind->dat->pxy;
//	pxy->PaintDat = self->TP1->cbF1On->Checked;
//	self->TP1->cbF1On->tag = (long)pxy;
//	self->TP2->cbF1On->tag = (long)pxy;
//	self->HP1->cbF1On->tag = (long)pxy;
//	self->BP1->cbF1On->tag = (long)pxy;
	self->BP2->cbF1On->tag = (long)pxy;
	self->NF1->cbF1On->tag = (long)pxy;
	self->BP2x->cbF1On->tag = (long)pxy;
	self->W1->cbF1On->tag = (long)pxy;
	self->SGS1->cbF1On->tag = (long)pxy;

	pxy = pxy->next;
//	pxy->PaintDat = self->TP1->cbF2On->Checked;
//	self->TP1->cbF2On->tag = (long)pxy;
//	self->TP2->cbF2On->tag = (long)pxy;
//	self->HP1->cbF2On->tag = (long)pxy;
//	self->BP1->cbF2On->tag = (long)pxy;
	self->BP2->cbF2On->tag = (long)pxy;
	self->NF1->cbF2On->tag = (long)pxy;
	self->BP2x->cbF2On->tag = (long)pxy;
	self->W1->cbF2On->tag = (long)pxy;
	self->SGS1->cbF2On->tag = (long)pxy;

	pxy = pxy->next;
//	pxy->PaintDat = self->TP1->cbF3On->Checked;
//	self->TP1->cbF3On->tag = (long)pxy;
//	self->TP2->cbF3On->tag = (long)pxy;
//	self->HP1->cbF3On->tag = (long)pxy;
//	self->BP1->cbF3On->tag = (long)pxy;
	self->BP2->cbF3On->tag = (long)pxy;
	self->NF1->cbF3On->tag = (long)pxy;
//	self->BP2x->cbF3On->tag = (long)pxy;
	self->W1->cbF3On->tag = (long)pxy;
	self->SGS1->cbF3On->tag = (long)pxy;

	pxy = pxy->next;
//	pxy->PaintDat = self->TP1->cbF4On->Checked;
//	self->TP1->cbF4On->tag = (long)pxy;
//	self->TP2->cbF4On->tag = (long)pxy;
//	self->HP1->cbF4On->tag = (long)pxy;
//	self->BP1->cbF4On->tag = (long)pxy;
	self->BP2->cbF4On->tag = (long)pxy;
//	self->BP2x->cbF4On->tag = (long)pxy;
	self->W1->cbF4On->tag = (long)pxy;
	self->SGS1->cbF4On->tag = (long)pxy;

//	pxy->PaintDat = true; //self->cbF4On->Checked;
//	self->cbF4On->tag = (long)pxy;

	self->ScaleMHz = TpcIni_RD_float(pcIni, "TP1", "ScaleMHz", 1000);
	self->fuLogX = sxysetfu_lin;
	TScaleXY_Set_Dat(self->xy_ind->dat->dx, "MHz", 0, self->ScaleMHz, 2, self->fuLogX);
	TScaleXY_Set_Dat(self->xy_ind->dat->dy, "dB", -70, 20, 2, sxysetfu_dB20);

//static TFormR *Create_fR(TPage_Filter *self, char *Caption, char *Gruppe, char *s, TDVal *dv) {

  TScaleXY_Txt *xy_txt;
	xy_txt = xcalloc(sizeof(TScaleXY_Txt));
	xy_txt->x = 2;
	xy_txt->y = 20;
	self->xy_ind->txt = xy_txt;
	xy_txt->next = xcalloc(sizeof(TScaleXY_Txt));
	xy_txt = xy_txt->next;
	xy_txt->x = 2;
	xy_txt->y = 36;

	free_appIni(pcIni);

	return self;
}
