/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "ElMath.h"
#include "ElGraf.h"
#include <math.h>

static void str_toGraf(TWins *Wins, const char *s, int x, int y) {
	vds_SetStr(&Wins->vds, s);
	DrawColorString(x, y, &Wins->vds);
}

static void DrawBP2x(Window WinID, int x, int y, int fu) {
  int xa;
  int x2 = x;
  int y2 = y+30;

	Draw_xP1_Rg(WinID, &x2, &xa, y, y2, fu);

	if ((fu & 0x0300) == 0x000) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lv(WinID, &x2, 9, y, get_Pic_cL1());
	    DrawTP1_Ch(WinID, &x2, y, get_Pic_cC2());
	    DrawTP1_Lv(WinID, &x2, 0, y, get_Pic_cL2());
	    DrawTP1_Cv(WinID, &x2, 9, y, get_Pic_cC3());
	}

	if ((fu & 0x0300) == 0x100) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lv(WinID, &x2, 9, y, get_Pic_cL1());
	    DrawTP1_Ch(WinID, &x2, y, get_Pic_cC2());
	    DrawTP1_Lv(WinID, &x2, 0, y, get_Pic_cL2());
	    DrawTP1_Cv(WinID, &x2, 9, y, get_Pic_cC3());
	}
	if ((fu & 0x0300) == 0x200) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lv(WinID, &x2, 9, y, get_Pic_cL1());
	    DrawTP1_Ch(WinID, &x2, y, get_Pic_cC2());
	    DrawTP1_Lv(WinID, &x2, 0, y, get_Pic_cL2());
	    DrawTP1_Cv(WinID, &x2, 9, y, get_Pic_cC2());
	}

	if ((fu & 0x0300) == 0x300) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lv(WinID, &x2, 9, y, get_Pic_cL1());
	    DrawTP1_Ch(WinID, &x2, y, get_Pic_cC2());
	    DrawTP1_Lv(WinID, &x2, 0, y, get_Pic_cL2());
	    DrawTP1_Cv(WinID, &x2, 9, y, get_Pic_cC2());
	}

	Draw_xP1_Ra(WinID, x2, xa, y, y2, fu);
}

static bool ImpBP2x_ZF1(TPage_BP2x *self, float w, TRLC *btIn, float *ret) {
  float res;
	if (w < 0.000001) {
	    if (self->cbCxOn->Checked || self->cbCsOn->Checked) return 0;
	    res = self->f_Rs->R->dval;
	    *ret = self->f_Ra->R->dval / (self->f_Ra->R->dval + res);
	    return False;
	}
	Form_R_math(self->f_Ra, w);

	res = 1;
	RLC_Copy(btIn, &self->f_Ra->rlc);
	if (self->cbCxOn->Checked) {
	    res = btIn->Z;
	    Form_C_math(self->f_Cx, w);
	    RLC_Add_Ser(btIn, &self->f_Cx->rlc);
	    res /= btIn->Z;
	}

	Form_C_math(self->f_C1, w);
	Form_L_math(self->f_L1, w);
	Form_C_math(self->f_C2, w);
	Form_C_math(self->f_C3, w);
	Form_L_math(self->f_L2, w);

	RLC_Add_Par(btIn, &self->f_C3->rlc);
	RLC_Add_Par(btIn, &self->f_L2->rlc);
	res *= btIn->Z;

	RLC_Add_Ser(btIn, &self->f_C2->rlc);
	res /= btIn->Z;

	RLC_Add_Par(btIn, &self->f_C1->rlc);
	RLC_Add_Par(btIn, &self->f_L1->rlc);
	res *= btIn->Z;

	if (self->cbCsOn->Checked) {
	    Form_C_math(self->f_Cs, w);
	    RLC_Add_Ser(btIn, &self->f_Cs->rlc);
	}

	*ret = res;
	return True;
}

static float ImpF1(TPage_BP2x *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!ImpBP2x_ZF1(self, w, &btIn, &res)) return res;

	*Z = btIn.Z;
	Form_R_math(self->f_Rg, w);
	RLC_Add_Ser(&btIn, &self->f_Rg->rlc);
	res /= btIn.Z;
	return res;
} 

static bool ImpBP2x_ZF2(TPage_BP2x *self, float w, TRLC *btIn, float *ret) {
  float res;
	if (w < 0.000001) {
	    if (self->cbCxOn->Checked || self->cbCsOn->Checked) return 0;
	    res = self->f_Rs->R->dval;
	    *ret = self->f_Ra->R->dval / (self->f_Ra->R->dval + res);
	    return False;
	}
	Form_R_math(self->f_Ra, w);

	res = 1;
	RLC_Copy(btIn, &self->f_Ra->rlc);
	if (self->cbCxOn->Checked) {
	    res = btIn->Z;
	    Form_C_math(self->f_Cx_F2, w);
	    RLC_Add_Ser(btIn, &self->f_Cx_F2->rlc);
	    res /= btIn->Z;
	}

	Form_C_math(self->f_C1_F2, w);
	Form_L_math(self->f_L1_F2, w);
	Form_C_math(self->f_C2_F2, w);
	Form_C_math(self->f_C3_F2, w);
	Form_L_math(self->f_L2_F2, w);

	RLC_Add_Par(btIn, &self->f_C3_F2->rlc);
	RLC_Add_Par(btIn, &self->f_L2_F2->rlc);
	res *= btIn->Z;

	RLC_Add_Ser(btIn, &self->f_C2_F2->rlc);
	res /= btIn->Z;

	RLC_Add_Par(btIn, &self->f_C1_F2->rlc);
	RLC_Add_Par(btIn, &self->f_L1_F2->rlc);
	res *= btIn->Z;

	if (self->cbCsOn->Checked) {
	    Form_C_math(self->f_Cs_F2, w);
	    RLC_Add_Ser(btIn, &self->f_Cs_F2->rlc);
	}

	*ret = res;
	return True;
}

static float ImpF2(TPage_BP2x *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!ImpBP2x_ZF2(self, w, &btIn, &res)) return res;

	*Z = btIn.Z;
	Form_R_math(self->f_Rg, w);
	RLC_Add_Ser(&btIn, &self->f_Rg->rlc);
	res /= btIn.Z;
	return res;
}
static float ImpBP2x_F3(TPage_BP2x *self, float MHz, float *Z) {
  float res, resF2;
  TRLC btIn, btInF2;
  float w = MHz * M_PI * 2;
	if (!ImpBP2x_ZF1(self, w, &btIn, &res)) return res;
	if (!ImpBP2x_ZF2(self, w, &btInF2, &resF2)) return res;
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &btInF2);
	res *= btIn.Z;
	*Z = btIn.Z;
	Form_R_math(self->f_Rg, w);
	RLC_Add_Ser(&btIn, &self->f_Rg->rlc);
	res /= btIn.Z;

//	ImpBP2x_MathIn_rc(self, w, &btIn, &res, Z);
	return res;
}
static float ImpBP2x_F4(TPage_BP2x *self, float MHz, float *Z) {
  float res, resF2;
  TRLC btIn, btInF2;
  float w = MHz * M_PI * 2;
	if (!ImpBP2x_ZF1(self, w, &btIn, &res)) return res;
	if (!ImpBP2x_ZF2(self, w, &btInF2, &resF2)) return res;
	resF2 /= btInF2.Z;
	RLC_Add_Par(&btInF2, &btIn);
	resF2 *= btInF2.Z;
	*Z = btInF2.Z;
	Form_R_math(self->f_Rg, w);
	RLC_Add_Ser(&btInF2, &self->f_Rg->rlc);
	resF2 /= btInF2.Z;
	return resF2;
}

static void math_Filter(TPage_BP2x *self) {
  TPage_Filter *pgf = self->pgfilt;
  float fcStep, fOff, fStep;
  TScale_pxy *pxy = pgf->xy_ind->dat->pxy;
  TScale_pxy *pxy2;
  TScale_pxy *pxy3 = NULL;
  float fc, Z, K;
  TScale_Point *sSP, *dSP;

	K = 1;
	if (pgf->cbRgRa->Checked)
	    K = (self->f_Rg->rlc.val + self->f_Ra->rlc.val) / self->f_Ra->rlc.val;

	fOff = pgf->ScaleMHzoff;
	fStep = pgf->ScaleMHz;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF1(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF1ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF2(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF2ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	pxy->PaintDat = True;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpBP2x_F3(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}

	pxy = pxy->next;
	pxy->PaintDat = True;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpBP2x_F4(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}

	Z = self->dv_Rg->dval;
	if (pxy3 && (Z > 0)) {
	    pxy2 = pgf->yInZ;
	    pxy2->PaintDat = True;
	    sSP = &pxy3->xy[0];
	    dSP = &pxy2->xy[0];
	    for (int i=0;i<pxy2->count;i++) {
		dSP->x = sSP->x;
		dSP->y = sSP->y2 / Z;
		dSP++; sSP++;
	    }
	} else {
	    pgf->yInZ->PaintDat = False;
	}

	SetCursor_Dat_txt(pgf);
	Refresh((TWins*)pgf->xy_ind);

}

static void SaveBP2xOnOff(TPage_BP2x *self, TpcIni *pcIni) {
  int OnOff = 0;

	if (self->cbCsOn->Checked) OnOff |= 0x01;
	if (self->cbRsOn->Checked) OnOff |= 0x02;
	if (self->cbCxOn->Checked) OnOff |= 0x04;
	if (self->cbRxOn->Checked) OnOff |= 0x08;

	if (self->cbF1On->Checked) OnOff |= 0x010;
	if (self->cbF2On->Checked) OnOff |= 0x020;

	if (self->cbF1ImpOn->Checked) OnOff |= 0x100;
	if (self->cbF2ImpOn->Checked) OnOff |= 0x200;

	TpcIni_WR_int(pcIni, self->gname, "OnOff", OnOff);
}

void BP2x_SetVisible(TPage_BP2x *self, bool On) {
	SetVisible((TWins*)self->f_Rg, self->f_Rg->tag && On);
	SetVisible((TWins*)self->f_Rs, self->f_Rs->tag && On);
	SetVisible((TWins*)self->f_Rx, self->f_Rx->tag && On);
	SetVisible((TWins*)self->f_Ra, self->f_Ra->tag && On);
	SetVisible((TWins*)self->f_Cs, self->f_Cs->tag && On);
	SetVisible((TWins*)self->f_Cx, self->f_Cx->tag && On);
	SetVisible((TWins*)self->f_C1, self->f_C1->tag && On);
	SetVisible((TWins*)self->f_C2, self->f_C2->tag && On);
	SetVisible((TWins*)self->f_C3, self->f_C3->tag && On);
	SetVisible((TWins*)self->f_L1, self->f_L1->tag && On);
	SetVisible((TWins*)self->f_L2, self->f_L2->tag && On);

	SetVisible((TWins*)self->f_Cs_F2, self->f_Cs_F2->tag && On);
	SetVisible((TWins*)self->f_Cx_F2, self->f_Cx_F2->tag && On);
	SetVisible((TWins*)self->f_C1_F2, self->f_C1_F2->tag && On);
	SetVisible((TWins*)self->f_C2_F2, self->f_C2_F2->tag && On);
	SetVisible((TWins*)self->f_C3_F2, self->f_C3_F2->tag && On);
	SetVisible((TWins*)self->f_L1_F2, self->f_L1_F2->tag && On);
	SetVisible((TWins*)self->f_L2_F2, self->f_L2_F2->tag && On);

  TScale_pxy *pxy;
	if (On) {
	    pxy = self->pgfilt->xy_ind->dat->pxy;
	    while (pxy) {
		pxy->PaintDat = False;
		pxy = pxy->next;
	    }
	    math_Filter(self);
	    ((TScale_pxy*)self->cbF1On->tag)->PaintDat = self->cbF1On->Checked;
	    ((TScale_pxy*)self->cbF2On->tag)->PaintDat = self->cbF2On->Checked;
	}
}

/* ************** BP2x Ext Ev ****************** */
/* bool ImpBP2x_Fx(TPage_BP2x *self, float MHz, float *yVal, float *Z) {
	if (self->cbF1ImpOn->Checked) {
		*yVal = ImpBP2x_F1(self, MHz, Z);
	} else if (self->cbF2ImpOn->Checked) {
		*yVal = ImpBP2x_F2(self, MHz, Z);
	} else return False;
	return True;
} */

static bool MathImp(TPage_BP2x *self, TMgsMathImp *pImp) {
	if (!pImp) return false;
	if (self->cbF1ImpOn->Checked) {
	    pImp->yVal = ImpF1(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF2ImpOn->Checked) {
	    pImp->yVal = ImpF2(self, pImp->MHz, &pImp->Z);
	    return true;
	}
	return false; 
}

static bool self_OnMsg(TPage_BP2x *self, int msg, void *p) {
  TpcIni *pcIni;
  int i;

	switch (msg & MSG_BEF_MASK) {
	case MSG_MATH_FILTER:
	    math_Filter(self);
	    return true;

	case MSG_MATH_IMP:
	    return MathImp(self, p);

	case MSG_NEW_SCALE_PARAM:
	    pcIni = get_appIni();
	    i = 1;
	    if (self->ScaleMHz >= 1) i = 0;
	    TpcIni_WR_float(pcIni, self->gname, "ScaleMHz", self->ScaleMHz, i);
	    TpcIni_Save(pcIni);
	    free_appIni(pcIni);
	    return true;
	}
	return false;
}

/* ************** BP2x Ev ****************** */
static void cbBP2xXxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_BP2x *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	SaveBP2xOnOff(pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	RePaint((TWins*)pg->pnl);
	math_Filter(pg);
}

static void cbBP2xFxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_BP2x *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	SaveBP2xOnOff(pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	((TScale_pxy*)self->tag)->PaintDat = ((TChkBox*)self)->Checked;
	Refresh((TWins*)pg->pgfilt->xy_ind);
}

static void cbBP2xFxImpOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_BP2x *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	if (pg->cbF1ImpOn != self) TChkBox_SetChecked(pg->cbF1ImpOn, False);
	if (pg->cbF2ImpOn != self) TChkBox_SetChecked(pg->cbF2ImpOn, False);
	SaveBP2xOnOff(pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	math_Filter(pg);
	Refresh((TWins*)pg->pgfilt->xy_ind);
}

static void pnl_BP2x_OnPaint(TWins *self) {
  TPage_BP2x *pg = self->subClass;
  int fu = 0;
  int x = pg->cbF1On->left + pg->cbF1On->width + 2;

	if (pg->cbCsOn->Checked) fu |= 0x01;
	if (pg->cbRsOn->Checked) fu |= 0x02;
	if (pg->cbCxOn->Checked) fu |= 0x04;
	if (pg->cbRxOn->Checked) fu |= 0x08;

	DrawBP2x(self->WinID, x, pg->cbF1On->top, fu);
	fu += 0x100;
	DrawBP2x(self->WinID, x, pg->cbF2On->top, fu);

	DrawPic(self->WinID, get_Pic_cOhm(), 
			pg->dv_Ra->left + pg->dv_Ra->width + 2,
			pg->dv_Ra->top + 5);

	str_toGraf(self, "pF", pg->dv_Cx->left + pg->dv_Cx->width + 1,
			       pg->dv_Cx->top + 3);

	str_toGraf(self, "pF", pg->dv_C3->left + pg->dv_C3->width + 1,
			       pg->dv_C3->top + 3);

	str_toGraf(self, "uH", pg->dv_L2->left + pg->dv_L2->width + 1,
			       pg->dv_L2->top + 3);
}

static void BP2x_DVal_OnValChange(TDVal *dv, double val) {
	if (dv->owner) TDVal_DoValChange((TDVal*)dv->owner, val, tdvfu_ChangeEv);
}

static void BP2x_fXxx_OnChange(TFormRLC* fRLC) {
	math_Filter((TPage_BP2x*)fRLC->subClass);
}
static void BP2x_OnRSync(TFormRLC* fRLC, int val) {
  TPage_BP2x *pg = fRLC->subClass;
	Form_R_sync(fRLC, pg->f_Rg, val);
	Form_R_sync(fRLC, pg->f_Rs, val);
	Form_R_sync(fRLC, pg->f_Rx, val);
	Form_R_sync(fRLC, pg->f_Ra, val);
}
static void BP2x_OnLSync(TFormRLC* fRLC, int val) {
  TPage_BP2x *pg = fRLC->subClass;
	Form_L_sync(fRLC, pg->f_L1, val);
	Form_L_sync(fRLC, pg->f_L2, val);

	Form_L_sync(fRLC, pg->f_L1_F2, val);
	Form_L_sync(fRLC, pg->f_L2_F2, val);
}
static void BP2x_OnCSync(TFormRLC* fRLC, int val) {
  TPage_BP2x *pg = fRLC->subClass;
	Form_C_sync(fRLC, pg->f_Cs, val);
	Form_C_sync(fRLC, pg->f_Cx, val);
	Form_C_sync(fRLC, pg->f_C1, val);
	Form_C_sync(fRLC, pg->f_C2, val);
	Form_C_sync(fRLC, pg->f_C3, val);

	Form_C_sync(fRLC, pg->f_Cs_F2, val);
	Form_C_sync(fRLC, pg->f_Cx_F2, val);
	Form_C_sync(fRLC, pg->f_C1_F2, val);
	Form_C_sync(fRLC, pg->f_C2_F2, val);
	Form_C_sync(fRLC, pg->f_C3_F2, val);
}
/* ************** Save Load ****************** */
static void SaveData(TPage_BP2x *self, TpcIni *ini) {
	self->f_Rg->SaveData(self->f_Rg, ini);
	self->f_Rs->SaveData(self->f_Rs, ini);
	self->f_Rx->SaveData(self->f_Rx, ini);
	self->f_Ra->SaveData(self->f_Ra, ini);

	self->f_L1->SaveData(self->f_L1, ini);
	self->f_L2->SaveData(self->f_L2, ini);

	self->f_Cs->SaveData(self->f_Cs, ini);
	self->f_Cx->SaveData(self->f_Cx, ini);
	self->f_C1->SaveData(self->f_C1, ini);
	self->f_C2->SaveData(self->f_C2, ini);
	self->f_C3->SaveData(self->f_C3, ini);

	self->f_L1_F2->SaveData(self->f_L1_F2, ini);
	self->f_L2_F2->SaveData(self->f_L2_F2, ini);

	self->f_Cs_F2->SaveData(self->f_Cs_F2, ini);
	self->f_Cx_F2->SaveData(self->f_Cx_F2, ini);
	self->f_C1_F2->SaveData(self->f_C1_F2, ini);
	self->f_C2_F2->SaveData(self->f_C2_F2, ini);
	self->f_C3_F2->SaveData(self->f_C3_F2, ini);
}

static void LoadData(TPage_BP2x *self, TpcIni *ini) {

	self->f_Rg->LoadData(self->f_Rg, ini);
	self->f_Rs->LoadData(self->f_Rs, ini);
	self->f_Rx->LoadData(self->f_Rx, ini);
	self->f_Ra->LoadData(self->f_Ra, ini);

	self->f_L1->LoadData(self->f_L1, ini);
	self->f_L2->LoadData(self->f_L2, ini);

	self->f_Cs->LoadData(self->f_Cs, ini);
	self->f_Cx->LoadData(self->f_Cx, ini);
	self->f_C1->LoadData(self->f_C1, ini);
	self->f_C2->LoadData(self->f_C2, ini);
	self->f_C3->LoadData(self->f_C3, ini);

	self->f_L1_F2->LoadData(self->f_L1_F2, ini);
	self->f_L2_F2->LoadData(self->f_L2_F2, ini);

	self->f_Cs_F2->LoadData(self->f_Cs_F2, ini);
	self->f_Cx_F2->LoadData(self->f_Cx_F2, ini);
	self->f_C1_F2->LoadData(self->f_C1_F2, ini);
	self->f_C2_F2->LoadData(self->f_C2_F2, ini);
	self->f_C3_F2->LoadData(self->f_C3_F2, ini);

	math_Filter(self);
	Refresh((TWins*)self->pgfilt->xy_ind);
}

/* ************** BP2x Create ****************** */
static void DVal_Create_Xxx(TWins *Parent, TDVal **dv, TFormRLConoff **onoff, int *x, int y, int w, int nk, TPic *pic) {
  int tx = *x;
  TDVal *tdv = TDVal_CreateS(Parent, tx+19, y, w, 19);
	tdv->nk = nk;
	tdv->dv_fu = tdv_fu_LostFocusEnter;
	tdv->OnValChange = BP2x_DVal_OnValChange;
	*dv = tdv;
	*onoff = TFormRLConoff_CreateU(Parent, tx, y+1, 18, 16, pic);
	*x = tx + w + 21;
}

static TFormR *Create_fR(TPage_BP2x *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormR *fR = Create_Form_R(appDat.obj, Caption, self->gname, s, dv, val);
	fR->subClass = self;
	TDVal_Set_Value(dv, fR->R->dval);
	fR->OnChange = BP2x_fXxx_OnChange;
	fR->OnSync = BP2x_OnRSync;
	return fR;
}
static TFormL *Create_fL(TPage_BP2x *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormL *fL = Create_Form_L(appDat.obj, Caption, self->gname, s, dv, val);
	fL->subClass = self;
	TDVal_Set_Value(dv, fL->L->dval);
	fL->OnChange = BP2x_fXxx_OnChange;
	fL->OnSync = BP2x_OnLSync;
	return fL;
}
static TFormC *Create_fC(TPage_BP2x *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormC *fC = Create_Form_C(appDat.obj, Caption, self->gname, s, dv, val);
	fC->subClass = self;
	TDVal_Set_Value(dv, fC->C->dval);
	fC->OnChange = BP2x_fXxx_OnChange;
	fC->OnSync = BP2x_OnCSync;
	return fC;
}

static TPage *page_Add_BP2x_Create(TPageControl *Parent) {
	return TPage_CreateParamAlg(Parent, sizeof(TPage_BP2x));
}

TPage_BP2x *page_Add_BP2x(TPageControl *pagc, TPage_Filter *pgfilt) {
  TPage_BP2x *self;
  TWins *pnl;
  TpcIni *pcIni;
  int x, y, m;

	pcIni = get_appIni();

	self = (TPage_BP2x*) TPageControl_AddNewUserPage(pagc, "BP2x", page_Add_BP2x_Create);
	self->art = PAGE_ART_FILTER;
	self->subClass = self;
	self->pgfilt = pgfilt;
	self->gname = "BP2x";

	pnl = TWins_CreateBA((TWins*)self, 0, 0, 318, 21, 0, alLeft);
	self->pnl = pnl;
	self->pnl->OnPaint = pnl_BP2x_OnPaint;

/* Rg Rs Rx Ra */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_Rg, &self->OnOff_Rg, &x, 2, 54, 1, get_Pic_cRg());
	DVal_Create_Xxx(pnl, &self->dv_Rs, &self->OnOff_Rs, &x, 2, 54, 1, get_Pic_cRs());
	DVal_Create_Xxx(pnl, &self->dv_Rx, &self->OnOff_Rx, &x, 2, 54, 1, get_Pic_cRx());
	DVal_Create_Xxx(pnl, &self->dv_Ra, &self->OnOff_Ra, &x, 2, 54, 1, get_Pic_cRa());
/* Cs Ca */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_Cs, &self->OnOff_Cs, &x, 22, 54, 1, get_Pic_cCs());
	DVal_Create_Xxx(pnl, &self->dv_Cx, &self->OnOff_Cx, &x, 22, 54, 1, get_Pic_cCx());
/* C1 C2 C3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_C1, &self->OnOff_C1, &x, 42, 54, 1, get_Pic_cC1());
	DVal_Create_Xxx(pnl, &self->dv_C2, &self->OnOff_C2, &x, 42, 54, 1, get_Pic_cC2());
	DVal_Create_Xxx(pnl, &self->dv_C3, &self->OnOff_C3, &x, 42, 54, 1, get_Pic_cC3());
/* L1 L2 L3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_L1, &self->OnOff_L1, &x, 62, 54, 3, get_Pic_cL1());
	DVal_Create_Xxx(pnl, &self->dv_L2, &self->OnOff_L2, &x, 62, 54, 3, get_Pic_cL2());

	m = TpcIni_RD_int(pcIni, "BP2x", "OnOff", 0xff05);

	self->cbCsOn = TChkBox_CreateS(pnl, 2, 82, 60, 16, "Cs on");
	self->cbRsOn = TChkBox_CreateS(pnl, 72, 82, 60, 16, "Rs on");
	self->cbCxOn = TChkBox_CreateS(pnl, 142, 82, 60, 16, "Cx on");
	self->cbRxOn = TChkBox_CreateS(pnl, 212, 82, 60, 16, "Rx on");
	self->cbCsOn->OnClick = cbBP2xXxOn_OnClick;
	self->cbRsOn->OnClick = cbBP2xXxOn_OnClick;
	self->cbCxOn->OnClick = cbBP2xXxOn_OnClick;
	self->cbRxOn->OnClick = cbBP2xXxOn_OnClick;
	self->cbCsOn->Checked = ((m & 0x01) != 0);
	self->cbRsOn->Checked = ((m & 0x02) != 0);
	self->cbCxOn->Checked = ((m & 0x04) != 0);
	self->cbRxOn->Checked = ((m & 0x08) != 0);

/* Filter On/Off */
	y = 105;
	self->cbF1On = TChkBox_CreateS(pnl, 2, y, 38, 17, "F1");
	self->cbF1ImpOn = TChkBox_CreateS(pnl, 2, y+17, 38, 17, "Imp");
	SetColor((TWins*)self->cbF1On, 0xa0a0ff);
	SetColor((TWins*)self->cbF1ImpOn, 0xe0e080);

/* Filter 2 */
	y += 45;
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_Cs_F2, &self->OnOff_Cs_F2, &x, y, 54, 1, get_Pic_cCs());
	DVal_Create_Xxx(pnl, &self->dv_Cx_F2, &self->OnOff_Cx_F2, &x, y, 54, 1, get_Pic_cCx());
	x = 0;
	y += 20;
	DVal_Create_Xxx(pnl, &self->dv_C1_F2, &self->OnOff_C1_F2, &x, y, 54, 1, get_Pic_cC1());
	DVal_Create_Xxx(pnl, &self->dv_C2_F2, &self->OnOff_C2_F2, &x, y, 54, 1, get_Pic_cC2());
	DVal_Create_Xxx(pnl, &self->dv_C3_F2, &self->OnOff_C3_F2, &x, y, 54, 1, get_Pic_cC3());
/* L1 L2 L3 */
	x = 0;
	y += 20;
	DVal_Create_Xxx(pnl, &self->dv_L1_F2, &self->OnOff_L1_F2, &x, y, 54, 3, get_Pic_cL1());
	DVal_Create_Xxx(pnl, &self->dv_L2_F2, &self->OnOff_L2_F2, &x, y, 54, 3, get_Pic_cL2());

	y += 26;
	self->cbF2On = TChkBox_CreateS(pnl, 2, y, 38, 17, "F2");
	self->cbF2ImpOn = TChkBox_CreateS(pnl, 2, y+17, 38, 17, "Imp");
	SetColor((TWins*)self->cbF2On, 0xa0ffa0);
	SetColor((TWins*)self->cbF2ImpOn, 0xe0e080);

	self->cbF1On->OnClick = cbBP2xFxOn_OnClick;
	self->cbF2On->OnClick = cbBP2xFxOn_OnClick;
	self->cbF1On->Checked = ((m & 0x010) != 0);
	self->cbF2On->Checked = ((m & 0x020) != 0);

	self->cbF1ImpOn->OnClick = cbBP2xFxImpOn_OnClick;
	self->cbF2ImpOn->OnClick = cbBP2xFxImpOn_OnClick;
	self->cbF1ImpOn->Checked = ((m & 0x700) == 0x100);
	self->cbF2ImpOn->Checked = ((m & 0x700) == 0x200);

	self->f_Rg = Create_fR(self, "BP2x Rg", "Rg", self->dv_Rg, 50);
	self->f_Rs = Create_fR(self, "BP2x Rs", "Rs", self->dv_Rs, 50);
	self->f_Rx = Create_fR(self, "BP2x Rx", "Rx", self->dv_Rx, 50);
	self->f_Ra = Create_fR(self, "BP2x Ra", "Ra", self->dv_Ra, 50);

	self->f_Cs = Create_fC(self, "BP2x Cs", "Cs", self->dv_Cs, 4);
	self->f_Cx = Create_fC(self, "BP2x Cx", "Cx", self->dv_Cx, 4);
	self->f_C1 = Create_fC(self, "BP2x C1", "C1", self->dv_C1, 8);
	self->f_C2 = Create_fC(self, "BP2x C2", "C2", self->dv_C2, 1.2);
	self->f_C3 = Create_fC(self, "BP2x C3", "C3", self->dv_C3, 8);

	self->f_L1 = Create_fL(self, "BP2x L1", "L1", self->dv_L1, 0.033);
	self->f_L2 = Create_fL(self, "BP2x L2", "L2", self->dv_L2, 0.033);


	self->f_Cs_F2 = Create_fC(self, "BP2x Cs F2", "CsF2", self->dv_Cs_F2, 3);
	self->f_Cx_F2 = Create_fC(self, "BP2x Cx F2", "CxF2", self->dv_Cx_F2, 3);
	self->f_C1_F2 = Create_fC(self, "BP2x C1 F2", "C1F2", self->dv_C1_F2, 8);
	self->f_C2_F2 = Create_fC(self, "BP2x C2 F2", "C2F2", self->dv_C2_F2, 0.9);
	self->f_C3_F2 = Create_fC(self, "BP2x C3 F2", "C3F2", self->dv_C3_F2, 8);

	self->f_L1_F2 = Create_fL(self, "BP2x L1 F2", "L1F2", self->dv_L1_F2, 0.02);
	self->f_L2_F2 = Create_fL(self, "BP2x L2 F2", "L2F2", self->dv_L2_F2, 0.02);

	TFormRLConoff_SetForm(self->OnOff_Rg, (TForm*)self->f_Rg);
	TFormRLConoff_SetForm(self->OnOff_Rs, (TForm*)self->f_Rs);
	TFormRLConoff_SetForm(self->OnOff_Rx, (TForm*)self->f_Rx);
	TFormRLConoff_SetForm(self->OnOff_Ra, (TForm*)self->f_Ra);
	TFormRLConoff_SetForm(self->OnOff_Cs, (TForm*)self->f_Cs);
	TFormRLConoff_SetForm(self->OnOff_Cx, (TForm*)self->f_Cx);
	TFormRLConoff_SetForm(self->OnOff_C1, (TForm*)self->f_C1);
	TFormRLConoff_SetForm(self->OnOff_C2, (TForm*)self->f_C2);
	TFormRLConoff_SetForm(self->OnOff_C3, (TForm*)self->f_C3);
	TFormRLConoff_SetForm(self->OnOff_L1, (TForm*)self->f_L1);
	TFormRLConoff_SetForm(self->OnOff_L2, (TForm*)self->f_L2);

	TFormRLConoff_SetForm(self->OnOff_Cs_F2, (TForm*)self->f_Cs_F2);
	TFormRLConoff_SetForm(self->OnOff_Cx_F2, (TForm*)self->f_Cx_F2);
	TFormRLConoff_SetForm(self->OnOff_C1_F2, (TForm*)self->f_C1_F2);
	TFormRLConoff_SetForm(self->OnOff_C2_F2, (TForm*)self->f_C2_F2);
	TFormRLConoff_SetForm(self->OnOff_C3_F2, (TForm*)self->f_C3_F2);
	TFormRLConoff_SetForm(self->OnOff_L1_F2, (TForm*)self->f_L1_F2);
	TFormRLConoff_SetForm(self->OnOff_L2_F2, (TForm*)self->f_L2_F2);

	self->ScaleMHz = TpcIni_RD_float(pcIni, "BP2x", "ScaleMHz", 1000);
	self->OnMsg = self_OnMsg;

	self->SaveData = SaveData;
	self->LoadData = LoadData;

	free_appIni(pcIni);

	return self;
}

