/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "ElMath.h"
#include "ElGraf.h"
#include <math.h>

static void str_toGraf(TWins *Wins, const char *s, int x, int y) {
	vds_SetStr(&Wins->vds, s);
	DrawColorString(x, y, &Wins->vds);
}

static void DrawNF1(Window WinID, int x, int y, int fu) {
  int xa, xn, y2;
  int x2 = x;

	y2 = y + 36;
	if ((fu & 0x0300) == 0x000) {
	    Draw_xP1_Rg(WinID, &x2, &xa, y, y2, fu);
	    x2 += 3;
	    xn = x2;
	    DrawTP1_Ch(WinID, &x2, y, get_Pic_cC1());
	    DrawLineRGB(WinID, 0, xn-1, y+3, xn-1, y+20);
	    DrawPic(WinID, get_Pic_pt(), xn-3, y-2);
	    DrawTP1_Lh(WinID, &xn, y + 20, get_Pic_cL1());
	    DrawLineRGB(WinID, 0, xn, y+3, xn, y+20);
	    DrawPic(WinID, get_Pic_pt(), xn-2, y-2);
	    x2 += 3;
	}

	if ((fu & 0x0300) == 0x100) {
	    y2 = y+46;
	    Draw_xP1_Rg(WinID, &x2, &xa, y, y2, fu);

	    xn = x2;
	    DrawTP1_Cv_pt(WinID, &x2, 0, y, get_Pic_cC2(), PT_TOP);
	    DrawTP1_Lv_pt(WinID, &xn, 0, y + 22, get_Pic_cL2(), PT_BOTTOM | PT_BOTTOM);
	}

	if ((fu & 0x0300) == 0x200) {
	    y2 = y+46;
	    Draw_xP1_Rg(WinID, &x2, &xa, y, y2, fu);

	    x2 += 3;
	    xn = x2;
	    DrawTP1_Ch(WinID, &x2, y, get_Pic_cC1());
	    DrawLineRGB(WinID, 0, xn-1, y+3, xn-1, y+20);
	    DrawPic(WinID, get_Pic_pt(), xn-3, y-2);
	    DrawTP1_Lh(WinID, &xn, y + 20, get_Pic_cL1());
	    DrawLineRGB(WinID, 0, xn, y+3, xn, y+20);
	    DrawPic(WinID, get_Pic_pt(), xn-2, y-2);

	    xn = x2;
	    DrawTP1_Cv_pt(WinID, &x2, 6, y, get_Pic_cC2(), PT_TOP);
	    DrawTP1_Lv_pt(WinID, &xn, 6, y + 22, get_Pic_cL2(), PT_BOTTOM | PT_BOTTOM);

	    DrawLineRGB(WinID, 0, x2, y, x2+10, y);
	    x2 += 14;
	    xn = x2;
	    DrawTP1_Ch(WinID, &x2, y, get_Pic_cC3());
	    DrawLineRGB(WinID, 0, xn-1, y+3, xn-1, y+20);
	    DrawPic(WinID, get_Pic_pt(), xn-3, y-2);
	    DrawTP1_Lh(WinID, &xn, y + 20, get_Pic_cL3());
	    DrawLineRGB(WinID, 0, xn, y+3, xn, y+20);
	    DrawPic(WinID, get_Pic_pt(), xn-2, y-2);
	    x2 += 3;
	}

	Draw_xP1_Ra(WinID, x2, xa, y, y2, fu);
}

static float ImpF1(TPage_NF1 *self, float MHz, float *Z) {
  float res, w;
  TRLC btIn;

	w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	res *= btIn.Z;

	Form_C_math(self->f_C1, w);
	Form_L_math(self->f_L1, w);

	RLC_Add_Par(&self->f_L1->rlc, &self->f_C1->rlc);
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
} 

static float ImpF2(TPage_NF1 *self, float MHz, float *Z) {
  float res, w;
  TRLC btIn;

	w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;

	Form_C_math(self->f_C2, w);
	Form_L_math(self->f_L2, w);

	RLC_Add_Ser(&self->f_L2->rlc, &self->f_C2->rlc);
	RLC_Add_Par(&btIn, &self->f_L2->rlc);
	res *= btIn.Z;

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
}

static float ImpF3(TPage_NF1 *self, float MHz, float *Z) {
  float res, w;
  TRLC btIn;

	w = MHz * M_PI * 2;
	if (!XF_MathOut((TPage_XFilter*)self, w, &btIn, &res)) return res;
	res *= btIn.Z;

	Form_C_math(self->f_C1, w);
	Form_C_math(self->f_C2, w);
	Form_C_math(self->f_C3, w);
	Form_L_math(self->f_L1, w);
	Form_L_math(self->f_L2, w);
	Form_L_math(self->f_L3, w);

	RLC_Add_Par(&self->f_L3->rlc, &self->f_C3->rlc);
	RLC_Add_Ser(&btIn, &self->f_L3->rlc);
	res /= btIn.Z;

	RLC_Add_Ser(&self->f_L2->rlc, &self->f_C2->rlc);
	RLC_Add_Par(&btIn, &self->f_L2->rlc);
	res *= btIn.Z;

	RLC_Add_Par(&self->f_L1->rlc, &self->f_C1->rlc);
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);

	XF_MathIn((TPage_XFilter*)self, w, &btIn, &res, Z);
	return res;
}



static void math_Filter(TPage_NF1 *self) {
  TPage_Filter *pgf = self->pgfilt;
  float fcStep, fOff, fStep;
  TScale_pxy *pxy = pgf->xy_ind->dat->pxy;
  TScale_pxy *pxy2;
  TScale_pxy *pxy3 = NULL;
  float fc, Z, K;
  TScale_Point *sSP, *dSP;

	K = 1;
	if (pgf->cbRgRa->Checked)
	    K = (self->f_Rg->rlc.val + self->f_Ra->rlc.val) / self->f_Ra->rlc.val;

	fOff = pgf->ScaleMHzoff;
	fStep = pgf->ScaleMHz;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF1(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF1ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF2(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF2ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF3(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF3ImpOn->Checked) pxy3 = pxy;

	Z = self->dv_Rg->dval;
	if (pxy3 && (Z > 0)) {
	    pxy2 = pgf->yInZ;
	    pxy2->PaintDat = True;
	    sSP = &pxy3->xy[0];
	    dSP = &pxy2->xy[0];
	    for (int i=0;i<pxy2->count;i++) {
		dSP->x = sSP->x;
		dSP->y = sSP->y2 / Z;
		dSP++; sSP++;
	    }
	} else {
	    pgf->yInZ->PaintDat = False;
	}

	SetCursor_Dat_txt(pgf);
	Refresh((TWins*)pgf->xy_ind);
}


void NF1_SetVisible(TPage_NF1 *self, bool On) {
	SetVisible((TWins*)self->f_C1, self->f_C1->tag && On);
	SetVisible((TWins*)self->f_C2, self->f_C2->tag && On);
	SetVisible((TWins*)self->f_C3, self->f_C3->tag && On);
	SetVisible((TWins*)self->f_L1, self->f_L1->tag && On);
	SetVisible((TWins*)self->f_L2, self->f_L2->tag && On);
	SetVisible((TWins*)self->f_L3, self->f_L3->tag && On);
	XF_SetVisible((TPage_XFilter*)self, On);
}

/* ************** NF1 Ext Ev ****************** */
static bool MathImp(TPage_NF1 *self, TMgsMathImp *pImp) {
	if (!pImp) return false;
	if (self->cbF1ImpOn->Checked) {
	    pImp->yVal = ImpF1(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF2ImpOn->Checked) {
	    pImp->yVal = ImpF2(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF3ImpOn->Checked) {
	    pImp->yVal = ImpF3(self, pImp->MHz, &pImp->Z);
	    return true;
	}
	return false; 
}

static void DVal_Update(TFormRLC *rlc, double val) {
	if (rlc->Update) rlc->Update(rlc, val, 0);
	if (rlc->SaveData) rlc->SaveData(rlc, NULL);
}

static bool self_OnMsg(TPage_NF1 *self, int msg, void *p) {
  TpcIni *pcIni;
  int i;
  TFormSetLC0 *LC0;
  float xf;

	switch (msg & MSG_BEF_MASK) {
	case MSG_MATH_FILTER:
	    math_Filter(self);
	    return true;

	case MSG_MATH_IMP:
	    return MathImp(self, p);

	case MSG_NEW_SCALE_PARAM:
	    pcIni = get_appIni();
	    i = 1;
	    if (self->ScaleMHz >= 1) i = 0;
	    TpcIni_WR_float(pcIni, self->gname, "ScaleMHz", self->ScaleMHz, i);
	    TpcIni_Save(pcIni);
	    free_appIni(pcIni);
	    return true;

	case MSG_SetLC0:
	    if (!p) return false;
	    LC0 = p;
	    switch (LC0->nr) {
	    case 1:
		xf = LC0->K / 2;
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * xf);
		DVal_Update((TFormRLC*)self->f_L1, LC0->L / xf);
		DVal_Update((TFormRLC*)self->f_C2, LC0->C / xf);
		DVal_Update((TFormRLC*)self->f_L2, LC0->L * xf);
		DVal_Update((TFormRLC*)self->f_C3, LC0->C * xf);
		DVal_Update((TFormRLC*)self->f_L3, LC0->L / xf);
		math_Filter(self);
		return true;
	    }
	    return false;
	}
	return false;
}

/* ************** NF1 Ev ****************** */
static void cbXxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_NF1 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	XF_SaveOnOff((TPage_XFilter*)pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	RePaint((TWins*)pg->pnl);
	math_Filter(pg);
}

static void cbFxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_NF1 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	XF_SaveOnOff((TPage_XFilter*)pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	((TScale_pxy*)self->tag)->PaintDat = ((TChkBox*)self)->Checked;
	Refresh((TWins*)pg->pgfilt->xy_ind);
}

static void cbFxImpOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_NF1 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	if (pg->cbF1ImpOn != self) TChkBox_SetChecked(pg->cbF1ImpOn, False);
	if (pg->cbF2ImpOn != self) TChkBox_SetChecked(pg->cbF2ImpOn, False);
	if (pg->cbF3ImpOn != self) TChkBox_SetChecked(pg->cbF3ImpOn, False);
	XF_SaveOnOff((TPage_XFilter*)pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	math_Filter(pg);
	Refresh((TWins*)pg->pgfilt->xy_ind);
}

static void pnl_OnPaint(TWins *self) {
  TPage_NF1 *pg = self->subClass;
  int fu, x;

	fu = XF_GetPaint_Fu((TPage_XFilter*)pg);
	x = pg->cbF1On->left + pg->cbF1On->width + 2;
	DrawNF1(self->WinID, x, pg->cbF1On->top, fu);
	fu += 0x100;
	DrawNF1(self->WinID, x, pg->cbF2On->top, fu);
	fu += 0x100;
	DrawNF1(self->WinID, x, pg->cbF3On->top, fu);

	DrawPic(self->WinID, get_Pic_cOhm(), 
			pg->dv_Ra->left + pg->dv_Ra->width + 2,
			pg->dv_Ra->top + 5);

	str_toGraf(self, "pF", pg->dv_Cx->left + pg->dv_Cx->width + 1,
			       pg->dv_Cx->top + 3);

	str_toGraf(self, "pF", pg->dv_C3->left + pg->dv_C3->width + 1,
			       pg->dv_C3->top + 3);

	str_toGraf(self, "uH", pg->dv_L3->left + pg->dv_L3->width + 1,
			       pg->dv_L3->top + 3);
}

static void DVal_OnValChange(TDVal *dv, double val) {
	if (dv->owner) TDVal_DoValChange((TDVal*)dv->owner, val, tdvfu_ChangeEv);
}

static void fXxx_OnChange(TFormRLC* fRLC) {
	math_Filter((TPage_NF1*)fRLC->subClass);
}
/*static void OnRSync(TFormRLC* fRLC, int val) {
  TPage_NF1 *pg = fRLC->subClass;
	Form_R_sync(fRLC, pg->f_Rg, val);
	Form_R_sync(fRLC, pg->f_Rs, val);
	Form_R_sync(fRLC, pg->f_Rx, val);
	Form_R_sync(fRLC, pg->f_Ra, val);
} */
static void OnLSync(TFormRLC* fRLC, int val) {
  TPage_NF1 *pg = fRLC->subClass;
	Form_L_sync(fRLC, pg->f_L1, val);
	Form_L_sync(fRLC, pg->f_L2, val);
	Form_L_sync(fRLC, pg->f_L3, val);
}
static void OnCSync(TFormRLC* fRLC, int val) {
  TPage_NF1 *pg = fRLC->subClass;
//	Form_C_sync(fRLC, pg->f_Cs, val);
//	Form_C_sync(fRLC, pg->f_Cx, val);
	Form_C_sync(fRLC, pg->f_C1, val);
	Form_C_sync(fRLC, pg->f_C2, val);
	Form_C_sync(fRLC, pg->f_C3, val);
}
/* ************** Save Load ****************** */
static void SaveData(TPage_NF1 *self, TpcIni *ini) {

	XF_SaveData((TPage_XFilter*)self, ini);

	self->f_L1->SaveData(self->f_L1, ini);
	self->f_L2->SaveData(self->f_L2, ini);
	self->f_L3->SaveData(self->f_L3, ini);

	self->f_C1->SaveData(self->f_C1, ini);
	self->f_C2->SaveData(self->f_C2, ini);
	self->f_C3->SaveData(self->f_C3, ini);
}

static void LoadData(TPage_NF1 *self, TpcIni *ini) {

	XF_LoadData((TPage_XFilter*)self, ini);

	self->f_L1->LoadData(self->f_L1, ini);
	self->f_L2->LoadData(self->f_L2, ini);
	self->f_L3->LoadData(self->f_L3, ini);

	self->f_C1->LoadData(self->f_C1, ini);
	self->f_C2->LoadData(self->f_C2, ini);
	self->f_C3->LoadData(self->f_C3, ini);

	math_Filter(self);
	Refresh((TWins*)self->pgfilt->xy_ind);
}

/* ************** NF1 Create ****************** */
static void DVal_Create_Xxx(TWins *Parent, TDVal **dv, TFormRLConoff **onoff, int *x, int y, int w, int nk, TPic *pic) {
  int tx = *x;
  TDVal *tdv = TDVal_CreateS(Parent, tx+19, y, w, 19);
	tdv->nk = nk;
	tdv->dv_fu = tdv_fu_LostFocusEnter;
	tdv->OnValChange = DVal_OnValChange;
	*dv = tdv;
	*onoff = TFormRLConoff_CreateU(Parent, tx, y+1, 18, 16, pic);
	*x = tx + w + 21;
}

static TFormL *Create_f_L(TPage_NF1 *pg, const char *Caption, const char *Gruppe, 
			const char *s, TDVal *dv, float val) {
  TFormL *fL = Create_Form_L(appDat.obj, Caption, Gruppe, s, dv, val);
	fL->subClass = pg;
	TDVal_Set_Value(dv, fL->L->dval);
	fL->OnChange = fXxx_OnChange;
	fL->OnSync = OnLSync;
	return fL;
}
static TFormC *Create_f_C(TPage_NF1 *pg, const char *Caption, const char *Gruppe, 
			const char *s, TDVal *dv, float val) {
  TFormC *fC = Create_Form_C(appDat.obj, Caption, Gruppe, s, dv, val);
	fC->subClass = pg;
	TDVal_Set_Value(dv, fC->C->dval);
	fC->OnChange = fXxx_OnChange;
	fC->OnSync = OnCSync;
	return fC;
}


TPage_NF1 *page_Add_NF1(TPageControl *pagc, TPage_Filter *pgfilt) {
  TPage_NF1 *self;
  TWins *pnl;
  int x;
  TAddXFilter xf;

	xf.pagc = pagc;
	xf.pgfilt = pgfilt;
	xf.gname = "NF1";
	xf.psize = sizeof(TPage_NF1);
	xf.ini = get_appIni();
	xf.TDValOnValChange = DVal_OnValChange;
	xf.fXxx_OnChange = fXxx_OnChange;
	xf.OnRSync = NULL;
	xf.OnLSync = OnLSync;
	xf.OnCSync = OnCSync;
	xf.dvfu = FU_RG|FU_RS|FU_RX|FU_RA|FU_CS|FU_CX;

	xf.cb_OnClick = cbXxOn_OnClick;
	xf.cbfu = FU_RS|FU_RX|FU_CS|FU_CX;
	xf.cbOnOff = 0xffe0;
	xf.cbY = 82;

	xf.cbFxOnClick = cbFxOn_OnClick;
	xf.cbFxImpOnClick = cbFxImpOn_OnClick;
	xf.cbF1Y = 105;
	xf.cbF2Y = 155;
	xf.cbF3Y = 215;
	xf.cbF4Y = 240;
	xf.cbFxfu = FU_F1|FU_F2|FU_F3;

	xf.LC0_Popupfu = FU_F1;

	self = (TPage_NF1*)PageAddXFilter(&xf);

	pnl = self->pnl;
	pnl->OnPaint = pnl_OnPaint;

/* C1 C2 C3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_C1, &self->OnOff_C1, &x, 42, 54, 1, get_Pic_cC1());
	DVal_Create_Xxx(pnl, &self->dv_C2, &self->OnOff_C2, &x, 42, 54, 1, get_Pic_cC2());
	DVal_Create_Xxx(pnl, &self->dv_C3, &self->OnOff_C3, &x, 42, 54, 1, get_Pic_cC3());
/* L1 L2 L3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_L1, &self->OnOff_L1, &x, 62, 54, 3, get_Pic_cL1());
	DVal_Create_Xxx(pnl, &self->dv_L2, &self->OnOff_L2, &x, 62, 54, 3, get_Pic_cL2());
	DVal_Create_Xxx(pnl, &self->dv_L3, &self->OnOff_L3, &x, 62, 54, 3, get_Pic_cL3());

	self->f_C1 = Create_f_C(self, "NF1 C1", self->gname, "C1", self->dv_C1, 50);
	self->f_C2 = Create_f_C(self, "NF1 C2", self->gname, "C2", self->dv_C2, 50);
	self->f_C3 = Create_f_C(self, "NF1 C3", self->gname, "C3", self->dv_C3, 50);

	self->f_L1 = Create_f_L(self, "NF1 L1", self->gname, "L1", self->dv_L1, 0.05);
	self->f_L2 = Create_f_L(self, "NF1 L2", self->gname, "L2", self->dv_L2, 0.05);
	self->f_L3 = Create_f_L(self, "NF1 L3", self->gname, "L3", self->dv_L3, 0.05);

	TFormRLConoff_SetForm(self->OnOff_C1, (TForm*)self->f_C1);
	TFormRLConoff_SetForm(self->OnOff_C2, (TForm*)self->f_C2);
	TFormRLConoff_SetForm(self->OnOff_C3, (TForm*)self->f_C3);
	TFormRLConoff_SetForm(self->OnOff_L1, (TForm*)self->f_L1);
	TFormRLConoff_SetForm(self->OnOff_L2, (TForm*)self->f_L2);
	TFormRLConoff_SetForm(self->OnOff_L3, (TForm*)self->f_L3);

	self->OnMsg = self_OnMsg;
	self->SaveData = SaveData;
	self->LoadData = LoadData;

	free_appIni(xf.ini);

	return self;
}
