/*
 Copyright (C) 2010-2015 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "ElMath.h"
#include "ElGraf.h"
#include <math.h>

static void str_toGraf(TWins *Wins, const char *s, int x, int y) {
	vds_SetStr(&Wins->vds, s);
	DrawColorString(x, y, &Wins->vds);
}

static void DrawSch(Window WinID, int x, int y, int fu) {
  int xa;
  int x2 = x;
  int y2 = y+30;

	Draw_xP1_Rg(WinID, &x2, &xa, y, y2, fu);

	if ((fu & 0x0300) == 0x000) {
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	}

	if ((fu & 0x0300) == 0x100) {
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL1());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL4());
	}
	if ((fu & 0x0300) == 0x200) {
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL1());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL2());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC3());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL4());
	}

	if ((fu & 0x0300) == 0x300) {
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL1());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC1());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL2());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC2());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL3());
	    DrawTP1_Cv(WinID, &x2, 0, y, get_Pic_cC3());
	    DrawTP1_Lh(WinID, &x2, y, get_Pic_cL4());
	}

	Draw_xP1_Ra(WinID, x2, xa, y, y2, fu);
}

static bool Imp_MathOut_rc(TPage_TP2 *self, float w, TRLC *btIn, float *ret) {
  float res;
	if (w < 0.000001) {
	    if (self->cbCxOn->Checked || self->cbCsOn->Checked) return 0;
	    res = self->f_Rs->R->dval;
	    if (self->cbRxOn->Checked) res += self->f_Rx->R->dval;
	    *ret = self->f_Ra->R->dval / (self->f_Ra->R->dval + res);
	    return False;
	}
	Form_R_math(self->f_Ra, w);

	res = 1;
	RLC_Copy(btIn, &self->f_Ra->rlc);
	if (self->cbCxOn->Checked || self->cbRxOn->Checked) {
	    res = btIn->Z;
	    if (self->cbCxOn->Checked) {
		Form_C_math(self->f_Cx, w);
		RLC_Add_Ser(btIn, &self->f_Cx->rlc);
	    }
	    if (self->cbRxOn->Checked) {
		Form_R_math(self->f_Rx, w);
		RLC_Add_Ser(btIn, &self->f_Rx->rlc);
	    }
	    res /= btIn->Z;
	}
	*ret = res;
	return True;
}

static void Imp_MathIn_rc(TPage_TP2 *self, float w, TRLC *btIn, float *ret, float *Z) {
	if (self->cbCsOn->Checked) {
	    Form_C_math(self->f_Cs, w);
	    RLC_Add_Ser(btIn, &self->f_Cs->rlc);
	}
	*Z = 0;
	if (self->cbRsOn->Checked) {
	    Form_R_math(self->f_Rs, w);
	    RLC_Add_Ser(btIn, &self->f_Rs->rlc);
	}
	if (self->f_Rg->R->dval > 0) {
	    *Z = btIn->Z;
	    Form_R_math(self->f_Rg, w);
	    RLC_Add_Ser(btIn, &self->f_Rg->rlc);
	}
  float res = *ret;
	res /= btIn->Z;
	*ret = res;
}

static float ImpF1(TPage_TP2 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!Imp_MathOut_rc(self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C1, w);
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;

	Imp_MathIn_rc(self, w, &btIn, &res, Z);
	return res;
} 
static float ImpF2(TPage_TP2 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!Imp_MathOut_rc(self, w, &btIn, &res)) return res;
	Form_L_math(self->f_L1, w);
	Form_C_math(self->f_C1, w);
	Form_L_math(self->f_L4, w);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L4->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);

	Imp_MathIn_rc(self, w, &btIn, &res, Z);
	return res;
}
static float ImpF3(TPage_TP2 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!Imp_MathOut_rc(self, w, &btIn, &res)) return res;
	Form_L_math(self->f_L1, w);
	Form_L_math(self->f_L2, w);
	Form_C_math(self->f_C1, w);
	Form_C_math(self->f_C3, w);
	Form_L_math(self->f_L4, w);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L4->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C3->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L2->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);

	Imp_MathIn_rc(self, w, &btIn, &res, Z);
	return res;
}
static float ImpF4(TPage_TP2 *self, float MHz, float *Z) {
  float res;
  TRLC btIn;
  float w = MHz * M_PI * 2;
	if (!Imp_MathOut_rc(self, w, &btIn, &res)) return res;
	Form_C_math(self->f_C1, w);
	Form_C_math(self->f_C2, w);
	Form_C_math(self->f_C3, w);
	Form_L_math(self->f_L1, w);
	Form_L_math(self->f_L2, w);
	Form_L_math(self->f_L3, w);
	Form_L_math(self->f_L4, w);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L4->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C3->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L3->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C2->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L2->rlc);
	res /= btIn.Z;
	RLC_Add_Par(&btIn, &self->f_C1->rlc);
	res *= btIn.Z;
	RLC_Add_Ser(&btIn, &self->f_L1->rlc);

	Imp_MathIn_rc(self, w, &btIn, &res, Z);
	return res;
}

static void math_Filter(TPage_TP2 *self) {
  TPage_Filter *pgf = self->pgfilt;
  float fcStep, fOff, fStep;
  TScale_pxy *pxy = pgf->xy_ind->dat->pxy;
  TScale_pxy *pxy2;
  TScale_pxy *pxy3 = NULL;
  float fc, Z, K;
  TScale_Point *sSP, *dSP;

// printf("%s(%d): math_Filter\n", FL);
	K = 1;
	if (pgf->cbRgRa->Checked)
	    K = (self->f_Rg->rlc.val + self->f_Ra->rlc.val) / self->f_Ra->rlc.val;

	fOff = pgf->ScaleMHzoff;
	fStep = pgf->ScaleMHz;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF1(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF1ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF2(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF2ImpOn->Checked) pxy3 = pxy;

	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF3(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF3ImpOn->Checked) pxy3 = pxy;


	pxy = pxy->next;
	fcStep = fStep / (pxy->count-1);
	for (int i=0;i<pxy->count;i++) {
	    fc = i * fcStep + fOff;
	    pxy->xy[i].x = fc;
	    pxy->xy[i].y = K * ImpF4(self, fc, &Z);
	    pxy->xy[i].y2 = Z;
	}
	if (self->cbF4ImpOn->Checked) pxy3 = pxy;

	Z = self->dv_Rg->dval;
	if (pxy3 && (Z > 0)) {
	    pxy2 = pgf->yInZ;
	    pxy2->PaintDat = True;
	    sSP = &pxy3->xy[0];
	    dSP = &pxy2->xy[0];
	    for (int i=0;i<pxy2->count;i++) {
		dSP->x = sSP->x;
		dSP->y = sSP->y2 / Z;
		dSP++; sSP++;
	    }
	} else {
	    pgf->yInZ->PaintDat = False;
	}

	SetCursor_Dat_txt(pgf);
	Refresh((TWins*)pgf->xy_ind);

}

static void SaveOnOff(TPage_TP2 *self, TpcIni *pcIni) {
  int OnOff = 0;

	if (self->cbCsOn->Checked) OnOff |= 0x01;
	if (self->cbRsOn->Checked) OnOff |= 0x02;
	if (self->cbCxOn->Checked) OnOff |= 0x04;
	if (self->cbRxOn->Checked) OnOff |= 0x08;

	if (self->cbF1On->Checked) OnOff |= 0x010;
	if (self->cbF2On->Checked) OnOff |= 0x020;
	if (self->cbF3On->Checked) OnOff |= 0x040;
	if (self->cbF4On->Checked) OnOff |= 0x080;

	if (self->cbF1ImpOn->Checked) OnOff |= 0x100;
	if (self->cbF2ImpOn->Checked) OnOff |= 0x200;
	if (self->cbF3ImpOn->Checked) OnOff |= 0x300;
	if (self->cbF4ImpOn->Checked) OnOff |= 0x400;

	TpcIni_WR_int(pcIni, self->gname, "OnOff", OnOff);
}

void TP2_SetVisible(TPage_TP2 *self, bool On) {
	SetVisible((TWins*)self->f_Rg, self->f_Rg->tag && On);
	SetVisible((TWins*)self->f_Rs, self->f_Rs->tag && On);
	SetVisible((TWins*)self->f_Rx, self->f_Rx->tag && On);
	SetVisible((TWins*)self->f_Ra, self->f_Ra->tag && On);
	SetVisible((TWins*)self->f_Cs, self->f_Cs->tag && On);
	SetVisible((TWins*)self->f_Cx, self->f_Cx->tag && On);
	SetVisible((TWins*)self->f_C1, self->f_C1->tag && On);
	SetVisible((TWins*)self->f_C2, self->f_C2->tag && On);
	SetVisible((TWins*)self->f_C3, self->f_C3->tag && On);
	SetVisible((TWins*)self->f_L1, self->f_L1->tag && On);
	SetVisible((TWins*)self->f_L2, self->f_L2->tag && On);
	SetVisible((TWins*)self->f_L3, self->f_L3->tag && On);
	SetVisible((TWins*)self->f_L4, self->f_L4->tag && On);
  TScale_pxy *pxy;
	if (On) {
	    pxy = self->pgfilt->xy_ind->dat->pxy;
	    while (pxy) {
		pxy->PaintDat = False;
		pxy = pxy->next;
	    }
	    math_Filter(self);
	    ((TScale_pxy*)self->cbF1On->tag)->PaintDat = self->cbF1On->Checked;
	    ((TScale_pxy*)self->cbF2On->tag)->PaintDat = self->cbF2On->Checked;
	    ((TScale_pxy*)self->cbF3On->tag)->PaintDat = self->cbF3On->Checked;
	    ((TScale_pxy*)self->cbF4On->tag)->PaintDat = self->cbF4On->Checked;
	}
}

/* ************** Ext Ev ****************** */

static bool MathImp(TPage_TP2 *self, TMgsMathImp *pImp) {
	if (!pImp) return false;
	if (self->cbF1ImpOn->Checked) {
	    pImp->yVal = ImpF1(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF2ImpOn->Checked) {
	    pImp->yVal = ImpF2(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF3ImpOn->Checked) {
	    pImp->yVal = ImpF3(self, pImp->MHz, &pImp->Z);
	    return true;
	} else if (self->cbF4ImpOn->Checked) {
	    pImp->yVal = ImpF4(self, pImp->MHz, &pImp->Z);
	    return true;
	}
	return false; 
}

static void DVal_Update(TFormRLC *rlc, double val) {
	if (rlc->Update) rlc->Update(rlc, val, 0);
	if (rlc->SaveData) rlc->SaveData(rlc, NULL);
}

static bool self_OnMsg(TPage_TP2 *self, int msg, void *p) {
  int i;
  TpcIni *pcIni;
  TFormSetLC0 *LC0;

	switch (msg & MSG_BEF_MASK) {
	case MSG_MATH_FILTER:
	    math_Filter(self);
	    return true;

	case MSG_MATH_IMP:
	    return MathImp(self, p);

	case MSG_NEW_SCALE_PARAM:
	    pcIni = get_appIni();
	    i = 1;
	    if (self->ScaleMHz >= 1) i = 0;
	    TpcIni_WR_float(pcIni, self->gname, "ScaleMHz", self->ScaleMHz, i);
	    TpcIni_Save(pcIni);
	    free_appIni(pcIni);
	    return true;

	case MSG_SetLC0:
	    if (!p) return false;
	    LC0 = p;
	    switch (LC0->nr) {
	    case 1:
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[0]);
		math_Filter(self);
		return true;
	    case 2:
		DVal_Update((TFormRLC*)self->f_L1, LC0->L * LC0->v[0]);
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[1]);
		DVal_Update((TFormRLC*)self->f_L4, LC0->L * LC0->v[2]);
		math_Filter(self);
		return true;
	    case 3:
		DVal_Update((TFormRLC*)self->f_L1, LC0->L * LC0->v[0]);
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[1]);
		DVal_Update((TFormRLC*)self->f_L2, LC0->L * LC0->v[2]);
		DVal_Update((TFormRLC*)self->f_C3, LC0->C * LC0->v[3]);
		DVal_Update((TFormRLC*)self->f_L4, LC0->L * LC0->v[4]);
		math_Filter(self);
		return true;
	    case 4:
		DVal_Update((TFormRLC*)self->f_L1, LC0->L * LC0->v[0]);
		DVal_Update((TFormRLC*)self->f_C1, LC0->C * LC0->v[1]);
		DVal_Update((TFormRLC*)self->f_L2, LC0->L * LC0->v[2]);
		DVal_Update((TFormRLC*)self->f_C2, LC0->C * LC0->v[3]);
		DVal_Update((TFormRLC*)self->f_L3, LC0->L * LC0->v[4]);
		DVal_Update((TFormRLC*)self->f_C3, LC0->C * LC0->v[5]);
		DVal_Update((TFormRLC*)self->f_L4, LC0->L * LC0->v[6]);
		math_Filter(self);
		return true;
	    }
	    return false;
	}
	return false;
}

/* ************** Ev ****************** */
static void cbXxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_TP2 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	SaveOnOff(pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	RePaint((TWins*)pg->pnl);
	math_Filter(pg);
}

static void cbFxOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_TP2 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	SaveOnOff(pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	((TScale_pxy*)self->tag)->PaintDat = ((TChkBox*)self)->Checked;
	Refresh((TWins*)pg->pgfilt->xy_ind);
}

static void cbFxImpOn_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TPage_TP2 *pg = self->subClass;
  TpcIni *pcIni = TpcIni_Create(appDat.inifile_app);
	if (pg->cbF1ImpOn != self) TChkBox_SetChecked(pg->cbF1ImpOn, False);
	if (pg->cbF2ImpOn != self) TChkBox_SetChecked(pg->cbF2ImpOn, False);
	if (pg->cbF3ImpOn != self) TChkBox_SetChecked(pg->cbF3ImpOn, False);
	if (pg->cbF4ImpOn != self) TChkBox_SetChecked(pg->cbF4ImpOn, False);
	SaveOnOff(pg, pcIni);
	TpcIni_SaveAndFree(pcIni);
	math_Filter(pg);
	Refresh((TWins*)pg->pgfilt->xy_ind);
}

static void pnl_OnPaint(TWins *self) {
  TPage_TP2 *pg = self->subClass;
  int fu = 0;
  int x = pg->cbF1On->left + pg->cbF1On->width + 2;

	if (pg->cbCsOn->Checked) fu |= 0x01;
	if (pg->cbRsOn->Checked) fu |= 0x02;
	if (pg->cbCxOn->Checked) fu |= 0x04;
	if (pg->cbRxOn->Checked) fu |= 0x08;

	DrawSch(self->WinID, x, pg->cbF1On->top, fu);
	fu += 0x100;
	DrawSch(self->WinID, x, pg->cbF2On->top, fu);
	fu += 0x100;
	DrawSch(self->WinID, x, pg->cbF3On->top, fu);
	fu += 0x100;
	DrawSch(self->WinID, x, pg->cbF4On->top, fu);

	DrawPic(self->WinID, get_Pic_cOhm(), 
			pg->dv_Ra->left + pg->dv_Ra->width + 2,
			pg->dv_Ra->top + 5);

	str_toGraf(self, "pF", pg->dv_Cx->left + pg->dv_Cx->width + 1,
			       pg->dv_Cx->top + 3);

	str_toGraf(self, "pF", pg->dv_C3->left + pg->dv_C3->width + 1,
			       pg->dv_C3->top + 3);

	str_toGraf(self, "uH", pg->dv_L4->left + pg->dv_L4->width + 1,
			       pg->dv_L4->top + 3);
}

static void DVal_OnValChange(TDVal *dv, double val) {
	if (dv->owner) TDVal_DoValChange((TDVal*)dv->owner, val, tdvfu_ChangeEv);
}

static void fXxx_OnChange(TFormRLC* fRLC) {
	math_Filter((TPage_TP2*)fRLC->subClass);
}
static void OnRSync(TFormRLC* fRLC, int val) {
  TPage_TP2 *pg = fRLC->subClass;
	Form_R_sync(fRLC, pg->f_Rg, val);
	Form_R_sync(fRLC, pg->f_Rs, val);
	Form_R_sync(fRLC, pg->f_Rx, val);
	Form_R_sync(fRLC, pg->f_Ra, val);
}
static void OnLSync(TFormRLC* fRLC, int val) {
  TPage_TP2 *pg = fRLC->subClass;
	Form_L_sync(fRLC, pg->f_L1, val);
	Form_L_sync(fRLC, pg->f_L2, val);
	Form_L_sync(fRLC, pg->f_L3, val);
	Form_L_sync(fRLC, pg->f_L4, val);
}
static void OnCSync(TFormRLC* fRLC, int val) {
  TPage_TP2 *pg = fRLC->subClass;
	Form_C_sync(fRLC, pg->f_Cs, val);
	Form_C_sync(fRLC, pg->f_Cx, val);
	Form_C_sync(fRLC, pg->f_C1, val);
	Form_C_sync(fRLC, pg->f_C2, val);
	Form_C_sync(fRLC, pg->f_C3, val);
}
/* ************** Save Load ****************** */
static void SaveData(TPage_TP2 *self, TpcIni *ini) {
	self->f_Rg->SaveData(self->f_Rg, ini);
	self->f_Rs->SaveData(self->f_Rs, ini);
	self->f_Rx->SaveData(self->f_Rx, ini);
	self->f_Ra->SaveData(self->f_Ra, ini);

	self->f_L1->SaveData(self->f_L1, ini);
	self->f_L2->SaveData(self->f_L2, ini);
	self->f_L3->SaveData(self->f_L3, ini);
	self->f_L4->SaveData(self->f_L4, ini);

	self->f_Cs->SaveData(self->f_Cs, ini);
	self->f_Cx->SaveData(self->f_Cx, ini);
	self->f_C1->SaveData(self->f_C1, ini);
	self->f_C2->SaveData(self->f_C2, ini);
	self->f_C3->SaveData(self->f_C3, ini);
}

static void LoadData(TPage_TP2 *self, TpcIni *ini) {

	self->f_Rg->LoadData(self->f_Rg, ini);
	self->f_Rs->LoadData(self->f_Rs, ini);
	self->f_Rx->LoadData(self->f_Rx, ini);
	self->f_Ra->LoadData(self->f_Ra, ini);

	self->f_L1->LoadData(self->f_L1, ini);
	self->f_L2->LoadData(self->f_L2, ini);
	self->f_L3->LoadData(self->f_L3, ini);
	self->f_L4->LoadData(self->f_L4, ini);

	self->f_Cs->LoadData(self->f_Cs, ini);
	self->f_Cx->LoadData(self->f_Cx, ini);
	self->f_C1->LoadData(self->f_C1, ini);
	self->f_C2->LoadData(self->f_C2, ini);
	self->f_C3->LoadData(self->f_C3, ini);

	math_Filter(self);
	Refresh((TWins*)self->pgfilt->xy_ind);
}

/* **************  Create ****************** */
static void DVal_Create_Xxx(TWins *Parent, TDVal **dv, TFormRLConoff **onoff, int *x, int y, int w, int nk, TPic *pic) {
  int tx = *x;
  TDVal *tdv = TDVal_CreateS(Parent, tx+19, y, w, 19);
	tdv->nk = nk;
	tdv->dv_fu = tdv_fu_LostFocusEnter;
	tdv->OnValChange = DVal_OnValChange;
	*dv = tdv;
	*onoff = TFormRLConoff_CreateU(Parent, tx, y+1, 18, 16, pic);
	*x = tx + w + 21;
}

static TFormL *Create_fL(TPage_TP2 *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormL *fL = Create_Form_L(appDat.obj, Caption, self->gname, s, dv, val);
	fL->subClass = self;
	TDVal_Set_Value(dv, fL->L->dval);
	fL->OnChange = fXxx_OnChange;
	fL->OnSync = OnLSync;
	return fL;
}
static TFormC *Create_fC(TPage_TP2 *self, const char *Caption,
			const char *s, TDVal *dv, float val) {
  TFormC *fC = Create_Form_C(appDat.obj, Caption, self->gname, s, dv, val);
	fC->subClass = self;
	TDVal_Set_Value(dv, fC->C->dval);
	fC->OnChange = fXxx_OnChange;
	fC->OnSync = OnCSync;
	return fC;
}

TPage_TP2 *page_Add_TP2(TPageControl *pagc, TPage_Filter *pgfilt) {
  TPage_TP2 *self;
  TWins *pnl;
  int x;
  TAddXFilter xf;

	xf.pagc = pagc;
	xf.pgfilt = pgfilt;
	xf.gname = "TP2";
	xf.psize = sizeof(TPage_TP2);
	xf.ini = get_appIni();
	xf.TDValOnValChange = DVal_OnValChange;
	xf.fXxx_OnChange = fXxx_OnChange;
	xf.OnRSync = OnRSync;
	xf.OnLSync = OnLSync;
	xf.OnCSync = OnCSync;
	xf.dvfu = FU_RG|FU_RS|FU_RX|FU_RA|FU_CS|FU_CX;

	xf.cb_OnClick = cbXxOn_OnClick;
	xf.cbfu = FU_RS|FU_RX|FU_CS|FU_CX;
	xf.cbOnOff = 0xffe0;
	xf.cbY = 82;

	xf.cbFxOnClick = cbFxOn_OnClick;
	xf.cbFxImpOnClick = cbFxImpOn_OnClick;
	xf.cbF1Y = 105;
	xf.cbF2Y = 150;
	xf.cbF3Y = 195;
	xf.cbF4Y = 240;
	xf.cbFxfu = FU_F1|FU_F2|FU_F3|FU_F4;

	xf.LC0_Popupfu = FU_F1|FU_F2|FU_F3|FU_F4;

	self = (TPage_TP2*)PageAddXFilter(&xf);


	self->pnl->OnPaint = pnl_OnPaint;
	pnl = self->pnl;
/* C1 C2 C3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_C1, &self->OnOff_C1, &x, 42, 54, 1, get_Pic_cC1());
	DVal_Create_Xxx(pnl, &self->dv_C2, &self->OnOff_C2, &x, 42, 54, 1, get_Pic_cC2());
	DVal_Create_Xxx(pnl, &self->dv_C3, &self->OnOff_C3, &x, 42, 54, 1, get_Pic_cC3());
//	DVal_Create_Xxx(pnl, &self->dv_C4, &self->OnOff_C4, &x, 42, 54, 1, get_Pic_cC4());
/* L1 L2 L3 */
	x = 0;
	DVal_Create_Xxx(pnl, &self->dv_L1, &self->OnOff_L1, &x, 62, 54, 3, get_Pic_cL1());
	DVal_Create_Xxx(pnl, &self->dv_L2, &self->OnOff_L2, &x, 62, 54, 3, get_Pic_cL2());
	DVal_Create_Xxx(pnl, &self->dv_L3, &self->OnOff_L3, &x, 62, 54, 3, get_Pic_cL3());
	DVal_Create_Xxx(pnl, &self->dv_L4, &self->OnOff_L4, &x, 62, 54, 3, get_Pic_cL4());

	self->f_C1 = Create_fC(self, "TP2 C1", "C1", self->dv_C1, 8);
	self->f_C2 = Create_fC(self, "TP2 C2", "C2", self->dv_C2, 16);
	self->f_C3 = Create_fC(self, "TP2 C3", "C3", self->dv_C3, 16);

	self->f_L1 = Create_fL(self, "TP2 L1", "L1", self->dv_L1, 0.027);
	self->f_L2 = Create_fL(self, "TP2 L2", "L2", self->dv_L2, 0.027);
	self->f_L3 = Create_fL(self, "TP2 L3", "L3", self->dv_L3, 0.027);
	self->f_L4 = Create_fL(self, "TP2 L4", "L4", self->dv_L4, 0.027);

	TFormRLConoff_SetForm(self->OnOff_C1, (TForm*)self->f_C1);
	TFormRLConoff_SetForm(self->OnOff_C2, (TForm*)self->f_C2);
	TFormRLConoff_SetForm(self->OnOff_C3, (TForm*)self->f_C3);
	TFormRLConoff_SetForm(self->OnOff_L1, (TForm*)self->f_L1);
	TFormRLConoff_SetForm(self->OnOff_L2, (TForm*)self->f_L2);
	TFormRLConoff_SetForm(self->OnOff_L3, (TForm*)self->f_L3);
	TFormRLConoff_SetForm(self->OnOff_L4, (TForm*)self->f_L4);

//	self->ScaleMHz = TpcIni_RD_float(pcIni, self->gname, "ScaleMHz", 1000);
	self->OnMsg = self_OnMsg;

	self->SaveData = SaveData;
	self->LoadData = LoadData;

	free_appIni(xf.ini);

	return self;
}
