/*
 Copyright (C) 2010-2012 by Hoefer Alfred

 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "TeCache.h"

typedef struct {
  TWins *lblSelAz;
  TChkBox *cbXM[max_cbXM];
} TPrivate;

static const char *sSpeedCPU[6]= {"PII 200MHz","PIII 700MHz","M1870MHz","Core2 2,4GHz","i7-870 3,0GHz", NULL};

static const char *sSpeed256[6]= {"t= 4,6 s", "t= 0,62 s","t= 0,176 s","t= 0,143 s","t= 0,111 s", NULL};
static const char *sSpeed512[6]= {"t= 6,7 s", "t= 2,15 s","t= 0,248 s","t= 0,202 s","t= 0,157 s", NULL};
static const char *sSpeed1M[6] = {"t= 12,7 s","t= 5,2 s", "t= 0,39 s", "t= 0,315 s","t= 0,246 s", NULL};
static const char *sSpeed2M[6] = {"t= 20,4 s","t= 8,4 s", "t= 0,56 s", "t= 0,44 s", "t= 0,342 s", NULL};
static const char *sSpeed4M[6] = {"t= 33,8 s","t= 13,9 s","t= 2,16 s", "t= 0,65 s", "t= 0,506 s", NULL};
static const char *sSpeed8M[6] = {"t= 45,8 s","t= 18,8 s","t= 3,7 s",  "t= 0,98 s", "t= 0,66 s", NULL};
static const char *sSpeed16M[6]= {"t= 72,9 s","t= 30,1 s","t= 7,1 s",  "t= 1,8 s",  "t= 1,09 s", NULL};
static const char *sSpeed32M[6]= {"t=101,3 s","t= 41,9 s","t= 10,9 s", "t= 2,7 s",  "t= 1,59 s", NULL};

static const char *sMove256[6]={"t= 88,6 s","t= 15,5 s","t= 3,67 s","t= 2,41 s","t= 0,73 s", NULL};
static const char *sMove512[6]= {"t= 107 s","t= 24,8 s","t= 4,03 s","t= 2,61 s","t= 0,83 s", NULL};
static const char *sMove1M[6] = {"t= 125 s","t= 33,1 s","t= 4,20 s","t= 2,70 s","t= 0,87 s", NULL};
static const char *sMove2M[6] = {"t= 143 s","t= 41,4 s","t= 5,18 s","t= 2,97 s","t= 0,98 s", NULL};
static const char *sMove4M[6] = {"t= 178 s","t= 56,0 s","t= 7,73 s","t= 4,13 s","t= 1,17 s", NULL};
static const char *sMove8M[6] = {"t= 246 s","t= 91,0 s","t= 12,9 s","t= 6,62 s","t= 1,85 s", NULL};
static const char *sMove16M[6]= {"t= 391 s","t= 159 s", "t= 23,8 s","t= 12,4 s","t= 3,43 s", NULL};
static const char *sMove32M[6]= {"t=3856 s","t= 293 s", "t= 46,3 s","t= 23,8 s","t= 6,63 s", NULL};

static const char **sSpeedX[max_cbXM] = {sSpeed256, sSpeed512, sSpeed1M, sSpeed2M, sSpeed4M, sSpeed8M, sSpeed16M, sSpeed32M};

static const char **sMoveX[max_cbXM] = {sMove256, sMove512, sMove1M, sMove2M, sMove4M, sMove8M, sMove16M, sMove32M};

static void str_to_pnlSelAz(TFormTeCache *self, const char *s, int x, int y) {
  TWins *az = self->pnlSelAz;
	vds_SetStr(&az->vds, s);
	DrawColorString(x, y, &az->vds);
}

static void pnlSelAz_z(TFormTeCache *form, const char **ppc, int y) {
	for (int i=0;i<5;i++) {
	    str_to_pnlSelAz(form, sSpeedCPU[i], 6, y);
	    y += 13;
	    str_to_pnlSelAz(form, *ppc, 9, y);
	    ppc++;
	    y += 16;
	}
}

static void TTeCache_pnlSelAz_OnPaint(TWins *self) {
  TFormTeCache *form = (TFormTeCache*)self->FormStruct;
  TPrivate *pv = self->private;
  int y;
  const char **ppc;
#ifdef use_PAGES
	if (form->PageControl->AktPage == form->pgExec) {
	    y = pv->cbXM[7]->top + 17;
	    ppc = sSpeedX[form->akt_MX];
	    pnlSelAz_z(form, ppc, y);
#endif
#ifdef use_PAGES
	} else if (form->PageControl->AktPage == form->pgMemMove) {
	    y = form->pnlAlign->top + form->pnlAlign->height;
	    ppc = sMoveX[form->akt_MX];
	    pnlSelAz_z(form, ppc, y);
	}
#endif
}

static void pnlSelAz_cbXM_OnClick(TChkBox *self, long KeyState, bool DblClick) {
  TFormTeCache *form = (TFormTeCache*)self->FormStruct;
  TWins *az = form->pnlSelAz;
  TPrivate *pv = az->private;
  int y;
	form->akt_MX = 0;
	for (int i=0;i<max_cbXM;i++) {
	    if (pv->cbXM[i] == self) form->akt_MX = i;
	    TChkBox_SetChecked(pv->cbXM[i], pv->cbXM[i] == self);
	}
	form->cd_view = &cd_Buf[form->akt_MX];
	form->cdm_view = &cdm_Buf[form->akt_MX];
	form->cdm_rd_view = &cdm_rd_Buf[form->akt_MX];

#ifdef use_exec_Xns
	form->ns_exec->xr = &exec_xr[form->akt_MX];
	form->ns_exec->xr2 = &exec_rd_xr[form->akt_MX];
	Refresh((TWins*)form->ns_exec);
#endif
	y = pv->cbXM[6]->top + 17;
	FillRectRGB(az->WinID, az->Color, 1, y, az->width-2, az->height-y-1);
	TTeCache_pnlSelAz_OnPaint(az);

	if (form->cd_default_arr) {
	    form->cd_default_view = &form->cd_default_arr[form->akt_MX];
	    if (!form->cd_default_view->sampz) {
		form->cd_default_view = NULL;
	    }
	}
	
	TTeCache_pnlExec_OnPaint(form->pnlExec);
#ifdef PAGES_MEM_MOVE
	form->xns_mov->xr = &dat_xr[form->akt_MX];
	form->xns_mov->xr2 = &dat_rd_xr[form->akt_MX];
	Refresh((TWins*)form->xns_mov);
#endif
}

/*____________________________ Create ________________________________*/
const char *sMemSize[8] = {"256k", "512k", "1M", "2M", "4M", "8M", "16M", "32M"};

void pnlSelAz_Create(TFormTeCache *form) {
  int i, x, y;
  TPrivate *pv;

	form->pnlSelAz = TWins_CreateBA(form->pnlMiddle, 0, 0, 96, 42, mBorder, alRight);
	form->pnlSelAz->OnPaint = TTeCache_pnlSelAz_OnPaint;
	pv = xcalloc(sizeof(TPrivate));
	form->pnlSelAz->private = pv;
	pv->lblSelAz = TWins_CreateLabel(form->pnlSelAz, 1, 1, 68, 16, "Mem Size", 0, 0);

	y = 16;
	x = 2;
	for (i=0;i<max_cbXM;i++) {
	    pv->cbXM[i] = TChkBox_CreateS(form->pnlSelAz, x, y, 46, 16, sMemSize[i]);
	    pv->cbXM[i]->OnClick = pnlSelAz_cbXM_OnClick;
	    x += 46;
	    if (i & 1) {
		x = 2;
		y += 16;
	    }
	}

#ifdef use_PAGES
	form->pnlAlign = TWins_CreateB(form->pnlSelAz, 1, y, 94, 54, mBorder | mBorderL);
	SetVisible(form->pnlAlign, False);
	form->pnlAlign->CaptionFu = mCaptionPaint | mCaptionTop;
	SetCaption(form->pnlAlign, "Align");
	form->pnlAlign->OnPaint = TTeCache_pnlAlign_OnPaint;
#endif

#ifdef _OS_WIN32
	SetFont(form->pnlSelAz, "-*-Arial-medium-r-*-*-10-*-*-*-*-*-*-*");
#else
	SetFont(form->pnlSelAz, "-*-helvetica-medium-r-*-*-10-*-75-75-*-*-*-*");
#endif /* ! _OS_WIN32 */

	TChkBox_SetChecked_ev(pv->cbXM[4], True);

}
