/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "appXbase.h"

#define APPNAME "form0"

#define TCSelf TFormMain
typedef struct TFormMain {
  AlgTForm
  TEdit *ed;
  TButton *buOk;
  TChkBox *cbUTF8;
  TChkBox *cbPasswd;
} TFormMain;
#undef TCSelf

static void TFormMain_OnResize(TFormMain *self) {
  int x, y;
	x = self->width - self->buOk->width - 2;
	if (x < 0) x = 0;
	y = self->height - self->buOk->height - 2;
	if (y < 0) y = 0;
	SetPosition((TWins*)self->buOk, x, y);
}

static void cbPasswd_OnClick(TChkBox* self, long KeyState, bool DblClick) {
  TFormMain *form;

	form = appDat.obj;
	if (self->Checked) {
	    TEdit_SetPasswdMod(form->ed, ED_PASSWD_ON, "¤");
//	    TEdit_SetPasswdMod(form->ed, ED_PASSWD_ON|ED_PASSWD_CHARSYNC, NULL);
//	    form->ed->Passwd |= ed_passwd_akt;
	} else {
//	    form->ed->Passwd &= ~ed_passwd_akt;
	    TEdit_SetPasswdMod(form->ed, ED_PASSWD_OFF, NULL);
	}
	RePaint((TWins*)form->ed);
}

static void cbUTF8_OnClick(TChkBox* self, long KeyState, bool DblClick) {
  TFormMain *form;

	form = appDat.obj;
	TWins_SetUTF8((TWins*)form->ed, self->Checked);
}

static void buOk_OnClick(TButton* self, long KeyState, bool DblClick) {
  Close();
}

/* ------------------------------------------------------------------- */
static const char *Help_Msg = {
  "\n"
  "Usage: form0\n"
  "\n"
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: "APPNAME" version "ALD_VERSION"  (build %d  %s)\n"
};

const int build_v = ALD_BUILD_V;
const char *build_date = __DATE__;

int main (int argc, char **argv) {
  int i;
  char *val;
//  char *InFile;
  TFormMain *self;
  bool help = False;
  bool err = False;


//	InFile = NULL;
	i = 1;
	while (!err && (i < argc)) {
	    val = argv[i];
	    if (*val == '-') {
		val++;
		switch (*val) {
		case 'h':
		    help = True;
		    break;
		case '-':
		    val++;
		    if (pcncmp(val, "help", 4)) {
			help = True;
		    } else err = True;
		    break;
		default: 
		    err = True;
		}
//	    } else {
//		InFile = val;
	    }
	    i++;
	}

//	err = err || !InFile;
	if (help || err) {
	    printf(Help_Msg, build_v, build_date);
	    exit(err);
	}

	initXapp(0, argc, argv);
#ifdef USE_DECOR
	appXDat.DefaultButtonDecor = TDecor_button_w3b3_Create();
//	appXDat.DefaultWinsDecor = TDecor_w3b3_Create(&TDecoBuR3B3_up_Tab, &TDecoBuR3B3_down_Tab);
#endif

	self = TForm_CreatParamAlg(NULL, sizeof(TFormMain));
	self->width = 200;
	self->height = 100;
	self->min_width = 100;
	self->min_height = 26;
	self->OnResize = TFormMain_OnResize;
	appDat.obj = self;
	SetCaption((TWins*)self, APPNAME);

	self->ed = TEdit_CreateS((TWins*)self, 0, 0, 60, 22, "AAAöäü.webm");
	self->ed->Align = alTop;
	TWins_SetUTF8((TWins*)self->ed, true);
  self->ed->NoCaret = true;

	self->buOk = TButton_CreateS((TWins*)self, 1, 21, 60, 22, "Ok");
	self->buOk->OnClick = buOk_OnClick;

	self->cbUTF8 = TChkBox_CreateS((TWins*)self, 0, 24, 40, 16, "Utf8");
	TChkBox_SetChecked(self->cbUTF8, true);
	self->cbUTF8->OnClick = cbUTF8_OnClick;

	self->cbPasswd = TChkBox_CreateS((TWins*)self, 54, 24, 60, 16, "Passwd");
	self->cbPasswd->OnClick = cbPasswd_OnClick;


	Set_Focus((TWins*)self->buOk);

	AppRun();
	return 0;
}
