/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "appXbase.h"
#include "tpixmap.h"

#define APPNAME "form_img"

#define Zoom_bpp_Test 1


#define TCSelf TFormMain
typedef struct TFormMain {
  AlgTForm
  char *fn;
  TPixMap *opxm;
  TPixMap *pxm;
#if Zoom_bpp_Test
  int pxm_format;  //pxm_4b_24bpp,...
  TPixMap *paint_pxm;
#endif
} TFormMain;
#undef TCSelf


static void TFormMain_OnPaint(TFormMain *self) {
  TFormMain *form = appDat.obj;
  TPixMap *pxm;

	if (!form->opxm) return;
	pxm = form->opxm;
	if (form->pxm) pxm = form->pxm;
#if Zoom_bpp_Test
	if (appXDat.RootW_depth == 16) {
	    if (!form->paint_pxm && ((pxm->pxm_LD & pxmLD_bppMask) != pxm_2b_16bpp)) {
		form->paint_pxm = (TPixMap*)TImage_rgb888to565((TImage*)pxm);
	    }
	} else {
	    if (!form->paint_pxm && ((pxm->pxm_LD & pxmLD_bppMask) == pxm_2b_16bpp)) {
		form->paint_pxm = (TPixMap*)TImage_rgb565to888m((TImage*)pxm);
	    }
	}
	if (form->paint_pxm) pxm = form->paint_pxm;
#endif
	TPixMap_PaintToWin(pxm, form->WinID, 0, 0);
}

static void TFormMain_OnResize(TFormMain *self) {
  TFormMain *form = appDat.obj;

	if (!form->opxm) return;
	if ((form->opxm->width == self->width) && 
	    (form->opxm->height == self->height)) {
	    ppTfree(&form->pxm);
	    return;
	}
	if (form->pxm &&
	    (form->pxm->width == self->width) && 
	    (form->pxm->height == self->height)) return;

#if Zoom_bpp_Test
	ppTfree(&form->paint_pxm);
#endif
	Tfree(form->pxm);
	form->pxm = TPixMap_dup_new_size(form->opxm, self->width, self->height);
}

static void TFormMain_SetCaption(TFormMain *self, char *info) {
	char *pc = pcadd_n(3, APPNAME, "   ", GetFileName_pc(self->fn));
	if (info) ppcadd2(&pc, "  ", info);
	SetCaption((TWins*)self, pc);
	xfree(pc);
}

static void TFormMain_OnAppStart() {
//  TFormMain *form = (TFormMain*)self->FormStruct;
  TFormMain *form = appDat.obj;


#if Zoom_bpp_Test
	form->opxm = TPixMap_Loader(form->fn, NULL, form->pxm_format);
#else
	form->opxm = TPixMap_Loader(form->fn, NULL, 0);
#endif
	if (form->opxm) {
	    Refresh((TWins*)form);
	    if (form->opxm->img_info) TFormMain_SetCaption(form, form->opxm->img_info);
	}
}

/*_____________________________ ErrorMsg _____________________________*/
static void Global_OnFileMsg(TFileMsg *fmsg) {
 file_msg_debug(fmsg, NULL);

	if ((fmsg->art == fmArt_errLogin) || (fmsg->art == fmArt_AlgErr)) {
	    default_Global_OnFileMsg(fmsg);
	}
	return;
}

/* ------------------------------------------------------------------- */
static const char *Help_Msg = {
  "\n"
  "Usage: "APPNAME" file\n"
  "\n"
#if Zoom_bpp_Test
  "  -b1                  read as 16bpp\n"
  "  -b2                  read as 24bpp\n"
#endif
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: "APPNAME" version "ALD_VERSION"  (build %d  %s)\n"
};

const int build_v = ALD_BUILD_V;
const char *build_date = __DATE__;

int main (int argc, char **argv) {
  int i;
#if Zoom_bpp_Test
  int m, pxm_format;  //pxm_4b_24bpp,...
#endif
  char *val;
  char *InFile;
  TFormMain *self;
  bool help = False;
  bool err = False;


#if Zoom_bpp_Test
	pxm_format = 0;
#endif
	InFile = NULL;
	i = 1;
	while (!err && (i < argc)) {
	    val = argv[i];
	    if (*val == '-') {
		val++;
		switch (*val) {
		case 'h':
		    help = True;
		    break;
#if Zoom_bpp_Test
		case 'b':
		    err = cmd_get_int(argc, argv, val, &i, &m);
		    if (!err) {
			if (m == 1) pxm_format = pxm_2b_16bpp;
			if (m == 2) pxm_format = pxm_4b_24bpp8a;
		    }
		    break;
#endif
		case '-':
		    val++;
		    if (pcncmp(val, "help", 4)) {
			help = True;
		    } else err = True;
		    break;
		default: 
		    err = True;
		}
	    } else {
		InFile = val;
	    }
	    i++;
	}

	err = err || !InFile;
	if (help || err) {
	    printf(Help_Msg, build_v, build_date);
	    exit(err);
	}

#ifdef _OS_WIN32
	char *pc = NT4_GetLongPathName(InFile);
	if (pc) InFile = pc;
#endif

	initXapp(0, argc, argv);

	ThImageRD_Reg(6, &hImageRD_xpm, &hImageRD_png, &hImageRD_jpeg, 
			 &hImageRD_gif, &hImageRD_bmp, &hImageRD_pnm);


	self = TForm_CreatParamAlg(NULL, sizeof(TFormMain));
	appDat.obj = self;
	self->width = 320;
	self->height = 200;
	self->min_width = 100;
	self->min_height = 26;
	self->OnResize = TFormMain_OnResize;
	self->OnPaint = TFormMain_OnPaint;
//	self->pxm_format = pxm_4b_24bpp8a;
#if Zoom_bpp_Test
	self->pxm_format = pxm_format;
#endif
	self->fn = path_rel2abs(InFile, NULL);
//	pc = pcadd_n(3, APPNAME, "   ", GetFileName_pc(self->fn));

	TImage *img = TImage_Loader(self->fn, NULL, pxmLD_OnlyInfo);
	if (img) {
	    m = img->width;
	    if (m < 100) m = 100;
	    if (m > 800) m = 800;
	    self->width = m;
	    m = img->height;
	    if (m < 26) m = 26;
	    if (m > 640) m = 640;
	    self->height = m;
	    Tfree(img);
	}
	TFormMain_SetCaption(self, NULL);

//	SetCaption((TWins*)self, pc);
//	xfree(pc);

	appXDat.OnAppStart = TFormMain_OnAppStart;
	appError_ev = default_Error_ev;
	Glob_OnFileMsg = Global_OnFileMsg;

	AppRun();
	return 0;
}
