/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "appXbase.h"
#include "dlg.h"
#include "cicons.h"

#define TCSelf TFormMain
typedef struct TFormMain {
  AlgTForm
  TWins *pnlTop;
  TWins *pnlMiddle;
  TWins *pnlBottom;

  TMenuButton *mbFile;
    TMenu *menuFile;

  TMenuButton *mbEdit;

  TMenuButton *mbHelp;
    TMenu *menuHelp;


  TWins *pnlL;
  TSplitterV *splittV;
  TWins *pnlR;

  TPageControl *pgcL;

  TPageControl *pgcR;

  TPage *pgR1;
  TPage *pgR2;
  TPage *pgR3;
  TPage *pgR4;

  TPage *AktPage;
  TButton *buNextPage;
  TButton *buPrevPage;

  TButton *buReparent;

  TButton *buTest;
  TButton *buClose;
} TFormMain;
#undef TCSelf

int build_v = ALD_BUILD_V;
static const char *build_date = __DATE__;

static void mHelp_Info(TMenu *self, long KeyState, TMenuItem *smi) {
  TFormMain *fmain = (TFormMain*)self->FormStruct;
  char buf[80];
	snprintf(buf, sizeof(buf), "ALDconsole %s   (build-%d  %s)", 
				ALD_VERSION, build_v, build_date);
	Copyright_MsgBox_Show((TWins*)fmain, buf);
}

static void mExit_OnClick(TMenu *self, long KeyState, TMenuItem *smi) {
  Close();
}

static void mFile_OnMenuOpen(TMenu *self) {
}

static void pgc_OnPageActivate(TPageControl *self, TPage *AktPage) {
  TFormMain *fmain = (TFormMain*)self->FormStruct;
	fmain->AktPage = AktPage;
 printf("%s(%d): 0x%x 0x%x\n", FL, (int)self, (int)AktPage);
	Set_Parent((TWins*)fmain->buReparent, (TWins*)AktPage);
}

static void buPrevPage_OnPaint(TButton *self) {
  TPic *pic;
	pic = get_pic9_hsb_minus(False);
	DrawPic(self->WinID, pic, 4, 4);
}

static void buNextPage_OnPaint(TButton *self) {
  TPic *pic;
	pic = get_pic9_hsb_plus(False);
	DrawPic(self->WinID, pic, 4, 4);
}

static void buPrevPage_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormMain *fmain = (TFormMain*)self->FormStruct;

	TPage_Move(fmain->AktPage, NULL, pgMove_ChkVisible|pgMove_Prev);

}

static void buNextPage_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormMain *fmain = (TFormMain*)self->FormStruct;

	TPage_Move(fmain->AktPage, NULL, pgMove_ChkVisible);
}

static void buTest_OnClick(TButton *self, long KeyState, bool DblClick) {
  TFormMain *fmain = (TFormMain*)self->FormStruct;
	ppTfree(&fmain->pgR1);
}

static void buClose_OnClick(TButton *self, long KeyState, bool DblClick) {
	Close();
}

static void pnlBottom_OnResize(TWins *self) {
  TFormMain *fmain = (TFormMain*)self->FormStruct;
	fmain->buClose->left = self->width - fmain->buClose->width - 1;
}

static const char *Help_Msg = {
  "\n"
  "Usage: edit\n"
  "\n"
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: edit version "ALD_VERSION"  (build %d  %s)\n"
};


int main (int argc, char **argv) {
  int i;
  char *val;
//  char *InFile;
  TFormMain *self;
  TMenuItem *mi;
  bool help = False;
  bool err = False;


//	InFile = NULL;
	i = 1;
	while (!err && (i<argc)) {
	    val = argv[i];
	    if (*val == '-') {
		val++;
		switch (*val) {
		case 'h':
		    help = True;
		    break;
		case '-':
		    val++;
		    if (pcncmp(val, "help", 4)) {
			help = True;
		    } else err = True;
		    break;
		default: 
		    err = True;
		}
	    } else {
//		InFile = val;
		err = True;
	    }
	    i++;
	}

//	err = err || !InFile;
	if (help || err) {
	    printf(Help_Msg, build_v, build_date);
	    exit(err);
	}

	initXapp(0, argc, argv);
	self = TForm_CreatParamAlg(NULL, sizeof(TFormMain));
	appDat.obj = self;

	self->pnlTop = TWins_CreateBA((TWins*)self, 0, 0, 200, 21, mBorder | mBorderL, alTop);
	self->pnlMiddle = TWins_CreateBA((TWins*)self, 0, 0, 200, 42, mBorder | mBorderL, alClient);
	self->pnlBottom = TWins_CreateBA((TWins*)self, 0, 0, 200, 23, 0, alBottom);
	self->pnlBottom->OnResize = pnlBottom_OnResize;

/* Menu File */
	self->mbFile = TMenuButton_Create(self->pnlTop, (TMenuButton*)self);
	self->mbFile->top = 2;
	self->mbFile->left = 1;
	SetCaption((TWins*)self->mbFile, "File");
	Set_Parent((TWins*)self->mbFile, self->pnlTop);

	self->menuFile = TMenu_Create((TWins*)self);
	self->mbFile->menu = self->menuFile;
	self->menuFile->OnMenuOpen = mFile_OnMenuOpen;
	TMenu_AddItem(self->menuFile, "Exit", 0, mExit_OnClick);

/* Menu Edit */
	self->mbEdit = TMenuButton_Create(self->pnlTop, self->mbFile);
	SetCaption((TWins*)self->mbEdit, "Edit");
	Set_Parent((TWins*)self->mbEdit, self->pnlTop);

/* Menu Help */
	self->mbHelp = TMenuButton_Create(self->pnlTop, self->mbFile);
	SetCaption((TWins*)self->mbHelp, "Help");
	Set_Parent((TWins*)self->mbHelp, self->pnlTop);

	self->menuHelp = TMenu_Create((TWins*)self);
	self->mbHelp->menu = self->menuHelp;
	mi = TMenu_AddItem(self->menuHelp, "Info", 0, mHelp_Info);
	mi->pic = GetZIcon(zico16info);
/* pnlMiddle */
	self->pnlL = TWins_CreateBA(self->pnlMiddle, 0, 0, 150, 42, mBorder, alLeft);
	self->pnlL->min_width = 60;
	self->splittV = TSplitterV_CreateP(self->pnlMiddle);
	self->pnlR = TWins_CreateBA(self->pnlMiddle, 0, 0, 150, 42, mBorder, alClient);
	self->pnlR->min_width = 60;

	self->pgcL = TPageControl_CreateA(self->pnlL);
	self->pgcL->OnPageActivate = pgc_OnPageActivate;
	self->pgcR = TPageControl_CreateA(self->pnlR);
	self->pgcR->OnPageActivate = pgc_OnPageActivate;

	TPageControl_AddNewPage(self->pgcL, "L1");
	TPageControl_AddNewPage(self->pgcL, "L2");
	TPageControl_AddNewPage(self->pgcL, "L3");
	TPageControl_AddNewPage(self->pgcL, "L4");

	self->pgR1 = TPageControl_AddNewPage(self->pgcR, " R1");
	self->pgR2 = TPageControl_AddNewPage(self->pgcR, " R2");
	self->pgR3 = TPageControl_AddNewPage(self->pgcR, " R3");
	self->pgR4 = TPageControl_AddNewPage(self->pgcR, " R4");
	TPageControl_AddNewPage(self->pgcR, " R5");
	TPageControl_AddNewPage(self->pgcR, " R6");
	TPageControl_AddNewPage(self->pgcR, " R7");
	TPageControl_AddNewPage(self->pgcR, " R8");

/* pnlBottom */
	self->buTest = TButton_CreateS(self->pnlBottom, 1, 1, 60, 22, "Test");
	self->buTest->OnClick = buTest_OnClick;

	self->buPrevPage = TButton_CreateS(self->pnlBottom, 100, 1, 17, 17, NULL);
	self->buPrevPage->KeyFu = 0;
	self->buPrevPage->OnPaint = buPrevPage_OnPaint;
	self->buPrevPage->OnClick = buPrevPage_OnClick;
	self->buNextPage = TButton_CreateS(self->pnlBottom, 120, 1, 17, 17, NULL);
	self->buNextPage->KeyFu = 0;
	self->buNextPage->OnPaint = buNextPage_OnPaint;
	self->buNextPage->OnClick = buNextPage_OnClick;

	self->buClose = TButton_CreateS(self->pnlBottom, 1, 1, 60, 22, "Close");
	self->buClose->OnClick = buClose_OnClick;

	self->buReparent = TButton_CreateS(self->pnlBottom, 64, 1, 30, 22, "X");

	Set_Focus((TWins*)self->buClose);


	AppRun();

#ifdef MEM_CHK
  Tfree(self);
  int lMCt;
  dword lMsize;
  MEM_CHK_Count(&lMCt, &lMsize);
  printf("Debug %s(%d): MEM_CHK_Count=%3d  size=%d wins=%d\n", FL, lMCt, lMsize, GetTWinsCount());
#endif

	return 0;
}

