/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#ifndef STANDALONE_BUILDIN
#include "../buildin_et.h"
#endif

#include "tsearch.h"
#include <unistd.h>
#include <sys/wait.h>
#include <fcntl.h>

#define APPNAME "daemonexec"


/* ------------------------------------------------------------------- */
const char *Help_Msg = {
  "\n"
  "Usage: "APPNAME" [-p path] file [args]\n"
  "\n"
  "  -o                   output dev (default /dev/null)\n"
  "  -p                   run path   (default / )\n"
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: "APPNAME" version "VERSION"  (build %d  %s)\n"
};

#ifdef STANDALONE_BUILDIN
const int build_v = ALD_BUILD_V;
const char *build_date = __DATE__;

int main (int argc, char **argv) {
#else

int bi_daemonexec_main (int argc, char **argv) {
#endif
  int i, InFileIdx, fd;
  TSearch_Rec *sr;
  char *val, *path, *exec, *fnexec, *odev;
  char **tmp_argv;
  const char *cpc, *cdev;
  TEnvPath *ep;
  Tstat64 *st;
  bool help = False;
  bool err = False;

	path = NULL;
	odev = NULL;
	tmp_argv = xmalloc(sizeof(void*) * argc);
	InFileIdx = 0;
	i = 1;
	while (!err && (i < argc)) {
	    val = argv[i];
	    if ((*val == '-') && !InFileIdx) {
		val++;
		switch (*val) {
		case 'h':
		    help = True;
		    break;
		case 'o':
		    err = cmd_get_pc(argc, argv, val, &i, &odev);
		    break;
		case 'p':
		    err = cmd_get_pc(argc, argv, val, &i, &path);
		    break;
		case '-':
		    val++;
		    if (pcncmp(val, "help", 4)) {
			help = True;
		    } else err = True;
		    break;
		default: 
		    err = True;
		}
	    } else {
		tmp_argv[InFileIdx] = pcdup(val);
		InFileIdx++;
	    }
	    i++;
	}

	if (help || err || !InFileIdx) {
	    printf(Help_Msg, build_v, build_date);
	    exit(EXIT_FAILURE);
	}

	tmp_argv[InFileIdx] = NULL;
	exec = tmp_argv[0];
	ep = TEnvPath_Create(NULL);
	fnexec = TEnvPath_GetExec(ep, exec);
	Tfree(ep);
	if (!fnexec) appError(EMSG_ERR_EXIT, "No such file \"%s\"", exec);
	st = file_stat_fn(fnexec, fsNoFileName | fsNoLinkName);
	err = !file_chk_access_st(st, X_OK);
	xfree(st);
	xfree(fnexec);
	if (err) appError(EMSG_ERR_EXIT, "Permission denied \"%s\"", exec);

	cdev = "/dev/null";
	if (odev) cdev = odev;
	st = file_stat_fn(cdev, fsNoFileName | fsNoLinkName);
	if (!st) appError(EMSG_ERR_EXIT, "No such file \"%s\"", cdev);
	if (st->FileType != stft_Chardev)
	    appError(EMSG_ERR_EXIT, "\"%s\" is not char dev", cdev);

	err = !file_chk_access_st(st, W_OK | R_OK);
	xfree(st);
	if (err) appError(EMSG_ERR_EXIT, "Permission denied \"%s\"", cdev);

	pid_t pid = fork();
	if (pid == -1) appError(EMSG_ERR_EXIT|EMSG_SYS, "cannot fork");
	if (pid != 0) exit (0);

	fd = open(cdev, O_RDWR);
//	fd = open("/dev/console", O_RDWR);
	if (fd <= 2) appError(EMSG_ERR_EXIT|EMSG_SYS, "/dev/null");
	dup2(fd, STDIN_FILENO);
	dup2(fd, STDOUT_FILENO);
	dup2(fd, STDERR_FILENO);
	close(fd);

	sr = TSearch_Create("/proc/self/fd/", findCHR|findFollowLink, fsFollowLink);
	if (sr && Find_First(sr)) {
sr_loop:
	    cpc = Get_ExFileDat_pc(sr->st, exfdt_FileName);
	    if ((*cpc >= '0') && (*cpc <= '9')) {
		i = pctoint(cpc);
		if (i > 2) close(i);
	    }
	    if (Find_Next(sr)) goto sr_loop;
	}
	Tfree(sr);

	pid = fork();
	if (pid == -1) exit(EXIT_FAILURE);
	if (pid == 0) {
	    setsid();
	    if (path) {
		if (chdir(path)) exit (EXIT_FAILURE);
	    } else {
		if (chdir("/")) exit (EXIT_FAILURE);
	    }

	    execvp(*tmp_argv, tmp_argv);
	    exit(EXIT_FAILURE);
	}
#if 0
	if (file_open_wr("/tmp/daemonexec.txt", NULL, &fd, 0)) {
	    file_write(fd, NULL, "run", 3, NULL);
	    file_close(fd, NULL);
	}
/*	while (1) {
	    fd_in_wait_sigle(0, 50000);
	    if (waitpid(-1, &stat_loc, WNOHANG) == pid) {
	    }
	}
*/
	waitpid(-1, &stat_loc, 0);
	if (file_open_wr("/tmp/daemonexec.txt", NULL, &fd, 0)) {
	    file_write(fd, NULL, "end", 3, NULL);
	    file_close(fd, NULL);
	}
#endif
	return 0;
}
