/*
 Copyright (C) 2010-2013 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#ifdef STANDALONE_BUILDIN
#include "sysbase.h"
#include "sysbaseio.h"
#else
#include "../buildin_et.h"
#endif

#define APPNAME "inidata"

typedef struct T_Item {
  struct T_Item *next;
  const char *name;
  const char *data;
} T_Item;

typedef struct T_Gruppe {
  struct T_Gruppe *next;
  const char *name;
  T_Item *item;
} T_Gruppe;

static T_Gruppe *gpd;


static T_Gruppe *NewGruppe(T_Gruppe *ng, char *name) {
  T_Gruppe *g = xcalloc(sizeof(T_Gruppe));

	g->name = name;
	if (ng) {
	    ng->next = g;
	} else {
	    gpd = g;
	}
	return g;
}

static T_Item *NewItem(T_Gruppe *ng, T_Item *ni, char *name) {
  T_Item *it = xcalloc(sizeof(T_Item));

	it->name = name;
	if (ni) {
	    ni->next = it;
	} else {
	    ng->item = it;
	}
	return it;
}

static const char *Help_Msg = {
  "\n"
  "Usage: "APPNAME" [options] -G \"val\" -I \"val\" \"File\"\n"
  "\n"
  "  -r                   Read Item\n"
  "  -w                   Write/Add Item\n"
  "  -rw                  Read Item is no Item Add Item\n"
  "  -wo                  Add Item only not exists\n"
  "  -G \"val\"             Gruppe\n"
  "  -I \"val\"             Item\n"
  "  -D \"val\"             r=Default w=Daten\n"
  "  -t                   Output whit \" \"dat\"\n"
  "  -x                   Output whit label \"item=dat\"\n"
  "  -x1                  Output whit label \"grupp_item=dat\"\n"
  "  -x2                  Output whit label \"grupp.item=dat\"\n"
  "  -h, --help           Prints out this screen\n"
  "Example:\n"
  "  "APPNAME" -w -G g1 -I i1 -D item1 /tmp/test.ini\n"
  "  "APPNAME" -r -G g1 -I i1 /tmp/test.ini\n"
  "  "APPNAME" -w -G g2 -I i2 -D item2 -I i3 -D item3 /tmp/test.ini\n"
  "  "APPNAME" -r -x2 -Gg1 -Ii1 -Gg2 -Ii2 -Ii3 /tmp/test.ini\n\n"
  "  ALDccp: "APPNAME" version "VERSION"  (build %d  %s)\n"
};

#ifdef STANDALONE_BUILDIN

const int build_v = ALD_BUILD_V;
const char *build_date = __DATE__;

int main (int argc, char **argv) {
#ifdef _OS_WIN32
	cmdUTF8korr(&argc, &argv);
#else
	cmdline_slash_korr(argc, argv);
#endif
#else

int bi_inidata_main (int argc, char **argv) {
#endif
  int i;
  char *pc, *tmp;
  const char *cpc;
  T_Gruppe *ng = NULL;
  T_Item *ni = NULL;
  char *File = NULL;
  TpcIni *ini;
  bool help = False;
  bool err = False;
  bool rd = False;
  bool wr = False;
  bool wrAddOnly = False;
  bool tMk = False;
  bool xMk = False;

	i = 1;
	while ((i < argc) && !help && !err) {
	    pc = argv[i];
	    if (*pc == '-') {
		pc++;
OptLoop:
		if (*pc == 'r') {
		    rd = True;
		    pc++; if (*pc) goto OptLoop;
		} else if (*pc == 'w') {
		    wr = True;
		    pc++; if (*pc) goto OptLoop;
		} else if (*pc == 'o') {
		    wrAddOnly = True;
		    pc++; if (*pc) goto OptLoop;
		} else if (*pc == 't') {
		    tMk = True;
		    pc++; if (*pc) goto OptLoop;
		} else if (*pc == 'x') {
		    xMk = 1;
		    pc++;
		    if ((*pc == '1') || (*pc == '2')) {
			if (*pc == '1') xMk = 2; else  xMk = 3;
			pc++;
		    }
		    if (*pc) goto OptLoop;

		} else if (*pc == 'h') {
		    help = True;

		} else if (*pc == 'G') {
		    err = cmd_get_pc(argc, argv, pc, &i, &tmp);
		    if (!err) {
			ng = NewGruppe(ng, tmp);
			ni = NULL;
		    }

		} else if (*pc == 'I') {
		    err = cmd_get_pc(argc, argv, pc, &i, &tmp);
		    if (!err) {
			if (ng) {
			    ni = NewItem(ng, ni, tmp);
			} else err = True;
		    }

		} else if (*pc == 'D') {
		    err = cmd_get_pc(argc, argv, pc, &i, &tmp);
		    if (!err) {
			if (ni) {
			    ni->data = tmp;
			} else err = True;
		    }

		} else if (*pc == '-') {
		    pc++;
		    if (pcncmp(pc, "help", 4)) {
			help = True;
		    } else err = True;
		} else err = True;

	    } else if (!File) {
		File = pc;
	    } else err = True;
	    i++;
	}

	if (!File) err = True;

	if (help || err) {
	    printf(Help_Msg, build_v, build_date);
	    exit(err);
	}
	if (!rd && !wr) rd = True;


	ini = TpcIni_Create(File);
	ng = gpd;
	while (ng) {
	    ni = ng->item;
	    while (ni) {
		if (rd) {
		    cpc = TpcIni_RD(ini, ng->name, ni->name);
		    if (!cpc) {
			if (wr) TpcIni_WR(ini, ng->name, ni->name, ni->data);
			cpc = ni->data;
		    }
		    if (xMk == 2) printf("%s_", ng->name);
		    if (xMk == 3) printf("%s.", ng->name);
		    if (xMk != 0) printf("%s=", ni->name);
		    if (tMk) printf("\"");
		    if (cpc) printf("%s", cpc);
		    if (tMk) printf("\"");
		    printf("\n");
		} else if (wr) {
		    if (wrAddOnly) {
			cpc = TpcIni_RD(ini, ng->name, ni->name);
			if (!cpc) TpcIni_WR(ini, ng->name, ni->name, ni->data);
		    } else TpcIni_WR(ini, ng->name, ni->name, ni->data);
		}
		ni = ni->next;
	    }
	    ng = ng->next;
	}
	
	if (ini->Changed) TpcIni_Save(ini);
	Tfree(ini);
	return 0;
}
