/*
 Copyright (C) 2010-2014 by Hoefer Alfred

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details. 

 You should have received a copy of the GNU General Public
 License along with this program; if not, write to the Free
 Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110, USA

 ----------------------------------------------------------------
 Licensed under GPL v2, see file COPYING in this tarball
*/

#include "sysbase.h"
#include "sysbaseio.h"
#include "zlib.h"

#define INBUF_SIZE 0x4000
#define OUTBUF_SIZE INBUF_SIZE+256


typedef struct {
  char *in_fn;
  FILE *outf;
  z_stream *s;
  byte *inBuf, *outBuf;
  dword total_in, total_out;
  dword out_count;
  char *pc_wr;
  char *cname;
  bool eof;
  char wrbuf[480];
} Tzsrc;

static void data_out(Tzsrc *zsrc) {

	if (!zsrc->outf) return;

	byte *pb = zsrc->outBuf;
	while (zsrc->out_count < zsrc->s->total_out) {
	    zsrc->out_count++;
	    int m = kCOMMA_ADD | 2;
	    if (zsrc->eof && (zsrc->out_count == zsrc->s->total_out)) m = 2;
	    hexto0xppc(&zsrc->pc_wr, *pb, m);
	    pb++;
	    if ((zsrc->out_count & 0xf) == 0) {
		*zsrc->pc_wr = '\n'; zsrc->pc_wr++; *zsrc->pc_wr = 0;
		fwrite(zsrc->wrbuf, 1, pclen(zsrc->wrbuf), zsrc->outf);
		zsrc->pc_wr = zsrc->wrbuf;
	    }
	}

	if (zsrc->eof) {
	    ppcmove(&zsrc->pc_wr, "};\n\n");
	    fwrite(zsrc->wrbuf, 1, pclen(zsrc->wrbuf), zsrc->outf);
	}
}

static bool run_txt(Tzsrc *zsrc) {
  FILE *inf;
  int inlg;

	inf = fopen(zsrc->in_fn, "rb");
	if (!inf) {
	    appError(EMSG_ERR | EMSG_SYS, " open %s ", zsrc->in_fn);
	    return False;
	}
	Fill0(zsrc->s, sizeof(z_stream));
	zsrc->out_count = 0;
	zsrc->wrbuf[0] = 0;
	zsrc->pc_wr = zsrc->wrbuf;
	zsrc->eof = False;

	while (!zsrc->eof) {
	    inlg = fread(zsrc->outBuf, 1, INBUF_SIZE, inf);
	    zsrc->eof = inlg < INBUF_SIZE;
	    zsrc->s->total_out += inlg;
	    data_out(zsrc);
	}

	fclose(inf);

	zsrc->total_in  = zsrc->s->total_out;
	zsrc->total_out = zsrc->s->total_out;

	return True;
}

static bool run_z(Tzsrc *zsrc) {
  FILE *inf;
  int inlg;

	inf = fopen(zsrc->in_fn, "rb");
	if (!inf) {
	    appError(EMSG_ERR | EMSG_SYS, " open %s ", zsrc->in_fn);
	    return False;
	}

	Fill0(zsrc->s, sizeof(z_stream));
	if (deflateInit2(zsrc->s, Z_BEST_COMPRESSION, Z_DEFLATED, -15, 8, Z_FILTERED) != Z_OK) {
	    fclose(inf);
	    appError(EMSG_ERR, " zlib_deflateInit");
	    return False;
	}
	zsrc->out_count = 0;
	zsrc->wrbuf[0] = 0;
	zsrc->pc_wr = zsrc->wrbuf;
	zsrc->eof = False;

	while (!zsrc->eof) {
	    inlg = fread(zsrc->inBuf, 1, INBUF_SIZE, inf);
	    zsrc->eof = inlg < INBUF_SIZE;
	    zsrc->s->avail_in = inlg;
	    zsrc->s->avail_out = OUTBUF_SIZE;
	    zsrc->s->next_in = zsrc->inBuf;
	    zsrc->s->next_out = zsrc->outBuf;

	    int m = Z_SYNC_FLUSH;
	    if (!inlg) m = Z_FINISH;
	    if (deflate(zsrc->s, m) < Z_OK) {
		fclose(inf);
		appError(EMSG_ERR, " zlib_deflate");
		return False;
	    }

	    data_out(zsrc);
	}
//	if (!zsrc->eof) goto loop;

//	if (zsrc->outf && (pc != zsrc->wrbuf)) 
//			fwrite(zsrc->wrbuf, 1, pclen(zsrc->wrbuf), zsrc->outf);

	fclose(inf);

	zsrc->total_in  = zsrc->s->total_in;
	zsrc->total_out = zsrc->s->total_out;
	deflateEnd(zsrc->s);

	return True;
}

static void const_int_out(Tzsrc *zsrc, int value, const char *s) {
  char *pc;

	if (!zsrc->outf) return;
	pc = zsrc->wrbuf;
	ppcmove(&pc, "const int ");
	ppcmove(&pc, zsrc->cname);
	ppcmove(&pc, "_");
	ppcmove(&pc, s);
	ppcmove(&pc, " = ");
	ppcmove_v32(&pc, value, kSEMICOL_ADD|kLF_ADD);
	fwrite(zsrc->wrbuf, 1, pclen(zsrc->wrbuf), zsrc->outf);
}

#define APPNAME "txt2zsrc"

static const char *Help_Msg = {
  "\n"
  "Usage: "APPNAME" [options] file\n"
  "\n"
  "  -o outfile           output file (default tmp_z.c)\n"
  "  -n                   no Compression\n"
  "  -z                   force Z-Compression\n"
  "  -t                   ThZsrc struct\n"
  "  -h, --help           Prints out this screen\n\n"
  "  ALDccp: "APPNAME" version "VERSION"  (build %d  %s)\n"
};

int main (int argc, char **argv) {
  int i;
  char *val;
  char *InFile, *OutFile, *fn, *pc;
  int do_compress = True;
  bool help = False;
  bool err = False;
  bool force_z = False;
  bool Tzsrc_struct = False;
  Tzsrc zsrc;

	InFile = NULL;
	OutFile = NULL;
	i = 1;
	while (!err && !help && (i<argc)) {
	    val = argv[i];
	    if (*val == '-') {
argv_loop:
		val++;
		switch (*val) {
		case 0: break;
		case 'h':
		    help = True;
		    goto argv_loop;
		case 'n':
		    do_compress = False;
		    goto argv_loop;
		case 't':
		    Tzsrc_struct = True;
		    goto argv_loop;
		case 'z':
		    force_z = True;
		    goto argv_loop;

		case 'o':
		    val++;
		    if (*val == '=') {
			val++;
			OutFile = pcdup(val);
		    } else if (((i+1)<argc) && (*argv[i+1] != '-')) {
			i++;
			OutFile = pcdup(argv[i]);
		    } else err = True;
		    break;
		    
		case '-':
		    val++;
		    if (pcncmp(val, "help", 4)) {
			help = True;
		    } else err = True;
		    break;
		default: 
		    err = True;
		}
	    } else {
		InFile = val;
	    }
	    i++;
	}

	err = err || !InFile;
	if (help || err) {
	    printf(Help_Msg, ALD_BUILD_V, __DATE__);
	    exit(1);
	}

	if (!OutFile) OutFile = pcadd(InFile, ".c");


	zsrc.in_fn = path_rel2abs(InFile, NULL);
	zsrc.s = xmalloc(sizeof(z_stream));
	zsrc.inBuf = xmalloc(INBUF_SIZE);
	zsrc.outBuf = xmalloc(OUTBUF_SIZE);
	zsrc.outf = NULL;

	if (do_compress) {
	    if (!run_z(&zsrc)) exit(1);
	    do_compress = force_z || (zsrc.s->total_in > zsrc.s->total_out); 

  printf("compr=%d total_in=%lu total_out=%lu %lu%%\n", 
	do_compress, zsrc.s->total_in, zsrc.s->total_out, zsrc.s->total_out * 100 / zsrc.s->total_in);

	}

	fn = path_rel2abs(OutFile, NULL);
	zsrc.outf = fopen(fn, "w");
	if (!zsrc.outf) {
	    appError(EMSG_ERR | EMSG_SYS, " open %s ", fn);
	    exit(1);
	}
	xfree(fn);

	zsrc.cname = ChangeFileExt(GetFileName_pc(OutFile), NULL);
	pc = zsrc.cname;
	while (*pc) {
	    if ((*pc == '.') || (*pc == '.')) *pc = '_';
	    pc++;
	}

	pc = zsrc.wrbuf;
	ppcmove(&pc, "//");
	if (do_compress) ppcmove(&pc, "zlib compressed");
	ppcmove(&pc, " file \"");
	ppcmove(&pc, GetFileName_pc(InFile));
	ppcmove(&pc, "\"\n\n");

	if (Tzsrc_struct) {
	    ppcmove(&pc, "#include \"zsrc.h\"\n\nstatic ");
	}
	ppcmove(&pc, "const unsigned char ");
	ppcmove(&pc, zsrc.cname);
	ppcmove(&pc, "[] = {\n");
	fwrite(zsrc.wrbuf, 1, pclen(zsrc.wrbuf), zsrc.outf);

	if (do_compress) {
	    run_z(&zsrc);
	} else {
	    run_txt(&zsrc);
	}

	if (Tzsrc_struct) {
	    pc = zsrc.wrbuf;
	    ppcmove(&pc, "const ThZsrc hZsrc_");
	    ppcmove(&pc, zsrc.cname);
	    ppcmove(&pc, " = {\n  \"");
	    ppcmove(&pc, GetFileName_pc(InFile));
	    ppcmove(&pc, "\",\n  ");
	    ppcmove(&pc, zsrc.cname);
	    ppcmove(&pc, ",\n  ");

	    ppcmove_v32(&pc, zsrc.total_out, kCOMMA_ADD|kLF_ADD);
	    ppcmove(&pc, "  ");
	    ppcmove_v32(&pc, zsrc.total_in, kCOMMA_ADD|kLF_ADD);
	    ppcmove(&pc, "  ");
	    ppcmove_v32(&pc, do_compress, kLF_ADD);
	    ppcmove(&pc, "};\n");

	    fwrite(zsrc.wrbuf, 1, pclen(zsrc.wrbuf), zsrc.outf);
	} else {
	    const_int_out(&zsrc, zsrc.total_out, "total_out");
	    const_int_out(&zsrc, zsrc.total_in,  "total_in");
	    const_int_out(&zsrc, do_compress,    "do_compress");
	}

	fclose(zsrc.outf);
	return 0;
}
